package org.geogebra.common.kernel.advanced;

import org.geogebra.common.euclidian.EuclidianViewInterfaceCommon;
import org.geogebra.common.kernel.Kernel;
import org.geogebra.common.kernel.algos.AlgoRoots;
import org.geogebra.common.kernel.arithmetic.Command;
import org.geogebra.common.kernel.commands.CommandProcessor;
import org.geogebra.common.kernel.geos.GeoElement;
import org.geogebra.common.kernel.geos.GeoFunctionable;
import org.geogebra.common.kernel.geos.GeoNumberValue;
import org.geogebra.common.main.MyError;

/**
 * Roots[ <GeoFunction>, <Number> , <Number> ] (Numerical version, more than one
 * root.)
 */
public class CmdRoots extends CommandProcessor {

	/**
	 * Create new command processor
	 *
	 * @param kernel
	 *            kernel
	 */
	public CmdRoots(Kernel kernel) {
		super(kernel);
	}// Constructor

	@Override
	final public GeoElement[] process(Command c) throws MyError {
		int n = c.getArgumentNumber();
		boolean[] ok = new boolean[n];
		GeoElement[] arg;

		switch (n) {
		case 1:
			arg = resArgs(c);
			if (arg[0].isGeoFunctionable()) {
				EuclidianViewInterfaceCommon view = this.kernelA
						.getApplication().getActiveEuclidianView();

				AlgoRoots algo = new AlgoRoots(cons, c.getLabels(),
						((GeoFunctionable) arg[0]).getGeoFunction(), view);
				return algo.getRootPoints();
			}
			throw argErr(app, c, arg[0]);

		case 3:
			arg = resArgs(c);
			if ((ok[0] = (arg[0].isGeoFunctionable()))
					&& (ok[1] = (arg[1] instanceof GeoNumberValue))
					&& (ok[2] = (arg[2] instanceof GeoNumberValue))) {

				AlgoRoots algo = new AlgoRoots(cons, c.getLabels(),
						((GeoFunctionable) arg[0]).getGeoFunction(),
						(GeoNumberValue) arg[1], (GeoNumberValue) arg[2]);

				GeoElement[] ret = algo.getRootPoints();
				return ret;
			}
			throw argErr(app, c, getBadArg(ok, arg));

		default:
			throw argNumErr(app, c, n);
		}// switch
	}// process(command)
}// class CmdRoots
