package org.geogebra.barycentric;

import java.io.IOException;
import java.io.OutputStream;
import java.util.TreeSet;

import javax.swing.JFrame;

import org.geogebra.desktop.CommandLineArguments;
import org.geogebra.desktop.main.AppD;
/**
 * This class parses the Mathematica data for cubics by Bernard Gibert an helps generate the Java code
 * @author Bernard Gibert
 * @author Zbynek Konecny 
 *
 */
public class ExpandConic {
	static AppD app;
	
	static TreeSet<String> sym=new TreeSet<String>(), asym=new TreeSet<String>(), nosym=new TreeSet<String>(), acyc=new TreeSet<String>();
	public static void main(String[] a){
		app = new AppD(new CommandLineArguments(new String[] { "--silent", "--giac" }), new JFrame(), false);
		try {
			app.getKernel().getGeoGebraCAS().evaluateRaw("pad3(l):=flatten({0,0,0,0,l})[size(l)..size(l)+3]");
			app.getKernel().getGeoGebraCAS().evaluateRaw("cyc(l):=subst(subst(l,{c=cc,b=c,a=b,C=CC,B=C,A=B}),{cc=a,CC=A})");
			app.getKernel().getGeoGebraCAS().evaluateRaw("swapab(l):=subst(subst(l,{b=bb,a=b,B=BB,A=B}),{bb=a,BB=A})");
			app.getKernel().getGeoGebraCAS().evaluateRaw("coeff3(s):=map(map(pad3(coeffs(s,x)),tt->pad3(coeffs(tt,y))),uu->map(uu,vv->pad3(coeffs(vv,z))))");
		} catch (Throwable e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		doExp();
		/*System.out.println(acyc.size());
		System.out.println(nosym.size());
		System.out.println(asym.size());
		System.out.println(sym.size());
		for(String s:nosym){
			System.out.println(s);
		}
		System.out.println("---");
		for(String s:acyc){
			System.out.println(s);
		}*/
	}
	private static void doExp() {
		exp("K001","a^4*c^2*x^2*y + a^2*b^2*c^2*x^2*y - 2*b^4*c^2*x^2*y - 2*a^2*c^4*x^2*y + b^2*c^4*x^2*y +      c^6*x^2*y + 2*a^4*c^2*x*y^2 - a^2*b^2*c^2*x*y^2 - b^4*c^2*x*y^2 - a^2*c^4*x*y^2 + 2*b^2*c^4*x*y^2 -      c^6*x*y^2 - a^4*b^2*x^2*z + 2*a^2*b^4*x^2*z - b^6*x^2*z - a^2*b^2*c^2*x^2*z - b^4*c^2*x^2*z +      2*b^2*c^4*x^2*z + a^6*y^2*z - 2*a^4*b^2*y^2*z + a^2*b^4*y^2*z + a^4*c^2*y^2*z + a^2*b^2*c^2*y^2*z -      2*a^2*c^4*y^2*z - 2*a^4*b^2*x*z^2 + a^2*b^4*x*z^2 + b^6*x*z^2 + a^2*b^2*c^2*x*z^2 -      2*b^4*c^2*x*z^2 + b^2*c^4*x*z^2 - a^6*y*z^2 - a^4*b^2*y*z^2 + 2*a^2*b^4*y*z^2 + 2*a^4*c^2*y*z^2 -      a^2*b^2*c^2*y*z^2 - a^2*c^4*y*z^2");
		exp("K002","c^2*x^2*y - c^2*x*y^2 - b^2*x^2*z + a^2*y^2*z + b^2*x*z^2 - a^2*y*z^2");
		exp("K003","a^2*b^2*c^2*x^2*y - b^4*c^2*x^2*y + b^2*c^4*x^2*y + a^4*c^2*x*y^2 - a^2*b^2*c^2*x*y^2 -      a^2*c^4*x*y^2 - a^2*b^2*c^2*x^2*z - b^4*c^2*x^2*z + b^2*c^4*x^2*z + a^4*c^2*y^2*z +      a^2*b^2*c^2*y^2*z - a^2*c^4*y^2*z - a^4*b^2*x*z^2 + a^2*b^4*x*z^2 + a^2*b^2*c^2*x*z^2 -      a^4*b^2*y*z^2 + a^2*b^4*y*z^2 - a^2*b^2*c^2*y*z^2");
		exp("K004","a^4*c^2*x^2*y + 2*a^2*b^2*c^2*x^2*y - 3*b^4*c^2*x^2*y - 2*a^2*c^4*x^2*y + 2*b^2*c^4*x^2*y +      c^6*x^2*y + 3*a^4*c^2*x*y^2 - 2*a^2*b^2*c^2*x*y^2 - b^4*c^2*x*y^2 - 2*a^2*c^4*x*y^2 +      2*b^2*c^4*x*y^2 - c^6*x*y^2 - a^4*b^2*x^2*z + 2*a^2*b^4*x^2*z - b^6*x^2*z - 2*a^2*b^2*c^2*x^2*z -      2*b^4*c^2*x^2*z + 3*b^2*c^4*x^2*z + a^6*y^2*z - 2*a^4*b^2*y^2*z + a^2*b^4*y^2*z + 2*a^4*c^2*y^2*z +      2*a^2*b^2*c^2*y^2*z - 3*a^2*c^4*y^2*z - 3*a^4*b^2*x*z^2 + 2*a^2*b^4*x*z^2 + b^6*x*z^2 +      2*a^2*b^2*c^2*x*z^2 - 2*b^4*c^2*x*z^2 + b^2*c^4*x*z^2 - a^6*y*z^2 - 2*a^4*b^2*y*z^2 +      3*a^2*b^4*y*z^2 + 2*a^4*c^2*y*z^2 - 2*a^2*b^2*c^2*y*z^2 - a^2*c^4*y*z^2");
		exp("K005","a^4*c^2*x^2*y - a^2*b^2*c^2*x^2*y - 2*a^2*c^4*x^2*y - b^2*c^4*x^2*y + c^6*x^2*y +      a^2*b^2*c^2*x*y^2 - b^4*c^2*x*y^2 + a^2*c^4*x*y^2 + 2*b^2*c^4*x*y^2 - c^6*x*y^2 - a^4*b^2*x^2*z +      2*a^2*b^4*x^2*z - b^6*x^2*z + a^2*b^2*c^2*x^2*z + b^4*c^2*x^2*z + a^6*y^2*z - 2*a^4*b^2*y^2*z +      a^2*b^4*y^2*z - a^4*c^2*y^2*z - a^2*b^2*c^2*y^2*z - a^2*b^4*x*z^2 + b^6*x*z^2 - a^2*b^2*c^2*x*z^2 -      2*b^4*c^2*x*z^2 + b^2*c^4*x*z^2 - a^6*y*z^2 + a^4*b^2*y*z^2 + 2*a^4*c^2*y*z^2 + a^2*b^2*c^2*y*z^2 -      a^2*c^4*y*z^2");
		exp("K006","(-a^4)*c^2*x^2*y + b^4*c^2*x^2*y + 2*a^2*c^4*x^2*y - c^6*x^2*y - a^4*c^2*x*y^2 + b^4*c^2*x*y^2 -      2*b^2*c^4*x*y^2 + c^6*x*y^2 + a^4*b^2*x^2*z - 2*a^2*b^4*x^2*z + b^6*x^2*z - b^2*c^4*x^2*z -      a^6*y^2*z + 2*a^4*b^2*y^2*z - a^2*b^4*y^2*z + a^2*c^4*y^2*z + a^4*b^2*x*z^2 - b^6*x*z^2 +      2*b^4*c^2*x*z^2 - b^2*c^4*x*z^2 + a^6*y*z^2 - a^2*b^4*y*z^2 - 2*a^4*c^2*y*z^2 + a^2*c^4*y*z^2");
		exp("K007","a^2*x^2*y - b^2*x^2*y + c^2*x^2*y + a^2*x*y^2 - b^2*x*y^2 - c^2*x*y^2 - a^2*x^2*z - b^2*x^2*z +      c^2*x^2*z + a^2*y^2*z + b^2*y^2*z - c^2*y^2*z - a^2*x*z^2 + b^2*x*z^2 + c^2*x*z^2 - a^2*y*z^2 +      b^2*y*z^2 - c^2*y*z^2");
		exp("K008","a^4*x^2*y - b^4*x^2*y - a^2*c^2*x^2*y + c^4*x^2*y + a^4*x*y^2 - b^4*x*y^2 + b^2*c^2*x*y^2 -      c^4*x*y^2 - a^4*x^2*z + a^2*b^2*x^2*z - b^4*x^2*z + c^4*x^2*z + a^4*y^2*z - a^2*b^2*y^2*z +      b^4*y^2*z - c^4*y^2*z - a^4*x*z^2 + b^4*x*z^2 - b^2*c^2*x*z^2 + c^4*x*z^2 - a^4*y*z^2 + b^4*y*z^2 +      a^2*c^2*y*z^2 - c^4*y*z^2");
		exp("K009","-(a^2*c^4*x^2*y) - b^2*c^4*x^2*y + c^6*x^2*y + a^2*c^4*x*y^2 + b^2*c^4*x*y^2 - c^6*x*y^2 +      a^2*b^4*x^2*z - b^6*x^2*z + b^4*c^2*x^2*z + 2*a^4*b^2*x*y*z - 2*a^2*b^4*x*y*z - 2*a^4*c^2*x*y*z +      2*b^4*c^2*x*y*z + 2*a^2*c^4*x*y*z - 2*b^2*c^4*x*y*z + a^6*y^2*z - a^4*b^2*y^2*z - a^4*c^2*y^2*z -      a^2*b^4*x*z^2 + b^6*x*z^2 - b^4*c^2*x*z^2 - a^6*y*z^2 + a^4*b^2*y*z^2 + a^4*c^2*y*z^2");
		exp("K010","-2*(a^2 + b^2 + c^2)*x*y*z + (a^2 + b^2 - c^2)*(x^2 + y^2)*z + (a^2 - b^2 + c^2)*y*(x^2 + z^2) +      (-a^2 + b^2 + c^2)*x*(y^2 + z^2)");
		exp("K011","a^6*x^2*y - a^4*b^2*x^2*y - a^2*b^4*x^2*y + b^6*x^2*y - a^4*c^2*x^2*y + 2*a^2*b^2*c^2*x^2*y -      b^4*c^2*x^2*y - a^2*c^4*x^2*y - b^2*c^4*x^2*y + c^6*x^2*y + a^6*x*y^2 - a^4*b^2*x*y^2 -      a^2*b^4*x*y^2 + b^6*x*y^2 - a^4*c^2*x*y^2 + 2*a^2*b^2*c^2*x*y^2 - b^4*c^2*x*y^2 - a^2*c^4*x*y^2 -      b^2*c^4*x*y^2 + c^6*x*y^2 + a^6*x^2*z - a^4*b^2*x^2*z - a^2*b^4*x^2*z + b^6*x^2*z - a^4*c^2*x^2*z +      2*a^2*b^2*c^2*x^2*z - b^4*c^2*x^2*z - a^2*c^4*x^2*z - b^2*c^4*x^2*z + c^6*x^2*z + 2*a^6*x*y*z -      2*a^4*b^2*x*y*z - 2*a^2*b^4*x*y*z + 2*b^6*x*y*z - 2*a^4*c^2*x*y*z + 12*a^2*b^2*c^2*x*y*z -      2*b^4*c^2*x*y*z - 2*a^2*c^4*x*y*z - 2*b^2*c^4*x*y*z + 2*c^6*x*y*z + a^6*y^2*z - a^4*b^2*y^2*z -      a^2*b^4*y^2*z + b^6*y^2*z - a^4*c^2*y^2*z + 2*a^2*b^2*c^2*y^2*z - b^4*c^2*y^2*z - a^2*c^4*y^2*z -      b^2*c^4*y^2*z + c^6*y^2*z + a^6*x*z^2 - a^4*b^2*x*z^2 - a^2*b^4*x*z^2 + b^6*x*z^2 - a^4*c^2*x*z^2 +      2*a^2*b^2*c^2*x*z^2 - b^4*c^2*x*z^2 - a^2*c^4*x*z^2 - b^2*c^4*x*z^2 + c^6*x*z^2 + a^6*y*z^2 -      a^4*b^2*y*z^2 - a^2*b^4*y*z^2 + b^6*y*z^2 - a^4*c^2*y*z^2 + 2*a^2*b^2*c^2*y*z^2 - b^4*c^2*y*z^2 -      a^2*c^4*y*z^2 - b^2*c^4*y*z^2 + c^6*y*z^2");
		exp("K012","a^2*b^2*c^2*x^2*y + a^2*b^2*c^2*x*y^2 + a^2*b^2*c^2*x^2*z - a^4*b^2*x*y*z - a^2*b^4*x*y*z -      a^4*c^2*x*y*z - b^4*c^2*x*y*z - a^2*c^4*x*y*z - b^2*c^4*x*y*z + a^2*b^2*c^2*y^2*z +      a^2*b^2*c^2*x*z^2 + a^2*b^2*c^2*y*z^2");
		exp("K013","a^3*x^2*y - a^2*b*x^2*y - a*b^2*x^2*y + b^3*x^2*y - a^2*c*x^2*y + 2*a*b*c*x^2*y - b^2*c*x^2*y -      a*c^2*x^2*y - b*c^2*x^2*y + c^3*x^2*y + a^3*x*y^2 - a^2*b*x*y^2 - a*b^2*x*y^2 + b^3*x*y^2 -      a^2*c*x*y^2 + 2*a*b*c*x*y^2 - b^2*c*x*y^2 - a*c^2*x*y^2 - b*c^2*x*y^2 + c^3*x*y^2 + a^3*x^2*z -      a^2*b*x^2*z - a*b^2*x^2*z + b^3*x^2*z - a^2*c*x^2*z + 2*a*b*c*x^2*z - b^2*c*x^2*z - a*c^2*x^2*z -      b*c^2*x^2*z + c^3*x^2*z + 2*a^3*x*y*z - 2*a^2*b*x*y*z - 2*a*b^2*x*y*z + 2*b^3*x*y*z -      2*a^2*c*x*y*z + 12*a*b*c*x*y*z - 2*b^2*c*x*y*z - 2*a*c^2*x*y*z - 2*b*c^2*x*y*z + 2*c^3*x*y*z +      a^3*y^2*z - a^2*b*y^2*z - a*b^2*y^2*z + b^3*y^2*z - a^2*c*y^2*z + 2*a*b*c*y^2*z - b^2*c*y^2*z -      a*c^2*y^2*z - b*c^2*y^2*z + c^3*y^2*z + a^3*x*z^2 - a^2*b*x*z^2 - a*b^2*x*z^2 + b^3*x*z^2 -      a^2*c*x*z^2 + 2*a*b*c*x*z^2 - b^2*c*x*z^2 - a*c^2*x*z^2 - b*c^2*x*z^2 + c^3*x*z^2 + a^3*y*z^2 -      a^2*b*y*z^2 - a*b^2*y*z^2 + b^3*y*z^2 - a^2*c*y*z^2 + 2*a*b*c*y*z^2 - b^2*c*y*z^2 - a*c^2*y*z^2 -      b*c^2*y*z^2 + c^3*y*z^2");
		exp("K014","a*b*c*x^2*y + a*b*c*x*y^2 + a*b*c*x^2*z - a^2*b*x*y*z - a*b^2*x*y*z - a^2*c*x*y*z - b^2*c*x*y*z -      a*c^2*x*y*z - b*c^2*x*y*z + a*b*c*y^2*z + a*b*c*x*z^2 + a*b*c*y*z^2");
		exp("K015","x^2*y + x*y^2 + x^2*z - 6*x*y*z + y^2*z + x*z^2 + y*z^2");
		exp("K016","x^2*y + x*y^2 + x^2*z + y^2*z + x*z^2 + y*z^2");
		exp("K017","b^4*c^2*x^2*y - a^2*c^4*x^2*y + a^4*c^2*x*y^2 - b^2*c^4*x*y^2 - a^2*b^4*x^2*z + b^2*c^4*x^2*z -      a^4*b^2*y^2*z + a^2*c^4*y^2*z + a^4*b^2*x*z^2 - b^4*c^2*x*z^2 + a^2*b^4*y*z^2 - a^4*c^2*y*z^2");
		exp("K018","-(a^2*c^2*x^2*y) + c^4*x^2*y + b^2*c^2*x*y^2 - c^4*x*y^2 + a^2*b^2*x^2*z - b^4*x^2*z +      a^4*y^2*z - a^2*b^2*y^2*z + b^4*x*z^2 - b^2*c^2*x*z^2 - a^4*y*z^2 + a^2*c^2*y*z^2");
		exp("K019","-(a^4*b^2*c^2*x^2*y) + a^2*b^4*c^2*x^2*y - b^4*c^4*x^2*y + b^2*c^6*x^2*y - a^4*b^2*c^2*x*y^2 +      a^2*b^4*c^2*x*y^2 + a^4*c^4*x*y^2 - a^2*c^6*x*y^2 + a^4*b^2*c^2*x^2*z - b^6*c^2*x^2*z -      a^2*b^2*c^4*x^2*z + b^4*c^4*x^2*z + a^6*c^2*y^2*z - a^2*b^4*c^2*y^2*z - a^4*c^4*y^2*z +      a^2*b^2*c^4*y^2*z - a^4*b^4*x*z^2 + a^2*b^6*x*z^2 + a^4*b^2*c^2*x*z^2 - a^2*b^2*c^4*x*z^2 -      a^6*b^2*y*z^2 + a^4*b^4*y*z^2 - a^2*b^4*c^2*y*z^2 + a^2*b^2*c^4*y*z^2");
		exp("K020","b^4*c^2*x^2*y + a^2*c^4*x^2*y - a^4*c^2*x*y^2 - b^2*c^4*x*y^2 - a^2*b^4*x^2*z - b^2*c^4*x^2*z +      a^4*b^2*y^2*z + a^2*c^4*y^2*z + a^4*b^2*x*z^2 + b^4*c^2*x*z^2 - a^2*b^4*y*z^2 - a^4*c^2*y*z^2");
		exp("K021","-(a^2*b^2*c^2*x^2*y) + b^2*c^4*x^2*y - a^2*b^2*c^2*x*y^2 + a^2*c^4*x*y^2 - a^2*b^2*c^2*x^2*z +      b^4*c^2*x^2*z + a^4*c^2*y^2*z - a^2*b^2*c^2*y^2*z + a^2*b^4*x*z^2 - a^2*b^2*c^2*x*z^2 +      a^4*b^2*y*z^2 - a^2*b^2*c^2*y*z^2");
		exp("K022","a^4*c^2*x^2*y - a^2*b^2*c^2*x^2*y - 2*b^4*c^2*x^2*y + 3*b^2*c^4*x^2*y - c^6*x^2*y -      2*a^4*c^2*x*y^2 - a^2*b^2*c^2*x*y^2 + b^4*c^2*x*y^2 + 3*a^2*c^4*x*y^2 - c^6*x*y^2 + a^4*b^2*x^2*z -      b^6*x^2*z - a^2*b^2*c^2*x^2*z + 3*b^4*c^2*x^2*z - 2*b^2*c^4*x^2*z + 2*a^6*x*y*z - 2*a^4*b^2*x*y*z -      2*a^2*b^4*x*y*z + 2*b^6*x*y*z - 2*a^4*c^2*x*y*z + 6*a^2*b^2*c^2*x*y*z - 2*b^4*c^2*x*y*z -      2*a^2*c^4*x*y*z - 2*b^2*c^4*x*y*z + 2*c^6*x*y*z - a^6*y^2*z + a^2*b^4*y^2*z + 3*a^4*c^2*y^2*z -      a^2*b^2*c^2*y^2*z - 2*a^2*c^4*y^2*z - 2*a^4*b^2*x*z^2 + 3*a^2*b^4*x*z^2 - b^6*x*z^2 -      a^2*b^2*c^2*x*z^2 + b^2*c^4*x*z^2 - a^6*y*z^2 + 3*a^4*b^2*y*z^2 - 2*a^2*b^4*y*z^2 -      a^2*b^2*c^2*y*z^2 + a^2*c^4*y*z^2");
		exp("K023","-(a^8*b^2*x^2*y) + 3*a^6*b^4*x^2*y - 3*a^4*b^6*x^2*y + a^2*b^8*x^2*y + a^6*b^2*c^2*x^2*y -      2*a^4*b^4*c^2*x^2*y + 2*a^2*b^6*c^2*x^2*y - b^8*c^2*x^2*y - a^6*c^4*x^2*y - a^4*b^2*c^4*x^2*y +      b^6*c^4*x^2*y + 3*a^4*c^6*x^2*y + a^2*b^2*c^6*x^2*y - b^4*c^6*x^2*y - 3*a^2*c^8*x^2*y +      c^10*x^2*y - a^8*b^2*x*y^2 + 3*a^6*b^4*x*y^2 - 3*a^4*b^6*x*y^2 + a^2*b^8*x*y^2 + a^8*c^2*x*y^2 -      2*a^6*b^2*c^2*x*y^2 + 2*a^4*b^4*c^2*x*y^2 - a^2*b^6*c^2*x*y^2 - a^6*c^4*x*y^2 + a^2*b^4*c^4*x*y^2 +      b^6*c^4*x*y^2 + a^4*c^6*x*y^2 - a^2*b^2*c^6*x*y^2 - 3*b^4*c^6*x*y^2 + 3*b^2*c^8*x*y^2 -      c^10*x*y^2 + a^6*b^4*x^2*z - 3*a^4*b^6*x^2*z + 3*a^2*b^8*x^2*z - b^10*x^2*z + a^8*c^2*x^2*z -      a^6*b^2*c^2*x^2*z + a^4*b^4*c^2*x^2*z - a^2*b^6*c^2*x^2*z - 3*a^6*c^4*x^2*z + 2*a^4*b^2*c^4*x^2*z +      b^6*c^4*x^2*z + 3*a^4*c^6*x^2*z - 2*a^2*b^2*c^6*x^2*z - b^4*c^6*x^2*z - a^2*c^8*x^2*z +      b^2*c^8*x^2*z + a^10*y^2*z - 3*a^8*b^2*y^2*z + 3*a^6*b^4*y^2*z - a^4*b^6*y^2*z +      a^6*b^2*c^2*y^2*z - a^4*b^4*c^2*y^2*z + a^2*b^6*c^2*y^2*z - b^8*c^2*y^2*z - a^6*c^4*y^2*z -      2*a^2*b^4*c^4*y^2*z + 3*b^6*c^4*y^2*z + a^4*c^6*y^2*z + 2*a^2*b^2*c^6*y^2*z - 3*b^4*c^6*y^2*z -      a^2*c^8*y^2*z + b^2*c^8*y^2*z - a^8*b^2*x*z^2 + a^6*b^4*x*z^2 - a^4*b^6*x*z^2 + b^10*x*z^2 +      a^8*c^2*x*z^2 + 2*a^6*b^2*c^2*x*z^2 + a^2*b^6*c^2*x*z^2 - 3*b^8*c^2*x*z^2 - 3*a^6*c^4*x*z^2 -      2*a^4*b^2*c^4*x*z^2 - a^2*b^4*c^4*x*z^2 + 3*b^6*c^4*x*z^2 + 3*a^4*c^6*x*z^2 + a^2*b^2*c^6*x*z^2 -      b^4*c^6*x*z^2 - a^2*c^8*x*z^2 - a^10*y*z^2 + a^6*b^4*y*z^2 - a^4*b^6*y*z^2 + a^2*b^8*y*z^2 +      3*a^8*c^2*y*z^2 - a^6*b^2*c^2*y*z^2 - 2*a^2*b^6*c^2*y*z^2 - b^8*c^2*y*z^2 - 3*a^6*c^4*y*z^2 +      a^4*b^2*c^4*y*z^2 + 2*a^2*b^4*c^4*y*z^2 + 3*b^6*c^4*y*z^2 + a^4*c^6*y*z^2 - a^2*b^2*c^6*y*z^2 -      3*b^4*c^6*y*z^2 + b^2*c^8*y*z^2");
		exp("K024","b^2*c^2*x^2*y + a^2*c^2*x*y^2 + b^2*c^2*x^2*z + a^2*c^2*y^2*z + a^2*b^2*x*z^2 + a^2*b^2*y*z^2 ");
		exp("K025","a^6*x^2*y - 3*a^4*b^2*x^2*y + 3*a^2*b^4*x^2*y - b^6*x^2*y + a^2*b^2*c^2*x^2*y - b^4*c^2*x^2*y -      2*a^2*c^4*x^2*y + b^2*c^4*x^2*y + c^6*x^2*y + a^6*x*y^2 - 3*a^4*b^2*x*y^2 + 3*a^2*b^4*x*y^2 -      b^6*x*y^2 + a^4*c^2*x*y^2 - a^2*b^2*c^2*x*y^2 - a^2*c^4*x*y^2 + 2*b^2*c^4*x*y^2 - c^6*x*y^2 -      a^6*x^2*z + 2*a^2*b^4*x^2*z - b^6*x^2*z + 3*a^4*c^2*x^2*z - a^2*b^2*c^2*x^2*z - b^4*c^2*x^2*z -      3*a^2*c^4*x^2*z + b^2*c^4*x^2*z + c^6*x^2*z - 2*a^4*b^2*x*y*z + 2*a^2*b^4*x*y*z + 2*a^4*c^2*x*y*z -      2*b^4*c^2*x*y*z - 2*a^2*c^4*x*y*z + 2*b^2*c^4*x*y*z + a^6*y^2*z - 2*a^4*b^2*y^2*z + b^6*y^2*z +      a^4*c^2*y^2*z + a^2*b^2*c^2*y^2*z - 3*b^4*c^2*y^2*z - a^2*c^4*y^2*z + 3*b^2*c^4*y^2*z - c^6*y^2*z -      a^6*x*z^2 - a^4*b^2*x*z^2 + a^2*b^4*x*z^2 + b^6*x*z^2 + 3*a^4*c^2*x*z^2 + a^2*b^2*c^2*x*z^2 -      2*b^4*c^2*x*z^2 - 3*a^2*c^4*x*z^2 + c^6*x*z^2 - a^6*y*z^2 - a^4*b^2*y*z^2 + a^2*b^4*y*z^2 +      b^6*y*z^2 + 2*a^4*c^2*y*z^2 - a^2*b^2*c^2*y*z^2 - 3*b^4*c^2*y*z^2 + 3*b^2*c^4*y*z^2 - c^6*y*z^2 ");
		exp("K026","-(a^4*c^2*x^2*y) + 2*a^2*b^2*c^2*x^2*y - b^4*c^2*x^2*y + a^2*c^4*x^2*y + b^2*c^4*x^2*y +      a^4*c^2*x*y^2 - 2*a^2*b^2*c^2*x*y^2 + b^4*c^2*x*y^2 - a^2*c^4*x*y^2 - b^2*c^4*x*y^2 +      a^4*b^2*x^2*z - a^2*b^4*x^2*z - 2*a^2*b^2*c^2*x^2*z - b^4*c^2*x^2*z + b^2*c^4*x^2*z +      2*a^4*b^2*x*y*z - 2*a^2*b^4*x*y*z - 2*a^4*c^2*x*y*z + 2*b^4*c^2*x*y*z + 2*a^2*c^4*x*y*z -      2*b^2*c^4*x*y*z + a^4*b^2*y^2*z - a^2*b^4*y^2*z + a^4*c^2*y^2*z + 2*a^2*b^2*c^2*y^2*z -      a^2*c^4*y^2*z - a^4*b^2*x*z^2 + a^2*b^4*x*z^2 + 2*a^2*b^2*c^2*x*z^2 + b^4*c^2*x*z^2 -      b^2*c^4*x*z^2 - a^4*b^2*y*z^2 + a^2*b^4*y*z^2 - a^4*c^2*y*z^2 - 2*a^2*b^2*c^2*y*z^2 +      a^2*c^4*y*z^2");
		exp("K027","a^6*c^2*x^2*y - 2*a^4*b^2*c^2*x^2*y + a^2*b^4*c^2*x^2*y - 2*a^4*c^4*x^2*y + 2*a^2*b^2*c^4*x^2*y +      a^2*c^6*x^2*y + a^4*b^2*c^2*x*y^2 - 2*a^2*b^4*c^2*x*y^2 + b^6*c^2*x*y^2 + 2*a^2*b^2*c^4*x*y^2 -      2*b^4*c^4*x*y^2 + b^2*c^6*x*y^2 + a^6*b^2*x^2*z - 2*a^4*b^4*x^2*z + a^2*b^6*x^2*z -      2*a^4*b^2*c^2*x^2*z + 2*a^2*b^4*c^2*x^2*z + a^2*b^2*c^4*x^2*z + 2*a^6*b^2*x*y*z - 4*a^4*b^4*x*y*z +      2*a^2*b^6*x*y*z + 2*a^6*c^2*x*y*z - 2*a^4*b^2*c^2*x*y*z - 2*a^2*b^4*c^2*x*y*z + 2*b^6*c^2*x*y*z -      4*a^4*c^4*x*y*z - 2*a^2*b^2*c^4*x*y*z - 4*b^4*c^4*x*y*z + 2*a^2*c^6*x*y*z + 2*b^2*c^6*x*y*z +      a^6*b^2*y^2*z - 2*a^4*b^4*y^2*z + a^2*b^6*y^2*z + 2*a^4*b^2*c^2*y^2*z - 2*a^2*b^4*c^2*y^2*z +      a^2*b^2*c^4*y^2*z + a^4*b^2*c^2*x*z^2 + 2*a^2*b^4*c^2*x*z^2 + b^6*c^2*x*z^2 - 2*a^2*b^2*c^4*x*z^2 -      2*b^4*c^4*x*z^2 + b^2*c^6*x*z^2 + a^6*c^2*y*z^2 + 2*a^4*b^2*c^2*y*z^2 + a^2*b^4*c^2*y*z^2 -      2*a^4*c^4*y*z^2 - 2*a^2*b^2*c^4*y*z^2 + a^2*c^6*y*z^2");
		exp("K028","a^4*c^2*x^2*y - a^2*b^2*c^2*x^2*y - a^2*c^4*x^2*y + a^2*b^2*c^2*x*y^2 - b^4*c^2*x*y^2 +      b^2*c^4*x*y^2 - a^4*b^2*x^2*z + a^2*b^4*x^2*z + a^2*b^2*c^2*x^2*z - 2*a^4*b^2*x*y*z +      2*a^2*b^4*x*y*z + 2*a^4*c^2*x*y*z - 2*b^4*c^2*x*y*z - 2*a^2*c^4*x*y*z + 2*b^2*c^4*x*y*z -      a^4*b^2*y^2*z + a^2*b^4*y^2*z - a^2*b^2*c^2*y^2*z - a^2*b^2*c^2*x*z^2 - b^4*c^2*x*z^2 +      b^2*c^4*x*z^2 + a^4*c^2*y*z^2 + a^2*b^2*c^2*y*z^2 - a^2*c^4*y*z^2");
		exp("K029","(-b^2)*c*x^2*z*Cos((1/3)*ArcCos((a^2 + b^2 - c^2)/(2*a*b))) +      a^2*c*y^2*z*Cos((1/3)*ArcCos((a^2 + b^2 - c^2)/(2*a*b))) +      b*c^2*x^2*y*Cos((1/3)*ArcCos((a^2 - b^2 + c^2)/(2*a*c))) -      a^2*b*y*z^2*Cos((1/3)*ArcCos((a^2 - b^2 + c^2)/(2*a*c))) -      2*a*c^2*x*y^2*Cos((1/3)*ArcCos((a^2 + b^2 - c^2)/(2*a*b)))*      Cos((1/3)*ArcCos((a^2 - b^2 + c^2)/(2*a*c))) + 2*a*b^2*x*z^2*      Cos((1/3)*ArcCos((a^2 + b^2 - c^2)/(2*a*b)))*Cos((1/3)*ArcCos((a^2 - b^2 + c^2)/(2*a*c))) -      a*c^2*x*y^2*Cos((1/3)*ArcCos((-a^2 + b^2 + c^2)/(2*b*c))) +      a*b^2*x*z^2*Cos((1/3)*ArcCos((-a^2 + b^2 + c^2)/(2*b*c))) +      2*b*c^2*x^2*y*Cos((1/3)*ArcCos((a^2 + b^2 - c^2)/(2*a*b)))*      Cos((1/3)*ArcCos((-a^2 + b^2 + c^2)/(2*b*c))) -      2*a^2*b*y*z^2*Cos((1/3)*ArcCos((a^2 + b^2 - c^2)/(2*a*b)))*      Cos((1/3)*ArcCos((-a^2 + b^2 + c^2)/(2*b*c))) -      2*b^2*c*x^2*z*Cos((1/3)*ArcCos((a^2 - b^2 + c^2)/(2*a*c)))*      Cos((1/3)*ArcCos((-a^2 + b^2 + c^2)/(2*b*c))) +      2*a^2*c*y^2*z*Cos((1/3)*ArcCos((a^2 - b^2 + c^2)/(2*a*c)))*      Cos((1/3)*ArcCos((-a^2 + b^2 + c^2)/(2*b*c)))");
		exp("K030","(-b^2)*c*x^2*z*Cos((1/3)*(2*Pi + ArcCos((a^2 + b^2 - c^2)/(2*a*b)))) +      a^2*c*y^2*z*Cos((1/3)*(2*Pi + ArcCos((a^2 + b^2 - c^2)/(2*a*b)))) +      b*c^2*x^2*y*Cos((1/3)*(2*Pi + ArcCos((a^2 - b^2 + c^2)/(2*a*c)))) -      a^2*b*y*z^2*Cos((1/3)*(2*Pi + ArcCos((a^2 - b^2 + c^2)/(2*a*c)))) -      2*a*c^2*x*y^2*Cos((1/3)*(2*Pi + ArcCos((a^2 + b^2 - c^2)/(2*a*b))))*      Cos((1/3)*(2*Pi + ArcCos((a^2 - b^2 + c^2)/(2*a*c)))) +      2*a*b^2*x*z^2*Cos((1/3)*(2*Pi + ArcCos((a^2 + b^2 - c^2)/(2*a*b))))*      Cos((1/3)*(2*Pi + ArcCos((a^2 - b^2 + c^2)/(2*a*c)))) -      a*c^2*x*y^2*Cos((1/3)*(2*Pi + ArcCos((-a^2 + b^2 + c^2)/(2*b*c)))) +      a*b^2*x*z^2*Cos((1/3)*(2*Pi + ArcCos((-a^2 + b^2 + c^2)/(2*b*c)))) +      2*b*c^2*x^2*y*Cos((1/3)*(2*Pi + ArcCos((a^2 + b^2 - c^2)/(2*a*b))))*      Cos((1/3)*(2*Pi + ArcCos((-a^2 + b^2 + c^2)/(2*b*c)))) -      2*a^2*b*y*z^2*Cos((1/3)*(2*Pi + ArcCos((a^2 + b^2 - c^2)/(2*a*b))))*      Cos((1/3)*(2*Pi + ArcCos((-a^2 + b^2 + c^2)/(2*b*c)))) -      2*b^2*c*x^2*z*Cos((1/3)*(2*Pi + ArcCos((a^2 - b^2 + c^2)/(2*a*c))))*      Cos((1/3)*(2*Pi + ArcCos((-a^2 + b^2 + c^2)/(2*b*c)))) +      2*a^2*c*y^2*z*Cos((1/3)*(2*Pi + ArcCos((a^2 - b^2 + c^2)/(2*a*c))))*      Cos((1/3)*(2*Pi + ArcCos((-a^2 + b^2 + c^2)/(2*b*c))))");
		exp("K031","(-b^2)*c*x^2*z*Cos((1/3)*(-2*Pi + ArcCos((a^2 + b^2 - c^2)/(2*a*b)))) +      a^2*c*y^2*z*Cos((1/3)*(-2*Pi + ArcCos((a^2 + b^2 - c^2)/(2*a*b)))) +      b*c^2*x^2*y*Cos((1/3)*(-2*Pi + ArcCos((a^2 - b^2 + c^2)/(2*a*c)))) -      a^2*b*y*z^2*Cos((1/3)*(-2*Pi + ArcCos((a^2 - b^2 + c^2)/(2*a*c)))) -      2*a*c^2*x*y^2*Cos((1/3)*(-2*Pi + ArcCos((a^2 + b^2 - c^2)/(2*a*b))))*      Cos((1/3)*(-2*Pi + ArcCos((a^2 - b^2 + c^2)/(2*a*c)))) +      2*a*b^2*x*z^2*Cos((1/3)*(-2*Pi + ArcCos((a^2 + b^2 - c^2)/(2*a*b))))*      Cos((1/3)*(-2*Pi + ArcCos((a^2 - b^2 + c^2)/(2*a*c)))) -      a*c^2*x*y^2*Cos((1/3)*(-2*Pi + ArcCos((-a^2 + b^2 + c^2)/(2*b*c)))) +      a*b^2*x*z^2*Cos((1/3)*(-2*Pi + ArcCos((-a^2 + b^2 + c^2)/(2*b*c)))) +      2*b*c^2*x^2*y*Cos((1/3)*(-2*Pi + ArcCos((a^2 + b^2 - c^2)/(2*a*b))))*      Cos((1/3)*(-2*Pi + ArcCos((-a^2 + b^2 + c^2)/(2*b*c)))) -      2*a^2*b*y*z^2*Cos((1/3)*(-2*Pi + ArcCos((a^2 + b^2 - c^2)/(2*a*b))))*      Cos((1/3)*(-2*Pi + ArcCos((-a^2 + b^2 + c^2)/(2*b*c)))) -      2*b^2*c*x^2*z*Cos((1/3)*(-2*Pi + ArcCos((a^2 - b^2 + c^2)/(2*a*c))))*      Cos((1/3)*(-2*Pi + ArcCos((-a^2 + b^2 + c^2)/(2*b*c)))) +      2*a^2*c*y^2*z*Cos((1/3)*(-2*Pi + ArcCos((a^2 - b^2 + c^2)/(2*a*c))))*      Cos((1/3)*(-2*Pi + ArcCos((-a^2 + b^2 + c^2)/(2*b*c))))");
		exp("K032","a^6*x^2*y - 3*a^4*b^2*x^2*y + 3*a^2*b^4*x^2*y - b^6*x^2*y + 3*a^4*c^2*x^2*y +      6*a^2*b^2*c^2*x^2*y - 9*b^4*c^2*x^2*y - 9*a^2*c^4*x^2*y + 5*b^2*c^4*x^2*y + 5*c^6*x^2*y +      a^6*x*y^2 - 3*a^4*b^2*x*y^2 + 3*a^2*b^4*x*y^2 - b^6*x*y^2 + 9*a^4*c^2*x*y^2 - 6*a^2*b^2*c^2*x*y^2 -      3*b^4*c^2*x*y^2 - 5*a^2*c^4*x*y^2 + 9*b^2*c^4*x*y^2 - 5*c^6*x*y^2 - a^6*x^2*z - 3*a^4*b^2*x^2*z +      9*a^2*b^4*x^2*z - 5*b^6*x^2*z + 3*a^4*c^2*x^2*z - 6*a^2*b^2*c^2*x^2*z - 5*b^4*c^2*x^2*z -      3*a^2*c^4*x^2*z + 9*b^2*c^4*x^2*z + c^6*x^2*z + 5*a^6*y^2*z - 9*a^4*b^2*y^2*z + 3*a^2*b^4*y^2*z +      b^6*y^2*z + 5*a^4*c^2*y^2*z + 6*a^2*b^2*c^2*y^2*z - 3*b^4*c^2*y^2*z - 9*a^2*c^4*y^2*z +      3*b^2*c^4*y^2*z - c^6*y^2*z - a^6*x*z^2 - 9*a^4*b^2*x*z^2 + 5*a^2*b^4*x*z^2 + 5*b^6*x*z^2 +      3*a^4*c^2*x*z^2 + 6*a^2*b^2*c^2*x*z^2 - 9*b^4*c^2*x*z^2 - 3*a^2*c^4*x*z^2 + 3*b^2*c^4*x*z^2 +      c^6*x*z^2 - 5*a^6*y*z^2 - 5*a^4*b^2*y*z^2 + 9*a^2*b^4*y*z^2 + b^6*y*z^2 + 9*a^4*c^2*y*z^2 -      6*a^2*b^2*c^2*y*z^2 - 3*b^4*c^2*y*z^2 - 3*a^2*c^4*y*z^2 + 3*b^2*c^4*y*z^2 - c^6*y*z^2");
		exp("K033","a^2*c*x^2*y - b^2*c*x^2*y + a*c^2*x^2*y + b*c^2*x^2*y + a^2*c*x*y^2 - b^2*c*x*y^2 - a*c^2*x*y^2 -      b*c^2*x*y^2 - a^2*b*x^2*z - a*b^2*x^2*z - b^2*c*x^2*z + b*c^2*x^2*z + a^2*b*y^2*z + a*b^2*y^2*z +      a^2*c*y^2*z - a*c^2*y^2*z - a^2*b*x*z^2 + a*b^2*x*z^2 + b^2*c*x*z^2 + b*c^2*x*z^2 - a^2*b*y*z^2 +      a*b^2*y*z^2 - a^2*c*y*z^2 - a*c^2*y*z^2");
		exp("K034","a*c*x^2*y - b*c*x*y^2 - a*b*x^2*z + a*b*y^2*z + b*c*x*z^2 - a*c*y*z^2");
		exp("K035","-(a^2*b^2*c^2*x^2*y) + b^4*c^2*x^2*y + a^2*c^4*x^2*y - b^2*c^4*x^2*y - a^4*c^2*x*y^2 +      a^2*b^2*c^2*x*y^2 + a^2*c^4*x*y^2 - b^2*c^4*x*y^2 - a^2*b^4*x^2*z + a^2*b^2*c^2*x^2*z +      b^4*c^2*x^2*z - b^2*c^4*x^2*z + a^4*b^2*y^2*z - a^4*c^2*y^2*z - a^2*b^2*c^2*y^2*z + a^2*c^4*y^2*z +      a^4*b^2*x*z^2 - a^2*b^4*x*z^2 - a^2*b^2*c^2*x*z^2 + b^4*c^2*x*z^2 + a^4*b^2*y*z^2 - a^2*b^4*y*z^2 -      a^4*c^2*y*z^2 + a^2*b^2*c^2*y*z^2");
		exp("K036","a^20*x^2*y - 5*a^18*b^2*x^2*y + 13*a^16*b^4*x^2*y - 27*a^14*b^6*x^2*y + 45*a^12*b^8*x^2*y -      53*a^10*b^10*x^2*y + 45*a^8*b^12*x^2*y - 33*a^6*b^14*x^2*y + 22*a^4*b^16*x^2*y -      10*a^2*b^18*x^2*y + 2*b^20*x^2*y - 3*a^18*c^2*x^2*y + 9*a^16*b^2*c^2*x^2*y - 8*a^14*b^4*c^2*x^2*y +      3*a^12*b^6*c^2*x^2*y - 5*a^10*b^8*c^2*x^2*y - 3*a^8*b^10*c^2*x^2*y + 30*a^6*b^12*c^2*x^2*y -      43*a^4*b^14*c^2*x^2*y + 26*a^2*b^16*c^2*x^2*y - 6*b^18*c^2*x^2*y + 6*a^16*c^4*x^2*y -      8*a^14*b^2*c^4*x^2*y - 18*a^12*b^4*c^4*x^2*y + 40*a^10*b^6*c^4*x^2*y - 20*a^8*b^8*c^4*x^2*y -      13*a^6*b^10*c^4*x^2*y + 30*a^4*b^12*c^4*x^2*y - 25*a^2*b^14*c^4*x^2*y + 8*b^16*c^4*x^2*y -      13*a^14*c^6*x^2*y + 22*a^12*b^2*c^6*x^2*y + 15*a^10*b^4*c^6*x^2*y - 40*a^8*b^6*c^6*x^2*y +      10*a^6*b^8*c^6*x^2*y - 3*a^4*b^10*c^6*x^2*y + 20*a^2*b^12*c^6*x^2*y - 11*b^14*c^6*x^2*y +      18*a^12*c^8*x^2*y - 40*a^10*b^2*c^8*x^2*y + 10*a^8*b^4*c^8*x^2*y + 30*a^6*b^6*c^8*x^2*y -      5*a^4*b^8*c^8*x^2*y - 28*a^2*b^10*c^8*x^2*y + 15*b^12*c^8*x^2*y - 13*a^10*c^10*x^2*y +      30*a^8*b^2*c^10*x^2*y - 20*a^6*b^4*c^10*x^2*y - 10*a^4*b^6*c^10*x^2*y + 25*a^2*b^8*c^10*x^2*y -      12*b^10*c^10*x^2*y + 6*a^8*c^12*x^2*y - 9*a^6*b^2*c^12*x^2*y + 6*a^4*b^4*c^12*x^2*y -      9*a^2*b^6*c^12*x^2*y + 6*b^8*c^12*x^2*y - 3*a^6*c^14*x^2*y + 3*a^4*b^2*c^14*x^2*y +      3*a^2*b^4*c^14*x^2*y - 3*b^6*c^14*x^2*y + a^4*c^16*x^2*y - 2*a^2*b^2*c^16*x^2*y + b^4*c^16*x^2*y +      2*a^20*x*y^2 - 10*a^18*b^2*x*y^2 + 22*a^16*b^4*x*y^2 - 33*a^14*b^6*x*y^2 + 45*a^12*b^8*x*y^2 -      53*a^10*b^10*x*y^2 + 45*a^8*b^12*x*y^2 - 27*a^6*b^14*x*y^2 + 13*a^4*b^16*x*y^2 - 5*a^2*b^18*x*y^2 +      b^20*x*y^2 - 6*a^18*c^2*x*y^2 + 26*a^16*b^2*c^2*x*y^2 - 43*a^14*b^4*c^2*x*y^2 +      30*a^12*b^6*c^2*x*y^2 - 3*a^10*b^8*c^2*x*y^2 - 5*a^8*b^10*c^2*x*y^2 + 3*a^6*b^12*c^2*x*y^2 -      8*a^4*b^14*c^2*x*y^2 + 9*a^2*b^16*c^2*x*y^2 - 3*b^18*c^2*x*y^2 + 8*a^16*c^4*x*y^2 -      25*a^14*b^2*c^4*x*y^2 + 30*a^12*b^4*c^4*x*y^2 - 13*a^10*b^6*c^4*x*y^2 - 20*a^8*b^8*c^4*x*y^2 +      40*a^6*b^10*c^4*x*y^2 - 18*a^4*b^12*c^4*x*y^2 - 8*a^2*b^14*c^4*x*y^2 + 6*b^16*c^4*x*y^2 -      11*a^14*c^6*x*y^2 + 20*a^12*b^2*c^6*x*y^2 - 3*a^10*b^4*c^6*x*y^2 + 10*a^8*b^6*c^6*x*y^2 -      40*a^6*b^8*c^6*x*y^2 + 15*a^4*b^10*c^6*x*y^2 + 22*a^2*b^12*c^6*x*y^2 - 13*b^14*c^6*x*y^2 +      15*a^12*c^8*x*y^2 - 28*a^10*b^2*c^8*x*y^2 - 5*a^8*b^4*c^8*x*y^2 + 30*a^6*b^6*c^8*x*y^2 +      10*a^4*b^8*c^8*x*y^2 - 40*a^2*b^10*c^8*x*y^2 + 18*b^12*c^8*x*y^2 - 12*a^10*c^10*x*y^2 +      25*a^8*b^2*c^10*x*y^2 - 10*a^6*b^4*c^10*x*y^2 - 20*a^4*b^6*c^10*x*y^2 + 30*a^2*b^8*c^10*x*y^2 -      13*b^10*c^10*x*y^2 + 6*a^8*c^12*x*y^2 - 9*a^6*b^2*c^12*x*y^2 + 6*a^4*b^4*c^12*x*y^2 -      9*a^2*b^6*c^12*x*y^2 + 6*b^8*c^12*x*y^2 - 3*a^6*c^14*x*y^2 + 3*a^4*b^2*c^14*x*y^2 +      3*a^2*b^4*c^14*x*y^2 - 3*b^6*c^14*x*y^2 + a^4*c^16*x*y^2 - 2*a^2*b^2*c^16*x*y^2 + b^4*c^16*x*y^2 +      a^20*x^2*z - 3*a^18*b^2*x^2*z + 6*a^16*b^4*x^2*z - 13*a^14*b^6*x^2*z + 18*a^12*b^8*x^2*z -      13*a^10*b^10*x^2*z + 6*a^8*b^12*x^2*z - 3*a^6*b^14*x^2*z + a^4*b^16*x^2*z - 5*a^18*c^2*x^2*z +      9*a^16*b^2*c^2*x^2*z - 8*a^14*b^4*c^2*x^2*z + 22*a^12*b^6*c^2*x^2*z - 40*a^10*b^8*c^2*x^2*z +      30*a^8*b^10*c^2*x^2*z - 9*a^6*b^12*c^2*x^2*z + 3*a^4*b^14*c^2*x^2*z - 2*a^2*b^16*c^2*x^2*z +      13*a^16*c^4*x^2*z - 8*a^14*b^2*c^4*x^2*z - 18*a^12*b^4*c^4*x^2*z + 15*a^10*b^6*c^4*x^2*z +      10*a^8*b^8*c^4*x^2*z - 20*a^6*b^10*c^4*x^2*z + 6*a^4*b^12*c^4*x^2*z + 3*a^2*b^14*c^4*x^2*z +      b^16*c^4*x^2*z - 27*a^14*c^6*x^2*z + 3*a^12*b^2*c^6*x^2*z + 40*a^10*b^4*c^6*x^2*z -      40*a^8*b^6*c^6*x^2*z + 30*a^6*b^8*c^6*x^2*z - 10*a^4*b^10*c^6*x^2*z - 9*a^2*b^12*c^6*x^2*z -      3*b^14*c^6*x^2*z + 45*a^12*c^8*x^2*z - 5*a^10*b^2*c^8*x^2*z - 20*a^8*b^4*c^8*x^2*z +      10*a^6*b^6*c^8*x^2*z - 5*a^4*b^8*c^8*x^2*z + 25*a^2*b^10*c^8*x^2*z + 6*b^12*c^8*x^2*z -      53*a^10*c^10*x^2*z - 3*a^8*b^2*c^10*x^2*z - 13*a^6*b^4*c^10*x^2*z - 3*a^4*b^6*c^10*x^2*z -      28*a^2*b^8*c^10*x^2*z - 12*b^10*c^10*x^2*z + 45*a^8*c^12*x^2*z + 30*a^6*b^2*c^12*x^2*z +      30*a^4*b^4*c^12*x^2*z + 20*a^2*b^6*c^12*x^2*z + 15*b^8*c^12*x^2*z - 33*a^6*c^14*x^2*z -      43*a^4*b^2*c^14*x^2*z - 25*a^2*b^4*c^14*x^2*z - 11*b^6*c^14*x^2*z + 22*a^4*c^16*x^2*z +      26*a^2*b^2*c^16*x^2*z + 8*b^4*c^16*x^2*z - 10*a^2*c^18*x^2*z - 6*b^2*c^18*x^2*z + 2*c^20*x^2*z +      4*a^20*x*y*z - 16*a^18*b^2*x*y*z + 30*a^16*b^4*x*y*z - 44*a^14*b^6*x*y*z + 62*a^12*b^8*x*y*z -      72*a^10*b^10*x*y*z + 62*a^8*b^12*x*y*z - 44*a^6*b^14*x*y*z + 30*a^4*b^16*x*y*z -      16*a^2*b^18*x*y*z + 4*b^20*x*y*z - 16*a^18*c^2*x*y*z + 52*a^16*b^2*c^2*x*y*z -      68*a^14*b^4*c^2*x*y*z + 42*a^12*b^6*c^2*x*y*z - 10*a^10*b^8*c^2*x*y*z - 10*a^8*b^10*c^2*x*y*z +      42*a^6*b^12*c^2*x*y*z - 68*a^4*b^14*c^2*x*y*z + 52*a^2*b^16*c^2*x*y*z - 16*b^18*c^2*x*y*z +      30*a^16*c^4*x*y*z - 68*a^14*b^2*c^4*x*y*z + 72*a^12*b^4*c^4*x*y*z - 30*a^10*b^6*c^4*x*y*z -      4*a^8*b^8*c^4*x*y*z - 30*a^6*b^10*c^4*x*y*z + 72*a^4*b^12*c^4*x*y*z - 68*a^2*b^14*c^4*x*y*z +      30*b^16*c^4*x*y*z - 44*a^14*c^6*x*y*z + 42*a^12*b^2*c^6*x*y*z - 30*a^10*b^4*c^6*x*y*z +      16*a^8*b^6*c^6*x*y*z + 16*a^6*b^8*c^6*x*y*z - 30*a^4*b^10*c^6*x*y*z + 42*a^2*b^12*c^6*x*y*z -      44*b^14*c^6*x*y*z + 62*a^12*c^8*x*y*z - 10*a^10*b^2*c^8*x*y*z - 4*a^8*b^4*c^8*x*y*z +      16*a^6*b^6*c^8*x*y*z - 4*a^4*b^8*c^8*x*y*z - 10*a^2*b^10*c^8*x*y*z + 62*b^12*c^8*x*y*z -      72*a^10*c^10*x*y*z - 10*a^8*b^2*c^10*x*y*z - 30*a^6*b^4*c^10*x*y*z - 30*a^4*b^6*c^10*x*y*z -      10*a^2*b^8*c^10*x*y*z - 72*b^10*c^10*x*y*z + 62*a^8*c^12*x*y*z + 42*a^6*b^2*c^12*x*y*z +      72*a^4*b^4*c^12*x*y*z + 42*a^2*b^6*c^12*x*y*z + 62*b^8*c^12*x*y*z - 44*a^6*c^14*x*y*z -      68*a^4*b^2*c^14*x*y*z - 68*a^2*b^4*c^14*x*y*z - 44*b^6*c^14*x*y*z + 30*a^4*c^16*x*y*z +      52*a^2*b^2*c^16*x*y*z + 30*b^4*c^16*x*y*z - 16*a^2*c^18*x*y*z - 16*b^2*c^18*x*y*z + 4*c^20*x*y*z +      a^16*b^4*y^2*z - 3*a^14*b^6*y^2*z + 6*a^12*b^8*y^2*z - 13*a^10*b^10*y^2*z + 18*a^8*b^12*y^2*z -      13*a^6*b^14*y^2*z + 6*a^4*b^16*y^2*z - 3*a^2*b^18*y^2*z + b^20*y^2*z - 2*a^16*b^2*c^2*y^2*z +      3*a^14*b^4*c^2*y^2*z - 9*a^12*b^6*c^2*y^2*z + 30*a^10*b^8*c^2*y^2*z - 40*a^8*b^10*c^2*y^2*z +      22*a^6*b^12*c^2*y^2*z - 8*a^4*b^14*c^2*y^2*z + 9*a^2*b^16*c^2*y^2*z - 5*b^18*c^2*y^2*z +      a^16*c^4*y^2*z + 3*a^14*b^2*c^4*y^2*z + 6*a^12*b^4*c^4*y^2*z - 20*a^10*b^6*c^4*y^2*z +      10*a^8*b^8*c^4*y^2*z + 15*a^6*b^10*c^4*y^2*z - 18*a^4*b^12*c^4*y^2*z - 8*a^2*b^14*c^4*y^2*z +      13*b^16*c^4*y^2*z - 3*a^14*c^6*y^2*z - 9*a^12*b^2*c^6*y^2*z - 10*a^10*b^4*c^6*y^2*z +      30*a^8*b^6*c^6*y^2*z - 40*a^6*b^8*c^6*y^2*z + 40*a^4*b^10*c^6*y^2*z + 3*a^2*b^12*c^6*y^2*z -      27*b^14*c^6*y^2*z + 6*a^12*c^8*y^2*z + 25*a^10*b^2*c^8*y^2*z - 5*a^8*b^4*c^8*y^2*z +      10*a^6*b^6*c^8*y^2*z - 20*a^4*b^8*c^8*y^2*z - 5*a^2*b^10*c^8*y^2*z + 45*b^12*c^8*y^2*z -      12*a^10*c^10*y^2*z - 28*a^8*b^2*c^10*y^2*z - 3*a^6*b^4*c^10*y^2*z - 13*a^4*b^6*c^10*y^2*z -      3*a^2*b^8*c^10*y^2*z - 53*b^10*c^10*y^2*z + 15*a^8*c^12*y^2*z + 20*a^6*b^2*c^12*y^2*z +      30*a^4*b^4*c^12*y^2*z + 30*a^2*b^6*c^12*y^2*z + 45*b^8*c^12*y^2*z - 11*a^6*c^14*y^2*z -      25*a^4*b^2*c^14*y^2*z - 43*a^2*b^4*c^14*y^2*z - 33*b^6*c^14*y^2*z + 8*a^4*c^16*y^2*z +      26*a^2*b^2*c^16*y^2*z + 22*b^4*c^16*y^2*z - 6*a^2*c^18*y^2*z - 10*b^2*c^18*y^2*z + 2*c^20*y^2*z +      2*a^20*x*z^2 - 6*a^18*b^2*x*z^2 + 8*a^16*b^4*x*z^2 - 11*a^14*b^6*x*z^2 + 15*a^12*b^8*x*z^2 -      12*a^10*b^10*x*z^2 + 6*a^8*b^12*x*z^2 - 3*a^6*b^14*x*z^2 + a^4*b^16*x*z^2 - 10*a^18*c^2*x*z^2 +      26*a^16*b^2*c^2*x*z^2 - 25*a^14*b^4*c^2*x*z^2 + 20*a^12*b^6*c^2*x*z^2 - 28*a^10*b^8*c^2*x*z^2 +      25*a^8*b^10*c^2*x*z^2 - 9*a^6*b^12*c^2*x*z^2 + 3*a^4*b^14*c^2*x*z^2 - 2*a^2*b^16*c^2*x*z^2 +      22*a^16*c^4*x*z^2 - 43*a^14*b^2*c^4*x*z^2 + 30*a^12*b^4*c^4*x*z^2 - 3*a^10*b^6*c^4*x*z^2 -      5*a^8*b^8*c^4*x*z^2 - 10*a^6*b^10*c^4*x*z^2 + 6*a^4*b^12*c^4*x*z^2 + 3*a^2*b^14*c^4*x*z^2 +      b^16*c^4*x*z^2 - 33*a^14*c^6*x*z^2 + 30*a^12*b^2*c^6*x*z^2 - 13*a^10*b^4*c^6*x*z^2 +      10*a^8*b^6*c^6*x*z^2 + 30*a^6*b^8*c^6*x*z^2 - 20*a^4*b^10*c^6*x*z^2 - 9*a^2*b^12*c^6*x*z^2 -      3*b^14*c^6*x*z^2 + 45*a^12*c^8*x*z^2 - 3*a^10*b^2*c^8*x*z^2 - 20*a^8*b^4*c^8*x*z^2 -      40*a^6*b^6*c^8*x*z^2 + 10*a^4*b^8*c^8*x*z^2 + 30*a^2*b^10*c^8*x*z^2 + 6*b^12*c^8*x*z^2 -      53*a^10*c^10*x*z^2 - 5*a^8*b^2*c^10*x*z^2 + 40*a^6*b^4*c^10*x*z^2 + 15*a^4*b^6*c^10*x*z^2 -      40*a^2*b^8*c^10*x*z^2 - 13*b^10*c^10*x*z^2 + 45*a^8*c^12*x*z^2 + 3*a^6*b^2*c^12*x*z^2 -      18*a^4*b^4*c^12*x*z^2 + 22*a^2*b^6*c^12*x*z^2 + 18*b^8*c^12*x*z^2 - 27*a^6*c^14*x*z^2 -      8*a^4*b^2*c^14*x*z^2 - 8*a^2*b^4*c^14*x*z^2 - 13*b^6*c^14*x*z^2 + 13*a^4*c^16*x*z^2 +      9*a^2*b^2*c^16*x*z^2 + 6*b^4*c^16*x*z^2 - 5*a^2*c^18*x*z^2 - 3*b^2*c^18*x*z^2 + c^20*x*z^2 +      a^16*b^4*y*z^2 - 3*a^14*b^6*y*z^2 + 6*a^12*b^8*y*z^2 - 12*a^10*b^10*y*z^2 + 15*a^8*b^12*y*z^2 -      11*a^6*b^14*y*z^2 + 8*a^4*b^16*y*z^2 - 6*a^2*b^18*y*z^2 + 2*b^20*y*z^2 - 2*a^16*b^2*c^2*y*z^2 +      3*a^14*b^4*c^2*y*z^2 - 9*a^12*b^6*c^2*y*z^2 + 25*a^10*b^8*c^2*y*z^2 - 28*a^8*b^10*c^2*y*z^2 +      20*a^6*b^12*c^2*y*z^2 - 25*a^4*b^14*c^2*y*z^2 + 26*a^2*b^16*c^2*y*z^2 - 10*b^18*c^2*y*z^2 +      a^16*c^4*y*z^2 + 3*a^14*b^2*c^4*y*z^2 + 6*a^12*b^4*c^4*y*z^2 - 10*a^10*b^6*c^4*y*z^2 -      5*a^8*b^8*c^4*y*z^2 - 3*a^6*b^10*c^4*y*z^2 + 30*a^4*b^12*c^4*y*z^2 - 43*a^2*b^14*c^4*y*z^2 +      22*b^16*c^4*y*z^2 - 3*a^14*c^6*y*z^2 - 9*a^12*b^2*c^6*y*z^2 - 20*a^10*b^4*c^6*y*z^2 +      30*a^8*b^6*c^6*y*z^2 + 10*a^6*b^8*c^6*y*z^2 - 13*a^4*b^10*c^6*y*z^2 + 30*a^2*b^12*c^6*y*z^2 -      33*b^14*c^6*y*z^2 + 6*a^12*c^8*y*z^2 + 30*a^10*b^2*c^8*y*z^2 + 10*a^8*b^4*c^8*y*z^2 -      40*a^6*b^6*c^8*y*z^2 - 20*a^4*b^8*c^8*y*z^2 - 3*a^2*b^10*c^8*y*z^2 + 45*b^12*c^8*y*z^2 -      13*a^10*c^10*y*z^2 - 40*a^8*b^2*c^10*y*z^2 + 15*a^6*b^4*c^10*y*z^2 + 40*a^4*b^6*c^10*y*z^2 -      5*a^2*b^8*c^10*y*z^2 - 53*b^10*c^10*y*z^2 + 18*a^8*c^12*y*z^2 + 22*a^6*b^2*c^12*y*z^2 -      18*a^4*b^4*c^12*y*z^2 + 3*a^2*b^6*c^12*y*z^2 + 45*b^8*c^12*y*z^2 - 13*a^6*c^14*y*z^2 -      8*a^4*b^2*c^14*y*z^2 - 8*a^2*b^4*c^14*y*z^2 - 27*b^6*c^14*y*z^2 + 6*a^4*c^16*y*z^2 +      9*a^2*b^2*c^16*y*z^2 + 13*b^4*c^16*y*z^2 - 3*a^2*c^18*y*z^2 - 5*b^2*c^18*y*z^2 + c^20*y*z^2");
		exp("K037","a^12*x^2*y - 3*a^10*b^2*x^2*y + 10*a^6*b^6*x^2*y - 15*a^4*b^8*x^2*y + 9*a^2*b^10*x^2*y -      2*b^12*x^2*y - 3*a^10*c^2*x^2*y + 9*a^8*b^2*c^2*x^2*y - 12*a^6*b^4*c^2*x^2*y +      12*a^4*b^6*c^2*x^2*y - 9*a^2*b^8*c^2*x^2*y + 3*b^10*c^2*x^2*y + 3*a^8*c^4*x^2*y -      6*a^6*b^2*c^4*x^2*y + 6*a^4*b^4*c^4*x^2*y - 3*a^2*b^6*c^4*x^2*y - 2*a^6*c^6*x^2*y -      3*a^4*b^2*c^6*x^2*y + 3*a^2*b^4*c^6*x^2*y + b^6*c^6*x^2*y + 3*a^4*c^8*x^2*y + 3*a^2*b^2*c^8*x^2*y -      3*b^4*c^8*x^2*y - 3*a^2*c^10*x^2*y + c^12*x^2*y + 2*a^12*x*y^2 - 9*a^10*b^2*x*y^2 +      15*a^8*b^4*x*y^2 - 10*a^6*b^6*x*y^2 + 3*a^2*b^10*x*y^2 - b^12*x*y^2 - 3*a^10*c^2*x*y^2 +      9*a^8*b^2*c^2*x*y^2 - 12*a^6*b^4*c^2*x*y^2 + 12*a^4*b^6*c^2*x*y^2 - 9*a^2*b^8*c^2*x*y^2 +      3*b^10*c^2*x*y^2 + 3*a^6*b^2*c^4*x*y^2 - 6*a^4*b^4*c^4*x*y^2 + 6*a^2*b^6*c^4*x*y^2 -      3*b^8*c^4*x*y^2 - a^6*c^6*x*y^2 - 3*a^4*b^2*c^6*x*y^2 + 3*a^2*b^4*c^6*x*y^2 + 2*b^6*c^6*x*y^2 +      3*a^4*c^8*x*y^2 - 3*a^2*b^2*c^8*x*y^2 - 3*b^4*c^8*x*y^2 + 3*b^2*c^10*x*y^2 - c^12*x*y^2 -      a^12*x^2*z + 3*a^10*b^2*x^2*z - 3*a^8*b^4*x^2*z + 2*a^6*b^6*x^2*z - 3*a^4*b^8*x^2*z +      3*a^2*b^10*x^2*z - b^12*x^2*z + 3*a^10*c^2*x^2*z - 9*a^8*b^2*c^2*x^2*z + 6*a^6*b^4*c^2*x^2*z +      3*a^4*b^6*c^2*x^2*z - 3*a^2*b^8*c^2*x^2*z + 12*a^6*b^2*c^4*x^2*z - 6*a^4*b^4*c^4*x^2*z -      3*a^2*b^6*c^4*x^2*z + 3*b^8*c^4*x^2*z - 10*a^6*c^6*x^2*z - 12*a^4*b^2*c^6*x^2*z +      3*a^2*b^4*c^6*x^2*z - b^6*c^6*x^2*z + 15*a^4*c^8*x^2*z + 9*a^2*b^2*c^8*x^2*z - 9*a^2*c^10*x^2*z -      3*b^2*c^10*x^2*z + 2*c^12*x^2*z + a^12*y^2*z - 3*a^10*b^2*y^2*z + 3*a^8*b^4*y^2*z -      2*a^6*b^6*y^2*z + 3*a^4*b^8*y^2*z - 3*a^2*b^10*y^2*z + b^12*y^2*z + 3*a^8*b^2*c^2*y^2*z -      3*a^6*b^4*c^2*y^2*z - 6*a^4*b^6*c^2*y^2*z + 9*a^2*b^8*c^2*y^2*z - 3*b^10*c^2*y^2*z -      3*a^8*c^4*y^2*z + 3*a^6*b^2*c^4*y^2*z + 6*a^4*b^4*c^4*y^2*z - 12*a^2*b^6*c^4*y^2*z +      a^6*c^6*y^2*z - 3*a^4*b^2*c^6*y^2*z + 12*a^2*b^4*c^6*y^2*z + 10*b^6*c^6*y^2*z -      9*a^2*b^2*c^8*y^2*z - 15*b^4*c^8*y^2*z + 3*a^2*c^10*y^2*z + 9*b^2*c^10*y^2*z - 2*c^12*y^2*z -      2*a^12*x*z^2 + 3*a^10*b^2*x*z^2 + a^6*b^6*x*z^2 - 3*a^4*b^8*x*z^2 + b^12*x*z^2 + 9*a^10*c^2*x*z^2 -      9*a^8*b^2*c^2*x*z^2 - 3*a^6*b^4*c^2*x*z^2 + 3*a^4*b^6*c^2*x*z^2 + 3*a^2*b^8*c^2*x*z^2 -      3*b^10*c^2*x*z^2 - 15*a^8*c^4*x*z^2 + 12*a^6*b^2*c^4*x*z^2 + 6*a^4*b^4*c^4*x*z^2 -      3*a^2*b^6*c^4*x*z^2 + 3*b^8*c^4*x*z^2 + 10*a^6*c^6*x*z^2 - 12*a^4*b^2*c^6*x*z^2 -      6*a^2*b^4*c^6*x*z^2 - 2*b^6*c^6*x*z^2 + 9*a^2*b^2*c^8*x*z^2 + 3*b^4*c^8*x*z^2 - 3*a^2*c^10*x*z^2 -      3*b^2*c^10*x*z^2 + c^12*x*z^2 - a^12*y*z^2 + 3*a^8*b^4*y*z^2 - a^6*b^6*y*z^2 - 3*a^2*b^10*y*z^2 +      2*b^12*y*z^2 + 3*a^10*c^2*y*z^2 - 3*a^8*b^2*c^2*y*z^2 - 3*a^6*b^4*c^2*y*z^2 + 3*a^4*b^6*c^2*y*z^2 +      9*a^2*b^8*c^2*y*z^2 - 9*b^10*c^2*y*z^2 - 3*a^8*c^4*y*z^2 + 3*a^6*b^2*c^4*y*z^2 -      6*a^4*b^4*c^4*y*z^2 - 12*a^2*b^6*c^4*y*z^2 + 15*b^8*c^4*y*z^2 + 2*a^6*c^6*y*z^2 +      6*a^4*b^2*c^6*y*z^2 + 12*a^2*b^4*c^6*y*z^2 - 10*b^6*c^6*y*z^2 - 3*a^4*c^8*y*z^2 -      9*a^2*b^2*c^8*y*z^2 + 3*a^2*c^10*y*z^2 + 3*b^2*c^10*y*z^2 - c^12*y*z^2");
		exp("K038","b^4*c^2*x^3 - b^2*c^4*x^3 + 2*a^2*b^2*c^2*x^2*y - b^4*c^2*x^2*y - 2*a^2*c^4*x^2*y -      b^2*c^4*x^2*y + 2*c^6*x^2*y + a^4*c^2*x*y^2 - 2*a^2*b^2*c^2*x*y^2 + a^2*c^4*x*y^2 +      2*b^2*c^4*x*y^2 - 2*c^6*x*y^2 - a^4*c^2*y^3 + a^2*c^4*y^3 + 2*a^2*b^4*x^2*z - 2*b^6*x^2*z -      2*a^2*b^2*c^2*x^2*z + b^4*c^2*x^2*z + b^2*c^4*x^2*z + 4*a^4*b^2*x*y*z - 4*a^2*b^4*x*y*z -      4*a^4*c^2*x*y*z + 4*b^4*c^2*x*y*z + 4*a^2*c^4*x*y*z - 4*b^2*c^4*x*y*z + 2*a^6*y^2*z -      2*a^4*b^2*y^2*z - a^4*c^2*y^2*z + 2*a^2*b^2*c^2*y^2*z - a^2*c^4*y^2*z - a^4*b^2*x*z^2 -      a^2*b^4*x*z^2 + 2*b^6*x*z^2 + 2*a^2*b^2*c^2*x*z^2 - 2*b^4*c^2*x*z^2 - 2*a^6*y*z^2 + a^4*b^2*y*z^2 +      a^2*b^4*y*z^2 + 2*a^4*c^2*y*z^2 - 2*a^2*b^2*c^2*y*z^2 + a^4*b^2*z^3 - a^2*b^4*z^3");
		exp("K039","(-a^6)*b^2*c^4*x^2*y - a^4*b^4*c^4*x^2*y + a^2*b^6*c^4*x^2*y + b^8*c^4*x^2*y +      2*a^4*b^2*c^6*x^2*y - a^2*b^4*c^6*x^2*y - 3*b^6*c^6*x^2*y + 3*b^4*c^8*x^2*y - b^2*c^10*x^2*y -      a^8*c^4*x*y^2 - a^6*b^2*c^4*x*y^2 + a^4*b^4*c^4*x*y^2 + a^2*b^6*c^4*x*y^2 + 3*a^6*c^6*x*y^2 +      a^4*b^2*c^6*x*y^2 - 2*a^2*b^4*c^6*x*y^2 - 3*a^4*c^8*x*y^2 + a^2*c^10*x*y^2 + a^6*b^4*c^2*x^2*z -      2*a^4*b^6*c^2*x^2*z + b^10*c^2*x^2*z + a^4*b^4*c^4*x^2*z + a^2*b^6*c^4*x^2*z - 3*b^8*c^4*x^2*z -      a^2*b^4*c^6*x^2*z + 3*b^6*c^6*x^2*z - b^4*c^8*x^2*z - 2*a^6*b^4*c^2*x*y*z + 2*a^4*b^6*c^2*x*y*z +      2*a^6*b^2*c^4*x*y*z - 2*a^2*b^6*c^4*x*y*z - 2*a^4*b^2*c^6*x*y*z + 2*a^2*b^4*c^6*x*y*z -      a^10*c^2*y^2*z + 2*a^6*b^4*c^2*y^2*z - a^4*b^6*c^2*y^2*z + 3*a^8*c^4*y^2*z - a^6*b^2*c^4*y^2*z -      a^4*b^4*c^4*y^2*z - 3*a^6*c^6*y^2*z + a^4*b^2*c^6*y^2*z + a^4*c^8*y^2*z + a^8*b^4*x*z^2 -      3*a^6*b^6*x*z^2 + 3*a^4*b^8*x*z^2 - a^2*b^10*x*z^2 + a^6*b^4*c^2*x*z^2 - a^4*b^6*c^2*x*z^2 -      a^4*b^4*c^4*x*z^2 + 2*a^2*b^6*c^4*x*z^2 - a^2*b^4*c^6*x*z^2 + a^10*b^2*y*z^2 - 3*a^8*b^4*y*z^2 +      3*a^6*b^6*y*z^2 - a^4*b^8*y*z^2 + a^6*b^4*c^2*y*z^2 - a^4*b^6*c^2*y*z^2 - 2*a^6*b^2*c^4*y*z^2 +      a^4*b^4*c^4*y*z^2 + a^4*b^2*c^6*y*z^2");
		exp("K040","a^2*b*c^2*x^2*y - a*b^2*c^2*x^2*y + b^2*c^3*x^2*y - b*c^4*x^2*y + a^2*b*c^2*x*y^2 -      a*b^2*c^2*x*y^2 - a^2*c^3*x*y^2 + a*c^4*x*y^2 - a^2*b^2*c*x^2*z + b^4*c*x^2*z + a*b^2*c^2*x^2*z -      b^3*c^2*x^2*z - a^4*c*y^2*z + a^2*b^2*c*y^2*z + a^3*c^2*y^2*z - a^2*b*c^2*y^2*z + a^2*b^3*x*z^2 -      a*b^4*x*z^2 - a^2*b^2*c*x*z^2 + a*b^2*c^2*x*z^2 + a^4*b*y*z^2 - a^3*b^2*y*z^2 + a^2*b^2*c*y*z^2 -      a^2*b*c^2*y*z^2");
		exp("K041","a^6*x^2*y - 3*a^4*b^2*x^2*y + 3*a^2*b^4*x^2*y - b^6*x^2*y + 3*a^4*c^2*x^2*y -      2*a^2*b^2*c^2*x^2*y - b^4*c^2*x^2*y - a^2*c^4*x^2*y + 5*b^2*c^4*x^2*y - 3*c^6*x^2*y + a^6*x*y^2 -      3*a^4*b^2*x*y^2 + 3*a^2*b^4*x*y^2 - b^6*x*y^2 + a^4*c^2*x*y^2 + 2*a^2*b^2*c^2*x*y^2 -      3*b^4*c^2*x*y^2 - 5*a^2*c^4*x*y^2 + b^2*c^4*x*y^2 + 3*c^6*x*y^2 - a^6*x^2*z - 3*a^4*b^2*x^2*z +      a^2*b^4*x^2*z + 3*b^6*x^2*z + 3*a^4*c^2*x^2*z + 2*a^2*b^2*c^2*x^2*z - 5*b^4*c^2*x^2*z -      3*a^2*c^4*x^2*z + b^2*c^4*x^2*z + c^6*x^2*z - 16*a^4*b^2*x*y*z + 16*a^2*b^4*x*y*z +      16*a^4*c^2*x*y*z - 16*b^4*c^2*x*y*z - 16*a^2*c^4*x*y*z + 16*b^2*c^4*x*y*z - 3*a^6*y^2*z -      a^4*b^2*y^2*z + 3*a^2*b^4*y^2*z + b^6*y^2*z + 5*a^4*c^2*y^2*z - 2*a^2*b^2*c^2*y^2*z -      3*b^4*c^2*y^2*z - a^2*c^4*y^2*z + 3*b^2*c^4*y^2*z - c^6*y^2*z - a^6*x*z^2 - a^4*b^2*x*z^2 +      5*a^2*b^4*x*z^2 - 3*b^6*x*z^2 + 3*a^4*c^2*x*z^2 - 2*a^2*b^2*c^2*x*z^2 - b^4*c^2*x*z^2 -      3*a^2*c^4*x*z^2 + 3*b^2*c^4*x*z^2 + c^6*x*z^2 + 3*a^6*y*z^2 - 5*a^4*b^2*y*z^2 + a^2*b^4*y*z^2 +      b^6*y*z^2 + a^4*c^2*y*z^2 + 2*a^2*b^2*c^2*y*z^2 - 3*b^4*c^2*y*z^2 - 3*a^2*c^4*y*z^2 +      3*b^2*c^4*y*z^2 - c^6*y*z^2");
		exp("K042","-(a^10*c^2*x^2*y) + a^8*b^2*c^2*x^2*y - 4*a^4*b^6*c^2*x^2*y + a^2*b^8*c^2*x^2*y +      3*b^10*c^2*x^2*y + 2*a^8*c^4*x^2*y - 5*a^6*b^2*c^4*x^2*y + 6*a^4*b^4*c^4*x^2*y +      5*a^2*b^6*c^4*x^2*y - 8*b^8*c^4*x^2*y + 2*a^6*c^6*x^2*y + 3*a^4*b^2*c^6*x^2*y -      9*a^2*b^4*c^6*x^2*y + 2*b^6*c^6*x^2*y - 4*a^4*c^8*x^2*y + 4*a^2*b^2*c^8*x^2*y + 6*b^4*c^8*x^2*y -      a^2*c^10*x^2*y - 5*b^2*c^10*x^2*y + 2*c^12*x^2*y - 3*a^10*c^2*x*y^2 - a^8*b^2*c^2*x*y^2 +      4*a^6*b^4*c^2*x*y^2 - a^2*b^8*c^2*x*y^2 + b^10*c^2*x*y^2 + 8*a^8*c^4*x*y^2 - 5*a^6*b^2*c^4*x*y^2 -      6*a^4*b^4*c^4*x*y^2 + 5*a^2*b^6*c^4*x*y^2 - 2*b^8*c^4*x*y^2 - 2*a^6*c^6*x*y^2 +      9*a^4*b^2*c^6*x*y^2 - 3*a^2*b^4*c^6*x*y^2 - 2*b^6*c^6*x*y^2 - 6*a^4*c^8*x*y^2 -      4*a^2*b^2*c^8*x*y^2 + 4*b^4*c^8*x*y^2 + 5*a^2*c^10*x*y^2 + b^2*c^10*x*y^2 - 2*c^12*x*y^2 +      a^10*b^2*x^2*z - 2*a^8*b^4*x^2*z - 2*a^6*b^6*x^2*z + 4*a^4*b^8*x^2*z + a^2*b^10*x^2*z -      2*b^12*x^2*z - a^8*b^2*c^2*x^2*z + 5*a^6*b^4*c^2*x^2*z - 3*a^4*b^6*c^2*x^2*z -      4*a^2*b^8*c^2*x^2*z + 5*b^10*c^2*x^2*z - 6*a^4*b^4*c^4*x^2*z + 9*a^2*b^6*c^4*x^2*z -      6*b^8*c^4*x^2*z + 4*a^4*b^2*c^6*x^2*z - 5*a^2*b^4*c^6*x^2*z - 2*b^6*c^6*x^2*z - a^2*b^2*c^8*x^2*z +      8*b^4*c^8*x^2*z - 3*b^2*c^10*x^2*z + 2*a^12*y^2*z - a^10*b^2*y^2*z - 4*a^8*b^4*y^2*z +      2*a^6*b^6*y^2*z + 2*a^4*b^8*y^2*z - a^2*b^10*y^2*z - 5*a^10*c^2*y^2*z + 4*a^8*b^2*c^2*y^2*z +      3*a^6*b^4*c^2*y^2*z - 5*a^4*b^6*c^2*y^2*z + a^2*b^8*c^2*y^2*z + 6*a^8*c^4*y^2*z -      9*a^6*b^2*c^4*y^2*z + 6*a^4*b^4*c^4*y^2*z + 2*a^6*c^6*y^2*z + 5*a^4*b^2*c^6*y^2*z -      4*a^2*b^4*c^6*y^2*z - 8*a^4*c^8*y^2*z + a^2*b^2*c^8*y^2*z + 3*a^2*c^10*y^2*z + 3*a^10*b^2*x*z^2 -      8*a^8*b^4*x*z^2 + 2*a^6*b^6*x*z^2 + 6*a^4*b^8*x*z^2 - 5*a^2*b^10*x*z^2 + 2*b^12*x*z^2 +      a^8*b^2*c^2*x*z^2 + 5*a^6*b^4*c^2*x*z^2 - 9*a^4*b^6*c^2*x*z^2 + 4*a^2*b^8*c^2*x*z^2 -      b^10*c^2*x*z^2 - 4*a^6*b^2*c^4*x*z^2 + 6*a^4*b^4*c^4*x*z^2 + 3*a^2*b^6*c^4*x*z^2 -      4*b^8*c^4*x*z^2 - 5*a^2*b^4*c^6*x*z^2 + 2*b^6*c^6*x*z^2 + a^2*b^2*c^8*x*z^2 + 2*b^4*c^8*x*z^2 -      b^2*c^10*x*z^2 - 2*a^12*y*z^2 + 5*a^10*b^2*y*z^2 - 6*a^8*b^4*y*z^2 - 2*a^6*b^6*y*z^2 +      8*a^4*b^8*y*z^2 - 3*a^2*b^10*y*z^2 + a^10*c^2*y*z^2 - 4*a^8*b^2*c^2*y*z^2 + 9*a^6*b^4*c^2*y*z^2 -      5*a^4*b^6*c^2*y*z^2 - a^2*b^8*c^2*y*z^2 + 4*a^8*c^4*y*z^2 - 3*a^6*b^2*c^4*y*z^2 -      6*a^4*b^4*c^4*y*z^2 + 4*a^2*b^6*c^4*y*z^2 - 2*a^6*c^6*y*z^2 + 5*a^4*b^2*c^6*y*z^2 -      2*a^4*c^8*y*z^2 - a^2*b^2*c^8*y*z^2 + a^2*c^10*y*z^2");
		exp("K043","a^2*c^2*x^2*y + b^2*c^2*x^2*y - 2*c^4*x^2*y - a^2*c^2*x*y^2 - b^2*c^2*x*y^2 + 2*c^4*x*y^2 -      a^2*b^2*x^2*z + 2*b^4*x^2*z - b^2*c^2*x^2*z - 2*a^4*y^2*z + a^2*b^2*y^2*z + a^2*c^2*y^2*z +      a^2*b^2*x*z^2 - 2*b^4*x*z^2 + b^2*c^2*x*z^2 + 2*a^4*y*z^2 - a^2*b^2*y*z^2 - a^2*c^2*y*z^2");
		exp("K044","a^10*c^2*x^2*y - a^8*b^2*c^2*x^2*y - 2*a^6*b^4*c^2*x^2*y + 2*a^4*b^6*c^2*x^2*y +      a^2*b^8*c^2*x^2*y - b^10*c^2*x^2*y - 4*a^8*c^4*x^2*y + 2*a^6*b^2*c^4*x^2*y + 2*a^4*b^4*c^4*x^2*y -      2*a^2*b^6*c^4*x^2*y + 2*b^8*c^4*x^2*y + 6*a^6*c^6*x^2*y + 2*a^2*b^4*c^6*x^2*y - 4*a^4*c^8*x^2*y -      2*a^2*b^2*c^8*x^2*y - 2*b^4*c^8*x^2*y + a^2*c^10*x^2*y + b^2*c^10*x^2*y + a^10*c^2*x*y^2 -      a^8*b^2*c^2*x*y^2 - 2*a^6*b^4*c^2*x*y^2 + 2*a^4*b^6*c^2*x*y^2 + a^2*b^8*c^2*x*y^2 -      b^10*c^2*x*y^2 - 2*a^8*c^4*x*y^2 + 2*a^6*b^2*c^4*x*y^2 - 2*a^4*b^4*c^4*x*y^2 -      2*a^2*b^6*c^4*x*y^2 + 4*b^8*c^4*x*y^2 - 2*a^4*b^2*c^6*x*y^2 - 6*b^6*c^6*x*y^2 + 2*a^4*c^8*x*y^2 +      2*a^2*b^2*c^8*x*y^2 + 4*b^4*c^8*x*y^2 - a^2*c^10*x*y^2 - b^2*c^10*x*y^2 - a^10*b^2*x^2*z +      4*a^8*b^4*x^2*z - 6*a^6*b^6*x^2*z + 4*a^4*b^8*x^2*z - a^2*b^10*x^2*z + a^8*b^2*c^2*x^2*z -      2*a^6*b^4*c^2*x^2*z + 2*a^2*b^8*c^2*x^2*z - b^10*c^2*x^2*z + 2*a^6*b^2*c^4*x^2*z -      2*a^4*b^4*c^4*x^2*z - 2*a^2*b^6*c^4*x^2*z + 2*b^8*c^4*x^2*z - 2*a^4*b^2*c^6*x^2*z +      2*a^2*b^4*c^6*x^2*z - a^2*b^2*c^8*x^2*z - 2*b^4*c^8*x^2*z + b^2*c^10*x^2*z + a^10*b^2*y^2*z -      4*a^8*b^4*y^2*z + 6*a^6*b^6*y^2*z - 4*a^4*b^8*y^2*z + a^2*b^10*y^2*z + a^10*c^2*y^2*z -      2*a^8*b^2*c^2*y^2*z + 2*a^4*b^6*c^2*y^2*z - a^2*b^8*c^2*y^2*z - 2*a^8*c^4*y^2*z +      2*a^6*b^2*c^4*y^2*z + 2*a^4*b^4*c^4*y^2*z - 2*a^2*b^6*c^4*y^2*z - 2*a^4*b^2*c^6*y^2*z +      2*a^2*b^4*c^6*y^2*z + 2*a^4*c^8*y^2*z + a^2*b^2*c^8*y^2*z - a^2*c^10*y^2*z - a^10*b^2*x*z^2 +      2*a^8*b^4*x*z^2 - 2*a^4*b^8*x*z^2 + a^2*b^10*x*z^2 + a^8*b^2*c^2*x*z^2 - 2*a^6*b^4*c^2*x*z^2 +      2*a^4*b^6*c^2*x*z^2 - 2*a^2*b^8*c^2*x*z^2 + b^10*c^2*x*z^2 + 2*a^6*b^2*c^4*x*z^2 +      2*a^4*b^4*c^4*x*z^2 - 4*b^8*c^4*x*z^2 - 2*a^4*b^2*c^6*x*z^2 + 2*a^2*b^4*c^6*x*z^2 +      6*b^6*c^6*x*z^2 - a^2*b^2*c^8*x*z^2 - 4*b^4*c^8*x*z^2 + b^2*c^10*x*z^2 - a^10*b^2*y*z^2 +      2*a^8*b^4*y*z^2 - 2*a^4*b^8*y*z^2 + a^2*b^10*y*z^2 - a^10*c^2*y*z^2 + 2*a^8*b^2*c^2*y*z^2 -      2*a^6*b^4*c^2*y*z^2 + 2*a^4*b^6*c^2*y*z^2 - a^2*b^8*c^2*y*z^2 + 4*a^8*c^4*y*z^2 -      2*a^4*b^4*c^4*y*z^2 - 2*a^2*b^6*c^4*y*z^2 - 6*a^6*c^6*y*z^2 - 2*a^4*b^2*c^6*y*z^2 +      2*a^2*b^4*c^6*y*z^2 + 4*a^4*c^8*y*z^2 + a^2*b^2*c^8*y*z^2 - a^2*c^10*y*z^2");
		exp("K045","a^6*c^2*x^2*y - a^2*b^4*c^2*x^2*y - 2*a^4*c^4*x^2*y + a^2*c^6*x^2*y + a^4*b^2*c^2*x*y^2 -      b^6*c^2*x*y^2 + 2*b^4*c^4*x*y^2 - b^2*c^6*x*y^2 - a^6*b^2*x^2*z + 2*a^4*b^4*x^2*z - a^2*b^6*x^2*z +      a^2*b^2*c^4*x^2*z + a^6*b^2*y^2*z - 2*a^4*b^4*y^2*z + a^2*b^6*y^2*z - a^2*b^2*c^4*y^2*z -      a^4*b^2*c^2*x*z^2 + b^6*c^2*x*z^2 - 2*b^4*c^4*x*z^2 + b^2*c^6*x*z^2 - a^6*c^2*y*z^2 +      a^2*b^4*c^2*y*z^2 + 2*a^4*c^4*y*z^2 - a^2*c^6*y*z^2");
		exp("K046a","Sqrt((a + b - c)*(a - b + c)*(-a + b + c)*(a + b + c))*((-a^4)*x^2*y - 4*a^2*b^2*x^2*y +        5*b^4*x^2*y - a^2*c^2*x^2*y - b^2*c^2*x^2*y - 4*c^4*x^2*y - 5*a^4*x*y^2 + 4*a^2*b^2*x*y^2 +        b^4*x*y^2 + a^2*c^2*x*y^2 + b^2*c^2*x*y^2 + 4*c^4*x*y^2 + a^4*x^2*z + a^2*b^2*x^2*z +        4*b^4*x^2*z + 4*a^2*c^2*x^2*z + b^2*c^2*x^2*z - 5*c^4*x^2*z - 4*a^4*y^2*z - a^2*b^2*y^2*z -        b^4*y^2*z - a^2*c^2*y^2*z - 4*b^2*c^2*y^2*z + 5*c^4*y^2*z + 5*a^4*x*z^2 - a^2*b^2*x*z^2 -        4*b^4*x*z^2 - 4*a^2*c^2*x*z^2 - b^2*c^2*x*z^2 - c^4*x*z^2 + 4*a^4*y*z^2 + a^2*b^2*y*z^2 -        5*b^4*y*z^2 + a^2*c^2*y*z^2 + 4*b^2*c^2*y*z^2 + c^4*y*z^2) +      Sqrt(3)*(a^6*x^2*y - 3*a^4*b^2*x^2*y + 3*a^2*b^4*x^2*y - b^6*x^2*y - 2*a^4*c^2*x^2*y -        6*a^2*b^2*c^2*x^2*y + 8*b^4*c^2*x^2*y + a^2*c^4*x^2*y - 7*b^2*c^4*x^2*y + a^6*x*y^2 -        3*a^4*b^2*x*y^2 + 3*a^2*b^4*x*y^2 - b^6*x*y^2 - 8*a^4*c^2*x*y^2 + 6*a^2*b^2*c^2*x*y^2 +        2*b^4*c^2*x*y^2 + 7*a^2*c^4*x*y^2 - b^2*c^4*x*y^2 - a^6*x^2*z + 2*a^4*b^2*x^2*z - a^2*b^4*x^2*z +        3*a^4*c^2*x^2*z + 6*a^2*b^2*c^2*x^2*z + 7*b^4*c^2*x^2*z - 3*a^2*c^4*x^2*z - 8*b^2*c^4*x^2*z +        c^6*x^2*z + a^4*b^2*y^2*z - 2*a^2*b^4*y^2*z + b^6*y^2*z - 7*a^4*c^2*y^2*z - 6*a^2*b^2*c^2*y^2*z -        3*b^4*c^2*y^2*z + 8*a^2*c^4*y^2*z + 3*b^2*c^4*y^2*z - c^6*y^2*z - a^6*x*z^2 + 8*a^4*b^2*x*z^2 -        7*a^2*b^4*x*z^2 + 3*a^4*c^2*x*z^2 - 6*a^2*b^2*c^2*x*z^2 + b^4*c^2*x*z^2 - 3*a^2*c^4*x*z^2 -        2*b^2*c^4*x*z^2 + c^6*x*z^2 + 7*a^4*b^2*y*z^2 - 8*a^2*b^4*y*z^2 + b^6*y*z^2 - a^4*c^2*y*z^2 +        6*a^2*b^2*c^2*y*z^2 - 3*b^4*c^2*y*z^2 + 2*a^2*c^4*y*z^2 + 3*b^2*c^4*y*z^2 - c^6*y*z^2)");
		exp("K046b","Sqrt((a + b - c)*(a - b + c)*(-a + b + c)*(a + b + c))*(a^4*x^2*y + 4*a^2*b^2*x^2*y -        5*b^4*x^2*y + a^2*c^2*x^2*y + b^2*c^2*x^2*y + 4*c^4*x^2*y + 5*a^4*x*y^2 - 4*a^2*b^2*x*y^2 -        b^4*x*y^2 - a^2*c^2*x*y^2 - b^2*c^2*x*y^2 - 4*c^4*x*y^2 - a^4*x^2*z - a^2*b^2*x^2*z -        4*b^4*x^2*z - 4*a^2*c^2*x^2*z - b^2*c^2*x^2*z + 5*c^4*x^2*z + 4*a^4*y^2*z + a^2*b^2*y^2*z +        b^4*y^2*z + a^2*c^2*y^2*z + 4*b^2*c^2*y^2*z - 5*c^4*y^2*z - 5*a^4*x*z^2 + a^2*b^2*x*z^2 +        4*b^4*x*z^2 + 4*a^2*c^2*x*z^2 + b^2*c^2*x*z^2 + c^4*x*z^2 - 4*a^4*y*z^2 - a^2*b^2*y*z^2 +        5*b^4*y*z^2 - a^2*c^2*y*z^2 - 4*b^2*c^2*y*z^2 - c^4*y*z^2) +      Sqrt(3)*(a^6*x^2*y - 3*a^4*b^2*x^2*y + 3*a^2*b^4*x^2*y - b^6*x^2*y - 2*a^4*c^2*x^2*y -        6*a^2*b^2*c^2*x^2*y + 8*b^4*c^2*x^2*y + a^2*c^4*x^2*y - 7*b^2*c^4*x^2*y + a^6*x*y^2 -        3*a^4*b^2*x*y^2 + 3*a^2*b^4*x*y^2 - b^6*x*y^2 - 8*a^4*c^2*x*y^2 + 6*a^2*b^2*c^2*x*y^2 +        2*b^4*c^2*x*y^2 + 7*a^2*c^4*x*y^2 - b^2*c^4*x*y^2 - a^6*x^2*z + 2*a^4*b^2*x^2*z - a^2*b^4*x^2*z +        3*a^4*c^2*x^2*z + 6*a^2*b^2*c^2*x^2*z + 7*b^4*c^2*x^2*z - 3*a^2*c^4*x^2*z - 8*b^2*c^4*x^2*z +        c^6*x^2*z + a^4*b^2*y^2*z - 2*a^2*b^4*y^2*z + b^6*y^2*z - 7*a^4*c^2*y^2*z - 6*a^2*b^2*c^2*y^2*z -        3*b^4*c^2*y^2*z + 8*a^2*c^4*y^2*z + 3*b^2*c^4*y^2*z - c^6*y^2*z - a^6*x*z^2 + 8*a^4*b^2*x*z^2 -        7*a^2*b^4*x*z^2 + 3*a^4*c^2*x*z^2 - 6*a^2*b^2*c^2*x*z^2 + b^4*c^2*x*z^2 - 3*a^2*c^4*x*z^2 -        2*b^2*c^4*x*z^2 + c^6*x*z^2 + 7*a^4*b^2*y*z^2 - 8*a^2*b^4*y*z^2 + b^6*y*z^2 - a^4*c^2*y*z^2 +        6*a^2*b^2*c^2*y*z^2 - 3*b^4*c^2*y*z^2 + 2*a^2*c^4*y*z^2 + 3*b^2*c^4*y*z^2 - c^6*y*z^2)");
		exp("K047","-3*a^4*c^2*x^2*y - 2*a^2*b^2*c^2*x^2*y + 5*b^4*c^2*x^2*y + 2*a^2*c^4*x^2*y - 6*b^2*c^4*x^2*y +      c^6*x^2*y - 5*a^4*c^2*x*y^2 + 2*a^2*b^2*c^2*x*y^2 + 3*b^4*c^2*x*y^2 + 6*a^2*c^4*x*y^2 -      2*b^2*c^4*x*y^2 - c^6*x*y^2 + 3*a^4*b^2*x^2*z - 2*a^2*b^4*x^2*z - b^6*x^2*z + 2*a^2*b^2*c^2*x^2*z +      6*b^4*c^2*x^2*z - 5*b^2*c^4*x^2*z + 8*a^4*b^2*x*y*z - 8*a^2*b^4*x*y*z - 8*a^4*c^2*x*y*z +      8*b^4*c^2*x*y*z + 8*a^2*c^4*x*y*z - 8*b^2*c^4*x*y*z + a^6*y^2*z + 2*a^4*b^2*y^2*z -      3*a^2*b^4*y^2*z - 6*a^4*c^2*y^2*z - 2*a^2*b^2*c^2*y^2*z + 5*a^2*c^4*y^2*z + 5*a^4*b^2*x*z^2 -      6*a^2*b^4*x*z^2 + b^6*x*z^2 - 2*a^2*b^2*c^2*x*z^2 + 2*b^4*c^2*x*z^2 - 3*b^2*c^4*x*z^2 - a^6*y*z^2 +      6*a^4*b^2*y*z^2 - 5*a^2*b^4*y*z^2 - 2*a^4*c^2*y*z^2 + 2*a^2*b^2*c^2*y*z^2 + 3*a^2*c^4*y*z^2");
		exp("K048","-(a^2*b^4*c^2*x^3) + b^6*c^2*x^3 + a^2*b^2*c^4*x^3 - b^2*c^6*x^3 - a^4*b^2*c^2*x^2*y +      b^6*c^2*x^2*y + a^2*b^2*c^4*x^2*y - 3*b^4*c^4*x^2*y + 2*b^2*c^6*x^2*y - a^6*c^2*x*y^2 +      a^2*b^4*c^2*x*y^2 + 3*a^4*c^4*x*y^2 - a^2*b^2*c^4*x*y^2 - 2*a^2*c^6*x*y^2 - a^6*c^2*y^3 +      a^4*b^2*c^2*y^3 - a^2*b^2*c^4*y^3 + a^2*c^6*y^3 + a^4*b^2*c^2*x^2*z - a^2*b^4*c^2*x^2*z -      2*b^6*c^2*x^2*z + 3*b^4*c^4*x^2*z - b^2*c^6*x^2*z + 2*a^6*c^2*y^2*z + a^4*b^2*c^2*y^2*z -      a^2*b^4*c^2*y^2*z - 3*a^4*c^4*y^2*z + a^2*c^6*y^2*z + a^6*b^2*x*z^2 - 3*a^4*b^4*x*z^2 +      2*a^2*b^6*x*z^2 + a^2*b^4*c^2*x*z^2 - a^2*b^2*c^4*x*z^2 - 2*a^6*b^2*y*z^2 + 3*a^4*b^4*y*z^2 -      a^2*b^6*y*z^2 - a^4*b^2*c^2*y*z^2 + a^2*b^2*c^4*y*z^2 + a^6*b^2*z^3 - a^2*b^6*z^3 -      a^4*b^2*c^2*z^3 + a^2*b^4*c^2*z^3");
		exp("K049","a^6*x^2*y - 3*a^4*b^2*x^2*y + 3*a^2*b^4*x^2*y - b^6*x^2*y - 2*a^4*c^2*x^2*y +      2*a^2*b^2*c^2*x^2*y + a^2*c^4*x^2*y + b^2*c^4*x^2*y + a^6*x*y^2 - 3*a^4*b^2*x*y^2 +      3*a^2*b^4*x*y^2 - b^6*x*y^2 - 2*a^2*b^2*c^2*x*y^2 + 2*b^4*c^2*x*y^2 - a^2*c^4*x*y^2 -      b^2*c^4*x*y^2 - a^6*x^2*z + 2*a^4*b^2*x^2*z - a^2*b^4*x^2*z + 3*a^4*c^2*x^2*z -      2*a^2*b^2*c^2*x^2*z - b^4*c^2*x^2*z - 3*a^2*c^4*x^2*z + c^6*x^2*z + a^4*b^2*y^2*z -      2*a^2*b^4*y^2*z + b^6*y^2*z + a^4*c^2*y^2*z + 2*a^2*b^2*c^2*y^2*z - 3*b^4*c^2*y^2*z +      3*b^2*c^4*y^2*z - c^6*y^2*z - a^6*x*z^2 + a^2*b^4*x*z^2 + 3*a^4*c^2*x*z^2 + 2*a^2*b^2*c^2*x*z^2 +      b^4*c^2*x*z^2 - 3*a^2*c^4*x*z^2 - 2*b^2*c^4*x*z^2 + c^6*x*z^2 - a^4*b^2*y*z^2 + b^6*y*z^2 -      a^4*c^2*y*z^2 - 2*a^2*b^2*c^2*y*z^2 - 3*b^4*c^2*y*z^2 + 2*a^2*c^4*y*z^2 + 3*b^2*c^4*y*z^2 -      c^6*y*z^2");
		exp("K050","a^10*c^2*x^2*y - 2*a^8*b^2*c^2*x^2*y + a^6*b^4*c^2*x^2*y - a^4*b^6*c^2*x^2*y +      2*a^2*b^8*c^2*x^2*y - b^10*c^2*x^2*y - 4*a^8*c^4*x^2*y + 4*a^6*b^2*c^4*x^2*y -      2*a^2*b^6*c^4*x^2*y + 2*b^8*c^4*x^2*y + 6*a^6*c^6*x^2*y - a^4*b^2*c^6*x^2*y + a^2*b^4*c^6*x^2*y -      4*a^4*c^8*x^2*y - 2*a^2*b^2*c^8*x^2*y - 2*b^4*c^8*x^2*y + a^2*c^10*x^2*y + b^2*c^10*x^2*y +      a^10*c^2*x*y^2 - 2*a^8*b^2*c^2*x*y^2 + a^6*b^4*c^2*x*y^2 - a^4*b^6*c^2*x*y^2 +      2*a^2*b^8*c^2*x*y^2 - b^10*c^2*x*y^2 - 2*a^8*c^4*x*y^2 + 2*a^6*b^2*c^4*x*y^2 -      4*a^2*b^6*c^4*x*y^2 + 4*b^8*c^4*x*y^2 - a^4*b^2*c^6*x*y^2 + a^2*b^4*c^6*x*y^2 - 6*b^6*c^6*x*y^2 +      2*a^4*c^8*x*y^2 + 2*a^2*b^2*c^8*x*y^2 + 4*b^4*c^8*x*y^2 - a^2*c^10*x*y^2 - b^2*c^10*x*y^2 -      a^10*b^2*x^2*z + 4*a^8*b^4*x^2*z - 6*a^6*b^6*x^2*z + 4*a^4*b^8*x^2*z - a^2*b^10*x^2*z +      2*a^8*b^2*c^2*x^2*z - 4*a^6*b^4*c^2*x^2*z + a^4*b^6*c^2*x^2*z + 2*a^2*b^8*c^2*x^2*z -      b^10*c^2*x^2*z - a^6*b^2*c^4*x^2*z - a^2*b^6*c^4*x^2*z + 2*b^8*c^4*x^2*z + a^4*b^2*c^6*x^2*z +      2*a^2*b^4*c^6*x^2*z - 2*a^2*b^2*c^8*x^2*z - 2*b^4*c^8*x^2*z + b^2*c^10*x^2*z + a^10*b^2*y^2*z -      4*a^8*b^4*y^2*z + 6*a^6*b^6*y^2*z - 4*a^4*b^8*y^2*z + a^2*b^10*y^2*z + a^10*c^2*y^2*z -      2*a^8*b^2*c^2*y^2*z - a^6*b^4*c^2*y^2*z + 4*a^4*b^6*c^2*y^2*z - 2*a^2*b^8*c^2*y^2*z -      2*a^8*c^4*y^2*z + a^6*b^2*c^4*y^2*z + a^2*b^6*c^4*y^2*z - 2*a^4*b^2*c^6*y^2*z - a^2*b^4*c^6*y^2*z +      2*a^4*c^8*y^2*z + 2*a^2*b^2*c^8*y^2*z - a^2*c^10*y^2*z - a^10*b^2*x*z^2 + 2*a^8*b^4*x*z^2 -      2*a^4*b^8*x*z^2 + a^2*b^10*x*z^2 + 2*a^8*b^2*c^2*x*z^2 - 2*a^6*b^4*c^2*x*z^2 + a^4*b^6*c^2*x*z^2 -      2*a^2*b^8*c^2*x*z^2 + b^10*c^2*x*z^2 - a^6*b^2*c^4*x*z^2 - a^2*b^6*c^4*x*z^2 - 4*b^8*c^4*x*z^2 +      a^4*b^2*c^6*x*z^2 + 4*a^2*b^4*c^6*x*z^2 + 6*b^6*c^6*x*z^2 - 2*a^2*b^2*c^8*x*z^2 - 4*b^4*c^8*x*z^2 +      b^2*c^10*x*z^2 - a^10*b^2*y*z^2 + 2*a^8*b^4*y*z^2 - 2*a^4*b^8*y*z^2 + a^2*b^10*y*z^2 -      a^10*c^2*y*z^2 + 2*a^8*b^2*c^2*y*z^2 - a^6*b^4*c^2*y*z^2 + 2*a^4*b^6*c^2*y*z^2 -      2*a^2*b^8*c^2*y*z^2 + 4*a^8*c^4*y*z^2 + a^6*b^2*c^4*y*z^2 + a^2*b^6*c^4*y*z^2 - 6*a^6*c^6*y*z^2 -      4*a^4*b^2*c^6*y*z^2 - a^2*b^4*c^6*y*z^2 + 4*a^4*c^8*y*z^2 + 2*a^2*b^2*c^8*y*z^2 - a^2*c^10*y*z^2 ");
		exp("K051","(-a^8)*x^3 + 2*a^6*b^2*x^3 - 2*a^2*b^6*x^3 + b^8*x^3 + 2*a^6*c^2*x^3 - 4*a^4*b^2*c^2*x^3 +      2*a^2*b^4*c^2*x^3 + 2*a^2*b^2*c^4*x^3 - 2*b^4*c^4*x^3 - 2*a^2*c^6*x^3 + c^8*x^3 - a^8*x^2*y +      2*a^6*b^2*x^2*y - 2*a^2*b^6*x^2*y + b^8*x^2*y - 2*a^4*b^2*c^2*x^2*y + 4*a^2*b^4*c^2*x^2*y -      2*b^6*c^2*x^2*y + 2*a^4*c^4*x^2*y - 2*a^2*b^2*c^4*x^2*y + 2*b^2*c^6*x^2*y - c^8*x^2*y + a^8*x*y^2 -      2*a^6*b^2*x*y^2 + 2*a^2*b^6*x*y^2 - b^8*x*y^2 - 2*a^6*c^2*x*y^2 + 4*a^4*b^2*c^2*x*y^2 -      2*a^2*b^4*c^2*x*y^2 - 2*a^2*b^2*c^4*x*y^2 + 2*b^4*c^4*x*y^2 + 2*a^2*c^6*x*y^2 - c^8*x*y^2 +      a^8*y^3 - 2*a^6*b^2*y^3 + 2*a^2*b^6*y^3 - b^8*y^3 + 2*a^4*b^2*c^2*y^3 - 4*a^2*b^4*c^2*y^3 +      2*b^6*c^2*y^3 - 2*a^4*c^4*y^3 + 2*a^2*b^2*c^4*y^3 - 2*b^2*c^6*y^3 + c^8*y^3 - a^8*x^2*z +      2*a^4*b^4*x^2*z - b^8*x^2*z + 2*a^6*c^2*x^2*z - 2*a^4*b^2*c^2*x^2*z - 2*a^2*b^4*c^2*x^2*z +      2*b^6*c^2*x^2*z + 4*a^2*b^2*c^4*x^2*z - 2*a^2*c^6*x^2*z - 2*b^2*c^6*x^2*z + c^8*x^2*z -      3*a^8*x*y*z + 4*a^6*b^2*x*y*z - 2*a^4*b^4*x*y*z + 4*a^2*b^6*x*y*z - 3*b^8*x*y*z + 4*a^6*c^2*x*y*z -      4*a^4*b^2*c^2*x*y*z - 4*a^2*b^4*c^2*x*y*z + 4*b^6*c^2*x*y*z - 2*a^4*c^4*x*y*z -      4*a^2*b^2*c^4*x*y*z - 2*b^4*c^4*x*y*z + 4*a^2*c^6*x*y*z + 4*b^2*c^6*x*y*z - 3*c^8*x*y*z -      a^8*y^2*z + 2*a^4*b^4*y^2*z - b^8*y^2*z + 2*a^6*c^2*y^2*z - 2*a^4*b^2*c^2*y^2*z -      2*a^2*b^4*c^2*y^2*z + 2*b^6*c^2*y^2*z + 4*a^2*b^2*c^4*y^2*z - 2*a^2*c^6*y^2*z - 2*b^2*c^6*y^2*z +      c^8*y^2*z + a^8*x*z^2 - 2*a^6*b^2*x*z^2 + 2*a^2*b^6*x*z^2 - b^8*x*z^2 - 2*a^6*c^2*x*z^2 +      4*a^4*b^2*c^2*x*z^2 - 2*a^2*b^4*c^2*x*z^2 - 2*a^2*b^2*c^4*x*z^2 + 2*b^4*c^4*x*z^2 +      2*a^2*c^6*x*z^2 - c^8*x*z^2 - a^8*y*z^2 + 2*a^6*b^2*y*z^2 - 2*a^2*b^6*y*z^2 + b^8*y*z^2 -      2*a^4*b^2*c^2*y*z^2 + 4*a^2*b^4*c^2*y*z^2 - 2*b^6*c^2*y*z^2 + 2*a^4*c^4*y*z^2 -      2*a^2*b^2*c^4*y*z^2 + 2*b^2*c^6*y*z^2 - c^8*y*z^2 + a^8*z^3 - 2*a^4*b^4*z^3 + b^8*z^3 -      2*a^6*c^2*z^3 + 2*a^4*b^2*c^2*z^3 + 2*a^2*b^4*c^2*z^3 - 2*b^6*c^2*z^3 - 4*a^2*b^2*c^4*z^3 +      2*a^2*c^6*z^3 + 2*b^2*c^6*z^3 - c^8*z^3");
		exp("K052","a^4*b^4*x^2*y - 2*a^4*b^2*c^2*x^2*y - 2*a^2*b^4*c^2*x^2*y + a^4*c^4*x^2*y + 4*a^2*b^2*c^4*x^2*y +      b^4*c^4*x^2*y - 2*a^2*c^6*x^2*y - 2*b^2*c^6*x^2*y + c^8*x^2*y + a^4*b^4*x*y^2 -      2*a^4*b^2*c^2*x*y^2 - 2*a^2*b^4*c^2*x*y^2 + a^4*c^4*x*y^2 + 4*a^2*b^2*c^4*x*y^2 + b^4*c^4*x*y^2 -      2*a^2*c^6*x*y^2 - 2*b^2*c^6*x*y^2 + c^8*x*y^2 + a^4*b^4*x^2*z - 2*a^2*b^6*x^2*z + b^8*x^2*z -      2*a^4*b^2*c^2*x^2*z + 4*a^2*b^4*c^2*x^2*z - 2*b^6*c^2*x^2*z + a^4*c^4*x^2*z - 2*a^2*b^2*c^4*x^2*z +      b^4*c^4*x^2*z + a^8*y^2*z - 2*a^6*b^2*y^2*z + a^4*b^4*y^2*z - 2*a^6*c^2*y^2*z +      4*a^4*b^2*c^2*y^2*z - 2*a^2*b^4*c^2*y^2*z + a^4*c^4*y^2*z - 2*a^2*b^2*c^4*y^2*z + b^4*c^4*y^2*z +      a^4*b^4*x*z^2 - 2*a^2*b^6*x*z^2 + b^8*x*z^2 - 2*a^4*b^2*c^2*x*z^2 + 4*a^2*b^4*c^2*x*z^2 -      2*b^6*c^2*x*z^2 + a^4*c^4*x*z^2 - 2*a^2*b^2*c^4*x*z^2 + b^4*c^4*x*z^2 + a^8*y*z^2 -      2*a^6*b^2*y*z^2 + a^4*b^4*y*z^2 - 2*a^6*c^2*y*z^2 + 4*a^4*b^2*c^2*y*z^2 - 2*a^2*b^4*c^2*y*z^2 +      a^4*c^4*y*z^2 - 2*a^2*b^2*c^4*y*z^2 + b^4*c^4*y*z^2");
		exp("K053A","c^2*x*y^2 + a^2*y^2*z - b^2*y^2*z + c^2*y^2*z - b^2*x*z^2 - a^2*y*z^2 - b^2*y*z^2 + c^2*y*z^2 ");
		exp("K053B","(-c^2)*x^2*y + a^2*x^2*z - b^2*x^2*z - c^2*x^2*z + a^2*x*z^2 + b^2*x*z^2 - c^2*x*z^2 +      a^2*y*z^2");
		exp("K053C","(-a^2)*x^2*y + b^2*x^2*y + c^2*x^2*y - a^2*x*y^2 + b^2*x*y^2 - c^2*x*y^2 + b^2*x^2*z -      a^2*y^2*z");
		exp("K054","a^6*x^2*y - 3*a^4*b^2*x^2*y + 3*a^2*b^4*x^2*y - b^6*x^2*y - 3*a^4*c^2*x^2*y +      4*a^2*b^2*c^2*x^2*y - b^4*c^2*x^2*y + 2*a^2*c^4*x^2*y + 2*b^2*c^4*x^2*y + a^6*x*y^2 -      3*a^4*b^2*x*y^2 + 3*a^2*b^4*x*y^2 - b^6*x*y^2 + a^4*c^2*x*y^2 - 4*a^2*b^2*c^2*x*y^2 +      3*b^4*c^2*x*y^2 - 2*a^2*c^4*x*y^2 - 2*b^2*c^4*x*y^2 - a^6*x^2*z + 3*a^4*b^2*x^2*z -      2*a^2*b^4*x^2*z + 3*a^4*c^2*x^2*z - 4*a^2*b^2*c^2*x^2*z - 2*b^4*c^2*x^2*z - 3*a^2*c^4*x^2*z +      b^2*c^4*x^2*z + c^6*x^2*z + 2*a^4*b^2*x*y*z - 2*a^2*b^4*x*y*z - 2*a^4*c^2*x*y*z + 2*b^4*c^2*x*y*z +      2*a^2*c^4*x*y*z - 2*b^2*c^4*x*y*z + 2*a^4*b^2*y^2*z - 3*a^2*b^4*y^2*z + b^6*y^2*z +      2*a^4*c^2*y^2*z + 4*a^2*b^2*c^2*y^2*z - 3*b^4*c^2*y^2*z - a^2*c^4*y^2*z + 3*b^2*c^4*y^2*z -      c^6*y^2*z - a^6*x*z^2 - a^4*b^2*x*z^2 + 2*a^2*b^4*x*z^2 + 3*a^4*c^2*x*z^2 + 4*a^2*b^2*c^2*x*z^2 +      2*b^4*c^2*x*z^2 - 3*a^2*c^4*x*z^2 - 3*b^2*c^4*x*z^2 + c^6*x*z^2 - 2*a^4*b^2*y*z^2 + a^2*b^4*y*z^2 +      b^6*y*z^2 - 2*a^4*c^2*y*z^2 - 4*a^2*b^2*c^2*y*z^2 - 3*b^4*c^2*y*z^2 + 3*a^2*c^4*y*z^2 +      3*b^2*c^4*y*z^2 - c^6*y*z^2");
		exp("K055","-(a^2*c^4*x^2*y) + 3*b^2*c^4*x^2*y + c^6*x^2*y - 3*a^2*c^4*x*y^2 + b^2*c^4*x*y^2 - c^6*x*y^2 +      a^2*b^4*x^2*z - b^6*x^2*z - 3*b^4*c^2*x^2*z - 2*a^4*b^2*x*y*z + 2*a^2*b^4*x*y*z + 2*a^4*c^2*x*y*z -      2*b^4*c^2*x*y*z - 2*a^2*c^4*x*y*z + 2*b^2*c^4*x*y*z + a^6*y^2*z - a^4*b^2*y^2*z + 3*a^4*c^2*y^2*z +      3*a^2*b^4*x*z^2 + b^6*x*z^2 - b^4*c^2*x*z^2 - a^6*y*z^2 - 3*a^4*b^2*y*z^2 + a^4*c^2*y*z^2");
		exp("K056","(-a^6)*b^4*x^2*y + a^4*b^6*x^2*y + a^4*b^4*c^2*x^2*y + 2*a^4*b^2*c^4*x^2*y - a^6*b^4*x*y^2 +      a^4*b^6*x*y^2 - a^4*b^4*c^2*x*y^2 - 2*a^2*b^4*c^4*x*y^2 - 2*a^4*b^4*c^2*x^2*z + a^6*c^4*x^2*z -      a^4*b^2*c^4*x^2*z - a^4*c^6*x^2*z + 2*a^4*b^4*c^2*y^2*z + a^2*b^4*c^4*y^2*z - b^6*c^4*y^2*z +      b^4*c^6*y^2*z + a^6*c^4*x*z^2 + a^4*b^2*c^4*x*z^2 + 2*a^2*b^4*c^4*x*z^2 - a^4*c^6*x*z^2 -      2*a^4*b^2*c^4*y*z^2 - a^2*b^4*c^4*y*z^2 - b^6*c^4*y*z^2 + b^4*c^6*y*z^2");
		exp("K057a","(-a^4)*b^2*c^2*x^2*y + a^2*b^4*c^2*x^2*y - b^6*c^2*x^2*y + a^4*c^4*x^2*y + 2*b^4*c^4*x^2*y -" +
				"      a^2*c^6*x^2*y - 2*b^2*c^6*x^2*y + c^8*x^2*y + a^4*b^2*c^2*x*y^2 - a^2*b^4*c^2*x*y^2 + " +
				"     b^6*c^2*x*y^2 - a^4*c^4*x*y^2 - 2*b^4*c^4*x*y^2 + a^2*c^6*x*y^2 + 2*b^2*c^6*x*y^2 - c^8*x*y^2 +  " +
				"    a^6*b^2*x^2*z - 2*a^4*b^4*x^2*z + 2*a^2*b^6*x^2*z - b^8*x^2*z - a^6*c^2*x^2*z - a^2*b^4*c^2*x^2*z +  " +
				"    b^6*c^2*x^2*z + 2*a^4*c^4*x^2*z + a^2*b^2*c^4*x^2*z - 2*a^2*c^6*x^2*z - b^2*c^6*x^2*z + c^8*x^2*z +   " +
				"   a^6*b^2*y^2*z - 2*a^4*b^4*y^2*z + 2*a^2*b^6*y^2*z - b^8*y^2*z - a^6*c^2*y^2*z +    " +
				"  2*a^4*b^2*c^2*y^2*z - 3*a^2*b^4*c^2*y^2*z + 3*b^6*c^2*y^2*z + a^2*b^2*c^4*y^2*z - 4*b^4*c^4*y^2*z +   " +
				"   3*b^2*c^6*y^2*z - c^8*y^2*z + a^6*b^2*x*z^2 - a^4*b^4*x*z^2 + a^2*b^6*x*z^2 - a^6*c^2*x*z^2 -   " +
				"   a^4*b^2*c^2*x*z^2 - a^2*b^4*c^2*x*z^2 - b^6*c^2*x*z^2 + 2*a^4*c^4*x*z^2 + 2*a^2*b^2*c^4*x*z^2 +  " +
				"    2*b^4*c^4*x*z^2 - 2*a^2*c^6*x*z^2 - 2*b^2*c^6*x*z^2 + c^8*x*z^2 - a^4*b^4*y*z^2 + a^2*b^6*y*z^2 -   " +
				"   b^8*y*z^2 + 2*a^4*b^2*c^2*y*z^2 - a^2*b^4*c^2*y*z^2 + 3*b^6*c^2*y*z^2 - a^4*c^4*y*z^2 -    " +
				"  a^2*b^2*c^4*y*z^2 - 4*b^4*c^4*y*z^2 + a^2*c^6*y*z^2 + 3*b^2*c^6*y*z^2 - c^8*y*z^2 +   " +
				"   Sqrt(a^4 + b^4 - b^2*c^2 + c^4 - a^2*(b^2 + c^2))*(-2*a^2*b^2*c^2*x^2*y + b^4*c^2*x^2*y +      " +
				"  2*a^2*c^4*x^2*y - c^6*x^2*y - a^2*b^2*c^2*x*y^2 + 2*b^4*c^2*x*y^2 + a^2*c^4*x*y^2 -    " +
				"    3*b^2*c^4*x*y^2 + c^6*x*y^2 + a^4*b^2*x^2*z - a^2*b^4*x^2*z - a^4*c^2*x^2*z + b^4*c^2*x^2*z +    " +
				"    a^2*c^4*x^2*z - b^2*c^4*x^2*z + a^2*b^4*y^2*z - b^6*y^2*z - 2*a^2*b^2*c^2*y^2*z +    " +
				"    2*b^4*c^2*y^2*z + a^2*c^4*y^2*z - b^2*c^4*y^2*z + a^4*b^2*x*z^2 + a^2*b^4*x*z^2 - b^6*x*z^2 -    " +
				"    a^4*c^2*x*z^2 - 2*a^2*b^2*c^2*x*z^2 + b^4*c^2*x*z^2 + a^2*c^4*x*z^2 + a^4*b^2*y*z^2 -    " +
				"    a^2*b^4*y*z^2 - b^6*y*z^2 - a^4*c^2*y*z^2 + a^2*b^2*c^2*y*z^2 + 2*b^4*c^2*y*z^2 -     " +
				"   b^2*c^4*y*z^2) + Sqrt(2*a^4 + (b^2 - c^2)^2 +     " +
				"    a^2*(-b^2 - c^2 + 2*Sqrt(a^4 + b^4 - b^2*c^2 + c^4 - a^2*(b^2 + c^2))))*   " +
				"   (a^4*c^2*x*y^2 - a^2*b^2*c^2*x*y^2 + b^4*c^2*x*y^2 - a^2*c^4*x*y^2 - b^2*c^4*x*y^2 + c^6*x*y^2 +     " +
				"   a^6*y^2*z - 2*a^4*b^2*y^2*z + 2*a^2*b^4*y^2*z - b^6*y^2*z - a^2*b^2*c^2*y^2*z + 2*b^4*c^2*y^2*z -  " +
				"      2*b^2*c^4*y^2*z + c^6*y^2*z - a^4*b^2*x*z^2 + a^2*b^4*x*z^2 - b^6*x*z^2 + a^2*b^2*c^2*x*z^2 +     " +
				"   b^4*c^2*x*z^2 - b^2*c^4*x*z^2 - a^6*y*z^2 - b^6*y*z^2 + 2*a^4*c^2*y*z^2 + a^2*b^2*c^2*y*z^2 +     " +
				"   2*b^4*c^2*y*z^2 - 2*a^2*c^4*y*z^2 - 2*b^2*c^4*y*z^2 + c^6*y*z^2 +     " +
				"   Sqrt(a^4 + b^4 - b^2*c^2 + c^4 - a^2*(b^2 + c^2))*((-b^2)*c^2*x^2*y + c^4*x^2*y - a^2*c^2*x*y^2 +    " +
				"      b^2*c^2*x*y^2 + a^2*b^2*x^2*z - b^4*x^2*z - a^2*c^2*x^2*z + c^4*x^2*z - a^4*y^2*z +    " +
				"      2*a^2*b^2*y^2*z - b^4*y^2*z - a^2*c^2*y^2*z + b^2*c^2*y^2*z + 2*a^2*b^2*x*z^2 - a^2*c^2*x*z^2 -   " +
				"       2*b^2*c^2*x*z^2 + c^4*x*z^2 + a^4*y*z^2 + a^2*b^2*y*z^2 - b^4*y*z^2 - 2*a^2*c^2*y*z^2 +        " +
				"  b^2*c^2*y*z^2))");
		exp("K057b","(-a^4)*b^2*c^2*x^2*y + a^2*b^4*c^2*x^2*y - b^6*c^2*x^2*y + a^4*c^4*x^2*y + 2*b^4*c^4*x^2*y -      a^2*c^6*x^2*y - 2*b^2*c^6*x^2*y + c^8*x^2*y + a^4*b^2*c^2*x*y^2 - a^2*b^4*c^2*x*y^2 +      b^6*c^2*x*y^2 - a^4*c^4*x*y^2 - 2*b^4*c^4*x*y^2 + a^2*c^6*x*y^2 + 2*b^2*c^6*x*y^2 - c^8*x*y^2 +      a^6*b^2*x^2*z - 2*a^4*b^4*x^2*z + 2*a^2*b^6*x^2*z - b^8*x^2*z - a^6*c^2*x^2*z - a^2*b^4*c^2*x^2*z +      b^6*c^2*x^2*z + 2*a^4*c^4*x^2*z + a^2*b^2*c^4*x^2*z - 2*a^2*c^6*x^2*z - b^2*c^6*x^2*z + c^8*x^2*z +      a^6*b^2*y^2*z - 2*a^4*b^4*y^2*z + 2*a^2*b^6*y^2*z - b^8*y^2*z - a^6*c^2*y^2*z +      2*a^4*b^2*c^2*y^2*z - 3*a^2*b^4*c^2*y^2*z + 3*b^6*c^2*y^2*z + a^2*b^2*c^4*y^2*z - 4*b^4*c^4*y^2*z +      3*b^2*c^6*y^2*z - c^8*y^2*z + a^6*b^2*x*z^2 - a^4*b^4*x*z^2 + a^2*b^6*x*z^2 - a^6*c^2*x*z^2 -      a^4*b^2*c^2*x*z^2 - a^2*b^4*c^2*x*z^2 - b^6*c^2*x*z^2 + 2*a^4*c^4*x*z^2 + 2*a^2*b^2*c^4*x*z^2 +      2*b^4*c^4*x*z^2 - 2*a^2*c^6*x*z^2 - 2*b^2*c^6*x*z^2 + c^8*x*z^2 - a^4*b^4*y*z^2 + a^2*b^6*y*z^2 -      b^8*y*z^2 + 2*a^4*b^2*c^2*y*z^2 - a^2*b^4*c^2*y*z^2 + 3*b^6*c^2*y*z^2 - a^4*c^4*y*z^2 -      a^2*b^2*c^4*y*z^2 - 4*b^4*c^4*y*z^2 + a^2*c^6*y*z^2 + 3*b^2*c^6*y*z^2 - c^8*y*z^2 +      Sqrt(a^4 + b^4 - b^2*c^2 + c^4 - a^2*(b^2 + c^2))*(-2*a^2*b^2*c^2*x^2*y + b^4*c^2*x^2*y +        2*a^2*c^4*x^2*y - c^6*x^2*y - a^2*b^2*c^2*x*y^2 + 2*b^4*c^2*x*y^2 + a^2*c^4*x*y^2 -        3*b^2*c^4*x*y^2 + c^6*x*y^2 + a^4*b^2*x^2*z - a^2*b^4*x^2*z - a^4*c^2*x^2*z + b^4*c^2*x^2*z +        a^2*c^4*x^2*z - b^2*c^4*x^2*z + a^2*b^4*y^2*z - b^6*y^2*z - 2*a^2*b^2*c^2*y^2*z +        2*b^4*c^2*y^2*z + a^2*c^4*y^2*z - b^2*c^4*y^2*z + a^4*b^2*x*z^2 + a^2*b^4*x*z^2 - b^6*x*z^2 -        a^4*c^2*x*z^2 - 2*a^2*b^2*c^2*x*z^2 + b^4*c^2*x*z^2 + a^2*c^4*x*z^2 + a^4*b^2*y*z^2 -        a^2*b^4*y*z^2 - b^6*y*z^2 - a^4*c^2*y*z^2 + a^2*b^2*c^2*y*z^2 + 2*b^4*c^2*y*z^2 -        b^2*c^4*y*z^2) + Sqrt(2*a^4 + (b^2 - c^2)^2 +         a^2*(-b^2 - c^2 + 2*Sqrt(a^4 + b^4 - b^2*c^2 + c^4 - a^2*(b^2 + c^2))))*      ((-a^4)*c^2*x*y^2 + a^2*b^2*c^2*x*y^2 - b^4*c^2*x*y^2 + a^2*c^4*x*y^2 + b^2*c^4*x*y^2 -        c^6*x*y^2 - a^6*y^2*z + 2*a^4*b^2*y^2*z - 2*a^2*b^4*y^2*z + b^6*y^2*z + a^2*b^2*c^2*y^2*z -        2*b^4*c^2*y^2*z + 2*b^2*c^4*y^2*z - c^6*y^2*z + a^4*b^2*x*z^2 - a^2*b^4*x*z^2 + b^6*x*z^2 -        a^2*b^2*c^2*x*z^2 - b^4*c^2*x*z^2 + b^2*c^4*x*z^2 + a^6*y*z^2 + b^6*y*z^2 - 2*a^4*c^2*y*z^2 -        a^2*b^2*c^2*y*z^2 - 2*b^4*c^2*y*z^2 + 2*a^2*c^4*y*z^2 + 2*b^2*c^4*y*z^2 - c^6*y*z^2 +        Sqrt(a^4 + b^4 - b^2*c^2 + c^4 - a^2*(b^2 + c^2))*(b^2*c^2*x^2*y - c^4*x^2*y + a^2*c^2*x*y^2 -          b^2*c^2*x*y^2 - a^2*b^2*x^2*z + b^4*x^2*z + a^2*c^2*x^2*z - c^4*x^2*z + a^4*y^2*z -          2*a^2*b^2*y^2*z + b^4*y^2*z + a^2*c^2*y^2*z - b^2*c^2*y^2*z - 2*a^2*b^2*x*z^2 + a^2*c^2*x*z^2 +          2*b^2*c^2*x*z^2 - c^4*x*z^2 - a^4*y*z^2 - a^2*b^2*y*z^2 + b^4*y*z^2 + 2*a^2*c^2*y*z^2 -          b^2*c^2*y*z^2))");
		exp("K058","(-a^2)*c*x^2*y + b^2*c*x^2*y - b*c^2*x^2*y + c^3*x^2*y - a^2*c*x*y^2 + b^2*c*x*y^2 +      a*c^2*x*y^2 - c^3*x*y^2 + a^2*b*x^2*z - b^3*x^2*z + b^2*c*x^2*z - b*c^2*x^2*z + a^3*y^2*z -      a*b^2*y^2*z - a^2*c*y^2*z + a*c^2*y^2*z + a^2*b*x*z^2 - a*b^2*x*z^2 + b^3*x*z^2 - b*c^2*x*z^2 -      a^3*y*z^2 + a^2*b*y*z^2 - a*b^2*y*z^2 + a*c^2*y*z^2");
		exp("K059","a^6*x^2*y - 3*a^4*b^2*x^2*y + 3*a^2*b^4*x^2*y - b^6*x^2*y + 2*a^2*b^2*c^2*x^2*y -      2*b^4*c^2*x^2*y - 3*a^2*c^4*x^2*y + b^2*c^4*x^2*y + 2*c^6*x^2*y + a^6*x*y^2 - 3*a^4*b^2*x*y^2 +      3*a^2*b^4*x*y^2 - b^6*x*y^2 + 2*a^4*c^2*x*y^2 - 2*a^2*b^2*c^2*x*y^2 - a^2*c^4*x*y^2 +      3*b^2*c^4*x*y^2 - 2*c^6*x*y^2 - a^6*x^2*z + 3*a^2*b^4*x^2*z - 2*b^6*x^2*z + 3*a^4*c^2*x^2*z -      2*a^2*b^2*c^2*x^2*z - b^4*c^2*x^2*z - 3*a^2*c^4*x^2*z + 2*b^2*c^4*x^2*z + c^6*x^2*z + 2*a^6*y^2*z -      3*a^4*b^2*y^2*z + b^6*y^2*z + a^4*c^2*y^2*z + 2*a^2*b^2*c^2*y^2*z - 3*b^4*c^2*y^2*z -      2*a^2*c^4*y^2*z + 3*b^2*c^4*y^2*z - c^6*y^2*z - a^6*x*z^2 - 2*a^4*b^2*x*z^2 + a^2*b^4*x*z^2 +      2*b^6*x*z^2 + 3*a^4*c^2*x*z^2 + 2*a^2*b^2*c^2*x*z^2 - 3*b^4*c^2*x*z^2 - 3*a^2*c^4*x*z^2 +      c^6*x*z^2 - 2*a^6*y*z^2 - a^4*b^2*y*z^2 + 2*a^2*b^4*y*z^2 + b^6*y*z^2 + 3*a^4*c^2*y*z^2 -      2*a^2*b^2*c^2*y*z^2 - 3*b^4*c^2*y*z^2 + 3*b^2*c^4*y*z^2 - c^6*y*z^2");
		exp("K060","(-a^6)*x^2*y + 3*a^4*b^2*x^2*y - 3*a^2*b^4*x^2*y + b^6*x^2*y + a^4*c^2*x^2*y -      a^2*b^2*c^2*x^2*y + a^2*c^4*x^2*y - c^6*x^2*y - a^6*x*y^2 + 3*a^4*b^2*x*y^2 - 3*a^2*b^4*x*y^2 +      b^6*x*y^2 + a^2*b^2*c^2*x*y^2 - b^4*c^2*x*y^2 - b^2*c^4*x*y^2 + c^6*x*y^2 + a^6*x^2*z -      a^4*b^2*x^2*z - a^2*b^4*x^2*z + b^6*x^2*z - 3*a^4*c^2*x^2*z + a^2*b^2*c^2*x^2*z + 3*a^2*c^4*x^2*z -      c^6*x^2*z - a^6*y^2*z + a^4*b^2*y^2*z + a^2*b^4*y^2*z - b^6*y^2*z - a^2*b^2*c^2*y^2*z +      3*b^4*c^2*y^2*z - 3*b^2*c^4*y^2*z + c^6*y^2*z + a^6*x*z^2 - b^6*x*z^2 - 3*a^4*c^2*x*z^2 -      a^2*b^2*c^2*x*z^2 + b^4*c^2*x*z^2 + 3*a^2*c^4*x*z^2 + b^2*c^4*x*z^2 - c^6*x*z^2 + a^6*y*z^2 -      b^6*y*z^2 - a^4*c^2*y*z^2 + a^2*b^2*c^2*y*z^2 + 3*b^4*c^2*y*z^2 - a^2*c^4*y*z^2 - 3*b^2*c^4*y*z^2 +      c^6*y*z^2");
		exp("K061a","2*a^6*x^2*y - 6*a^4*b^2*x^2*y + 6*a^2*b^4*x^2*y - 2*b^6*x^2*y - a^4*c^2*x^2*y +      3*a^2*b^2*c^2*x^2*y - 2*b^4*c^2*x^2*y - 4*a^2*c^4*x^2*y + b^2*c^4*x^2*y + 3*c^6*x^2*y +      2*a^6*x*y^2 - 6*a^4*b^2*x*y^2 + 6*a^2*b^4*x*y^2 - 2*b^6*x*y^2 + 2*a^4*c^2*x*y^2 -      3*a^2*b^2*c^2*x*y^2 + b^4*c^2*x*y^2 - a^2*c^4*x*y^2 + 4*b^2*c^4*x*y^2 - 3*c^6*x*y^2 - 2*a^6*x^2*z +      a^4*b^2*x^2*z + 4*a^2*b^4*x^2*z - 3*b^6*x^2*z + 6*a^4*c^2*x^2*z - 3*a^2*b^2*c^2*x^2*z -      b^4*c^2*x^2*z - 6*a^2*c^4*x^2*z + 2*b^2*c^4*x^2*z + 2*c^6*x^2*z + 3*a^6*y^2*z - 4*a^4*b^2*y^2*z -      a^2*b^4*y^2*z + 2*b^6*y^2*z + a^4*c^2*y^2*z + 3*a^2*b^2*c^2*y^2*z - 6*b^4*c^2*y^2*z -      2*a^2*c^4*y^2*z + 6*b^2*c^4*y^2*z - 2*c^6*y^2*z - 2*a^6*x*z^2 - 2*a^4*b^2*x*z^2 + a^2*b^4*x*z^2 +      3*b^6*x*z^2 + 6*a^4*c^2*x*z^2 + 3*a^2*b^2*c^2*x*z^2 - 4*b^4*c^2*x*z^2 - 6*a^2*c^4*x*z^2 -      b^2*c^4*x*z^2 + 2*c^6*x*z^2 - 3*a^6*y*z^2 - a^4*b^2*y*z^2 + 2*a^2*b^4*y*z^2 + 2*b^6*y*z^2 +      4*a^4*c^2*y*z^2 - 3*a^2*b^2*c^2*y*z^2 - 6*b^4*c^2*y*z^2 + a^2*c^4*y*z^2 + 6*b^2*c^4*y*z^2 -      2*c^6*y*z^2 + Sqrt(3)*Sqrt((a + b - c)*(a - b + c)*(-a + b + c)*(a + b + c))*      ((-a^2)*c^2*x^2*y + c^4*x^2*y + b^2*c^2*x*y^2 - c^4*x*y^2 + a^2*b^2*x^2*z - b^4*x^2*z +        a^4*y^2*z - a^2*b^2*y^2*z + b^4*x*z^2 - b^2*c^2*x*z^2 - a^4*y*z^2 + a^2*c^2*y*z^2)");
		exp("K061b","2*a^6*x^2*y - 6*a^4*b^2*x^2*y + 6*a^2*b^4*x^2*y - 2*b^6*x^2*y - a^4*c^2*x^2*y +      3*a^2*b^2*c^2*x^2*y - 2*b^4*c^2*x^2*y - 4*a^2*c^4*x^2*y + b^2*c^4*x^2*y + 3*c^6*x^2*y +      2*a^6*x*y^2 - 6*a^4*b^2*x*y^2 + 6*a^2*b^4*x*y^2 - 2*b^6*x*y^2 + 2*a^4*c^2*x*y^2 -      3*a^2*b^2*c^2*x*y^2 + b^4*c^2*x*y^2 - a^2*c^4*x*y^2 + 4*b^2*c^4*x*y^2 - 3*c^6*x*y^2 - 2*a^6*x^2*z +      a^4*b^2*x^2*z + 4*a^2*b^4*x^2*z - 3*b^6*x^2*z + 6*a^4*c^2*x^2*z - 3*a^2*b^2*c^2*x^2*z -      b^4*c^2*x^2*z - 6*a^2*c^4*x^2*z + 2*b^2*c^4*x^2*z + 2*c^6*x^2*z + 3*a^6*y^2*z - 4*a^4*b^2*y^2*z -      a^2*b^4*y^2*z + 2*b^6*y^2*z + a^4*c^2*y^2*z + 3*a^2*b^2*c^2*y^2*z - 6*b^4*c^2*y^2*z -      2*a^2*c^4*y^2*z + 6*b^2*c^4*y^2*z - 2*c^6*y^2*z - 2*a^6*x*z^2 - 2*a^4*b^2*x*z^2 + a^2*b^4*x*z^2 +      3*b^6*x*z^2 + 6*a^4*c^2*x*z^2 + 3*a^2*b^2*c^2*x*z^2 - 4*b^4*c^2*x*z^2 - 6*a^2*c^4*x*z^2 -      b^2*c^4*x*z^2 + 2*c^6*x*z^2 - 3*a^6*y*z^2 - a^4*b^2*y*z^2 + 2*a^2*b^4*y*z^2 + 2*b^6*y*z^2 +      4*a^4*c^2*y*z^2 - 3*a^2*b^2*c^2*y*z^2 - 6*b^4*c^2*y*z^2 + a^2*c^4*y*z^2 + 6*b^2*c^4*y*z^2 -      2*c^6*y*z^2 + Sqrt(3)*Sqrt((a + b - c)*(a - b + c)*(-a + b + c)*(a + b + c))*      (a^2*c^2*x^2*y - c^4*x^2*y - b^2*c^2*x*y^2 + c^4*x*y^2 - a^2*b^2*x^2*z + b^4*x^2*z - a^4*y^2*z +        a^2*b^2*y^2*z - b^4*x*z^2 + b^2*c^2*x*z^2 + a^4*y*z^2 - a^2*c^2*y*z^2)");
		exp("K062","a^6*c^2*x^2*y - 2*a^4*b^2*c^2*x^2*y + 2*a^2*b^4*c^2*x^2*y - b^6*c^2*x^2*y - 2*a^4*c^4*x^2*y -      b^4*c^4*x^2*y + a^2*c^6*x^2*y + 2*b^2*c^6*x^2*y + a^6*c^2*x*y^2 - 2*a^4*b^2*c^2*x*y^2 +      2*a^2*b^4*c^2*x*y^2 - b^6*c^2*x*y^2 + a^4*c^4*x*y^2 + 2*b^4*c^4*x*y^2 - 2*a^2*c^6*x*y^2 -      b^2*c^6*x*y^2 - a^6*b^2*x^2*z + 2*a^4*b^4*x^2*z - a^2*b^6*x^2*z + 2*a^4*b^2*c^2*x^2*z -      2*b^6*c^2*x^2*z - 2*a^2*b^2*c^4*x^2*z + b^4*c^4*x^2*z + b^2*c^6*x^2*z + a^6*b^2*y^2*z -      2*a^4*b^4*y^2*z + a^2*b^6*y^2*z + 2*a^6*c^2*y^2*z - 2*a^2*b^4*c^2*y^2*z - a^4*c^4*y^2*z +      2*a^2*b^2*c^4*y^2*z - a^2*c^6*y^2*z - a^6*b^2*x*z^2 - a^4*b^4*x*z^2 + 2*a^2*b^6*x*z^2 +      2*a^4*b^2*c^2*x*z^2 + b^6*c^2*x*z^2 - 2*a^2*b^2*c^4*x*z^2 - 2*b^4*c^4*x*z^2 + b^2*c^6*x*z^2 -      2*a^6*b^2*y*z^2 + a^4*b^4*y*z^2 + a^2*b^6*y*z^2 - a^6*c^2*y*z^2 - 2*a^2*b^4*c^2*y*z^2 +      2*a^4*c^4*y*z^2 + 2*a^2*b^2*c^4*y*z^2 - a^2*c^6*y*z^2");
		exp("K063","2*a^6*c^2*x^2*y - 5*a^4*b^2*c^2*x^2*y + 8*a^2*b^4*c^2*x^2*y - 3*b^6*c^2*x^2*y - a^4*c^4*x^2*y -      2*b^4*c^4*x^2*y - 2*a^2*c^6*x^2*y + 2*b^2*c^6*x^2*y + c^8*x^2*y + 3*a^6*c^2*x*y^2 -      8*a^4*b^2*c^2*x*y^2 + 5*a^2*b^4*c^2*x*y^2 - 2*b^6*c^2*x*y^2 + 2*a^4*c^4*x*y^2 + b^4*c^4*x*y^2 -      2*a^2*c^6*x*y^2 + 2*b^2*c^6*x*y^2 - c^8*x*y^2 - 2*a^6*b^2*x^2*z + a^4*b^4*x^2*z + 2*a^2*b^6*x^2*z -      b^8*x^2*z + 5*a^4*b^2*c^2*x^2*z - 2*b^6*c^2*x^2*z - 8*a^2*b^2*c^4*x^2*z + 2*b^4*c^4*x^2*z +      3*b^2*c^6*x^2*z + a^8*y^2*z - 2*a^6*b^2*y^2*z - a^4*b^4*y^2*z + 2*a^2*b^6*y^2*z + 2*a^6*c^2*y^2*z -      5*a^2*b^4*c^2*y^2*z - 2*a^4*c^4*y^2*z + 8*a^2*b^2*c^4*y^2*z - 3*a^2*c^6*y^2*z - 3*a^6*b^2*x*z^2 -      2*a^4*b^4*x*z^2 + 2*a^2*b^6*x*z^2 + b^8*x*z^2 + 8*a^4*b^2*c^2*x*z^2 - 2*b^6*c^2*x*z^2 -      5*a^2*b^2*c^4*x*z^2 - b^4*c^4*x*z^2 + 2*b^2*c^6*x*z^2 - a^8*y*z^2 - 2*a^6*b^2*y*z^2 +      2*a^4*b^4*y*z^2 + 3*a^2*b^6*y*z^2 + 2*a^6*c^2*y*z^2 - 8*a^2*b^4*c^2*y*z^2 + a^4*c^4*y*z^2 +      5*a^2*b^2*c^4*y*z^2 - 2*a^2*c^6*y*z^2");
		exp("K064","(-a^4)*x^2*y + 2*a^2*b^2*x^2*y - b^4*x^2*y + 2*a^2*c^2*x^2*y - b^2*c^2*x^2*y - c^4*x^2*y -      a^4*x*y^2 + 2*a^2*b^2*x*y^2 - b^4*x*y^2 - a^2*c^2*x*y^2 + 2*b^2*c^2*x*y^2 - c^4*x*y^2 - a^4*x^2*z +      2*a^2*b^2*x^2*z - b^4*x^2*z + 2*a^2*c^2*x^2*z - b^2*c^2*x^2*z - c^4*x^2*z - a^4*y^2*z +      2*a^2*b^2*y^2*z - b^4*y^2*z - a^2*c^2*y^2*z + 2*b^2*c^2*y^2*z - c^4*y^2*z - a^4*x*z^2 -      a^2*b^2*x*z^2 - b^4*x*z^2 + 2*a^2*c^2*x*z^2 + 2*b^2*c^2*x*z^2 - c^4*x*z^2 - a^4*y*z^2 -      a^2*b^2*y*z^2 - b^4*y*z^2 + 2*a^2*c^2*y*z^2 + 2*b^2*c^2*y*z^2 - c^4*y*z^2");
		exp("K065","(-a^4)*x^2*y + b^4*x^2*y + 2*a^2*c^2*x^2*y + b^2*c^2*x^2*y - 3*c^4*x^2*y - a^4*x*y^2 +      b^4*x*y^2 - a^2*c^2*x*y^2 - 2*b^2*c^2*x*y^2 + 3*c^4*x*y^2 + a^4*x^2*z - 2*a^2*b^2*x^2*z +      3*b^4*x^2*z - b^2*c^2*x^2*z - c^4*x^2*z - 3*a^4*y^2*z + 2*a^2*b^2*y^2*z - b^4*y^2*z +      a^2*c^2*y^2*z + c^4*y^2*z + a^4*x*z^2 + a^2*b^2*x*z^2 - 3*b^4*x*z^2 + 2*b^2*c^2*x*z^2 - c^4*x*z^2 +      3*a^4*y*z^2 - a^2*b^2*y*z^2 - b^4*y*z^2 - 2*a^2*c^2*y*z^2 + c^4*y*z^2");
		exp("K066a","a^6*x^2*y - 3*a^4*b^2*x^2*y + 3*a^2*b^4*x^2*y - b^6*x^2*y - 4*a^4*c^2*x^2*y -      2*a^2*b^2*c^2*x^2*y + 6*b^4*c^2*x^2*y + 5*a^2*c^4*x^2*y - 3*b^2*c^4*x^2*y - 2*c^6*x^2*y +      a^6*x*y^2 - 3*a^4*b^2*x*y^2 + 3*a^2*b^4*x*y^2 - b^6*x*y^2 - 6*a^4*c^2*x*y^2 + 2*a^2*b^2*c^2*x*y^2 +      4*b^4*c^2*x*y^2 + 3*a^2*c^4*x*y^2 - 5*b^2*c^4*x*y^2 + 2*c^6*x*y^2 - a^6*x^2*z + 4*a^4*b^2*x^2*z -      5*a^2*b^4*x^2*z + 2*b^6*x^2*z + 3*a^4*c^2*x^2*z + 2*a^2*b^2*c^2*x^2*z + 3*b^4*c^2*x^2*z -      3*a^2*c^4*x^2*z - 6*b^2*c^4*x^2*z + c^6*x^2*z - 2*a^6*y^2*z + 5*a^4*b^2*y^2*z - 4*a^2*b^4*y^2*z +      b^6*y^2*z - 3*a^4*c^2*y^2*z - 2*a^2*b^2*c^2*y^2*z - 3*b^4*c^2*y^2*z + 6*a^2*c^4*y^2*z +      3*b^2*c^4*y^2*z - c^6*y^2*z - a^6*x*z^2 + 6*a^4*b^2*x*z^2 - 3*a^2*b^4*x*z^2 - 2*b^6*x*z^2 +      3*a^4*c^2*x*z^2 - 2*a^2*b^2*c^2*x*z^2 + 5*b^4*c^2*x*z^2 - 3*a^2*c^4*x*z^2 - 4*b^2*c^4*x*z^2 +      c^6*x*z^2 + 2*a^6*y*z^2 + 3*a^4*b^2*y*z^2 - 6*a^2*b^4*y*z^2 + b^6*y*z^2 - 5*a^4*c^2*y*z^2 +      2*a^2*b^2*c^2*y*z^2 - 3*b^4*c^2*y*z^2 + 4*a^2*c^4*y*z^2 + 3*b^2*c^4*y*z^2 - c^6*y*z^2 +      Sqrt(3)*Sqrt((a + b - c)*(a - b + c)*(-a + b + c)*(a + b + c))*((-a^4)*x^2*y + b^4*x^2*y +        a^2*c^2*x^2*y + b^2*c^2*x^2*y - 2*c^4*x^2*y - a^4*x*y^2 + b^4*x*y^2 - a^2*c^2*x*y^2 -        b^2*c^2*x*y^2 + 2*c^4*x*y^2 + a^4*x^2*z - a^2*b^2*x^2*z + 2*b^4*x^2*z - b^2*c^2*x^2*z -        c^4*x^2*z - 2*a^4*y^2*z + a^2*b^2*y^2*z - b^4*y^2*z + a^2*c^2*y^2*z + c^4*y^2*z + a^4*x*z^2 +        a^2*b^2*x*z^2 - 2*b^4*x*z^2 + b^2*c^2*x*z^2 - c^4*x*z^2 + 2*a^4*y*z^2 - a^2*b^2*y*z^2 -        b^4*y*z^2 - a^2*c^2*y*z^2 + c^4*y*z^2)");
		exp("K066b","a^6*x^2*y - 3*a^4*b^2*x^2*y + 3*a^2*b^4*x^2*y - b^6*x^2*y - 4*a^4*c^2*x^2*y -      2*a^2*b^2*c^2*x^2*y + 6*b^4*c^2*x^2*y + 5*a^2*c^4*x^2*y - 3*b^2*c^4*x^2*y - 2*c^6*x^2*y +      a^6*x*y^2 - 3*a^4*b^2*x*y^2 + 3*a^2*b^4*x*y^2 - b^6*x*y^2 - 6*a^4*c^2*x*y^2 + 2*a^2*b^2*c^2*x*y^2 +      4*b^4*c^2*x*y^2 + 3*a^2*c^4*x*y^2 - 5*b^2*c^4*x*y^2 + 2*c^6*x*y^2 - a^6*x^2*z + 4*a^4*b^2*x^2*z -      5*a^2*b^4*x^2*z + 2*b^6*x^2*z + 3*a^4*c^2*x^2*z + 2*a^2*b^2*c^2*x^2*z + 3*b^4*c^2*x^2*z -      3*a^2*c^4*x^2*z - 6*b^2*c^4*x^2*z + c^6*x^2*z - 2*a^6*y^2*z + 5*a^4*b^2*y^2*z - 4*a^2*b^4*y^2*z +      b^6*y^2*z - 3*a^4*c^2*y^2*z - 2*a^2*b^2*c^2*y^2*z - 3*b^4*c^2*y^2*z + 6*a^2*c^4*y^2*z +      3*b^2*c^4*y^2*z - c^6*y^2*z - a^6*x*z^2 + 6*a^4*b^2*x*z^2 - 3*a^2*b^4*x*z^2 - 2*b^6*x*z^2 +      3*a^4*c^2*x*z^2 - 2*a^2*b^2*c^2*x*z^2 + 5*b^4*c^2*x*z^2 - 3*a^2*c^4*x*z^2 - 4*b^2*c^4*x*z^2 +      c^6*x*z^2 + 2*a^6*y*z^2 + 3*a^4*b^2*y*z^2 - 6*a^2*b^4*y*z^2 + b^6*y*z^2 - 5*a^4*c^2*y*z^2 +      2*a^2*b^2*c^2*y*z^2 - 3*b^4*c^2*y*z^2 + 4*a^2*c^4*y*z^2 + 3*b^2*c^4*y*z^2 - c^6*y*z^2 +      Sqrt(3)*Sqrt((a + b - c)*(a - b + c)*(-a + b + c)*(a + b + c))*(a^4*x^2*y - b^4*x^2*y -        a^2*c^2*x^2*y - b^2*c^2*x^2*y + 2*c^4*x^2*y + a^4*x*y^2 - b^4*x*y^2 + a^2*c^2*x*y^2 +        b^2*c^2*x*y^2 - 2*c^4*x*y^2 - a^4*x^2*z + a^2*b^2*x^2*z - 2*b^4*x^2*z + b^2*c^2*x^2*z +        c^4*x^2*z + 2*a^4*y^2*z - a^2*b^2*y^2*z + b^4*y^2*z - a^2*c^2*y^2*z - c^4*y^2*z - a^4*x*z^2 -        a^2*b^2*x*z^2 + 2*b^4*x*z^2 - b^2*c^2*x*z^2 + c^4*x*z^2 - 2*a^4*y*z^2 + a^2*b^2*y*z^2 +        b^4*y*z^2 + a^2*c^2*y*z^2 - c^4*y*z^2)");
		exp("K067","a^10*c^2*x^2*y - 2*a^8*b^2*c^2*x^2*y + 2*a^4*b^6*c^2*x^2*y - a^2*b^8*c^2*x^2*y -      5*a^8*c^4*x^2*y + 5*a^6*b^2*c^4*x^2*y - a^4*b^4*c^4*x^2*y + 2*a^2*b^6*c^4*x^2*y - b^8*c^4*x^2*y +      10*a^6*c^6*x^2*y + a^2*b^4*c^6*x^2*y + 4*b^6*c^6*x^2*y - 10*a^4*c^8*x^2*y - 7*a^2*b^2*c^8*x^2*y -      6*b^4*c^8*x^2*y + 5*a^2*c^10*x^2*y + 4*b^2*c^10*x^2*y - c^12*x^2*y + a^8*b^2*c^2*x*y^2 -      2*a^6*b^4*c^2*x*y^2 + 2*a^2*b^8*c^2*x*y^2 - b^10*c^2*x*y^2 + a^8*c^4*x*y^2 - 2*a^6*b^2*c^4*x*y^2 +      a^4*b^4*c^4*x*y^2 - 5*a^2*b^6*c^4*x*y^2 + 5*b^8*c^4*x*y^2 - 4*a^6*c^6*x*y^2 - a^4*b^2*c^6*x*y^2 -      10*b^6*c^6*x*y^2 + 6*a^4*c^8*x*y^2 + 7*a^2*b^2*c^8*x*y^2 + 10*b^4*c^8*x*y^2 - 4*a^2*c^10*x*y^2 -      5*b^2*c^10*x*y^2 + c^12*x*y^2 - a^10*b^2*x^2*z + 5*a^8*b^4*x^2*z - 10*a^6*b^6*x^2*z +      10*a^4*b^8*x^2*z - 5*a^2*b^10*x^2*z + b^12*x^2*z + 2*a^8*b^2*c^2*x^2*z - 5*a^6*b^4*c^2*x^2*z +      7*a^2*b^8*c^2*x^2*z - 4*b^10*c^2*x^2*z + a^4*b^4*c^4*x^2*z - a^2*b^6*c^4*x^2*z + 6*b^8*c^4*x^2*z -      2*a^4*b^2*c^6*x^2*z - 2*a^2*b^4*c^6*x^2*z - 4*b^6*c^6*x^2*z + a^2*b^2*c^8*x^2*z + b^4*c^8*x^2*z -      a^12*y^2*z + 5*a^10*b^2*y^2*z - 10*a^8*b^4*y^2*z + 10*a^6*b^6*y^2*z - 5*a^4*b^8*y^2*z +      a^2*b^10*y^2*z + 4*a^10*c^2*y^2*z - 7*a^8*b^2*c^2*y^2*z + 5*a^4*b^6*c^2*y^2*z -      2*a^2*b^8*c^2*y^2*z - 6*a^8*c^4*y^2*z + a^6*b^2*c^4*y^2*z - a^4*b^4*c^4*y^2*z + 4*a^6*c^6*y^2*z +      2*a^4*b^2*c^6*y^2*z + 2*a^2*b^4*c^6*y^2*z - a^4*c^8*y^2*z - a^2*b^2*c^8*y^2*z - a^8*b^4*x*z^2 +      4*a^6*b^6*x*z^2 - 6*a^4*b^8*x*z^2 + 4*a^2*b^10*x*z^2 - b^12*x*z^2 - a^8*b^2*c^2*x*z^2 +      2*a^6*b^4*c^2*x*z^2 + a^4*b^6*c^2*x*z^2 - 7*a^2*b^8*c^2*x*z^2 + 5*b^10*c^2*x*z^2 +      2*a^6*b^2*c^4*x*z^2 - a^4*b^4*c^4*x*z^2 - 10*b^8*c^4*x*z^2 + 5*a^2*b^4*c^6*x*z^2 +      10*b^6*c^6*x*z^2 - 2*a^2*b^2*c^8*x*z^2 - 5*b^4*c^8*x*z^2 + b^2*c^10*x*z^2 + a^12*y*z^2 -      4*a^10*b^2*y*z^2 + 6*a^8*b^4*y*z^2 - 4*a^6*b^6*y*z^2 + a^4*b^8*y*z^2 - 5*a^10*c^2*y*z^2 +      7*a^8*b^2*c^2*y*z^2 - a^6*b^4*c^2*y*z^2 - 2*a^4*b^6*c^2*y*z^2 + a^2*b^8*c^2*y*z^2 +      10*a^8*c^4*y*z^2 + a^4*b^4*c^4*y*z^2 - 2*a^2*b^6*c^4*y*z^2 - 10*a^6*c^6*y*z^2 -      5*a^4*b^2*c^6*y*z^2 + 5*a^4*c^8*y*z^2 + 2*a^2*b^2*c^8*y*z^2 - a^2*c^10*y*z^2");
		exp("K068","a^4*x^2*y - 3*a^2*b^2*x^2*y + 2*b^4*x^2*y + a^2*c^2*x^2*y - b^2*c^2*x^2*y - 2*a^4*x*y^2 +      3*a^2*b^2*x*y^2 - b^4*x*y^2 + a^2*c^2*x*y^2 - b^2*c^2*x*y^2 - a^4*x^2*z - a^2*b^2*x^2*z +      3*a^2*c^2*x^2*z + b^2*c^2*x^2*z - 2*c^4*x^2*z + a^2*b^2*y^2*z + b^4*y^2*z - a^2*c^2*y^2*z -      3*b^2*c^2*y^2*z + 2*c^4*y^2*z + 2*a^4*x*z^2 - a^2*b^2*x*z^2 - 3*a^2*c^2*x*z^2 + b^2*c^2*x*z^2 +      c^4*x*z^2 + a^2*b^2*y*z^2 - 2*b^4*y*z^2 - a^2*c^2*y*z^2 + 3*b^2*c^2*y*z^2 - c^4*y*z^2");
		exp("K069","a^10*c^2*x^2*y - 4*a^6*b^4*c^2*x^2*y + 2*a^4*b^6*c^2*x^2*y + 3*a^2*b^8*c^2*x^2*y -      2*b^10*c^2*x^2*y - 2*a^8*c^4*x^2*y + 5*a^6*b^2*c^4*x^2*y + a^4*b^4*c^4*x^2*y -      9*a^2*b^6*c^4*x^2*y + 5*b^8*c^4*x^2*y - 5*a^4*b^2*c^6*x^2*y + 8*a^2*b^4*c^6*x^2*y -      3*b^6*c^6*x^2*y + 2*a^4*c^8*x^2*y - a^2*b^2*c^8*x^2*y - b^4*c^8*x^2*y - a^2*c^10*x^2*y +      b^2*c^10*x^2*y + 2*a^10*c^2*x*y^2 - 3*a^8*b^2*c^2*x*y^2 - 2*a^6*b^4*c^2*x*y^2 +      4*a^4*b^6*c^2*x*y^2 - b^10*c^2*x*y^2 - 5*a^8*c^4*x*y^2 + 9*a^6*b^2*c^4*x*y^2 - a^4*b^4*c^4*x*y^2 -      5*a^2*b^6*c^4*x*y^2 + 2*b^8*c^4*x*y^2 + 3*a^6*c^6*x*y^2 - 8*a^4*b^2*c^6*x*y^2 +      5*a^2*b^4*c^6*x*y^2 + a^4*c^8*x*y^2 + a^2*b^2*c^8*x*y^2 - 2*b^4*c^8*x*y^2 - a^2*c^10*x*y^2 +      b^2*c^10*x*y^2 - a^10*b^2*x^2*z + 2*a^8*b^4*x^2*z - 2*a^4*b^8*x^2*z + a^2*b^10*x^2*z -      5*a^6*b^4*c^2*x^2*z + 5*a^4*b^6*c^2*x^2*z + a^2*b^8*c^2*x^2*z - b^10*c^2*x^2*z +      4*a^6*b^2*c^4*x^2*z - a^4*b^4*c^4*x^2*z - 8*a^2*b^6*c^4*x^2*z + b^8*c^4*x^2*z -      2*a^4*b^2*c^6*x^2*z + 9*a^2*b^4*c^6*x^2*z + 3*b^6*c^6*x^2*z - 3*a^2*b^2*c^8*x^2*z -      5*b^4*c^8*x^2*z + 2*b^2*c^10*x^2*z - 2*a^10*b^2*x*y*z + 4*a^8*b^4*x*y*z - 4*a^4*b^8*x*y*z +      2*a^2*b^10*x*y*z + 2*a^10*c^2*x*y*z - 18*a^6*b^4*c^2*x*y*z + 18*a^4*b^6*c^2*x*y*z -      2*b^10*c^2*x*y*z - 4*a^8*c^4*x*y*z + 18*a^6*b^2*c^4*x*y*z - 18*a^2*b^6*c^4*x*y*z +      4*b^8*c^4*x*y*z - 18*a^4*b^2*c^6*x*y*z + 18*a^2*b^4*c^6*x*y*z + 4*a^4*c^8*x*y*z - 4*b^4*c^8*x*y*z -      2*a^2*c^10*x*y*z + 2*b^2*c^10*x*y*z - a^10*b^2*y^2*z + 2*a^8*b^4*y^2*z - 2*a^4*b^8*y^2*z +      a^2*b^10*y^2*z + a^10*c^2*y^2*z - a^8*b^2*c^2*y^2*z - 5*a^6*b^4*c^2*y^2*z + 5*a^4*b^6*c^2*y^2*z -      a^8*c^4*y^2*z + 8*a^6*b^2*c^4*y^2*z + a^4*b^4*c^4*y^2*z - 4*a^2*b^6*c^4*y^2*z - 3*a^6*c^6*y^2*z -      9*a^4*b^2*c^6*y^2*z + 2*a^2*b^4*c^6*y^2*z + 5*a^4*c^8*y^2*z + 3*a^2*b^2*c^8*y^2*z -      2*a^2*c^10*y^2*z - 2*a^10*b^2*x*z^2 + 5*a^8*b^4*x*z^2 - 3*a^6*b^6*x*z^2 - a^4*b^8*x*z^2 +      a^2*b^10*x*z^2 + 3*a^8*b^2*c^2*x*z^2 - 9*a^6*b^4*c^2*x*z^2 + 8*a^4*b^6*c^2*x*z^2 -      a^2*b^8*c^2*x*z^2 - b^10*c^2*x*z^2 + 2*a^6*b^2*c^4*x*z^2 + a^4*b^4*c^4*x*z^2 -      5*a^2*b^6*c^4*x*z^2 + 2*b^8*c^4*x*z^2 - 4*a^4*b^2*c^6*x*z^2 + 5*a^2*b^4*c^6*x*z^2 -      2*b^4*c^8*x*z^2 + b^2*c^10*x*z^2 - a^10*b^2*y*z^2 + a^8*b^4*y*z^2 + 3*a^6*b^6*y*z^2 -      5*a^4*b^8*y*z^2 + 2*a^2*b^10*y*z^2 + a^10*c^2*y*z^2 + a^8*b^2*c^2*y*z^2 - 8*a^6*b^4*c^2*y*z^2 +      9*a^4*b^6*c^2*y*z^2 - 3*a^2*b^8*c^2*y*z^2 - 2*a^8*c^4*y*z^2 + 5*a^6*b^2*c^4*y*z^2 -      a^4*b^4*c^4*y*z^2 - 2*a^2*b^6*c^4*y*z^2 - 5*a^4*b^2*c^6*y*z^2 + 4*a^2*b^4*c^6*y*z^2 +      2*a^4*c^8*y*z^2 - a^2*c^10*y*z^2");
		exp("K070a","(-a^4)*x^2*y + 2*a^2*b^2*x^2*y - b^4*x^2*y + c^4*x^2*y + a^4*x*y^2 - 2*a^2*b^2*x*y^2 +      b^4*x*y^2 - c^4*x*y^2 + a^4*x^2*z - b^4*x^2*z - 2*a^2*c^2*x^2*z + c^4*x^2*z + a^4*y^2*z -      b^4*y^2*z + 2*b^2*c^2*y^2*z - c^4*y^2*z - a^4*x*z^2 + b^4*x*z^2 + 2*a^2*c^2*x*z^2 - c^4*x*z^2 -      a^4*y*z^2 + b^4*y*z^2 - 2*b^2*c^2*y*z^2 + c^4*y*z^2 +      Sqrt((a + b - c)*(a - b + c)*(-a + b + c)*(a + b + c))*(a^2*x^2*y - b^2*x^2*y - c^2*x^2*y +        a^2*x*y^2 - b^2*x*y^2 + c^2*x*y^2 - a^2*x^2*z + b^2*x^2*z + c^2*x^2*z - a^2*y^2*z + b^2*y^2*z -        c^2*y^2*z - a^2*x*z^2 - b^2*x*z^2 + c^2*x*z^2 + a^2*y*z^2 + b^2*y*z^2 - c^2*y*z^2)");
		exp("K070b","(-a^4)*x^2*y + 2*a^2*b^2*x^2*y - b^4*x^2*y + c^4*x^2*y + a^4*x*y^2 - 2*a^2*b^2*x*y^2 +      b^4*x*y^2 - c^4*x*y^2 + a^4*x^2*z - b^4*x^2*z - 2*a^2*c^2*x^2*z + c^4*x^2*z + a^4*y^2*z -      b^4*y^2*z + 2*b^2*c^2*y^2*z - c^4*y^2*z - a^4*x*z^2 + b^4*x*z^2 + 2*a^2*c^2*x*z^2 - c^4*x*z^2 -      a^4*y*z^2 + b^4*y*z^2 - 2*b^2*c^2*y*z^2 + c^4*y*z^2 +      Sqrt((a + b - c)*(a - b + c)*(-a + b + c)*(a + b + c))*((-a^2)*x^2*y + b^2*x^2*y + c^2*x^2*y -        a^2*x*y^2 + b^2*x*y^2 - c^2*x*y^2 + a^2*x^2*z - b^2*x^2*z - c^2*x^2*z + a^2*y^2*z - b^2*y^2*z +        c^2*y^2*z + a^2*x*z^2 + b^2*x*z^2 - c^2*x*z^2 - a^2*y*z^2 - b^2*y*z^2 + c^2*y*z^2)");
		exp("K071","a^6*x^2*y - 3*a^4*b^2*x^2*y + 3*a^2*b^4*x^2*y - b^6*x^2*y - 2*a^2*b^2*c^2*x^2*y +      2*b^4*c^2*x^2*y - a^2*c^4*x^2*y - b^2*c^4*x^2*y + a^6*x*y^2 - 3*a^4*b^2*x*y^2 + 3*a^2*b^4*x*y^2 -      b^6*x*y^2 - 2*a^4*c^2*x*y^2 + 2*a^2*b^2*c^2*x*y^2 + a^2*c^4*x*y^2 + b^2*c^4*x*y^2 - a^6*x^2*z +      a^2*b^4*x^2*z + 3*a^4*c^2*x^2*z + 2*a^2*b^2*c^2*x^2*z + b^4*c^2*x^2*z - 3*a^2*c^4*x^2*z -      2*b^2*c^4*x^2*z + c^6*x^2*z - 4*a^4*b^2*x*y*z + 4*a^2*b^4*x*y*z + 4*a^4*c^2*x*y*z -      4*b^4*c^2*x*y*z - 4*a^2*c^4*x*y*z + 4*b^2*c^4*x*y*z - a^4*b^2*y^2*z + b^6*y^2*z - a^4*c^2*y^2*z -      2*a^2*b^2*c^2*y^2*z - 3*b^4*c^2*y^2*z + 2*a^2*c^4*y^2*z + 3*b^2*c^4*y^2*z - c^6*y^2*z - a^6*x*z^2 +      2*a^4*b^2*x*z^2 - a^2*b^4*x*z^2 + 3*a^4*c^2*x*z^2 - 2*a^2*b^2*c^2*x*z^2 - b^4*c^2*x*z^2 -      3*a^2*c^4*x*z^2 + c^6*x*z^2 + a^4*b^2*y*z^2 - 2*a^2*b^4*y*z^2 + b^6*y*z^2 + a^4*c^2*y*z^2 +      2*a^2*b^2*c^2*y*z^2 - 3*b^4*c^2*y*z^2 + 3*b^2*c^4*y*z^2 - c^6*y*z^2");
		exp("K072","-(a^6*c^2*x^2*y) + a^2*b^4*c^2*x^2*y + 2*a^4*c^4*x^2*y - b^4*c^4*x^2*y - 2*a^2*c^6*x^2*y +      c^8*x^2*y - a^4*b^2*c^2*x*y^2 + b^6*c^2*x*y^2 + a^4*c^4*x*y^2 - 2*b^4*c^4*x*y^2 + 2*b^2*c^6*x*y^2 -      c^8*x*y^2 + a^6*b^2*x^2*z - 2*a^4*b^4*x^2*z + 2*a^2*b^6*x^2*z - b^8*x^2*z - a^2*b^2*c^4*x^2*z +      b^4*c^4*x^2*z + a^6*b^2*x*y*z - a^2*b^6*x*y*z - a^6*c^2*x*y*z + b^6*c^2*x*y*z + a^2*c^6*x*y*z -      b^2*c^6*x*y*z + a^8*y^2*z - 2*a^6*b^2*y^2*z + 2*a^4*b^4*y^2*z - a^2*b^6*y^2*z - a^4*c^4*y^2*z +      a^2*b^2*c^4*y^2*z - a^4*b^4*x*z^2 + b^8*x*z^2 + a^4*b^2*c^2*x*z^2 - 2*b^6*c^2*x*z^2 +      2*b^4*c^4*x*z^2 - b^2*c^6*x*z^2 - a^8*y*z^2 + a^4*b^4*y*z^2 + 2*a^6*c^2*y*z^2 - a^2*b^4*c^2*y*z^2 -      2*a^4*c^4*y*z^2 + a^2*c^6*y*z^2");
		exp("K073","a^6*b^2*c^4*x^2*y - b^8*c^4*x^2*y - a^4*b^2*c^6*x^2*y + a^2*b^4*c^6*x^2*y + 3*b^6*c^6*x^2*y -      a^2*b^2*c^8*x^2*y - 3*b^4*c^8*x^2*y + b^2*c^10*x^2*y + a^8*c^4*x*y^2 - a^2*b^6*c^4*x*y^2 -      3*a^6*c^6*x*y^2 - a^4*b^2*c^6*x*y^2 + a^2*b^4*c^6*x*y^2 + 3*a^4*c^8*x*y^2 + a^2*b^2*c^8*x*y^2 -      a^2*c^10*x*y^2 - a^6*b^4*c^2*x^2*z + a^4*b^6*c^2*x^2*z + a^2*b^8*c^2*x^2*z - b^10*c^2*x^2*z -      a^2*b^6*c^4*x^2*z + 3*b^8*c^4*x^2*z - 3*b^6*c^6*x^2*z + b^4*c^8*x^2*z + a^10*c^2*y^2*z -      a^8*b^2*c^2*y^2*z - a^6*b^4*c^2*y^2*z + a^4*b^6*c^2*y^2*z - 3*a^8*c^4*y^2*z + a^6*b^2*c^4*y^2*z +      3*a^6*c^6*y^2*z - a^4*c^8*y^2*z - a^8*b^4*x*z^2 + 3*a^6*b^6*x*z^2 - 3*a^4*b^8*x*z^2 +      a^2*b^10*x*z^2 + a^4*b^6*c^2*x*z^2 - a^2*b^8*c^2*x*z^2 - a^2*b^6*c^4*x*z^2 + a^2*b^4*c^6*x*z^2 -      a^10*b^2*y*z^2 + 3*a^8*b^4*y*z^2 - 3*a^6*b^6*y*z^2 + a^4*b^8*y*z^2 + a^8*b^2*c^2*y*z^2 -      a^6*b^4*c^2*y*z^2 + a^6*b^2*c^4*y*z^2 - a^4*b^2*c^6*y*z^2");
		exp("K074","a^6*c^2*x^2*y - a^4*b^2*c^2*x^2*y - a^2*b^4*c^2*x^2*y + b^6*c^2*x^2*y - a^4*c^4*x^2*y +      2*a^2*b^2*c^4*x^2*y - b^4*c^4*x^2*y - a^2*c^6*x^2*y - b^2*c^6*x^2*y + c^8*x^2*y + a^6*c^2*x*y^2 -      a^4*b^2*c^2*x*y^2 - a^2*b^4*c^2*x*y^2 + b^6*c^2*x*y^2 - a^4*c^4*x*y^2 + 2*a^2*b^2*c^4*x*y^2 -      b^4*c^4*x*y^2 - a^2*c^6*x*y^2 - b^2*c^6*x*y^2 + c^8*x*y^2 + a^6*b^2*x^2*z - a^4*b^4*x^2*z -      a^2*b^6*x^2*z + b^8*x^2*z - a^4*b^2*c^2*x^2*z + 2*a^2*b^4*c^2*x^2*z - b^6*c^2*x^2*z -      a^2*b^2*c^4*x^2*z - b^4*c^4*x^2*z + b^2*c^6*x^2*z + 2*a^8*x*y*z - 4*a^6*b^2*x*y*z +      4*a^4*b^4*x*y*z - 4*a^2*b^6*x*y*z + 2*b^8*x*y*z - 4*a^6*c^2*x*y*z + 4*a^4*b^2*c^2*x*y*z +      4*a^2*b^4*c^2*x*y*z - 4*b^6*c^2*x*y*z + 4*a^4*c^4*x*y*z + 4*a^2*b^2*c^4*x*y*z + 4*b^4*c^4*x*y*z -      4*a^2*c^6*x*y*z - 4*b^2*c^6*x*y*z + 2*c^8*x*y*z + a^8*y^2*z - a^6*b^2*y^2*z - a^4*b^4*y^2*z +      a^2*b^6*y^2*z - a^6*c^2*y^2*z + 2*a^4*b^2*c^2*y^2*z - a^2*b^4*c^2*y^2*z - a^4*c^4*y^2*z -      a^2*b^2*c^4*y^2*z + a^2*c^6*y^2*z + a^6*b^2*x*z^2 - a^4*b^4*x*z^2 - a^2*b^6*x*z^2 + b^8*x*z^2 -      a^4*b^2*c^2*x*z^2 + 2*a^2*b^4*c^2*x*z^2 - b^6*c^2*x*z^2 - a^2*b^2*c^4*x*z^2 - b^4*c^4*x*z^2 +      b^2*c^6*x*z^2 + a^8*y*z^2 - a^6*b^2*y*z^2 - a^4*b^4*y*z^2 + a^2*b^6*y*z^2 - a^6*c^2*y*z^2 +      2*a^4*b^2*c^2*y*z^2 - a^2*b^4*c^2*y*z^2 - a^4*c^4*y*z^2 - a^2*b^2*c^4*y*z^2 + a^2*c^6*y*z^2");
		exp("K075","2*b^8*c^4*x^3 - 2*b^4*c^8*x^3 + a^4*b^4*c^4*x^2*y + b^8*c^4*x^2*y - b^4*c^8*x^2*y -      a^8*c^4*x*y^2 - a^4*b^4*c^4*x*y^2 + a^4*c^8*x*y^2 - 2*a^8*c^4*y^3 + 2*a^4*c^8*y^3 -      a^4*b^4*c^4*x^2*z + b^8*c^4*x^2*z - b^4*c^8*x^2*z - a^8*c^4*y^2*z + a^4*b^4*c^4*y^2*z +      a^4*c^8*y^2*z + a^8*b^4*x*z^2 - a^4*b^8*x*z^2 + a^4*b^4*c^4*x*z^2 + a^8*b^4*y*z^2 - a^4*b^8*y*z^2 -      a^4*b^4*c^4*y*z^2 + 2*a^8*b^4*z^3 - 2*a^4*b^8*z^3");
		exp("K076","2*b^8*c^4*x^3 - 2*b^4*c^8*x^3 + 3*a^4*b^4*c^4*x^2*y - b^8*c^4*x^2*y + b^4*c^8*x^2*y +      a^8*c^4*x*y^2 - 3*a^4*b^4*c^4*x*y^2 - a^4*c^8*x*y^2 - 2*a^8*c^4*y^3 + 2*a^4*c^8*y^3 -      3*a^4*b^4*c^4*x^2*z - b^8*c^4*x^2*z + b^4*c^8*x^2*z + a^8*c^4*y^2*z + 3*a^4*b^4*c^4*y^2*z -      a^4*c^8*y^2*z - a^8*b^4*x*z^2 + a^4*b^8*x*z^2 + 3*a^4*b^4*c^4*x*z^2 - a^8*b^4*y*z^2 +      a^4*b^8*y*z^2 - 3*a^4*b^4*c^4*y*z^2 + 2*a^8*b^4*z^3 - 2*a^4*b^8*z^3");
		exp("K077","2*b^4*c^2*x^3 - 2*b^2*c^4*x^3 + a^2*b^2*c^2*x^2*y + b^4*c^2*x^2*y - b^2*c^4*x^2*y -      a^4*c^2*x*y^2 - a^2*b^2*c^2*x*y^2 + a^2*c^4*x*y^2 - 2*a^4*c^2*y^3 + 2*a^2*c^4*y^3 -      a^2*b^2*c^2*x^2*z + b^4*c^2*x^2*z - b^2*c^4*x^2*z - a^4*c^2*y^2*z + a^2*b^2*c^2*y^2*z +      a^2*c^4*y^2*z + a^4*b^2*x*z^2 - a^2*b^4*x*z^2 + a^2*b^2*c^2*x*z^2 + a^4*b^2*y*z^2 - a^2*b^4*y*z^2 -      a^2*b^2*c^2*y*z^2 + 2*a^4*b^2*z^3 - 2*a^2*b^4*z^3");
		exp("K078","2*b^4*c^2*x^3 - 2*b^2*c^4*x^3 + 3*a^2*b^2*c^2*x^2*y - b^4*c^2*x^2*y + b^2*c^4*x^2*y +      a^4*c^2*x*y^2 - 3*a^2*b^2*c^2*x*y^2 - a^2*c^4*x*y^2 - 2*a^4*c^2*y^3 + 2*a^2*c^4*y^3 -      3*a^2*b^2*c^2*x^2*z - b^4*c^2*x^2*z + b^2*c^4*x^2*z + a^4*c^2*y^2*z + 3*a^2*b^2*c^2*y^2*z -      a^2*c^4*y^2*z - a^4*b^2*x*z^2 + a^2*b^4*x*z^2 + 3*a^2*b^2*c^2*x*z^2 - a^4*b^2*y*z^2 +      a^2*b^4*y*z^2 - 3*a^2*b^2*c^2*y*z^2 + 2*a^4*b^2*z^3 - 2*a^2*b^4*z^3");
		exp("K079","2*a^6*x^3 - 3*a^4*b^2*x^3 + a^2*b^4*x^3 - 3*a^4*c^2*x^3 + 4*a^2*b^2*c^2*x^3 - b^4*c^2*x^3 +      a^2*c^4*x^3 - b^2*c^4*x^3 + a^6*x^2*y - a^4*b^2*x^2*y - 2*a^4*c^2*x^2*y + 2*a^2*b^2*c^2*x^2*y +      a^2*c^4*x^2*y - b^2*c^4*x^2*y - a^2*b^4*x*y^2 + b^6*x*y^2 + 2*a^2*b^2*c^2*x*y^2 - 2*b^4*c^2*x*y^2 -      a^2*c^4*x*y^2 + b^2*c^4*x*y^2 + a^4*b^2*y^3 - 3*a^2*b^4*y^3 + 2*b^6*y^3 - a^4*c^2*y^3 +      4*a^2*b^2*c^2*y^3 - 3*b^4*c^2*y^3 - a^2*c^4*y^3 + b^2*c^4*y^3 + a^6*x^2*z - 2*a^4*b^2*x^2*z +      a^2*b^4*x^2*z - a^4*c^2*x^2*z + 2*a^2*b^2*c^2*x^2*z - b^4*c^2*x^2*z + a^4*b^2*y^2*z -      2*a^2*b^4*y^2*z + b^6*y^2*z - a^4*c^2*y^2*z + 2*a^2*b^2*c^2*y^2*z - b^4*c^2*y^2*z - a^2*b^4*x*z^2 +      2*a^2*b^2*c^2*x*z^2 + b^4*c^2*x*z^2 - a^2*c^4*x*z^2 - 2*b^2*c^4*x*z^2 + c^6*x*z^2 - a^4*b^2*y*z^2 +      a^4*c^2*y*z^2 + 2*a^2*b^2*c^2*y*z^2 - 2*a^2*c^4*y*z^2 - b^2*c^4*y*z^2 + c^6*y*z^2 - a^4*b^2*z^3 -      a^2*b^4*z^3 + a^4*c^2*z^3 + 4*a^2*b^2*c^2*z^3 + b^4*c^2*z^3 - 3*a^2*c^4*z^3 - 3*b^2*c^4*z^3 +      2*c^6*z^3");
		exp("K080","a^4*c^2*x^2*y + a^2*b^2*c^2*x^2*y - 2*b^4*c^2*x^2*y - a^2*c^4*x^2*y + 2*b^2*c^4*x^2*y +      2*a^4*c^2*x*y^2 - a^2*b^2*c^2*x*y^2 - b^4*c^2*x*y^2 - 2*a^2*c^4*x*y^2 + b^2*c^4*x*y^2 -      a^4*b^2*x^2*z + a^2*b^4*x^2*z - a^2*b^2*c^2*x^2*z - 2*b^4*c^2*x^2*z + 2*b^2*c^4*x^2*z -      2*a^4*b^2*x*y*z + 2*a^2*b^4*x*y*z + 2*a^4*c^2*x*y*z - 2*b^4*c^2*x*y*z - 2*a^2*c^4*x*y*z +      2*b^2*c^4*x*y*z - a^4*b^2*y^2*z + a^2*b^4*y^2*z + 2*a^4*c^2*y^2*z + a^2*b^2*c^2*y^2*z -      2*a^2*c^4*y^2*z - 2*a^4*b^2*x*z^2 + 2*a^2*b^4*x*z^2 + a^2*b^2*c^2*x*z^2 - b^4*c^2*x*z^2 +      b^2*c^4*x*z^2 - 2*a^4*b^2*y*z^2 + 2*a^2*b^4*y*z^2 + a^4*c^2*y*z^2 - a^2*b^2*c^2*y*z^2 -      a^2*c^4*y*z^2");
		exp("K081","2*a^6*c^2*x^2*y - 2*a^4*b^2*c^2*x^2*y - 2*a^2*b^4*c^2*x^2*y + 2*b^6*c^2*x^2*y - 2*a^4*c^4*x^2*y +      4*a^2*b^2*c^4*x^2*y - 2*b^4*c^4*x^2*y - 2*a^2*c^6*x^2*y - 2*b^2*c^6*x^2*y + 2*c^8*x^2*y +      2*a^6*c^2*x*y^2 - 2*a^4*b^2*c^2*x*y^2 - 2*a^2*b^4*c^2*x*y^2 + 2*b^6*c^2*x*y^2 - 2*a^4*c^4*x*y^2 +      4*a^2*b^2*c^4*x*y^2 - 2*b^4*c^4*x*y^2 - 2*a^2*c^6*x*y^2 - 2*b^2*c^6*x*y^2 + 2*c^8*x*y^2 +      2*a^6*b^2*x^2*z - 2*a^4*b^4*x^2*z - 2*a^2*b^6*x^2*z + 2*b^8*x^2*z - 2*a^4*b^2*c^2*x^2*z +      4*a^2*b^4*c^2*x^2*z - 2*b^6*c^2*x^2*z - 2*a^2*b^2*c^4*x^2*z - 2*b^4*c^4*x^2*z + 2*b^2*c^6*x^2*z +      3*a^8*x*y*z - 4*a^6*b^2*x*y*z + 2*a^4*b^4*x*y*z - 4*a^2*b^6*x*y*z + 3*b^8*x*y*z - 4*a^6*c^2*x*y*z +      4*a^4*b^2*c^2*x*y*z + 4*a^2*b^4*c^2*x*y*z - 4*b^6*c^2*x*y*z + 2*a^4*c^4*x*y*z +      4*a^2*b^2*c^4*x*y*z + 2*b^4*c^4*x*y*z - 4*a^2*c^6*x*y*z - 4*b^2*c^6*x*y*z + 3*c^8*x*y*z +      2*a^8*y^2*z - 2*a^6*b^2*y^2*z - 2*a^4*b^4*y^2*z + 2*a^2*b^6*y^2*z - 2*a^6*c^2*y^2*z +      4*a^4*b^2*c^2*y^2*z - 2*a^2*b^4*c^2*y^2*z - 2*a^4*c^4*y^2*z - 2*a^2*b^2*c^4*y^2*z +      2*a^2*c^6*y^2*z + 2*a^6*b^2*x*z^2 - 2*a^4*b^4*x*z^2 - 2*a^2*b^6*x*z^2 + 2*b^8*x*z^2 -      2*a^4*b^2*c^2*x*z^2 + 4*a^2*b^4*c^2*x*z^2 - 2*b^6*c^2*x*z^2 - 2*a^2*b^2*c^4*x*z^2 -      2*b^4*c^4*x*z^2 + 2*b^2*c^6*x*z^2 + 2*a^8*y*z^2 - 2*a^6*b^2*y*z^2 - 2*a^4*b^4*y*z^2 +      2*a^2*b^6*y*z^2 - 2*a^6*c^2*y*z^2 + 4*a^4*b^2*c^2*y*z^2 - 2*a^2*b^4*c^2*y*z^2 - 2*a^4*c^4*y*z^2 -      2*a^2*b^2*c^4*y*z^2 + 2*a^2*c^6*y*z^2");
		exp("K082","c^2*x^2*y + c^2*x*y^2 + b^2*x^2*z + a^2*y^2*z + b^2*x*z^2 + a^2*y*z^2");
		exp("K083A","(-c^2)*x^2*y - c^2*x*y^2 - c^2*y^3 + b^2*x^2*z + 2*b^2*x*y*z - 2*c^2*x*y*z + a^2*y^2*z -      c^2*y^2*z + b^2*x*z^2 - a^2*y*z^2 + b^2*y*z^2 + b^2*z^3");
		exp("K083B","c^2*x^3 + c^2*x^2*y + c^2*x*y^2 - b^2*x^2*z + c^2*x^2*z - 2*a^2*x*y*z + 2*c^2*x*y*z -      a^2*y^2*z - a^2*x*z^2 + b^2*x*z^2 - a^2*y*z^2 - a^2*z^3");
		exp("K083C","(-b^2)*x^3 - b^2*x^2*y + c^2*x^2*y + a^2*x*y^2 - c^2*x*y^2 + a^2*y^3 - b^2*x^2*z + 2*a^2*x*y*z -      2*b^2*x*y*z + a^2*y^2*z - b^2*x*z^2 + a^2*y*z^2");
		exp("K084","a^6*b^2*c^2*x^2*y - 3*a^4*b^4*c^2*x^2*y + a^2*b^6*c^2*x^2*y + 2*a^4*b^2*c^4*x^2*y +      b^6*c^4*x^2*y - 2*a^4*c^6*x^2*y + 2*a^2*b^2*c^6*x^2*y - 3*b^4*c^6*x^2*y + b^2*c^8*x^2*y +      a^6*b^2*c^2*x*y^2 - 3*a^4*b^4*c^2*x*y^2 + a^2*b^6*c^2*x*y^2 + a^6*c^4*x*y^2 + 2*a^2*b^4*c^4*x*y^2 -      3*a^4*c^6*x*y^2 + 2*a^2*b^2*c^6*x*y^2 - 2*b^4*c^6*x*y^2 + a^2*c^8*x*y^2 - 2*a^4*b^6*x^2*z +      a^6*b^2*c^2*x^2*z + 2*a^4*b^4*c^2*x^2*z + 2*a^2*b^6*c^2*x^2*z + b^8*c^2*x^2*z -      3*a^4*b^2*c^4*x^2*z - 3*b^6*c^4*x^2*z + a^2*b^2*c^6*x^2*z + b^4*c^6*x^2*z - 2*a^6*b^4*x*y*z -      2*a^4*b^6*x*y*z + 8*a^6*b^2*c^2*x*y*z - 4*a^4*b^4*c^2*x*y*z + 8*a^2*b^6*c^2*x*y*z -      2*a^6*c^4*x*y*z - 4*a^4*b^2*c^4*x*y*z - 4*a^2*b^4*c^4*x*y*z - 2*b^6*c^4*x*y*z - 2*a^4*c^6*x*y*z +      8*a^2*b^2*c^6*x*y*z - 2*b^4*c^6*x*y*z - 2*a^6*b^4*y^2*z + a^8*c^2*y^2*z + 2*a^6*b^2*c^2*y^2*z +      2*a^4*b^4*c^2*y^2*z + a^2*b^6*c^2*y^2*z - 3*a^6*c^4*y^2*z - 3*a^2*b^4*c^4*y^2*z + a^4*c^6*y^2*z +      a^2*b^2*c^6*y^2*z + a^6*b^4*x*z^2 - 3*a^4*b^6*x*z^2 + a^2*b^8*x*z^2 + a^6*b^2*c^2*x*z^2 +      2*a^2*b^6*c^2*x*z^2 - 3*a^4*b^2*c^4*x*z^2 + 2*a^2*b^4*c^4*x*z^2 - 2*b^6*c^4*x*z^2 +      a^2*b^2*c^6*x*z^2 + a^8*b^2*y*z^2 - 3*a^6*b^4*y*z^2 + a^4*b^6*y*z^2 + 2*a^6*b^2*c^2*y*z^2 +      a^2*b^6*c^2*y*z^2 - 2*a^6*c^4*y*z^2 + 2*a^4*b^2*c^4*y*z^2 - 3*a^2*b^4*c^4*y*z^2 +      a^2*b^2*c^6*y*z^2");
		exp("K085","a*b^2*c^2*x^2*y + b^3*c^2*x^2*y - 2*a*b*c^3*x^2*y + b^2*c^3*x^2*y + a^3*c^2*x*y^2 +      a^2*b*c^2*x*y^2 + a^2*c^3*x*y^2 - 2*a*b*c^3*x*y^2 - 2*a*b^3*c*x^2*z + a*b^2*c^2*x^2*z +      b^3*c^2*x^2*z + b^2*c^3*x^2*z - 2*a^3*b*c*x*y*z - 2*a*b^3*c*x*y*z - 2*a*b*c^3*x*y*z -      2*a^3*b*c*y^2*z + a^3*c^2*y^2*z + a^2*b*c^2*y^2*z + a^2*c^3*y^2*z + a^3*b^2*x*z^2 + a^2*b^3*x*z^2 +      a^2*b^2*c*x*z^2 - 2*a*b^3*c*x*z^2 + a^3*b^2*y*z^2 + a^2*b^3*y*z^2 - 2*a^3*b*c*y*z^2 +      a^2*b^2*c*y*z^2");
		exp("K086","(-a)*c^2*x^2*y + c^3*x^2*y + b*c^2*x*y^2 - c^3*x*y^2 + a*b^2*x^2*z - b^3*x^2*z - 2*a^2*b*x*y*z +      2*a*b^2*x*y*z + 2*a^2*c*x*y*z - 2*b^2*c*x*y*z - 2*a*c^2*x*y*z + 2*b*c^2*x*y*z + a^3*y^2*z -      a^2*b*y^2*z + b^3*x*z^2 - b^2*c*x*z^2 - a^3*y*z^2 + a^2*c*y*z^2");
		exp("K087","a^6*x^2*y - a^4*b^2*x^2*y - 3*a^2*b^4*x^2*y + 2*b^6*x^2*y - 2*a^4*c^2*x^2*y +      8*a^2*b^2*c^2*x^2*y - 3*b^4*c^2*x^2*y - 2*a^2*c^4*x^2*y - b^2*c^4*x^2*y + c^6*x^2*y + 2*a^6*x*y^2 -      3*a^4*b^2*x*y^2 - a^2*b^4*x*y^2 + b^6*x*y^2 - 3*a^4*c^2*x*y^2 + 8*a^2*b^2*c^2*x*y^2 -      2*b^4*c^2*x*y^2 - a^2*c^4*x*y^2 - 2*b^2*c^4*x*y^2 + c^6*x*y^2 + a^6*x^2*z - 2*a^4*b^2*x^2*z -      2*a^2*b^4*x^2*z + b^6*x^2*z - a^4*c^2*x^2*z + 8*a^2*b^2*c^2*x^2*z - b^4*c^2*x^2*z -      3*a^2*c^4*x^2*z - 3*b^2*c^4*x^2*z + 2*c^6*x^2*z + 4*a^6*x*y*z - 6*a^4*b^2*x*y*z - 6*a^2*b^4*x*y*z +      4*b^6*x*y*z - 6*a^4*c^2*x*y*z + 24*a^2*b^2*c^2*x*y*z - 6*b^4*c^2*x*y*z - 6*a^2*c^4*x*y*z -      6*b^2*c^4*x*y*z + 4*c^6*x*y*z + a^6*y^2*z - 2*a^4*b^2*y^2*z - 2*a^2*b^4*y^2*z + b^6*y^2*z -      a^4*c^2*y^2*z + 8*a^2*b^2*c^2*y^2*z - b^4*c^2*y^2*z - 3*a^2*c^4*y^2*z - 3*b^2*c^4*y^2*z +      2*c^6*y^2*z + 2*a^6*x*z^2 - 3*a^4*b^2*x*z^2 - a^2*b^4*x*z^2 + b^6*x*z^2 - 3*a^4*c^2*x*z^2 +      8*a^2*b^2*c^2*x*z^2 - 2*b^4*c^2*x*z^2 - a^2*c^4*x*z^2 - 2*b^2*c^4*x*z^2 + c^6*x*z^2 + a^6*y*z^2 -      a^4*b^2*y*z^2 - 3*a^2*b^4*y*z^2 + 2*b^6*y*z^2 - 2*a^4*c^2*y*z^2 + 8*a^2*b^2*c^2*y*z^2 -      3*b^4*c^2*y*z^2 - 2*a^2*c^4*y*z^2 - b^2*c^4*y*z^2 + c^6*y*z^2");
		exp("K088","a^4*x^2*y + 2*a^2*b^2*x^2*y + b^4*x^2*y - 7*a^2*c^2*x^2*y + 2*b^2*c^2*x^2*y + c^4*x^2*y +      a^4*x*y^2 + 2*a^2*b^2*x*y^2 + b^4*x*y^2 + 2*a^2*c^2*x*y^2 - 7*b^2*c^2*x*y^2 + c^4*x*y^2 +      a^4*x^2*z - 7*a^2*b^2*x^2*z + b^4*x^2*z + 2*a^2*c^2*x^2*z + 2*b^2*c^2*x^2*z + c^4*x^2*z -      6*a^4*x*y*z + 6*a^2*b^2*x*y*z - 6*b^4*x*y*z + 6*a^2*c^2*x*y*z + 6*b^2*c^2*x*y*z - 6*c^4*x*y*z +      a^4*y^2*z - 7*a^2*b^2*y^2*z + b^4*y^2*z + 2*a^2*c^2*y^2*z + 2*b^2*c^2*y^2*z + c^4*y^2*z +      a^4*x*z^2 + 2*a^2*b^2*x*z^2 + b^4*x*z^2 + 2*a^2*c^2*x*z^2 - 7*b^2*c^2*x*z^2 + c^4*x*z^2 +      a^4*y*z^2 + 2*a^2*b^2*y*z^2 + b^4*y*z^2 - 7*a^2*c^2*y*z^2 + 2*b^2*c^2*y*z^2 + c^4*y*z^2");
		exp("K089","4*a^2*b^2*x^2*y - 5*a^2*c^2*x^2*y + 4*b^2*c^2*x^2*y + 4*a^2*b^2*x*y^2 + 4*a^2*c^2*x*y^2 -      5*b^2*c^2*x*y^2 - 5*a^2*b^2*x^2*z + 4*a^2*c^2*x^2*z + 4*b^2*c^2*x^2*z - 6*a^2*b^2*x*y*z -      6*a^2*c^2*x*y*z - 6*b^2*c^2*x*y*z - 5*a^2*b^2*y^2*z + 4*a^2*c^2*y^2*z + 4*b^2*c^2*y^2*z +      4*a^2*b^2*x*z^2 + 4*a^2*c^2*x*z^2 - 5*b^2*c^2*x*z^2 + 4*a^2*b^2*y*z^2 - 5*a^2*c^2*y*z^2 +      4*b^2*c^2*y*z^2");
		exp("K090","a*x^2*y - 3*b*x^2*y + c*x^2*y - 3*a*x*y^2 + b*x*y^2 + c*x*y^2 + a*x^2*z + b*x^2*z - 3*c*x^2*z +      2*a*x*y*z + 2*b*x*y*z + 2*c*x*y*z + a*y^2*z + b*y^2*z - 3*c*y^2*z - 3*a*x*z^2 + b*x*z^2 + c*x*z^2 +      a*y*z^2 - 3*b*y*z^2 + c*y*z^2");
		exp("K091","a^6*x^2*y - a^4*b^2*x^2*y - a^2*b^4*x^2*y + b^6*x^2*y - 2*a^4*c^2*x^2*y - a^2*b^2*c^2*x^2*y -      b^4*c^2*x^2*y - 2*a^2*c^4*x^2*y - b^2*c^4*x^2*y + c^6*x^2*y + a^6*x*y^2 - a^4*b^2*x*y^2 -      a^2*b^4*x*y^2 + b^6*x*y^2 - a^4*c^2*x*y^2 - a^2*b^2*c^2*x*y^2 - 2*b^4*c^2*x*y^2 - a^2*c^4*x*y^2 -      2*b^2*c^4*x*y^2 + c^6*x*y^2 + a^6*x^2*z - 2*a^4*b^2*x^2*z - 2*a^2*b^4*x^2*z + b^6*x^2*z -      a^4*c^2*x^2*z - a^2*b^2*c^2*x^2*z - b^4*c^2*x^2*z - a^2*c^4*x^2*z - b^2*c^4*x^2*z + c^6*x^2*z +      2*a^6*x*y*z - 4*a^4*b^2*x*y*z - 4*a^2*b^4*x*y*z + 2*b^6*x*y*z - 4*a^4*c^2*x*y*z - 4*b^4*c^2*x*y*z -      4*a^2*c^4*x*y*z - 4*b^2*c^4*x*y*z + 2*c^6*x*y*z + a^6*y^2*z - 2*a^4*b^2*y^2*z - 2*a^2*b^4*y^2*z +      b^6*y^2*z - a^4*c^2*y^2*z - a^2*b^2*c^2*y^2*z - b^4*c^2*y^2*z - a^2*c^4*y^2*z - b^2*c^4*y^2*z +      c^6*y^2*z + a^6*x*z^2 - a^4*b^2*x*z^2 - a^2*b^4*x*z^2 + b^6*x*z^2 - a^4*c^2*x*z^2 -      a^2*b^2*c^2*x*z^2 - 2*b^4*c^2*x*z^2 - a^2*c^4*x*z^2 - 2*b^2*c^4*x*z^2 + c^6*x*z^2 + a^6*y*z^2 -      a^4*b^2*y*z^2 - a^2*b^4*y*z^2 + b^6*y*z^2 - 2*a^4*c^2*y*z^2 - a^2*b^2*c^2*y*z^2 - b^4*c^2*y*z^2 -      2*a^2*c^4*y*z^2 - b^2*c^4*y*z^2 + c^6*y*z^2");
		exp("K092","2*a^4*x^2*y + 2*a^2*b^2*x^2*y - 4*b^4*x^2*y - a^2*c^2*x^2*y + 2*b^2*c^2*x^2*y + 2*c^4*x^2*y +      4*a^4*x*y^2 - 2*a^2*b^2*x*y^2 - 2*b^4*x*y^2 - 2*a^2*c^2*x*y^2 + b^2*c^2*x*y^2 - 2*c^4*x*y^2 -      2*a^4*x^2*z + a^2*b^2*x^2*z - 2*b^4*x^2*z - 2*a^2*c^2*x^2*z - 2*b^2*c^2*x^2*z + 4*c^4*x^2*z +      2*a^4*y^2*z - a^2*b^2*y^2*z + 2*b^4*y^2*z + 2*a^2*c^2*y^2*z + 2*b^2*c^2*y^2*z - 4*c^4*y^2*z -      4*a^4*x*z^2 + 2*a^2*b^2*x*z^2 + 2*b^4*x*z^2 + 2*a^2*c^2*x*z^2 - b^2*c^2*x*z^2 + 2*c^4*x*z^2 -      2*a^4*y*z^2 - 2*a^2*b^2*y*z^2 + 4*b^4*y*z^2 + a^2*c^2*y*z^2 - 2*b^2*c^2*y*z^2 - 2*c^4*y*z^2");
		exp("K093","2*a^2*b^2*x^2*y - a^2*c^2*x^2*y + 2*b^2*c^2*x^2*y + 2*a^2*b^2*x*y^2 + 2*a^2*c^2*x*y^2 -      b^2*c^2*x*y^2 - a^2*b^2*x^2*z + 2*a^2*c^2*x^2*z + 2*b^2*c^2*x^2*z - a^2*b^2*y^2*z +      2*a^2*c^2*y^2*z + 2*b^2*c^2*y^2*z + 2*a^2*b^2*x*z^2 + 2*a^2*c^2*x*z^2 - b^2*c^2*x*z^2 +      2*a^2*b^2*y*z^2 - a^2*c^2*y*z^2 + 2*b^2*c^2*y*z^2");
		exp("K094","a^4*c^2*x^2*y - 5*a^2*b^2*c^2*x^2*y + a^2*c^4*x^2*y - 5*a^2*b^2*c^2*x*y^2 + b^4*c^2*x*y^2 +      b^2*c^4*x*y^2 + a^4*b^2*x^2*z + a^2*b^4*x^2*z - 5*a^2*b^2*c^2*x^2*z + 2*a^4*b^2*x*y*z +      2*a^2*b^4*x*y*z + 2*a^4*c^2*x*y*z - 12*a^2*b^2*c^2*x*y*z + 2*b^4*c^2*x*y*z + 2*a^2*c^4*x*y*z +      2*b^2*c^4*x*y*z + a^4*b^2*y^2*z + a^2*b^4*y^2*z - 5*a^2*b^2*c^2*y^2*z - 5*a^2*b^2*c^2*x*z^2 +      b^4*c^2*x*z^2 + b^2*c^4*x*z^2 + a^4*c^2*y*z^2 - 5*a^2*b^2*c^2*y*z^2 + a^2*c^4*y*z^2");
		exp("K095","-(a^4*c^2*x^2*y) + 2*a^2*b^2*c^2*x^2*y - b^4*c^2*x^2*y + 2*a^2*c^4*x^2*y - b^2*c^4*x^2*y -      c^6*x^2*y + a^4*c^2*x*y^2 - 2*a^2*b^2*c^2*x*y^2 + b^4*c^2*x*y^2 + a^2*c^4*x*y^2 - 2*b^2*c^4*x*y^2 +      c^6*x*y^2 + a^4*b^2*x^2*z - 2*a^2*b^4*x^2*z + b^6*x^2*z - 2*a^2*b^2*c^2*x^2*z + b^4*c^2*x^2*z +      b^2*c^4*x^2*z - a^6*y^2*z + 2*a^4*b^2*y^2*z - a^2*b^4*y^2*z - a^4*c^2*y^2*z + 2*a^2*b^2*c^2*y^2*z -      a^2*c^4*y^2*z - a^4*b^2*x*z^2 - a^2*b^4*x*z^2 - b^6*x*z^2 + 2*a^2*b^2*c^2*x*z^2 + 2*b^4*c^2*x*z^2 -      b^2*c^4*x*z^2 + a^6*y*z^2 + a^4*b^2*y*z^2 + a^2*b^4*y*z^2 - 2*a^4*c^2*y*z^2 - 2*a^2*b^2*c^2*y*z^2 +      a^2*c^4*y*z^2");
		exp("K096","(-a^10)*c^2*x^2*y - a^8*b^2*c^2*x^2*y + 6*a^6*b^4*c^2*x^2*y - 2*a^4*b^6*c^2*x^2*y -      5*a^2*b^8*c^2*x^2*y + 3*b^10*c^2*x^2*y + 4*a^8*c^4*x^2*y - 4*a^4*b^4*c^4*x^2*y +      8*a^2*b^6*c^4*x^2*y - 8*b^8*c^4*x^2*y - 6*a^6*c^6*x^2*y + 2*a^4*b^2*c^6*x^2*y -      2*a^2*b^4*c^6*x^2*y + 6*b^6*c^6*x^2*y + 4*a^4*c^8*x^2*y - a^2*c^10*x^2*y - b^2*c^10*x^2*y -      3*a^10*c^2*x*y^2 + 5*a^8*b^2*c^2*x*y^2 + 2*a^6*b^4*c^2*x*y^2 - 6*a^4*b^6*c^2*x*y^2 +      a^2*b^8*c^2*x*y^2 + b^10*c^2*x*y^2 + 8*a^8*c^4*x*y^2 - 8*a^6*b^2*c^4*x*y^2 + 4*a^4*b^4*c^4*x*y^2 -      4*b^8*c^4*x*y^2 - 6*a^6*c^6*x*y^2 + 2*a^4*b^2*c^6*x*y^2 - 2*a^2*b^4*c^6*x*y^2 + 6*b^6*c^6*x*y^2 -      4*b^4*c^8*x*y^2 + a^2*c^10*x*y^2 + b^2*c^10*x*y^2 + a^10*b^2*x^2*z - 4*a^8*b^4*x^2*z +      6*a^6*b^6*x^2*z - 4*a^4*b^8*x^2*z + a^2*b^10*x^2*z + a^8*b^2*c^2*x^2*z - 2*a^4*b^6*c^2*x^2*z +      b^10*c^2*x^2*z - 6*a^6*b^2*c^4*x^2*z + 4*a^4*b^4*c^4*x^2*z + 2*a^2*b^6*c^4*x^2*z +      2*a^4*b^2*c^6*x^2*z - 8*a^2*b^4*c^6*x^2*z - 6*b^6*c^6*x^2*z + 5*a^2*b^2*c^8*x^2*z +      8*b^4*c^8*x^2*z - 3*b^2*c^10*x^2*z - a^10*b^2*y^2*z + 4*a^8*b^4*y^2*z - 6*a^6*b^6*y^2*z +      4*a^4*b^8*y^2*z - a^2*b^10*y^2*z - a^10*c^2*y^2*z + 2*a^6*b^4*c^2*y^2*z - a^2*b^8*c^2*y^2*z -      2*a^6*b^2*c^4*y^2*z - 4*a^4*b^4*c^4*y^2*z + 6*a^2*b^6*c^4*y^2*z + 6*a^6*c^6*y^2*z +      8*a^4*b^2*c^6*y^2*z - 2*a^2*b^4*c^6*y^2*z - 8*a^4*c^8*y^2*z - 5*a^2*b^2*c^8*y^2*z +      3*a^2*c^10*y^2*z + 3*a^10*b^2*x*z^2 - 8*a^8*b^4*x*z^2 + 6*a^6*b^6*x*z^2 - a^2*b^10*x*z^2 -      5*a^8*b^2*c^2*x*z^2 + 8*a^6*b^4*c^2*x*z^2 - 2*a^4*b^6*c^2*x*z^2 - b^10*c^2*x*z^2 -      2*a^6*b^2*c^4*x*z^2 - 4*a^4*b^4*c^4*x*z^2 + 2*a^2*b^6*c^4*x*z^2 + 4*b^8*c^4*x*z^2 +      6*a^4*b^2*c^6*x*z^2 - 6*b^6*c^6*x*z^2 - a^2*b^2*c^8*x*z^2 + 4*b^4*c^8*x*z^2 - b^2*c^10*x*z^2 +      a^10*b^2*y*z^2 - 6*a^6*b^6*y*z^2 + 8*a^4*b^8*y*z^2 - 3*a^2*b^10*y*z^2 + a^10*c^2*y*z^2 +      2*a^6*b^4*c^2*y*z^2 - 8*a^4*b^6*c^2*y*z^2 + 5*a^2*b^8*c^2*y*z^2 - 4*a^8*c^4*y*z^2 -      2*a^6*b^2*c^4*y*z^2 + 4*a^4*b^4*c^4*y*z^2 + 2*a^2*b^6*c^4*y*z^2 + 6*a^6*c^6*y*z^2 -      6*a^2*b^4*c^6*y*z^2 - 4*a^4*c^8*y*z^2 + a^2*b^2*c^8*y*z^2 + a^2*c^10*y*z^2");
		exp("K097","-(a^4*b*c*x^2*y) + 2*a^2*b^3*c*x^2*y - b^5*c*x^2*y - a^3*b*c^2*x^2*y + a^2*b^2*c^2*x^2*y +      a*b^3*c^2*x^2*y - b^4*c^2*x^2*y + a*b^2*c^3*x^2*y + a*b*c^4*x^2*y + b^2*c^4*x^2*y + b*c^5*x^2*y +      a^5*c*x*y^2 - 2*a^3*b^2*c*x*y^2 + a*b^4*c*x*y^2 + a^4*c^2*x*y^2 - a^3*b*c^2*x*y^2 -      a^2*b^2*c^2*x*y^2 + a*b^3*c^2*x*y^2 - a^2*b*c^3*x*y^2 - a^2*c^4*x*y^2 - a*b*c^4*x*y^2 -      a*c^5*x*y^2 + a^4*b*c*x^2*z + a^3*b^2*c*x^2*z - a*b^4*c*x^2*z - b^5*c*x^2*z - a^2*b^2*c^2*x^2*z -      a*b^3*c^2*x^2*z - b^4*c^2*x^2*z - 2*a^2*b*c^3*x^2*z - a*b^2*c^3*x^2*z + b^2*c^4*x^2*z +      b*c^5*x^2*z + a^5*c*y^2*z + a^4*b*c*y^2*z - a^2*b^3*c*y^2*z - a*b^4*c*y^2*z + a^4*c^2*y^2*z +      a^3*b*c^2*y^2*z + a^2*b^2*c^2*y^2*z + a^2*b*c^3*y^2*z + 2*a*b^2*c^3*y^2*z - a^2*c^4*y^2*z -      a*c^5*y^2*z - a^5*b*x*z^2 - a^4*b^2*x*z^2 + a^2*b^4*x*z^2 + a*b^5*x*z^2 + a^3*b^2*c*x*z^2 +      a^2*b^3*c*x*z^2 + a*b^4*c*x*z^2 + 2*a^3*b*c^2*x*z^2 + a^2*b^2*c^2*x*z^2 - a*b^2*c^3*x*z^2 -      a*b*c^4*x*z^2 - a^5*b*y*z^2 - a^4*b^2*y*z^2 + a^2*b^4*y*z^2 + a*b^5*y*z^2 - a^4*b*c*y*z^2 -      a^3*b^2*c*y*z^2 - a^2*b^3*c*y*z^2 - a^2*b^2*c^2*y*z^2 - 2*a*b^3*c^2*y*z^2 + a^2*b*c^3*y*z^2 +      a*b*c^4*y*z^2");
		exp("K098","a^4*b^2*c^2*x^2*y - 2*a^2*b^4*c^2*x^2*y + b^6*c^2*x^2*y - a^2*b^2*c^4*x^2*y - 2*b^4*c^4*x^2*y +      b^2*c^6*x^2*y + a^6*c^2*x*y^2 - 2*a^4*b^2*c^2*x*y^2 + a^2*b^4*c^2*x*y^2 - 2*a^4*c^4*x*y^2 -      a^2*b^2*c^4*x*y^2 + a^2*c^6*x*y^2 + a^4*b^2*c^2*x^2*z - a^2*b^4*c^2*x^2*z + b^6*c^2*x^2*z -      2*a^2*b^2*c^4*x^2*z - 2*b^4*c^4*x^2*z + b^2*c^6*x^2*z + a^4*b^2*c^2*x*y*z + a^2*b^4*c^2*x*y*z +      a^2*b^2*c^4*x*y*z + a^6*c^2*y^2*z - a^4*b^2*c^2*y^2*z + a^2*b^4*c^2*y^2*z - 2*a^4*c^4*y^2*z -      2*a^2*b^2*c^4*y^2*z + a^2*c^6*y^2*z + a^6*b^2*x*z^2 - 2*a^4*b^4*x*z^2 + a^2*b^6*x*z^2 -      2*a^4*b^2*c^2*x*z^2 - a^2*b^4*c^2*x*z^2 + a^2*b^2*c^4*x*z^2 + a^6*b^2*y*z^2 - 2*a^4*b^4*y*z^2 +      a^2*b^6*y*z^2 - a^4*b^2*c^2*y*z^2 - 2*a^2*b^4*c^2*y*z^2 + a^2*b^2*c^4*y*z^2");
		exp("K099","a^6*c^2*x^2*y + a^4*b^2*c^2*x^2*y - a^2*b^4*c^2*x^2*y - b^6*c^2*x^2*y - a^4*c^4*x^2*y +      2*a^2*b^2*c^4*x^2*y + 3*b^4*c^4*x^2*y - a^2*c^6*x^2*y - 3*b^2*c^6*x^2*y + c^8*x^2*y +      a^6*c^2*x*y^2 + a^4*b^2*c^2*x*y^2 - a^2*b^4*c^2*x*y^2 - b^6*c^2*x*y^2 - 3*a^4*c^4*x*y^2 -      2*a^2*b^2*c^4*x*y^2 + b^4*c^4*x*y^2 + 3*a^2*c^6*x*y^2 + b^2*c^6*x*y^2 - c^8*x*y^2 - a^6*b^2*x^2*z +      a^4*b^4*x^2*z + a^2*b^6*x^2*z - b^8*x^2*z - a^4*b^2*c^2*x^2*z - 2*a^2*b^4*c^2*x^2*z +      3*b^6*c^2*x^2*z + a^2*b^2*c^4*x^2*z - 3*b^4*c^4*x^2*z + b^2*c^6*x^2*z + a^8*y^2*z - a^6*b^2*y^2*z -      a^4*b^4*y^2*z + a^2*b^6*y^2*z - 3*a^6*c^2*y^2*z + 2*a^4*b^2*c^2*y^2*z + a^2*b^4*c^2*y^2*z +      3*a^4*c^4*y^2*z - a^2*b^2*c^4*y^2*z - a^2*c^6*y^2*z - a^6*b^2*x*z^2 + 3*a^4*b^4*x*z^2 -      3*a^2*b^6*x*z^2 + b^8*x*z^2 - a^4*b^2*c^2*x*z^2 + 2*a^2*b^4*c^2*x*z^2 - b^6*c^2*x*z^2 +      a^2*b^2*c^4*x*z^2 - b^4*c^4*x*z^2 + b^2*c^6*x*z^2 - a^8*y*z^2 + 3*a^6*b^2*y*z^2 - 3*a^4*b^4*y*z^2 +      a^2*b^6*y*z^2 + a^6*c^2*y*z^2 - 2*a^4*b^2*c^2*y*z^2 + a^2*b^4*c^2*y*z^2 + a^4*c^4*y*z^2 -      a^2*b^2*c^4*y*z^2 - a^2*c^6*y*z^2");
		exp("K100","b^4*c^2*x^3 - b^2*c^4*x^3 + a^2*b^2*c^2*x^2*y - a^2*b^2*c^2*x*y^2 - a^4*c^2*y^3 + a^2*c^4*y^3 -      a^2*b^2*c^2*x^2*z + a^2*b^2*c^2*y^2*z + a^2*b^2*c^2*x*z^2 - a^2*b^2*c^2*y*z^2 + a^4*b^2*z^3 -      a^2*b^4*z^3");
		exp("K101","b*c*x^2*y - a*c*x*y^2 - b*c*x^2*z + a*c*y^2*z + a*b*x*z^2 - a*b*y*z^2");
		exp("K102","b^2*c^2*x^2*y - a^2*c^2*x*y^2 - b^2*c^2*x^2*z + a^2*c^2*y^2*z + a^2*b^2*x*z^2 - a^2*b^2*y*z^2 ");
		exp("K103","a^4*x^2*y - b^4*x^2*y + b^2*c^2*x^2*y - c^4*x^2*y + a^4*x*y^2 - b^4*x*y^2 - a^2*c^2*x*y^2 +      c^4*x*y^2 - a^4*x^2*z + b^4*x^2*z - b^2*c^2*x^2*z + c^4*x^2*z - a^4*y^2*z + b^4*y^2*z +      a^2*c^2*y^2*z - c^4*y^2*z - a^4*x*z^2 + a^2*b^2*x*z^2 - b^4*x*z^2 + c^4*x*z^2 + a^4*y*z^2 -      a^2*b^2*y*z^2 + b^4*y*z^2 - c^4*y*z^2");
		exp("K104","4*a^4*x^2*y - 2*a^2*b^2*x^2*y - 2*b^4*x^2*y - 2*a^2*c^2*x^2*y + b^2*c^2*x^2*y - 2*c^4*x^2*y +      2*a^4*x*y^2 + 2*a^2*b^2*x*y^2 - 4*b^4*x*y^2 - a^2*c^2*x*y^2 + 2*b^2*c^2*x*y^2 + 2*c^4*x*y^2 -      4*a^4*x^2*z + 2*a^2*b^2*x^2*z + 2*b^4*x^2*z + 2*a^2*c^2*x^2*z - b^2*c^2*x^2*z + 2*c^4*x^2*z -      2*a^4*y^2*z - 2*a^2*b^2*y^2*z + 4*b^4*y^2*z + a^2*c^2*y^2*z - 2*b^2*c^2*y^2*z - 2*c^4*y^2*z -      2*a^4*x*z^2 + a^2*b^2*x*z^2 - 2*b^4*x*z^2 - 2*a^2*c^2*x*z^2 - 2*b^2*c^2*x*z^2 + 4*c^4*x*z^2 +      2*a^4*y*z^2 - a^2*b^2*y*z^2 + 2*b^4*y*z^2 + 2*a^2*c^2*y*z^2 + 2*b^2*c^2*y*z^2 - 4*c^4*y*z^2");
		exp("K105","a^4*b^2*c^2*x^2*y - 2*a^2*b^4*c^2*x^2*y + b^6*c^2*x^2*y - 2*b^4*c^4*x^2*y + b^2*c^6*x^2*y +      a^6*c^2*x*y^2 - 2*a^4*b^2*c^2*x*y^2 + a^2*b^4*c^2*x*y^2 - 2*a^4*c^4*x*y^2 + a^2*c^6*x*y^2 +      a^4*b^2*c^2*x^2*z + b^6*c^2*x^2*z - 2*a^2*b^2*c^4*x^2*z - 2*b^4*c^4*x^2*z + b^2*c^6*x^2*z +      2*a^4*b^2*c^2*x*y*z + 2*a^2*b^4*c^2*x*y*z + 2*a^2*b^2*c^4*x*y*z + a^6*c^2*y^2*z +      a^2*b^4*c^2*y^2*z - 2*a^4*c^4*y^2*z - 2*a^2*b^2*c^4*y^2*z + a^2*c^6*y^2*z + a^6*b^2*x*z^2 -      2*a^4*b^4*x*z^2 + a^2*b^6*x*z^2 - 2*a^4*b^2*c^2*x*z^2 + a^2*b^2*c^4*x*z^2 + a^6*b^2*y*z^2 -      2*a^4*b^4*y*z^2 + a^2*b^6*y*z^2 - 2*a^2*b^4*c^2*y*z^2 + a^2*b^2*c^4*y*z^2");
		exp("K106","a^2*b^2*x^2*y - b^4*x^2*y - a^2*c^2*x^2*y + 2*b^2*c^2*x^2*y - c^4*x^2*y + a^4*x*y^2 -      a^2*b^2*x*y^2 - 2*a^2*c^2*x*y^2 + b^2*c^2*x*y^2 + c^4*x*y^2 + a^2*b^2*x^2*z + b^4*x^2*z -      a^2*c^2*x^2*z - 2*b^2*c^2*x^2*z + c^4*x^2*z - a^4*y^2*z - a^2*b^2*y^2*z + 2*a^2*c^2*y^2*z +      b^2*c^2*y^2*z - c^4*y^2*z - a^4*x*z^2 + 2*a^2*b^2*x*z^2 - b^4*x*z^2 + a^2*c^2*x*z^2 -      b^2*c^2*x*z^2 + a^4*y*z^2 - 2*a^2*b^2*y*z^2 + b^4*y*z^2 + a^2*c^2*y*z^2 - b^2*c^2*y*z^2");
		exp("K107","-(a^2*b^2*c^2*x^2*y) + b^4*c^2*x^2*y - b^2*c^4*x^2*y + a^4*c^2*x*y^2 - a^2*b^2*c^2*x*y^2 -      a^2*c^4*x*y^2 - a^2*b^2*c^2*x^2*z - b^4*c^2*x^2*z + b^2*c^4*x^2*z + 6*a^2*b^2*c^2*x*y*z -      a^4*c^2*y^2*z - a^2*b^2*c^2*y^2*z + a^2*c^4*y^2*z + a^4*b^2*x*z^2 - a^2*b^4*x*z^2 -      a^2*b^2*c^2*x*z^2 - a^4*b^2*y*z^2 + a^2*b^4*y*z^2 - a^2*b^2*c^2*y*z^2");
		exp("K108","a^4*b^2*c^4*x^2*y - b^6*c^4*x^2*y - a^2*b^2*c^6*x^2*y + b^2*c^8*x^2*y + a^6*c^4*x*y^2 -      a^2*b^4*c^4*x*y^2 + a^2*b^2*c^6*x*y^2 - a^2*c^8*x*y^2 - a^4*b^4*c^2*x^2*z + a^2*b^6*c^2*x^2*z -      b^8*c^2*x^2*z + b^4*c^6*x^2*z + a^8*c^2*y^2*z - a^6*b^2*c^2*y^2*z + a^4*b^4*c^2*y^2*z -      a^4*c^6*y^2*z - a^6*b^4*x*z^2 + a^2*b^8*x*z^2 - a^2*b^6*c^2*x*z^2 + a^2*b^4*c^4*x*z^2 -      a^8*b^2*y*z^2 + a^4*b^6*y*z^2 + a^6*b^2*c^2*y*z^2 - a^4*b^2*c^4*y*z^2");
		exp("K109","a^5*b*c^2*x^2*y + a^4*b^2*c^2*x^2*y - a*b^5*c^2*x^2*y - b^6*c^2*x^2*y + a^5*c^3*x^2*y +      a^4*b*c^3*x^2*y - a*b^4*c^3*x^2*y - b^5*c^3*x^2*y - 2*a^3*b*c^4*x^2*y - 2*a^2*b^2*c^4*x^2*y -      2*a^3*c^5*x^2*y - 2*a^2*b*c^5*x^2*y + a*b*c^6*x^2*y + b^2*c^6*x^2*y + a*c^7*x^2*y + b*c^7*x^2*y +      a^6*c^2*x*y^2 + a^5*b*c^2*x*y^2 - a^2*b^4*c^2*x*y^2 - a*b^5*c^2*x*y^2 + a^5*c^3*x*y^2 +      a^4*b*c^3*x*y^2 - a*b^4*c^3*x*y^2 - b^5*c^3*x*y^2 + 2*a^2*b^2*c^4*x*y^2 + 2*a*b^3*c^4*x*y^2 +      2*a*b^2*c^5*x*y^2 + 2*b^3*c^5*x*y^2 - a^2*c^6*x*y^2 - a*b*c^6*x*y^2 - a*c^7*x*y^2 - b*c^7*x*y^2 -      a^5*b^3*x^2*z + 2*a^3*b^5*x^2*z - a*b^7*x^2*z - a^5*b^2*c*x^2*z - a^4*b^3*c*x^2*z +      2*a^3*b^4*c*x^2*z + 2*a^2*b^5*c*x^2*z - a*b^6*c*x^2*z - b^7*c*x^2*z - a^4*b^2*c^2*x^2*z +      2*a^2*b^4*c^2*x^2*z - b^6*c^2*x^2*z + a*b^3*c^4*x^2*z + a*b^2*c^5*x^2*z + b^3*c^5*x^2*z +      b^2*c^6*x^2*z + a^7*b*y^2*z - 2*a^5*b^3*y^2*z + a^3*b^5*y^2*z + a^7*c*y^2*z + a^6*b*c*y^2*z -      2*a^5*b^2*c*y^2*z - 2*a^4*b^3*c*y^2*z + a^3*b^4*c*y^2*z + a^2*b^5*c*y^2*z + a^6*c^2*y^2*z -      2*a^4*b^2*c^2*y^2*z + a^2*b^4*c^2*y^2*z - a^3*b*c^4*y^2*z - a^3*c^5*y^2*z - a^2*b*c^5*y^2*z -      a^2*c^6*y^2*z - a^6*b^2*x*z^2 - a^5*b^3*x*z^2 + a^2*b^6*x*z^2 + a*b^7*x*z^2 - a^5*b^2*c*x*z^2 -      a^4*b^3*c*x*z^2 + a*b^6*c*x*z^2 + b^7*c*x*z^2 - 2*a^2*b^4*c^2*x*z^2 - 2*a*b^5*c^2*x*z^2 -      2*a*b^4*c^3*x*z^2 - 2*b^5*c^3*x*z^2 + a^2*b^2*c^4*x*z^2 + a*b^3*c^4*x*z^2 + a*b^2*c^5*x*z^2 +      b^3*c^5*x*z^2 - a^7*b*y*z^2 - a^6*b^2*y*z^2 + a^3*b^5*y*z^2 + a^2*b^6*y*z^2 - a^7*c*y*z^2 -      a^6*b*c*y*z^2 + a^3*b^4*c*y*z^2 + a^2*b^5*c*y*z^2 + 2*a^5*b*c^2*y*z^2 + 2*a^4*b^2*c^2*y*z^2 +      2*a^5*c^3*y*z^2 + 2*a^4*b*c^3*y*z^2 - a^3*b*c^4*y*z^2 - a^2*b^2*c^4*y*z^2 - a^3*c^5*y*z^2 -      a^2*b*c^5*y*z^2");
		exp("K110A","a^4*b^2*c^2*x^2*y - b^6*c^2*x^2*y + a^4*c^4*x^2*y + 2*a^2*b^2*c^4*x^2*y + 3*b^4*c^4*x^2*y -      2*a^2*c^6*x^2*y - 3*b^2*c^6*x^2*y + c^8*x^2*y + a^6*c^2*x*y^2 - a^2*b^4*c^2*x*y^2 +      2*a^2*b^2*c^4*x*y^2 - a^2*c^6*x*y^2 - a^4*b^4*x^2*z + 2*a^2*b^6*x^2*z - b^8*x^2*z -      a^4*b^2*c^2*x^2*z - 2*a^2*b^4*c^2*x^2*z + 3*b^6*c^2*x^2*z - 3*b^4*c^4*x^2*z + b^2*c^6*x^2*z +      a^8*y^2*z - 2*a^6*b^2*y^2*z + a^4*b^4*y^2*z - a^4*c^4*y^2*z - a^6*b^2*x*z^2 + a^2*b^6*x*z^2 -      2*a^2*b^4*c^2*x*z^2 + a^2*b^2*c^4*x*z^2 - a^8*y*z^2 + a^4*b^4*y*z^2 + 2*a^6*c^2*y*z^2 -      a^4*c^4*y*z^2");
		exp("K110B","a^4*b^2*c^2*x^2*y - b^6*c^2*x^2*y - 2*a^2*b^2*c^4*x^2*y + b^2*c^6*x^2*y + a^6*c^2*x*y^2 -      a^2*b^4*c^2*x*y^2 - 3*a^4*c^4*x*y^2 - 2*a^2*b^2*c^4*x*y^2 - b^4*c^4*x*y^2 + 3*a^2*c^6*x*y^2 +      2*b^2*c^6*x*y^2 - c^8*x*y^2 - a^4*b^4*x^2*z + 2*a^2*b^6*x^2*z - b^8*x^2*z + b^4*c^4*x^2*z +      a^8*y^2*z - 2*a^6*b^2*y^2*z + a^4*b^4*y^2*z - 3*a^6*c^2*y^2*z + 2*a^4*b^2*c^2*y^2*z +      a^2*b^4*c^2*y^2*z + 3*a^4*c^4*y^2*z - a^2*c^6*y^2*z - a^4*b^4*x*z^2 + b^8*x*z^2 - 2*b^6*c^2*x*z^2 +      b^4*c^4*x*z^2 - a^6*b^2*y*z^2 + a^2*b^6*y*z^2 + 2*a^4*b^2*c^2*y*z^2 - a^2*b^2*c^4*y*z^2");
		exp("K110C","a^4*c^4*x^2*y - b^4*c^4*x^2*y - 2*a^2*c^6*x^2*y + c^8*x^2*y + a^4*c^4*x*y^2 - b^4*c^4*x*y^2 +      2*b^2*c^6*x*y^2 - c^8*x*y^2 - a^4*b^2*c^2*x^2*z + 2*a^2*b^4*c^2*x^2*z - b^6*c^2*x^2*z +      b^2*c^6*x^2*z + a^6*c^2*y^2*z - 2*a^4*b^2*c^2*y^2*z + a^2*b^4*c^2*y^2*z - a^2*c^6*y^2*z -      a^6*b^2*x*z^2 + 3*a^4*b^4*x*z^2 - 3*a^2*b^6*x*z^2 + b^8*x*z^2 + 2*a^2*b^4*c^2*x*z^2 -      2*b^6*c^2*x*z^2 + a^2*b^2*c^4*x*z^2 + b^4*c^4*x*z^2 - a^8*y*z^2 + 3*a^6*b^2*y*z^2 -      3*a^4*b^4*y*z^2 + a^2*b^6*y*z^2 + 2*a^6*c^2*y*z^2 - 2*a^4*b^2*c^2*y*z^2 - a^4*c^4*y*z^2 -      a^2*b^2*c^4*y*z^2");
		exp("K111A","a^2*b^2*c^2*x^2*y + b^4*c^2*x^2*y - a^2*c^4*x^2*y - 2*b^2*c^4*x^2*y + c^6*x^2*y +      a^4*c^2*x*y^2 + a^2*b^2*c^2*x*y^2 - a^2*c^4*x*y^2 - a^2*b^4*x^2*z + b^6*x^2*z + a^2*b^2*c^2*x^2*z -      2*b^4*c^2*x^2*z + b^2*c^4*x^2*z + a^6*y^2*z - a^4*b^2*y^2*z - a^4*c^2*y^2*z + a^4*b^2*x*z^2 -      a^2*b^4*x*z^2 + a^2*b^2*c^2*x*z^2 + a^6*y*z^2 - a^4*b^2*y*z^2 - a^4*c^2*y*z^2");
		exp("K111B","a^2*b^2*c^2*x^2*y + b^4*c^2*x^2*y - b^2*c^4*x^2*y + a^4*c^2*x*y^2 + a^2*b^2*c^2*x*y^2 -      2*a^2*c^4*x*y^2 - b^2*c^4*x*y^2 + c^6*x*y^2 - a^2*b^4*x^2*z + b^6*x^2*z - b^4*c^2*x^2*z +      a^6*y^2*z - a^4*b^2*y^2*z - 2*a^4*c^2*y^2*z + a^2*b^2*c^2*y^2*z + a^2*c^4*y^2*z - a^2*b^4*x*z^2 +      b^6*x*z^2 - b^4*c^2*x*z^2 - a^4*b^2*y*z^2 + a^2*b^4*y*z^2 + a^2*b^2*c^2*y*z^2");
		exp("K111C","(-a^2)*c^4*x^2*y - b^2*c^4*x^2*y + c^6*x^2*y - a^2*c^4*x*y^2 - b^2*c^4*x*y^2 + c^6*x*y^2 +      a^2*b^2*c^2*x^2*z - b^4*c^2*x^2*z + b^2*c^4*x^2*z - a^4*c^2*y^2*z + a^2*b^2*c^2*y^2*z +      a^2*c^4*y^2*z + a^4*b^2*x*z^2 - 2*a^2*b^4*x*z^2 + b^6*x*z^2 + a^2*b^2*c^2*x*z^2 - b^4*c^2*x*z^2 +      a^6*y*z^2 - 2*a^4*b^2*y*z^2 + a^2*b^4*y*z^2 - a^4*c^2*y*z^2 + a^2*b^2*c^2*y*z^2");
		exp("K112","(-a^8)*b^2*c^2*x^2*y + 2*a^6*b^4*c^2*x^2*y - 2*a^2*b^8*c^2*x^2*y + b^10*c^2*x^2*y -      a^8*c^4*x^2*y + 2*a^6*b^2*c^4*x^2*y - a^4*b^4*c^4*x^2*y + 2*a^2*b^6*c^4*x^2*y - 2*b^8*c^4*x^2*y +      4*a^6*c^6*x^2*y + a^4*b^2*c^6*x^2*y + b^6*c^6*x^2*y - 6*a^4*c^8*x^2*y - 4*a^2*b^2*c^8*x^2*y -      b^4*c^8*x^2*y + 4*a^2*c^10*x^2*y + 2*b^2*c^10*x^2*y - c^12*x^2*y - a^10*c^2*x*y^2 +      2*a^8*b^2*c^2*x*y^2 - 2*a^4*b^6*c^2*x*y^2 + a^2*b^8*c^2*x*y^2 + 2*a^8*c^4*x*y^2 -      2*a^6*b^2*c^4*x*y^2 + a^4*b^4*c^4*x*y^2 - 2*a^2*b^6*c^4*x*y^2 + b^8*c^4*x*y^2 - a^6*c^6*x*y^2 -      a^2*b^4*c^6*x*y^2 - 4*b^6*c^6*x*y^2 + a^4*c^8*x*y^2 + 4*a^2*b^2*c^8*x*y^2 + 6*b^4*c^8*x*y^2 -      2*a^2*c^10*x*y^2 - 4*b^2*c^10*x*y^2 + c^12*x*y^2 + a^8*b^4*x^2*z - 4*a^6*b^6*x^2*z +      6*a^4*b^8*x^2*z - 4*a^2*b^10*x^2*z + b^12*x^2*z + a^8*b^2*c^2*x^2*z - 2*a^6*b^4*c^2*x^2*z -      a^4*b^6*c^2*x^2*z + 4*a^2*b^8*c^2*x^2*z - 2*b^10*c^2*x^2*z - 2*a^6*b^2*c^4*x^2*z +      a^4*b^4*c^4*x^2*z + b^8*c^4*x^2*z - 2*a^2*b^4*c^6*x^2*z - b^6*c^6*x^2*z + 2*a^2*b^2*c^8*x^2*z +      2*b^4*c^8*x^2*z - b^2*c^10*x^2*z - a^12*y^2*z + 4*a^10*b^2*y^2*z - 6*a^8*b^4*y^2*z +      4*a^6*b^6*y^2*z - a^4*b^8*y^2*z + 2*a^10*c^2*y^2*z - 4*a^8*b^2*c^2*y^2*z + a^6*b^4*c^2*y^2*z +      2*a^4*b^6*c^2*y^2*z - a^2*b^8*c^2*y^2*z - a^8*c^4*y^2*z - a^4*b^4*c^4*y^2*z + 2*a^2*b^6*c^4*y^2*z +      a^6*c^6*y^2*z + 2*a^4*b^2*c^6*y^2*z - 2*a^4*c^8*y^2*z - 2*a^2*b^2*c^8*y^2*z + a^2*c^10*y^2*z +      a^10*b^2*x*z^2 - 2*a^8*b^4*x*z^2 + a^6*b^6*x*z^2 - a^4*b^8*x*z^2 + 2*a^2*b^10*x*z^2 - b^12*x*z^2 -      2*a^8*b^2*c^2*x*z^2 + 2*a^6*b^4*c^2*x*z^2 - 4*a^2*b^8*c^2*x*z^2 + 4*b^10*c^2*x*z^2 -      a^4*b^4*c^4*x*z^2 + a^2*b^6*c^4*x*z^2 - 6*b^8*c^4*x*z^2 + 2*a^4*b^2*c^6*x*z^2 +      2*a^2*b^4*c^6*x*z^2 + 4*b^6*c^6*x*z^2 - a^2*b^2*c^8*x*z^2 - b^4*c^8*x*z^2 + a^12*y*z^2 -      2*a^10*b^2*y*z^2 + a^8*b^4*y*z^2 - a^6*b^6*y*z^2 + 2*a^4*b^8*y*z^2 - a^2*b^10*y*z^2 -      4*a^10*c^2*y*z^2 + 4*a^8*b^2*c^2*y*z^2 - 2*a^4*b^6*c^2*y*z^2 + 2*a^2*b^8*c^2*y*z^2 +      6*a^8*c^4*y*z^2 - a^6*b^2*c^4*y*z^2 + a^4*b^4*c^4*y*z^2 - 4*a^6*c^6*y*z^2 - 2*a^4*b^2*c^6*y*z^2 -      2*a^2*b^4*c^6*y*z^2 + a^4*c^8*y*z^2 + a^2*b^2*c^8*y*z^2");
		exp("K113","(-a^6)*b^2*c^2*x^2*y - a^4*b^4*c^2*x^2*y + a^2*b^6*c^2*x^2*y + b^8*c^2*x^2*y - a^6*c^4*x^2*y +      2*a^4*b^2*c^4*x^2*y + a^2*b^4*c^4*x^2*y - 2*b^6*c^4*x^2*y + a^4*c^6*x^2*y - 3*a^2*b^2*c^6*x^2*y +      a^2*c^8*x^2*y + 2*b^2*c^8*x^2*y - c^10*x^2*y - a^8*c^2*x*y^2 - a^6*b^2*c^2*x*y^2 +      a^4*b^4*c^2*x*y^2 + a^2*b^6*c^2*x*y^2 + 2*a^6*c^4*x*y^2 - a^4*b^2*c^4*x*y^2 - 2*a^2*b^4*c^4*x*y^2 +      b^6*c^4*x*y^2 + 3*a^2*b^2*c^6*x*y^2 - b^4*c^6*x*y^2 - 2*a^2*c^8*x*y^2 - b^2*c^8*x*y^2 +      c^10*x*y^2 + a^6*b^4*x^2*z - a^4*b^6*x^2*z - a^2*b^8*x^2*z + b^10*x^2*z + a^6*b^2*c^2*x^2*z -      2*a^4*b^4*c^2*x^2*z + 3*a^2*b^6*c^2*x^2*z - 2*b^8*c^2*x^2*z + a^4*b^2*c^4*x^2*z -      a^2*b^4*c^4*x^2*z - a^2*b^2*c^6*x^2*z + 2*b^4*c^6*x^2*z - b^2*c^8*x^2*z - a^10*y^2*z +      a^8*b^2*y^2*z + a^6*b^4*y^2*z - a^4*b^6*y^2*z + 2*a^8*c^2*y^2*z - 3*a^6*b^2*c^2*y^2*z +      2*a^4*b^4*c^2*y^2*z - a^2*b^6*c^2*y^2*z + a^4*b^2*c^4*y^2*z - a^2*b^4*c^4*y^2*z - 2*a^4*c^6*y^2*z +      a^2*b^2*c^6*y^2*z + a^2*c^8*y^2*z + a^8*b^2*x*z^2 - 2*a^6*b^4*x*z^2 + 2*a^2*b^8*x*z^2 -      b^10*x*z^2 + a^6*b^2*c^2*x*z^2 + a^4*b^4*c^2*x*z^2 - 3*a^2*b^6*c^2*x*z^2 + b^8*c^2*x*z^2 -      a^4*b^2*c^4*x*z^2 + 2*a^2*b^4*c^4*x*z^2 + b^6*c^4*x*z^2 - a^2*b^2*c^6*x*z^2 - b^4*c^6*x*z^2 +      a^10*y*z^2 - 2*a^8*b^2*y*z^2 + 2*a^4*b^6*y*z^2 - a^2*b^8*y*z^2 - a^8*c^2*y*z^2 +      3*a^6*b^2*c^2*y*z^2 - a^4*b^4*c^2*y*z^2 - a^2*b^6*c^2*y*z^2 - a^6*c^4*y*z^2 - 2*a^4*b^2*c^4*y*z^2 +      a^2*b^4*c^4*y*z^2 + a^4*c^6*y*z^2 + a^2*b^2*c^6*y*z^2");
		exp("K114","-2*a^6*b^2*c^4*x^2*y - a^4*b^4*c^4*x^2*y + 2*a^2*b^6*c^4*x^2*y + b^8*c^4*x^2*y +      3*a^4*b^2*c^6*x^2*y - 2*a^2*b^4*c^6*x^2*y - 3*b^6*c^6*x^2*y + 3*b^4*c^8*x^2*y - b^2*c^10*x^2*y -      a^8*c^4*x*y^2 - 2*a^6*b^2*c^4*x*y^2 + a^4*b^4*c^4*x*y^2 + 2*a^2*b^6*c^4*x*y^2 + 3*a^6*c^6*x*y^2 +      2*a^4*b^2*c^6*x*y^2 - 3*a^2*b^4*c^6*x*y^2 - 3*a^4*c^8*x*y^2 + a^2*c^10*x*y^2 +      2*a^6*b^4*c^2*x^2*z - 3*a^4*b^6*c^2*x^2*z + b^10*c^2*x^2*z + a^4*b^4*c^4*x^2*z +      2*a^2*b^6*c^4*x^2*z - 3*b^8*c^4*x^2*z - 2*a^2*b^4*c^6*x^2*z + 3*b^6*c^6*x^2*z - b^4*c^8*x^2*z -      a^10*c^2*y^2*z + 3*a^6*b^4*c^2*y^2*z - 2*a^4*b^6*c^2*y^2*z + 3*a^8*c^4*y^2*z -      2*a^6*b^2*c^4*y^2*z - a^4*b^4*c^4*y^2*z - 3*a^6*c^6*y^2*z + 2*a^4*b^2*c^6*y^2*z + a^4*c^8*y^2*z +      a^8*b^4*x*z^2 - 3*a^6*b^6*x*z^2 + 3*a^4*b^8*x*z^2 - a^2*b^10*x*z^2 + 2*a^6*b^4*c^2*x*z^2 -      2*a^4*b^6*c^2*x*z^2 - a^4*b^4*c^4*x*z^2 + 3*a^2*b^6*c^4*x*z^2 - 2*a^2*b^4*c^6*x*z^2 +      a^10*b^2*y*z^2 - 3*a^8*b^4*y*z^2 + 3*a^6*b^6*y*z^2 - a^4*b^8*y*z^2 + 2*a^6*b^4*c^2*y*z^2 -      2*a^4*b^6*c^2*y*z^2 - 3*a^6*b^2*c^4*y*z^2 + a^4*b^4*c^4*y*z^2 + 2*a^4*b^2*c^6*y*z^2");
		exp("K115","a^6*x^2*y - 3*a^4*b^2*x^2*y + 3*a^2*b^4*x^2*y - b^6*x^2*y - 2*a^4*c^2*x^2*y + a^2*b^2*c^2*x^2*y +      b^4*c^2*x^2*y + a^2*c^4*x^2*y + a^6*x*y^2 - 3*a^4*b^2*x*y^2 + 3*a^2*b^4*x*y^2 - b^6*x*y^2 -      a^4*c^2*x*y^2 - a^2*b^2*c^2*x*y^2 + 2*b^4*c^2*x*y^2 - b^2*c^4*x*y^2 - a^6*x^2*z + 2*a^4*b^2*x^2*z -      a^2*b^4*x^2*z + 3*a^4*c^2*x^2*z - a^2*b^2*c^2*x^2*z - 3*a^2*c^4*x^2*z - b^2*c^4*x^2*z + c^6*x^2*z +      a^4*b^2*y^2*z - 2*a^2*b^4*y^2*z + b^6*y^2*z + a^2*b^2*c^2*y^2*z - 3*b^4*c^2*y^2*z + a^2*c^4*y^2*z +      3*b^2*c^4*y^2*z - c^6*y^2*z - a^6*x*z^2 + a^4*b^2*x*z^2 + 3*a^4*c^2*x*z^2 + a^2*b^2*c^2*x*z^2 +      b^4*c^2*x*z^2 - 3*a^2*c^4*x*z^2 - 2*b^2*c^4*x*z^2 + c^6*x*z^2 - a^2*b^4*y*z^2 + b^6*y*z^2 -      a^4*c^2*y*z^2 - a^2*b^2*c^2*y*z^2 - 3*b^4*c^2*y*z^2 + 2*a^2*c^4*y*z^2 + 3*b^2*c^4*y*z^2 -      c^6*y*z^2");
		exp("K116","2*a^6*x^2*y - 6*a^4*b^2*x^2*y + 6*a^2*b^4*x^2*y - 2*b^6*x^2*y - 3*a^4*c^2*x^2*y +      3*a^2*b^2*c^2*x^2*y + b^2*c^4*x^2*y + c^6*x^2*y + 2*a^6*x*y^2 - 6*a^4*b^2*x*y^2 + 6*a^2*b^4*x*y^2 -      2*b^6*x*y^2 - 3*a^2*b^2*c^2*x*y^2 + 3*b^4*c^2*x*y^2 - a^2*c^4*x*y^2 - c^6*x*y^2 - 2*a^6*x^2*z +      3*a^4*b^2*x^2*z - b^6*x^2*z + 6*a^4*c^2*x^2*z - 3*a^2*b^2*c^2*x^2*z - b^4*c^2*x^2*z -      6*a^2*c^4*x^2*z + 2*c^6*x^2*z + a^6*y^2*z - 3*a^2*b^4*y^2*z + 2*b^6*y^2*z + a^4*c^2*y^2*z +      3*a^2*b^2*c^2*y^2*z - 6*b^4*c^2*y^2*z + 6*b^2*c^4*y^2*z - 2*c^6*y^2*z - 2*a^6*x*z^2 +      a^2*b^4*x*z^2 + b^6*x*z^2 + 6*a^4*c^2*x*z^2 + 3*a^2*b^2*c^2*x*z^2 - 6*a^2*c^4*x*z^2 -      3*b^2*c^4*x*z^2 + 2*c^6*x*z^2 - a^6*y*z^2 - a^4*b^2*y*z^2 + 2*b^6*y*z^2 - 3*a^2*b^2*c^2*y*z^2 -      6*b^4*c^2*y*z^2 + 3*a^2*c^4*y*z^2 + 6*b^2*c^4*y*z^2 - 2*c^6*y*z^2");
		exp("K117","3*a^2*x^2*y - 3*b^2*x^2*y - c^2*x^2*y + 3*a^2*x*y^2 - 3*b^2*x*y^2 + c^2*x*y^2 - 3*a^2*x^2*z +      b^2*x^2*z + 3*c^2*x^2*z - a^2*y^2*z + 3*b^2*y^2*z - 3*c^2*y^2*z - 3*a^2*x*z^2 - b^2*x*z^2 +      3*c^2*x*z^2 + a^2*y*z^2 + 3*b^2*y*z^2 - 3*c^2*y*z^2");
		exp("K118","-2*a^6*x^2*y + 6*a^4*b^2*x^2*y - 6*a^2*b^4*x^2*y + 2*b^6*x^2*y - 7*a^4*c^2*x^2*y -      13*a^2*b^2*c^2*x^2*y + 20*b^4*c^2*x^2*y + 20*a^2*c^4*x^2*y - 11*b^2*c^4*x^2*y - 11*c^6*x^2*y -      2*a^6*x*y^2 + 6*a^4*b^2*x*y^2 - 6*a^2*b^4*x*y^2 + 2*b^6*x*y^2 - 20*a^4*c^2*x*y^2 +      13*a^2*b^2*c^2*x*y^2 + 7*b^4*c^2*x*y^2 + 11*a^2*c^4*x*y^2 - 20*b^2*c^4*x*y^2 + 11*c^6*x*y^2 +      2*a^6*x^2*z + 7*a^4*b^2*x^2*z - 20*a^2*b^4*x^2*z + 11*b^6*x^2*z - 6*a^4*c^2*x^2*z +      13*a^2*b^2*c^2*x^2*z + 11*b^4*c^2*x^2*z + 6*a^2*c^4*x^2*z - 20*b^2*c^4*x^2*z - 2*c^6*x^2*z -      11*a^6*y^2*z + 20*a^4*b^2*y^2*z - 7*a^2*b^4*y^2*z - 2*b^6*y^2*z - 11*a^4*c^2*y^2*z -      13*a^2*b^2*c^2*y^2*z + 6*b^4*c^2*y^2*z + 20*a^2*c^4*y^2*z - 6*b^2*c^4*y^2*z + 2*c^6*y^2*z +      2*a^6*x*z^2 + 20*a^4*b^2*x*z^2 - 11*a^2*b^4*x*z^2 - 11*b^6*x*z^2 - 6*a^4*c^2*x*z^2 -      13*a^2*b^2*c^2*x*z^2 + 20*b^4*c^2*x*z^2 + 6*a^2*c^4*x*z^2 - 7*b^2*c^4*x*z^2 - 2*c^6*x*z^2 +      11*a^6*y*z^2 + 11*a^4*b^2*y*z^2 - 20*a^2*b^4*y*z^2 - 2*b^6*y*z^2 - 20*a^4*c^2*y*z^2 +      13*a^2*b^2*c^2*y*z^2 + 6*b^4*c^2*y*z^2 + 7*a^2*c^4*y*z^2 - 6*b^2*c^4*y*z^2 + 2*c^6*y*z^2");
		exp("K119","2*a^6*x^2*y - 6*a^4*b^2*x^2*y + 6*a^2*b^4*x^2*y - 2*b^6*x^2*y - 5*a^4*c^2*x^2*y +      a^2*b^2*c^2*x^2*y + 4*b^4*c^2*x^2*y + 4*a^2*c^4*x^2*y - b^2*c^4*x^2*y - c^6*x^2*y + 2*a^6*x*y^2 -      6*a^4*b^2*x*y^2 + 6*a^2*b^4*x*y^2 - 2*b^6*x*y^2 - 4*a^4*c^2*x*y^2 - a^2*b^2*c^2*x*y^2 +      5*b^4*c^2*x*y^2 + a^2*c^4*x*y^2 - 4*b^2*c^4*x*y^2 + c^6*x*y^2 - 2*a^6*x^2*z + 5*a^4*b^2*x^2*z -      4*a^2*b^4*x^2*z + b^6*x^2*z + 6*a^4*c^2*x^2*z - a^2*b^2*c^2*x^2*z + b^4*c^2*x^2*z -      6*a^2*c^4*x^2*z - 4*b^2*c^4*x^2*z + 2*c^6*x^2*z - a^6*y^2*z + 4*a^4*b^2*y^2*z - 5*a^2*b^4*y^2*z +      2*b^6*y^2*z - a^4*c^2*y^2*z + a^2*b^2*c^2*y^2*z - 6*b^4*c^2*y^2*z + 4*a^2*c^4*y^2*z +      6*b^2*c^4*y^2*z - 2*c^6*y^2*z - 2*a^6*x*z^2 + 4*a^4*b^2*x*z^2 - a^2*b^4*x*z^2 - b^6*x*z^2 +      6*a^4*c^2*x*z^2 + a^2*b^2*c^2*x*z^2 + 4*b^4*c^2*x*z^2 - 6*a^2*c^4*x*z^2 - 5*b^2*c^4*x*z^2 +      2*c^6*x*z^2 + a^6*y*z^2 + a^4*b^2*y*z^2 - 4*a^2*b^4*y*z^2 + 2*b^6*y*z^2 - 4*a^4*c^2*y*z^2 -      a^2*b^2*c^2*y*z^2 - 6*b^4*c^2*y*z^2 + 5*a^2*c^4*y*z^2 + 6*b^2*c^4*y*z^2 - 2*c^6*y*z^2");
		exp("K120","a^6*x^2*y - 3*a^4*b^2*x^2*y + 3*a^2*b^4*x^2*y - b^6*x^2*y - 3*a^4*c^2*x^2*y + 3*b^4*c^2*x^2*y +      3*a^2*c^4*x^2*y - b^2*c^4*x^2*y - c^6*x^2*y + a^6*x*y^2 - 3*a^4*b^2*x*y^2 + 3*a^2*b^4*x*y^2 -      b^6*x*y^2 - 3*a^4*c^2*x*y^2 + 3*b^4*c^2*x*y^2 + a^2*c^4*x*y^2 - 3*b^2*c^4*x*y^2 + c^6*x*y^2 -      a^6*x^2*z + 3*a^4*b^2*x^2*z - 3*a^2*b^4*x^2*z + b^6*x^2*z + 3*a^4*c^2*x^2*z + b^4*c^2*x^2*z -      3*a^2*c^4*x^2*z - 3*b^2*c^4*x^2*z + c^6*x^2*z - a^6*y^2*z + 3*a^4*b^2*y^2*z - 3*a^2*b^4*y^2*z +      b^6*y^2*z - a^4*c^2*y^2*z - 3*b^4*c^2*y^2*z + 3*a^2*c^4*y^2*z + 3*b^2*c^4*y^2*z - c^6*y^2*z -      a^6*x*z^2 + 3*a^4*b^2*x*z^2 - a^2*b^4*x*z^2 - b^6*x*z^2 + 3*a^4*c^2*x*z^2 + 3*b^4*c^2*x*z^2 -      3*a^2*c^4*x*z^2 - 3*b^2*c^4*x*z^2 + c^6*x*z^2 + a^6*y*z^2 + a^4*b^2*y*z^2 - 3*a^2*b^4*y*z^2 +      b^6*y*z^2 - 3*a^4*c^2*y*z^2 - 3*b^4*c^2*y*z^2 + 3*a^2*c^4*y*z^2 + 3*b^2*c^4*y*z^2 - c^6*y*z^2");
		exp("K121","-2*a^6*x^2*y + 6*a^4*b^2*x^2*y - 6*a^2*b^4*x^2*y + 2*b^6*x^2*y + a^4*c^2*x^2*y -      a^2*b^2*c^2*x^2*y + 4*a^2*c^4*x^2*y + b^2*c^4*x^2*y - 3*c^6*x^2*y - 2*a^6*x*y^2 + 6*a^4*b^2*x*y^2 -      6*a^2*b^4*x*y^2 + 2*b^6*x*y^2 + a^2*b^2*c^2*x*y^2 - b^4*c^2*x*y^2 - a^2*c^4*x*y^2 -      4*b^2*c^4*x*y^2 + 3*c^6*x*y^2 + 2*a^6*x^2*z - a^4*b^2*x^2*z - 4*a^2*b^4*x^2*z + 3*b^6*x^2*z -      6*a^4*c^2*x^2*z + a^2*b^2*c^2*x^2*z - b^4*c^2*x^2*z + 6*a^2*c^4*x^2*z - 2*c^6*x^2*z - 3*a^6*y^2*z +      4*a^4*b^2*y^2*z + a^2*b^4*y^2*z - 2*b^6*y^2*z + a^4*c^2*y^2*z - a^2*b^2*c^2*y^2*z +      6*b^4*c^2*y^2*z - 6*b^2*c^4*y^2*z + 2*c^6*y^2*z + 2*a^6*x*z^2 + a^2*b^4*x*z^2 - 3*b^6*x*z^2 -      6*a^4*c^2*x*z^2 - a^2*b^2*c^2*x*z^2 + 4*b^4*c^2*x*z^2 + 6*a^2*c^4*x*z^2 + b^2*c^4*x*z^2 -      2*c^6*x*z^2 + 3*a^6*y*z^2 - a^4*b^2*y*z^2 - 2*b^6*y*z^2 - 4*a^4*c^2*y*z^2 + a^2*b^2*c^2*y*z^2 +      6*b^4*c^2*y*z^2 - a^2*c^4*y*z^2 - 6*b^2*c^4*y*z^2 + 2*c^6*y*z^2");
		exp("K122","(-a^6)*x^2*y + 3*a^4*b^2*x^2*y - 3*a^2*b^4*x^2*y + b^6*x^2*y + 3*a^2*c^4*x^2*y + b^2*c^4*x^2*y -      2*c^6*x^2*y - a^6*x*y^2 + 3*a^4*b^2*x*y^2 - 3*a^2*b^4*x*y^2 + b^6*x*y^2 - a^2*c^4*x*y^2 -      3*b^2*c^4*x*y^2 + 2*c^6*x*y^2 + a^6*x^2*z - 3*a^2*b^4*x^2*z + 2*b^6*x^2*z - 3*a^4*c^2*x^2*z -      b^4*c^2*x^2*z + 3*a^2*c^4*x^2*z - c^6*x^2*z - 2*a^6*y^2*z + 3*a^4*b^2*y^2*z - b^6*y^2*z +      a^4*c^2*y^2*z + 3*b^4*c^2*y^2*z - 3*b^2*c^4*y^2*z + c^6*y^2*z + a^6*x*z^2 + a^2*b^4*x*z^2 -      2*b^6*x*z^2 - 3*a^4*c^2*x*z^2 + 3*b^4*c^2*x*z^2 + 3*a^2*c^4*x*z^2 - c^6*x*z^2 + 2*a^6*y*z^2 -      a^4*b^2*y*z^2 - b^6*y*z^2 - 3*a^4*c^2*y*z^2 + 3*b^4*c^2*y*z^2 - 3*b^2*c^4*y*z^2 + c^6*y*z^2");
		exp("K123","(-a^6)*x^2*y + 3*a^4*b^2*x^2*y - 3*a^2*b^4*x^2*y + b^6*x^2*y - a^4*c^2*x^2*y +      a^2*b^2*c^2*x^2*y + 5*a^2*c^4*x^2*y + 2*b^2*c^4*x^2*y - 3*c^6*x^2*y - a^6*x*y^2 + 3*a^4*b^2*x*y^2 -      3*a^2*b^4*x*y^2 + b^6*x*y^2 - a^2*b^2*c^2*x*y^2 + b^4*c^2*x*y^2 - 2*a^2*c^4*x*y^2 -      5*b^2*c^4*x*y^2 + 3*c^6*x*y^2 + a^6*x^2*z + a^4*b^2*x^2*z - 5*a^2*b^4*x^2*z + 3*b^6*x^2*z -      3*a^4*c^2*x^2*z - a^2*b^2*c^2*x^2*z - 2*b^4*c^2*x^2*z + 3*a^2*c^4*x^2*z - c^6*x^2*z - 3*a^6*y^2*z +      5*a^4*b^2*y^2*z - a^2*b^4*y^2*z - b^6*y^2*z + 2*a^4*c^2*y^2*z + a^2*b^2*c^2*y^2*z +      3*b^4*c^2*y^2*z - 3*b^2*c^4*y^2*z + c^6*y^2*z + a^6*x*z^2 + 2*a^2*b^4*x*z^2 - 3*b^6*x*z^2 -      3*a^4*c^2*x*z^2 + a^2*b^2*c^2*x*z^2 + 5*b^4*c^2*x*z^2 + 3*a^2*c^4*x*z^2 - b^2*c^4*x*z^2 -      c^6*x*z^2 + 3*a^6*y*z^2 - 2*a^4*b^2*y*z^2 - b^6*y*z^2 - 5*a^4*c^2*y*z^2 - a^2*b^2*c^2*y*z^2 +      3*b^4*c^2*y*z^2 + a^2*c^4*y*z^2 - 3*b^2*c^4*y*z^2 + c^6*y*z^2");
		exp("K124","a^2*x^2*y - b^2*x^2*y - 2*c^2*x^2*y + a^2*x*y^2 - b^2*x*y^2 + 2*c^2*x*y^2 - a^2*x^2*z +      2*b^2*x^2*z + c^2*x^2*z - 2*a^2*y^2*z + b^2*y^2*z - c^2*y^2*z - a^2*x*z^2 - 2*b^2*x*z^2 +      c^2*x*z^2 + 2*a^2*y*z^2 + b^2*y*z^2 - c^2*y*z^2");
		exp("K125","a^6*x^2*y - 3*a^4*b^2*x^2*y + 3*a^2*b^4*x^2*y - b^6*x^2*y - 3*a^4*c^2*x^2*y +      3*a^2*b^2*c^2*x^2*y + 3*a^2*c^4*x^2*y + 2*b^2*c^4*x^2*y - c^6*x^2*y + a^6*x*y^2 - 3*a^4*b^2*x*y^2 +      3*a^2*b^4*x*y^2 - b^6*x*y^2 - 3*a^2*b^2*c^2*x*y^2 + 3*b^4*c^2*x*y^2 - 2*a^2*c^4*x*y^2 -      3*b^2*c^4*x*y^2 + c^6*x*y^2 - a^6*x^2*z + 3*a^4*b^2*x^2*z - 3*a^2*b^4*x^2*z + b^6*x^2*z +      3*a^4*c^2*x^2*z - 3*a^2*b^2*c^2*x^2*z - 2*b^4*c^2*x^2*z - 3*a^2*c^4*x^2*z + c^6*x^2*z - a^6*y^2*z +      3*a^4*b^2*y^2*z - 3*a^2*b^4*y^2*z + b^6*y^2*z + 2*a^4*c^2*y^2*z + 3*a^2*b^2*c^2*y^2*z -      3*b^4*c^2*y^2*z + 3*b^2*c^4*y^2*z - c^6*y^2*z - a^6*x*z^2 + 2*a^2*b^4*x*z^2 - b^6*x*z^2 +      3*a^4*c^2*x*z^2 + 3*a^2*b^2*c^2*x*z^2 + 3*b^4*c^2*x*z^2 - 3*a^2*c^4*x*z^2 - 3*b^2*c^4*x*z^2 +      c^6*x*z^2 + a^6*y*z^2 - 2*a^4*b^2*y*z^2 + b^6*y*z^2 - 3*a^4*c^2*y*z^2 - 3*a^2*b^2*c^2*y*z^2 -      3*b^4*c^2*y*z^2 + 3*a^2*c^4*y*z^2 + 3*b^2*c^4*y*z^2 - c^6*y*z^2");
		exp("K126","a^6*c^2*x^2*y - 2*a^4*b^2*c^2*x^2*y + a^2*b^4*c^2*x^2*y - a^4*c^4*x^2*y - b^4*c^4*x^2*y -      a^2*c^6*x^2*y + c^8*x^2*y - a^4*b^2*c^2*x*y^2 + 2*a^2*b^4*c^2*x*y^2 - b^6*c^2*x*y^2 +      a^4*c^4*x*y^2 + b^4*c^4*x*y^2 + b^2*c^6*x*y^2 - c^8*x*y^2 - a^6*b^2*x^2*z + a^4*b^4*x^2*z +      a^2*b^6*x^2*z - b^8*x^2*z + 2*a^4*b^2*c^2*x^2*z - a^2*b^2*c^4*x^2*z + b^4*c^4*x^2*z + a^8*y^2*z -      a^6*b^2*y^2*z - a^4*b^4*y^2*z + a^2*b^6*y^2*z - 2*a^2*b^4*c^2*y^2*z - a^4*c^4*y^2*z +      a^2*b^2*c^4*y^2*z - a^4*b^4*x*z^2 + b^8*x*z^2 + a^4*b^2*c^2*x*z^2 - b^6*c^2*x*z^2 -      2*a^2*b^2*c^4*x*z^2 - b^4*c^4*x*z^2 + b^2*c^6*x*z^2 - a^8*y*z^2 + a^4*b^4*y*z^2 + a^6*c^2*y*z^2 -      a^2*b^4*c^2*y*z^2 + a^4*c^4*y*z^2 + 2*a^2*b^2*c^4*y*z^2 - a^2*c^6*y*z^2");
		exp("K127","3*a^6*x^2*y - 9*a^4*b^2*x^2*y + 9*a^2*b^4*x^2*y - 3*b^6*x^2*y - 4*a^4*c^2*x^2*y +      4*a^2*b^2*c^2*x^2*y - a^2*c^4*x^2*y + b^2*c^4*x^2*y + 2*c^6*x^2*y + 3*a^6*x*y^2 - 9*a^4*b^2*x*y^2 +      9*a^2*b^4*x*y^2 - 3*b^6*x*y^2 - 4*a^2*b^2*c^2*x*y^2 + 4*b^4*c^2*x*y^2 - a^2*c^4*x*y^2 +      b^2*c^4*x*y^2 - 2*c^6*x*y^2 - 3*a^6*x^2*z + 4*a^4*b^2*x^2*z + a^2*b^4*x^2*z - 2*b^6*x^2*z +      9*a^4*c^2*x^2*z - 4*a^2*b^2*c^2*x^2*z - b^4*c^2*x^2*z - 9*a^2*c^4*x^2*z + 3*c^6*x^2*z +      2*a^6*y^2*z - a^4*b^2*y^2*z - 4*a^2*b^4*y^2*z + 3*b^6*y^2*z + a^4*c^2*y^2*z + 4*a^2*b^2*c^2*y^2*z -      9*b^4*c^2*y^2*z + 9*b^2*c^4*y^2*z - 3*c^6*y^2*z - 3*a^6*x*z^2 + a^2*b^4*x*z^2 + 2*b^6*x*z^2 +      9*a^4*c^2*x*z^2 + 4*a^2*b^2*c^2*x*z^2 - b^4*c^2*x*z^2 - 9*a^2*c^4*x*z^2 - 4*b^2*c^4*x*z^2 +      3*c^6*x*z^2 - 2*a^6*y*z^2 - a^4*b^2*y*z^2 + 3*b^6*y*z^2 + a^4*c^2*y*z^2 - 4*a^2*b^2*c^2*y*z^2 -      9*b^4*c^2*y*z^2 + 4*a^2*c^4*y*z^2 + 9*b^2*c^4*y*z^2 - 3*c^6*y*z^2");
		exp("K128","(-b^4)*c^2*x^2*y + a^2*c^4*x^2*y + a^4*c^2*x*y^2 - b^2*c^4*x*y^2 - a^2*b^4*x^2*z +      b^2*c^4*x^2*z + a^4*b^2*y^2*z - a^2*c^4*y^2*z - a^4*b^2*x*z^2 + b^4*c^2*x*z^2 + a^2*b^4*y*z^2 -      a^4*c^2*y*z^2");
		exp("K129a","Sqrt((a + b - c)*(a - b + c)*(-a + b + c)*(a + b + c))*(c^2*x^2*y - c^2*x*y^2 - b^2*x^2*z +        a^2*y^2*z + b^2*x*z^2 - a^2*y*z^2) + Sqrt(3)*((-b^2)*c^2*x^2*y + a^2*c^2*x*y^2 + b^2*c^2*x^2*z -        a^2*c^2*y^2*z - a^2*b^2*x*z^2 + a^2*b^2*y*z^2)");
		exp("K129b","Sqrt((a + b - c)*(a - b + c)*(-a + b + c)*(a + b + c))*((-c^2)*x^2*y + c^2*x*y^2 + b^2*x^2*z -        a^2*y^2*z - b^2*x*z^2 + a^2*y*z^2) + Sqrt(3)*((-b^2)*c^2*x^2*y + a^2*c^2*x*y^2 + b^2*c^2*x^2*z -        a^2*c^2*y^2*z - a^2*b^2*x*z^2 + a^2*b^2*y*z^2)");
		exp("K130","(-a^10)*b^2*c^2*x^2*y + 2*a^8*b^4*c^2*x^2*y - a^6*b^6*c^2*x^2*y + a^4*b^8*c^2*x^2*y -      2*a^2*b^10*c^2*x^2*y + b^12*c^2*x^2*y + a^10*c^4*x^2*y + 2*a^8*b^2*c^4*x^2*y -      4*a^6*b^4*c^4*x^2*y - a^4*b^6*c^4*x^2*y + 4*a^2*b^8*c^4*x^2*y - 2*b^10*c^4*x^2*y -      4*a^8*c^6*x^2*y - a^6*b^2*c^6*x^2*y + 5*a^4*b^4*c^6*x^2*y - a^2*b^6*c^6*x^2*y + b^8*c^6*x^2*y +      6*a^6*c^8*x^2*y - a^4*b^2*c^8*x^2*y - 4*a^2*b^4*c^8*x^2*y - b^6*c^8*x^2*y - 4*a^4*c^10*x^2*y +      2*a^2*b^2*c^10*x^2*y + 2*b^4*c^10*x^2*y + a^2*c^12*x^2*y - b^2*c^12*x^2*y - a^12*c^2*x*y^2 +      2*a^10*b^2*c^2*x*y^2 - a^8*b^4*c^2*x*y^2 + a^6*b^6*c^2*x*y^2 - 2*a^4*b^8*c^2*x*y^2 +      a^2*b^10*c^2*x*y^2 + 2*a^10*c^4*x*y^2 - 4*a^8*b^2*c^4*x*y^2 + a^6*b^4*c^4*x*y^2 +      4*a^4*b^6*c^4*x*y^2 - 2*a^2*b^8*c^4*x*y^2 - b^10*c^4*x*y^2 - a^8*c^6*x*y^2 + a^6*b^2*c^6*x*y^2 -      5*a^4*b^4*c^6*x*y^2 + a^2*b^6*c^6*x*y^2 + 4*b^8*c^6*x*y^2 + a^6*c^8*x*y^2 + 4*a^4*b^2*c^8*x*y^2 +      a^2*b^4*c^8*x*y^2 - 6*b^6*c^8*x*y^2 - 2*a^4*c^10*x*y^2 - 2*a^2*b^2*c^10*x*y^2 + 4*b^4*c^10*x*y^2 +      a^2*c^12*x*y^2 - b^2*c^12*x*y^2 - a^10*b^4*x^2*z + 4*a^8*b^6*x^2*z - 6*a^6*b^8*x^2*z +      4*a^4*b^10*x^2*z - a^2*b^12*x^2*z + a^10*b^2*c^2*x^2*z - 2*a^8*b^4*c^2*x^2*z + a^6*b^6*c^2*x^2*z +      a^4*b^8*c^2*x^2*z - 2*a^2*b^10*c^2*x^2*z + b^12*c^2*x^2*z - 2*a^8*b^2*c^4*x^2*z +      4*a^6*b^4*c^4*x^2*z - 5*a^4*b^6*c^4*x^2*z + 4*a^2*b^8*c^4*x^2*z - 2*b^10*c^4*x^2*z +      a^6*b^2*c^6*x^2*z + a^4*b^4*c^6*x^2*z + a^2*b^6*c^6*x^2*z + b^8*c^6*x^2*z - a^4*b^2*c^8*x^2*z -      4*a^2*b^4*c^8*x^2*z - b^6*c^8*x^2*z + 2*a^2*b^2*c^10*x^2*z + 2*b^4*c^10*x^2*z - b^2*c^12*x^2*z +      a^12*b^2*y^2*z - 4*a^10*b^4*y^2*z + 6*a^8*b^6*y^2*z - 4*a^6*b^8*y^2*z + a^4*b^10*y^2*z -      a^12*c^2*y^2*z + 2*a^10*b^2*c^2*y^2*z - a^8*b^4*c^2*y^2*z - a^6*b^6*c^2*y^2*z +      2*a^4*b^8*c^2*y^2*z - a^2*b^10*c^2*y^2*z + 2*a^10*c^4*y^2*z - 4*a^8*b^2*c^4*y^2*z +      5*a^6*b^4*c^4*y^2*z - 4*a^4*b^6*c^4*y^2*z + 2*a^2*b^8*c^4*y^2*z - a^8*c^6*y^2*z -      a^6*b^2*c^6*y^2*z - a^4*b^4*c^6*y^2*z - a^2*b^6*c^6*y^2*z + a^6*c^8*y^2*z + 4*a^4*b^2*c^8*y^2*z +      a^2*b^4*c^8*y^2*z - 2*a^4*c^10*y^2*z - 2*a^2*b^2*c^10*y^2*z + a^2*c^12*y^2*z + a^12*b^2*x*z^2 -      2*a^10*b^4*x*z^2 + a^8*b^6*x*z^2 - a^6*b^8*x*z^2 + 2*a^4*b^10*x*z^2 - a^2*b^12*x*z^2 -      2*a^10*b^2*c^2*x*z^2 + 4*a^8*b^4*c^2*x*z^2 - a^6*b^6*c^2*x*z^2 - 4*a^4*b^8*c^2*x*z^2 +      2*a^2*b^10*c^2*x*z^2 + b^12*c^2*x*z^2 + a^8*b^2*c^4*x*z^2 - a^6*b^4*c^4*x*z^2 +      5*a^4*b^6*c^4*x*z^2 - a^2*b^8*c^4*x*z^2 - 4*b^10*c^4*x*z^2 - a^6*b^2*c^6*x*z^2 -      4*a^4*b^4*c^6*x*z^2 - a^2*b^6*c^6*x*z^2 + 6*b^8*c^6*x*z^2 + 2*a^4*b^2*c^8*x*z^2 +      2*a^2*b^4*c^8*x*z^2 - 4*b^6*c^8*x*z^2 - a^2*b^2*c^10*x*z^2 + b^4*c^10*x*z^2 + a^12*b^2*y*z^2 -      2*a^10*b^4*y*z^2 + a^8*b^6*y*z^2 - a^6*b^8*y*z^2 + 2*a^4*b^10*y*z^2 - a^2*b^12*y*z^2 -      a^12*c^2*y*z^2 - 2*a^10*b^2*c^2*y*z^2 + 4*a^8*b^4*c^2*y*z^2 + a^6*b^6*c^2*y*z^2 -      4*a^4*b^8*c^2*y*z^2 + 2*a^2*b^10*c^2*y*z^2 + 4*a^10*c^4*y*z^2 + a^8*b^2*c^4*y*z^2 -      5*a^6*b^4*c^4*y*z^2 + a^4*b^6*c^4*y*z^2 - a^2*b^8*c^4*y*z^2 - 6*a^8*c^6*y*z^2 + a^6*b^2*c^6*y*z^2 +      4*a^4*b^4*c^6*y*z^2 + a^2*b^6*c^6*y*z^2 + 4*a^6*c^8*y*z^2 - 2*a^4*b^2*c^8*y*z^2 -      2*a^2*b^4*c^8*y*z^2 - a^4*c^10*y*z^2 + a^2*b^2*c^10*y*z^2");
		exp("K131","(-b^3)*c^3*x^2*y - a*b*c^4*x^2*y + a^3*c^3*x*y^2 + a*b*c^4*x*y^2 + a*b^4*c*x^2*z +      b^3*c^3*x^2*z - a^4*b*c*y^2*z - a^3*c^3*y^2*z - a^3*b^3*x*z^2 - a*b^4*c*x*z^2 + a^3*b^3*y*z^2 +      a^4*b*c*y*z^2");
		exp("K132","(-b^2)*c*x^2*y - a*c^2*x^2*y + a^2*c*x*y^2 + b*c^2*x*y^2 + a*b^2*x^2*z + b*c^2*x^2*z -      a^2*b*y^2*z - a*c^2*y^2*z - a^2*b*x*z^2 - b^2*c*x*z^2 + a*b^2*y*z^2 + a^2*c*y*z^2");
		exp("K133","(-a^7)*c*x^2*y + 3*a^5*b^2*c*x^2*y - 3*a^3*b^4*c*x^2*y + a*b^6*c*x^2*y - 2*a^6*c^2*x^2*y -      4*a^4*b^2*c^2*x^2*y + 6*a^2*b^4*c^2*x^2*y + a^5*c^3*x^2*y + 2*a^3*b^2*c^3*x^2*y -      3*a*b^4*c^3*x^2*y + 4*a^4*c^4*x^2*y - 4*a^2*b^2*c^4*x^2*y + a^3*c^5*x^2*y + 3*a*b^2*c^5*x^2*y -      2*a^2*c^6*x^2*y - a*c^7*x^2*y - a^6*b*c*x*y^2 + 3*a^4*b^3*c*x*y^2 - 3*a^2*b^5*c*x*y^2 +      b^7*c*x*y^2 - 6*a^4*b^2*c^2*x*y^2 + 4*a^2*b^4*c^2*x*y^2 + 2*b^6*c^2*x*y^2 + 3*a^4*b*c^3*x*y^2 -      2*a^2*b^3*c^3*x*y^2 - b^5*c^3*x*y^2 + 4*a^2*b^2*c^4*x*y^2 - 4*b^4*c^4*x*y^2 - 3*a^2*b*c^5*x*y^2 -      b^3*c^5*x*y^2 + 2*b^2*c^6*x*y^2 + b*c^7*x*y^2 + a^7*b*x^2*z + 2*a^6*b^2*x^2*z - a^5*b^3*x^2*z -      4*a^4*b^4*x^2*z - a^3*b^5*x^2*z + 2*a^2*b^6*x^2*z + a*b^7*x^2*z - 3*a^5*b*c^2*x^2*z +      4*a^4*b^2*c^2*x^2*z - 2*a^3*b^3*c^2*x^2*z + 4*a^2*b^4*c^2*x^2*z - 3*a*b^5*c^2*x^2*z +      3*a^3*b*c^4*x^2*z - 6*a^2*b^2*c^4*x^2*z + 3*a*b^3*c^4*x^2*z - a*b*c^6*x^2*z - a^7*b*y^2*z -      2*a^6*b^2*y^2*z + a^5*b^3*y^2*z + 4*a^4*b^4*y^2*z + a^3*b^5*y^2*z - 2*a^2*b^6*y^2*z - a*b^7*y^2*z +      3*a^5*b*c^2*y^2*z - 4*a^4*b^2*c^2*y^2*z + 2*a^3*b^3*c^2*y^2*z - 4*a^2*b^4*c^2*y^2*z +      3*a*b^5*c^2*y^2*z - 3*a^3*b*c^4*y^2*z + 6*a^2*b^2*c^4*y^2*z - 3*a*b^3*c^4*y^2*z + a*b*c^6*y^2*z +      a^6*b*c*x*z^2 - 3*a^4*b^3*c*x*z^2 + 3*a^2*b^5*c*x*z^2 - b^7*c*x*z^2 + 6*a^4*b^2*c^2*x*z^2 -      4*a^2*b^4*c^2*x*z^2 - 2*b^6*c^2*x*z^2 - 3*a^4*b*c^3*x*z^2 + 2*a^2*b^3*c^3*x*z^2 + b^5*c^3*x*z^2 -      4*a^2*b^2*c^4*x*z^2 + 4*b^4*c^4*x*z^2 + 3*a^2*b*c^5*x*z^2 + b^3*c^5*x*z^2 - 2*b^2*c^6*x*z^2 -      b*c^7*x*z^2 + a^7*c*y*z^2 - 3*a^5*b^2*c*y*z^2 + 3*a^3*b^4*c*y*z^2 - a*b^6*c*y*z^2 +      2*a^6*c^2*y*z^2 + 4*a^4*b^2*c^2*y*z^2 - 6*a^2*b^4*c^2*y*z^2 - a^5*c^3*y*z^2 - 2*a^3*b^2*c^3*y*z^2 +      3*a*b^4*c^3*y*z^2 - 4*a^4*c^4*y*z^2 + 4*a^2*b^2*c^4*y*z^2 - a^3*c^5*y*z^2 - 3*a*b^2*c^5*y*z^2 +      2*a^2*c^6*y*z^2 + a*c^7*y*z^2");
		exp("K134","a^2*c*x^2*y - a*b*c*x^2*y - b*c^2*x^2*y - c^3*x^2*y + a*b*c*x*y^2 - b^2*c*x*y^2 + a*c^2*x*y^2 +      c^3*x*y^2 - a^2*b*x^2*z + b^3*x^2*z + a*b*c*x^2*z + b^2*c*x^2*z - a^3*y^2*z + a*b^2*y^2*z -      a^2*c*y^2*z - a*b*c*y^2*z - a*b^2*x*z^2 - b^3*x*z^2 - a*b*c*x*z^2 + b*c^2*x*z^2 + a^3*y*z^2 +      a^2*b*y*z^2 + a*b*c*y*z^2 - a*c^2*y*z^2");
		exp("K135","(-a^2)*b*c^3*x^2*y + b^2*c^4*x^2*y + a*b^2*c^3*x*y^2 - a^2*c^4*x*y^2 + a^2*b^3*c*x^2*z -      b^4*c^2*x^2*z - a^3*b^2*c*y^2*z + a^4*c^2*y^2*z + a^2*b^4*x*z^2 - a*b^3*c^2*x*z^2 - a^4*b^2*y*z^2 +      a^3*b*c^2*y*z^2");
		exp("K136","a^2*b^2*c^2*x^2*y - b^3*c^3*x^2*y - a^2*b^2*c^2*x*y^2 + a^3*c^3*x*y^2 - a^2*b^2*c^2*x^2*z +      b^3*c^3*x^2*z + a^2*b^2*c^2*y^2*z - a^3*c^3*y^2*z - a^3*b^3*x*z^2 + a^2*b^2*c^2*x*z^2 +      a^3*b^3*y*z^2 - a^2*b^2*c^2*y*z^2");
		exp("K137","(-a)*b*c^2*x^2*y + b*c^3*x^2*y + a*b*c^2*x*y^2 - a*c^3*x*y^2 + a*b^2*c*x^2*z - b^3*c*x^2*z +      a^3*c*y^2*z - a^2*b*c*y^2*z + a*b^3*x*z^2 - a*b^2*c*x*z^2 - a^3*b*y*z^2 + a^2*b*c*y*z^2");
		exp("K138","-2*b^4*c^2*x^3 - 2*b^2*c^4*x^3 + a^2*b^2*c^2*x^2*y + b^4*c^2*x^2*y + b^2*c^4*x^2*y +      a^4*c^2*x*y^2 + a^2*b^2*c^2*x*y^2 + a^2*c^4*x*y^2 - 2*a^4*c^2*y^3 - 2*a^2*c^4*y^3 +      a^2*b^2*c^2*x^2*z + b^4*c^2*x^2*z + b^2*c^4*x^2*z - 6*a^2*b^2*c^2*x*y*z + a^4*c^2*y^2*z +      a^2*b^2*c^2*y^2*z + a^2*c^4*y^2*z + a^4*b^2*x*z^2 + a^2*b^4*x*z^2 + a^2*b^2*c^2*x*z^2 +      a^4*b^2*y*z^2 + a^2*b^4*y*z^2 + a^2*b^2*c^2*y*z^2 - 2*a^4*b^2*z^3 - 2*a^2*b^4*z^3");
		exp("K139","a^18*x^2*y - 8*a^16*b^2*x^2*y + 28*a^14*b^4*x^2*y - 56*a^12*b^6*x^2*y + 70*a^10*b^8*x^2*y -      56*a^8*b^10*x^2*y + 28*a^6*b^12*x^2*y - 8*a^4*b^14*x^2*y + a^2*b^16*x^2*y + 6*a^14*b^2*c^2*x^2*y -      28*a^12*b^4*c^2*x^2*y + 51*a^10*b^6*c^2*x^2*y - 45*a^8*b^8*c^2*x^2*y + 20*a^6*b^10*c^2*x^2*y -      6*a^4*b^12*c^2*x^2*y + 3*a^2*b^14*c^2*x^2*y - b^16*c^2*x^2*y - 9*a^14*c^4*x^2*y +      29*a^12*b^2*c^4*x^2*y - 36*a^10*b^4*c^4*x^2*y + 21*a^8*b^6*c^4*x^2*y - 7*a^6*b^8*c^4*x^2*y +      9*a^4*b^10*c^4*x^2*y - 12*a^2*b^12*c^4*x^2*y + 5*b^14*c^4*x^2*y + 15*a^12*c^6*x^2*y -      41*a^10*b^2*c^6*x^2*y + 52*a^8*b^4*c^6*x^2*y - 36*a^6*b^6*c^6*x^2*y + 4*a^4*b^8*c^6*x^2*y +      16*a^2*b^10*c^6*x^2*y - 10*b^12*c^6*x^2*y - 9*a^10*c^8*x^2*y + 3*a^8*b^2*c^8*x^2*y -      7*a^6*b^4*c^8*x^2*y + 15*a^4*b^6*c^8*x^2*y - 12*a^2*b^8*c^8*x^2*y + 11*b^10*c^8*x^2*y +      9*a^8*c^10*x^2*y + 20*a^6*b^2*c^10*x^2*y - 18*a^4*b^4*c^10*x^2*y - 6*a^2*b^6*c^10*x^2*y -      10*b^8*c^10*x^2*y - 15*a^6*c^12*x^2*y - 5*a^4*b^2*c^12*x^2*y + 19*a^2*b^4*c^12*x^2*y +      11*b^6*c^12*x^2*y + 9*a^4*c^14*x^2*y - 9*a^2*b^2*c^14*x^2*y - 10*b^4*c^14*x^2*y +      5*b^2*c^16*x^2*y - c^18*x^2*y - a^16*b^2*x*y^2 + 8*a^14*b^4*x*y^2 - 28*a^12*b^6*x*y^2 +      56*a^10*b^8*x*y^2 - 70*a^8*b^10*x*y^2 + 56*a^6*b^12*x*y^2 - 28*a^4*b^14*x*y^2 + 8*a^2*b^16*x*y^2 -      b^18*x*y^2 + a^16*c^2*x*y^2 - 3*a^14*b^2*c^2*x*y^2 + 6*a^12*b^4*c^2*x*y^2 - 20*a^10*b^6*c^2*x*y^2 +      45*a^8*b^8*c^2*x*y^2 - 51*a^6*b^10*c^2*x*y^2 + 28*a^4*b^12*c^2*x*y^2 - 6*a^2*b^14*c^2*x*y^2 -      5*a^14*c^4*x*y^2 + 12*a^12*b^2*c^4*x*y^2 - 9*a^10*b^4*c^4*x*y^2 + 7*a^8*b^6*c^4*x*y^2 -      21*a^6*b^8*c^4*x*y^2 + 36*a^4*b^10*c^4*x*y^2 - 29*a^2*b^12*c^4*x*y^2 + 9*b^14*c^4*x*y^2 +      10*a^12*c^6*x*y^2 - 16*a^10*b^2*c^6*x*y^2 - 4*a^8*b^4*c^6*x*y^2 + 36*a^6*b^6*c^6*x*y^2 -      52*a^4*b^8*c^6*x*y^2 + 41*a^2*b^10*c^6*x*y^2 - 15*b^12*c^6*x*y^2 - 11*a^10*c^8*x*y^2 +      12*a^8*b^2*c^8*x*y^2 - 15*a^6*b^4*c^8*x*y^2 + 7*a^4*b^6*c^8*x*y^2 - 3*a^2*b^8*c^8*x*y^2 +      9*b^10*c^8*x*y^2 + 10*a^8*c^10*x*y^2 + 6*a^6*b^2*c^10*x*y^2 + 18*a^4*b^4*c^10*x*y^2 -      20*a^2*b^6*c^10*x*y^2 - 9*b^8*c^10*x*y^2 - 11*a^6*c^12*x*y^2 - 19*a^4*b^2*c^12*x*y^2 +      5*a^2*b^4*c^12*x*y^2 + 15*b^6*c^12*x*y^2 + 10*a^4*c^14*x*y^2 + 9*a^2*b^2*c^14*x*y^2 -      9*b^4*c^14*x*y^2 - 5*a^2*c^16*x*y^2 + c^18*x*y^2 - a^18*x^2*z + 9*a^14*b^4*x^2*z -      15*a^12*b^6*x^2*z + 9*a^10*b^8*x^2*z - 9*a^8*b^10*x^2*z + 15*a^6*b^12*x^2*z - 9*a^4*b^14*x^2*z +      b^18*x^2*z + 8*a^16*c^2*x^2*z - 6*a^14*b^2*c^2*x^2*z - 29*a^12*b^4*c^2*x^2*z +      41*a^10*b^6*c^2*x^2*z - 3*a^8*b^8*c^2*x^2*z - 20*a^6*b^10*c^2*x^2*z + 5*a^4*b^12*c^2*x^2*z +      9*a^2*b^14*c^2*x^2*z - 5*b^16*c^2*x^2*z - 28*a^14*c^4*x^2*z + 28*a^12*b^2*c^4*x^2*z +      36*a^10*b^4*c^4*x^2*z - 52*a^8*b^6*c^4*x^2*z + 7*a^6*b^8*c^4*x^2*z + 18*a^4*b^10*c^4*x^2*z -      19*a^2*b^12*c^4*x^2*z + 10*b^14*c^4*x^2*z + 56*a^12*c^6*x^2*z - 51*a^10*b^2*c^6*x^2*z -      21*a^8*b^4*c^6*x^2*z + 36*a^6*b^6*c^6*x^2*z - 15*a^4*b^8*c^6*x^2*z + 6*a^2*b^10*c^6*x^2*z -      11*b^12*c^6*x^2*z - 70*a^10*c^8*x^2*z + 45*a^8*b^2*c^8*x^2*z + 7*a^6*b^4*c^8*x^2*z -      4*a^4*b^6*c^8*x^2*z + 12*a^2*b^8*c^8*x^2*z + 10*b^10*c^8*x^2*z + 56*a^8*c^10*x^2*z -      20*a^6*b^2*c^10*x^2*z - 9*a^4*b^4*c^10*x^2*z - 16*a^2*b^6*c^10*x^2*z - 11*b^8*c^10*x^2*z -      28*a^6*c^12*x^2*z + 6*a^4*b^2*c^12*x^2*z + 12*a^2*b^4*c^12*x^2*z + 10*b^6*c^12*x^2*z +      8*a^4*c^14*x^2*z - 3*a^2*b^2*c^14*x^2*z - 5*b^4*c^14*x^2*z - a^2*c^16*x^2*z + b^2*c^16*x^2*z -      12*a^16*b^2*x*y*z + 48*a^14*b^4*x*y*z - 72*a^12*b^6*x*y*z + 60*a^10*b^8*x*y*z - 60*a^8*b^10*x*y*z +      72*a^6*b^12*x*y*z - 48*a^4*b^14*x*y*z + 12*a^2*b^16*x*y*z + 12*a^16*c^2*x*y*z -      84*a^12*b^4*c^2*x*y*z + 120*a^10*b^6*c^2*x*y*z - 120*a^6*b^10*c^2*x*y*z + 84*a^4*b^12*c^2*x*y*z -      12*b^16*c^2*x*y*z - 48*a^14*c^4*x*y*z + 84*a^12*b^2*c^4*x*y*z - 120*a^8*b^6*c^4*x*y*z +      120*a^6*b^8*c^4*x*y*z - 84*a^2*b^12*c^4*x*y*z + 48*b^14*c^4*x*y*z + 72*a^12*c^6*x*y*z -      120*a^10*b^2*c^6*x*y*z + 120*a^8*b^4*c^6*x*y*z - 120*a^4*b^8*c^6*x*y*z + 120*a^2*b^10*c^6*x*y*z -      72*b^12*c^6*x*y*z - 60*a^10*c^8*x*y*z - 120*a^6*b^4*c^8*x*y*z + 120*a^4*b^6*c^8*x*y*z +      60*b^10*c^8*x*y*z + 60*a^8*c^10*x*y*z + 120*a^6*b^2*c^10*x*y*z - 120*a^2*b^6*c^10*x*y*z -      60*b^8*c^10*x*y*z - 72*a^6*c^12*x*y*z - 84*a^4*b^2*c^12*x*y*z + 84*a^2*b^4*c^12*x*y*z +      72*b^6*c^12*x*y*z + 48*a^4*c^14*x*y*z - 48*b^4*c^14*x*y*z - 12*a^2*c^16*x*y*z + 12*b^2*c^16*x*y*z -      a^18*y^2*z + 9*a^14*b^4*y^2*z - 15*a^12*b^6*y^2*z + 9*a^10*b^8*y^2*z - 9*a^8*b^10*y^2*z +      15*a^6*b^12*y^2*z - 9*a^4*b^14*y^2*z + b^18*y^2*z + 5*a^16*c^2*y^2*z - 9*a^14*b^2*c^2*y^2*z -      5*a^12*b^4*c^2*y^2*z + 20*a^10*b^6*c^2*y^2*z + 3*a^8*b^8*c^2*y^2*z - 41*a^6*b^10*c^2*y^2*z +      29*a^4*b^12*c^2*y^2*z + 6*a^2*b^14*c^2*y^2*z - 8*b^16*c^2*y^2*z - 10*a^14*c^4*y^2*z +      19*a^12*b^2*c^4*y^2*z - 18*a^10*b^4*c^4*y^2*z - 7*a^8*b^6*c^4*y^2*z + 52*a^6*b^8*c^4*y^2*z -      36*a^4*b^10*c^4*y^2*z - 28*a^2*b^12*c^4*y^2*z + 28*b^14*c^4*y^2*z + 11*a^12*c^6*y^2*z -      6*a^10*b^2*c^6*y^2*z + 15*a^8*b^4*c^6*y^2*z - 36*a^6*b^6*c^6*y^2*z + 21*a^4*b^8*c^6*y^2*z +      51*a^2*b^10*c^6*y^2*z - 56*b^12*c^6*y^2*z - 10*a^10*c^8*y^2*z - 12*a^8*b^2*c^8*y^2*z +      4*a^6*b^4*c^8*y^2*z - 7*a^4*b^6*c^8*y^2*z - 45*a^2*b^8*c^8*y^2*z + 70*b^10*c^8*y^2*z +      11*a^8*c^10*y^2*z + 16*a^6*b^2*c^10*y^2*z + 9*a^4*b^4*c^10*y^2*z + 20*a^2*b^6*c^10*y^2*z -      56*b^8*c^10*y^2*z - 10*a^6*c^12*y^2*z - 12*a^4*b^2*c^12*y^2*z - 6*a^2*b^4*c^12*y^2*z +      28*b^6*c^12*y^2*z + 5*a^4*c^14*y^2*z + 3*a^2*b^2*c^14*y^2*z - 8*b^4*c^14*y^2*z - a^2*c^16*y^2*z +      b^2*c^16*y^2*z - a^16*b^2*x*z^2 + 5*a^14*b^4*x*z^2 - 10*a^12*b^6*x*z^2 + 11*a^10*b^8*x*z^2 -      10*a^8*b^10*x*z^2 + 11*a^6*b^12*x*z^2 - 10*a^4*b^14*x*z^2 + 5*a^2*b^16*x*z^2 - b^18*x*z^2 +      a^16*c^2*x*z^2 + 3*a^14*b^2*c^2*x*z^2 - 12*a^12*b^4*c^2*x*z^2 + 16*a^10*b^6*c^2*x*z^2 -      12*a^8*b^8*c^2*x*z^2 - 6*a^6*b^10*c^2*x*z^2 + 19*a^4*b^12*c^2*x*z^2 - 9*a^2*b^14*c^2*x*z^2 -      8*a^14*c^4*x*z^2 - 6*a^12*b^2*c^4*x*z^2 + 9*a^10*b^4*c^4*x*z^2 + 4*a^8*b^6*c^4*x*z^2 +      15*a^6*b^8*c^4*x*z^2 - 18*a^4*b^10*c^4*x*z^2 - 5*a^2*b^12*c^4*x*z^2 + 9*b^14*c^4*x*z^2 +      28*a^12*c^6*x*z^2 + 20*a^10*b^2*c^6*x*z^2 - 7*a^8*b^4*c^6*x*z^2 - 36*a^6*b^6*c^6*x*z^2 -      7*a^4*b^8*c^6*x*z^2 + 20*a^2*b^10*c^6*x*z^2 - 15*b^12*c^6*x*z^2 - 56*a^10*c^8*x*z^2 -      45*a^8*b^2*c^8*x*z^2 + 21*a^6*b^4*c^8*x*z^2 + 52*a^4*b^6*c^8*x*z^2 + 3*a^2*b^8*c^8*x*z^2 +      9*b^10*c^8*x*z^2 + 70*a^8*c^10*x*z^2 + 51*a^6*b^2*c^10*x*z^2 - 36*a^4*b^4*c^10*x*z^2 -      41*a^2*b^6*c^10*x*z^2 - 9*b^8*c^10*x*z^2 - 56*a^6*c^12*x*z^2 - 28*a^4*b^2*c^12*x*z^2 +      29*a^2*b^4*c^12*x*z^2 + 15*b^6*c^12*x*z^2 + 28*a^4*c^14*x*z^2 + 6*a^2*b^2*c^14*x*z^2 -      9*b^4*c^14*x*z^2 - 8*a^2*c^16*x*z^2 + c^18*x*z^2 + a^18*y*z^2 - 5*a^16*b^2*y*z^2 +      10*a^14*b^4*y*z^2 - 11*a^12*b^6*y*z^2 + 10*a^10*b^8*y*z^2 - 11*a^8*b^10*y*z^2 + 10*a^6*b^12*y*z^2 -      5*a^4*b^14*y*z^2 + a^2*b^16*y*z^2 + 9*a^14*b^2*c^2*y*z^2 - 19*a^12*b^4*c^2*y*z^2 +      6*a^10*b^6*c^2*y*z^2 + 12*a^8*b^8*c^2*y*z^2 - 16*a^6*b^10*c^2*y*z^2 + 12*a^4*b^12*c^2*y*z^2 -      3*a^2*b^14*c^2*y*z^2 - b^16*c^2*y*z^2 - 9*a^14*c^4*y*z^2 + 5*a^12*b^2*c^4*y*z^2 +      18*a^10*b^4*c^4*y*z^2 - 15*a^8*b^6*c^4*y*z^2 - 4*a^6*b^8*c^4*y*z^2 - 9*a^4*b^10*c^4*y*z^2 +      6*a^2*b^12*c^4*y*z^2 + 8*b^14*c^4*y*z^2 + 15*a^12*c^6*y*z^2 - 20*a^10*b^2*c^6*y*z^2 +      7*a^8*b^4*c^6*y*z^2 + 36*a^6*b^6*c^6*y*z^2 + 7*a^4*b^8*c^6*y*z^2 - 20*a^2*b^10*c^6*y*z^2 -      28*b^12*c^6*y*z^2 - 9*a^10*c^8*y*z^2 - 3*a^8*b^2*c^8*y*z^2 - 52*a^6*b^4*c^8*y*z^2 -      21*a^4*b^6*c^8*y*z^2 + 45*a^2*b^8*c^8*y*z^2 + 56*b^10*c^8*y*z^2 + 9*a^8*c^10*y*z^2 +      41*a^6*b^2*c^10*y*z^2 + 36*a^4*b^4*c^10*y*z^2 - 51*a^2*b^6*c^10*y*z^2 - 70*b^8*c^10*y*z^2 -      15*a^6*c^12*y*z^2 - 29*a^4*b^2*c^12*y*z^2 + 28*a^2*b^4*c^12*y*z^2 + 56*b^6*c^12*y*z^2 +      9*a^4*c^14*y*z^2 - 6*a^2*b^2*c^14*y*z^2 - 28*b^4*c^14*y*z^2 + 8*b^2*c^16*y*z^2 - c^18*y*z^2");
		exp("K140","(-a^4)*c^2*x^2*y + b^4*c^2*x^2*y - a^2*c^4*x^2*y - b^2*c^4*x^2*y - a^4*c^2*x*y^2 +      b^4*c^2*x*y^2 + a^2*c^4*x*y^2 + b^2*c^4*x*y^2 + a^4*b^2*x^2*z + a^2*b^4*x^2*z + b^4*c^2*x^2*z -      b^2*c^4*x^2*z - a^4*b^2*y^2*z - a^2*b^4*y^2*z - a^4*c^2*y^2*z + a^2*c^4*y^2*z + a^4*b^2*x*z^2 -      a^2*b^4*x*z^2 - b^4*c^2*x*z^2 - b^2*c^4*x*z^2 + a^4*b^2*y*z^2 - a^2*b^4*y*z^2 + a^4*c^2*y*z^2 +      a^2*c^4*y*z^2");
		exp("K141","(-a^2)*c^2*x^2*y + b^2*c^2*x*y^2 + a^2*b^2*x^2*z - a^2*b^2*y^2*z - b^2*c^2*x*z^2 +      a^2*c^2*y*z^2");
		exp("K142","a^2*c^2*x^2*y + 2*b^2*c^2*x^2*y + 2*a^2*c^2*x*y^2 + b^2*c^2*x*y^2 + a^2*b^2*x^2*z +      2*b^2*c^2*x^2*z + 2*a^2*b^2*x*y*z + 2*a^2*c^2*x*y*z + 2*b^2*c^2*x*y*z + a^2*b^2*y^2*z +      2*a^2*c^2*y^2*z + 2*a^2*b^2*x*z^2 + b^2*c^2*x*z^2 + 2*a^2*b^2*y*z^2 + a^2*c^2*y*z^2");
		exp("K143A","a^4*x^2*y - 2*a^2*b^2*x^2*y + b^4*x^2*y + 2*a^2*c^2*x^2*y + 2*b^2*c^2*x^2*y - 3*c^4*x^2*y +      a^4*x*y^2 - 2*a^2*b^2*x*y^2 + b^4*x*y^2 - c^4*x*y^2 + a^4*y^3 - 2*a^2*b^2*y^3 + b^4*y^3 -      2*a^2*c^2*y^3 - 2*b^2*c^2*y^3 + c^4*y^3 - a^4*x^2*z - 2*a^2*b^2*x^2*z + 3*b^4*x^2*z +      2*a^2*c^2*x^2*z - 2*b^2*c^2*x^2*z - c^4*x^2*z - 2*a^2*b^2*x*y*z + 2*b^4*x*y*z + 2*a^2*c^2*x*y*z -      2*c^4*x*y*z + 2*a^4*y^2*z - 2*a^2*b^2*y^2*z - 2*a^2*c^2*y^2*z - a^4*x*z^2 + b^4*x*z^2 +      2*a^2*c^2*x*z^2 - c^4*x*z^2 - 2*a^4*y*z^2 + 2*a^2*b^2*y*z^2 + 2*a^2*c^2*y*z^2 - a^4*z^3 +      2*a^2*b^2*z^3 - b^4*z^3 + 2*a^2*c^2*z^3 + 2*b^2*c^2*z^3 - c^4*z^3");
		exp("K143B","(-a^4)*x^3 + 2*a^2*b^2*x^3 - b^4*x^3 + 2*a^2*c^2*x^3 + 2*b^2*c^2*x^3 - c^4*x^3 - a^4*x^2*y +      2*a^2*b^2*x^2*y - b^4*x^2*y + c^4*x^2*y - a^4*x*y^2 + 2*a^2*b^2*x*y^2 - b^4*x*y^2 -      2*a^2*c^2*x*y^2 - 2*b^2*c^2*x*y^2 + 3*c^4*x*y^2 + 2*a^2*b^2*x^2*z - 2*b^4*x^2*z + 2*b^2*c^2*x^2*z -      2*a^4*x*y*z + 2*a^2*b^2*x*y*z - 2*b^2*c^2*x*y*z + 2*c^4*x*y*z - 3*a^4*y^2*z + 2*a^2*b^2*y^2*z +      b^4*y^2*z + 2*a^2*c^2*y^2*z - 2*b^2*c^2*y^2*z + c^4*y^2*z - 2*a^2*b^2*x*z^2 + 2*b^4*x*z^2 -      2*b^2*c^2*x*z^2 - a^4*y*z^2 + b^4*y*z^2 - 2*b^2*c^2*y*z^2 + c^4*y*z^2 + a^4*z^3 - 2*a^2*b^2*z^3 +      b^4*z^3 - 2*a^2*c^2*z^3 - 2*b^2*c^2*z^3 + c^4*z^3");
		exp("K143C","a^4*x^3 - 2*a^2*b^2*x^3 + b^4*x^3 - 2*a^2*c^2*x^3 - 2*b^2*c^2*x^3 + c^4*x^3 - 2*a^2*c^2*x^2*y -      2*b^2*c^2*x^2*y + 2*c^4*x^2*y + 2*a^2*c^2*x*y^2 + 2*b^2*c^2*x*y^2 - 2*c^4*x*y^2 - a^4*y^3 +      2*a^2*b^2*y^3 - b^4*y^3 + 2*a^2*c^2*y^3 + 2*b^2*c^2*y^3 - c^4*y^3 + a^4*x^2*z - b^4*x^2*z -      2*a^2*c^2*x^2*z + c^4*x^2*z + 2*a^4*x*y*z - 2*b^4*x*y*z - 2*a^2*c^2*x*y*z + 2*b^2*c^2*x*y*z +      a^4*y^2*z - b^4*y^2*z + 2*b^2*c^2*y^2*z - c^4*y^2*z + a^4*x*z^2 + 2*a^2*b^2*x*z^2 - 3*b^4*x*z^2 -      2*a^2*c^2*x*z^2 + 2*b^2*c^2*x*z^2 + c^4*x*z^2 + 3*a^4*y*z^2 - 2*a^2*b^2*y*z^2 - b^4*y*z^2 -      2*a^2*c^2*y*z^2 + 2*b^2*c^2*y*z^2 - c^4*y*z^2");
		exp("K144","a^6*b^2*x^3 - 3*a^4*b^4*x^3 + 3*a^2*b^6*x^3 - b^8*x^3 - a^6*c^2*x^3 - a^2*b^4*c^2*x^3 +      2*b^6*c^2*x^3 + 3*a^4*c^4*x^3 + a^2*b^2*c^4*x^3 - 3*a^2*c^6*x^3 - 2*b^2*c^6*x^3 + c^8*x^3 +      a^8*x^2*y - 2*a^6*b^2*x^2*y + 2*a^2*b^6*x^2*y - b^8*x^2*y + a^6*c^2*x^2*y + 3*a^4*b^2*c^2*x^2*y -      5*a^2*b^4*c^2*x^2*y + b^6*c^2*x^2*y - 3*a^4*c^4*x^2*y + 4*a^2*b^2*c^4*x^2*y + 3*b^4*c^4*x^2*y -      a^2*c^6*x^2*y - 5*b^2*c^6*x^2*y + 2*c^8*x^2*y + a^8*x*y^2 - 2*a^6*b^2*x*y^2 + 2*a^2*b^6*x*y^2 -      b^8*x*y^2 - a^6*c^2*x*y^2 + 5*a^4*b^2*c^2*x*y^2 - 3*a^2*b^4*c^2*x*y^2 - b^6*c^2*x*y^2 -      3*a^4*c^4*x*y^2 - 4*a^2*b^2*c^4*x*y^2 + 3*b^4*c^4*x*y^2 + 5*a^2*c^6*x*y^2 + b^2*c^6*x*y^2 -      2*c^8*x*y^2 + a^8*y^3 - 3*a^6*b^2*y^3 + 3*a^4*b^4*y^3 - a^2*b^6*y^3 - 2*a^6*c^2*y^3 +      a^4*b^2*c^2*y^3 + b^6*c^2*y^3 - a^2*b^2*c^4*y^3 - 3*b^4*c^4*y^3 + 2*a^2*c^6*y^3 + 3*b^2*c^6*y^3 -      c^8*y^3 - a^8*x^2*z - a^6*b^2*x^2*z + 3*a^4*b^4*x^2*z + a^2*b^6*x^2*z - 2*b^8*x^2*z +      2*a^6*c^2*x^2*z - 3*a^4*b^2*c^2*x^2*z - 4*a^2*b^4*c^2*x^2*z + 5*b^6*c^2*x^2*z +      5*a^2*b^2*c^4*x^2*z - 3*b^4*c^4*x^2*z - 2*a^2*c^6*x^2*z - b^2*c^6*x^2*z + c^8*x^2*z + 2*a^8*y^2*z -      a^6*b^2*y^2*z - 3*a^4*b^4*y^2*z + a^2*b^6*y^2*z + b^8*y^2*z - 5*a^6*c^2*y^2*z +      4*a^4*b^2*c^2*y^2*z + 3*a^2*b^4*c^2*y^2*z - 2*b^6*c^2*y^2*z + 3*a^4*c^4*y^2*z -      5*a^2*b^2*c^4*y^2*z + a^2*c^6*y^2*z + 2*b^2*c^6*y^2*z - c^8*y^2*z - a^8*x*z^2 + a^6*b^2*x*z^2 +      3*a^4*b^4*x*z^2 - 5*a^2*b^6*x*z^2 + 2*b^8*x*z^2 + 2*a^6*c^2*x*z^2 - 5*a^4*b^2*c^2*x*z^2 +      4*a^2*b^4*c^2*x*z^2 - b^6*c^2*x*z^2 + 3*a^2*b^2*c^4*x*z^2 - 3*b^4*c^4*x*z^2 - 2*a^2*c^6*x*z^2 +      b^2*c^6*x*z^2 + c^8*x*z^2 - 2*a^8*y*z^2 + 5*a^6*b^2*y*z^2 - 3*a^4*b^4*y*z^2 - a^2*b^6*y*z^2 +      b^8*y*z^2 + a^6*c^2*y*z^2 - 4*a^4*b^2*c^2*y*z^2 + 5*a^2*b^4*c^2*y*z^2 - 2*b^6*c^2*y*z^2 +      3*a^4*c^4*y*z^2 - 3*a^2*b^2*c^4*y*z^2 - a^2*c^6*y*z^2 + 2*b^2*c^6*y*z^2 - c^8*y*z^2 - a^8*z^3 +      2*a^6*b^2*z^3 - 2*a^2*b^6*z^3 + b^8*z^3 + 3*a^6*c^2*z^3 - a^4*b^2*c^2*z^3 + a^2*b^4*c^2*z^3 -      3*b^6*c^2*z^3 - 3*a^4*c^4*z^3 + 3*b^4*c^4*z^3 + a^2*c^6*z^3 - b^2*c^6*z^3");
		exp("K145","a^2*b^2*c^3*x^2*y - b^4*c^3*x^2*y + b^2*c^5*x^2*y + a^4*c^3*x*y^2 - a^2*b^2*c^3*x*y^2 -      a^2*c^5*x*y^2 - a^2*b^3*c^2*x^2*z - b^5*c^2*x^2*z + b^3*c^4*x^2*z + a^5*c^2*y^2*z +      a^3*b^2*c^2*y^2*z - a^3*c^4*y^2*z - a^4*b^3*x*z^2 + a^2*b^5*x*z^2 + a^2*b^3*c^2*x*z^2 -      a^5*b^2*y*z^2 + a^3*b^4*y*z^2 - a^3*b^2*c^2*y*z^2");
		exp("K146","a^2*b^2*x^2*y - b^4*x^2*y + b^2*c^2*x^2*y + a^4*x*y^2 - a^2*b^2*x*y^2 - a^2*c^2*x*y^2 -      a^2*c^2*x^2*z - b^2*c^2*x^2*z + c^4*x^2*z + a^2*c^2*y^2*z + b^2*c^2*y^2*z - c^4*y^2*z - a^4*x*z^2 +      a^2*b^2*x*z^2 + a^2*c^2*x*z^2 - a^2*b^2*y*z^2 + b^4*y*z^2 - b^2*c^2*y*z^2");
		exp("K147","a^4*b^6*c^4*x^2*y - 2*a^4*b^4*c^6*x^2*y - 2*a^2*b^6*c^6*x^2*y + a^4*b^2*c^8*x^2*y +      4*a^2*b^4*c^8*x^2*y + b^6*c^8*x^2*y - 2*a^2*b^2*c^10*x^2*y - 2*b^4*c^10*x^2*y + b^2*c^12*x^2*y +      a^6*b^4*c^4*x*y^2 - 2*a^6*b^2*c^6*x*y^2 - 2*a^4*b^4*c^6*x*y^2 + a^6*c^8*x*y^2 +      4*a^4*b^2*c^8*x*y^2 + a^2*b^4*c^8*x*y^2 - 2*a^4*c^10*x*y^2 - 2*a^2*b^2*c^10*x*y^2 +      a^2*c^12*x*y^2 + a^4*b^8*c^2*x^2*z - 2*a^2*b^10*c^2*x^2*z + b^12*c^2*x^2*z - 2*a^4*b^6*c^4*x^2*z +      4*a^2*b^8*c^4*x^2*z - 2*b^10*c^4*x^2*z + a^4*b^4*c^6*x^2*z - 2*a^2*b^6*c^6*x^2*z + b^8*c^6*x^2*z +      a^12*c^2*y^2*z - 2*a^10*b^2*c^2*y^2*z + a^8*b^4*c^2*y^2*z - 2*a^10*c^4*y^2*z +      4*a^8*b^2*c^4*y^2*z - 2*a^6*b^4*c^4*y^2*z + a^8*c^6*y^2*z - 2*a^6*b^2*c^6*y^2*z +      a^4*b^4*c^6*y^2*z + a^6*b^8*x*z^2 - 2*a^4*b^10*x*z^2 + a^2*b^12*x*z^2 - 2*a^6*b^6*c^2*x*z^2 +      4*a^4*b^8*c^2*x*z^2 - 2*a^2*b^10*c^2*x*z^2 + a^6*b^4*c^4*x*z^2 - 2*a^4*b^6*c^4*x*z^2 +      a^2*b^8*c^4*x*z^2 + a^12*b^2*y*z^2 - 2*a^10*b^4*y*z^2 + a^8*b^6*y*z^2 - 2*a^10*b^2*c^2*y*z^2 +      4*a^8*b^4*c^2*y*z^2 - 2*a^6*b^6*c^2*y*z^2 + a^8*b^2*c^4*y*z^2 - 2*a^6*b^4*c^4*y*z^2 +      a^4*b^6*c^4*y*z^2");
		exp("K148","a^4*b^2*c^4*x^2*y + a^2*b^4*c^4*x^2*y + b^6*c^4*x^2*y - 2*a^2*b^2*c^6*x^2*y - 2*b^4*c^6*x^2*y +      b^2*c^8*x^2*y + a^6*c^4*x*y^2 + a^4*b^2*c^4*x*y^2 + a^2*b^4*c^4*x*y^2 - 2*a^4*c^6*x*y^2 -      2*a^2*b^2*c^6*x*y^2 + a^2*c^8*x*y^2 + a^4*b^4*c^2*x^2*z - 2*a^2*b^6*c^2*x^2*z + b^8*c^2*x^2*z +      a^2*b^4*c^4*x^2*z - 2*b^6*c^4*x^2*z + b^4*c^6*x^2*z + a^8*c^2*y^2*z - 2*a^6*b^2*c^2*y^2*z +      a^4*b^4*c^2*y^2*z - 2*a^6*c^4*y^2*z + a^4*b^2*c^4*y^2*z + a^4*c^6*y^2*z + a^6*b^4*x*z^2 -      2*a^4*b^6*x*z^2 + a^2*b^8*x*z^2 + a^4*b^4*c^2*x*z^2 - 2*a^2*b^6*c^2*x*z^2 + a^2*b^4*c^4*x*z^2 +      a^8*b^2*y*z^2 - 2*a^6*b^4*y*z^2 + a^4*b^6*y*z^2 - 2*a^6*b^2*c^2*y*z^2 + a^4*b^4*c^2*y*z^2 +      a^4*b^2*c^4*y*z^2");
		exp("K149","b^2*x^2*y + a^2*x*y^2 + c^2*x^2*z + c^2*y^2*z + a^2*x*z^2 + b^2*y*z^2");
		exp("K150","(-a^2)*c^4*x^2*y - b^2*c^4*x^2*y + 2*c^6*x^2*y - a^2*c^4*x*y^2 - b^2*c^4*x*y^2 + 2*c^6*x*y^2 -      a^2*b^4*x^2*z + 2*b^6*x^2*z - b^4*c^2*x^2*z - a^4*b^2*x*y*z - a^2*b^4*x*y*z - a^4*c^2*x*y*z +      6*a^2*b^2*c^2*x*y*z - b^4*c^2*x*y*z - a^2*c^4*x*y*z - b^2*c^4*x*y*z + 2*a^6*y^2*z - a^4*b^2*y^2*z -      a^4*c^2*y^2*z - a^2*b^4*x*z^2 + 2*b^6*x*z^2 - b^4*c^2*x*z^2 + 2*a^6*y*z^2 - a^4*b^2*y*z^2 -      a^4*c^2*y*z^2");
		exp("K151","a^2*b^4*x^2*y - a^2*b^2*c^2*x^2*y - b^4*c^2*x^2*y + b^2*c^4*x^2*y + a^4*b^2*x*y^2 -      a^4*c^2*x*y^2 - a^2*b^2*c^2*x*y^2 + a^2*c^4*x*y^2 - a^2*b^2*c^2*x^2*z + b^4*c^2*x^2*z +      a^2*c^4*x^2*z - b^2*c^4*x^2*z + a^4*c^2*y^2*z - a^2*b^2*c^2*y^2*z - a^2*c^4*y^2*z + b^2*c^4*y^2*z -      a^4*b^2*x*z^2 + a^2*b^4*x*z^2 + a^4*c^2*x*z^2 - a^2*b^2*c^2*x*z^2 + a^4*b^2*y*z^2 - a^2*b^4*y*z^2 -      a^2*b^2*c^2*y*z^2 + b^4*c^2*y*z^2");
		exp("K152","a^10*c^2*x^2*y - a^8*b^2*c^2*x^2*y - 2*a^6*b^4*c^2*x^2*y + 2*a^4*b^6*c^2*x^2*y +      a^2*b^8*c^2*x^2*y - b^10*c^2*x^2*y - 3*a^8*c^4*x^2*y + 4*a^6*b^2*c^4*x^2*y + 18*a^4*b^4*c^4*x^2*y -      20*a^2*b^6*c^4*x^2*y + b^8*c^4*x^2*y + 2*a^6*c^6*x^2*y - 6*a^4*b^2*c^6*x^2*y +      18*a^2*b^4*c^6*x^2*y + 2*b^6*c^6*x^2*y + 2*a^4*c^8*x^2*y + 4*a^2*b^2*c^8*x^2*y - 2*b^4*c^8*x^2*y -      3*a^2*c^10*x^2*y - b^2*c^10*x^2*y + c^12*x^2*y + a^10*c^2*x*y^2 - a^8*b^2*c^2*x*y^2 -      2*a^6*b^4*c^2*x*y^2 + 2*a^4*b^6*c^2*x*y^2 + a^2*b^8*c^2*x*y^2 - b^10*c^2*x*y^2 - a^8*c^4*x*y^2 +      20*a^6*b^2*c^4*x*y^2 - 18*a^4*b^4*c^4*x*y^2 - 4*a^2*b^6*c^4*x*y^2 + 3*b^8*c^4*x*y^2 -      2*a^6*c^6*x*y^2 - 18*a^4*b^2*c^6*x*y^2 + 6*a^2*b^4*c^6*x*y^2 - 2*b^6*c^6*x*y^2 + 2*a^4*c^8*x*y^2 -      4*a^2*b^2*c^8*x*y^2 - 2*b^4*c^8*x*y^2 + a^2*c^10*x*y^2 + 3*b^2*c^10*x*y^2 - c^12*x*y^2 -      a^10*b^2*x^2*z + 3*a^8*b^4*x^2*z - 2*a^6*b^6*x^2*z - 2*a^4*b^8*x^2*z + 3*a^2*b^10*x^2*z -      b^12*x^2*z + a^8*b^2*c^2*x^2*z - 4*a^6*b^4*c^2*x^2*z + 6*a^4*b^6*c^2*x^2*z - 4*a^2*b^8*c^2*x^2*z +      b^10*c^2*x^2*z + 2*a^6*b^2*c^4*x^2*z - 18*a^4*b^4*c^4*x^2*z - 18*a^2*b^6*c^4*x^2*z +      2*b^8*c^4*x^2*z - 2*a^4*b^2*c^6*x^2*z + 20*a^2*b^4*c^6*x^2*z - 2*b^6*c^6*x^2*z -      a^2*b^2*c^8*x^2*z - b^4*c^8*x^2*z + b^2*c^10*x^2*z + a^12*y^2*z - 3*a^10*b^2*y^2*z +      2*a^8*b^4*y^2*z + 2*a^6*b^6*y^2*z - 3*a^4*b^8*y^2*z + a^2*b^10*y^2*z - a^10*c^2*y^2*z +      4*a^8*b^2*c^2*y^2*z - 6*a^6*b^4*c^2*y^2*z + 4*a^4*b^6*c^2*y^2*z - a^2*b^8*c^2*y^2*z -      2*a^8*c^4*y^2*z + 18*a^6*b^2*c^4*y^2*z + 18*a^4*b^4*c^4*y^2*z - 2*a^2*b^6*c^4*y^2*z +      2*a^6*c^6*y^2*z - 20*a^4*b^2*c^6*y^2*z + 2*a^2*b^4*c^6*y^2*z + a^4*c^8*y^2*z + a^2*b^2*c^8*y^2*z -      a^2*c^10*y^2*z - a^10*b^2*x*z^2 + a^8*b^4*x*z^2 + 2*a^6*b^6*x*z^2 - 2*a^4*b^8*x*z^2 -      a^2*b^10*x*z^2 + b^12*x*z^2 + a^8*b^2*c^2*x*z^2 - 20*a^6*b^4*c^2*x*z^2 + 18*a^4*b^6*c^2*x*z^2 +      4*a^2*b^8*c^2*x*z^2 - 3*b^10*c^2*x*z^2 + 2*a^6*b^2*c^4*x*z^2 + 18*a^4*b^4*c^4*x*z^2 -      6*a^2*b^6*c^4*x*z^2 + 2*b^8*c^4*x*z^2 - 2*a^4*b^2*c^6*x*z^2 + 4*a^2*b^4*c^6*x*z^2 +      2*b^6*c^6*x*z^2 - a^2*b^2*c^8*x*z^2 - 3*b^4*c^8*x*z^2 + b^2*c^10*x*z^2 - a^12*y*z^2 +      a^10*b^2*y*z^2 + 2*a^8*b^4*y*z^2 - 2*a^6*b^6*y*z^2 - a^4*b^8*y*z^2 + a^2*b^10*y*z^2 +      3*a^10*c^2*y*z^2 - 4*a^8*b^2*c^2*y*z^2 - 18*a^6*b^4*c^2*y*z^2 + 20*a^4*b^6*c^2*y*z^2 -      a^2*b^8*c^2*y*z^2 - 2*a^8*c^4*y*z^2 + 6*a^6*b^2*c^4*y*z^2 - 18*a^4*b^4*c^4*y*z^2 -      2*a^2*b^6*c^4*y*z^2 - 2*a^6*c^6*y*z^2 - 4*a^4*b^2*c^6*y*z^2 + 2*a^2*b^4*c^6*y*z^2 +      3*a^4*c^8*y*z^2 + a^2*b^2*c^8*y*z^2 - a^2*c^10*y*z^2");
		exp("K153","a^2*b^2*c^2*(x + y + z)^3 - (a^2 + b^2 + c^2)^3*x*y*z");
		exp("K154","a^4*c*x^2*y + a^3*b*c*x^2*y - a^2*b^2*c*x^2*y - a*b^3*c*x^2*y - a^3*c^2*x^2*y +      2*a^2*b*c^2*x^2*y - a*b^2*c^2*x^2*y - a^2*c^3*x^2*y + a*b*c^3*x^2*y + a*c^4*x^2*y + a^3*b*c*x*y^2 +      a^2*b^2*c*x*y^2 - a*b^3*c*x*y^2 - b^4*c*x*y^2 + a^2*b*c^2*x*y^2 - 2*a*b^2*c^2*x*y^2 +      b^3*c^2*x*y^2 - a*b*c^3*x*y^2 + b^2*c^3*x*y^2 - b*c^4*x*y^2 - a^4*b*x^2*z + a^3*b^2*x^2*z +      a^2*b^3*x^2*z - a*b^4*x^2*z - a^3*b*c*x^2*z - 2*a^2*b^2*c*x^2*z - a*b^3*c*x^2*z + a^2*b*c^2*x^2*z +      a*b^2*c^2*x^2*z + a*b*c^3*x^2*z + a^4*b*y^2*z - a^3*b^2*y^2*z - a^2*b^3*y^2*z + a*b^4*y^2*z +      a^3*b*c*y^2*z + 2*a^2*b^2*c*y^2*z + a*b^3*c*y^2*z - a^2*b*c^2*y^2*z - a*b^2*c^2*y^2*z -      a*b*c^3*y^2*z - a^3*b*c*x*z^2 - a^2*b^2*c*x*z^2 + a*b^3*c*x*z^2 + b^4*c*x*z^2 - a^2*b*c^2*x*z^2 +      2*a*b^2*c^2*x*z^2 - b^3*c^2*x*z^2 + a*b*c^3*x*z^2 - b^2*c^3*x*z^2 + b*c^4*x*z^2 - a^4*c*y*z^2 -      a^3*b*c*y*z^2 + a^2*b^2*c*y*z^2 + a*b^3*c*y*z^2 + a^3*c^2*y*z^2 - 2*a^2*b*c^2*y*z^2 +      a*b^2*c^2*y*z^2 + a^2*c^3*y*z^2 - a*b*c^3*y*z^2 - a*c^4*y*z^2");
		exp("K155","b^3*c^3*x^2*y - a*b*c^4*x^2*y - a^3*c^3*x*y^2 + a*b*c^4*x*y^2 + a*b^4*c*x^2*z - b^3*c^3*x^2*z -      a^4*b*c*y^2*z + a^3*c^3*y^2*z + a^3*b^3*x*z^2 - a*b^4*c*x*z^2 - a^3*b^3*y*z^2 + a^4*b*c*y*z^2");
		exp("K156","5*a^4*c^2*x^2*y + 6*a^2*b^2*c^2*x^2*y - 11*b^4*c^2*x^2*y - 10*a^2*c^4*x^2*y + 6*b^2*c^4*x^2*y +      5*c^6*x^2*y + 11*a^4*c^2*x*y^2 - 6*a^2*b^2*c^2*x*y^2 - 5*b^4*c^2*x*y^2 - 6*a^2*c^4*x*y^2 +      10*b^2*c^4*x*y^2 - 5*c^6*x*y^2 - 5*a^4*b^2*x^2*z + 10*a^2*b^4*x^2*z - 5*b^6*x^2*z -      6*a^2*b^2*c^2*x^2*z - 6*b^4*c^2*x^2*z + 11*b^2*c^4*x^2*z + 5*a^6*y^2*z - 10*a^4*b^2*y^2*z +      5*a^2*b^4*y^2*z + 6*a^4*c^2*y^2*z + 6*a^2*b^2*c^2*y^2*z - 11*a^2*c^4*y^2*z - 11*a^4*b^2*x*z^2 +      6*a^2*b^4*x*z^2 + 5*b^6*x*z^2 + 6*a^2*b^2*c^2*x*z^2 - 10*b^4*c^2*x*z^2 + 5*b^2*c^4*x*z^2 -      5*a^6*y*z^2 - 6*a^4*b^2*y*z^2 + 11*a^2*b^4*y*z^2 + 10*a^4*c^2*y*z^2 - 6*a^2*b^2*c^2*y*z^2 -      5*a^2*c^4*y*z^2");
		exp("K157","(-a)*b*c*x^2*y - b^2*c*x^2*y + b*c^2*x^2*y + a^2*c*x*y^2 + a*b*c*x*y^2 - a*c^2*x*y^2 +      a*b*c*x^2*z - b^2*c*x^2*z + b*c^2*x^2*z + a^2*c*y^2*z - a*b*c*y^2*z - a*c^2*y^2*z - a^2*b*x*z^2 +      a*b^2*x*z^2 - a*b*c*x*z^2 - a^2*b*y*z^2 + a*b^2*y*z^2 + a*b*c*y*z^2");
		exp("K158","-(a^8*b^2*c^2*x^2*y) + 3*a^6*b^4*c^2*x^2*y - 3*a^4*b^6*c^2*x^2*y + a^2*b^8*c^2*x^2*y +      a^6*b^2*c^4*x^2*y - a^4*b^4*c^4*x^2*y - a^2*b^6*c^4*x^2*y + b^8*c^4*x^2*y - a^2*b^4*c^6*x^2*y -      3*b^6*c^6*x^2*y + a^2*b^2*c^8*x^2*y + 3*b^4*c^8*x^2*y - b^2*c^10*x^2*y - a^8*b^2*c^2*x*y^2 +      3*a^6*b^4*c^2*x*y^2 - 3*a^4*b^6*c^2*x*y^2 + a^2*b^8*c^2*x*y^2 - a^8*c^4*x*y^2 + a^6*b^2*c^4*x*y^2 +      a^4*b^4*c^4*x*y^2 - a^2*b^6*c^4*x*y^2 + 3*a^6*c^6*x*y^2 + a^4*b^2*c^6*x*y^2 - 3*a^4*c^8*x*y^2 -      a^2*b^2*c^8*x*y^2 + a^2*c^10*x*y^2 + a^8*b^2*c^2*x^2*z - a^6*b^4*c^2*x^2*z - a^2*b^8*c^2*x^2*z +      b^10*c^2*x^2*z - 3*a^6*b^2*c^4*x^2*z + a^4*b^4*c^4*x^2*z + a^2*b^6*c^4*x^2*z - 3*b^8*c^4*x^2*z +      3*a^4*b^2*c^6*x^2*z + a^2*b^4*c^6*x^2*z + 3*b^6*c^6*x^2*z - a^2*b^2*c^8*x^2*z - b^4*c^8*x^2*z -      a^10*c^2*y^2*z + a^8*b^2*c^2*y^2*z + a^4*b^6*c^2*y^2*z - a^2*b^8*c^2*y^2*z + 3*a^8*c^4*y^2*z -      a^6*b^2*c^4*y^2*z - a^4*b^4*c^4*y^2*z + 3*a^2*b^6*c^4*y^2*z - 3*a^6*c^6*y^2*z - a^4*b^2*c^6*y^2*z -      3*a^2*b^4*c^6*y^2*z + a^4*c^8*y^2*z + a^2*b^2*c^8*y^2*z + a^8*b^4*x*z^2 - 3*a^6*b^6*x*z^2 +      3*a^4*b^8*x*z^2 - a^2*b^10*x*z^2 + a^8*b^2*c^2*x*z^2 - a^6*b^4*c^2*x*z^2 - a^4*b^6*c^2*x*z^2 +      a^2*b^8*c^2*x*z^2 - 3*a^6*b^2*c^4*x*z^2 - a^4*b^4*c^4*x*z^2 + 3*a^4*b^2*c^6*x*z^2 +      a^2*b^4*c^6*x*z^2 - a^2*b^2*c^8*x*z^2 + a^10*b^2*y*z^2 - 3*a^8*b^4*y*z^2 + 3*a^6*b^6*y*z^2 -      a^4*b^8*y*z^2 - a^8*b^2*c^2*y*z^2 + a^6*b^4*c^2*y*z^2 + a^4*b^6*c^2*y*z^2 - a^2*b^8*c^2*y*z^2 +      a^4*b^4*c^4*y*z^2 + 3*a^2*b^6*c^4*y*z^2 - a^4*b^2*c^6*y*z^2 - 3*a^2*b^4*c^6*y*z^2 +      a^2*b^2*c^8*y*z^2");
		exp("K159","(-a^6)*x^2*y - a^4*b^2*x^2*y + 5*a^2*b^4*x^2*y - 3*b^6*x^2*y + 3*a^4*c^2*x^2*y -      2*a^2*b^2*c^2*x^2*y - b^4*c^2*x^2*y - 3*a^2*c^4*x^2*y + 3*b^2*c^4*x^2*y + c^6*x^2*y + 3*a^6*x*y^2 -      5*a^4*b^2*x*y^2 + a^2*b^4*x*y^2 + b^6*x*y^2 + a^4*c^2*x*y^2 + 2*a^2*b^2*c^2*x*y^2 -      3*b^4*c^2*x*y^2 - 3*a^2*c^4*x*y^2 + 3*b^2*c^4*x*y^2 - c^6*x*y^2 + a^6*x^2*z - 3*a^4*b^2*x^2*z +      3*a^2*b^4*x^2*z - b^6*x^2*z + a^4*c^2*x^2*z + 2*a^2*b^2*c^2*x^2*z - 3*b^4*c^2*x^2*z -      5*a^2*c^4*x^2*z + b^2*c^4*x^2*z + 3*c^6*x^2*z + a^6*y^2*z - 3*a^4*b^2*y^2*z + 3*a^2*b^4*y^2*z -      b^6*y^2*z + 3*a^4*c^2*y^2*z - 2*a^2*b^2*c^2*y^2*z - b^4*c^2*y^2*z - a^2*c^4*y^2*z +      5*b^2*c^4*y^2*z - 3*c^6*y^2*z - 3*a^6*x*z^2 - a^4*b^2*x*z^2 + 3*a^2*b^4*x*z^2 + b^6*x*z^2 +      5*a^4*c^2*x*z^2 - 2*a^2*b^2*c^2*x*z^2 - 3*b^4*c^2*x*z^2 - a^2*c^4*x*z^2 + 3*b^2*c^4*x*z^2 -      c^6*x*z^2 - a^6*y*z^2 - 3*a^4*b^2*y*z^2 + a^2*b^4*y*z^2 + 3*b^6*y*z^2 + 3*a^4*c^2*y*z^2 +      2*a^2*b^2*c^2*y*z^2 - 5*b^4*c^2*y*z^2 - 3*a^2*c^4*y*z^2 + b^2*c^4*y*z^2 + c^6*y*z^2");
		exp("K160","a^4*b^4*c^4*x^2*y - b^8*c^4*x^2*y + b^4*c^8*x^2*y + a^8*c^4*x*y^2 - a^4*b^4*c^4*x*y^2 -      a^4*c^8*x*y^2 - a^4*b^4*c^4*x^2*z - b^8*c^4*x^2*z + b^4*c^8*x^2*z + a^8*c^4*y^2*z +      a^4*b^4*c^4*y^2*z - a^4*c^8*y^2*z - a^8*b^4*x*z^2 + a^4*b^8*x*z^2 + a^4*b^4*c^4*x*z^2 -      a^8*b^4*y*z^2 + a^4*b^8*y*z^2 - a^4*b^4*c^4*y*z^2");
		exp("K161","(-a^8)*c^4*x^2*y - 2*a^4*b^4*c^4*x^2*y + 3*b^8*c^4*x^2*y + 2*a^4*c^8*x^2*y - 2*b^4*c^8*x^2*y -      c^12*x^2*y - 3*a^8*c^4*x*y^2 + 2*a^4*b^4*c^4*x*y^2 + b^8*c^4*x*y^2 + 2*a^4*c^8*x*y^2 -      2*b^4*c^8*x*y^2 + c^12*x*y^2 + a^8*b^4*x^2*z - 2*a^4*b^8*x^2*z + b^12*x^2*z + 2*a^4*b^4*c^4*x^2*z +      2*b^8*c^4*x^2*z - 3*b^4*c^8*x^2*z - a^12*y^2*z + 2*a^8*b^4*y^2*z - a^4*b^8*y^2*z -      2*a^8*c^4*y^2*z - 2*a^4*b^4*c^4*y^2*z + 3*a^4*c^8*y^2*z + 3*a^8*b^4*x*z^2 - 2*a^4*b^8*x*z^2 -      b^12*x*z^2 - 2*a^4*b^4*c^4*x*z^2 + 2*b^8*c^4*x*z^2 - b^4*c^8*x*z^2 + a^12*y*z^2 + 2*a^8*b^4*y*z^2 -      3*a^4*b^8*y*z^2 - 2*a^8*c^4*y*z^2 + 2*a^4*b^4*c^4*y*z^2 + a^4*c^8*y*z^2");
		exp("K162","a^2*b^2*c^4*x^2*y - b^4*c^4*x^2*y + b^2*c^6*x^2*y - a^4*c^4*x*y^2 + a^2*b^2*c^4*x*y^2 +      a^2*c^6*x*y^2 + a^2*b^4*c^2*x^2*z + b^6*c^2*x^2*z - b^4*c^4*x^2*z - 2*a^4*b^2*c^2*x*y*z -      2*a^2*b^4*c^2*x*y*z - 2*a^2*b^2*c^4*x*y*z + a^6*c^2*y^2*z + a^4*b^2*c^2*y^2*z - a^4*c^4*y^2*z -      a^4*b^4*x*z^2 + a^2*b^6*x*z^2 + a^2*b^4*c^2*x*z^2 + a^6*b^2*y*z^2 - a^4*b^4*y*z^2 +      a^4*b^2*c^2*y*z^2");
		exp("K163","a^6*c^2*x^2*y - a^4*b^2*c^2*x^2*y - a^2*b^4*c^2*x^2*y + b^6*c^2*x^2*y - 3*a^4*c^4*x^2*y +      2*a^2*b^2*c^4*x^2*y + b^4*c^4*x^2*y + 3*a^2*c^6*x^2*y - b^2*c^6*x^2*y - c^8*x^2*y - a^6*c^2*x*y^2 +      a^4*b^2*c^2*x*y^2 + a^2*b^4*c^2*x*y^2 - b^6*c^2*x*y^2 - a^4*c^4*x*y^2 - 2*a^2*b^2*c^4*x*y^2 +      3*b^4*c^4*x*y^2 + a^2*c^6*x*y^2 - 3*b^2*c^6*x*y^2 + c^8*x*y^2 - a^6*b^2*x^2*z + 3*a^4*b^4*x^2*z -      3*a^2*b^6*x^2*z + b^8*x^2*z + a^4*b^2*c^2*x^2*z - 2*a^2*b^4*c^2*x^2*z + b^6*c^2*x^2*z +      a^2*b^2*c^4*x^2*z - b^4*c^4*x^2*z - b^2*c^6*x^2*z - a^8*y^2*z + 3*a^6*b^2*y^2*z - 3*a^4*b^4*y^2*z +      a^2*b^6*y^2*z - a^6*c^2*y^2*z + 2*a^4*b^2*c^2*y^2*z - a^2*b^4*c^2*y^2*z + a^4*c^4*y^2*z -      a^2*b^2*c^4*y^2*z + a^2*c^6*y^2*z + a^6*b^2*x*z^2 + a^4*b^4*x*z^2 - a^2*b^6*x*z^2 - b^8*x*z^2 -      a^4*b^2*c^2*x*z^2 + 2*a^2*b^4*c^2*x*z^2 + 3*b^6*c^2*x*z^2 - a^2*b^2*c^4*x*z^2 - 3*b^4*c^4*x*z^2 +      b^2*c^6*x*z^2 + a^8*y*z^2 + a^6*b^2*y*z^2 - a^4*b^4*y*z^2 - a^2*b^6*y*z^2 - 3*a^6*c^2*y*z^2 -      2*a^4*b^2*c^2*y*z^2 + a^2*b^4*c^2*y*z^2 + 3*a^4*c^4*y*z^2 + a^2*b^2*c^4*y*z^2 - a^2*c^6*y*z^2");
		exp("K164","a^6*c^2*x^2*y - a^2*b^4*c^2*x^2*y - 3*a^4*c^4*x^2*y + b^4*c^4*x^2*y + 3*a^2*c^6*x^2*y -      c^8*x^2*y + a^4*b^2*c^2*x*y^2 - b^6*c^2*x*y^2 - a^4*c^4*x*y^2 + 3*b^4*c^4*x*y^2 - 3*b^2*c^6*x*y^2 +      c^8*x*y^2 - a^6*b^2*x^2*z + 3*a^4*b^4*x^2*z - 3*a^2*b^6*x^2*z + b^8*x^2*z + a^2*b^2*c^4*x^2*z -      b^4*c^4*x^2*z - a^8*y^2*z + 3*a^6*b^2*y^2*z - 3*a^4*b^4*y^2*z + a^2*b^6*y^2*z + a^4*c^4*y^2*z -      a^2*b^2*c^4*y^2*z + a^4*b^4*x*z^2 - b^8*x*z^2 - a^4*b^2*c^2*x*z^2 + 3*b^6*c^2*x*z^2 -      3*b^4*c^4*x*z^2 + b^2*c^6*x*z^2 + a^8*y*z^2 - a^4*b^4*y*z^2 - 3*a^6*c^2*y*z^2 + a^2*b^4*c^2*y*z^2 +      3*a^4*c^4*y*z^2 - a^2*c^6*y*z^2");
		exp("K165","(-a^4)*c^2*x^2*y + a^2*b^2*c^2*x^2*y + 2*a^3*c^3*x^2*y - 2*a^2*b*c^3*x^2*y + 2*a*b*c^4*x^2*y -      b^2*c^4*x^2*y - 2*a*c^5*x^2*y + c^6*x^2*y - a^2*b^2*c^2*x*y^2 + b^4*c^2*x*y^2 + 2*a*b^2*c^3*x*y^2 -      2*b^3*c^3*x*y^2 + a^2*c^4*x*y^2 - 2*a*b*c^4*x*y^2 + 2*b*c^5*x*y^2 - c^6*x*y^2 + a^4*b^2*x^2*z -      2*a^3*b^3*x^2*z + 2*a*b^5*x^2*z - b^6*x^2*z + 2*a^2*b^3*c*x^2*z - 2*a*b^4*c*x^2*z -      a^2*b^2*c^2*x^2*z + b^4*c^2*x^2*z - 2*a^5*b*x*y*z + 4*a^4*b^2*x*y*z - 4*a^2*b^4*x*y*z +      2*a*b^5*x*y*z + 2*a^5*c*x*y*z - 6*a^3*b^2*c*x*y*z + 6*a^2*b^3*c*x*y*z - 2*b^5*c*x*y*z -      4*a^4*c^2*x*y*z + 6*a^3*b*c^2*x*y*z - 6*a*b^3*c^2*x*y*z + 4*b^4*c^2*x*y*z - 6*a^2*b*c^3*x*y*z +      6*a*b^2*c^3*x*y*z + 4*a^2*c^4*x*y*z - 4*b^2*c^4*x*y*z - 2*a*c^5*x*y*z + 2*b*c^5*x*y*z + a^6*y^2*z -      2*a^5*b*y^2*z + 2*a^3*b^3*y^2*z - a^2*b^4*y^2*z + 2*a^4*b*c*y^2*z - 2*a^3*b^2*c*y^2*z -      a^4*c^2*y^2*z + a^2*b^2*c^2*y^2*z - a^2*b^4*x*z^2 + b^6*x*z^2 + 2*a*b^4*c*x*z^2 - 2*b^5*c*x*z^2 +      a^2*b^2*c^2*x*z^2 - 2*a*b^3*c^2*x*z^2 + 2*b^3*c^3*x*z^2 - b^2*c^4*x*z^2 - a^6*y*z^2 +      a^4*b^2*y*z^2 + 2*a^5*c*y*z^2 - 2*a^4*b*c*y*z^2 + 2*a^3*b*c^2*y*z^2 - a^2*b^2*c^2*y*z^2 -      2*a^3*c^3*y*z^2 + a^2*c^4*y*z^2");
		exp("K166","(-a^6)*b^2*c^2*x^2*y + a^4*b^4*c^2*x^2*y + a^4*b^2*c^4*x^2*y - a^2*b^2*c^6*x^2*y -      b^4*c^6*x^2*y + b^2*c^8*x^2*y - a^4*b^4*c^2*x*y^2 + a^2*b^6*c^2*x*y^2 - a^2*b^4*c^4*x*y^2 +      a^4*c^6*x*y^2 + a^2*b^2*c^6*x*y^2 - a^2*c^8*x*y^2 + a^6*b^2*c^2*x^2*z - a^4*b^4*c^2*x^2*z +      a^2*b^6*c^2*x^2*z - b^8*c^2*x^2*z - a^4*b^2*c^4*x^2*z + b^6*c^4*x^2*z - a^8*b^2*x*y*z +      3*a^6*b^4*x*y*z - 3*a^4*b^6*x*y*z + a^2*b^8*x*y*z + a^8*c^2*x*y*z - b^8*c^2*x*y*z -      3*a^6*c^4*x*y*z + 3*b^6*c^4*x*y*z + 3*a^4*c^6*x*y*z - 3*b^4*c^6*x*y*z - a^2*c^8*x*y*z +      b^2*c^8*x*y*z + a^8*c^2*y^2*z - a^6*b^2*c^2*y^2*z + a^4*b^4*c^2*y^2*z - a^2*b^6*c^2*y^2*z -      a^6*c^4*y^2*z + a^2*b^4*c^4*y^2*z - a^4*b^6*x*z^2 + a^2*b^8*x*z^2 - a^2*b^6*c^2*x*z^2 +      a^4*b^2*c^4*x*z^2 + a^2*b^4*c^4*x*z^2 - a^2*b^2*c^6*x*z^2 - a^8*b^2*y*z^2 + a^6*b^4*y*z^2 +      a^6*b^2*c^2*y*z^2 - a^4*b^2*c^4*y*z^2 - a^2*b^4*c^4*y*z^2 + a^2*b^2*c^6*y*z^2");
		exp("K167","(-a^2)*b^2*c^4*x^2*y - b^4*c^4*x^2*y + b^2*c^6*x^2*y + a^4*c^4*x*y^2 + a^2*b^2*c^4*x*y^2 -      a^2*c^6*x*y^2 + a^2*b^4*c^2*x^2*z - b^6*c^2*x^2*z + b^4*c^4*x^2*z + a^6*c^2*y^2*z -      a^4*b^2*c^2*y^2*z - a^4*c^4*y^2*z - a^4*b^4*x*z^2 + a^2*b^6*x*z^2 - a^2*b^4*c^2*x*z^2 -      a^6*b^2*y*z^2 + a^4*b^4*y*z^2 + a^4*b^2*c^2*y*z^2");
		exp("K168","(-a^2)*c^2*x^2*y - b^2*c^2*x^2*y + c^4*x^2*y + a^2*c^2*x*y^2 + b^2*c^2*x*y^2 - c^4*x*y^2 +      a^2*b^2*x^2*z - b^4*x^2*z + b^2*c^2*x^2*z + a^4*y^2*z - a^2*b^2*y^2*z - a^2*c^2*y^2*z -      a^2*b^2*x*z^2 + b^4*x*z^2 - b^2*c^2*x*z^2 - a^4*y*z^2 + a^2*b^2*y*z^2 + a^2*c^2*y*z^2");
		exp("K169","(-a^2)*c^2*x^2*y + b^2*c^2*x^2*y - c^4*x^2*y - a^2*c^2*x*y^2 + b^2*c^2*x*y^2 + c^4*x*y^2 +      a^2*b^2*x^2*z + b^4*x^2*z - b^2*c^2*x^2*z - a^4*y^2*z - a^2*b^2*y^2*z + a^2*c^2*y^2*z +      a^2*b^2*x*z^2 - b^4*x*z^2 - b^2*c^2*x*z^2 + a^4*y*z^2 - a^2*b^2*y*z^2 + a^2*c^2*y*z^2");
		exp("K170","a^4*x^2*y - b^4*x^2*y - 2*a^2*c^2*x^2*y + c^4*x^2*y + a^4*x*y^2 - b^4*x*y^2 + 2*b^2*c^2*x*y^2 -      c^4*x*y^2 - a^4*x^2*z + 2*a^2*b^2*x^2*z - b^4*x^2*z + c^4*x^2*z + a^4*y^2*z - 2*a^2*b^2*y^2*z +      b^4*y^2*z - c^4*y^2*z - a^4*x*z^2 + b^4*x*z^2 - 2*b^2*c^2*x*z^2 + c^4*x*z^2 - a^4*y*z^2 +      b^4*y*z^2 + 2*a^2*c^2*y*z^2 - c^4*y*z^2");
		exp("K171","a^4*b^2*c^4*x^2*y - b^6*c^4*x^2*y - 2*a^2*b^2*c^6*x^2*y + b^2*c^8*x^2*y + a^6*c^4*x*y^2 -      a^2*b^4*c^4*x*y^2 + 2*a^2*b^2*c^6*x*y^2 - a^2*c^8*x*y^2 - a^4*b^4*c^2*x^2*z + 2*a^2*b^6*c^2*x^2*z -      b^8*c^2*x^2*z + b^4*c^6*x^2*z + a^8*c^2*y^2*z - 2*a^6*b^2*c^2*y^2*z + a^4*b^4*c^2*y^2*z -      a^4*c^6*y^2*z - a^6*b^4*x*z^2 + a^2*b^8*x*z^2 - 2*a^2*b^6*c^2*x*z^2 + a^2*b^4*c^4*x*z^2 -      a^8*b^2*y*z^2 + a^4*b^6*y*z^2 + 2*a^6*b^2*c^2*y*z^2 - a^4*b^2*c^4*y*z^2");
		exp("K172","(-a^2)*b^2*c^4*x^2*y + b^4*c^4*x^2*y - b^2*c^6*x^2*y - a^4*c^4*x*y^2 + a^2*b^2*c^4*x*y^2 +      a^2*c^6*x*y^2 + a^2*b^4*c^2*x^2*z + b^6*c^2*x^2*z - b^4*c^4*x^2*z - a^6*c^2*y^2*z -      a^4*b^2*c^2*y^2*z + a^4*c^4*y^2*z + a^4*b^4*x*z^2 - a^2*b^6*x*z^2 - a^2*b^4*c^2*x*z^2 +      a^6*b^2*y*z^2 - a^4*b^4*y*z^2 + a^4*b^2*c^2*y*z^2");
		exp("K173","-(a^6*c^2*x^2*y) + a^4*b^2*c^2*x^2*y - 3*a^2*b^4*c^2*x^2*y + 3*b^6*c^2*x^2*y + a^4*c^4*x^2*y +      6*a^2*b^2*c^4*x^2*y - 3*b^4*c^4*x^2*y + a^2*c^6*x^2*y + b^2*c^6*x^2*y - c^8*x^2*y -      3*a^6*c^2*x*y^2 + 3*a^4*b^2*c^2*x*y^2 - a^2*b^4*c^2*x*y^2 + b^6*c^2*x*y^2 + 3*a^4*c^4*x*y^2 -      6*a^2*b^2*c^4*x*y^2 - b^4*c^4*x*y^2 - a^2*c^6*x*y^2 - b^2*c^6*x*y^2 + c^8*x*y^2 + a^6*b^2*x^2*z -      a^4*b^4*x^2*z - a^2*b^6*x^2*z + b^8*x^2*z - a^4*b^2*c^2*x^2*z - 6*a^2*b^4*c^2*x^2*z -      b^6*c^2*x^2*z + 3*a^2*b^2*c^4*x^2*z + 3*b^4*c^4*x^2*z - 3*b^2*c^6*x^2*z - a^8*y^2*z +      a^6*b^2*y^2*z + a^4*b^4*y^2*z - a^2*b^6*y^2*z + a^6*c^2*y^2*z + 6*a^4*b^2*c^2*y^2*z +      a^2*b^4*c^2*y^2*z - 3*a^4*c^4*y^2*z - 3*a^2*b^2*c^4*y^2*z + 3*a^2*c^6*y^2*z + 3*a^6*b^2*x*z^2 -      3*a^4*b^4*x*z^2 + a^2*b^6*x*z^2 - b^8*x*z^2 - 3*a^4*b^2*c^2*x*z^2 + 6*a^2*b^4*c^2*x*z^2 +      b^6*c^2*x*z^2 + a^2*b^2*c^4*x*z^2 + b^4*c^4*x*z^2 - b^2*c^6*x*z^2 + a^8*y*z^2 - a^6*b^2*y*z^2 +      3*a^4*b^4*y*z^2 - 3*a^2*b^6*y*z^2 - a^6*c^2*y*z^2 - 6*a^4*b^2*c^2*y*z^2 + 3*a^2*b^4*c^2*y*z^2 -      a^4*c^4*y*z^2 - a^2*b^2*c^4*y*z^2 + a^2*c^6*y*z^2");
		exp("K174","(-a^4)*b^2*c^4*x^2*y + b^6*c^4*x^2*y - b^2*c^8*x^2*y - a^6*c^4*x*y^2 + a^2*b^4*c^4*x*y^2 +      a^2*c^8*x*y^2 + a^4*b^4*c^2*x^2*z + b^8*c^2*x^2*z - b^4*c^6*x^2*z - a^8*c^2*y^2*z -      a^4*b^4*c^2*y^2*z + a^4*c^6*y^2*z + a^6*b^4*x*z^2 - a^2*b^8*x*z^2 - a^2*b^4*c^4*x*z^2 +      a^8*b^2*y*z^2 - a^4*b^6*y*z^2 + a^4*b^2*c^4*y*z^2");
		exp("K175","(-b)*c^4*x^2*y + a*c^4*x*y^2 + b^4*c*x^2*z - a^4*c*y^2*z - a*b^4*x*z^2 + a^4*b*y*z^2");
		exp("K176","a^4*c^4*x^2*y - b^4*c^4*x^2*y - 2*a^2*c^6*x^2*y + c^8*x^2*y + a^4*c^4*x*y^2 - b^4*c^4*x*y^2 +      2*b^2*c^6*x*y^2 - c^8*x*y^2 - a^4*b^4*x^2*z + 2*a^2*b^6*x^2*z - b^8*x^2*z + b^4*c^4*x^2*z +      a^8*y^2*z - 2*a^6*b^2*y^2*z + a^4*b^4*y^2*z - a^4*c^4*y^2*z - a^4*b^4*x*z^2 + b^8*x*z^2 -      2*b^6*c^2*x*z^2 + b^4*c^4*x*z^2 - a^8*y*z^2 + a^4*b^4*y*z^2 + 2*a^6*c^2*y*z^2 - a^4*c^4*y*z^2");
		exp("K177","(-c^4)*x^2*y + c^4*x*y^2 + b^4*x^2*z - a^4*y^2*z - b^4*x*z^2 + a^4*y*z^2");
		exp("K178","(-a^2)*c^4*x^2*y + b^2*c^4*x^2*y - c^6*x^2*y - a^2*c^4*x*y^2 + b^2*c^4*x*y^2 + c^6*x*y^2 +      a^2*b^4*x^2*z + b^6*x^2*z - b^4*c^2*x^2*z - a^6*y^2*z - a^4*b^2*y^2*z + a^4*c^2*y^2*z +      a^2*b^4*x*z^2 - b^6*x*z^2 - b^4*c^2*x*z^2 + a^6*y*z^2 - a^4*b^2*y*z^2 + a^4*c^2*y*z^2");
		exp("K179","(-a^3)*b*c^4*x^2*y - a^2*b^2*c^4*x^2*y + a*b^3*c^4*x^2*y + b^4*c^4*x^2*y + a^2*b*c^5*x^2*y -      2*a*b^2*c^5*x^2*y + b^3*c^5*x^2*y + a*b*c^6*x^2*y - b^2*c^6*x^2*y - b*c^7*x^2*y - a^4*c^4*x*y^2 -      a^3*b*c^4*x*y^2 + a^2*b^2*c^4*x*y^2 + a*b^3*c^4*x*y^2 - a^3*c^5*x*y^2 + 2*a^2*b*c^5*x*y^2 -      a*b^2*c^5*x*y^2 + a^2*c^6*x*y^2 - a*b*c^6*x*y^2 + a*c^7*x*y^2 + a^3*b^4*c*x^2*z - a^2*b^5*c*x^2*z -      a*b^6*c*x^2*z + b^7*c*x^2*z + a^2*b^4*c^2*x^2*z + 2*a*b^5*c^2*x^2*z + b^6*c^2*x^2*z -      a*b^4*c^3*x^2*z - b^5*c^3*x^2*z - b^4*c^4*x^2*z - a^7*c*y^2*z + a^6*b*c*y^2*z + a^5*b^2*c*y^2*z -      a^4*b^3*c*y^2*z - a^6*c^2*y^2*z - 2*a^5*b*c^2*y^2*z - a^4*b^2*c^2*y^2*z + a^5*c^3*y^2*z +      a^4*b*c^3*y^2*z + a^4*c^4*y^2*z + a^4*b^4*x*z^2 + a^3*b^5*x*z^2 - a^2*b^6*x*z^2 - a*b^7*x*z^2 +      a^3*b^4*c*x*z^2 - 2*a^2*b^5*c*x*z^2 + a*b^6*c*x*z^2 - a^2*b^4*c^2*x*z^2 + a*b^5*c^2*x*z^2 -      a*b^4*c^3*x*z^2 + a^7*b*y*z^2 + a^6*b^2*y*z^2 - a^5*b^3*y*z^2 - a^4*b^4*y*z^2 - a^6*b*c*y*z^2 +      2*a^5*b^2*c*y*z^2 - a^4*b^3*c*y*z^2 - a^5*b*c^2*y*z^2 + a^4*b^2*c^2*y*z^2 + a^4*b*c^3*y*z^2");
		exp("K180","a^6*b*c^4*x^2*y - 3*a^4*b^3*c^4*x^2*y + 3*a^2*b^5*c^4*x^2*y - b^7*c^4*x^2*y + 2*a^5*b*c^5*x^2*y +      4*a^3*b^3*c^5*x^2*y - 6*a*b^5*c^5*x^2*y - a^4*b*c^6*x^2*y - 2*a^2*b^3*c^6*x^2*y + 3*b^5*c^6*x^2*y -      4*a^3*b*c^7*x^2*y + 4*a*b^3*c^7*x^2*y - a^2*b*c^8*x^2*y - 3*b^3*c^8*x^2*y + 2*a*b*c^9*x^2*y +      b*c^10*x^2*y + a^7*c^4*x*y^2 - 3*a^5*b^2*c^4*x*y^2 + 3*a^3*b^4*c^4*x*y^2 - a*b^6*c^4*x*y^2 +      6*a^5*b*c^5*x*y^2 - 4*a^3*b^3*c^5*x*y^2 - 2*a*b^5*c^5*x*y^2 - 3*a^5*c^6*x*y^2 +      2*a^3*b^2*c^6*x*y^2 + a*b^4*c^6*x*y^2 - 4*a^3*b*c^7*x*y^2 + 4*a*b^3*c^7*x*y^2 + 3*a^3*c^8*x*y^2 +      a*b^2*c^8*x*y^2 - 2*a*b*c^9*x*y^2 - a*c^10*x*y^2 - a^6*b^4*c*x^2*z - 2*a^5*b^5*c*x^2*z +      a^4*b^6*c*x^2*z + 4*a^3*b^7*c*x^2*z + a^2*b^8*c*x^2*z - 2*a*b^9*c*x^2*z - b^10*c*x^2*z +      3*a^4*b^4*c^3*x^2*z - 4*a^3*b^5*c^3*x^2*z + 2*a^2*b^6*c^3*x^2*z - 4*a*b^7*c^3*x^2*z +      3*b^8*c^3*x^2*z - 3*a^2*b^4*c^5*x^2*z + 6*a*b^5*c^5*x^2*z - 3*b^6*c^5*x^2*z + b^4*c^7*x^2*z +      a^10*c*y^2*z + 2*a^9*b*c*y^2*z - a^8*b^2*c*y^2*z - 4*a^7*b^3*c*y^2*z - a^6*b^4*c*y^2*z +      2*a^5*b^5*c*y^2*z + a^4*b^6*c*y^2*z - 3*a^8*c^3*y^2*z + 4*a^7*b*c^3*y^2*z - 2*a^6*b^2*c^3*y^2*z +      4*a^5*b^3*c^3*y^2*z - 3*a^4*b^4*c^3*y^2*z + 3*a^6*c^5*y^2*z - 6*a^5*b*c^5*y^2*z +      3*a^4*b^2*c^5*y^2*z - a^4*c^7*y^2*z - a^7*b^4*x*z^2 + 3*a^5*b^6*x*z^2 - 3*a^3*b^8*x*z^2 +      a*b^10*x*z^2 - 6*a^5*b^5*c*x*z^2 + 4*a^3*b^7*c*x*z^2 + 2*a*b^9*c*x*z^2 + 3*a^5*b^4*c^2*x*z^2 -      2*a^3*b^6*c^2*x*z^2 - a*b^8*c^2*x*z^2 + 4*a^3*b^5*c^3*x*z^2 - 4*a*b^7*c^3*x*z^2 -      3*a^3*b^4*c^4*x*z^2 - a*b^6*c^4*x*z^2 + 2*a*b^5*c^5*x*z^2 + a*b^4*c^6*x*z^2 - a^10*b*y*z^2 +      3*a^8*b^3*y*z^2 - 3*a^6*b^5*y*z^2 + a^4*b^7*y*z^2 - 2*a^9*b*c*y*z^2 - 4*a^7*b^3*c*y*z^2 +      6*a^5*b^5*c*y*z^2 + a^8*b*c^2*y*z^2 + 2*a^6*b^3*c^2*y*z^2 - 3*a^4*b^5*c^2*y*z^2 +      4*a^7*b*c^3*y*z^2 - 4*a^5*b^3*c^3*y*z^2 + a^6*b*c^4*y*z^2 + 3*a^4*b^3*c^4*y*z^2 -      2*a^5*b*c^5*y*z^2 - a^4*b*c^6*y*z^2");
		exp("K181","(-a^2)*x^2*y + b^2*x^2*y + c^2*x^2*y - a^2*x*y^2 + b^2*x*y^2 - c^2*x*y^2 + a^2*x^2*z -      b^2*x^2*z - c^2*x^2*z + a^2*y^2*z - b^2*y^2*z + c^2*y^2*z + a^2*x*z^2 + b^2*x*z^2 - c^2*x*z^2 -      a^2*y*z^2 - b^2*y*z^2 + c^2*y*z^2");
		exp("K182","a^8*x^2*y - 2*a^6*b^2*x^2*y + 2*a^2*b^6*x^2*y - b^8*x^2*y + 6*a^4*b^2*c^2*x^2*y -      4*a^2*b^4*c^2*x^2*y - 2*b^6*c^2*x^2*y - 6*a^4*c^4*x^2*y - 6*a^2*b^2*c^4*x^2*y + 4*b^4*c^4*x^2*y +      8*a^2*c^6*x^2*y + 2*b^2*c^6*x^2*y - 3*c^8*x^2*y + a^8*x*y^2 - 2*a^6*b^2*x*y^2 + 2*a^2*b^6*x*y^2 -      b^8*x*y^2 + 2*a^6*c^2*x*y^2 + 4*a^4*b^2*c^2*x*y^2 - 6*a^2*b^4*c^2*x*y^2 - 4*a^4*c^4*x*y^2 +      6*a^2*b^2*c^4*x*y^2 + 6*b^4*c^4*x*y^2 - 2*a^2*c^6*x*y^2 - 8*b^2*c^6*x*y^2 + 3*c^8*x*y^2 -      a^8*x^2*z + 6*a^4*b^4*x^2*z - 8*a^2*b^6*x^2*z + 3*b^8*x^2*z + 2*a^6*c^2*x^2*z -      6*a^4*b^2*c^2*x^2*z + 6*a^2*b^4*c^2*x^2*z - 2*b^6*c^2*x^2*z + 4*a^2*b^2*c^4*x^2*z -      4*b^4*c^4*x^2*z - 2*a^2*c^6*x^2*z + 2*b^2*c^6*x^2*z + c^8*x^2*z - 3*a^8*y^2*z + 8*a^6*b^2*y^2*z -      6*a^4*b^4*y^2*z + b^8*y^2*z + 2*a^6*c^2*y^2*z - 6*a^4*b^2*c^2*y^2*z + 6*a^2*b^4*c^2*y^2*z -      2*b^6*c^2*y^2*z + 4*a^4*c^4*y^2*z - 4*a^2*b^2*c^4*y^2*z - 2*a^2*c^6*y^2*z + 2*b^2*c^6*y^2*z -      c^8*y^2*z - a^8*x*z^2 - 2*a^6*b^2*x*z^2 + 4*a^4*b^4*x*z^2 + 2*a^2*b^6*x*z^2 - 3*b^8*x*z^2 +      2*a^6*c^2*x*z^2 - 4*a^4*b^2*c^2*x*z^2 - 6*a^2*b^4*c^2*x*z^2 + 8*b^6*c^2*x*z^2 +      6*a^2*b^2*c^4*x*z^2 - 6*b^4*c^4*x*z^2 - 2*a^2*c^6*x*z^2 + c^8*x*z^2 + 3*a^8*y*z^2 -      2*a^6*b^2*y*z^2 - 4*a^4*b^4*y*z^2 + 2*a^2*b^6*y*z^2 + b^8*y*z^2 - 8*a^6*c^2*y*z^2 +      6*a^4*b^2*c^2*y*z^2 + 4*a^2*b^4*c^2*y*z^2 - 2*b^6*c^2*y*z^2 + 6*a^4*c^4*y*z^2 -      6*a^2*b^2*c^4*y*z^2 + 2*b^2*c^6*y*z^2 - c^8*y*z^2");
		exp("K183","(-a^6)*x^2*y - 2*a^4*b^2*x^2*y + 3*a^2*b^4*x^2*y + 2*a^4*c^2*x^2*y - 2*a^2*b^2*c^2*x^2*y -      a^2*c^4*x^2*y - 3*a^4*b^2*x*y^2 + 2*a^2*b^4*x*y^2 + b^6*x*y^2 + 2*a^2*b^2*c^2*x*y^2 -      2*b^4*c^2*x*y^2 + b^2*c^4*x*y^2 + a^6*x^2*z - 2*a^4*b^2*x^2*z + a^2*b^4*x^2*z + 2*a^4*c^2*x^2*z +      2*a^2*b^2*c^2*x^2*z - 3*a^2*c^4*x^2*z - a^4*b^2*y^2*z + 2*a^2*b^4*y^2*z - b^6*y^2*z -      2*a^2*b^2*c^2*y^2*z - 2*b^4*c^2*y^2*z + 3*b^2*c^4*y^2*z + 3*a^4*c^2*x*z^2 - 2*a^2*b^2*c^2*x*z^2 -      b^4*c^2*x*z^2 - 2*a^2*c^4*x*z^2 + 2*b^2*c^4*x*z^2 - c^6*x*z^2 + a^4*c^2*y*z^2 +      2*a^2*b^2*c^2*y*z^2 - 3*b^4*c^2*y*z^2 - 2*a^2*c^4*y*z^2 + 2*b^2*c^4*y*z^2 + c^6*y*z^2");
		exp("K184","a^2*x^2*y - b^2*x*y^2 - a^2*x^2*z + b^2*y^2*z + c^2*x*z^2 - c^2*y*z^2");
		exp("K185","(-a^2)*x^2*y + c^2*x^2*y + b^2*x*y^2 - c^2*x*y^2 + a^2*x^2*z - b^2*x^2*z + a^2*y^2*z -      b^2*y^2*z + b^2*x*z^2 - c^2*x*z^2 - a^2*y*z^2 + c^2*y*z^2");
		exp("K186","a^8*x^2*y - 4*a^6*b^2*x^2*y + 6*a^4*b^4*x^2*y - 4*a^2*b^6*x^2*y + b^8*x^2*y - a^6*c^2*x^2*y +      2*a^4*b^2*c^2*x^2*y - a^2*b^4*c^2*x^2*y + 2*a^2*b^2*c^4*x^2*y - 2*b^4*c^4*x^2*y - a^2*c^6*x^2*y +      c^8*x^2*y + a^8*x*y^2 - 4*a^6*b^2*x*y^2 + 6*a^4*b^4*x*y^2 - 4*a^2*b^6*x*y^2 + b^8*x*y^2 -      a^4*b^2*c^2*x*y^2 + 2*a^2*b^4*c^2*x*y^2 - b^6*c^2*x*y^2 - 2*a^4*c^4*x*y^2 + 2*a^2*b^2*c^4*x*y^2 -      b^2*c^6*x*y^2 + c^8*x*y^2 + a^8*x^2*z - a^6*b^2*x^2*z - a^2*b^6*x^2*z + b^8*x^2*z -      4*a^6*c^2*x^2*z + 2*a^4*b^2*c^2*x^2*z + 2*a^2*b^4*c^2*x^2*z + 6*a^4*c^4*x^2*z - a^2*b^2*c^4*x^2*z -      2*b^4*c^4*x^2*z - 4*a^2*c^6*x^2*z + c^8*x^2*z + 2*a^8*x*y*z - 2*a^6*b^2*x*y*z - 2*a^2*b^6*x*y*z +      2*b^8*x*y*z - 2*a^6*c^2*x*y*z + 2*a^4*b^2*c^2*x*y*z + 2*a^2*b^4*c^2*x*y*z - 2*b^6*c^2*x*y*z +      2*a^2*b^2*c^4*x*y*z - 2*a^2*c^6*x*y*z - 2*b^2*c^6*x*y*z + 2*c^8*x*y*z + a^8*y^2*z - a^6*b^2*y^2*z -      a^2*b^6*y^2*z + b^8*y^2*z + 2*a^4*b^2*c^2*y^2*z + 2*a^2*b^4*c^2*y^2*z - 4*b^6*c^2*y^2*z -      2*a^4*c^4*y^2*z - a^2*b^2*c^4*y^2*z + 6*b^4*c^4*y^2*z - 4*b^2*c^6*y^2*z + c^8*y^2*z + a^8*x*z^2 -      2*a^4*b^4*x*z^2 + b^8*x*z^2 - 4*a^6*c^2*x*z^2 - a^4*b^2*c^2*x*z^2 + 2*a^2*b^4*c^2*x*z^2 -      b^6*c^2*x*z^2 + 6*a^4*c^4*x*z^2 + 2*a^2*b^2*c^4*x*z^2 - 4*a^2*c^6*x*z^2 - b^2*c^6*x*z^2 +      c^8*x*z^2 + a^8*y*z^2 - 2*a^4*b^4*y*z^2 + b^8*y*z^2 - a^6*c^2*y*z^2 + 2*a^4*b^2*c^2*y*z^2 -      a^2*b^4*c^2*y*z^2 - 4*b^6*c^2*y*z^2 + 2*a^2*b^2*c^4*y*z^2 + 6*b^4*c^4*y*z^2 - a^2*c^6*y*z^2 -      4*b^2*c^6*y*z^2 + c^8*y*z^2");
		exp("K187","a^4*c^2*x^2*y - a^2*b^2*c^2*x^2*y + b^2*c^4*x^2*y - c^6*x^2*y + a^2*b^2*c^2*x*y^2 -      b^4*c^2*x*y^2 - a^2*c^4*x*y^2 + c^6*x*y^2 - a^4*b^2*x^2*z + b^6*x^2*z + a^2*b^2*c^2*x^2*z -      b^4*c^2*x^2*z - 4*a^4*b^2*x*y*z + 4*a^2*b^4*x*y*z + 4*a^4*c^2*x*y*z - 4*b^4*c^2*x*y*z -      4*a^2*c^4*x*y*z + 4*b^2*c^4*x*y*z - a^6*y^2*z + a^2*b^4*y^2*z + a^4*c^2*y^2*z - a^2*b^2*c^2*y^2*z +      a^2*b^4*x*z^2 - b^6*x*z^2 - a^2*b^2*c^2*x*z^2 + b^2*c^4*x*z^2 + a^6*y*z^2 - a^4*b^2*y*z^2 +      a^2*b^2*c^2*y*z^2 - a^2*c^4*y*z^2");
		exp("K188","(-a^6)*c^2*x^2*y - a^4*b^2*c^2*x^2*y + 2*a^2*b^4*c^2*x^2*y + 3*a^4*c^4*x^2*y - 2*b^4*c^4*x^2*y -      3*a^2*c^6*x^2*y + b^2*c^6*x^2*y + c^8*x^2*y - 2*a^4*b^2*c^2*x*y^2 + a^2*b^4*c^2*x*y^2 +      b^6*c^2*x*y^2 + 2*a^4*c^4*x*y^2 - 3*b^4*c^4*x*y^2 - a^2*c^6*x*y^2 + 3*b^2*c^6*x*y^2 - c^8*x*y^2 +      a^6*b^2*x^2*z - 3*a^4*b^4*x^2*z + 3*a^2*b^6*x^2*z - b^8*x^2*z + a^4*b^2*c^2*x^2*z - b^6*c^2*x^2*z -      2*a^2*b^2*c^4*x^2*z + 2*b^4*c^4*x^2*z + a^8*y^2*z - 3*a^6*b^2*y^2*z + 3*a^4*b^4*y^2*z -      a^2*b^6*y^2*z + a^6*c^2*y^2*z - a^2*b^4*c^2*y^2*z - 2*a^4*c^4*y^2*z + 2*a^2*b^2*c^4*y^2*z -      2*a^4*b^4*x*z^2 + a^2*b^6*x*z^2 + b^8*x*z^2 + 2*a^4*b^2*c^2*x*z^2 - 3*b^6*c^2*x*z^2 -      a^2*b^2*c^4*x*z^2 + 3*b^4*c^4*x*z^2 - b^2*c^6*x*z^2 - a^8*y*z^2 - a^6*b^2*y*z^2 + 2*a^4*b^4*y*z^2 +      3*a^6*c^2*y*z^2 - 2*a^2*b^4*c^2*y*z^2 - 3*a^4*c^4*y*z^2 + a^2*b^2*c^4*y*z^2 + a^2*c^6*y*z^2");
		exp("K189","(-a^4)*c^2*x^2*y + a^2*b^2*c^2*x^2*y - 2*b^4*c^2*x^2*y + 2*a^2*c^4*x^2*y + b^2*c^4*x^2*y -      c^6*x^2*y - 2*a^4*c^2*x*y^2 + a^2*b^2*c^2*x*y^2 - b^4*c^2*x*y^2 + a^2*c^4*x*y^2 + 2*b^2*c^4*x*y^2 -      c^6*x*y^2 - a^4*b^2*x^2*z + 2*a^2*b^4*x^2*z - b^6*x^2*z + a^2*b^2*c^2*x^2*z + b^4*c^2*x^2*z -      2*b^2*c^4*x^2*z - a^6*y^2*z + 2*a^4*b^2*y^2*z - a^2*b^4*y^2*z + a^4*c^2*y^2*z + a^2*b^2*c^2*y^2*z -      2*a^2*c^4*y^2*z - 2*a^4*b^2*x*z^2 + a^2*b^4*x*z^2 - b^6*x*z^2 + a^2*b^2*c^2*x*z^2 +      2*b^4*c^2*x*z^2 - b^2*c^4*x*z^2 - a^6*y*z^2 + a^4*b^2*y*z^2 - 2*a^2*b^4*y*z^2 + 2*a^4*c^2*y*z^2 +      a^2*b^2*c^2*y*z^2 - a^2*c^4*y*z^2");
		exp("K190","a^6*c^2*x^2*y - 2*a^4*b^2*c^2*x^2*y - a^2*b^4*c^2*x^2*y + 2*b^6*c^2*x^2*y - a^4*c^4*x^2*y +      4*a^2*b^2*c^4*x^2*y - b^4*c^4*x^2*y - a^2*c^6*x^2*y - 2*b^2*c^6*x^2*y + c^8*x^2*y +      2*a^6*c^2*x*y^2 - a^4*b^2*c^2*x*y^2 - 2*a^2*b^4*c^2*x*y^2 + b^6*c^2*x*y^2 - a^4*c^4*x*y^2 +      4*a^2*b^2*c^4*x*y^2 - b^4*c^4*x*y^2 - 2*a^2*c^6*x*y^2 - b^2*c^6*x*y^2 + c^8*x*y^2 + a^6*b^2*x^2*z -      a^4*b^4*x^2*z - a^2*b^6*x^2*z + b^8*x^2*z - 2*a^4*b^2*c^2*x^2*z + 4*a^2*b^4*c^2*x^2*z -      2*b^6*c^2*x^2*z - a^2*b^2*c^4*x^2*z - b^4*c^4*x^2*z + 2*b^2*c^6*x^2*z + a^8*y^2*z - a^6*b^2*y^2*z -      a^4*b^4*y^2*z + a^2*b^6*y^2*z - 2*a^6*c^2*y^2*z + 4*a^4*b^2*c^2*y^2*z - 2*a^2*b^4*c^2*y^2*z -      a^4*c^4*y^2*z - a^2*b^2*c^4*y^2*z + 2*a^2*c^6*y^2*z + 2*a^6*b^2*x*z^2 - a^4*b^4*x*z^2 -      2*a^2*b^6*x*z^2 + b^8*x*z^2 - a^4*b^2*c^2*x*z^2 + 4*a^2*b^4*c^2*x*z^2 - b^6*c^2*x*z^2 -      2*a^2*b^2*c^4*x*z^2 - b^4*c^4*x*z^2 + b^2*c^6*x*z^2 + a^8*y*z^2 - 2*a^6*b^2*y*z^2 - a^4*b^4*y*z^2 +      2*a^2*b^6*y*z^2 - a^6*c^2*y*z^2 + 4*a^4*b^2*c^2*y*z^2 - a^2*b^4*c^2*y*z^2 - a^4*c^4*y*z^2 -      2*a^2*b^2*c^4*y*z^2 + a^2*c^6*y*z^2");
		exp("K191","-2*b^2*c^2*x^2*y - 2*a^2*c^2*x*y^2 - 2*b^2*c^2*x^2*z + a^4*x*y*z - 2*a^2*b^2*x*y*z + b^4*x*y*z -      2*a^2*c^2*x*y*z - 2*b^2*c^2*x*y*z + c^4*x*y*z - 2*a^2*c^2*y^2*z - 2*a^2*b^2*x*z^2 -      2*a^2*b^2*y*z^2");
		exp("K192","(-b^2)*c^2*x^2*y - a^2*c^2*x*y^2 - b^2*c^2*x^2*z + a^4*x*y*z - 2*a^2*b^2*x*y*z + b^4*x*y*z -      2*a^2*c^2*x*y*z - 2*b^2*c^2*x*y*z + c^4*x*y*z - a^2*c^2*y^2*z - a^2*b^2*x*z^2 - a^2*b^2*y*z^2");
		exp("K193","a^2*b^6*c^4*x^3 + a^2*b^4*c^6*x^3 + a^4*b^4*c^4*x^2*y + a^2*b^6*c^4*x^2*y - 2*a^2*b^4*c^6*x^2*y +      a^6*b^2*c^4*x*y^2 + a^4*b^4*c^4*x*y^2 - 2*a^4*b^2*c^6*x*y^2 + a^6*b^2*c^4*y^3 + a^4*b^2*c^6*y^3 +      a^4*b^4*c^4*x^2*z - 2*a^2*b^6*c^4*x^2*z + a^2*b^4*c^6*x^2*z - 6*a^4*b^4*c^4*x*y*z -      2*a^6*b^2*c^4*y^2*z + a^4*b^4*c^4*y^2*z + a^4*b^2*c^6*y^2*z + a^6*b^4*c^2*x*z^2 -      2*a^4*b^6*c^2*x*z^2 + a^4*b^4*c^4*x*z^2 - 2*a^6*b^4*c^2*y*z^2 + a^4*b^6*c^2*y*z^2 +      a^4*b^4*c^4*y*z^2 + a^6*b^4*c^2*z^3 + a^4*b^6*c^2*z^3");
		exp("K194","4*a^6*x^2*y - 12*a^4*b^2*x^2*y + 12*a^2*b^4*x^2*y - 4*b^6*x^2*y - 4*a^4*c^2*x^2*y -      a^2*b^2*c^2*x^2*y + 5*b^4*c^2*x^2*y - 4*a^2*c^4*x^2*y - 5*b^2*c^4*x^2*y + 4*c^6*x^2*y +      4*a^6*x*y^2 - 12*a^4*b^2*x*y^2 + 12*a^2*b^4*x*y^2 - 4*b^6*x*y^2 - 5*a^4*c^2*x*y^2 +      a^2*b^2*c^2*x*y^2 + 4*b^4*c^2*x*y^2 + 5*a^2*c^4*x*y^2 + 4*b^2*c^4*x*y^2 - 4*c^6*x*y^2 -      4*a^6*x^2*z + 4*a^4*b^2*x^2*z + 4*a^2*b^4*x^2*z - 4*b^6*x^2*z + 12*a^4*c^2*x^2*z +      a^2*b^2*c^2*x^2*z + 5*b^4*c^2*x^2*z - 12*a^2*c^4*x^2*z - 5*b^2*c^4*x^2*z + 4*c^6*x^2*z +      4*a^6*y^2*z - 4*a^4*b^2*y^2*z - 4*a^2*b^4*y^2*z + 4*b^6*y^2*z - 5*a^4*c^2*y^2*z -      a^2*b^2*c^2*y^2*z - 12*b^4*c^2*y^2*z + 5*a^2*c^4*y^2*z + 12*b^2*c^4*y^2*z - 4*c^6*y^2*z -      4*a^6*x*z^2 + 5*a^4*b^2*x*z^2 - 5*a^2*b^4*x*z^2 + 4*b^6*x*z^2 + 12*a^4*c^2*x*z^2 -      a^2*b^2*c^2*x*z^2 - 4*b^4*c^2*x*z^2 - 12*a^2*c^4*x*z^2 - 4*b^2*c^4*x*z^2 + 4*c^6*x*z^2 -      4*a^6*y*z^2 + 5*a^4*b^2*y*z^2 - 5*a^2*b^4*y*z^2 + 4*b^6*y*z^2 + 4*a^4*c^2*y*z^2 +      a^2*b^2*c^2*y*z^2 - 12*b^4*c^2*y*z^2 + 4*a^2*c^4*y*z^2 + 12*b^2*c^4*y*z^2 - 4*c^6*y*z^2");
		exp("K195","9*a^6*x^2*y - 27*a^4*b^2*x^2*y + 27*a^2*b^4*x^2*y - 9*b^6*x^2*y - 9*a^4*c^2*x^2*y +      2*a^2*b^2*c^2*x^2*y + 7*b^4*c^2*x^2*y - 9*a^2*c^4*x^2*y - 7*b^2*c^4*x^2*y + 9*c^6*x^2*y +      9*a^6*x*y^2 - 27*a^4*b^2*x*y^2 + 27*a^2*b^4*x*y^2 - 9*b^6*x*y^2 - 7*a^4*c^2*x*y^2 -      2*a^2*b^2*c^2*x*y^2 + 9*b^4*c^2*x*y^2 + 7*a^2*c^4*x*y^2 + 9*b^2*c^4*x*y^2 - 9*c^6*x*y^2 -      9*a^6*x^2*z + 9*a^4*b^2*x^2*z + 9*a^2*b^4*x^2*z - 9*b^6*x^2*z + 27*a^4*c^2*x^2*z -      2*a^2*b^2*c^2*x^2*z + 7*b^4*c^2*x^2*z - 27*a^2*c^4*x^2*z - 7*b^2*c^4*x^2*z + 9*c^6*x^2*z +      9*a^6*y^2*z - 9*a^4*b^2*y^2*z - 9*a^2*b^4*y^2*z + 9*b^6*y^2*z - 7*a^4*c^2*y^2*z +      2*a^2*b^2*c^2*y^2*z - 27*b^4*c^2*y^2*z + 7*a^2*c^4*y^2*z + 27*b^2*c^4*y^2*z - 9*c^6*y^2*z -      9*a^6*x*z^2 + 7*a^4*b^2*x*z^2 - 7*a^2*b^4*x*z^2 + 9*b^6*x*z^2 + 27*a^4*c^2*x*z^2 +      2*a^2*b^2*c^2*x*z^2 - 9*b^4*c^2*x*z^2 - 27*a^2*c^4*x*z^2 - 9*b^2*c^4*x*z^2 + 9*c^6*x*z^2 -      9*a^6*y*z^2 + 7*a^4*b^2*y*z^2 - 7*a^2*b^4*y*z^2 + 9*b^6*y*z^2 + 9*a^4*c^2*y*z^2 -      2*a^2*b^2*c^2*y*z^2 - 27*b^4*c^2*y*z^2 + 9*a^2*c^4*y*z^2 + 27*b^2*c^4*y*z^2 - 9*c^6*y*z^2");
		exp("K196","a^2*b^2*x^2*y + a^2*b^2*x*y^2 + a^2*c^2*x^2*z + b^2*c^2*y^2*z + a^2*c^2*x*z^2 + b^2*c^2*y*z^2 ");
		exp("K197","(-a^4)*x^2*y - b^4*x^2*y + 3*a^2*c^2*x^2*y - c^4*x^2*y - a^4*x*y^2 - b^4*x*y^2 +      3*b^2*c^2*x*y^2 - c^4*x*y^2 - a^4*x^2*z + 3*a^2*b^2*x^2*z - b^4*x^2*z - c^4*x^2*z - a^4*y^2*z +      3*a^2*b^2*y^2*z - b^4*y^2*z - c^4*y^2*z - a^4*x*z^2 - b^4*x*z^2 + 3*b^2*c^2*x*z^2 - c^4*x*z^2 -      a^4*y*z^2 - b^4*y*z^2 + 3*a^2*c^2*y*z^2 - c^4*y*z^2");
		exp("K198","a^2*c^2*x^2*y + b^2*c^2*x*y^2 + a^2*b^2*x^2*z + a^2*b^2*y^2*z + b^2*c^2*x*z^2 + a^2*c^2*y*z^2 +      (2*a^2*b^2 + 2*a^2*c^2 + 2*b^2*c^2)*x*y*z");
		exp("K199","a^2*c*x^2*y - b^2*c*x^2*y + 2*b*c^2*x^2*y - c^3*x^2*y + a^2*c*x*y^2 - b^2*c*x*y^2 -      2*a*c^2*x*y^2 + c^3*x*y^2 - a^2*b*x^2*z + b^3*x^2*z - 2*b^2*c*x^2*z + b*c^2*x^2*z - a^3*y^2*z +      a*b^2*y^2*z + 2*a^2*c*y^2*z - a*c^2*y^2*z - a^2*b*x*z^2 + 2*a*b^2*x*z^2 - b^3*x*z^2 + b*c^2*x*z^2 +      a^3*y*z^2 - 2*a^2*b*y*z^2 + a*b^2*y*z^2 - a*c^2*y*z^2");
		exp("K200","a*x^2*y - b*x^2*y + c*x^2*y + a*x*y^2 - b*x*y^2 - c*x*y^2 - a*x^2*z - b*x^2*z + c*x^2*z +      a*y^2*z + b*y^2*z - c*y^2*z - a*x*z^2 + b*x*z^2 + c*x*z^2 - a*y*z^2 + b*y*z^2 - c*y*z^2");
		exp("K201","a^2*c*x^2*y - 2*a*b*c*x^2*y - 3*b^2*c*x^2*y + 4*b*c^2*x^2*y - c^3*x^2*y + 3*a^2*c*x*y^2 +      2*a*b*c*x*y^2 - b^2*c*x*y^2 - 4*a*c^2*x*y^2 + c^3*x*y^2 - a^2*b*x^2*z + b^3*x^2*z + 2*a*b*c*x^2*z -      4*b^2*c*x^2*z + 3*b*c^2*x^2*z - a^3*y^2*z + a*b^2*y^2*z + 4*a^2*c*y^2*z - 2*a*b*c*y^2*z -      3*a*c^2*y^2*z - 3*a^2*b*x*z^2 + 4*a*b^2*x*z^2 - b^3*x*z^2 - 2*a*b*c*x*z^2 + b*c^2*x*z^2 +      a^3*y*z^2 - 4*a^2*b*y*z^2 + 3*a*b^2*y*z^2 + 2*a*b*c*y*z^2 - a*c^2*y*z^2");
		exp("K202","(-a^2)*c*x^2*y - 2*a*b*c*x^2*y + 3*b^2*c*x^2*y + 2*a*c^2*x^2*y - 2*b*c^2*x^2*y - c^3*x^2*y -      3*a^2*c*x*y^2 + 2*a*b*c*x*y^2 + b^2*c*x*y^2 + 2*a*c^2*x*y^2 - 2*b*c^2*x*y^2 + c^3*x*y^2 +      a^2*b*x^2*z - 2*a*b^2*x^2*z + b^3*x^2*z + 2*a*b*c*x^2*z + 2*b^2*c*x^2*z - 3*b*c^2*x^2*z -      a^3*y^2*z + 2*a^2*b*y^2*z - a*b^2*y^2*z - 2*a^2*c*y^2*z - 2*a*b*c*y^2*z + 3*a*c^2*y^2*z +      3*a^2*b*x*z^2 - 2*a*b^2*x*z^2 - b^3*x*z^2 - 2*a*b*c*x*z^2 + 2*b^2*c*x*z^2 - b*c^2*x*z^2 +      a^3*y*z^2 + 2*a^2*b*y*z^2 - 3*a*b^2*y*z^2 - 2*a^2*c*y*z^2 + 2*a*b*c*y*z^2 + a*c^2*y*z^2");
		exp("K203","(-a^8)*x^3 + 2*a^6*b^2*x^3 - a^4*b^4*x^3 + 2*a^6*c^2*x^3 - 4*a^4*b^2*c^2*x^3 +      2*a^2*b^4*c^2*x^3 - a^4*c^4*x^3 + 2*a^2*b^2*c^4*x^3 - b^4*c^4*x^3 + a^4*b^4*x^2*y -      2*a^2*b^6*x^2*y + b^8*x^2*y - 2*a^4*b^2*c^2*x^2*y + 4*a^2*b^4*c^2*x^2*y - 2*b^6*c^2*x^2*y +      a^4*c^4*x^2*y - 2*a^2*b^2*c^4*x^2*y + b^4*c^4*x^2*y + a^8*x*y^2 - 2*a^6*b^2*x*y^2 + a^4*b^4*x*y^2 -      2*a^6*c^2*x*y^2 + 4*a^4*b^2*c^2*x*y^2 - 2*a^2*b^4*c^2*x*y^2 + a^4*c^4*x*y^2 - 2*a^2*b^2*c^4*x*y^2 +      b^4*c^4*x*y^2 - a^4*b^4*y^3 + 2*a^2*b^6*y^3 - b^8*y^3 + 2*a^4*b^2*c^2*y^3 - 4*a^2*b^4*c^2*y^3 +      2*b^6*c^2*y^3 - a^4*c^4*y^3 + 2*a^2*b^2*c^4*y^3 - b^4*c^4*y^3 + a^4*b^4*x^2*z -      2*a^4*b^2*c^2*x^2*z - 2*a^2*b^4*c^2*x^2*z + a^4*c^4*x^2*z + 4*a^2*b^2*c^4*x^2*z + b^4*c^4*x^2*z -      2*a^2*c^6*x^2*z - 2*b^2*c^6*x^2*z + c^8*x^2*z - 2*a^8*x*y*z + 4*a^6*b^2*x*y*z - 6*a^4*b^4*x*y*z +      4*a^2*b^6*x*y*z - 2*b^8*x*y*z + 4*a^6*c^2*x*y*z + 4*b^6*c^2*x*y*z - 6*a^4*c^4*x*y*z -      6*b^4*c^4*x*y*z + 4*a^2*c^6*x*y*z + 4*b^2*c^6*x*y*z - 2*c^8*x*y*z + a^4*b^4*y^2*z -      2*a^4*b^2*c^2*y^2*z - 2*a^2*b^4*c^2*y^2*z + a^4*c^4*y^2*z + 4*a^2*b^2*c^4*y^2*z + b^4*c^4*y^2*z -      2*a^2*c^6*y^2*z - 2*b^2*c^6*y^2*z + c^8*y^2*z + a^8*x*z^2 - 2*a^6*b^2*x*z^2 + a^4*b^4*x*z^2 -      2*a^6*c^2*x*z^2 + 4*a^4*b^2*c^2*x*z^2 - 2*a^2*b^4*c^2*x*z^2 + a^4*c^4*x*z^2 - 2*a^2*b^2*c^4*x*z^2 +      b^4*c^4*x*z^2 + a^4*b^4*y*z^2 - 2*a^2*b^6*y*z^2 + b^8*y*z^2 - 2*a^4*b^2*c^2*y*z^2 +      4*a^2*b^4*c^2*y*z^2 - 2*b^6*c^2*y*z^2 + a^4*c^4*y*z^2 - 2*a^2*b^2*c^4*y*z^2 + b^4*c^4*y*z^2 -      a^4*b^4*z^3 + 2*a^4*b^2*c^2*z^3 + 2*a^2*b^4*c^2*z^3 - a^4*c^4*z^3 - 4*a^2*b^2*c^4*z^3 -      b^4*c^4*z^3 + 2*a^2*c^6*z^3 + 2*b^2*c^6*z^3 - c^8*z^3");
		exp("K204","(-a^8)*b^4*c^2*x^2*y + 4*a^6*b^6*c^2*x^2*y - 6*a^4*b^8*c^2*x^2*y + 4*a^2*b^10*c^2*x^2*y -      b^12*c^2*x^2*y + a^8*b^2*c^4*x^2*y - 3*a^6*b^4*c^4*x^2*y + 5*a^4*b^6*c^4*x^2*y -      5*a^2*b^8*c^4*x^2*y + 2*b^10*c^4*x^2*y - a^6*b^2*c^6*x^2*y + a^4*b^4*c^6*x^2*y - b^8*c^6*x^2*y +      2*a^2*b^4*c^8*x^2*y + b^6*c^8*x^2*y - a^2*b^2*c^10*x^2*y - 2*b^4*c^10*x^2*y + b^2*c^12*x^2*y +      a^12*c^2*x*y^2 - 4*a^10*b^2*c^2*x*y^2 + 6*a^8*b^4*c^2*x*y^2 - 4*a^6*b^6*c^2*x*y^2 +      a^4*b^8*c^2*x*y^2 - 2*a^10*c^4*x*y^2 + 5*a^8*b^2*c^4*x*y^2 - 5*a^6*b^4*c^4*x*y^2 +      3*a^4*b^6*c^4*x*y^2 - a^2*b^8*c^4*x*y^2 + a^8*c^6*x*y^2 - a^4*b^4*c^6*x*y^2 + a^2*b^6*c^6*x*y^2 -      a^6*c^8*x*y^2 - 2*a^4*b^2*c^8*x*y^2 + 2*a^4*c^10*x*y^2 + a^2*b^2*c^10*x*y^2 - a^2*c^12*x*y^2 -      a^8*b^4*c^2*x^2*z + a^6*b^6*c^2*x^2*z + a^2*b^10*c^2*x^2*z - b^12*c^2*x^2*z + a^8*b^2*c^4*x^2*z +      3*a^6*b^4*c^4*x^2*z - a^4*b^6*c^4*x^2*z - 2*a^2*b^8*c^4*x^2*z + 2*b^10*c^4*x^2*z -      4*a^6*b^2*c^6*x^2*z - 5*a^4*b^4*c^6*x^2*z - b^8*c^6*x^2*z + 6*a^4*b^2*c^8*x^2*z +      5*a^2*b^4*c^8*x^2*z + b^6*c^8*x^2*z - 4*a^2*b^2*c^10*x^2*z - 2*b^4*c^10*x^2*z + b^2*c^12*x^2*z +      a^12*c^2*y^2*z - a^10*b^2*c^2*y^2*z - a^6*b^6*c^2*y^2*z + a^4*b^8*c^2*y^2*z - 2*a^10*c^4*y^2*z +      2*a^8*b^2*c^4*y^2*z + a^6*b^4*c^4*y^2*z - 3*a^4*b^6*c^4*y^2*z - a^2*b^8*c^4*y^2*z + a^8*c^6*y^2*z +      5*a^4*b^4*c^6*y^2*z + 4*a^2*b^6*c^6*y^2*z - a^6*c^8*y^2*z - 5*a^4*b^2*c^8*y^2*z -      6*a^2*b^4*c^8*y^2*z + 2*a^4*c^10*y^2*z + 4*a^2*b^2*c^10*y^2*z - a^2*c^12*y^2*z - a^12*b^2*x*z^2 +      2*a^10*b^4*x*z^2 - a^8*b^6*x*z^2 + a^6*b^8*x*z^2 - 2*a^4*b^10*x*z^2 + a^2*b^12*x*z^2 +      4*a^10*b^2*c^2*x*z^2 - 5*a^8*b^4*c^2*x*z^2 + 2*a^4*b^8*c^2*x*z^2 - a^2*b^10*c^2*x*z^2 -      6*a^8*b^2*c^4*x*z^2 + 5*a^6*b^4*c^4*x*z^2 + a^4*b^6*c^4*x*z^2 + 4*a^6*b^2*c^6*x*z^2 -      3*a^4*b^4*c^6*x*z^2 - a^2*b^6*c^6*x*z^2 - a^4*b^2*c^8*x*z^2 + a^2*b^4*c^8*x*z^2 - a^12*b^2*y*z^2 +      2*a^10*b^4*y*z^2 - a^8*b^6*y*z^2 + a^6*b^8*y*z^2 - 2*a^4*b^10*y*z^2 + a^2*b^12*y*z^2 +      a^10*b^2*c^2*y*z^2 - 2*a^8*b^4*c^2*y*z^2 + 5*a^4*b^8*c^2*y*z^2 - 4*a^2*b^10*c^2*y*z^2 -      a^6*b^4*c^4*y*z^2 - 5*a^4*b^6*c^4*y*z^2 + 6*a^2*b^8*c^4*y*z^2 + a^6*b^2*c^6*y*z^2 +      3*a^4*b^4*c^6*y*z^2 - 4*a^2*b^6*c^6*y*z^2 - a^4*b^2*c^8*y*z^2 + a^2*b^4*c^8*y*z^2");
		exp("K205","(-a^10)*x^2*y + 4*a^8*b^2*x^2*y - 6*a^6*b^4*x^2*y + 4*a^4*b^6*x^2*y - a^2*b^8*x^2*y +      2*a^8*c^2*x^2*y - 5*a^6*b^2*c^2*x^2*y + 5*a^4*b^4*c^2*x^2*y - 3*a^2*b^6*c^2*x^2*y + b^8*c^2*x^2*y -      a^6*c^4*x^2*y + a^2*b^4*c^4*x^2*y - b^6*c^4*x^2*y + a^4*c^6*x^2*y + 2*a^2*b^2*c^6*x^2*y -      2*a^2*c^8*x^2*y - b^2*c^8*x^2*y + c^10*x^2*y + a^8*b^2*x*y^2 - 4*a^6*b^4*x*y^2 + 6*a^4*b^6*x*y^2 -      4*a^2*b^8*x*y^2 + b^10*x*y^2 - a^8*c^2*x*y^2 + 3*a^6*b^2*c^2*x*y^2 - 5*a^4*b^4*c^2*x*y^2 +      5*a^2*b^6*c^2*x*y^2 - 2*b^8*c^2*x*y^2 + a^6*c^4*x*y^2 - a^4*b^2*c^4*x*y^2 + b^6*c^4*x*y^2 -      2*a^2*b^2*c^6*x*y^2 - b^4*c^6*x*y^2 + a^2*c^8*x*y^2 + 2*b^2*c^8*x*y^2 - c^10*x*y^2 + a^10*x^2*z -      2*a^8*b^2*x^2*z + a^6*b^4*x^2*z - a^4*b^6*x^2*z + 2*a^2*b^8*x^2*z - b^10*x^2*z - 4*a^8*c^2*x^2*z +      5*a^6*b^2*c^2*x^2*z - 2*a^2*b^6*c^2*x^2*z + b^8*c^2*x^2*z + 6*a^6*c^4*x^2*z - 5*a^4*b^2*c^4*x^2*z -      a^2*b^4*c^4*x^2*z - 4*a^4*c^6*x^2*z + 3*a^2*b^2*c^6*x^2*z + b^4*c^6*x^2*z + a^2*c^8*x^2*z -      b^2*c^8*x^2*z + a^10*y^2*z - 2*a^8*b^2*y^2*z + a^6*b^4*y^2*z - a^4*b^6*y^2*z + 2*a^2*b^8*y^2*z -      b^10*y^2*z - a^8*c^2*y^2*z + 2*a^6*b^2*c^2*y^2*z - 5*a^2*b^6*c^2*y^2*z + 4*b^8*c^2*y^2*z +      a^4*b^2*c^4*y^2*z + 5*a^2*b^4*c^4*y^2*z - 6*b^6*c^4*y^2*z - a^4*c^6*y^2*z - 3*a^2*b^2*c^6*y^2*z +      4*b^4*c^6*y^2*z + a^2*c^8*y^2*z - b^2*c^8*y^2*z + a^8*b^2*x*z^2 - a^6*b^4*x*z^2 - a^2*b^8*x*z^2 +      b^10*x*z^2 - a^8*c^2*x*z^2 - 3*a^6*b^2*c^2*x*z^2 + a^4*b^4*c^2*x*z^2 + 2*a^2*b^6*c^2*x*z^2 -      2*b^8*c^2*x*z^2 + 4*a^6*c^4*x*z^2 + 5*a^4*b^2*c^4*x*z^2 + b^6*c^4*x*z^2 - 6*a^4*c^6*x*z^2 -      5*a^2*b^2*c^6*x*z^2 - b^4*c^6*x*z^2 + 4*a^2*c^8*x*z^2 + 2*b^2*c^8*x*z^2 - c^10*x*z^2 - a^10*y*z^2 +      a^8*b^2*y*z^2 + a^4*b^6*y*z^2 - a^2*b^8*y*z^2 + 2*a^8*c^2*y*z^2 - 2*a^6*b^2*c^2*y*z^2 -      a^4*b^4*c^2*y*z^2 + 3*a^2*b^6*c^2*y*z^2 + b^8*c^2*y*z^2 - a^6*c^4*y*z^2 - 5*a^2*b^4*c^4*y*z^2 -      4*b^6*c^4*y*z^2 + a^4*c^6*y*z^2 + 5*a^2*b^2*c^6*y*z^2 + 6*b^4*c^6*y*z^2 - 2*a^2*c^8*y*z^2 -      4*b^2*c^8*y*z^2 + c^10*y*z^2");
		exp("K206","(-a^2)*b*c^3*x^2*y + a*b^2*c^3*x^2*y - b^3*c^3*x^2*y + b*c^5*x^2*y + a^3*c^3*x*y^2 -      a^2*b*c^3*x*y^2 + a*b^2*c^3*x*y^2 - a*c^5*x*y^2 + a^2*b^3*c*x^2*z - b^5*c*x^2*z - a*b^3*c^2*x^2*z +      b^3*c^3*x^2*z + a^5*c*y^2*z - a^3*b^2*c*y^2*z + a^3*b*c^2*y^2*z - a^3*c^3*y^2*z - a^3*b^3*x*z^2 +      a*b^5*x*z^2 + a^2*b^3*c*x*z^2 - a*b^3*c^2*x*z^2 - a^5*b*y*z^2 + a^3*b^3*y*z^2 - a^3*b^2*c*y*z^2 +      a^3*b*c^2*y*z^2");
		exp("K207","(-a)*c^2*x^2*y - b*c^2*x^2*y - 2*c^3*x^2*y + a*c^2*x*y^2 + b*c^2*x*y^2 + 2*c^3*x*y^2 +      a*b^2*x^2*z + 2*b^3*x^2*z + b^2*c*x^2*z - a^2*b*x*y*z + a*b^2*x*y*z + a^2*c*x*y*z - b^2*c*x*y*z -      a*c^2*x*y*z + b*c^2*x*y*z - 2*a^3*y^2*z - a^2*b*y^2*z - a^2*c*y^2*z - a*b^2*x*z^2 - 2*b^3*x*z^2 -      b^2*c*x*z^2 + 2*a^3*y*z^2 + a^2*b*y*z^2 + a^2*c*y*z^2");
		exp("K208","a^12*x^2*y - 6*a^10*b^2*x^2*y + 15*a^8*b^4*x^2*y - 20*a^6*b^6*x^2*y + 15*a^4*b^8*x^2*y -      6*a^2*b^10*x^2*y + b^12*x^2*y - a^10*c^2*x^2*y + 3*a^8*b^2*c^2*x^2*y - 2*a^6*b^4*c^2*x^2*y -      2*a^4*b^6*c^2*x^2*y + 3*a^2*b^8*c^2*x^2*y - b^10*c^2*x^2*y - 2*a^8*c^4*x^2*y +      8*a^6*b^2*c^4*x^2*y - 12*a^4*b^4*c^4*x^2*y + 8*a^2*b^6*c^4*x^2*y - 2*b^8*c^4*x^2*y +      2*a^6*c^6*x^2*y - 2*a^4*b^2*c^6*x^2*y - 2*a^2*b^4*c^6*x^2*y + 2*b^6*c^6*x^2*y + a^4*c^8*x^2*y -      2*a^2*b^2*c^8*x^2*y + b^4*c^8*x^2*y - a^2*c^10*x^2*y - b^2*c^10*x^2*y - a^12*x*y^2 +      6*a^10*b^2*x*y^2 - 15*a^8*b^4*x*y^2 + 20*a^6*b^6*x*y^2 - 15*a^4*b^8*x*y^2 + 6*a^2*b^10*x*y^2 -      b^12*x*y^2 + a^10*c^2*x*y^2 - 3*a^8*b^2*c^2*x*y^2 + 2*a^6*b^4*c^2*x*y^2 + 2*a^4*b^6*c^2*x*y^2 -      3*a^2*b^8*c^2*x*y^2 + b^10*c^2*x*y^2 + 2*a^8*c^4*x*y^2 - 8*a^6*b^2*c^4*x*y^2 +      12*a^4*b^4*c^4*x*y^2 - 8*a^2*b^6*c^4*x*y^2 + 2*b^8*c^4*x*y^2 - 2*a^6*c^6*x*y^2 +      2*a^4*b^2*c^6*x*y^2 + 2*a^2*b^4*c^6*x*y^2 - 2*b^6*c^6*x*y^2 - a^4*c^8*x*y^2 + 2*a^2*b^2*c^8*x*y^2 -      b^4*c^8*x*y^2 + a^2*c^10*x*y^2 + b^2*c^10*x*y^2 - a^12*x^2*z + a^10*b^2*x^2*z + 2*a^8*b^4*x^2*z -      2*a^6*b^6*x^2*z - a^4*b^8*x^2*z + a^2*b^10*x^2*z + 6*a^10*c^2*x^2*z - 3*a^8*b^2*c^2*x^2*z -      8*a^6*b^4*c^2*x^2*z + 2*a^4*b^6*c^2*x^2*z + 2*a^2*b^8*c^2*x^2*z + b^10*c^2*x^2*z -      15*a^8*c^4*x^2*z + 2*a^6*b^2*c^4*x^2*z + 12*a^4*b^4*c^4*x^2*z + 2*a^2*b^6*c^4*x^2*z -      b^8*c^4*x^2*z + 20*a^6*c^6*x^2*z + 2*a^4*b^2*c^6*x^2*z - 8*a^2*b^4*c^6*x^2*z - 2*b^6*c^6*x^2*z -      15*a^4*c^8*x^2*z - 3*a^2*b^2*c^8*x^2*z + 2*b^4*c^8*x^2*z + 6*a^2*c^10*x^2*z + b^2*c^10*x^2*z -      c^12*x^2*z + 4*a^10*b^2*x*y*z - 8*a^8*b^4*x*y*z + 8*a^4*b^8*x*y*z - 4*a^2*b^10*x*y*z -      4*a^10*c^2*x*y*z + 20*a^6*b^4*c^2*x*y*z - 20*a^4*b^6*c^2*x*y*z + 4*b^10*c^2*x*y*z +      8*a^8*c^4*x*y*z - 20*a^6*b^2*c^4*x*y*z + 20*a^2*b^6*c^4*x*y*z - 8*b^8*c^4*x*y*z +      20*a^4*b^2*c^6*x*y*z - 20*a^2*b^4*c^6*x*y*z - 8*a^4*c^8*x*y*z + 8*b^4*c^8*x*y*z +      4*a^2*c^10*x*y*z - 4*b^2*c^10*x*y*z - a^10*b^2*y^2*z + a^8*b^4*y^2*z + 2*a^6*b^6*y^2*z -      2*a^4*b^8*y^2*z - a^2*b^10*y^2*z + b^12*y^2*z - a^10*c^2*y^2*z - 2*a^8*b^2*c^2*y^2*z -      2*a^6*b^4*c^2*y^2*z + 8*a^4*b^6*c^2*y^2*z + 3*a^2*b^8*c^2*y^2*z - 6*b^10*c^2*y^2*z +      a^8*c^4*y^2*z - 2*a^6*b^2*c^4*y^2*z - 12*a^4*b^4*c^4*y^2*z - 2*a^2*b^6*c^4*y^2*z +      15*b^8*c^4*y^2*z + 2*a^6*c^6*y^2*z + 8*a^4*b^2*c^6*y^2*z - 2*a^2*b^4*c^6*y^2*z - 20*b^6*c^6*y^2*z -      2*a^4*c^8*y^2*z + 3*a^2*b^2*c^8*y^2*z + 15*b^4*c^8*y^2*z - a^2*c^10*y^2*z - 6*b^2*c^10*y^2*z +      c^12*y^2*z + a^12*x*z^2 - a^10*b^2*x*z^2 - 2*a^8*b^4*x*z^2 + 2*a^6*b^6*x*z^2 + a^4*b^8*x*z^2 -      a^2*b^10*x*z^2 - 6*a^10*c^2*x*z^2 + 3*a^8*b^2*c^2*x*z^2 + 8*a^6*b^4*c^2*x*z^2 -      2*a^4*b^6*c^2*x*z^2 - 2*a^2*b^8*c^2*x*z^2 - b^10*c^2*x*z^2 + 15*a^8*c^4*x*z^2 -      2*a^6*b^2*c^4*x*z^2 - 12*a^4*b^4*c^4*x*z^2 - 2*a^2*b^6*c^4*x*z^2 + b^8*c^4*x*z^2 -      20*a^6*c^6*x*z^2 - 2*a^4*b^2*c^6*x*z^2 + 8*a^2*b^4*c^6*x*z^2 + 2*b^6*c^6*x*z^2 + 15*a^4*c^8*x*z^2 +      3*a^2*b^2*c^8*x*z^2 - 2*b^4*c^8*x*z^2 - 6*a^2*c^10*x*z^2 - b^2*c^10*x*z^2 + c^12*x*z^2 +      a^10*b^2*y*z^2 - a^8*b^4*y*z^2 - 2*a^6*b^6*y*z^2 + 2*a^4*b^8*y*z^2 + a^2*b^10*y*z^2 - b^12*y*z^2 +      a^10*c^2*y*z^2 + 2*a^8*b^2*c^2*y*z^2 + 2*a^6*b^4*c^2*y*z^2 - 8*a^4*b^6*c^2*y*z^2 -      3*a^2*b^8*c^2*y*z^2 + 6*b^10*c^2*y*z^2 - a^8*c^4*y*z^2 + 2*a^6*b^2*c^4*y*z^2 +      12*a^4*b^4*c^4*y*z^2 + 2*a^2*b^6*c^4*y*z^2 - 15*b^8*c^4*y*z^2 - 2*a^6*c^6*y*z^2 -      8*a^4*b^2*c^6*y*z^2 + 2*a^2*b^4*c^6*y*z^2 + 20*b^6*c^6*y*z^2 + 2*a^4*c^8*y*z^2 -      3*a^2*b^2*c^8*y*z^2 - 15*b^4*c^8*y*z^2 + a^2*c^10*y*z^2 + 6*b^2*c^10*y*z^2 - c^12*y*z^2");
		exp("K209","a^4*x^2*y - b^4*x^2*y - 3*a^2*c^2*x^2*y + b^2*c^2*x^2*y + 2*c^4*x^2*y + a^4*x*y^2 - b^4*x*y^2 -      a^2*c^2*x*y^2 + 3*b^2*c^2*x*y^2 - 2*c^4*x*y^2 - a^4*x^2*z + 3*a^2*b^2*x^2*z - 2*b^4*x^2*z -      b^2*c^2*x^2*z + c^4*x^2*z + 2*a^4*y^2*z - 3*a^2*b^2*y^2*z + b^4*y^2*z + a^2*c^2*y^2*z - c^4*y^2*z -      a^4*x*z^2 + a^2*b^2*x*z^2 + 2*b^4*x*z^2 - 3*b^2*c^2*x*z^2 + c^4*x*z^2 - 2*a^4*y*z^2 -      a^2*b^2*y*z^2 + b^4*y*z^2 + 3*a^2*c^2*y*z^2 - c^4*y*z^2");
		exp("K210","a^4*b^2*c^2*x^2*y - 2*a^2*b^4*c^2*x^2*y + b^6*c^2*x^2*y - a^4*c^4*x^2*y + a^2*b^2*c^4*x^2*y +      a^2*c^6*x^2*y - b^2*c^6*x^2*y - a^6*c^2*x*y^2 + 2*a^4*b^2*c^2*x*y^2 - a^2*b^4*c^2*x*y^2 -      a^2*b^2*c^4*x*y^2 + b^4*c^4*x*y^2 + a^2*c^6*x*y^2 - b^2*c^6*x*y^2 + a^4*b^4*x^2*z - a^2*b^6*x^2*z -      a^4*b^2*c^2*x^2*z - a^2*b^4*c^2*x^2*z + b^6*c^2*x^2*z + 2*a^2*b^2*c^4*x^2*z - b^2*c^6*x^2*z +      a^6*b^2*x*y*z - a^2*b^6*x*y*z - a^6*c^2*x*y*z + b^6*c^2*x*y*z + a^2*c^6*x*y*z - b^2*c^6*x*y*z +      a^6*b^2*y^2*z - a^4*b^4*y^2*z - a^6*c^2*y^2*z + a^4*b^2*c^2*y^2*z + a^2*b^4*c^2*y^2*z -      2*a^2*b^2*c^4*y^2*z + a^2*c^6*y^2*z + a^6*b^2*x*z^2 - a^2*b^6*x*z^2 - 2*a^4*b^2*c^2*x*z^2 +      a^2*b^4*c^2*x*z^2 + b^6*c^2*x*z^2 + a^2*b^2*c^4*x*z^2 - b^4*c^4*x*z^2 + a^6*b^2*y*z^2 -      a^2*b^6*y*z^2 - a^6*c^2*y*z^2 - a^4*b^2*c^2*y*z^2 + 2*a^2*b^4*c^2*y*z^2 + a^4*c^4*y*z^2 -      a^2*b^2*c^4*y*z^2");
		exp("K211","-(a^4*c^2*x^2*y) + a^2*b^2*c^2*x^2*y + a^2*c^4*x^2*y + a^2*b^2*c^2*x*y^2 - b^4*c^2*x*y^2 +      b^2*c^4*x*y^2 - a^4*b^2*x^2*z + a^2*b^4*x^2*z + a^2*b^2*c^2*x^2*z + 2*a^2*b^2*c^2*x*y*z +      a^4*b^2*y^2*z - a^2*b^4*y^2*z + a^2*b^2*c^2*y^2*z + a^2*b^2*c^2*x*z^2 + b^4*c^2*x*z^2 -      b^2*c^4*x*z^2 + a^4*c^2*y*z^2 + a^2*b^2*c^2*y*z^2 - a^2*c^4*y*z^2");
		exp("K212","a^4*c^2*x^2*y - 2*a^2*b^2*c^2*x^2*y + b^4*c^2*x^2*y - a^2*c^4*x^2*y + b^2*c^4*x^2*y +      a^4*c^2*x*y^2 - 2*a^2*b^2*c^2*x*y^2 + b^4*c^2*x*y^2 + a^2*c^4*x*y^2 - b^2*c^4*x*y^2 +      a^4*b^2*x^2*z - a^2*b^4*x^2*z - 2*a^2*b^2*c^2*x^2*z + b^4*c^2*x^2*z + b^2*c^4*x^2*z -      2*a^2*b^2*c^2*x*y*z - a^4*b^2*y^2*z + a^2*b^4*y^2*z + a^4*c^2*y^2*z - 2*a^2*b^2*c^2*y^2*z +      a^2*c^4*y^2*z + a^4*b^2*x*z^2 + a^2*b^4*x*z^2 - 2*a^2*b^2*c^2*x*z^2 - b^4*c^2*x*z^2 +      b^2*c^4*x*z^2 + a^4*b^2*y*z^2 + a^2*b^4*y*z^2 - a^4*c^2*y*z^2 - 2*a^2*b^2*c^2*y*z^2 +      a^2*c^4*y*z^2");
		exp("K213","2*a^2*b^2*x^2*y - 2*b^4*x^2*y + b^2*c^2*x^2*y + 2*c^4*x^2*y + 2*a^4*x*y^2 - 2*a^2*b^2*x*y^2 -      a^2*c^2*x*y^2 - 2*c^4*x*y^2 - 2*b^4*x^2*z - 2*a^2*c^2*x^2*z - b^2*c^2*x^2*z + 2*c^4*x^2*z +      2*a^4*y^2*z + a^2*c^2*y^2*z + 2*b^2*c^2*y^2*z - 2*c^4*y^2*z - 2*a^4*x*z^2 + a^2*b^2*x*z^2 +      2*b^4*x*z^2 + 2*a^2*c^2*x*z^2 - 2*a^4*y*z^2 - a^2*b^2*y*z^2 + 2*b^4*y*z^2 - 2*b^2*c^2*y*z^2");
		exp("K214","b^2*c^4*x^2*y + a^2*c^4*x*y^2 + b^4*c^2*x^2*z + a^4*c^2*y^2*z + a^2*b^4*x*z^2 + a^4*b^2*y*z^2 +      (-a^6 + a^4*b^2 + a^2*b^4 - b^6 + a^4*c^2 + b^4*c^2 + a^2*c^4 + b^2*c^4 - c^6)*x*y*z");
		exp("K215","b^4*c^4*x^3 - a^2*b^2*c^4*x^2*y - a^2*b^2*c^4*x*y^2 + a^4*c^4*y^3 - a^2*b^4*c^2*x^2*z +      a^4*b^2*c^2*x*y*z + a^2*b^4*c^2*x*y*z + a^2*b^2*c^4*x*y*z - a^4*b^2*c^2*y^2*z - a^2*b^4*c^2*x*z^2 -      a^4*b^2*c^2*y*z^2 + a^4*b^4*z^3");
		exp("K216","-(a^4*c^2*x^2*y) + a^2*b^2*c^2*x^2*y + 2*a^2*c^4*x^2*y + b^2*c^4*x^2*y - c^6*x^2*y +      a^2*b^2*c^2*x*y^2 - b^4*c^2*x*y^2 + a^2*c^4*x*y^2 + 2*b^2*c^4*x*y^2 - c^6*x*y^2 - a^4*b^2*x^2*z +      2*a^2*b^4*x^2*z - b^6*x^2*z + a^2*b^2*c^2*x^2*z + b^4*c^2*x^2*z - 2*a^6*x*y*z + 2*a^4*b^2*x*y*z +      2*a^2*b^4*x*y*z - 2*b^6*x*y*z + 2*a^4*c^2*x*y*z - 2*a^2*b^2*c^2*x*y*z + 2*b^4*c^2*x*y*z +      2*a^2*c^4*x*y*z + 2*b^2*c^4*x*y*z - 2*c^6*x*y*z - a^6*y^2*z + 2*a^4*b^2*y^2*z - a^2*b^4*y^2*z +      a^4*c^2*y^2*z + a^2*b^2*c^2*y^2*z + a^2*b^4*x*z^2 - b^6*x*z^2 + a^2*b^2*c^2*x*z^2 +      2*b^4*c^2*x*z^2 - b^2*c^4*x*z^2 - a^6*y*z^2 + a^4*b^2*y*z^2 + 2*a^4*c^2*y*z^2 + a^2*b^2*c^2*y*z^2 -      a^2*c^4*y*z^2");
		exp("K217","(-a^12)*x^2*y + 3*a^10*b^2*x^2*y - 5*a^8*b^4*x^2*y + 7*a^6*b^6*x^2*y - 6*a^4*b^8*x^2*y +      2*a^2*b^10*x^2*y + 2*a^10*c^2*x^2*y - 5*a^8*b^2*c^2*x^2*y + 4*a^6*b^4*c^2*x^2*y -      3*a^4*b^6*c^2*x^2*y + 4*a^2*b^8*c^2*x^2*y - 2*b^10*c^2*x^2*y + a^6*b^2*c^4*x^2*y -      3*a^2*b^6*c^4*x^2*y + 2*b^8*c^4*x^2*y - 2*a^6*c^6*x^2*y + 3*a^4*b^2*c^6*x^2*y - b^6*c^6*x^2*y +      a^4*c^8*x^2*y - 2*a^2*b^2*c^8*x^2*y + b^4*c^8*x^2*y - 2*a^10*b^2*x*y^2 + 6*a^8*b^4*x*y^2 -      7*a^6*b^6*x*y^2 + 5*a^4*b^8*x*y^2 - 3*a^2*b^10*x*y^2 + b^12*x*y^2 + 2*a^10*c^2*x*y^2 -      4*a^8*b^2*c^2*x*y^2 + 3*a^6*b^4*c^2*x*y^2 - 4*a^4*b^6*c^2*x*y^2 + 5*a^2*b^8*c^2*x*y^2 -      2*b^10*c^2*x*y^2 - 2*a^8*c^4*x*y^2 + 3*a^6*b^2*c^4*x*y^2 - a^2*b^6*c^4*x*y^2 + a^6*c^6*x*y^2 -      3*a^2*b^4*c^6*x*y^2 + 2*b^6*c^6*x*y^2 - a^4*c^8*x*y^2 + 2*a^2*b^2*c^8*x*y^2 - b^4*c^8*x*y^2 +      a^12*x^2*z - 2*a^10*b^2*x^2*z + 2*a^6*b^6*x^2*z - a^4*b^8*x^2*z - 3*a^10*c^2*x^2*z +      5*a^8*b^2*c^2*x^2*z - a^6*b^4*c^2*x^2*z - 3*a^4*b^6*c^2*x^2*z + 2*a^2*b^8*c^2*x^2*z +      5*a^8*c^4*x^2*z - 4*a^6*b^2*c^4*x^2*z - b^8*c^4*x^2*z - 7*a^6*c^6*x^2*z + 3*a^4*b^2*c^6*x^2*z +      3*a^2*b^4*c^6*x^2*z + b^6*c^6*x^2*z + 6*a^4*c^8*x^2*z - 4*a^2*b^2*c^8*x^2*z - 2*b^4*c^8*x^2*z -      2*a^2*c^10*x^2*z + 2*b^2*c^10*x^2*z + a^8*b^4*y^2*z - 2*a^6*b^6*y^2*z + 2*a^2*b^10*y^2*z -      b^12*y^2*z - 2*a^8*b^2*c^2*y^2*z + 3*a^6*b^4*c^2*y^2*z + a^4*b^6*c^2*y^2*z - 5*a^2*b^8*c^2*y^2*z +      3*b^10*c^2*y^2*z + a^8*c^4*y^2*z + 4*a^2*b^6*c^4*y^2*z - 5*b^8*c^4*y^2*z - a^6*c^6*y^2*z -      3*a^4*b^2*c^6*y^2*z - 3*a^2*b^4*c^6*y^2*z + 7*b^6*c^6*y^2*z + 2*a^4*c^8*y^2*z +      4*a^2*b^2*c^8*y^2*z - 6*b^4*c^8*y^2*z - 2*a^2*c^10*y^2*z + 2*b^2*c^10*y^2*z - 2*a^10*b^2*x*z^2 +      2*a^8*b^4*x*z^2 - a^6*b^6*x*z^2 + a^4*b^8*x*z^2 + 2*a^10*c^2*x*z^2 + 4*a^8*b^2*c^2*x*z^2 -      3*a^6*b^4*c^2*x*z^2 - 2*a^2*b^8*c^2*x*z^2 - 6*a^8*c^4*x*z^2 - 3*a^6*b^2*c^4*x*z^2 +      3*a^2*b^6*c^4*x*z^2 + b^8*c^4*x*z^2 + 7*a^6*c^6*x*z^2 + 4*a^4*b^2*c^6*x*z^2 + a^2*b^4*c^6*x*z^2 -      2*b^6*c^6*x*z^2 - 5*a^4*c^8*x*z^2 - 5*a^2*b^2*c^8*x*z^2 + 3*a^2*c^10*x*z^2 + 2*b^2*c^10*x*z^2 -      c^12*x*z^2 - a^8*b^4*y*z^2 + a^6*b^6*y*z^2 - 2*a^4*b^8*y*z^2 + 2*a^2*b^10*y*z^2 +      2*a^8*b^2*c^2*y*z^2 + 3*a^4*b^6*c^2*y*z^2 - 4*a^2*b^8*c^2*y*z^2 - 2*b^10*c^2*y*z^2 -      a^8*c^4*y*z^2 - 3*a^6*b^2*c^4*y*z^2 + 3*a^2*b^6*c^4*y*z^2 + 6*b^8*c^4*y*z^2 + 2*a^6*c^6*y*z^2 -      a^4*b^2*c^6*y*z^2 - 4*a^2*b^4*c^6*y*z^2 - 7*b^6*c^6*y*z^2 + 5*a^2*b^2*c^8*y*z^2 + 5*b^4*c^8*y*z^2 -      2*a^2*c^10*y*z^2 - 3*b^2*c^10*y*z^2 + c^12*y*z^2");
		exp("K218","(-a^10)*x^2*y + 4*a^8*b^2*x^2*y - 5*a^6*b^4*x^2*y + 2*a^4*b^6*x^2*y + a^8*c^2*x^2*y -      6*a^6*b^2*c^2*x^2*y + 9*a^4*b^4*c^2*x^2*y - 4*a^2*b^6*c^2*x^2*y + a^6*c^4*x^2*y -      3*a^2*b^4*c^4*x^2*y + 2*b^6*c^4*x^2*y - a^4*c^6*x^2*y + 2*a^2*b^2*c^6*x^2*y - b^4*c^6*x^2*y +      2*a^6*b^4*x*y^2 - 5*a^4*b^6*x*y^2 + 4*a^2*b^8*x*y^2 - b^10*x*y^2 - 4*a^6*b^2*c^2*x*y^2 +      9*a^4*b^4*c^2*x*y^2 - 6*a^2*b^6*c^2*x*y^2 + b^8*c^2*x*y^2 + 2*a^6*c^4*x*y^2 - 3*a^4*b^2*c^4*x*y^2 +      b^6*c^4*x*y^2 - a^4*c^6*x*y^2 + 2*a^2*b^2*c^6*x*y^2 - b^4*c^6*x*y^2 - a^10*x^2*z + a^8*b^2*x^2*z +      a^6*b^4*x^2*z - a^4*b^6*x^2*z + 4*a^8*c^2*x^2*z - 6*a^6*b^2*c^2*x^2*z + 2*a^2*b^6*c^2*x^2*z -      5*a^6*c^4*x^2*z + 9*a^4*b^2*c^4*x^2*z - 3*a^2*b^4*c^4*x^2*z - b^6*c^4*x^2*z + 2*a^4*c^6*x^2*z -      4*a^2*b^2*c^6*x^2*z + 2*b^4*c^6*x^2*z - a^6*b^4*y^2*z + a^4*b^6*y^2*z + a^2*b^8*y^2*z -      b^10*y^2*z + 2*a^6*b^2*c^2*y^2*z - 6*a^2*b^6*c^2*y^2*z + 4*b^8*c^2*y^2*z - a^6*c^4*y^2*z -      3*a^4*b^2*c^4*y^2*z + 9*a^2*b^4*c^4*y^2*z - 5*b^6*c^4*y^2*z + 2*a^4*c^6*y^2*z -      4*a^2*b^2*c^6*y^2*z + 2*b^4*c^6*y^2*z + 2*a^6*b^4*x*z^2 - a^4*b^6*x*z^2 - 4*a^6*b^2*c^2*x*z^2 -      3*a^4*b^4*c^2*x*z^2 + 2*a^2*b^6*c^2*x*z^2 + 2*a^6*c^4*x*z^2 + 9*a^4*b^2*c^4*x*z^2 - b^6*c^4*x*z^2 -      5*a^4*c^6*x*z^2 - 6*a^2*b^2*c^6*x*z^2 + b^4*c^6*x*z^2 + 4*a^2*c^8*x*z^2 + b^2*c^8*x*z^2 -      c^10*x*z^2 - a^6*b^4*y*z^2 + 2*a^4*b^6*y*z^2 + 2*a^6*b^2*c^2*y*z^2 - 3*a^4*b^4*c^2*y*z^2 -      4*a^2*b^6*c^2*y*z^2 - a^6*c^4*y*z^2 + 9*a^2*b^4*c^4*y*z^2 + 2*b^6*c^4*y*z^2 + a^4*c^6*y*z^2 -      6*a^2*b^2*c^6*y*z^2 - 5*b^4*c^6*y*z^2 + a^2*c^8*y*z^2 + 4*b^2*c^8*y*z^2 - c^10*y*z^2");
		exp("K219","-x^3 + x^2*y + x*y^2 - y^3 + x^2*z - 3*x*y*z + y^2*z + x*z^2 + y*z^2 - z^3");
		exp("K220","a^2*(-a + b + c)*y*(y - z)*z + b^2*(a - b + c)*x*z*(-x + z) + (a + b - c)*c^2*x*(x - y)*y +      2*(b - c)*(c - a)*(a - b)*x*y*z");
		exp("K221","(-a^2)*b*c^2*x^2*y + b*c^4*x^2*y + a*b^2*c^2*x*y^2 - a*c^4*x*y^2 + a^2*b^2*c*x^2*z -      b^4*c*x^2*z + a^4*c*y^2*z - a^2*b^2*c*y^2*z + a*b^4*x*z^2 - a*b^2*c^2*x*z^2 - a^4*b*y*z^2 +      a^2*b*c^2*y*z^2");
		exp("K222","(-a^2)*b^2*c^4*x^2*y + b^2*c^6*x^2*y + a^2*b^2*c^4*x*y^2 - a^2*c^6*x*y^2 + a^2*b^4*c^2*x^2*z -      b^6*c^2*x^2*z + a^6*c^2*y^2*z - a^4*b^2*c^2*y^2*z + a^2*b^6*x*z^2 - a^2*b^4*c^2*x*z^2 -      a^6*b^2*y*z^2 + a^4*b^2*c^2*y*z^2");
		exp("K223","(-a^4)*b^2*c^4*x^2*y + a^2*b^4*c^4*x^2*y - b^4*c^6*x^2*y + b^2*c^8*x^2*y - a^4*b^2*c^4*x*y^2 +      a^2*b^4*c^4*x*y^2 + a^4*c^6*x*y^2 - a^2*c^8*x*y^2 + a^4*b^4*c^2*x^2*z - b^8*c^2*x^2*z -      a^2*b^4*c^4*x^2*z + b^6*c^4*x^2*z + a^8*c^2*y^2*z - a^4*b^4*c^2*y^2*z - a^6*c^4*y^2*z +      a^4*b^2*c^4*y^2*z - a^4*b^6*x*z^2 + a^2*b^8*x*z^2 + a^4*b^4*c^2*x*z^2 - a^2*b^4*c^4*x*z^2 -      a^8*b^2*y*z^2 + a^6*b^4*y*z^2 - a^4*b^4*c^2*y*z^2 + a^4*b^2*c^4*y*z^2");
		exp("K224","(-a)*b^2*c^4*x^2*y + b^2*c^5*x^2*y + a^2*b*c^4*x*y^2 - a^2*c^5*x*y^2 + a*b^4*c^2*x^2*z -      b^5*c^2*x^2*z + a^5*c^2*y^2*z - a^4*b*c^2*y^2*z + a^2*b^5*x*z^2 - a^2*b^4*c*x*z^2 - a^5*b^2*y*z^2 +      a^4*b^2*c*y*z^2");
		exp("K225","(-a^2)*b^2*c^4*x^2*y + a*b^3*c^4*x^2*y - b^3*c^5*x^2*y + b^2*c^6*x^2*y - a^3*b*c^4*x*y^2 +      a^2*b^2*c^4*x*y^2 + a^3*c^5*x*y^2 - a^2*c^6*x*y^2 + a^2*b^4*c^2*x^2*z - b^6*c^2*x^2*z -      a*b^4*c^3*x^2*z + b^5*c^3*x^2*z + a^6*c^2*y^2*z - a^4*b^2*c^2*y^2*z - a^5*c^3*y^2*z +      a^4*b*c^3*y^2*z - a^3*b^5*x*z^2 + a^2*b^6*x*z^2 + a^3*b^4*c*x*z^2 - a^2*b^4*c^2*x*z^2 -      a^6*b^2*y*z^2 + a^5*b^3*y*z^2 - a^4*b^3*c*y*z^2 + a^4*b^2*c^2*y*z^2");
		exp("K226","2*a^4*b^2*c^2*x^3 - 2*a^2*b^4*c^2*x^3 - 2*a^2*b^2*c^4*x^3 + a^6*c^2*x^2*y - a^4*b^2*c^2*x^2*y +      a^2*b^4*c^2*x^2*y - b^6*c^2*x^2*y - 2*a^4*c^4*x^2*y + 2*a^2*b^2*c^4*x^2*y + 2*b^4*c^4*x^2*y +      a^2*c^6*x^2*y - b^2*c^6*x^2*y - a^6*c^2*x*y^2 + a^4*b^2*c^2*x*y^2 - a^2*b^4*c^2*x*y^2 +      b^6*c^2*x*y^2 + 2*a^4*c^4*x*y^2 + 2*a^2*b^2*c^4*x*y^2 - 2*b^4*c^4*x*y^2 - a^2*c^6*x*y^2 +      b^2*c^6*x*y^2 - 2*a^4*b^2*c^2*y^3 + 2*a^2*b^4*c^2*y^3 - 2*a^2*b^2*c^4*y^3 + a^6*b^2*x^2*z -      2*a^4*b^4*x^2*z + a^2*b^6*x^2*z - a^4*b^2*c^2*x^2*z + 2*a^2*b^4*c^2*x^2*z - b^6*c^2*x^2*z +      a^2*b^2*c^4*x^2*z + 2*b^4*c^4*x^2*z - b^2*c^6*x^2*z + 2*a^6*b^2*x*y*z - 4*a^4*b^4*x*y*z +      2*a^2*b^6*x*y*z + 2*a^6*c^2*x*y*z - 2*a^4*b^2*c^2*x*y*z - 2*a^2*b^4*c^2*x*y*z + 2*b^6*c^2*x*y*z -      4*a^4*c^4*x*y*z - 2*a^2*b^2*c^4*x*y*z - 4*b^4*c^4*x*y*z + 2*a^2*c^6*x*y*z + 2*b^2*c^6*x*y*z +      a^6*b^2*y^2*z - 2*a^4*b^4*y^2*z + a^2*b^6*y^2*z - a^6*c^2*y^2*z + 2*a^4*b^2*c^2*y^2*z -      a^2*b^4*c^2*y^2*z + 2*a^4*c^4*y^2*z + a^2*b^2*c^4*y^2*z - a^2*c^6*y^2*z - a^6*b^2*x*z^2 +      2*a^4*b^4*x*z^2 - a^2*b^6*x*z^2 + a^4*b^2*c^2*x*z^2 + 2*a^2*b^4*c^2*x*z^2 + b^6*c^2*x*z^2 -      a^2*b^2*c^4*x*z^2 - 2*b^4*c^4*x*z^2 + b^2*c^6*x*z^2 - a^6*b^2*y*z^2 + 2*a^4*b^4*y*z^2 -      a^2*b^6*y*z^2 + a^6*c^2*y*z^2 + 2*a^4*b^2*c^2*y*z^2 + a^2*b^4*c^2*y*z^2 - 2*a^4*c^4*y*z^2 -      a^2*b^2*c^4*y*z^2 + a^2*c^6*y*z^2 - 2*a^4*b^2*c^2*z^3 - 2*a^2*b^4*c^2*z^3 + 2*a^2*b^2*c^4*z^3");
		exp("K227A","a*c^2*x*y^2*Csc(A/3 - B/3 + Pi/6)*Csc(A/3 - C/3 + Pi/6) - a*b^2*x*z^2*Csc(A/3 - B/3 + Pi/6)*Csc(A/3 - C/3 + Pi/6) + b*c^2*x^2*y*Csc(A/3 - B/3 + Pi/6)*Sec(B/3 - C/3) -      a^2*b*y*z^2*Csc(A/3 - B/3 + Pi/6)*Sec(B/3 - C/3) - b^2*c*x^2*z*Csc(A/3 - C/3 + Pi/6)*Sec(B/3 - C/3) + a^2*c*y^2*z*Csc(A/3 - C/3 + Pi/6)*Sec(B/3 - C/3)");
		exp("K227B","(-b^2)*c*x^2*z*Csc(B/3 - C/3 + Pi/6) + a^2*c*y^2*z*Csc(B/3 - C/3 + Pi/6) + b*c^2*x^2*y*Csc(A/3 - B/3 - Pi/6)*Csc(B/3 - C/3 + Pi/6) -      a^2*b*y*z^2*Csc(A/3 - B/3 - Pi/6)*Csc(B/3 - C/3 + Pi/6) + a*c^2*x*y^2*Csc(A/3 - B/3 - Pi/6)*Sec(A/3 - C/3) - a*b^2*x*z^2*Csc(A/3 - B/3 - Pi/6)*Sec(A/3 - C/3)");
		exp("K227C","b^2*c*x^2*z*Csc(A/3 - C/3 - Pi/6)*Csc(B/3 - C/3 - Pi/6) - a^2*c*y^2*z*Csc(A/3 - C/3 - Pi/6)*Csc(B/3 - C/3 - Pi/6) + a*c^2*x*y^2*Csc(A/3 - C/3 - Pi/6)*Sec(A/3 - B/3) -      a*b^2*x*z^2*Csc(A/3 - C/3 - Pi/6)*Sec(A/3 - B/3) - b*c^2*x^2*y*Csc(B/3 - C/3 - Pi/6)*Sec(A/3 - B/3) + a^2*b*y*z^2*Csc(B/3 - C/3 - Pi/6)*Sec(A/3 - B/3)");
		exp("K228","a*x*(c*y - b*z)^2 + b*y*((-c)*x + a*z)^2 + c*(b*x - a*y)^2*z");
		exp("K229","a^2*x*(c^2*y - b^2*z)^2 + b^2*y*((-c^2)*x + a^2*z)^2 + c^2*(b^2*x - a^2*y)^2*z");
		exp("K230","a^10*c*x^2*y - 2*a^9*b*c*x^2*y - 4*a^8*b^2*c*x^2*y + 8*a^7*b^3*c*x^2*y + 6*a^6*b^4*c*x^2*y -      12*a^5*b^5*c*x^2*y - 4*a^4*b^6*c*x^2*y + 8*a^3*b^7*c*x^2*y + a^2*b^8*c*x^2*y - 2*a*b^9*c*x^2*y -      2*a^9*c^2*x^2*y + 8*a^8*b*c^2*x^2*y + a^7*b^2*c^2*x^2*y - 24*a^6*b^3*c^2*x^2*y +      9*a^5*b^4*c^2*x^2*y + 24*a^4*b^5*c^2*x^2*y - 13*a^3*b^6*c^2*x^2*y - 8*a^2*b^7*c^2*x^2*y +      5*a*b^8*c^2*x^2*y - a^8*c^3*x^2*y - 8*a^7*b*c^3*x^2*y + 18*a^6*b^2*c^3*x^2*y +      12*a^5*b^3*c^3*x^2*y - 33*a^4*b^4*c^3*x^2*y + 16*a^2*b^6*c^3*x^2*y - 4*a*b^7*c^3*x^2*y +      5*a^7*c^4*x^2*y - 6*a^6*b*c^4*x^2*y - 24*a^5*b^2*c^4*x^2*y + 22*a^4*b^3*c^4*x^2*y +      22*a^3*b^4*c^4*x^2*y - 16*a^2*b^5*c^4*x^2*y - 3*a*b^6*c^4*x^2*y - 3*a^6*c^5*x^2*y +      18*a^5*b*c^5*x^2*y + 3*a^4*b^2*c^5*x^2*y - 32*a^3*b^3*c^5*x^2*y + 2*a^2*b^4*c^5*x^2*y +      10*a*b^5*c^5*x^2*y - 3*a^5*c^6*x^2*y - 12*a^4*b*c^6*x^2*y + 18*a^3*b^2*c^6*x^2*y +      14*a^2*b^3*c^6*x^2*y - 8*a*b^4*c^6*x^2*y + 5*a^4*c^7*x^2*y - 4*a^3*b*c^7*x^2*y -      17*a^2*b^2*c^7*x^2*y - a^3*c^8*x^2*y + 10*a^2*b*c^8*x^2*y + 5*a*b^2*c^8*x^2*y - 2*a^2*c^9*x^2*y -      4*a*b*c^9*x^2*y + a*c^10*x^2*y - 2*a^9*b*c*x*y^2 + a^8*b^2*c*x*y^2 + 8*a^7*b^3*c*x*y^2 -      4*a^6*b^4*c*x*y^2 - 12*a^5*b^5*c*x*y^2 + 6*a^4*b^6*c*x*y^2 + 8*a^3*b^7*c*x*y^2 -      4*a^2*b^8*c*x*y^2 - 2*a*b^9*c*x*y^2 + b^10*c*x*y^2 + 5*a^8*b*c^2*x*y^2 - 8*a^7*b^2*c^2*x*y^2 -      13*a^6*b^3*c^2*x*y^2 + 24*a^5*b^4*c^2*x*y^2 + 9*a^4*b^5*c^2*x*y^2 - 24*a^3*b^6*c^2*x*y^2 +      a^2*b^7*c^2*x*y^2 + 8*a*b^8*c^2*x*y^2 - 2*b^9*c^2*x*y^2 - 4*a^7*b*c^3*x*y^2 +      16*a^6*b^2*c^3*x*y^2 - 33*a^4*b^4*c^3*x*y^2 + 12*a^3*b^5*c^3*x*y^2 + 18*a^2*b^6*c^3*x*y^2 -      8*a*b^7*c^3*x*y^2 - b^8*c^3*x*y^2 - 3*a^6*b*c^4*x*y^2 - 16*a^5*b^2*c^4*x*y^2 +      22*a^4*b^3*c^4*x*y^2 + 22*a^3*b^4*c^4*x*y^2 - 24*a^2*b^5*c^4*x*y^2 - 6*a*b^6*c^4*x*y^2 +      5*b^7*c^4*x*y^2 + 10*a^5*b*c^5*x*y^2 + 2*a^4*b^2*c^5*x*y^2 - 32*a^3*b^3*c^5*x*y^2 +      3*a^2*b^4*c^5*x*y^2 + 18*a*b^5*c^5*x*y^2 - 3*b^6*c^5*x*y^2 - 8*a^4*b*c^6*x*y^2 +      14*a^3*b^2*c^6*x*y^2 + 18*a^2*b^3*c^6*x*y^2 - 12*a*b^4*c^6*x*y^2 - 3*b^5*c^6*x*y^2 -      17*a^2*b^2*c^7*x*y^2 - 4*a*b^3*c^7*x*y^2 + 5*b^4*c^7*x*y^2 + 5*a^2*b*c^8*x*y^2 +      10*a*b^2*c^8*x*y^2 - b^3*c^8*x*y^2 - 4*a*b*c^9*x*y^2 - 2*b^2*c^9*x*y^2 + b*c^10*x*y^2 +      a^10*b*x^2*z - 2*a^9*b^2*x^2*z - a^8*b^3*x^2*z + 5*a^7*b^4*x^2*z - 3*a^6*b^5*x^2*z -      3*a^5*b^6*x^2*z + 5*a^4*b^7*x^2*z - a^3*b^8*x^2*z - 2*a^2*b^9*x^2*z + a*b^10*x^2*z -      2*a^9*b*c*x^2*z + 8*a^8*b^2*c*x^2*z - 8*a^7*b^3*c*x^2*z - 6*a^6*b^4*c*x^2*z + 18*a^5*b^5*c*x^2*z -      12*a^4*b^6*c*x^2*z - 4*a^3*b^7*c*x^2*z + 10*a^2*b^8*c*x^2*z - 4*a*b^9*c*x^2*z - 4*a^8*b*c^2*x^2*z +      a^7*b^2*c^2*x^2*z + 18*a^6*b^3*c^2*x^2*z - 24*a^5*b^4*c^2*x^2*z + 3*a^4*b^5*c^2*x^2*z +      18*a^3*b^6*c^2*x^2*z - 17*a^2*b^7*c^2*x^2*z + 5*a*b^8*c^2*x^2*z + 8*a^7*b*c^3*x^2*z -      24*a^6*b^2*c^3*x^2*z + 12*a^5*b^3*c^3*x^2*z + 22*a^4*b^4*c^3*x^2*z - 32*a^3*b^5*c^3*x^2*z +      14*a^2*b^6*c^3*x^2*z + 6*a^6*b*c^4*x^2*z + 9*a^5*b^2*c^4*x^2*z - 33*a^4*b^3*c^4*x^2*z +      22*a^3*b^4*c^4*x^2*z + 2*a^2*b^5*c^4*x^2*z - 8*a*b^6*c^4*x^2*z - 12*a^5*b*c^5*x^2*z +      24*a^4*b^2*c^5*x^2*z - 16*a^2*b^4*c^5*x^2*z + 10*a*b^5*c^5*x^2*z - 4*a^4*b*c^6*x^2*z -      13*a^3*b^2*c^6*x^2*z + 16*a^2*b^3*c^6*x^2*z - 3*a*b^4*c^6*x^2*z + 8*a^3*b*c^7*x^2*z -      8*a^2*b^2*c^7*x^2*z - 4*a*b^3*c^7*x^2*z + a^2*b*c^8*x^2*z + 5*a*b^2*c^8*x^2*z - 2*a*b*c^9*x^2*z +      2*a^10*b*x*y*z - 4*a^9*b^2*x*y*z - 2*a^8*b^3*x*y*z + 10*a^7*b^4*x*y*z - 6*a^6*b^5*x*y*z -      6*a^5*b^6*x*y*z + 10*a^4*b^7*x*y*z - 2*a^3*b^8*x*y*z - 4*a^2*b^9*x*y*z + 2*a*b^10*x*y*z +      2*a^10*c*x*y*z - 8*a^9*b*c*x*y*z + 12*a^8*b^2*c*x*y*z - 4*a^7*b^3*c*x*y*z - 14*a^6*b^4*c*x*y*z +      24*a^5*b^5*c*x*y*z - 14*a^4*b^6*c*x*y*z - 4*a^3*b^7*c*x*y*z + 12*a^2*b^8*c*x*y*z -      8*a*b^9*c*x*y*z + 2*b^10*c*x*y*z - 4*a^9*c^2*x*y*z + 12*a^8*b*c^2*x*y*z - 12*a^7*b^2*c^2*x*y*z +      10*a^6*b^3*c^2*x*y*z - 6*a^5*b^4*c^2*x*y*z - 6*a^4*b^5*c^2*x*y*z + 10*a^3*b^6*c^2*x*y*z -      12*a^2*b^7*c^2*x*y*z + 12*a*b^8*c^2*x*y*z - 4*b^9*c^2*x*y*z - 2*a^8*c^3*x*y*z - 4*a^7*b*c^3*x*y*z +      10*a^6*b^2*c^3*x*y*z - 8*a^5*b^3*c^3*x*y*z + 8*a^4*b^4*c^3*x*y*z - 8*a^3*b^5*c^3*x*y*z +      10*a^2*b^6*c^3*x*y*z - 4*a*b^7*c^3*x*y*z - 2*b^8*c^3*x*y*z + 10*a^7*c^4*x*y*z -      14*a^6*b*c^4*x*y*z - 6*a^5*b^2*c^4*x*y*z + 8*a^4*b^3*c^4*x*y*z + 8*a^3*b^4*c^4*x*y*z -      6*a^2*b^5*c^4*x*y*z - 14*a*b^6*c^4*x*y*z + 10*b^7*c^4*x*y*z - 6*a^6*c^5*x*y*z +      24*a^5*b*c^5*x*y*z - 6*a^4*b^2*c^5*x*y*z - 8*a^3*b^3*c^5*x*y*z - 6*a^2*b^4*c^5*x*y*z +      24*a*b^5*c^5*x*y*z - 6*b^6*c^5*x*y*z - 6*a^5*c^6*x*y*z - 14*a^4*b*c^6*x*y*z +      10*a^3*b^2*c^6*x*y*z + 10*a^2*b^3*c^6*x*y*z - 14*a*b^4*c^6*x*y*z - 6*b^5*c^6*x*y*z +      10*a^4*c^7*x*y*z - 4*a^3*b*c^7*x*y*z - 12*a^2*b^2*c^7*x*y*z - 4*a*b^3*c^7*x*y*z +      10*b^4*c^7*x*y*z - 2*a^3*c^8*x*y*z + 12*a^2*b*c^8*x*y*z + 12*a*b^2*c^8*x*y*z - 2*b^3*c^8*x*y*z -      4*a^2*c^9*x*y*z - 8*a*b*c^9*x*y*z - 4*b^2*c^9*x*y*z + 2*a*c^10*x*y*z + 2*b*c^10*x*y*z +      a^10*b*y^2*z - 2*a^9*b^2*y^2*z - a^8*b^3*y^2*z + 5*a^7*b^4*y^2*z - 3*a^6*b^5*y^2*z -      3*a^5*b^6*y^2*z + 5*a^4*b^7*y^2*z - a^3*b^8*y^2*z - 2*a^2*b^9*y^2*z + a*b^10*y^2*z -      4*a^9*b*c*y^2*z + 10*a^8*b^2*c*y^2*z - 4*a^7*b^3*c*y^2*z - 12*a^6*b^4*c*y^2*z +      18*a^5*b^5*c*y^2*z - 6*a^4*b^6*c*y^2*z - 8*a^3*b^7*c*y^2*z + 8*a^2*b^8*c*y^2*z - 2*a*b^9*c*y^2*z +      5*a^8*b*c^2*y^2*z - 17*a^7*b^2*c^2*y^2*z + 18*a^6*b^3*c^2*y^2*z + 3*a^5*b^4*c^2*y^2*z -      24*a^4*b^5*c^2*y^2*z + 18*a^3*b^6*c^2*y^2*z + a^2*b^7*c^2*y^2*z - 4*a*b^8*c^2*y^2*z +      14*a^6*b^2*c^3*y^2*z - 32*a^5*b^3*c^3*y^2*z + 22*a^4*b^4*c^3*y^2*z + 12*a^3*b^5*c^3*y^2*z -      24*a^2*b^6*c^3*y^2*z + 8*a*b^7*c^3*y^2*z - 8*a^6*b*c^4*y^2*z + 2*a^5*b^2*c^4*y^2*z +      22*a^4*b^3*c^4*y^2*z - 33*a^3*b^4*c^4*y^2*z + 9*a^2*b^5*c^4*y^2*z + 6*a*b^6*c^4*y^2*z +      10*a^5*b*c^5*y^2*z - 16*a^4*b^2*c^5*y^2*z + 24*a^2*b^4*c^5*y^2*z - 12*a*b^5*c^5*y^2*z -      3*a^4*b*c^6*y^2*z + 16*a^3*b^2*c^6*y^2*z - 13*a^2*b^3*c^6*y^2*z - 4*a*b^4*c^6*y^2*z -      4*a^3*b*c^7*y^2*z - 8*a^2*b^2*c^7*y^2*z + 8*a*b^3*c^7*y^2*z + 5*a^2*b*c^8*y^2*z + a*b^2*c^8*y^2*z -      2*a*b*c^9*y^2*z - 2*a^9*b*c*x*z^2 + 5*a^8*b^2*c*x*z^2 - 4*a^7*b^3*c*x*z^2 - 3*a^6*b^4*c*x*z^2 +      10*a^5*b^5*c*x*z^2 - 8*a^4*b^6*c*x*z^2 + 5*a^2*b^8*c*x*z^2 - 4*a*b^9*c*x*z^2 + b^10*c*x*z^2 +      a^8*b*c^2*x*z^2 - 8*a^7*b^2*c^2*x*z^2 + 16*a^6*b^3*c^2*x*z^2 - 16*a^5*b^4*c^2*x*z^2 +      2*a^4*b^5*c^2*x*z^2 + 14*a^3*b^6*c^2*x*z^2 - 17*a^2*b^7*c^2*x*z^2 + 10*a*b^8*c^2*x*z^2 -      2*b^9*c^2*x*z^2 + 8*a^7*b*c^3*x*z^2 - 13*a^6*b^2*c^3*x*z^2 + 22*a^4*b^4*c^3*x*z^2 -      32*a^3*b^5*c^3*x*z^2 + 18*a^2*b^6*c^3*x*z^2 - 4*a*b^7*c^3*x*z^2 - b^8*c^3*x*z^2 -      4*a^6*b*c^4*x*z^2 + 24*a^5*b^2*c^4*x*z^2 - 33*a^4*b^3*c^4*x*z^2 + 22*a^3*b^4*c^4*x*z^2 +      3*a^2*b^5*c^4*x*z^2 - 12*a*b^6*c^4*x*z^2 + 5*b^7*c^4*x*z^2 - 12*a^5*b*c^5*x*z^2 +      9*a^4*b^2*c^5*x*z^2 + 12*a^3*b^3*c^5*x*z^2 - 24*a^2*b^4*c^5*x*z^2 + 18*a*b^5*c^5*x*z^2 -      3*b^6*c^5*x*z^2 + 6*a^4*b*c^6*x*z^2 - 24*a^3*b^2*c^6*x*z^2 + 18*a^2*b^3*c^6*x*z^2 -      6*a*b^4*c^6*x*z^2 - 3*b^5*c^6*x*z^2 + 8*a^3*b*c^7*x*z^2 + a^2*b^2*c^7*x*z^2 - 8*a*b^3*c^7*x*z^2 +      5*b^4*c^7*x*z^2 - 4*a^2*b*c^8*x*z^2 + 8*a*b^2*c^8*x*z^2 - b^3*c^8*x*z^2 - 2*a*b*c^9*x*z^2 -      2*b^2*c^9*x*z^2 + b*c^10*x*z^2 + a^10*c*y*z^2 - 4*a^9*b*c*y*z^2 + 5*a^8*b^2*c*y*z^2 -      8*a^6*b^4*c*y*z^2 + 10*a^5*b^5*c*y*z^2 - 3*a^4*b^6*c*y*z^2 - 4*a^3*b^7*c*y*z^2 +      5*a^2*b^8*c*y*z^2 - 2*a*b^9*c*y*z^2 - 2*a^9*c^2*y*z^2 + 10*a^8*b*c^2*y*z^2 - 17*a^7*b^2*c^2*y*z^2 +      14*a^6*b^3*c^2*y*z^2 + 2*a^5*b^4*c^2*y*z^2 - 16*a^4*b^5*c^2*y*z^2 + 16*a^3*b^6*c^2*y*z^2 -      8*a^2*b^7*c^2*y*z^2 + a*b^8*c^2*y*z^2 - a^8*c^3*y*z^2 - 4*a^7*b*c^3*y*z^2 + 18*a^6*b^2*c^3*y*z^2 -      32*a^5*b^3*c^3*y*z^2 + 22*a^4*b^4*c^3*y*z^2 - 13*a^2*b^6*c^3*y*z^2 + 8*a*b^7*c^3*y*z^2 +      5*a^7*c^4*y*z^2 - 12*a^6*b*c^4*y*z^2 + 3*a^5*b^2*c^4*y*z^2 + 22*a^4*b^3*c^4*y*z^2 -      33*a^3*b^4*c^4*y*z^2 + 24*a^2*b^5*c^4*y*z^2 - 4*a*b^6*c^4*y*z^2 - 3*a^6*c^5*y*z^2 +      18*a^5*b*c^5*y*z^2 - 24*a^4*b^2*c^5*y*z^2 + 12*a^3*b^3*c^5*y*z^2 + 9*a^2*b^4*c^5*y*z^2 -      12*a*b^5*c^5*y*z^2 - 3*a^5*c^6*y*z^2 - 6*a^4*b*c^6*y*z^2 + 18*a^3*b^2*c^6*y*z^2 -      24*a^2*b^3*c^6*y*z^2 + 6*a*b^4*c^6*y*z^2 + 5*a^4*c^7*y*z^2 - 8*a^3*b*c^7*y*z^2 +      a^2*b^2*c^7*y*z^2 + 8*a*b^3*c^7*y*z^2 - a^3*c^8*y*z^2 + 8*a^2*b*c^8*y*z^2 - 4*a*b^2*c^8*y*z^2 -      2*a^2*c^9*y*z^2 - 2*a*b*c^9*y*z^2 + a*c^10*y*z^2");
		exp("K231","(-a^4)*c^2*x^2*y - 2*a^2*b^2*c^2*x^2*y + 3*b^4*c^2*x^2*y + 2*a^2*c^4*x^2*y - 2*b^2*c^4*x^2*y -      c^6*x^2*y + 3*a^4*c^2*x*y^2 - 2*a^2*b^2*c^2*x*y^2 - b^4*c^2*x*y^2 - 2*a^2*c^4*x*y^2 +      2*b^2*c^4*x*y^2 - c^6*x*y^2 - a^4*b^2*x^2*z + 2*a^2*b^4*x^2*z - b^6*x^2*z - 2*a^2*b^2*c^2*x^2*z -      2*b^4*c^2*x^2*z + 3*b^2*c^4*x^2*z + 2*a^6*x*y*z - 2*a^4*b^2*x*y*z - 2*a^2*b^4*x*y*z + 2*b^6*x*y*z -      2*a^4*c^2*x*y*z + 12*a^2*b^2*c^2*x*y*z - 2*b^4*c^2*x*y*z - 2*a^2*c^4*x*y*z - 2*b^2*c^4*x*y*z +      2*c^6*x*y*z - a^6*y^2*z + 2*a^4*b^2*y^2*z - a^2*b^4*y^2*z - 2*a^4*c^2*y^2*z - 2*a^2*b^2*c^2*y^2*z +      3*a^2*c^4*y^2*z + 3*a^4*b^2*x*z^2 - 2*a^2*b^4*x*z^2 - b^6*x*z^2 - 2*a^2*b^2*c^2*x*z^2 +      2*b^4*c^2*x*z^2 - b^2*c^4*x*z^2 - a^6*y*z^2 - 2*a^4*b^2*y*z^2 + 3*a^2*b^4*y*z^2 + 2*a^4*c^2*y*z^2 -      2*a^2*b^2*c^2*y*z^2 - a^2*c^4*y*z^2");
		exp("K232","-2*a^2*b^2*c^2*x^2*y + 2*b^4*c^2*x^2*y - 2*b^2*c^4*x^2*y + a^4*c^2*x*y^2 - b^4*c^2*x*y^2 +      2*b^2*c^4*x*y^2 - c^6*x*y^2 - a^4*b^2*x^2*z + 2*a^2*b^4*x^2*z - b^6*x^2*z + b^2*c^4*x^2*z +      a^6*x*y*z - a^4*b^2*x*y*z - a^2*b^4*x*y*z + b^6*x*y*z - a^4*c^2*x*y*z + 6*a^2*b^2*c^2*x*y*z -      b^4*c^2*x*y*z - a^2*c^4*x*y*z - b^2*c^4*x*y*z + c^6*x*y*z - 2*a^4*c^2*y^2*z - 2*a^2*b^2*c^2*y^2*z +      2*a^2*c^4*y^2*z + 2*a^4*b^2*x*z^2 - 2*a^2*b^4*x*z^2 - 2*a^2*b^2*c^2*x*z^2 - a^6*y*z^2 +      a^2*b^4*y*z^2 + 2*a^4*c^2*y*z^2 - a^2*c^4*y*z^2");
		exp("K233","(-a^2)*c^2*x^2*y + b^2*c^2*x^2*y + c^4*x^2*y - a^2*c^2*x*y^2 + b^2*c^2*x*y^2 - c^4*x*y^2 +      a^2*b^2*x^2*z - b^4*x^2*z - b^2*c^2*x^2*z + a^4*y^2*z - a^2*b^2*y^2*z + a^2*c^2*y^2*z +      a^2*b^2*x*z^2 + b^4*x*z^2 - b^2*c^2*x*z^2 - a^4*y*z^2 - a^2*b^2*y*z^2 + a^2*c^2*y*z^2");
		exp("K234","a^4*c^2*x^2*y - b^4*c^2*x^2*y - 2*a^2*c^4*x^2*y + c^6*x^2*y - 2*a^4*c^2*x*y^2 +      2*a^2*b^2*c^2*x*y^2 + 2*a^2*c^4*x*y^2 + 2*a^2*b^2*c^2*x^2*z + 2*b^4*c^2*x^2*z - 2*b^2*c^4*x^2*z -      a^6*x*y*z + a^4*b^2*x*y*z + a^2*b^4*x*y*z - b^6*x*y*z + a^4*c^2*x*y*z - 6*a^2*b^2*c^2*x*y*z +      b^4*c^2*x*y*z + a^2*c^4*x*y*z + b^2*c^4*x*y*z - c^6*x*y*z + a^6*y^2*z - 2*a^4*b^2*y^2*z +      a^2*b^4*y^2*z - a^2*c^4*y^2*z - a^4*b^2*x*z^2 + b^6*x*z^2 - 2*b^4*c^2*x*z^2 + b^2*c^4*x*z^2 +      2*a^4*b^2*y*z^2 - 2*a^2*b^4*y*z^2 + 2*a^2*b^2*c^2*y*z^2");
		exp("K235","(-a^6)*c^2*x^2*y - 2*a^4*b^2*c^2*x^2*y + 3*a^2*b^4*c^2*x^2*y + 2*a^4*c^4*x^2*y -      2*a^2*b^2*c^4*x^2*y - a^2*c^6*x^2*y - 3*a^4*b^2*c^2*x*y^2 + 2*a^2*b^4*c^2*x*y^2 + b^6*c^2*x*y^2 +      2*a^2*b^2*c^4*x*y^2 - 2*b^4*c^4*x*y^2 + b^2*c^6*x*y^2 + a^6*b^2*x^2*z - 2*a^4*b^4*x^2*z +      a^2*b^6*x^2*z + 2*a^4*b^2*c^2*x^2*z + 2*a^2*b^4*c^2*x^2*z - 3*a^2*b^2*c^4*x^2*z - a^6*b^2*y^2*z +      2*a^4*b^4*y^2*z - a^2*b^6*y^2*z - 2*a^4*b^2*c^2*y^2*z - 2*a^2*b^4*c^2*y^2*z + 3*a^2*b^2*c^4*y^2*z +      3*a^4*b^2*c^2*x*z^2 - 2*a^2*b^4*c^2*x*z^2 - b^6*c^2*x*z^2 - 2*a^2*b^2*c^4*x*z^2 + 2*b^4*c^4*x*z^2 -      b^2*c^6*x*z^2 + a^6*c^2*y*z^2 + 2*a^4*b^2*c^2*y*z^2 - 3*a^2*b^4*c^2*y*z^2 - 2*a^4*c^4*y*z^2 +      2*a^2*b^2*c^4*y*z^2 + a^2*c^6*y*z^2");
		exp("K236","a^4*c^4*x^2*y + 2*a^2*b^2*c^4*x^2*y - 3*b^4*c^4*x^2*y - 2*a^2*c^6*x^2*y + 2*b^2*c^6*x^2*y +      c^8*x^2*y + 3*a^4*c^4*x*y^2 - 2*a^2*b^2*c^4*x*y^2 - b^4*c^4*x*y^2 - 2*a^2*c^6*x*y^2 +      2*b^2*c^6*x*y^2 - c^8*x*y^2 - a^4*b^4*x^2*z + 2*a^2*b^6*x^2*z - b^8*x^2*z - 2*a^2*b^4*c^2*x^2*z -      2*b^6*c^2*x^2*z + 3*b^4*c^4*x^2*z + a^8*y^2*z - 2*a^6*b^2*y^2*z + a^4*b^4*y^2*z + 2*a^6*c^2*y^2*z +      2*a^4*b^2*c^2*y^2*z - 3*a^4*c^4*y^2*z - 3*a^4*b^4*x*z^2 + 2*a^2*b^6*x*z^2 + b^8*x*z^2 +      2*a^2*b^4*c^2*x*z^2 - 2*b^6*c^2*x*z^2 + b^4*c^4*x*z^2 - a^8*y*z^2 - 2*a^6*b^2*y*z^2 +      3*a^4*b^4*y*z^2 + 2*a^6*c^2*y*z^2 - 2*a^4*b^2*c^2*y*z^2 - a^4*c^4*y*z^2");
		exp("K237","a^4*x^2*y - 2*a^2*b^2*x^2*y + b^4*x^2*y - a^4*x*y^2 + 2*a^2*b^2*x*y^2 - b^4*x*y^2 - a^4*x^2*z +      2*a^2*c^2*x^2*z - c^4*x^2*z + b^4*y^2*z - 2*b^2*c^2*y^2*z + c^4*y^2*z + a^4*x*z^2 -      2*a^2*c^2*x*z^2 + c^4*x*z^2 - b^4*y*z^2 + 2*b^2*c^2*y*z^2 - c^4*y*z^2");
		exp("K238","(-a^8)*x^2*y + 2*a^6*b^2*x^2*y - 2*a^2*b^6*x^2*y + b^8*x^2*y + 2*a^6*c^2*x^2*y -      4*a^4*b^2*c^2*x^2*y + 2*a^2*b^4*c^2*x^2*y - a^4*c^4*x^2*y + 2*a^2*b^2*c^4*x^2*y - b^4*c^4*x^2*y -      a^8*x*y^2 + 2*a^6*b^2*x*y^2 - 2*a^2*b^6*x*y^2 + b^8*x*y^2 - 2*a^4*b^2*c^2*x*y^2 +      4*a^2*b^4*c^2*x*y^2 - 2*b^6*c^2*x*y^2 + a^4*c^4*x*y^2 - 2*a^2*b^2*c^4*x*y^2 + b^4*c^4*x*y^2 +      a^8*x^2*z - 2*a^6*b^2*x^2*z + a^4*b^4*x^2*z - 2*a^6*c^2*x^2*z + 4*a^4*b^2*c^2*x^2*z -      2*a^2*b^4*c^2*x^2*z - 2*a^2*b^2*c^4*x^2*z + b^4*c^4*x^2*z + 2*a^2*c^6*x^2*z - c^8*x^2*z -      a^4*b^4*y^2*z + 2*a^2*b^6*y^2*z - b^8*y^2*z + 2*a^4*b^2*c^2*y^2*z - 4*a^2*b^4*c^2*y^2*z +      2*b^6*c^2*y^2*z - a^4*c^4*y^2*z + 2*a^2*b^2*c^4*y^2*z - 2*b^2*c^6*y^2*z + c^8*y^2*z + a^8*x*z^2 -      a^4*b^4*x*z^2 - 2*a^6*c^2*x*z^2 + 2*a^4*b^2*c^2*x*z^2 + 2*a^2*b^4*c^2*x*z^2 - 4*a^2*b^2*c^4*x*z^2 -      b^4*c^4*x*z^2 + 2*a^2*c^6*x*z^2 + 2*b^2*c^6*x*z^2 - c^8*x*z^2 + a^4*b^4*y*z^2 - b^8*y*z^2 -      2*a^4*b^2*c^2*y*z^2 - 2*a^2*b^4*c^2*y*z^2 + 2*b^6*c^2*y*z^2 + a^4*c^4*y*z^2 + 4*a^2*b^2*c^4*y*z^2 -      2*a^2*c^6*y*z^2 - 2*b^2*c^6*y*z^2 + c^8*y*z^2");
		exp("K239","-2*a^8*x^2*y + 3*a^6*b^2*x^2*y + a^4*b^4*x^2*y - 3*a^2*b^6*x^2*y + b^8*x^2*y + 5*a^6*c^2*x^2*y -      11*a^4*b^2*c^2*x^2*y + 7*a^2*b^4*c^2*x^2*y - b^6*c^2*x^2*y - 2*a^4*c^4*x^2*y +      4*a^2*b^2*c^4*x^2*y - 2*b^4*c^4*x^2*y - a^8*x*y^2 + 3*a^6*b^2*x*y^2 - a^4*b^4*x*y^2 -      3*a^2*b^6*x*y^2 + 2*b^8*x*y^2 + a^6*c^2*x*y^2 - 7*a^4*b^2*c^2*x*y^2 + 11*a^2*b^4*c^2*x*y^2 -      5*b^6*c^2*x*y^2 + 2*a^4*c^4*x*y^2 - 4*a^2*b^2*c^4*x*y^2 + 2*b^4*c^4*x*y^2 + 2*a^8*x^2*z -      5*a^6*b^2*x^2*z + 2*a^4*b^4*x^2*z - 3*a^6*c^2*x^2*z + 11*a^4*b^2*c^2*x^2*z - 4*a^2*b^4*c^2*x^2*z -      a^4*c^4*x^2*z - 7*a^2*b^2*c^4*x^2*z + 2*b^4*c^4*x^2*z + 3*a^2*c^6*x^2*z + b^2*c^6*x^2*z -      c^8*x^2*z - 2*a^4*b^4*y^2*z + 5*a^2*b^6*y^2*z - 2*b^8*y^2*z + 4*a^4*b^2*c^2*y^2*z -      11*a^2*b^4*c^2*y^2*z + 3*b^6*c^2*y^2*z - 2*a^4*c^4*y^2*z + 7*a^2*b^2*c^4*y^2*z + b^4*c^4*y^2*z -      a^2*c^6*y^2*z - 3*b^2*c^6*y^2*z + c^8*y^2*z + a^8*x*z^2 - a^6*b^2*x*z^2 - 2*a^4*b^4*x*z^2 -      3*a^6*c^2*x*z^2 + 7*a^4*b^2*c^2*x*z^2 + 4*a^2*b^4*c^2*x*z^2 + a^4*c^4*x*z^2 -      11*a^2*b^2*c^4*x*z^2 - 2*b^4*c^4*x*z^2 + 3*a^2*c^6*x*z^2 + 5*b^2*c^6*x*z^2 - 2*c^8*x*z^2 +      2*a^4*b^4*y*z^2 + a^2*b^6*y*z^2 - b^8*y*z^2 - 4*a^4*b^2*c^2*y*z^2 - 7*a^2*b^4*c^2*y*z^2 +      3*b^6*c^2*y*z^2 + 2*a^4*c^4*y*z^2 + 11*a^2*b^2*c^4*y*z^2 - b^4*c^4*y*z^2 - 5*a^2*c^6*y*z^2 -      3*b^2*c^6*y*z^2 + 2*c^8*y*z^2");
		exp("K240","-2*a^6*b^2*x^2*y + a^4*b^4*x^2*y + 2*a^2*b^6*x^2*y - b^8*x^2*y + 2*a^6*c^2*x^2*y +      4*a^4*b^2*c^2*x^2*y - 8*a^2*b^4*c^2*x^2*y + 2*b^6*c^2*x^2*y - 5*a^4*c^4*x^2*y +      4*a^2*b^2*c^4*x^2*y + b^4*c^4*x^2*y + 2*a^2*c^6*x^2*y - 2*b^2*c^6*x^2*y + a^8*x*y^2 -      2*a^6*b^2*x*y^2 - a^4*b^4*x*y^2 + 2*a^2*b^6*x*y^2 - 2*a^6*c^2*x*y^2 + 8*a^4*b^2*c^2*x*y^2 -      4*a^2*b^4*c^2*x*y^2 - 2*b^6*c^2*x*y^2 - a^4*c^4*x*y^2 - 4*a^2*b^2*c^4*x*y^2 + 5*b^4*c^4*x*y^2 +      2*a^2*c^6*x*y^2 - 2*b^2*c^6*x*y^2 - 2*a^6*b^2*x^2*z + 5*a^4*b^4*x^2*z - 2*a^2*b^6*x^2*z +      2*a^6*c^2*x^2*z - 4*a^4*b^2*c^2*x^2*z - 4*a^2*b^4*c^2*x^2*z + 2*b^6*c^2*x^2*z - a^4*c^4*x^2*z +      8*a^2*b^2*c^4*x^2*z - b^4*c^4*x^2*z - 2*a^2*c^6*x^2*z - 2*b^2*c^6*x^2*z + c^8*x^2*z +      2*a^6*b^2*y^2*z - 5*a^4*b^4*y^2*z + 2*a^2*b^6*y^2*z - 2*a^6*c^2*y^2*z + 4*a^4*b^2*c^2*y^2*z +      4*a^2*b^4*c^2*y^2*z - 2*b^6*c^2*y^2*z + a^4*c^4*y^2*z - 8*a^2*b^2*c^4*y^2*z + b^4*c^4*y^2*z +      2*a^2*c^6*y^2*z + 2*b^2*c^6*y^2*z - c^8*y^2*z - a^8*x*z^2 + 2*a^6*b^2*x*z^2 + a^4*b^4*x*z^2 -      2*a^2*b^6*x*z^2 + 2*a^6*c^2*x*z^2 - 8*a^4*b^2*c^2*x*z^2 + 4*a^2*b^4*c^2*x*z^2 + 2*b^6*c^2*x*z^2 +      a^4*c^4*x*z^2 + 4*a^2*b^2*c^4*x*z^2 - 5*b^4*c^4*x*z^2 - 2*a^2*c^6*x*z^2 + 2*b^2*c^6*x*z^2 +      2*a^6*b^2*y*z^2 - a^4*b^4*y*z^2 - 2*a^2*b^6*y*z^2 + b^8*y*z^2 - 2*a^6*c^2*y*z^2 -      4*a^4*b^2*c^2*y*z^2 + 8*a^2*b^4*c^2*y*z^2 - 2*b^6*c^2*y*z^2 + 5*a^4*c^4*y*z^2 -      4*a^2*b^2*c^4*y*z^2 - b^4*c^4*y*z^2 - 2*a^2*c^6*y*z^2 + 2*b^2*c^6*y*z^2");
		exp("K241","-2*a^8*x^2*y + a^6*b^2*x^2*y + 2*a^4*b^4*x^2*y - a^2*b^6*x^2*y + 7*a^6*c^2*x^2*y -      7*a^4*b^2*c^2*x^2*y - a^2*b^4*c^2*x^2*y + b^6*c^2*x^2*y - 7*a^4*c^4*x^2*y + 8*a^2*b^2*c^4*x^2*y -      b^4*c^4*x^2*y + 2*a^2*c^6*x^2*y - 2*b^2*c^6*x^2*y + a^6*b^2*x*y^2 - 2*a^4*b^4*x*y^2 -      a^2*b^6*x*y^2 + 2*b^8*x*y^2 - a^6*c^2*x*y^2 + a^4*b^2*c^2*x*y^2 + 7*a^2*b^4*c^2*x*y^2 -      7*b^6*c^2*x*y^2 + a^4*c^4*x*y^2 - 8*a^2*b^2*c^4*x*y^2 + 7*b^4*c^4*x*y^2 + 2*a^2*c^6*x*y^2 -      2*b^2*c^6*x*y^2 + 2*a^8*x^2*z - 7*a^6*b^2*x^2*z + 7*a^4*b^4*x^2*z - 2*a^2*b^6*x^2*z -      a^6*c^2*x^2*z + 7*a^4*b^2*c^2*x^2*z - 8*a^2*b^4*c^2*x^2*z + 2*b^6*c^2*x^2*z - 2*a^4*c^4*x^2*z +      a^2*b^2*c^4*x^2*z + b^4*c^4*x^2*z + a^2*c^6*x^2*z - b^2*c^6*x^2*z + 2*a^6*b^2*y^2*z -      7*a^4*b^4*y^2*z + 7*a^2*b^6*y^2*z - 2*b^8*y^2*z - 2*a^6*c^2*y^2*z + 8*a^4*b^2*c^2*y^2*z -      7*a^2*b^4*c^2*y^2*z + b^6*c^2*y^2*z - a^4*c^4*y^2*z - a^2*b^2*c^4*y^2*z + 2*b^4*c^4*y^2*z +      a^2*c^6*y^2*z - b^2*c^6*y^2*z + a^6*b^2*x*z^2 - a^4*b^4*x*z^2 - 2*a^2*b^6*x*z^2 - a^6*c^2*x*z^2 -      a^4*b^2*c^2*x*z^2 + 8*a^2*b^4*c^2*x*z^2 + 2*b^6*c^2*x*z^2 + 2*a^4*c^4*x*z^2 - 7*a^2*b^2*c^4*x*z^2 -      7*b^4*c^4*x*z^2 + a^2*c^6*x*z^2 + 7*b^2*c^6*x*z^2 - 2*c^8*x*z^2 + 2*a^6*b^2*y*z^2 + a^4*b^4*y*z^2 -      a^2*b^6*y*z^2 - 2*a^6*c^2*y*z^2 - 8*a^4*b^2*c^2*y*z^2 + a^2*b^4*c^2*y*z^2 + b^6*c^2*y*z^2 +      7*a^4*c^4*y*z^2 + 7*a^2*b^2*c^4*y*z^2 - 2*b^4*c^4*y*z^2 - 7*a^2*c^6*y*z^2 - b^2*c^6*y*z^2 +      2*c^8*y*z^2");
		exp("K242","(-a^2)*b^2*x^2*y + b^4*x^2*y + a^2*c^2*x^2*y - b^2*c^2*x^2*y - a^4*x*y^2 + a^2*b^2*x*y^2 +      a^2*c^2*x*y^2 - b^2*c^2*x*y^2 - a^2*b^2*x^2*z + a^2*c^2*x^2*z + b^2*c^2*x^2*z - c^4*x^2*z +      a^2*b^2*y^2*z - a^2*c^2*y^2*z - b^2*c^2*y^2*z + c^4*y^2*z + a^4*x*z^2 - a^2*b^2*x*z^2 -      a^2*c^2*x*z^2 + b^2*c^2*x*z^2 + a^2*b^2*y*z^2 - b^4*y*z^2 - a^2*c^2*y*z^2 + b^2*c^2*y*z^2");
		exp("K243","a^4*c^2*x^2*y + 4*a^2*b^2*c^2*x^2*y - 5*b^4*c^2*x^2*y - 2*a^2*c^4*x^2*y + 4*b^2*c^4*x^2*y +      c^6*x^2*y + 5*a^4*c^2*x*y^2 - 4*a^2*b^2*c^2*x*y^2 - b^4*c^2*x*y^2 - 4*a^2*c^4*x*y^2 +      2*b^2*c^4*x*y^2 - c^6*x*y^2 - a^4*b^2*x^2*z + 2*a^2*b^4*x^2*z - b^6*x^2*z - 4*a^2*b^2*c^2*x^2*z -      4*b^4*c^2*x^2*z + 5*b^2*c^4*x^2*z + a^6*y^2*z - 2*a^4*b^2*y^2*z + a^2*b^4*y^2*z + 4*a^4*c^2*y^2*z +      4*a^2*b^2*c^2*y^2*z - 5*a^2*c^4*y^2*z - 5*a^4*b^2*x*z^2 + 4*a^2*b^4*x*z^2 + b^6*x*z^2 +      4*a^2*b^2*c^2*x*z^2 - 2*b^4*c^2*x*z^2 + b^2*c^4*x*z^2 - a^6*y*z^2 - 4*a^4*b^2*y*z^2 +      5*a^2*b^4*y*z^2 + 2*a^4*c^2*y*z^2 - 4*a^2*b^2*c^2*y*z^2 - a^2*c^4*y*z^2");
		exp("K244","-27*a^2*b^2*c^2*x*y*z + (a^2*x + b^2*y + c^2*z)^3");
		exp("K245","(-a^6)*x^2*y + 2*a^4*b^2*x^2*y - a^2*b^4*x^2*y - a^2*b^2*c^2*x^2*y + b^4*c^2*x^2*y +      a^2*c^4*x^2*y - b^2*c^4*x^2*y + a^4*b^2*x*y^2 - 2*a^2*b^4*x*y^2 + b^6*x*y^2 - a^4*c^2*x*y^2 +      a^2*b^2*c^2*x*y^2 + a^2*c^4*x*y^2 - b^2*c^4*x*y^2 + a^6*x^2*z - a^2*b^4*x^2*z - 2*a^4*c^2*x^2*z +      a^2*b^2*c^2*x^2*z + b^4*c^2*x^2*z + a^2*c^4*x^2*z - b^2*c^4*x^2*z + a^4*b^2*y^2*z - b^6*y^2*z -      a^4*c^2*y^2*z - a^2*b^2*c^2*y^2*z + 2*b^4*c^2*y^2*z + a^2*c^4*y^2*z - b^2*c^4*y^2*z +      a^4*b^2*x*z^2 - a^2*b^4*x*z^2 - a^4*c^2*x*z^2 - a^2*b^2*c^2*x*z^2 + b^4*c^2*x*z^2 +      2*a^2*c^4*x*z^2 - c^6*x*z^2 + a^4*b^2*y*z^2 - a^2*b^4*y*z^2 - a^4*c^2*y*z^2 + a^2*b^2*c^2*y*z^2 +      b^4*c^2*y*z^2 - 2*b^2*c^4*y*z^2 + c^6*y*z^2");
		exp("K246","(-a^10)*b^2*c^2*x^2*y + 2*a^8*b^4*c^2*x^2*y - 2*a^4*b^8*c^2*x^2*y + a^2*b^10*c^2*x^2*y -      3*a^6*b^4*c^4*x^2*y + 5*a^4*b^6*c^4*x^2*y - a^2*b^8*c^4*x^2*y - b^10*c^4*x^2*y +      2*a^6*b^2*c^6*x^2*y - 3*a^4*b^4*c^6*x^2*y - 2*a^2*b^6*c^6*x^2*y + 3*b^8*c^6*x^2*y +      3*a^2*b^4*c^8*x^2*y - 3*b^6*c^8*x^2*y - a^2*b^2*c^10*x^2*y + b^4*c^10*x^2*y - a^10*b^2*c^2*x*y^2 +      2*a^8*b^4*c^2*x*y^2 - 2*a^4*b^8*c^2*x*y^2 + a^2*b^10*c^2*x*y^2 + a^10*c^4*x*y^2 +      a^8*b^2*c^4*x*y^2 - 5*a^6*b^4*c^4*x*y^2 + 3*a^4*b^6*c^4*x*y^2 - 3*a^8*c^6*x*y^2 +      2*a^6*b^2*c^6*x*y^2 + 3*a^4*b^4*c^6*x*y^2 - 2*a^2*b^6*c^6*x*y^2 + 3*a^6*c^8*x*y^2 -      3*a^4*b^2*c^8*x*y^2 - a^4*c^10*x*y^2 + a^2*b^2*c^10*x*y^2 + a^10*b^2*c^2*x^2*z -      2*a^6*b^6*c^2*x^2*z + a^2*b^10*c^2*x^2*z - 2*a^8*b^2*c^4*x^2*z + 3*a^6*b^4*c^4*x^2*z +      3*a^4*b^6*c^4*x^2*z - 3*a^2*b^8*c^4*x^2*z - b^10*c^4*x^2*z - 5*a^4*b^4*c^6*x^2*z +      2*a^2*b^6*c^6*x^2*z + 3*b^8*c^6*x^2*z + 2*a^4*b^2*c^8*x^2*z + a^2*b^4*c^8*x^2*z - 3*b^6*c^8*x^2*z -      a^2*b^2*c^10*x^2*z + b^4*c^10*x^2*z - a^10*b^2*c^2*y^2*z + 2*a^6*b^6*c^2*y^2*z -      a^2*b^10*c^2*y^2*z + a^10*c^4*y^2*z + 3*a^8*b^2*c^4*y^2*z - 3*a^6*b^4*c^4*y^2*z -      3*a^4*b^6*c^4*y^2*z + 2*a^2*b^8*c^4*y^2*z - 3*a^8*c^6*y^2*z - 2*a^6*b^2*c^6*y^2*z +      5*a^4*b^4*c^6*y^2*z + 3*a^6*c^8*y^2*z - a^4*b^2*c^8*y^2*z - 2*a^2*b^4*c^8*y^2*z - a^4*c^10*y^2*z +      a^2*b^2*c^10*y^2*z - a^10*b^4*x*z^2 + 3*a^8*b^6*x*z^2 - 3*a^6*b^8*x*z^2 + a^4*b^10*x*z^2 +      a^10*b^2*c^2*x*z^2 - a^8*b^4*c^2*x*z^2 - 2*a^6*b^6*c^2*x*z^2 + 3*a^4*b^8*c^2*x*z^2 -      a^2*b^10*c^2*x*z^2 - 2*a^8*b^2*c^4*x*z^2 + 5*a^6*b^4*c^4*x*z^2 - 3*a^4*b^6*c^4*x*z^2 -      3*a^4*b^4*c^6*x*z^2 + 2*a^2*b^6*c^6*x*z^2 + 2*a^4*b^2*c^8*x*z^2 - a^2*b^2*c^10*x*z^2 -      a^10*b^4*y*z^2 + 3*a^8*b^6*y*z^2 - 3*a^6*b^8*y*z^2 + a^4*b^10*y*z^2 + a^10*b^2*c^2*y*z^2 -      3*a^8*b^4*c^2*y*z^2 + 2*a^6*b^6*c^2*y*z^2 + a^4*b^8*c^2*y*z^2 - a^2*b^10*c^2*y*z^2 +      3*a^6*b^4*c^4*y*z^2 - 5*a^4*b^6*c^4*y*z^2 + 2*a^2*b^8*c^4*y*z^2 - 2*a^6*b^2*c^6*y*z^2 +      3*a^4*b^4*c^6*y*z^2 - 2*a^2*b^4*c^8*y*z^2 + a^2*b^2*c^10*y*z^2");
		exp("K247","a^6*b*c*x^2*y - a^5*b^2*c*x^2*y - 2*a^4*b^3*c*x^2*y + 2*a^3*b^4*c*x^2*y + a^2*b^5*c*x^2*y -      a*b^6*c*x^2*y - a^5*b*c^2*x^2*y + 3*a^4*b^2*c^2*x^2*y - 4*a^2*b^4*c^2*x^2*y + a*b^5*c^2*x^2*y +      b^6*c^2*x^2*y - 2*a^3*b^2*c^3*x^2*y + 2*a^2*b^3*c^3*x^2*y + 2*a*b^4*c^3*x^2*y - 2*b^5*c^3*x^2*y +      2*a^2*b^2*c^4*x^2*y - 2*a*b^3*c^4*x^2*y - a^2*b*c^5*x^2*y - a*b^2*c^5*x^2*y + 2*b^3*c^5*x^2*y +      a*b*c^6*x^2*y - b^2*c^6*x^2*y + a^6*b*c*x*y^2 - a^5*b^2*c*x*y^2 - 2*a^4*b^3*c*x*y^2 +      2*a^3*b^4*c*x*y^2 + a^2*b^5*c*x*y^2 - a*b^6*c*x*y^2 - a^6*c^2*x*y^2 - a^5*b*c^2*x*y^2 +      4*a^4*b^2*c^2*x*y^2 - 3*a^2*b^4*c^2*x*y^2 + a*b^5*c^2*x*y^2 + 2*a^5*c^3*x*y^2 - 2*a^4*b*c^3*x*y^2 -      2*a^3*b^2*c^3*x*y^2 + 2*a^2*b^3*c^3*x*y^2 + 2*a^3*b*c^4*x*y^2 - 2*a^2*b^2*c^4*x*y^2 -      2*a^3*c^5*x*y^2 + a^2*b*c^5*x*y^2 + a*b^2*c^5*x*y^2 + a^2*c^6*x*y^2 - a*b*c^6*x*y^2 -      a^6*b*c*x^2*z + a^5*b^2*c*x^2*z + a^2*b^5*c*x^2*z - a*b^6*c*x^2*z + a^5*b*c^2*x^2*z -      3*a^4*b^2*c^2*x^2*z + 2*a^3*b^3*c^2*x^2*z - 2*a^2*b^4*c^2*x^2*z + a*b^5*c^2*x^2*z + b^6*c^2*x^2*z +      2*a^4*b*c^3*x^2*z - 2*a^2*b^3*c^3*x^2*z + 2*a*b^4*c^3*x^2*z - 2*b^5*c^3*x^2*z - 2*a^3*b*c^4*x^2*z +      4*a^2*b^2*c^4*x^2*z - 2*a*b^3*c^4*x^2*z - a^2*b*c^5*x^2*z - a*b^2*c^5*x^2*z + 2*b^3*c^5*x^2*z +      a*b*c^6*x^2*z - b^2*c^6*x^2*z + a^6*b*c*y^2*z - a^5*b^2*c*y^2*z - a^2*b^5*c*y^2*z + a*b^6*c*y^2*z -      a^6*c^2*y^2*z - a^5*b*c^2*y^2*z + 2*a^4*b^2*c^2*y^2*z - 2*a^3*b^3*c^2*y^2*z + 3*a^2*b^4*c^2*y^2*z -      a*b^5*c^2*y^2*z + 2*a^5*c^3*y^2*z - 2*a^4*b*c^3*y^2*z + 2*a^3*b^2*c^3*y^2*z - 2*a*b^4*c^3*y^2*z +      2*a^3*b*c^4*y^2*z - 4*a^2*b^2*c^4*y^2*z + 2*a*b^3*c^4*y^2*z - 2*a^3*c^5*y^2*z + a^2*b*c^5*y^2*z +      a*b^2*c^5*y^2*z + a^2*c^6*y^2*z - a*b*c^6*y^2*z + a^6*b^2*x*z^2 - 2*a^5*b^3*x*z^2 +      2*a^3*b^5*x*z^2 - a^2*b^6*x*z^2 - a^6*b*c*x*z^2 + a^5*b^2*c*x*z^2 + 2*a^4*b^3*c*x*z^2 -      2*a^3*b^4*c*x*z^2 - a^2*b^5*c*x*z^2 + a*b^6*c*x*z^2 + a^5*b*c^2*x*z^2 - 4*a^4*b^2*c^2*x*z^2 +      2*a^3*b^3*c^2*x*z^2 + 2*a^2*b^4*c^2*x*z^2 - a*b^5*c^2*x*z^2 + 2*a^4*b*c^3*x*z^2 -      2*a^2*b^3*c^3*x*z^2 - 2*a^3*b*c^4*x*z^2 + 3*a^2*b^2*c^4*x*z^2 - a^2*b*c^5*x*z^2 - a*b^2*c^5*x*z^2 +      a*b*c^6*x*z^2 + a^6*b^2*y*z^2 - 2*a^5*b^3*y*z^2 + 2*a^3*b^5*y*z^2 - a^2*b^6*y*z^2 - a^6*b*c*y*z^2 +      a^5*b^2*c*y*z^2 + 2*a^4*b^3*c*y*z^2 - 2*a^3*b^4*c*y*z^2 - a^2*b^5*c*y*z^2 + a*b^6*c*y*z^2 +      a^5*b*c^2*y*z^2 - 2*a^4*b^2*c^2*y*z^2 - 2*a^3*b^3*c^2*y*z^2 + 4*a^2*b^4*c^2*y*z^2 -      a*b^5*c^2*y*z^2 + 2*a^3*b^2*c^3*y*z^2 - 2*a*b^4*c^3*y*z^2 - 3*a^2*b^2*c^4*y*z^2 +      2*a*b^3*c^4*y*z^2 + a^2*b*c^5*y*z^2 + a*b^2*c^5*y*z^2 - a*b*c^6*y*z^2");
		exp("K248","(-a^2)*b^2*c^2*x^2*y + b^2*c^4*x^2*y + a^2*b^2*c^2*x*y^2 - a^2*c^4*x*y^2 + a^2*b^2*c^2*x^2*z -      b^4*c^2*x^2*z - 2*a^4*b^2*x*y*z + 2*a^2*b^4*x*y*z + 2*a^4*c^2*x*y*z - 2*b^4*c^2*x*y*z -      2*a^2*c^4*x*y*z + 2*b^2*c^4*x*y*z + a^4*c^2*y^2*z - a^2*b^2*c^2*y^2*z + a^2*b^4*x*z^2 -      a^2*b^2*c^2*x*z^2 - a^4*b^2*y*z^2 + a^2*b^2*c^2*y*z^2");
		exp("K249","(-a^6)*c^2*x^2*y + 3*a^4*b^2*c^2*x^2*y - 5*a^2*b^4*c^2*x^2*y + 2*a^4*c^4*x^2*y +      5*b^4*c^4*x^2*y - 2*a^2*c^6*x^2*y - 3*b^2*c^6*x^2*y + c^8*x^2*y + 5*a^4*b^2*c^2*x*y^2 -      3*a^2*b^4*c^2*x*y^2 + b^6*c^2*x*y^2 - 5*a^4*c^4*x*y^2 - 2*b^4*c^4*x*y^2 + 3*a^2*c^6*x*y^2 +      2*b^2*c^6*x*y^2 - c^8*x*y^2 + a^6*b^2*x^2*z - 2*a^4*b^4*x^2*z + 2*a^2*b^6*x^2*z - b^8*x^2*z -      3*a^4*b^2*c^2*x^2*z + 3*b^6*c^2*x^2*z + 5*a^2*b^2*c^4*x^2*z - 5*b^4*c^4*x^2*z - 2*a^6*b^2*x*y*z +      2*a^2*b^6*x*y*z + 2*a^6*c^2*x*y*z - 2*b^6*c^2*x*y*z - 2*a^2*c^6*x*y*z + 2*b^2*c^6*x*y*z +      a^8*y^2*z - 2*a^6*b^2*y^2*z + 2*a^4*b^4*y^2*z - a^2*b^6*y^2*z - 3*a^6*c^2*y^2*z +      3*a^2*b^4*c^2*y^2*z + 5*a^4*c^4*y^2*z - 5*a^2*b^2*c^4*y^2*z + 5*a^4*b^4*x*z^2 - 3*a^2*b^6*x*z^2 +      b^8*x*z^2 - 5*a^4*b^2*c^2*x*z^2 - 2*b^6*c^2*x*z^2 + 3*a^2*b^2*c^4*x*z^2 + 2*b^4*c^4*x*z^2 -      b^2*c^6*x*z^2 - a^8*y*z^2 + 3*a^6*b^2*y*z^2 - 5*a^4*b^4*y*z^2 + 2*a^6*c^2*y*z^2 +      5*a^2*b^4*c^2*y*z^2 - 2*a^4*c^4*y*z^2 - 3*a^2*b^2*c^4*y*z^2 + a^2*c^6*y*z^2");
		exp("K250","(-a^6)*c^2*x^2*y + 2*a^4*b^2*c^2*x^2*y - 3*a^2*b^4*c^2*x^2*y + 2*a^4*c^4*x^2*y +      3*b^4*c^4*x^2*y - 2*a^2*c^6*x^2*y - 2*b^2*c^6*x^2*y + c^8*x^2*y + 3*a^4*b^2*c^2*x*y^2 -      2*a^2*b^4*c^2*x*y^2 + b^6*c^2*x*y^2 - 3*a^4*c^4*x*y^2 - 2*b^4*c^4*x*y^2 + 2*a^2*c^6*x*y^2 +      2*b^2*c^6*x*y^2 - c^8*x*y^2 + a^6*b^2*x^2*z - 2*a^4*b^4*x^2*z + 2*a^2*b^6*x^2*z - b^8*x^2*z -      2*a^4*b^2*c^2*x^2*z + 2*b^6*c^2*x^2*z + 3*a^2*b^2*c^4*x^2*z - 3*b^4*c^4*x^2*z - a^6*b^2*x*y*z +      a^2*b^6*x*y*z + a^6*c^2*x*y*z - b^6*c^2*x*y*z - a^2*c^6*x*y*z + b^2*c^6*x*y*z + a^8*y^2*z -      2*a^6*b^2*y^2*z + 2*a^4*b^4*y^2*z - a^2*b^6*y^2*z - 2*a^6*c^2*y^2*z + 2*a^2*b^4*c^2*y^2*z +      3*a^4*c^4*y^2*z - 3*a^2*b^2*c^4*y^2*z + 3*a^4*b^4*x*z^2 - 2*a^2*b^6*x*z^2 + b^8*x*z^2 -      3*a^4*b^2*c^2*x*z^2 - 2*b^6*c^2*x*z^2 + 2*a^2*b^2*c^4*x*z^2 + 2*b^4*c^4*x*z^2 - b^2*c^6*x*z^2 -      a^8*y*z^2 + 2*a^6*b^2*y*z^2 - 3*a^4*b^4*y*z^2 + 2*a^6*c^2*y*z^2 + 3*a^2*b^4*c^2*y*z^2 -      2*a^4*c^4*y*z^2 - 2*a^2*b^2*c^4*y*z^2 + a^2*c^6*y*z^2");
		exp("K251","(-a)*b*c*x^2*y + c^3*x^2*y + a*b*c*x*y^2 - c^3*x*y^2 - b^3*x^2*z + a*b*c*x^2*z + a^3*y^2*z -      a*b*c*y^2*z + b^3*x*z^2 - a*b*c*x*z^2 - a^3*y*z^2 + a*b*c*y*z^2");
		exp("K252","(-a^2)*b^2*c^2*x^2*y + c^6*x^2*y + a^2*b^2*c^2*x*y^2 - c^6*x*y^2 - b^6*x^2*z +      a^2*b^2*c^2*x^2*z + a^6*y^2*z - a^2*b^2*c^2*y^2*z + b^6*x*z^2 - a^2*b^2*c^2*x*z^2 - a^6*y*z^2 +      a^2*b^2*c^2*y*z^2");
		exp("K253","a^3*c^2*x^2*y + a^2*b*c^2*x^2*y + a*b^2*c^2*x^2*y + b^3*c^2*x^2*y + a^2*c^3*x^2*y +      2*a*b*c^3*x^2*y + b^2*c^3*x^2*y - a^3*c^2*x*y^2 - a^2*b*c^2*x*y^2 - a*b^2*c^2*x*y^2 -      b^3*c^2*x*y^2 - a^2*c^3*x*y^2 - 2*a*b*c^3*x*y^2 - b^2*c^3*x*y^2 - a^3*b^2*x^2*z - a^2*b^3*x^2*z -      a^2*b^2*c*x^2*z - 2*a*b^3*c*x^2*z - a*b^2*c^2*x^2*z - b^3*c^2*x^2*z - b^2*c^3*x^2*z +      a^3*b^2*y^2*z + a^2*b^3*y^2*z + 2*a^3*b*c*y^2*z + a^2*b^2*c*y^2*z + a^3*c^2*y^2*z +      a^2*b*c^2*y^2*z + a^2*c^3*y^2*z + a^3*b^2*x*z^2 + a^2*b^3*x*z^2 + a^2*b^2*c*x*z^2 +      2*a*b^3*c*x*z^2 + a*b^2*c^2*x*z^2 + b^3*c^2*x*z^2 + b^2*c^3*x*z^2 - a^3*b^2*y*z^2 - a^2*b^3*y*z^2 -      2*a^3*b*c*y*z^2 - a^2*b^2*c*y*z^2 - a^3*c^2*y*z^2 - a^2*b*c^2*y*z^2 - a^2*c^3*y*z^2");
		exp("K254","a^3*b*c*x^2*y - a*b^3*c*x^2*y + a^3*c^2*x^2*y + a^2*b*c^2*x^2*y + a^2*c^3*x^2*y + a*b*c^3*x^2*y +      a^3*b*c*x*y^2 - a*b^3*c*x*y^2 - a*b^2*c^2*x*y^2 - b^3*c^2*x*y^2 - a*b*c^3*x*y^2 - b^2*c^3*x*y^2 -      a^3*b^2*x^2*z - a^2*b^3*x^2*z - a^3*b*c*x^2*z - a^2*b^2*c*x^2*z - a*b^3*c*x^2*z + a*b*c^3*x^2*z +      a^3*b^2*y^2*z + a^2*b^3*y^2*z + a^3*b*c*y^2*z + a^2*b^2*c*y^2*z + a*b^3*c*y^2*z - a*b*c^3*y^2*z -      a^3*b*c*x*z^2 + a*b^3*c*x*z^2 + a*b^2*c^2*x*z^2 + b^3*c^2*x*z^2 + a*b*c^3*x*z^2 + b^2*c^3*x*z^2 -      a^3*b*c*y*z^2 + a*b^3*c*y*z^2 - a^3*c^2*y*z^2 - a^2*b*c^2*y*z^2 - a^2*c^3*y*z^2 - a*b*c^3*y*z^2 ");
		exp("K255","-(a^16*c^2*x^2*y) + 10*a^12*b^4*c^2*x^2*y - 16*a^10*b^6*c^2*x^2*y + 16*a^6*b^10*c^2*x^2*y -      10*a^4*b^12*c^2*x^2*y + b^16*c^2*x^2*y + 5*a^14*c^4*x^2*y - 10*a^12*b^2*c^4*x^2*y -      18*a^10*b^4*c^4*x^2*y + 59*a^8*b^6*c^4*x^2*y - 43*a^6*b^8*c^4*x^2*y + 8*a^2*b^12*c^4*x^2*y -      b^14*c^4*x^2*y - 9*a^12*c^6*x^2*y + 36*a^10*b^2*c^6*x^2*y - 21*a^8*b^4*c^6*x^2*y -      36*a^6*b^6*c^6*x^2*y + 51*a^4*b^8*c^6*x^2*y - 12*a^2*b^10*c^6*x^2*y - 9*b^12*c^6*x^2*y +      5*a^10*c^8*x^2*y - 45*a^8*b^2*c^8*x^2*y + 52*a^6*b^4*c^8*x^2*y - 28*a^4*b^6*c^8*x^2*y -      9*a^2*b^8*c^8*x^2*y + 25*b^10*c^8*x^2*y + 5*a^8*c^10*x^2*y + 20*a^6*b^2*c^10*x^2*y -      18*a^4*b^4*c^10*x^2*y + 20*a^2*b^6*c^10*x^2*y - 25*b^8*c^10*x^2*y - 9*a^6*c^12*x^2*y -      6*a^2*b^4*c^12*x^2*y + 9*b^6*c^12*x^2*y + 5*a^4*c^14*x^2*y + b^4*c^14*x^2*y - a^2*c^16*x^2*y -      b^2*c^16*x^2*y - a^16*c^2*x*y^2 + 10*a^12*b^4*c^2*x*y^2 - 16*a^10*b^6*c^2*x*y^2 +      16*a^6*b^10*c^2*x*y^2 - 10*a^4*b^12*c^2*x*y^2 + b^16*c^2*x*y^2 + a^14*c^4*x*y^2 -      8*a^12*b^2*c^4*x*y^2 + 43*a^8*b^6*c^4*x*y^2 - 59*a^6*b^8*c^4*x*y^2 + 18*a^4*b^10*c^4*x*y^2 +      10*a^2*b^12*c^4*x*y^2 - 5*b^14*c^4*x*y^2 + 9*a^12*c^6*x*y^2 + 12*a^10*b^2*c^6*x*y^2 -      51*a^8*b^4*c^6*x*y^2 + 36*a^6*b^6*c^6*x*y^2 + 21*a^4*b^8*c^6*x*y^2 - 36*a^2*b^10*c^6*x*y^2 +      9*b^12*c^6*x*y^2 - 25*a^10*c^8*x*y^2 + 9*a^8*b^2*c^8*x*y^2 + 28*a^6*b^4*c^8*x*y^2 -      52*a^4*b^6*c^8*x*y^2 + 45*a^2*b^8*c^8*x*y^2 - 5*b^10*c^8*x*y^2 + 25*a^8*c^10*x*y^2 -      20*a^6*b^2*c^10*x*y^2 + 18*a^4*b^4*c^10*x*y^2 - 20*a^2*b^6*c^10*x*y^2 - 5*b^8*c^10*x*y^2 -      9*a^6*c^12*x*y^2 + 6*a^4*b^2*c^12*x*y^2 + 9*b^6*c^12*x*y^2 - a^4*c^14*x*y^2 - 5*b^4*c^14*x*y^2 +      a^2*c^16*x*y^2 + b^2*c^16*x*y^2 + a^16*b^2*x^2*z - 5*a^14*b^4*x^2*z + 9*a^12*b^6*x^2*z -      5*a^10*b^8*x^2*z - 5*a^8*b^10*x^2*z + 9*a^6*b^12*x^2*z - 5*a^4*b^14*x^2*z + a^2*b^16*x^2*z +      10*a^12*b^4*c^2*x^2*z - 36*a^10*b^6*c^2*x^2*z + 45*a^8*b^8*c^2*x^2*z - 20*a^6*b^10*c^2*x^2*z +      b^16*c^2*x^2*z - 10*a^12*b^2*c^4*x^2*z + 18*a^10*b^4*c^4*x^2*z + 21*a^8*b^6*c^4*x^2*z -      52*a^6*b^8*c^4*x^2*z + 18*a^4*b^10*c^4*x^2*z + 6*a^2*b^12*c^4*x^2*z - b^14*c^4*x^2*z +      16*a^10*b^2*c^6*x^2*z - 59*a^8*b^4*c^6*x^2*z + 36*a^6*b^6*c^6*x^2*z + 28*a^4*b^8*c^6*x^2*z -      20*a^2*b^10*c^6*x^2*z - 9*b^12*c^6*x^2*z + 43*a^6*b^4*c^8*x^2*z - 51*a^4*b^6*c^8*x^2*z +      9*a^2*b^8*c^8*x^2*z + 25*b^10*c^8*x^2*z - 16*a^6*b^2*c^10*x^2*z + 12*a^2*b^6*c^10*x^2*z -      25*b^8*c^10*x^2*z + 10*a^4*b^2*c^12*x^2*z - 8*a^2*b^4*c^12*x^2*z + 9*b^6*c^12*x^2*z +      b^4*c^14*x^2*z - b^2*c^16*x^2*z - a^16*b^2*y^2*z + 5*a^14*b^4*y^2*z - 9*a^12*b^6*y^2*z +      5*a^10*b^8*y^2*z + 5*a^8*b^10*y^2*z - 9*a^6*b^12*y^2*z + 5*a^4*b^14*y^2*z - a^2*b^16*y^2*z -      a^16*c^2*y^2*z + 20*a^10*b^6*c^2*y^2*z - 45*a^8*b^8*c^2*y^2*z + 36*a^6*b^10*c^2*y^2*z -      10*a^4*b^12*c^2*y^2*z + a^14*c^4*y^2*z - 6*a^12*b^2*c^4*y^2*z - 18*a^10*b^4*c^4*y^2*z +      52*a^8*b^6*c^4*y^2*z - 21*a^6*b^8*c^4*y^2*z - 18*a^4*b^10*c^4*y^2*z + 10*a^2*b^12*c^4*y^2*z +      9*a^12*c^6*y^2*z + 20*a^10*b^2*c^6*y^2*z - 28*a^8*b^4*c^6*y^2*z - 36*a^6*b^6*c^6*y^2*z +      59*a^4*b^8*c^6*y^2*z - 16*a^2*b^10*c^6*y^2*z - 25*a^10*c^8*y^2*z - 9*a^8*b^2*c^8*y^2*z +      51*a^6*b^4*c^8*y^2*z - 43*a^4*b^6*c^8*y^2*z + 25*a^8*c^10*y^2*z - 12*a^6*b^2*c^10*y^2*z +      16*a^2*b^6*c^10*y^2*z - 9*a^6*c^12*y^2*z + 8*a^4*b^2*c^12*y^2*z - 10*a^2*b^4*c^12*y^2*z -      a^4*c^14*y^2*z + a^2*c^16*y^2*z + a^16*b^2*x*z^2 - a^14*b^4*x*z^2 - 9*a^12*b^6*x*z^2 +      25*a^10*b^8*x*z^2 - 25*a^8*b^10*x*z^2 + 9*a^6*b^12*x*z^2 + a^4*b^14*x*z^2 - a^2*b^16*x*z^2 +      8*a^12*b^4*c^2*x*z^2 - 12*a^10*b^6*c^2*x*z^2 - 9*a^8*b^8*c^2*x*z^2 + 20*a^6*b^10*c^2*x*z^2 -      6*a^4*b^12*c^2*x*z^2 - b^16*c^2*x*z^2 - 10*a^12*b^2*c^4*x*z^2 + 51*a^8*b^6*c^4*x*z^2 -      28*a^6*b^8*c^4*x*z^2 - 18*a^4*b^10*c^4*x*z^2 + 5*b^14*c^4*x*z^2 + 16*a^10*b^2*c^6*x*z^2 -      43*a^8*b^4*c^6*x*z^2 - 36*a^6*b^6*c^6*x*z^2 + 52*a^4*b^8*c^6*x*z^2 + 20*a^2*b^10*c^6*x*z^2 -      9*b^12*c^6*x*z^2 + 59*a^6*b^4*c^8*x*z^2 - 21*a^4*b^6*c^8*x*z^2 - 45*a^2*b^8*c^8*x*z^2 +      5*b^10*c^8*x*z^2 - 16*a^6*b^2*c^10*x*z^2 - 18*a^4*b^4*c^10*x*z^2 + 36*a^2*b^6*c^10*x*z^2 +      5*b^8*c^10*x*z^2 + 10*a^4*b^2*c^12*x*z^2 - 10*a^2*b^4*c^12*x*z^2 - 9*b^6*c^12*x*z^2 +      5*b^4*c^14*x*z^2 - b^2*c^16*x*z^2 + a^16*b^2*y*z^2 - a^14*b^4*y*z^2 - 9*a^12*b^6*y*z^2 +      25*a^10*b^8*y*z^2 - 25*a^8*b^10*y*z^2 + 9*a^6*b^12*y*z^2 + a^4*b^14*y*z^2 - a^2*b^16*y*z^2 +      a^16*c^2*y*z^2 + 6*a^12*b^4*c^2*y*z^2 - 20*a^10*b^6*c^2*y*z^2 + 9*a^8*b^8*c^2*y*z^2 +      12*a^6*b^10*c^2*y*z^2 - 8*a^4*b^12*c^2*y*z^2 - 5*a^14*c^4*y*z^2 + 18*a^10*b^4*c^4*y*z^2 +      28*a^8*b^6*c^4*y*z^2 - 51*a^6*b^8*c^4*y*z^2 + 10*a^2*b^12*c^4*y*z^2 + 9*a^12*c^6*y*z^2 -      20*a^10*b^2*c^6*y*z^2 - 52*a^8*b^4*c^6*y*z^2 + 36*a^6*b^6*c^6*y*z^2 + 43*a^4*b^8*c^6*y*z^2 -      16*a^2*b^10*c^6*y*z^2 - 5*a^10*c^8*y*z^2 + 45*a^8*b^2*c^8*y*z^2 + 21*a^6*b^4*c^8*y*z^2 -      59*a^4*b^6*c^8*y*z^2 - 5*a^8*c^10*y*z^2 - 36*a^6*b^2*c^10*y*z^2 + 18*a^4*b^4*c^10*y*z^2 +      16*a^2*b^6*c^10*y*z^2 + 9*a^6*c^12*y*z^2 + 10*a^4*b^2*c^12*y*z^2 - 10*a^2*b^4*c^12*y*z^2 -      5*a^4*c^14*y*z^2 + a^2*c^16*y*z^2");
		exp("K256","-(a^6*b^6*c^2*x^2*y) + 2*a^4*b^8*c^2*x^2*y - a^2*b^10*c^2*x^2*y + a^6*b^4*c^4*x^2*y -      5*a^4*b^6*c^4*x^2*y + a^2*b^8*c^4*x^2*y + b^10*c^4*x^2*y + a^6*b^2*c^6*x^2*y +      7*a^4*b^4*c^6*x^2*y + 4*a^2*b^6*c^6*x^2*y - 3*b^8*c^6*x^2*y - a^6*c^8*x^2*y - 7*a^4*b^2*c^8*x^2*y -      10*a^2*b^4*c^8*x^2*y + 2*b^6*c^8*x^2*y + 3*a^4*c^10*x^2*y + 9*a^2*b^2*c^10*x^2*y +      2*b^4*c^10*x^2*y - 3*a^2*c^12*x^2*y - 3*b^2*c^12*x^2*y + c^14*x^2*y - a^10*b^2*c^2*x*y^2 +      2*a^8*b^4*c^2*x*y^2 - a^6*b^6*c^2*x*y^2 + a^10*c^4*x*y^2 + a^8*b^2*c^4*x*y^2 -      5*a^6*b^4*c^4*x*y^2 + a^4*b^6*c^4*x*y^2 - 3*a^8*c^6*x*y^2 + 4*a^6*b^2*c^6*x*y^2 +      7*a^4*b^4*c^6*x*y^2 + a^2*b^6*c^6*x*y^2 + 2*a^6*c^8*x*y^2 - 10*a^4*b^2*c^8*x*y^2 -      7*a^2*b^4*c^8*x*y^2 - b^6*c^8*x*y^2 + 2*a^4*c^10*x*y^2 + 9*a^2*b^2*c^10*x*y^2 + 3*b^4*c^10*x*y^2 -      3*a^2*c^12*x*y^2 - 3*b^2*c^12*x*y^2 + c^14*x*y^2 - a^6*b^8*x^2*z + 3*a^4*b^10*x^2*z -      3*a^2*b^12*x^2*z + b^14*x^2*z + a^6*b^6*c^2*x^2*z - 7*a^4*b^8*c^2*x^2*z + 9*a^2*b^10*c^2*x^2*z -      3*b^12*c^2*x^2*z + a^6*b^4*c^4*x^2*z + 7*a^4*b^6*c^4*x^2*z - 10*a^2*b^8*c^4*x^2*z +      2*b^10*c^4*x^2*z - a^6*b^2*c^6*x^2*z - 5*a^4*b^4*c^6*x^2*z + 4*a^2*b^6*c^6*x^2*z +      2*b^8*c^6*x^2*z + 2*a^4*b^2*c^8*x^2*z + a^2*b^4*c^8*x^2*z - 3*b^6*c^8*x^2*z - a^2*b^2*c^10*x^2*z +      b^4*c^10*x^2*z + a^14*y^2*z - 3*a^12*b^2*y^2*z + 3*a^10*b^4*y^2*z - a^8*b^6*y^2*z -      3*a^12*c^2*y^2*z + 9*a^10*b^2*c^2*y^2*z - 7*a^8*b^4*c^2*y^2*z + a^6*b^6*c^2*y^2*z +      2*a^10*c^4*y^2*z - 10*a^8*b^2*c^4*y^2*z + 7*a^6*b^4*c^4*y^2*z + a^4*b^6*c^4*y^2*z +      2*a^8*c^6*y^2*z + 4*a^6*b^2*c^6*y^2*z - 5*a^4*b^4*c^6*y^2*z - a^2*b^6*c^6*y^2*z - 3*a^6*c^8*y^2*z +      a^4*b^2*c^8*y^2*z + 2*a^2*b^4*c^8*y^2*z + a^4*c^10*y^2*z - a^2*b^2*c^10*y^2*z + a^10*b^4*x*z^2 -      3*a^8*b^6*x*z^2 + 2*a^6*b^8*x*z^2 + 2*a^4*b^10*x*z^2 - 3*a^2*b^12*x*z^2 + b^14*x*z^2 -      a^10*b^2*c^2*x*z^2 + a^8*b^4*c^2*x*z^2 + 4*a^6*b^6*c^2*x*z^2 - 10*a^4*b^8*c^2*x*z^2 +      9*a^2*b^10*c^2*x*z^2 - 3*b^12*c^2*x*z^2 + 2*a^8*b^2*c^4*x*z^2 - 5*a^6*b^4*c^4*x*z^2 +      7*a^4*b^6*c^4*x*z^2 - 7*a^2*b^8*c^4*x*z^2 + 3*b^10*c^4*x*z^2 - a^6*b^2*c^6*x*z^2 +      a^4*b^4*c^6*x*z^2 + a^2*b^6*c^6*x*z^2 - b^8*c^6*x*z^2 + a^14*y*z^2 - 3*a^12*b^2*y*z^2 +      2*a^10*b^4*y*z^2 + 2*a^8*b^6*y*z^2 - 3*a^6*b^8*y*z^2 + a^4*b^10*y*z^2 - 3*a^12*c^2*y*z^2 +      9*a^10*b^2*c^2*y*z^2 - 10*a^8*b^4*c^2*y*z^2 + 4*a^6*b^6*c^2*y*z^2 + a^4*b^8*c^2*y*z^2 -      a^2*b^10*c^2*y*z^2 + 3*a^10*c^4*y*z^2 - 7*a^8*b^2*c^4*y*z^2 + 7*a^6*b^4*c^4*y*z^2 -      5*a^4*b^6*c^4*y*z^2 + 2*a^2*b^8*c^4*y*z^2 - a^8*c^6*y*z^2 + a^6*b^2*c^6*y*z^2 + a^4*b^4*c^6*y*z^2 -      a^2*b^6*c^6*y*z^2");
		exp("K257","-(a^4*c^2*x^2*y) - a^2*b^2*c^2*x^2*y + a^2*c^4*x^2*y + a^2*b^2*c^2*x*y^2 + b^4*c^2*x*y^2 -      b^2*c^4*x*y^2 + a^4*b^2*x^2*z - a^2*b^4*x^2*z + a^2*b^2*c^2*x^2*z + 2*a^4*b^2*x*y*z -      2*a^2*b^4*x*y*z - 2*a^4*c^2*x*y*z + 2*b^4*c^2*x*y*z + 2*a^2*c^4*x*y*z - 2*b^2*c^4*x*y*z +      a^4*b^2*y^2*z - a^2*b^4*y^2*z - a^2*b^2*c^2*y^2*z - a^2*b^2*c^2*x*z^2 + b^4*c^2*x*z^2 -      b^2*c^4*x*z^2 - a^4*c^2*y*z^2 + a^2*b^2*c^2*y*z^2 + a^2*c^4*y*z^2");
		exp("K258","b^4*c^2*x^3 - b^2*c^4*x^3 + 2*a^2*b^2*c^2*x^2*y - b^4*c^2*x^2*y + b^2*c^4*x^2*y + a^4*c^2*x*y^2 -      2*a^2*b^2*c^2*x*y^2 - a^2*c^4*x*y^2 - a^4*c^2*y^3 + a^2*c^4*y^3 - 2*a^2*b^2*c^2*x^2*z -      b^4*c^2*x^2*z + b^2*c^4*x^2*z + a^4*c^2*y^2*z + 2*a^2*b^2*c^2*y^2*z - a^2*c^4*y^2*z -      a^4*b^2*x*z^2 + a^2*b^4*x*z^2 + 2*a^2*b^2*c^2*x*z^2 - a^4*b^2*y*z^2 + a^2*b^4*y*z^2 -      2*a^2*b^2*c^2*y*z^2 + a^4*b^2*z^3 - a^2*b^4*z^3");
		exp("K259","a*c^2*x^2*y + b*c^2*x^2*y - c^3*x^2*y - a*c^2*x*y^2 - b*c^2*x*y^2 + c^3*x*y^2 - a*b^2*x^2*z +      b^3*x^2*z - b^2*c*x^2*z + 2*a^2*b*x*y*z - 2*a*b^2*x*y*z - 2*a^2*c*x*y*z + 2*b^2*c*x*y*z +      2*a*c^2*x*y*z - 2*b*c^2*x*y*z - a^3*y^2*z + a^2*b*y^2*z + a^2*c*y^2*z + a*b^2*x*z^2 - b^3*x*z^2 +      b^2*c*x*z^2 + a^3*y*z^2 - a^2*b*y*z^2 - a^2*c*y*z^2");
		exp("K260","a^2*c^4*x^2*y + b^2*c^4*x^2*y - c^6*x^2*y - a^2*c^4*x*y^2 - b^2*c^4*x*y^2 + c^6*x*y^2 -      a^2*b^4*x^2*z + b^6*x^2*z - b^4*c^2*x^2*z + 2*a^4*b^2*x*y*z - 2*a^2*b^4*x*y*z - 2*a^4*c^2*x*y*z +      2*b^4*c^2*x*y*z + 2*a^2*c^4*x*y*z - 2*b^2*c^4*x*y*z - a^6*y^2*z + a^4*b^2*y^2*z + a^4*c^2*y^2*z +      a^2*b^4*x*z^2 - b^6*x*z^2 + b^4*c^2*x*z^2 + a^6*y*z^2 - a^4*b^2*y*z^2 - a^4*c^2*y*z^2");
		exp("K261a","a^4*c^2*x^2*y + a^2*b^2*c^2*x^2*y - 2*b^4*c^2*x^2*y - 2*a^2*c^4*x^2*y + b^2*c^4*x^2*y +      c^6*x^2*y + 2*a^4*c^2*x*y^2 - a^2*b^2*c^2*x*y^2 - b^4*c^2*x*y^2 - a^2*c^4*x*y^2 + 2*b^2*c^4*x*y^2 -      c^6*x*y^2 - a^4*b^2*x^2*z + 2*a^2*b^4*x^2*z - b^6*x^2*z - a^2*b^2*c^2*x^2*z - b^4*c^2*x^2*z +      2*b^2*c^4*x^2*z + a^6*y^2*z - 2*a^4*b^2*y^2*z + a^2*b^4*y^2*z + a^4*c^2*y^2*z + a^2*b^2*c^2*y^2*z -      2*a^2*c^4*y^2*z - 2*a^4*b^2*x*z^2 + a^2*b^4*x*z^2 + b^6*x*z^2 + a^2*b^2*c^2*x*z^2 -      2*b^4*c^2*x*z^2 + b^2*c^4*x*z^2 - a^6*y*z^2 - a^4*b^2*y*z^2 + 2*a^2*b^4*y*z^2 + 2*a^4*c^2*y*z^2 -      a^2*b^2*c^2*y*z^2 - a^2*c^4*y*z^2 + Sqrt(3)*Sqrt((a + b - c)*(a - b + c)*(-a + b + c)*(a + b + c))*      ((-a^2)*c^2*x^2*y + c^4*x^2*y + b^2*c^2*x*y^2 - c^4*x*y^2 + a^2*b^2*x^2*z - b^4*x^2*z +        a^4*y^2*z - a^2*b^2*y^2*z + b^4*x*z^2 - b^2*c^2*x*z^2 - a^4*y*z^2 + a^2*c^2*y*z^2)");
		exp("K261b","(-a^4)*c^2*x^2*y - a^2*b^2*c^2*x^2*y + 2*b^4*c^2*x^2*y + 2*a^2*c^4*x^2*y - b^2*c^4*x^2*y -      c^6*x^2*y - 2*a^4*c^2*x*y^2 + a^2*b^2*c^2*x*y^2 + b^4*c^2*x*y^2 + a^2*c^4*x*y^2 - 2*b^2*c^4*x*y^2 +      c^6*x*y^2 + a^4*b^2*x^2*z - 2*a^2*b^4*x^2*z + b^6*x^2*z + a^2*b^2*c^2*x^2*z + b^4*c^2*x^2*z -      2*b^2*c^4*x^2*z - a^6*y^2*z + 2*a^4*b^2*y^2*z - a^2*b^4*y^2*z - a^4*c^2*y^2*z - a^2*b^2*c^2*y^2*z +      2*a^2*c^4*y^2*z + 2*a^4*b^2*x*z^2 - a^2*b^4*x*z^2 - b^6*x*z^2 - a^2*b^2*c^2*x*z^2 +      2*b^4*c^2*x*z^2 - b^2*c^4*x*z^2 + a^6*y*z^2 + a^4*b^2*y*z^2 - 2*a^2*b^4*y*z^2 - 2*a^4*c^2*y*z^2 +      a^2*b^2*c^2*y*z^2 + a^2*c^4*y*z^2 + Sqrt(3)*Sqrt((a + b - c)*(a - b + c)*(-a + b + c)*(a + b + c))*      ((-a^2)*c^2*x^2*y + c^4*x^2*y + b^2*c^2*x*y^2 - c^4*x*y^2 + a^2*b^2*x^2*z - b^4*x^2*z +        a^4*y^2*z - a^2*b^2*y^2*z + b^4*x*z^2 - b^2*c^2*x*z^2 - a^4*y*z^2 + a^2*c^2*y*z^2)");
		exp("K262a","Sqrt((a + b - c)*(a - b + c)*(-a + b + c)*(a + b + c))*((-a^2)*c^2*x^2*y + c^4*x^2*y +        b^2*c^2*x*y^2 - c^4*x*y^2 + a^2*b^2*x^2*z - b^4*x^2*z + a^4*y^2*z - a^2*b^2*y^2*z + b^4*x*z^2 -        b^2*c^2*x*z^2 - a^4*y*z^2 + a^2*c^2*y*z^2) + Sqrt(3)*((-a^4)*c^2*x^2*y - a^2*b^2*c^2*x^2*y +        2*b^4*c^2*x^2*y + 2*a^2*c^4*x^2*y - b^2*c^4*x^2*y - c^6*x^2*y - 2*a^4*c^2*x*y^2 +        a^2*b^2*c^2*x*y^2 + b^4*c^2*x*y^2 + a^2*c^4*x*y^2 - 2*b^2*c^4*x*y^2 + c^6*x*y^2 + a^4*b^2*x^2*z -        2*a^2*b^4*x^2*z + b^6*x^2*z + a^2*b^2*c^2*x^2*z + b^4*c^2*x^2*z - 2*b^2*c^4*x^2*z - a^6*y^2*z +        2*a^4*b^2*y^2*z - a^2*b^4*y^2*z - a^4*c^2*y^2*z - a^2*b^2*c^2*y^2*z + 2*a^2*c^4*y^2*z +        2*a^4*b^2*x*z^2 - a^2*b^4*x*z^2 - b^6*x*z^2 - a^2*b^2*c^2*x*z^2 + 2*b^4*c^2*x*z^2 -        b^2*c^4*x*z^2 + a^6*y*z^2 + a^4*b^2*y*z^2 - 2*a^2*b^4*y*z^2 - 2*a^4*c^2*y*z^2 +        a^2*b^2*c^2*y*z^2 + a^2*c^4*y*z^2)");
		exp("K262b","Sqrt((a + b - c)*(a - b + c)*(-a + b + c)*(a + b + c))*((-a^2)*c^2*x^2*y + c^4*x^2*y +        b^2*c^2*x*y^2 - c^4*x*y^2 + a^2*b^2*x^2*z - b^4*x^2*z + a^4*y^2*z - a^2*b^2*y^2*z + b^4*x*z^2 -        b^2*c^2*x*z^2 - a^4*y*z^2 + a^2*c^2*y*z^2) + Sqrt(3)*(a^4*c^2*x^2*y + a^2*b^2*c^2*x^2*y -        2*b^4*c^2*x^2*y - 2*a^2*c^4*x^2*y + b^2*c^4*x^2*y + c^6*x^2*y + 2*a^4*c^2*x*y^2 -        a^2*b^2*c^2*x*y^2 - b^4*c^2*x*y^2 - a^2*c^4*x*y^2 + 2*b^2*c^4*x*y^2 - c^6*x*y^2 - a^4*b^2*x^2*z +        2*a^2*b^4*x^2*z - b^6*x^2*z - a^2*b^2*c^2*x^2*z - b^4*c^2*x^2*z + 2*b^2*c^4*x^2*z + a^6*y^2*z -        2*a^4*b^2*y^2*z + a^2*b^4*y^2*z + a^4*c^2*y^2*z + a^2*b^2*c^2*y^2*z - 2*a^2*c^4*y^2*z -        2*a^4*b^2*x*z^2 + a^2*b^4*x*z^2 + b^6*x*z^2 + a^2*b^2*c^2*x*z^2 - 2*b^4*c^2*x*z^2 +        b^2*c^4*x*z^2 - a^6*y*z^2 - a^4*b^2*y*z^2 + 2*a^2*b^4*y*z^2 + 2*a^4*c^2*y*z^2 -        a^2*b^2*c^2*y*z^2 - a^2*c^4*y*z^2)");
		exp("K263","a^6*c^2*x^2*y - b^6*c^2*x^2*y - 2*a^4*c^4*x^2*y - b^4*c^4*x^2*y + a^2*c^6*x^2*y +      2*b^2*c^6*x^2*y + a^6*c^2*x*y^2 - b^6*c^2*x*y^2 + a^4*c^4*x*y^2 + 2*b^4*c^4*x*y^2 -      2*a^2*c^6*x*y^2 - b^2*c^6*x*y^2 - a^6*b^2*x^2*z + 2*a^4*b^4*x^2*z - a^2*b^6*x^2*z -      2*b^6*c^2*x^2*z + b^4*c^4*x^2*z + b^2*c^6*x^2*z + a^6*b^2*y^2*z - 2*a^4*b^4*y^2*z + a^2*b^6*y^2*z +      2*a^6*c^2*y^2*z - a^4*c^4*y^2*z - a^2*c^6*y^2*z - a^6*b^2*x*z^2 - a^4*b^4*x*z^2 + 2*a^2*b^6*x*z^2 +      b^6*c^2*x*z^2 - 2*b^4*c^4*x*z^2 + b^2*c^6*x*z^2 - 2*a^6*b^2*y*z^2 + a^4*b^4*y*z^2 + a^2*b^6*y*z^2 -      a^6*c^2*y*z^2 + 2*a^4*c^4*y*z^2 - a^2*c^6*y*z^2");
		exp("K264a","(-a^8)*x^2*y + a^6*b^2*x^2*y + 3*a^4*b^4*x^2*y - 5*a^2*b^6*x^2*y + 2*b^8*x^2*y -      2*a^6*c^2*x^2*y + 2*a^4*b^2*c^2*x^2*y + 5*a^2*b^4*c^2*x^2*y - 5*b^6*c^2*x^2*y + 6*a^4*c^4*x^2*y +      2*a^2*b^2*c^4*x^2*y + 3*b^4*c^4*x^2*y - 2*a^2*c^6*x^2*y + b^2*c^6*x^2*y - c^8*x^2*y - 2*a^8*x*y^2 +      5*a^6*b^2*x*y^2 - 3*a^4*b^4*x*y^2 - a^2*b^6*x*y^2 + b^8*x*y^2 + 5*a^6*c^2*x*y^2 -      5*a^4*b^2*c^2*x*y^2 - 2*a^2*b^4*c^2*x*y^2 + 2*b^6*c^2*x*y^2 - 3*a^4*c^4*x*y^2 -      2*a^2*b^2*c^4*x*y^2 - 6*b^4*c^4*x*y^2 - a^2*c^6*x*y^2 + 2*b^2*c^6*x*y^2 + c^8*x*y^2 + a^8*x^2*z +      2*a^6*b^2*x^2*z - 6*a^4*b^4*x^2*z + 2*a^2*b^6*x^2*z + b^8*x^2*z - a^6*c^2*x^2*z -      2*a^4*b^2*c^2*x^2*z - 2*a^2*b^4*c^2*x^2*z - b^6*c^2*x^2*z - 3*a^4*c^4*x^2*z - 5*a^2*b^2*c^4*x^2*z -      3*b^4*c^4*x^2*z + 5*a^2*c^6*x^2*z + 5*b^2*c^6*x^2*z - 2*c^8*x^2*z - a^8*y^2*z - 2*a^6*b^2*y^2*z +      6*a^4*b^4*y^2*z - 2*a^2*b^6*y^2*z - b^8*y^2*z + a^6*c^2*y^2*z + 2*a^4*b^2*c^2*y^2*z +      2*a^2*b^4*c^2*y^2*z + b^6*c^2*y^2*z + 3*a^4*c^4*y^2*z + 5*a^2*b^2*c^4*y^2*z + 3*b^4*c^4*y^2*z -      5*a^2*c^6*y^2*z - 5*b^2*c^6*y^2*z + 2*c^8*y^2*z + 2*a^8*x*z^2 - 5*a^6*b^2*x*z^2 + 3*a^4*b^4*x*z^2 +      a^2*b^6*x*z^2 - b^8*x*z^2 - 5*a^6*c^2*x*z^2 + 5*a^4*b^2*c^2*x*z^2 + 2*a^2*b^4*c^2*x*z^2 -      2*b^6*c^2*x*z^2 + 3*a^4*c^4*x*z^2 + 2*a^2*b^2*c^4*x*z^2 + 6*b^4*c^4*x*z^2 + a^2*c^6*x*z^2 -      2*b^2*c^6*x*z^2 - c^8*x*z^2 + a^8*y*z^2 - a^6*b^2*y*z^2 - 3*a^4*b^4*y*z^2 + 5*a^2*b^6*y*z^2 -      2*b^8*y*z^2 + 2*a^6*c^2*y*z^2 - 2*a^4*b^2*c^2*y*z^2 - 5*a^2*b^4*c^2*y*z^2 + 5*b^6*c^2*y*z^2 -      6*a^4*c^4*y*z^2 - 2*a^2*b^2*c^4*y*z^2 - 3*b^4*c^4*y*z^2 + 2*a^2*c^6*y*z^2 - b^2*c^6*y*z^2 +      c^8*y*z^2 + Sqrt(3)*Sqrt((a + b - c)*(a - b + c)*(-a + b + c)*(a + b + c))*      ((-a^6)*x^2*y + 2*a^4*b^2*x^2*y - a^2*b^4*x^2*y + a^4*c^2*x^2*y + a^2*b^2*c^2*x^2*y -        b^4*c^2*x^2*y + a^2*c^4*x^2*y + 2*b^2*c^4*x^2*y - c^6*x^2*y + a^4*b^2*x*y^2 - 2*a^2*b^4*x*y^2 +        b^6*x*y^2 + a^4*c^2*x*y^2 - a^2*b^2*c^2*x*y^2 - b^4*c^2*x*y^2 - 2*a^2*c^4*x*y^2 - b^2*c^4*x*y^2 +        c^6*x*y^2 + a^6*x^2*z - a^4*b^2*x^2*z - a^2*b^4*x^2*z + b^6*x^2*z - 2*a^4*c^2*x^2*z -        a^2*b^2*c^2*x^2*z - 2*b^4*c^2*x^2*z + a^2*c^4*x^2*z + b^2*c^4*x^2*z - a^6*y^2*z + a^4*b^2*y^2*z +        a^2*b^4*y^2*z - b^6*y^2*z + 2*a^4*c^2*y^2*z + a^2*b^2*c^2*y^2*z + 2*b^4*c^2*y^2*z -        a^2*c^4*y^2*z - b^2*c^4*y^2*z - a^4*b^2*x*z^2 + 2*a^2*b^4*x*z^2 - b^6*x*z^2 - a^4*c^2*x*z^2 +        a^2*b^2*c^2*x*z^2 + b^4*c^2*x*z^2 + 2*a^2*c^4*x*z^2 + b^2*c^4*x*z^2 - c^6*x*z^2 + a^6*y*z^2 -        2*a^4*b^2*y*z^2 + a^2*b^4*y*z^2 - a^4*c^2*y*z^2 - a^2*b^2*c^2*y*z^2 + b^4*c^2*y*z^2 -        a^2*c^4*y*z^2 - 2*b^2*c^4*y*z^2 + c^6*y*z^2)");
		exp("K264b","(-a^8)*x^2*y + a^6*b^2*x^2*y + 3*a^4*b^4*x^2*y - 5*a^2*b^6*x^2*y + 2*b^8*x^2*y -      2*a^6*c^2*x^2*y + 2*a^4*b^2*c^2*x^2*y + 5*a^2*b^4*c^2*x^2*y - 5*b^6*c^2*x^2*y + 6*a^4*c^4*x^2*y +      2*a^2*b^2*c^4*x^2*y + 3*b^4*c^4*x^2*y - 2*a^2*c^6*x^2*y + b^2*c^6*x^2*y - c^8*x^2*y - 2*a^8*x*y^2 +      5*a^6*b^2*x*y^2 - 3*a^4*b^4*x*y^2 - a^2*b^6*x*y^2 + b^8*x*y^2 + 5*a^6*c^2*x*y^2 -      5*a^4*b^2*c^2*x*y^2 - 2*a^2*b^4*c^2*x*y^2 + 2*b^6*c^2*x*y^2 - 3*a^4*c^4*x*y^2 -      2*a^2*b^2*c^4*x*y^2 - 6*b^4*c^4*x*y^2 - a^2*c^6*x*y^2 + 2*b^2*c^6*x*y^2 + c^8*x*y^2 + a^8*x^2*z +      2*a^6*b^2*x^2*z - 6*a^4*b^4*x^2*z + 2*a^2*b^6*x^2*z + b^8*x^2*z - a^6*c^2*x^2*z -      2*a^4*b^2*c^2*x^2*z - 2*a^2*b^4*c^2*x^2*z - b^6*c^2*x^2*z - 3*a^4*c^4*x^2*z - 5*a^2*b^2*c^4*x^2*z -      3*b^4*c^4*x^2*z + 5*a^2*c^6*x^2*z + 5*b^2*c^6*x^2*z - 2*c^8*x^2*z - a^8*y^2*z - 2*a^6*b^2*y^2*z +      6*a^4*b^4*y^2*z - 2*a^2*b^6*y^2*z - b^8*y^2*z + a^6*c^2*y^2*z + 2*a^4*b^2*c^2*y^2*z +      2*a^2*b^4*c^2*y^2*z + b^6*c^2*y^2*z + 3*a^4*c^4*y^2*z + 5*a^2*b^2*c^4*y^2*z + 3*b^4*c^4*y^2*z -      5*a^2*c^6*y^2*z - 5*b^2*c^6*y^2*z + 2*c^8*y^2*z + 2*a^8*x*z^2 - 5*a^6*b^2*x*z^2 + 3*a^4*b^4*x*z^2 +      a^2*b^6*x*z^2 - b^8*x*z^2 - 5*a^6*c^2*x*z^2 + 5*a^4*b^2*c^2*x*z^2 + 2*a^2*b^4*c^2*x*z^2 -      2*b^6*c^2*x*z^2 + 3*a^4*c^4*x*z^2 + 2*a^2*b^2*c^4*x*z^2 + 6*b^4*c^4*x*z^2 + a^2*c^6*x*z^2 -      2*b^2*c^6*x*z^2 - c^8*x*z^2 + a^8*y*z^2 - a^6*b^2*y*z^2 - 3*a^4*b^4*y*z^2 + 5*a^2*b^6*y*z^2 -      2*b^8*y*z^2 + 2*a^6*c^2*y*z^2 - 2*a^4*b^2*c^2*y*z^2 - 5*a^2*b^4*c^2*y*z^2 + 5*b^6*c^2*y*z^2 -      6*a^4*c^4*y*z^2 - 2*a^2*b^2*c^4*y*z^2 - 3*b^4*c^4*y*z^2 + 2*a^2*c^6*y*z^2 - b^2*c^6*y*z^2 +      c^8*y*z^2 + Sqrt(3)*Sqrt((a + b - c)*(a - b + c)*(-a + b + c)*(a + b + c))*      (a^6*x^2*y - 2*a^4*b^2*x^2*y + a^2*b^4*x^2*y - a^4*c^2*x^2*y - a^2*b^2*c^2*x^2*y + b^4*c^2*x^2*y -        a^2*c^4*x^2*y - 2*b^2*c^4*x^2*y + c^6*x^2*y - a^4*b^2*x*y^2 + 2*a^2*b^4*x*y^2 - b^6*x*y^2 -        a^4*c^2*x*y^2 + a^2*b^2*c^2*x*y^2 + b^4*c^2*x*y^2 + 2*a^2*c^4*x*y^2 + b^2*c^4*x*y^2 - c^6*x*y^2 -        a^6*x^2*z + a^4*b^2*x^2*z + a^2*b^4*x^2*z - b^6*x^2*z + 2*a^4*c^2*x^2*z + a^2*b^2*c^2*x^2*z +        2*b^4*c^2*x^2*z - a^2*c^4*x^2*z - b^2*c^4*x^2*z + a^6*y^2*z - a^4*b^2*y^2*z - a^2*b^4*y^2*z +        b^6*y^2*z - 2*a^4*c^2*y^2*z - a^2*b^2*c^2*y^2*z - 2*b^4*c^2*y^2*z + a^2*c^4*y^2*z +        b^2*c^4*y^2*z + a^4*b^2*x*z^2 - 2*a^2*b^4*x*z^2 + b^6*x*z^2 + a^4*c^2*x*z^2 - a^2*b^2*c^2*x*z^2 -        b^4*c^2*x*z^2 - 2*a^2*c^4*x*z^2 - b^2*c^4*x*z^2 + c^6*x*z^2 - a^6*y*z^2 + 2*a^4*b^2*y*z^2 -        a^2*b^4*y*z^2 + a^4*c^2*y*z^2 + a^2*b^2*c^2*y*z^2 - b^4*c^2*y*z^2 + a^2*c^4*y*z^2 +        2*b^2*c^4*y*z^2 - c^6*y*z^2)");
		exp("K265","a^10*b^2*x^3 - 4*a^8*b^4*x^3 + 6*a^6*b^6*x^3 - 4*a^4*b^8*x^3 + a^2*b^10*x^3 - a^10*c^2*x^3 +      2*a^6*b^4*c^2*x^3 - a^2*b^8*c^2*x^3 + 4*a^8*c^4*x^3 - 2*a^6*b^2*c^4*x^3 - 2*a^2*b^6*c^4*x^3 -      6*a^6*c^6*x^3 + 2*a^2*b^4*c^6*x^3 + 4*a^4*c^8*x^3 + a^2*b^2*c^8*x^3 - a^2*c^10*x^3 -      a^8*b^2*c^2*x^2*y + 2*a^4*b^6*c^2*x^2*y - b^10*c^2*x^2*y - a^8*c^4*x^2*y - 2*a^4*b^4*c^4*x^2*y +      3*b^8*c^4*x^2*y + 2*a^6*c^6*x^2*y - 2*a^2*b^4*c^6*x^2*y - 4*b^6*c^6*x^2*y + 4*a^2*b^2*c^8*x^2*y +      4*b^4*c^8*x^2*y - 2*a^2*c^10*x^2*y - 3*b^2*c^10*x^2*y + c^12*x^2*y + a^10*c^2*x*y^2 -      2*a^6*b^4*c^2*x*y^2 + a^2*b^8*c^2*x*y^2 - 3*a^8*c^4*x*y^2 + 2*a^4*b^4*c^4*x*y^2 + b^8*c^4*x*y^2 +      4*a^6*c^6*x*y^2 + 2*a^4*b^2*c^6*x*y^2 - 2*b^6*c^6*x*y^2 - 4*a^4*c^8*x*y^2 - 4*a^2*b^2*c^8*x*y^2 +      3*a^2*c^10*x*y^2 + 2*b^2*c^10*x*y^2 - c^12*x*y^2 - a^10*b^2*y^3 + 4*a^8*b^4*y^3 - 6*a^6*b^6*y^3 +      4*a^4*b^8*y^3 - a^2*b^10*y^3 + a^8*b^2*c^2*y^3 - 2*a^4*b^6*c^2*y^3 + b^10*c^2*y^3 +      2*a^6*b^2*c^4*y^3 + 2*a^2*b^6*c^4*y^3 - 4*b^8*c^4*y^3 - 2*a^4*b^2*c^6*y^3 + 6*b^6*c^6*y^3 -      a^2*b^2*c^8*y^3 - 4*b^4*c^8*y^3 + b^2*c^10*y^3 + a^8*b^4*x^2*z - 2*a^6*b^6*x^2*z +      2*a^2*b^10*x^2*z - b^12*x^2*z + a^8*b^2*c^2*x^2*z - 4*a^2*b^8*c^2*x^2*z + 3*b^10*c^2*x^2*z +      2*a^4*b^4*c^4*x^2*z + 2*a^2*b^6*c^4*x^2*z - 4*b^8*c^4*x^2*z - 2*a^4*b^2*c^6*x^2*z +      4*b^6*c^6*x^2*z - 3*b^4*c^8*x^2*z + b^2*c^10*x^2*z + 2*a^10*b^2*x*y*z - 4*a^8*b^4*x*y*z +      4*a^4*b^8*x*y*z - 2*a^2*b^10*x*y*z - 2*a^10*c^2*x*y*z + 10*a^6*b^4*c^2*x*y*z -      10*a^4*b^6*c^2*x*y*z + 2*b^10*c^2*x*y*z + 4*a^8*c^4*x*y*z - 10*a^6*b^2*c^4*x*y*z +      10*a^2*b^6*c^4*x*y*z - 4*b^8*c^4*x*y*z + 10*a^4*b^2*c^6*x*y*z - 10*a^2*b^4*c^6*x*y*z -      4*a^4*c^8*x*y*z + 4*b^4*c^8*x*y*z + 2*a^2*c^10*x*y*z - 2*b^2*c^10*x*y*z + a^12*y^2*z -      2*a^10*b^2*y^2*z + 2*a^6*b^6*y^2*z - a^4*b^8*y^2*z - 3*a^10*c^2*y^2*z + 4*a^8*b^2*c^2*y^2*z -      a^2*b^8*c^2*y^2*z + 4*a^8*c^4*y^2*z - 2*a^6*b^2*c^4*y^2*z - 2*a^4*b^4*c^4*y^2*z - 4*a^6*c^6*y^2*z +      2*a^2*b^4*c^6*y^2*z + 3*a^4*c^8*y^2*z - a^2*c^10*y^2*z - a^10*b^2*x*z^2 + 3*a^8*b^4*x*z^2 -      4*a^6*b^6*x*z^2 + 4*a^4*b^8*x*z^2 - 3*a^2*b^10*x*z^2 + b^12*x*z^2 - 2*a^4*b^6*c^2*x*z^2 +      4*a^2*b^8*c^2*x*z^2 - 2*b^10*c^2*x*z^2 + 2*a^6*b^2*c^4*x*z^2 - 2*a^4*b^4*c^4*x*z^2 +      2*b^6*c^6*x*z^2 - a^2*b^2*c^8*x*z^2 - b^4*c^8*x*z^2 - a^12*y*z^2 + 3*a^10*b^2*y*z^2 -      4*a^8*b^4*y*z^2 + 4*a^6*b^6*y*z^2 - 3*a^4*b^8*y*z^2 + a^2*b^10*y*z^2 + 2*a^10*c^2*y*z^2 -      4*a^8*b^2*c^2*y*z^2 + 2*a^6*b^4*c^2*y*z^2 + 2*a^4*b^4*c^4*y*z^2 - 2*a^2*b^6*c^4*y*z^2 -      2*a^6*c^6*y*z^2 + a^4*c^8*y*z^2 + a^2*b^2*c^8*y*z^2 + a^10*c^2*z^3 - a^8*b^2*c^2*z^3 -      2*a^6*b^4*c^2*z^3 + 2*a^4*b^6*c^2*z^3 + a^2*b^8*c^2*z^3 - b^10*c^2*z^3 - 4*a^8*c^4*z^3 +      4*b^8*c^4*z^3 + 6*a^6*c^6*z^3 + 2*a^4*b^2*c^6*z^3 - 2*a^2*b^4*c^6*z^3 - 6*b^6*c^6*z^3 -      4*a^4*c^8*z^3 + 4*b^4*c^8*z^3 + a^2*c^10*z^3 - b^2*c^10*z^3");
		exp("K266","5*a^16*b^2*x^3 - 31*a^14*b^4*x^3 + 77*a^12*b^6*x^3 - 91*a^10*b^8*x^3 + 35*a^8*b^10*x^3 +      35*a^6*b^12*x^3 - 49*a^4*b^14*x^3 + 23*a^2*b^16*x^3 - 4*b^18*x^3 - 5*a^16*c^2*x^3 +      51*a^12*b^4*c^2*x^3 - 98*a^10*b^6*c^2*x^3 + 75*a^8*b^8*c^2*x^3 - 60*a^6*b^10*c^2*x^3 +      85*a^4*b^12*c^2*x^3 - 66*a^2*b^14*c^2*x^3 + 18*b^16*c^2*x^3 + 31*a^14*c^4*x^3 -      51*a^12*b^2*c^4*x^3 + 16*a^8*b^6*c^4*x^3 + 15*a^6*b^8*c^4*x^3 - 27*a^4*b^10*c^4*x^3 +      34*a^2*b^12*c^4*x^3 - 18*b^14*c^4*x^3 - 77*a^12*c^6*x^3 + 98*a^10*b^2*c^6*x^3 -      16*a^8*b^4*c^6*x^3 - a^4*b^8*c^6*x^3 + 38*a^2*b^10*c^6*x^3 - 42*b^12*c^6*x^3 + 91*a^10*c^8*x^3 -      75*a^8*b^2*c^8*x^3 - 15*a^6*b^4*c^8*x^3 + a^4*b^6*c^8*x^3 + 126*b^10*c^8*x^3 - 35*a^8*c^10*x^3 +      60*a^6*b^2*c^10*x^3 + 27*a^4*b^4*c^10*x^3 - 38*a^2*b^6*c^10*x^3 - 126*b^8*c^10*x^3 -      35*a^6*c^12*x^3 - 85*a^4*b^2*c^12*x^3 - 34*a^2*b^4*c^12*x^3 + 42*b^6*c^12*x^3 + 49*a^4*c^14*x^3 +      66*a^2*b^2*c^14*x^3 + 18*b^4*c^14*x^3 - 23*a^2*c^16*x^3 - 18*b^2*c^16*x^3 + 4*c^18*x^3 +      4*a^18*x^2*y - 18*a^16*b^2*x^2*y + 18*a^14*b^4*x^2*y + 42*a^12*b^6*x^2*y - 126*a^10*b^8*x^2*y +      126*a^8*b^10*x^2*y - 42*a^6*b^12*x^2*y - 18*a^4*b^14*x^2*y + 18*a^2*b^16*x^2*y - 4*b^18*x^2*y -      15*a^16*c^2*x^2*y + 39*a^14*b^2*c^2*x^2*y - 23*a^12*b^4*c^2*x^2*y + 24*a^10*b^6*c^2*x^2*y -      90*a^8*b^8*c^2*x^2*y + 55*a^6*b^10*c^2*x^2*y + 81*a^4*b^12*c^2*x^2*y - 102*a^2*b^14*c^2*x^2*y +      31*b^16*c^2*x^2*y + 14*a^14*c^4*x^2*y - 45*a^12*b^2*c^4*x^2*y - 9*a^10*b^4*c^4*x^2*y +      95*a^8*b^6*c^4*x^2*y + 48*a^6*b^8*c^4*x^2*y - 243*a^4*b^10*c^4*x^2*y + 179*a^2*b^12*c^4*x^2*y -      39*b^14*c^4*x^2*y + 7*a^12*c^6*x^2*y + 48*a^10*b^2*c^6*x^2*y - 114*a^8*b^4*c^6*x^2*y -      186*a^6*b^6*c^6*x^2*y + 393*a^4*b^8*c^6*x^2*y - 54*a^2*b^10*c^6*x^2*y - 94*b^12*c^6*x^2*y +      72*a^8*b^2*c^8*x^2*y + 280*a^6*b^4*c^8*x^2*y - 246*a^4*b^6*c^8*x^2*y - 228*a^2*b^8*c^8*x^2*y +      250*b^10*c^8*x^2*y - 49*a^8*c^10*x^2*y - 225*a^6*b^2*c^10*x^2*y - 51*a^4*b^4*c^10*x^2*y +      362*a^2*b^6*c^10*x^2*y - 141*b^8*c^10*x^2*y + 70*a^6*c^12*x^2*y + 123*a^4*b^2*c^12*x^2*y -      225*a^2*b^4*c^12*x^2*y - 91*b^6*c^12*x^2*y - 39*a^4*c^14*x^2*y + 42*a^2*b^2*c^14*x^2*y +      124*b^4*c^14*x^2*y + 8*a^2*c^16*x^2*y - 36*b^2*c^16*x^2*y + 4*a^18*x*y^2 - 18*a^16*b^2*x*y^2 +      18*a^14*b^4*x*y^2 + 42*a^12*b^6*x*y^2 - 126*a^10*b^8*x*y^2 + 126*a^8*b^10*x*y^2 -      42*a^6*b^12*x*y^2 - 18*a^4*b^14*x*y^2 + 18*a^2*b^16*x*y^2 - 4*b^18*x*y^2 - 31*a^16*c^2*x*y^2 +      102*a^14*b^2*c^2*x*y^2 - 81*a^12*b^4*c^2*x*y^2 - 55*a^10*b^6*c^2*x*y^2 + 90*a^8*b^8*c^2*x*y^2 -      24*a^6*b^10*c^2*x*y^2 + 23*a^4*b^12*c^2*x*y^2 - 39*a^2*b^14*c^2*x*y^2 + 15*b^16*c^2*x*y^2 +      39*a^14*c^4*x*y^2 - 179*a^12*b^2*c^4*x*y^2 + 243*a^10*b^4*c^4*x*y^2 - 48*a^8*b^6*c^4*x*y^2 -      95*a^6*b^8*c^4*x*y^2 + 9*a^4*b^10*c^4*x*y^2 + 45*a^2*b^12*c^4*x*y^2 - 14*b^14*c^4*x*y^2 +      94*a^12*c^6*x*y^2 + 54*a^10*b^2*c^6*x*y^2 - 393*a^8*b^4*c^6*x*y^2 + 186*a^6*b^6*c^6*x*y^2 +      114*a^4*b^8*c^6*x*y^2 - 48*a^2*b^10*c^6*x*y^2 - 7*b^12*c^6*x*y^2 - 250*a^10*c^8*x*y^2 +      228*a^8*b^2*c^8*x*y^2 + 246*a^6*b^4*c^8*x*y^2 - 280*a^4*b^6*c^8*x*y^2 - 72*a^2*b^8*c^8*x*y^2 +      141*a^8*c^10*x*y^2 - 362*a^6*b^2*c^10*x*y^2 + 51*a^4*b^4*c^10*x*y^2 + 225*a^2*b^6*c^10*x*y^2 +      49*b^8*c^10*x*y^2 + 91*a^6*c^12*x*y^2 + 225*a^4*b^2*c^12*x*y^2 - 123*a^2*b^4*c^12*x*y^2 -      70*b^6*c^12*x*y^2 - 124*a^4*c^14*x*y^2 - 42*a^2*b^2*c^14*x*y^2 + 39*b^4*c^14*x*y^2 +      36*a^2*c^16*x*y^2 - 8*b^2*c^16*x*y^2 + 4*a^18*y^3 - 23*a^16*b^2*y^3 + 49*a^14*b^4*y^3 -      35*a^12*b^6*y^3 - 35*a^10*b^8*y^3 + 91*a^8*b^10*y^3 - 77*a^6*b^12*y^3 + 31*a^4*b^14*y^3 -      5*a^2*b^16*y^3 - 18*a^16*c^2*y^3 + 66*a^14*b^2*c^2*y^3 - 85*a^12*b^4*c^2*y^3 +      60*a^10*b^6*c^2*y^3 - 75*a^8*b^8*c^2*y^3 + 98*a^6*b^10*c^2*y^3 - 51*a^4*b^12*c^2*y^3 +      5*b^16*c^2*y^3 + 18*a^14*c^4*y^3 - 34*a^12*b^2*c^4*y^3 + 27*a^10*b^4*c^4*y^3 - 15*a^8*b^6*c^4*y^3 -      16*a^6*b^8*c^4*y^3 + 51*a^2*b^12*c^4*y^3 - 31*b^14*c^4*y^3 + 42*a^12*c^6*y^3 -      38*a^10*b^2*c^6*y^3 + a^8*b^4*c^6*y^3 + 16*a^4*b^8*c^6*y^3 - 98*a^2*b^10*c^6*y^3 +      77*b^12*c^6*y^3 - 126*a^10*c^8*y^3 - a^6*b^4*c^8*y^3 + 15*a^4*b^6*c^8*y^3 + 75*a^2*b^8*c^8*y^3 -      91*b^10*c^8*y^3 + 126*a^8*c^10*y^3 + 38*a^6*b^2*c^10*y^3 - 27*a^4*b^4*c^10*y^3 -      60*a^2*b^6*c^10*y^3 + 35*b^8*c^10*y^3 - 42*a^6*c^12*y^3 + 34*a^4*b^2*c^12*y^3 +      85*a^2*b^4*c^12*y^3 + 35*b^6*c^12*y^3 - 18*a^4*c^14*y^3 - 66*a^2*b^2*c^14*y^3 - 49*b^4*c^14*y^3 +      18*a^2*c^16*y^3 + 23*b^2*c^16*y^3 - 4*c^18*y^3 - 4*a^18*x^2*z + 15*a^16*b^2*x^2*z -      14*a^14*b^4*x^2*z - 7*a^12*b^6*x^2*z + 49*a^8*b^10*x^2*z - 70*a^6*b^12*x^2*z + 39*a^4*b^14*x^2*z -      8*a^2*b^16*x^2*z + 18*a^16*c^2*x^2*z - 39*a^14*b^2*c^2*x^2*z + 45*a^12*b^4*c^2*x^2*z -      48*a^10*b^6*c^2*x^2*z - 72*a^8*b^8*c^2*x^2*z + 225*a^6*b^10*c^2*x^2*z - 123*a^4*b^12*c^2*x^2*z -      42*a^2*b^14*c^2*x^2*z + 36*b^16*c^2*x^2*z - 18*a^14*c^4*x^2*z + 23*a^12*b^2*c^4*x^2*z +      9*a^10*b^4*c^4*x^2*z + 114*a^8*b^6*c^4*x^2*z - 280*a^6*b^8*c^4*x^2*z + 51*a^4*b^10*c^4*x^2*z +      225*a^2*b^12*c^4*x^2*z - 124*b^14*c^4*x^2*z - 42*a^12*c^6*x^2*z - 24*a^10*b^2*c^6*x^2*z -      95*a^8*b^4*c^6*x^2*z + 186*a^6*b^6*c^6*x^2*z + 246*a^4*b^8*c^6*x^2*z - 362*a^2*b^10*c^6*x^2*z +      91*b^12*c^6*x^2*z + 126*a^10*c^8*x^2*z + 90*a^8*b^2*c^8*x^2*z - 48*a^6*b^4*c^8*x^2*z -      393*a^4*b^6*c^8*x^2*z + 228*a^2*b^8*c^8*x^2*z + 141*b^10*c^8*x^2*z - 126*a^8*c^10*x^2*z -      55*a^6*b^2*c^10*x^2*z + 243*a^4*b^4*c^10*x^2*z + 54*a^2*b^6*c^10*x^2*z - 250*b^8*c^10*x^2*z +      42*a^6*c^12*x^2*z - 81*a^4*b^2*c^12*x^2*z - 179*a^2*b^4*c^12*x^2*z + 94*b^6*c^12*x^2*z +      18*a^4*c^14*x^2*z + 102*a^2*b^2*c^14*x^2*z + 39*b^4*c^14*x^2*z - 18*a^2*c^16*x^2*z -      31*b^2*c^16*x^2*z + 4*c^18*x^2*z + 2*a^16*b^2*x*y*z - 14*a^14*b^4*x*y*z + 42*a^12*b^6*x*y*z -      70*a^10*b^8*x*y*z + 70*a^8*b^10*x*y*z - 42*a^6*b^12*x*y*z + 14*a^4*b^14*x*y*z - 2*a^2*b^16*x*y*z -      2*a^16*c^2*x*y*z + 20*a^12*b^4*c^2*x*y*z - 32*a^10*b^6*c^2*x*y*z + 32*a^6*b^10*c^2*x*y*z -      20*a^4*b^12*c^2*x*y*z + 2*b^16*c^2*x*y*z + 14*a^14*c^4*x*y*z - 20*a^12*b^2*c^4*x*y*z +      2*a^8*b^6*c^4*x*y*z - 2*a^6*b^8*c^4*x*y*z + 20*a^2*b^12*c^4*x*y*z - 14*b^14*c^4*x*y*z -      42*a^12*c^6*x*y*z + 32*a^10*b^2*c^6*x*y*z - 2*a^8*b^4*c^6*x*y*z + 2*a^4*b^8*c^6*x*y*z -      32*a^2*b^10*c^6*x*y*z + 42*b^12*c^6*x*y*z + 70*a^10*c^8*x*y*z + 2*a^6*b^4*c^8*x*y*z -      2*a^4*b^6*c^8*x*y*z - 70*b^10*c^8*x*y*z - 70*a^8*c^10*x*y*z - 32*a^6*b^2*c^10*x*y*z +      32*a^2*b^6*c^10*x*y*z + 70*b^8*c^10*x*y*z + 42*a^6*c^12*x*y*z + 20*a^4*b^2*c^12*x*y*z -      20*a^2*b^4*c^12*x*y*z - 42*b^6*c^12*x*y*z - 14*a^4*c^14*x*y*z + 14*b^4*c^14*x*y*z +      2*a^2*c^16*x*y*z - 2*b^2*c^16*x*y*z + 8*a^16*b^2*y^2*z - 39*a^14*b^4*y^2*z + 70*a^12*b^6*y^2*z -      49*a^10*b^8*y^2*z + 7*a^6*b^12*y^2*z + 14*a^4*b^14*y^2*z - 15*a^2*b^16*y^2*z + 4*b^18*y^2*z -      36*a^16*c^2*y^2*z + 42*a^14*b^2*c^2*y^2*z + 123*a^12*b^4*c^2*y^2*z - 225*a^10*b^6*c^2*y^2*z +      72*a^8*b^8*c^2*y^2*z + 48*a^6*b^10*c^2*y^2*z - 45*a^4*b^12*c^2*y^2*z + 39*a^2*b^14*c^2*y^2*z -      18*b^16*c^2*y^2*z + 124*a^14*c^4*y^2*z - 225*a^12*b^2*c^4*y^2*z - 51*a^10*b^4*c^4*y^2*z +      280*a^8*b^6*c^4*y^2*z - 114*a^6*b^8*c^4*y^2*z - 9*a^4*b^10*c^4*y^2*z - 23*a^2*b^12*c^4*y^2*z +      18*b^14*c^4*y^2*z - 91*a^12*c^6*y^2*z + 362*a^10*b^2*c^6*y^2*z - 246*a^8*b^4*c^6*y^2*z -      186*a^6*b^6*c^6*y^2*z + 95*a^4*b^8*c^6*y^2*z + 24*a^2*b^10*c^6*y^2*z + 42*b^12*c^6*y^2*z -      141*a^10*c^8*y^2*z - 228*a^8*b^2*c^8*y^2*z + 393*a^6*b^4*c^8*y^2*z + 48*a^4*b^6*c^8*y^2*z -      90*a^2*b^8*c^8*y^2*z - 126*b^10*c^8*y^2*z + 250*a^8*c^10*y^2*z - 54*a^6*b^2*c^10*y^2*z -      243*a^4*b^4*c^10*y^2*z + 55*a^2*b^6*c^10*y^2*z + 126*b^8*c^10*y^2*z - 94*a^6*c^12*y^2*z +      179*a^4*b^2*c^12*y^2*z + 81*a^2*b^4*c^12*y^2*z - 42*b^6*c^12*y^2*z - 39*a^4*c^14*y^2*z -      102*a^2*b^2*c^14*y^2*z - 18*b^4*c^14*y^2*z + 31*a^2*c^16*y^2*z + 18*b^2*c^16*y^2*z - 4*c^18*y^2*z -      4*a^18*x*z^2 + 31*a^16*b^2*x*z^2 - 39*a^14*b^4*x*z^2 - 94*a^12*b^6*x*z^2 + 250*a^10*b^8*x*z^2 -      141*a^8*b^10*x*z^2 - 91*a^6*b^12*x*z^2 + 124*a^4*b^14*x*z^2 - 36*a^2*b^16*x*z^2 +      18*a^16*c^2*x*z^2 - 102*a^14*b^2*c^2*x*z^2 + 179*a^12*b^4*c^2*x*z^2 - 54*a^10*b^6*c^2*x*z^2 -      228*a^8*b^8*c^2*x*z^2 + 362*a^6*b^10*c^2*x*z^2 - 225*a^4*b^12*c^2*x*z^2 + 42*a^2*b^14*c^2*x*z^2 +      8*b^16*c^2*x*z^2 - 18*a^14*c^4*x*z^2 + 81*a^12*b^2*c^4*x*z^2 - 243*a^10*b^4*c^4*x*z^2 +      393*a^8*b^6*c^4*x*z^2 - 246*a^6*b^8*c^4*x*z^2 - 51*a^4*b^10*c^4*x*z^2 + 123*a^2*b^12*c^4*x*z^2 -      39*b^14*c^4*x*z^2 - 42*a^12*c^6*x*z^2 + 55*a^10*b^2*c^6*x*z^2 + 48*a^8*b^4*c^6*x*z^2 -      186*a^6*b^6*c^6*x*z^2 + 280*a^4*b^8*c^6*x*z^2 - 225*a^2*b^10*c^6*x*z^2 + 70*b^12*c^6*x*z^2 +      126*a^10*c^8*x*z^2 - 90*a^8*b^2*c^8*x*z^2 + 95*a^6*b^4*c^8*x*z^2 - 114*a^4*b^6*c^8*x*z^2 +      72*a^2*b^8*c^8*x*z^2 - 49*b^10*c^8*x*z^2 - 126*a^8*c^10*x*z^2 + 24*a^6*b^2*c^10*x*z^2 -      9*a^4*b^4*c^10*x*z^2 + 48*a^2*b^6*c^10*x*z^2 + 42*a^6*c^12*x*z^2 - 23*a^4*b^2*c^12*x*z^2 -      45*a^2*b^4*c^12*x*z^2 + 7*b^6*c^12*x*z^2 + 18*a^4*c^14*x*z^2 + 39*a^2*b^2*c^14*x*z^2 +      14*b^4*c^14*x*z^2 - 18*a^2*c^16*x*z^2 - 15*b^2*c^16*x*z^2 + 4*c^18*x*z^2 + 36*a^16*b^2*y*z^2 -      124*a^14*b^4*y*z^2 + 91*a^12*b^6*y*z^2 + 141*a^10*b^8*y*z^2 - 250*a^8*b^10*y*z^2 +      94*a^6*b^12*y*z^2 + 39*a^4*b^14*y*z^2 - 31*a^2*b^16*y*z^2 + 4*b^18*y*z^2 - 8*a^16*c^2*y*z^2 -      42*a^14*b^2*c^2*y*z^2 + 225*a^12*b^4*c^2*y*z^2 - 362*a^10*b^6*c^2*y*z^2 + 228*a^8*b^8*c^2*y*z^2 +      54*a^6*b^10*c^2*y*z^2 - 179*a^4*b^12*c^2*y*z^2 + 102*a^2*b^14*c^2*y*z^2 - 18*b^16*c^2*y*z^2 +      39*a^14*c^4*y*z^2 - 123*a^12*b^2*c^4*y*z^2 + 51*a^10*b^4*c^4*y*z^2 + 246*a^8*b^6*c^4*y*z^2 -      393*a^6*b^8*c^4*y*z^2 + 243*a^4*b^10*c^4*y*z^2 - 81*a^2*b^12*c^4*y*z^2 + 18*b^14*c^4*y*z^2 -      70*a^12*c^6*y*z^2 + 225*a^10*b^2*c^6*y*z^2 - 280*a^8*b^4*c^6*y*z^2 + 186*a^6*b^6*c^6*y*z^2 -      48*a^4*b^8*c^6*y*z^2 - 55*a^2*b^10*c^6*y*z^2 + 42*b^12*c^6*y*z^2 + 49*a^10*c^8*y*z^2 -      72*a^8*b^2*c^8*y*z^2 + 114*a^6*b^4*c^8*y*z^2 - 95*a^4*b^6*c^8*y*z^2 + 90*a^2*b^8*c^8*y*z^2 -      126*b^10*c^8*y*z^2 - 48*a^6*b^2*c^10*y*z^2 + 9*a^4*b^4*c^10*y*z^2 - 24*a^2*b^6*c^10*y*z^2 +      126*b^8*c^10*y*z^2 - 7*a^6*c^12*y*z^2 + 45*a^4*b^2*c^12*y*z^2 + 23*a^2*b^4*c^12*y*z^2 -      42*b^6*c^12*y*z^2 - 14*a^4*c^14*y*z^2 - 39*a^2*b^2*c^14*y*z^2 - 18*b^4*c^14*y*z^2 +      15*a^2*c^16*y*z^2 + 18*b^2*c^16*y*z^2 - 4*c^18*y*z^2 - 4*a^18*z^3 + 18*a^16*b^2*z^3 -      18*a^14*b^4*z^3 - 42*a^12*b^6*z^3 + 126*a^10*b^8*z^3 - 126*a^8*b^10*z^3 + 42*a^6*b^12*z^3 +      18*a^4*b^14*z^3 - 18*a^2*b^16*z^3 + 4*b^18*z^3 + 23*a^16*c^2*z^3 - 66*a^14*b^2*c^2*z^3 +      34*a^12*b^4*c^2*z^3 + 38*a^10*b^6*c^2*z^3 - 38*a^6*b^10*c^2*z^3 - 34*a^4*b^12*c^2*z^3 +      66*a^2*b^14*c^2*z^3 - 23*b^16*c^2*z^3 - 49*a^14*c^4*z^3 + 85*a^12*b^2*c^4*z^3 -      27*a^10*b^4*c^4*z^3 - a^8*b^6*c^4*z^3 + a^6*b^8*c^4*z^3 + 27*a^4*b^10*c^4*z^3 -      85*a^2*b^12*c^4*z^3 + 49*b^14*c^4*z^3 + 35*a^12*c^6*z^3 - 60*a^10*b^2*c^6*z^3 +      15*a^8*b^4*c^6*z^3 - 15*a^4*b^8*c^6*z^3 + 60*a^2*b^10*c^6*z^3 - 35*b^12*c^6*z^3 + 35*a^10*c^8*z^3 +      75*a^8*b^2*c^8*z^3 + 16*a^6*b^4*c^8*z^3 - 16*a^4*b^6*c^8*z^3 - 75*a^2*b^8*c^8*z^3 -      35*b^10*c^8*z^3 - 91*a^8*c^10*z^3 - 98*a^6*b^2*c^10*z^3 + 98*a^2*b^6*c^10*z^3 + 91*b^8*c^10*z^3 +      77*a^6*c^12*z^3 + 51*a^4*b^2*c^12*z^3 - 51*a^2*b^4*c^12*z^3 - 77*b^6*c^12*z^3 - 31*a^4*c^14*z^3 +      31*b^4*c^14*z^3 + 5*a^2*c^16*z^3 - 5*b^2*c^16*z^3");
		exp("K267","a^6*b^2*x^2*y - 2*a^4*b^4*x^2*y + a^2*b^6*x^2*y - a^6*c^2*x^2*y + b^6*c^2*x^2*y +      2*a^4*c^4*x^2*y - 2*b^4*c^4*x^2*y - a^2*c^6*x^2*y + b^2*c^6*x^2*y + a^6*b^2*x*y^2 -      2*a^4*b^4*x*y^2 + a^2*b^6*x*y^2 + a^6*c^2*x*y^2 - b^6*c^2*x*y^2 - 2*a^4*c^4*x*y^2 +      2*b^4*c^4*x*y^2 + a^2*c^6*x*y^2 - b^2*c^6*x*y^2 - a^6*b^2*x^2*z + 2*a^4*b^4*x^2*z - a^2*b^6*x^2*z +      a^6*c^2*x^2*z + b^6*c^2*x^2*z - 2*a^4*c^4*x^2*z - 2*b^4*c^4*x^2*z + a^2*c^6*x^2*z + b^2*c^6*x^2*z -      2*a^6*b^2*x*y*z + 4*a^4*b^4*x*y*z - 2*a^2*b^6*x*y*z - 2*a^6*c^2*x*y*z - 2*b^6*c^2*x*y*z +      4*a^4*c^4*x*y*z + 4*b^4*c^4*x*y*z - 2*a^2*c^6*x*y*z - 2*b^2*c^6*x*y*z - a^6*b^2*y^2*z +      2*a^4*b^4*y^2*z - a^2*b^6*y^2*z + a^6*c^2*y^2*z + b^6*c^2*y^2*z - 2*a^4*c^4*y^2*z -      2*b^4*c^4*y^2*z + a^2*c^6*y^2*z + b^2*c^6*y^2*z + a^6*b^2*x*z^2 - 2*a^4*b^4*x*z^2 + a^2*b^6*x*z^2 +      a^6*c^2*x*z^2 - b^6*c^2*x*z^2 - 2*a^4*c^4*x*z^2 + 2*b^4*c^4*x*z^2 + a^2*c^6*x*z^2 - b^2*c^6*x*z^2 +      a^6*b^2*y*z^2 - 2*a^4*b^4*y*z^2 + a^2*b^6*y*z^2 - a^6*c^2*y*z^2 + b^6*c^2*y*z^2 + 2*a^4*c^4*y*z^2 -      2*b^4*c^4*y*z^2 - a^2*c^6*y*z^2 + b^2*c^6*y*z^2");
		exp("K268","a^6*x^2*y - 3*a^4*b^2*x^2*y + 3*a^2*b^4*x^2*y - b^6*x^2*y - 2*a^4*c^2*x^2*y -      4*a^2*b^2*c^2*x^2*y + 6*b^4*c^2*x^2*y + a^2*c^4*x^2*y - 5*b^2*c^4*x^2*y + a^6*x*y^2 -      3*a^4*b^2*x*y^2 + 3*a^2*b^4*x*y^2 - b^6*x*y^2 - 6*a^4*c^2*x*y^2 + 4*a^2*b^2*c^2*x*y^2 +      2*b^4*c^2*x*y^2 + 5*a^2*c^4*x*y^2 - b^2*c^4*x*y^2 - a^6*x^2*z + 2*a^4*b^2*x^2*z - a^2*b^4*x^2*z +      3*a^4*c^2*x^2*z + 4*a^2*b^2*c^2*x^2*z + 5*b^4*c^2*x^2*z - 3*a^2*c^4*x^2*z - 6*b^2*c^4*x^2*z +      c^6*x^2*z + a^4*b^2*y^2*z - 2*a^2*b^4*y^2*z + b^6*y^2*z - 5*a^4*c^2*y^2*z - 4*a^2*b^2*c^2*y^2*z -      3*b^4*c^2*y^2*z + 6*a^2*c^4*y^2*z + 3*b^2*c^4*y^2*z - c^6*y^2*z - a^6*x*z^2 + 6*a^4*b^2*x*z^2 -      5*a^2*b^4*x*z^2 + 3*a^4*c^2*x*z^2 - 4*a^2*b^2*c^2*x*z^2 + b^4*c^2*x*z^2 - 3*a^2*c^4*x*z^2 -      2*b^2*c^4*x*z^2 + c^6*x*z^2 + 5*a^4*b^2*y*z^2 - 6*a^2*b^4*y*z^2 + b^6*y*z^2 - a^4*c^2*y*z^2 +      4*a^2*b^2*c^2*y*z^2 - 3*b^4*c^2*y*z^2 + 2*a^2*c^4*y*z^2 + 3*b^2*c^4*y*z^2 - c^6*y*z^2");
		exp("K269","-(a^4*c^2*x^2*y) + a^3*b*c^2*x^2*y - a^2*b^2*c^2*x^2*y - a*b^3*c^2*x^2*y + 2*b^4*c^2*x^2*y -      a^2*b*c^3*x^2*y + 2*a*b^2*c^3*x^2*y - b^3*c^3*x^2*y + 2*a^2*c^4*x^2*y - a*b*c^4*x^2*y -      b^2*c^4*x^2*y + b*c^5*x^2*y - c^6*x^2*y - 2*a^4*c^2*x*y^2 + a^3*b*c^2*x*y^2 + a^2*b^2*c^2*x*y^2 -      a*b^3*c^2*x*y^2 + b^4*c^2*x*y^2 + a^3*c^3*x*y^2 - 2*a^2*b*c^3*x*y^2 + a*b^2*c^3*x*y^2 +      a^2*c^4*x*y^2 + a*b*c^4*x*y^2 - 2*b^2*c^4*x*y^2 - a*c^5*x*y^2 + c^6*x*y^2 + a^4*b^2*x^2*z -      2*a^2*b^4*x^2*z + b^6*x^2*z - a^3*b^2*c*x^2*z + a^2*b^3*c*x^2*z + a*b^4*c*x^2*z - b^5*c*x^2*z +      a^2*b^2*c^2*x^2*z - 2*a*b^3*c^2*x^2*z + b^4*c^2*x^2*z + a*b^2*c^3*x^2*z + b^3*c^3*x^2*z -      2*b^2*c^4*x^2*z - a^6*y^2*z + 2*a^4*b^2*y^2*z - a^2*b^4*y^2*z + a^5*c*y^2*z - a^4*b*c*y^2*z -      a^3*b^2*c*y^2*z + a^2*b^3*c*y^2*z - a^4*c^2*y^2*z + 2*a^3*b*c^2*y^2*z - a^2*b^2*c^2*y^2*z -      a^3*c^3*y^2*z - a^2*b*c^3*y^2*z + 2*a^2*c^4*y^2*z + 2*a^4*b^2*x*z^2 - a^3*b^3*x*z^2 -      a^2*b^4*x*z^2 + a*b^5*x*z^2 - b^6*x*z^2 - a^3*b^2*c*x*z^2 + 2*a^2*b^3*c*x*z^2 - a*b^4*c*x*z^2 -      a^2*b^2*c^2*x*z^2 - a*b^3*c^2*x*z^2 + 2*b^4*c^2*x*z^2 + a*b^2*c^3*x*z^2 - b^2*c^4*x*z^2 +      a^6*y*z^2 - a^5*b*y*z^2 + a^4*b^2*y*z^2 + a^3*b^3*y*z^2 - 2*a^2*b^4*y*z^2 + a^4*b*c*y*z^2 -      2*a^3*b^2*c*y*z^2 + a^2*b^3*c*y*z^2 - 2*a^4*c^2*y*z^2 + a^3*b*c^2*y*z^2 + a^2*b^2*c^2*y*z^2 -      a^2*b*c^3*y*z^2 + a^2*c^4*y*z^2");
		exp("K270","a^6*c^2*x^2*y + a^2*b^4*c^2*x^2*y - 2*b^6*c^2*x^2*y - a^4*c^4*x^2*y + b^4*c^4*x^2*y -      a^2*c^6*x^2*y + c^8*x^2*y + 2*a^6*c^2*x*y^2 - a^4*b^2*c^2*x*y^2 - b^6*c^2*x*y^2 - a^4*c^4*x*y^2 +      b^4*c^4*x*y^2 + b^2*c^6*x*y^2 - c^8*x*y^2 - a^6*b^2*x^2*z + a^4*b^4*x^2*z + a^2*b^6*x^2*z -      b^8*x^2*z - a^2*b^2*c^4*x^2*z - b^4*c^4*x^2*z + 2*b^2*c^6*x^2*z + a^8*y^2*z - a^6*b^2*y^2*z -      a^4*b^4*y^2*z + a^2*b^6*y^2*z + a^4*c^4*y^2*z + a^2*b^2*c^4*y^2*z - 2*a^2*c^6*y^2*z -      2*a^6*b^2*x*z^2 + a^4*b^4*x*z^2 + b^8*x*z^2 + a^4*b^2*c^2*x*z^2 - b^6*c^2*x*z^2 - b^4*c^4*x*z^2 +      b^2*c^6*x*z^2 - a^8*y*z^2 - a^4*b^4*y*z^2 + 2*a^2*b^6*y*z^2 + a^6*c^2*y*z^2 - a^2*b^4*c^2*y*z^2 +      a^4*c^4*y*z^2 - a^2*c^6*y*z^2");
		exp("K271","-(b^8*x^3) + a^4*b^2*c^2*x^3 - 2*a^2*b^4*c^2*x^3 + 3*b^6*c^2*x^3 - 2*a^2*b^2*c^4*x^3 -      b^4*c^4*x^3 + 3*b^2*c^6*x^3 - c^8*x^3 - a^4*b^4*x^2*y - 2*a^2*b^6*x^2*y + a^6*c^2*x^2*y -      3*a^4*b^2*c^2*x^2*y + 9*a^2*b^4*c^2*x^2*y - b^6*c^2*x^2*y - 2*a^4*c^4*x^2*y + 5*a^2*c^6*x^2*y -      7*b^2*c^6*x^2*y + c^8*x^2*y - 2*a^6*b^2*x*y^2 - a^4*b^4*x*y^2 - a^6*c^2*x*y^2 +      9*a^4*b^2*c^2*x*y^2 - 3*a^2*b^4*c^2*x*y^2 + b^6*c^2*x*y^2 - 2*b^4*c^4*x*y^2 - 7*a^2*c^6*x*y^2 +      5*b^2*c^6*x*y^2 + c^8*x*y^2 - a^8*y^3 + 3*a^6*c^2*y^3 - 2*a^4*b^2*c^2*y^3 + a^2*b^4*c^2*y^3 -      a^4*c^4*y^3 - 2*a^2*b^2*c^4*y^3 + 3*a^2*c^6*y^3 - c^8*y^3 + a^6*b^2*x^2*z - 2*a^4*b^4*x^2*z +      5*a^2*b^6*x^2*z + b^8*x^2*z - 3*a^4*b^2*c^2*x^2*z - 7*b^6*c^2*x^2*z - a^4*c^4*x^2*z +      9*a^2*b^2*c^4*x^2*z - 2*a^2*c^6*x^2*z - b^2*c^6*x^2*z + a^8*x*y*z - a^6*b^2*x*y*z +      10*a^4*b^4*x*y*z - a^2*b^6*x*y*z + b^8*x*y*z - a^6*c^2*x*y*z - 9*a^4*b^2*c^2*x*y*z -      9*a^2*b^4*c^2*x*y*z - b^6*c^2*x*y*z + 10*a^4*c^4*x*y*z - 9*a^2*b^2*c^4*x*y*z + 10*b^4*c^4*x*y*z -      a^2*c^6*x*y*z - b^2*c^6*x*y*z + c^8*x*y*z + a^8*y^2*z + 5*a^6*b^2*y^2*z - 2*a^4*b^4*y^2*z +      a^2*b^6*y^2*z - 7*a^6*c^2*y^2*z - 3*a^2*b^4*c^2*y^2*z + 9*a^2*b^2*c^4*y^2*z - b^4*c^4*y^2*z -      a^2*c^6*y^2*z - 2*b^2*c^6*y^2*z - a^6*b^2*x*z^2 - 7*a^2*b^6*x*z^2 + b^8*x*z^2 - 2*a^6*c^2*x*z^2 +      9*a^4*b^2*c^2*x*z^2 + 5*b^6*c^2*x*z^2 - a^4*c^4*x*z^2 - 3*a^2*b^2*c^4*x*z^2 - 2*b^4*c^4*x*z^2 +      b^2*c^6*x*z^2 + a^8*y*z^2 - 7*a^6*b^2*y*z^2 - a^2*b^6*y*z^2 + 5*a^6*c^2*y*z^2 +      9*a^2*b^4*c^2*y*z^2 - 2*b^6*c^2*y*z^2 - 2*a^4*c^4*y*z^2 - 3*a^2*b^2*c^4*y*z^2 - b^4*c^4*y*z^2 +      a^2*c^6*y*z^2 - a^8*z^3 + 3*a^6*b^2*z^3 - a^4*b^4*z^3 + 3*a^2*b^6*z^3 - b^8*z^3 -      2*a^4*b^2*c^2*z^3 - 2*a^2*b^4*c^2*z^3 + a^2*b^2*c^4*z^3");
		exp("K272","a^8*b^2*c^2*x^3 - 2*a^2*b^8*c^2*x^3 - 9*a^4*b^4*c^4*x^3 + 12*b^6*c^6*x^3 - 2*a^2*b^2*c^8*x^3 -      3*a^2*b^8*c^2*x^2*y + 9*a^4*b^4*c^4*x^2*y + 3*a^6*c^6*x^2*y - 12*b^6*c^6*x^2*y +      3*a^2*b^2*c^8*x^2*y - 3*a^8*b^2*c^2*x*y^2 + 9*a^4*b^4*c^4*x*y^2 - 12*a^6*c^6*x*y^2 +      3*b^6*c^6*x*y^2 + 3*a^2*b^2*c^8*x*y^2 - 2*a^8*b^2*c^2*y^3 + a^2*b^8*c^2*y^3 - 9*a^4*b^4*c^4*y^3 +      12*a^6*c^6*y^3 - 2*a^2*b^2*c^8*y^3 + 3*a^6*b^6*x^2*z + 3*a^2*b^8*c^2*x^2*z + 9*a^4*b^4*c^4*x^2*z -      12*b^6*c^6*x^2*z - 3*a^2*b^2*c^8*x^2*z + 6*a^6*b^6*x*y*z + 3*a^8*b^2*c^2*x*y*z +      3*a^2*b^8*c^2*x*y*z - 27*a^4*b^4*c^4*x*y*z + 6*a^6*c^6*x*y*z + 6*b^6*c^6*x*y*z +      3*a^2*b^2*c^8*x*y*z + 3*a^6*b^6*y^2*z + 3*a^8*b^2*c^2*y^2*z + 9*a^4*b^4*c^4*y^2*z -      12*a^6*c^6*y^2*z - 3*a^2*b^2*c^8*y^2*z - 12*a^6*b^6*x*z^2 - 3*a^8*b^2*c^2*x*z^2 +      3*a^2*b^8*c^2*x*z^2 + 9*a^4*b^4*c^4*x*z^2 + 3*b^6*c^6*x*z^2 - 12*a^6*b^6*y*z^2 +      3*a^8*b^2*c^2*y*z^2 - 3*a^2*b^8*c^2*y*z^2 + 9*a^4*b^4*c^4*y*z^2 + 3*a^6*c^6*y*z^2 +      12*a^6*b^6*z^3 - 2*a^8*b^2*c^2*z^3 - 2*a^2*b^8*c^2*z^3 - 9*a^4*b^4*c^4*z^3 + a^2*b^2*c^8*z^3");
		exp("K273","-2*a^2*c^2*x^2*y + b^2*c^2*x^2*y + c^4*x^2*y - a^2*c^2*x*y^2 + 2*b^2*c^2*x*y^2 - c^4*x*y^2 +      2*a^2*b^2*x^2*z - b^4*x^2*z - b^2*c^2*x^2*z + a^4*y^2*z - 2*a^2*b^2*y^2*z + a^2*c^2*y^2*z +      a^2*b^2*x*z^2 + b^4*x*z^2 - 2*b^2*c^2*x*z^2 - a^4*y*z^2 - a^2*b^2*y*z^2 + 2*a^2*c^2*y*z^2");
		exp("K274","(-a^6)*b^2*c^4*x^2*y + 2*a^5*b^3*c^4*x^2*y - 2*a^4*b^4*c^4*x^2*y + 2*a^2*b^6*c^4*x^2*y -      2*a*b^7*c^4*x^2*y + b^8*c^4*x^2*y - 2*a^4*b^3*c^5*x^2*y + 4*a^3*b^4*c^5*x^2*y -      4*a^2*b^5*c^5*x^2*y + 2*a*b^6*c^5*x^2*y + 3*a^4*b^2*c^6*x^2*y - 2*a^3*b^3*c^6*x^2*y -      a^2*b^4*c^6*x^2*y + 4*a*b^5*c^6*x^2*y - 3*b^6*c^6*x^2*y - 2*a^3*b^2*c^7*x^2*y +      4*a^2*b^3*c^7*x^2*y - 4*a*b^4*c^7*x^2*y - a^2*b^2*c^8*x^2*y - 2*a*b^3*c^8*x^2*y + 3*b^4*c^8*x^2*y +      2*a*b^2*c^9*x^2*y - b^2*c^10*x^2*y - a^8*c^4*x*y^2 + 2*a^7*b*c^4*x*y^2 - 2*a^6*b^2*c^4*x*y^2 +      2*a^4*b^4*c^4*x*y^2 - 2*a^3*b^5*c^4*x*y^2 + a^2*b^6*c^4*x*y^2 - 2*a^6*b*c^5*x*y^2 +      4*a^5*b^2*c^5*x*y^2 - 4*a^4*b^3*c^5*x*y^2 + 2*a^3*b^4*c^5*x*y^2 + 3*a^6*c^6*x*y^2 -      4*a^5*b*c^6*x*y^2 + a^4*b^2*c^6*x*y^2 + 2*a^3*b^3*c^6*x*y^2 - 3*a^2*b^4*c^6*x*y^2 +      4*a^4*b*c^7*x*y^2 - 4*a^3*b^2*c^7*x*y^2 + 2*a^2*b^3*c^7*x*y^2 - 3*a^4*c^8*x*y^2 +      2*a^3*b*c^8*x*y^2 + a^2*b^2*c^8*x*y^2 - 2*a^2*b*c^9*x*y^2 + a^2*c^10*x*y^2 + a^6*b^4*c^2*x^2*z -      3*a^4*b^6*c^2*x^2*z + 2*a^3*b^7*c^2*x^2*z + a^2*b^8*c^2*x^2*z - 2*a*b^9*c^2*x^2*z +      b^10*c^2*x^2*z - 2*a^5*b^4*c^3*x^2*z + 2*a^4*b^5*c^3*x^2*z + 2*a^3*b^6*c^3*x^2*z -      4*a^2*b^7*c^3*x^2*z + 2*a*b^8*c^3*x^2*z + 2*a^4*b^4*c^4*x^2*z - 4*a^3*b^5*c^4*x^2*z +      a^2*b^6*c^4*x^2*z + 4*a*b^7*c^4*x^2*z - 3*b^8*c^4*x^2*z + 4*a^2*b^5*c^5*x^2*z - 4*a*b^6*c^5*x^2*z -      2*a^2*b^4*c^6*x^2*z - 2*a*b^5*c^6*x^2*z + 3*b^6*c^6*x^2*z + 2*a*b^4*c^7*x^2*z - b^4*c^8*x^2*z +      2*a^7*b^3*c^2*x*y*z - 4*a^6*b^4*c^2*x*y*z + 4*a^4*b^6*c^2*x*y*z - 2*a^3*b^7*c^2*x*y*z -      2*a^7*b^2*c^3*x*y*z + 6*a^5*b^4*c^3*x*y*z - 6*a^4*b^5*c^3*x*y*z + 2*a^2*b^7*c^3*x*y*z +      4*a^6*b^2*c^4*x*y*z - 6*a^5*b^3*c^4*x*y*z + 6*a^3*b^5*c^4*x*y*z - 4*a^2*b^6*c^4*x*y*z +      6*a^4*b^3*c^5*x*y*z - 6*a^3*b^4*c^5*x*y*z - 4*a^4*b^2*c^6*x*y*z + 4*a^2*b^4*c^6*x*y*z +      2*a^3*b^2*c^7*x*y*z - 2*a^2*b^3*c^7*x*y*z - a^10*c^2*y^2*z + 2*a^9*b*c^2*y^2*z -      a^8*b^2*c^2*y^2*z - 2*a^7*b^3*c^2*y^2*z + 3*a^6*b^4*c^2*y^2*z - a^4*b^6*c^2*y^2*z -      2*a^8*b*c^3*y^2*z + 4*a^7*b^2*c^3*y^2*z - 2*a^6*b^3*c^3*y^2*z - 2*a^5*b^4*c^3*y^2*z +      2*a^4*b^5*c^3*y^2*z + 3*a^8*c^4*y^2*z - 4*a^7*b*c^4*y^2*z - a^6*b^2*c^4*y^2*z +      4*a^5*b^3*c^4*y^2*z - 2*a^4*b^4*c^4*y^2*z + 4*a^6*b*c^5*y^2*z - 4*a^5*b^2*c^5*y^2*z -      3*a^6*c^6*y^2*z + 2*a^5*b*c^6*y^2*z + 2*a^4*b^2*c^6*y^2*z - 2*a^4*b*c^7*y^2*z + a^4*c^8*y^2*z +      a^8*b^4*x*z^2 - 3*a^6*b^6*x*z^2 + 3*a^4*b^8*x*z^2 - a^2*b^10*x*z^2 - 2*a^7*b^4*c*x*z^2 +      2*a^6*b^5*c*x*z^2 + 4*a^5*b^6*c*x*z^2 - 4*a^4*b^7*c*x*z^2 - 2*a^3*b^8*c*x*z^2 + 2*a^2*b^9*c*x*z^2 +      2*a^6*b^4*c^2*x*z^2 - 4*a^5*b^5*c^2*x*z^2 - a^4*b^6*c^2*x*z^2 + 4*a^3*b^7*c^2*x*z^2 -      a^2*b^8*c^2*x*z^2 + 4*a^4*b^5*c^3*x*z^2 - 2*a^3*b^6*c^3*x*z^2 - 2*a^2*b^7*c^3*x*z^2 -      2*a^4*b^4*c^4*x*z^2 - 2*a^3*b^5*c^4*x*z^2 + 3*a^2*b^6*c^4*x*z^2 + 2*a^3*b^4*c^5*x*z^2 -      a^2*b^4*c^6*x*z^2 + a^10*b^2*y*z^2 - 3*a^8*b^4*y*z^2 + 3*a^6*b^6*y*z^2 - a^4*b^8*y*z^2 -      2*a^9*b^2*c*y*z^2 + 2*a^8*b^3*c*y*z^2 + 4*a^7*b^4*c*y*z^2 - 4*a^6*b^5*c*y*z^2 - 2*a^5*b^6*c*y*z^2 +      2*a^4*b^7*c*y*z^2 + a^8*b^2*c^2*y*z^2 - 4*a^7*b^3*c^2*y*z^2 + a^6*b^4*c^2*y*z^2 +      4*a^5*b^5*c^2*y*z^2 - 2*a^4*b^6*c^2*y*z^2 + 2*a^7*b^2*c^3*y*z^2 + 2*a^6*b^3*c^3*y*z^2 -      4*a^5*b^4*c^3*y*z^2 - 3*a^6*b^2*c^4*y*z^2 + 2*a^5*b^3*c^4*y*z^2 + 2*a^4*b^4*c^4*y*z^2 -      2*a^4*b^3*c^5*y*z^2 + a^4*b^2*c^6*y*z^2");
		exp("K275","a^6*x^2*y - 3*a^4*b^2*x^2*y + 3*a^2*b^4*x^2*y - b^6*x^2*y - 2*a^5*c*x^2*y + 2*a^4*b*c*x^2*y +      4*a^3*b^2*c*x^2*y - 4*a^2*b^3*c*x^2*y - 2*a*b^4*c*x^2*y + 2*b^5*c*x^2*y + a^4*c^2*x^2*y -      4*a^3*b*c^2*x^2*y + a^2*b^2*c^2*x^2*y + 4*a*b^3*c^2*x^2*y - 2*b^4*c^2*x^2*y + 2*a^3*c^3*x^2*y +      2*a^2*b*c^3*x^2*y - 4*a*b^2*c^3*x^2*y - 3*a^2*c^4*x^2*y + 2*a*b*c^4*x^2*y + 2*b^2*c^4*x^2*y -      2*b*c^5*x^2*y + c^6*x^2*y + a^6*x*y^2 - 3*a^4*b^2*x*y^2 + 3*a^2*b^4*x*y^2 - b^6*x*y^2 -      2*a^5*c*x*y^2 + 2*a^4*b*c*x*y^2 + 4*a^3*b^2*c*x*y^2 - 4*a^2*b^3*c*x*y^2 - 2*a*b^4*c*x*y^2 +      2*b^5*c*x*y^2 + 2*a^4*c^2*x*y^2 - 4*a^3*b*c^2*x*y^2 - a^2*b^2*c^2*x*y^2 + 4*a*b^3*c^2*x*y^2 -      b^4*c^2*x*y^2 + 4*a^2*b*c^3*x*y^2 - 2*a*b^2*c^3*x*y^2 - 2*b^3*c^3*x*y^2 - 2*a^2*c^4*x*y^2 -      2*a*b*c^4*x*y^2 + 3*b^2*c^4*x*y^2 + 2*a*c^5*x*y^2 - c^6*x*y^2 - a^6*x^2*z + 2*a^5*b*x^2*z -      a^4*b^2*x^2*z - 2*a^3*b^3*x^2*z + 3*a^2*b^4*x^2*z - b^6*x^2*z - 2*a^4*b*c*x^2*z +      4*a^3*b^2*c*x^2*z - 2*a^2*b^3*c*x^2*z - 2*a*b^4*c*x^2*z + 2*b^5*c*x^2*z + 3*a^4*c^2*x^2*z -      4*a^3*b*c^2*x^2*z - a^2*b^2*c^2*x^2*z + 4*a*b^3*c^2*x^2*z - 2*b^4*c^2*x^2*z + 4*a^2*b*c^3*x^2*z -      4*a*b^2*c^3*x^2*z - 3*a^2*c^4*x^2*z + 2*a*b*c^4*x^2*z + 2*b^2*c^4*x^2*z - 2*b*c^5*x^2*z +      c^6*x^2*z + 2*a^5*b*x*y*z - 4*a^4*b^2*x*y*z + 4*a^2*b^4*x*y*z - 2*a*b^5*x*y*z - 2*a^5*c*x*y*z +      6*a^3*b^2*c*x*y*z - 6*a^2*b^3*c*x*y*z + 2*b^5*c*x*y*z + 4*a^4*c^2*x*y*z - 6*a^3*b*c^2*x*y*z +      6*a*b^3*c^2*x*y*z - 4*b^4*c^2*x*y*z + 6*a^2*b*c^3*x*y*z - 6*a*b^2*c^3*x*y*z - 4*a^2*c^4*x*y*z +      4*b^2*c^4*x*y*z + 2*a*c^5*x*y*z - 2*b*c^5*x*y*z + a^6*y^2*z - 3*a^4*b^2*y^2*z + 2*a^3*b^3*y^2*z +      a^2*b^4*y^2*z - 2*a*b^5*y^2*z + b^6*y^2*z - 2*a^5*c*y^2*z + 2*a^4*b*c*y^2*z + 2*a^3*b^2*c*y^2*z -      4*a^2*b^3*c*y^2*z + 2*a*b^4*c*y^2*z + 2*a^4*c^2*y^2*z - 4*a^3*b*c^2*y^2*z + a^2*b^2*c^2*y^2*z +      4*a*b^3*c^2*y^2*z - 3*b^4*c^2*y^2*z + 4*a^2*b*c^3*y^2*z - 4*a*b^2*c^3*y^2*z - 2*a^2*c^4*y^2*z -      2*a*b*c^4*y^2*z + 3*b^2*c^4*y^2*z + 2*a*c^5*y^2*z - c^6*y^2*z - a^6*x*z^2 + 2*a^5*b*x*z^2 -      2*a^4*b^2*x*z^2 + 2*a^2*b^4*x*z^2 - 2*a*b^5*x*z^2 + b^6*x*z^2 - 2*a^4*b*c*x*z^2 +      4*a^3*b^2*c*x*z^2 - 4*a^2*b^3*c*x*z^2 + 2*a*b^4*c*x*z^2 + 3*a^4*c^2*x*z^2 - 4*a^3*b*c^2*x*z^2 +      a^2*b^2*c^2*x*z^2 + 2*a*b^3*c^2*x*z^2 - 3*b^4*c^2*x*z^2 + 4*a^2*b*c^3*x*z^2 - 4*a*b^2*c^3*x*z^2 +      2*b^3*c^3*x*z^2 - 3*a^2*c^4*x*z^2 + 2*a*b*c^4*x*z^2 + b^2*c^4*x*z^2 - 2*b*c^5*x*z^2 + c^6*x*z^2 -      a^6*y*z^2 + 2*a^5*b*y*z^2 - 2*a^4*b^2*y*z^2 + 2*a^2*b^4*y*z^2 - 2*a*b^5*y*z^2 + b^6*y*z^2 -      2*a^4*b*c*y*z^2 + 4*a^3*b^2*c*y*z^2 - 4*a^2*b^3*c*y*z^2 + 2*a*b^4*c*y*z^2 + 3*a^4*c^2*y*z^2 -      2*a^3*b*c^2*y*z^2 - a^2*b^2*c^2*y*z^2 + 4*a*b^3*c^2*y*z^2 - 3*b^4*c^2*y*z^2 - 2*a^3*c^3*y*z^2 +      4*a^2*b*c^3*y*z^2 - 4*a*b^2*c^3*y*z^2 - a^2*c^4*y*z^2 - 2*a*b*c^4*y*z^2 + 3*b^2*c^4*y*z^2 +      2*a*c^5*y*z^2 - c^6*y*z^2");
		exp("K276","a^6*x^2*y + a^4*b^2*x^2*y - 2*a^2*b^4*x^2*y - 2*a^4*c^2*x^2*y + a^2*b^2*c^2*x^2*y +      a^2*c^4*x^2*y + 2*a^4*b^2*x*y^2 - a^2*b^4*x*y^2 - b^6*x*y^2 - a^2*b^2*c^2*x*y^2 + 2*b^4*c^2*x*y^2 -      b^2*c^4*x*y^2 - a^6*x^2*z + 2*a^4*b^2*x^2*z - a^2*b^4*x^2*z - a^4*c^2*x^2*z - a^2*b^2*c^2*x^2*z +      2*a^2*c^4*x^2*z + a^4*b^2*y^2*z - 2*a^2*b^4*y^2*z + b^6*y^2*z + a^2*b^2*c^2*y^2*z + b^4*c^2*y^2*z -      2*b^2*c^4*y^2*z - 2*a^4*c^2*x*z^2 + a^2*b^2*c^2*x*z^2 + b^4*c^2*x*z^2 + a^2*c^4*x*z^2 -      2*b^2*c^4*x*z^2 + c^6*x*z^2 - a^4*c^2*y*z^2 - a^2*b^2*c^2*y*z^2 + 2*b^4*c^2*y*z^2 +      2*a^2*c^4*y*z^2 - b^2*c^4*y*z^2 - c^6*y*z^2");
		exp("K277","(-a^4)*b^2*x^3 + 2*a^2*b^4*x^3 - b^6*x^3 + a^4*c^2*x^3 - 2*a^2*c^4*x^3 + c^6*x^3 + a^6*x^2*y -      3*a^2*b^4*x^2*y + 2*b^6*x^2*y - 4*a^4*c^2*x^2*y + 4*a^2*b^2*c^2*x^2*y - 3*b^4*c^2*x^2*y +      2*a^2*c^4*x^2*y + c^6*x^2*y - 2*a^6*x*y^2 + 3*a^4*b^2*x*y^2 - b^6*x*y^2 + 3*a^4*c^2*x*y^2 -      4*a^2*b^2*c^2*x*y^2 + 4*b^4*c^2*x*y^2 - 2*b^2*c^4*x*y^2 - c^6*x*y^2 + a^6*y^3 - 2*a^4*b^2*y^3 +      a^2*b^4*y^3 - b^4*c^2*y^3 + 2*b^2*c^4*y^3 - c^6*y^3 - a^6*x^2*z + 4*a^4*b^2*x^2*z -      2*a^2*b^4*x^2*z - b^6*x^2*z - 4*a^2*b^2*c^2*x^2*z + 3*a^2*c^4*x^2*z + 3*b^2*c^4*x^2*z -      2*c^6*x^2*z - 6*a^4*b^2*x*y*z + 6*a^2*b^4*x*y*z + 6*a^4*c^2*x*y*z - 6*b^4*c^2*x*y*z -      6*a^2*c^4*x*y*z + 6*b^2*c^4*x*y*z + a^6*y^2*z + 2*a^4*b^2*y^2*z - 4*a^2*b^4*y^2*z + b^6*y^2*z +      4*a^2*b^2*c^2*y^2*z - 3*a^2*c^4*y^2*z - 3*b^2*c^4*y^2*z + 2*c^6*y^2*z + 2*a^6*x*z^2 -      3*a^4*b^2*x*z^2 + b^6*x*z^2 - 3*a^4*c^2*x*z^2 + 4*a^2*b^2*c^2*x*z^2 + 2*b^4*c^2*x*z^2 -      4*b^2*c^4*x*z^2 + c^6*x*z^2 - a^6*y*z^2 + 3*a^2*b^4*y*z^2 - 2*b^6*y*z^2 - 2*a^4*c^2*y*z^2 -      4*a^2*b^2*c^2*y*z^2 + 3*b^4*c^2*y*z^2 + 4*a^2*c^4*y*z^2 - c^6*y*z^2 - a^6*z^3 + b^6*z^3 +      2*a^4*c^2*z^3 - 2*b^4*c^2*z^3 - a^2*c^4*z^3 + b^2*c^4*z^3");
		exp("K278","a^4*x^2*y - 2*a^2*b^2*x^2*y + b^4*x^2*y - 2*a^2*c^2*x^2*y + b^2*c^2*x^2*y + c^4*x^2*y -      a^4*x*y^2 + 2*a^2*b^2*x*y^2 - b^4*x*y^2 - a^2*c^2*x*y^2 + 2*b^2*c^2*x*y^2 - c^4*x*y^2 - a^4*x^2*z +      2*a^2*b^2*x^2*z - b^4*x^2*z + 2*a^2*c^2*x^2*z - b^2*c^2*x^2*z - c^4*x^2*z + a^4*y^2*z -      2*a^2*b^2*y^2*z + b^4*y^2*z + a^2*c^2*y^2*z - 2*b^2*c^2*y^2*z + c^4*y^2*z + a^4*x*z^2 +      a^2*b^2*x*z^2 + b^4*x*z^2 - 2*a^2*c^2*x*z^2 - 2*b^2*c^2*x*z^2 + c^4*x*z^2 - a^4*y*z^2 -      a^2*b^2*y*z^2 - b^4*y*z^2 + 2*a^2*c^2*y*z^2 + 2*b^2*c^2*y*z^2 - c^4*y*z^2");
		exp("K279","a^6*c^2*x^2*y + a^4*b^2*c^2*x^2*y - 2*a^2*b^4*c^2*x^2*y - 2*a^4*c^4*x^2*y + a^2*b^2*c^4*x^2*y +      a^2*c^6*x^2*y + 2*a^4*b^2*c^2*x*y^2 - a^2*b^4*c^2*x*y^2 - b^6*c^2*x*y^2 - a^2*b^2*c^4*x*y^2 +      2*b^4*c^4*x*y^2 - b^2*c^6*x*y^2 - a^6*b^2*x^2*z + 2*a^4*b^4*x^2*z - a^2*b^6*x^2*z -      a^4*b^2*c^2*x^2*z - a^2*b^4*c^2*x^2*z + 2*a^2*b^2*c^4*x^2*z + a^6*b^2*y^2*z - 2*a^4*b^4*y^2*z +      a^2*b^6*y^2*z + a^4*b^2*c^2*y^2*z + a^2*b^4*c^2*y^2*z - 2*a^2*b^2*c^4*y^2*z - 2*a^4*b^2*c^2*x*z^2 +      a^2*b^4*c^2*x*z^2 + b^6*c^2*x*z^2 + a^2*b^2*c^4*x*z^2 - 2*b^4*c^4*x*z^2 + b^2*c^6*x*z^2 -      a^6*c^2*y*z^2 - a^4*b^2*c^2*y*z^2 + 2*a^2*b^4*c^2*y*z^2 + 2*a^4*c^4*y*z^2 - a^2*b^2*c^4*y*z^2 -      a^2*c^6*y*z^2");
		exp("K280","(-a^2)*c^2*x^2*y - 2*b^2*c^2*x^2*y + 2*a^2*c^2*x*y^2 + b^2*c^2*x*y^2 + a^2*b^2*x^2*z +      2*b^2*c^2*x^2*z - a^2*b^2*y^2*z - 2*a^2*c^2*y^2*z - 2*a^2*b^2*x*z^2 - b^2*c^2*x*z^2 +      2*a^2*b^2*y*z^2 + a^2*c^2*y*z^2");
		exp("K281","2*b^2*c^2*x^2*y + c^4*x^2*y - 2*a^2*c^2*x*y^2 - c^4*x*y^2 - b^4*x^2*z - 2*b^2*c^2*x^2*z +      a^4*y^2*z + 2*a^2*c^2*y^2*z + 2*a^2*b^2*x*z^2 + b^4*x*z^2 - a^4*y*z^2 - 2*a^2*b^2*y*z^2");
		exp("K282","a^2*c^2*x^2*y + 4*b^2*c^2*x^2*y + c^4*x^2*y - 4*a^2*c^2*x*y^2 - b^2*c^2*x*y^2 - c^4*x*y^2 -      a^2*b^2*x^2*z - b^4*x^2*z - 4*b^2*c^2*x^2*z + a^4*y^2*z + a^2*b^2*y^2*z + 4*a^2*c^2*y^2*z +      4*a^2*b^2*x*z^2 + b^4*x*z^2 + b^2*c^2*x*z^2 - a^4*y*z^2 - 4*a^2*b^2*y*z^2 - a^2*c^2*y*z^2");
		exp("K283","(-a^2)*c^2*x^2*y + 2*b^2*c^2*x^2*y + 2*c^4*x^2*y - 2*a^2*c^2*x*y^2 + b^2*c^2*x*y^2 -      2*c^4*x*y^2 + a^2*b^2*x^2*z - 2*b^4*x^2*z - 2*b^2*c^2*x^2*z + 2*a^4*y^2*z - a^2*b^2*y^2*z +      2*a^2*c^2*y^2*z + 2*a^2*b^2*x*z^2 + 2*b^4*x*z^2 - b^2*c^2*x*z^2 - 2*a^4*y*z^2 - 2*a^2*b^2*y*z^2 +      a^2*c^2*y*z^2");
		exp("K284","-2*a^2*c^2*x^2*y - 2*b^2*c^2*x^2*y + c^4*x^2*y + 2*a^2*c^2*x*y^2 + 2*b^2*c^2*x*y^2 - c^4*x*y^2 +      2*a^2*b^2*x^2*z - b^4*x^2*z + 2*b^2*c^2*x^2*z + a^4*y^2*z - 2*a^2*b^2*y^2*z - 2*a^2*c^2*y^2*z -      2*a^2*b^2*x*z^2 + b^4*x*z^2 - 2*b^2*c^2*x*z^2 - a^4*y*z^2 + 2*a^2*b^2*y*z^2 + 2*a^2*c^2*y*z^2");
		exp("K285","a*b^4*c^4*x^2*y - b^5*c^4*x^2*y - a^2*b^2*c^5*x^2*y + b^4*c^5*x^2*y + a^2*b*c^6*x^2*y -      a*b^2*c^6*x^2*y - a^5*c^4*x*y^2 + a^4*b*c^4*x*y^2 + a^4*c^5*x*y^2 - a^2*b^2*c^5*x*y^2 -      a^2*b*c^6*x*y^2 + a*b^2*c^6*x*y^2 + a^2*b^6*c*x^2*z - a^2*b^5*c^2*x^2*z - a*b^6*c^2*x^2*z +      a*b^4*c^4*x^2*z + b^5*c^4*x^2*z - b^4*c^5*x^2*z - 2*a^4*b^4*c*x*y*z + 2*a^5*b^2*c^2*x*y*z +      2*a^2*b^5*c^2*x*y*z - 2*a^4*b*c^4*x*y*z - 2*a*b^4*c^4*x*y*z + 2*a^2*b^2*c^5*x*y*z +      a^6*b^2*c*y^2*z - a^6*b*c^2*y^2*z - a^5*b^2*c^2*y^2*z + a^5*c^4*y^2*z + a^4*b*c^4*y^2*z -      a^4*c^5*y^2*z - a^5*b^4*x*z^2 + a^4*b^5*x*z^2 + a^4*b^4*c*x*z^2 - a^2*b^6*c*x*z^2 -      a^2*b^5*c^2*x*z^2 + a*b^6*c^2*x*z^2 + a^5*b^4*y*z^2 - a^4*b^5*y*z^2 - a^6*b^2*c*y*z^2 +      a^4*b^4*c*y*z^2 + a^6*b*c^2*y*z^2 - a^5*b^2*c^2*y*z^2");
		exp("K286","(-a^2)*b^3*c*x^2*y + a*b^4*c*x^2*y + a^3*b*c^2*x^2*y - a*b^3*c^2*x^2*y - a^3*c^3*x^2*y +      a^2*b*c^3*x^2*y + a^4*b*c*x*y^2 - a^3*b^2*c*x*y^2 - a^3*b*c^2*x*y^2 + a*b^3*c^2*x*y^2 +      a*b^2*c^3*x*y^2 - b^3*c^3*x*y^2 - a^3*b^3*x^2*z + a^3*b^2*c*x^2*z + a^2*b^3*c*x^2*z -      a^2*b*c^3*x^2*z - a*b^2*c^3*x^2*z + a*b*c^4*x^2*z + 2*a^3*b^3*x*y*z - 2*a^4*b*c*x*y*z -      2*a*b^4*c*x*y*z + 2*a^3*c^3*x*y*z + 2*b^3*c^3*x*y*z - 2*a*b*c^4*x*y*z - a^3*b^3*y^2*z +      a^3*b^2*c*y^2*z + a^2*b^3*c*y^2*z - a^2*b*c^3*y^2*z - a*b^2*c^3*y^2*z + a*b*c^4*y^2*z +      a^4*b*c*x*z^2 - a^3*b^2*c*x*z^2 - a^3*b*c^2*x*z^2 + a*b^3*c^2*x*z^2 + a*b^2*c^3*x*z^2 -      b^3*c^3*x*z^2 - a^2*b^3*c*y*z^2 + a*b^4*c*y*z^2 + a^3*b*c^2*y*z^2 - a*b^3*c^2*y*z^2 -      a^3*c^3*y*z^2 + a^2*b*c^3*y*z^2");
		exp("K287","2*a^2*c^2*x^2*y + b^2*c^2*x^2*y - a^2*c^2*x*y^2 - 2*b^2*c^2*x*y^2 - 2*a^2*b^2*x^2*z -      b^2*c^2*x^2*z + 2*a^2*b^2*y^2*z + a^2*c^2*y^2*z + a^2*b^2*x*z^2 + 2*b^2*c^2*x*z^2 - a^2*b^2*y*z^2 -      2*a^2*c^2*y*z^2");
		exp("K288","a^8*x^2*y - 2*a^6*b^2*x^2*y + 2*a^2*b^6*x^2*y - b^8*x^2*y - a^6*c^2*x^2*y + 2*a^4*b^2*c^2*x^2*y -      a^2*b^4*c^2*x^2*y - a^2*c^6*x^2*y + c^8*x^2*y + a^8*x*y^2 - 2*a^6*b^2*x*y^2 + 2*a^2*b^6*x*y^2 -      b^8*x*y^2 + a^4*b^2*c^2*x*y^2 - 2*a^2*b^4*c^2*x*y^2 + b^6*c^2*x*y^2 + b^2*c^6*x*y^2 - c^8*x*y^2 -      a^8*x^2*z + a^6*b^2*x^2*z + a^2*b^6*x^2*z - b^8*x^2*z + 2*a^6*c^2*x^2*z - 2*a^4*b^2*c^2*x^2*z +      a^2*b^2*c^4*x^2*z - 2*a^2*c^6*x^2*z + c^8*x^2*z + a^8*y^2*z - a^6*b^2*y^2*z - a^2*b^6*y^2*z +      b^8*y^2*z + 2*a^2*b^4*c^2*y^2*z - 2*b^6*c^2*y^2*z - a^2*b^2*c^4*y^2*z + 2*b^2*c^6*y^2*z -      c^8*y^2*z - a^8*x*z^2 + b^8*x*z^2 + 2*a^6*c^2*x*z^2 - a^4*b^2*c^2*x*z^2 - b^6*c^2*x*z^2 +      2*a^2*b^2*c^4*x*z^2 - 2*a^2*c^6*x*z^2 - b^2*c^6*x*z^2 + c^8*x*z^2 - a^8*y*z^2 + b^8*y*z^2 +      a^6*c^2*y*z^2 + a^2*b^4*c^2*y*z^2 - 2*b^6*c^2*y*z^2 - 2*a^2*b^2*c^4*y*z^2 + a^2*c^6*y*z^2 +      2*b^2*c^6*y*z^2 - c^8*y*z^2");
		exp("K289","a^6*c^2*x^2*y - 2*a^4*b^2*c^2*x^2*y + 2*a^2*b^4*c^2*x^2*y - b^6*c^2*x^2*y - a^4*c^4*x^2*y +      b^2*c^6*x^2*y + a^6*c^2*x*y^2 - 2*a^4*b^2*c^2*x*y^2 + 2*a^2*b^4*c^2*x*y^2 - b^6*c^2*x*y^2 +      b^4*c^4*x*y^2 - a^2*c^6*x*y^2 - a^6*b^2*x^2*z + a^4*b^4*x^2*z + 2*a^4*b^2*c^2*x^2*z -      b^6*c^2*x^2*z - 2*a^2*b^2*c^4*x^2*z + b^2*c^6*x^2*z - a^6*b^2*x*y*z + a^2*b^6*x*y*z +      a^6*c^2*x*y*z - b^6*c^2*x*y*z - a^2*c^6*x*y*z + b^2*c^6*x*y*z - a^4*b^4*y^2*z + a^2*b^6*y^2*z +      a^6*c^2*y^2*z - 2*a^2*b^4*c^2*y^2*z + 2*a^2*b^2*c^4*y^2*z - a^2*c^6*y^2*z - a^6*b^2*x*z^2 +      a^2*b^6*x*z^2 + 2*a^4*b^2*c^2*x*z^2 - 2*a^2*b^2*c^4*x*z^2 - b^4*c^4*x*z^2 + b^2*c^6*x*z^2 -      a^6*b^2*y*z^2 + a^2*b^6*y*z^2 - 2*a^2*b^4*c^2*y*z^2 + a^4*c^4*y*z^2 + 2*a^2*b^2*c^4*y*z^2 -      a^2*c^6*y*z^2");
		exp("K290","(-a^4)*c^2*x^2*y - a^2*b^2*c^2*x^2*y + b^4*c^2*x^2*y + a^2*c^4*x^2*y - a^4*c^2*x*y^2 +      a^2*b^2*c^2*x*y^2 + b^4*c^2*x*y^2 - b^2*c^4*x*y^2 + a^4*b^2*x^2*z - a^2*b^4*x^2*z +      a^2*b^2*c^2*x^2*z - b^2*c^4*x^2*z + a^4*b^2*y^2*z - a^2*b^4*y^2*z - a^2*b^2*c^2*y^2*z +      a^2*c^4*y^2*z + a^4*b^2*x*z^2 - a^2*b^2*c^2*x*z^2 + b^4*c^2*x*z^2 - b^2*c^4*x*z^2 - a^2*b^4*y*z^2 -      a^4*c^2*y*z^2 + a^2*b^2*c^2*y*z^2 + a^2*c^4*y*z^2");
		exp("K291","(-b^4)*c^2*x^2*y - a^2*c^4*x^2*y + b^2*c^4*x^2*y + c^6*x^2*y + a^4*c^2*x*y^2 - a^2*c^4*x*y^2 +      b^2*c^4*x*y^2 - c^6*x*y^2 + a^2*b^4*x^2*z - b^6*x^2*z - b^4*c^2*x^2*z + b^2*c^4*x^2*z + a^6*y^2*z -      a^4*b^2*y^2*z + a^4*c^2*y^2*z - a^2*c^4*y^2*z - a^4*b^2*x*z^2 + a^2*b^4*x*z^2 + b^6*x*z^2 -      b^4*c^2*x*z^2 - a^6*y*z^2 - a^4*b^2*y*z^2 + a^2*b^4*y*z^2 + a^4*c^2*y*z^2");
		exp("K292","-2*a^4*b^2*c^2*x^2*y + a^2*b^4*c^2*x^2*y + b^6*c^2*x^2*y - a^4*c^4*x^2*y + 2*a^2*c^6*x^2*y -      c^8*x^2*y - a^6*c^2*x*y^2 - a^4*b^2*c^2*x*y^2 + 2*a^2*b^4*c^2*x*y^2 + b^4*c^4*x*y^2 -      2*b^2*c^6*x*y^2 + c^8*x*y^2 + a^4*b^4*x^2*z - 2*a^2*b^6*x^2*z + b^8*x^2*z + 2*a^4*b^2*c^2*x^2*z -      a^2*b^2*c^4*x^2*z - b^2*c^6*x^2*z - a^8*y^2*z + 2*a^6*b^2*y^2*z - a^4*b^4*y^2*z -      2*a^2*b^4*c^2*y^2*z + a^2*b^2*c^4*y^2*z + a^2*c^6*y^2*z + a^6*b^2*x*z^2 - b^8*x*z^2 +      a^4*b^2*c^2*x*z^2 + 2*b^6*c^2*x*z^2 - 2*a^2*b^2*c^4*x*z^2 - b^4*c^4*x*z^2 + a^8*y*z^2 -      a^2*b^6*y*z^2 - 2*a^6*c^2*y*z^2 - a^2*b^4*c^2*y*z^2 + a^4*c^4*y*z^2 + 2*a^2*b^2*c^4*y*z^2");
		exp("K293a","a^6*x^2*y - 5*a^4*b^2*x^2*y + 5*a^2*b^4*x^2*y - b^6*x^2*y + 2*a^4*c^2*x^2*y +      6*a^2*b^2*c^2*x^2*y - 5*b^4*c^2*x^2*y - 8*a^2*c^4*x^2*y + 2*b^2*c^4*x^2*y + 3*c^6*x^2*y +      a^6*x*y^2 - 5*a^4*b^2*x*y^2 + 5*a^2*b^4*x*y^2 - b^6*x*y^2 + 5*a^4*c^2*x*y^2 - 6*a^2*b^2*c^2*x*y^2 -      2*b^4*c^2*x*y^2 - 2*a^2*c^4*x*y^2 + 8*b^2*c^4*x*y^2 - 3*c^6*x*y^2 - a^6*x^2*z - 2*a^4*b^2*x^2*z +      8*a^2*b^4*x^2*z - 3*b^6*x^2*z + 5*a^4*c^2*x^2*z - 6*a^2*b^2*c^2*x^2*z - 2*b^4*c^2*x^2*z -      5*a^2*c^4*x^2*z + 5*b^2*c^4*x^2*z + c^6*x^2*z + 3*a^6*y^2*z - 8*a^4*b^2*y^2*z + 2*a^2*b^4*y^2*z +      b^6*y^2*z + 2*a^4*c^2*y^2*z + 6*a^2*b^2*c^2*y^2*z - 5*b^4*c^2*y^2*z - 5*a^2*c^4*y^2*z +      5*b^2*c^4*y^2*z - c^6*y^2*z - a^6*x*z^2 - 5*a^4*b^2*x*z^2 + 2*a^2*b^4*x*z^2 + 3*b^6*x*z^2 +      5*a^4*c^2*x*z^2 + 6*a^2*b^2*c^2*x*z^2 - 8*b^4*c^2*x*z^2 - 5*a^2*c^4*x*z^2 + 2*b^2*c^4*x*z^2 +      c^6*x*z^2 - 3*a^6*y*z^2 - 2*a^4*b^2*y*z^2 + 5*a^2*b^4*y*z^2 + b^6*y*z^2 + 8*a^4*c^2*y*z^2 -      6*a^2*b^2*c^2*y*z^2 - 5*b^4*c^2*y*z^2 - 2*a^2*c^4*y*z^2 + 5*b^2*c^4*y*z^2 - c^6*y*z^2 +      Sqrt(a^4 - a^2*b^2 + b^4 - a^2*c^2 - b^2*c^2 + c^4)*((-a^4)*x^2*y + b^4*x^2*y + 2*a^2*c^2*x^2*y +        b^2*c^2*x^2*y - 3*c^4*x^2*y - a^4*x*y^2 + b^4*x*y^2 - a^2*c^2*x*y^2 - 2*b^2*c^2*x*y^2 +        3*c^4*x*y^2 + a^4*x^2*z - 2*a^2*b^2*x^2*z + 3*b^4*x^2*z - b^2*c^2*x^2*z - c^4*x^2*z -        3*a^4*y^2*z + 2*a^2*b^2*y^2*z - b^4*y^2*z + a^2*c^2*y^2*z + c^4*y^2*z + a^4*x*z^2 +        a^2*b^2*x*z^2 - 3*b^4*x*z^2 + 2*b^2*c^2*x*z^2 - c^4*x*z^2 + 3*a^4*y*z^2 - a^2*b^2*y*z^2 -        b^4*y*z^2 - 2*a^2*c^2*y*z^2 + c^4*y*z^2)");
		exp("K293b","a^6*x^2*y - 5*a^4*b^2*x^2*y + 5*a^2*b^4*x^2*y - b^6*x^2*y + 2*a^4*c^2*x^2*y +      6*a^2*b^2*c^2*x^2*y - 5*b^4*c^2*x^2*y - 8*a^2*c^4*x^2*y + 2*b^2*c^4*x^2*y + 3*c^6*x^2*y +      a^6*x*y^2 - 5*a^4*b^2*x*y^2 + 5*a^2*b^4*x*y^2 - b^6*x*y^2 + 5*a^4*c^2*x*y^2 - 6*a^2*b^2*c^2*x*y^2 -      2*b^4*c^2*x*y^2 - 2*a^2*c^4*x*y^2 + 8*b^2*c^4*x*y^2 - 3*c^6*x*y^2 - a^6*x^2*z - 2*a^4*b^2*x^2*z +      8*a^2*b^4*x^2*z - 3*b^6*x^2*z + 5*a^4*c^2*x^2*z - 6*a^2*b^2*c^2*x^2*z - 2*b^4*c^2*x^2*z -      5*a^2*c^4*x^2*z + 5*b^2*c^4*x^2*z + c^6*x^2*z + 3*a^6*y^2*z - 8*a^4*b^2*y^2*z + 2*a^2*b^4*y^2*z +      b^6*y^2*z + 2*a^4*c^2*y^2*z + 6*a^2*b^2*c^2*y^2*z - 5*b^4*c^2*y^2*z - 5*a^2*c^4*y^2*z +      5*b^2*c^4*y^2*z - c^6*y^2*z - a^6*x*z^2 - 5*a^4*b^2*x*z^2 + 2*a^2*b^4*x*z^2 + 3*b^6*x*z^2 +      5*a^4*c^2*x*z^2 + 6*a^2*b^2*c^2*x*z^2 - 8*b^4*c^2*x*z^2 - 5*a^2*c^4*x*z^2 + 2*b^2*c^4*x*z^2 +      c^6*x*z^2 - 3*a^6*y*z^2 - 2*a^4*b^2*y*z^2 + 5*a^2*b^4*y*z^2 + b^6*y*z^2 + 8*a^4*c^2*y*z^2 -      6*a^2*b^2*c^2*y*z^2 - 5*b^4*c^2*y*z^2 - 2*a^2*c^4*y*z^2 + 5*b^2*c^4*y*z^2 - c^6*y*z^2 +      Sqrt(a^4 - a^2*b^2 + b^4 - a^2*c^2 - b^2*c^2 + c^4)*(a^4*x^2*y - b^4*x^2*y - 2*a^2*c^2*x^2*y -        b^2*c^2*x^2*y + 3*c^4*x^2*y + a^4*x*y^2 - b^4*x*y^2 + a^2*c^2*x*y^2 + 2*b^2*c^2*x*y^2 -        3*c^4*x*y^2 - a^4*x^2*z + 2*a^2*b^2*x^2*z - 3*b^4*x^2*z + b^2*c^2*x^2*z + c^4*x^2*z +        3*a^4*y^2*z - 2*a^2*b^2*y^2*z + b^4*y^2*z - a^2*c^2*y^2*z - c^4*y^2*z - a^4*x*z^2 -        a^2*b^2*x*z^2 + 3*b^4*x*z^2 - 2*b^2*c^2*x*z^2 + c^4*x*z^2 - 3*a^4*y*z^2 + a^2*b^2*y*z^2 +        b^4*y*z^2 + 2*a^2*c^2*y*z^2 - c^4*y*z^2)");
		exp("K294","a^3*b*c^2*x^2*y - 2*a^2*b^2*c^2*x^2*y + a*b^3*c^2*x^2*y - a^2*b*c^3*x^2*y + b^3*c^3*x^2*y +      a*b*c^4*x^2*y - b*c^5*x^2*y - a^3*b*c^2*x*y^2 + 2*a^2*b^2*c^2*x*y^2 - a*b^3*c^2*x*y^2 -      a^3*c^3*x*y^2 + a*b^2*c^3*x*y^2 - a*b*c^4*x*y^2 + a*c^5*x*y^2 - a^3*b^2*c*x^2*z + a^2*b^3*c*x^2*z -      a*b^4*c*x^2*z + b^5*c*x^2*z + 2*a^2*b^2*c^2*x^2*z - a*b^2*c^3*x^2*z - b^3*c^3*x^2*z - a^5*c*y^2*z +      a^4*b*c*y^2*z - a^3*b^2*c*y^2*z + a^2*b^3*c*y^2*z - 2*a^2*b^2*c^2*y^2*z + a^3*c^3*y^2*z +      a^2*b*c^3*y^2*z + a^3*b^3*x*z^2 - a*b^5*x*z^2 + a^3*b^2*c*x*z^2 + a*b^4*c*x*z^2 -      2*a^2*b^2*c^2*x*z^2 - a*b^3*c^2*x*z^2 + a*b^2*c^3*x*z^2 + a^5*b*y*z^2 - a^3*b^3*y*z^2 -      a^4*b*c*y*z^2 - a^2*b^3*c*y*z^2 + a^3*b*c^2*y*z^2 + 2*a^2*b^2*c^2*y*z^2 - a^2*b*c^3*y*z^2");
		exp("K295","a^2*x^2*y - b^2*x^2*y - 3*c^2*x^2*y + a^2*x*y^2 - b^2*x*y^2 + 3*c^2*x*y^2 - a^2*x^2*z +      3*b^2*x^2*z + c^2*x^2*z - 3*a^2*y^2*z + b^2*y^2*z - c^2*y^2*z - a^2*x*z^2 - 3*b^2*x*z^2 +      c^2*x*z^2 + 3*a^2*y*z^2 + b^2*y*z^2 - c^2*y*z^2");
		exp("K296","a*x^2*y - c*x^2*y - b*x*y^2 + c*x*y^2 - a*x^2*z + b*x^2*z - a*y^2*z + b*y^2*z - b*x*z^2 +      c*x*z^2 + a*y*z^2 - c*y*z^2");
		exp("K297","3*a^2*b^2*c^4*x^2*y + b^4*c^4*x^2*y - b^2*c^6*x^2*y - a^4*c^4*x*y^2 - 3*a^2*b^2*c^4*x*y^2 +      a^2*c^6*x*y^2 - 3*a^2*b^4*c^2*x^2*z + b^6*c^2*x^2*z - b^4*c^4*x^2*z - a^6*c^2*y^2*z +      3*a^4*b^2*c^2*y^2*z + a^4*c^4*y^2*z + a^4*b^4*x*z^2 - a^2*b^6*x*z^2 + 3*a^2*b^4*c^2*x*z^2 +      a^6*b^2*y*z^2 - a^4*b^4*y*z^2 - 3*a^4*b^2*c^2*y*z^2");
		exp("K298","2*a^6*c^2*x^2*y - 3*a^4*b^2*c^2*x^2*y + 4*a^2*b^4*c^2*x^2*y - 3*b^6*c^2*x^2*y - a^4*c^4*x^2*y -      b^4*c^4*x^2*y - 2*a^2*c^6*x^2*y + 3*b^2*c^6*x^2*y + c^8*x^2*y + 3*a^6*c^2*x*y^2 -      4*a^4*b^2*c^2*x*y^2 + 3*a^2*b^4*c^2*x*y^2 - 2*b^6*c^2*x*y^2 + a^4*c^4*x*y^2 + b^4*c^4*x*y^2 -      3*a^2*c^6*x*y^2 + 2*b^2*c^6*x*y^2 - c^8*x*y^2 - 2*a^6*b^2*x^2*z + a^4*b^4*x^2*z + 2*a^2*b^6*x^2*z -      b^8*x^2*z + 3*a^4*b^2*c^2*x^2*z - 3*b^6*c^2*x^2*z - 4*a^2*b^2*c^4*x^2*z + b^4*c^4*x^2*z +      3*b^2*c^6*x^2*z - 2*a^6*b^2*x*y*z + 2*a^2*b^6*x*y*z + 2*a^6*c^2*x*y*z - 2*b^6*c^2*x*y*z -      2*a^2*c^6*x*y*z + 2*b^2*c^6*x*y*z + a^8*y^2*z - 2*a^6*b^2*y^2*z - a^4*b^4*y^2*z + 2*a^2*b^6*y^2*z +      3*a^6*c^2*y^2*z - 3*a^2*b^4*c^2*y^2*z - a^4*c^4*y^2*z + 4*a^2*b^2*c^4*y^2*z - 3*a^2*c^6*y^2*z -      3*a^6*b^2*x*z^2 - a^4*b^4*x*z^2 + 3*a^2*b^6*x*z^2 + b^8*x*z^2 + 4*a^4*b^2*c^2*x*z^2 -      2*b^6*c^2*x*z^2 - 3*a^2*b^2*c^4*x*z^2 - b^4*c^4*x*z^2 + 2*b^2*c^6*x*z^2 - a^8*y*z^2 -      3*a^6*b^2*y*z^2 + a^4*b^4*y*z^2 + 3*a^2*b^6*y*z^2 + 2*a^6*c^2*y*z^2 - 4*a^2*b^4*c^2*y*z^2 +      a^4*c^4*y*z^2 + 3*a^2*b^2*c^4*y*z^2 - 2*a^2*c^6*y*z^2");
		exp("K299","2*a^3*b*c*x^2*y - 2*a*b^3*c*x^2*y - 2*a^3*c^2*x^2*y - 2*a^2*b*c^2*x^2*y + a*b^2*c^2*x^2*y +      3*b^3*c^2*x^2*y + 2*a^2*c^3*x^2*y - 3*b^2*c^3*x^2*y + a*c^4*x^2*y + b*c^4*x^2*y - c^5*x^2*y +      2*a^3*b*c*x*y^2 - 2*a*b^3*c*x*y^2 - 3*a^3*c^2*x*y^2 - a^2*b*c^2*x*y^2 + 2*a*b^2*c^2*x*y^2 +      2*b^3*c^2*x*y^2 + 3*a^2*c^3*x*y^2 - 2*b^2*c^3*x*y^2 - a*c^4*x*y^2 - b*c^4*x*y^2 + c^5*x*y^2 +      2*a^3*b^2*x^2*z - 2*a^2*b^3*x^2*z - a*b^4*x^2*z + b^5*x^2*z - 2*a^3*b*c*x^2*z + 2*a^2*b^2*c*x^2*z -      b^4*c*x^2*z - a*b^2*c^2*x^2*z + 3*b^3*c^2*x^2*z + 2*a*b*c^3*x^2*z - 3*b^2*c^3*x^2*z +      2*a^3*b^2*x*y*z - 2*a^2*b^3*x*y*z - 2*a^3*c^2*x*y*z + 2*b^3*c^2*x*y*z + 2*a^2*c^3*x*y*z -      2*b^2*c^3*x*y*z - a^5*y^2*z + a^4*b*y^2*z + 2*a^3*b^2*y^2*z - 2*a^2*b^3*y^2*z + a^4*c*y^2*z -      2*a^2*b^2*c*y^2*z + 2*a*b^3*c*y^2*z - 3*a^3*c^2*y^2*z + a^2*b*c^2*y^2*z + 3*a^2*c^3*y^2*z -      2*a*b*c^3*y^2*z + 3*a^3*b^2*x*z^2 - 3*a^2*b^3*x*z^2 + a*b^4*x*z^2 - b^5*x*z^2 - 2*a^3*b*c*x*z^2 +      a^2*b^2*c*x*z^2 + b^4*c*x*z^2 - 2*a*b^2*c^2*x*z^2 + 2*b^3*c^2*x*z^2 + 2*a*b*c^3*x*z^2 -      2*b^2*c^3*x*z^2 + a^5*y*z^2 - a^4*b*y*z^2 + 3*a^3*b^2*y*z^2 - 3*a^2*b^3*y*z^2 - a^4*c*y*z^2 -      a^2*b^2*c*y*z^2 + 2*a*b^3*c*y*z^2 - 2*a^3*c^2*y*z^2 + 2*a^2*b*c^2*y*z^2 + 2*a^2*c^3*y*z^2 -      2*a*b*c^3*y*z^2");
		exp("K300","2*a^6*c^2*x^2*y - 4*a^4*b^2*c^2*x^2*y + 4*a^2*b^4*c^2*x^2*y - 2*b^6*c^2*x^2*y - 3*a^4*c^4*x^2*y +      a^2*c^6*x^2*y + 2*b^2*c^6*x^2*y + 2*a^6*c^2*x*y^2 - 4*a^4*b^2*c^2*x*y^2 + 4*a^2*b^4*c^2*x*y^2 -      2*b^6*c^2*x*y^2 + 3*b^4*c^4*x*y^2 - 2*a^2*c^6*x*y^2 - b^2*c^6*x*y^2 - 2*a^6*b^2*x^2*z +      3*a^4*b^4*x^2*z - a^2*b^6*x^2*z + 4*a^4*b^2*c^2*x^2*z - 2*b^6*c^2*x^2*z - 4*a^2*b^2*c^4*x^2*z +      2*b^2*c^6*x^2*z - a^6*b^2*x*y*z + a^2*b^6*x*y*z + a^6*c^2*x*y*z - b^6*c^2*x*y*z - a^2*c^6*x*y*z +      b^2*c^6*x*y*z + a^6*b^2*y^2*z - 3*a^4*b^4*y^2*z + 2*a^2*b^6*y^2*z + 2*a^6*c^2*y^2*z -      4*a^2*b^4*c^2*y^2*z + 4*a^2*b^2*c^4*y^2*z - 2*a^2*c^6*y^2*z - 2*a^6*b^2*x*z^2 + 2*a^2*b^6*x*z^2 +      4*a^4*b^2*c^2*x*z^2 + b^6*c^2*x*z^2 - 4*a^2*b^2*c^4*x*z^2 - 3*b^4*c^4*x*z^2 + 2*b^2*c^6*x*z^2 -      2*a^6*b^2*y*z^2 + 2*a^2*b^6*y*z^2 - a^6*c^2*y*z^2 - 4*a^2*b^4*c^2*y*z^2 + 3*a^4*c^4*y*z^2 +      4*a^2*b^2*c^4*y*z^2 - 2*a^2*c^6*y*z^2");
		exp("K301","a^8*x^2*y - 2*a^6*b^2*x^2*y + 2*a^2*b^6*x^2*y - b^8*x^2*y - a^6*c^2*x^2*y + 2*a^4*b^2*c^2*x^2*y -      a^2*b^4*c^2*x^2*y + a^4*c^4*x^2*y - 2*a^2*c^6*x^2*y + c^8*x^2*y + a^8*x*y^2 - 2*a^6*b^2*x*y^2 +      2*a^2*b^6*x*y^2 - b^8*x*y^2 + a^4*b^2*c^2*x*y^2 - 2*a^2*b^4*c^2*x*y^2 + b^6*c^2*x*y^2 -      b^4*c^4*x*y^2 + 2*b^2*c^6*x*y^2 - c^8*x*y^2 - a^8*x^2*z + a^6*b^2*x^2*z - a^4*b^4*x^2*z +      2*a^2*b^6*x^2*z - b^8*x^2*z + 2*a^6*c^2*x^2*z - 2*a^4*b^2*c^2*x^2*z + a^2*b^2*c^4*x^2*z -      2*a^2*c^6*x^2*z + c^8*x^2*z - a^6*b^2*x*y*z + a^2*b^6*x*y*z + a^6*c^2*x*y*z - b^6*c^2*x*y*z -      a^2*c^6*x*y*z + b^2*c^6*x*y*z + a^8*y^2*z - 2*a^6*b^2*y^2*z + a^4*b^4*y^2*z - a^2*b^6*y^2*z +      b^8*y^2*z + 2*a^2*b^4*c^2*y^2*z - 2*b^6*c^2*y^2*z - a^2*b^2*c^4*y^2*z + 2*b^2*c^6*y^2*z -      c^8*y^2*z - a^8*x*z^2 + b^8*x*z^2 + 2*a^6*c^2*x*z^2 - a^4*b^2*c^2*x*z^2 - 2*b^6*c^2*x*z^2 +      2*a^2*b^2*c^4*x*z^2 + b^4*c^4*x*z^2 - 2*a^2*c^6*x*z^2 - b^2*c^6*x*z^2 + c^8*x*z^2 - a^8*y*z^2 +      b^8*y*z^2 + 2*a^6*c^2*y*z^2 + a^2*b^4*c^2*y*z^2 - 2*b^6*c^2*y*z^2 - a^4*c^4*y*z^2 -      2*a^2*b^2*c^4*y*z^2 + a^2*c^6*y*z^2 + 2*b^2*c^6*y*z^2 - c^8*y*z^2");
		exp("K302","(-a^4)*b^2*c^2*x^2*y + b^6*c^2*x^2*y - 2*a^4*c^4*x^2*y + b^4*c^4*x^2*y + 3*a^2*c^6*x^2*y -      b^2*c^6*x^2*y - c^8*x^2*y - a^6*c^2*x*y^2 + a^2*b^4*c^2*x*y^2 - a^4*c^4*x*y^2 + 2*b^4*c^4*x*y^2 +      a^2*c^6*x*y^2 - 3*b^2*c^6*x*y^2 + c^8*x*y^2 + 2*a^4*b^4*x^2*z - 3*a^2*b^6*x^2*z + b^8*x^2*z +      a^4*b^2*c^2*x^2*z + b^6*c^2*x^2*z - b^4*c^4*x^2*z - b^2*c^6*x^2*z + a^6*b^2*x*y*z - a^2*b^6*x*y*z -      a^6*c^2*x*y*z + b^6*c^2*x*y*z + a^2*c^6*x*y*z - b^2*c^6*x*y*z - a^8*y^2*z + 3*a^6*b^2*y^2*z -      2*a^4*b^4*y^2*z - a^6*c^2*y^2*z - a^2*b^4*c^2*y^2*z + a^4*c^4*y^2*z + a^2*c^6*y^2*z +      a^6*b^2*x*z^2 + a^4*b^4*x*z^2 - a^2*b^6*x*z^2 - b^8*x*z^2 + 3*b^6*c^2*x*z^2 - a^2*b^2*c^4*x*z^2 -      2*b^4*c^4*x*z^2 + a^8*y*z^2 + a^6*b^2*y*z^2 - a^4*b^4*y*z^2 - a^2*b^6*y*z^2 - 3*a^6*c^2*y*z^2 +      2*a^4*c^4*y*z^2 + a^2*b^2*c^4*y*z^2");
		exp("K303a","Sqrt((a + b - c)*(a - b + c)*(-a + b + c)*(a + b + c))*(2*a^2*c^2*x^2*y - b^2*c^2*x^2*y -        c^4*x^2*y + a^2*c^2*x*y^2 - 2*b^2*c^2*x*y^2 + c^4*x*y^2 - 2*a^2*b^2*x^2*z + b^4*x^2*z +        b^2*c^2*x^2*z - a^4*y^2*z + 2*a^2*b^2*y^2*z - a^2*c^2*y^2*z - a^2*b^2*x*z^2 - b^4*x*z^2 +        2*b^2*c^2*x*z^2 + a^4*y*z^2 + a^2*b^2*y*z^2 - 2*a^2*c^2*y*z^2) +      Sqrt(3)*(2*a^4*c^2*x^2*y + a^2*b^2*c^2*x^2*y - 3*b^4*c^2*x^2*y - 3*a^2*c^4*x^2*y +        2*b^2*c^4*x^2*y + c^6*x^2*y + 3*a^4*c^2*x*y^2 - a^2*b^2*c^2*x*y^2 - 2*b^4*c^2*x*y^2 -        2*a^2*c^4*x*y^2 + 3*b^2*c^4*x*y^2 - c^6*x*y^2 - 2*a^4*b^2*x^2*z + 3*a^2*b^4*x^2*z - b^6*x^2*z -        a^2*b^2*c^2*x^2*z - 2*b^4*c^2*x^2*z + 3*b^2*c^4*x^2*z - 2*a^4*b^2*x*y*z + 2*a^2*b^4*x*y*z +        2*a^4*c^2*x*y*z - 2*b^4*c^2*x*y*z - 2*a^2*c^4*x*y*z + 2*b^2*c^4*x*y*z + a^6*y^2*z -        3*a^4*b^2*y^2*z + 2*a^2*b^4*y^2*z + 2*a^4*c^2*y^2*z + a^2*b^2*c^2*y^2*z - 3*a^2*c^4*y^2*z -        3*a^4*b^2*x*z^2 + 2*a^2*b^4*x*z^2 + b^6*x*z^2 + a^2*b^2*c^2*x*z^2 - 3*b^4*c^2*x*z^2 +        2*b^2*c^4*x*z^2 - a^6*y*z^2 - 2*a^4*b^2*y*z^2 + 3*a^2*b^4*y*z^2 + 3*a^4*c^2*y*z^2 -        a^2*b^2*c^2*y*z^2 - 2*a^2*c^4*y*z^2)");
		exp("K303b","Sqrt((a + b - c)*(a - b + c)*(-a + b + c)*(a + b + c))*(2*a^2*c^2*x^2*y - b^2*c^2*x^2*y -        c^4*x^2*y + a^2*c^2*x*y^2 - 2*b^2*c^2*x*y^2 + c^4*x*y^2 - 2*a^2*b^2*x^2*z + b^4*x^2*z +        b^2*c^2*x^2*z - a^4*y^2*z + 2*a^2*b^2*y^2*z - a^2*c^2*y^2*z - a^2*b^2*x*z^2 - b^4*x*z^2 +        2*b^2*c^2*x*z^2 + a^4*y*z^2 + a^2*b^2*y*z^2 - 2*a^2*c^2*y*z^2) +      Sqrt(3)*(-2*a^4*c^2*x^2*y - a^2*b^2*c^2*x^2*y + 3*b^4*c^2*x^2*y + 3*a^2*c^4*x^2*y -        2*b^2*c^4*x^2*y - c^6*x^2*y - 3*a^4*c^2*x*y^2 + a^2*b^2*c^2*x*y^2 + 2*b^4*c^2*x*y^2 +        2*a^2*c^4*x*y^2 - 3*b^2*c^4*x*y^2 + c^6*x*y^2 + 2*a^4*b^2*x^2*z - 3*a^2*b^4*x^2*z + b^6*x^2*z +        a^2*b^2*c^2*x^2*z + 2*b^4*c^2*x^2*z - 3*b^2*c^4*x^2*z + 2*a^4*b^2*x*y*z - 2*a^2*b^4*x*y*z -        2*a^4*c^2*x*y*z + 2*b^4*c^2*x*y*z + 2*a^2*c^4*x*y*z - 2*b^2*c^4*x*y*z - a^6*y^2*z +        3*a^4*b^2*y^2*z - 2*a^2*b^4*y^2*z - 2*a^4*c^2*y^2*z - a^2*b^2*c^2*y^2*z + 3*a^2*c^4*y^2*z +        3*a^4*b^2*x*z^2 - 2*a^2*b^4*x*z^2 - b^6*x*z^2 - a^2*b^2*c^2*x*z^2 + 3*b^4*c^2*x*z^2 -        2*b^2*c^4*x*z^2 + a^6*y*z^2 + 2*a^4*b^2*y*z^2 - 3*a^2*b^4*y*z^2 - 3*a^4*c^2*y*z^2 +        a^2*b^2*c^2*y*z^2 + 2*a^2*c^4*y*z^2)");
		exp("K304","-2*a^6*c^2*x^2*y + 2*a^4*b^2*c^2*x^2*y - a^2*b^4*c^2*x^2*y + b^6*c^2*x^2*y + 5*a^4*c^4*x^2*y -      b^4*c^4*x^2*y - 4*a^2*c^6*x^2*y - b^2*c^6*x^2*y + c^8*x^2*y - a^6*c^2*x*y^2 + a^4*b^2*c^2*x*y^2 -      2*a^2*b^4*c^2*x*y^2 + 2*b^6*c^2*x*y^2 + a^4*c^4*x*y^2 - 5*b^4*c^4*x*y^2 + a^2*c^6*x*y^2 +      4*b^2*c^6*x*y^2 - c^8*x*y^2 + 2*a^6*b^2*x^2*z - 5*a^4*b^4*x^2*z + 4*a^2*b^6*x^2*z - b^8*x^2*z -      2*a^4*b^2*c^2*x^2*z + b^6*c^2*x^2*z + a^2*b^2*c^4*x^2*z + b^4*c^4*x^2*z - b^2*c^6*x^2*z +      a^8*y^2*z - 4*a^6*b^2*y^2*z + 5*a^4*b^4*y^2*z - 2*a^2*b^6*y^2*z - a^6*c^2*y^2*z +      2*a^2*b^4*c^2*y^2*z - a^4*c^4*y^2*z - a^2*b^2*c^4*y^2*z + a^2*c^6*y^2*z + a^6*b^2*x*z^2 -      a^4*b^4*x*z^2 - a^2*b^6*x*z^2 + b^8*x*z^2 - a^4*b^2*c^2*x*z^2 - 4*b^6*c^2*x*z^2 +      2*a^2*b^2*c^4*x*z^2 + 5*b^4*c^4*x*z^2 - 2*b^2*c^6*x*z^2 - a^8*y*z^2 + a^6*b^2*y*z^2 +      a^4*b^4*y*z^2 - a^2*b^6*y*z^2 + 4*a^6*c^2*y*z^2 + a^2*b^4*c^2*y*z^2 - 5*a^4*c^4*y*z^2 -      2*a^2*b^2*c^4*y*z^2 + 2*a^2*c^6*y*z^2");
		exp("K305","-2*a^6*c^2*x^2*y + a^4*b^2*c^2*x^2*y - a^2*b^4*c^2*x^2*y + 2*b^6*c^2*x^2*y + 3*a^4*c^4*x^2*y -      a^2*c^6*x^2*y - 2*b^2*c^6*x^2*y - 2*a^6*c^2*x*y^2 + a^4*b^2*c^2*x*y^2 - a^2*b^4*c^2*x*y^2 +      2*b^6*c^2*x*y^2 - 3*b^4*c^4*x*y^2 + 2*a^2*c^6*x*y^2 + b^2*c^6*x*y^2 + 2*a^6*b^2*x^2*z -      3*a^4*b^4*x^2*z + a^2*b^6*x^2*z - a^4*b^2*c^2*x^2*z + 2*b^6*c^2*x^2*z + a^2*b^2*c^4*x^2*z -      2*b^2*c^6*x^2*z + a^6*b^2*x*y*z - a^2*b^6*x*y*z - a^6*c^2*x*y*z + b^6*c^2*x*y*z + a^2*c^6*x*y*z -      b^2*c^6*x*y*z - a^6*b^2*y^2*z + 3*a^4*b^4*y^2*z - 2*a^2*b^6*y^2*z - 2*a^6*c^2*y^2*z +      a^2*b^4*c^2*y^2*z - a^2*b^2*c^4*y^2*z + 2*a^2*c^6*y^2*z + 2*a^6*b^2*x*z^2 - 2*a^2*b^6*x*z^2 -      a^4*b^2*c^2*x*z^2 - b^6*c^2*x*z^2 + a^2*b^2*c^4*x*z^2 + 3*b^4*c^4*x*z^2 - 2*b^2*c^6*x*z^2 +      2*a^6*b^2*y*z^2 - 2*a^2*b^6*y*z^2 + a^6*c^2*y*z^2 + a^2*b^4*c^2*y*z^2 - 3*a^4*c^4*y*z^2 -      a^2*b^2*c^4*y*z^2 + 2*a^2*c^6*y*z^2");
		exp("K306","(-a^3)*b*c^2*x^2*y + a*b^3*c^2*x^2*y - a^3*c^3*x^2*y + a*c^5*x^2*y - a^3*b*c^2*x*y^2 +      a*b^3*c^2*x*y^2 + b^3*c^3*x*y^2 - b*c^5*x*y^2 + a^3*b^3*x^2*z - a*b^5*x^2*z + a^3*b^2*c*x^2*z -      a*b^2*c^3*x^2*z + a^5*b*x*y*z - a*b^5*x*y*z - a^5*c*x*y*z + 2*a^3*b^2*c*x*y*z - 2*a^2*b^3*c*x*y*z +      b^5*c*x*y*z - 2*a^3*b*c^2*x*y*z + 2*a*b^3*c^2*x*y*z + 2*a^2*b*c^3*x*y*z - 2*a*b^2*c^3*x*y*z +      a*c^5*x*y*z - b*c^5*x*y*z + a^5*b*y^2*z - a^3*b^3*y^2*z - a^2*b^3*c*y^2*z + a^2*b*c^3*y^2*z +      a^3*b^2*c*x*z^2 + b^5*c*x*z^2 - a*b^2*c^3*x*z^2 - b^3*c^3*x*z^2 - a^5*c*y*z^2 - a^2*b^3*c*y*z^2 +      a^3*c^3*y*z^2 + a^2*b*c^3*y*z^2");
		exp("K307","a^4*c^2*x^2*y - 2*a^2*b^2*c^2*x^2*y + b^4*c^2*x^2*y - a^2*c^4*x^2*y - b^2*c^4*x^2*y -      a^4*c^2*x*y^2 + 2*a^2*b^2*c^2*x*y^2 - b^4*c^2*x*y^2 + a^2*c^4*x*y^2 + b^2*c^4*x*y^2 -      a^4*b^2*x^2*z + a^2*b^4*x^2*z + 2*a^2*b^2*c^2*x^2*z + b^4*c^2*x^2*z - b^2*c^4*x^2*z +      2*a^4*b^2*x*y*z - 2*a^2*b^4*x*y*z - 2*a^4*c^2*x*y*z + 2*b^4*c^2*x*y*z + 2*a^2*c^4*x*y*z -      2*b^2*c^4*x*y*z - a^4*b^2*y^2*z + a^2*b^4*y^2*z - a^4*c^2*y^2*z - 2*a^2*b^2*c^2*y^2*z +      a^2*c^4*y^2*z + a^4*b^2*x*z^2 - a^2*b^4*x*z^2 - 2*a^2*b^2*c^2*x*z^2 - b^4*c^2*x*z^2 +      b^2*c^4*x*z^2 + a^4*b^2*y*z^2 - a^2*b^4*y*z^2 + a^4*c^2*y*z^2 + 2*a^2*b^2*c^2*y*z^2 -      a^2*c^4*y*z^2");
		exp("K308","a*c*x^2*y - b*c*x^2*y + c^2*x^2*y + a*c*x*y^2 - b*c*x*y^2 - c^2*x*y^2 - a*b*x^2*z - b^2*x^2*z +      b*c*x^2*z + a^2*y^2*z + a*b*y^2*z - a*c*y^2*z - a*b*x*z^2 + b^2*x*z^2 + b*c*x*z^2 - a^2*y*z^2 +      a*b*y*z^2 - a*c*y*z^2");
		exp("K309","(-a^4)*c^2*x^2*y - 3*a^2*b^2*c^2*x^2*y + 4*b^4*c^2*x^2*y + a^2*c^4*x^2*y - 4*b^2*c^4*x^2*y -      4*a^4*c^2*x*y^2 + 3*a^2*b^2*c^2*x*y^2 + b^4*c^2*x*y^2 + 4*a^2*c^4*x*y^2 - b^2*c^4*x*y^2 +      a^4*b^2*x^2*z - a^2*b^4*x^2*z + 3*a^2*b^2*c^2*x^2*z + 4*b^4*c^2*x^2*z - 4*b^2*c^4*x^2*z +      2*a^4*b^2*x*y*z - 2*a^2*b^4*x*y*z - 2*a^4*c^2*x*y*z + 2*b^4*c^2*x*y*z + 2*a^2*c^4*x*y*z -      2*b^2*c^4*x*y*z + a^4*b^2*y^2*z - a^2*b^4*y^2*z - 4*a^4*c^2*y^2*z - 3*a^2*b^2*c^2*y^2*z +      4*a^2*c^4*y^2*z + 4*a^4*b^2*x*z^2 - 4*a^2*b^4*x*z^2 - 3*a^2*b^2*c^2*x*z^2 + b^4*c^2*x*z^2 -      b^2*c^4*x*z^2 + 4*a^4*b^2*y*z^2 - 4*a^2*b^4*y*z^2 - a^4*c^2*y*z^2 + 3*a^2*b^2*c^2*y*z^2 +      a^2*c^4*y*z^2");
		exp("K310","-7*a^6*x^2*y + 21*a^4*b^2*x^2*y - 21*a^2*b^4*x^2*y + 7*b^6*x^2*y - a^4*c^2*x^2*y -      10*a^2*b^2*c^2*x^2*y + 11*b^4*c^2*x^2*y + 15*a^2*c^4*x^2*y - 11*b^2*c^4*x^2*y - 7*c^6*x^2*y -      7*a^6*x*y^2 + 21*a^4*b^2*x*y^2 - 21*a^2*b^4*x*y^2 + 7*b^6*x*y^2 - 11*a^4*c^2*x*y^2 +      10*a^2*b^2*c^2*x*y^2 + b^4*c^2*x*y^2 + 11*a^2*c^4*x*y^2 - 15*b^2*c^4*x*y^2 + 7*c^6*x*y^2 +      7*a^6*x^2*z + a^4*b^2*x^2*z - 15*a^2*b^4*x^2*z + 7*b^6*x^2*z - 21*a^4*c^2*x^2*z +      10*a^2*b^2*c^2*x^2*z + 11*b^4*c^2*x^2*z + 21*a^2*c^4*x^2*z - 11*b^2*c^4*x^2*z - 7*c^6*x^2*z +      16*a^4*b^2*x*y*z - 16*a^2*b^4*x*y*z - 16*a^4*c^2*x*y*z + 16*b^4*c^2*x*y*z + 16*a^2*c^4*x*y*z -      16*b^2*c^4*x*y*z - 7*a^6*y^2*z + 15*a^4*b^2*y^2*z - a^2*b^4*y^2*z - 7*b^6*y^2*z -      11*a^4*c^2*y^2*z - 10*a^2*b^2*c^2*y^2*z + 21*b^4*c^2*y^2*z + 11*a^2*c^4*y^2*z - 21*b^2*c^4*y^2*z +      7*c^6*y^2*z + 7*a^6*x*z^2 + 11*a^4*b^2*x*z^2 - 11*a^2*b^4*x*z^2 - 7*b^6*x*z^2 - 21*a^4*c^2*x*z^2 -      10*a^2*b^2*c^2*x*z^2 + 15*b^4*c^2*x*z^2 + 21*a^2*c^4*x*z^2 - b^2*c^4*x*z^2 - 7*c^6*x*z^2 +      7*a^6*y*z^2 + 11*a^4*b^2*y*z^2 - 11*a^2*b^4*y*z^2 - 7*b^6*y*z^2 - 15*a^4*c^2*y*z^2 +      10*a^2*b^2*c^2*y*z^2 + 21*b^4*c^2*y*z^2 + a^2*c^4*y*z^2 - 21*b^2*c^4*y*z^2 + 7*c^6*y*z^2");
		exp("K311","(-a^2)*x^2*y + b^2*x^2*y + a*c*x^2*y - c^2*x^2*y - a^2*x*y^2 + b^2*x*y^2 - b*c*x*y^2 +      c^2*x*y^2 + a^2*x^2*z - a*b*x^2*z + b^2*x^2*z - c^2*x^2*z - a^2*y^2*z + a*b*y^2*z - b^2*y^2*z +      c^2*y^2*z + a^2*x*z^2 - b^2*x*z^2 + b*c*x*z^2 - c^2*x*z^2 + a^2*y*z^2 - b^2*y*z^2 - a*c*y*z^2 +      c^2*y*z^2");
		exp("K312","a^2*b^2*c^4*x^2*y - b^4*c^4*x^2*y - a*b^2*c^5*x^2*y + b^2*c^6*x^2*y + a^4*c^4*x*y^2 -      a^2*b^2*c^4*x*y^2 + a^2*b*c^5*x*y^2 - a^2*c^6*x*y^2 - a^2*b^4*c^2*x^2*z + a*b^5*c^2*x^2*z -      b^6*c^2*x^2*z + b^4*c^4*x^2*z + a^6*c^2*y^2*z - a^5*b*c^2*y^2*z + a^4*b^2*c^2*y^2*z -      a^4*c^4*y^2*z - a^4*b^4*x*z^2 + a^2*b^6*x*z^2 - a^2*b^5*c*x*z^2 + a^2*b^4*c^2*x*z^2 -      a^6*b^2*y*z^2 + a^4*b^4*y*z^2 + a^5*b^2*c*y*z^2 - a^4*b^2*c^2*y*z^2");
		exp("K313","a^6*x^2*y - 3*a^4*b^2*x^2*y + 3*a^2*b^4*x^2*y - b^6*x^2*y + 2*a^4*c^2*x^2*y +      4*a^2*b^2*c^2*x^2*y - 6*b^4*c^2*x^2*y - 7*a^2*c^4*x^2*y + 3*b^2*c^4*x^2*y + 4*c^6*x^2*y +      a^6*x*y^2 - 3*a^4*b^2*x*y^2 + 3*a^2*b^4*x*y^2 - b^6*x*y^2 + 6*a^4*c^2*x*y^2 - 4*a^2*b^2*c^2*x*y^2 -      2*b^4*c^2*x*y^2 - 3*a^2*c^4*x*y^2 + 7*b^2*c^4*x*y^2 - 4*c^6*x*y^2 - a^6*x^2*z - 2*a^4*b^2*x^2*z +      7*a^2*b^4*x^2*z - 4*b^6*x^2*z + 3*a^4*c^2*x^2*z - 4*a^2*b^2*c^2*x^2*z - 3*b^4*c^2*x^2*z -      3*a^2*c^4*x^2*z + 6*b^2*c^4*x^2*z + c^6*x^2*z + 4*a^6*y^2*z - 7*a^4*b^2*y^2*z + 2*a^2*b^4*y^2*z +      b^6*y^2*z + 3*a^4*c^2*y^2*z + 4*a^2*b^2*c^2*y^2*z - 3*b^4*c^2*y^2*z - 6*a^2*c^4*y^2*z +      3*b^2*c^4*y^2*z - c^6*y^2*z - a^6*x*z^2 - 6*a^4*b^2*x*z^2 + 3*a^2*b^4*x*z^2 + 4*b^6*x*z^2 +      3*a^4*c^2*x*z^2 + 4*a^2*b^2*c^2*x*z^2 - 7*b^4*c^2*x*z^2 - 3*a^2*c^4*x*z^2 + 2*b^2*c^4*x*z^2 +      c^6*x*z^2 - 4*a^6*y*z^2 - 3*a^4*b^2*y*z^2 + 6*a^2*b^4*y*z^2 + b^6*y*z^2 + 7*a^4*c^2*y*z^2 -      4*a^2*b^2*c^2*y*z^2 - 3*b^4*c^2*y*z^2 - 2*a^2*c^4*y*z^2 + 3*b^2*c^4*y*z^2 - c^6*y*z^2");
		exp("K314","(-a^2)*c^2*x^2*y + 3*b^2*c^2*x^2*y + c^4*x^2*y - 3*a^2*c^2*x*y^2 + b^2*c^2*x*y^2 - c^4*x*y^2 +      a^2*b^2*x^2*z - b^4*x^2*z - 3*b^2*c^2*x^2*z + a^4*y^2*z - a^2*b^2*y^2*z + 3*a^2*c^2*y^2*z +      3*a^2*b^2*x*z^2 + b^4*x*z^2 - b^2*c^2*x*z^2 - a^4*y*z^2 - 3*a^2*b^2*y*z^2 + a^2*c^2*y*z^2");
		exp("K315","a^2*x^2*y - b^2*x^2*y + 3*c^2*x^2*y + a^2*x*y^2 - b^2*x*y^2 - 3*c^2*x*y^2 - a^2*x^2*z -      3*b^2*x^2*z + c^2*x^2*z + 3*a^2*y^2*z + b^2*y^2*z - c^2*y^2*z - a^2*x*z^2 + 3*b^2*x*z^2 +      c^2*x*z^2 - 3*a^2*y*z^2 + b^2*y*z^2 - c^2*y*z^2");
		exp("K316","(-a^2)*b^4*c^2*x^2*y + b^6*c^2*x^2*y + a^2*b^2*c^4*x^2*y - b^4*c^4*x^2*y - a^6*c^2*x*y^2 +      a^4*b^2*c^2*x*y^2 + a^4*c^4*x*y^2 - a^2*b^2*c^4*x*y^2 - a^2*b^4*c^2*x^2*z + a^2*b^2*c^4*x^2*z +      b^4*c^4*x^2*z - b^2*c^6*x^2*z + a^4*b^2*c^2*y^2*z - a^4*c^4*y^2*z - a^2*b^2*c^4*y^2*z +      a^2*c^6*y^2*z + a^6*b^2*x*z^2 - a^4*b^4*x*z^2 - a^4*b^2*c^2*x*z^2 + a^2*b^4*c^2*x*z^2 +      a^4*b^4*y*z^2 - a^2*b^6*y*z^2 - a^4*b^2*c^2*y*z^2 + a^2*b^4*c^2*y*z^2");
		exp("K317","b*c*x^2*y + c^2*x^2*y - a*c*x*y^2 - c^2*x*y^2 - b^2*x^2*z - b*c*x^2*z + a^2*y^2*z + a*c*y^2*z +      a*b*x*z^2 + b^2*x*z^2 - a^2*y*z^2 - a*b*y*z^2");
		exp("K318","(-a)*b^2*c^3*x^2*y + b^3*c^3*x^2*y - a*b*c^4*x^2*y - b*c^5*x^2*y - a^3*c^3*x*y^2 +      a^2*b*c^3*x*y^2 + a*b*c^4*x*y^2 + a*c^5*x*y^2 + a*b^4*c*x^2*z + b^5*c*x^2*z + a*b^3*c^2*x^2*z -      b^3*c^3*x^2*z - a^5*c*y^2*z - a^4*b*c*y^2*z - a^3*b*c^2*y^2*z + a^3*c^3*y^2*z + a^3*b^3*x*z^2 -      a*b^5*x*z^2 - a^2*b^3*c*x*z^2 - a*b^4*c*x*z^2 + a^5*b*y*z^2 - a^3*b^3*y*z^2 + a^4*b*c*y*z^2 +      a^3*b^2*c*y*z^2");
		exp("K319","b^2*c^3*x^2*y + b*c^4*x^2*y - a^2*c^3*x*y^2 - a*c^4*x*y^2 - b^4*c*x^2*z - b^3*c^2*x^2*z +      a^4*c*y^2*z + a^3*c^2*y^2*z + a^2*b^3*x*z^2 + a*b^4*x*z^2 - a^4*b*y*z^2 - a^3*b^2*y*z^2");
		exp("K320","(-a)*b^2*c^2*x^2*y + b^3*c^2*x^2*y - 2*a*b*c^3*x^2*y + b^2*c^3*x^2*y - a*c^4*x^2*y -      b*c^4*x^2*y - c^5*x^2*y - a^3*c^2*x*y^2 + a^2*b*c^2*x*y^2 - a^2*c^3*x*y^2 + 2*a*b*c^3*x*y^2 +      a*c^4*x*y^2 + b*c^4*x*y^2 + c^5*x*y^2 + a*b^4*x^2*z + b^5*x^2*z + 2*a*b^3*c*x^2*z + b^4*c*x^2*z +      a*b^2*c^2*x^2*z - b^3*c^2*x^2*z - b^2*c^3*x^2*z - a^5*y^2*z - a^4*b*y^2*z - a^4*c*y^2*z -      2*a^3*b*c*y^2*z + a^3*c^2*y^2*z - a^2*b*c^2*y^2*z + a^2*c^3*y^2*z + a^3*b^2*x*z^2 + a^2*b^3*x*z^2 -      a*b^4*x*z^2 - b^5*x*z^2 - a^2*b^2*c*x*z^2 - 2*a*b^3*c*x*z^2 - b^4*c*x*z^2 + a^5*y*z^2 +      a^4*b*y*z^2 - a^3*b^2*y*z^2 - a^2*b^3*y*z^2 + a^4*c*y*z^2 + 2*a^3*b*c*y*z^2 + a^2*b^2*c*y*z^2");
		exp("K321","2*a*b*c^3*x^2*y + a*c^4*x^2*y + b*c^4*x^2*y + c^5*x^2*y - 2*a*b*c^3*x*y^2 - a*c^4*x*y^2 -      b*c^4*x*y^2 - c^5*x*y^2 - a*b^4*x^2*z - b^5*x^2*z - 2*a*b^3*c*x^2*z - b^4*c*x^2*z + a^5*y^2*z +      a^4*b*y^2*z + a^4*c*y^2*z + 2*a^3*b*c*y^2*z + a*b^4*x*z^2 + b^5*x*z^2 + 2*a*b^3*c*x*z^2 +      b^4*c*x*z^2 - a^5*y*z^2 - a^4*b*y*z^2 - a^4*c*y*z^2 - 2*a^3*b*c*y*z^2");
		exp("K322","(-a^4)*b^2*x^2*y + b^4*c^2*x^2*y + a^2*b^4*x*y^2 - a^4*c^2*x*y^2 + a^4*c^2*x^2*z -      b^2*c^4*x^2*z - b^4*c^2*y^2*z + a^2*c^4*y^2*z + a^4*b^2*x*z^2 - a^2*c^4*x*z^2 - a^2*b^4*y*z^2 +      b^2*c^4*y*z^2");
		exp("K323","b^2*c*x^2*y - a*c^2*x^2*y - a^2*c*x*y^2 + b*c^2*x*y^2 + a*b^2*x^2*z - b*c^2*x^2*z - a^2*b*y^2*z +      a*c^2*y^2*z + a^2*b*x*z^2 - b^2*c*x*z^2 - a*b^2*y*z^2 + a^2*c*y*z^2");
		exp("K324","b^2*c*x^2*y - a*c^2*x^2*y + a^2*c*x*y^2 - b*c^2*x*y^2 - a*b^2*x^2*z + b*c^2*x^2*z - a^2*b*y^2*z +      a*c^2*y^2*z + a^2*b*x*z^2 - b^2*c*x*z^2 + a*b^2*y*z^2 - a^2*c*y*z^2");
		exp("K325","(-a^2)*b^2*c^2*x^2*y + b^2*c^4*x^2*y + a^2*b^2*c^2*x*y^2 - a^2*c^4*x*y^2 + a^2*b^2*c^2*x^2*z -      b^4*c^2*x^2*z + a^4*c^2*y^2*z - a^2*b^2*c^2*y^2*z + a^2*b^4*x*z^2 - a^2*b^2*c^2*x*z^2 -      a^4*b^2*y*z^2 + a^2*b^2*c^2*y*z^2");
		exp("K326","a^2*b^2*c^2*x^2*y + b^2*c^4*x^2*y - a^2*b^2*c^2*x*y^2 - a^2*c^4*x*y^2 - a^2*b^2*c^2*x^2*z -      b^4*c^2*x^2*z + a^4*c^2*y^2*z + a^2*b^2*c^2*y^2*z + a^2*b^4*x*z^2 + a^2*b^2*c^2*x*z^2 -      a^4*b^2*y*z^2 - a^2*b^2*c^2*y*z^2");
		exp("K327","x*(y^2 + z^2) + y*(z^2 + x^2) + z*(x^2 + y^2) + x*y*z");
		exp("K328","a*c*x^2*y + c^2*x^2*y - b*c*x*y^2 - c^2*x*y^2 - a*b*x^2*z - b^2*x^2*z + a^2*y^2*z + a*b*y^2*z +      b^2*x*z^2 + b*c*x*z^2 - a^2*y*z^2 - a*c*y*z^2");
		exp("K329","a^6*x^2*y - 3*a^4*b^2*x^2*y + 3*a^2*b^4*x^2*y - b^6*x^2*y + a^4*c^2*x^2*y + 2*a^2*b^2*c^2*x^2*y -      3*b^4*c^2*x^2*y - 5*a^2*c^4*x^2*y + b^2*c^4*x^2*y + 3*c^6*x^2*y + a^6*x*y^2 - 3*a^4*b^2*x*y^2 +      3*a^2*b^4*x*y^2 - b^6*x*y^2 + 3*a^4*c^2*x*y^2 - 2*a^2*b^2*c^2*x*y^2 - b^4*c^2*x*y^2 -      a^2*c^4*x*y^2 + 5*b^2*c^4*x*y^2 - 3*c^6*x*y^2 - a^6*x^2*z - a^4*b^2*x^2*z + 5*a^2*b^4*x^2*z -      3*b^6*x^2*z + 3*a^4*c^2*x^2*z - 2*a^2*b^2*c^2*x^2*z - b^4*c^2*x^2*z - 3*a^2*c^4*x^2*z +      3*b^2*c^4*x^2*z + c^6*x^2*z + 3*a^6*y^2*z - 5*a^4*b^2*y^2*z + a^2*b^4*y^2*z + b^6*y^2*z +      a^4*c^2*y^2*z + 2*a^2*b^2*c^2*y^2*z - 3*b^4*c^2*y^2*z - 3*a^2*c^4*y^2*z + 3*b^2*c^4*y^2*z -      c^6*y^2*z - a^6*x*z^2 - 3*a^4*b^2*x*z^2 + a^2*b^4*x*z^2 + 3*b^6*x*z^2 + 3*a^4*c^2*x*z^2 +      2*a^2*b^2*c^2*x*z^2 - 5*b^4*c^2*x*z^2 - 3*a^2*c^4*x*z^2 + b^2*c^4*x*z^2 + c^6*x*z^2 - 3*a^6*y*z^2 -      a^4*b^2*y*z^2 + 3*a^2*b^4*y*z^2 + b^6*y*z^2 + 5*a^4*c^2*y*z^2 - 2*a^2*b^2*c^2*y*z^2 -      3*b^4*c^2*y*z^2 - a^2*c^4*y*z^2 + 3*b^2*c^4*y*z^2 - c^6*y*z^2");
		exp("K330","2*a^6*c^2*x^2*y + 4*a^4*b^2*c^2*x^2*y - 2*a^2*b^4*c^2*x^2*y - 4*b^6*c^2*x^2*y - 5*a^4*c^4*x^2*y +      4*b^4*c^4*x^2*y + a^2*c^6*x^2*y - 2*b^2*c^6*x^2*y + 2*c^8*x^2*y + 4*a^6*c^2*x*y^2 +      2*a^4*b^2*c^2*x*y^2 - 4*a^2*b^4*c^2*x*y^2 - 2*b^6*c^2*x*y^2 - 4*a^4*c^4*x*y^2 + 5*b^4*c^4*x*y^2 +      2*a^2*c^6*x*y^2 - b^2*c^6*x*y^2 - 2*c^8*x*y^2 - 2*a^6*b^2*x^2*z + 5*a^4*b^4*x^2*z - a^2*b^6*x^2*z -      2*b^8*x^2*z - 4*a^4*b^2*c^2*x^2*z + 2*b^6*c^2*x^2*z + 2*a^2*b^2*c^4*x^2*z - 4*b^4*c^4*x^2*z +      4*b^2*c^6*x^2*z + 3*a^6*b^2*x*y*z - 3*a^2*b^6*x*y*z - 3*a^6*c^2*x*y*z + 3*b^6*c^2*x*y*z +      3*a^2*c^6*x*y*z - 3*b^2*c^6*x*y*z + 2*a^8*y^2*z + a^6*b^2*y^2*z - 5*a^4*b^4*y^2*z +      2*a^2*b^6*y^2*z - 2*a^6*c^2*y^2*z + 4*a^2*b^4*c^2*y^2*z + 4*a^4*c^4*y^2*z - 2*a^2*b^2*c^4*y^2*z -      4*a^2*c^6*y^2*z - 4*a^6*b^2*x*z^2 + 4*a^4*b^4*x*z^2 - 2*a^2*b^6*x*z^2 + 2*b^8*x*z^2 -      2*a^4*b^2*c^2*x*z^2 + b^6*c^2*x*z^2 + 4*a^2*b^2*c^4*x*z^2 - 5*b^4*c^4*x*z^2 + 2*b^2*c^6*x*z^2 -      2*a^8*y*z^2 + 2*a^6*b^2*y*z^2 - 4*a^4*b^4*y*z^2 + 4*a^2*b^6*y*z^2 - a^6*c^2*y*z^2 +      2*a^2*b^4*c^2*y*z^2 + 5*a^4*c^4*y*z^2 - 4*a^2*b^2*c^4*y*z^2 - 2*a^2*c^6*y*z^2");
		exp("K331","2*a^4*b^2*c^2*x^2*y + 4*a^2*b^4*c^2*x^2*y - a^4*c^4*x^2*y - 6*a^2*b^2*c^4*x^2*y -      2*b^4*c^4*x^2*y + 3*a^2*c^6*x^2*y + 4*a^4*b^2*c^2*x*y^2 + 2*a^2*b^4*c^2*x*y^2 - 2*a^4*c^4*x*y^2 -      6*a^2*b^2*c^4*x*y^2 - b^4*c^4*x*y^2 + 3*b^2*c^6*x*y^2 - a^4*b^4*x^2*z + 3*a^2*b^6*x^2*z +      2*a^4*b^2*c^2*x^2*z - 6*a^2*b^4*c^2*x^2*z + 4*a^2*b^2*c^4*x^2*z - 2*b^4*c^4*x^2*z + a^6*b^2*x*y*z +      2*a^4*b^4*x*y*z + a^2*b^6*x*y*z + a^6*c^2*x*y*z - 4*a^4*b^2*c^2*x*y*z - 4*a^2*b^4*c^2*x*y*z +      b^6*c^2*x*y*z + 2*a^4*c^4*x*y*z - 4*a^2*b^2*c^4*x*y*z + 2*b^4*c^4*x*y*z + a^2*c^6*x*y*z +      b^2*c^6*x*y*z + 3*a^6*b^2*y^2*z - a^4*b^4*y^2*z - 6*a^4*b^2*c^2*y^2*z + 2*a^2*b^4*c^2*y^2*z -      2*a^4*c^4*y^2*z + 4*a^2*b^2*c^4*y^2*z - 2*a^4*b^4*x*z^2 + 4*a^4*b^2*c^2*x*z^2 -      6*a^2*b^4*c^2*x*z^2 + 3*b^6*c^2*x*z^2 + 2*a^2*b^2*c^4*x*z^2 - b^4*c^4*x*z^2 - 2*a^4*b^4*y*z^2 +      3*a^6*c^2*y*z^2 - 6*a^4*b^2*c^2*y*z^2 + 4*a^2*b^4*c^2*y*z^2 - a^4*c^4*y*z^2 +      2*a^2*b^2*c^4*y*z^2");
		exp("K332","(-a^3)*c*x^2*y - a^2*b*c*x^2*y + a*b^2*c*x^2*y + b^3*c*x^2*y + a^2*c^2*x^2*y - 2*a*b*c^2*x^2*y +      b^2*c^2*x^2*y + a*c^3*x^2*y - b*c^3*x^2*y - c^4*x^2*y - a^3*c*x*y^2 - a^2*b*c*x*y^2 +      a*b^2*c*x*y^2 + b^3*c*x*y^2 - a^2*c^2*x*y^2 + 2*a*b*c^2*x*y^2 - b^2*c^2*x*y^2 + a*c^3*x*y^2 -      b*c^3*x*y^2 + c^4*x*y^2 + a^3*b*x^2*z - a^2*b^2*x^2*z - a*b^3*x^2*z + b^4*x^2*z + a^2*b*c*x^2*z +      2*a*b^2*c*x^2*z + b^3*c*x^2*z - a*b*c^2*x^2*z - b^2*c^2*x^2*z - b*c^3*x^2*z - a^4*y^2*z +      a^3*b*y^2*z + a^2*b^2*y^2*z - a*b^3*y^2*z - a^3*c*y^2*z - 2*a^2*b*c*y^2*z - a*b^2*c*y^2*z +      a^2*c^2*y^2*z + a*b*c^2*y^2*z + a*c^3*y^2*z + a^3*b*x*z^2 + a^2*b^2*x*z^2 - a*b^3*x*z^2 -      b^4*x*z^2 + a^2*b*c*x*z^2 - 2*a*b^2*c*x*z^2 + b^3*c*x*z^2 - a*b*c^2*x*z^2 + b^2*c^2*x*z^2 -      b*c^3*x*z^2 + a^4*y*z^2 + a^3*b*y*z^2 - a^2*b^2*y*z^2 - a*b^3*y*z^2 - a^3*c*y*z^2 +      2*a^2*b*c*y*z^2 - a*b^2*c*y*z^2 - a^2*c^2*y*z^2 + a*b*c^2*y*z^2 + a*c^3*y*z^2");
		exp("K333","a^4*c*x^2*y - 2*a^3*b*c*x^2*y + 2*a*b^3*c*x^2*y - b^4*c*x^2*y + 2*a^2*b*c^2*x^2*y -      2*b^3*c^2*x^2*y - 2*a^2*c^3*x^2*y - 2*a*b*c^3*x^2*y + 2*b*c^4*x^2*y + c^5*x^2*y + a^4*c*x*y^2 -      2*a^3*b*c*x*y^2 + 2*a*b^3*c*x*y^2 - b^4*c*x*y^2 + 2*a^3*c^2*x*y^2 - 2*a*b^2*c^2*x*y^2 +      2*a*b*c^3*x*y^2 + 2*b^2*c^3*x*y^2 - 2*a*c^4*x*y^2 - c^5*x*y^2 - a^4*b*x^2*z + 2*a^2*b^3*x^2*z -      b^5*x^2*z + 2*a^3*b*c*x^2*z - 2*a^2*b^2*c*x^2*z + 2*a*b^3*c*x^2*z - 2*b^4*c*x^2*z -      2*a*b*c^3*x^2*z + 2*b^2*c^3*x^2*z + b*c^4*x^2*z + a^5*y^2*z - 2*a^3*b^2*y^2*z + a*b^4*y^2*z +      2*a^4*c*y^2*z - 2*a^3*b*c*y^2*z + 2*a^2*b^2*c*y^2*z - 2*a*b^3*c*y^2*z - 2*a^2*c^3*y^2*z +      2*a*b*c^3*y^2*z - a*c^4*y^2*z - a^4*b*x*z^2 - 2*a^3*b^2*x*z^2 + 2*a*b^4*x*z^2 + b^5*x*z^2 +      2*a^3*b*c*x*z^2 - 2*a*b^3*c*x*z^2 + 2*a*b^2*c^2*x*z^2 - 2*b^3*c^2*x*z^2 - 2*a*b*c^3*x*z^2 +      b*c^4*x*z^2 - a^5*y*z^2 - 2*a^4*b*y*z^2 + 2*a^2*b^3*y*z^2 + a*b^4*y*z^2 + 2*a^3*b*c*y*z^2 -      2*a*b^3*c*y*z^2 + 2*a^3*c^2*y*z^2 - 2*a^2*b*c^2*y*z^2 + 2*a*b*c^3*y*z^2 - a*c^4*y*z^2");
		exp("K334","a^5*c*x^2*y - a^4*b*c*x^2*y - 2*a^3*b^2*c*x^2*y + 2*a^2*b^3*c*x^2*y + a*b^4*c*x^2*y -      b^5*c*x^2*y + 2*a^3*b*c^2*x^2*y - 2*a*b^3*c^2*x^2*y - 2*a^3*c^3*x^2*y + 2*a*b^2*c^3*x^2*y -      2*a*b*c^4*x^2*y + a*c^5*x^2*y + b*c^5*x^2*y + a^5*c*x*y^2 - a^4*b*c*x*y^2 - 2*a^3*b^2*c*x*y^2 +      2*a^2*b^3*c*x*y^2 + a*b^4*c*x*y^2 - b^5*c*x*y^2 + 2*a^3*b*c^2*x*y^2 - 2*a*b^3*c^2*x*y^2 -      2*a^2*b*c^3*x*y^2 + 2*b^3*c^3*x*y^2 + 2*a*b*c^4*x*y^2 - a*c^5*x*y^2 - b*c^5*x*y^2 - a^5*b*x^2*z +      2*a^3*b^3*x^2*z - a*b^5*x^2*z + a^4*b*c*x^2*z - 2*a^3*b^2*c*x^2*z + 2*a*b^4*c*x^2*z - b^5*c*x^2*z +      2*a^3*b*c^2*x^2*z - 2*a*b^3*c^2*x^2*z - 2*a^2*b*c^3*x^2*z + 2*a*b^2*c^3*x^2*z - a*b*c^4*x^2*z +      b*c^5*x^2*z + a^5*b*y^2*z - 2*a^3*b^3*y^2*z + a*b^5*y^2*z + a^5*c*y^2*z - 2*a^4*b*c*y^2*z +      2*a^2*b^3*c*y^2*z - a*b^4*c*y^2*z + 2*a^3*b*c^2*y^2*z - 2*a*b^3*c^2*y^2*z - 2*a^2*b*c^3*y^2*z +      2*a*b^2*c^3*y^2*z + a*b*c^4*y^2*z - a*c^5*y^2*z - a^5*b*x*z^2 + a*b^5*x*z^2 + a^4*b*c*x*z^2 -      2*a^3*b^2*c*x*z^2 + 2*a^2*b^3*c*x*z^2 - 2*a*b^4*c*x*z^2 + b^5*c*x*z^2 + 2*a^3*b*c^2*x*z^2 -      2*a^2*b*c^3*x*z^2 + 2*a*b^2*c^3*x*z^2 - 2*b^3*c^3*x*z^2 - a*b*c^4*x*z^2 + b*c^5*x*z^2 -      a^5*b*y*z^2 + a*b^5*y*z^2 - a^5*c*y*z^2 + 2*a^4*b*c*y*z^2 - 2*a^3*b^2*c*y*z^2 + 2*a^2*b^3*c*y*z^2 -      a*b^4*c*y*z^2 - 2*a*b^3*c^2*y*z^2 + 2*a^3*c^3*y*z^2 - 2*a^2*b*c^3*y*z^2 + 2*a*b^2*c^3*y*z^2 +      a*b*c^4*y*z^2 - a*c^5*y*z^2");
		exp("K335","-2*a^18*b^2*c^4*x^2*y + 3*a^16*b^4*c^4*x^2*y + 7*a^14*b^6*c^4*x^2*y - 10*a^12*b^8*c^4*x^2*y -      9*a^10*b^10*c^4*x^2*y + 12*a^8*b^12*c^4*x^2*y + 5*a^6*b^14*c^4*x^2*y - 6*a^4*b^16*c^4*x^2*y -      a^2*b^18*c^4*x^2*y + b^20*c^4*x^2*y + 3*a^16*b^2*c^6*x^2*y - 16*a^14*b^4*c^6*x^2*y +      5*a^12*b^6*c^6*x^2*y + 32*a^10*b^8*c^6*x^2*y - 19*a^8*b^10*c^6*x^2*y - 16*a^6*b^12*c^6*x^2*y +      11*a^4*b^14*c^6*x^2*y + 3*a^14*b^2*c^8*x^2*y + 15*a^12*b^4*c^8*x^2*y - 46*a^10*b^6*c^8*x^2*y +      9*a^8*b^8*c^8*x^2*y + 31*a^6*b^10*c^8*x^2*y - 19*a^4*b^12*c^8*x^2*y + 12*a^2*b^14*c^8*x^2*y -      5*b^16*c^8*x^2*y - 8*a^12*b^2*c^10*x^2*y + 20*a^10*b^4*c^10*x^2*y + 29*a^8*b^6*c^10*x^2*y -      56*a^6*b^8*c^10*x^2*y + 26*a^4*b^10*c^10*x^2*y - 12*a^2*b^12*c^10*x^2*y + b^14*c^10*x^2*y +      3*a^10*b^2*c^12*x^2*y - 37*a^8*b^4*c^12*x^2*y + 35*a^6*b^6*c^12*x^2*y + 8*a^4*b^8*c^12*x^2*y -      18*a^2*b^10*c^12*x^2*y + 9*b^12*c^12*x^2*y + 6*a^8*b^2*c^14*x^2*y + 8*a^6*b^4*c^14*x^2*y -      37*a^4*b^6*c^14*x^2*y + 24*a^2*b^8*c^14*x^2*y - 3*b^10*c^14*x^2*y - 7*a^6*b^2*c^16*x^2*y +      17*a^4*b^4*c^16*x^2*y + 4*a^2*b^6*c^16*x^2*y - 7*b^8*c^16*x^2*y - 12*a^2*b^4*c^18*x^2*y +      3*b^6*c^18*x^2*y + 3*a^2*b^2*c^20*x^2*y + 2*b^4*c^20*x^2*y - b^2*c^22*x^2*y - a^20*c^4*x*y^2 +      a^18*b^2*c^4*x*y^2 + 6*a^16*b^4*c^4*x*y^2 - 5*a^14*b^6*c^4*x*y^2 - 12*a^12*b^8*c^4*x*y^2 +      9*a^10*b^10*c^4*x*y^2 + 10*a^8*b^12*c^4*x*y^2 - 7*a^6*b^14*c^4*x*y^2 - 3*a^4*b^16*c^4*x*y^2 +      2*a^2*b^18*c^4*x*y^2 - 11*a^14*b^4*c^6*x*y^2 + 16*a^12*b^6*c^6*x*y^2 + 19*a^10*b^8*c^6*x*y^2 -      32*a^8*b^10*c^6*x*y^2 - 5*a^6*b^12*c^6*x*y^2 + 16*a^4*b^14*c^6*x*y^2 - 3*a^2*b^16*c^6*x*y^2 +      5*a^16*c^8*x*y^2 - 12*a^14*b^2*c^8*x*y^2 + 19*a^12*b^4*c^8*x*y^2 - 31*a^10*b^6*c^8*x*y^2 -      9*a^8*b^8*c^8*x*y^2 + 46*a^6*b^10*c^8*x*y^2 - 15*a^4*b^12*c^8*x*y^2 - 3*a^2*b^14*c^8*x*y^2 -      a^14*c^10*x*y^2 + 12*a^12*b^2*c^10*x*y^2 - 26*a^10*b^4*c^10*x*y^2 + 56*a^8*b^6*c^10*x*y^2 -      29*a^6*b^8*c^10*x*y^2 - 20*a^4*b^10*c^10*x*y^2 + 8*a^2*b^12*c^10*x*y^2 - 9*a^12*c^12*x*y^2 +      18*a^10*b^2*c^12*x*y^2 - 8*a^8*b^4*c^12*x*y^2 - 35*a^6*b^6*c^12*x*y^2 + 37*a^4*b^8*c^12*x*y^2 -      3*a^2*b^10*c^12*x*y^2 + 3*a^10*c^14*x*y^2 - 24*a^8*b^2*c^14*x*y^2 + 37*a^6*b^4*c^14*x*y^2 -      8*a^4*b^6*c^14*x*y^2 - 6*a^2*b^8*c^14*x*y^2 + 7*a^8*c^16*x*y^2 - 4*a^6*b^2*c^16*x*y^2 -      17*a^4*b^4*c^16*x*y^2 + 7*a^2*b^6*c^16*x*y^2 - 3*a^6*c^18*x*y^2 + 12*a^4*b^2*c^18*x*y^2 -      2*a^4*c^20*x*y^2 - 3*a^2*b^2*c^20*x*y^2 + a^2*c^22*x*y^2 + 2*a^18*b^4*c^2*x^2*z -      3*a^16*b^6*c^2*x^2*z - 3*a^14*b^8*c^2*x^2*z + 8*a^12*b^10*c^2*x^2*z - 3*a^10*b^12*c^2*x^2*z -      6*a^8*b^14*c^2*x^2*z + 7*a^6*b^16*c^2*x^2*z - 3*a^2*b^20*c^2*x^2*z + b^22*c^2*x^2*z -      3*a^16*b^4*c^4*x^2*z + 16*a^14*b^6*c^4*x^2*z - 15*a^12*b^8*c^4*x^2*z - 20*a^10*b^10*c^4*x^2*z +      37*a^8*b^12*c^4*x^2*z - 8*a^6*b^14*c^4*x^2*z - 17*a^4*b^16*c^4*x^2*z + 12*a^2*b^18*c^4*x^2*z -      2*b^20*c^4*x^2*z - 7*a^14*b^4*c^6*x^2*z - 5*a^12*b^6*c^6*x^2*z + 46*a^10*b^8*c^6*x^2*z -      29*a^8*b^10*c^6*x^2*z - 35*a^6*b^12*c^6*x^2*z + 37*a^4*b^14*c^6*x^2*z - 4*a^2*b^16*c^6*x^2*z -      3*b^18*c^6*x^2*z + 10*a^12*b^4*c^8*x^2*z - 32*a^10*b^6*c^8*x^2*z - 9*a^8*b^8*c^8*x^2*z +      56*a^6*b^10*c^8*x^2*z - 8*a^4*b^12*c^8*x^2*z - 24*a^2*b^14*c^8*x^2*z + 7*b^16*c^8*x^2*z +      9*a^10*b^4*c^10*x^2*z + 19*a^8*b^6*c^10*x^2*z - 31*a^6*b^8*c^10*x^2*z - 26*a^4*b^10*c^10*x^2*z +      18*a^2*b^12*c^10*x^2*z + 3*b^14*c^10*x^2*z - 12*a^8*b^4*c^12*x^2*z + 16*a^6*b^6*c^12*x^2*z +      19*a^4*b^8*c^12*x^2*z + 12*a^2*b^10*c^12*x^2*z - 9*b^12*c^12*x^2*z - 5*a^6*b^4*c^14*x^2*z -      11*a^4*b^6*c^14*x^2*z - 12*a^2*b^8*c^14*x^2*z - b^10*c^14*x^2*z + 6*a^4*b^4*c^16*x^2*z +      5*b^8*c^16*x^2*z + a^2*b^4*c^18*x^2*z - b^4*c^20*x^2*z - a^22*c^2*y^2*z + 3*a^20*b^2*c^2*y^2*z -      7*a^16*b^6*c^2*y^2*z + 6*a^14*b^8*c^2*y^2*z + 3*a^12*b^10*c^2*y^2*z - 8*a^10*b^12*c^2*y^2*z +      3*a^8*b^14*c^2*y^2*z + 3*a^6*b^16*c^2*y^2*z - 2*a^4*b^18*c^2*y^2*z + 2*a^20*c^4*y^2*z -      12*a^18*b^2*c^4*y^2*z + 17*a^16*b^4*c^4*y^2*z + 8*a^14*b^6*c^4*y^2*z - 37*a^12*b^8*c^4*y^2*z +      20*a^10*b^10*c^4*y^2*z + 15*a^8*b^12*c^4*y^2*z - 16*a^6*b^14*c^4*y^2*z + 3*a^4*b^16*c^4*y^2*z +      3*a^18*c^6*y^2*z + 4*a^16*b^2*c^6*y^2*z - 37*a^14*b^4*c^6*y^2*z + 35*a^12*b^6*c^6*y^2*z +      29*a^10*b^8*c^6*y^2*z - 46*a^8*b^10*c^6*y^2*z + 5*a^6*b^12*c^6*y^2*z + 7*a^4*b^14*c^6*y^2*z -      7*a^16*c^8*y^2*z + 24*a^14*b^2*c^8*y^2*z + 8*a^12*b^4*c^8*y^2*z - 56*a^10*b^6*c^8*y^2*z +      9*a^8*b^8*c^8*y^2*z + 32*a^6*b^10*c^8*y^2*z - 10*a^4*b^12*c^8*y^2*z - 3*a^14*c^10*y^2*z -      18*a^12*b^2*c^10*y^2*z + 26*a^10*b^4*c^10*y^2*z + 31*a^8*b^6*c^10*y^2*z - 19*a^6*b^8*c^10*y^2*z -      9*a^4*b^10*c^10*y^2*z + 9*a^12*c^12*y^2*z - 12*a^10*b^2*c^12*y^2*z - 19*a^8*b^4*c^12*y^2*z -      16*a^6*b^6*c^12*y^2*z + 12*a^4*b^8*c^12*y^2*z + a^10*c^14*y^2*z + 12*a^8*b^2*c^14*y^2*z +      11*a^6*b^4*c^14*y^2*z + 5*a^4*b^6*c^14*y^2*z - 5*a^8*c^16*y^2*z - 6*a^4*b^4*c^16*y^2*z -      a^4*b^2*c^18*y^2*z + a^4*c^20*y^2*z + a^20*b^4*x*z^2 - 5*a^16*b^8*x*z^2 + a^14*b^10*x*z^2 +      9*a^12*b^12*x*z^2 - 3*a^10*b^14*x*z^2 - 7*a^8*b^16*x*z^2 + 3*a^6*b^18*x*z^2 + 2*a^4*b^20*x*z^2 -      a^2*b^22*x*z^2 - a^18*b^4*c^2*x*z^2 + 12*a^14*b^8*c^2*x*z^2 - 12*a^12*b^10*c^2*x*z^2 -      18*a^10*b^12*c^2*x*z^2 + 24*a^8*b^14*c^2*x*z^2 + 4*a^6*b^16*c^2*x*z^2 - 12*a^4*b^18*c^2*x*z^2 +      3*a^2*b^20*c^2*x*z^2 - 6*a^16*b^4*c^4*x*z^2 + 11*a^14*b^6*c^4*x*z^2 - 19*a^12*b^8*c^4*x*z^2 +      26*a^10*b^10*c^4*x*z^2 + 8*a^8*b^12*c^4*x*z^2 - 37*a^6*b^14*c^4*x*z^2 + 17*a^4*b^16*c^4*x*z^2 +      5*a^14*b^4*c^6*x*z^2 - 16*a^12*b^6*c^6*x*z^2 + 31*a^10*b^8*c^6*x*z^2 - 56*a^8*b^10*c^6*x*z^2 +      35*a^6*b^12*c^6*x*z^2 + 8*a^4*b^14*c^6*x*z^2 - 7*a^2*b^16*c^6*x*z^2 + 12*a^12*b^4*c^8*x*z^2 -      19*a^10*b^6*c^8*x*z^2 + 9*a^8*b^8*c^8*x*z^2 + 29*a^6*b^10*c^8*x*z^2 - 37*a^4*b^12*c^8*x*z^2 +      6*a^2*b^14*c^8*x*z^2 - 9*a^10*b^4*c^10*x*z^2 + 32*a^8*b^6*c^10*x*z^2 - 46*a^6*b^8*c^10*x*z^2 +      20*a^4*b^10*c^10*x*z^2 + 3*a^2*b^12*c^10*x*z^2 - 10*a^8*b^4*c^12*x*z^2 + 5*a^6*b^6*c^12*x*z^2 +      15*a^4*b^8*c^12*x*z^2 - 8*a^2*b^10*c^12*x*z^2 + 7*a^6*b^4*c^14*x*z^2 - 16*a^4*b^6*c^14*x*z^2 +      3*a^2*b^8*c^14*x*z^2 + 3*a^4*b^4*c^16*x*z^2 + 3*a^2*b^6*c^16*x*z^2 - 2*a^2*b^4*c^18*x*z^2 +      a^22*b^2*y*z^2 - 2*a^20*b^4*y*z^2 - 3*a^18*b^6*y*z^2 + 7*a^16*b^8*y*z^2 + 3*a^14*b^10*y*z^2 -      9*a^12*b^12*y*z^2 - a^10*b^14*y*z^2 + 5*a^8*b^16*y*z^2 - a^4*b^20*y*z^2 - 3*a^20*b^2*c^2*y*z^2 +      12*a^18*b^4*c^2*y*z^2 - 4*a^16*b^6*c^2*y*z^2 - 24*a^14*b^8*c^2*y*z^2 + 18*a^12*b^10*c^2*y*z^2 +      12*a^10*b^12*c^2*y*z^2 - 12*a^8*b^14*c^2*y*z^2 + a^4*b^18*c^2*y*z^2 - 17*a^16*b^4*c^4*y*z^2 +      37*a^14*b^6*c^4*y*z^2 - 8*a^12*b^8*c^4*y*z^2 - 26*a^10*b^10*c^4*y*z^2 + 19*a^8*b^12*c^4*y*z^2 -      11*a^6*b^14*c^4*y*z^2 + 6*a^4*b^16*c^4*y*z^2 + 7*a^16*b^2*c^6*y*z^2 - 8*a^14*b^4*c^6*y*z^2 -      35*a^12*b^6*c^6*y*z^2 + 56*a^10*b^8*c^6*y*z^2 - 31*a^8*b^10*c^6*y*z^2 + 16*a^6*b^12*c^6*y*z^2 -      5*a^4*b^14*c^6*y*z^2 - 6*a^14*b^2*c^8*y*z^2 + 37*a^12*b^4*c^8*y*z^2 - 29*a^10*b^6*c^8*y*z^2 -      9*a^8*b^8*c^8*y*z^2 + 19*a^6*b^10*c^8*y*z^2 - 12*a^4*b^12*c^8*y*z^2 - 3*a^12*b^2*c^10*y*z^2 -      20*a^10*b^4*c^10*y*z^2 + 46*a^8*b^6*c^10*y*z^2 - 32*a^6*b^8*c^10*y*z^2 + 9*a^4*b^10*c^10*y*z^2 +      8*a^10*b^2*c^12*y*z^2 - 15*a^8*b^4*c^12*y*z^2 - 5*a^6*b^6*c^12*y*z^2 + 10*a^4*b^8*c^12*y*z^2 -      3*a^8*b^2*c^14*y*z^2 + 16*a^6*b^4*c^14*y*z^2 - 7*a^4*b^6*c^14*y*z^2 - 3*a^6*b^2*c^16*y*z^2 -      3*a^4*b^4*c^16*y*z^2 + 2*a^4*b^2*c^18*y*z^2");
		exp("K336","a^4*b^2*c^2*x^2*y - b^6*c^2*x^2*y + a^4*c^4*x^2*y + b^4*c^4*x^2*y - 2*a^2*c^6*x^2*y -      b^2*c^6*x^2*y + c^8*x^2*y + a^6*c^2*x*y^2 - a^2*b^4*c^2*x*y^2 - a^4*c^4*x*y^2 - b^4*c^4*x*y^2 +      a^2*c^6*x*y^2 + 2*b^2*c^6*x*y^2 - c^8*x*y^2 - a^4*b^4*x^2*z + 2*a^2*b^6*x^2*z - b^8*x^2*z -      a^4*b^2*c^2*x^2*z + b^6*c^2*x^2*z - b^4*c^4*x^2*z + b^2*c^6*x^2*z + a^8*y^2*z - 2*a^6*b^2*y^2*z +      a^4*b^4*y^2*z - a^6*c^2*y^2*z + a^2*b^4*c^2*y^2*z + a^4*c^4*y^2*z - a^2*c^6*y^2*z - a^6*b^2*x*z^2 +      a^4*b^4*x*z^2 - a^2*b^6*x*z^2 + b^8*x*z^2 - 2*b^6*c^2*x*z^2 + a^2*b^2*c^4*x*z^2 + b^4*c^4*x*z^2 -      a^8*y*z^2 + a^6*b^2*y*z^2 - a^4*b^4*y*z^2 + a^2*b^6*y*z^2 + 2*a^6*c^2*y*z^2 - a^4*c^4*y*z^2 -      a^2*b^2*c^4*y*z^2");
		exp("K337","a^6*c^2*x^2*y - a^4*b^2*c^2*x^2*y + a^2*b^4*c^2*x^2*y - b^6*c^2*x^2*y - 2*a^4*c^4*x^2*y +      a^2*c^6*x^2*y + b^2*c^6*x^2*y + a^6*c^2*x*y^2 - a^4*b^2*c^2*x*y^2 + a^2*b^4*c^2*x*y^2 -      b^6*c^2*x*y^2 + 2*b^4*c^4*x*y^2 - a^2*c^6*x*y^2 - b^2*c^6*x*y^2 - a^6*b^2*x^2*z + 2*a^4*b^4*x^2*z -      a^2*b^6*x^2*z + a^4*b^2*c^2*x^2*z - b^6*c^2*x^2*z - a^2*b^2*c^4*x^2*z + b^2*c^6*x^2*z +      a^6*b^2*y^2*z - 2*a^4*b^4*y^2*z + a^2*b^6*y^2*z + a^6*c^2*y^2*z - a^2*b^4*c^2*y^2*z +      a^2*b^2*c^4*y^2*z - a^2*c^6*y^2*z - a^6*b^2*x*z^2 + a^2*b^6*x*z^2 + a^4*b^2*c^2*x*z^2 +      b^6*c^2*x*z^2 - a^2*b^2*c^4*x*z^2 - 2*b^4*c^4*x*z^2 + b^2*c^6*x*z^2 - a^6*b^2*y*z^2 +      a^2*b^6*y*z^2 - a^6*c^2*y*z^2 - a^2*b^4*c^2*y*z^2 + 2*a^4*c^4*y*z^2 + a^2*b^2*c^4*y*z^2 -      a^2*c^6*y*z^2");
		exp("K338","a^2*c*x^2*y - b^2*c*x^2*y - a*c^2*x^2*y + 3*b*c^2*x^2*y - 2*c^3*x^2*y + a^2*c*x*y^2 -      b^2*c*x*y^2 - 3*a*c^2*x*y^2 + b*c^2*x*y^2 + 2*c^3*x*y^2 - a^2*b*x^2*z + a*b^2*x^2*z + 2*b^3*x^2*z -      3*b^2*c*x^2*z + b*c^2*x^2*z - 2*a^3*y^2*z - a^2*b*y^2*z + a*b^2*y^2*z + 3*a^2*c*y^2*z -      a*c^2*y^2*z - a^2*b*x*z^2 + 3*a*b^2*x*z^2 - 2*b^3*x*z^2 - b^2*c*x*z^2 + b*c^2*x*z^2 + 2*a^3*y*z^2 -      3*a^2*b*y*z^2 + a*b^2*y*z^2 + a^2*c*y*z^2 - a*c^2*y*z^2");
		exp("K339","(-a^10)*c^2*x^2*y + a^8*b^2*c^2*x^2*y + 2*a^6*b^4*c^2*x^2*y - 2*a^4*b^6*c^2*x^2*y -      a^2*b^8*c^2*x^2*y + b^10*c^2*x^2*y + 4*a^8*c^4*x^2*y - 4*a^6*b^2*c^4*x^2*y - 2*a^4*b^4*c^4*x^2*y +      4*a^2*b^6*c^4*x^2*y - 2*b^8*c^4*x^2*y - 6*a^6*c^6*x^2*y + 4*a^4*b^2*c^6*x^2*y -      2*a^2*b^4*c^6*x^2*y + 4*a^4*c^8*x^2*y + 2*b^4*c^8*x^2*y - a^2*c^10*x^2*y - b^2*c^10*x^2*y -      a^10*c^2*x*y^2 + a^8*b^2*c^2*x*y^2 + 2*a^6*b^4*c^2*x*y^2 - 2*a^4*b^6*c^2*x*y^2 -      a^2*b^8*c^2*x*y^2 + b^10*c^2*x*y^2 + 2*a^8*c^4*x*y^2 - 4*a^6*b^2*c^4*x*y^2 + 2*a^4*b^4*c^4*x*y^2 +      4*a^2*b^6*c^4*x*y^2 - 4*b^8*c^4*x*y^2 + 2*a^4*b^2*c^6*x*y^2 - 4*a^2*b^4*c^6*x*y^2 +      6*b^6*c^6*x*y^2 - 2*a^4*c^8*x*y^2 - 4*b^4*c^8*x*y^2 + a^2*c^10*x*y^2 + b^2*c^10*x*y^2 +      a^10*b^2*x^2*z - 4*a^8*b^4*x^2*z + 6*a^6*b^6*x^2*z - 4*a^4*b^8*x^2*z + a^2*b^10*x^2*z -      a^8*b^2*c^2*x^2*z + 4*a^6*b^4*c^2*x^2*z - 4*a^4*b^6*c^2*x^2*z + b^10*c^2*x^2*z -      2*a^6*b^2*c^4*x^2*z + 2*a^4*b^4*c^4*x^2*z + 2*a^2*b^6*c^4*x^2*z - 2*b^8*c^4*x^2*z +      2*a^4*b^2*c^6*x^2*z - 4*a^2*b^4*c^6*x^2*z + a^2*b^2*c^8*x^2*z + 2*b^4*c^8*x^2*z - b^2*c^10*x^2*z -      a^10*b^2*y^2*z + 4*a^8*b^4*y^2*z - 6*a^6*b^6*y^2*z + 4*a^4*b^8*y^2*z - a^2*b^10*y^2*z -      a^10*c^2*y^2*z + 4*a^6*b^4*c^2*y^2*z - 4*a^4*b^6*c^2*y^2*z + a^2*b^8*c^2*y^2*z + 2*a^8*c^4*y^2*z -      2*a^6*b^2*c^4*y^2*z - 2*a^4*b^4*c^4*y^2*z + 2*a^2*b^6*c^4*y^2*z + 4*a^4*b^2*c^6*y^2*z -      2*a^2*b^4*c^6*y^2*z - 2*a^4*c^8*y^2*z - a^2*b^2*c^8*y^2*z + a^2*c^10*y^2*z + a^10*b^2*x*z^2 -      2*a^8*b^4*x*z^2 + 2*a^4*b^8*x*z^2 - a^2*b^10*x*z^2 - a^8*b^2*c^2*x*z^2 + 4*a^6*b^4*c^2*x*z^2 -      2*a^4*b^6*c^2*x*z^2 - b^10*c^2*x*z^2 - 2*a^6*b^2*c^4*x*z^2 - 2*a^4*b^4*c^4*x*z^2 +      4*a^2*b^6*c^4*x*z^2 + 4*b^8*c^4*x*z^2 + 2*a^4*b^2*c^6*x*z^2 - 4*a^2*b^4*c^6*x*z^2 -      6*b^6*c^6*x*z^2 + a^2*b^2*c^8*x*z^2 + 4*b^4*c^8*x*z^2 - b^2*c^10*x*z^2 + a^10*b^2*y*z^2 -      2*a^8*b^4*y*z^2 + 2*a^4*b^8*y*z^2 - a^2*b^10*y*z^2 + a^10*c^2*y*z^2 + 2*a^6*b^4*c^2*y*z^2 -      4*a^4*b^6*c^2*y*z^2 + a^2*b^8*c^2*y*z^2 - 4*a^8*c^4*y*z^2 - 4*a^6*b^2*c^4*y*z^2 +      2*a^4*b^4*c^4*y*z^2 + 2*a^2*b^6*c^4*y*z^2 + 6*a^6*c^6*y*z^2 + 4*a^4*b^2*c^6*y*z^2 -      2*a^2*b^4*c^6*y*z^2 - 4*a^4*c^8*y*z^2 - a^2*b^2*c^8*y*z^2 + a^2*c^10*y*z^2");
		exp("K340","(-a^4)*b^2*c^3*x^2*y + b^6*c^3*x^2*y - a^4*b*c^4*x^2*y - a^2*b^3*c^4*x^2*y + a*b^4*c^4*x^2*y +      b^5*c^4*x^2*y + a^2*b^2*c^5*x^2*y + 2*a^2*b*c^6*x^2*y - a*b^2*c^6*x^2*y - b^2*c^7*x^2*y -      b*c^8*x^2*y - a^6*c^3*x*y^2 + a^2*b^4*c^3*x*y^2 - a^5*c^4*x*y^2 - a^4*b*c^4*x*y^2 +      a^3*b^2*c^4*x*y^2 + a*b^4*c^4*x*y^2 - a^2*b^2*c^5*x*y^2 + a^2*b*c^6*x*y^2 - 2*a*b^2*c^6*x*y^2 +      a^2*c^7*x*y^2 + a*c^8*x*y^2 + a^4*b^4*c*x^2*z - 2*a^2*b^6*c*x^2*z + b^8*c*x^2*z +      a^4*b^3*c^2*x^2*z - a^2*b^5*c^2*x^2*z + a*b^6*c^2*x^2*z + b^7*c^2*x^2*z + a^2*b^4*c^3*x^2*z -      a*b^4*c^4*x^2*z - b^4*c^5*x^2*z - b^3*c^6*x^2*z - a^8*c*y^2*z + 2*a^6*b^2*c*y^2*z -      a^4*b^4*c*y^2*z - a^7*c^2*y^2*z - a^6*b*c^2*y^2*z + a^5*b^2*c^2*y^2*z - a^3*b^4*c^2*y^2*z -      a^4*b^2*c^3*y^2*z + a^4*b*c^4*y^2*z + a^4*c^5*y^2*z + a^3*c^6*y^2*z + a^6*b^3*x*z^2 +      a^5*b^4*x*z^2 - a^2*b^7*x*z^2 - a*b^8*x*z^2 + a^4*b^4*c*x*z^2 - a^2*b^6*c*x*z^2 -      a^3*b^4*c^2*x*z^2 + a^2*b^5*c^2*x*z^2 + 2*a*b^6*c^2*x*z^2 - a^2*b^3*c^4*x*z^2 - a*b^4*c^4*x*z^2 +      a^8*b*y*z^2 + a^7*b^2*y*z^2 - a^4*b^5*y*z^2 - a^3*b^6*y*z^2 + a^6*b^2*c*y*z^2 - a^4*b^4*c*y*z^2 -      2*a^6*b*c^2*y*z^2 - a^5*b^2*c^2*y*z^2 + a^4*b^3*c^2*y*z^2 + a^4*b*c^4*y*z^2 + a^3*b^2*c^4*y*z^2 ");
		exp("K341a","Sqrt((a + b - c)*(a - b + c)*(-a + b + c)*(a + b + c))*(c^2*x^2*y - c^2*x*y^2 - b^2*x^2*z +        a^2*y^2*z + b^2*x*z^2 - a^2*y*z^2) + Sqrt(3)*(a^2*c^2*x^2*y + b^2*c^2*x^2*y - c^4*x^2*y -        a^2*c^2*x*y^2 - b^2*c^2*x*y^2 + c^4*x*y^2 - a^2*b^2*x^2*z + b^4*x^2*z - b^2*c^2*x^2*z -        a^4*y^2*z + a^2*b^2*y^2*z + a^2*c^2*y^2*z + a^2*b^2*x*z^2 - b^4*x*z^2 + b^2*c^2*x*z^2 +        a^4*y*z^2 - a^2*b^2*y*z^2 - a^2*c^2*y*z^2)");
		exp("K341b","Sqrt((a + b - c)*(a - b + c)*(-a + b + c)*(a + b + c))*(c^2*x^2*y - c^2*x*y^2 - b^2*x^2*z +        a^2*y^2*z + b^2*x*z^2 - a^2*y*z^2) + Sqrt(3)*((-a^2)*c^2*x^2*y - b^2*c^2*x^2*y + c^4*x^2*y +        a^2*c^2*x*y^2 + b^2*c^2*x*y^2 - c^4*x*y^2 + a^2*b^2*x^2*z - b^4*x^2*z + b^2*c^2*x^2*z +        a^4*y^2*z - a^2*b^2*y^2*z - a^2*c^2*y^2*z - a^2*b^2*x*z^2 + b^4*x*z^2 - b^2*c^2*x*z^2 -        a^4*y*z^2 + a^2*b^2*y*z^2 + a^2*c^2*y*z^2)");
		exp("K342a","2*a^10*x^2*y - 5*a^8*b^2*x^2*y + 3*a^6*b^4*x^2*y + a^4*b^6*x^2*y - a^2*b^8*x^2*y -      5*a^8*c^2*x^2*y + 5*a^6*b^2*c^2*x^2*y + 2*a^4*b^4*c^2*x^2*y - 2*a^2*b^6*c^2*x^2*y +      3*a^6*c^4*x^2*y + 2*a^4*b^2*c^4*x^2*y + 6*a^2*b^4*c^4*x^2*y + a^4*c^6*x^2*y - 2*a^2*b^2*c^6*x^2*y -      a^2*c^8*x^2*y + a^8*b^2*x*y^2 - a^6*b^4*x*y^2 - 3*a^4*b^6*x*y^2 + 5*a^2*b^8*x*y^2 - 2*b^10*x*y^2 +      2*a^6*b^2*c^2*x*y^2 - 2*a^4*b^4*c^2*x*y^2 - 5*a^2*b^6*c^2*x*y^2 + 5*b^8*c^2*x*y^2 -      6*a^4*b^2*c^4*x*y^2 - 2*a^2*b^4*c^4*x*y^2 - 3*b^6*c^4*x*y^2 + 2*a^2*b^2*c^6*x*y^2 - b^4*c^6*x*y^2 +      b^2*c^8*x*y^2 - 2*a^10*x^2*z + 5*a^8*b^2*x^2*z - 3*a^6*b^4*x^2*z - a^4*b^6*x^2*z + a^2*b^8*x^2*z +      5*a^8*c^2*x^2*z - 5*a^6*b^2*c^2*x^2*z - 2*a^4*b^4*c^2*x^2*z + 2*a^2*b^6*c^2*x^2*z -      3*a^6*c^4*x^2*z - 2*a^4*b^2*c^4*x^2*z - 6*a^2*b^4*c^4*x^2*z - a^4*c^6*x^2*z + 2*a^2*b^2*c^6*x^2*z +      a^2*c^8*x^2*z - a^8*b^2*y^2*z + a^6*b^4*y^2*z + 3*a^4*b^6*y^2*z - 5*a^2*b^8*y^2*z + 2*b^10*y^2*z -      2*a^6*b^2*c^2*y^2*z + 2*a^4*b^4*c^2*y^2*z + 5*a^2*b^6*c^2*y^2*z - 5*b^8*c^2*y^2*z +      6*a^4*b^2*c^4*y^2*z + 2*a^2*b^4*c^4*y^2*z + 3*b^6*c^4*y^2*z - 2*a^2*b^2*c^6*y^2*z + b^4*c^6*y^2*z -      b^2*c^8*y^2*z - a^8*c^2*x*z^2 - 2*a^6*b^2*c^2*x*z^2 + 6*a^4*b^4*c^2*x*z^2 - 2*a^2*b^6*c^2*x*z^2 -      b^8*c^2*x*z^2 + a^6*c^4*x*z^2 + 2*a^4*b^2*c^4*x*z^2 + 2*a^2*b^4*c^4*x*z^2 + b^6*c^4*x*z^2 +      3*a^4*c^6*x*z^2 + 5*a^2*b^2*c^6*x*z^2 + 3*b^4*c^6*x*z^2 - 5*a^2*c^8*x*z^2 - 5*b^2*c^8*x*z^2 +      2*c^10*x*z^2 + a^8*c^2*y*z^2 + 2*a^6*b^2*c^2*y*z^2 - 6*a^4*b^4*c^2*y*z^2 + 2*a^2*b^6*c^2*y*z^2 +      b^8*c^2*y*z^2 - a^6*c^4*y*z^2 - 2*a^4*b^2*c^4*y*z^2 - 2*a^2*b^4*c^4*y*z^2 - b^6*c^4*y*z^2 -      3*a^4*c^6*y*z^2 - 5*a^2*b^2*c^6*y*z^2 - 3*b^4*c^6*y*z^2 + 5*a^2*c^8*y*z^2 + 5*b^2*c^8*y*z^2 -      2*c^10*y*z^2 + Sqrt(3)*Sqrt((a + b - c)*(a - b + c)*(-a + b + c)*(a + b + c))*      ((-a^6)*b^2*x^2*y + 2*a^4*b^4*x^2*y - a^2*b^6*x^2*y - a^6*c^2*x^2*y + a^4*b^2*c^2*x^2*y +        a^2*b^4*c^2*x^2*y + 2*a^4*c^4*x^2*y + a^2*b^2*c^4*x^2*y - a^2*c^6*x^2*y + a^6*b^2*x*y^2 -        2*a^4*b^4*x*y^2 + a^2*b^6*x*y^2 - a^4*b^2*c^2*x*y^2 - a^2*b^4*c^2*x*y^2 + b^6*c^2*x*y^2 -        a^2*b^2*c^4*x*y^2 - 2*b^4*c^4*x*y^2 + b^2*c^6*x*y^2 + a^6*b^2*x^2*z - 2*a^4*b^4*x^2*z +        a^2*b^6*x^2*z + a^6*c^2*x^2*z - a^4*b^2*c^2*x^2*z - a^2*b^4*c^2*x^2*z - 2*a^4*c^4*x^2*z -        a^2*b^2*c^4*x^2*z + a^2*c^6*x^2*z - a^6*b^2*y^2*z + 2*a^4*b^4*y^2*z - a^2*b^6*y^2*z +        a^4*b^2*c^2*y^2*z + a^2*b^4*c^2*y^2*z - b^6*c^2*y^2*z + a^2*b^2*c^4*y^2*z + 2*b^4*c^4*y^2*z -        b^2*c^6*y^2*z - a^6*c^2*x*z^2 + a^4*b^2*c^2*x*z^2 + a^2*b^4*c^2*x*z^2 - b^6*c^2*x*z^2 +        2*a^4*c^4*x*z^2 + a^2*b^2*c^4*x*z^2 + 2*b^4*c^4*x*z^2 - a^2*c^6*x*z^2 - b^2*c^6*x*z^2 +        a^6*c^2*y*z^2 - a^4*b^2*c^2*y*z^2 - a^2*b^4*c^2*y*z^2 + b^6*c^2*y*z^2 - 2*a^4*c^4*y*z^2 -        a^2*b^2*c^4*y*z^2 - 2*b^4*c^4*y*z^2 + a^2*c^6*y*z^2 + b^2*c^6*y*z^2)");
		exp("K342b","2*a^10*x^2*y - 5*a^8*b^2*x^2*y + 3*a^6*b^4*x^2*y + a^4*b^6*x^2*y - a^2*b^8*x^2*y -      5*a^8*c^2*x^2*y + 5*a^6*b^2*c^2*x^2*y + 2*a^4*b^4*c^2*x^2*y - 2*a^2*b^6*c^2*x^2*y +      3*a^6*c^4*x^2*y + 2*a^4*b^2*c^4*x^2*y + 6*a^2*b^4*c^4*x^2*y + a^4*c^6*x^2*y - 2*a^2*b^2*c^6*x^2*y -      a^2*c^8*x^2*y + a^8*b^2*x*y^2 - a^6*b^4*x*y^2 - 3*a^4*b^6*x*y^2 + 5*a^2*b^8*x*y^2 - 2*b^10*x*y^2 +      2*a^6*b^2*c^2*x*y^2 - 2*a^4*b^4*c^2*x*y^2 - 5*a^2*b^6*c^2*x*y^2 + 5*b^8*c^2*x*y^2 -      6*a^4*b^2*c^4*x*y^2 - 2*a^2*b^4*c^4*x*y^2 - 3*b^6*c^4*x*y^2 + 2*a^2*b^2*c^6*x*y^2 - b^4*c^6*x*y^2 +      b^2*c^8*x*y^2 - 2*a^10*x^2*z + 5*a^8*b^2*x^2*z - 3*a^6*b^4*x^2*z - a^4*b^6*x^2*z + a^2*b^8*x^2*z +      5*a^8*c^2*x^2*z - 5*a^6*b^2*c^2*x^2*z - 2*a^4*b^4*c^2*x^2*z + 2*a^2*b^6*c^2*x^2*z -      3*a^6*c^4*x^2*z - 2*a^4*b^2*c^4*x^2*z - 6*a^2*b^4*c^4*x^2*z - a^4*c^6*x^2*z + 2*a^2*b^2*c^6*x^2*z +      a^2*c^8*x^2*z - a^8*b^2*y^2*z + a^6*b^4*y^2*z + 3*a^4*b^6*y^2*z - 5*a^2*b^8*y^2*z + 2*b^10*y^2*z -      2*a^6*b^2*c^2*y^2*z + 2*a^4*b^4*c^2*y^2*z + 5*a^2*b^6*c^2*y^2*z - 5*b^8*c^2*y^2*z +      6*a^4*b^2*c^4*y^2*z + 2*a^2*b^4*c^4*y^2*z + 3*b^6*c^4*y^2*z - 2*a^2*b^2*c^6*y^2*z + b^4*c^6*y^2*z -      b^2*c^8*y^2*z - a^8*c^2*x*z^2 - 2*a^6*b^2*c^2*x*z^2 + 6*a^4*b^4*c^2*x*z^2 - 2*a^2*b^6*c^2*x*z^2 -      b^8*c^2*x*z^2 + a^6*c^4*x*z^2 + 2*a^4*b^2*c^4*x*z^2 + 2*a^2*b^4*c^4*x*z^2 + b^6*c^4*x*z^2 +      3*a^4*c^6*x*z^2 + 5*a^2*b^2*c^6*x*z^2 + 3*b^4*c^6*x*z^2 - 5*a^2*c^8*x*z^2 - 5*b^2*c^8*x*z^2 +      2*c^10*x*z^2 + a^8*c^2*y*z^2 + 2*a^6*b^2*c^2*y*z^2 - 6*a^4*b^4*c^2*y*z^2 + 2*a^2*b^6*c^2*y*z^2 +      b^8*c^2*y*z^2 - a^6*c^4*y*z^2 - 2*a^4*b^2*c^4*y*z^2 - 2*a^2*b^4*c^4*y*z^2 - b^6*c^4*y*z^2 -      3*a^4*c^6*y*z^2 - 5*a^2*b^2*c^6*y*z^2 - 3*b^4*c^6*y*z^2 + 5*a^2*c^8*y*z^2 + 5*b^2*c^8*y*z^2 -      2*c^10*y*z^2 + Sqrt(3)*Sqrt((a + b - c)*(a - b + c)*(-a + b + c)*(a + b + c))*      (a^6*b^2*x^2*y - 2*a^4*b^4*x^2*y + a^2*b^6*x^2*y + a^6*c^2*x^2*y - a^4*b^2*c^2*x^2*y -        a^2*b^4*c^2*x^2*y - 2*a^4*c^4*x^2*y - a^2*b^2*c^4*x^2*y + a^2*c^6*x^2*y - a^6*b^2*x*y^2 +        2*a^4*b^4*x*y^2 - a^2*b^6*x*y^2 + a^4*b^2*c^2*x*y^2 + a^2*b^4*c^2*x*y^2 - b^6*c^2*x*y^2 +        a^2*b^2*c^4*x*y^2 + 2*b^4*c^4*x*y^2 - b^2*c^6*x*y^2 - a^6*b^2*x^2*z + 2*a^4*b^4*x^2*z -        a^2*b^6*x^2*z - a^6*c^2*x^2*z + a^4*b^2*c^2*x^2*z + a^2*b^4*c^2*x^2*z + 2*a^4*c^4*x^2*z +        a^2*b^2*c^4*x^2*z - a^2*c^6*x^2*z + a^6*b^2*y^2*z - 2*a^4*b^4*y^2*z + a^2*b^6*y^2*z -        a^4*b^2*c^2*y^2*z - a^2*b^4*c^2*y^2*z + b^6*c^2*y^2*z - a^2*b^2*c^4*y^2*z - 2*b^4*c^4*y^2*z +        b^2*c^6*y^2*z + a^6*c^2*x*z^2 - a^4*b^2*c^2*x*z^2 - a^2*b^4*c^2*x*z^2 + b^6*c^2*x*z^2 -        2*a^4*c^4*x*z^2 - a^2*b^2*c^4*x*z^2 - 2*b^4*c^4*x*z^2 + a^2*c^6*x*z^2 + b^2*c^6*x*z^2 -        a^6*c^2*y*z^2 + a^4*b^2*c^2*y*z^2 + a^2*b^4*c^2*y*z^2 - b^6*c^2*y*z^2 + 2*a^4*c^4*y*z^2 +        a^2*b^2*c^4*y*z^2 + 2*b^4*c^4*y*z^2 - a^2*c^6*y*z^2 - b^2*c^6*y*z^2)");
		exp("K343","a^2*b*c^2*x^2*y - b^3*c^2*x^2*y + b*c^4*x^2*y + a^3*c^2*x*y^2 - a*b^2*c^2*x*y^2 - a*c^4*x*y^2 -      a^2*b^2*c*x^2*z - b^4*c*x^2*z + b^2*c^3*x^2*z + a^4*c*y^2*z + a^2*b^2*c*y^2*z - a^2*c^3*y^2*z -      a^3*b^2*x*z^2 + a*b^4*x*z^2 + a*b^2*c^2*x*z^2 - a^4*b*y*z^2 + a^2*b^3*y*z^2 - a^2*b*c^2*y*z^2");
		exp("K344","(-a^3)*b*c*x^2*y - a^2*b^2*c*x^2*y + a*b^3*c*x^2*y + b^4*c*x^2*y - a^3*c^2*x^2*y -      a*b^2*c^2*x^2*y + 2*b^3*c^2*x^2*y + a^2*c^3*x^2*y - a*b*c^3*x^2*y + a*c^4*x^2*y - 2*b*c^4*x^2*y -      c^5*x^2*y - a^4*c*x*y^2 - a^3*b*c*x*y^2 + a^2*b^2*c*x*y^2 + a*b^3*c*x*y^2 - 2*a^3*c^2*x*y^2 +      a^2*b*c^2*x*y^2 + b^3*c^2*x*y^2 + a*b*c^3*x*y^2 - b^2*c^3*x*y^2 + 2*a*c^4*x*y^2 - b*c^4*x*y^2 +      c^5*x*y^2 + a^3*b^2*x^2*z - a^2*b^3*x^2*z - a*b^4*x^2*z + b^5*x^2*z + a^3*b*c*x^2*z +      a*b^3*c*x^2*z + 2*b^4*c*x^2*z + a^2*b*c^2*x^2*z + a*b^2*c^2*x^2*z - a*b*c^3*x^2*z -      2*b^2*c^3*x^2*z - b*c^4*x^2*z - a^5*y^2*z + a^4*b*y^2*z + a^3*b^2*y^2*z - a^2*b^3*y^2*z -      2*a^4*c*y^2*z - a^3*b*c*y^2*z - a*b^3*c*y^2*z - a^2*b*c^2*y^2*z - a*b^2*c^2*y^2*z +      2*a^2*c^3*y^2*z + a*b*c^3*y^2*z + a*c^4*y^2*z + a^4*b*x*z^2 + 2*a^3*b^2*x*z^2 - 2*a*b^4*x*z^2 -      b^5*x*z^2 + a^3*b*c*x*z^2 - a^2*b^2*c*x*z^2 - a*b^3*c*x*z^2 + b^4*c*x*z^2 - a^2*b*c^2*x*z^2 +      b^3*c^2*x*z^2 - a*b*c^3*x*z^2 - b^2*c^3*x*z^2 + a^5*y*z^2 + 2*a^4*b*y*z^2 - 2*a^2*b^3*y*z^2 -      a*b^4*y*z^2 - a^4*c*y*z^2 + a^3*b*c*y*z^2 + a^2*b^2*c*y*z^2 - a*b^3*c*y*z^2 - a^3*c^2*y*z^2 +      a*b^2*c^2*y*z^2 + a^2*c^3*y*z^2 + a*b*c^3*y*z^2");
		exp("K345","a*c*x^2*y + b*c*x^2*y - a*c*x*y^2 - b*c*x*y^2 - a*b*x^2*z - b*c*x^2*z + a*b*y^2*z + a*c*y^2*z +      a*b*x*z^2 + b*c*x*z^2 - a*b*y*z^2 - a*c*y*z^2");
		exp("K346","b^2*c^6*x^2*y - a^2*c^6*x*y^2 - b^6*c^2*x^2*z + a^6*c^2*y^2*z + a^2*b^6*x*z^2 - a^6*b^2*y*z^2 ");
		exp("K347","9*a^8*x^2*y - 24*a^6*b^2*x^2*y + 18*a^4*b^4*x^2*y - 3*b^8*x^2*y + 20*a^4*b^2*c^2*x^2*y -      24*a^2*b^4*c^2*x^2*y + 4*b^6*c^2*x^2*y - 22*a^4*c^4*x^2*y + 16*a^2*b^2*c^4*x^2*y +      6*b^4*c^4*x^2*y + 8*a^2*c^6*x^2*y - 12*b^2*c^6*x^2*y + 5*c^8*x^2*y + 3*a^8*x*y^2 -      18*a^4*b^4*x*y^2 + 24*a^2*b^6*x*y^2 - 9*b^8*x*y^2 - 4*a^6*c^2*x*y^2 + 24*a^4*b^2*c^2*x*y^2 -      20*a^2*b^4*c^2*x*y^2 - 6*a^4*c^4*x*y^2 - 16*a^2*b^2*c^4*x*y^2 + 22*b^4*c^4*x*y^2 +      12*a^2*c^6*x*y^2 - 8*b^2*c^6*x*y^2 - 5*c^8*x*y^2 - 9*a^8*x^2*z + 22*a^4*b^4*x^2*z -      8*a^2*b^6*x^2*z - 5*b^8*x^2*z + 24*a^6*c^2*x^2*z - 20*a^4*b^2*c^2*x^2*z - 16*a^2*b^4*c^2*x^2*z +      12*b^6*c^2*x^2*z - 18*a^4*c^4*x^2*z + 24*a^2*b^2*c^4*x^2*z - 6*b^4*c^4*x^2*z - 4*b^2*c^6*x^2*z +      3*c^8*x^2*z + 5*a^8*y^2*z + 8*a^6*b^2*y^2*z - 22*a^4*b^4*y^2*z + 9*b^8*y^2*z - 12*a^6*c^2*y^2*z +      16*a^4*b^2*c^2*y^2*z + 20*a^2*b^4*c^2*y^2*z - 24*b^6*c^2*y^2*z + 6*a^4*c^4*y^2*z -      24*a^2*b^2*c^4*y^2*z + 18*b^4*c^4*y^2*z + 4*a^2*c^6*y^2*z - 3*c^8*y^2*z - 3*a^8*x*z^2 +      4*a^6*b^2*x*z^2 + 6*a^4*b^4*x*z^2 - 12*a^2*b^6*x*z^2 + 5*b^8*x*z^2 - 24*a^4*b^2*c^2*x*z^2 +      16*a^2*b^4*c^2*x*z^2 + 8*b^6*c^2*x*z^2 + 18*a^4*c^4*x*z^2 + 20*a^2*b^2*c^4*x*z^2 -      22*b^4*c^4*x*z^2 - 24*a^2*c^6*x*z^2 + 9*c^8*x*z^2 - 5*a^8*y*z^2 + 12*a^6*b^2*y*z^2 -      6*a^4*b^4*y*z^2 - 4*a^2*b^6*y*z^2 + 3*b^8*y*z^2 - 8*a^6*c^2*y*z^2 - 16*a^4*b^2*c^2*y*z^2 +      24*a^2*b^4*c^2*y*z^2 + 22*a^4*c^4*y*z^2 - 20*a^2*b^2*c^4*y*z^2 - 18*b^4*c^4*y*z^2 +      24*b^2*c^6*y*z^2 - 9*c^8*y*z^2");
		exp("K348","3*a^10*x^2*y - 8*a^8*b^2*x^2*y + 6*a^6*b^4*x^2*y - a^2*b^8*x^2*y + a^8*c^2*x^2*y +      4*a^6*b^2*c^2*x^2*y - 10*a^4*b^4*c^2*x^2*y + 4*a^2*b^6*c^2*x^2*y + b^8*c^2*x^2*y -      6*a^6*c^4*x^2*y + 12*a^4*b^2*c^4*x^2*y - 2*a^2*b^4*c^4*x^2*y - 4*b^6*c^4*x^2*y - 2*a^4*c^6*x^2*y -      4*a^2*b^2*c^6*x^2*y + 6*b^4*c^6*x^2*y + 3*a^2*c^8*x^2*y - 4*b^2*c^8*x^2*y + c^10*x^2*y +      a^8*b^2*x*y^2 - 6*a^4*b^6*x*y^2 + 8*a^2*b^8*x*y^2 - 3*b^10*x*y^2 - a^8*c^2*x*y^2 -      4*a^6*b^2*c^2*x*y^2 + 10*a^4*b^4*c^2*x*y^2 - 4*a^2*b^6*c^2*x*y^2 - b^8*c^2*x*y^2 +      4*a^6*c^4*x*y^2 + 2*a^4*b^2*c^4*x*y^2 - 12*a^2*b^4*c^4*x*y^2 + 6*b^6*c^4*x*y^2 - 6*a^4*c^6*x*y^2 +      4*a^2*b^2*c^6*x*y^2 + 2*b^4*c^6*x*y^2 + 4*a^2*c^8*x*y^2 - 3*b^2*c^8*x*y^2 - c^10*x*y^2 -      3*a^10*x^2*z - a^8*b^2*x^2*z + 6*a^6*b^4*x^2*z + 2*a^4*b^6*x^2*z - 3*a^2*b^8*x^2*z - b^10*x^2*z +      8*a^8*c^2*x^2*z - 4*a^6*b^2*c^2*x^2*z - 12*a^4*b^4*c^2*x^2*z + 4*a^2*b^6*c^2*x^2*z +      4*b^8*c^2*x^2*z - 6*a^6*c^4*x^2*z + 10*a^4*b^2*c^4*x^2*z + 2*a^2*b^4*c^4*x^2*z - 6*b^6*c^4*x^2*z -      4*a^2*b^2*c^6*x^2*z + 4*b^4*c^6*x^2*z + a^2*c^8*x^2*z - b^2*c^8*x^2*z + a^10*y^2*z +      3*a^8*b^2*y^2*z - 2*a^6*b^4*y^2*z - 6*a^4*b^6*y^2*z + a^2*b^8*y^2*z + 3*b^10*y^2*z -      4*a^8*c^2*y^2*z - 4*a^6*b^2*c^2*y^2*z + 12*a^4*b^4*c^2*y^2*z + 4*a^2*b^6*c^2*y^2*z -      8*b^8*c^2*y^2*z + 6*a^6*c^4*y^2*z - 2*a^4*b^2*c^4*y^2*z - 10*a^2*b^4*c^4*y^2*z + 6*b^6*c^4*y^2*z -      4*a^4*c^6*y^2*z + 4*a^2*b^2*c^6*y^2*z + a^2*c^8*y^2*z - b^2*c^8*y^2*z + a^8*b^2*x*z^2 -      4*a^6*b^4*x*z^2 + 6*a^4*b^6*x*z^2 - 4*a^2*b^8*x*z^2 + b^10*x*z^2 - a^8*c^2*x*z^2 +      4*a^6*b^2*c^2*x*z^2 - 2*a^4*b^4*c^2*x*z^2 - 4*a^2*b^6*c^2*x*z^2 + 3*b^8*c^2*x*z^2 -      10*a^4*b^2*c^4*x*z^2 + 12*a^2*b^4*c^4*x*z^2 - 2*b^6*c^4*x*z^2 + 6*a^4*c^6*x*z^2 +      4*a^2*b^2*c^6*x*z^2 - 6*b^4*c^6*x*z^2 - 8*a^2*c^8*x*z^2 + b^2*c^8*x*z^2 + 3*c^10*x*z^2 -      a^10*y*z^2 + 4*a^8*b^2*y*z^2 - 6*a^6*b^4*y*z^2 + 4*a^4*b^6*y*z^2 - a^2*b^8*y*z^2 -      3*a^8*c^2*y*z^2 + 4*a^6*b^2*c^2*y*z^2 + 2*a^4*b^4*c^2*y*z^2 - 4*a^2*b^6*c^2*y*z^2 + b^8*c^2*y*z^2 +      2*a^6*c^4*y*z^2 - 12*a^4*b^2*c^4*y*z^2 + 10*a^2*b^4*c^4*y*z^2 + 6*a^4*c^6*y*z^2 -      4*a^2*b^2*c^6*y*z^2 - 6*b^4*c^6*y*z^2 - a^2*c^8*y*z^2 + 8*b^2*c^8*y*z^2 - 3*c^10*y*z^2");
		exp("K349","a^6*b^2*c^4*x^2*y - 2*a^4*b^4*c^4*x^2*y + 2*a^2*b^6*c^4*x^2*y - b^8*c^4*x^2*y -      a^4*b^2*c^6*x^2*y - a^2*b^4*c^6*x^2*y + 3*b^6*c^6*x^2*y - a^2*b^2*c^8*x^2*y - 3*b^4*c^8*x^2*y +      b^2*c^10*x^2*y + a^8*c^4*x*y^2 - 2*a^6*b^2*c^4*x*y^2 + 2*a^4*b^4*c^4*x*y^2 - a^2*b^6*c^4*x*y^2 -      3*a^6*c^6*x*y^2 + a^4*b^2*c^6*x*y^2 + a^2*b^4*c^6*x*y^2 + 3*a^4*c^8*x*y^2 + a^2*b^2*c^8*x*y^2 -      a^2*c^10*x*y^2 - a^6*b^4*c^2*x^2*z + a^4*b^6*c^2*x^2*z + a^2*b^8*c^2*x^2*z - b^10*c^2*x^2*z +      2*a^4*b^4*c^4*x^2*z + a^2*b^6*c^4*x^2*z + 3*b^8*c^4*x^2*z - 2*a^2*b^4*c^6*x^2*z - 3*b^6*c^6*x^2*z +      b^4*c^8*x^2*z + a^10*c^2*y^2*z - a^8*b^2*c^2*y^2*z - a^6*b^4*c^2*y^2*z + a^4*b^6*c^2*y^2*z -      3*a^8*c^4*y^2*z - a^6*b^2*c^4*y^2*z - 2*a^4*b^4*c^4*y^2*z + 3*a^6*c^6*y^2*z + 2*a^4*b^2*c^6*y^2*z -      a^4*c^8*y^2*z - a^8*b^4*x*z^2 + 3*a^6*b^6*x*z^2 - 3*a^4*b^8*x*z^2 + a^2*b^10*x*z^2 +      2*a^6*b^4*c^2*x*z^2 - a^4*b^6*c^2*x*z^2 - a^2*b^8*c^2*x*z^2 - 2*a^4*b^4*c^4*x*z^2 -      a^2*b^6*c^4*x*z^2 + a^2*b^4*c^6*x*z^2 - a^10*b^2*y*z^2 + 3*a^8*b^4*y*z^2 - 3*a^6*b^6*y*z^2 +      a^4*b^8*y*z^2 + a^8*b^2*c^2*y*z^2 + a^6*b^4*c^2*y*z^2 - 2*a^4*b^6*c^2*y*z^2 + a^6*b^2*c^4*y*z^2 +      2*a^4*b^4*c^4*y*z^2 - a^4*b^2*c^6*y*z^2");
		exp("K350","a^6*c^2*x^2*y - 3*a^4*b^2*c^2*x^2*y + 3*a^2*b^4*c^2*x^2*y - b^6*c^2*x^2*y - 2*a^4*c^4*x^2*y +      2*a^2*b^2*c^4*x^2*y + a^2*c^6*x^2*y + b^2*c^6*x^2*y + a^6*c^2*x*y^2 - 3*a^4*b^2*c^2*x*y^2 +      3*a^2*b^4*c^2*x*y^2 - b^6*c^2*x*y^2 - 2*a^2*b^2*c^4*x*y^2 + 2*b^4*c^4*x*y^2 - a^2*c^6*x*y^2 -      b^2*c^6*x*y^2 - a^6*b^2*x^2*z + 2*a^4*b^4*x^2*z - a^2*b^6*x^2*z + 3*a^4*b^2*c^2*x^2*z -      2*a^2*b^4*c^2*x^2*z - b^6*c^2*x^2*z - 3*a^2*b^2*c^4*x^2*z + b^2*c^6*x^2*z + a^6*b^2*y^2*z -      2*a^4*b^4*y^2*z + a^2*b^6*y^2*z + a^6*c^2*y^2*z + 2*a^4*b^2*c^2*y^2*z - 3*a^2*b^4*c^2*y^2*z +      3*a^2*b^2*c^4*y^2*z - a^2*c^6*y^2*z - a^6*b^2*x*z^2 + a^2*b^6*x*z^2 + 3*a^4*b^2*c^2*x*z^2 +      2*a^2*b^4*c^2*x*z^2 + b^6*c^2*x*z^2 - 3*a^2*b^2*c^4*x*z^2 - 2*b^4*c^4*x*z^2 + b^2*c^6*x*z^2 -      a^6*b^2*y*z^2 + a^2*b^6*y*z^2 - a^6*c^2*y*z^2 - 2*a^4*b^2*c^2*y*z^2 - 3*a^2*b^4*c^2*y*z^2 +      2*a^4*c^4*y*z^2 + 3*a^2*b^2*c^4*y*z^2 - a^2*c^6*y*z^2");
		exp("K351","a*b*c^2*x^2*y - b^2*c^2*x^2*y + b*c^3*x^2*y + a^2*c^2*x*y^2 - a*b*c^2*x*y^2 - a*c^3*x*y^2 -      a*b^2*c*x^2*z - b^3*c*x^2*z + b^2*c^2*x^2*z + a^3*c*y^2*z + a^2*b*c*y^2*z - a^2*c^2*y^2*z -      a^2*b^2*x*z^2 + a*b^3*x*z^2 + a*b^2*c*x*z^2 - a^3*b*y*z^2 + a^2*b^2*y*z^2 - a^2*b*c*y*z^2");
		exp("K352","(-a^2)*c^2*x^2*y + a*b*c^2*x^2*y - b*c^3*x^2*y + c^4*x^2*y - a*b*c^2*x*y^2 + b^2*c^2*x*y^2 +      a*c^3*x*y^2 - c^4*x*y^2 + a^2*b^2*x^2*z - b^4*x^2*z - a*b^2*c*x^2*z + b^3*c*x^2*z + 2*a^3*b*x*y*z -      2*a*b^3*x*y*z - 2*a^3*c*x*y*z + 2*b^3*c*x*y*z + 2*a*c^3*x*y*z - 2*b*c^3*x*y*z + a^4*y^2*z -      a^2*b^2*y^2*z - a^3*c*y^2*z + a^2*b*c*y^2*z - a*b^3*x*z^2 + b^4*x*z^2 + a*b^2*c*x*z^2 -      b^2*c^2*x*z^2 - a^4*y*z^2 + a^3*b*y*z^2 - a^2*b*c*y*z^2 + a^2*c^2*y*z^2");
		exp("K353","-2*a^4*c^2*x^2*y - b^4*c^2*x^2*y + a^2*c^4*x^2*y + 2*b^2*c^4*x^2*y + a^4*c^2*x*y^2 +      2*b^4*c^2*x*y^2 - 2*a^2*c^4*x*y^2 - b^2*c^4*x*y^2 + 2*a^4*b^2*x^2*z - a^2*b^4*x^2*z -      2*b^4*c^2*x^2*z + b^2*c^4*x^2*z + a^4*b^2*y^2*z - 2*a^2*b^4*y^2*z + 2*a^4*c^2*y^2*z -      a^2*c^4*y^2*z - a^4*b^2*x*z^2 + 2*a^2*b^4*x*z^2 + b^4*c^2*x*z^2 - 2*b^2*c^4*x*z^2 -      2*a^4*b^2*y*z^2 + a^2*b^4*y*z^2 - a^4*c^2*y*z^2 + 2*a^2*c^4*y*z^2");
		exp("K354","(-a^4)*c^2*x^2*y + b^2*c^4*x^2*y + b^4*c^2*x*y^2 - a^2*c^4*x*y^2 + a^4*b^2*x^2*z -      b^4*c^2*x^2*z - a^2*b^4*y^2*z + a^4*c^2*y^2*z + a^2*b^4*x*z^2 - b^2*c^4*x*z^2 - a^4*b^2*y*z^2 +      a^2*c^4*y*z^2");
		exp("K355","(-a^4)*b^2*x^2*y + a^2*b^4*x^2*y + b^4*c^2*x^2*y - b^2*c^4*x^2*y - a^4*b^2*x*y^2 +      a^2*b^4*x*y^2 - a^4*c^2*x*y^2 + a^2*c^4*x*y^2 + a^4*c^2*x^2*z + b^4*c^2*x^2*z - a^2*c^4*x^2*z -      b^2*c^4*x^2*z - a^4*c^2*y^2*z - b^4*c^2*y^2*z + a^2*c^4*y^2*z + b^2*c^4*y^2*z + a^4*b^2*x*z^2 -      a^2*b^4*x*z^2 + a^4*c^2*x*z^2 - a^2*c^4*x*z^2 + a^4*b^2*y*z^2 - a^2*b^4*y*z^2 - b^4*c^2*y*z^2 +      b^2*c^4*y*z^2");
		exp("K356","a^4*b^2*x^2*y - a^2*c^4*x^2*y - a^2*b^4*x*y^2 + b^2*c^4*x*y^2 + a^2*b^4*x^2*z - a^4*c^2*x^2*z -      a^4*b^2*y^2*z + b^4*c^2*y^2*z - b^4*c^2*x*z^2 + a^2*c^4*x*z^2 + a^4*c^2*y*z^2 - b^2*c^4*y*z^2");
		exp("K357","(-a^4)*c^2*x^2*y - b^4*c^2*x^2*y + a^2*c^4*x^2*y + b^2*c^4*x^2*y + a^4*c^2*x*y^2 +      b^4*c^2*x*y^2 - a^2*c^4*x*y^2 - b^2*c^4*x*y^2 + a^4*b^2*x^2*z - a^2*b^4*x^2*z - b^4*c^2*x^2*z +      b^2*c^4*x^2*z + a^4*b^2*y^2*z - a^2*b^4*y^2*z + a^4*c^2*y^2*z - a^2*c^4*y^2*z - a^4*b^2*x*z^2 +      a^2*b^4*x*z^2 + b^4*c^2*x*z^2 - b^2*c^4*x*z^2 - a^4*b^2*y*z^2 + a^2*b^4*y*z^2 - a^4*c^2*y*z^2 +      a^2*c^4*y*z^2");
		exp("K358","2*a^4*c^2*x^2*y - 3*a^2*b^2*c^2*x^2*y + b^4*c^2*x^2*y - 2*a^2*c^4*x^2*y - b^2*c^4*x^2*y -      a^4*c^2*x*y^2 + 3*a^2*b^2*c^2*x*y^2 - 2*b^4*c^2*x*y^2 + a^2*c^4*x*y^2 + 2*b^2*c^4*x*y^2 -      2*a^4*b^2*x^2*z + 2*a^2*b^4*x^2*z + 3*a^2*b^2*c^2*x^2*z + b^4*c^2*x^2*z - b^2*c^4*x^2*z -      4*a^4*b^2*x*y*z + 4*a^2*b^4*x*y*z + 4*a^4*c^2*x*y*z - 4*b^4*c^2*x*y*z - 4*a^2*c^4*x*y*z +      4*b^2*c^4*x*y*z - 2*a^4*b^2*y^2*z + 2*a^2*b^4*y^2*z - a^4*c^2*y^2*z - 3*a^2*b^2*c^2*y^2*z +      a^2*c^4*y^2*z + a^4*b^2*x*z^2 - a^2*b^4*x*z^2 - 3*a^2*b^2*c^2*x*z^2 - 2*b^4*c^2*x*z^2 +      2*b^2*c^4*x*z^2 + a^4*b^2*y*z^2 - a^2*b^4*y*z^2 + 2*a^4*c^2*y*z^2 + 3*a^2*b^2*c^2*y*z^2 -      2*a^2*c^4*y*z^2");
		exp("K359","a^3*b^2*c^2*x^2*y - a^2*b^3*c^2*x^2*y - a^2*b^2*c^3*x^2*y + a*b^2*c^4*x^2*y + b^3*c^4*x^2*y -      b^2*c^5*x^2*y + a^3*b^2*c^2*x*y^2 - a^2*b^3*c^2*x*y^2 + a^2*b^2*c^3*x*y^2 - a^3*c^4*x*y^2 -      a^2*b*c^4*x*y^2 + a^2*c^5*x*y^2 - a^3*b^2*c^2*x^2*z + a^2*b^3*c^2*x^2*z - a*b^4*c^2*x^2*z +      b^5*c^2*x^2*z + a^2*b^2*c^3*x^2*z - b^4*c^3*x^2*z - 2*a^4*b^2*c*x*y*z + 2*a^2*b^4*c*x*y*z +      2*a^4*b*c^2*x*y*z - 2*a*b^4*c^2*x*y*z - 2*a^2*b*c^4*x*y*z + 2*a*b^2*c^4*x*y*z - a^5*c^2*y^2*z +      a^4*b*c^2*y^2*z - a^3*b^2*c^2*y^2*z + a^2*b^3*c^2*y^2*z + a^4*c^3*y^2*z - a^2*b^2*c^3*y^2*z +      a^3*b^4*x*z^2 - a^2*b^5*x*z^2 + a^2*b^4*c*x*z^2 - a^3*b^2*c^2*x*z^2 - a^2*b^3*c^2*x*z^2 +      a^2*b^2*c^3*x*z^2 + a^5*b^2*y*z^2 - a^4*b^3*y*z^2 - a^4*b^2*c*y*z^2 + a^3*b^2*c^2*y*z^2 +      a^2*b^3*c^2*y*z^2 - a^2*b^2*c^3*y*z^2");
		exp("K360","(-a)*c^2*x^2*y + b*c^2*x^2*y + c^3*x^2*y - a*c^2*x*y^2 + b*c^2*x*y^2 - c^3*x*y^2 + a*b^2*x^2*z -      b^3*x^2*z - b^2*c*x^2*z - 2*a^2*b*x*y*z + 2*a*b^2*x*y*z + 2*a^2*c*x*y*z - 2*b^2*c*x*y*z -      2*a*c^2*x*y*z + 2*b*c^2*x*y*z + a^3*y^2*z - a^2*b*y^2*z + a^2*c*y^2*z + a*b^2*x*z^2 + b^3*x*z^2 -      b^2*c*x*z^2 - a^3*y*z^2 - a^2*b*y*z^2 + a^2*c*y*z^2");
		exp("K361","(-a^2)*b^2*c^2*x^2*y + b^4*c^2*x^2*y - a^2*c^4*x^2*y - 2*b^2*c^4*x^2*y + c^6*x^2*y -      a^4*c^2*x*y^2 + a^2*b^2*c^2*x*y^2 + 2*a^2*c^4*x*y^2 + b^2*c^4*x*y^2 - c^6*x*y^2 + a^2*b^4*x^2*z -      b^6*x^2*z + a^2*b^2*c^2*x^2*z + 2*b^4*c^2*x^2*z - b^2*c^4*x^2*z + 2*a^4*b^2*x*y*z -      2*a^2*b^4*x*y*z - 2*a^4*c^2*x*y*z + 2*b^4*c^2*x*y*z + 2*a^2*c^4*x*y*z - 2*b^2*c^4*x*y*z +      a^6*y^2*z - a^4*b^2*y^2*z - 2*a^4*c^2*y^2*z - a^2*b^2*c^2*y^2*z + a^2*c^4*y^2*z + a^4*b^2*x*z^2 -      2*a^2*b^4*x*z^2 + b^6*x*z^2 - a^2*b^2*c^2*x*z^2 - b^4*c^2*x*z^2 - a^6*y*z^2 + 2*a^4*b^2*y*z^2 -      a^2*b^4*y*z^2 + a^4*c^2*y*z^2 + a^2*b^2*c^2*y*z^2");
		exp("K362","a*b*c^3*x^2*y + b^2*c^3*x^2*y - a^2*c^3*x*y^2 - a*b*c^3*x*y^2 - a*b^3*c*x^2*z - b^3*c^2*x^2*z +      a^3*b*c*y^2*z + a^3*c^2*y^2*z + a^2*b^3*x*z^2 + a*b^3*c*x*z^2 - a^3*b^2*y*z^2 - a^3*b*c*y*z^2");
		exp("K363","-3*a^6*x^2*y + 9*a^4*b^2*x^2*y - 9*a^2*b^4*x^2*y + 3*b^6*x^2*y + 4*a^4*c^2*x^2*y -      4*a^2*b^2*c^2*x^2*y + a^2*c^4*x^2*y - b^2*c^4*x^2*y - 2*c^6*x^2*y - 3*a^6*x*y^2 + 9*a^4*b^2*x*y^2 -      9*a^2*b^4*x*y^2 + 3*b^6*x*y^2 + 4*a^2*b^2*c^2*x*y^2 - 4*b^4*c^2*x*y^2 + a^2*c^4*x*y^2 -      b^2*c^4*x*y^2 + 2*c^6*x*y^2 + 3*a^6*x^2*z - 4*a^4*b^2*x^2*z - a^2*b^4*x^2*z + 2*b^6*x^2*z -      9*a^4*c^2*x^2*z + 4*a^2*b^2*c^2*x^2*z + b^4*c^2*x^2*z + 9*a^2*c^4*x^2*z - 3*c^6*x^2*z -      2*a^6*y^2*z + a^4*b^2*y^2*z + 4*a^2*b^4*y^2*z - 3*b^6*y^2*z - a^4*c^2*y^2*z - 4*a^2*b^2*c^2*y^2*z +      9*b^4*c^2*y^2*z - 9*b^2*c^4*y^2*z + 3*c^6*y^2*z + 3*a^6*x*z^2 - a^2*b^4*x*z^2 - 2*b^6*x*z^2 -      9*a^4*c^2*x*z^2 - 4*a^2*b^2*c^2*x*z^2 + b^4*c^2*x*z^2 + 9*a^2*c^4*x*z^2 + 4*b^2*c^4*x*z^2 -      3*c^6*x*z^2 + 2*a^6*y*z^2 + a^4*b^2*y*z^2 - 3*b^6*y*z^2 - a^4*c^2*y*z^2 + 4*a^2*b^2*c^2*y*z^2 +      9*b^4*c^2*y*z^2 - 4*a^2*c^4*y*z^2 - 9*b^2*c^4*y*z^2 + 3*c^6*y*z^2");
		exp("K364","a^10*c^2*x^2*y - 2*a^8*b^2*c^2*x^2*y + 2*a^6*b^4*c^2*x^2*y - 4*a^4*b^6*c^2*x^2*y +      5*a^2*b^8*c^2*x^2*y - 2*b^10*c^2*x^2*y - 3*a^8*c^4*x^2*y + 2*a^6*b^2*c^4*x^2*y +      2*a^4*b^4*c^4*x^2*y - 6*a^2*b^6*c^4*x^2*y + 5*b^8*c^4*x^2*y + 2*a^6*c^6*x^2*y +      2*a^2*b^4*c^6*x^2*y - 4*b^6*c^6*x^2*y + 2*a^4*c^8*x^2*y + 2*a^2*b^2*c^8*x^2*y + 2*b^4*c^8*x^2*y -      3*a^2*c^10*x^2*y - 2*b^2*c^10*x^2*y + c^12*x^2*y + 2*a^10*c^2*x*y^2 - 5*a^8*b^2*c^2*x*y^2 +      4*a^6*b^4*c^2*x*y^2 - 2*a^4*b^6*c^2*x*y^2 + 2*a^2*b^8*c^2*x*y^2 - b^10*c^2*x*y^2 -      5*a^8*c^4*x*y^2 + 6*a^6*b^2*c^4*x*y^2 - 2*a^4*b^4*c^4*x*y^2 - 2*a^2*b^6*c^4*x*y^2 +      3*b^8*c^4*x*y^2 + 4*a^6*c^6*x*y^2 - 2*a^4*b^2*c^6*x*y^2 - 2*b^6*c^6*x*y^2 - 2*a^4*c^8*x*y^2 -      2*a^2*b^2*c^8*x*y^2 - 2*b^4*c^8*x*y^2 + 2*a^2*c^10*x*y^2 + 3*b^2*c^10*x*y^2 - c^12*x*y^2 -      a^10*b^2*x^2*z + 3*a^8*b^4*x^2*z - 2*a^6*b^6*x^2*z - 2*a^4*b^8*x^2*z + 3*a^2*b^10*x^2*z -      b^12*x^2*z + 2*a^8*b^2*c^2*x^2*z - 2*a^6*b^4*c^2*x^2*z - 2*a^2*b^8*c^2*x^2*z + 2*b^10*c^2*x^2*z -      2*a^6*b^2*c^4*x^2*z - 2*a^4*b^4*c^4*x^2*z - 2*a^2*b^6*c^4*x^2*z - 2*b^8*c^4*x^2*z +      4*a^4*b^2*c^6*x^2*z + 6*a^2*b^4*c^6*x^2*z + 4*b^6*c^6*x^2*z - 5*a^2*b^2*c^8*x^2*z -      5*b^4*c^8*x^2*z + 2*b^2*c^10*x^2*z + a^12*y^2*z - 3*a^10*b^2*y^2*z + 2*a^8*b^4*y^2*z +      2*a^6*b^6*y^2*z - 3*a^4*b^8*y^2*z + a^2*b^10*y^2*z - 2*a^10*c^2*y^2*z + 2*a^8*b^2*c^2*y^2*z +      2*a^4*b^6*c^2*y^2*z - 2*a^2*b^8*c^2*y^2*z + 2*a^8*c^4*y^2*z + 2*a^6*b^2*c^4*y^2*z +      2*a^4*b^4*c^4*y^2*z + 2*a^2*b^6*c^4*y^2*z - 4*a^6*c^6*y^2*z - 6*a^4*b^2*c^6*y^2*z -      4*a^2*b^4*c^6*y^2*z + 5*a^4*c^8*y^2*z + 5*a^2*b^2*c^8*y^2*z - 2*a^2*c^10*y^2*z - 2*a^10*b^2*x*z^2 +      5*a^8*b^4*x*z^2 - 4*a^6*b^6*x*z^2 + 2*a^4*b^8*x*z^2 - 2*a^2*b^10*x*z^2 + b^12*x*z^2 +      5*a^8*b^2*c^2*x*z^2 - 6*a^6*b^4*c^2*x*z^2 + 2*a^4*b^6*c^2*x*z^2 + 2*a^2*b^8*c^2*x*z^2 -      3*b^10*c^2*x*z^2 - 4*a^6*b^2*c^4*x*z^2 + 2*a^4*b^4*c^4*x*z^2 + 2*b^8*c^4*x*z^2 +      2*a^4*b^2*c^6*x*z^2 + 2*a^2*b^4*c^6*x*z^2 + 2*b^6*c^6*x*z^2 - 2*a^2*b^2*c^8*x*z^2 -      3*b^4*c^8*x*z^2 + b^2*c^10*x*z^2 - a^12*y*z^2 + 2*a^10*b^2*y*z^2 - 2*a^8*b^4*y*z^2 +      4*a^6*b^6*y*z^2 - 5*a^4*b^8*y*z^2 + 2*a^2*b^10*y*z^2 + 3*a^10*c^2*y*z^2 - 2*a^8*b^2*c^2*y*z^2 -      2*a^6*b^4*c^2*y*z^2 + 6*a^4*b^6*c^2*y*z^2 - 5*a^2*b^8*c^2*y*z^2 - 2*a^8*c^4*y*z^2 -      2*a^4*b^4*c^4*y*z^2 + 4*a^2*b^6*c^4*y*z^2 - 2*a^6*c^6*y*z^2 - 2*a^4*b^2*c^6*y*z^2 -      2*a^2*b^4*c^6*y*z^2 + 3*a^4*c^8*y*z^2 + 2*a^2*b^2*c^8*y*z^2 - a^2*c^10*y*z^2");
		exp("K365","(-a)*c*x^2*y + b*c*x^2*y + c^2*x^2*y - a*c*x*y^2 + b*c*x*y^2 - c^2*x*y^2 + a*b*x^2*z -      b^2*x^2*z - b*c*x^2*z + a^2*y^2*z - a*b*y^2*z + a*c*y^2*z + a*b*x*z^2 + b^2*x*z^2 - b*c*x*z^2 -      a^2*y*z^2 - a*b*y*z^2 + a*c*y*z^2");
		exp("K366","a^2*x^2*y + a*b*x^2*y - a*b*x*y^2 - b^2*x*y^2 - a^2*x^2*z - a*c*x^2*z + b^2*y^2*z + b*c*y^2*z +      a*c*x*z^2 + c^2*x*z^2 - b*c*y*z^2 - c^2*y*z^2");
		exp("K367","a^2*b^2*c^4*x^2*y - b^4*c^4*x^2*y - a^4*c^4*x*y^2 + a^2*b^2*c^4*x*y^2 + a^2*b^4*c^2*x^2*z -      b^4*c^4*x^2*z + a^4*b^2*c^2*y^2*z - a^4*c^4*y^2*z - a^4*b^4*x*z^2 + a^2*b^4*c^2*x*z^2 -      a^4*b^4*y*z^2 + a^4*b^2*c^2*y*z^2");
		exp("K368","a^2*b^2*c^4*x^2*y + b^4*c^4*x^2*y - a^4*c^4*x*y^2 - a^2*b^2*c^4*x*y^2 - a^2*b^4*c^2*x^2*z -      b^4*c^4*x^2*z + a^4*b^2*c^2*y^2*z + a^4*c^4*y^2*z + a^4*b^4*x*z^2 + a^2*b^4*c^2*x*z^2 -      a^4*b^4*y*z^2 - a^4*b^2*c^2*y*z^2");
		exp("K369","b^10*c^4*x^3 - 3*b^8*c^6*x^3 + 3*b^6*c^8*x^3 - b^4*c^10*x^3 - a^10*b^4*x^2*y + 4*a^8*b^6*x^2*y -      6*a^6*b^8*x^2*y + 4*a^4*b^10*x^2*y - a^2*b^12*x^2*y + 2*a^10*b^2*c^2*x^2*y - 5*a^8*b^4*c^2*x^2*y -      2*a^6*b^6*c^2*x^2*y + 12*a^4*b^8*c^2*x^2*y - 8*a^2*b^10*c^2*x^2*y + b^12*c^2*x^2*y -      a^10*c^4*x^2*y - 2*a^8*b^2*c^4*x^2*y + 19*a^6*b^4*c^4*x^2*y - 26*a^4*b^6*c^4*x^2*y +      3*a^2*b^8*c^4*x^2*y + 4*b^10*c^4*x^2*y + 3*a^8*c^6*x^2*y - 8*a^6*b^2*c^6*x^2*y +      a^4*b^4*c^6*x^2*y + 22*a^2*b^6*c^6*x^2*y - 9*b^8*c^6*x^2*y - 3*a^6*c^8*x^2*y +      8*a^4*b^2*c^8*x^2*y - 16*a^2*b^4*c^8*x^2*y + 2*b^6*c^8*x^2*y + a^4*c^10*x^2*y + 2*b^4*c^10*x^2*y +      a^12*b^2*x*y^2 - 4*a^10*b^4*x*y^2 + 6*a^8*b^6*x*y^2 - 4*a^6*b^8*x*y^2 + a^4*b^10*x*y^2 -      a^12*c^2*x*y^2 + 8*a^10*b^2*c^2*x*y^2 - 12*a^8*b^4*c^2*x*y^2 + 2*a^6*b^6*c^2*x*y^2 +      5*a^4*b^8*c^2*x*y^2 - 2*a^2*b^10*c^2*x*y^2 - 4*a^10*c^4*x*y^2 - 3*a^8*b^2*c^4*x*y^2 +      26*a^6*b^4*c^4*x*y^2 - 19*a^4*b^6*c^4*x*y^2 + 2*a^2*b^8*c^4*x*y^2 + b^10*c^4*x*y^2 +      9*a^8*c^6*x*y^2 - 22*a^6*b^2*c^6*x*y^2 - a^4*b^4*c^6*x*y^2 + 8*a^2*b^6*c^6*x*y^2 -      3*b^8*c^6*x*y^2 - 2*a^6*c^8*x*y^2 + 16*a^4*b^2*c^8*x*y^2 - 8*a^2*b^4*c^8*x*y^2 + 3*b^6*c^8*x*y^2 -      2*a^4*c^10*x*y^2 - b^4*c^10*x*y^2 - a^10*c^4*y^3 + 3*a^8*c^6*y^3 - 3*a^6*c^8*y^3 + a^4*c^10*y^3 +      a^10*b^4*x^2*z - 3*a^8*b^6*x^2*z + 3*a^6*b^8*x^2*z - a^4*b^10*x^2*z - 2*a^10*b^2*c^2*x^2*z +      2*a^8*b^4*c^2*x^2*z + 8*a^6*b^6*c^2*x^2*z - 8*a^4*b^8*c^2*x^2*z + a^10*c^4*x^2*z +      5*a^8*b^2*c^4*x^2*z - 19*a^6*b^4*c^4*x^2*z - a^4*b^6*c^4*x^2*z + 16*a^2*b^8*c^4*x^2*z -      2*b^10*c^4*x^2*z - 4*a^8*c^6*x^2*z + 2*a^6*b^2*c^6*x^2*z + 26*a^4*b^4*c^6*x^2*z -      22*a^2*b^6*c^6*x^2*z - 2*b^8*c^6*x^2*z + 6*a^6*c^8*x^2*z - 12*a^4*b^2*c^8*x^2*z -      3*a^2*b^4*c^8*x^2*z + 9*b^6*c^8*x^2*z - 4*a^4*c^10*x^2*z + 8*a^2*b^2*c^10*x^2*z -      4*b^4*c^10*x^2*z + a^2*c^12*x^2*z - b^2*c^12*x^2*z - 2*a^12*b^2*x*y*z + 10*a^8*b^6*x*y*z -      10*a^6*b^8*x*y*z + 2*a^2*b^12*x*y*z + 2*a^12*c^2*x*y*z - 12*a^8*b^4*c^2*x*y*z +      12*a^4*b^8*c^2*x*y*z - 2*b^12*c^2*x*y*z + 12*a^8*b^2*c^4*x*y*z - 12*a^2*b^8*c^4*x*y*z -      10*a^8*c^6*x*y*z + 10*b^8*c^6*x*y*z + 10*a^6*c^8*x*y*z - 12*a^4*b^2*c^8*x*y*z +      12*a^2*b^4*c^8*x*y*z - 10*b^6*c^8*x*y*z - 2*a^2*c^12*x*y*z + 2*b^2*c^12*x*y*z + a^10*b^4*y^2*z -      3*a^8*b^6*y^2*z + 3*a^6*b^8*y^2*z - a^4*b^10*y^2*z + 8*a^8*b^4*c^2*y^2*z - 8*a^6*b^6*c^2*y^2*z -      2*a^4*b^8*c^2*y^2*z + 2*a^2*b^10*c^2*y^2*z + 2*a^10*c^4*y^2*z - 16*a^8*b^2*c^4*y^2*z +      a^6*b^4*c^4*y^2*z + 19*a^4*b^6*c^4*y^2*z - 5*a^2*b^8*c^4*y^2*z - b^10*c^4*y^2*z + 2*a^8*c^6*y^2*z +      22*a^6*b^2*c^6*y^2*z - 26*a^4*b^4*c^6*y^2*z - 2*a^2*b^6*c^6*y^2*z + 4*b^8*c^6*y^2*z -      9*a^6*c^8*y^2*z + 3*a^4*b^2*c^8*y^2*z + 12*a^2*b^4*c^8*y^2*z - 6*b^6*c^8*y^2*z + 4*a^4*c^10*y^2*z -      8*a^2*b^2*c^10*y^2*z + 4*b^4*c^10*y^2*z + a^2*c^12*y^2*z - b^2*c^12*y^2*z + a^12*b^2*x*z^2 +      4*a^10*b^4*x*z^2 - 9*a^8*b^6*x*z^2 + 2*a^6*b^8*x*z^2 + 2*a^4*b^10*x*z^2 - a^12*c^2*x*z^2 -      8*a^10*b^2*c^2*x*z^2 + 3*a^8*b^4*c^2*x*z^2 + 22*a^6*b^6*c^2*x*z^2 - 16*a^4*b^8*c^2*x*z^2 +      4*a^10*c^4*x*z^2 + 12*a^8*b^2*c^4*x*z^2 - 26*a^6*b^4*c^4*x*z^2 + a^4*b^6*c^4*x*z^2 +      8*a^2*b^8*c^4*x*z^2 + b^10*c^4*x*z^2 - 6*a^8*c^6*x*z^2 - 2*a^6*b^2*c^6*x*z^2 +      19*a^4*b^4*c^6*x*z^2 - 8*a^2*b^6*c^6*x*z^2 - 3*b^8*c^6*x*z^2 + 4*a^6*c^8*x*z^2 -      5*a^4*b^2*c^8*x*z^2 - 2*a^2*b^4*c^8*x*z^2 + 3*b^6*c^8*x*z^2 - a^4*c^10*x*z^2 +      2*a^2*b^2*c^10*x*z^2 - b^4*c^10*x*z^2 - 2*a^10*b^4*y*z^2 - 2*a^8*b^6*y*z^2 + 9*a^6*b^8*y*z^2 -      4*a^4*b^10*y*z^2 - a^2*b^12*y*z^2 + 16*a^8*b^4*c^2*y*z^2 - 22*a^6*b^6*c^2*y*z^2 -      3*a^4*b^8*c^2*y*z^2 + 8*a^2*b^10*c^2*y*z^2 + b^12*c^2*y*z^2 - a^10*c^4*y*z^2 -      8*a^8*b^2*c^4*y*z^2 - a^6*b^4*c^4*y*z^2 + 26*a^4*b^6*c^4*y*z^2 - 12*a^2*b^8*c^4*y*z^2 -      4*b^10*c^4*y*z^2 + 3*a^8*c^6*y*z^2 + 8*a^6*b^2*c^6*y*z^2 - 19*a^4*b^4*c^6*y*z^2 +      2*a^2*b^6*c^6*y*z^2 + 6*b^8*c^6*y*z^2 - 3*a^6*c^8*y*z^2 + 2*a^4*b^2*c^8*y*z^2 +      5*a^2*b^4*c^8*y*z^2 - 4*b^6*c^8*y*z^2 + a^4*c^10*y*z^2 - 2*a^2*b^2*c^10*y*z^2 + b^4*c^10*y*z^2 +      a^10*b^4*z^3 - 3*a^8*b^6*z^3 + 3*a^6*b^8*z^3 - a^4*b^10*z^3");
		exp("K370","-11*a^4*b^2*x^2*y + 8*a^2*b^4*x^2*y - 5*b^6*x^2*y + 11*a^4*c^2*x^2*y + 6*a^2*b^2*c^2*x^2*y +     7*b^4*c^2*x^2*y - 14*a^2*c^4*x^2*y - 10*b^2*c^4*x^2*y + 8*c^6*x^2*y + 5*a^6*x*y^2 -     8*a^4*b^2*x*y^2 + 11*a^2*b^4*x*y^2 - 7*a^4*c^2*x*y^2 - 6*a^2*b^2*c^2*x*y^2 - 11*b^4*c^2*x*y^2 +     10*a^2*c^4*x*y^2 + 14*b^2*c^4*x*y^2 - 8*c^6*x*y^2 - 11*a^4*b^2*x^2*z + 14*a^2*b^4*x^2*z -     8*b^6*x^2*z + 11*a^4*c^2*x^2*z - 6*a^2*b^2*c^2*x^2*z + 10*b^4*c^2*x^2*z - 8*a^2*c^4*x^2*z -     7*b^2*c^4*x^2*z + 5*c^6*x^2*z + 8*a^6*y^2*z - 14*a^4*b^2*y^2*z + 11*a^2*b^4*y^2*z -     10*a^4*c^2*y^2*z + 6*a^2*b^2*c^2*y^2*z - 11*b^4*c^2*y^2*z + 7*a^2*c^4*y^2*z + 8*b^2*c^4*y^2*z -     5*c^6*y^2*z - 5*a^6*x*z^2 + 7*a^4*b^2*x*z^2 - 10*a^2*b^4*x*z^2 + 8*b^6*x*z^2 + 8*a^4*c^2*x*z^2 +     6*a^2*b^2*c^2*x*z^2 - 14*b^4*c^2*x*z^2 - 11*a^2*c^4*x*z^2 + 11*b^2*c^4*x*z^2 - 8*a^6*y*z^2 +     10*a^4*b^2*y*z^2 - 7*a^2*b^4*y*z^2 + 5*b^6*y*z^2 + 14*a^4*c^2*y*z^2 - 6*a^2*b^2*c^2*y*z^2 -     8*b^4*c^2*y*z^2 - 11*a^2*c^4*y*z^2 + 11*b^2*c^4*y*z^2");
		exp("K371","4*a^2*b^2*x^2*y - 5*a^2*c^2*x^2*y + 4*b^2*c^2*x^2*y - 4*a^2*b^2*x*y^2 - 4*a^2*c^2*x*y^2 +      5*b^2*c^2*x*y^2 + 5*a^2*b^2*x^2*z - 4*a^2*c^2*x^2*z - 4*b^2*c^2*x^2*z - 5*a^2*b^2*y^2*z +      4*a^2*c^2*y^2*z + 4*b^2*c^2*y^2*z + 4*a^2*b^2*x*z^2 + 4*a^2*c^2*x*z^2 - 5*b^2*c^2*x*z^2 -      4*a^2*b^2*y*z^2 + 5*a^2*c^2*y*z^2 - 4*b^2*c^2*y*z^2");
		exp("K372","a^2*c*x^2*y - 2*a*b*c*x^2*y - 3*b^2*c*x^2*y + a*c^2*x^2*y + b*c^2*x^2*y + 3*a^2*c*x*y^2 +      2*a*b*c*x*y^2 - b^2*c*x*y^2 - a*c^2*x*y^2 - b*c^2*x*y^2 - a^2*b*x^2*z - a*b^2*x^2*z +      2*a*b*c*x^2*z - b^2*c*x^2*z + 3*b*c^2*x^2*z + a^2*b*y^2*z + a*b^2*y^2*z + a^2*c*y^2*z -      2*a*b*c*y^2*z - 3*a*c^2*y^2*z - 3*a^2*b*x*z^2 + a*b^2*x*z^2 - 2*a*b*c*x*z^2 + b^2*c*x*z^2 +      b*c^2*x*z^2 - a^2*b*y*z^2 + 3*a*b^2*y*z^2 - a^2*c*y*z^2 + 2*a*b*c*y*z^2 - a*c^2*y*z^2");
		exp("K373","(-a^4)*b^4*c^4*x^2*y + b^8*c^4*x^2*y - a^4*b^2*c^6*x^2*y - 2*a^2*b^4*c^6*x^2*y -      3*b^6*c^6*x^2*y + 2*a^2*b^2*c^8*x^2*y + 3*b^4*c^8*x^2*y - b^2*c^10*x^2*y - a^8*c^4*x*y^2 +      a^4*b^4*c^4*x*y^2 + 3*a^6*c^6*x*y^2 + 2*a^4*b^2*c^6*x*y^2 + a^2*b^4*c^6*x*y^2 - 3*a^4*c^8*x*y^2 -      2*a^2*b^2*c^8*x*y^2 + a^2*c^10*x*y^2 + a^4*b^6*c^2*x^2*z - 2*a^2*b^8*c^2*x^2*z + b^10*c^2*x^2*z +      a^4*b^4*c^4*x^2*z + 2*a^2*b^6*c^4*x^2*z - 3*b^8*c^4*x^2*z + 3*b^6*c^6*x^2*z - b^4*c^8*x^2*z -      a^10*c^2*y^2*z + 2*a^8*b^2*c^2*y^2*z - a^6*b^4*c^2*y^2*z + 3*a^8*c^4*y^2*z - 2*a^6*b^2*c^4*y^2*z -      a^4*b^4*c^4*y^2*z - 3*a^6*c^6*y^2*z + a^4*c^8*y^2*z + a^8*b^4*x*z^2 - 3*a^6*b^6*x*z^2 +      3*a^4*b^8*x*z^2 - a^2*b^10*x*z^2 - 2*a^4*b^6*c^2*x*z^2 + 2*a^2*b^8*c^2*x*z^2 - a^4*b^4*c^4*x*z^2 -      a^2*b^6*c^4*x*z^2 + a^10*b^2*y*z^2 - 3*a^8*b^4*y*z^2 + 3*a^6*b^6*y*z^2 - a^4*b^8*y*z^2 -      2*a^8*b^2*c^2*y*z^2 + 2*a^6*b^4*c^2*y*z^2 + a^6*b^2*c^4*y*z^2 + a^4*b^4*c^4*y*z^2");
		exp("K374","(-a^12)*b^2*c^4*x^2*y + 3*a^8*b^6*c^4*x^2*y - a^6*b^8*c^4*x^2*y - 3*a^2*b^12*c^4*x^2*y +      2*b^14*c^4*x^2*y + 3*a^10*b^2*c^6*x^2*y - 3*a^8*b^4*c^6*x^2*y - 3*a^6*b^6*c^6*x^2*y +      3*a^4*b^8*c^6*x^2*y + 9*a^2*b^10*c^6*x^2*y - 9*b^12*c^6*x^2*y - 3*a^8*b^2*c^8*x^2*y +      3*a^6*b^4*c^8*x^2*y - 6*a^4*b^6*c^8*x^2*y - 12*a^2*b^8*c^8*x^2*y + 15*b^10*c^8*x^2*y +      2*a^6*b^2*c^10*x^2*y + 6*a^4*b^4*c^10*x^2*y + 12*a^2*b^6*c^10*x^2*y - 10*b^8*c^10*x^2*y -      3*a^4*b^2*c^12*x^2*y - 9*a^2*b^4*c^12*x^2*y + 3*a^2*b^2*c^14*x^2*y + 3*b^4*c^14*x^2*y -      b^2*c^16*x^2*y - 2*a^14*c^4*x*y^2 + 3*a^12*b^2*c^4*x*y^2 + a^8*b^6*c^4*x*y^2 -      3*a^6*b^8*c^4*x*y^2 + a^2*b^12*c^4*x*y^2 + 9*a^12*c^6*x*y^2 - 9*a^10*b^2*c^6*x*y^2 -      3*a^8*b^4*c^6*x*y^2 + 3*a^6*b^6*c^6*x*y^2 + 3*a^4*b^8*c^6*x*y^2 - 3*a^2*b^10*c^6*x*y^2 -      15*a^10*c^8*x*y^2 + 12*a^8*b^2*c^8*x*y^2 + 6*a^6*b^4*c^8*x*y^2 - 3*a^4*b^6*c^8*x*y^2 +      3*a^2*b^8*c^8*x*y^2 + 10*a^8*c^10*x*y^2 - 12*a^6*b^2*c^10*x*y^2 - 6*a^4*b^4*c^10*x*y^2 -      2*a^2*b^6*c^10*x*y^2 + 9*a^4*b^2*c^12*x*y^2 + 3*a^2*b^4*c^12*x*y^2 - 3*a^4*c^14*x*y^2 -      3*a^2*b^2*c^14*x*y^2 + a^2*c^16*x*y^2 + a^12*b^4*c^2*x^2*z - 3*a^10*b^6*c^2*x^2*z +      3*a^8*b^8*c^2*x^2*z - 2*a^6*b^10*c^2*x^2*z + 3*a^4*b^12*c^2*x^2*z - 3*a^2*b^14*c^2*x^2*z +      b^16*c^2*x^2*z + 3*a^8*b^6*c^4*x^2*z - 3*a^6*b^8*c^4*x^2*z - 6*a^4*b^10*c^4*x^2*z +      9*a^2*b^12*c^4*x^2*z - 3*b^14*c^4*x^2*z - 3*a^8*b^4*c^6*x^2*z + 3*a^6*b^6*c^6*x^2*z +      6*a^4*b^8*c^6*x^2*z - 12*a^2*b^10*c^6*x^2*z + a^6*b^4*c^8*x^2*z - 3*a^4*b^6*c^8*x^2*z +      12*a^2*b^8*c^8*x^2*z + 10*b^10*c^8*x^2*z - 9*a^2*b^6*c^10*x^2*z - 15*b^8*c^10*x^2*z +      3*a^2*b^4*c^12*x^2*z + 9*b^6*c^12*x^2*z - 2*b^4*c^14*x^2*z - a^16*c^2*y^2*z +      3*a^14*b^2*c^2*y^2*z - 3*a^12*b^4*c^2*y^2*z + 2*a^10*b^6*c^2*y^2*z - 3*a^8*b^8*c^2*y^2*z +      3*a^6*b^10*c^2*y^2*z - a^4*b^12*c^2*y^2*z + 3*a^14*c^4*y^2*z - 9*a^12*b^2*c^4*y^2*z +      6*a^10*b^4*c^4*y^2*z + 3*a^8*b^6*c^4*y^2*z - 3*a^6*b^8*c^4*y^2*z + 12*a^10*b^2*c^6*y^2*z -      6*a^8*b^4*c^6*y^2*z - 3*a^6*b^6*c^6*y^2*z + 3*a^4*b^8*c^6*y^2*z - 10*a^10*c^8*y^2*z -      12*a^8*b^2*c^8*y^2*z + 3*a^6*b^4*c^8*y^2*z - a^4*b^6*c^8*y^2*z + 15*a^8*c^10*y^2*z +      9*a^6*b^2*c^10*y^2*z - 9*a^6*c^12*y^2*z - 3*a^4*b^2*c^12*y^2*z + 2*a^4*c^14*y^2*z +      2*a^14*b^4*x*z^2 - 9*a^12*b^6*x*z^2 + 15*a^10*b^8*x*z^2 - 10*a^8*b^10*x*z^2 + 3*a^4*b^14*x*z^2 -      a^2*b^16*x*z^2 - 3*a^12*b^4*c^2*x*z^2 + 9*a^10*b^6*c^2*x*z^2 - 12*a^8*b^8*c^2*x*z^2 +      12*a^6*b^10*c^2*x*z^2 - 9*a^4*b^12*c^2*x*z^2 + 3*a^2*b^14*c^2*x*z^2 + 3*a^8*b^6*c^4*x*z^2 -      6*a^6*b^8*c^4*x*z^2 + 6*a^4*b^10*c^4*x*z^2 - 3*a^2*b^12*c^4*x*z^2 - a^8*b^4*c^6*x*z^2 -      3*a^6*b^6*c^6*x*z^2 + 3*a^4*b^8*c^6*x*z^2 + 2*a^2*b^10*c^6*x*z^2 + 3*a^6*b^4*c^8*x*z^2 -      3*a^4*b^6*c^8*x*z^2 - 3*a^2*b^8*c^8*x*z^2 + 3*a^2*b^6*c^10*x*z^2 - a^2*b^4*c^12*x*z^2 +      a^16*b^2*y*z^2 - 3*a^14*b^4*y*z^2 + 10*a^10*b^8*y*z^2 - 15*a^8*b^10*y*z^2 + 9*a^6*b^12*y*z^2 -      2*a^4*b^14*y*z^2 - 3*a^14*b^2*c^2*y*z^2 + 9*a^12*b^4*c^2*y*z^2 - 12*a^10*b^6*c^2*y*z^2 +      12*a^8*b^8*c^2*y*z^2 - 9*a^6*b^10*c^2*y*z^2 + 3*a^4*b^12*c^2*y*z^2 + 3*a^12*b^2*c^4*y*z^2 -      6*a^10*b^4*c^4*y*z^2 + 6*a^8*b^6*c^4*y*z^2 - 3*a^6*b^8*c^4*y*z^2 - 2*a^10*b^2*c^6*y*z^2 -      3*a^8*b^4*c^6*y*z^2 + 3*a^6*b^6*c^6*y*z^2 + a^4*b^8*c^6*y*z^2 + 3*a^8*b^2*c^8*y*z^2 +      3*a^6*b^4*c^8*y*z^2 - 3*a^4*b^6*c^8*y*z^2 - 3*a^6*b^2*c^10*y*z^2 + a^4*b^2*c^12*y*z^2");
		exp("K375","a^8*c^2*x^2*y - a^6*b^2*c^2*x^2*y - a^4*b^4*c^2*x^2*y + a^2*b^6*c^2*x^2*y - a^6*c^4*x^2*y +      4*a^4*b^2*c^4*x^2*y - 3*a^2*b^4*c^4*x^2*y - 2*b^6*c^4*x^2*y - a^4*c^6*x^2*y - 3*a^2*b^2*c^6*x^2*y +      4*b^4*c^6*x^2*y + a^2*c^8*x^2*y - 2*b^2*c^8*x^2*y - a^6*b^2*c^2*x*y^2 + a^4*b^4*c^2*x*y^2 +      a^2*b^6*c^2*x*y^2 - b^8*c^2*x*y^2 + 2*a^6*c^4*x*y^2 + 3*a^4*b^2*c^4*x*y^2 - 4*a^2*b^4*c^4*x*y^2 +      b^6*c^4*x*y^2 - 4*a^4*c^6*x*y^2 + 3*a^2*b^2*c^6*x*y^2 + b^4*c^6*x*y^2 + 2*a^2*c^8*x*y^2 -      b^2*c^8*x*y^2 - a^8*b^2*x^2*z + a^6*b^4*x^2*z + a^4*b^6*x^2*z - a^2*b^8*x^2*z + a^6*b^2*c^2*x^2*z -      4*a^4*b^4*c^2*x^2*z + 3*a^2*b^6*c^2*x^2*z + 2*b^8*c^2*x^2*z + a^4*b^2*c^4*x^2*z +      3*a^2*b^4*c^4*x^2*z - 4*b^6*c^4*x^2*z - a^2*b^2*c^6*x^2*z + 2*b^4*c^6*x^2*z + a^8*b^2*y^2*z -      a^6*b^4*y^2*z - a^4*b^6*y^2*z + a^2*b^8*y^2*z - 2*a^8*c^2*y^2*z - 3*a^6*b^2*c^2*y^2*z +      4*a^4*b^4*c^2*y^2*z - a^2*b^6*c^2*y^2*z + 4*a^6*c^4*y^2*z - 3*a^4*b^2*c^4*y^2*z -      a^2*b^4*c^4*y^2*z - 2*a^4*c^6*y^2*z + a^2*b^2*c^6*y^2*z - 2*a^6*b^4*x*z^2 + 4*a^4*b^6*x*z^2 -      2*a^2*b^8*x*z^2 + a^6*b^2*c^2*x*z^2 - 3*a^4*b^4*c^2*x*z^2 - 3*a^2*b^6*c^2*x*z^2 + b^8*c^2*x*z^2 -      a^4*b^2*c^4*x*z^2 + 4*a^2*b^4*c^4*x*z^2 - b^6*c^4*x*z^2 - a^2*b^2*c^6*x*z^2 - b^4*c^6*x*z^2 +      b^2*c^8*x*z^2 + 2*a^8*b^2*y*z^2 - 4*a^6*b^4*y*z^2 + 2*a^4*b^6*y*z^2 - a^8*c^2*y*z^2 +      3*a^6*b^2*c^2*y*z^2 + 3*a^4*b^4*c^2*y*z^2 - a^2*b^6*c^2*y*z^2 + a^6*c^4*y*z^2 -      4*a^4*b^2*c^4*y*z^2 + a^2*b^4*c^4*y*z^2 + a^4*c^6*y*z^2 + a^2*b^2*c^6*y*z^2 - a^2*c^8*y*z^2");
		exp("K376","(-a^4)*c^2*x^2*y + 2*a^2*b^2*c^2*x^2*y - b^4*c^2*x^2*y + c^6*x^2*y + a^4*c^2*x*y^2 -      2*a^2*b^2*c^2*x*y^2 + b^4*c^2*x*y^2 - c^6*x*y^2 + a^4*b^2*x^2*z - b^6*x^2*z - 2*a^2*b^2*c^2*x^2*z +      b^2*c^4*x^2*z + 4*a^4*b^2*x*y*z - 4*a^2*b^4*x*y*z - 4*a^4*c^2*x*y*z + 4*b^4*c^2*x*y*z +      4*a^2*c^4*x*y*z - 4*b^2*c^4*x*y*z + a^6*y^2*z - a^2*b^4*y^2*z + 2*a^2*b^2*c^2*y^2*z -      a^2*c^4*y^2*z - a^4*b^2*x*z^2 + b^6*x*z^2 + 2*a^2*b^2*c^2*x*z^2 - b^2*c^4*x*z^2 - a^6*y*z^2 +      a^2*b^4*y*z^2 - 2*a^2*b^2*c^2*y*z^2 + a^2*c^4*y*z^2");
		exp("K377","(-a^8)*b^2*x^2*y + 2*a^6*b^4*x^2*y - 2*a^2*b^8*x^2*y + b^10*x^2*y - a^6*b^2*c^2*x^2*y +      2*a^4*b^4*c^2*x^2*y + a^2*b^6*c^2*x^2*y - 2*b^8*c^2*x^2*y + a^4*b^2*c^4*x^2*y +      2*a^2*b^4*c^4*x^2*y - a^2*b^2*c^6*x^2*y + 2*b^4*c^6*x^2*y - b^2*c^8*x^2*y - a^10*x*y^2 +      2*a^8*b^2*x*y^2 - 2*a^4*b^6*x*y^2 + a^2*b^8*x*y^2 + 2*a^8*c^2*x*y^2 - a^6*b^2*c^2*x*y^2 -      2*a^4*b^4*c^2*x*y^2 + a^2*b^6*c^2*x*y^2 - 2*a^4*b^2*c^4*x*y^2 - a^2*b^4*c^4*x*y^2 -      2*a^4*c^6*x*y^2 + a^2*b^2*c^6*x*y^2 + a^2*c^8*x*y^2 + a^8*c^2*x^2*z + a^6*b^2*c^2*x^2*z -      a^4*b^4*c^2*x^2*z + a^2*b^6*c^2*x^2*z + b^8*c^2*x^2*z - 2*a^6*c^4*x^2*z - 2*a^4*b^2*c^4*x^2*z -      2*a^2*b^4*c^4*x^2*z - 2*b^6*c^4*x^2*z - a^2*b^2*c^6*x^2*z + 2*a^2*c^8*x^2*z + 2*b^2*c^8*x^2*z -      c^10*x^2*z - a^8*c^2*y^2*z - a^6*b^2*c^2*y^2*z + a^4*b^4*c^2*y^2*z - a^2*b^6*c^2*y^2*z -      b^8*c^2*y^2*z + 2*a^6*c^4*y^2*z + 2*a^4*b^2*c^4*y^2*z + 2*a^2*b^4*c^4*y^2*z + 2*b^6*c^4*y^2*z +      a^2*b^2*c^6*y^2*z - 2*a^2*c^8*y^2*z - 2*b^2*c^8*y^2*z + c^10*y^2*z + a^10*x*z^2 - 2*a^8*b^2*x*z^2 +      2*a^4*b^6*x*z^2 - a^2*b^8*x*z^2 - 2*a^8*c^2*x*z^2 + a^6*b^2*c^2*x*z^2 + 2*a^4*b^4*c^2*x*z^2 -      a^2*b^6*c^2*x*z^2 + 2*a^4*b^2*c^4*x*z^2 + a^2*b^4*c^4*x*z^2 + 2*a^4*c^6*x*z^2 - a^2*b^2*c^6*x*z^2 -      a^2*c^8*x*z^2 + a^8*b^2*y*z^2 - 2*a^6*b^4*y*z^2 + 2*a^2*b^8*y*z^2 - b^10*y*z^2 +      a^6*b^2*c^2*y*z^2 - 2*a^4*b^4*c^2*y*z^2 - a^2*b^6*c^2*y*z^2 + 2*b^8*c^2*y*z^2 - a^4*b^2*c^4*y*z^2 -      2*a^2*b^4*c^4*y*z^2 + a^2*b^2*c^6*y*z^2 - 2*b^4*c^6*y*z^2 + b^2*c^8*y*z^2");
		exp("K378","a^2*b^4*c^6*x^2*y - b^6*c^6*x^2*y + a^2*b^2*c^8*x^2*y + 2*b^4*c^8*x^2*y - b^2*c^10*x^2*y +      a^6*c^6*x*y^2 - a^4*b^2*c^6*x*y^2 - 2*a^4*c^8*x*y^2 - a^2*b^2*c^8*x*y^2 + a^2*c^10*x*y^2 -      a^2*b^8*c^2*x^2*z + b^10*c^2*x^2*z - a^2*b^6*c^4*x^2*z - 2*b^8*c^4*x^2*z + b^6*c^6*x^2*z +      2*a^6*b^4*c^2*x*y*z - 2*a^4*b^6*c^2*x*y*z - 2*a^6*b^2*c^4*x*y*z + 2*a^2*b^6*c^4*x*y*z +      2*a^4*b^2*c^6*x*y*z - 2*a^2*b^4*c^6*x*y*z - a^10*c^2*y^2*z + a^8*b^2*c^2*y^2*z + 2*a^8*c^4*y^2*z +      a^6*b^2*c^4*y^2*z - a^6*c^6*y^2*z - a^6*b^6*x*z^2 + 2*a^4*b^8*x*z^2 - a^2*b^10*x*z^2 +      a^4*b^6*c^2*x*z^2 + a^2*b^8*c^2*x*z^2 + a^10*b^2*y*z^2 - 2*a^8*b^4*y*z^2 + a^6*b^6*y*z^2 -      a^8*b^2*c^2*y*z^2 - a^6*b^4*c^2*y*z^2");
		exp("K379","a*b^2*c^2*x^2*y + b^3*c^2*x^2*y + 2*a*b*c^3*x^2*y + b^2*c^3*x^2*y + a*c^4*x^2*y + b*c^4*x^2*y +      c^5*x^2*y - a^3*c^2*x*y^2 - a^2*b*c^2*x*y^2 - a^2*c^3*x*y^2 - 2*a*b*c^3*x*y^2 - a*c^4*x*y^2 -      b*c^4*x*y^2 - c^5*x*y^2 - a*b^4*x^2*z - b^5*x^2*z - 2*a*b^3*c*x^2*z - b^4*c*x^2*z -      a*b^2*c^2*x^2*z - b^3*c^2*x^2*z - b^2*c^3*x^2*z + a^5*y^2*z + a^4*b*y^2*z + a^4*c*y^2*z +      2*a^3*b*c*y^2*z + a^3*c^2*y^2*z + a^2*b*c^2*y^2*z + a^2*c^3*y^2*z + a^3*b^2*x*z^2 + a^2*b^3*x*z^2 +      a*b^4*x*z^2 + b^5*x*z^2 + a^2*b^2*c*x*z^2 + 2*a*b^3*c*x*z^2 + b^4*c*x*z^2 - a^5*y*z^2 -      a^4*b*y*z^2 - a^3*b^2*y*z^2 - a^2*b^3*y*z^2 - a^4*c*y*z^2 - 2*a^3*b*c*y*z^2 - a^2*b^2*c*y*z^2");
		exp("K380","(-a^2)*b^2*c^2*x^2*y + b^4*c^2*x^2*y - a^2*c^4*x^2*y + c^6*x^2*y - a^4*c^2*x*y^2 +      a^2*b^2*c^2*x*y^2 + b^2*c^4*x*y^2 - c^6*x*y^2 + a^2*b^4*x^2*z - b^6*x^2*z + a^2*b^2*c^2*x^2*z -      b^2*c^4*x^2*z + a^6*y^2*z - a^4*b^2*y^2*z - a^2*b^2*c^2*y^2*z + a^2*c^4*y^2*z + a^4*b^2*x*z^2 +      b^6*x*z^2 - a^2*b^2*c^2*x*z^2 - b^4*c^2*x*z^2 - a^6*y*z^2 - a^2*b^4*y*z^2 + a^4*c^2*y*z^2 +      a^2*b^2*c^2*y*z^2");
		exp("K381","(-a^2)*c^4*x^2*y + c^6*x^2*y + b^2*c^4*x*y^2 - c^6*x*y^2 + a^2*b^4*x^2*z - b^6*x^2*z -      2*a^4*b^2*x*y*z + 2*a^2*b^4*x*y*z + 2*a^4*c^2*x*y*z - 2*b^4*c^2*x*y*z - 2*a^2*c^4*x*y*z +      2*b^2*c^4*x*y*z + a^6*y^2*z - a^4*b^2*y^2*z + b^6*x*z^2 - b^4*c^2*x*z^2 - a^6*y*z^2 +      a^4*c^2*y*z^2");
		exp("K382","a^6*b*c^2*x^2*y - 2*a^5*b^2*c^2*x^2*y + a^4*b^3*c^2*x^2*y - a^2*b^5*c^2*x^2*y +      2*a*b^6*c^2*x^2*y - b^7*c^2*x^2*y - a^6*c^3*x^2*y + a^4*b^2*c^3*x^2*y - 3*a^2*b^4*c^3*x^2*y +      3*b^6*c^3*x^2*y + 2*a^5*c^4*x^2*y - 2*a^4*b*c^4*x^2*y + 2*a^3*b^2*c^4*x^2*y - 2*b^5*c^4*x^2*y +      2*a^2*b^2*c^5*x^2*y - 2*b^4*c^5*x^2*y - 2*a^3*c^6*x^2*y + a^2*b*c^6*x^2*y - 2*a*b^2*c^6*x^2*y +      3*b^3*c^6*x^2*y + a^2*c^7*x^2*y - b^2*c^7*x^2*y - a^7*c^2*x*y^2 + 2*a^6*b*c^2*x*y^2 -      a^5*b^2*c^2*x*y^2 + a^3*b^4*c^2*x*y^2 - 2*a^2*b^5*c^2*x*y^2 + a*b^6*c^2*x*y^2 + 3*a^6*c^3*x*y^2 -      3*a^4*b^2*c^3*x*y^2 + a^2*b^4*c^3*x*y^2 - b^6*c^3*x*y^2 - 2*a^5*c^4*x*y^2 + 2*a^2*b^3*c^4*x*y^2 -      2*a*b^4*c^4*x*y^2 + 2*b^5*c^4*x*y^2 - 2*a^4*c^5*x*y^2 + 2*a^2*b^2*c^5*x*y^2 + 3*a^3*c^6*x*y^2 -      2*a^2*b*c^6*x*y^2 + a*b^2*c^6*x*y^2 - 2*b^3*c^6*x*y^2 - a^2*c^7*x*y^2 + b^2*c^7*x*y^2 -      a^6*b^3*x^2*z + 2*a^5*b^4*x^2*z - 2*a^3*b^6*x^2*z + a^2*b^7*x^2*z + a^6*b^2*c*x^2*z -      2*a^4*b^4*c*x^2*z + a^2*b^6*c*x^2*z - 2*a^5*b^2*c^2*x^2*z + a^4*b^3*c^2*x^2*z +      2*a^3*b^4*c^2*x^2*z + 2*a^2*b^5*c^2*x^2*z - 2*a*b^6*c^2*x^2*z - b^7*c^2*x^2*z + a^4*b^2*c^3*x^2*z +      3*b^6*c^3*x^2*z - 3*a^2*b^3*c^4*x^2*z - 2*b^5*c^4*x^2*z - a^2*b^2*c^5*x^2*z - 2*b^4*c^5*x^2*z +      2*a*b^2*c^6*x^2*z + 3*b^3*c^6*x^2*z - b^2*c^7*x^2*z + a^7*b^2*x*y*z - 3*a^6*b^3*x*y*z +      2*a^5*b^4*x*y*z + 2*a^4*b^5*x*y*z - 3*a^3*b^6*x*y*z + a^2*b^7*x*y*z - 2*a^6*b^2*c*x*y*z +      4*a^4*b^4*c*x*y*z - 2*a^2*b^6*c*x*y*z + a^7*c^2*x*y*z - 2*a^6*b*c^2*x*y*z + 2*a^5*b^2*c^2*x*y*z -      a^4*b^3*c^2*x*y*z - a^3*b^4*c^2*x*y*z + 2*a^2*b^5*c^2*x*y*z - 2*a*b^6*c^2*x*y*z + b^7*c^2*x*y*z -      3*a^6*c^3*x*y*z - a^4*b^2*c^3*x*y*z - a^2*b^4*c^3*x*y*z - 3*b^6*c^3*x*y*z + 2*a^5*c^4*x*y*z +      4*a^4*b*c^4*x*y*z - a^3*b^2*c^4*x*y*z - a^2*b^3*c^4*x*y*z + 4*a*b^4*c^4*x*y*z + 2*b^5*c^4*x*y*z +      2*a^4*c^5*x*y*z + 2*a^2*b^2*c^5*x*y*z + 2*b^4*c^5*x*y*z - 3*a^3*c^6*x*y*z - 2*a^2*b*c^6*x*y*z -      2*a*b^2*c^6*x*y*z - 3*b^3*c^6*x*y*z + a^2*c^7*x*y*z + b^2*c^7*x*y*z + a^7*b^2*y^2*z -      2*a^6*b^3*y^2*z + 2*a^4*b^5*y^2*z - a^3*b^6*y^2*z + a^6*b^2*c*y^2*z - 2*a^4*b^4*c*y^2*z +      a^2*b^6*c*y^2*z - a^7*c^2*y^2*z - 2*a^6*b*c^2*y^2*z + 2*a^5*b^2*c^2*y^2*z + 2*a^4*b^3*c^2*y^2*z +      a^3*b^4*c^2*y^2*z - 2*a^2*b^5*c^2*y^2*z + 3*a^6*c^3*y^2*z + a^2*b^4*c^3*y^2*z - 2*a^5*c^4*y^2*z -      3*a^3*b^2*c^4*y^2*z - 2*a^4*c^5*y^2*z - a^2*b^2*c^5*y^2*z + 3*a^3*c^6*y^2*z + 2*a^2*b*c^6*y^2*z -      a^2*c^7*y^2*z - a^7*b^2*x*z^2 + 3*a^6*b^3*x*z^2 - 2*a^5*b^4*x*z^2 - 2*a^4*b^5*x*z^2 +      3*a^3*b^6*x*z^2 - a^2*b^7*x*z^2 + 2*a^6*b^2*c*x*z^2 - 2*a^2*b^6*c*x*z^2 - a^5*b^2*c^2*x*z^2 -      3*a^4*b^3*c^2*x*z^2 + 2*a^2*b^5*c^2*x*z^2 + a*b^6*c^2*x*z^2 + b^7*c^2*x*z^2 + 2*a^2*b^4*c^3*x*z^2 -      2*b^6*c^3*x*z^2 + a^3*b^2*c^4*x*z^2 + a^2*b^3*c^4*x*z^2 - 2*a*b^4*c^4*x*z^2 - 2*a^2*b^2*c^5*x*z^2 +      2*b^4*c^5*x*z^2 + a*b^2*c^6*x*z^2 - b^3*c^6*x*z^2 - a^7*b^2*y*z^2 + 3*a^6*b^3*y*z^2 -      2*a^5*b^4*y*z^2 - 2*a^4*b^5*y*z^2 + 3*a^3*b^6*y*z^2 - a^2*b^7*y*z^2 - 2*a^6*b^2*c*y*z^2 +      2*a^2*b^6*c*y*z^2 + a^7*c^2*y*z^2 + a^6*b*c^2*y*z^2 + 2*a^5*b^2*c^2*y*z^2 - 3*a^3*b^4*c^2*y*z^2 -      a^2*b^5*c^2*y*z^2 - 2*a^6*c^3*y*z^2 + 2*a^4*b^2*c^3*y*z^2 - 2*a^4*b*c^4*y*z^2 + a^3*b^2*c^4*y*z^2 +      a^2*b^3*c^4*y*z^2 + 2*a^4*c^5*y*z^2 - 2*a^2*b^2*c^5*y*z^2 - a^3*c^6*y*z^2 + a^2*b*c^6*y*z^2");
		exp("K383","(-a^4)*b*c^2*x^2*y + a^3*b^2*c^2*x^2*y - a^2*b^3*c^2*x^2*y - a*b^4*c^2*x^2*y + 2*b^5*c^2*x^2*y +      a^4*c^3*x^2*y - b^4*c^3*x^2*y - a^3*c^4*x^2*y + 2*a^2*b*c^4*x^2*y - b^3*c^4*x^2*y - a^2*c^5*x^2*y +      b^2*c^5*x^2*y + a*c^6*x^2*y - b*c^6*x^2*y + 2*a^5*c^2*x*y^2 - a^4*b*c^2*x*y^2 - a^3*b^2*c^2*x*y^2 +      a^2*b^3*c^2*x*y^2 - a*b^4*c^2*x*y^2 - a^4*c^3*x*y^2 + b^4*c^3*x*y^2 - a^3*c^4*x*y^2 +      2*a*b^2*c^4*x*y^2 - b^3*c^4*x*y^2 + a^2*c^5*x*y^2 - b^2*c^5*x*y^2 - a*c^6*x*y^2 + b*c^6*x*y^2 +      a^4*b^3*x^2*z - a^3*b^4*x^2*z - a^2*b^5*x^2*z + a*b^6*x^2*z - a^4*b^2*c*x^2*z + 2*a^2*b^4*c*x^2*z -      b^6*c*x^2*z + a^3*b^2*c^2*x^2*z + b^5*c^2*x^2*z - a^2*b^2*c^3*x^2*z - b^4*c^3*x^2*z -      a*b^2*c^4*x^2*z - b^3*c^4*x^2*z + 2*b^2*c^5*x^2*z - 2*a^5*b^2*x*y*z + 2*a^4*b^3*x*y*z +      2*a^3*b^4*x*y*z - 2*a^2*b^5*x*y*z - 2*a^5*c^2*x*y*z - 2*b^5*c^2*x*y*z + 2*a^4*c^3*x*y*z +      2*b^4*c^3*x*y*z + 2*a^3*c^4*x*y*z + 2*b^3*c^4*x*y*z - 2*a^2*c^5*x*y*z - 2*b^2*c^5*x*y*z +      a^6*b*y^2*z - a^5*b^2*y^2*z - a^4*b^3*y^2*z + a^3*b^4*y^2*z - a^6*c*y^2*z + 2*a^4*b^2*c*y^2*z -      a^2*b^4*c*y^2*z + a^5*c^2*y^2*z + a^2*b^3*c^2*y^2*z - a^4*c^3*y^2*z - a^2*b^2*c^3*y^2*z -      a^3*c^4*y^2*z - a^2*b*c^4*y^2*z + 2*a^2*c^5*y^2*z + 2*a^5*b^2*x*z^2 - a^4*b^3*x*z^2 -      a^3*b^4*x*z^2 + a^2*b^5*x*z^2 - a*b^6*x*z^2 - a^4*b^2*c*x*z^2 + b^6*c*x*z^2 - a^3*b^2*c^2*x*z^2 +      2*a*b^4*c^2*x*z^2 - b^5*c^2*x*z^2 + a^2*b^2*c^3*x*z^2 - b^4*c^3*x*z^2 - a*b^2*c^4*x*z^2 +      b^3*c^4*x*z^2 - a^6*b*y*z^2 + a^5*b^2*y*z^2 - a^4*b^3*y*z^2 - a^3*b^4*y*z^2 + 2*a^2*b^5*y*z^2 +      a^6*c*y*z^2 - a^2*b^4*c*y*z^2 - a^5*c^2*y*z^2 + 2*a^4*b*c^2*y*z^2 - a^2*b^3*c^2*y*z^2 -      a^4*c^3*y*z^2 + a^2*b^2*c^3*y*z^2 + a^3*c^4*y*z^2 - a^2*b*c^4*y*z^2");
		exp("K384","(-a^5)*b*c^2*x^2*y + 2*a^2*b^4*c^2*x^2*y + a*b^5*c^2*x^2*y - 2*b^6*c^2*x^2*y + a^5*c^3*x^2*y +      a^3*b^2*c^3*x^2*y - a^2*b^3*c^3*x^2*y - 2*a*b^4*c^3*x^2*y + b^5*c^3*x^2*y + a^3*b*c^4*x^2*y -      2*a^2*b^2*c^4*x^2*y - a*b^3*c^4*x^2*y + 2*b^4*c^4*x^2*y - 2*a^3*c^5*x^2*y + a^2*b*c^5*x^2*y +      a*b^2*c^5*x^2*y + a*c^7*x^2*y - b*c^7*x^2*y - 2*a^6*c^2*x*y^2 + a^5*b*c^2*x*y^2 +      2*a^4*b^2*c^2*x*y^2 - a*b^5*c^2*x*y^2 + a^5*c^3*x*y^2 - 2*a^4*b*c^3*x*y^2 - a^3*b^2*c^3*x*y^2 +      a^2*b^3*c^3*x*y^2 + b^5*c^3*x*y^2 + 2*a^4*c^4*x*y^2 - a^3*b*c^4*x*y^2 - 2*a^2*b^2*c^4*x*y^2 +      a*b^3*c^4*x*y^2 + a^2*b*c^5*x*y^2 + a*b^2*c^5*x*y^2 - 2*b^3*c^5*x*y^2 - a*c^7*x*y^2 + b*c^7*x*y^2 +      a^5*b^3*x^2*z - 2*a^3*b^5*x^2*z + a*b^7*x^2*z - a^5*b^2*c*x^2*z + a^3*b^4*c*x^2*z +      a^2*b^5*c*x^2*z - b^7*c*x^2*z + a^3*b^3*c^2*x^2*z - 2*a^2*b^4*c^2*x^2*z + a*b^5*c^2*x^2*z -      a^2*b^3*c^3*x^2*z - a*b^4*c^3*x^2*z + 2*a^2*b^2*c^4*x^2*z - 2*a*b^3*c^4*x^2*z + 2*b^4*c^4*x^2*z +      a*b^2*c^5*x^2*z + b^3*c^5*x^2*z - 2*b^2*c^6*x^2*z + 2*a^6*b^2*x*y*z - 4*a^4*b^4*x*y*z +      2*a^2*b^6*x*y*z - 2*a^5*b^2*c*x*y*z + 2*a^4*b^3*c*x*y*z + 2*a^3*b^4*c*x*y*z - 2*a^2*b^5*c*x*y*z +      2*a^6*c^2*x*y*z - 2*a^5*b*c^2*x*y*z - 2*a*b^5*c^2*x*y*z + 2*b^6*c^2*x*y*z + 2*a^4*b*c^3*x*y*z +      2*a*b^4*c^3*x*y*z - 4*a^4*c^4*x*y*z + 2*a^3*b*c^4*x*y*z + 2*a*b^3*c^4*x*y*z - 4*b^4*c^4*x*y*z -      2*a^2*b*c^5*x*y*z - 2*a*b^2*c^5*x*y*z + 2*a^2*c^6*x*y*z + 2*b^2*c^6*x*y*z + a^7*b*y^2*z -      2*a^5*b^3*y^2*z + a^3*b^5*y^2*z - a^7*c*y^2*z + a^5*b^2*c*y^2*z + a^4*b^3*c*y^2*z -      a^2*b^5*c*y^2*z + a^5*b*c^2*y^2*z - 2*a^4*b^2*c^2*y^2*z + a^3*b^3*c^2*y^2*z - a^4*b*c^3*y^2*z -      a^3*b^2*c^3*y^2*z + 2*a^4*c^4*y^2*z - 2*a^3*b*c^4*y^2*z + 2*a^2*b^2*c^4*y^2*z + a^3*c^5*y^2*z +      a^2*b*c^5*y^2*z - 2*a^2*c^6*y^2*z - 2*a^6*b^2*x*z^2 + a^5*b^3*x*z^2 + 2*a^4*b^4*x*z^2 -      a*b^7*x*z^2 + a^5*b^2*c*x*z^2 - 2*a^4*b^3*c*x*z^2 - a^3*b^4*c*x*z^2 + a^2*b^5*c*x*z^2 +      b^7*c*x*z^2 + 2*a^4*b^2*c^2*x*z^2 - a^3*b^3*c^2*x*z^2 - 2*a^2*b^4*c^2*x*z^2 + a*b^5*c^2*x*z^2 +      a^2*b^3*c^3*x*z^2 + a*b^4*c^3*x*z^2 - 2*b^5*c^3*x*z^2 - a*b^2*c^5*x*z^2 + b^3*c^5*x*z^2 -      a^7*b*y*z^2 + 2*a^4*b^4*y*z^2 + a^3*b^5*y*z^2 - 2*a^2*b^6*y*z^2 + a^7*c*y*z^2 + a^5*b^2*c*y*z^2 -      a^4*b^3*c*y*z^2 - 2*a^3*b^4*c*y*z^2 + a^2*b^5*c*y*z^2 + a^5*b*c^2*y*z^2 - 2*a^4*b^2*c^2*y*z^2 -      a^3*b^3*c^2*y*z^2 + 2*a^2*b^4*c^2*y*z^2 - 2*a^5*c^3*y*z^2 + a^4*b*c^3*y*z^2 + a^3*b^2*c^3*y*z^2 +      a^3*c^5*y*z^2 - a^2*b*c^5*y*z^2");
		exp("K385","(-a^8)*b^2*c^2*x^2*y + 2*a^7*b^3*c^2*x^2*y - 2*a^5*b^5*c^2*x^2*y + 2*a^4*b^6*c^2*x^2*y -      2*a^3*b^7*c^2*x^2*y + 2*a*b^9*c^2*x^2*y - b^10*c^2*x^2*y - 2*a^5*b^4*c^3*x^2*y +      2*a^4*b^5*c^3*x^2*y + 4*a^3*b^6*c^3*x^2*y - 4*a^2*b^7*c^3*x^2*y - 2*a*b^8*c^3*x^2*y +      2*b^9*c^3*x^2*y + a^8*c^4*x^2*y + a^6*b^2*c^4*x^2*y - a^4*b^4*c^4*x^2*y + 4*a^3*b^5*c^4*x^2*y -      a^2*b^6*c^4*x^2*y - 4*a*b^7*c^4*x^2*y - 2*a^7*c^5*x^2*y - 2*a^4*b^3*c^5*x^2*y -      2*a^3*b^4*c^5*x^2*y + 4*a^2*b^5*c^5*x^2*y + 4*a*b^6*c^5*x^2*y - 2*b^7*c^5*x^2*y - a^6*c^6*x^2*y -      2*a^3*b^3*c^6*x^2*y - a^2*b^4*c^6*x^2*y + 2*a*b^5*c^6*x^2*y + 2*b^6*c^6*x^2*y + 4*a^5*c^7*x^2*y -      2*a*b^4*c^7*x^2*y - 2*b^5*c^7*x^2*y - a^4*c^8*x^2*y + a^2*b^2*c^8*x^2*y - 2*a^3*c^9*x^2*y +      2*b^3*c^9*x^2*y + a^2*c^10*x^2*y - b^2*c^10*x^2*y - a^10*c^2*x*y^2 + 2*a^9*b*c^2*x*y^2 -      2*a^7*b^3*c^2*x*y^2 + 2*a^6*b^4*c^2*x*y^2 - 2*a^5*b^5*c^2*x*y^2 + 2*a^3*b^7*c^2*x*y^2 -      a^2*b^8*c^2*x*y^2 + 2*a^9*c^3*x*y^2 - 2*a^8*b*c^3*x*y^2 - 4*a^7*b^2*c^3*x*y^2 +      4*a^6*b^3*c^3*x*y^2 + 2*a^5*b^4*c^3*x*y^2 - 2*a^4*b^5*c^3*x*y^2 - 4*a^7*b*c^4*x*y^2 -      a^6*b^2*c^4*x*y^2 + 4*a^5*b^3*c^4*x*y^2 - a^4*b^4*c^4*x*y^2 + a^2*b^6*c^4*x*y^2 + b^8*c^4*x*y^2 -      2*a^7*c^5*x*y^2 + 4*a^6*b*c^5*x*y^2 + 4*a^5*b^2*c^5*x*y^2 - 2*a^4*b^3*c^5*x*y^2 -      2*a^3*b^4*c^5*x*y^2 - 2*b^7*c^5*x*y^2 + 2*a^6*c^6*x*y^2 + 2*a^5*b*c^6*x*y^2 - a^4*b^2*c^6*x*y^2 -      2*a^3*b^3*c^6*x*y^2 - b^6*c^6*x*y^2 - 2*a^5*c^7*x*y^2 - 2*a^4*b*c^7*x*y^2 + 4*b^5*c^7*x*y^2 +      a^2*b^2*c^8*x*y^2 - b^4*c^8*x*y^2 + 2*a^3*c^9*x*y^2 - 2*b^3*c^9*x*y^2 - a^2*c^10*x*y^2 +      b^2*c^10*x*y^2 + a^8*b^4*x^2*z - 2*a^7*b^5*x^2*z - a^6*b^6*x^2*z + 4*a^5*b^7*x^2*z -      a^4*b^8*x^2*z - 2*a^3*b^9*x^2*z + a^2*b^10*x^2*z - a^8*b^2*c^2*x^2*z + a^6*b^4*c^2*x^2*z +      a^2*b^8*c^2*x^2*z - b^10*c^2*x^2*z + 2*a^7*b^2*c^3*x^2*z - 2*a^4*b^5*c^3*x^2*z -      2*a^3*b^6*c^3*x^2*z + 2*b^9*c^3*x^2*z - 2*a^5*b^3*c^4*x^2*z - a^4*b^4*c^4*x^2*z -      2*a^3*b^5*c^4*x^2*z - a^2*b^6*c^4*x^2*z - 2*a*b^7*c^4*x^2*z - 2*a^5*b^2*c^5*x^2*z +      2*a^4*b^3*c^5*x^2*z + 4*a^3*b^4*c^5*x^2*z + 4*a^2*b^5*c^5*x^2*z + 2*a*b^6*c^5*x^2*z -      2*b^7*c^5*x^2*z + 2*a^4*b^2*c^6*x^2*z + 4*a^3*b^3*c^6*x^2*z - a^2*b^4*c^6*x^2*z +      4*a*b^5*c^6*x^2*z + 2*b^6*c^6*x^2*z - 2*a^3*b^2*c^7*x^2*z - 4*a^2*b^3*c^7*x^2*z -      4*a*b^4*c^7*x^2*z - 2*b^5*c^7*x^2*z - 2*a*b^3*c^8*x^2*z + 2*a*b^2*c^9*x^2*z + 2*b^3*c^9*x^2*z -      b^2*c^10*x^2*z + a^10*b^2*x*y*z - 2*a^9*b^3*x*y*z + 2*a^7*b^5*x*y*z - 2*a^6*b^6*x*y*z +      2*a^5*b^7*x*y*z - 2*a^3*b^9*x*y*z + a^2*b^10*x*y*z - 2*a^9*b^2*c*x*y*z + 2*a^8*b^3*c*x*y*z +      6*a^7*b^4*c*x*y*z - 6*a^6*b^5*c*x*y*z - 6*a^5*b^6*c*x*y*z + 6*a^4*b^7*c*x*y*z + 2*a^3*b^8*c*x*y*z -      2*a^2*b^9*c*x*y*z + a^10*c^2*x*y*z - 2*a^9*b*c^2*x*y*z + 4*a^7*b^3*c^2*x*y*z - a^6*b^4*c^2*x*y*z -      4*a^5*b^5*c^2*x*y*z - a^4*b^6*c^2*x*y*z + 4*a^3*b^7*c^2*x*y*z - 2*a*b^9*c^2*x*y*z +      b^10*c^2*x*y*z - 2*a^9*c^3*x*y*z + 2*a^8*b*c^3*x*y*z + 4*a^7*b^2*c^3*x*y*z - 4*a^6*b^3*c^3*x*y*z -      4*a^3*b^6*c^3*x*y*z + 4*a^2*b^7*c^3*x*y*z + 2*a*b^8*c^3*x*y*z - 2*b^9*c^3*x*y*z +      6*a^7*b*c^4*x*y*z - a^6*b^2*c^4*x*y*z + 6*a^4*b^4*c^4*x*y*z - a^2*b^6*c^4*x*y*z +      6*a*b^7*c^4*x*y*z + 2*a^7*c^5*x*y*z - 6*a^6*b*c^5*x*y*z - 4*a^5*b^2*c^5*x*y*z -      4*a^2*b^5*c^5*x*y*z - 6*a*b^6*c^5*x*y*z + 2*b^7*c^5*x*y*z - 2*a^6*c^6*x*y*z - 6*a^5*b*c^6*x*y*z -      a^4*b^2*c^6*x*y*z - 4*a^3*b^3*c^6*x*y*z - a^2*b^4*c^6*x*y*z - 6*a*b^5*c^6*x*y*z - 2*b^6*c^6*x*y*z +      2*a^5*c^7*x*y*z + 6*a^4*b*c^7*x*y*z + 4*a^3*b^2*c^7*x*y*z + 4*a^2*b^3*c^7*x*y*z +      6*a*b^4*c^7*x*y*z + 2*b^5*c^7*x*y*z + 2*a^3*b*c^8*x*y*z + 2*a*b^3*c^8*x*y*z - 2*a^3*c^9*x*y*z -      2*a^2*b*c^9*x*y*z - 2*a*b^2*c^9*x*y*z - 2*b^3*c^9*x*y*z + a^2*c^10*x*y*z + b^2*c^10*x*y*z +      a^10*b^2*y^2*z - 2*a^9*b^3*y^2*z - a^8*b^4*y^2*z + 4*a^7*b^5*y^2*z - a^6*b^6*y^2*z -      2*a^5*b^7*y^2*z + a^4*b^8*y^2*z - a^10*c^2*y^2*z + a^8*b^2*c^2*y^2*z + a^4*b^6*c^2*y^2*z -      a^2*b^8*c^2*y^2*z + 2*a^9*c^3*y^2*z - 2*a^6*b^3*c^3*y^2*z - 2*a^5*b^4*c^3*y^2*z +      2*a^2*b^7*c^3*y^2*z - 2*a^7*b*c^4*y^2*z - a^6*b^2*c^4*y^2*z - 2*a^5*b^3*c^4*y^2*z -      a^4*b^4*c^4*y^2*z - 2*a^3*b^5*c^4*y^2*z - 2*a^7*c^5*y^2*z + 2*a^6*b*c^5*y^2*z +      4*a^5*b^2*c^5*y^2*z + 4*a^4*b^3*c^5*y^2*z + 2*a^3*b^4*c^5*y^2*z - 2*a^2*b^5*c^5*y^2*z +      2*a^6*c^6*y^2*z + 4*a^5*b*c^6*y^2*z - a^4*b^2*c^6*y^2*z + 4*a^3*b^3*c^6*y^2*z +      2*a^2*b^4*c^6*y^2*z - 2*a^5*c^7*y^2*z - 4*a^4*b*c^7*y^2*z - 4*a^3*b^2*c^7*y^2*z -      2*a^2*b^3*c^7*y^2*z - 2*a^3*b*c^8*y^2*z + 2*a^3*c^9*y^2*z + 2*a^2*b*c^9*y^2*z - a^2*c^10*y^2*z -      a^10*b^2*x*z^2 + 2*a^9*b^3*x*z^2 - 2*a^7*b^5*x*z^2 + 2*a^6*b^6*x*z^2 - 2*a^5*b^7*x*z^2 +      2*a^3*b^9*x*z^2 - a^2*b^10*x*z^2 + 2*a^9*b^2*c*x*z^2 - 2*a^8*b^3*c*x*z^2 - 4*a^7*b^4*c*x*z^2 +      4*a^6*b^5*c*x*z^2 + 2*a^5*b^6*c*x*z^2 - 2*a^4*b^7*c*x*z^2 - 4*a^7*b^3*c^2*x*z^2 -      a^6*b^4*c^2*x*z^2 + 4*a^5*b^5*c^2*x*z^2 - a^4*b^6*c^2*x*z^2 + a^2*b^8*c^2*x*z^2 + b^10*c^2*x*z^2 -      2*a^7*b^2*c^3*x*z^2 + 4*a^6*b^3*c^3*x*z^2 + 4*a^5*b^4*c^3*x*z^2 - 2*a^4*b^5*c^3*x*z^2 -      2*a^3*b^6*c^3*x*z^2 - 2*b^9*c^3*x*z^2 + 2*a^6*b^2*c^4*x*z^2 + 2*a^5*b^3*c^4*x*z^2 -      a^4*b^4*c^4*x*z^2 - 2*a^3*b^5*c^4*x*z^2 - b^8*c^4*x*z^2 - 2*a^5*b^2*c^5*x*z^2 -      2*a^4*b^3*c^5*x*z^2 + 4*b^7*c^5*x*z^2 + a^2*b^4*c^6*x*z^2 - b^6*c^6*x*z^2 + 2*a^3*b^2*c^7*x*z^2 -      2*b^5*c^7*x*z^2 - a^2*b^2*c^8*x*z^2 + b^4*c^8*x*z^2 - a^10*b^2*y*z^2 + 2*a^9*b^3*y*z^2 -      2*a^7*b^5*y*z^2 + 2*a^6*b^6*y*z^2 - 2*a^5*b^7*y*z^2 + 2*a^3*b^9*y*z^2 - a^2*b^10*y*z^2 -      2*a^7*b^4*c*y*z^2 + 2*a^6*b^5*c*y*z^2 + 4*a^5*b^6*c*y*z^2 - 4*a^4*b^7*c*y*z^2 - 2*a^3*b^8*c*y*z^2 +      2*a^2*b^9*c*y*z^2 + a^10*c^2*y*z^2 + a^8*b^2*c^2*y*z^2 - a^6*b^4*c^2*y*z^2 + 4*a^5*b^5*c^2*y*z^2 -      a^4*b^6*c^2*y*z^2 - 4*a^3*b^7*c^2*y*z^2 - 2*a^9*c^3*y*z^2 - 2*a^6*b^3*c^3*y*z^2 -      2*a^5*b^4*c^3*y*z^2 + 4*a^4*b^5*c^3*y*z^2 + 4*a^3*b^6*c^3*y*z^2 - 2*a^2*b^7*c^3*y*z^2 -      a^8*c^4*y*z^2 - 2*a^5*b^3*c^4*y*z^2 - a^4*b^4*c^4*y*z^2 + 2*a^3*b^5*c^4*y*z^2 +      2*a^2*b^6*c^4*y*z^2 + 4*a^7*c^5*y*z^2 - 2*a^3*b^4*c^5*y*z^2 - 2*a^2*b^5*c^5*y*z^2 - a^6*c^6*y*z^2 +      a^4*b^2*c^6*y*z^2 - 2*a^5*c^7*y*z^2 + 2*a^2*b^3*c^7*y*z^2 + a^4*c^8*y*z^2 - a^2*b^2*c^8*y*z^2");
		exp("K386","(-a^6)*b^2*c^2*x^2*y + a^4*b^4*c^2*x^2*y + a^2*b^6*c^2*x^2*y - b^8*c^2*x^2*y +      2*a^5*b^2*c^3*x^2*y - 2*a^4*b^3*c^3*x^2*y + 2*a^3*b^4*c^3*x^2*y + 2*a^2*b^5*c^3*x^2*y -      4*a*b^6*c^3*x^2*y + a^6*c^4*x^2*y - 2*a^5*b*c^4*x^2*y + a^4*b^2*c^4*x^2*y - 3*a^2*b^4*c^4*x^2*y +      2*a*b^5*c^4*x^2*y + b^6*c^4*x^2*y + 2*a^4*b*c^5*x^2*y - 4*a^3*b^2*c^5*x^2*y + 2*a*b^4*c^5*x^2*y -      2*a^4*c^6*x^2*y + 2*a^3*b*c^6*x^2*y + a^2*b^2*c^6*x^2*y - 2*a*b^3*c^6*x^2*y + b^4*c^6*x^2*y -      2*a^2*b*c^7*x^2*y + 2*a*b^2*c^7*x^2*y + a^2*c^8*x^2*y - b^2*c^8*x^2*y - a^8*c^2*x*y^2 +      a^6*b^2*c^2*x*y^2 + a^4*b^4*c^2*x*y^2 - a^2*b^6*c^2*x*y^2 - 4*a^6*b*c^3*x*y^2 +      2*a^5*b^2*c^3*x*y^2 + 2*a^4*b^3*c^3*x*y^2 - 2*a^3*b^4*c^3*x*y^2 + 2*a^2*b^5*c^3*x*y^2 +      a^6*c^4*x*y^2 + 2*a^5*b*c^4*x*y^2 - 3*a^4*b^2*c^4*x*y^2 + a^2*b^4*c^4*x*y^2 - 2*a*b^5*c^4*x*y^2 +      b^6*c^4*x*y^2 + 2*a^4*b*c^5*x*y^2 - 4*a^2*b^3*c^5*x*y^2 + 2*a*b^4*c^5*x*y^2 + a^4*c^6*x*y^2 -      2*a^3*b*c^6*x*y^2 + a^2*b^2*c^6*x*y^2 + 2*a*b^3*c^6*x*y^2 - 2*b^4*c^6*x*y^2 + 2*a^2*b*c^7*x*y^2 -      2*a*b^2*c^7*x*y^2 - a^2*c^8*x*y^2 + b^2*c^8*x*y^2 + a^6*b^4*x^2*z - 2*a^4*b^6*x^2*z +      a^2*b^8*x^2*z - 2*a^5*b^4*c*x^2*z + 2*a^4*b^5*c*x^2*z + 2*a^3*b^6*c*x^2*z - 2*a^2*b^7*c*x^2*z -      a^6*b^2*c^2*x^2*z + 2*a^5*b^3*c^2*x^2*z + a^4*b^4*c^2*x^2*z - 4*a^3*b^5*c^2*x^2*z +      a^2*b^6*c^2*x^2*z + 2*a*b^7*c^2*x^2*z - b^8*c^2*x^2*z - 2*a^4*b^3*c^3*x^2*z - 2*a*b^6*c^3*x^2*z +      a^4*b^2*c^4*x^2*z + 2*a^3*b^3*c^4*x^2*z - 3*a^2*b^4*c^4*x^2*z + 2*a*b^5*c^4*x^2*z + b^6*c^4*x^2*z +      2*a^2*b^3*c^5*x^2*z + 2*a*b^4*c^5*x^2*z + a^2*b^2*c^6*x^2*z - 4*a*b^3*c^6*x^2*z + b^4*c^6*x^2*z -      b^2*c^8*x^2*z + a^8*b^2*x*y*z - a^6*b^4*x*y*z - a^4*b^6*x*y*z + a^2*b^8*x*y*z + 4*a^6*b^3*c*x*y*z -      4*a^5*b^4*c*x*y*z - 4*a^4*b^5*c*x*y*z + 4*a^3*b^6*c*x*y*z + a^8*c^2*x*y*z - 2*a^6*b^2*c^2*x*y*z +      2*a^4*b^4*c^2*x*y*z - 2*a^2*b^6*c^2*x*y*z + b^8*c^2*x*y*z + 4*a^6*b*c^3*x*y*z + 4*a*b^6*c^3*x*y*z -      a^6*c^4*x*y*z - 4*a^5*b*c^4*x*y*z + 2*a^4*b^2*c^4*x*y*z + 2*a^2*b^4*c^4*x*y*z - 4*a*b^5*c^4*x*y*z -      b^6*c^4*x*y*z - 4*a^4*b*c^5*x*y*z - 4*a*b^4*c^5*x*y*z - a^4*c^6*x*y*z + 4*a^3*b*c^6*x*y*z -      2*a^2*b^2*c^6*x*y*z + 4*a*b^3*c^6*x*y*z - b^4*c^6*x*y*z + a^2*c^8*x*y*z + b^2*c^8*x*y*z +      a^8*b^2*y^2*z - 2*a^6*b^4*y^2*z + a^4*b^6*y^2*z - 2*a^7*b^2*c*y^2*z + 2*a^6*b^3*c*y^2*z +      2*a^5*b^4*c*y^2*z - 2*a^4*b^5*c*y^2*z - a^8*c^2*y^2*z + 2*a^7*b*c^2*y^2*z + a^6*b^2*c^2*y^2*z -      4*a^5*b^3*c^2*y^2*z + a^4*b^4*c^2*y^2*z + 2*a^3*b^5*c^2*y^2*z - a^2*b^6*c^2*y^2*z -      2*a^6*b*c^3*y^2*z - 2*a^3*b^4*c^3*y^2*z + a^6*c^4*y^2*z + 2*a^5*b*c^4*y^2*z - 3*a^4*b^2*c^4*y^2*z +      2*a^3*b^3*c^4*y^2*z + a^2*b^4*c^4*y^2*z + 2*a^4*b*c^5*y^2*z + 2*a^3*b^2*c^5*y^2*z + a^4*c^6*y^2*z -      4*a^3*b*c^6*y^2*z + a^2*b^2*c^6*y^2*z - a^2*c^8*y^2*z - a^8*b^2*x*z^2 + a^6*b^4*x*z^2 +      a^4*b^6*x*z^2 - a^2*b^8*x*z^2 - 4*a^6*b^3*c*x*z^2 + 2*a^5*b^4*c*x*z^2 + 2*a^4*b^5*c*x*z^2 -      2*a^3*b^6*c*x*z^2 + 2*a^2*b^7*c*x*z^2 + a^6*b^2*c^2*x*z^2 + 2*a^5*b^3*c^2*x*z^2 -      3*a^4*b^4*c^2*x*z^2 + a^2*b^6*c^2*x*z^2 - 2*a*b^7*c^2*x*z^2 + b^8*c^2*x*z^2 + 2*a^4*b^3*c^3*x*z^2 -      4*a^2*b^5*c^3*x*z^2 + 2*a*b^6*c^3*x*z^2 + a^4*b^2*c^4*x*z^2 - 2*a^3*b^3*c^4*x*z^2 +      a^2*b^4*c^4*x*z^2 + 2*a*b^5*c^4*x*z^2 - 2*b^6*c^4*x*z^2 + 2*a^2*b^3*c^5*x*z^2 - 2*a*b^4*c^5*x*z^2 -      a^2*b^2*c^6*x*z^2 + b^4*c^6*x*z^2 - a^8*b^2*y*z^2 + a^6*b^4*y*z^2 + a^4*b^6*y*z^2 - a^2*b^8*y*z^2 +      2*a^7*b^2*c*y*z^2 - 2*a^6*b^3*c*y*z^2 + 2*a^5*b^4*c*y*z^2 + 2*a^4*b^5*c*y*z^2 - 4*a^3*b^6*c*y*z^2 +      a^8*c^2*y*z^2 - 2*a^7*b*c^2*y*z^2 + a^6*b^2*c^2*y*z^2 - 3*a^4*b^4*c^2*y*z^2 + 2*a^3*b^5*c^2*y*z^2 +      a^2*b^6*c^2*y*z^2 + 2*a^6*b*c^3*y*z^2 - 4*a^5*b^2*c^3*y*z^2 + 2*a^3*b^4*c^3*y*z^2 -      2*a^6*c^4*y*z^2 + 2*a^5*b*c^4*y*z^2 + a^4*b^2*c^4*y*z^2 - 2*a^3*b^3*c^4*y*z^2 + a^2*b^4*c^4*y*z^2 -      2*a^4*b*c^5*y*z^2 + 2*a^3*b^2*c^5*y*z^2 + a^4*c^6*y*z^2 - a^2*b^2*c^6*y*z^2");
		exp("K387","(-a^5)*b*c^2*x^2*y + a^4*b^2*c^2*x^2*y + a*b^5*c^2*x^2*y - b^6*c^2*x^2*y + a^5*c^3*x^2*y -      4*a^3*b^2*c^3*x^2*y + 4*a^2*b^3*c^3*x^2*y - a*b^4*c^3*x^2*y - a^4*c^4*x^2*y + 5*a^3*b*c^4*x^2*y -      a^2*b^2*c^4*x^2*y - 5*a*b^3*c^4*x^2*y + 2*b^4*c^4*x^2*y - a^3*c^5*x^2*y - 4*a^2*b*c^5*x^2*y +      5*a*b^2*c^5*x^2*y + a^2*c^6*x^2*y - b^2*c^6*x^2*y - a^6*c^2*x*y^2 + a^5*b*c^2*x*y^2 +      a^2*b^4*c^2*x*y^2 - a*b^5*c^2*x*y^2 - a^4*b*c^3*x*y^2 + 4*a^3*b^2*c^3*x*y^2 - 4*a^2*b^3*c^3*x*y^2 +      b^5*c^3*x*y^2 + 2*a^4*c^4*x*y^2 - 5*a^3*b*c^4*x*y^2 - a^2*b^2*c^4*x*y^2 + 5*a*b^3*c^4*x*y^2 -      b^4*c^4*x*y^2 + 5*a^2*b*c^5*x*y^2 - 4*a*b^2*c^5*x*y^2 - b^3*c^5*x*y^2 - a^2*c^6*x*y^2 +      b^2*c^6*x*y^2 + a^5*b^3*x^2*z - a^4*b^4*x^2*z - a^3*b^5*x^2*z + a^2*b^6*x^2*z - a^5*b^2*c*x^2*z +      5*a^3*b^4*c*x^2*z - 4*a^2*b^5*c*x^2*z + a^4*b^2*c^2*x^2*z - 4*a^3*b^3*c^2*x^2*z -      a^2*b^4*c^2*x^2*z + 5*a*b^5*c^2*x^2*z - b^6*c^2*x^2*z + 4*a^2*b^3*c^3*x^2*z - 5*a*b^4*c^3*x^2*z -      a*b^3*c^4*x^2*z + 2*b^4*c^4*x^2*z + a*b^2*c^5*x^2*z - b^2*c^6*x^2*z + a^6*b^2*x*y*z -      2*a^4*b^4*x*y*z + a^2*b^6*x*y*z - a^5*b^2*c*x*y*z + a^4*b^3*c*x*y*z + a^3*b^4*c*x*y*z -      a^2*b^5*c*x*y*z + a^6*c^2*x*y*z - a^5*b*c^2*x*y*z - a*b^5*c^2*x*y*z + b^6*c^2*x*y*z +      a^4*b*c^3*x*y*z + a*b^4*c^3*x*y*z - 2*a^4*c^4*x*y*z + a^3*b*c^4*x*y*z + a*b^3*c^4*x*y*z -      2*b^4*c^4*x*y*z - a^2*b*c^5*x*y*z - a*b^2*c^5*x*y*z + a^2*c^6*x*y*z + b^2*c^6*x*y*z +      a^6*b^2*y^2*z - a^5*b^3*y^2*z - a^4*b^4*y^2*z + a^3*b^5*y^2*z - 4*a^5*b^2*c*y^2*z +      5*a^4*b^3*c*y^2*z - a^2*b^5*c*y^2*z - a^6*c^2*y^2*z + 5*a^5*b*c^2*y^2*z - a^4*b^2*c^2*y^2*z -      4*a^3*b^3*c^2*y^2*z + a^2*b^4*c^2*y^2*z - 5*a^4*b*c^3*y^2*z + 4*a^3*b^2*c^3*y^2*z +      2*a^4*c^4*y^2*z - a^3*b*c^4*y^2*z + a^2*b*c^5*y^2*z - a^2*c^6*y^2*z - a^6*b^2*x*z^2 +      2*a^4*b^4*x*z^2 - a^2*b^6*x*z^2 + a^5*b^2*c*x*z^2 - a^4*b^3*c*x*z^2 - 5*a^3*b^4*c*x*z^2 +      5*a^2*b^5*c*x*z^2 + 4*a^3*b^3*c^2*x*z^2 - a^2*b^4*c^2*x*z^2 - 4*a*b^5*c^2*x*z^2 + b^6*c^2*x*z^2 -      4*a^2*b^3*c^3*x*z^2 + 5*a*b^4*c^3*x*z^2 - b^5*c^3*x*z^2 + a^2*b^2*c^4*x*z^2 - b^4*c^4*x*z^2 -      a*b^2*c^5*x*z^2 + b^3*c^5*x*z^2 - a^6*b^2*y*z^2 + 2*a^4*b^4*y*z^2 - a^2*b^6*y*z^2 +      5*a^5*b^2*c*y*z^2 - 5*a^4*b^3*c*y*z^2 - a^3*b^4*c*y*z^2 + a^2*b^5*c*y*z^2 + a^6*c^2*y*z^2 -      4*a^5*b*c^2*y*z^2 - a^4*b^2*c^2*y*z^2 + 4*a^3*b^3*c^2*y*z^2 - a^5*c^3*y*z^2 + 5*a^4*b*c^3*y*z^2 -      4*a^3*b^2*c^3*y*z^2 - a^4*c^4*y*z^2 + a^2*b^2*c^4*y*z^2 + a^3*c^5*y*z^2 - a^2*b*c^5*y*z^2");
		exp("K388","a^12*b^2*c^4*x^2*y - 2*a^10*b^4*c^4*x^2*y + a^8*b^6*c^4*x^2*y - a^4*b^10*c^4*x^2*y +      2*a^2*b^12*c^4*x^2*y - b^14*c^4*x^2*y - 3*a^10*b^2*c^6*x^2*y + 2*a^8*b^4*c^6*x^2*y +      2*a^6*b^6*c^6*x^2*y - 2*a^4*b^8*c^6*x^2*y - 3*a^2*b^10*c^6*x^2*y + 4*b^12*c^6*x^2*y +      3*a^8*b^2*c^8*x^2*y + 2*a^4*b^6*c^8*x^2*y - 5*b^10*c^8*x^2*y - 2*a^6*b^2*c^10*x^2*y -      2*a^4*b^4*c^10*x^2*y - 2*a^2*b^6*c^10*x^2*y + 3*a^4*b^2*c^12*x^2*y + 6*a^2*b^4*c^12*x^2*y +      5*b^6*c^12*x^2*y - 3*a^2*b^2*c^14*x^2*y - 4*b^4*c^14*x^2*y + b^2*c^16*x^2*y + a^14*c^4*x*y^2 -      2*a^12*b^2*c^4*x*y^2 + a^10*b^4*c^4*x*y^2 - a^6*b^8*c^4*x*y^2 + 2*a^4*b^10*c^4*x*y^2 -      a^2*b^12*c^4*x*y^2 - 4*a^12*c^6*x*y^2 + 3*a^10*b^2*c^6*x*y^2 + 2*a^8*b^4*c^6*x*y^2 -      2*a^6*b^6*c^6*x*y^2 - 2*a^4*b^8*c^6*x*y^2 + 3*a^2*b^10*c^6*x*y^2 + 5*a^10*c^8*x*y^2 -      2*a^6*b^4*c^8*x*y^2 - 3*a^2*b^8*c^8*x*y^2 + 2*a^6*b^2*c^10*x*y^2 + 2*a^4*b^4*c^10*x*y^2 +      2*a^2*b^6*c^10*x*y^2 - 5*a^6*c^12*x*y^2 - 6*a^4*b^2*c^12*x*y^2 - 3*a^2*b^4*c^12*x*y^2 +      4*a^4*c^14*x*y^2 + 3*a^2*b^2*c^14*x*y^2 - a^2*c^16*x*y^2 - a^12*b^4*c^2*x^2*z +      3*a^10*b^6*c^2*x^2*z - 3*a^8*b^8*c^2*x^2*z + 2*a^6*b^10*c^2*x^2*z - 3*a^4*b^12*c^2*x^2*z +      3*a^2*b^14*c^2*x^2*z - b^16*c^2*x^2*z + 2*a^10*b^4*c^4*x^2*z - 2*a^8*b^6*c^4*x^2*z +      2*a^4*b^10*c^4*x^2*z - 6*a^2*b^12*c^4*x^2*z + 4*b^14*c^4*x^2*z - a^8*b^4*c^6*x^2*z -      2*a^6*b^6*c^6*x^2*z - 2*a^4*b^8*c^6*x^2*z + 2*a^2*b^10*c^6*x^2*z - 5*b^12*c^6*x^2*z +      2*a^4*b^6*c^8*x^2*z + a^4*b^4*c^10*x^2*z + 3*a^2*b^6*c^10*x^2*z + 5*b^8*c^10*x^2*z -      2*a^2*b^4*c^12*x^2*z - 4*b^6*c^12*x^2*z + b^4*c^14*x^2*z + a^16*c^2*y^2*z - 3*a^14*b^2*c^2*y^2*z +      3*a^12*b^4*c^2*y^2*z - 2*a^10*b^6*c^2*y^2*z + 3*a^8*b^8*c^2*y^2*z - 3*a^6*b^10*c^2*y^2*z +      a^4*b^12*c^2*y^2*z - 4*a^14*c^4*y^2*z + 6*a^12*b^2*c^4*y^2*z - 2*a^10*b^4*c^4*y^2*z +      2*a^6*b^8*c^4*y^2*z - 2*a^4*b^10*c^4*y^2*z + 5*a^12*c^6*y^2*z - 2*a^10*b^2*c^6*y^2*z +      2*a^8*b^4*c^6*y^2*z + 2*a^6*b^6*c^6*y^2*z + a^4*b^8*c^6*y^2*z - 2*a^6*b^4*c^8*y^2*z -      5*a^8*c^10*y^2*z - 3*a^6*b^2*c^10*y^2*z - a^4*b^4*c^10*y^2*z + 4*a^6*c^12*y^2*z +      2*a^4*b^2*c^12*y^2*z - a^4*c^14*y^2*z - a^14*b^4*x*z^2 + 4*a^12*b^6*x*z^2 - 5*a^10*b^8*x*z^2 +      5*a^6*b^12*x*z^2 - 4*a^4*b^14*x*z^2 + a^2*b^16*x*z^2 + 2*a^12*b^4*c^2*x*z^2 -      3*a^10*b^6*c^2*x*z^2 - 2*a^6*b^10*c^2*x*z^2 + 6*a^4*b^12*c^2*x*z^2 - 3*a^2*b^14*c^2*x*z^2 -      a^10*b^4*c^4*x*z^2 - 2*a^8*b^6*c^4*x*z^2 + 2*a^6*b^8*c^4*x*z^2 - 2*a^4*b^10*c^4*x*z^2 +      3*a^2*b^12*c^4*x*z^2 + 2*a^6*b^6*c^6*x*z^2 - 2*a^2*b^10*c^6*x*z^2 + a^6*b^4*c^8*x*z^2 +      2*a^4*b^6*c^8*x*z^2 + 3*a^2*b^8*c^8*x*z^2 - 2*a^4*b^4*c^10*x*z^2 - 3*a^2*b^6*c^10*x*z^2 +      a^2*b^4*c^12*x*z^2 - a^16*b^2*y*z^2 + 4*a^14*b^4*y*z^2 - 5*a^12*b^6*y*z^2 + 5*a^8*b^10*y*z^2 -      4*a^6*b^12*y*z^2 + a^4*b^14*y*z^2 + 3*a^14*b^2*c^2*y*z^2 - 6*a^12*b^4*c^2*y*z^2 +      2*a^10*b^6*c^2*y*z^2 + 3*a^6*b^10*c^2*y*z^2 - 2*a^4*b^12*c^2*y*z^2 - 3*a^12*b^2*c^4*y*z^2 +      2*a^10*b^4*c^4*y*z^2 - 2*a^8*b^6*c^4*y*z^2 + 2*a^6*b^8*c^4*y*z^2 + a^4*b^10*c^4*y*z^2 +      2*a^10*b^2*c^6*y*z^2 - 2*a^6*b^6*c^6*y*z^2 - 3*a^8*b^2*c^8*y*z^2 - 2*a^6*b^4*c^8*y*z^2 -      a^4*b^6*c^8*y*z^2 + 3*a^6*b^2*c^10*y*z^2 + 2*a^4*b^4*c^10*y*z^2 - a^4*b^2*c^12*y*z^2");
		exp("K389","(-a^6)*b^2*c^4*x^2*y - a^4*b^4*c^4*x^2*y + a^2*b^6*c^4*x^2*y + b^8*c^4*x^2*y +      3*a^4*b^2*c^6*x^2*y - 3*b^6*c^6*x^2*y - a^2*b^2*c^8*x^2*y + 3*b^4*c^8*x^2*y - b^2*c^10*x^2*y -      a^8*c^4*x*y^2 - a^6*b^2*c^4*x*y^2 + a^4*b^4*c^4*x*y^2 + a^2*b^6*c^4*x*y^2 + 3*a^6*c^6*x*y^2 -      3*a^2*b^4*c^6*x*y^2 - 3*a^4*c^8*x*y^2 + a^2*b^2*c^8*x*y^2 + a^2*c^10*x*y^2 + a^6*b^4*c^2*x^2*z -      3*a^4*b^6*c^2*x^2*z + a^2*b^8*c^2*x^2*z + b^10*c^2*x^2*z + a^4*b^4*c^4*x^2*z - 3*b^8*c^4*x^2*z -      a^2*b^4*c^6*x^2*z + 3*b^6*c^6*x^2*z - b^4*c^8*x^2*z - 4*a^6*b^4*c^2*x*y*z + 4*a^4*b^6*c^2*x*y*z +      4*a^6*b^2*c^4*x*y*z - 4*a^2*b^6*c^4*x*y*z - 4*a^4*b^2*c^6*x*y*z + 4*a^2*b^4*c^6*x*y*z -      a^10*c^2*y^2*z - a^8*b^2*c^2*y^2*z + 3*a^6*b^4*c^2*y^2*z - a^4*b^6*c^2*y^2*z + 3*a^8*c^4*y^2*z -      a^4*b^4*c^4*y^2*z - 3*a^6*c^6*y^2*z + a^4*b^2*c^6*y^2*z + a^4*c^8*y^2*z + a^8*b^4*x*z^2 -      3*a^6*b^6*x*z^2 + 3*a^4*b^8*x*z^2 - a^2*b^10*x*z^2 + a^6*b^4*c^2*x*z^2 - a^2*b^8*c^2*x*z^2 -      a^4*b^4*c^4*x*z^2 + 3*a^2*b^6*c^4*x*z^2 - a^2*b^4*c^6*x*z^2 + a^10*b^2*y*z^2 - 3*a^8*b^4*y*z^2 +      3*a^6*b^6*y*z^2 - a^4*b^8*y*z^2 + a^8*b^2*c^2*y*z^2 - a^4*b^6*c^2*y*z^2 - 3*a^6*b^2*c^4*y*z^2 +      a^4*b^4*c^4*y*z^2 + a^4*b^2*c^6*y*z^2");
		exp("K390","a^4*b^2*c^4*x^2*y + a^2*b^4*c^4*x^2*y + b^6*c^4*x^2*y - 2*a^2*b^2*c^6*x^2*y - 2*b^4*c^6*x^2*y +      b^2*c^8*x^2*y - a^6*c^4*x*y^2 - a^4*b^2*c^4*x*y^2 - a^2*b^4*c^4*x*y^2 + 2*a^4*c^6*x*y^2 +      2*a^2*b^2*c^6*x*y^2 - a^2*c^8*x*y^2 - a^4*b^4*c^2*x^2*z + 2*a^2*b^6*c^2*x^2*z - b^8*c^2*x^2*z -      a^2*b^4*c^4*x^2*z + 2*b^6*c^4*x^2*z - b^4*c^6*x^2*z + a^8*c^2*y^2*z - 2*a^6*b^2*c^2*y^2*z +      a^4*b^4*c^2*y^2*z - 2*a^6*c^4*y^2*z + a^4*b^2*c^4*y^2*z + a^4*c^6*y^2*z + a^6*b^4*x*z^2 -      2*a^4*b^6*x*z^2 + a^2*b^8*x*z^2 + a^4*b^4*c^2*x*z^2 - 2*a^2*b^6*c^2*x*z^2 + a^2*b^4*c^4*x*z^2 -      a^8*b^2*y*z^2 + 2*a^6*b^4*y*z^2 - a^4*b^6*y*z^2 + 2*a^6*b^2*c^2*y*z^2 - a^4*b^4*c^2*y*z^2 -      a^4*b^2*c^4*y*z^2");
		exp("K391","(-a^3)*c^2*x^2*y - a^2*b*c^2*x^2*y + a*b^2*c^2*x^2*y + b^3*c^2*x^2*y + a*c^4*x^2*y +      b*c^4*x^2*y - a^3*c^2*x*y^2 - a^2*b*c^2*x*y^2 + a*b^2*c^2*x*y^2 + b^3*c^2*x*y^2 - a*c^4*x*y^2 -      b*c^4*x*y^2 + a^3*b^2*x^2*z - a*b^4*x^2*z + a^2*b^2*c*x^2*z - b^4*c*x^2*z - a*b^2*c^2*x^2*z -      b^2*c^3*x^2*z + a^4*b*x*y*z + a^3*b^2*x*y*z - a^2*b^3*x*y*z - a*b^4*x*y*z - a^4*c*x*y*z +      b^4*c*x*y*z - a^3*c^2*x*y*z + b^3*c^2*x*y*z + a^2*c^3*x*y*z - b^2*c^3*x*y*z + a*c^4*x*y*z -      b*c^4*x*y*z + a^4*b*y^2*z - a^2*b^3*y^2*z + a^4*c*y^2*z - a^2*b^2*c*y^2*z + a^2*b*c^2*y^2*z +      a^2*c^3*y^2*z + a^3*b^2*x*z^2 + a*b^4*x*z^2 + a^2*b^2*c*x*z^2 + b^4*c*x*z^2 - a*b^2*c^2*x*z^2 -      b^2*c^3*x*z^2 - a^4*b*y*z^2 - a^2*b^3*y*z^2 - a^4*c*y*z^2 - a^2*b^2*c*y*z^2 + a^2*b*c^2*y*z^2 +      a^2*c^3*y*z^2");
		exp("K392","a^6*x^2*y - a^4*b^2*x^2*y - 2*a^4*c^2*x^2*y + 2*a^2*b^2*c^2*x^2*y + a^2*c^4*x^2*y -      b^2*c^4*x^2*y - a^2*b^4*x*y^2 + b^6*x*y^2 + 2*a^2*b^2*c^2*x*y^2 - 2*b^4*c^2*x*y^2 - a^2*c^4*x*y^2 +      b^2*c^4*x*y^2 + a^6*x^2*z - 2*a^4*b^2*x^2*z + a^2*b^4*x^2*z - a^4*c^2*x^2*z + 2*a^2*b^2*c^2*x^2*z -      b^4*c^2*x^2*z + a^4*b^2*y^2*z - 2*a^2*b^4*y^2*z + b^6*y^2*z - a^4*c^2*y^2*z + 2*a^2*b^2*c^2*y^2*z -      b^4*c^2*y^2*z - a^2*b^4*x*z^2 + 2*a^2*b^2*c^2*x*z^2 + b^4*c^2*x*z^2 - a^2*c^4*x*z^2 -      2*b^2*c^4*x*z^2 + c^6*x*z^2 - a^4*b^2*y*z^2 + a^4*c^2*y*z^2 + 2*a^2*b^2*c^2*y*z^2 -      2*a^2*c^4*y*z^2 - b^2*c^4*y*z^2 + c^6*y*z^2");
		exp("K393","a^6*x^2*y - 3*a^4*b^2*x^2*y + 3*a^2*b^4*x^2*y - b^6*x^2*y + 2*a^2*b^2*c^2*x^2*y -      2*b^4*c^2*x^2*y - 3*a^2*c^4*x^2*y + b^2*c^4*x^2*y + 2*c^6*x^2*y - a^6*x*y^2 + 3*a^4*b^2*x*y^2 -      3*a^2*b^4*x*y^2 + b^6*x*y^2 - 2*a^4*c^2*x*y^2 + 2*a^2*b^2*c^2*x*y^2 + a^2*c^4*x*y^2 -      3*b^2*c^4*x*y^2 + 2*c^6*x*y^2 + a^6*x^2*z - 3*a^2*b^4*x^2*z + 2*b^6*x^2*z - 3*a^4*c^2*x^2*z +      2*a^2*b^2*c^2*x^2*z + b^4*c^2*x^2*z + 3*a^2*c^4*x^2*z - 2*b^2*c^4*x^2*z - c^6*x^2*z + 2*a^6*y^2*z -      3*a^4*b^2*y^2*z + b^6*y^2*z + a^4*c^2*y^2*z + 2*a^2*b^2*c^2*y^2*z - 3*b^4*c^2*y^2*z -      2*a^2*c^4*y^2*z + 3*b^2*c^4*y^2*z - c^6*y^2*z - a^6*x*z^2 - 2*a^4*b^2*x*z^2 + a^2*b^4*x*z^2 +      2*b^6*x*z^2 + 3*a^4*c^2*x*z^2 + 2*a^2*b^2*c^2*x*z^2 - 3*b^4*c^2*x*z^2 - 3*a^2*c^4*x*z^2 +      c^6*x*z^2 + 2*a^6*y*z^2 + a^4*b^2*y*z^2 - 2*a^2*b^4*y*z^2 - b^6*y*z^2 - 3*a^4*c^2*y*z^2 +      2*a^2*b^2*c^2*y*z^2 + 3*b^4*c^2*y*z^2 - 3*b^2*c^4*y*z^2 + c^6*y*z^2");
		exp("K394","a^4*x^2*y - b^4*x^2*y + a^2*c^2*x^2*y + 2*b^2*c^2*x^2*y - 3*c^4*x^2*y + a^4*x*y^2 - b^4*x*y^2 -      2*a^2*c^2*x*y^2 - b^2*c^2*x*y^2 + 3*c^4*x*y^2 - a^4*x^2*z - a^2*b^2*x^2*z + 3*b^4*x^2*z -      2*b^2*c^2*x^2*z + c^4*x^2*z - 3*a^4*y^2*z + a^2*b^2*y^2*z + b^4*y^2*z + 2*a^2*c^2*y^2*z -      c^4*y^2*z - a^4*x*z^2 + 2*a^2*b^2*x*z^2 - 3*b^4*x*z^2 + b^2*c^2*x*z^2 + c^4*x*z^2 + 3*a^4*y*z^2 -      2*a^2*b^2*y*z^2 + b^4*y*z^2 - a^2*c^2*y*z^2 - c^4*y*z^2");
		exp("K395","2*a^4*x^2*y - 2*a^2*b^2*x^2*y - a^2*c^2*x^2*y - 2*c^4*x^2*y + 2*a^2*b^2*x*y^2 - 2*b^4*x*y^2 +      b^2*c^2*x*y^2 + 2*c^4*x*y^2 - 2*a^4*x^2*z + a^2*b^2*x^2*z + 2*b^4*x^2*z + 2*a^2*c^2*x^2*z -      2*a^4*y^2*z - a^2*b^2*y^2*z + 2*b^4*y^2*z - 2*b^2*c^2*y^2*z - 2*b^4*x*z^2 - 2*a^2*c^2*x*z^2 -      b^2*c^2*x*z^2 + 2*c^4*x*z^2 + 2*a^4*y*z^2 + a^2*c^2*y*z^2 + 2*b^2*c^2*y*z^2 - 2*c^4*y*z^2");
		exp("K396","(-a^4)*c^2*x^2*y + 2*a^2*b^2*c^2*x^2*y - b^4*c^2*x^2*y + 2*a^2*c^4*x^2*y - b^2*c^4*x^2*y -      c^6*x^2*y - a^4*c^2*x*y^2 + 2*a^2*b^2*c^2*x*y^2 - b^4*c^2*x*y^2 - a^2*c^4*x*y^2 + 2*b^2*c^4*x*y^2 -      c^6*x*y^2 - a^4*b^2*x^2*z + 2*a^2*b^4*x^2*z - b^6*x^2*z + 2*a^2*b^2*c^2*x^2*z - b^4*c^2*x^2*z -      b^2*c^4*x^2*z + 2*a^6*x*y*z - 2*a^4*b^2*x*y*z - 2*a^2*b^4*x*y*z + 2*b^6*x*y*z - 2*a^4*c^2*x*y*z +      6*a^2*b^2*c^2*x*y*z - 2*b^4*c^2*x*y*z - 2*a^2*c^4*x*y*z - 2*b^2*c^4*x*y*z + 2*c^6*x*y*z -      a^6*y^2*z + 2*a^4*b^2*y^2*z - a^2*b^4*y^2*z - a^4*c^2*y^2*z + 2*a^2*b^2*c^2*y^2*z - a^2*c^4*y^2*z -      a^4*b^2*x*z^2 - a^2*b^4*x*z^2 - b^6*x*z^2 + 2*a^2*b^2*c^2*x*z^2 + 2*b^4*c^2*x*z^2 - b^2*c^4*x*z^2 -      a^6*y*z^2 - a^4*b^2*y*z^2 - a^2*b^4*y*z^2 + 2*a^4*c^2*y*z^2 + 2*a^2*b^2*c^2*y*z^2 -      a^2*c^4*y*z^2");
		exp("K397","a^4*c^2*x^2*y + a^2*b^2*c^2*x^2*y - 2*b^4*c^2*x^2*y - 2*a^2*c^4*x^2*y + b^2*c^4*x^2*y +      c^6*x^2*y - 2*a^4*c^2*x*y^2 + a^2*b^2*c^2*x*y^2 + b^4*c^2*x*y^2 + a^2*c^4*x*y^2 - 2*b^2*c^4*x*y^2 +      c^6*x*y^2 + a^4*b^2*x^2*z - 2*a^2*b^4*x^2*z + b^6*x^2*z + a^2*b^2*c^2*x^2*z + b^4*c^2*x^2*z -      2*b^2*c^4*x^2*z - 2*a^6*x*y*z + 2*a^4*b^2*x*y*z + 2*a^2*b^4*x*y*z - 2*b^6*x*y*z + 2*a^4*c^2*x*y*z -      6*a^2*b^2*c^2*x*y*z + 2*b^4*c^2*x*y*z + 2*a^2*c^4*x*y*z + 2*b^2*c^4*x*y*z - 2*c^6*x*y*z +      a^6*y^2*z - 2*a^4*b^2*y^2*z + a^2*b^4*y^2*z + a^4*c^2*y^2*z + a^2*b^2*c^2*y^2*z - 2*a^2*c^4*y^2*z -      2*a^4*b^2*x*z^2 + a^2*b^4*x*z^2 + b^6*x*z^2 + a^2*b^2*c^2*x*z^2 - 2*b^4*c^2*x*z^2 + b^2*c^4*x*z^2 +      a^6*y*z^2 + a^4*b^2*y*z^2 - 2*a^2*b^4*y*z^2 - 2*a^4*c^2*y*z^2 + a^2*b^2*c^2*y*z^2 +      a^2*c^4*y*z^2");
		exp("K398","(-a^8)*x^2*y - 2*a^6*b^2*x^2*y + 3*a^4*b^4*x^2*y + 2*a^2*b^6*x^2*y - 2*b^8*x^2*y +      6*a^6*c^2*x^2*y - 12*a^2*b^4*c^2*x^2*y + 6*b^6*c^2*x^2*y - 9*a^4*c^4*x^2*y + 12*a^2*b^2*c^4*x^2*y -      3*b^4*c^4*x^2*y + 2*a^2*c^6*x^2*y - 2*b^2*c^6*x^2*y + 2*a^8*x*y^2 - 2*a^6*b^2*x*y^2 -      3*a^4*b^4*x*y^2 + 2*a^2*b^6*x*y^2 + b^8*x*y^2 - 6*a^6*c^2*x*y^2 + 12*a^4*b^2*c^2*x*y^2 -      6*b^6*c^2*x*y^2 + 3*a^4*c^4*x*y^2 - 12*a^2*b^2*c^4*x*y^2 + 9*b^4*c^4*x*y^2 + 2*a^2*c^6*x*y^2 -      2*b^2*c^6*x*y^2 + a^8*x^2*z - 6*a^6*b^2*x^2*z + 9*a^4*b^4*x^2*z - 2*a^2*b^6*x^2*z +      2*a^6*c^2*x^2*z - 12*a^2*b^4*c^2*x^2*z + 2*b^6*c^2*x^2*z - 3*a^4*c^4*x^2*z + 12*a^2*b^2*c^4*x^2*z +      3*b^4*c^4*x^2*z - 2*a^2*c^6*x^2*z - 6*b^2*c^6*x^2*z + 2*c^8*x^2*z + 2*a^6*b^2*y^2*z -      9*a^4*b^4*y^2*z + 6*a^2*b^6*y^2*z - b^8*y^2*z - 2*a^6*c^2*y^2*z + 12*a^4*b^2*c^2*y^2*z -      2*b^6*c^2*y^2*z - 3*a^4*c^4*y^2*z - 12*a^2*b^2*c^4*y^2*z + 3*b^4*c^4*y^2*z + 6*a^2*c^6*y^2*z +      2*b^2*c^6*y^2*z - 2*c^8*y^2*z - 2*a^8*x*z^2 + 6*a^6*b^2*x*z^2 - 3*a^4*b^4*x*z^2 - 2*a^2*b^6*x*z^2 +      2*a^6*c^2*x*z^2 - 12*a^4*b^2*c^2*x*z^2 + 12*a^2*b^4*c^2*x*z^2 + 2*b^6*c^2*x*z^2 + 3*a^4*c^4*x*z^2 -      9*b^4*c^4*x*z^2 - 2*a^2*c^6*x*z^2 + 6*b^2*c^6*x*z^2 - c^8*x*z^2 + 2*a^6*b^2*y*z^2 +      3*a^4*b^4*y*z^2 - 6*a^2*b^6*y*z^2 + 2*b^8*y*z^2 - 2*a^6*c^2*y*z^2 - 12*a^4*b^2*c^2*y*z^2 +      12*a^2*b^4*c^2*y*z^2 - 2*b^6*c^2*y*z^2 + 9*a^4*c^4*y*z^2 - 3*b^4*c^4*y*z^2 - 6*a^2*c^6*y*z^2 +      2*b^2*c^6*y*z^2 + c^8*y*z^2");
		exp("K399","(-a^4)*x^2*y - 2*a^3*b*x^2*y + 3*a^2*b^2*x^2*y + 2*a*b^3*x^2*y - 2*b^4*x^2*y + 6*a^3*c*x^2*y -      12*a*b^2*c*x^2*y + 6*b^3*c*x^2*y - 9*a^2*c^2*x^2*y + 12*a*b*c^2*x^2*y - 3*b^2*c^2*x^2*y +      2*a*c^3*x^2*y - 2*b*c^3*x^2*y + 2*a^4*x*y^2 - 2*a^3*b*x*y^2 - 3*a^2*b^2*x*y^2 + 2*a*b^3*x*y^2 +      b^4*x*y^2 - 6*a^3*c*x*y^2 + 12*a^2*b*c*x*y^2 - 6*b^3*c*x*y^2 + 3*a^2*c^2*x*y^2 - 12*a*b*c^2*x*y^2 +      9*b^2*c^2*x*y^2 + 2*a*c^3*x*y^2 - 2*b*c^3*x*y^2 + a^4*x^2*z - 6*a^3*b*x^2*z + 9*a^2*b^2*x^2*z -      2*a*b^3*x^2*z + 2*a^3*c*x^2*z - 12*a*b^2*c*x^2*z + 2*b^3*c*x^2*z - 3*a^2*c^2*x^2*z +      12*a*b*c^2*x^2*z + 3*b^2*c^2*x^2*z - 2*a*c^3*x^2*z - 6*b*c^3*x^2*z + 2*c^4*x^2*z + 2*a^3*b*y^2*z -      9*a^2*b^2*y^2*z + 6*a*b^3*y^2*z - b^4*y^2*z - 2*a^3*c*y^2*z + 12*a^2*b*c*y^2*z - 2*b^3*c*y^2*z -      3*a^2*c^2*y^2*z - 12*a*b*c^2*y^2*z + 3*b^2*c^2*y^2*z + 6*a*c^3*y^2*z + 2*b*c^3*y^2*z -      2*c^4*y^2*z - 2*a^4*x*z^2 + 6*a^3*b*x*z^2 - 3*a^2*b^2*x*z^2 - 2*a*b^3*x*z^2 + 2*a^3*c*x*z^2 -      12*a^2*b*c*x*z^2 + 12*a*b^2*c*x*z^2 + 2*b^3*c*x*z^2 + 3*a^2*c^2*x*z^2 - 9*b^2*c^2*x*z^2 -      2*a*c^3*x*z^2 + 6*b*c^3*x*z^2 - c^4*x*z^2 + 2*a^3*b*y*z^2 + 3*a^2*b^2*y*z^2 - 6*a*b^3*y*z^2 +      2*b^4*y*z^2 - 2*a^3*c*y*z^2 - 12*a^2*b*c*y*z^2 + 12*a*b^2*c*y*z^2 - 2*b^3*c*y*z^2 +      9*a^2*c^2*y*z^2 - 3*b^2*c^2*y*z^2 - 6*a*c^3*y*z^2 + 2*b*c^3*y*z^2 + c^4*y*z^2");
		exp("K400a","(-b^2)*x^2*y + c^2*x^2*y + a^2*x*y^2 - c^2*x*y^2 - b^2*x^2*z + c^2*x^2*z + a^2*y^2*z - c^2*y^2*z - a^2*x*z^2 + b^2*x*z^2 - a^2*y*z^2 + b^2*y*z^2 -      Sqrt(a^4 - a^2*b^2 + b^4 - a^2*c^2 - b^2*c^2 + c^4)*(x^2*y - x*y^2 - x^2*z + y^2*z + x*z^2 - y*z^2)");
		exp("K400b","(-b^2)*x^2*y + c^2*x^2*y + a^2*x*y^2 - c^2*x*y^2 - b^2*x^2*z + c^2*x^2*z + a^2*y^2*z - c^2*y^2*z - a^2*x*z^2 + b^2*x*z^2 - a^2*y*z^2 + b^2*y*z^2 +      Sqrt(a^4 - a^2*b^2 + b^4 - a^2*c^2 - b^2*c^2 + c^4)*(x^2*y - x*y^2 - x^2*z + y^2*z + x*z^2 - y*z^2)");
		exp("K401","a^10*b^2*x^3 - a^8*b^4*x^3 - 2*a^6*b^6*x^3 + 2*a^4*b^8*x^3 + a^2*b^10*x^3 - b^12*x^3 -      a^10*c^2*x^3 + 6*a^6*b^4*c^2*x^3 - 4*a^4*b^6*c^2*x^3 - 5*a^2*b^8*c^2*x^3 + 4*b^10*c^2*x^3 +      a^8*c^4*x^3 - 6*a^6*b^2*c^4*x^3 + 10*a^2*b^6*c^4*x^3 - 5*b^8*c^4*x^3 + 2*a^6*c^6*x^3 +      4*a^4*b^2*c^6*x^3 - 10*a^2*b^4*c^6*x^3 - 2*a^4*c^8*x^3 + 5*a^2*b^2*c^8*x^3 + 5*b^4*c^8*x^3 -      a^2*c^10*x^3 - 4*b^2*c^10*x^3 + c^12*x^3 + a^12*x^2*y + a^10*b^2*x^2*y - 4*a^8*b^4*x^2*y -      2*a^6*b^6*x^2*y + 5*a^4*b^8*x^2*y + a^2*b^10*x^2*y - 2*b^12*x^2*y - 6*a^10*c^2*x^2*y +      5*a^8*b^2*c^2*x^2*y + 16*a^6*b^4*c^2*x^2*y - 14*a^4*b^6*c^2*x^2*y - 10*a^2*b^8*c^2*x^2*y +      9*b^10*c^2*x^2*y + 7*a^8*c^4*x^2*y - 18*a^6*b^2*c^4*x^2*y + 4*a^4*b^4*c^4*x^2*y +      22*a^2*b^6*c^4*x^2*y - 15*b^8*c^4*x^2*y + 4*a^6*c^6*x^2*y + 14*a^4*b^2*c^6*x^2*y -      16*a^2*b^4*c^6*x^2*y + 10*b^6*c^6*x^2*y - 9*a^4*c^8*x^2*y + a^2*b^2*c^8*x^2*y + 2*a^2*c^10*x^2*y -      3*b^2*c^10*x^2*y + c^12*x^2*y + 2*a^12*x*y^2 - a^10*b^2*x*y^2 - 5*a^8*b^4*x*y^2 + 2*a^6*b^6*x*y^2 +      4*a^4*b^8*x*y^2 - a^2*b^10*x*y^2 - b^12*x*y^2 - 9*a^10*c^2*x*y^2 + 10*a^8*b^2*c^2*x*y^2 +      14*a^6*b^4*c^2*x*y^2 - 16*a^4*b^6*c^2*x*y^2 - 5*a^2*b^8*c^2*x*y^2 + 6*b^10*c^2*x*y^2 +      15*a^8*c^4*x*y^2 - 22*a^6*b^2*c^4*x*y^2 - 4*a^4*b^4*c^4*x*y^2 + 18*a^2*b^6*c^4*x*y^2 -      7*b^8*c^4*x*y^2 - 10*a^6*c^6*x*y^2 + 16*a^4*b^2*c^6*x*y^2 - 14*a^2*b^4*c^6*x*y^2 -      4*b^6*c^6*x*y^2 - a^2*b^2*c^8*x*y^2 + 9*b^4*c^8*x*y^2 + 3*a^2*c^10*x*y^2 - 2*b^2*c^10*x*y^2 -      c^12*x*y^2 + a^12*y^3 - a^10*b^2*y^3 - 2*a^8*b^4*y^3 + 2*a^6*b^6*y^3 + a^4*b^8*y^3 - a^2*b^10*y^3 -      4*a^10*c^2*y^3 + 5*a^8*b^2*c^2*y^3 + 4*a^6*b^4*c^2*y^3 - 6*a^4*b^6*c^2*y^3 + b^10*c^2*y^3 +      5*a^8*c^4*y^3 - 10*a^6*b^2*c^4*y^3 + 6*a^2*b^6*c^4*y^3 - b^8*c^4*y^3 + 10*a^4*b^2*c^6*y^3 -      4*a^2*b^4*c^6*y^3 - 2*b^6*c^6*y^3 - 5*a^4*c^8*y^3 - 5*a^2*b^2*c^8*y^3 + 2*b^4*c^8*y^3 +      4*a^2*c^10*y^3 + b^2*c^10*y^3 - c^12*y^3 - a^12*x^2*z + 6*a^10*b^2*x^2*z - 7*a^8*b^4*x^2*z -      4*a^6*b^6*x^2*z + 9*a^4*b^8*x^2*z - 2*a^2*b^10*x^2*z - b^12*x^2*z - a^10*c^2*x^2*z -      5*a^8*b^2*c^2*x^2*z + 18*a^6*b^4*c^2*x^2*z - 14*a^4*b^6*c^2*x^2*z - a^2*b^8*c^2*x^2*z +      3*b^10*c^2*x^2*z + 4*a^8*c^4*x^2*z - 16*a^6*b^2*c^4*x^2*z - 4*a^4*b^4*c^4*x^2*z +      16*a^2*b^6*c^4*x^2*z + 2*a^6*c^6*x^2*z + 14*a^4*b^2*c^6*x^2*z - 22*a^2*b^4*c^6*x^2*z -      10*b^6*c^6*x^2*z - 5*a^4*c^8*x^2*z + 10*a^2*b^2*c^8*x^2*z + 15*b^4*c^8*x^2*z - a^2*c^10*x^2*z -      9*b^2*c^10*x^2*z + 2*c^12*x^2*z + 8*a^10*b^2*x*y*z - 16*a^8*b^4*x*y*z + 16*a^4*b^8*x*y*z -      8*a^2*b^10*x*y*z - 8*a^10*c^2*x*y*z + 40*a^6*b^4*c^2*x*y*z - 40*a^4*b^6*c^2*x*y*z +      8*b^10*c^2*x*y*z + 16*a^8*c^4*x*y*z - 40*a^6*b^2*c^4*x*y*z + 40*a^2*b^6*c^4*x*y*z -      16*b^8*c^4*x*y*z + 40*a^4*b^2*c^6*x*y*z - 40*a^2*b^4*c^6*x*y*z - 16*a^4*c^8*x*y*z +      16*b^4*c^8*x*y*z + 8*a^2*c^10*x*y*z - 8*b^2*c^10*x*y*z + a^12*y^2*z + 2*a^10*b^2*y^2*z -      9*a^8*b^4*y^2*z + 4*a^6*b^6*y^2*z + 7*a^4*b^8*y^2*z - 6*a^2*b^10*y^2*z + b^12*y^2*z -      3*a^10*c^2*y^2*z + a^8*b^2*c^2*y^2*z + 14*a^6*b^4*c^2*y^2*z - 18*a^4*b^6*c^2*y^2*z +      5*a^2*b^8*c^2*y^2*z + b^10*c^2*y^2*z - 16*a^6*b^2*c^4*y^2*z + 4*a^4*b^4*c^4*y^2*z +      16*a^2*b^6*c^4*y^2*z - 4*b^8*c^4*y^2*z + 10*a^6*c^6*y^2*z + 22*a^4*b^2*c^6*y^2*z -      14*a^2*b^4*c^6*y^2*z - 2*b^6*c^6*y^2*z - 15*a^4*c^8*y^2*z - 10*a^2*b^2*c^8*y^2*z +      5*b^4*c^8*y^2*z + 9*a^2*c^10*y^2*z + b^2*c^10*y^2*z - 2*c^12*y^2*z - 2*a^12*x*z^2 +      9*a^10*b^2*x*z^2 - 15*a^8*b^4*x*z^2 + 10*a^6*b^6*x*z^2 - 3*a^2*b^10*x*z^2 + b^12*x*z^2 +      a^10*c^2*x*z^2 - 10*a^8*b^2*c^2*x*z^2 + 22*a^6*b^4*c^2*x*z^2 - 16*a^4*b^6*c^2*x*z^2 +      a^2*b^8*c^2*x*z^2 + 2*b^10*c^2*x*z^2 + 5*a^8*c^4*x*z^2 - 14*a^6*b^2*c^4*x*z^2 +      4*a^4*b^4*c^4*x*z^2 + 14*a^2*b^6*c^4*x*z^2 - 9*b^8*c^4*x*z^2 - 2*a^6*c^6*x*z^2 +      16*a^4*b^2*c^6*x*z^2 - 18*a^2*b^4*c^6*x*z^2 + 4*b^6*c^6*x*z^2 - 4*a^4*c^8*x*z^2 +      5*a^2*b^2*c^8*x*z^2 + 7*b^4*c^8*x*z^2 + a^2*c^10*x*z^2 - 6*b^2*c^10*x*z^2 + c^12*x*z^2 -      a^12*y*z^2 + 3*a^10*b^2*y*z^2 - 10*a^6*b^6*y*z^2 + 15*a^4*b^8*y*z^2 - 9*a^2*b^10*y*z^2 +      2*b^12*y*z^2 - 2*a^10*c^2*y*z^2 - a^8*b^2*c^2*y*z^2 + 16*a^6*b^4*c^2*y*z^2 - 22*a^4*b^6*c^2*y*z^2 +      10*a^2*b^8*c^2*y*z^2 - b^10*c^2*y*z^2 + 9*a^8*c^4*y*z^2 - 14*a^6*b^2*c^4*y*z^2 -      4*a^4*b^4*c^4*y*z^2 + 14*a^2*b^6*c^4*y*z^2 - 5*b^8*c^4*y*z^2 - 4*a^6*c^6*y*z^2 +      18*a^4*b^2*c^6*y*z^2 - 16*a^2*b^4*c^6*y*z^2 + 2*b^6*c^6*y*z^2 - 7*a^4*c^8*y*z^2 -      5*a^2*b^2*c^8*y*z^2 + 4*b^4*c^8*y*z^2 + 6*a^2*c^10*y*z^2 - b^2*c^10*y*z^2 - c^12*y*z^2 - a^12*z^3 +      4*a^10*b^2*z^3 - 5*a^8*b^4*z^3 + 5*a^4*b^8*z^3 - 4*a^2*b^10*z^3 + b^12*z^3 + a^10*c^2*z^3 -      5*a^8*b^2*c^2*z^3 + 10*a^6*b^4*c^2*z^3 - 10*a^4*b^6*c^2*z^3 + 5*a^2*b^8*c^2*z^3 - b^10*c^2*z^3 +      2*a^8*c^4*z^3 - 4*a^6*b^2*c^4*z^3 + 4*a^2*b^6*c^4*z^3 - 2*b^8*c^4*z^3 - 2*a^6*c^6*z^3 +      6*a^4*b^2*c^6*z^3 - 6*a^2*b^4*c^6*z^3 + 2*b^6*c^6*z^3 - a^4*c^8*z^3 + b^4*c^8*z^3 + a^2*c^10*z^3 -      b^2*c^10*z^3");
		exp("K402","b^4*c^6*x^2*y - b^2*c^8*x^2*y + a^4*c^6*x*y^2 - a^2*c^8*x*y^2 - b^8*c^2*x^2*z + b^6*c^4*x^2*z +      2*a^4*b^4*c^2*x*y*z + 2*a^4*b^2*c^4*x*y*z + 2*a^2*b^4*c^4*x*y*z - a^8*c^2*y^2*z + a^6*c^4*y^2*z +      a^4*b^6*x*z^2 - a^2*b^8*x*z^2 - a^8*b^2*y*z^2 + a^6*b^4*y*z^2");
		exp("K403","b^2*c^2*x^2*y - c^4*x^2*y + a^2*c^2*x*y^2 - c^4*x*y^2 - b^4*x^2*z + b^2*c^2*x^2*z -      2*a^2*b^2*x*y*z - 2*a^2*c^2*x*y*z - 2*b^2*c^2*x*y*z - a^4*y^2*z + a^2*c^2*y^2*z + a^2*b^2*x*z^2 -      b^4*x*z^2 - a^4*y*z^2 + a^2*b^2*y*z^2");
		exp("K404","(-a^10)*b^2*c^2*x^2*y + 3*a^8*b^4*c^2*x^2*y - 4*a^6*b^6*c^2*x^2*y + 4*a^4*b^8*c^2*x^2*y -      3*a^2*b^10*c^2*x^2*y + b^12*c^2*x^2*y + a^8*b^2*c^4*x^2*y - a^6*b^4*c^4*x^2*y -      3*a^4*b^6*c^4*x^2*y + 7*a^2*b^8*c^4*x^2*y - 4*b^10*c^4*x^2*y - 6*a^2*b^6*c^6*x^2*y +      6*b^8*c^6*x^2*y + a^4*b^2*c^8*x^2*y + 3*a^2*b^4*c^8*x^2*y - 4*b^6*c^8*x^2*y - a^2*b^2*c^10*x^2*y +      b^4*c^10*x^2*y - a^12*c^2*x*y^2 + 3*a^10*b^2*c^2*x*y^2 - 4*a^8*b^4*c^2*x*y^2 +      4*a^6*b^6*c^2*x*y^2 - 3*a^4*b^8*c^2*x*y^2 + a^2*b^10*c^2*x*y^2 + 4*a^10*c^4*x*y^2 -      7*a^8*b^2*c^4*x*y^2 + 3*a^6*b^4*c^4*x*y^2 + a^4*b^6*c^4*x*y^2 - a^2*b^8*c^4*x*y^2 -      6*a^8*c^6*x*y^2 + 6*a^6*b^2*c^6*x*y^2 + 4*a^6*c^8*x*y^2 - 3*a^4*b^2*c^8*x*y^2 - a^2*b^4*c^8*x*y^2 -      a^4*c^10*x*y^2 + a^2*b^2*c^10*x*y^2 + a^10*b^2*c^2*x^2*z - a^8*b^4*c^2*x^2*z - a^4*b^8*c^2*x^2*z +      a^2*b^10*c^2*x^2*z - 3*a^8*b^2*c^4*x^2*z + a^6*b^4*c^4*x^2*z - 3*a^2*b^8*c^4*x^2*z -      b^10*c^4*x^2*z + 4*a^6*b^2*c^6*x^2*z + 3*a^4*b^4*c^6*x^2*z + 6*a^2*b^6*c^6*x^2*z +      4*b^8*c^6*x^2*z - 4*a^4*b^2*c^8*x^2*z - 7*a^2*b^4*c^8*x^2*z - 6*b^6*c^8*x^2*z +      3*a^2*b^2*c^10*x^2*z + 4*b^4*c^10*x^2*z - b^2*c^12*x^2*z - a^10*b^2*c^2*y^2*z + a^8*b^4*c^2*y^2*z +      a^4*b^8*c^2*y^2*z - a^2*b^10*c^2*y^2*z + a^10*c^4*y^2*z + 3*a^8*b^2*c^4*y^2*z - a^4*b^6*c^4*y^2*z +      3*a^2*b^8*c^4*y^2*z - 4*a^8*c^6*y^2*z - 6*a^6*b^2*c^6*y^2*z - 3*a^4*b^4*c^6*y^2*z -      4*a^2*b^6*c^6*y^2*z + 6*a^6*c^8*y^2*z + 7*a^4*b^2*c^8*y^2*z + 4*a^2*b^4*c^8*y^2*z -      4*a^4*c^10*y^2*z - 3*a^2*b^2*c^10*y^2*z + a^2*c^12*y^2*z + a^12*b^2*x*z^2 - 4*a^10*b^4*x*z^2 +      6*a^8*b^6*x*z^2 - 4*a^6*b^8*x*z^2 + a^4*b^10*x*z^2 - 3*a^10*b^2*c^2*x*z^2 + 7*a^8*b^4*c^2*x*z^2 -      6*a^6*b^6*c^2*x*z^2 + 3*a^4*b^8*c^2*x*z^2 - a^2*b^10*c^2*x*z^2 + 4*a^8*b^2*c^4*x*z^2 -      3*a^6*b^4*c^4*x*z^2 + a^2*b^8*c^4*x*z^2 - 4*a^6*b^2*c^6*x*z^2 - a^4*b^4*c^6*x*z^2 +      3*a^4*b^2*c^8*x*z^2 + a^2*b^4*c^8*x*z^2 - a^2*b^2*c^10*x*z^2 - a^10*b^4*y*z^2 + 4*a^8*b^6*y*z^2 -      6*a^6*b^8*y*z^2 + 4*a^4*b^10*y*z^2 - a^2*b^12*y*z^2 + a^10*b^2*c^2*y*z^2 - 3*a^8*b^4*c^2*y*z^2 +      6*a^6*b^6*c^2*y*z^2 - 7*a^4*b^8*c^2*y*z^2 + 3*a^2*b^10*c^2*y*z^2 - a^8*b^2*c^4*y*z^2 +      3*a^4*b^6*c^4*y*z^2 - 4*a^2*b^8*c^4*y*z^2 + a^4*b^4*c^6*y*z^2 + 4*a^2*b^6*c^6*y*z^2 -      a^4*b^2*c^8*y*z^2 - 3*a^2*b^4*c^8*y*z^2 + a^2*b^2*c^10*y*z^2");
		exp("K405","2*a^2*b^2*c^2*x^2*y - 2*b^4*c^2*x^2*y - a^2*c^4*x^2*y + b^2*c^4*x^2*y + c^6*x^2*y +      2*a^4*c^2*x*y^2 - 2*a^2*b^2*c^2*x*y^2 - a^2*c^4*x*y^2 + b^2*c^4*x*y^2 - c^6*x*y^2 + a^2*b^4*x^2*z -      b^6*x^2*z - 2*a^2*b^2*c^2*x^2*z - b^4*c^2*x^2*z + 2*b^2*c^4*x^2*z + 2*a^4*b^2*x*y*z -      2*a^2*b^4*x*y*z - 2*a^4*c^2*x*y*z + 2*b^4*c^2*x*y*z + 2*a^2*c^4*x*y*z - 2*b^2*c^4*x*y*z +      a^6*y^2*z - a^4*b^2*y^2*z + a^4*c^2*y^2*z + 2*a^2*b^2*c^2*y^2*z - 2*a^2*c^4*y^2*z -      2*a^4*b^2*x*z^2 + a^2*b^4*x*z^2 + b^6*x*z^2 + 2*a^2*b^2*c^2*x*z^2 - b^4*c^2*x*z^2 - a^6*y*z^2 -      a^4*b^2*y*z^2 + 2*a^2*b^4*y*z^2 + a^4*c^2*y*z^2 - 2*a^2*b^2*c^2*y*z^2");
		exp("K406","a^8*x^2*y - 4*a^6*b^2*x^2*y + 6*a^4*b^4*x^2*y - 4*a^2*b^6*x^2*y + b^8*x^2*y - 2*a^4*c^4*x^2*y +      4*a^2*b^2*c^4*x^2*y - 2*b^4*c^4*x^2*y + c^8*x^2*y + a^8*x*y^2 - 4*a^6*b^2*x*y^2 + 6*a^4*b^4*x*y^2 -      4*a^2*b^6*x*y^2 + b^8*x*y^2 - 2*a^4*c^4*x*y^2 + 4*a^2*b^2*c^4*x*y^2 - 2*b^4*c^4*x*y^2 + c^8*x*y^2 +      a^8*x^2*z - 2*a^4*b^4*x^2*z + b^8*x^2*z - 4*a^6*c^2*x^2*z + 4*a^2*b^4*c^2*x^2*z + 6*a^4*c^4*x^2*z -      2*b^4*c^4*x^2*z - 4*a^2*c^6*x^2*z + c^8*x^2*z + 2*a^8*x*y*z - 4*a^4*b^4*x*y*z + 2*b^8*x*y*z -      4*a^4*c^4*x*y*z - 4*b^4*c^4*x*y*z + 2*c^8*x*y*z + a^8*y^2*z - 2*a^4*b^4*y^2*z + b^8*y^2*z +      4*a^4*b^2*c^2*y^2*z - 4*b^6*c^2*y^2*z - 2*a^4*c^4*y^2*z + 6*b^4*c^4*y^2*z - 4*b^2*c^6*y^2*z +      c^8*y^2*z + a^8*x*z^2 - 2*a^4*b^4*x*z^2 + b^8*x*z^2 - 4*a^6*c^2*x*z^2 + 4*a^2*b^4*c^2*x*z^2 +      6*a^4*c^4*x*z^2 - 2*b^4*c^4*x*z^2 - 4*a^2*c^6*x*z^2 + c^8*x*z^2 + a^8*y*z^2 - 2*a^4*b^4*y*z^2 +      b^8*y*z^2 + 4*a^4*b^2*c^2*y*z^2 - 4*b^6*c^2*y*z^2 - 2*a^4*c^4*y*z^2 + 6*b^4*c^4*y*z^2 -      4*b^2*c^6*y*z^2 + c^8*y*z^2");
		exp("K407","(-a^2)*b*c^2*x^2*y + b^3*c^2*x^2*y + 2*a*b*c^3*x^2*y - b*c^4*x^2*y + a^3*c^2*x*y^2 -      a*b^2*c^2*x*y^2 + 2*a*b*c^3*x*y^2 - a*c^4*x*y^2 - a^2*b^2*c*x^2*z + 2*a*b^3*c*x^2*z - b^4*c*x^2*z +      b^2*c^3*x^2*z + 2*a^3*b*c*x*y*z - 4*a^2*b^2*c*x*y*z + 2*a*b^3*c*x*y*z - 4*a^2*b*c^2*x*y*z -      4*a*b^2*c^2*x*y*z + 2*a*b*c^3*x*y*z - a^4*c*y^2*z + 2*a^3*b*c*y^2*z - a^2*b^2*c*y^2*z +      a^2*c^3*y^2*z + a^3*b^2*x*z^2 - a*b^4*x*z^2 + 2*a*b^3*c*x*z^2 - a*b^2*c^2*x*z^2 - a^4*b*y*z^2 +      a^2*b^3*y*z^2 + 2*a^3*b*c*y*z^2 - a^2*b*c^2*y*z^2");
		exp("K408","a^2*x^2*y - 5*b^2*x^2*y + c^2*x^2*y - 5*a^2*x*y^2 + b^2*x*y^2 + c^2*x*y^2 + a^2*x^2*z +      b^2*x^2*z - 5*c^2*x^2*z + 6*a^2*x*y*z + 6*b^2*x*y*z + 6*c^2*x*y*z + a^2*y^2*z + b^2*y^2*z -      5*c^2*y^2*z - 5*a^2*x*z^2 + b^2*x*z^2 + c^2*x*z^2 + a^2*y*z^2 - 5*b^2*y*z^2 + c^2*y*z^2");
		exp("K409","2*(a^2*b^2 + a^2*c^2 + b^2*c^2)*x*y*z + (b^4*x^2 + a^4*y^2)*z + y*(c^4*x^2 + a^4*z^2) +      x*(c^4*y^2 + b^4*z^2)");
		exp("K410","a^2*b^2*c^2*x^2*y - a^2*c^4*x^2*y + b^2*c^4*x^2*y - a^2*b^2*c^2*x*y^2 - a^2*c^4*x*y^2 +      b^2*c^4*x*y^2 + a^2*b^4*x^2*z - a^2*b^2*c^2*x^2*z - b^4*c^2*x^2*z - a^4*b^2*y^2*z + a^4*c^2*y^2*z +      a^2*b^2*c^2*y^2*z + a^2*b^4*x*z^2 + a^2*b^2*c^2*x*z^2 - b^4*c^2*x*z^2 - a^4*b^2*y*z^2 +      a^4*c^2*y*z^2 - a^2*b^2*c^2*y*z^2");
		exp("K411","a^2*b^4*c^6*x^2*y + b^6*c^6*x^2*y - b^4*c^8*x^2*y - a^6*c^6*x*y^2 - a^4*b^2*c^6*x*y^2 +      a^4*c^8*x*y^2 - a^2*b^6*c^4*x^2*z + b^8*c^4*x^2*z - b^6*c^6*x^2*z - a^8*c^4*y^2*z +      a^6*b^2*c^4*y^2*z + a^6*c^6*y^2*z + a^6*b^6*x*z^2 - a^4*b^8*x*z^2 + a^4*b^6*c^2*x*z^2 +      a^8*b^4*y*z^2 - a^6*b^6*y*z^2 - a^6*b^4*c^2*y*z^2");
		exp("K412","a^6*x^2*y - 3*a^4*b^2*x^2*y + 3*a^2*b^4*x^2*y - b^6*x^2*y - 4*a^4*c^2*x^2*y +      6*a^2*b^2*c^2*x^2*y - 2*b^4*c^2*x^2*y + 3*a^2*c^4*x^2*y + 3*b^2*c^4*x^2*y + a^6*x*y^2 -      3*a^4*b^2*x*y^2 + 3*a^2*b^4*x*y^2 - b^6*x*y^2 + 2*a^4*c^2*x*y^2 - 6*a^2*b^2*c^2*x*y^2 +      4*b^4*c^2*x*y^2 - 3*a^2*c^4*x*y^2 - 3*b^2*c^4*x*y^2 - a^6*x^2*z + 4*a^4*b^2*x^2*z -      3*a^2*b^4*x^2*z + 3*a^4*c^2*x^2*z - 6*a^2*b^2*c^2*x^2*z - 3*b^4*c^2*x^2*z - 3*a^2*c^4*x^2*z +      2*b^2*c^4*x^2*z + c^6*x^2*z + 4*a^4*b^2*x*y*z - 4*a^2*b^4*x*y*z - 4*a^4*c^2*x*y*z +      4*b^4*c^2*x*y*z + 4*a^2*c^4*x*y*z - 4*b^2*c^4*x*y*z + 3*a^4*b^2*y^2*z - 4*a^2*b^4*y^2*z +      b^6*y^2*z + 3*a^4*c^2*y^2*z + 6*a^2*b^2*c^2*y^2*z - 3*b^4*c^2*y^2*z - 2*a^2*c^4*y^2*z +      3*b^2*c^4*y^2*z - c^6*y^2*z - a^6*x*z^2 - 2*a^4*b^2*x*z^2 + 3*a^2*b^4*x*z^2 + 3*a^4*c^2*x*z^2 +      6*a^2*b^2*c^2*x*z^2 + 3*b^4*c^2*x*z^2 - 3*a^2*c^4*x*z^2 - 4*b^2*c^4*x*z^2 + c^6*x*z^2 -      3*a^4*b^2*y*z^2 + 2*a^2*b^4*y*z^2 + b^6*y*z^2 - 3*a^4*c^2*y*z^2 - 6*a^2*b^2*c^2*y*z^2 -      3*b^4*c^2*y*z^2 + 4*a^2*c^4*y*z^2 + 3*b^2*c^4*y*z^2 - c^6*y*z^2");
		exp("K413","a^10*b^2*x^2*y - 5*a^8*b^4*x^2*y + 10*a^6*b^6*x^2*y - 10*a^4*b^8*x^2*y + 5*a^2*b^10*x^2*y -      b^12*x^2*y - 3*a^8*b^2*c^2*x^2*y + 4*a^6*b^4*c^2*x^2*y + 6*a^4*b^6*c^2*x^2*y -      12*a^2*b^8*c^2*x^2*y + 5*b^10*c^2*x^2*y + a^6*b^2*c^4*x^2*y + a^4*b^4*c^4*x^2*y +      7*a^2*b^6*c^4*x^2*y - 9*b^8*c^4*x^2*y + 3*a^4*b^2*c^6*x^2*y + 2*a^2*b^4*c^6*x^2*y +      7*b^6*c^6*x^2*y - 2*a^2*b^2*c^8*x^2*y - 2*b^4*c^8*x^2*y + a^12*x*y^2 - 5*a^10*b^2*x*y^2 +      10*a^8*b^4*x*y^2 - 10*a^6*b^6*x*y^2 + 5*a^4*b^8*x*y^2 - a^2*b^10*x*y^2 - 5*a^10*c^2*x*y^2 +      12*a^8*b^2*c^2*x*y^2 - 6*a^6*b^4*c^2*x*y^2 - 4*a^4*b^6*c^2*x*y^2 + 3*a^2*b^8*c^2*x*y^2 +      9*a^8*c^4*x*y^2 - 7*a^6*b^2*c^4*x*y^2 - a^4*b^4*c^4*x*y^2 - a^2*b^6*c^4*x*y^2 - 7*a^6*c^6*x*y^2 -      2*a^4*b^2*c^6*x*y^2 - 3*a^2*b^4*c^6*x*y^2 + 2*a^4*c^8*x*y^2 + 2*a^2*b^2*c^8*x*y^2 -      a^10*c^2*x^2*z + 3*a^8*b^2*c^2*x^2*z - a^6*b^4*c^2*x^2*z - 3*a^4*b^6*c^2*x^2*z +      2*a^2*b^8*c^2*x^2*z + 5*a^8*c^4*x^2*z - 4*a^6*b^2*c^4*x^2*z - a^4*b^4*c^4*x^2*z -      2*a^2*b^6*c^4*x^2*z + 2*b^8*c^4*x^2*z - 10*a^6*c^6*x^2*z - 6*a^4*b^2*c^6*x^2*z -      7*a^2*b^4*c^6*x^2*z - 7*b^6*c^6*x^2*z + 10*a^4*c^8*x^2*z + 12*a^2*b^2*c^8*x^2*z + 9*b^4*c^8*x^2*z -      5*a^2*c^10*x^2*z - 5*b^2*c^10*x^2*z + c^12*x^2*z - 2*a^8*b^2*c^2*y^2*z + 3*a^6*b^4*c^2*y^2*z +      a^4*b^6*c^2*y^2*z - 3*a^2*b^8*c^2*y^2*z + b^10*c^2*y^2*z - 2*a^8*c^4*y^2*z + 2*a^6*b^2*c^4*y^2*z +      a^4*b^4*c^4*y^2*z + 4*a^2*b^6*c^4*y^2*z - 5*b^8*c^4*y^2*z + 7*a^6*c^6*y^2*z + 7*a^4*b^2*c^6*y^2*z +      6*a^2*b^4*c^6*y^2*z + 10*b^6*c^6*y^2*z - 9*a^4*c^8*y^2*z - 12*a^2*b^2*c^8*y^2*z -      10*b^4*c^8*y^2*z + 5*a^2*c^10*y^2*z + 5*b^2*c^10*y^2*z - c^12*y^2*z - a^12*x*z^2 +      5*a^10*b^2*x*z^2 - 9*a^8*b^4*x*z^2 + 7*a^6*b^6*x*z^2 - 2*a^4*b^8*x*z^2 + 5*a^10*c^2*x*z^2 -      12*a^8*b^2*c^2*x*z^2 + 7*a^6*b^4*c^2*x*z^2 + 2*a^4*b^6*c^2*x*z^2 - 2*a^2*b^8*c^2*x*z^2 -      10*a^8*c^4*x*z^2 + 6*a^6*b^2*c^4*x*z^2 + a^4*b^4*c^4*x*z^2 + 3*a^2*b^6*c^4*x*z^2 +      10*a^6*c^6*x*z^2 + 4*a^4*b^2*c^6*x*z^2 + a^2*b^4*c^6*x*z^2 - 5*a^4*c^8*x*z^2 -      3*a^2*b^2*c^8*x*z^2 + a^2*c^10*x*z^2 + 2*a^8*b^4*y*z^2 - 7*a^6*b^6*y*z^2 + 9*a^4*b^8*y*z^2 -      5*a^2*b^10*y*z^2 + b^12*y*z^2 + 2*a^8*b^2*c^2*y*z^2 - 2*a^6*b^4*c^2*y*z^2 - 7*a^4*b^6*c^2*y*z^2 +      12*a^2*b^8*c^2*y*z^2 - 5*b^10*c^2*y*z^2 - 3*a^6*b^2*c^4*y*z^2 - a^4*b^4*c^4*y*z^2 -      6*a^2*b^6*c^4*y*z^2 + 10*b^8*c^4*y*z^2 - a^4*b^2*c^6*y*z^2 - 4*a^2*b^4*c^6*y*z^2 -      10*b^6*c^6*y*z^2 + 3*a^2*b^2*c^8*y*z^2 + 5*b^4*c^8*y*z^2 - b^2*c^10*y*z^2");
		exp("K414","a^3*b*c^2*x^2*y + a^2*b^2*c^2*x^2*y - a*b^3*c^2*x^2*y - b^4*c^2*x^2*y - a^2*b*c^3*x^2*y +      2*a*b^2*c^3*x^2*y - b^3*c^3*x^2*y - a*b*c^4*x^2*y + b^2*c^4*x^2*y + b*c^5*x^2*y + a^4*c^2*x*y^2 +      a^3*b*c^2*x*y^2 - a^2*b^2*c^2*x*y^2 - a*b^3*c^2*x*y^2 + a^3*c^3*x*y^2 - 2*a^2*b*c^3*x*y^2 +      a*b^2*c^3*x*y^2 - a^2*c^4*x*y^2 + a*b*c^4*x*y^2 - a*c^5*x*y^2 - a^3*b^2*c*x^2*z + a^2*b^3*c*x^2*z +      a*b^4*c*x^2*z - b^5*c*x^2*z - a^2*b^2*c^2*x^2*z - 2*a*b^3*c^2*x^2*z - b^4*c^2*x^2*z +      a*b^2*c^3*x^2*z + b^3*c^3*x^2*z + b^2*c^4*x^2*z + a^5*c*y^2*z - a^4*b*c*y^2*z - a^3*b^2*c*y^2*z +      a^2*b^3*c*y^2*z + a^4*c^2*y^2*z + 2*a^3*b*c^2*y^2*z + a^2*b^2*c^2*y^2*z - a^3*c^3*y^2*z -      a^2*b*c^3*y^2*z - a^2*c^4*y^2*z - a^4*b^2*x*z^2 - a^3*b^3*x*z^2 + a^2*b^4*x*z^2 + a*b^5*x*z^2 -      a^3*b^2*c*x*z^2 + 2*a^2*b^3*c*x*z^2 - a*b^4*c*x*z^2 + a^2*b^2*c^2*x*z^2 - a*b^3*c^2*x*z^2 +      a*b^2*c^3*x*z^2 - a^5*b*y*z^2 - a^4*b^2*y*z^2 + a^3*b^3*y*z^2 + a^2*b^4*y*z^2 + a^4*b*c*y*z^2 -      2*a^3*b^2*c*y*z^2 + a^2*b^3*c*y*z^2 + a^3*b*c^2*y*z^2 - a^2*b^2*c^2*y*z^2 - a^2*b*c^3*y*z^2");
		exp("K415","(-a^10)*c^2*x^2*y + 3*a^8*b^2*c^2*x^2*y - 4*a^6*b^4*c^2*x^2*y + 4*a^4*b^6*c^2*x^2*y -      3*a^2*b^8*c^2*x^2*y + b^10*c^2*x^2*y + 4*a^8*c^4*x^2*y - 6*a^6*b^2*c^4*x^2*y +      2*a^4*b^4*c^4*x^2*y + 2*a^2*b^6*c^4*x^2*y - 2*b^8*c^4*x^2*y - 6*a^6*c^6*x^2*y +      2*a^4*b^2*c^6*x^2*y + 4*a^4*c^8*x^2*y + 2*a^2*b^2*c^8*x^2*y + 2*b^4*c^8*x^2*y - a^2*c^10*x^2*y -      b^2*c^10*x^2*y - a^10*c^2*x*y^2 + 3*a^8*b^2*c^2*x*y^2 - 4*a^6*b^4*c^2*x*y^2 + 4*a^4*b^6*c^2*x*y^2 -      3*a^2*b^8*c^2*x*y^2 + b^10*c^2*x*y^2 + 2*a^8*c^4*x*y^2 - 2*a^6*b^2*c^4*x*y^2 -      2*a^4*b^4*c^4*x*y^2 + 6*a^2*b^6*c^4*x*y^2 - 4*b^8*c^4*x*y^2 - 2*a^2*b^4*c^6*x*y^2 +      6*b^6*c^6*x*y^2 - 2*a^4*c^8*x*y^2 - 2*a^2*b^2*c^8*x*y^2 - 4*b^4*c^8*x*y^2 + a^2*c^10*x*y^2 +      b^2*c^10*x*y^2 + a^10*b^2*x^2*z - 4*a^8*b^4*x^2*z + 6*a^6*b^6*x^2*z - 4*a^4*b^8*x^2*z +      a^2*b^10*x^2*z - 3*a^8*b^2*c^2*x^2*z + 6*a^6*b^4*c^2*x^2*z - 2*a^4*b^6*c^2*x^2*z -      2*a^2*b^8*c^2*x^2*z + b^10*c^2*x^2*z + 4*a^6*b^2*c^4*x^2*z - 2*a^4*b^4*c^4*x^2*z -      2*b^8*c^4*x^2*z - 4*a^4*b^2*c^6*x^2*z - 2*a^2*b^4*c^6*x^2*z + 3*a^2*b^2*c^8*x^2*z +      2*b^4*c^8*x^2*z - b^2*c^10*x^2*z - a^10*b^2*y^2*z + 4*a^8*b^4*y^2*z - 6*a^6*b^6*y^2*z +      4*a^4*b^8*y^2*z - a^2*b^10*y^2*z - a^10*c^2*y^2*z + 2*a^8*b^2*c^2*y^2*z + 2*a^6*b^4*c^2*y^2*z -      6*a^4*b^6*c^2*y^2*z + 3*a^2*b^8*c^2*y^2*z + 2*a^8*c^4*y^2*z + 2*a^4*b^4*c^4*y^2*z -      4*a^2*b^6*c^4*y^2*z + 2*a^4*b^2*c^6*y^2*z + 4*a^2*b^4*c^6*y^2*z - 2*a^4*c^8*y^2*z -      3*a^2*b^2*c^8*y^2*z + a^2*c^10*y^2*z + a^10*b^2*x*z^2 - 2*a^8*b^4*x*z^2 + 2*a^4*b^8*x*z^2 -      a^2*b^10*x*z^2 - 3*a^8*b^2*c^2*x*z^2 + 2*a^6*b^4*c^2*x*z^2 + 2*a^2*b^8*c^2*x*z^2 - b^10*c^2*x*z^2 +      4*a^6*b^2*c^4*x*z^2 + 2*a^4*b^4*c^4*x*z^2 + 2*a^2*b^6*c^4*x*z^2 + 4*b^8*c^4*x*z^2 -      4*a^4*b^2*c^6*x*z^2 - 6*a^2*b^4*c^6*x*z^2 - 6*b^6*c^6*x*z^2 + 3*a^2*b^2*c^8*x*z^2 +      4*b^4*c^8*x*z^2 - b^2*c^10*x*z^2 + a^10*b^2*y*z^2 - 2*a^8*b^4*y*z^2 + 2*a^4*b^8*y*z^2 -      a^2*b^10*y*z^2 + a^10*c^2*y*z^2 - 2*a^8*b^2*c^2*y*z^2 - 2*a^4*b^6*c^2*y*z^2 + 3*a^2*b^8*c^2*y*z^2 -      4*a^8*c^4*y*z^2 - 2*a^6*b^2*c^4*y*z^2 - 2*a^4*b^4*c^4*y*z^2 - 4*a^2*b^6*c^4*y*z^2 +      6*a^6*c^6*y*z^2 + 6*a^4*b^2*c^6*y*z^2 + 4*a^2*b^4*c^6*y*z^2 - 4*a^4*c^8*y*z^2 -      3*a^2*b^2*c^8*y*z^2 + a^2*c^10*y*z^2");
		exp("K416","(-a^10)*c^2*x^2*y + 4*a^8*b^2*c^2*x^2*y - 7*a^6*b^4*c^2*x^2*y + 7*a^4*b^6*c^2*x^2*y -      4*a^2*b^8*c^2*x^2*y + b^10*c^2*x^2*y + 4*a^8*c^4*x^2*y - 8*a^6*b^2*c^4*x^2*y +      4*a^4*b^4*c^4*x^2*y + 2*a^2*b^6*c^4*x^2*y - 2*b^8*c^4*x^2*y - 6*a^6*c^6*x^2*y +      3*a^4*b^2*c^6*x^2*y + a^2*b^4*c^6*x^2*y + 4*a^4*c^8*x^2*y + 2*a^2*b^2*c^8*x^2*y + 2*b^4*c^8*x^2*y -      a^2*c^10*x^2*y - b^2*c^10*x^2*y - a^10*c^2*x*y^2 + 4*a^8*b^2*c^2*x*y^2 - 7*a^6*b^4*c^2*x*y^2 +      7*a^4*b^6*c^2*x*y^2 - 4*a^2*b^8*c^2*x*y^2 + b^10*c^2*x*y^2 + 2*a^8*c^4*x*y^2 -      2*a^6*b^2*c^4*x*y^2 - 4*a^4*b^4*c^4*x*y^2 + 8*a^2*b^6*c^4*x*y^2 - 4*b^8*c^4*x*y^2 -      a^4*b^2*c^6*x*y^2 - 3*a^2*b^4*c^6*x*y^2 + 6*b^6*c^6*x*y^2 - 2*a^4*c^8*x*y^2 - 2*a^2*b^2*c^8*x*y^2 -      4*b^4*c^8*x*y^2 + a^2*c^10*x*y^2 + b^2*c^10*x*y^2 + a^10*b^2*x^2*z - 4*a^8*b^4*x^2*z +      6*a^6*b^6*x^2*z - 4*a^4*b^8*x^2*z + a^2*b^10*x^2*z - 4*a^8*b^2*c^2*x^2*z + 8*a^6*b^4*c^2*x^2*z -      3*a^4*b^6*c^2*x^2*z - 2*a^2*b^8*c^2*x^2*z + b^10*c^2*x^2*z + 7*a^6*b^2*c^4*x^2*z -      4*a^4*b^4*c^4*x^2*z - a^2*b^6*c^4*x^2*z - 2*b^8*c^4*x^2*z - 7*a^4*b^2*c^6*x^2*z -      2*a^2*b^4*c^6*x^2*z + 4*a^2*b^2*c^8*x^2*z + 2*b^4*c^8*x^2*z - b^2*c^10*x^2*z - a^10*b^2*y^2*z +      4*a^8*b^4*y^2*z - 6*a^6*b^6*y^2*z + 4*a^4*b^8*y^2*z - a^2*b^10*y^2*z - a^10*c^2*y^2*z +      2*a^8*b^2*c^2*y^2*z + 3*a^6*b^4*c^2*y^2*z - 8*a^4*b^6*c^2*y^2*z + 4*a^2*b^8*c^2*y^2*z +      2*a^8*c^4*y^2*z + a^6*b^2*c^4*y^2*z + 4*a^4*b^4*c^4*y^2*z - 7*a^2*b^6*c^4*y^2*z +      2*a^4*b^2*c^6*y^2*z + 7*a^2*b^4*c^6*y^2*z - 2*a^4*c^8*y^2*z - 4*a^2*b^2*c^8*y^2*z +      a^2*c^10*y^2*z + a^10*b^2*x*z^2 - 2*a^8*b^4*x*z^2 + 2*a^4*b^8*x*z^2 - a^2*b^10*x*z^2 -      4*a^8*b^2*c^2*x*z^2 + 2*a^6*b^4*c^2*x*z^2 + a^4*b^6*c^2*x*z^2 + 2*a^2*b^8*c^2*x*z^2 -      b^10*c^2*x*z^2 + 7*a^6*b^2*c^4*x*z^2 + 4*a^4*b^4*c^4*x*z^2 + 3*a^2*b^6*c^4*x*z^2 +      4*b^8*c^4*x*z^2 - 7*a^4*b^2*c^6*x*z^2 - 8*a^2*b^4*c^6*x*z^2 - 6*b^6*c^6*x*z^2 +      4*a^2*b^2*c^8*x*z^2 + 4*b^4*c^8*x*z^2 - b^2*c^10*x*z^2 + a^10*b^2*y*z^2 - 2*a^8*b^4*y*z^2 +      2*a^4*b^8*y*z^2 - a^2*b^10*y*z^2 + a^10*c^2*y*z^2 - 2*a^8*b^2*c^2*y*z^2 - a^6*b^4*c^2*y*z^2 -      2*a^4*b^6*c^2*y*z^2 + 4*a^2*b^8*c^2*y*z^2 - 4*a^8*c^4*y*z^2 - 3*a^6*b^2*c^4*y*z^2 -      4*a^4*b^4*c^4*y*z^2 - 7*a^2*b^6*c^4*y*z^2 + 6*a^6*c^6*y*z^2 + 8*a^4*b^2*c^6*y*z^2 +      7*a^2*b^4*c^6*y*z^2 - 4*a^4*c^8*y*z^2 - 4*a^2*b^2*c^8*y*z^2 + a^2*c^10*y*z^2");
		exp("K417","(-a^2)*b^4*c^2*x^3 + b^6*c^2*x^3 + a^2*b^2*c^4*x^3 - b^2*c^6*x^3 + a^4*b^2*c^2*x^2*y -      2*a^2*b^4*c^2*x^2*y + b^6*c^2*x^2*y + a^2*b^2*c^4*x^2*y - b^4*c^4*x^2*y - a^6*c^2*x*y^2 +      2*a^4*b^2*c^2*x*y^2 - a^2*b^4*c^2*x*y^2 + a^4*c^4*x*y^2 - a^2*b^2*c^4*x*y^2 - a^6*c^2*y^3 +      a^4*b^2*c^2*y^3 - a^2*b^2*c^4*y^3 + a^2*c^6*y^3 - a^4*b^2*c^2*x^2*z - a^2*b^4*c^2*x^2*z +      2*a^2*b^2*c^4*x^2*z + b^4*c^4*x^2*z - b^2*c^6*x^2*z + a^4*b^2*c^2*y^2*z + a^2*b^4*c^2*y^2*z -      a^4*c^4*y^2*z - 2*a^2*b^2*c^4*y^2*z + a^2*c^6*y^2*z + a^6*b^2*x*z^2 - a^4*b^4*x*z^2 -      2*a^4*b^2*c^2*x*z^2 + a^2*b^4*c^2*x*z^2 + a^2*b^2*c^4*x*z^2 + a^4*b^4*y*z^2 - a^2*b^6*y*z^2 -      a^4*b^2*c^2*y*z^2 + 2*a^2*b^4*c^2*y*z^2 - a^2*b^2*c^4*y*z^2 + a^6*b^2*z^3 - a^2*b^6*z^3 -      a^4*b^2*c^2*z^3 + a^2*b^4*c^2*z^3");
		exp("K418","(-a^2)*b^4*c^2*x^3 + b^6*c^2*x^3 + a^2*b^2*c^4*x^3 - b^2*c^6*x^3 - 2*a^4*b^2*c^2*x^2*y +      a^2*b^4*c^2*x^2*y + b^6*c^2*x^2*y + a^2*b^2*c^4*x^2*y - 4*b^4*c^4*x^2*y + 3*b^2*c^6*x^2*y -      a^6*c^2*x*y^2 - a^4*b^2*c^2*x*y^2 + 2*a^2*b^4*c^2*x*y^2 + 4*a^4*c^4*x*y^2 - a^2*b^2*c^4*x*y^2 -      3*a^2*c^6*x*y^2 - a^6*c^2*y^3 + a^4*b^2*c^2*y^3 - a^2*b^2*c^4*y^3 + a^2*c^6*y^3 +      2*a^4*b^2*c^2*x^2*z - a^2*b^4*c^2*x^2*z - 3*b^6*c^2*x^2*z - a^2*b^2*c^4*x^2*z + 4*b^4*c^4*x^2*z -      b^2*c^6*x^2*z + 3*a^6*c^2*y^2*z + a^4*b^2*c^2*y^2*z - 2*a^2*b^4*c^2*y^2*z - 4*a^4*c^4*y^2*z +      a^2*b^2*c^4*y^2*z + a^2*c^6*y^2*z + a^6*b^2*x*z^2 - 4*a^4*b^4*x*z^2 + 3*a^2*b^6*x*z^2 +      a^4*b^2*c^2*x*z^2 + a^2*b^4*c^2*x*z^2 - 2*a^2*b^2*c^4*x*z^2 - 3*a^6*b^2*y*z^2 + 4*a^4*b^4*y*z^2 -      a^2*b^6*y*z^2 - a^4*b^2*c^2*y*z^2 - a^2*b^4*c^2*y*z^2 + 2*a^2*b^2*c^4*y*z^2 + a^6*b^2*z^3 -      a^2*b^6*z^3 - a^4*b^2*c^2*z^3 + a^2*b^4*c^2*z^3");
		exp("K419a","Sqrt((a + b - c)*(a - b + c)*(-a + b + c)*(a + b + c))*(-3*a^2*x^2*y + 3*b^2*x^2*y -        3*a^2*x*y^2 + 3*b^2*x*y^2 + 3*a^2*x^2*z - 3*c^2*x^2*z - 3*b^2*y^2*z + 3*c^2*y^2*z + 3*a^2*x*z^2 -        3*c^2*x*z^2 - 3*b^2*y*z^2 + 3*c^2*y*z^2) + Sqrt(3)*(a^4*x^2*y - 2*a^2*b^2*x^2*y + b^4*x^2*y +        a^2*c^2*x^2*y + b^2*c^2*x^2*y - 2*c^4*x^2*y - a^4*x*y^2 + 2*a^2*b^2*x*y^2 - b^4*x*y^2 -        a^2*c^2*x*y^2 - b^2*c^2*x*y^2 + 2*c^4*x*y^2 - a^4*x^2*z - a^2*b^2*x^2*z + 2*b^4*x^2*z +        2*a^2*c^2*x^2*z - b^2*c^2*x^2*z - c^4*x^2*z - 2*a^4*y^2*z + a^2*b^2*y^2*z + b^4*y^2*z +        a^2*c^2*y^2*z - 2*b^2*c^2*y^2*z + c^4*y^2*z + a^4*x*z^2 + a^2*b^2*x*z^2 - 2*b^4*x*z^2 -        2*a^2*c^2*x*z^2 + b^2*c^2*x*z^2 + c^4*x*z^2 + 2*a^4*y*z^2 - a^2*b^2*y*z^2 - b^4*y*z^2 -        a^2*c^2*y*z^2 + 2*b^2*c^2*y*z^2 - c^4*y*z^2)");
		exp("K419b","Sqrt((a + b - c)*(a - b + c)*(-a + b + c)*(a + b + c))*(3*a^2*x^2*y - 3*b^2*x^2*y +        3*a^2*x*y^2 - 3*b^2*x*y^2 - 3*a^2*x^2*z + 3*c^2*x^2*z + 3*b^2*y^2*z - 3*c^2*y^2*z - 3*a^2*x*z^2 +        3*c^2*x*z^2 + 3*b^2*y*z^2 - 3*c^2*y*z^2) + Sqrt(3)*(a^4*x^2*y - 2*a^2*b^2*x^2*y + b^4*x^2*y +        a^2*c^2*x^2*y + b^2*c^2*x^2*y - 2*c^4*x^2*y - a^4*x*y^2 + 2*a^2*b^2*x*y^2 - b^4*x*y^2 -        a^2*c^2*x*y^2 - b^2*c^2*x*y^2 + 2*c^4*x*y^2 - a^4*x^2*z - a^2*b^2*x^2*z + 2*b^4*x^2*z +        2*a^2*c^2*x^2*z - b^2*c^2*x^2*z - c^4*x^2*z - 2*a^4*y^2*z + a^2*b^2*y^2*z + b^4*y^2*z +        a^2*c^2*y^2*z - 2*b^2*c^2*y^2*z + c^4*y^2*z + a^4*x*z^2 + a^2*b^2*x*z^2 - 2*b^4*x*z^2 -        2*a^2*c^2*x*z^2 + b^2*c^2*x*z^2 + c^4*x*z^2 + 2*a^4*y*z^2 - a^2*b^2*y*z^2 - b^4*y*z^2 -        a^2*c^2*y*z^2 + 2*b^2*c^2*y*z^2 - c^4*y*z^2)");
		exp("K420a","Sqrt((a + b - c)*(a - b + c)*(-a + b + c)*(a + b + c))*((-a^2)*x^2*y + b^2*x^2*y + 2*c^2*x^2*y -        a^2*x*y^2 + b^2*x*y^2 - 2*c^2*x*y^2 + a^2*x^2*z - 2*b^2*x^2*z - c^2*x^2*z + 2*a^2*y^2*z -        b^2*y^2*z + c^2*y^2*z + a^2*x*z^2 + 2*b^2*x*z^2 - c^2*x*z^2 - 2*a^2*y*z^2 - b^2*y*z^2 +        c^2*y*z^2) + Sqrt(3)*(a^4*x^2*y - 2*a^2*b^2*x^2*y + b^4*x^2*y - a^2*c^2*x^2*y - b^2*c^2*x^2*y -        a^4*x*y^2 + 2*a^2*b^2*x*y^2 - b^4*x*y^2 + a^2*c^2*x*y^2 + b^2*c^2*x*y^2 - a^4*x^2*z +        a^2*b^2*x^2*z + 2*a^2*c^2*x^2*z + b^2*c^2*x^2*z - c^4*x^2*z - a^2*b^2*y^2*z + b^4*y^2*z -        a^2*c^2*y^2*z - 2*b^2*c^2*y^2*z + c^4*y^2*z + a^4*x*z^2 - a^2*b^2*x*z^2 - 2*a^2*c^2*x*z^2 -        b^2*c^2*x*z^2 + c^4*x*z^2 + a^2*b^2*y*z^2 - b^4*y*z^2 + a^2*c^2*y*z^2 + 2*b^2*c^2*y*z^2 -        c^4*y*z^2)");
		exp("K420b","Sqrt((a + b - c)*(a - b + c)*(-a + b + c)*(a + b + c))*(a^2*x^2*y - b^2*x^2*y - 2*c^2*x^2*y +        a^2*x*y^2 - b^2*x*y^2 + 2*c^2*x*y^2 - a^2*x^2*z + 2*b^2*x^2*z + c^2*x^2*z - 2*a^2*y^2*z +        b^2*y^2*z - c^2*y^2*z - a^2*x*z^2 - 2*b^2*x*z^2 + c^2*x*z^2 + 2*a^2*y*z^2 + b^2*y*z^2 -        c^2*y*z^2) + Sqrt(3)*(a^4*x^2*y - 2*a^2*b^2*x^2*y + b^4*x^2*y - a^2*c^2*x^2*y - b^2*c^2*x^2*y -        a^4*x*y^2 + 2*a^2*b^2*x*y^2 - b^4*x*y^2 + a^2*c^2*x*y^2 + b^2*c^2*x*y^2 - a^4*x^2*z +        a^2*b^2*x^2*z + 2*a^2*c^2*x^2*z + b^2*c^2*x^2*z - c^4*x^2*z - a^2*b^2*y^2*z + b^4*y^2*z -        a^2*c^2*y^2*z - 2*b^2*c^2*y^2*z + c^4*y^2*z + a^4*x*z^2 - a^2*b^2*x*z^2 - 2*a^2*c^2*x*z^2 -        b^2*c^2*x*z^2 + c^4*x*z^2 + a^2*b^2*y*z^2 - b^4*y*z^2 + a^2*c^2*y*z^2 + 2*b^2*c^2*y*z^2 -        c^4*y*z^2)");
		exp("K421","a^2*b^4*x^2*y + a^2*b^2*c^2*x^2*y + b^4*c^2*x^2*y + a^2*c^4*x^2*y + b^2*c^4*x^2*y + c^6*x^2*y -      a^4*b^2*x*y^2 - a^4*c^2*x*y^2 - a^2*b^2*c^2*x*y^2 - a^2*c^4*x*y^2 - b^2*c^4*x*y^2 - c^6*x*y^2 -      a^2*b^4*x^2*z - b^6*x^2*z - a^2*b^2*c^2*x^2*z - b^4*c^2*x^2*z - a^2*c^4*x^2*z - b^2*c^4*x^2*z +      a^6*y^2*z + a^4*b^2*y^2*z + a^4*c^2*y^2*z + a^2*b^2*c^2*y^2*z + a^2*c^4*y^2*z + b^2*c^4*y^2*z +      a^4*b^2*x*z^2 + a^2*b^4*x*z^2 + b^6*x*z^2 + a^4*c^2*x*z^2 + a^2*b^2*c^2*x*z^2 + b^4*c^2*x*z^2 -      a^6*y*z^2 - a^4*b^2*y*z^2 - a^2*b^4*y*z^2 - a^4*c^2*y*z^2 - a^2*b^2*c^2*y*z^2 - b^4*c^2*y*z^2");
		exp("K422","2*a^6*b^2*c^2*x^2*y - a^4*b^4*c^2*x^2*y - b^8*c^2*x^2*y + a^6*c^4*x^2*y - a^2*b^4*c^4*x^2*y -      2*a^4*c^6*x^2*y - b^4*c^6*x^2*y + a^2*c^8*x^2*y + 2*b^2*c^8*x^2*y + a^8*c^2*x*y^2 +      a^4*b^4*c^2*x*y^2 - 2*a^2*b^6*c^2*x*y^2 + a^4*b^2*c^4*x*y^2 - b^6*c^4*x*y^2 + a^4*c^6*x*y^2 +      2*b^4*c^6*x*y^2 - 2*a^2*c^8*x*y^2 - b^2*c^8*x*y^2 - a^6*b^4*x^2*z + 2*a^4*b^6*x^2*z -      a^2*b^8*x^2*z - 2*a^6*b^2*c^2*x^2*z - 2*b^8*c^2*x^2*z + a^4*b^2*c^4*x^2*z + a^2*b^4*c^4*x^2*z +      b^6*c^4*x^2*z + b^2*c^8*x^2*z + a^8*b^2*y^2*z - 2*a^6*b^4*y^2*z + a^4*b^6*y^2*z + 2*a^8*c^2*y^2*z +      2*a^2*b^6*c^2*y^2*z - a^6*c^4*y^2*z - a^4*b^2*c^4*y^2*z - a^2*b^4*c^4*y^2*z - a^2*c^8*y^2*z -      a^8*b^2*x*z^2 - a^4*b^6*x*z^2 + 2*a^2*b^8*x*z^2 - a^4*b^4*c^2*x*z^2 + b^8*c^2*x*z^2 -      a^4*b^2*c^4*x*z^2 - 2*b^6*c^4*x*z^2 + 2*a^2*b^2*c^6*x*z^2 + b^4*c^6*x*z^2 - 2*a^8*b^2*y*z^2 +      a^6*b^4*y*z^2 + a^2*b^8*y*z^2 - a^8*c^2*y*z^2 + a^4*b^4*c^2*y*z^2 + 2*a^6*c^4*y*z^2 +      a^2*b^4*c^4*y*z^2 - a^4*c^6*y*z^2 - 2*a^2*b^2*c^6*y*z^2");
		exp("K423","2*a^2*b^2*c^2*x^2*y + b^4*c^2*x^2*y + a^2*c^4*x^2*y + 2*b^2*c^4*x^2*y - a^4*c^2*x*y^2 -      2*a^2*b^2*c^2*x*y^2 - 2*a^2*c^4*x*y^2 - b^2*c^4*x*y^2 - a^2*b^4*x^2*z - 2*a^2*b^2*c^2*x^2*z -      2*b^4*c^2*x^2*z - b^2*c^4*x^2*z + a^4*b^2*y^2*z + 2*a^4*c^2*y^2*z + 2*a^2*b^2*c^2*y^2*z +      a^2*c^4*y^2*z + a^4*b^2*x*z^2 + 2*a^2*b^4*x*z^2 + 2*a^2*b^2*c^2*x*z^2 + b^4*c^2*x*z^2 -      2*a^4*b^2*y*z^2 - a^2*b^4*y*z^2 - a^4*c^2*y*z^2 - 2*a^2*b^2*c^2*y*z^2");
		exp("K424a","2*b^2*c^2*x^2*y - 2*a^2*c^2*x*y^2 - 2*b^2*c^2*x^2*z + 2*a^2*c^2*y^2*z + 2*a^2*b^2*x*z^2 -      2*a^2*b^2*y*z^2 + Sqrt((a + b - c)*(a - b + c)*(-a + b + c)*(a + b + c))*      (c^2*x^2*y - c^2*x*y^2 - b^2*x^2*z + a^2*y^2*z + b^2*x*z^2 - a^2*y*z^2)");
		exp("K424b","2*b^2*c^2*x^2*y - 2*a^2*c^2*x*y^2 - 2*b^2*c^2*x^2*z + 2*a^2*c^2*y^2*z + 2*a^2*b^2*x*z^2 -      2*a^2*b^2*y*z^2 + Sqrt((a + b - c)*(a - b + c)*(-a + b + c)*(a + b + c))*      ((-c^2)*x^2*y + c^2*x*y^2 + b^2*x^2*z - a^2*y^2*z - b^2*x*z^2 + a^2*y*z^2)");
		exp("K425","-3*a^6*x^2*y + 9*a^4*b^2*x^2*y - 9*a^2*b^4*x^2*y + 3*b^6*x^2*y - 5*a^4*c^2*x^2*y -      2*a^2*b^2*c^2*x^2*y + 7*b^4*c^2*x^2*y + 11*a^2*c^4*x^2*y - 7*b^2*c^4*x^2*y - 3*c^6*x^2*y -      3*a^6*x*y^2 + 9*a^4*b^2*x*y^2 - 9*a^2*b^4*x*y^2 + 3*b^6*x*y^2 - 7*a^4*c^2*x*y^2 +      2*a^2*b^2*c^2*x*y^2 + 5*b^4*c^2*x*y^2 + 7*a^2*c^4*x*y^2 - 11*b^2*c^4*x*y^2 + 3*c^6*x*y^2 +      3*a^6*x^2*z + 5*a^4*b^2*x^2*z - 11*a^2*b^4*x^2*z + 3*b^6*x^2*z - 9*a^4*c^2*x^2*z +      2*a^2*b^2*c^2*x^2*z + 7*b^4*c^2*x^2*z + 9*a^2*c^4*x^2*z - 7*b^2*c^4*x^2*z - 3*c^6*x^2*z +      16*a^4*b^2*x*y*z - 16*a^2*b^4*x*y*z - 16*a^4*c^2*x*y*z + 16*b^4*c^2*x*y*z + 16*a^2*c^4*x*y*z -      16*b^2*c^4*x*y*z - 3*a^6*y^2*z + 11*a^4*b^2*y^2*z - 5*a^2*b^4*y^2*z - 3*b^6*y^2*z -      7*a^4*c^2*y^2*z - 2*a^2*b^2*c^2*y^2*z + 9*b^4*c^2*y^2*z + 7*a^2*c^4*y^2*z - 9*b^2*c^4*y^2*z +      3*c^6*y^2*z + 3*a^6*x*z^2 + 7*a^4*b^2*x*z^2 - 7*a^2*b^4*x*z^2 - 3*b^6*x*z^2 - 9*a^4*c^2*x*z^2 -      2*a^2*b^2*c^2*x*z^2 + 11*b^4*c^2*x*z^2 + 9*a^2*c^4*x*z^2 - 5*b^2*c^4*x*z^2 - 3*c^6*x*z^2 +      3*a^6*y*z^2 + 7*a^4*b^2*y*z^2 - 7*a^2*b^4*y*z^2 - 3*b^6*y*z^2 - 11*a^4*c^2*y*z^2 +      2*a^2*b^2*c^2*y*z^2 + 9*b^4*c^2*y*z^2 + 5*a^2*c^4*y*z^2 - 9*b^2*c^4*y*z^2 + 3*c^6*y*z^2");
		exp("K426","(-a^4)*c^2*x^2*y + 2*a^2*b^2*c^2*x^2*y - b^4*c^2*x^2*y - 2*a^2*c^4*x^2*y - 2*b^2*c^4*x^2*y +      3*c^6*x^2*y + a^4*c^2*x*y^2 - 2*a^2*b^2*c^2*x*y^2 + b^4*c^2*x*y^2 + 2*a^2*c^4*x*y^2 +      2*b^2*c^4*x*y^2 - 3*c^6*x*y^2 + a^4*b^2*x^2*z + 2*a^2*b^4*x^2*z - 3*b^6*x^2*z -      2*a^2*b^2*c^2*x^2*z + 2*b^4*c^2*x^2*z + b^2*c^4*x^2*z + 8*a^4*b^2*x*y*z - 8*a^2*b^4*x*y*z -      8*a^4*c^2*x*y*z + 8*b^4*c^2*x*y*z + 8*a^2*c^4*x*y*z - 8*b^2*c^4*x*y*z + 3*a^6*y^2*z -      2*a^4*b^2*y^2*z - a^2*b^4*y^2*z - 2*a^4*c^2*y^2*z + 2*a^2*b^2*c^2*y^2*z - a^2*c^4*y^2*z -      a^4*b^2*x*z^2 - 2*a^2*b^4*x*z^2 + 3*b^6*x*z^2 + 2*a^2*b^2*c^2*x*z^2 - 2*b^4*c^2*x*z^2 -      b^2*c^4*x*z^2 - 3*a^6*y*z^2 + 2*a^4*b^2*y*z^2 + a^2*b^4*y*z^2 + 2*a^4*c^2*y*z^2 -      2*a^2*b^2*c^2*y*z^2 + a^2*c^4*y*z^2");
		exp("K427","-2*a^6*x^2*y + 6*a^4*b^2*x^2*y - 6*a^2*b^4*x^2*y + 2*b^6*x^2*y - a^4*c^2*x^2*y -      2*a^2*b^2*c^2*x^2*y + 3*b^4*c^2*x^2*y + 5*a^2*c^4*x^2*y - 3*b^2*c^4*x^2*y - 2*c^6*x^2*y -      2*a^6*x*y^2 + 6*a^4*b^2*x*y^2 - 6*a^2*b^4*x*y^2 + 2*b^6*x*y^2 - 3*a^4*c^2*x*y^2 +      2*a^2*b^2*c^2*x*y^2 + b^4*c^2*x*y^2 + 3*a^2*c^4*x*y^2 - 5*b^2*c^4*x*y^2 + 2*c^6*x*y^2 +      2*a^6*x^2*z + a^4*b^2*x^2*z - 5*a^2*b^4*x^2*z + 2*b^6*x^2*z - 6*a^4*c^2*x^2*z +      2*a^2*b^2*c^2*x^2*z + 3*b^4*c^2*x^2*z + 6*a^2*c^4*x^2*z - 3*b^2*c^4*x^2*z - 2*c^6*x^2*z +      6*a^4*b^2*x*y*z - 6*a^2*b^4*x*y*z - 6*a^4*c^2*x*y*z + 6*b^4*c^2*x*y*z + 6*a^2*c^4*x*y*z -      6*b^2*c^4*x*y*z - 2*a^6*y^2*z + 5*a^4*b^2*y^2*z - a^2*b^4*y^2*z - 2*b^6*y^2*z - 3*a^4*c^2*y^2*z -      2*a^2*b^2*c^2*y^2*z + 6*b^4*c^2*y^2*z + 3*a^2*c^4*y^2*z - 6*b^2*c^4*y^2*z + 2*c^6*y^2*z +      2*a^6*x*z^2 + 3*a^4*b^2*x*z^2 - 3*a^2*b^4*x*z^2 - 2*b^6*x*z^2 - 6*a^4*c^2*x*z^2 -      2*a^2*b^2*c^2*x*z^2 + 5*b^4*c^2*x*z^2 + 6*a^2*c^4*x*z^2 - b^2*c^4*x*z^2 - 2*c^6*x*z^2 +      2*a^6*y*z^2 + 3*a^4*b^2*y*z^2 - 3*a^2*b^4*y*z^2 - 2*b^6*y*z^2 - 5*a^4*c^2*y*z^2 +      2*a^2*b^2*c^2*y*z^2 + 6*b^4*c^2*y*z^2 + a^2*c^4*y*z^2 - 6*b^2*c^4*y*z^2 + 2*c^6*y*z^2");
		exp("K428","(-a^8)*b^4*c^4*x^2*y + 2*a^6*b^6*c^4*x^2*y - 2*a^2*b^10*c^4*x^2*y + b^12*c^4*x^2*y -      a^6*b^4*c^6*x^2*y + 2*a^4*b^6*c^6*x^2*y + a^2*b^8*c^6*x^2*y - 2*b^10*c^6*x^2*y +      a^4*b^4*c^8*x^2*y + 2*a^2*b^6*c^8*x^2*y - a^2*b^4*c^10*x^2*y + 2*b^6*c^10*x^2*y - b^4*c^12*x^2*y -      a^12*c^4*x*y^2 + 2*a^10*b^2*c^4*x*y^2 - 2*a^6*b^6*c^4*x*y^2 + a^4*b^8*c^4*x*y^2 +      2*a^10*c^6*x*y^2 - a^8*b^2*c^6*x*y^2 - 2*a^6*b^4*c^6*x*y^2 + a^4*b^6*c^6*x*y^2 -      2*a^6*b^2*c^8*x*y^2 - a^4*b^4*c^8*x*y^2 - 2*a^6*c^10*x*y^2 + a^4*b^2*c^10*x*y^2 + a^4*c^12*x*y^2 +      a^8*b^4*c^4*x^2*z + a^6*b^6*c^4*x^2*z - a^4*b^8*c^4*x^2*z + a^2*b^10*c^4*x^2*z + b^12*c^4*x^2*z -      2*a^6*b^4*c^6*x^2*z - 2*a^4*b^6*c^6*x^2*z - 2*a^2*b^8*c^6*x^2*z - 2*b^10*c^6*x^2*z -      a^2*b^6*c^8*x^2*z + 2*a^2*b^4*c^10*x^2*z + 2*b^6*c^10*x^2*z - b^4*c^12*x^2*z - a^12*c^4*y^2*z -      a^10*b^2*c^4*y^2*z + a^8*b^4*c^4*y^2*z - a^6*b^6*c^4*y^2*z - a^4*b^8*c^4*y^2*z + 2*a^10*c^6*y^2*z +      2*a^8*b^2*c^6*y^2*z + 2*a^6*b^4*c^6*y^2*z + 2*a^4*b^6*c^6*y^2*z + a^6*b^2*c^8*y^2*z -      2*a^6*c^10*y^2*z - 2*a^4*b^2*c^10*y^2*z + a^4*c^12*y^2*z + a^12*b^4*x*z^2 - 2*a^10*b^6*x*z^2 +      2*a^6*b^10*x*z^2 - a^4*b^12*x*z^2 - 2*a^10*b^4*c^2*x*z^2 + a^8*b^6*c^2*x*z^2 +      2*a^6*b^8*c^2*x*z^2 - a^4*b^10*c^2*x*z^2 + 2*a^6*b^6*c^4*x*z^2 + a^4*b^8*c^4*x*z^2 +      2*a^6*b^4*c^6*x*z^2 - a^4*b^6*c^6*x*z^2 - a^4*b^4*c^8*x*z^2 + a^12*b^4*y*z^2 - 2*a^10*b^6*y*z^2 +      2*a^6*b^10*y*z^2 - a^4*b^12*y*z^2 + a^10*b^4*c^2*y*z^2 - 2*a^8*b^6*c^2*y*z^2 - a^6*b^8*c^2*y*z^2 +      2*a^4*b^10*c^2*y*z^2 - a^8*b^4*c^4*y*z^2 - 2*a^6*b^6*c^4*y*z^2 + a^6*b^4*c^6*y*z^2 -      2*a^4*b^6*c^6*y*z^2 + a^4*b^4*c^8*y*z^2");
		exp("K429","(-a^2)*c^4*x^2*y + b^2*c^4*x^2*y + c^6*x^2*y - a^2*c^4*x*y^2 + b^2*c^4*x*y^2 - c^6*x*y^2 +      a^2*b^4*x^2*z - b^6*x^2*z - b^4*c^2*x^2*z - 2*a^4*b^2*x*y*z + 2*a^2*b^4*x*y*z + 2*a^4*c^2*x*y*z -      2*b^4*c^2*x*y*z - 2*a^2*c^4*x*y*z + 2*b^2*c^4*x*y*z + a^6*y^2*z - a^4*b^2*y^2*z + a^4*c^2*y^2*z +      a^2*b^4*x*z^2 + b^6*x*z^2 - b^4*c^2*x*z^2 - a^6*y*z^2 - a^4*b^2*y*z^2 + a^4*c^2*y*z^2");
		exp("K430","a^2*b^2*c^4*x^2*y - b^4*c^4*x^2*y + a^2*b*c^5*x^2*y + a*b^2*c^5*x^2*y + a*b*c^6*x^2*y +      b^2*c^6*x^2*y + a^4*c^4*x*y^2 - a^2*b^2*c^4*x*y^2 - a^2*b*c^5*x*y^2 - a*b^2*c^5*x*y^2 -      a^2*c^6*x*y^2 - a*b*c^6*x*y^2 - a^2*b^5*c*x^2*z - a*b^6*c*x^2*z - a^2*b^4*c^2*x^2*z -      a*b^5*c^2*x^2*z - b^6*c^2*x^2*z + b^4*c^4*x^2*z + a^6*b*c*y^2*z + a^5*b^2*c*y^2*z + a^6*c^2*y^2*z +      a^5*b*c^2*y^2*z + a^4*b^2*c^2*y^2*z - a^4*c^4*y^2*z - a^4*b^4*x*z^2 + a^2*b^6*x*z^2 +      a^2*b^5*c*x*z^2 + a*b^6*c*x*z^2 + a^2*b^4*c^2*x*z^2 + a*b^5*c^2*x*z^2 - a^6*b^2*y*z^2 +      a^4*b^4*y*z^2 - a^6*b*c*y*z^2 - a^5*b^2*c*y*z^2 - a^5*b*c^2*y*z^2 - a^4*b^2*c^2*y*z^2");
		exp("K431","(-a^5)*b^2*c^4*x^2*y - a^4*b^3*c^4*x^2*y + a*b^6*c^4*x^2*y + b^7*c^4*x^2*y - a^5*b*c^5*x^2*y -      a^4*b^2*c^5*x^2*y + a*b^5*c^5*x^2*y + b^6*c^5*x^2*y + 2*a^3*b^2*c^6*x^2*y + 2*a^2*b^3*c^6*x^2*y +      2*a^3*b*c^7*x^2*y + 2*a^2*b^2*c^7*x^2*y - a*b^2*c^8*x^2*y - b^3*c^8*x^2*y - a*b*c^9*x^2*y -      b^2*c^9*x^2*y - a^7*c^4*x*y^2 - a^6*b*c^4*x*y^2 + a^3*b^4*c^4*x*y^2 + a^2*b^5*c^4*x*y^2 -      a^6*c^5*x*y^2 - a^5*b*c^5*x*y^2 + a^2*b^4*c^5*x*y^2 + a*b^5*c^5*x*y^2 - 2*a^3*b^2*c^6*x*y^2 -      2*a^2*b^3*c^6*x*y^2 - 2*a^2*b^2*c^7*x*y^2 - 2*a*b^3*c^7*x*y^2 + a^3*c^8*x*y^2 + a^2*b*c^8*x*y^2 +      a^2*c^9*x*y^2 + a*b*c^9*x*y^2 + a^5*b^5*c*x^2*z - 2*a^3*b^7*c*x^2*z + a*b^9*c*x^2*z +      a^5*b^4*c^2*x^2*z + a^4*b^5*c^2*x^2*z - 2*a^3*b^6*c^2*x^2*z - 2*a^2*b^7*c^2*x^2*z +      a*b^8*c^2*x^2*z + b^9*c^2*x^2*z + a^4*b^4*c^3*x^2*z - 2*a^2*b^6*c^3*x^2*z + b^8*c^3*x^2*z -      a*b^5*c^5*x^2*z - a*b^4*c^6*x^2*z - b^5*c^6*x^2*z - b^4*c^7*x^2*z - a^9*b*c*y^2*z +      2*a^7*b^3*c*y^2*z - a^5*b^5*c*y^2*z - a^9*c^2*y^2*z - a^8*b*c^2*y^2*z + 2*a^7*b^2*c^2*y^2*z +      2*a^6*b^3*c^2*y^2*z - a^5*b^4*c^2*y^2*z - a^4*b^5*c^2*y^2*z - a^8*c^3*y^2*z + 2*a^6*b^2*c^3*y^2*z -      a^4*b^4*c^3*y^2*z + a^5*b*c^5*y^2*z + a^5*c^6*y^2*z + a^4*b*c^6*y^2*z + a^4*c^7*y^2*z +      a^7*b^4*x*z^2 + a^6*b^5*x*z^2 - a^3*b^8*x*z^2 - a^2*b^9*x*z^2 + a^6*b^4*c*x*z^2 + a^5*b^5*c*x*z^2 -      a^2*b^8*c*x*z^2 - a*b^9*c*x*z^2 + 2*a^3*b^6*c^2*x*z^2 + 2*a^2*b^7*c^2*x*z^2 + 2*a^2*b^6*c^3*x*z^2 +      2*a*b^7*c^3*x*z^2 - a^3*b^4*c^4*x*z^2 - a^2*b^5*c^4*x*z^2 - a^2*b^4*c^5*x*z^2 - a*b^5*c^5*x*z^2 +      a^9*b^2*y*z^2 + a^8*b^3*y*z^2 - a^5*b^6*y*z^2 - a^4*b^7*y*z^2 + a^9*b*c*y*z^2 + a^8*b^2*c*y*z^2 -      a^5*b^5*c*y*z^2 - a^4*b^6*c*y*z^2 - 2*a^7*b^2*c^2*y*z^2 - 2*a^6*b^3*c^2*y*z^2 - 2*a^7*b*c^3*y*z^2 -      2*a^6*b^2*c^3*y*z^2 + a^5*b^2*c^4*y*z^2 + a^4*b^3*c^4*y*z^2 + a^5*b*c^5*y*z^2 +      a^4*b^2*c^5*y*z^2");
		exp("K432","b^5*c^4*x^2*y - a^2*b*c^6*x^2*y - a^5*c^4*x*y^2 + a*b^2*c^6*x*y^2 + a^2*b^6*c*x^2*z -      b^4*c^5*x^2*z - a^6*b^2*c*y^2*z + a^4*c^5*y^2*z + a^5*b^4*x*z^2 - a*b^6*c^2*x*z^2 - a^4*b^5*y*z^2 +      a^6*b*c^2*y*z^2");
		exp("K433","(-a^4)*c^4*x^2*y + a^2*c^6*x^2*y + b^4*c^4*x*y^2 - b^2*c^6*x*y^2 + a^4*b^4*x^2*z -      a^2*b^6*x^2*z + a^6*b^2*x*y*z - a^2*b^6*x*y*z - a^6*c^2*x*y*z + b^6*c^2*x*y*z + a^2*c^6*x*y*z -      b^2*c^6*x*y*z + a^6*b^2*y^2*z - a^4*b^4*y^2*z + b^6*c^2*x*z^2 - b^4*c^4*x*z^2 - a^6*c^2*y*z^2 +      a^4*c^4*y*z^2");
		exp("K434","a^4*c^2*x^2*y + a^3*b*c^2*x^2*y + a^2*b^2*c^2*x^2*y - a*b^3*c^2*x^2*y - 2*b^4*c^2*x^2*y -      a^2*b*c^3*x^2*y - b^3*c^3*x^2*y - 2*a^2*c^4*x^2*y - a*b*c^4*x^2*y + b^2*c^4*x^2*y + b*c^5*x^2*y +      c^6*x^2*y + 2*a^4*c^2*x*y^2 + a^3*b*c^2*x*y^2 - a^2*b^2*c^2*x*y^2 - a*b^3*c^2*x*y^2 -      b^4*c^2*x*y^2 + a^3*c^3*x*y^2 + a*b^2*c^3*x*y^2 - a^2*c^4*x*y^2 + a*b*c^4*x*y^2 + 2*b^2*c^4*x*y^2 -      a*c^5*x*y^2 - c^6*x*y^2 - a^4*b^2*x^2*z + 2*a^2*b^4*x^2*z - b^6*x^2*z - a^3*b^2*c*x^2*z +      a^2*b^3*c*x^2*z + a*b^4*c*x^2*z - b^5*c*x^2*z - a^2*b^2*c^2*x^2*z - b^4*c^2*x^2*z +      a*b^2*c^3*x^2*z + b^3*c^3*x^2*z + 2*b^2*c^4*x^2*z + a^6*y^2*z - 2*a^4*b^2*y^2*z + a^2*b^4*y^2*z +      a^5*c*y^2*z - a^4*b*c*y^2*z - a^3*b^2*c*y^2*z + a^2*b^3*c*y^2*z + a^4*c^2*y^2*z +      a^2*b^2*c^2*y^2*z - a^3*c^3*y^2*z - a^2*b*c^3*y^2*z - 2*a^2*c^4*y^2*z - 2*a^4*b^2*x*z^2 -      a^3*b^3*x*z^2 + a^2*b^4*x*z^2 + a*b^5*x*z^2 + b^6*x*z^2 - a^3*b^2*c*x*z^2 - a*b^4*c*x*z^2 +      a^2*b^2*c^2*x*z^2 - a*b^3*c^2*x*z^2 - 2*b^4*c^2*x*z^2 + a*b^2*c^3*x*z^2 + b^2*c^4*x*z^2 -      a^6*y*z^2 - a^5*b*y*z^2 - a^4*b^2*y*z^2 + a^3*b^3*y*z^2 + 2*a^2*b^4*y*z^2 + a^4*b*c*y*z^2 +      a^2*b^3*c*y*z^2 + 2*a^4*c^2*y*z^2 + a^3*b*c^2*y*z^2 - a^2*b^2*c^2*y*z^2 - a^2*b*c^3*y*z^2 -      a^2*c^4*y*z^2");
		exp("K435","a^6*c^2*x^2*y + 2*a^4*b^2*c^2*x^2*y - 2*a^2*b^4*c^2*x^2*y - 3*b^6*c^2*x^2*y - 2*a^4*c^4*x^2*y +      8*b^4*c^4*x^2*y - a^2*c^6*x^2*y - 5*b^2*c^6*x^2*y + 2*c^8*x^2*y + 3*a^6*c^2*x*y^2 +      2*a^4*b^2*c^2*x*y^2 - 2*a^2*b^4*c^2*x*y^2 - b^6*c^2*x*y^2 - 8*a^4*c^4*x*y^2 + 2*b^4*c^4*x*y^2 +      5*a^2*c^6*x*y^2 + b^2*c^6*x*y^2 - 2*c^8*x*y^2 - a^6*b^2*x^2*z + 2*a^4*b^4*x^2*z + a^2*b^6*x^2*z -      2*b^8*x^2*z - 2*a^4*b^2*c^2*x^2*z + 5*b^6*c^2*x^2*z + 2*a^2*b^2*c^4*x^2*z - 8*b^4*c^4*x^2*z +      3*b^2*c^6*x^2*z + 2*a^8*y^2*z - a^6*b^2*y^2*z - 2*a^4*b^4*y^2*z + a^2*b^6*y^2*z - 5*a^6*c^2*y^2*z +      2*a^2*b^4*c^2*y^2*z + 8*a^4*c^4*y^2*z - 2*a^2*b^2*c^4*y^2*z - 3*a^2*c^6*y^2*z - 3*a^6*b^2*x*z^2 +      8*a^4*b^4*x*z^2 - 5*a^2*b^6*x*z^2 + 2*b^8*x*z^2 - 2*a^4*b^2*c^2*x*z^2 - b^6*c^2*x*z^2 +      2*a^2*b^2*c^4*x*z^2 - 2*b^4*c^4*x*z^2 + b^2*c^6*x*z^2 - 2*a^8*y*z^2 + 5*a^6*b^2*y*z^2 -      8*a^4*b^4*y*z^2 + 3*a^2*b^6*y*z^2 + a^6*c^2*y*z^2 + 2*a^2*b^4*c^2*y*z^2 + 2*a^4*c^4*y*z^2 -      2*a^2*b^2*c^4*y*z^2 - a^2*c^6*y*z^2");
		exp("K436","(-a^4)*b^2*c^3*x^2*y + b^6*c^3*x^2*y - a^4*b*c^4*x^2*y + 2*a^3*b^2*c^4*x^2*y -      2*a^2*b^3*c^4*x^2*y + 2*a*b^4*c^4*x^2*y - b^5*c^4*x^2*y - 2*b^4*c^5*x^2*y + 2*a^2*b*c^6*x^2*y -      2*a*b^2*c^6*x^2*y + 2*b^3*c^6*x^2*y + b^2*c^7*x^2*y - b*c^8*x^2*y - a^6*c^3*x*y^2 +      a^2*b^4*c^3*x*y^2 + a^5*c^4*x*y^2 - 2*a^4*b*c^4*x*y^2 + 2*a^3*b^2*c^4*x*y^2 - 2*a^2*b^3*c^4*x*y^2 +      a*b^4*c^4*x*y^2 + 2*a^4*c^5*x*y^2 - 2*a^3*c^6*x*y^2 + 2*a^2*b*c^6*x*y^2 - 2*a*b^2*c^6*x*y^2 -      a^2*c^7*x*y^2 + a*c^8*x*y^2 + a^4*b^4*c*x^2*z - 2*a^2*b^6*c*x^2*z + b^8*c*x^2*z +      a^4*b^3*c^2*x^2*z - 2*a^3*b^4*c^2*x^2*z + 2*a*b^6*c^2*x^2*z - b^7*c^2*x^2*z + 2*a^2*b^4*c^3*x^2*z -      2*b^6*c^3*x^2*z - 2*a*b^4*c^4*x^2*z + 2*b^5*c^4*x^2*z + b^4*c^5*x^2*z - b^3*c^6*x^2*z -      a^8*c*y^2*z + 2*a^6*b^2*c*y^2*z - a^4*b^4*c*y^2*z + a^7*c^2*y^2*z - 2*a^6*b*c^2*y^2*z +      2*a^4*b^3*c^2*y^2*z - a^3*b^4*c^2*y^2*z + 2*a^6*c^3*y^2*z - 2*a^4*b^2*c^3*y^2*z - 2*a^5*c^4*y^2*z +      2*a^4*b*c^4*y^2*z - a^4*c^5*y^2*z + a^3*c^6*y^2*z + a^6*b^3*x*z^2 - a^5*b^4*x*z^2 -      2*a^4*b^5*x*z^2 + 2*a^3*b^6*x*z^2 + a^2*b^7*x*z^2 - a*b^8*x*z^2 + 2*a^4*b^4*c*x*z^2 -      2*a^2*b^6*c*x*z^2 - 2*a^3*b^4*c^2*x*z^2 + 2*a*b^6*c^2*x*z^2 + 2*a^2*b^4*c^3*x*z^2 -      a^2*b^3*c^4*x*z^2 - a*b^4*c^4*x*z^2 + a^8*b*y*z^2 - a^7*b^2*y*z^2 - 2*a^6*b^3*y*z^2 +      2*a^5*b^4*y*z^2 + a^4*b^5*y*z^2 - a^3*b^6*y*z^2 + 2*a^6*b^2*c*y*z^2 - 2*a^4*b^4*c*y*z^2 -      2*a^6*b*c^2*y*z^2 + 2*a^4*b^3*c^2*y*z^2 - 2*a^4*b^2*c^3*y*z^2 + a^4*b*c^4*y*z^2 +      a^3*b^2*c^4*y*z^2");
		exp("K437","b^4*c^2*x^3 - b^2*c^4*x^3 - 2*a^2*b^2*c^2*x^2*y - b^4*c^2*x^2*y - b^2*c^4*x^2*y + a^4*c^2*x*y^2 + 2*a^2*b^2*c^2*x*y^2 + a^2*c^4*x*y^2 - a^4*c^2*y^3 + a^2*c^4*y^3 +      2*a^2*b^2*c^2*x^2*z + b^4*c^2*x^2*z + b^2*c^4*x^2*z - a^4*c^2*y^2*z - 2*a^2*b^2*c^2*y^2*z - a^2*c^4*y^2*z - a^4*b^2*x*z^2 - a^2*b^4*x*z^2 - 2*a^2*b^2*c^2*x*z^2 + a^4*b^2*y*z^2 +      a^2*b^4*y*z^2 + 2*a^2*b^2*c^2*y*z^2 + a^4*b^2*z^3 - a^2*b^4*z^3");
		exp("K438a","Sqrt((a + b - c)*(a - b + c)*(-a + b + c)*(a + b + c))*((-a^10)*c^2*x^2*y +        6*a^8*b^2*c^2*x^2*y + a^6*b^4*c^2*x^2*y + a^4*b^6*c^2*x^2*y - 6*a^2*b^8*c^2*x^2*y -        b^10*c^2*x^2*y + 6*a^8*c^4*x^2*y + 4*a^6*b^2*c^4*x^2*y - 2*a^4*b^4*c^4*x^2*y +        10*a^2*b^6*c^4*x^2*y - 12*b^8*c^4*x^2*y - 8*a^6*c^6*x^2*y - 11*a^4*b^2*c^6*x^2*y +        13*a^2*b^4*c^6*x^2*y + 40*b^6*c^6*x^2*y + 4*a^4*c^8*x^2*y - 14*a^2*b^2*c^8*x^2*y -        38*b^4*c^8*x^2*y - 3*a^2*c^10*x^2*y + 9*b^2*c^10*x^2*y + 2*c^12*x^2*y + a^10*c^2*x*y^2 +        6*a^8*b^2*c^2*x*y^2 - a^6*b^4*c^2*x*y^2 - a^4*b^6*c^2*x*y^2 - 6*a^2*b^8*c^2*x*y^2 +        b^10*c^2*x*y^2 + 12*a^8*c^4*x*y^2 - 10*a^6*b^2*c^4*x*y^2 + 2*a^4*b^4*c^4*x*y^2 -        4*a^2*b^6*c^4*x*y^2 - 6*b^8*c^4*x*y^2 - 40*a^6*c^6*x*y^2 - 13*a^4*b^2*c^6*x*y^2 +        11*a^2*b^4*c^6*x*y^2 + 8*b^6*c^6*x*y^2 + 38*a^4*c^8*x*y^2 + 14*a^2*b^2*c^8*x*y^2 -        4*b^4*c^8*x*y^2 - 9*a^2*c^10*x*y^2 + 3*b^2*c^10*x*y^2 - 2*c^12*x*y^2 + a^10*b^2*x^2*z -        6*a^8*b^4*x^2*z + 8*a^6*b^6*x^2*z - 4*a^4*b^8*x^2*z + 3*a^2*b^10*x^2*z - 2*b^12*x^2*z -        6*a^8*b^2*c^2*x^2*z - 4*a^6*b^4*c^2*x^2*z + 11*a^4*b^6*c^2*x^2*z + 14*a^2*b^8*c^2*x^2*z -        9*b^10*c^2*x^2*z - a^6*b^2*c^4*x^2*z + 2*a^4*b^4*c^4*x^2*z - 13*a^2*b^6*c^4*x^2*z +        38*b^8*c^4*x^2*z - a^4*b^2*c^6*x^2*z - 10*a^2*b^4*c^6*x^2*z - 40*b^6*c^6*x^2*z +        6*a^2*b^2*c^8*x^2*z + 12*b^4*c^8*x^2*z + b^2*c^10*x^2*z + 2*a^12*y^2*z - 3*a^10*b^2*y^2*z +        4*a^8*b^4*y^2*z - 8*a^6*b^6*y^2*z + 6*a^4*b^8*y^2*z - a^2*b^10*y^2*z + 9*a^10*c^2*y^2*z -        14*a^8*b^2*c^2*y^2*z - 11*a^6*b^4*c^2*y^2*z + 4*a^4*b^6*c^2*y^2*z + 6*a^2*b^8*c^2*y^2*z -        38*a^8*c^4*y^2*z + 13*a^6*b^2*c^4*y^2*z - 2*a^4*b^4*c^4*y^2*z + a^2*b^6*c^4*y^2*z +        40*a^6*c^6*y^2*z + 10*a^4*b^2*c^6*y^2*z + a^2*b^4*c^6*y^2*z - 12*a^4*c^8*y^2*z -        6*a^2*b^2*c^8*y^2*z - a^2*c^10*y^2*z - a^10*b^2*x*z^2 - 12*a^8*b^4*x*z^2 + 40*a^6*b^6*x*z^2 -        38*a^4*b^8*x*z^2 + 9*a^2*b^10*x*z^2 + 2*b^12*x*z^2 - 6*a^8*b^2*c^2*x*z^2 + 10*a^6*b^4*c^2*x*z^2 +        13*a^4*b^6*c^2*x*z^2 - 14*a^2*b^8*c^2*x*z^2 - 3*b^10*c^2*x*z^2 + a^6*b^2*c^4*x*z^2 -        2*a^4*b^4*c^4*x*z^2 - 11*a^2*b^6*c^4*x*z^2 + 4*b^8*c^4*x*z^2 + a^4*b^2*c^6*x*z^2 +        4*a^2*b^4*c^6*x*z^2 - 8*b^6*c^6*x*z^2 + 6*a^2*b^2*c^8*x*z^2 + 6*b^4*c^8*x*z^2 - b^2*c^10*x*z^2 -        2*a^12*y*z^2 - 9*a^10*b^2*y*z^2 + 38*a^8*b^4*y*z^2 - 40*a^6*b^6*y*z^2 + 12*a^4*b^8*y*z^2 +        a^2*b^10*y*z^2 + 3*a^10*c^2*y*z^2 + 14*a^8*b^2*c^2*y*z^2 - 13*a^6*b^4*c^2*y*z^2 -        10*a^4*b^6*c^2*y*z^2 + 6*a^2*b^8*c^2*y*z^2 - 4*a^8*c^4*y*z^2 + 11*a^6*b^2*c^4*y*z^2 +        2*a^4*b^4*c^4*y*z^2 - a^2*b^6*c^4*y*z^2 + 8*a^6*c^6*y*z^2 - 4*a^4*b^2*c^6*y*z^2 -        a^2*b^4*c^6*y*z^2 - 6*a^4*c^8*y*z^2 - 6*a^2*b^2*c^8*y*z^2 + a^2*c^10*y*z^2) -      Sqrt(3)*(a^12*c^2*x^2*y + a^10*b^2*c^2*x^2*y - 5*a^8*b^4*c^2*x^2*y + 2*a^6*b^6*c^2*x^2*y -        a^4*b^8*c^2*x^2*y + 5*a^2*b^10*c^2*x^2*y - 3*b^12*c^2*x^2*y - 3*a^10*c^4*x^2*y -        14*a^8*b^2*c^4*x^2*y + 3*a^6*b^4*c^4*x^2*y - 3*a^4*b^6*c^4*x^2*y + 2*a^2*b^8*c^4*x^2*y +        15*b^10*c^4*x^2*y + 4*a^8*c^6*x^2*y + 15*a^6*b^2*c^6*x^2*y - 27*a^2*b^6*c^6*x^2*y -        22*b^8*c^6*x^2*y - 6*a^6*c^8*x^2*y - 3*a^4*b^2*c^8*x^2*y + 15*a^2*b^4*c^8*x^2*y +        4*b^6*c^8*x^2*y + 9*a^4*c^10*x^2*y + 12*a^2*b^2*c^10*x^2*y + 15*b^4*c^10*x^2*y -        7*a^2*c^12*x^2*y - 11*b^2*c^12*x^2*y + 2*c^14*x^2*y + 3*a^12*c^2*x*y^2 - 5*a^10*b^2*c^2*x*y^2 +        a^8*b^4*c^2*x*y^2 - 2*a^6*b^6*c^2*x*y^2 + 5*a^4*b^8*c^2*x*y^2 - a^2*b^10*c^2*x*y^2 -        b^12*c^2*x*y^2 - 15*a^10*c^4*x*y^2 - 2*a^8*b^2*c^4*x*y^2 + 3*a^6*b^4*c^4*x*y^2 -        3*a^4*b^6*c^4*x*y^2 + 14*a^2*b^8*c^4*x*y^2 + 3*b^10*c^4*x*y^2 + 22*a^8*c^6*x*y^2 +        27*a^6*b^2*c^6*x*y^2 - 15*a^2*b^6*c^6*x*y^2 - 4*b^8*c^6*x*y^2 - 4*a^6*c^8*x*y^2 -        15*a^4*b^2*c^8*x*y^2 + 3*a^2*b^4*c^8*x*y^2 + 6*b^6*c^8*x*y^2 - 15*a^4*c^10*x*y^2 -        12*a^2*b^2*c^10*x*y^2 - 9*b^4*c^10*x*y^2 + 11*a^2*c^12*x*y^2 + 7*b^2*c^12*x*y^2 - 2*c^14*x*y^2 -        a^12*b^2*x^2*z + 3*a^10*b^4*x^2*z - 4*a^8*b^6*x^2*z + 6*a^6*b^8*x^2*z - 9*a^4*b^10*x^2*z +        7*a^2*b^12*x^2*z - 2*b^14*x^2*z - a^10*b^2*c^2*x^2*z + 14*a^8*b^4*c^2*x^2*z -        15*a^6*b^6*c^2*x^2*z + 3*a^4*b^8*c^2*x^2*z - 12*a^2*b^10*c^2*x^2*z + 11*b^12*c^2*x^2*z +        5*a^8*b^2*c^4*x^2*z - 3*a^6*b^4*c^4*x^2*z - 15*a^2*b^8*c^4*x^2*z - 15*b^10*c^4*x^2*z -        2*a^6*b^2*c^6*x^2*z + 3*a^4*b^4*c^6*x^2*z + 27*a^2*b^6*c^6*x^2*z - 4*b^8*c^6*x^2*z +        a^4*b^2*c^8*x^2*z - 2*a^2*b^4*c^8*x^2*z + 22*b^6*c^8*x^2*z - 5*a^2*b^2*c^10*x^2*z -        15*b^4*c^10*x^2*z + 3*b^2*c^12*x^2*z + 2*a^14*y^2*z - 7*a^12*b^2*y^2*z + 9*a^10*b^4*y^2*z -        6*a^8*b^6*y^2*z + 4*a^6*b^8*y^2*z - 3*a^4*b^10*y^2*z + a^2*b^12*y^2*z - 11*a^12*c^2*y^2*z +        12*a^10*b^2*c^2*y^2*z - 3*a^8*b^4*c^2*y^2*z + 15*a^6*b^6*c^2*y^2*z - 14*a^4*b^8*c^2*y^2*z +        a^2*b^10*c^2*y^2*z + 15*a^10*c^4*y^2*z + 15*a^8*b^2*c^4*y^2*z + 3*a^4*b^6*c^4*y^2*z -        5*a^2*b^8*c^4*y^2*z + 4*a^8*c^6*y^2*z - 27*a^6*b^2*c^6*y^2*z - 3*a^4*b^4*c^6*y^2*z +        2*a^2*b^6*c^6*y^2*z - 22*a^6*c^8*y^2*z + 2*a^4*b^2*c^8*y^2*z - a^2*b^4*c^8*y^2*z +        15*a^4*c^10*y^2*z + 5*a^2*b^2*c^10*y^2*z - 3*a^2*c^12*y^2*z - 3*a^12*b^2*x*z^2 +        15*a^10*b^4*x*z^2 - 22*a^8*b^6*x*z^2 + 4*a^6*b^8*x*z^2 + 15*a^4*b^10*x*z^2 - 11*a^2*b^12*x*z^2 +        2*b^14*x*z^2 + 5*a^10*b^2*c^2*x*z^2 + 2*a^8*b^4*c^2*x*z^2 - 27*a^6*b^6*c^2*x*z^2 +        15*a^4*b^8*c^2*x*z^2 + 12*a^2*b^10*c^2*x*z^2 - 7*b^12*c^2*x*z^2 - a^8*b^2*c^4*x*z^2 -        3*a^6*b^4*c^4*x*z^2 - 3*a^2*b^8*c^4*x*z^2 + 9*b^10*c^4*x*z^2 + 2*a^6*b^2*c^6*x*z^2 +        3*a^4*b^4*c^6*x*z^2 + 15*a^2*b^6*c^6*x*z^2 - 6*b^8*c^6*x*z^2 - 5*a^4*b^2*c^8*x*z^2 -        14*a^2*b^4*c^8*x*z^2 + 4*b^6*c^8*x*z^2 + a^2*b^2*c^10*x*z^2 - 3*b^4*c^10*x*z^2 + b^2*c^12*x*z^2 -        2*a^14*y*z^2 + 11*a^12*b^2*y*z^2 - 15*a^10*b^4*y*z^2 - 4*a^8*b^6*y*z^2 + 22*a^6*b^8*y*z^2 -        15*a^4*b^10*y*z^2 + 3*a^2*b^12*y*z^2 + 7*a^12*c^2*y*z^2 - 12*a^10*b^2*c^2*y*z^2 -        15*a^8*b^4*c^2*y*z^2 + 27*a^6*b^6*c^2*y*z^2 - 2*a^4*b^8*c^2*y*z^2 - 5*a^2*b^10*c^2*y*z^2 -        9*a^10*c^4*y*z^2 + 3*a^8*b^2*c^4*y*z^2 + 3*a^4*b^6*c^4*y*z^2 + a^2*b^8*c^4*y*z^2 +        6*a^8*c^6*y*z^2 - 15*a^6*b^2*c^6*y*z^2 - 3*a^4*b^4*c^6*y*z^2 - 2*a^2*b^6*c^6*y*z^2 -        4*a^6*c^8*y*z^2 + 14*a^4*b^2*c^8*y*z^2 + 5*a^2*b^4*c^8*y*z^2 + 3*a^4*c^10*y*z^2 -        a^2*b^2*c^10*y*z^2 - a^2*c^12*y*z^2)");
		exp("K438b","Sqrt((a + b - c)*(a - b + c)*(-a + b + c)*(a + b + c))*((-a^10)*c^2*x^2*y +        6*a^8*b^2*c^2*x^2*y + a^6*b^4*c^2*x^2*y + a^4*b^6*c^2*x^2*y - 6*a^2*b^8*c^2*x^2*y -        b^10*c^2*x^2*y + 6*a^8*c^4*x^2*y + 4*a^6*b^2*c^4*x^2*y - 2*a^4*b^4*c^4*x^2*y +        10*a^2*b^6*c^4*x^2*y - 12*b^8*c^4*x^2*y - 8*a^6*c^6*x^2*y - 11*a^4*b^2*c^6*x^2*y +        13*a^2*b^4*c^6*x^2*y + 40*b^6*c^6*x^2*y + 4*a^4*c^8*x^2*y - 14*a^2*b^2*c^8*x^2*y -        38*b^4*c^8*x^2*y - 3*a^2*c^10*x^2*y + 9*b^2*c^10*x^2*y + 2*c^12*x^2*y + a^10*c^2*x*y^2 +        6*a^8*b^2*c^2*x*y^2 - a^6*b^4*c^2*x*y^2 - a^4*b^6*c^2*x*y^2 - 6*a^2*b^8*c^2*x*y^2 +        b^10*c^2*x*y^2 + 12*a^8*c^4*x*y^2 - 10*a^6*b^2*c^4*x*y^2 + 2*a^4*b^4*c^4*x*y^2 -        4*a^2*b^6*c^4*x*y^2 - 6*b^8*c^4*x*y^2 - 40*a^6*c^6*x*y^2 - 13*a^4*b^2*c^6*x*y^2 +        11*a^2*b^4*c^6*x*y^2 + 8*b^6*c^6*x*y^2 + 38*a^4*c^8*x*y^2 + 14*a^2*b^2*c^8*x*y^2 -        4*b^4*c^8*x*y^2 - 9*a^2*c^10*x*y^2 + 3*b^2*c^10*x*y^2 - 2*c^12*x*y^2 + a^10*b^2*x^2*z -        6*a^8*b^4*x^2*z + 8*a^6*b^6*x^2*z - 4*a^4*b^8*x^2*z + 3*a^2*b^10*x^2*z - 2*b^12*x^2*z -        6*a^8*b^2*c^2*x^2*z - 4*a^6*b^4*c^2*x^2*z + 11*a^4*b^6*c^2*x^2*z + 14*a^2*b^8*c^2*x^2*z -        9*b^10*c^2*x^2*z - a^6*b^2*c^4*x^2*z + 2*a^4*b^4*c^4*x^2*z - 13*a^2*b^6*c^4*x^2*z +        38*b^8*c^4*x^2*z - a^4*b^2*c^6*x^2*z - 10*a^2*b^4*c^6*x^2*z - 40*b^6*c^6*x^2*z +        6*a^2*b^2*c^8*x^2*z + 12*b^4*c^8*x^2*z + b^2*c^10*x^2*z + 2*a^12*y^2*z - 3*a^10*b^2*y^2*z +        4*a^8*b^4*y^2*z - 8*a^6*b^6*y^2*z + 6*a^4*b^8*y^2*z - a^2*b^10*y^2*z + 9*a^10*c^2*y^2*z -        14*a^8*b^2*c^2*y^2*z - 11*a^6*b^4*c^2*y^2*z + 4*a^4*b^6*c^2*y^2*z + 6*a^2*b^8*c^2*y^2*z -        38*a^8*c^4*y^2*z + 13*a^6*b^2*c^4*y^2*z - 2*a^4*b^4*c^4*y^2*z + a^2*b^6*c^4*y^2*z +        40*a^6*c^6*y^2*z + 10*a^4*b^2*c^6*y^2*z + a^2*b^4*c^6*y^2*z - 12*a^4*c^8*y^2*z -        6*a^2*b^2*c^8*y^2*z - a^2*c^10*y^2*z - a^10*b^2*x*z^2 - 12*a^8*b^4*x*z^2 + 40*a^6*b^6*x*z^2 -        38*a^4*b^8*x*z^2 + 9*a^2*b^10*x*z^2 + 2*b^12*x*z^2 - 6*a^8*b^2*c^2*x*z^2 + 10*a^6*b^4*c^2*x*z^2 +        13*a^4*b^6*c^2*x*z^2 - 14*a^2*b^8*c^2*x*z^2 - 3*b^10*c^2*x*z^2 + a^6*b^2*c^4*x*z^2 -        2*a^4*b^4*c^4*x*z^2 - 11*a^2*b^6*c^4*x*z^2 + 4*b^8*c^4*x*z^2 + a^4*b^2*c^6*x*z^2 +        4*a^2*b^4*c^6*x*z^2 - 8*b^6*c^6*x*z^2 + 6*a^2*b^2*c^8*x*z^2 + 6*b^4*c^8*x*z^2 - b^2*c^10*x*z^2 -        2*a^12*y*z^2 - 9*a^10*b^2*y*z^2 + 38*a^8*b^4*y*z^2 - 40*a^6*b^6*y*z^2 + 12*a^4*b^8*y*z^2 +        a^2*b^10*y*z^2 + 3*a^10*c^2*y*z^2 + 14*a^8*b^2*c^2*y*z^2 - 13*a^6*b^4*c^2*y*z^2 -        10*a^4*b^6*c^2*y*z^2 + 6*a^2*b^8*c^2*y*z^2 - 4*a^8*c^4*y*z^2 + 11*a^6*b^2*c^4*y*z^2 +        2*a^4*b^4*c^4*y*z^2 - a^2*b^6*c^4*y*z^2 + 8*a^6*c^6*y*z^2 - 4*a^4*b^2*c^6*y*z^2 -        a^2*b^4*c^6*y*z^2 - 6*a^4*c^8*y*z^2 - 6*a^2*b^2*c^8*y*z^2 + a^2*c^10*y*z^2) +      Sqrt(3)*(a^12*c^2*x^2*y + a^10*b^2*c^2*x^2*y - 5*a^8*b^4*c^2*x^2*y + 2*a^6*b^6*c^2*x^2*y -        a^4*b^8*c^2*x^2*y + 5*a^2*b^10*c^2*x^2*y - 3*b^12*c^2*x^2*y - 3*a^10*c^4*x^2*y -        14*a^8*b^2*c^4*x^2*y + 3*a^6*b^4*c^4*x^2*y - 3*a^4*b^6*c^4*x^2*y + 2*a^2*b^8*c^4*x^2*y +        15*b^10*c^4*x^2*y + 4*a^8*c^6*x^2*y + 15*a^6*b^2*c^6*x^2*y - 27*a^2*b^6*c^6*x^2*y -        22*b^8*c^6*x^2*y - 6*a^6*c^8*x^2*y - 3*a^4*b^2*c^8*x^2*y + 15*a^2*b^4*c^8*x^2*y +        4*b^6*c^8*x^2*y + 9*a^4*c^10*x^2*y + 12*a^2*b^2*c^10*x^2*y + 15*b^4*c^10*x^2*y -        7*a^2*c^12*x^2*y - 11*b^2*c^12*x^2*y + 2*c^14*x^2*y + 3*a^12*c^2*x*y^2 - 5*a^10*b^2*c^2*x*y^2 +        a^8*b^4*c^2*x*y^2 - 2*a^6*b^6*c^2*x*y^2 + 5*a^4*b^8*c^2*x*y^2 - a^2*b^10*c^2*x*y^2 -        b^12*c^2*x*y^2 - 15*a^10*c^4*x*y^2 - 2*a^8*b^2*c^4*x*y^2 + 3*a^6*b^4*c^4*x*y^2 -        3*a^4*b^6*c^4*x*y^2 + 14*a^2*b^8*c^4*x*y^2 + 3*b^10*c^4*x*y^2 + 22*a^8*c^6*x*y^2 +        27*a^6*b^2*c^6*x*y^2 - 15*a^2*b^6*c^6*x*y^2 - 4*b^8*c^6*x*y^2 - 4*a^6*c^8*x*y^2 -        15*a^4*b^2*c^8*x*y^2 + 3*a^2*b^4*c^8*x*y^2 + 6*b^6*c^8*x*y^2 - 15*a^4*c^10*x*y^2 -        12*a^2*b^2*c^10*x*y^2 - 9*b^4*c^10*x*y^2 + 11*a^2*c^12*x*y^2 + 7*b^2*c^12*x*y^2 - 2*c^14*x*y^2 -        a^12*b^2*x^2*z + 3*a^10*b^4*x^2*z - 4*a^8*b^6*x^2*z + 6*a^6*b^8*x^2*z - 9*a^4*b^10*x^2*z +        7*a^2*b^12*x^2*z - 2*b^14*x^2*z - a^10*b^2*c^2*x^2*z + 14*a^8*b^4*c^2*x^2*z -        15*a^6*b^6*c^2*x^2*z + 3*a^4*b^8*c^2*x^2*z - 12*a^2*b^10*c^2*x^2*z + 11*b^12*c^2*x^2*z +        5*a^8*b^2*c^4*x^2*z - 3*a^6*b^4*c^4*x^2*z - 15*a^2*b^8*c^4*x^2*z - 15*b^10*c^4*x^2*z -        2*a^6*b^2*c^6*x^2*z + 3*a^4*b^4*c^6*x^2*z + 27*a^2*b^6*c^6*x^2*z - 4*b^8*c^6*x^2*z +        a^4*b^2*c^8*x^2*z - 2*a^2*b^4*c^8*x^2*z + 22*b^6*c^8*x^2*z - 5*a^2*b^2*c^10*x^2*z -        15*b^4*c^10*x^2*z + 3*b^2*c^12*x^2*z + 2*a^14*y^2*z - 7*a^12*b^2*y^2*z + 9*a^10*b^4*y^2*z -        6*a^8*b^6*y^2*z + 4*a^6*b^8*y^2*z - 3*a^4*b^10*y^2*z + a^2*b^12*y^2*z - 11*a^12*c^2*y^2*z +        12*a^10*b^2*c^2*y^2*z - 3*a^8*b^4*c^2*y^2*z + 15*a^6*b^6*c^2*y^2*z - 14*a^4*b^8*c^2*y^2*z +        a^2*b^10*c^2*y^2*z + 15*a^10*c^4*y^2*z + 15*a^8*b^2*c^4*y^2*z + 3*a^4*b^6*c^4*y^2*z -        5*a^2*b^8*c^4*y^2*z + 4*a^8*c^6*y^2*z - 27*a^6*b^2*c^6*y^2*z - 3*a^4*b^4*c^6*y^2*z +        2*a^2*b^6*c^6*y^2*z - 22*a^6*c^8*y^2*z + 2*a^4*b^2*c^8*y^2*z - a^2*b^4*c^8*y^2*z +        15*a^4*c^10*y^2*z + 5*a^2*b^2*c^10*y^2*z - 3*a^2*c^12*y^2*z - 3*a^12*b^2*x*z^2 +        15*a^10*b^4*x*z^2 - 22*a^8*b^6*x*z^2 + 4*a^6*b^8*x*z^2 + 15*a^4*b^10*x*z^2 - 11*a^2*b^12*x*z^2 +        2*b^14*x*z^2 + 5*a^10*b^2*c^2*x*z^2 + 2*a^8*b^4*c^2*x*z^2 - 27*a^6*b^6*c^2*x*z^2 +        15*a^4*b^8*c^2*x*z^2 + 12*a^2*b^10*c^2*x*z^2 - 7*b^12*c^2*x*z^2 - a^8*b^2*c^4*x*z^2 -        3*a^6*b^4*c^4*x*z^2 - 3*a^2*b^8*c^4*x*z^2 + 9*b^10*c^4*x*z^2 + 2*a^6*b^2*c^6*x*z^2 +        3*a^4*b^4*c^6*x*z^2 + 15*a^2*b^6*c^6*x*z^2 - 6*b^8*c^6*x*z^2 - 5*a^4*b^2*c^8*x*z^2 -        14*a^2*b^4*c^8*x*z^2 + 4*b^6*c^8*x*z^2 + a^2*b^2*c^10*x*z^2 - 3*b^4*c^10*x*z^2 + b^2*c^12*x*z^2 -        2*a^14*y*z^2 + 11*a^12*b^2*y*z^2 - 15*a^10*b^4*y*z^2 - 4*a^8*b^6*y*z^2 + 22*a^6*b^8*y*z^2 -        15*a^4*b^10*y*z^2 + 3*a^2*b^12*y*z^2 + 7*a^12*c^2*y*z^2 - 12*a^10*b^2*c^2*y*z^2 -        15*a^8*b^4*c^2*y*z^2 + 27*a^6*b^6*c^2*y*z^2 - 2*a^4*b^8*c^2*y*z^2 - 5*a^2*b^10*c^2*y*z^2 -        9*a^10*c^4*y*z^2 + 3*a^8*b^2*c^4*y*z^2 + 3*a^4*b^6*c^4*y*z^2 + a^2*b^8*c^4*y*z^2 +        6*a^8*c^6*y*z^2 - 15*a^6*b^2*c^6*y*z^2 - 3*a^4*b^4*c^6*y*z^2 - 2*a^2*b^6*c^6*y*z^2 -        4*a^6*c^8*y*z^2 + 14*a^4*b^2*c^8*y*z^2 + 5*a^2*b^4*c^8*y*z^2 + 3*a^4*c^10*y*z^2 -        a^2*b^2*c^10*y*z^2 - a^2*c^12*y*z^2)");
		exp("K439","(-a^10)*c^2*x^2*y + 4*a^8*b^2*c^2*x^2*y - 6*a^6*b^4*c^2*x^2*y + 4*a^4*b^6*c^2*x^2*y -      a^2*b^8*c^2*x^2*y + 5*a^8*c^4*x^2*y - 7*a^6*b^2*c^4*x^2*y + a^4*b^4*c^4*x^2*y +      2*a^2*b^6*c^4*x^2*y - b^8*c^4*x^2*y - 10*a^6*c^6*x^2*y - a^2*b^4*c^6*x^2*y + 2*b^6*c^6*x^2*y +      10*a^4*c^8*x^2*y + 5*a^2*b^2*c^8*x^2*y - 5*a^2*c^10*x^2*y - 2*b^2*c^10*x^2*y + c^12*x^2*y +      a^8*b^2*c^2*x*y^2 - 4*a^6*b^4*c^2*x*y^2 + 6*a^4*b^6*c^2*x*y^2 - 4*a^2*b^8*c^2*x*y^2 +      b^10*c^2*x*y^2 + a^8*c^4*x*y^2 - 2*a^6*b^2*c^4*x*y^2 - a^4*b^4*c^4*x*y^2 + 7*a^2*b^6*c^4*x*y^2 -      5*b^8*c^4*x*y^2 - 2*a^6*c^6*x*y^2 + a^4*b^2*c^6*x*y^2 + 10*b^6*c^6*x*y^2 - 5*a^2*b^2*c^8*x*y^2 -      10*b^4*c^8*x*y^2 + 2*a^2*c^10*x*y^2 + 5*b^2*c^10*x*y^2 - c^12*x*y^2 + a^10*b^2*x^2*z -      5*a^8*b^4*x^2*z + 10*a^6*b^6*x^2*z - 10*a^4*b^8*x^2*z + 5*a^2*b^10*x^2*z - b^12*x^2*z -      4*a^8*b^2*c^2*x^2*z + 7*a^6*b^4*c^2*x^2*z - 5*a^2*b^8*c^2*x^2*z + 2*b^10*c^2*x^2*z +      6*a^6*b^2*c^4*x^2*z - a^4*b^4*c^4*x^2*z + a^2*b^6*c^4*x^2*z - 4*a^4*b^2*c^6*x^2*z -      2*a^2*b^4*c^6*x^2*z - 2*b^6*c^6*x^2*z + a^2*b^2*c^8*x^2*z + b^4*c^8*x^2*z + a^12*y^2*z -      5*a^10*b^2*y^2*z + 10*a^8*b^4*y^2*z - 10*a^6*b^6*y^2*z + 5*a^4*b^8*y^2*z - a^2*b^10*y^2*z -      2*a^10*c^2*y^2*z + 5*a^8*b^2*c^2*y^2*z - 7*a^4*b^6*c^2*y^2*z + 4*a^2*b^8*c^2*y^2*z -      a^6*b^2*c^4*y^2*z + a^4*b^4*c^4*y^2*z - 6*a^2*b^6*c^4*y^2*z + 2*a^6*c^6*y^2*z +      2*a^4*b^2*c^6*y^2*z + 4*a^2*b^4*c^6*y^2*z - a^4*c^8*y^2*z - a^2*b^2*c^8*y^2*z - a^8*b^4*x*z^2 +      2*a^6*b^6*x*z^2 - 2*a^2*b^10*x*z^2 + b^12*x*z^2 - a^8*b^2*c^2*x*z^2 + 2*a^6*b^4*c^2*x*z^2 -      a^4*b^6*c^2*x*z^2 + 5*a^2*b^8*c^2*x*z^2 - 5*b^10*c^2*x*z^2 + 4*a^6*b^2*c^4*x*z^2 +      a^4*b^4*c^4*x*z^2 + 10*b^8*c^4*x*z^2 - 6*a^4*b^2*c^6*x*z^2 - 7*a^2*b^4*c^6*x*z^2 -      10*b^6*c^6*x*z^2 + 4*a^2*b^2*c^8*x*z^2 + 5*b^4*c^8*x*z^2 - b^2*c^10*x*z^2 - a^12*y*z^2 +      2*a^10*b^2*y*z^2 - 2*a^6*b^6*y*z^2 + a^4*b^8*y*z^2 + 5*a^10*c^2*y*z^2 - 5*a^8*b^2*c^2*y*z^2 +      a^6*b^4*c^2*y*z^2 - 2*a^4*b^6*c^2*y*z^2 + a^2*b^8*c^2*y*z^2 - 10*a^8*c^4*y*z^2 -      a^4*b^4*c^4*y*z^2 - 4*a^2*b^6*c^4*y*z^2 + 10*a^6*c^6*y*z^2 + 7*a^4*b^2*c^6*y*z^2 +      6*a^2*b^4*c^6*y*z^2 - 5*a^4*c^8*y*z^2 - 4*a^2*b^2*c^8*y*z^2 + a^2*c^10*y*z^2");
		exp("K440","(-a^6)*c^2*x^2*y + b^6*c^2*x^2*y + 2*a^4*c^4*x^2*y - a^2*c^6*x^2*y - b^2*c^6*x^2*y -      a^6*c^2*x*y^2 + b^6*c^2*x*y^2 - 2*b^4*c^4*x*y^2 + a^2*c^6*x*y^2 + b^2*c^6*x*y^2 + a^6*b^2*x^2*z -      2*a^4*b^4*x^2*z + a^2*b^6*x^2*z + b^6*c^2*x^2*z - b^2*c^6*x^2*z - a^6*b^2*y^2*z + 2*a^4*b^4*y^2*z -      a^2*b^6*y^2*z - a^6*c^2*y^2*z + a^2*c^6*y^2*z + a^6*b^2*x*z^2 - a^2*b^6*x*z^2 - b^6*c^2*x*z^2 +      2*b^4*c^4*x*z^2 - b^2*c^6*x*z^2 + a^6*b^2*y*z^2 - a^2*b^6*y*z^2 + a^6*c^2*y*z^2 - 2*a^4*c^4*y*z^2 +      a^2*c^6*y*z^2");
		exp("K441","(-a^6)*c^2*x^2*y + 2*a^4*b^2*c^2*x^2*y - a^2*b^4*c^2*x^2*y + 3*a^4*c^4*x^2*y - b^4*c^4*x^2*y -      3*a^2*c^6*x^2*y + c^8*x^2*y + a^4*b^2*c^2*x*y^2 - 2*a^2*b^4*c^2*x*y^2 + b^6*c^2*x*y^2 +      a^4*c^4*x*y^2 - 3*b^4*c^4*x*y^2 + 3*b^2*c^6*x*y^2 - c^8*x*y^2 + a^6*b^2*x^2*z - 3*a^4*b^4*x^2*z +      3*a^2*b^6*x^2*z - b^8*x^2*z - 2*a^4*b^2*c^2*x^2*z + a^2*b^2*c^4*x^2*z + b^4*c^4*x^2*z + a^8*y^2*z -      3*a^6*b^2*y^2*z + 3*a^4*b^4*y^2*z - a^2*b^6*y^2*z + 2*a^2*b^4*c^2*y^2*z - a^4*c^4*y^2*z -      a^2*b^2*c^4*y^2*z - a^4*b^4*x*z^2 + b^8*x*z^2 - a^4*b^2*c^2*x*z^2 - 3*b^6*c^2*x*z^2 +      2*a^2*b^2*c^4*x*z^2 + 3*b^4*c^4*x*z^2 - b^2*c^6*x*z^2 - a^8*y*z^2 + a^4*b^4*y*z^2 +      3*a^6*c^2*y*z^2 + a^2*b^4*c^2*y*z^2 - 3*a^4*c^4*y*z^2 - 2*a^2*b^2*c^4*y*z^2 + a^2*c^6*y*z^2");
		exp("K442","(-a^4)*x^2*y + b^4*x^2*y + a^2*c^2*x^2*y + b^2*c^2*x^2*y - 2*c^4*x^2*y - a^4*x*y^2 + b^4*x*y^2 -      a^2*c^2*x*y^2 - b^2*c^2*x*y^2 + 2*c^4*x*y^2 + a^4*x^2*z - a^2*b^2*x^2*z + 2*b^4*x^2*z -      b^2*c^2*x^2*z - c^4*x^2*z - 2*a^4*y^2*z + a^2*b^2*y^2*z - b^4*y^2*z + a^2*c^2*y^2*z + c^4*y^2*z +      a^4*x*z^2 + a^2*b^2*x*z^2 - 2*b^4*x*z^2 + b^2*c^2*x*z^2 - c^4*x*z^2 + 2*a^4*y*z^2 - a^2*b^2*y*z^2 -      b^4*y*z^2 - a^2*c^2*y*z^2 + c^4*y*z^2");
		exp("K443","(-a^4)*c^2*x^2*y + b^4*c^2*x^2*y - 2*b^2*c^4*x^2*y + c^6*x^2*y - a^4*c^2*x*y^2 + b^4*c^2*x*y^2 +      2*a^2*c^4*x*y^2 - c^6*x*y^2 + a^4*b^2*x^2*z - b^6*x^2*z + 2*b^4*c^2*x^2*z - b^2*c^4*x^2*z +      4*a^4*b^2*x*y*z - 4*a^2*b^4*x*y*z - 4*a^4*c^2*x*y*z + 4*b^4*c^2*x*y*z + 4*a^2*c^4*x*y*z -      4*b^2*c^4*x*y*z + a^6*y^2*z - a^2*b^4*y^2*z - 2*a^4*c^2*y^2*z + a^2*c^4*y^2*z + a^4*b^2*x*z^2 -      2*a^2*b^4*x*z^2 + b^6*x*z^2 - b^2*c^4*x*z^2 - a^6*y*z^2 + 2*a^4*b^2*y*z^2 - a^2*b^4*y*z^2 +      a^2*c^4*y*z^2");
		exp("K444","(-b^2)*c^4*x^2*y + a^2*c^4*x*y^2 + b^4*c^2*x^2*z + a^4*b^2*x*y*z - a^2*b^4*x*y*z -      a^4*c^2*x*y*z + b^4*c^2*x*y*z + a^2*c^4*x*y*z - b^2*c^4*x*y*z - a^4*c^2*y^2*z - a^2*b^4*x*z^2 +      a^4*b^2*y*z^2");
		exp("K445","(-a^10)*c^2*x^2*y + 5*a^8*b^2*c^2*x^2*y - 10*a^6*b^4*c^2*x^2*y + 10*a^4*b^6*c^2*x^2*y -      5*a^2*b^8*c^2*x^2*y + b^10*c^2*x^2*y + 3*a^8*c^4*x^2*y - 8*a^6*b^2*c^4*x^2*y +      6*a^4*b^4*c^4*x^2*y - b^8*c^4*x^2*y - 2*a^6*c^6*x^2*y + 2*a^4*b^2*c^6*x^2*y + 2*a^2*b^4*c^6*x^2*y -      2*b^6*c^6*x^2*y - 2*a^4*c^8*x^2*y + 2*b^4*c^8*x^2*y + 3*a^2*c^10*x^2*y + b^2*c^10*x^2*y -      c^12*x^2*y - a^10*c^2*x*y^2 + 5*a^8*b^2*c^2*x*y^2 - 10*a^6*b^4*c^2*x*y^2 + 10*a^4*b^6*c^2*x*y^2 -      5*a^2*b^8*c^2*x*y^2 + b^10*c^2*x*y^2 + a^8*c^4*x*y^2 - 6*a^4*b^4*c^4*x*y^2 + 8*a^2*b^6*c^4*x*y^2 -      3*b^8*c^4*x*y^2 + 2*a^6*c^6*x*y^2 - 2*a^4*b^2*c^6*x*y^2 - 2*a^2*b^4*c^6*x*y^2 + 2*b^6*c^6*x*y^2 -      2*a^4*c^8*x*y^2 + 2*b^4*c^8*x*y^2 - a^2*c^10*x*y^2 - 3*b^2*c^10*x*y^2 + c^12*x*y^2 +      a^10*b^2*x^2*z - 3*a^8*b^4*x^2*z + 2*a^6*b^6*x^2*z + 2*a^4*b^8*x^2*z - 3*a^2*b^10*x^2*z +      b^12*x^2*z - 5*a^8*b^2*c^2*x^2*z + 8*a^6*b^4*c^2*x^2*z - 2*a^4*b^6*c^2*x^2*z - b^10*c^2*x^2*z +      10*a^6*b^2*c^4*x^2*z - 6*a^4*b^4*c^4*x^2*z - 2*a^2*b^6*c^4*x^2*z - 2*b^8*c^4*x^2*z -      10*a^4*b^2*c^6*x^2*z + 2*b^6*c^6*x^2*z + 5*a^2*b^2*c^8*x^2*z + b^4*c^8*x^2*z - b^2*c^10*x^2*z -      a^12*y^2*z + 3*a^10*b^2*y^2*z - 2*a^8*b^4*y^2*z - 2*a^6*b^6*y^2*z + 3*a^4*b^8*y^2*z -      a^2*b^10*y^2*z + a^10*c^2*y^2*z + 2*a^6*b^4*c^2*y^2*z - 8*a^4*b^6*c^2*y^2*z + 5*a^2*b^8*c^2*y^2*z +      2*a^8*c^4*y^2*z + 2*a^6*b^2*c^4*y^2*z + 6*a^4*b^4*c^4*y^2*z - 10*a^2*b^6*c^4*y^2*z -      2*a^6*c^6*y^2*z + 10*a^2*b^4*c^6*y^2*z - a^4*c^8*y^2*z - 5*a^2*b^2*c^8*y^2*z + a^2*c^10*y^2*z +      a^10*b^2*x*z^2 - a^8*b^4*x*z^2 - 2*a^6*b^6*x*z^2 + 2*a^4*b^8*x*z^2 + a^2*b^10*x*z^2 - b^12*x*z^2 -      5*a^8*b^2*c^2*x*z^2 + 2*a^4*b^6*c^2*x*z^2 + 3*b^10*c^2*x*z^2 + 10*a^6*b^2*c^4*x*z^2 +      6*a^4*b^4*c^4*x*z^2 + 2*a^2*b^6*c^4*x*z^2 - 2*b^8*c^4*x*z^2 - 10*a^4*b^2*c^6*x*z^2 -      8*a^2*b^4*c^6*x*z^2 - 2*b^6*c^6*x*z^2 + 5*a^2*b^2*c^8*x*z^2 + 3*b^4*c^8*x*z^2 - b^2*c^10*x*z^2 +      a^12*y*z^2 - a^10*b^2*y*z^2 - 2*a^8*b^4*y*z^2 + 2*a^6*b^6*y*z^2 + a^4*b^8*y*z^2 - a^2*b^10*y*z^2 -      3*a^10*c^2*y*z^2 - 2*a^6*b^4*c^2*y*z^2 + 5*a^2*b^8*c^2*y*z^2 + 2*a^8*c^4*y*z^2 -      2*a^6*b^2*c^4*y*z^2 - 6*a^4*b^4*c^4*y*z^2 - 10*a^2*b^6*c^4*y*z^2 + 2*a^6*c^6*y*z^2 +      8*a^4*b^2*c^6*y*z^2 + 10*a^2*b^4*c^6*y*z^2 - 3*a^4*c^8*y*z^2 - 5*a^2*b^2*c^8*y*z^2 +      a^2*c^10*y*z^2");
		exp("K446","a^4*c^2*x^2*y - 2*a^2*b^2*c^2*x^2*y + b^4*c^2*x^2*y + a^2*c^4*x^2*y + b^2*c^4*x^2*y -      2*c^6*x^2*y - a^4*c^2*x*y^2 + 2*a^2*b^2*c^2*x*y^2 - b^4*c^2*x*y^2 - a^2*c^4*x*y^2 - b^2*c^4*x*y^2 +      2*c^6*x*y^2 - a^4*b^2*x^2*z - a^2*b^4*x^2*z + 2*b^6*x^2*z + 2*a^2*b^2*c^2*x^2*z - b^4*c^2*x^2*z -      b^2*c^4*x^2*z - 6*a^4*b^2*x*y*z + 6*a^2*b^4*x*y*z + 6*a^4*c^2*x*y*z - 6*b^4*c^2*x*y*z -      6*a^2*c^4*x*y*z + 6*b^2*c^4*x*y*z - 2*a^6*y^2*z + a^4*b^2*y^2*z + a^2*b^4*y^2*z + a^4*c^2*y^2*z -      2*a^2*b^2*c^2*y^2*z + a^2*c^4*y^2*z + a^4*b^2*x*z^2 + a^2*b^4*x*z^2 - 2*b^6*x*z^2 -      2*a^2*b^2*c^2*x*z^2 + b^4*c^2*x*z^2 + b^2*c^4*x*z^2 + 2*a^6*y*z^2 - a^4*b^2*y*z^2 - a^2*b^4*y*z^2 -      a^4*c^2*y*z^2 + 2*a^2*b^2*c^2*y*z^2 - a^2*c^4*y*z^2");
		exp("K447","2*a^4*c^2*x^2*y - a^2*b^2*c^2*x^2*y - b^4*c^2*x^2*y - a^2*c^4*x^2*y + 2*b^2*c^4*x^2*y -      c^6*x^2*y + a^4*c^2*x*y^2 + a^2*b^2*c^2*x*y^2 - 2*b^4*c^2*x*y^2 - 2*a^2*c^4*x*y^2 + b^2*c^4*x*y^2 +      c^6*x*y^2 - 2*a^4*b^2*x^2*z + a^2*b^4*x^2*z + b^6*x^2*z + a^2*b^2*c^2*x^2*z - 2*b^4*c^2*x^2*z +      b^2*c^4*x^2*z - 6*a^4*b^2*x*y*z + 6*a^2*b^4*x*y*z + 6*a^4*c^2*x*y*z - 6*b^4*c^2*x*y*z -      6*a^2*c^4*x*y*z + 6*b^2*c^4*x*y*z - a^6*y^2*z - a^4*b^2*y^2*z + 2*a^2*b^4*y^2*z + 2*a^4*c^2*y^2*z -      a^2*b^2*c^2*y^2*z - a^2*c^4*y^2*z - a^4*b^2*x*z^2 + 2*a^2*b^4*x*z^2 - b^6*x*z^2 -      a^2*b^2*c^2*x*z^2 - b^4*c^2*x*z^2 + 2*b^2*c^4*x*z^2 + a^6*y*z^2 - 2*a^4*b^2*y*z^2 + a^2*b^4*y*z^2 +      a^4*c^2*y*z^2 + a^2*b^2*c^2*y*z^2 - 2*a^2*c^4*y*z^2");
		exp("K448","a^16*c^2*x^2*y - 2*a^14*b^2*c^2*x^2*y - a^12*b^4*c^2*x^2*y + a^10*b^6*c^2*x^2*y +      10*a^8*b^8*c^2*x^2*y - 16*a^6*b^10*c^2*x^2*y + 7*a^4*b^12*c^2*x^2*y + a^2*b^14*c^2*x^2*y -      b^16*c^2*x^2*y - 3*a^14*c^4*x^2*y + 11*a^12*b^2*c^4*x^2*y - 30*a^8*b^6*c^4*x^2*y +      23*a^6*b^8*c^4*x^2*y + 9*a^4*b^10*c^4*x^2*y - 12*a^2*b^12*c^4*x^2*y + 2*b^14*c^4*x^2*y -      21*a^10*b^2*c^6*x^2*y + 30*a^8*b^4*c^6*x^2*y + 24*a^6*b^6*c^6*x^2*y - 51*a^4*b^8*c^6*x^2*y +      15*a^2*b^10*c^6*x^2*y + 3*b^12*c^6*x^2*y + 10*a^10*c^8*x^2*y + 10*a^8*b^2*c^8*x^2*y -      55*a^6*b^4*c^8*x^2*y + 31*a^4*b^6*c^8*x^2*y + 16*a^2*b^8*c^8*x^2*y - 10*b^10*c^8*x^2*y -      15*a^8*c^10*x^2*y + 14*a^6*b^2*c^10*x^2*y + 21*a^4*b^4*c^10*x^2*y - 33*a^2*b^6*c^10*x^2*y +      5*b^8*c^10*x^2*y + 9*a^6*c^12*x^2*y - 15*a^4*b^2*c^12*x^2*y + 12*a^2*b^4*c^12*x^2*y +      6*b^6*c^12*x^2*y - 2*a^4*c^14*x^2*y + a^2*b^2*c^14*x^2*y - 7*b^4*c^14*x^2*y + 2*b^2*c^16*x^2*y +      a^16*c^2*x*y^2 - a^14*b^2*c^2*x*y^2 - 7*a^12*b^4*c^2*x*y^2 + 16*a^10*b^6*c^2*x*y^2 -      10*a^8*b^8*c^2*x*y^2 - a^6*b^10*c^2*x*y^2 + a^4*b^12*c^2*x*y^2 + 2*a^2*b^14*c^2*x*y^2 -      b^16*c^2*x*y^2 - 2*a^14*c^4*x*y^2 + 12*a^12*b^2*c^4*x*y^2 - 9*a^10*b^4*c^4*x*y^2 -      23*a^8*b^6*c^4*x*y^2 + 30*a^6*b^8*c^4*x*y^2 - 11*a^2*b^12*c^4*x*y^2 + 3*b^14*c^4*x*y^2 -      3*a^12*c^6*x*y^2 - 15*a^10*b^2*c^6*x*y^2 + 51*a^8*b^4*c^6*x*y^2 - 24*a^6*b^6*c^6*x*y^2 -      30*a^4*b^8*c^6*x*y^2 + 21*a^2*b^10*c^6*x*y^2 + 10*a^10*c^8*x*y^2 - 16*a^8*b^2*c^8*x*y^2 -      31*a^6*b^4*c^8*x*y^2 + 55*a^4*b^6*c^8*x*y^2 - 10*a^2*b^8*c^8*x*y^2 - 10*b^10*c^8*x*y^2 -      5*a^8*c^10*x*y^2 + 33*a^6*b^2*c^10*x*y^2 - 21*a^4*b^4*c^10*x*y^2 - 14*a^2*b^6*c^10*x*y^2 +      15*b^8*c^10*x*y^2 - 6*a^6*c^12*x*y^2 - 12*a^4*b^2*c^12*x*y^2 + 15*a^2*b^4*c^12*x*y^2 -      9*b^6*c^12*x*y^2 + 7*a^4*c^14*x*y^2 - a^2*b^2*c^14*x*y^2 + 2*b^4*c^14*x*y^2 - 2*a^2*c^16*x*y^2 -      a^16*b^2*x^2*z + 3*a^14*b^4*x^2*z - 10*a^10*b^8*x^2*z + 15*a^8*b^10*x^2*z - 9*a^6*b^12*x^2*z +      2*a^4*b^14*x^2*z + 2*a^14*b^2*c^2*x^2*z - 11*a^12*b^4*c^2*x^2*z + 21*a^10*b^6*c^2*x^2*z -      10*a^8*b^8*c^2*x^2*z - 14*a^6*b^10*c^2*x^2*z + 15*a^4*b^12*c^2*x^2*z - a^2*b^14*c^2*x^2*z -      2*b^16*c^2*x^2*z + a^12*b^2*c^4*x^2*z - 30*a^8*b^6*c^4*x^2*z + 55*a^6*b^8*c^4*x^2*z -      21*a^4*b^10*c^4*x^2*z - 12*a^2*b^12*c^4*x^2*z + 7*b^14*c^4*x^2*z - a^10*b^2*c^6*x^2*z +      30*a^8*b^4*c^6*x^2*z - 24*a^6*b^6*c^6*x^2*z - 31*a^4*b^8*c^6*x^2*z + 33*a^2*b^10*c^6*x^2*z -      6*b^12*c^6*x^2*z - 10*a^8*b^2*c^8*x^2*z - 23*a^6*b^4*c^8*x^2*z + 51*a^4*b^6*c^8*x^2*z -      16*a^2*b^8*c^8*x^2*z - 5*b^10*c^8*x^2*z + 16*a^6*b^2*c^10*x^2*z - 9*a^4*b^4*c^10*x^2*z -      15*a^2*b^6*c^10*x^2*z + 10*b^8*c^10*x^2*z - 7*a^4*b^2*c^12*x^2*z + 12*a^2*b^4*c^12*x^2*z -      3*b^6*c^12*x^2*z - a^2*b^2*c^14*x^2*z - 2*b^4*c^14*x^2*z + b^2*c^16*x^2*z - 2*a^16*b^2*x*y*z +      8*a^14*b^4*x*y*z - 12*a^12*b^6*x*y*z + 10*a^10*b^8*x*y*z - 10*a^8*b^10*x*y*z + 12*a^6*b^12*x*y*z -      8*a^4*b^14*x*y*z + 2*a^2*b^16*x*y*z + 2*a^16*c^2*x*y*z - 14*a^12*b^4*c^2*x*y*z +      20*a^10*b^6*c^2*x*y*z - 20*a^6*b^10*c^2*x*y*z + 14*a^4*b^12*c^2*x*y*z - 2*b^16*c^2*x*y*z -      8*a^14*c^4*x*y*z + 14*a^12*b^2*c^4*x*y*z - 20*a^8*b^6*c^4*x*y*z + 20*a^6*b^8*c^4*x*y*z -      14*a^2*b^12*c^4*x*y*z + 8*b^14*c^4*x*y*z + 12*a^12*c^6*x*y*z - 20*a^10*b^2*c^6*x*y*z +      20*a^8*b^4*c^6*x*y*z - 20*a^4*b^8*c^6*x*y*z + 20*a^2*b^10*c^6*x*y*z - 12*b^12*c^6*x*y*z -      10*a^10*c^8*x*y*z - 20*a^6*b^4*c^8*x*y*z + 20*a^4*b^6*c^8*x*y*z + 10*b^10*c^8*x*y*z +      10*a^8*c^10*x*y*z + 20*a^6*b^2*c^10*x*y*z - 20*a^2*b^6*c^10*x*y*z - 10*b^8*c^10*x*y*z -      12*a^6*c^12*x*y*z - 14*a^4*b^2*c^12*x*y*z + 14*a^2*b^4*c^12*x*y*z + 12*b^6*c^12*x*y*z +      8*a^4*c^14*x*y*z - 8*b^4*c^14*x*y*z - 2*a^2*c^16*x*y*z + 2*b^2*c^16*x*y*z - 2*a^14*b^4*y^2*z +      9*a^12*b^6*y^2*z - 15*a^10*b^8*y^2*z + 10*a^8*b^10*y^2*z - 3*a^4*b^14*y^2*z + a^2*b^16*y^2*z +      2*a^16*c^2*y^2*z + a^14*b^2*c^2*y^2*z - 15*a^12*b^4*c^2*y^2*z + 14*a^10*b^6*c^2*y^2*z +      10*a^8*b^8*c^2*y^2*z - 21*a^6*b^10*c^2*y^2*z + 11*a^4*b^12*c^2*y^2*z - 2*a^2*b^14*c^2*y^2*z -      7*a^14*c^4*y^2*z + 12*a^12*b^2*c^4*y^2*z + 21*a^10*b^4*c^4*y^2*z - 55*a^8*b^6*c^4*y^2*z +      30*a^6*b^8*c^4*y^2*z - a^2*b^12*c^4*y^2*z + 6*a^12*c^6*y^2*z - 33*a^10*b^2*c^6*y^2*z +      31*a^8*b^4*c^6*y^2*z + 24*a^6*b^6*c^6*y^2*z - 30*a^4*b^8*c^6*y^2*z + a^2*b^10*c^6*y^2*z +      5*a^10*c^8*y^2*z + 16*a^8*b^2*c^8*y^2*z - 51*a^6*b^4*c^8*y^2*z + 23*a^4*b^6*c^8*y^2*z +      10*a^2*b^8*c^8*y^2*z - 10*a^8*c^10*y^2*z + 15*a^6*b^2*c^10*y^2*z + 9*a^4*b^4*c^10*y^2*z -      16*a^2*b^6*c^10*y^2*z + 3*a^6*c^12*y^2*z - 12*a^4*b^2*c^12*y^2*z + 7*a^2*b^4*c^12*y^2*z +      2*a^4*c^14*y^2*z + a^2*b^2*c^14*y^2*z - a^2*c^16*y^2*z - a^16*b^2*x*z^2 + 2*a^14*b^4*x*z^2 +      3*a^12*b^6*x*z^2 - 10*a^10*b^8*x*z^2 + 5*a^8*b^10*x*z^2 + 6*a^6*b^12*x*z^2 - 7*a^4*b^14*x*z^2 +      2*a^2*b^16*x*z^2 + a^14*b^2*c^2*x*z^2 - 12*a^12*b^4*c^2*x*z^2 + 15*a^10*b^6*c^2*x*z^2 +      16*a^8*b^8*c^2*x*z^2 - 33*a^6*b^10*c^2*x*z^2 + 12*a^4*b^12*c^2*x*z^2 + a^2*b^14*c^2*x*z^2 +      7*a^12*b^2*c^4*x*z^2 + 9*a^10*b^4*c^4*x*z^2 - 51*a^8*b^6*c^4*x*z^2 + 31*a^6*b^8*c^4*x*z^2 +      21*a^4*b^10*c^4*x*z^2 - 15*a^2*b^12*c^4*x*z^2 - 2*b^14*c^4*x*z^2 - 16*a^10*b^2*c^6*x*z^2 +      23*a^8*b^4*c^6*x*z^2 + 24*a^6*b^6*c^6*x*z^2 - 55*a^4*b^8*c^6*x*z^2 + 14*a^2*b^10*c^6*x*z^2 +      9*b^12*c^6*x*z^2 + 10*a^8*b^2*c^8*x*z^2 - 30*a^6*b^4*c^8*x*z^2 + 30*a^4*b^6*c^8*x*z^2 +      10*a^2*b^8*c^8*x*z^2 - 15*b^10*c^8*x*z^2 + a^6*b^2*c^10*x*z^2 - 21*a^2*b^6*c^10*x*z^2 +      10*b^8*c^10*x*z^2 - a^4*b^2*c^12*x*z^2 + 11*a^2*b^4*c^12*x*z^2 - 2*a^2*b^2*c^14*x*z^2 -      3*b^4*c^14*x*z^2 + b^2*c^16*x*z^2 - 2*a^16*b^2*y*z^2 + 7*a^14*b^4*y*z^2 - 6*a^12*b^6*y*z^2 -      5*a^10*b^8*y*z^2 + 10*a^8*b^10*y*z^2 - 3*a^6*b^12*y*z^2 - 2*a^4*b^14*y*z^2 + a^2*b^16*y*z^2 -      a^14*b^2*c^2*y*z^2 - 12*a^12*b^4*c^2*y*z^2 + 33*a^10*b^6*c^2*y*z^2 - 16*a^8*b^8*c^2*y*z^2 -      15*a^6*b^10*c^2*y*z^2 + 12*a^4*b^12*c^2*y*z^2 - a^2*b^14*c^2*y*z^2 + 2*a^14*c^4*y*z^2 +      15*a^12*b^2*c^4*y*z^2 - 21*a^10*b^4*c^4*y*z^2 - 31*a^8*b^6*c^4*y*z^2 + 51*a^6*b^8*c^4*y*z^2 -      9*a^4*b^10*c^4*y*z^2 - 7*a^2*b^12*c^4*y*z^2 - 9*a^12*c^6*y*z^2 - 14*a^10*b^2*c^6*y*z^2 +      55*a^8*b^4*c^6*y*z^2 - 24*a^6*b^6*c^6*y*z^2 - 23*a^4*b^8*c^6*y*z^2 + 16*a^2*b^10*c^6*y*z^2 +      15*a^10*c^8*y*z^2 - 10*a^8*b^2*c^8*y*z^2 - 30*a^6*b^4*c^8*y*z^2 + 30*a^4*b^6*c^8*y*z^2 -      10*a^2*b^8*c^8*y*z^2 - 10*a^8*c^10*y*z^2 + 21*a^6*b^2*c^10*y*z^2 - a^2*b^6*c^10*y*z^2 -      11*a^4*b^2*c^12*y*z^2 + a^2*b^4*c^12*y*z^2 + 3*a^4*c^14*y*z^2 + 2*a^2*b^2*c^14*y*z^2 -      a^2*c^16*y*z^2");
		exp("K449","a^6*x^2*y - 3*a^4*b^2*x^2*y + 3*a^2*b^4*x^2*y - b^6*x^2*y + 2*a^4*c^2*x^2*y + a^2*b^2*c^2*x^2*y -      3*b^4*c^2*x^2*y - 4*a^2*c^4*x^2*y + 3*b^2*c^4*x^2*y + c^6*x^2*y + a^6*x*y^2 - 3*a^4*b^2*x*y^2 +      3*a^2*b^4*x*y^2 - b^6*x*y^2 + 3*a^4*c^2*x*y^2 - a^2*b^2*c^2*x*y^2 - 2*b^4*c^2*x*y^2 -      3*a^2*c^4*x*y^2 + 4*b^2*c^4*x*y^2 - c^6*x*y^2 - a^6*x^2*z - 2*a^4*b^2*x^2*z + 4*a^2*b^4*x^2*z -      b^6*x^2*z + 3*a^4*c^2*x^2*z - a^2*b^2*c^2*x^2*z - 3*b^4*c^2*x^2*z - 3*a^2*c^4*x^2*z +      3*b^2*c^4*x^2*z + c^6*x^2*z - 6*a^4*b^2*x*y*z + 6*a^2*b^4*x*y*z + 6*a^4*c^2*x*y*z -      6*b^4*c^2*x*y*z - 6*a^2*c^4*x*y*z + 6*b^2*c^4*x*y*z + a^6*y^2*z - 4*a^4*b^2*y^2*z +      2*a^2*b^4*y^2*z + b^6*y^2*z + 3*a^4*c^2*y^2*z + a^2*b^2*c^2*y^2*z - 3*b^4*c^2*y^2*z -      3*a^2*c^4*y^2*z + 3*b^2*c^4*y^2*z - c^6*y^2*z - a^6*x*z^2 - 3*a^4*b^2*x*z^2 + 3*a^2*b^4*x*z^2 +      b^6*x*z^2 + 3*a^4*c^2*x*z^2 + a^2*b^2*c^2*x*z^2 - 4*b^4*c^2*x*z^2 - 3*a^2*c^4*x*z^2 +      2*b^2*c^4*x*z^2 + c^6*x*z^2 - a^6*y*z^2 - 3*a^4*b^2*y*z^2 + 3*a^2*b^4*y*z^2 + b^6*y*z^2 +      4*a^4*c^2*y*z^2 - a^2*b^2*c^2*y*z^2 - 3*b^4*c^2*y*z^2 - 2*a^2*c^4*y*z^2 + 3*b^2*c^4*y*z^2 -      c^6*y*z^2");
		exp("K450","(-a^16)*x^2*y + 4*a^14*b^2*x^2*y - 6*a^12*b^4*x^2*y + 4*a^10*b^6*x^2*y - 4*a^6*b^10*x^2*y +      6*a^4*b^12*x^2*y - 4*a^2*b^14*x^2*y + b^16*x^2*y + 3*a^14*c^2*x^2*y - 9*a^12*b^2*c^2*x^2*y +      8*a^10*b^4*c^2*x^2*y - a^8*b^6*c^2*x^2*y + a^6*b^8*c^2*x^2*y - 5*a^4*b^10*c^2*x^2*y +      4*a^2*b^12*c^2*x^2*y - b^14*c^2*x^2*y - 5*a^12*c^4*x^2*y + 10*a^10*b^2*c^4*x^2*y -      4*a^8*b^4*c^4*x^2*y - 2*a^6*b^6*c^4*x^2*y + 3*a^4*b^8*c^4*x^2*y - 2*a^2*b^10*c^4*x^2*y +      8*a^10*c^6*x^2*y - 11*a^8*b^2*c^6*x^2*y + a^6*b^4*c^6*x^2*y - 4*a^4*b^6*c^6*x^2*y +      7*a^2*b^8*c^6*x^2*y + b^10*c^6*x^2*y - 9*a^8*c^8*x^2*y + 8*a^6*b^2*c^8*x^2*y +      3*a^4*b^4*c^8*x^2*y - 6*a^2*b^6*c^8*x^2*y - 6*b^8*c^8*x^2*y + 7*a^6*c^10*x^2*y +      2*a^4*b^2*c^10*x^2*y + 3*a^2*b^4*c^10*x^2*y + 8*b^6*c^10*x^2*y - 7*a^4*c^12*x^2*y -      8*a^2*b^2*c^12*x^2*y - 5*b^4*c^12*x^2*y + 6*a^2*c^14*x^2*y + 4*b^2*c^14*x^2*y - 2*c^16*x^2*y -      a^16*x*y^2 + 4*a^14*b^2*x*y^2 - 6*a^12*b^4*x*y^2 + 4*a^10*b^6*x*y^2 - 4*a^6*b^10*x*y^2 +      6*a^4*b^12*x*y^2 - 4*a^2*b^14*x*y^2 + b^16*x*y^2 + a^14*c^2*x*y^2 - 4*a^12*b^2*c^2*x*y^2 +      5*a^10*b^4*c^2*x*y^2 - a^8*b^6*c^2*x*y^2 + a^6*b^8*c^2*x*y^2 - 8*a^4*b^10*c^2*x*y^2 +      9*a^2*b^12*c^2*x*y^2 - 3*b^14*c^2*x*y^2 + 2*a^10*b^2*c^4*x*y^2 - 3*a^8*b^4*c^4*x*y^2 +      2*a^6*b^6*c^4*x*y^2 + 4*a^4*b^8*c^4*x*y^2 - 10*a^2*b^10*c^4*x*y^2 + 5*b^12*c^4*x*y^2 -      a^10*c^6*x*y^2 - 7*a^8*b^2*c^6*x*y^2 + 4*a^6*b^4*c^6*x*y^2 - a^4*b^6*c^6*x*y^2 +      11*a^2*b^8*c^6*x*y^2 - 8*b^10*c^6*x*y^2 + 6*a^8*c^8*x*y^2 + 6*a^6*b^2*c^8*x*y^2 -      3*a^4*b^4*c^8*x*y^2 - 8*a^2*b^6*c^8*x*y^2 + 9*b^8*c^8*x*y^2 - 8*a^6*c^10*x*y^2 -      3*a^4*b^2*c^10*x*y^2 - 2*a^2*b^4*c^10*x*y^2 - 7*b^6*c^10*x*y^2 + 5*a^4*c^12*x*y^2 +      8*a^2*b^2*c^12*x*y^2 + 7*b^4*c^12*x*y^2 - 4*a^2*c^14*x*y^2 - 6*b^2*c^14*x*y^2 + 2*c^16*x*y^2 +      a^16*x^2*z - 3*a^14*b^2*x^2*z + 5*a^12*b^4*x^2*z - 8*a^10*b^6*x^2*z + 9*a^8*b^8*x^2*z -      7*a^6*b^10*x^2*z + 7*a^4*b^12*x^2*z - 6*a^2*b^14*x^2*z + 2*b^16*x^2*z - 4*a^14*c^2*x^2*z +      9*a^12*b^2*c^2*x^2*z - 10*a^10*b^4*c^2*x^2*z + 11*a^8*b^6*c^2*x^2*z - 8*a^6*b^8*c^2*x^2*z -      2*a^4*b^10*c^2*x^2*z + 8*a^2*b^12*c^2*x^2*z - 4*b^14*c^2*x^2*z + 6*a^12*c^4*x^2*z -      8*a^10*b^2*c^4*x^2*z + 4*a^8*b^4*c^4*x^2*z - a^6*b^6*c^4*x^2*z - 3*a^4*b^8*c^4*x^2*z -      3*a^2*b^10*c^4*x^2*z + 5*b^12*c^4*x^2*z - 4*a^10*c^6*x^2*z + a^8*b^2*c^6*x^2*z +      2*a^6*b^4*c^6*x^2*z + 4*a^4*b^6*c^6*x^2*z + 6*a^2*b^8*c^6*x^2*z - 8*b^10*c^6*x^2*z -      a^6*b^2*c^8*x^2*z - 3*a^4*b^4*c^8*x^2*z - 7*a^2*b^6*c^8*x^2*z + 6*b^8*c^8*x^2*z +      4*a^6*c^10*x^2*z + 5*a^4*b^2*c^10*x^2*z + 2*a^2*b^4*c^10*x^2*z - b^6*c^10*x^2*z -      6*a^4*c^12*x^2*z - 4*a^2*b^2*c^12*x^2*z + 4*a^2*c^14*x^2*z + b^2*c^14*x^2*z - c^16*x^2*z -      2*a^16*y^2*z + 6*a^14*b^2*y^2*z - 7*a^12*b^4*y^2*z + 7*a^10*b^6*y^2*z - 9*a^8*b^8*y^2*z +      8*a^6*b^10*y^2*z - 5*a^4*b^12*y^2*z + 3*a^2*b^14*y^2*z - b^16*y^2*z + 4*a^14*c^2*y^2*z -      8*a^12*b^2*c^2*y^2*z + 2*a^10*b^4*c^2*y^2*z + 8*a^8*b^6*c^2*y^2*z - 11*a^6*b^8*c^2*y^2*z +      10*a^4*b^10*c^2*y^2*z - 9*a^2*b^12*c^2*y^2*z + 4*b^14*c^2*y^2*z - 5*a^12*c^4*y^2*z +      3*a^10*b^2*c^4*y^2*z + 3*a^8*b^4*c^4*y^2*z + a^6*b^6*c^4*y^2*z - 4*a^4*b^8*c^4*y^2*z +      8*a^2*b^10*c^4*y^2*z - 6*b^12*c^4*y^2*z + 8*a^10*c^6*y^2*z - 6*a^8*b^2*c^6*y^2*z -      4*a^6*b^4*c^6*y^2*z - 2*a^4*b^6*c^6*y^2*z - a^2*b^8*c^6*y^2*z + 4*b^10*c^6*y^2*z -      6*a^8*c^8*y^2*z + 7*a^6*b^2*c^8*y^2*z + 3*a^4*b^4*c^8*y^2*z + a^2*b^6*c^8*y^2*z + a^6*c^10*y^2*z -      2*a^4*b^2*c^10*y^2*z - 5*a^2*b^4*c^10*y^2*z - 4*b^6*c^10*y^2*z + 4*a^2*b^2*c^12*y^2*z +      6*b^4*c^12*y^2*z - a^2*c^14*y^2*z - 4*b^2*c^14*y^2*z + c^16*y^2*z + a^16*x*z^2 - a^14*b^2*x*z^2 +      a^10*b^6*x*z^2 - 6*a^8*b^8*x*z^2 + 8*a^6*b^10*x*z^2 - 5*a^4*b^12*x*z^2 + 4*a^2*b^14*x*z^2 -      2*b^16*x*z^2 - 4*a^14*c^2*x*z^2 + 4*a^12*b^2*c^2*x*z^2 - 2*a^10*b^4*c^2*x*z^2 +      7*a^8*b^6*c^2*x*z^2 - 6*a^6*b^8*c^2*x*z^2 + 3*a^4*b^10*c^2*x*z^2 - 8*a^2*b^12*c^2*x*z^2 +      6*b^14*c^2*x*z^2 + 6*a^12*c^4*x*z^2 - 5*a^10*b^2*c^4*x*z^2 + 3*a^8*b^4*c^4*x*z^2 -      4*a^6*b^6*c^4*x*z^2 + 3*a^4*b^8*c^4*x*z^2 + 2*a^2*b^10*c^4*x*z^2 - 7*b^12*c^4*x*z^2 -      4*a^10*c^6*x*z^2 + a^8*b^2*c^6*x*z^2 - 2*a^6*b^4*c^6*x*z^2 + a^4*b^6*c^6*x*z^2 +      8*a^2*b^8*c^6*x*z^2 + 7*b^10*c^6*x*z^2 - a^6*b^2*c^8*x*z^2 - 4*a^4*b^4*c^8*x*z^2 -      11*a^2*b^6*c^8*x*z^2 - 9*b^8*c^8*x*z^2 + 4*a^6*c^10*x*z^2 + 8*a^4*b^2*c^10*x*z^2 +      10*a^2*b^4*c^10*x*z^2 + 8*b^6*c^10*x*z^2 - 6*a^4*c^12*x*z^2 - 9*a^2*b^2*c^12*x*z^2 -      5*b^4*c^12*x*z^2 + 4*a^2*c^14*x*z^2 + 3*b^2*c^14*x*z^2 - c^16*x*z^2 + 2*a^16*y*z^2 -      4*a^14*b^2*y*z^2 + 5*a^12*b^4*y*z^2 - 8*a^10*b^6*y*z^2 + 6*a^8*b^8*y*z^2 - a^6*b^10*y*z^2 +      a^2*b^14*y*z^2 - b^16*y*z^2 - 6*a^14*c^2*y*z^2 + 8*a^12*b^2*c^2*y*z^2 - 3*a^10*b^4*c^2*y*z^2 +      6*a^8*b^6*c^2*y*z^2 - 7*a^6*b^8*c^2*y*z^2 + 2*a^4*b^10*c^2*y*z^2 - 4*a^2*b^12*c^2*y*z^2 +      4*b^14*c^2*y*z^2 + 7*a^12*c^4*y*z^2 - 2*a^10*b^2*c^4*y*z^2 - 3*a^8*b^4*c^4*y*z^2 +      4*a^6*b^6*c^4*y*z^2 - 3*a^4*b^8*c^4*y*z^2 + 5*a^2*b^10*c^4*y*z^2 - 6*b^12*c^4*y*z^2 -      7*a^10*c^6*y*z^2 - 8*a^8*b^2*c^6*y*z^2 - a^6*b^4*c^6*y*z^2 + 2*a^4*b^6*c^6*y*z^2 -      a^2*b^8*c^6*y*z^2 + 4*b^10*c^6*y*z^2 + 9*a^8*c^8*y*z^2 + 11*a^6*b^2*c^8*y*z^2 +      4*a^4*b^4*c^8*y*z^2 + a^2*b^6*c^8*y*z^2 - 8*a^6*c^10*y*z^2 - 10*a^4*b^2*c^10*y*z^2 -      8*a^2*b^4*c^10*y*z^2 - 4*b^6*c^10*y*z^2 + 5*a^4*c^12*y*z^2 + 9*a^2*b^2*c^12*y*z^2 +      6*b^4*c^12*y*z^2 - 3*a^2*c^14*y*z^2 - 4*b^2*c^14*y*z^2 + c^16*y*z^2");
		exp("K451","a^12*x^2*y - 3*a^8*b^4*x^2*y + 3*a^4*b^8*x^2*y - b^12*x^2*y - 5*a^10*c^2*x^2*y -      3*a^8*b^2*c^2*x^2*y + 29*a^6*b^4*c^2*x^2*y - 20*a^4*b^6*c^2*x^2*y - 6*a^2*b^8*c^2*x^2*y +      5*b^10*c^2*x^2*y + 16*a^8*c^4*x^2*y - 22*a^6*b^2*c^4*x^2*y - 27*a^4*b^4*c^4*x^2*y +      40*a^2*b^6*c^4*x^2*y - 7*b^8*c^4*x^2*y - 17*a^6*c^6*x^2*y + 48*a^4*b^2*c^6*x^2*y -      21*a^2*b^4*c^6*x^2*y - 10*b^6*c^6*x^2*y + a^4*c^8*x^2*y - 18*a^2*b^2*c^8*x^2*y + 17*b^4*c^8*x^2*y +      4*a^2*c^10*x^2*y - 4*b^2*c^10*x^2*y + a^12*x*y^2 - 3*a^8*b^4*x*y^2 + 3*a^4*b^8*x*y^2 - b^12*x*y^2 -      5*a^10*c^2*x*y^2 + 6*a^8*b^2*c^2*x*y^2 + 20*a^6*b^4*c^2*x*y^2 - 29*a^4*b^6*c^2*x*y^2 +      3*a^2*b^8*c^2*x*y^2 + 5*b^10*c^2*x*y^2 + 7*a^8*c^4*x*y^2 - 40*a^6*b^2*c^4*x*y^2 +      27*a^4*b^4*c^4*x*y^2 + 22*a^2*b^6*c^4*x*y^2 - 16*b^8*c^4*x*y^2 + 10*a^6*c^6*x*y^2 +      21*a^4*b^2*c^6*x*y^2 - 48*a^2*b^4*c^6*x*y^2 + 17*b^6*c^6*x*y^2 - 17*a^4*c^8*x*y^2 +      18*a^2*b^2*c^8*x*y^2 - b^4*c^8*x*y^2 + 4*a^2*c^10*x*y^2 - 4*b^2*c^10*x*y^2 - a^12*x^2*z +      5*a^10*b^2*x^2*z - 16*a^8*b^4*x^2*z + 17*a^6*b^6*x^2*z - a^4*b^8*x^2*z - 4*a^2*b^10*x^2*z +      3*a^8*b^2*c^2*x^2*z + 22*a^6*b^4*c^2*x^2*z - 48*a^4*b^6*c^2*x^2*z + 18*a^2*b^8*c^2*x^2*z +      4*b^10*c^2*x^2*z + 3*a^8*c^4*x^2*z - 29*a^6*b^2*c^4*x^2*z + 27*a^4*b^4*c^4*x^2*z +      21*a^2*b^6*c^4*x^2*z - 17*b^8*c^4*x^2*z + 20*a^4*b^2*c^6*x^2*z - 40*a^2*b^4*c^6*x^2*z +      10*b^6*c^6*x^2*z - 3*a^4*c^8*x^2*z + 6*a^2*b^2*c^8*x^2*z + 7*b^4*c^8*x^2*z - 5*b^2*c^10*x^2*z +      c^12*x^2*z + 4*a^10*b^2*y^2*z + a^8*b^4*y^2*z - 17*a^6*b^6*y^2*z + 16*a^4*b^8*y^2*z -      5*a^2*b^10*y^2*z + b^12*y^2*z - 4*a^10*c^2*y^2*z - 18*a^8*b^2*c^2*y^2*z + 48*a^6*b^4*c^2*y^2*z -      22*a^4*b^6*c^2*y^2*z - 3*a^2*b^8*c^2*y^2*z + 17*a^8*c^4*y^2*z - 21*a^6*b^2*c^4*y^2*z -      27*a^4*b^4*c^4*y^2*z + 29*a^2*b^6*c^4*y^2*z - 3*b^8*c^4*y^2*z - 10*a^6*c^6*y^2*z +      40*a^4*b^2*c^6*y^2*z - 20*a^2*b^4*c^6*y^2*z - 7*a^4*c^8*y^2*z - 6*a^2*b^2*c^8*y^2*z +      3*b^4*c^8*y^2*z + 5*a^2*c^10*y^2*z - c^12*y^2*z - a^12*x*z^2 + 5*a^10*b^2*x*z^2 - 7*a^8*b^4*x*z^2 -      10*a^6*b^6*x*z^2 + 17*a^4*b^8*x*z^2 - 4*a^2*b^10*x*z^2 - 6*a^8*b^2*c^2*x*z^2 +      40*a^6*b^4*c^2*x*z^2 - 21*a^4*b^6*c^2*x*z^2 - 18*a^2*b^8*c^2*x*z^2 + 4*b^10*c^2*x*z^2 +      3*a^8*c^4*x*z^2 - 20*a^6*b^2*c^4*x*z^2 - 27*a^4*b^4*c^4*x*z^2 + 48*a^2*b^6*c^4*x*z^2 +      b^8*c^4*x*z^2 + 29*a^4*b^2*c^6*x*z^2 - 22*a^2*b^4*c^6*x*z^2 - 17*b^6*c^6*x*z^2 - 3*a^4*c^8*x*z^2 -      3*a^2*b^2*c^8*x*z^2 + 16*b^4*c^8*x*z^2 - 5*b^2*c^10*x*z^2 + c^12*x*z^2 + 4*a^10*b^2*y*z^2 -      17*a^8*b^4*y*z^2 + 10*a^6*b^6*y*z^2 + 7*a^4*b^8*y*z^2 - 5*a^2*b^10*y*z^2 + b^12*y*z^2 -      4*a^10*c^2*y*z^2 + 18*a^8*b^2*c^2*y*z^2 + 21*a^6*b^4*c^2*y*z^2 - 40*a^4*b^6*c^2*y*z^2 +      6*a^2*b^8*c^2*y*z^2 - a^8*c^4*y*z^2 - 48*a^6*b^2*c^4*y*z^2 + 27*a^4*b^4*c^4*y*z^2 +      20*a^2*b^6*c^4*y*z^2 - 3*b^8*c^4*y*z^2 + 17*a^6*c^6*y*z^2 + 22*a^4*b^2*c^6*y*z^2 -      29*a^2*b^4*c^6*y*z^2 - 16*a^4*c^8*y*z^2 + 3*a^2*b^2*c^8*y*z^2 + 3*b^4*c^8*y*z^2 +      5*a^2*c^10*y*z^2 - c^12*y*z^2");
		exp("K452","(-a^4)*x^2*y + b^4*x^2*y + 5*a^2*c^2*x^2*y + b^2*c^2*x^2*y - 6*c^4*x^2*y - a^4*x*y^2 +      b^4*x*y^2 - a^2*c^2*x*y^2 - 5*b^2*c^2*x*y^2 + 6*c^4*x*y^2 + a^4*x^2*z - 5*a^2*b^2*x^2*z +      6*b^4*x^2*z - b^2*c^2*x^2*z - c^4*x^2*z - 6*a^4*y^2*z + 5*a^2*b^2*y^2*z - b^4*y^2*z +      a^2*c^2*y^2*z + c^4*y^2*z + a^4*x*z^2 + a^2*b^2*x*z^2 - 6*b^4*x*z^2 + 5*b^2*c^2*x*z^2 - c^4*x*z^2 +      6*a^4*y*z^2 - a^2*b^2*y*z^2 - b^4*y*z^2 - 5*a^2*c^2*y*z^2 + c^4*y*z^2");
		exp("K453","a*c*x^2*y + b*c*x^2*y - 2*c^2*x^2*y - a*c*x*y^2 - b*c*x*y^2 + 2*c^2*x*y^2 - a*b*x^2*z +      2*b^2*x^2*z - b*c*x^2*z - 2*a^2*y^2*z + a*b*y^2*z + a*c*y^2*z + a*b*x*z^2 - 2*b^2*x*z^2 +      b*c*x*z^2 + 2*a^2*y*z^2 - a*b*y*z^2 - a*c*y*z^2");
		exp("K454","2*a*b*c^3*x^2*y - b^2*c^3*x^2*y - b*c^4*x^2*y + a^2*c^3*x*y^2 - 2*a*b*c^3*x*y^2 + a*c^4*x*y^2 -      2*a*b^3*c*x^2*z + b^4*c*x^2*z + b^3*c^2*x^2*z - a^4*c*y^2*z + 2*a^3*b*c*y^2*z - a^3*c^2*y^2*z -      a^2*b^3*x*z^2 - a*b^4*x*z^2 + 2*a*b^3*c*x*z^2 + a^4*b*y*z^2 + a^3*b^2*y*z^2 - 2*a^3*b*c*y*z^2");
		exp("K455","a^2*x^2*y - b^2*x^2*y + a*c*x^2*y + c^2*x^2*y + a^2*x*y^2 - b^2*x*y^2 - b*c*x*y^2 - c^2*x*y^2 -      a^2*x^2*z - a*b*x^2*z - b^2*x^2*z + c^2*x^2*z + a^2*y^2*z + a*b*y^2*z + b^2*y^2*z - c^2*y^2*z -      a^2*x*z^2 + b^2*x*z^2 + b*c*x*z^2 + c^2*x*z^2 - a^2*y*z^2 + b^2*y*z^2 - a*c*y*z^2 - c^2*y*z^2");
		exp("K456A","(-a^2)*b^2*x^3 + b^4*x^3 - a^2*c^2*x^3 - 2*b^2*c^2*x^3 + c^4*x^3 - 2*a^2*c^2*x^2*y + a^4*x*y^2 - a^2*b^2*x*y^2 + a^2*c^2*x*y^2 - 2*a^2*b^2*x^2*z + 2*a^4*y^2*z +      a^4*x*z^2 + a^2*b^2*x*z^2 - a^2*c^2*x*z^2 + 2*a^4*y*z^2");
		exp("K456B","(-a^2)*b^2*x^2*y + b^4*x^2*y + b^2*c^2*x^2*y - 2*b^2*c^2*x*y^2 + a^4*y^3 - a^2*b^2*y^3 - 2*a^2*c^2*y^3 - b^2*c^2*y^3 + c^4*y^3 + 2*b^4*x^2*z - 2*a^2*b^2*y^2*z +      2*b^4*x*z^2 + a^2*b^2*y*z^2 + b^4*y*z^2 - b^2*c^2*y*z^2");
		exp("K456C","2*c^4*x^2*y + 2*c^4*x*y^2 - a^2*c^2*x^2*z + b^2*c^2*x^2*z + c^4*x^2*z + a^2*c^2*y^2*z - b^2*c^2*y^2*z + c^4*y^2*z - 2*b^2*c^2*x*z^2 - 2*a^2*c^2*y*z^2 + a^4*z^3 -      2*a^2*b^2*z^3 + b^4*z^3 - a^2*c^2*z^3 - b^2*c^2*z^3");
		exp("K457","a^6*b^3*c^2*x^2*y - 2*a^4*b^5*c^2*x^2*y + a^2*b^7*c^2*x^2*y - a^6*b*c^4*x^2*y + b^7*c^4*x^2*y +      2*a^4*b*c^6*x^2*y - 2*b^5*c^6*x^2*y - a^2*b*c^8*x^2*y + b^3*c^8*x^2*y + a^7*b^2*c^2*x*y^2 -      2*a^5*b^4*c^2*x*y^2 + a^3*b^6*c^2*x*y^2 + a^7*c^4*x*y^2 - a*b^6*c^4*x*y^2 - 2*a^5*c^6*x*y^2 +      2*a*b^4*c^6*x*y^2 + a^3*c^8*x*y^2 - a*b^2*c^8*x*y^2 - a^6*b^4*c*x^2*z + 2*a^4*b^6*c*x^2*z -      a^2*b^8*c*x^2*z + a^6*b^2*c^3*x^2*z + b^8*c^3*x^2*z - 2*a^4*b^2*c^5*x^2*z - 2*b^6*c^5*x^2*z +      a^2*b^2*c^7*x^2*z + b^4*c^7*x^2*z - 2*a^7*b^3*c*x*y*z + 4*a^5*b^5*c*x*y*z - 2*a^3*b^7*c*x*y*z -      2*a^7*b*c^3*x*y*z - 2*a*b^7*c^3*x*y*z + 4*a^5*b*c^5*x*y*z + 4*a*b^5*c^5*x*y*z - 2*a^3*b*c^7*x*y*z -      2*a*b^3*c^7*x*y*z - a^8*b^2*c*y^2*z + 2*a^6*b^4*c*y^2*z - a^4*b^6*c*y^2*z + a^8*c^3*y^2*z +      a^2*b^6*c^3*y^2*z - 2*a^6*c^5*y^2*z - 2*a^2*b^4*c^5*y^2*z + a^4*c^7*y^2*z + a^2*b^2*c^7*y^2*z +      a^7*b^4*x*z^2 - 2*a^5*b^6*x*z^2 + a^3*b^8*x*z^2 + a^7*b^2*c^2*x*z^2 - a*b^8*c^2*x*z^2 -      2*a^5*b^2*c^4*x*z^2 + 2*a*b^6*c^4*x*z^2 + a^3*b^2*c^6*x*z^2 - a*b^4*c^6*x*z^2 + a^8*b^3*y*z^2 -      2*a^6*b^5*y*z^2 + a^4*b^7*y*z^2 - a^8*b*c^2*y*z^2 + a^2*b^7*c^2*y*z^2 + 2*a^6*b*c^4*y*z^2 -      2*a^2*b^5*c^4*y*z^2 - a^4*b*c^6*y*z^2 + a^2*b^3*c^6*y*z^2");
		exp("K458","(-a^8)*b^6*x^3 + 4*a^6*b^8*x^3 - 6*a^4*b^10*x^3 + 4*a^2*b^12*x^3 - b^14*x^3 + 3*a^8*b^4*c^2*x^3 -      8*a^6*b^6*c^2*x^3 + 8*a^4*b^8*c^2*x^3 - 4*a^2*b^10*c^2*x^3 + b^12*c^2*x^3 - 3*a^8*b^2*c^4*x^3 +      6*a^4*b^6*c^4*x^3 - 4*a^2*b^8*c^4*x^3 + a^8*c^6*x^3 + 8*a^6*b^2*c^6*x^3 - 6*a^4*b^4*c^6*x^3 +      2*b^8*c^6*x^3 - 4*a^6*c^8*x^3 - 8*a^4*b^2*c^8*x^3 + 4*a^2*b^4*c^8*x^3 - 2*b^6*c^8*x^3 +      6*a^4*c^10*x^3 + 4*a^2*b^2*c^10*x^3 - 4*a^2*c^12*x^3 - b^2*c^12*x^3 + c^14*x^3 -      2*a^6*b^6*c^2*x^2*y + 4*a^4*b^8*c^2*x^2*y - 2*a^2*b^10*c^2*x^2*y + 7*a^6*b^4*c^4*x^2*y -      6*a^4*b^6*c^4*x^2*y + 2*b^10*c^4*x^2*y - 8*a^6*b^2*c^6*x^2*y - 9*a^4*b^4*c^6*x^2*y +      6*a^2*b^6*c^6*x^2*y - 4*b^8*c^6*x^2*y + 3*a^6*c^8*x^2*y + 20*a^4*b^2*c^8*x^2*y +      5*a^2*b^4*c^8*x^2*y + 2*b^6*c^8*x^2*y - 9*a^4*c^10*x^2*y - 18*a^2*b^2*c^10*x^2*y -      3*b^4*c^10*x^2*y + 9*a^2*c^12*x^2*y + 6*b^2*c^12*x^2*y - 3*c^14*x^2*y + 2*a^10*b^2*c^2*x*y^2 -      4*a^8*b^4*c^2*x*y^2 + 2*a^6*b^6*c^2*x*y^2 - 2*a^10*c^4*x*y^2 + 6*a^6*b^4*c^4*x*y^2 -      7*a^4*b^6*c^4*x*y^2 + 4*a^8*c^6*x*y^2 - 6*a^6*b^2*c^6*x*y^2 + 9*a^4*b^4*c^6*x*y^2 +      8*a^2*b^6*c^6*x*y^2 - 2*a^6*c^8*x*y^2 - 5*a^4*b^2*c^8*x*y^2 - 20*a^2*b^4*c^8*x*y^2 -      3*b^6*c^8*x*y^2 + 3*a^4*c^10*x*y^2 + 18*a^2*b^2*c^10*x*y^2 + 9*b^4*c^10*x*y^2 - 6*a^2*c^12*x*y^2 -      9*b^2*c^12*x*y^2 + 3*c^14*x*y^2 + a^14*y^3 - 4*a^12*b^2*y^3 + 6*a^10*b^4*y^3 - 4*a^8*b^6*y^3 +      a^6*b^8*y^3 - a^12*c^2*y^3 + 4*a^10*b^2*c^2*y^3 - 8*a^8*b^4*c^2*y^3 + 8*a^6*b^6*c^2*y^3 -      3*a^4*b^8*c^2*y^3 + 4*a^8*b^2*c^4*y^3 - 6*a^6*b^4*c^4*y^3 + 3*a^2*b^8*c^4*y^3 - 2*a^8*c^6*y^3 +      6*a^4*b^4*c^6*y^3 - 8*a^2*b^6*c^6*y^3 - b^8*c^6*y^3 + 2*a^6*c^8*y^3 - 4*a^4*b^2*c^8*y^3 +      8*a^2*b^4*c^8*y^3 + 4*b^6*c^8*y^3 - 4*a^2*b^2*c^10*y^3 - 6*b^4*c^10*y^3 + a^2*c^12*y^3 +      4*b^2*c^12*y^3 - c^14*y^3 - 3*a^6*b^8*x^2*z + 9*a^4*b^10*x^2*z - 9*a^2*b^12*x^2*z + 3*b^14*x^2*z +      8*a^6*b^6*c^2*x^2*z - 20*a^4*b^8*c^2*x^2*z + 18*a^2*b^10*c^2*x^2*z - 6*b^12*c^2*x^2*z -      7*a^6*b^4*c^4*x^2*z + 9*a^4*b^6*c^4*x^2*z - 5*a^2*b^8*c^4*x^2*z + 3*b^10*c^4*x^2*z +      2*a^6*b^2*c^6*x^2*z + 6*a^4*b^4*c^6*x^2*z - 6*a^2*b^6*c^6*x^2*z - 2*b^8*c^6*x^2*z -      4*a^4*b^2*c^8*x^2*z + 4*b^6*c^8*x^2*z + 2*a^2*b^2*c^10*x^2*z - 2*b^4*c^10*x^2*z -      2*a^10*b^4*x*y*z + 6*a^8*b^6*x*y*z - 6*a^6*b^8*x*y*z + 2*a^4*b^10*x*y*z - 2*a^8*b^4*c^2*x*y*z +      2*a^4*b^8*c^2*x*y*z + 2*a^10*c^4*x*y*z + 2*a^8*b^2*c^4*x*y*z - 2*a^2*b^8*c^4*x*y*z -      2*b^10*c^4*x*y*z - 6*a^8*c^6*x*y*z + 6*b^8*c^6*x*y*z + 6*a^6*c^8*x*y*z - 2*a^4*b^2*c^8*x*y*z +      2*a^2*b^4*c^8*x*y*z - 6*b^6*c^8*x*y*z - 2*a^4*c^10*x*y*z + 2*b^4*c^10*x*y*z - 3*a^14*y^2*z +      9*a^12*b^2*y^2*z - 9*a^10*b^4*y^2*z + 3*a^8*b^6*y^2*z + 6*a^12*c^2*y^2*z - 18*a^10*b^2*c^2*y^2*z +      20*a^8*b^4*c^2*y^2*z - 8*a^6*b^6*c^2*y^2*z - 3*a^10*c^4*y^2*z + 5*a^8*b^2*c^4*y^2*z -      9*a^6*b^4*c^4*y^2*z + 7*a^4*b^6*c^4*y^2*z + 2*a^8*c^6*y^2*z + 6*a^6*b^2*c^6*y^2*z -      6*a^4*b^4*c^6*y^2*z - 2*a^2*b^6*c^6*y^2*z - 4*a^6*c^8*y^2*z + 4*a^2*b^4*c^8*y^2*z +      2*a^4*c^10*y^2*z - 2*a^2*b^2*c^10*y^2*z + 2*a^10*b^4*x*z^2 - 4*a^8*b^6*x*z^2 + 2*a^6*b^8*x*z^2 -      3*a^4*b^10*x*z^2 + 6*a^2*b^12*x*z^2 - 3*b^14*x*z^2 - 2*a^10*b^2*c^2*x*z^2 + 6*a^6*b^6*c^2*x*z^2 +      5*a^4*b^8*c^2*x*z^2 - 18*a^2*b^10*c^2*x*z^2 + 9*b^12*c^2*x*z^2 + 4*a^8*b^2*c^4*x*z^2 -      6*a^6*b^4*c^4*x*z^2 - 9*a^4*b^6*c^4*x*z^2 + 20*a^2*b^8*c^4*x*z^2 - 9*b^10*c^4*x*z^2 -      2*a^6*b^2*c^6*x*z^2 + 7*a^4*b^4*c^6*x*z^2 - 8*a^2*b^6*c^6*x*z^2 + 3*b^8*c^6*x*z^2 + 3*a^14*y*z^2 -      6*a^12*b^2*y*z^2 + 3*a^10*b^4*y*z^2 - 2*a^8*b^6*y*z^2 + 4*a^6*b^8*y*z^2 - 2*a^4*b^10*y*z^2 -      9*a^12*c^2*y*z^2 + 18*a^10*b^2*c^2*y*z^2 - 5*a^8*b^4*c^2*y*z^2 - 6*a^6*b^6*c^2*y*z^2 +      2*a^2*b^10*c^2*y*z^2 + 9*a^10*c^4*y*z^2 - 20*a^8*b^2*c^4*y*z^2 + 9*a^6*b^4*c^4*y*z^2 +      6*a^4*b^6*c^4*y*z^2 - 4*a^2*b^8*c^4*y*z^2 - 3*a^8*c^6*y*z^2 + 8*a^6*b^2*c^6*y*z^2 -      7*a^4*b^4*c^6*y*z^2 + 2*a^2*b^6*c^6*y*z^2 - a^14*z^3 + a^12*b^2*z^3 + 2*a^8*b^6*z^3 -      2*a^6*b^8*z^3 - a^2*b^12*z^3 + b^14*z^3 + 4*a^12*c^2*z^3 - 4*a^10*b^2*c^2*z^3 - 4*a^8*b^4*c^2*z^3 +      4*a^4*b^8*c^2*z^3 + 4*a^2*b^10*c^2*z^3 - 4*b^12*c^2*z^3 - 6*a^10*c^4*z^3 + 8*a^8*b^2*c^4*z^3 +      6*a^6*b^4*c^4*z^3 - 6*a^4*b^6*c^4*z^3 - 8*a^2*b^8*c^4*z^3 + 6*b^10*c^4*z^3 + 4*a^8*c^6*z^3 -      8*a^6*b^2*c^6*z^3 + 8*a^2*b^6*c^6*z^3 - 4*b^8*c^6*z^3 - a^6*c^8*z^3 + 3*a^4*b^2*c^8*z^3 -      3*a^2*b^4*c^8*z^3 + b^6*c^8*z^3");
		exp("K459","a^8*b^4*x^3 - 2*a^6*b^6*x^3 + a^4*b^8*x^3 - 2*a^8*b^2*c^2*x^3 + 2*a^6*b^4*c^2*x^3 -      6*a^4*b^6*c^2*x^3 + 6*a^2*b^8*c^2*x^3 - 4*b^10*c^2*x^3 + a^8*c^4*x^3 + 2*a^6*b^2*c^4*x^3 +      10*a^4*b^4*c^4*x^3 - 6*a^2*b^6*c^4*x^3 + 9*b^8*c^4*x^3 - 2*a^6*c^6*x^3 - 6*a^4*b^2*c^6*x^3 -      6*a^2*b^4*c^6*x^3 - 10*b^6*c^6*x^3 + a^4*c^8*x^3 + 6*a^2*b^2*c^8*x^3 + 9*b^4*c^8*x^3 -      4*b^2*c^10*x^3 - a^8*b^4*x^2*y + 2*a^6*b^6*x^2*y - a^4*b^8*x^2*y + 2*a^8*b^2*c^2*x^2*y -      6*a^6*b^4*c^2*x^2*y + 10*a^4*b^6*c^2*x^2*y - 6*a^2*b^8*c^2*x^2*y + 4*b^10*c^2*x^2*y -      a^8*c^4*x^2*y - 2*a^6*b^2*c^4*x^2*y - 2*a^4*b^4*c^4*x^2*y - 2*a^2*b^6*c^4*x^2*y - 9*b^8*c^4*x^2*y +      6*a^6*c^6*x^2*y + 2*a^4*b^2*c^6*x^2*y + 2*a^2*b^4*c^6*x^2*y + 14*b^6*c^6*x^2*y - 9*a^4*c^8*x^2*y +      2*a^2*b^2*c^8*x^2*y - 9*b^4*c^8*x^2*y + 4*a^2*c^10*x^2*y - a^8*b^4*x*y^2 + 2*a^6*b^6*x*y^2 -      a^4*b^8*x*y^2 + 4*a^10*c^2*x*y^2 - 6*a^8*b^2*c^2*x*y^2 + 10*a^6*b^4*c^2*x*y^2 -      6*a^4*b^6*c^2*x*y^2 + 2*a^2*b^8*c^2*x*y^2 - 9*a^8*c^4*x*y^2 - 2*a^6*b^2*c^4*x*y^2 -      2*a^4*b^4*c^4*x*y^2 - 2*a^2*b^6*c^4*x*y^2 - b^8*c^4*x*y^2 + 14*a^6*c^6*x*y^2 +      2*a^4*b^2*c^6*x*y^2 + 2*a^2*b^4*c^6*x*y^2 + 6*b^6*c^6*x*y^2 - 9*a^4*c^8*x*y^2 +      2*a^2*b^2*c^8*x*y^2 - 9*b^4*c^8*x*y^2 + 4*b^2*c^10*x*y^2 + a^8*b^4*y^3 - 2*a^6*b^6*y^3 +      a^4*b^8*y^3 - 4*a^10*c^2*y^3 + 6*a^8*b^2*c^2*y^3 - 6*a^6*b^4*c^2*y^3 + 2*a^4*b^6*c^2*y^3 -      2*a^2*b^8*c^2*y^3 + 9*a^8*c^4*y^3 - 6*a^6*b^2*c^4*y^3 + 10*a^4*b^4*c^4*y^3 + 2*a^2*b^6*c^4*y^3 +      b^8*c^4*y^3 - 10*a^6*c^6*y^3 - 6*a^4*b^2*c^6*y^3 - 6*a^2*b^4*c^6*y^3 - 2*b^6*c^6*y^3 +      9*a^4*c^8*y^3 + 6*a^2*b^2*c^8*y^3 + b^4*c^8*y^3 - 4*a^2*c^10*y^3 - a^8*b^4*x^2*z +      6*a^6*b^6*x^2*z - 9*a^4*b^8*x^2*z + 4*a^2*b^10*x^2*z + 2*a^8*b^2*c^2*x^2*z - 2*a^6*b^4*c^2*x^2*z +      2*a^4*b^6*c^2*x^2*z + 2*a^2*b^8*c^2*x^2*z - a^8*c^4*x^2*z - 6*a^6*b^2*c^4*x^2*z -      2*a^4*b^4*c^4*x^2*z + 2*a^2*b^6*c^4*x^2*z - 9*b^8*c^4*x^2*z + 2*a^6*c^6*x^2*z +      10*a^4*b^2*c^6*x^2*z - 2*a^2*b^4*c^6*x^2*z + 14*b^6*c^6*x^2*z - a^4*c^8*x^2*z -      6*a^2*b^2*c^8*x^2*z - 9*b^4*c^8*x^2*z + 4*b^2*c^10*x^2*z - 4*a^10*b^2*x*y*z + 10*a^8*b^4*x*y*z -      12*a^6*b^6*x*y*z + 10*a^4*b^8*x*y*z - 4*a^2*b^10*x*y*z - 4*a^10*c^2*x*y*z + 12*a^8*b^2*c^2*x*y*z -      12*a^6*b^4*c^2*x*y*z - 12*a^4*b^6*c^2*x*y*z + 12*a^2*b^8*c^2*x*y*z - 4*b^10*c^2*x*y*z +      10*a^8*c^4*x*y*z - 12*a^6*b^2*c^4*x*y*z + 36*a^4*b^4*c^4*x*y*z - 12*a^2*b^6*c^4*x*y*z +      10*b^8*c^4*x*y*z - 12*a^6*c^6*x*y*z - 12*a^4*b^2*c^6*x*y*z - 12*a^2*b^4*c^6*x*y*z -      12*b^6*c^6*x*y*z + 10*a^4*c^8*x*y*z + 12*a^2*b^2*c^8*x*y*z + 10*b^4*c^8*x*y*z - 4*a^2*c^10*x*y*z -      4*b^2*c^10*x*y*z + 4*a^10*b^2*y^2*z - 9*a^8*b^4*y^2*z + 6*a^6*b^6*y^2*z - a^4*b^8*y^2*z +      2*a^8*b^2*c^2*y^2*z + 2*a^6*b^4*c^2*y^2*z - 2*a^4*b^6*c^2*y^2*z + 2*a^2*b^8*c^2*y^2*z -      9*a^8*c^4*y^2*z + 2*a^6*b^2*c^4*y^2*z - 2*a^4*b^4*c^4*y^2*z - 6*a^2*b^6*c^4*y^2*z - b^8*c^4*y^2*z +      14*a^6*c^6*y^2*z - 2*a^4*b^2*c^6*y^2*z + 10*a^2*b^4*c^6*y^2*z + 2*b^6*c^6*y^2*z - 9*a^4*c^8*y^2*z -      6*a^2*b^2*c^8*y^2*z - b^4*c^8*y^2*z + 4*a^2*c^10*y^2*z + 4*a^10*b^2*x*z^2 - 9*a^8*b^4*x*z^2 +      14*a^6*b^6*x*z^2 - 9*a^4*b^8*x*z^2 - 6*a^8*b^2*c^2*x*z^2 - 2*a^6*b^4*c^2*x*z^2 +      2*a^4*b^6*c^2*x*z^2 + 2*a^2*b^8*c^2*x*z^2 + 4*b^10*c^2*x*z^2 - a^8*c^4*x*z^2 +      10*a^6*b^2*c^4*x*z^2 - 2*a^4*b^4*c^4*x*z^2 + 2*a^2*b^6*c^4*x*z^2 - 9*b^8*c^4*x*z^2 +      2*a^6*c^6*x*z^2 - 6*a^4*b^2*c^6*x*z^2 - 2*a^2*b^4*c^6*x*z^2 + 6*b^6*c^6*x*z^2 - a^4*c^8*x*z^2 +      2*a^2*b^2*c^8*x*z^2 - b^4*c^8*x*z^2 - 9*a^8*b^4*y*z^2 + 14*a^6*b^6*y*z^2 - 9*a^4*b^8*y*z^2 +      4*a^2*b^10*y*z^2 + 4*a^10*c^2*y*z^2 + 2*a^8*b^2*c^2*y*z^2 + 2*a^6*b^4*c^2*y*z^2 -      2*a^4*b^6*c^2*y*z^2 - 6*a^2*b^8*c^2*y*z^2 - 9*a^8*c^4*y*z^2 + 2*a^6*b^2*c^4*y*z^2 -      2*a^4*b^4*c^4*y*z^2 + 10*a^2*b^6*c^4*y*z^2 - b^8*c^4*y*z^2 + 6*a^6*c^6*y*z^2 -      2*a^4*b^2*c^6*y*z^2 - 6*a^2*b^4*c^6*y*z^2 + 2*b^6*c^6*y*z^2 - a^4*c^8*y*z^2 + 2*a^2*b^2*c^8*y*z^2 -      b^4*c^8*y*z^2 - 4*a^10*b^2*z^3 + 9*a^8*b^4*z^3 - 10*a^6*b^6*z^3 + 9*a^4*b^8*z^3 - 4*a^2*b^10*z^3 +      6*a^8*b^2*c^2*z^3 - 6*a^6*b^4*c^2*z^3 - 6*a^4*b^6*c^2*z^3 + 6*a^2*b^8*c^2*z^3 + a^8*c^4*z^3 -      6*a^6*b^2*c^4*z^3 + 10*a^4*b^4*c^4*z^3 - 6*a^2*b^6*c^4*z^3 + b^8*c^4*z^3 - 2*a^6*c^6*z^3 +      2*a^4*b^2*c^6*z^3 + 2*a^2*b^4*c^6*z^3 - 2*b^6*c^6*z^3 + a^4*c^8*z^3 - 2*a^2*b^2*c^8*z^3 +      b^4*c^8*z^3");
		exp("K460","-2*a^4*b^4*x^3 + 2*a^2*b^6*x^3 - b^8*x^3 + 4*a^4*b^2*c^2*x^3 - 2*a^2*b^4*c^2*x^3 +      2*b^6*c^2*x^3 - 2*a^4*c^4*x^3 - 2*a^2*b^2*c^4*x^3 - 2*b^4*c^4*x^3 + 2*a^2*c^6*x^3 + 2*b^2*c^6*x^3 -      c^8*x^3 - 2*a^6*b^2*x^2*y + 4*a^4*b^4*x^2*y - 2*a^2*b^6*x^2*y + b^8*x^2*y + 2*a^6*c^2*x^2*y -      2*a^4*b^2*c^2*x^2*y - 2*a^2*b^4*c^2*x^2*y - 2*b^6*c^2*x^2*y - 2*a^4*c^4*x^2*y +      4*a^2*b^2*c^4*x^2*y + 4*b^4*c^4*x^2*y - 4*b^2*c^6*x^2*y + c^8*x^2*y + a^8*x*y^2 - 2*a^6*b^2*x*y^2 +      4*a^4*b^4*x*y^2 - 2*a^2*b^6*x*y^2 - 2*a^6*c^2*x*y^2 - 2*a^4*b^2*c^2*x*y^2 - 2*a^2*b^4*c^2*x*y^2 +      2*b^6*c^2*x*y^2 + 4*a^4*c^4*x*y^2 + 4*a^2*b^2*c^4*x*y^2 - 2*b^4*c^4*x*y^2 - 4*a^2*c^6*x*y^2 +      c^8*x*y^2 - a^8*y^3 + 2*a^6*b^2*y^3 - 2*a^4*b^4*y^3 + 2*a^6*c^2*y^3 - 2*a^4*b^2*c^2*y^3 +      4*a^2*b^4*c^2*y^3 - 2*a^4*c^4*y^3 - 2*a^2*b^2*c^4*y^3 - 2*b^4*c^4*y^3 + 2*a^2*c^6*y^3 +      2*b^2*c^6*y^3 - c^8*y^3 + 2*a^6*b^2*x^2*z - 2*a^4*b^4*x^2*z + b^8*x^2*z - 2*a^6*c^2*x^2*z -      2*a^4*b^2*c^2*x^2*z + 4*a^2*b^4*c^2*x^2*z - 4*b^6*c^2*x^2*z + 4*a^4*c^4*x^2*z -      2*a^2*b^2*c^4*x^2*z + 4*b^4*c^4*x^2*z - 2*a^2*c^6*x^2*z - 2*b^2*c^6*x^2*z + c^8*x^2*z -      2*a^8*x*y*z + 4*a^6*b^2*x*y*z - 6*a^4*b^4*x*y*z + 4*a^2*b^6*x*y*z - 2*b^8*x*y*z + 4*a^6*c^2*x*y*z +      4*b^6*c^2*x*y*z - 6*a^4*c^4*x*y*z - 6*b^4*c^4*x*y*z + 4*a^2*c^6*x*y*z + 4*b^2*c^6*x*y*z -      2*c^8*x*y*z + a^8*y^2*z - 2*a^4*b^4*y^2*z + 2*a^2*b^6*y^2*z - 4*a^6*c^2*y^2*z +      4*a^4*b^2*c^2*y^2*z - 2*a^2*b^4*c^2*y^2*z - 2*b^6*c^2*y^2*z + 4*a^4*c^4*y^2*z -      2*a^2*b^2*c^4*y^2*z + 4*b^4*c^4*y^2*z - 2*a^2*c^6*y^2*z - 2*b^2*c^6*y^2*z + c^8*y^2*z + a^8*x*z^2 -      2*a^6*b^2*x*z^2 + 4*a^4*b^4*x*z^2 - 4*a^2*b^6*x*z^2 + b^8*x*z^2 - 2*a^6*c^2*x*z^2 -      2*a^4*b^2*c^2*x*z^2 + 4*a^2*b^4*c^2*x*z^2 + 4*a^4*c^4*x*z^2 - 2*a^2*b^2*c^4*x*z^2 -      2*b^4*c^4*x*z^2 - 2*a^2*c^6*x*z^2 + 2*b^2*c^6*x*z^2 + a^8*y*z^2 - 4*a^6*b^2*y*z^2 +      4*a^4*b^4*y*z^2 - 2*a^2*b^6*y*z^2 + b^8*y*z^2 + 4*a^4*b^2*c^2*y*z^2 - 2*a^2*b^4*c^2*y*z^2 -      2*b^6*c^2*y*z^2 - 2*a^4*c^4*y*z^2 - 2*a^2*b^2*c^4*y*z^2 + 4*b^4*c^4*y*z^2 + 2*a^2*c^6*y*z^2 -      2*b^2*c^6*y*z^2 - a^8*z^3 + 2*a^6*b^2*z^3 - 2*a^4*b^4*z^3 + 2*a^2*b^6*z^3 - b^8*z^3 +      2*a^6*c^2*z^3 - 2*a^4*b^2*c^2*z^3 - 2*a^2*b^4*c^2*z^3 + 2*b^6*c^2*z^3 - 2*a^4*c^4*z^3 +      4*a^2*b^2*c^4*z^3 - 2*b^4*c^4*z^3");
		exp("K461","3*a^4*b^2*x^3 - 2*a^2*b^4*x^3 - b^6*x^3 - 3*a^4*c^2*x^3 + 3*b^4*c^2*x^3 + 2*a^2*c^4*x^3 -      3*b^2*c^4*x^3 + c^6*x^3 + 3*a^6*x^2*y + 2*a^4*b^2*x^2*y - a^2*b^4*x^2*y - 4*b^6*x^2*y -      12*a^4*c^2*x^2*y + 2*a^2*b^2*c^2*x^2*y + 10*b^4*c^2*x^2*y + 7*a^2*c^4*x^2*y - 8*b^2*c^4*x^2*y +      2*c^6*x^2*y + 4*a^6*x*y^2 + a^4*b^2*x*y^2 - 2*a^2*b^4*x*y^2 - 3*b^6*x*y^2 - 10*a^4*c^2*x*y^2 -      2*a^2*b^2*c^2*x*y^2 + 12*b^4*c^2*x*y^2 + 8*a^2*c^4*x*y^2 - 7*b^2*c^4*x*y^2 - 2*c^6*x*y^2 +      a^6*y^3 + 2*a^4*b^2*y^3 - 3*a^2*b^4*y^3 - 3*a^4*c^2*y^3 + 3*b^4*c^2*y^3 + 3*a^2*c^4*y^3 -      2*b^2*c^4*y^3 - c^6*y^3 - 3*a^6*x^2*z + 12*a^4*b^2*x^2*z - 7*a^2*b^4*x^2*z - 2*b^6*x^2*z -      2*a^4*c^2*x^2*z - 2*a^2*b^2*c^2*x^2*z + 8*b^4*c^2*x^2*z + a^2*c^4*x^2*z - 10*b^2*c^4*x^2*z +      4*c^6*x^2*z + 16*a^4*b^2*x*y*z - 16*a^2*b^4*x*y*z - 16*a^4*c^2*x*y*z + 16*b^4*c^2*x*y*z +      16*a^2*c^4*x*y*z - 16*b^2*c^4*x*y*z + 2*a^6*y^2*z + 7*a^4*b^2*y^2*z - 12*a^2*b^4*y^2*z +      3*b^6*y^2*z - 8*a^4*c^2*y^2*z + 2*a^2*b^2*c^2*y^2*z + 2*b^4*c^2*y^2*z + 10*a^2*c^4*y^2*z -      b^2*c^4*y^2*z - 4*c^6*y^2*z - 4*a^6*x*z^2 + 10*a^4*b^2*x*z^2 - 8*a^2*b^4*x*z^2 + 2*b^6*x*z^2 -      a^4*c^2*x*z^2 + 2*a^2*b^2*c^2*x*z^2 + 7*b^4*c^2*x*z^2 + 2*a^2*c^4*x*z^2 - 12*b^2*c^4*x*z^2 +      3*c^6*x*z^2 - 2*a^6*y*z^2 + 8*a^4*b^2*y*z^2 - 10*a^2*b^4*y*z^2 + 4*b^6*y*z^2 - 7*a^4*c^2*y*z^2 -      2*a^2*b^2*c^2*y*z^2 + b^4*c^2*y*z^2 + 12*a^2*c^4*y*z^2 - 2*b^2*c^4*y*z^2 - 3*c^6*y*z^2 - a^6*z^3 +      3*a^4*b^2*z^3 - 3*a^2*b^4*z^3 + b^6*z^3 - 2*a^4*c^2*z^3 + 2*b^4*c^2*z^3 + 3*a^2*c^4*z^3 -      3*b^2*c^4*z^3");
		exp("K462","a^6*x^3 + 6*a^4*b^2*x^3 + 9*a^2*b^4*x^3 + 6*a^4*c^2*x^3 - 18*a^2*b^2*c^2*x^3 + 9*a^2*c^4*x^3 +      6*a^6*x^2*y + 21*a^4*b^2*x^2*y + 12*a^2*b^4*x^2*y + 9*b^6*x^2*y + 15*a^4*c^2*x^2*y -      60*a^2*b^2*c^2*x^2*y + 9*b^4*c^2*x^2*y - 45*b^2*c^4*x^2*y + 27*c^6*x^2*y + 9*a^6*x*y^2 +      12*a^4*b^2*x*y^2 + 21*a^2*b^4*x*y^2 + 6*b^6*x*y^2 + 9*a^4*c^2*x*y^2 - 60*a^2*b^2*c^2*x*y^2 +      15*b^4*c^2*x*y^2 - 45*a^2*c^4*x*y^2 + 27*c^6*x*y^2 + 9*a^4*b^2*y^3 + 6*a^2*b^4*y^3 + b^6*y^3 -      18*a^2*b^2*c^2*y^3 + 6*b^4*c^2*y^3 + 9*b^2*c^4*y^3 + 6*a^6*x^2*z + 15*a^4*b^2*x^2*z +      27*b^6*x^2*z + 21*a^4*c^2*x^2*z - 60*a^2*b^2*c^2*x^2*z - 45*b^4*c^2*x^2*z + 12*a^2*c^4*x^2*z +      9*b^2*c^4*x^2*z + 9*c^6*x^2*z + 27*a^6*x*y*z + 21*a^4*b^2*x*y*z + 21*a^2*b^4*x*y*z + 27*b^6*x*y*z +      21*a^4*c^2*x*y*z - 210*a^2*b^2*c^2*x*y*z + 21*b^4*c^2*x*y*z + 21*a^2*c^4*x*y*z + 21*b^2*c^4*x*y*z +      27*c^6*x*y*z + 27*a^6*y^2*z + 15*a^2*b^4*y^2*z + 6*b^6*y^2*z - 45*a^4*c^2*y^2*z -      60*a^2*b^2*c^2*y^2*z + 21*b^4*c^2*y^2*z + 9*a^2*c^4*y^2*z + 12*b^2*c^4*y^2*z + 9*c^6*y^2*z +      9*a^6*x*z^2 + 9*a^4*b^2*x*z^2 - 45*a^2*b^4*x*z^2 + 27*b^6*x*z^2 + 12*a^4*c^2*x*z^2 -      60*a^2*b^2*c^2*x*z^2 + 21*a^2*c^4*x*z^2 + 15*b^2*c^4*x*z^2 + 6*c^6*x*z^2 + 27*a^6*y*z^2 -      45*a^4*b^2*y*z^2 + 9*a^2*b^4*y*z^2 + 9*b^6*y*z^2 - 60*a^2*b^2*c^2*y*z^2 + 12*b^4*c^2*y*z^2 +      15*a^2*c^4*y*z^2 + 21*b^2*c^4*y*z^2 + 6*c^6*y*z^2 + 9*a^4*c^2*z^3 - 18*a^2*b^2*c^2*z^3 +      9*b^4*c^2*z^3 + 6*a^2*c^4*z^3 + 6*b^2*c^4*z^3 + c^6*z^3");
		exp("K463","-2*b^4*c^2*x^3 + 2*b^2*c^4*x^3 - a^4*c^2*x^2*y - 3*a^2*b^2*c^2*x^2*y + 2*b^4*c^2*x^2*y +      4*a^2*c^4*x^2*y + b^2*c^4*x^2*y - 3*c^6*x^2*y - 2*a^4*c^2*x*y^2 + 3*a^2*b^2*c^2*x*y^2 +      b^4*c^2*x*y^2 - a^2*c^4*x*y^2 - 4*b^2*c^4*x*y^2 + 3*c^6*x*y^2 + 2*a^4*c^2*y^3 - 2*a^2*c^4*y^3 +      a^4*b^2*x^2*z - 4*a^2*b^4*x^2*z + 3*b^6*x^2*z + 3*a^2*b^2*c^2*x^2*z - b^4*c^2*x^2*z -      2*b^2*c^4*x^2*z - 4*a^4*b^2*x*y*z + 4*a^2*b^4*x*y*z + 4*a^4*c^2*x*y*z - 4*b^4*c^2*x*y*z -      4*a^2*c^4*x*y*z + 4*b^2*c^4*x*y*z - 3*a^6*y^2*z + 4*a^4*b^2*y^2*z - a^2*b^4*y^2*z + a^4*c^2*y^2*z -      3*a^2*b^2*c^2*y^2*z + 2*a^2*c^4*y^2*z + 2*a^4*b^2*x*z^2 + a^2*b^4*x*z^2 - 3*b^6*x*z^2 -      3*a^2*b^2*c^2*x*z^2 + 4*b^4*c^2*x*z^2 - b^2*c^4*x*z^2 + 3*a^6*y*z^2 - a^4*b^2*y*z^2 -      2*a^2*b^4*y*z^2 - 4*a^4*c^2*y*z^2 + 3*a^2*b^2*c^2*y*z^2 + a^2*c^4*y*z^2 - 2*a^4*b^2*z^3 +      2*a^2*b^4*z^3");
		exp("K464","a^10*c^2*x^2*y - 3*a^8*b^2*c^2*x^2*y + 4*a^6*b^4*c^2*x^2*y - 4*a^4*b^6*c^2*x^2*y +      3*a^2*b^8*c^2*x^2*y - b^10*c^2*x^2*y - 4*a^8*c^4*x^2*y + 5*a^6*b^2*c^4*x^2*y -      3*a^2*b^6*c^4*x^2*y + 2*b^8*c^4*x^2*y + 6*a^6*c^6*x^2*y - a^4*b^2*c^6*x^2*y + a^2*b^4*c^6*x^2*y -      4*a^4*c^8*x^2*y - 2*a^2*b^2*c^8*x^2*y - 2*b^4*c^8*x^2*y + a^2*c^10*x^2*y + b^2*c^10*x^2*y +      a^10*c^2*x*y^2 - 3*a^8*b^2*c^2*x*y^2 + 4*a^6*b^4*c^2*x*y^2 - 4*a^4*b^6*c^2*x*y^2 +      3*a^2*b^8*c^2*x*y^2 - b^10*c^2*x*y^2 - 2*a^8*c^4*x*y^2 + 3*a^6*b^2*c^4*x*y^2 -      5*a^2*b^6*c^4*x*y^2 + 4*b^8*c^4*x*y^2 - a^4*b^2*c^6*x*y^2 + a^2*b^4*c^6*x*y^2 - 6*b^6*c^6*x*y^2 +      2*a^4*c^8*x*y^2 + 2*a^2*b^2*c^8*x*y^2 + 4*b^4*c^8*x*y^2 - a^2*c^10*x*y^2 - b^2*c^10*x*y^2 -      a^10*b^2*x^2*z + 4*a^8*b^4*x^2*z - 6*a^6*b^6*x^2*z + 4*a^4*b^8*x^2*z - a^2*b^10*x^2*z +      3*a^8*b^2*c^2*x^2*z - 5*a^6*b^4*c^2*x^2*z + a^4*b^6*c^2*x^2*z + 2*a^2*b^8*c^2*x^2*z -      b^10*c^2*x^2*z - 4*a^6*b^2*c^4*x^2*z - a^2*b^6*c^4*x^2*z + 2*b^8*c^4*x^2*z + 4*a^4*b^2*c^6*x^2*z +      3*a^2*b^4*c^6*x^2*z - 3*a^2*b^2*c^8*x^2*z - 2*b^4*c^8*x^2*z + b^2*c^10*x^2*z +      2*a^6*b^4*c^2*x*y*z - 2*a^4*b^6*c^2*x*y*z - 2*a^6*b^2*c^4*x*y*z + 2*a^2*b^6*c^4*x*y*z +      2*a^4*b^2*c^6*x*y*z - 2*a^2*b^4*c^6*x*y*z + a^10*b^2*y^2*z - 4*a^8*b^4*y^2*z + 6*a^6*b^6*y^2*z -      4*a^4*b^8*y^2*z + a^2*b^10*y^2*z + a^10*c^2*y^2*z - 2*a^8*b^2*c^2*y^2*z - a^6*b^4*c^2*y^2*z +      5*a^4*b^6*c^2*y^2*z - 3*a^2*b^8*c^2*y^2*z - 2*a^8*c^4*y^2*z + a^6*b^2*c^4*y^2*z +      4*a^2*b^6*c^4*y^2*z - 3*a^4*b^2*c^6*y^2*z - 4*a^2*b^4*c^6*y^2*z + 2*a^4*c^8*y^2*z +      3*a^2*b^2*c^8*y^2*z - a^2*c^10*y^2*z - a^10*b^2*x*z^2 + 2*a^8*b^4*x*z^2 - 2*a^4*b^8*x*z^2 +      a^2*b^10*x*z^2 + 3*a^8*b^2*c^2*x*z^2 - 3*a^6*b^4*c^2*x*z^2 + a^4*b^6*c^2*x*z^2 -      2*a^2*b^8*c^2*x*z^2 + b^10*c^2*x*z^2 - 4*a^6*b^2*c^4*x*z^2 - a^2*b^6*c^4*x*z^2 - 4*b^8*c^4*x*z^2 +      4*a^4*b^2*c^6*x*z^2 + 5*a^2*b^4*c^6*x*z^2 + 6*b^6*c^6*x*z^2 - 3*a^2*b^2*c^8*x*z^2 -      4*b^4*c^8*x*z^2 + b^2*c^10*x*z^2 - a^10*b^2*y*z^2 + 2*a^8*b^4*y*z^2 - 2*a^4*b^8*y*z^2 +      a^2*b^10*y*z^2 - a^10*c^2*y*z^2 + 2*a^8*b^2*c^2*y*z^2 - a^6*b^4*c^2*y*z^2 + 3*a^4*b^6*c^2*y*z^2 -      3*a^2*b^8*c^2*y*z^2 + 4*a^8*c^4*y*z^2 + a^6*b^2*c^4*y*z^2 + 4*a^2*b^6*c^4*y*z^2 - 6*a^6*c^6*y*z^2 -      5*a^4*b^2*c^6*y*z^2 - 4*a^2*b^4*c^6*y*z^2 + 4*a^4*c^8*y*z^2 + 3*a^2*b^2*c^8*y*z^2 -      a^2*c^10*y*z^2");
		exp("K465","(-a^10)*c^2*x^2*y + a^8*b^2*c^2*x^2*y + 2*a^6*b^4*c^2*x^2*y - 2*a^4*b^6*c^2*x^2*y -      a^2*b^8*c^2*x^2*y + b^10*c^2*x^2*y + 4*a^8*c^4*x^2*y - 3*a^6*b^2*c^4*x^2*y + a^2*b^6*c^4*x^2*y -      2*b^8*c^4*x^2*y - 6*a^6*c^6*x^2*y + a^4*b^2*c^6*x^2*y - a^2*b^4*c^6*x^2*y + 4*a^4*c^8*x^2*y +      2*a^2*b^2*c^8*x^2*y + 2*b^4*c^8*x^2*y - a^2*c^10*x^2*y - b^2*c^10*x^2*y - a^10*c^2*x*y^2 +      a^8*b^2*c^2*x*y^2 + 2*a^6*b^4*c^2*x*y^2 - 2*a^4*b^6*c^2*x*y^2 - a^2*b^8*c^2*x*y^2 +      b^10*c^2*x*y^2 + 2*a^8*c^4*x*y^2 - a^6*b^2*c^4*x*y^2 + 3*a^2*b^6*c^4*x*y^2 - 4*b^8*c^4*x*y^2 +      a^4*b^2*c^6*x*y^2 - a^2*b^4*c^6*x*y^2 + 6*b^6*c^6*x*y^2 - 2*a^4*c^8*x*y^2 - 2*a^2*b^2*c^8*x*y^2 -      4*b^4*c^8*x*y^2 + a^2*c^10*x*y^2 + b^2*c^10*x*y^2 + a^10*b^2*x^2*z - 4*a^8*b^4*x^2*z +      6*a^6*b^6*x^2*z - 4*a^4*b^8*x^2*z + a^2*b^10*x^2*z - a^8*b^2*c^2*x^2*z + 3*a^6*b^4*c^2*x^2*z -      a^4*b^6*c^2*x^2*z - 2*a^2*b^8*c^2*x^2*z + b^10*c^2*x^2*z - 2*a^6*b^2*c^4*x^2*z +      a^2*b^6*c^4*x^2*z - 2*b^8*c^4*x^2*z + 2*a^4*b^2*c^6*x^2*z - a^2*b^4*c^6*x^2*z + a^2*b^2*c^8*x^2*z +      2*b^4*c^8*x^2*z - b^2*c^10*x^2*z + 2*a^6*b^4*c^2*x*y*z - 2*a^4*b^6*c^2*x*y*z -      2*a^6*b^2*c^4*x*y*z + 2*a^2*b^6*c^4*x*y*z + 2*a^4*b^2*c^6*x*y*z - 2*a^2*b^4*c^6*x*y*z -      a^10*b^2*y^2*z + 4*a^8*b^4*y^2*z - 6*a^6*b^6*y^2*z + 4*a^4*b^8*y^2*z - a^2*b^10*y^2*z -      a^10*c^2*y^2*z + 2*a^8*b^2*c^2*y^2*z + a^6*b^4*c^2*y^2*z - 3*a^4*b^6*c^2*y^2*z +      a^2*b^8*c^2*y^2*z + 2*a^8*c^4*y^2*z - a^6*b^2*c^4*y^2*z + 2*a^2*b^6*c^4*y^2*z + a^4*b^2*c^6*y^2*z -      2*a^2*b^4*c^6*y^2*z - 2*a^4*c^8*y^2*z - a^2*b^2*c^8*y^2*z + a^2*c^10*y^2*z + a^10*b^2*x*z^2 -      2*a^8*b^4*x*z^2 + 2*a^4*b^8*x*z^2 - a^2*b^10*x*z^2 - a^8*b^2*c^2*x*z^2 + a^6*b^4*c^2*x*z^2 -      a^4*b^6*c^2*x*z^2 + 2*a^2*b^8*c^2*x*z^2 - b^10*c^2*x*z^2 - 2*a^6*b^2*c^4*x*z^2 +      a^2*b^6*c^4*x*z^2 + 4*b^8*c^4*x*z^2 + 2*a^4*b^2*c^6*x*z^2 - 3*a^2*b^4*c^6*x*z^2 - 6*b^6*c^6*x*z^2 +      a^2*b^2*c^8*x*z^2 + 4*b^4*c^8*x*z^2 - b^2*c^10*x*z^2 + a^10*b^2*y*z^2 - 2*a^8*b^4*y*z^2 +      2*a^4*b^8*y*z^2 - a^2*b^10*y*z^2 + a^10*c^2*y*z^2 - 2*a^8*b^2*c^2*y*z^2 + a^6*b^4*c^2*y*z^2 -      a^4*b^6*c^2*y*z^2 + a^2*b^8*c^2*y*z^2 - 4*a^8*c^4*y*z^2 - a^6*b^2*c^4*y*z^2 + 2*a^2*b^6*c^4*y*z^2 +      6*a^6*c^6*y*z^2 + 3*a^4*b^2*c^6*y*z^2 - 2*a^2*b^4*c^6*y*z^2 - 4*a^4*c^8*y*z^2 - a^2*b^2*c^8*y*z^2 +      a^2*c^10*y*z^2");
		exp("K466","a^8*b^2*c^2*x^2*y - 2*a^6*b^4*c^2*x^2*y + 2*a^2*b^8*c^2*x^2*y - b^10*c^2*x^2*y + a^8*c^4*x^2*y -      2*a^6*b^2*c^4*x^2*y + a^4*b^4*c^4*x^2*y - a^2*b^6*c^4*x^2*y + b^8*c^4*x^2*y - 4*a^6*c^6*x^2*y -      a^4*b^2*c^6*x^2*y + 2*b^6*c^6*x^2*y + 6*a^4*c^8*x^2*y + 3*a^2*b^2*c^8*x^2*y - 2*b^4*c^8*x^2*y -      4*a^2*c^10*x^2*y - b^2*c^10*x^2*y + c^12*x^2*y + a^10*c^2*x*y^2 - 2*a^8*b^2*c^2*x*y^2 +      2*a^4*b^6*c^2*x*y^2 - a^2*b^8*c^2*x*y^2 - a^8*c^4*x*y^2 + a^6*b^2*c^4*x*y^2 - a^4*b^4*c^4*x*y^2 +      2*a^2*b^6*c^4*x*y^2 - b^8*c^4*x*y^2 - 2*a^6*c^6*x*y^2 + a^2*b^4*c^6*x*y^2 + 4*b^6*c^6*x*y^2 +      2*a^4*c^8*x*y^2 - 3*a^2*b^2*c^8*x*y^2 - 6*b^4*c^8*x*y^2 + a^2*c^10*x*y^2 + 4*b^2*c^10*x*y^2 -      c^12*x*y^2 - a^8*b^4*x^2*z + 4*a^6*b^6*x^2*z - 6*a^4*b^8*x^2*z + 4*a^2*b^10*x^2*z - b^12*x^2*z -      a^8*b^2*c^2*x^2*z + 2*a^6*b^4*c^2*x^2*z + a^4*b^6*c^2*x^2*z - 3*a^2*b^8*c^2*x^2*z +      b^10*c^2*x^2*z + 2*a^6*b^2*c^4*x^2*z - a^4*b^4*c^4*x^2*z + 2*b^8*c^4*x^2*z + a^2*b^4*c^6*x^2*z -      2*b^6*c^6*x^2*z - 2*a^2*b^2*c^8*x^2*z - b^4*c^8*x^2*z + b^2*c^10*x^2*z + 2*a^6*b^4*c^2*x*y*z -      2*a^4*b^6*c^2*x*y*z - 2*a^6*b^2*c^4*x*y*z + 2*a^2*b^6*c^4*x*y*z + 2*a^4*b^2*c^6*x*y*z -      2*a^2*b^4*c^6*x*y*z + a^12*y^2*z - 4*a^10*b^2*y^2*z + 6*a^8*b^4*y^2*z - 4*a^6*b^6*y^2*z +      a^4*b^8*y^2*z - a^10*c^2*y^2*z + 3*a^8*b^2*c^2*y^2*z - a^6*b^4*c^2*y^2*z - 2*a^4*b^6*c^2*y^2*z +      a^2*b^8*c^2*y^2*z - 2*a^8*c^4*y^2*z + a^4*b^4*c^4*y^2*z - 2*a^2*b^6*c^4*y^2*z + 2*a^6*c^6*y^2*z -      a^4*b^2*c^6*y^2*z + a^4*c^8*y^2*z + 2*a^2*b^2*c^8*y^2*z - a^2*c^10*y^2*z - a^10*b^2*x*z^2 +      a^8*b^4*x*z^2 + 2*a^6*b^6*x*z^2 - 2*a^4*b^8*x*z^2 - a^2*b^10*x*z^2 + b^12*x*z^2 +      2*a^8*b^2*c^2*x*z^2 - a^6*b^4*c^2*x*z^2 + 3*a^2*b^8*c^2*x*z^2 - 4*b^10*c^2*x*z^2 +      a^4*b^4*c^4*x*z^2 - a^2*b^6*c^4*x*z^2 + 6*b^8*c^4*x*z^2 - 2*a^4*b^2*c^6*x*z^2 -      2*a^2*b^4*c^6*x*z^2 - 4*b^6*c^6*x*z^2 + a^2*b^2*c^8*x*z^2 + b^4*c^8*x*z^2 - a^12*y*z^2 +      a^10*b^2*y*z^2 + 2*a^8*b^4*y*z^2 - 2*a^6*b^6*y*z^2 - a^4*b^8*y*z^2 + a^2*b^10*y*z^2 +      4*a^10*c^2*y*z^2 - 3*a^8*b^2*c^2*y*z^2 + a^4*b^6*c^2*y*z^2 - 2*a^2*b^8*c^2*y*z^2 -      6*a^8*c^4*y*z^2 + a^6*b^2*c^4*y*z^2 - a^4*b^4*c^4*y*z^2 + 4*a^6*c^6*y*z^2 + 2*a^4*b^2*c^6*y*z^2 +      2*a^2*b^4*c^6*y*z^2 - a^4*c^8*y*z^2 - a^2*b^2*c^8*y*z^2");
		exp("K467","a^8*b^2*c^2*x^2*y - 2*a^6*b^4*c^2*x^2*y + 2*a^2*b^8*c^2*x^2*y - b^10*c^2*x^2*y + a^8*c^4*x^2*y -      2*a^6*b^2*c^4*x^2*y + a^4*b^4*c^4*x^2*y - 3*a^2*b^6*c^4*x^2*y + 3*b^8*c^4*x^2*y - 4*a^6*c^6*x^2*y -      a^4*b^2*c^6*x^2*y - 4*b^6*c^6*x^2*y + 6*a^4*c^8*x^2*y + 5*a^2*b^2*c^8*x^2*y + 4*b^4*c^8*x^2*y -      4*a^2*c^10*x^2*y - 3*b^2*c^10*x^2*y + c^12*x^2*y + a^10*c^2*x*y^2 - 2*a^8*b^2*c^2*x*y^2 +      2*a^4*b^6*c^2*x*y^2 - a^2*b^8*c^2*x*y^2 - 3*a^8*c^4*x*y^2 + 3*a^6*b^2*c^4*x*y^2 -      a^4*b^4*c^4*x*y^2 + 2*a^2*b^6*c^4*x*y^2 - b^8*c^4*x*y^2 + 4*a^6*c^6*x*y^2 + a^2*b^4*c^6*x*y^2 +      4*b^6*c^6*x*y^2 - 4*a^4*c^8*x*y^2 - 5*a^2*b^2*c^8*x*y^2 - 6*b^4*c^8*x*y^2 + 3*a^2*c^10*x*y^2 +      4*b^2*c^10*x*y^2 - c^12*x*y^2 - a^8*b^4*x^2*z + 4*a^6*b^6*x^2*z - 6*a^4*b^8*x^2*z +      4*a^2*b^10*x^2*z - b^12*x^2*z - a^8*b^2*c^2*x^2*z + 2*a^6*b^4*c^2*x^2*z + a^4*b^6*c^2*x^2*z -      5*a^2*b^8*c^2*x^2*z + 3*b^10*c^2*x^2*z + 2*a^6*b^2*c^4*x^2*z - a^4*b^4*c^4*x^2*z -      4*b^8*c^4*x^2*z + 3*a^2*b^4*c^6*x^2*z + 4*b^6*c^6*x^2*z - 2*a^2*b^2*c^8*x^2*z - 3*b^4*c^8*x^2*z +      b^2*c^10*x^2*z - 2*a^6*b^4*c^2*x*y*z + 2*a^4*b^6*c^2*x*y*z + 2*a^6*b^2*c^4*x*y*z -      2*a^2*b^6*c^4*x*y*z - 2*a^4*b^2*c^6*x*y*z + 2*a^2*b^4*c^6*x*y*z + a^12*y^2*z - 4*a^10*b^2*y^2*z +      6*a^8*b^4*y^2*z - 4*a^6*b^6*y^2*z + a^4*b^8*y^2*z - 3*a^10*c^2*y^2*z + 5*a^8*b^2*c^2*y^2*z -      a^6*b^4*c^2*y^2*z - 2*a^4*b^6*c^2*y^2*z + a^2*b^8*c^2*y^2*z + 4*a^8*c^4*y^2*z + a^4*b^4*c^4*y^2*z -      2*a^2*b^6*c^4*y^2*z - 4*a^6*c^6*y^2*z - 3*a^4*b^2*c^6*y^2*z + 3*a^4*c^8*y^2*z +      2*a^2*b^2*c^8*y^2*z - a^2*c^10*y^2*z - a^10*b^2*x*z^2 + 3*a^8*b^4*x*z^2 - 4*a^6*b^6*x*z^2 +      4*a^4*b^8*x*z^2 - 3*a^2*b^10*x*z^2 + b^12*x*z^2 + 2*a^8*b^2*c^2*x*z^2 - 3*a^6*b^4*c^2*x*z^2 +      5*a^2*b^8*c^2*x*z^2 - 4*b^10*c^2*x*z^2 + a^4*b^4*c^4*x*z^2 - a^2*b^6*c^4*x*z^2 + 6*b^8*c^4*x*z^2 -      2*a^4*b^2*c^6*x*z^2 - 2*a^2*b^4*c^6*x*z^2 - 4*b^6*c^6*x*z^2 + a^2*b^2*c^8*x*z^2 + b^4*c^8*x*z^2 -      a^12*y*z^2 + 3*a^10*b^2*y*z^2 - 4*a^8*b^4*y*z^2 + 4*a^6*b^6*y*z^2 - 3*a^4*b^8*y*z^2 +      a^2*b^10*y*z^2 + 4*a^10*c^2*y*z^2 - 5*a^8*b^2*c^2*y*z^2 + 3*a^4*b^6*c^2*y*z^2 -      2*a^2*b^8*c^2*y*z^2 - 6*a^8*c^4*y*z^2 + a^6*b^2*c^4*y*z^2 - a^4*b^4*c^4*y*z^2 + 4*a^6*c^6*y*z^2 +      2*a^4*b^2*c^6*y*z^2 + 2*a^2*b^4*c^6*y*z^2 - a^4*c^8*y*z^2 - a^2*b^2*c^8*y*z^2");
		exp("K468","2*a^4*b^2*c^2*x^2*y - a^2*b^4*c^2*x^2*y - b^6*c^2*x^2*y + a^4*c^4*x^2*y + 2*b^4*c^4*x^2*y -      2*a^2*c^6*x^2*y - 2*b^2*c^6*x^2*y + c^8*x^2*y + a^6*c^2*x*y^2 + a^4*b^2*c^2*x*y^2 -      2*a^2*b^4*c^2*x*y^2 - 2*a^4*c^4*x*y^2 - b^4*c^4*x*y^2 + 2*a^2*c^6*x*y^2 + 2*b^2*c^6*x*y^2 -      c^8*x*y^2 - a^4*b^4*x^2*z + 2*a^2*b^6*x^2*z - b^8*x^2*z - 2*a^4*b^2*c^2*x^2*z + 2*b^6*c^2*x^2*z +      a^2*b^2*c^4*x^2*z - 2*b^4*c^4*x^2*z + b^2*c^6*x^2*z + a^8*y^2*z - 2*a^6*b^2*y^2*z + a^4*b^4*y^2*z -      2*a^6*c^2*y^2*z + 2*a^2*b^4*c^2*y^2*z + 2*a^4*c^4*y^2*z - a^2*b^2*c^4*y^2*z - a^2*c^6*y^2*z -      a^6*b^2*x*z^2 + 2*a^4*b^4*x*z^2 - 2*a^2*b^6*x*z^2 + b^8*x*z^2 - a^4*b^2*c^2*x*z^2 -      2*b^6*c^2*x*z^2 + 2*a^2*b^2*c^4*x*z^2 + b^4*c^4*x*z^2 - a^8*y*z^2 + 2*a^6*b^2*y*z^2 -      2*a^4*b^4*y*z^2 + a^2*b^6*y*z^2 + 2*a^6*c^2*y*z^2 + a^2*b^4*c^2*y*z^2 - a^4*c^4*y*z^2 -      2*a^2*b^2*c^4*y*z^2");
		exp("K469","(-a^8)*b^2*c^2*x^2*y + 2*a^6*b^4*c^2*x^2*y - 3*a^4*b^6*c^2*x^2*y + 4*a^2*b^8*c^2*x^2*y -      2*b^10*c^2*x^2*y + a^8*c^4*x^2*y + a^6*b^2*c^4*x^2*y - 2*a^4*b^4*c^4*x^2*y - 6*a^2*b^6*c^4*x^2*y +      6*b^8*c^4*x^2*y - 3*a^6*c^6*x^2*y + 2*a^4*b^2*c^6*x^2*y + 6*a^2*b^4*c^6*x^2*y - 5*b^6*c^6*x^2*y +      3*a^4*c^8*x^2*y - 3*a^2*b^2*c^8*x^2*y - a^2*c^10*x^2*y + b^2*c^10*x^2*y - 2*a^10*c^2*x*y^2 +      4*a^8*b^2*c^2*x*y^2 - 3*a^6*b^4*c^2*x*y^2 + 2*a^4*b^6*c^2*x*y^2 - a^2*b^8*c^2*x*y^2 +      6*a^8*c^4*x*y^2 - 6*a^6*b^2*c^4*x*y^2 - 2*a^4*b^4*c^4*x*y^2 + a^2*b^6*c^4*x*y^2 + b^8*c^4*x*y^2 -      5*a^6*c^6*x*y^2 + 6*a^4*b^2*c^6*x*y^2 + 2*a^2*b^4*c^6*x*y^2 - 3*b^6*c^6*x*y^2 -      3*a^2*b^2*c^8*x*y^2 + 3*b^4*c^8*x*y^2 + a^2*c^10*x*y^2 - b^2*c^10*x*y^2 + a^8*b^4*x^2*z -      3*a^6*b^6*x^2*z + 3*a^4*b^8*x^2*z - a^2*b^10*x^2*z - a^8*b^2*c^2*x^2*z + a^6*b^4*c^2*x^2*z +      2*a^4*b^6*c^2*x^2*z - 3*a^2*b^8*c^2*x^2*z + b^10*c^2*x^2*z + 2*a^6*b^2*c^4*x^2*z -      2*a^4*b^4*c^4*x^2*z + 6*a^2*b^6*c^4*x^2*z - 3*a^4*b^2*c^6*x^2*z - 6*a^2*b^4*c^6*x^2*z -      5*b^6*c^6*x^2*z + 4*a^2*b^2*c^8*x^2*z + 6*b^4*c^8*x^2*z - 2*b^2*c^10*x^2*z + 2*a^10*b^2*x*y*z -      8*a^8*b^4*x*y*z + 12*a^6*b^6*x*y*z - 8*a^4*b^8*x*y*z + 2*a^2*b^10*x*y*z + 2*a^10*c^2*x*y*z -      4*a^8*b^2*c^2*x*y*z + 2*a^6*b^4*c^2*x*y*z + 2*a^4*b^6*c^2*x*y*z - 4*a^2*b^8*c^2*x*y*z +      2*b^10*c^2*x*y*z - 8*a^8*c^4*x*y*z + 2*a^6*b^2*c^4*x*y*z + 2*a^2*b^6*c^4*x*y*z - 8*b^8*c^4*x*y*z +      12*a^6*c^6*x*y*z + 2*a^4*b^2*c^6*x*y*z + 2*a^2*b^4*c^6*x*y*z + 12*b^6*c^6*x*y*z - 8*a^4*c^8*x*y*z -      4*a^2*b^2*c^8*x*y*z - 8*b^4*c^8*x*y*z + 2*a^2*c^10*x*y*z + 2*b^2*c^10*x*y*z - a^10*b^2*y^2*z +      3*a^8*b^4*y^2*z - 3*a^6*b^6*y^2*z + a^4*b^8*y^2*z + a^10*c^2*y^2*z - 3*a^8*b^2*c^2*y^2*z +      2*a^6*b^4*c^2*y^2*z + a^4*b^6*c^2*y^2*z - a^2*b^8*c^2*y^2*z + 6*a^6*b^2*c^4*y^2*z -      2*a^4*b^4*c^4*y^2*z + 2*a^2*b^6*c^4*y^2*z - 5*a^6*c^6*y^2*z - 6*a^4*b^2*c^6*y^2*z -      3*a^2*b^4*c^6*y^2*z + 6*a^4*c^8*y^2*z + 4*a^2*b^2*c^8*y^2*z - 2*a^2*c^10*y^2*z - 2*a^10*b^2*x*z^2 +      6*a^8*b^4*x*z^2 - 5*a^6*b^6*x*z^2 + a^2*b^10*x*z^2 + 4*a^8*b^2*c^2*x*z^2 - 6*a^6*b^4*c^2*x*z^2 +      6*a^4*b^6*c^2*x*z^2 - 3*a^2*b^8*c^2*x*z^2 - b^10*c^2*x*z^2 - 3*a^6*b^2*c^4*x*z^2 -      2*a^4*b^4*c^4*x*z^2 + 2*a^2*b^6*c^4*x*z^2 + 3*b^8*c^4*x*z^2 + 2*a^4*b^2*c^6*x*z^2 +      a^2*b^4*c^6*x*z^2 - 3*b^6*c^6*x*z^2 - a^2*b^2*c^8*x*z^2 + b^4*c^8*x*z^2 + a^10*b^2*y*z^2 -      5*a^6*b^6*y*z^2 + 6*a^4*b^8*y*z^2 - 2*a^2*b^10*y*z^2 - a^10*c^2*y*z^2 - 3*a^8*b^2*c^2*y*z^2 +      6*a^6*b^4*c^2*y*z^2 - 6*a^4*b^6*c^2*y*z^2 + 4*a^2*b^8*c^2*y*z^2 + 3*a^8*c^4*y*z^2 +      2*a^6*b^2*c^4*y*z^2 - 2*a^4*b^4*c^4*y*z^2 - 3*a^2*b^6*c^4*y*z^2 - 3*a^6*c^6*y*z^2 +      a^4*b^2*c^6*y*z^2 + 2*a^2*b^4*c^6*y*z^2 + a^4*c^8*y*z^2 - a^2*b^2*c^8*y*z^2");
		exp("K470","a^8*b^2*c^2*x^2*y - 5*a^4*b^6*c^2*x^2*y + 6*a^2*b^8*c^2*x^2*y - 2*b^10*c^2*x^2*y -      a^8*c^4*x^2*y - 3*a^6*b^2*c^4*x^2*y + 6*a^4*b^4*c^4*x^2*y - 6*a^2*b^6*c^4*x^2*y + 4*b^8*c^4*x^2*y +      3*a^6*c^6*x^2*y + 2*a^4*b^2*c^6*x^2*y - 2*a^2*b^4*c^6*x^2*y - 3*b^6*c^6*x^2*y - 3*a^4*c^8*x^2*y +      a^2*b^2*c^8*x^2*y + 2*b^4*c^8*x^2*y + a^2*c^10*x^2*y - b^2*c^10*x^2*y - 2*a^10*c^2*x*y^2 +      6*a^8*b^2*c^2*x*y^2 - 5*a^6*b^4*c^2*x*y^2 + a^2*b^8*c^2*x*y^2 + 4*a^8*c^4*x*y^2 -      6*a^6*b^2*c^4*x*y^2 + 6*a^4*b^4*c^4*x*y^2 - 3*a^2*b^6*c^4*x*y^2 - b^8*c^4*x*y^2 - 3*a^6*c^6*x*y^2 -      2*a^4*b^2*c^6*x*y^2 + 2*a^2*b^4*c^6*x*y^2 + 3*b^6*c^6*x*y^2 + 2*a^4*c^8*x*y^2 + a^2*b^2*c^8*x*y^2 -      3*b^4*c^8*x*y^2 - a^2*c^10*x*y^2 + b^2*c^10*x*y^2 - a^8*b^4*x^2*z + 3*a^6*b^6*x^2*z -      3*a^4*b^8*x^2*z + a^2*b^10*x^2*z + a^8*b^2*c^2*x^2*z - 3*a^6*b^4*c^2*x^2*z + 2*a^4*b^6*c^2*x^2*z +      a^2*b^8*c^2*x^2*z - b^10*c^2*x^2*z + 6*a^4*b^4*c^4*x^2*z - 2*a^2*b^6*c^4*x^2*z + 2*b^8*c^4*x^2*z -      5*a^4*b^2*c^6*x^2*z - 6*a^2*b^4*c^6*x^2*z - 3*b^6*c^6*x^2*z + 6*a^2*b^2*c^8*x^2*z +      4*b^4*c^8*x^2*z - 2*b^2*c^10*x^2*z + 2*a^10*b^2*x*y*z - 8*a^8*b^4*x*y*z + 12*a^6*b^6*x*y*z -      8*a^4*b^8*x*y*z + 2*a^2*b^10*x*y*z + 2*a^10*c^2*x*y*z - 4*a^8*b^2*c^2*x*y*z + 2*a^6*b^4*c^2*x*y*z +      2*a^4*b^6*c^2*x*y*z - 4*a^2*b^8*c^2*x*y*z + 2*b^10*c^2*x*y*z - 8*a^8*c^4*x*y*z +      2*a^6*b^2*c^4*x*y*z + 2*a^2*b^6*c^4*x*y*z - 8*b^8*c^4*x*y*z + 12*a^6*c^6*x*y*z +      2*a^4*b^2*c^6*x*y*z + 2*a^2*b^4*c^6*x*y*z + 12*b^6*c^6*x*y*z - 8*a^4*c^8*x*y*z -      4*a^2*b^2*c^8*x*y*z - 8*b^4*c^8*x*y*z + 2*a^2*c^10*x*y*z + 2*b^2*c^10*x*y*z + a^10*b^2*y^2*z -      3*a^8*b^4*y^2*z + 3*a^6*b^6*y^2*z - a^4*b^8*y^2*z - a^10*c^2*y^2*z + a^8*b^2*c^2*y^2*z +      2*a^6*b^4*c^2*y^2*z - 3*a^4*b^6*c^2*y^2*z + a^2*b^8*c^2*y^2*z + 2*a^8*c^4*y^2*z -      2*a^6*b^2*c^4*y^2*z + 6*a^4*b^4*c^4*y^2*z - 3*a^6*c^6*y^2*z - 6*a^4*b^2*c^6*y^2*z -      5*a^2*b^4*c^6*y^2*z + 4*a^4*c^8*y^2*z + 6*a^2*b^2*c^8*y^2*z - 2*a^2*c^10*y^2*z - 2*a^10*b^2*x*z^2 +      4*a^8*b^4*x*z^2 - 3*a^6*b^6*x*z^2 + 2*a^4*b^8*x*z^2 - a^2*b^10*x*z^2 + 6*a^8*b^2*c^2*x*z^2 -      6*a^6*b^4*c^2*x*z^2 - 2*a^4*b^6*c^2*x*z^2 + a^2*b^8*c^2*x*z^2 + b^10*c^2*x*z^2 -      5*a^6*b^2*c^4*x*z^2 + 6*a^4*b^4*c^4*x*z^2 + 2*a^2*b^6*c^4*x*z^2 - 3*b^8*c^4*x*z^2 -      3*a^2*b^4*c^6*x*z^2 + 3*b^6*c^6*x*z^2 + a^2*b^2*c^8*x*z^2 - b^4*c^8*x*z^2 - a^10*b^2*y*z^2 +      2*a^8*b^4*y*z^2 - 3*a^6*b^6*y*z^2 + 4*a^4*b^8*y*z^2 - 2*a^2*b^10*y*z^2 + a^10*c^2*y*z^2 +      a^8*b^2*c^2*y*z^2 - 2*a^6*b^4*c^2*y*z^2 - 6*a^4*b^6*c^2*y*z^2 + 6*a^2*b^8*c^2*y*z^2 -      3*a^8*c^4*y*z^2 + 2*a^6*b^2*c^4*y*z^2 + 6*a^4*b^4*c^4*y*z^2 - 5*a^2*b^6*c^4*y*z^2 +      3*a^6*c^6*y*z^2 - 3*a^4*b^2*c^6*y*z^2 - a^4*c^8*y*z^2 + a^2*b^2*c^8*y*z^2");
		exp("K471","(-a^10)*b^2*c^2*x^2*y + 3*a^8*b^4*c^2*x^2*y - 3*a^6*b^6*c^2*x^2*y + a^4*b^8*c^2*x^2*y +      a^10*c^4*x^2*y + a^8*b^2*c^4*x^2*y - a^6*b^4*c^4*x^2*y - a^4*b^6*c^4*x^2*y + 2*a^2*b^8*c^4*x^2*y -      2*b^10*c^4*x^2*y - 4*a^8*c^6*x^2*y - 2*a^6*b^2*c^6*x^2*y - a^2*b^6*c^6*x^2*y + 7*b^8*c^6*x^2*y +      6*a^6*c^8*x^2*y + 4*a^4*b^2*c^8*x^2*y - a^2*b^4*c^8*x^2*y - 9*b^6*c^8*x^2*y - 4*a^4*c^10*x^2*y -      a^2*b^2*c^10*x^2*y + 5*b^4*c^10*x^2*y + a^2*c^12*x^2*y - b^2*c^12*x^2*y + a^8*b^4*c^2*x*y^2 -      3*a^6*b^6*c^2*x*y^2 + 3*a^4*b^8*c^2*x*y^2 - a^2*b^10*c^2*x*y^2 - 2*a^10*c^4*x*y^2 +      2*a^8*b^2*c^4*x*y^2 - a^6*b^4*c^4*x*y^2 - a^4*b^6*c^4*x*y^2 + a^2*b^8*c^4*x*y^2 + b^10*c^4*x*y^2 +      7*a^8*c^6*x*y^2 - a^6*b^2*c^6*x*y^2 - 2*a^2*b^6*c^6*x*y^2 - 4*b^8*c^6*x*y^2 - 9*a^6*c^8*x*y^2 -      a^4*b^2*c^8*x*y^2 + 4*a^2*b^4*c^8*x*y^2 + 6*b^6*c^8*x*y^2 + 5*a^4*c^10*x*y^2 - a^2*b^2*c^10*x*y^2 -      4*b^4*c^10*x*y^2 - a^2*c^12*x*y^2 + b^2*c^12*x*y^2 + a^10*b^4*x^2*z - 4*a^8*b^6*x^2*z +      6*a^6*b^8*x^2*z - 4*a^4*b^10*x^2*z + a^2*b^12*x^2*z - a^10*b^2*c^2*x^2*z + a^8*b^4*c^2*x^2*z -      2*a^6*b^6*c^2*x^2*z + 4*a^4*b^8*c^2*x^2*z - a^2*b^10*c^2*x^2*z - b^12*c^2*x^2*z +      3*a^8*b^2*c^4*x^2*z - a^6*b^4*c^4*x^2*z - a^2*b^8*c^4*x^2*z + 5*b^10*c^4*x^2*z -      3*a^6*b^2*c^6*x^2*z - a^4*b^4*c^6*x^2*z - a^2*b^6*c^6*x^2*z - 9*b^8*c^6*x^2*z + a^4*b^2*c^8*x^2*z +      2*a^2*b^4*c^8*x^2*z + 7*b^6*c^8*x^2*z - 2*b^4*c^10*x^2*z + 4*a^10*b^2*c^2*x*y*z -      10*a^8*b^4*c^2*x*y*z + 12*a^6*b^6*c^2*x*y*z - 10*a^4*b^8*c^2*x*y*z + 4*a^2*b^10*c^2*x*y*z -      10*a^8*b^2*c^4*x*y*z + 4*a^6*b^4*c^4*x*y*z + 4*a^4*b^6*c^4*x*y*z - 10*a^2*b^8*c^4*x*y*z +      12*a^6*b^2*c^6*x*y*z + 4*a^4*b^4*c^6*x*y*z + 12*a^2*b^6*c^6*x*y*z - 10*a^4*b^2*c^8*x*y*z -      10*a^2*b^4*c^8*x*y*z + 4*a^2*b^2*c^10*x*y*z + a^12*b^2*y^2*z - 4*a^10*b^4*y^2*z + 6*a^8*b^6*y^2*z -      4*a^6*b^8*y^2*z + a^4*b^10*y^2*z - a^12*c^2*y^2*z - a^10*b^2*c^2*y^2*z + 4*a^8*b^4*c^2*y^2*z -      2*a^6*b^6*c^2*y^2*z + a^4*b^8*c^2*y^2*z - a^2*b^10*c^2*y^2*z + 5*a^10*c^4*y^2*z -      a^8*b^2*c^4*y^2*z - a^4*b^6*c^4*y^2*z + 3*a^2*b^8*c^4*y^2*z - 9*a^8*c^6*y^2*z - a^6*b^2*c^6*y^2*z -      a^4*b^4*c^6*y^2*z - 3*a^2*b^6*c^6*y^2*z + 7*a^6*c^8*y^2*z + 2*a^4*b^2*c^8*y^2*z +      a^2*b^4*c^8*y^2*z - 2*a^4*c^10*y^2*z - 2*a^10*b^4*x*z^2 + 7*a^8*b^6*x*z^2 - 9*a^6*b^8*x*z^2 +      5*a^4*b^10*x*z^2 - a^2*b^12*x*z^2 + 2*a^8*b^4*c^2*x*z^2 - a^6*b^6*c^2*x*z^2 - a^4*b^8*c^2*x*z^2 -      a^2*b^10*c^2*x*z^2 + b^12*c^2*x*z^2 + a^8*b^2*c^4*x*z^2 - a^6*b^4*c^4*x*z^2 + 4*a^2*b^8*c^4*x*z^2 -      4*b^10*c^4*x*z^2 - 3*a^6*b^2*c^6*x*z^2 - a^4*b^4*c^6*x*z^2 - 2*a^2*b^6*c^6*x*z^2 +      6*b^8*c^6*x*z^2 + 3*a^4*b^2*c^8*x*z^2 + a^2*b^4*c^8*x*z^2 - 4*b^6*c^8*x*z^2 - a^2*b^2*c^10*x*z^2 +      b^4*c^10*x*z^2 - a^12*b^2*y*z^2 + 5*a^10*b^4*y*z^2 - 9*a^8*b^6*y*z^2 + 7*a^6*b^8*y*z^2 -      2*a^4*b^10*y*z^2 + a^12*c^2*y*z^2 - a^10*b^2*c^2*y*z^2 - a^8*b^4*c^2*y*z^2 - a^6*b^6*c^2*y*z^2 +      2*a^4*b^8*c^2*y*z^2 - 4*a^10*c^4*y*z^2 + 4*a^8*b^2*c^4*y*z^2 - a^4*b^6*c^4*y*z^2 +      a^2*b^8*c^4*y*z^2 + 6*a^8*c^6*y*z^2 - 2*a^6*b^2*c^6*y*z^2 - a^4*b^4*c^6*y*z^2 -      3*a^2*b^6*c^6*y*z^2 - 4*a^6*c^8*y*z^2 + a^4*b^2*c^8*y*z^2 + 3*a^2*b^4*c^8*y*z^2 + a^4*c^10*y*z^2 -      a^2*b^2*c^10*y*z^2");
		exp("K472","a^4*x^2*y - 2*a^2*b^2*x^2*y + b^4*x^2*y + a^2*c^2*x^2*y + b^2*c^2*x^2*y - 2*c^4*x^2*y -      a^4*x*y^2 + 2*a^2*b^2*x*y^2 - b^4*x*y^2 - a^2*c^2*x*y^2 - b^2*c^2*x*y^2 + 2*c^4*x*y^2 - a^4*x^2*z -      a^2*b^2*x^2*z + 2*b^4*x^2*z + 2*a^2*c^2*x^2*z - b^2*c^2*x^2*z - c^4*x^2*z - 2*a^4*y^2*z +      a^2*b^2*y^2*z + b^4*y^2*z + a^2*c^2*y^2*z - 2*b^2*c^2*y^2*z + c^4*y^2*z + a^4*x*z^2 +      a^2*b^2*x*z^2 - 2*b^4*x*z^2 - 2*a^2*c^2*x*z^2 + b^2*c^2*x*z^2 + c^4*x*z^2 + 2*a^4*y*z^2 -      a^2*b^2*y*z^2 - b^4*y*z^2 - a^2*c^2*y*z^2 + 2*b^2*c^2*y*z^2 - c^4*y*z^2");
		exp("K473","a^6*c^2*x^2*y + a^4*b^2*c^2*x^2*y - a^2*b^4*c^2*x^2*y - b^6*c^2*x^2*y - 3*a^4*c^4*x^2*y +      2*a^2*c^6*x^2*y + b^2*c^6*x^2*y + a^6*c^2*x*y^2 + a^4*b^2*c^2*x*y^2 - a^2*b^4*c^2*x*y^2 -      b^6*c^2*x*y^2 + 3*b^4*c^4*x*y^2 - a^2*c^6*x*y^2 - 2*b^2*c^6*x*y^2 - a^6*b^2*x^2*z +      3*a^4*b^4*x^2*z - 2*a^2*b^6*x^2*z - a^4*b^2*c^2*x^2*z - b^6*c^2*x^2*z + a^2*b^2*c^4*x^2*z +      b^2*c^6*x^2*z + a^6*b^2*x*y*z - a^2*b^6*x*y*z - a^6*c^2*x*y*z + b^6*c^2*x*y*z + a^2*c^6*x*y*z -      b^2*c^6*x*y*z + 2*a^6*b^2*y^2*z - 3*a^4*b^4*y^2*z + a^2*b^6*y^2*z + a^6*c^2*y^2*z +      a^2*b^4*c^2*y^2*z - a^2*b^2*c^4*y^2*z - a^2*c^6*y^2*z - a^6*b^2*x*z^2 + a^2*b^6*x*z^2 -      a^4*b^2*c^2*x*z^2 + 2*b^6*c^2*x*z^2 + a^2*b^2*c^4*x*z^2 - 3*b^4*c^4*x*z^2 + b^2*c^6*x*z^2 -      a^6*b^2*y*z^2 + a^2*b^6*y*z^2 - 2*a^6*c^2*y*z^2 + a^2*b^4*c^2*y*z^2 + 3*a^4*c^4*y*z^2 -      a^2*b^2*c^4*y*z^2 - a^2*c^6*y*z^2");
		exp("K474","a^6*c^2*x^2*y + a^4*b^2*c^2*x^2*y - a^2*b^4*c^2*x^2*y - b^6*c^2*x^2*y - 2*a^4*c^4*x^2*y +      a^2*c^6*x^2*y + b^2*c^6*x^2*y + a^6*c^2*x*y^2 + a^4*b^2*c^2*x*y^2 - a^2*b^4*c^2*x*y^2 -      b^6*c^2*x*y^2 + 2*b^4*c^4*x*y^2 - a^2*c^6*x*y^2 - b^2*c^6*x*y^2 - a^6*b^2*x^2*z + 2*a^4*b^4*x^2*z -      a^2*b^6*x^2*z - a^4*b^2*c^2*x^2*z - b^6*c^2*x^2*z + a^2*b^2*c^4*x^2*z + b^2*c^6*x^2*z +      a^6*b^2*y^2*z - 2*a^4*b^4*y^2*z + a^2*b^6*y^2*z + a^6*c^2*y^2*z + a^2*b^4*c^2*y^2*z -      a^2*b^2*c^4*y^2*z - a^2*c^6*y^2*z - a^6*b^2*x*z^2 + a^2*b^6*x*z^2 - a^4*b^2*c^2*x*z^2 +      b^6*c^2*x*z^2 + a^2*b^2*c^4*x*z^2 - 2*b^4*c^4*x*z^2 + b^2*c^6*x*z^2 - a^6*b^2*y*z^2 +      a^2*b^6*y*z^2 - a^6*c^2*y*z^2 + a^2*b^4*c^2*y*z^2 + 2*a^4*c^4*y*z^2 - a^2*b^2*c^4*y*z^2 -      a^2*c^6*y*z^2");
		exp("K475","(-a^8)*c^2*x^2*y + 2*a^6*b^2*c^2*x^2*y - 2*a^2*b^6*c^2*x^2*y + b^8*c^2*x^2*y + a^6*c^4*x^2*y -      3*a^4*b^2*c^4*x^2*y + a^2*b^4*c^4*x^2*y + b^6*c^4*x^2*y + a^4*c^6*x^2*y + 2*a^2*b^2*c^6*x^2*y -      b^4*c^6*x^2*y - a^2*c^8*x^2*y - b^2*c^8*x^2*y - a^8*c^2*x*y^2 + 2*a^6*b^2*c^2*x*y^2 -      2*a^2*b^6*c^2*x*y^2 + b^8*c^2*x*y^2 - a^6*c^4*x*y^2 - a^4*b^2*c^4*x*y^2 + 3*a^2*b^4*c^4*x*y^2 -      b^6*c^4*x*y^2 + a^4*c^6*x*y^2 - 2*a^2*b^2*c^6*x*y^2 - b^4*c^6*x*y^2 + a^2*c^8*x*y^2 +      b^2*c^8*x*y^2 + a^8*b^2*x^2*z - a^6*b^4*x^2*z - a^4*b^6*x^2*z + a^2*b^8*x^2*z -      2*a^6*b^2*c^2*x^2*z + 3*a^4*b^4*c^2*x^2*z - 2*a^2*b^6*c^2*x^2*z + b^8*c^2*x^2*z -      a^2*b^4*c^4*x^2*z + b^6*c^4*x^2*z + 2*a^2*b^2*c^6*x^2*z - b^4*c^6*x^2*z - b^2*c^8*x^2*z -      a^8*b^2*y^2*z + a^6*b^4*y^2*z + a^4*b^6*y^2*z - a^2*b^8*y^2*z - a^8*c^2*y^2*z +      2*a^6*b^2*c^2*y^2*z - 3*a^4*b^4*c^2*y^2*z + 2*a^2*b^6*c^2*y^2*z - a^6*c^4*y^2*z +      a^4*b^2*c^4*y^2*z + a^4*c^6*y^2*z - 2*a^2*b^2*c^6*y^2*z + a^2*c^8*y^2*z + a^8*b^2*x*z^2 +      a^6*b^4*x*z^2 - a^4*b^6*x*z^2 - a^2*b^8*x*z^2 - 2*a^6*b^2*c^2*x*z^2 + a^4*b^4*c^2*x*z^2 +      2*a^2*b^6*c^2*x*z^2 - b^8*c^2*x*z^2 - 3*a^2*b^4*c^4*x*z^2 + b^6*c^4*x*z^2 + 2*a^2*b^2*c^6*x*z^2 +      b^4*c^6*x*z^2 - b^2*c^8*x*z^2 + a^8*b^2*y*z^2 + a^6*b^4*y*z^2 - a^4*b^6*y*z^2 - a^2*b^8*y*z^2 +      a^8*c^2*y*z^2 - 2*a^6*b^2*c^2*y*z^2 - a^4*b^4*c^2*y*z^2 + 2*a^2*b^6*c^2*y*z^2 - a^6*c^4*y*z^2 +      3*a^4*b^2*c^4*y*z^2 - a^4*c^6*y*z^2 - 2*a^2*b^2*c^6*y*z^2 + a^2*c^8*y*z^2");
		exp("K476","a^6*x^2*y - a^4*b^2*x^2*y - a^2*b^4*x^2*y + b^6*x^2*y - a^4*c^2*x^2*y + a^2*b^2*c^2*x^2*y -      a^2*c^4*x^2*y + c^6*x^2*y + a^6*x*y^2 - a^4*b^2*x*y^2 - a^2*b^4*x*y^2 + b^6*x*y^2 +      a^2*b^2*c^2*x*y^2 - b^4*c^2*x*y^2 - b^2*c^4*x*y^2 + c^6*x*y^2 + a^6*x^2*z - a^4*b^2*x^2*z -      a^2*b^4*x^2*z + b^6*x^2*z - a^4*c^2*x^2*z + a^2*b^2*c^2*x^2*z - a^2*c^4*x^2*z + c^6*x^2*z +      2*a^6*x*y*z - 2*a^4*b^2*x*y*z - 2*a^2*b^4*x*y*z + 2*b^6*x*y*z - 2*a^4*c^2*x*y*z +      6*a^2*b^2*c^2*x*y*z - 2*b^4*c^2*x*y*z - 2*a^2*c^4*x*y*z - 2*b^2*c^4*x*y*z + 2*c^6*x*y*z +      a^6*y^2*z - a^4*b^2*y^2*z - a^2*b^4*y^2*z + b^6*y^2*z + a^2*b^2*c^2*y^2*z - b^4*c^2*y^2*z -      b^2*c^4*y^2*z + c^6*y^2*z + a^6*x*z^2 + b^6*x*z^2 - a^4*c^2*x*z^2 + a^2*b^2*c^2*x*z^2 -      b^4*c^2*x*z^2 - a^2*c^4*x*z^2 - b^2*c^4*x*z^2 + c^6*x*z^2 + a^6*y*z^2 + b^6*y*z^2 - a^4*c^2*y*z^2 +      a^2*b^2*c^2*y*z^2 - b^4*c^2*y*z^2 - a^2*c^4*y*z^2 - b^2*c^4*y*z^2 + c^6*y*z^2");
		exp("K477","(-a^8)*x^2*y + 2*a^6*b^2*x^2*y - 2*a^2*b^6*x^2*y + b^8*x^2*y + a^6*c^2*x^2*y -      2*a^4*b^2*c^2*x^2*y + a^2*b^4*c^2*x^2*y + 2*a^4*c^4*x^2*y - a^2*c^6*x^2*y - c^8*x^2*y - a^8*x*y^2 +      2*a^6*b^2*x*y^2 - 2*a^2*b^6*x*y^2 + b^8*x*y^2 - a^4*b^2*c^2*x*y^2 + 2*a^2*b^4*c^2*x*y^2 -      b^6*c^2*x*y^2 - 2*b^4*c^4*x*y^2 + b^2*c^6*x*y^2 + c^8*x*y^2 + a^8*x^2*z - a^6*b^2*x^2*z -      2*a^4*b^4*x^2*z + a^2*b^6*x^2*z + b^8*x^2*z - 2*a^6*c^2*x^2*z + 2*a^4*b^2*c^2*x^2*z -      a^2*b^2*c^4*x^2*z + 2*a^2*c^6*x^2*z - c^8*x^2*z - 2*a^6*b^2*x*y*z + 2*a^2*b^6*x*y*z +      2*a^6*c^2*x*y*z - 2*b^6*c^2*x*y*z - 2*a^2*c^6*x*y*z + 2*b^2*c^6*x*y*z - a^8*y^2*z - a^6*b^2*y^2*z +      2*a^4*b^4*y^2*z + a^2*b^6*y^2*z - b^8*y^2*z - 2*a^2*b^4*c^2*y^2*z + 2*b^6*c^2*y^2*z +      a^2*b^2*c^4*y^2*z - 2*b^2*c^6*y^2*z + c^8*y^2*z + a^8*x*z^2 - b^8*x*z^2 - 2*a^6*c^2*x*z^2 +      a^4*b^2*c^2*x*z^2 - b^6*c^2*x*z^2 - 2*a^2*b^2*c^4*x*z^2 + 2*b^4*c^4*x*z^2 + 2*a^2*c^6*x*z^2 +      b^2*c^6*x*z^2 - c^8*x*z^2 + a^8*y*z^2 - b^8*y*z^2 + a^6*c^2*y*z^2 - a^2*b^4*c^2*y*z^2 +      2*b^6*c^2*y*z^2 - 2*a^4*c^4*y*z^2 + 2*a^2*b^2*c^4*y*z^2 - a^2*c^6*y*z^2 - 2*b^2*c^6*y*z^2 +      c^8*y*z^2");
		exp("K478","a^6*c^4*x^2*y - 2*a^4*b^2*c^4*x^2*y - a^2*b^4*c^4*x^2*y + 2*b^6*c^4*x^2*y - a^4*c^6*x^2*y +      4*a^2*b^2*c^6*x^2*y - b^4*c^6*x^2*y - a^2*c^8*x^2*y - 2*b^2*c^8*x^2*y + c^10*x^2*y -      2*a^6*c^4*x*y^2 + a^4*b^2*c^4*x*y^2 + 2*a^2*b^4*c^4*x*y^2 - b^6*c^4*x*y^2 + a^4*c^6*x*y^2 -      4*a^2*b^2*c^6*x*y^2 + b^4*c^6*x*y^2 + 2*a^2*c^8*x*y^2 + b^2*c^8*x*y^2 - c^10*x*y^2 -      a^6*b^4*x^2*z + a^4*b^6*x^2*z + a^2*b^8*x^2*z - b^10*x^2*z + 2*a^4*b^4*c^2*x^2*z -      4*a^2*b^6*c^2*x^2*z + 2*b^8*c^2*x^2*z + a^2*b^4*c^4*x^2*z + b^6*c^4*x^2*z - 2*b^4*c^6*x^2*z +      a^10*y^2*z - a^8*b^2*y^2*z - a^6*b^4*y^2*z + a^4*b^6*y^2*z - 2*a^8*c^2*y^2*z +      4*a^6*b^2*c^2*y^2*z - 2*a^4*b^4*c^2*y^2*z - a^6*c^4*y^2*z - a^4*b^2*c^4*y^2*z + 2*a^4*c^6*y^2*z +      2*a^6*b^4*x*z^2 - a^4*b^6*x*z^2 - 2*a^2*b^8*x*z^2 + b^10*x*z^2 - a^4*b^4*c^2*x*z^2 +      4*a^2*b^6*c^2*x*z^2 - b^8*c^2*x*z^2 - 2*a^2*b^4*c^4*x*z^2 - b^6*c^4*x*z^2 + b^4*c^6*x*z^2 -      a^10*y*z^2 + 2*a^8*b^2*y*z^2 + a^6*b^4*y*z^2 - 2*a^4*b^6*y*z^2 + a^8*c^2*y*z^2 -      4*a^6*b^2*c^2*y*z^2 + a^4*b^4*c^2*y*z^2 + a^6*c^4*y*z^2 + 2*a^4*b^2*c^4*y*z^2 - a^4*c^6*y*z^2");
		exp("K479","a^8*x^2*y - 2*a^6*b^2*x^2*y + 2*a^2*b^6*x^2*y - b^8*x^2*y + 3*a^4*b^2*c^2*x^2*y -      2*a^2*b^4*c^2*x^2*y - b^6*c^2*x^2*y - 2*a^4*c^4*x^2*y + b^2*c^6*x^2*y + c^8*x^2*y + a^8*x*y^2 -      2*a^6*b^2*x*y^2 + 2*a^2*b^6*x*y^2 - b^8*x*y^2 + a^6*c^2*x*y^2 + 2*a^4*b^2*c^2*x*y^2 -      3*a^2*b^4*c^2*x*y^2 + 2*b^4*c^4*x*y^2 - a^2*c^6*x*y^2 - c^8*x*y^2 - a^8*x^2*z + 2*a^4*b^4*x^2*z -      b^8*x^2*z + 2*a^6*c^2*x^2*z - 3*a^4*b^2*c^2*x^2*z - b^6*c^2*x^2*z + 2*a^2*b^2*c^4*x^2*z -      2*a^2*c^6*x^2*z + b^2*c^6*x^2*z + c^8*x^2*z + a^8*y^2*z - 2*a^4*b^4*y^2*z + b^8*y^2*z +      a^6*c^2*y^2*z + 3*a^2*b^4*c^2*y^2*z - 2*b^6*c^2*y^2*z - 2*a^2*b^2*c^4*y^2*z - a^2*c^6*y^2*z +      2*b^2*c^6*y^2*z - c^8*y^2*z - a^8*x*z^2 - a^6*b^2*x*z^2 + a^2*b^6*x*z^2 + b^8*x*z^2 +      2*a^6*c^2*x*z^2 - 2*a^4*b^2*c^2*x*z^2 + 3*a^2*b^2*c^4*x*z^2 - 2*b^4*c^4*x*z^2 - 2*a^2*c^6*x*z^2 +      c^8*x*z^2 - a^8*y*z^2 - a^6*b^2*y*z^2 + a^2*b^6*y*z^2 + b^8*y*z^2 + 2*a^2*b^4*c^2*y*z^2 -      2*b^6*c^2*y*z^2 + 2*a^4*c^4*y*z^2 - 3*a^2*b^2*c^4*y*z^2 + 2*b^2*c^6*y*z^2 - c^8*y*z^2");
		exp("K480","(-a^12)*c^2*x^2*y + 2*a^10*b^2*c^2*x^2*y - a^8*b^4*c^2*x^2*y + a^4*b^8*c^2*x^2*y -      2*a^2*b^10*c^2*x^2*y + b^12*c^2*x^2*y + 3*a^10*c^4*x^2*y - 6*a^8*b^2*c^4*x^2*y -      a^6*b^4*c^4*x^2*y + 3*a^4*b^6*c^4*x^2*y + 2*a^2*b^8*c^4*x^2*y - b^10*c^4*x^2*y - 2*a^8*c^6*x^2*y +      10*a^6*b^2*c^6*x^2*y - 2*a^2*b^6*c^6*x^2*y - 2*b^8*c^6*x^2*y - 2*a^6*c^8*x^2*y -      9*a^4*b^2*c^8*x^2*y - a^2*b^4*c^8*x^2*y + 2*b^6*c^8*x^2*y + 3*a^4*c^10*x^2*y +      4*a^2*b^2*c^10*x^2*y + b^4*c^10*x^2*y - a^2*c^12*x^2*y - b^2*c^12*x^2*y - a^12*c^2*x*y^2 +      2*a^10*b^2*c^2*x*y^2 - a^8*b^4*c^2*x*y^2 + a^4*b^8*c^2*x*y^2 - 2*a^2*b^10*c^2*x*y^2 +      b^12*c^2*x*y^2 + a^10*c^4*x*y^2 - 2*a^8*b^2*c^4*x*y^2 - 3*a^6*b^4*c^4*x*y^2 + a^4*b^6*c^4*x*y^2 +      6*a^2*b^8*c^4*x*y^2 - 3*b^10*c^4*x*y^2 + 2*a^8*c^6*x*y^2 + 2*a^6*b^2*c^6*x*y^2 -      10*a^2*b^6*c^6*x*y^2 + 2*b^8*c^6*x*y^2 - 2*a^6*c^8*x*y^2 + a^4*b^2*c^8*x*y^2 +      9*a^2*b^4*c^8*x*y^2 + 2*b^6*c^8*x*y^2 - a^4*c^10*x*y^2 - 4*a^2*b^2*c^10*x*y^2 - 3*b^4*c^10*x*y^2 +      a^2*c^12*x*y^2 + b^2*c^12*x*y^2 + a^12*b^2*x^2*z - 3*a^10*b^4*x^2*z + 2*a^8*b^6*x^2*z +      2*a^6*b^8*x^2*z - 3*a^4*b^10*x^2*z + a^2*b^12*x^2*z - 2*a^10*b^2*c^2*x^2*z + 6*a^8*b^4*c^2*x^2*z -      10*a^6*b^6*c^2*x^2*z + 9*a^4*b^8*c^2*x^2*z - 4*a^2*b^10*c^2*x^2*z + b^12*c^2*x^2*z +      a^8*b^2*c^4*x^2*z + a^6*b^4*c^4*x^2*z + a^2*b^8*c^4*x^2*z - b^10*c^4*x^2*z - 3*a^4*b^4*c^6*x^2*z +      2*a^2*b^6*c^6*x^2*z - 2*b^8*c^6*x^2*z - a^4*b^2*c^8*x^2*z - 2*a^2*b^4*c^8*x^2*z + 2*b^6*c^8*x^2*z +      2*a^2*b^2*c^10*x^2*z + b^4*c^10*x^2*z - b^2*c^12*x^2*z - 2*a^8*b^4*c^2*x*y*z +      2*a^4*b^8*c^2*x*y*z + 2*a^8*b^2*c^4*x*y*z - 2*a^2*b^8*c^4*x*y*z - 2*a^4*b^2*c^8*x*y*z +      2*a^2*b^4*c^8*x*y*z - a^12*b^2*y^2*z + 3*a^10*b^4*y^2*z - 2*a^8*b^6*y^2*z - 2*a^6*b^8*y^2*z +      3*a^4*b^10*y^2*z - a^2*b^12*y^2*z - a^12*c^2*y^2*z + 4*a^10*b^2*c^2*y^2*z - 9*a^8*b^4*c^2*y^2*z +      10*a^6*b^6*c^2*y^2*z - 6*a^4*b^8*c^2*y^2*z + 2*a^2*b^10*c^2*y^2*z + a^10*c^4*y^2*z -      a^8*b^2*c^4*y^2*z - a^4*b^6*c^4*y^2*z - a^2*b^8*c^4*y^2*z + 2*a^8*c^6*y^2*z - 2*a^6*b^2*c^6*y^2*z +      3*a^4*b^4*c^6*y^2*z - 2*a^6*c^8*y^2*z + 2*a^4*b^2*c^8*y^2*z + a^2*b^4*c^8*y^2*z - a^4*c^10*y^2*z -      2*a^2*b^2*c^10*y^2*z + a^2*c^12*y^2*z + a^12*b^2*x*z^2 - a^10*b^4*x*z^2 - 2*a^8*b^6*x*z^2 +      2*a^6*b^8*x*z^2 + a^4*b^10*x*z^2 - a^2*b^12*x*z^2 - 2*a^10*b^2*c^2*x*z^2 + 2*a^8*b^4*c^2*x*z^2 -      2*a^6*b^6*c^2*x*z^2 - a^4*b^8*c^2*x*z^2 + 4*a^2*b^10*c^2*x*z^2 - b^12*c^2*x*z^2 +      a^8*b^2*c^4*x*z^2 + 3*a^6*b^4*c^4*x*z^2 - 9*a^2*b^8*c^4*x*z^2 + 3*b^10*c^4*x*z^2 -      a^4*b^4*c^6*x*z^2 + 10*a^2*b^6*c^6*x*z^2 - 2*b^8*c^6*x*z^2 - a^4*b^2*c^8*x*z^2 -      6*a^2*b^4*c^8*x*z^2 - 2*b^6*c^8*x*z^2 + 2*a^2*b^2*c^10*x*z^2 + 3*b^4*c^10*x*z^2 - b^2*c^12*x*z^2 +      a^12*b^2*y*z^2 - a^10*b^4*y*z^2 - 2*a^8*b^6*y*z^2 + 2*a^6*b^8*y*z^2 + a^4*b^10*y*z^2 -      a^2*b^12*y*z^2 + a^12*c^2*y*z^2 - 4*a^10*b^2*c^2*y*z^2 + a^8*b^4*c^2*y*z^2 + 2*a^6*b^6*c^2*y*z^2 -      2*a^4*b^8*c^2*y*z^2 + 2*a^2*b^10*c^2*y*z^2 - 3*a^10*c^4*y*z^2 + 9*a^8*b^2*c^4*y*z^2 -      3*a^4*b^6*c^4*y*z^2 - a^2*b^8*c^4*y*z^2 + 2*a^8*c^6*y*z^2 - 10*a^6*b^2*c^6*y*z^2 +      a^4*b^4*c^6*y*z^2 + 2*a^6*c^8*y*z^2 + 6*a^4*b^2*c^8*y*z^2 + a^2*b^4*c^8*y*z^2 - 3*a^4*c^10*y*z^2 -      2*a^2*b^2*c^10*y*z^2 + a^2*c^12*y*z^2");
		exp("K481","a^8*x^2*y - 2*a^6*b^2*x^2*y + 2*a^2*b^6*x^2*y - b^8*x^2*y - 2*a^6*c^2*x^2*y + a^4*b^2*c^2*x^2*y +      b^6*c^2*x^2*y + a^4*c^4*x^2*y - a^2*c^6*x^2*y - b^2*c^6*x^2*y + c^8*x^2*y + a^8*x*y^2 -      2*a^6*b^2*x*y^2 + 2*a^2*b^6*x*y^2 - b^8*x*y^2 - a^6*c^2*x*y^2 - a^2*b^4*c^2*x*y^2 +      2*b^6*c^2*x*y^2 - b^4*c^4*x*y^2 + a^2*c^6*x*y^2 + b^2*c^6*x*y^2 - c^8*x*y^2 - a^8*x^2*z +      2*a^6*b^2*x^2*z - a^4*b^4*x^2*z + a^2*b^6*x^2*z - b^8*x^2*z + 2*a^6*c^2*x^2*z - a^4*b^2*c^2*x^2*z +      b^6*c^2*x^2*z - 2*a^2*c^6*x^2*z - b^2*c^6*x^2*z + c^8*x^2*z + a^6*b^2*x*y*z - a^2*b^6*x*y*z -      a^6*c^2*x*y*z + b^6*c^2*x*y*z + a^2*c^6*x*y*z - b^2*c^6*x*y*z + a^8*y^2*z - a^6*b^2*y^2*z +      a^4*b^4*y^2*z - 2*a^2*b^6*y^2*z + b^8*y^2*z - a^6*c^2*y^2*z + a^2*b^4*c^2*y^2*z - 2*b^6*c^2*y^2*z +      a^2*c^6*y^2*z + 2*b^2*c^6*y^2*z - c^8*y^2*z - a^8*x*z^2 + a^6*b^2*x*z^2 - a^2*b^6*x*z^2 +      b^8*x*z^2 + 2*a^6*c^2*x*z^2 - b^6*c^2*x*z^2 + a^2*b^2*c^4*x*z^2 + b^4*c^4*x*z^2 - 2*a^2*c^6*x*z^2 -      2*b^2*c^6*x*z^2 + c^8*x*z^2 - a^8*y*z^2 + a^6*b^2*y*z^2 - a^2*b^6*y*z^2 + b^8*y*z^2 +      a^6*c^2*y*z^2 - 2*b^6*c^2*y*z^2 - a^4*c^4*y*z^2 - a^2*b^2*c^4*y*z^2 + 2*a^2*c^6*y*z^2 +      2*b^2*c^6*y*z^2 - c^8*y*z^2");
		exp("K482","(-a^4)*b^2*c^2*x^2*y - a^2*b^4*c^2*x^2*y + a^4*c^4*x^2*y + 2*a^2*b^2*c^4*x^2*y +      2*b^4*c^4*x^2*y - a^2*c^6*x^2*y - 3*b^2*c^6*x^2*y + c^8*x^2*y + a^4*b^2*c^2*x*y^2 +      a^2*b^4*c^2*x*y^2 - 2*a^4*c^4*x*y^2 - 2*a^2*b^2*c^4*x*y^2 - b^4*c^4*x*y^2 + 3*a^2*c^6*x*y^2 +      b^2*c^6*x*y^2 - c^8*x*y^2 - a^4*b^4*x^2*z + a^2*b^6*x^2*z - b^8*x^2*z + a^4*b^2*c^2*x^2*z -      2*a^2*b^4*c^2*x^2*z + 3*b^6*c^2*x^2*z + a^2*b^2*c^4*x^2*z - 2*b^4*c^4*x^2*z + a^6*b^2*x*y*z -      a^2*b^6*x*y*z - a^6*c^2*x*y*z + b^6*c^2*x*y*z + a^2*c^6*x*y*z - b^2*c^6*x*y*z + a^8*y^2*z -      a^6*b^2*y^2*z + a^4*b^4*y^2*z - 3*a^6*c^2*y^2*z + 2*a^4*b^2*c^2*y^2*z - a^2*b^4*c^2*y^2*z +      2*a^4*c^4*y^2*z - a^2*b^2*c^4*y^2*z + 2*a^4*b^4*x*z^2 - 3*a^2*b^6*x*z^2 + b^8*x*z^2 -      a^4*b^2*c^2*x*z^2 + 2*a^2*b^4*c^2*x*z^2 - b^6*c^2*x*z^2 - a^2*b^2*c^4*x*z^2 + b^4*c^4*x*z^2 -      a^8*y*z^2 + 3*a^6*b^2*y*z^2 - 2*a^4*b^4*y*z^2 + a^6*c^2*y*z^2 - 2*a^4*b^2*c^2*y*z^2 +      a^2*b^4*c^2*y*z^2 - a^4*c^4*y*z^2 + a^2*b^2*c^4*y*z^2");
		exp("K483","(-a^6)*c^2*x^2*y + a^4*b^2*c^2*x^2*y + a^2*b^4*c^2*x^2*y - b^6*c^2*x^2*y + 2*a^4*c^4*x^2*y -      2*a^2*b^2*c^4*x^2*y + b^4*c^4*x^2*y - 2*a^2*c^6*x^2*y + c^8*x^2*y + a^6*c^2*x*y^2 -      a^4*b^2*c^2*x*y^2 - a^2*b^4*c^2*x*y^2 + b^6*c^2*x*y^2 - a^4*c^4*x*y^2 + 2*a^2*b^2*c^4*x*y^2 -      2*b^4*c^4*x*y^2 + 2*b^2*c^6*x*y^2 - c^8*x*y^2 + a^6*b^2*x^2*z - 2*a^4*b^4*x^2*z + 2*a^2*b^6*x^2*z -      b^8*x^2*z - a^4*b^2*c^2*x^2*z + 2*a^2*b^4*c^2*x^2*z - a^2*b^2*c^4*x^2*z - b^4*c^4*x^2*z +      b^2*c^6*x^2*z - a^6*b^2*x*y*z + a^2*b^6*x*y*z + a^6*c^2*x*y*z - b^6*c^2*x*y*z - a^2*c^6*x*y*z +      b^2*c^6*x*y*z + a^8*y^2*z - 2*a^6*b^2*y^2*z + 2*a^4*b^4*y^2*z - a^2*b^6*y^2*z -      2*a^4*b^2*c^2*y^2*z + a^2*b^4*c^2*y^2*z + a^4*c^4*y^2*z + a^2*b^2*c^4*y^2*z - a^2*c^6*y^2*z -      a^6*b^2*x*z^2 + a^4*b^4*x*z^2 + b^8*x*z^2 + a^4*b^2*c^2*x*z^2 - 2*a^2*b^4*c^2*x*z^2 -      2*b^6*c^2*x*z^2 + a^2*b^2*c^4*x*z^2 + 2*b^4*c^4*x*z^2 - b^2*c^6*x*z^2 - a^8*y*z^2 - a^4*b^4*y*z^2 +      a^2*b^6*y*z^2 + 2*a^6*c^2*y*z^2 + 2*a^4*b^2*c^2*y*z^2 - a^2*b^4*c^2*y*z^2 - 2*a^4*c^4*y*z^2 -      a^2*b^2*c^4*y*z^2 + a^2*c^6*y*z^2");
		exp("K484","(-a^6)*c^2*x^2*y + 2*a^4*b^2*c^2*x^2*y + 2*a^2*b^4*c^2*x^2*y - b^6*c^2*x^2*y + a^4*c^4*x^2*y -      4*a^2*b^2*c^4*x^2*y - b^4*c^4*x^2*y - a^2*c^6*x^2*y + 3*b^2*c^6*x^2*y + a^6*c^2*x*y^2 -      2*a^4*b^2*c^2*x*y^2 - 2*a^2*b^4*c^2*x*y^2 + b^6*c^2*x*y^2 + a^4*c^4*x*y^2 + 4*a^2*b^2*c^4*x*y^2 -      b^4*c^4*x*y^2 - 3*a^2*c^6*x*y^2 + b^2*c^6*x*y^2 + a^6*b^2*x^2*z - a^4*b^4*x^2*z + a^2*b^6*x^2*z -      2*a^4*b^2*c^2*x^2*z + 4*a^2*b^4*c^2*x^2*z - 3*b^6*c^2*x^2*z - 2*a^2*b^2*c^4*x^2*z + b^4*c^4*x^2*z +      b^2*c^6*x^2*z - 2*a^6*b^2*x*y*z + 2*a^2*b^6*x*y*z + 2*a^6*c^2*x*y*z - 2*b^6*c^2*x*y*z -      2*a^2*c^6*x*y*z + 2*b^2*c^6*x*y*z - a^6*b^2*y^2*z + a^4*b^4*y^2*z - a^2*b^6*y^2*z +      3*a^6*c^2*y^2*z - 4*a^4*b^2*c^2*y^2*z + 2*a^2*b^4*c^2*y^2*z - a^4*c^4*y^2*z + 2*a^2*b^2*c^4*y^2*z -      a^2*c^6*y^2*z - a^6*b^2*x*z^2 - a^4*b^4*x*z^2 + 3*a^2*b^6*x*z^2 + 2*a^4*b^2*c^2*x*z^2 -      4*a^2*b^4*c^2*x*z^2 - b^6*c^2*x*z^2 + 2*a^2*b^2*c^4*x*z^2 + b^4*c^4*x*z^2 - b^2*c^6*x*z^2 -      3*a^6*b^2*y*z^2 + a^4*b^4*y*z^2 + a^2*b^6*y*z^2 + a^6*c^2*y*z^2 + 4*a^4*b^2*c^2*y*z^2 -      2*a^2*b^4*c^2*y*z^2 - a^4*c^4*y*z^2 - 2*a^2*b^2*c^4*y*z^2 + a^2*c^6*y*z^2");
		exp("K485","2*a^4*x^2*y - 4*a^2*b^2*x^2*y + 2*b^4*x^2*y - a^2*c^2*x^2*y - b^2*c^2*x^2*y - 4*c^4*x^2*y -      2*a^4*x*y^2 + 4*a^2*b^2*x*y^2 - 2*b^4*x*y^2 + a^2*c^2*x*y^2 + b^2*c^2*x*y^2 + 4*c^4*x*y^2 -      2*a^4*x^2*z + a^2*b^2*x^2*z + 4*b^4*x^2*z + 4*a^2*c^2*x^2*z + b^2*c^2*x^2*z - 2*c^4*x^2*z -      4*a^4*y^2*z - a^2*b^2*y^2*z + 2*b^4*y^2*z - a^2*c^2*y^2*z - 4*b^2*c^2*y^2*z + 2*c^4*y^2*z +      2*a^4*x*z^2 - a^2*b^2*x*z^2 - 4*b^4*x*z^2 - 4*a^2*c^2*x*z^2 - b^2*c^2*x*z^2 + 2*c^4*x*z^2 +      4*a^4*y*z^2 + a^2*b^2*y*z^2 - 2*b^4*y*z^2 + a^2*c^2*y*z^2 + 4*b^2*c^2*y*z^2 - 2*c^4*y*z^2");
		exp("K486","(-a^2)*c^4*x^2*y + 2*b^2*c^4*x^2*y + c^6*x^2*y - 2*a^2*c^4*x*y^2 + b^2*c^4*x*y^2 - c^6*x*y^2 +      a^2*b^4*x^2*z - b^6*x^2*z - 2*b^4*c^2*x^2*z - 2*a^4*b^2*x*y*z + 2*a^2*b^4*x*y*z + 2*a^4*c^2*x*y*z -      2*b^4*c^2*x*y*z - 2*a^2*c^4*x*y*z + 2*b^2*c^4*x*y*z + a^6*y^2*z - a^4*b^2*y^2*z + 2*a^4*c^2*y^2*z +      2*a^2*b^4*x*z^2 + b^6*x*z^2 - b^4*c^2*x*z^2 - a^6*y*z^2 - 2*a^4*b^2*y*z^2 + a^4*c^2*y*z^2");
		exp("K487","2*a^6*c^2*x^2*y - 2*a^4*b^2*c^2*x^2*y - 2*a^2*b^4*c^2*x^2*y + 2*b^6*c^2*x^2*y - 5*a^4*c^4*x^2*y +      3*a^2*b^2*c^4*x^2*y + b^4*c^4*x^2*y + a^2*c^6*x^2*y - 2*b^2*c^6*x^2*y + 2*c^8*x^2*y -      2*a^6*c^2*x*y^2 + 2*a^4*b^2*c^2*x*y^2 + 2*a^2*b^4*c^2*x*y^2 - 2*b^6*c^2*x*y^2 - a^4*c^4*x*y^2 -      3*a^2*b^2*c^4*x*y^2 + 5*b^4*c^4*x*y^2 + 2*a^2*c^6*x*y^2 - b^2*c^6*x*y^2 - 2*c^8*x*y^2 -      2*a^6*b^2*x^2*z + 5*a^4*b^4*x^2*z - a^2*b^6*x^2*z - 2*b^8*x^2*z + 2*a^4*b^2*c^2*x^2*z -      3*a^2*b^4*c^2*x^2*z + 2*b^6*c^2*x^2*z + 2*a^2*b^2*c^4*x^2*z - b^4*c^4*x^2*z - 2*b^2*c^6*x^2*z -      3*a^6*b^2*x*y*z + 3*a^2*b^6*x*y*z + 3*a^6*c^2*x*y*z - 3*b^6*c^2*x*y*z - 3*a^2*c^6*x*y*z +      3*b^2*c^6*x*y*z + 2*a^8*y^2*z + a^6*b^2*y^2*z - 5*a^4*b^4*y^2*z + 2*a^2*b^6*y^2*z -      2*a^6*c^2*y^2*z + 3*a^4*b^2*c^2*y^2*z - 2*a^2*b^4*c^2*y^2*z + a^4*c^4*y^2*z - 2*a^2*b^2*c^4*y^2*z +      2*a^2*c^6*y^2*z + 2*a^6*b^2*x*z^2 + a^4*b^4*x*z^2 - 2*a^2*b^6*x*z^2 + 2*b^8*x*z^2 -      2*a^4*b^2*c^2*x*z^2 + 3*a^2*b^4*c^2*x*z^2 + b^6*c^2*x*z^2 - 2*a^2*b^2*c^4*x*z^2 - 5*b^4*c^4*x*z^2 +      2*b^2*c^6*x*z^2 - 2*a^8*y*z^2 + 2*a^6*b^2*y*z^2 - a^4*b^4*y*z^2 - 2*a^2*b^6*y*z^2 - a^6*c^2*y*z^2 -      3*a^4*b^2*c^2*y*z^2 + 2*a^2*b^4*c^2*y*z^2 + 5*a^4*c^4*y*z^2 + 2*a^2*b^2*c^4*y*z^2 -      2*a^2*c^6*y*z^2");
		exp("K488","-2*a^8*c^4*x^2*y - 4*a^6*b^2*c^4*x^2*y - 3*a^4*b^4*c^4*x^2*y + 8*a^2*b^6*c^4*x^2*y +      b^8*c^4*x^2*y + 5*a^6*c^6*x^2*y + 7*a^4*b^2*c^6*x^2*y - 8*a^2*b^4*c^6*x^2*y - 4*b^6*c^6*x^2*y -      3*a^4*c^8*x^2*y + a^2*b^2*c^8*x^2*y + 6*b^4*c^8*x^2*y - a^2*c^10*x^2*y - 4*b^2*c^10*x^2*y +      c^12*x^2*y - a^8*c^4*x*y^2 - 8*a^6*b^2*c^4*x*y^2 + 3*a^4*b^4*c^4*x*y^2 + 4*a^2*b^6*c^4*x*y^2 +      2*b^8*c^4*x*y^2 + 4*a^6*c^6*x*y^2 + 8*a^4*b^2*c^6*x*y^2 - 7*a^2*b^4*c^6*x*y^2 - 5*b^6*c^6*x*y^2 -      6*a^4*c^8*x*y^2 - a^2*b^2*c^8*x*y^2 + 3*b^4*c^8*x*y^2 + 4*a^2*c^10*x*y^2 + b^2*c^10*x*y^2 -      c^12*x*y^2 + 2*a^8*b^4*x^2*z - 5*a^6*b^6*x^2*z + 3*a^4*b^8*x^2*z + a^2*b^10*x^2*z - b^12*x^2*z +      4*a^6*b^4*c^2*x^2*z - 7*a^4*b^6*c^2*x^2*z - a^2*b^8*c^2*x^2*z + 4*b^10*c^2*x^2*z +      3*a^4*b^4*c^4*x^2*z + 8*a^2*b^6*c^4*x^2*z - 6*b^8*c^4*x^2*z - 8*a^2*b^4*c^6*x^2*z +      4*b^6*c^6*x^2*z - b^4*c^8*x^2*z + 3*a^10*b^2*x*y*z - 6*a^8*b^4*x*y*z + 6*a^4*b^8*x*y*z -      3*a^2*b^10*x*y*z - 3*a^10*c^2*x*y*z + 9*a^6*b^4*c^2*x*y*z - 9*a^4*b^6*c^2*x*y*z +      3*b^10*c^2*x*y*z + 6*a^8*c^4*x*y*z - 9*a^6*b^2*c^4*x*y*z + 9*a^2*b^6*c^4*x*y*z - 6*b^8*c^4*x*y*z +      9*a^4*b^2*c^6*x*y*z - 9*a^2*b^4*c^6*x*y*z - 6*a^4*c^8*x*y*z + 6*b^4*c^8*x*y*z + 3*a^2*c^10*x*y*z -      3*b^2*c^10*x*y*z + a^12*y^2*z - a^10*b^2*y^2*z - 3*a^8*b^4*y^2*z + 5*a^6*b^6*y^2*z -      2*a^4*b^8*y^2*z - 4*a^10*c^2*y^2*z + a^8*b^2*c^2*y^2*z + 7*a^6*b^4*c^2*y^2*z -      4*a^4*b^6*c^2*y^2*z + 6*a^8*c^4*y^2*z - 8*a^6*b^2*c^4*y^2*z - 3*a^4*b^4*c^4*y^2*z -      4*a^6*c^6*y^2*z + 8*a^4*b^2*c^6*y^2*z + a^4*c^8*y^2*z + a^8*b^4*x*z^2 - 4*a^6*b^6*x*z^2 +      6*a^4*b^8*x*z^2 - 4*a^2*b^10*x*z^2 + b^12*x*z^2 + 8*a^6*b^4*c^2*x*z^2 - 8*a^4*b^6*c^2*x*z^2 +      a^2*b^8*c^2*x*z^2 - b^10*c^2*x*z^2 - 3*a^4*b^4*c^4*x*z^2 + 7*a^2*b^6*c^4*x*z^2 - 3*b^8*c^4*x*z^2 -      4*a^2*b^4*c^6*x*z^2 + 5*b^6*c^6*x*z^2 - 2*b^4*c^8*x*z^2 - a^12*y*z^2 + 4*a^10*b^2*y*z^2 -      6*a^8*b^4*y*z^2 + 4*a^6*b^6*y*z^2 - a^4*b^8*y*z^2 + a^10*c^2*y*z^2 - a^8*b^2*c^2*y*z^2 +      8*a^6*b^4*c^2*y*z^2 - 8*a^4*b^6*c^2*y*z^2 + 3*a^8*c^4*y*z^2 - 7*a^6*b^2*c^4*y*z^2 +      3*a^4*b^4*c^4*y*z^2 - 5*a^6*c^6*y*z^2 + 4*a^4*b^2*c^6*y*z^2 + 2*a^4*c^8*y*z^2");
		exp("K489","(-a^6)*c^2*x^2*y + a^4*b^2*c^2*x^2*y + a^2*b^4*c^2*x^2*y - b^6*c^2*x^2*y + 2*a^4*c^4*x^2*y -      2*a^2*b^2*c^4*x^2*y + 2*b^4*c^4*x^2*y - a^2*c^6*x^2*y - b^2*c^6*x^2*y + a^6*c^2*x*y^2 -      a^4*b^2*c^2*x*y^2 - a^2*b^4*c^2*x*y^2 + b^6*c^2*x*y^2 - 2*a^4*c^4*x*y^2 + 2*a^2*b^2*c^4*x*y^2 -      2*b^4*c^4*x*y^2 + a^2*c^6*x*y^2 + b^2*c^6*x*y^2 + a^6*b^2*x^2*z - 2*a^4*b^4*x^2*z + a^2*b^6*x^2*z -      a^4*b^2*c^2*x^2*z + 2*a^2*b^4*c^2*x^2*z + b^6*c^2*x^2*z - a^2*b^2*c^4*x^2*z - 2*b^4*c^4*x^2*z +      b^2*c^6*x^2*z - a^6*b^2*y^2*z + 2*a^4*b^4*y^2*z - a^2*b^6*y^2*z - a^6*c^2*y^2*z -      2*a^4*b^2*c^2*y^2*z + a^2*b^4*c^2*y^2*z + 2*a^4*c^4*y^2*z + a^2*b^2*c^4*y^2*z - a^2*c^6*y^2*z -      a^6*b^2*x*z^2 + 2*a^4*b^4*x*z^2 - a^2*b^6*x*z^2 + a^4*b^2*c^2*x*z^2 - 2*a^2*b^4*c^2*x*z^2 -      b^6*c^2*x*z^2 + a^2*b^2*c^4*x*z^2 + 2*b^4*c^4*x*z^2 - b^2*c^6*x*z^2 + a^6*b^2*y*z^2 -      2*a^4*b^4*y*z^2 + a^2*b^6*y*z^2 + a^6*c^2*y*z^2 + 2*a^4*b^2*c^2*y*z^2 - a^2*b^4*c^2*y*z^2 -      2*a^4*c^4*y*z^2 - a^2*b^2*c^4*y*z^2 + a^2*c^6*y*z^2");
		exp("K490","a^8*x^2*y - 4*a^6*b^2*x^2*y + 6*a^4*b^4*x^2*y - 4*a^2*b^6*x^2*y + b^8*x^2*y - a^6*c^2*x^2*y +      2*a^4*b^2*c^2*x^2*y - a^2*b^4*c^2*x^2*y + 2*a^2*b^2*c^4*x^2*y - 2*b^4*c^4*x^2*y - a^2*c^6*x^2*y +      c^8*x^2*y - a^8*x*y^2 + 4*a^6*b^2*x*y^2 - 6*a^4*b^4*x*y^2 + 4*a^2*b^6*x*y^2 - b^8*x*y^2 +      a^4*b^2*c^2*x*y^2 - 2*a^2*b^4*c^2*x*y^2 + b^6*c^2*x*y^2 + 2*a^4*c^4*x*y^2 - 2*a^2*b^2*c^4*x*y^2 +      b^2*c^6*x*y^2 - c^8*x*y^2 - a^8*x^2*z + a^6*b^2*x^2*z + a^2*b^6*x^2*z - b^8*x^2*z +      4*a^6*c^2*x^2*z - 2*a^4*b^2*c^2*x^2*z - 2*a^2*b^4*c^2*x^2*z - 6*a^4*c^4*x^2*z + a^2*b^2*c^4*x^2*z +      2*b^4*c^4*x^2*z + 4*a^2*c^6*x^2*z - c^8*x^2*z + a^8*y^2*z - a^6*b^2*y^2*z - a^2*b^6*y^2*z +      b^8*y^2*z + 2*a^4*b^2*c^2*y^2*z + 2*a^2*b^4*c^2*y^2*z - 4*b^6*c^2*y^2*z - 2*a^4*c^4*y^2*z -      a^2*b^2*c^4*y^2*z + 6*b^4*c^4*y^2*z - 4*b^2*c^6*y^2*z + c^8*y^2*z + a^8*x*z^2 - 2*a^4*b^4*x*z^2 +      b^8*x*z^2 - 4*a^6*c^2*x*z^2 - a^4*b^2*c^2*x*z^2 + 2*a^2*b^4*c^2*x*z^2 - b^6*c^2*x*z^2 +      6*a^4*c^4*x*z^2 + 2*a^2*b^2*c^4*x*z^2 - 4*a^2*c^6*x*z^2 - b^2*c^6*x*z^2 + c^8*x*z^2 - a^8*y*z^2 +      2*a^4*b^4*y*z^2 - b^8*y*z^2 + a^6*c^2*y*z^2 - 2*a^4*b^2*c^2*y*z^2 + a^2*b^4*c^2*y*z^2 +      4*b^6*c^2*y*z^2 - 2*a^2*b^2*c^4*y*z^2 - 6*b^4*c^4*y*z^2 + a^2*c^6*y*z^2 + 4*b^2*c^6*y*z^2 -      c^8*y*z^2");
		exp("K491","(-a^8)*x^2*y + 4*a^6*b^2*x^2*y - 6*a^4*b^4*x^2*y + 4*a^2*b^6*x^2*y - b^8*x^2*y +      2*a^6*c^2*x^2*y - 3*a^4*b^2*c^2*x^2*y + b^6*c^2*x^2*y - 2*a^4*c^4*x^2*y + 2*a^2*c^6*x^2*y +      b^2*c^6*x^2*y - c^8*x^2*y + a^8*x*y^2 - 4*a^6*b^2*x*y^2 + 6*a^4*b^4*x*y^2 - 4*a^2*b^6*x*y^2 +      b^8*x*y^2 - a^6*c^2*x*y^2 + 3*a^2*b^4*c^2*x*y^2 - 2*b^6*c^2*x*y^2 + 2*b^4*c^4*x*y^2 -      a^2*c^6*x*y^2 - 2*b^2*c^6*x*y^2 + c^8*x*y^2 + a^8*x^2*z - 2*a^6*b^2*x^2*z + 2*a^4*b^4*x^2*z -      2*a^2*b^6*x^2*z + b^8*x^2*z - 4*a^6*c^2*x^2*z + 3*a^4*b^2*c^2*x^2*z - b^6*c^2*x^2*z +      6*a^4*c^4*x^2*z - 4*a^2*c^6*x^2*z - b^2*c^6*x^2*z + c^8*x^2*z - a^8*y^2*z + 2*a^6*b^2*y^2*z -      2*a^4*b^4*y^2*z + 2*a^2*b^6*y^2*z - b^8*y^2*z + a^6*c^2*y^2*z - 3*a^2*b^4*c^2*y^2*z +      4*b^6*c^2*y^2*z - 6*b^4*c^4*y^2*z + a^2*c^6*y^2*z + 4*b^2*c^6*y^2*z - c^8*y^2*z - a^8*x*z^2 +      a^6*b^2*x*z^2 + a^2*b^6*x*z^2 - b^8*x*z^2 + 4*a^6*c^2*x*z^2 + 2*b^6*c^2*x*z^2 - 6*a^4*c^4*x*z^2 -      3*a^2*b^2*c^4*x*z^2 - 2*b^4*c^4*x*z^2 + 4*a^2*c^6*x*z^2 + 2*b^2*c^6*x*z^2 - c^8*x*z^2 + a^8*y*z^2 -      a^6*b^2*y*z^2 - a^2*b^6*y*z^2 + b^8*y*z^2 - 2*a^6*c^2*y*z^2 - 4*b^6*c^2*y*z^2 + 2*a^4*c^4*y*z^2 +      3*a^2*b^2*c^4*y*z^2 + 6*b^4*c^4*y*z^2 - 2*a^2*c^6*y*z^2 - 4*b^2*c^6*y*z^2 + c^8*y*z^2");
		exp("K492","(-a^4)*x^2*y + 2*a^2*b^2*x^2*y - b^4*x^2*y + a^2*c^2*x^2*y - c^4*x^2*y + a^4*x*y^2 -      2*a^2*b^2*x*y^2 + b^4*x*y^2 - b^2*c^2*x*y^2 + c^4*x*y^2 + a^4*x^2*z - a^2*b^2*x^2*z + b^4*x^2*z -      2*a^2*c^2*x^2*z + c^4*x^2*z - a^4*y^2*z + a^2*b^2*y^2*z - b^4*y^2*z + 2*b^2*c^2*y^2*z - c^4*y^2*z -      a^4*x*z^2 - b^4*x*z^2 + 2*a^2*c^2*x*z^2 + b^2*c^2*x*z^2 - c^4*x*z^2 + a^4*y*z^2 + b^4*y*z^2 -      a^2*c^2*y*z^2 - 2*b^2*c^2*y*z^2 + c^4*y*z^2");
		exp("K493","(-a^6)*c^4*x^2*y + a^4*b^2*c^4*x^2*y + 2*a^4*c^6*x^2*y - a^2*b^2*c^6*x^2*y - a^2*c^8*x^2*y -      a^2*b^4*c^4*x*y^2 + b^6*c^4*x*y^2 + a^2*b^2*c^6*x*y^2 - 2*b^4*c^6*x*y^2 + b^2*c^8*x*y^2 +      a^6*b^4*x^2*z - 2*a^4*b^6*x^2*z + a^2*b^8*x^2*z - a^4*b^4*c^2*x^2*z + a^2*b^6*c^2*x^2*z +      a^8*b^2*x*y*z - 3*a^6*b^4*x*y*z + 3*a^4*b^6*x*y*z - a^2*b^8*x*y*z - a^8*c^2*x*y*z + b^8*c^2*x*y*z +      3*a^6*c^4*x*y*z - 3*b^6*c^4*x*y*z - 3*a^4*c^6*x*y*z + 3*b^4*c^6*x*y*z + a^2*c^8*x*y*z -      b^2*c^8*x*y*z - a^8*b^2*y^2*z + 2*a^6*b^4*y^2*z - a^4*b^6*y^2*z - a^6*b^2*c^2*y^2*z +      a^4*b^4*c^2*y^2*z - a^2*b^6*c^2*x*z^2 - b^8*c^2*x*z^2 + a^2*b^4*c^4*x*z^2 + 2*b^6*c^4*x*z^2 -      b^4*c^6*x*z^2 + a^8*c^2*y*z^2 + a^6*b^2*c^2*y*z^2 - 2*a^6*c^4*y*z^2 - a^4*b^2*c^4*y*z^2 +      a^4*c^6*y*z^2");
		exp("K494","a^6*c^4*x^2*y + a^4*b^2*c^4*x^2*y - 2*a^4*c^6*x^2*y - a^2*b^2*c^6*x^2*y + a^2*c^8*x^2*y -      a^2*b^4*c^4*x*y^2 - b^6*c^4*x*y^2 + a^2*b^2*c^6*x*y^2 + 2*b^4*c^6*x*y^2 - b^2*c^8*x*y^2 -      a^6*b^4*x^2*z + 2*a^4*b^6*x^2*z - a^2*b^8*x^2*z - a^4*b^4*c^2*x^2*z + a^2*b^6*c^2*x^2*z +      a^8*b^2*x*y*z - 3*a^6*b^4*x*y*z + 3*a^4*b^6*x*y*z - a^2*b^8*x*y*z - a^8*c^2*x*y*z + b^8*c^2*x*y*z +      3*a^6*c^4*x*y*z - 3*b^6*c^4*x*y*z - 3*a^4*c^6*x*y*z + 3*b^4*c^6*x*y*z + a^2*c^8*x*y*z -      b^2*c^8*x*y*z + a^8*b^2*y^2*z - 2*a^6*b^4*y^2*z + a^4*b^6*y^2*z - a^6*b^2*c^2*y^2*z +      a^4*b^4*c^2*y^2*z - a^2*b^6*c^2*x*z^2 + b^8*c^2*x*z^2 + a^2*b^4*c^4*x*z^2 - 2*b^6*c^4*x*z^2 +      b^4*c^6*x*z^2 - a^8*c^2*y*z^2 + a^6*b^2*c^2*y*z^2 + 2*a^6*c^4*y*z^2 - a^4*b^2*c^4*y*z^2 -      a^4*c^6*y*z^2");
		exp("K495","(-a^4)*c^4*x^2*y - a^2*b^2*c^4*x^2*y + 2*b^4*c^4*x^2*y + 2*a^2*c^6*x^2*y - b^2*c^6*x^2*y -      c^8*x^2*y - 2*a^4*c^4*x*y^2 + a^2*b^2*c^4*x*y^2 + b^4*c^4*x*y^2 + a^2*c^6*x*y^2 - 2*b^2*c^6*x*y^2 +      c^8*x*y^2 + a^4*b^4*x^2*z - 2*a^2*b^6*x^2*z + b^8*x^2*z + a^2*b^4*c^2*x^2*z + b^6*c^2*x^2*z -      2*b^4*c^4*x^2*z - a^8*y^2*z + 2*a^6*b^2*y^2*z - a^4*b^4*y^2*z - a^6*c^2*y^2*z - a^4*b^2*c^2*y^2*z +      2*a^4*c^4*y^2*z + 2*a^4*b^4*x*z^2 - a^2*b^6*x*z^2 - b^8*x*z^2 - a^2*b^4*c^2*x*z^2 +      2*b^6*c^2*x*z^2 - b^4*c^4*x*z^2 + a^8*y*z^2 + a^6*b^2*y*z^2 - 2*a^4*b^4*y*z^2 - 2*a^6*c^2*y*z^2 +      a^4*b^2*c^2*y*z^2 + a^4*c^4*y*z^2");
		exp("K496","a^6*c^2*x^2*y - 3*a^4*b^2*c^2*x^2*y + 3*a^2*b^4*c^2*x^2*y - b^6*c^2*x^2*y + 2*a^2*b^2*c^4*x^2*y -      2*b^4*c^4*x^2*y - 3*a^2*c^6*x^2*y + b^2*c^6*x^2*y + 2*c^8*x^2*y + a^6*c^2*x*y^2 -      3*a^4*b^2*c^2*x*y^2 + 3*a^2*b^4*c^2*x*y^2 - b^6*c^2*x*y^2 + 2*a^4*c^4*x*y^2 - 2*a^2*b^2*c^4*x*y^2 -      a^2*c^6*x*y^2 + 3*b^2*c^6*x*y^2 - 2*c^8*x*y^2 - a^6*b^2*x^2*z + 3*a^2*b^6*x^2*z - 2*b^8*x^2*z +      3*a^4*b^2*c^2*x^2*z - 2*a^2*b^4*c^2*x^2*z - b^6*c^2*x^2*z - 3*a^2*b^2*c^4*x^2*z + 2*b^4*c^4*x^2*z +      b^2*c^6*x^2*z + 2*a^8*y^2*z - 3*a^6*b^2*y^2*z + a^2*b^6*y^2*z + a^6*c^2*y^2*z +      2*a^4*b^2*c^2*y^2*z - 3*a^2*b^4*c^2*y^2*z - 2*a^4*c^4*y^2*z + 3*a^2*b^2*c^4*y^2*z - a^2*c^6*y^2*z -      a^6*b^2*x*z^2 - 2*a^4*b^4*x*z^2 + a^2*b^6*x*z^2 + 2*b^8*x*z^2 + 3*a^4*b^2*c^2*x*z^2 +      2*a^2*b^4*c^2*x*z^2 - 3*b^6*c^2*x*z^2 - 3*a^2*b^2*c^4*x*z^2 + b^2*c^6*x*z^2 - 2*a^8*y*z^2 -      a^6*b^2*y*z^2 + 2*a^4*b^4*y*z^2 + a^2*b^6*y*z^2 + 3*a^6*c^2*y*z^2 - 2*a^4*b^2*c^2*y*z^2 -      3*a^2*b^4*c^2*y*z^2 + 3*a^2*b^2*c^4*y*z^2 - a^2*c^6*y*z^2");
		exp("K497","(-a^6)*c^2*x^2*y + 3*a^4*b^2*c^2*x^2*y - 3*a^2*b^4*c^2*x^2*y + b^6*c^2*x^2*y + a^4*c^4*x^2*y -      a^2*b^2*c^4*x^2*y + a^2*c^6*x^2*y - c^8*x^2*y - a^6*c^2*x*y^2 + 3*a^4*b^2*c^2*x*y^2 -      3*a^2*b^4*c^2*x*y^2 + b^6*c^2*x*y^2 + a^2*b^2*c^4*x*y^2 - b^4*c^4*x*y^2 - b^2*c^6*x*y^2 +      c^8*x*y^2 + a^6*b^2*x^2*z - a^4*b^4*x^2*z - a^2*b^6*x^2*z + b^8*x^2*z - 3*a^4*b^2*c^2*x^2*z +      a^2*b^4*c^2*x^2*z + 3*a^2*b^2*c^4*x^2*z - b^2*c^6*x^2*z - a^8*y^2*z + a^6*b^2*y^2*z +      a^4*b^4*y^2*z - a^2*b^6*y^2*z - a^4*b^2*c^2*y^2*z + 3*a^2*b^4*c^2*y^2*z - 3*a^2*b^2*c^4*y^2*z +      a^2*c^6*y^2*z + a^6*b^2*x*z^2 - b^8*x*z^2 - 3*a^4*b^2*c^2*x*z^2 - a^2*b^4*c^2*x*z^2 +      b^6*c^2*x*z^2 + 3*a^2*b^2*c^4*x*z^2 + b^4*c^4*x*z^2 - b^2*c^6*x*z^2 + a^8*y*z^2 - a^2*b^6*y*z^2 -      a^6*c^2*y*z^2 + a^4*b^2*c^2*y*z^2 + 3*a^2*b^4*c^2*y*z^2 - a^4*c^4*y*z^2 - 3*a^2*b^2*c^4*y*z^2 +      a^2*c^6*y*z^2");
		exp("K498","(-a^2)*c^3*x^2*y + b^2*c^3*x^2*y - b*c^4*x^2*y + c^5*x^2*y - a^2*c^3*x*y^2 + b^2*c^3*x*y^2 +      a*c^4*x*y^2 - c^5*x*y^2 + a^2*b^3*x^2*z - b^5*x^2*z + b^4*c*x^2*z - b^3*c^2*x^2*z - a^4*b*x*y*z -      a^3*b^2*x*y*z + a^2*b^3*x*y*z + a*b^4*x*y*z + a^4*c*x*y*z - b^4*c*x*y*z + a^3*c^2*x*y*z -      b^3*c^2*x*y*z - a^2*c^3*x*y*z + b^2*c^3*x*y*z - a*c^4*x*y*z + b*c^4*x*y*z + a^5*y^2*z -      a^3*b^2*y^2*z - a^4*c*y^2*z + a^3*c^2*y^2*z + a^2*b^3*x*z^2 - a*b^4*x*z^2 + b^5*x*z^2 -      b^3*c^2*x*z^2 - a^5*y*z^2 + a^4*b*y*z^2 - a^3*b^2*y*z^2 + a^3*c^2*y*z^2");
		exp("K499","a^6*c^2*x^2*y - b^6*c^2*x^2*y - a^4*c^4*x^2*y + a^2*b^2*c^4*x^2*y + 3*b^4*c^4*x^2*y -      a^2*c^6*x^2*y - 3*b^2*c^6*x^2*y + c^8*x^2*y + a^6*c^2*x*y^2 - b^6*c^2*x*y^2 - 3*a^4*c^4*x*y^2 -      a^2*b^2*c^4*x*y^2 + b^4*c^4*x*y^2 + 3*a^2*c^6*x*y^2 + b^2*c^6*x*y^2 - c^8*x*y^2 - a^6*b^2*x^2*z +      a^4*b^4*x^2*z + a^2*b^6*x^2*z - b^8*x^2*z - a^2*b^4*c^2*x^2*z + 3*b^6*c^2*x^2*z - 3*b^4*c^4*x^2*z +      b^2*c^6*x^2*z + a^8*y^2*z - a^6*b^2*y^2*z - a^4*b^4*y^2*z + a^2*b^6*y^2*z - 3*a^6*c^2*y^2*z +      a^4*b^2*c^2*y^2*z + 3*a^4*c^4*y^2*z - a^2*c^6*y^2*z - a^6*b^2*x*z^2 + 3*a^4*b^4*x*z^2 -      3*a^2*b^6*x*z^2 + b^8*x*z^2 + a^2*b^4*c^2*x*z^2 - b^6*c^2*x*z^2 - b^4*c^4*x*z^2 + b^2*c^6*x*z^2 -      a^8*y*z^2 + 3*a^6*b^2*y*z^2 - 3*a^4*b^4*y*z^2 + a^2*b^6*y*z^2 + a^6*c^2*y*z^2 - a^4*b^2*c^2*y*z^2 +      a^4*c^4*y*z^2 - a^2*c^6*y*z^2");
		exp("K500","(-a^8)*b^2*c^4*x^2*y + a^6*b^4*c^4*x^2*y + a^2*b^8*c^4*x^2*y - b^10*c^4*x^2*y +      2*a^6*b^2*c^6*x^2*y + 4*b^8*c^6*x^2*y - 2*a^4*b^2*c^8*x^2*y - 3*a^2*b^4*c^8*x^2*y -      6*b^6*c^8*x^2*y + 2*a^2*b^2*c^10*x^2*y + 4*b^4*c^10*x^2*y - b^2*c^12*x^2*y + a^10*c^4*x*y^2 -      a^8*b^2*c^4*x*y^2 - a^4*b^6*c^4*x*y^2 + a^2*b^8*c^4*x*y^2 - 4*a^8*c^6*x*y^2 - 2*a^2*b^6*c^6*x*y^2 +      6*a^6*c^8*x*y^2 + 3*a^4*b^2*c^8*x*y^2 + 2*a^2*b^4*c^8*x*y^2 - 4*a^4*c^10*x*y^2 -      2*a^2*b^2*c^10*x*y^2 + a^2*c^12*x*y^2 + a^8*b^4*c^2*x^2*z - 2*a^6*b^6*c^2*x^2*z +      2*a^4*b^8*c^2*x^2*z - 2*a^2*b^10*c^2*x^2*z + b^12*c^2*x^2*z - a^6*b^4*c^4*x^2*z +      3*a^2*b^8*c^4*x^2*z - 4*b^10*c^4*x^2*z + 6*b^8*c^6*x^2*z - a^2*b^4*c^8*x^2*z - 4*b^6*c^8*x^2*z +      b^4*c^10*x^2*z - a^12*c^2*y^2*z + 2*a^10*b^2*c^2*y^2*z - 2*a^8*b^4*c^2*y^2*z +      2*a^6*b^6*c^2*y^2*z - a^4*b^8*c^2*y^2*z + 4*a^10*c^4*y^2*z - 3*a^8*b^2*c^4*y^2*z +      a^4*b^6*c^4*y^2*z - 6*a^8*c^6*y^2*z + 4*a^6*c^8*y^2*z + a^4*b^2*c^8*y^2*z - a^4*c^10*y^2*z -      a^10*b^4*x*z^2 + 4*a^8*b^6*x*z^2 - 6*a^6*b^8*x*z^2 + 4*a^4*b^10*x*z^2 - a^2*b^12*x*z^2 +      a^8*b^4*c^2*x*z^2 - 3*a^4*b^8*c^2*x*z^2 + 2*a^2*b^10*c^2*x*z^2 - 2*a^2*b^8*c^4*x*z^2 +      a^4*b^4*c^6*x*z^2 + 2*a^2*b^6*c^6*x*z^2 - a^2*b^4*c^8*x*z^2 + a^12*b^2*y*z^2 - 4*a^10*b^4*y*z^2 +      6*a^8*b^6*y*z^2 - 4*a^6*b^8*y*z^2 + a^4*b^10*y*z^2 - 2*a^10*b^2*c^2*y*z^2 + 3*a^8*b^4*c^2*y*z^2 -      a^4*b^8*c^2*y*z^2 + 2*a^8*b^2*c^4*y*z^2 - 2*a^6*b^2*c^6*y*z^2 - a^4*b^4*c^6*y*z^2 +      a^4*b^2*c^8*y*z^2");
		exp("K501","(-a^8)*x^2*y + 3*a^6*b^2*x^2*y - 3*a^4*b^4*x^2*y + a^2*b^6*x^2*y + 2*a^6*c^2*x^2*y -      4*a^4*b^2*c^2*x^2*y + 3*a^2*b^4*c^2*x^2*y - b^6*c^2*x^2*y + a^2*b^2*c^4*x^2*y - 2*a^2*c^6*x^2*y +      c^8*x^2*y - a^6*b^2*x*y^2 + 3*a^4*b^4*x*y^2 - 3*a^2*b^6*x*y^2 + b^8*x*y^2 + a^6*c^2*x*y^2 -      3*a^4*b^2*c^2*x*y^2 + 4*a^2*b^4*c^2*x*y^2 - 2*b^6*c^2*x*y^2 - a^2*b^2*c^4*x*y^2 + 2*b^2*c^6*x*y^2 -      c^8*x*y^2 + a^8*x^2*z - 2*a^6*b^2*x^2*z + 2*a^2*b^6*x^2*z - b^8*x^2*z - 3*a^6*c^2*x^2*z +      4*a^4*b^2*c^2*x^2*z - a^2*b^4*c^2*x^2*z + 3*a^4*c^4*x^2*z - 3*a^2*b^2*c^4*x^2*z - a^2*c^6*x^2*z +      b^2*c^6*x^2*z + a^8*y^2*z - 2*a^6*b^2*y^2*z + 2*a^2*b^6*y^2*z - b^8*y^2*z + a^4*b^2*c^2*y^2*z -      4*a^2*b^4*c^2*y^2*z + 3*b^6*c^2*y^2*z + 3*a^2*b^2*c^4*y^2*z - 3*b^4*c^4*y^2*z - a^2*c^6*y^2*z +      b^2*c^6*y^2*z - a^6*b^2*x*z^2 + b^8*x*z^2 + a^6*c^2*x*z^2 + 3*a^4*b^2*c^2*x*z^2 +      a^2*b^4*c^2*x*z^2 - 2*b^6*c^2*x*z^2 - 3*a^4*c^4*x*z^2 - 4*a^2*b^2*c^4*x*z^2 + 3*a^2*c^6*x*z^2 +      2*b^2*c^6*x*z^2 - c^8*x*z^2 - a^8*y*z^2 + a^2*b^6*y*z^2 + 2*a^6*c^2*y*z^2 - a^4*b^2*c^2*y*z^2 -      3*a^2*b^4*c^2*y*z^2 - b^6*c^2*y*z^2 + 4*a^2*b^2*c^4*y*z^2 + 3*b^4*c^4*y*z^2 - 2*a^2*c^6*y*z^2 -      3*b^2*c^6*y*z^2 + c^8*y*z^2");
		exp("K502","(-a^6)*c^2*x^2*y + 3*a^2*b^4*c^2*x^2*y - 2*b^6*c^2*x^2*y + a^4*c^4*x^2*y + 2*a^2*b^2*c^4*x^2*y +      3*b^4*c^4*x^2*y + a^2*c^6*x^2*y - c^8*x^2*y + 2*a^6*c^2*x*y^2 - 3*a^4*b^2*c^2*x*y^2 +      b^6*c^2*x*y^2 - 3*a^4*c^4*x*y^2 - 2*a^2*b^2*c^4*x*y^2 - b^4*c^4*x*y^2 - b^2*c^6*x*y^2 + c^8*x*y^2 +      a^6*b^2*x^2*z - a^4*b^4*x^2*z - a^2*b^6*x^2*z + b^8*x^2*z - 2*a^2*b^4*c^2*x^2*z -      3*a^2*b^2*c^4*x^2*z - 3*b^4*c^4*x^2*z + 2*b^2*c^6*x^2*z - a^8*y^2*z + a^6*b^2*y^2*z +      a^4*b^4*y^2*z - a^2*b^6*y^2*z + 2*a^4*b^2*c^2*y^2*z + 3*a^4*c^4*y^2*z + 3*a^2*b^2*c^4*y^2*z -      2*a^2*c^6*y^2*z - 2*a^6*b^2*x*z^2 + 3*a^4*b^4*x*z^2 - b^8*x*z^2 + 3*a^4*b^2*c^2*x*z^2 +      2*a^2*b^4*c^2*x*z^2 + b^6*c^2*x*z^2 + b^4*c^4*x*z^2 - b^2*c^6*x*z^2 + a^8*y*z^2 - 3*a^4*b^4*y*z^2 +      2*a^2*b^6*y*z^2 - a^6*c^2*y*z^2 - 2*a^4*b^2*c^2*y*z^2 - 3*a^2*b^4*c^2*y*z^2 - a^4*c^4*y*z^2 +      a^2*c^6*y*z^2");
		exp("K503","a^6*c^4*x^2*y + a^4*b^2*c^4*x^2*y - 2*a^2*b^4*c^4*x^2*y + 3*a^2*b^2*c^6*x^2*y + 2*b^4*c^6*x^2*y -      3*a^2*c^8*x^2*y - 4*b^2*c^8*x^2*y + 2*c^10*x^2*y + 2*a^4*b^2*c^4*x*y^2 - a^2*b^4*c^4*x*y^2 -      b^6*c^4*x*y^2 - 2*a^4*c^6*x*y^2 - 3*a^2*b^2*c^6*x*y^2 + 4*a^2*c^8*x*y^2 + 3*b^2*c^8*x*y^2 -      2*c^10*x*y^2 - a^6*b^4*x^2*z + 3*a^2*b^8*x^2*z - 2*b^10*x^2*z - a^4*b^4*c^2*x^2*z -      3*a^2*b^6*c^2*x^2*z + 4*b^8*c^2*x^2*z + 2*a^2*b^4*c^4*x^2*z - 2*b^6*c^4*x^2*z - a^8*b^2*x*y*z +      3*a^6*b^4*x*y*z - 3*a^4*b^6*x*y*z + a^2*b^8*x*y*z + a^8*c^2*x*y*z - b^8*c^2*x*y*z -      3*a^6*c^4*x*y*z + 3*b^6*c^4*x*y*z + 3*a^4*c^6*x*y*z - 3*b^4*c^6*x*y*z - a^2*c^8*x*y*z +      b^2*c^8*x*y*z + 2*a^10*y^2*z - 3*a^8*b^2*y^2*z + a^4*b^6*y^2*z - 4*a^8*c^2*y^2*z +      3*a^6*b^2*c^2*y^2*z + a^4*b^4*c^2*y^2*z + 2*a^6*c^4*y^2*z - 2*a^4*b^2*c^4*y^2*z + 2*a^4*b^6*x*z^2 -      4*a^2*b^8*x*z^2 + 2*b^10*x*z^2 - 2*a^4*b^4*c^2*x*z^2 + 3*a^2*b^6*c^2*x*z^2 - 3*b^8*c^2*x*z^2 +      a^2*b^4*c^4*x*z^2 + b^4*c^6*x*z^2 - 2*a^10*y*z^2 + 4*a^8*b^2*y*z^2 - 2*a^6*b^4*y*z^2 +      3*a^8*c^2*y*z^2 - 3*a^6*b^2*c^2*y*z^2 + 2*a^4*b^4*c^2*y*z^2 - a^4*b^2*c^4*y*z^2 - a^4*c^6*y*z^2 ");
		exp("K504","-2*a^8*c^2*x^2*y + 4*a^6*b^2*c^2*x^2*y - 2*a^4*b^4*c^2*x^2*y + 3*a^6*c^4*x^2*y -      3*a^4*b^2*c^4*x^2*y + 2*a^2*b^4*c^4*x^2*y - a^2*b^2*c^6*x^2*y - a^2*c^8*x^2*y +      2*a^4*b^4*c^2*x*y^2 - 4*a^2*b^6*c^2*x*y^2 + 2*b^8*c^2*x*y^2 - 2*a^4*b^2*c^4*x*y^2 +      3*a^2*b^4*c^4*x*y^2 - 3*b^6*c^4*x*y^2 + a^2*b^2*c^6*x*y^2 + b^2*c^8*x*y^2 + 2*a^8*b^2*x^2*z -      3*a^6*b^4*x^2*z + a^2*b^8*x^2*z - 4*a^6*b^2*c^2*x^2*z + 3*a^4*b^4*c^2*x^2*z + a^2*b^6*c^2*x^2*z +      2*a^4*b^2*c^4*x^2*z - 2*a^2*b^4*c^4*x^2*z - a^8*b^2*x*y*z + 3*a^6*b^4*x*y*z - 3*a^4*b^6*x*y*z +      a^2*b^8*x*y*z + a^8*c^2*x*y*z - b^8*c^2*x*y*z - 3*a^6*c^4*x*y*z + 3*b^6*c^4*x*y*z +      3*a^4*c^6*x*y*z - 3*b^4*c^6*x*y*z - a^2*c^8*x*y*z + b^2*c^8*x*y*z - a^8*b^2*y^2*z +      3*a^4*b^6*y^2*z - 2*a^2*b^8*y^2*z - a^6*b^2*c^2*y^2*z - 3*a^4*b^4*c^2*y^2*z + 4*a^2*b^6*c^2*y^2*z +      2*a^4*b^2*c^4*y^2*z - 2*a^2*b^4*c^4*y^2*z + 2*a^4*b^4*c^2*x*z^2 - a^2*b^6*c^2*x*z^2 -      b^8*c^2*x*z^2 - 2*a^4*b^2*c^4*x*z^2 - 3*a^2*b^4*c^4*x*z^2 + 4*a^2*b^2*c^6*x*z^2 + 3*b^4*c^6*x*z^2 -      2*b^2*c^8*x*z^2 + a^8*c^2*y*z^2 + a^6*b^2*c^2*y*z^2 - 2*a^4*b^4*c^2*y*z^2 + 3*a^4*b^2*c^4*y*z^2 +      2*a^2*b^4*c^4*y*z^2 - 3*a^4*c^6*y*z^2 - 4*a^2*b^2*c^6*y*z^2 + 2*a^2*c^8*y*z^2");
		exp("K505","(-a^10)*b^2*c^2*x^2*y + 5*a^8*b^4*c^2*x^2*y - 7*a^6*b^6*c^2*x^2*y + a^4*b^8*c^2*x^2*y +      4*a^2*b^10*c^2*x^2*y - 2*b^12*c^2*x^2*y + a^10*c^4*x^2*y - a^8*b^2*c^4*x^2*y - a^6*b^4*c^4*x^2*y +      11*a^4*b^6*c^4*x^2*y - 14*a^2*b^8*c^4*x^2*y + 4*b^10*c^4*x^2*y - 4*a^8*c^6*x^2*y +      2*a^6*b^2*c^6*x^2*y - 10*a^4*b^4*c^6*x^2*y + 11*a^2*b^6*c^6*x^2*y + b^8*c^6*x^2*y +      6*a^6*c^8*x^2*y + 2*a^4*b^2*c^8*x^2*y - a^2*b^4*c^8*x^2*y - 7*b^6*c^8*x^2*y - 4*a^4*c^10*x^2*y -      a^2*b^2*c^10*x^2*y + 5*b^4*c^10*x^2*y + a^2*c^12*x^2*y - b^2*c^12*x^2*y - 2*a^12*c^2*x*y^2 +      4*a^10*b^2*c^2*x*y^2 + a^8*b^4*c^2*x*y^2 - 7*a^6*b^6*c^2*x*y^2 + 5*a^4*b^8*c^2*x*y^2 -      a^2*b^10*c^2*x*y^2 + 4*a^10*c^4*x*y^2 - 14*a^8*b^2*c^4*x*y^2 + 11*a^6*b^4*c^4*x*y^2 -      a^4*b^6*c^4*x*y^2 - a^2*b^8*c^4*x*y^2 + b^10*c^4*x*y^2 + a^8*c^6*x*y^2 + 11*a^6*b^2*c^6*x*y^2 -      10*a^4*b^4*c^6*x*y^2 + 2*a^2*b^6*c^6*x*y^2 - 4*b^8*c^6*x*y^2 - 7*a^6*c^8*x*y^2 -      a^4*b^2*c^8*x*y^2 + 2*a^2*b^4*c^8*x*y^2 + 6*b^6*c^8*x*y^2 + 5*a^4*c^10*x*y^2 - a^2*b^2*c^10*x*y^2 -      4*b^4*c^10*x*y^2 - a^2*c^12*x*y^2 + b^2*c^12*x*y^2 + a^10*b^4*x^2*z - 4*a^8*b^6*x^2*z +      6*a^6*b^8*x^2*z - 4*a^4*b^10*x^2*z + a^2*b^12*x^2*z - a^10*b^2*c^2*x^2*z - a^8*b^4*c^2*x^2*z +      2*a^6*b^6*c^2*x^2*z + 2*a^4*b^8*c^2*x^2*z - a^2*b^10*c^2*x^2*z - b^12*c^2*x^2*z +      5*a^8*b^2*c^4*x^2*z - a^6*b^4*c^4*x^2*z - 10*a^4*b^6*c^4*x^2*z - a^2*b^8*c^4*x^2*z +      5*b^10*c^4*x^2*z - 7*a^6*b^2*c^6*x^2*z + 11*a^4*b^4*c^6*x^2*z + 11*a^2*b^6*c^6*x^2*z -      7*b^8*c^6*x^2*z + a^4*b^2*c^8*x^2*z - 14*a^2*b^4*c^8*x^2*z + b^6*c^8*x^2*z + 4*a^2*b^2*c^10*x^2*z +      4*b^4*c^10*x^2*z - 2*b^2*c^12*x^2*z + 2*a^12*b^2*x*y*z - 6*a^10*b^4*x*y*z + 4*a^8*b^6*x*y*z +      4*a^6*b^8*x*y*z - 6*a^4*b^10*x*y*z + 2*a^2*b^12*x*y*z + 2*a^12*c^2*x*y*z - 4*a^10*b^2*c^2*x*y*z +      8*a^8*b^4*c^2*x*y*z - 12*a^6*b^6*c^2*x*y*z + 8*a^4*b^8*c^2*x*y*z - 4*a^2*b^10*c^2*x*y*z +      2*b^12*c^2*x*y*z - 6*a^10*c^4*x*y*z + 8*a^8*b^2*c^4*x*y*z + 8*a^2*b^8*c^4*x*y*z -      6*b^10*c^4*x*y*z + 4*a^8*c^6*x*y*z - 12*a^6*b^2*c^6*x*y*z - 12*a^2*b^6*c^6*x*y*z +      4*b^8*c^6*x*y*z + 4*a^6*c^8*x*y*z + 8*a^4*b^2*c^8*x*y*z + 8*a^2*b^4*c^8*x*y*z + 4*b^6*c^8*x*y*z -      6*a^4*c^10*x*y*z - 4*a^2*b^2*c^10*x*y*z - 6*b^4*c^10*x*y*z + 2*a^2*c^12*x*y*z + 2*b^2*c^12*x*y*z +      a^12*b^2*y^2*z - 4*a^10*b^4*y^2*z + 6*a^8*b^6*y^2*z - 4*a^6*b^8*y^2*z + a^4*b^10*y^2*z -      a^12*c^2*y^2*z - a^10*b^2*c^2*y^2*z + 2*a^8*b^4*c^2*y^2*z + 2*a^6*b^6*c^2*y^2*z -      a^4*b^8*c^2*y^2*z - a^2*b^10*c^2*y^2*z + 5*a^10*c^4*y^2*z - a^8*b^2*c^4*y^2*z -      10*a^6*b^4*c^4*y^2*z - a^4*b^6*c^4*y^2*z + 5*a^2*b^8*c^4*y^2*z - 7*a^8*c^6*y^2*z +      11*a^6*b^2*c^6*y^2*z + 11*a^4*b^4*c^6*y^2*z - 7*a^2*b^6*c^6*y^2*z + a^6*c^8*y^2*z -      14*a^4*b^2*c^8*y^2*z + a^2*b^4*c^8*y^2*z + 4*a^4*c^10*y^2*z + 4*a^2*b^2*c^10*y^2*z -      2*a^2*c^12*y^2*z - 2*a^12*b^2*x*z^2 + 4*a^10*b^4*x*z^2 + a^8*b^6*x*z^2 - 7*a^6*b^8*x*z^2 +      5*a^4*b^10*x*z^2 - a^2*b^12*x*z^2 + 4*a^10*b^2*c^2*x*z^2 - 14*a^8*b^4*c^2*x*z^2 +      11*a^6*b^6*c^2*x*z^2 - a^4*b^8*c^2*x*z^2 - a^2*b^10*c^2*x*z^2 + b^12*c^2*x*z^2 +      a^8*b^2*c^4*x*z^2 + 11*a^6*b^4*c^4*x*z^2 - 10*a^4*b^6*c^4*x*z^2 + 2*a^2*b^8*c^4*x*z^2 -      4*b^10*c^4*x*z^2 - 7*a^6*b^2*c^6*x*z^2 - a^4*b^4*c^6*x*z^2 + 2*a^2*b^6*c^6*x*z^2 +      6*b^8*c^6*x*z^2 + 5*a^4*b^2*c^8*x*z^2 - a^2*b^4*c^8*x*z^2 - 4*b^6*c^8*x*z^2 - a^2*b^2*c^10*x*z^2 +      b^4*c^10*x*z^2 - a^12*b^2*y*z^2 + 5*a^10*b^4*y*z^2 - 7*a^8*b^6*y*z^2 + a^6*b^8*y*z^2 +      4*a^4*b^10*y*z^2 - 2*a^2*b^12*y*z^2 + a^12*c^2*y*z^2 - a^10*b^2*c^2*y*z^2 - a^8*b^4*c^2*y*z^2 +      11*a^6*b^6*c^2*y*z^2 - 14*a^4*b^8*c^2*y*z^2 + 4*a^2*b^10*c^2*y*z^2 - 4*a^10*c^4*y*z^2 +      2*a^8*b^2*c^4*y*z^2 - 10*a^6*b^4*c^4*y*z^2 + 11*a^4*b^6*c^4*y*z^2 + a^2*b^8*c^4*y*z^2 +      6*a^8*c^6*y*z^2 + 2*a^6*b^2*c^6*y*z^2 - a^4*b^4*c^6*y*z^2 - 7*a^2*b^6*c^6*y*z^2 - 4*a^6*c^8*y*z^2 -      a^4*b^2*c^8*y*z^2 + 5*a^2*b^4*c^8*y*z^2 + a^4*c^10*y*z^2 - a^2*b^2*c^10*y*z^2");
		exp("K506","(-a^5)*b^2*c^2*x^2*y + a^4*b^3*c^2*x^2*y + a*b^6*c^2*x^2*y - b^7*c^2*x^2*y + a^4*b^2*c^3*x^2*y +      2*a^2*b^4*c^3*x^2*y - 4*a*b^5*c^3*x^2*y + b^6*c^3*x^2*y + a^5*c^4*x^2*y - a^4*b*c^4*x^2*y -      a^3*b^2*c^4*x^2*y - a^2*b^3*c^4*x^2*y + 2*a*b^4*c^4*x^2*y - a^4*c^5*x^2*y + 2*a^3*b*c^5*x^2*y -      a^2*b^2*c^5*x^2*y - a^3*c^6*x^2*y - a^2*b*c^6*x^2*y + a*b^2*c^6*x^2*y + b^3*c^6*x^2*y +      a^2*c^7*x^2*y - b^2*c^7*x^2*y - a^7*c^2*x*y^2 + a^6*b*c^2*x*y^2 + a^3*b^4*c^2*x*y^2 -      a^2*b^5*c^2*x*y^2 + a^6*c^3*x*y^2 - 4*a^5*b*c^3*x*y^2 + 2*a^4*b^2*c^3*x*y^2 + a^2*b^4*c^3*x*y^2 +      2*a^4*b*c^4*x*y^2 - a^3*b^2*c^4*x*y^2 - a^2*b^3*c^4*x*y^2 - a*b^4*c^4*x*y^2 + b^5*c^4*x*y^2 -      a^2*b^2*c^5*x*y^2 + 2*a*b^3*c^5*x*y^2 - b^4*c^5*x*y^2 + a^3*c^6*x*y^2 + a^2*b*c^6*x*y^2 -      a*b^2*c^6*x*y^2 - b^3*c^6*x*y^2 - a^2*c^7*x*y^2 + b^2*c^7*x*y^2 + a^5*b^4*x^2*z - a^4*b^5*x^2*z -      a^3*b^6*x^2*z + a^2*b^7*x^2*z - a^4*b^4*c*x^2*z + 2*a^3*b^5*c*x^2*z - a^2*b^6*c*x^2*z -      a^5*b^2*c^2*x^2*z + a^4*b^3*c^2*x^2*z - a^3*b^4*c^2*x^2*z - a^2*b^5*c^2*x^2*z + a*b^6*c^2*x^2*z -      b^7*c^2*x^2*z + a^4*b^2*c^3*x^2*z - a^2*b^4*c^3*x^2*z + b^6*c^3*x^2*z + 2*a^2*b^3*c^4*x^2*z +      2*a*b^4*c^4*x^2*z - 4*a*b^3*c^5*x^2*z + a*b^2*c^6*x^2*z + b^3*c^6*x^2*z - b^2*c^7*x^2*z +      a^7*b^2*x*y*z - a^6*b^3*x*y*z - a^3*b^6*x*y*z + a^2*b^7*x*y*z - a^6*b^2*c*x*y*z +      2*a^5*b^3*c*x*y*z - 2*a^4*b^4*c*x*y*z + 2*a^3*b^5*c*x*y*z - a^2*b^6*c*x*y*z + a^7*c^2*x*y*z -      a^6*b*c^2*x*y*z + 4*a^5*b^2*c^2*x*y*z - 2*a^4*b^3*c^2*x*y*z - 2*a^3*b^4*c^2*x*y*z +      4*a^2*b^5*c^2*x*y*z - a*b^6*c^2*x*y*z + b^7*c^2*x*y*z - a^6*c^3*x*y*z + 2*a^5*b*c^3*x*y*z -      2*a^4*b^2*c^3*x*y*z - 2*a^2*b^4*c^3*x*y*z + 2*a*b^5*c^3*x*y*z - b^6*c^3*x*y*z - 2*a^4*b*c^4*x*y*z -      2*a^3*b^2*c^4*x*y*z - 2*a^2*b^3*c^4*x*y*z - 2*a*b^4*c^4*x*y*z + 2*a^3*b*c^5*x*y*z +      4*a^2*b^2*c^5*x*y*z + 2*a*b^3*c^5*x*y*z - a^3*c^6*x*y*z - a^2*b*c^6*x*y*z - a*b^2*c^6*x*y*z -      b^3*c^6*x*y*z + a^2*c^7*x*y*z + b^2*c^7*x*y*z + a^7*b^2*y^2*z - a^6*b^3*y^2*z - a^5*b^4*y^2*z +      a^4*b^5*y^2*z - a^6*b^2*c*y^2*z + 2*a^5*b^3*c*y^2*z - a^4*b^4*c*y^2*z - a^7*c^2*y^2*z +      a^6*b*c^2*y^2*z - a^5*b^2*c^2*y^2*z - a^4*b^3*c^2*y^2*z + a^3*b^4*c^2*y^2*z - a^2*b^5*c^2*y^2*z +      a^6*c^3*y^2*z - a^4*b^2*c^3*y^2*z + a^2*b^4*c^3*y^2*z + 2*a^4*b*c^4*y^2*z + 2*a^3*b^2*c^4*y^2*z -      4*a^3*b*c^5*y^2*z + a^3*c^6*y^2*z + a^2*b*c^6*y^2*z - a^2*c^7*y^2*z - a^7*b^2*x*z^2 +      a^6*b^3*x*z^2 + a^3*b^6*x*z^2 - a^2*b^7*x*z^2 + a^6*b^2*c*x*z^2 - 4*a^5*b^3*c*x*z^2 +      2*a^4*b^4*c*x*z^2 + a^2*b^6*c*x*z^2 + 2*a^4*b^3*c^2*x*z^2 - a^3*b^4*c^2*x*z^2 - a^2*b^5*c^2*x*z^2 -      a*b^6*c^2*x*z^2 + b^7*c^2*x*z^2 - a^2*b^4*c^3*x*z^2 + 2*a*b^5*c^3*x*z^2 - b^6*c^3*x*z^2 +      a^3*b^2*c^4*x*z^2 + a^2*b^3*c^4*x*z^2 - a*b^4*c^4*x*z^2 - b^5*c^4*x*z^2 - a^2*b^2*c^5*x*z^2 +      b^4*c^5*x*z^2 - a^7*b^2*y*z^2 + a^6*b^3*y*z^2 + a^3*b^6*y*z^2 - a^2*b^7*y*z^2 + a^6*b^2*c*y*z^2 +      2*a^4*b^4*c*y*z^2 - 4*a^3*b^5*c*y*z^2 + a^2*b^6*c*y*z^2 + a^7*c^2*y*z^2 - a^6*b*c^2*y*z^2 -      a^5*b^2*c^2*y*z^2 - a^4*b^3*c^2*y*z^2 + 2*a^3*b^4*c^2*y*z^2 - a^6*c^3*y*z^2 + 2*a^5*b*c^3*y*z^2 -      a^4*b^2*c^3*y*z^2 - a^5*c^4*y*z^2 - a^4*b*c^4*y*z^2 + a^3*b^2*c^4*y*z^2 + a^2*b^3*c^4*y*z^2 +      a^4*c^5*y*z^2 - a^2*b^2*c^5*y*z^2");
		exp("K507","a^4*b^5*c^3*x^2*y - a*b^8*c^3*x^2*y - a^5*b^3*c^4*x^2*y + a^3*b^5*c^4*x^2*y - a^4*b^3*c^5*x^2*y +      a^2*b^5*c^5*x^2*y - a^3*b^3*c^6*x^2*y + a*b^5*c^6*x^2*y + a^5*c^7*x^2*y - a^2*b^3*c^7*x^2*y -      a^8*b*c^3*x*y^2 + a^5*b^4*c^3*x*y^2 + a^5*b^3*c^4*x*y^2 - a^3*b^5*c^4*x*y^2 + a^5*b^2*c^5*x*y^2 -      a^3*b^4*c^5*x*y^2 + a^5*b*c^6*x*y^2 - a^3*b^3*c^6*x*y^2 - a^3*b^2*c^7*x*y^2 + b^5*c^7*x*y^2 +      a^5*b^7*x^2*z - a^5*b^4*c^3*x^2*z - a^4*b^5*c^3*x^2*z - a^3*b^6*c^3*x^2*z - a^2*b^7*c^3*x^2*z +      a^4*b^3*c^5*x^2*z + a^3*b^4*c^5*x^2*z + a^2*b^5*c^5*x^2*z + a*b^6*c^5*x^2*z - a*b^3*c^8*x^2*z -      a^7*b^5*x*y*z - a^5*b^7*x*y*z + a^8*b^3*c*x*y*z - a^6*b^5*c*x*y*z - a^5*b^6*c*x*y*z +      a^3*b^8*c*x*y*z + a^7*b^3*c^2*x*y*z - 2*a^5*b^5*c^2*x*y*z + a^3*b^7*c^2*x*y*z + a^8*b*c^3*x*y*z +      a^7*b^2*c^3*x*y*z + 2*a^6*b^3*c^3*x*y*z + 2*a^3*b^6*c^3*x*y*z + a^2*b^7*c^3*x*y*z +      a*b^8*c^3*x*y*z - a^7*c^5*x*y*z - a^6*b*c^5*x*y*z - 2*a^5*b^2*c^5*x*y*z - 2*a^2*b^5*c^5*x*y*z -      a*b^6*c^5*x*y*z - b^7*c^5*x*y*z - a^5*b*c^6*x*y*z + 2*a^3*b^3*c^6*x*y*z - a*b^5*c^6*x*y*z -      a^5*c^7*x*y*z + a^3*b^2*c^7*x*y*z + a^2*b^3*c^7*x*y*z - b^5*c^7*x*y*z + a^3*b*c^8*x*y*z +      a*b^3*c^8*x*y*z + a^7*b^5*y^2*z - a^7*b^2*c^3*y^2*z - a^6*b^3*c^3*y^2*z - a^5*b^4*c^3*y^2*z -      a^4*b^5*c^3*y^2*z + a^6*b*c^5*y^2*z + a^5*b^2*c^5*y^2*z + a^4*b^3*c^5*y^2*z + a^3*b^4*c^5*y^2*z -      a^3*b*c^8*y^2*z - a^8*b^3*c*x*z^2 + a^5*b^6*c*x*z^2 + a^5*b^5*c^2*x*z^2 - a^3*b^7*c^2*x*z^2 +      a^5*b^4*c^3*x*z^2 - a^3*b^6*c^3*x*z^2 + a^5*b^3*c^4*x*z^2 - a^3*b^5*c^4*x*z^2 - a^3*b^4*c^5*x*z^2 +      b^7*c^5*x*z^2 + a^6*b^5*c*y*z^2 - a^3*b^8*c*y*z^2 - a^7*b^3*c^2*y*z^2 + a^5*b^5*c^2*y*z^2 -      a^6*b^3*c^3*y*z^2 + a^4*b^5*c^3*y*z^2 - a^5*b^3*c^4*y*z^2 + a^3*b^5*c^4*y*z^2 + a^7*c^5*y*z^2 -      a^4*b^3*c^5*y*z^2");
		exp("K508","a^6*b^4*x^3 - 3*a^4*b^6*x^3 + 3*a^2*b^8*x^3 - b^10*x^3 - a^4*b^4*c^2*x^3 + a^2*b^6*c^2*x^3 -      a^6*c^4*x^3 + a^4*b^2*c^4*x^3 - b^6*c^4*x^3 + 3*a^4*c^6*x^3 - a^2*b^2*c^6*x^3 + b^4*c^6*x^3 -      3*a^2*c^8*x^3 + c^10*x^3 + a^8*b^2*x^2*y - 2*a^6*b^4*x^2*y + 2*a^2*b^8*x^2*y - b^10*x^2*y -      2*a^6*b^2*c^2*x^2*y + 5*a^4*b^4*c^2*x^2*y - 6*a^2*b^6*c^2*x^2*y + 3*b^8*c^2*x^2*y + a^6*c^4*x^2*y +      2*a^4*b^2*c^4*x^2*y + 2*a^2*b^4*c^4*x^2*y - 3*b^6*c^4*x^2*y - 4*a^4*c^6*x^2*y -      4*a^2*b^2*c^6*x^2*y + 2*b^4*c^6*x^2*y + 5*a^2*c^8*x^2*y + b^2*c^8*x^2*y - 2*c^10*x^2*y +      a^10*x*y^2 - 2*a^8*b^2*x*y^2 + 2*a^4*b^6*x*y^2 - a^2*b^8*x*y^2 - 3*a^8*c^2*x*y^2 +      6*a^6*b^2*c^2*x*y^2 - 5*a^4*b^4*c^2*x*y^2 + 2*a^2*b^6*c^2*x*y^2 + 3*a^6*c^4*x*y^2 -      2*a^4*b^2*c^4*x*y^2 - 2*a^2*b^4*c^4*x*y^2 - b^6*c^4*x*y^2 - 2*a^4*c^6*x*y^2 + 4*a^2*b^2*c^6*x*y^2 +      4*b^4*c^6*x*y^2 - a^2*c^8*x*y^2 - 5*b^2*c^8*x*y^2 + 2*c^10*x*y^2 + a^10*y^3 - 3*a^8*b^2*y^3 +      3*a^6*b^4*y^3 - a^4*b^6*y^3 - a^6*b^2*c^2*y^3 + a^4*b^4*c^2*y^3 + a^6*c^4*y^3 - a^2*b^4*c^4*y^3 +      b^6*c^4*y^3 - a^4*c^6*y^3 + a^2*b^2*c^6*y^3 - 3*b^4*c^6*y^3 + 3*b^2*c^8*y^3 - c^10*y^3 -      a^6*b^4*x^2*z + 4*a^4*b^6*x^2*z - 5*a^2*b^8*x^2*z + 2*b^10*x^2*z - a^8*c^2*x^2*z +      2*a^6*b^2*c^2*x^2*z - 2*a^4*b^4*c^2*x^2*z + 4*a^2*b^6*c^2*x^2*z - b^8*c^2*x^2*z + 2*a^6*c^4*x^2*z -      5*a^4*b^2*c^4*x^2*z - 2*a^2*b^4*c^4*x^2*z - 2*b^6*c^4*x^2*z + 6*a^2*b^2*c^6*x^2*z +      3*b^4*c^6*x^2*z - 2*a^2*c^8*x^2*z - 3*b^2*c^8*x^2*z + c^10*x^2*z - 3*a^8*b^2*x*y*z +      9*a^6*b^4*x*y*z - 9*a^4*b^6*x*y*z + 3*a^2*b^8*x*y*z + 3*a^8*c^2*x*y*z - 3*b^8*c^2*x*y*z -      9*a^6*c^4*x*y*z + 9*b^6*c^4*x*y*z + 9*a^4*c^6*x*y*z - 9*b^4*c^6*x*y*z - 3*a^2*c^8*x*y*z +      3*b^2*c^8*x*y*z - 2*a^10*y^2*z + 5*a^8*b^2*y^2*z - 4*a^6*b^4*y^2*z + a^4*b^6*y^2*z +      a^8*c^2*y^2*z - 4*a^6*b^2*c^2*y^2*z + 2*a^4*b^4*c^2*y^2*z - 2*a^2*b^6*c^2*y^2*z + b^8*c^2*y^2*z +      2*a^6*c^4*y^2*z + 2*a^4*b^2*c^4*y^2*z + 5*a^2*b^4*c^4*y^2*z - 2*b^6*c^4*y^2*z - 3*a^4*c^6*y^2*z -      6*a^2*b^2*c^6*y^2*z + 3*a^2*c^8*y^2*z + 2*b^2*c^8*y^2*z - c^10*y^2*z - a^10*x*z^2 +      3*a^8*b^2*x*z^2 - 3*a^6*b^4*x*z^2 + 2*a^4*b^6*x*z^2 + a^2*b^8*x*z^2 - 2*b^10*x*z^2 +      2*a^8*c^2*x*z^2 - 6*a^6*b^2*c^2*x*z^2 + 2*a^4*b^4*c^2*x*z^2 - 4*a^2*b^6*c^2*x*z^2 +      5*b^8*c^2*x*z^2 + 5*a^4*b^2*c^4*x*z^2 + 2*a^2*b^4*c^4*x*z^2 - 4*b^6*c^4*x*z^2 - 2*a^4*c^6*x*z^2 -      2*a^2*b^2*c^6*x*z^2 + b^4*c^6*x*z^2 + a^2*c^8*x*z^2 + 2*a^10*y*z^2 - a^8*b^2*y*z^2 -      2*a^6*b^4*y*z^2 + 3*a^4*b^6*y*z^2 - 3*a^2*b^8*y*z^2 + b^10*y*z^2 - 5*a^8*c^2*y*z^2 +      4*a^6*b^2*c^2*y*z^2 - 2*a^4*b^4*c^2*y*z^2 + 6*a^2*b^6*c^2*y*z^2 - 2*b^8*c^2*y*z^2 +      4*a^6*c^4*y*z^2 - 2*a^4*b^2*c^4*y*z^2 - 5*a^2*b^4*c^4*y*z^2 - a^4*c^6*y*z^2 + 2*a^2*b^2*c^6*y*z^2 +      2*b^4*c^6*y*z^2 - b^2*c^8*y*z^2 - a^10*z^3 - a^6*b^4*z^3 + a^4*b^6*z^3 + b^10*z^3 + 3*a^8*c^2*z^3 +      a^6*b^2*c^2*z^3 - a^2*b^6*c^2*z^3 - 3*b^8*c^2*z^3 - 3*a^6*c^4*z^3 - a^4*b^2*c^4*z^3 +      a^2*b^4*c^4*z^3 + 3*b^6*c^4*z^3 + a^4*c^6*z^3 - b^4*c^6*z^3");
		exp("K509","a^4*b^2*x^3 - 2*a^2*b^4*x^3 + b^6*x^3 - a^4*c^2*x^3 + 2*a^2*c^4*x^3 - c^6*x^3 + a^6*x^2*y -      3*a^4*b^2*x^2*y + 2*a^2*b^4*x^2*y - a^4*c^2*x^2*y + 6*a^2*b^2*c^2*x^2*y - 3*b^4*c^2*x^2*y -      3*a^2*c^4*x^2*y - b^2*c^4*x^2*y + 2*c^6*x^2*y - 2*a^4*b^2*x*y^2 + 3*a^2*b^4*x*y^2 - b^6*x*y^2 +      3*a^4*c^2*x*y^2 - 6*a^2*b^2*c^2*x*y^2 + b^4*c^2*x*y^2 + a^2*c^4*x*y^2 + 3*b^2*c^4*x*y^2 -      2*c^6*x*y^2 - a^6*y^3 + 2*a^4*b^2*y^3 - a^2*b^4*y^3 + b^4*c^2*y^3 - 2*b^2*c^4*y^3 + c^6*y^3 -      a^6*x^2*z + a^4*b^2*x^2*z + 3*a^2*b^4*x^2*z - 2*b^6*x^2*z + 3*a^4*c^2*x^2*z - 6*a^2*b^2*c^2*x^2*z +      b^4*c^2*x^2*z - 2*a^2*c^4*x^2*z + 3*b^2*c^4*x^2*z + 6*a^4*b^2*x*y*z - 6*a^2*b^4*x*y*z -      6*a^4*c^2*x*y*z + 6*b^4*c^2*x*y*z + 6*a^2*c^4*x*y*z - 6*b^2*c^4*x*y*z + 2*a^6*y^2*z -      3*a^4*b^2*y^2*z - a^2*b^4*y^2*z + b^6*y^2*z - a^4*c^2*y^2*z + 6*a^2*b^2*c^2*y^2*z -      3*b^4*c^2*y^2*z - 3*a^2*c^4*y^2*z + 2*b^2*c^4*y^2*z - 3*a^4*b^2*x*z^2 - a^2*b^4*x*z^2 +      2*b^6*x*z^2 + 2*a^4*c^2*x*z^2 + 6*a^2*b^2*c^2*x*z^2 - 3*b^4*c^2*x*z^2 - 3*a^2*c^4*x*z^2 -      b^2*c^4*x*z^2 + c^6*x*z^2 - 2*a^6*y*z^2 + a^4*b^2*y*z^2 + 3*a^2*b^4*y*z^2 + 3*a^4*c^2*y*z^2 -      6*a^2*b^2*c^2*y*z^2 - 2*b^4*c^2*y*z^2 + a^2*c^4*y*z^2 + 3*b^2*c^4*y*z^2 - c^6*y*z^2 + a^6*z^3 -      b^6*z^3 - 2*a^4*c^2*z^3 + 2*b^4*c^2*z^3 + a^2*c^4*z^3 - b^2*c^4*z^3");
		exp("K510","(-a^5)*c*x^2*y + a^4*b*c*x^2*y - 4*a^2*b^3*c*x^2*y + a*b^4*c*x^2*y + 3*b^5*c*x^2*y +      2*a^4*c^2*x^2*y - 5*a^3*b*c^2*x^2*y + 6*a^2*b^2*c^2*x^2*y + 5*a*b^3*c^2*x^2*y - 8*b^4*c^2*x^2*y +      2*a^3*c^3*x^2*y + 3*a^2*b*c^3*x^2*y - 9*a*b^2*c^3*x^2*y + 2*b^3*c^3*x^2*y - 4*a^2*c^4*x^2*y +      4*a*b*c^4*x^2*y + 6*b^2*c^4*x^2*y - a*c^5*x^2*y - 5*b*c^5*x^2*y + 2*c^6*x^2*y - 3*a^5*c*x*y^2 -      a^4*b*c*x*y^2 + 4*a^3*b^2*c*x*y^2 - a*b^4*c*x*y^2 + b^5*c*x*y^2 + 8*a^4*c^2*x*y^2 -      5*a^3*b*c^2*x*y^2 - 6*a^2*b^2*c^2*x*y^2 + 5*a*b^3*c^2*x*y^2 - 2*b^4*c^2*x*y^2 - 2*a^3*c^3*x*y^2 +      9*a^2*b*c^3*x*y^2 - 3*a*b^2*c^3*x*y^2 - 2*b^3*c^3*x*y^2 - 6*a^2*c^4*x*y^2 - 4*a*b*c^4*x*y^2 +      4*b^2*c^4*x*y^2 + 5*a*c^5*x*y^2 + b*c^5*x*y^2 - 2*c^6*x*y^2 + a^5*b*x^2*z - 2*a^4*b^2*x^2*z -      2*a^3*b^3*x^2*z + 4*a^2*b^4*x^2*z + a*b^5*x^2*z - 2*b^6*x^2*z - a^4*b*c*x^2*z + 5*a^3*b^2*c*x^2*z -      3*a^2*b^3*c*x^2*z - 4*a*b^4*c*x^2*z + 5*b^5*c*x^2*z - 6*a^2*b^2*c^2*x^2*z + 9*a*b^3*c^2*x^2*z -      6*b^4*c^2*x^2*z + 4*a^2*b*c^3*x^2*z - 5*a*b^2*c^3*x^2*z - 2*b^3*c^3*x^2*z - a*b*c^4*x^2*z +      8*b^2*c^4*x^2*z - 3*b*c^5*x^2*z + 2*a^6*y^2*z - a^5*b*y^2*z - 4*a^4*b^2*y^2*z + 2*a^3*b^3*y^2*z +      2*a^2*b^4*y^2*z - a*b^5*y^2*z - 5*a^5*c*y^2*z + 4*a^4*b*c*y^2*z + 3*a^3*b^2*c*y^2*z -      5*a^2*b^3*c*y^2*z + a*b^4*c*y^2*z + 6*a^4*c^2*y^2*z - 9*a^3*b*c^2*y^2*z + 6*a^2*b^2*c^2*y^2*z +      2*a^3*c^3*y^2*z + 5*a^2*b*c^3*y^2*z - 4*a*b^2*c^3*y^2*z - 8*a^2*c^4*y^2*z + a*b*c^4*y^2*z +      3*a*c^5*y^2*z + 3*a^5*b*x*z^2 - 8*a^4*b^2*x*z^2 + 2*a^3*b^3*x*z^2 + 6*a^2*b^4*x*z^2 -      5*a*b^5*x*z^2 + 2*b^6*x*z^2 + a^4*b*c*x*z^2 + 5*a^3*b^2*c*x*z^2 - 9*a^2*b^3*c*x*z^2 +      4*a*b^4*c*x*z^2 - b^5*c*x*z^2 - 4*a^3*b*c^2*x*z^2 + 6*a^2*b^2*c^2*x*z^2 + 3*a*b^3*c^2*x*z^2 -      4*b^4*c^2*x*z^2 - 5*a*b^2*c^3*x*z^2 + 2*b^3*c^3*x*z^2 + a*b*c^4*x*z^2 + 2*b^2*c^4*x*z^2 -      b*c^5*x*z^2 - 2*a^6*y*z^2 + 5*a^5*b*y*z^2 - 6*a^4*b^2*y*z^2 - 2*a^3*b^3*y*z^2 + 8*a^2*b^4*y*z^2 -      3*a*b^5*y*z^2 + a^5*c*y*z^2 - 4*a^4*b*c*y*z^2 + 9*a^3*b^2*c*y*z^2 - 5*a^2*b^3*c*y*z^2 -      a*b^4*c*y*z^2 + 4*a^4*c^2*y*z^2 - 3*a^3*b*c^2*y*z^2 - 6*a^2*b^2*c^2*y*z^2 + 4*a*b^3*c^2*y*z^2 -      2*a^3*c^3*y*z^2 + 5*a^2*b*c^3*y*z^2 - 2*a^2*c^4*y*z^2 - a*b*c^4*y*z^2 + a*c^5*y*z^2");
		exp("K511","(-a^2)*b^4*c^2*x^3 + b^6*c^2*x^3 + a^2*b^2*c^4*x^3 - b^2*c^6*x^3 + a^4*b^2*c^2*x^2*y -      a^2*b^4*c^2*x^2*y + b^4*c^4*x^2*y - b^2*c^6*x^2*y + a^4*b^2*c^2*x*y^2 - a^2*b^4*c^2*x*y^2 -      a^4*c^4*x*y^2 + a^2*c^6*x*y^2 - a^6*c^2*y^3 + a^4*b^2*c^2*y^3 - a^2*b^2*c^4*y^3 + a^2*c^6*y^3 -      a^4*b^2*c^2*x^2*z + b^6*c^2*x^2*z + a^2*b^2*c^4*x^2*z - b^4*c^4*x^2*z - a^6*c^2*y^2*z +      a^2*b^4*c^2*y^2*z + a^4*c^4*y^2*z - a^2*b^2*c^4*y^2*z + a^4*b^4*x*z^2 - a^2*b^6*x*z^2 -      a^4*b^2*c^2*x*z^2 + a^2*b^2*c^4*x*z^2 + a^6*b^2*y*z^2 - a^4*b^4*y*z^2 + a^2*b^4*c^2*y*z^2 -      a^2*b^2*c^4*y*z^2 + a^6*b^2*z^3 - a^2*b^6*z^3 - a^4*b^2*c^2*z^3 + a^2*b^4*c^2*z^3");
		exp("K512","a^2*b^2*c^2*x^2*y - a^2*b^2*c^2*x*y^2 - a^2*b^2*c^2*x^2*z + a^4*b^2*x*y*z - a^2*b^4*x*y*z -      a^4*c^2*x*y*z + b^4*c^2*x*y*z + a^2*c^4*x*y*z - b^2*c^4*x*y*z + a^2*b^2*c^2*y^2*z +      a^2*b^2*c^2*x*z^2 - a^2*b^2*c^2*y*z^2");
		exp("K513","(-a^8)*b^2*c^4*x^2*y + 3*a^6*b^4*c^4*x^2*y - 2*a^4*b^6*c^4*x^2*y - a^2*b^8*c^4*x^2*y +      b^10*c^4*x^2*y + 2*a^6*b^2*c^6*x^2*y - a^4*b^4*c^6*x^2*y + a^2*b^6*c^6*x^2*y - 3*b^8*c^6*x^2*y -      a^4*b^2*c^8*x^2*y + 3*b^6*c^8*x^2*y - b^4*c^10*x^2*y - a^10*c^4*x*y^2 + a^8*b^2*c^4*x*y^2 +      2*a^6*b^4*c^4*x*y^2 - 3*a^4*b^6*c^4*x*y^2 + a^2*b^8*c^4*x*y^2 + 3*a^8*c^6*x*y^2 -      a^6*b^2*c^6*x*y^2 + a^4*b^4*c^6*x*y^2 - 2*a^2*b^6*c^6*x*y^2 - 3*a^6*c^8*x*y^2 + a^2*b^4*c^8*x*y^2 +      a^4*c^10*x*y^2 + a^8*b^4*c^2*x^2*z - 2*a^6*b^6*c^2*x^2*z + a^4*b^8*c^2*x^2*z -      3*a^6*b^4*c^4*x^2*z + a^4*b^6*c^4*x^2*z + b^10*c^4*x^2*z + 2*a^4*b^4*c^6*x^2*z -      a^2*b^6*c^6*x^2*z - 3*b^8*c^6*x^2*z + a^2*b^4*c^8*x^2*z + 3*b^6*c^8*x^2*z - b^4*c^10*x^2*z -      a^8*b^4*c^2*y^2*z + 2*a^6*b^6*c^2*y^2*z - a^4*b^8*c^2*y^2*z - a^10*c^4*y^2*z - a^6*b^4*c^4*y^2*z +      3*a^4*b^6*c^4*y^2*z + 3*a^8*c^6*y^2*z + a^6*b^2*c^6*y^2*z - 2*a^4*b^4*c^6*y^2*z - 3*a^6*c^8*y^2*z -      a^4*b^2*c^8*y^2*z + a^4*c^10*y^2*z + a^10*b^4*x*z^2 - 3*a^8*b^6*x*z^2 + 3*a^6*b^8*x*z^2 -      a^4*b^10*x*z^2 - a^8*b^4*c^2*x*z^2 + a^6*b^6*c^2*x*z^2 - 2*a^6*b^4*c^4*x*z^2 - a^4*b^6*c^4*x*z^2 -      a^2*b^8*c^4*x*z^2 + 3*a^4*b^4*c^6*x*z^2 + 2*a^2*b^6*c^6*x*z^2 - a^2*b^4*c^8*x*z^2 +      a^10*b^4*y*z^2 - 3*a^8*b^6*y*z^2 + 3*a^6*b^8*y*z^2 - a^4*b^10*y*z^2 - a^6*b^6*c^2*y*z^2 +      a^4*b^8*c^2*y*z^2 + a^8*b^2*c^4*y*z^2 + a^6*b^4*c^4*y*z^2 + 2*a^4*b^6*c^4*y*z^2 -      2*a^6*b^2*c^6*y*z^2 - 3*a^4*b^4*c^6*y*z^2 + a^4*b^2*c^8*y*z^2");
		exp("K514","(-a^8)*c^2*x^2*y + a^6*b^2*c^2*x^2*y + 4*a^4*b^4*c^2*x^2*y - 7*a^2*b^6*c^2*x^2*y +      3*b^8*c^2*x^2*y + 3*a^6*c^4*x^2*y + 2*a^2*b^4*c^4*x^2*y - 7*b^6*c^4*x^2*y - 3*a^4*c^6*x^2*y +      7*b^4*c^6*x^2*y + a^2*c^8*x^2*y - 3*b^2*c^8*x^2*y - 3*a^8*c^2*x*y^2 + 7*a^6*b^2*c^2*x*y^2 -      4*a^4*b^4*c^2*x*y^2 - a^2*b^6*c^2*x*y^2 + b^8*c^2*x*y^2 + 7*a^6*c^4*x*y^2 - 2*a^4*b^2*c^4*x*y^2 -      3*b^6*c^4*x*y^2 - 7*a^4*c^6*x*y^2 + 3*b^4*c^6*x*y^2 + 3*a^2*c^8*x*y^2 - b^2*c^8*x*y^2 +      a^8*b^2*x^2*z - 3*a^6*b^4*x^2*z + 3*a^4*b^6*x^2*z - a^2*b^8*x^2*z - a^6*b^2*c^2*x^2*z +      3*b^8*c^2*x^2*z - 4*a^4*b^2*c^4*x^2*z - 2*a^2*b^4*c^4*x^2*z - 7*b^6*c^4*x^2*z +      7*a^2*b^2*c^6*x^2*z + 7*b^4*c^6*x^2*z - 3*b^2*c^8*x^2*z + 2*a^8*b^2*x*y*z - 6*a^6*b^4*x*y*z +      6*a^4*b^6*x*y*z - 2*a^2*b^8*x*y*z - 2*a^8*c^2*x*y*z + 2*b^8*c^2*x*y*z + 6*a^6*c^4*x*y*z -      6*b^6*c^4*x*y*z - 6*a^4*c^6*x*y*z + 6*b^4*c^6*x*y*z + 2*a^2*c^8*x*y*z - 2*b^2*c^8*x*y*z +      a^8*b^2*y^2*z - 3*a^6*b^4*y^2*z + 3*a^4*b^6*y^2*z - a^2*b^8*y^2*z - 3*a^8*c^2*y^2*z +      a^2*b^6*c^2*y^2*z + 7*a^6*c^4*y^2*z + 2*a^4*b^2*c^4*y^2*z + 4*a^2*b^4*c^4*y^2*z - 7*a^4*c^6*y^2*z -      7*a^2*b^2*c^6*y^2*z + 3*a^2*c^8*y^2*z + 3*a^8*b^2*x*z^2 - 7*a^6*b^4*x*z^2 + 7*a^4*b^6*x*z^2 -      3*a^2*b^8*x*z^2 - 7*a^6*b^2*c^2*x*z^2 + 2*a^4*b^4*c^2*x*z^2 + b^8*c^2*x*z^2 + 4*a^4*b^2*c^4*x*z^2 -      3*b^6*c^4*x*z^2 + a^2*b^2*c^6*x*z^2 + 3*b^4*c^6*x*z^2 - b^2*c^8*x*z^2 + 3*a^8*b^2*y*z^2 -      7*a^6*b^4*y*z^2 + 7*a^4*b^6*y*z^2 - 3*a^2*b^8*y*z^2 - a^8*c^2*y*z^2 - 2*a^4*b^4*c^2*y*z^2 +      7*a^2*b^6*c^2*y*z^2 + 3*a^6*c^4*y*z^2 - 4*a^2*b^4*c^4*y*z^2 - 3*a^4*c^6*y*z^2 - a^2*b^2*c^6*y*z^2 +      a^2*c^8*y*z^2");
		exp("K515","a^12*x^2*y - 6*a^10*b^2*x^2*y + 15*a^8*b^4*x^2*y - 20*a^6*b^6*x^2*y + 15*a^4*b^8*x^2*y -      6*a^2*b^10*x^2*y + b^12*x^2*y - 3*a^10*c^2*x^2*y + 6*a^8*b^2*c^2*x^2*y - 3*a^6*b^4*c^2*x^2*y +      3*a^4*b^6*c^2*x^2*y - 6*a^2*b^8*c^2*x^2*y + 3*b^10*c^2*x^2*y - 6*a^6*b^2*c^4*x^2*y -      12*a^4*b^4*c^4*x^2*y + 24*a^2*b^6*c^4*x^2*y - 6*b^8*c^4*x^2*y + 10*a^6*c^6*x^2*y +      15*a^4*b^2*c^6*x^2*y - 15*a^2*b^4*c^6*x^2*y - 8*b^6*c^6*x^2*y - 15*a^4*c^8*x^2*y -      6*a^2*b^2*c^8*x^2*y + 15*b^4*c^8*x^2*y + 9*a^2*c^10*x^2*y - 3*b^2*c^10*x^2*y - 2*c^12*x^2*y -      a^12*x*y^2 + 6*a^10*b^2*x*y^2 - 15*a^8*b^4*x*y^2 + 20*a^6*b^6*x*y^2 - 15*a^4*b^8*x*y^2 +      6*a^2*b^10*x*y^2 - b^12*x*y^2 - 3*a^10*c^2*x*y^2 + 6*a^8*b^2*c^2*x*y^2 - 3*a^6*b^4*c^2*x*y^2 +      3*a^4*b^6*c^2*x*y^2 - 6*a^2*b^8*c^2*x*y^2 + 3*b^10*c^2*x*y^2 + 6*a^8*c^4*x*y^2 -      24*a^6*b^2*c^4*x*y^2 + 12*a^4*b^4*c^4*x*y^2 + 6*a^2*b^6*c^4*x*y^2 + 8*a^6*c^6*x*y^2 +      15*a^4*b^2*c^6*x*y^2 - 15*a^2*b^4*c^6*x*y^2 - 10*b^6*c^6*x*y^2 - 15*a^4*c^8*x*y^2 +      6*a^2*b^2*c^8*x*y^2 + 15*b^4*c^8*x*y^2 + 3*a^2*c^10*x*y^2 - 9*b^2*c^10*x*y^2 + 2*c^12*x*y^2 -      a^12*x^2*z + 3*a^10*b^2*x^2*z - 10*a^6*b^6*x^2*z + 15*a^4*b^8*x^2*z - 9*a^2*b^10*x^2*z +      2*b^12*x^2*z + 6*a^10*c^2*x^2*z - 6*a^8*b^2*c^2*x^2*z + 6*a^6*b^4*c^2*x^2*z -      15*a^4*b^6*c^2*x^2*z + 6*a^2*b^8*c^2*x^2*z + 3*b^10*c^2*x^2*z - 15*a^8*c^4*x^2*z +      3*a^6*b^2*c^4*x^2*z + 12*a^4*b^4*c^4*x^2*z + 15*a^2*b^6*c^4*x^2*z - 15*b^8*c^4*x^2*z +      20*a^6*c^6*x^2*z - 3*a^4*b^2*c^6*x^2*z - 24*a^2*b^4*c^6*x^2*z + 8*b^6*c^6*x^2*z -      15*a^4*c^8*x^2*z + 6*a^2*b^2*c^8*x^2*z + 6*b^4*c^8*x^2*z + 6*a^2*c^10*x^2*z - 3*b^2*c^10*x^2*z -      c^12*x^2*z - 2*a^12*y^2*z + 9*a^10*b^2*y^2*z - 15*a^8*b^4*y^2*z + 10*a^6*b^6*y^2*z -      3*a^2*b^10*y^2*z + b^12*y^2*z - 3*a^10*c^2*y^2*z - 6*a^8*b^2*c^2*y^2*z + 15*a^6*b^4*c^2*y^2*z -      6*a^4*b^6*c^2*y^2*z + 6*a^2*b^8*c^2*y^2*z - 6*b^10*c^2*y^2*z + 15*a^8*c^4*y^2*z -      15*a^6*b^2*c^4*y^2*z - 12*a^4*b^4*c^4*y^2*z - 3*a^2*b^6*c^4*y^2*z + 15*b^8*c^4*y^2*z -      8*a^6*c^6*y^2*z + 24*a^4*b^2*c^6*y^2*z + 3*a^2*b^4*c^6*y^2*z - 20*b^6*c^6*y^2*z - 6*a^4*c^8*y^2*z -      6*a^2*b^2*c^8*y^2*z + 15*b^4*c^8*y^2*z + 3*a^2*c^10*y^2*z - 6*b^2*c^10*y^2*z + c^12*y^2*z +      a^12*x*z^2 + 3*a^10*b^2*x*z^2 - 6*a^8*b^4*x*z^2 - 8*a^6*b^6*x*z^2 + 15*a^4*b^8*x*z^2 -      3*a^2*b^10*x*z^2 - 2*b^12*x*z^2 - 6*a^10*c^2*x*z^2 - 6*a^8*b^2*c^2*x*z^2 + 24*a^6*b^4*c^2*x*z^2 -      15*a^4*b^6*c^2*x*z^2 - 6*a^2*b^8*c^2*x*z^2 + 9*b^10*c^2*x*z^2 + 15*a^8*c^4*x*z^2 +      3*a^6*b^2*c^4*x*z^2 - 12*a^4*b^4*c^4*x*z^2 + 15*a^2*b^6*c^4*x*z^2 - 15*b^8*c^4*x*z^2 -      20*a^6*c^6*x*z^2 - 3*a^4*b^2*c^6*x*z^2 - 6*a^2*b^4*c^6*x*z^2 + 10*b^6*c^6*x*z^2 +      15*a^4*c^8*x*z^2 + 6*a^2*b^2*c^8*x*z^2 - 6*a^2*c^10*x*z^2 - 3*b^2*c^10*x*z^2 + c^12*x*z^2 +      2*a^12*y*z^2 + 3*a^10*b^2*y*z^2 - 15*a^8*b^4*y*z^2 + 8*a^6*b^6*y*z^2 + 6*a^4*b^8*y*z^2 -      3*a^2*b^10*y*z^2 - b^12*y*z^2 - 9*a^10*c^2*y*z^2 + 6*a^8*b^2*c^2*y*z^2 + 15*a^6*b^4*c^2*y*z^2 -      24*a^4*b^6*c^2*y*z^2 + 6*a^2*b^8*c^2*y*z^2 + 6*b^10*c^2*y*z^2 + 15*a^8*c^4*y*z^2 -      15*a^6*b^2*c^4*y*z^2 + 12*a^4*b^4*c^4*y*z^2 - 3*a^2*b^6*c^4*y*z^2 - 15*b^8*c^4*y*z^2 -      10*a^6*c^6*y*z^2 + 6*a^4*b^2*c^6*y*z^2 + 3*a^2*b^4*c^6*y*z^2 + 20*b^6*c^6*y*z^2 -      6*a^2*b^2*c^8*y*z^2 - 15*b^4*c^8*y*z^2 + 3*a^2*c^10*y*z^2 + 6*b^2*c^10*y*z^2 - c^12*y*z^2");
		exp("K516","(-a^6)*b^2*c^2*x^2*y + a^4*b^4*c^2*x^2*y + 2*a^2*b^4*c^4*x^2*y - b^6*c^4*x^2*y +      a^2*b^2*c^6*x^2*y + b^4*c^6*x^2*y - a^4*b^4*c^2*x*y^2 + a^2*b^6*c^2*x*y^2 + a^6*c^4*x*y^2 -      2*a^4*b^2*c^4*x*y^2 - a^4*c^6*x*y^2 - a^2*b^2*c^6*x*y^2 + a^6*b^2*c^2*x^2*z - a^2*b^6*c^2*x^2*z -      a^4*b^2*c^4*x^2*z - 2*a^2*b^4*c^4*x^2*z - b^6*c^4*x^2*z + b^4*c^6*x^2*z + a^6*b^2*c^2*y^2*z -      a^2*b^6*c^2*y^2*z + a^6*c^4*y^2*z + 2*a^4*b^2*c^4*y^2*z + a^2*b^4*c^4*y^2*z - a^4*c^6*y^2*z -      a^6*b^4*x*z^2 + a^4*b^6*x*z^2 + 2*a^4*b^4*c^2*x*z^2 + a^2*b^6*c^2*x*z^2 + a^4*b^2*c^4*x*z^2 -      a^2*b^2*c^6*x*z^2 - a^6*b^4*y*z^2 + a^4*b^6*y*z^2 - a^6*b^2*c^2*y*z^2 - 2*a^4*b^4*c^2*y*z^2 -      a^2*b^4*c^4*y*z^2 + a^2*b^2*c^6*y*z^2");
		exp("K517","a^4*x^2*y - b^4*x^2*y - a^2*c^2*x^2*y - b^2*c^2*x^2*y + a^4*x*y^2 - b^4*x*y^2 + a^2*c^2*x*y^2 +      b^2*c^2*x*y^2 - a^4*x^2*z + a^2*b^2*x^2*z + b^2*c^2*x^2*z + c^4*x^2*z - a^2*b^2*y^2*z + b^4*y^2*z -      a^2*c^2*y^2*z - c^4*y^2*z - a^4*x*z^2 - a^2*b^2*x*z^2 - b^2*c^2*x*z^2 + c^4*x*z^2 + a^2*b^2*y*z^2 +      b^4*y*z^2 + a^2*c^2*y*z^2 - c^4*y*z^2");
		exp("K518","(-a^6)*x^2*y + 3*a^4*b^2*x^2*y - 3*a^2*b^4*x^2*y + b^6*x^2*y - 4*b^4*c^2*x^2*y + a^2*c^4*x^2*y +      3*b^2*c^4*x^2*y - a^6*x*y^2 + 3*a^4*b^2*x*y^2 - 3*a^2*b^4*x*y^2 + b^6*x*y^2 + 4*a^4*c^2*x*y^2 -      3*a^2*c^4*x*y^2 - b^2*c^4*x*y^2 + a^6*x^2*z - a^2*b^4*x^2*z - 3*a^4*c^2*x^2*z - 3*b^4*c^2*x^2*z +      3*a^2*c^4*x^2*z + 4*b^2*c^4*x^2*z - c^6*x^2*z + a^4*b^2*y^2*z - b^6*y^2*z + 3*a^4*c^2*y^2*z +      3*b^4*c^2*y^2*z - 4*a^2*c^4*y^2*z - 3*b^2*c^4*y^2*z + c^6*y^2*z + a^6*x*z^2 - 4*a^4*b^2*x*z^2 +      3*a^2*b^4*x*z^2 - 3*a^4*c^2*x*z^2 + b^4*c^2*x*z^2 + 3*a^2*c^4*x*z^2 - c^6*x*z^2 - 3*a^4*b^2*y*z^2 +      4*a^2*b^4*y*z^2 - b^6*y*z^2 - a^4*c^2*y*z^2 + 3*b^4*c^2*y*z^2 - 3*b^2*c^4*y*z^2 + c^6*y*z^2");
		exp("K519","-2*a^6*b^4*c^2*x^2*y + 6*a^4*b^6*c^2*x^2*y - 6*a^2*b^8*c^2*x^2*y + 2*b^10*c^2*x^2*y -      a^8*c^4*x^2*y - 2*a^4*b^4*c^4*x^2*y + 8*a^2*b^6*c^4*x^2*y - 5*b^8*c^4*x^2*y + 4*a^6*c^6*x^2*y +      2*a^4*b^2*c^6*x^2*y - 2*a^2*b^4*c^6*x^2*y + 4*b^6*c^6*x^2*y - 6*a^4*c^8*x^2*y -      4*a^2*b^2*c^8*x^2*y - 2*b^4*c^8*x^2*y + 4*a^2*c^10*x^2*y + 2*b^2*c^10*x^2*y - c^12*x^2*y -      2*a^10*c^2*x*y^2 + 6*a^8*b^2*c^2*x*y^2 - 6*a^6*b^4*c^2*x*y^2 + 2*a^4*b^6*c^2*x*y^2 +      5*a^8*c^4*x*y^2 - 8*a^6*b^2*c^4*x*y^2 + 2*a^4*b^4*c^4*x*y^2 + b^8*c^4*x*y^2 - 4*a^6*c^6*x*y^2 +      2*a^4*b^2*c^6*x*y^2 - 2*a^2*b^4*c^6*x*y^2 - 4*b^6*c^6*x*y^2 + 2*a^4*c^8*x*y^2 +      4*a^2*b^2*c^8*x*y^2 + 6*b^4*c^8*x*y^2 - 2*a^2*c^10*x*y^2 - 4*b^2*c^10*x*y^2 + c^12*x*y^2 +      a^8*b^4*x^2*z - 4*a^6*b^6*x^2*z + 6*a^4*b^8*x^2*z - 4*a^2*b^10*x^2*z + b^12*x^2*z -      2*a^4*b^6*c^2*x^2*z + 4*a^2*b^8*c^2*x^2*z - 2*b^10*c^2*x^2*z + 2*a^6*b^2*c^4*x^2*z +      2*a^4*b^4*c^4*x^2*z + 2*a^2*b^6*c^4*x^2*z + 2*b^8*c^4*x^2*z - 6*a^4*b^2*c^6*x^2*z -      8*a^2*b^4*c^6*x^2*z - 4*b^6*c^6*x^2*z + 6*a^2*b^2*c^8*x^2*z + 5*b^4*c^8*x^2*z - 2*b^2*c^10*x^2*z -      a^12*y^2*z + 4*a^10*b^2*y^2*z - 6*a^8*b^4*y^2*z + 4*a^6*b^6*y^2*z - a^4*b^8*y^2*z +      2*a^10*c^2*y^2*z - 4*a^8*b^2*c^2*y^2*z + 2*a^6*b^4*c^2*y^2*z - 2*a^8*c^4*y^2*z -      2*a^6*b^2*c^4*y^2*z - 2*a^4*b^4*c^4*y^2*z - 2*a^2*b^6*c^4*y^2*z + 4*a^6*c^6*y^2*z +      8*a^4*b^2*c^6*y^2*z + 6*a^2*b^4*c^6*y^2*z - 5*a^4*c^8*y^2*z - 6*a^2*b^2*c^8*y^2*z +      2*a^2*c^10*y^2*z + 2*a^10*b^2*x*z^2 - 5*a^8*b^4*x*z^2 + 4*a^6*b^6*x*z^2 - 2*a^4*b^8*x*z^2 +      2*a^2*b^10*x*z^2 - b^12*x*z^2 - 6*a^8*b^2*c^2*x*z^2 + 8*a^6*b^4*c^2*x*z^2 - 2*a^4*b^6*c^2*x*z^2 -      4*a^2*b^8*c^2*x*z^2 + 4*b^10*c^2*x*z^2 + 6*a^6*b^2*c^4*x*z^2 - 2*a^4*b^4*c^4*x*z^2 +      2*a^2*b^6*c^4*x*z^2 - 6*b^8*c^4*x*z^2 - 2*a^4*b^2*c^6*x*z^2 + 4*b^6*c^6*x*z^2 - b^4*c^8*x*z^2 +      a^12*y*z^2 - 2*a^10*b^2*y*z^2 + 2*a^8*b^4*y*z^2 - 4*a^6*b^6*y*z^2 + 5*a^4*b^8*y*z^2 -      2*a^2*b^10*y*z^2 - 4*a^10*c^2*y*z^2 + 4*a^8*b^2*c^2*y*z^2 + 2*a^6*b^4*c^2*y*z^2 -      8*a^4*b^6*c^2*y*z^2 + 6*a^2*b^8*c^2*y*z^2 + 6*a^8*c^4*y*z^2 - 2*a^6*b^2*c^4*y*z^2 +      2*a^4*b^4*c^4*y*z^2 - 6*a^2*b^6*c^4*y*z^2 - 4*a^6*c^6*y*z^2 + 2*a^2*b^4*c^6*y*z^2 +      a^4*c^8*y*z^2");
		exp("K520","-2*a^8*b^2*c^2*x^2*y + 4*a^6*b^4*c^2*x^2*y - 4*a^2*b^8*c^2*x^2*y + 2*b^10*c^2*x^2*y -      a^8*c^4*x^2*y - 2*a^4*b^4*c^4*x^2*y + 8*a^2*b^6*c^4*x^2*y - 5*b^8*c^4*x^2*y + 4*a^6*c^6*x^2*y +      8*a^4*b^2*c^6*x^2*y + 4*b^6*c^6*x^2*y - 6*a^4*c^8*x^2*y - 8*a^2*b^2*c^8*x^2*y - 2*b^4*c^8*x^2*y +      4*a^2*c^10*x^2*y + 2*b^2*c^10*x^2*y - c^12*x^2*y - 2*a^10*c^2*x*y^2 + 4*a^8*b^2*c^2*x*y^2 -      4*a^4*b^6*c^2*x*y^2 + 2*a^2*b^8*c^2*x*y^2 + 5*a^8*c^4*x*y^2 - 8*a^6*b^2*c^4*x*y^2 +      2*a^4*b^4*c^4*x*y^2 + b^8*c^4*x*y^2 - 4*a^6*c^6*x*y^2 - 8*a^2*b^4*c^6*x*y^2 - 4*b^6*c^6*x*y^2 +      2*a^4*c^8*x*y^2 + 8*a^2*b^2*c^8*x*y^2 + 6*b^4*c^8*x*y^2 - 2*a^2*c^10*x*y^2 - 4*b^2*c^10*x*y^2 +      c^12*x*y^2 + a^8*b^4*x^2*z - 4*a^6*b^6*x^2*z + 6*a^4*b^8*x^2*z - 4*a^2*b^10*x^2*z + b^12*x^2*z +      2*a^8*b^2*c^2*x^2*z - 8*a^4*b^6*c^2*x^2*z + 8*a^2*b^8*c^2*x^2*z - 2*b^10*c^2*x^2*z -      4*a^6*b^2*c^4*x^2*z + 2*a^4*b^4*c^4*x^2*z + 2*b^8*c^4*x^2*z - 8*a^2*b^4*c^6*x^2*z -      4*b^6*c^6*x^2*z + 4*a^2*b^2*c^8*x^2*z + 5*b^4*c^8*x^2*z - 2*b^2*c^10*x^2*z - a^12*y^2*z +      4*a^10*b^2*y^2*z - 6*a^8*b^4*y^2*z + 4*a^6*b^6*y^2*z - a^4*b^8*y^2*z + 2*a^10*c^2*y^2*z -      8*a^8*b^2*c^2*y^2*z + 8*a^6*b^4*c^2*y^2*z - 2*a^2*b^8*c^2*y^2*z - 2*a^8*c^4*y^2*z -      2*a^4*b^4*c^4*y^2*z + 4*a^2*b^6*c^4*y^2*z + 4*a^6*c^6*y^2*z + 8*a^4*b^2*c^6*y^2*z -      5*a^4*c^8*y^2*z - 4*a^2*b^2*c^8*y^2*z + 2*a^2*c^10*y^2*z + 2*a^10*b^2*x*z^2 - 5*a^8*b^4*x*z^2 +      4*a^6*b^6*x*z^2 - 2*a^4*b^8*x*z^2 + 2*a^2*b^10*x*z^2 - b^12*x*z^2 - 4*a^8*b^2*c^2*x*z^2 +      8*a^6*b^4*c^2*x*z^2 - 8*a^2*b^8*c^2*x*z^2 + 4*b^10*c^2*x*z^2 - 2*a^4*b^4*c^4*x*z^2 +      8*a^2*b^6*c^4*x*z^2 - 6*b^8*c^4*x*z^2 + 4*a^4*b^2*c^6*x*z^2 + 4*b^6*c^6*x*z^2 -      2*a^2*b^2*c^8*x*z^2 - b^4*c^8*x*z^2 + a^12*y*z^2 - 2*a^10*b^2*y*z^2 + 2*a^8*b^4*y*z^2 -      4*a^6*b^6*y*z^2 + 5*a^4*b^8*y*z^2 - 2*a^2*b^10*y*z^2 - 4*a^10*c^2*y*z^2 + 8*a^8*b^2*c^2*y*z^2 -      8*a^4*b^6*c^2*y*z^2 + 4*a^2*b^8*c^2*y*z^2 + 6*a^8*c^4*y*z^2 - 8*a^6*b^2*c^4*y*z^2 +      2*a^4*b^4*c^4*y*z^2 - 4*a^6*c^6*y*z^2 - 4*a^2*b^4*c^6*y*z^2 + a^4*c^8*y*z^2 +      2*a^2*b^2*c^8*y*z^2");
		exp("K521","a^6*x^2*y - 3*a^4*b^2*x^2*y + 3*a^2*b^4*x^2*y - b^6*x^2*y - a^4*c^2*x^2*y + 8*a^3*b*c^2*x^2*y -      2*a^2*b^2*c^2*x^2*y - 8*a*b^3*c^2*x^2*y + 3*b^4*c^2*x^2*y + 16*a*b^2*c^3*x^2*y - a^2*c^4*x^2*y -      8*a*b*c^4*x^2*y - 3*b^2*c^4*x^2*y + c^6*x^2*y + a^6*x*y^2 - 3*a^4*b^2*x*y^2 + 3*a^2*b^4*x*y^2 -      b^6*x*y^2 - 3*a^4*c^2*x*y^2 + 8*a^3*b*c^2*x*y^2 + 2*a^2*b^2*c^2*x*y^2 - 8*a*b^3*c^2*x*y^2 +      b^4*c^2*x*y^2 - 16*a^2*b*c^3*x*y^2 + 3*a^2*c^4*x*y^2 + 8*a*b*c^4*x*y^2 + b^2*c^4*x*y^2 -      c^6*x*y^2 - a^6*x^2*z + a^4*b^2*x^2*z + a^2*b^4*x^2*z - b^6*x^2*z - 8*a^3*b^2*c*x^2*z +      8*a*b^4*c*x^2*z + 3*a^4*c^2*x^2*z + 2*a^2*b^2*c^2*x^2*z - 16*a*b^3*c^2*x^2*z + 3*b^4*c^2*x^2*z +      8*a*b^2*c^3*x^2*z - 3*a^2*c^4*x^2*z - 3*b^2*c^4*x^2*z + c^6*x^2*z + a^6*y^2*z - a^4*b^2*y^2*z -      a^2*b^4*y^2*z + b^6*y^2*z - 8*a^4*b*c*y^2*z + 8*a^2*b^3*c*y^2*z - 3*a^4*c^2*y^2*z +      16*a^3*b*c^2*y^2*z - 2*a^2*b^2*c^2*y^2*z - 3*b^4*c^2*y^2*z - 8*a^2*b*c^3*y^2*z + 3*a^2*c^4*y^2*z +      3*b^2*c^4*y^2*z - c^6*y^2*z - a^6*x*z^2 + 3*a^4*b^2*x*z^2 - 3*a^2*b^4*x*z^2 + b^6*x*z^2 -      8*a^3*b^2*c*x*z^2 + 16*a^2*b^3*c*x*z^2 - 8*a*b^4*c*x*z^2 + 3*a^4*c^2*x*z^2 - 2*a^2*b^2*c^2*x*z^2 -      b^4*c^2*x*z^2 + 8*a*b^2*c^3*x*z^2 - 3*a^2*c^4*x*z^2 - b^2*c^4*x*z^2 + c^6*x*z^2 - a^6*y*z^2 +      3*a^4*b^2*y*z^2 - 3*a^2*b^4*y*z^2 + b^6*y*z^2 + 8*a^4*b*c*y*z^2 - 16*a^3*b^2*c*y*z^2 +      8*a^2*b^3*c*y*z^2 + a^4*c^2*y*z^2 + 2*a^2*b^2*c^2*y*z^2 - 3*b^4*c^2*y*z^2 - 8*a^2*b*c^3*y*z^2 +      a^2*c^4*y*z^2 + 3*b^2*c^4*y*z^2 - c^6*y*z^2");
		exp("K522","(-a^6)*x^2*y + 3*a^4*b^2*x^2*y - 3*a^2*b^4*x^2*y + b^6*x^2*y + 3*a^4*c^2*x^2*y +      6*a^2*b^2*c^2*x^2*y - 9*b^4*c^2*x^2*y - 3*a^2*c^4*x^2*y + 7*b^2*c^4*x^2*y + c^6*x^2*y - a^6*x*y^2 +      3*a^4*b^2*x*y^2 - 3*a^2*b^4*x*y^2 + b^6*x*y^2 + 9*a^4*c^2*x*y^2 - 6*a^2*b^2*c^2*x*y^2 -      3*b^4*c^2*x*y^2 - 7*a^2*c^4*x*y^2 + 3*b^2*c^4*x*y^2 - c^6*x*y^2 + a^6*x^2*z - 3*a^4*b^2*x^2*z +      3*a^2*b^4*x^2*z - b^6*x^2*z - 3*a^4*c^2*x^2*z - 6*a^2*b^2*c^2*x^2*z - 7*b^4*c^2*x^2*z +      3*a^2*c^4*x^2*z + 9*b^2*c^4*x^2*z - c^6*x^2*z + a^6*y^2*z - 3*a^4*b^2*y^2*z + 3*a^2*b^4*y^2*z -      b^6*y^2*z + 7*a^4*c^2*y^2*z + 6*a^2*b^2*c^2*y^2*z + 3*b^4*c^2*y^2*z - 9*a^2*c^4*y^2*z -      3*b^2*c^4*y^2*z + c^6*y^2*z + a^6*x*z^2 - 9*a^4*b^2*x*z^2 + 7*a^2*b^4*x*z^2 + b^6*x*z^2 -      3*a^4*c^2*x*z^2 + 6*a^2*b^2*c^2*x*z^2 - 3*b^4*c^2*x*z^2 + 3*a^2*c^4*x*z^2 + 3*b^2*c^4*x*z^2 -      c^6*x*z^2 - a^6*y*z^2 - 7*a^4*b^2*y*z^2 + 9*a^2*b^4*y*z^2 - b^6*y*z^2 + 3*a^4*c^2*y*z^2 -      6*a^2*b^2*c^2*y*z^2 + 3*b^4*c^2*y*z^2 - 3*a^2*c^4*y*z^2 - 3*b^2*c^4*y*z^2 + c^6*y*z^2");
		exp("K523","-4*a^6*b^2*c^4*x^2*y - 3*a^4*b^4*c^4*x^2*y + 6*a^2*b^6*c^4*x^2*y + b^8*c^4*x^2*y +      7*a^4*b^2*c^6*x^2*y - 4*a^2*b^4*c^6*x^2*y - 3*b^6*c^6*x^2*y - 2*a^2*b^2*c^8*x^2*y +      3*b^4*c^8*x^2*y - b^2*c^10*x^2*y - a^8*c^4*x*y^2 - 6*a^6*b^2*c^4*x*y^2 + 3*a^4*b^4*c^4*x*y^2 +      4*a^2*b^6*c^4*x*y^2 + 3*a^6*c^6*x*y^2 + 4*a^4*b^2*c^6*x*y^2 - 7*a^2*b^4*c^6*x*y^2 -      3*a^4*c^8*x*y^2 + 2*a^2*b^2*c^8*x*y^2 + a^2*c^10*x*y^2 + 4*a^6*b^4*c^2*x^2*z -      7*a^4*b^6*c^2*x^2*z + 2*a^2*b^8*c^2*x^2*z + b^10*c^2*x^2*z + 3*a^4*b^4*c^4*x^2*z +      4*a^2*b^6*c^4*x^2*z - 3*b^8*c^4*x^2*z - 6*a^2*b^4*c^6*x^2*z + 3*b^6*c^6*x^2*z - b^4*c^8*x^2*z -      a^10*c^2*y^2*z - 2*a^8*b^2*c^2*y^2*z + 7*a^6*b^4*c^2*y^2*z - 4*a^4*b^6*c^2*y^2*z +      3*a^8*c^4*y^2*z - 4*a^6*b^2*c^4*y^2*z - 3*a^4*b^4*c^4*y^2*z - 3*a^6*c^6*y^2*z +      6*a^4*b^2*c^6*y^2*z + a^4*c^8*y^2*z + a^8*b^4*x*z^2 - 3*a^6*b^6*x*z^2 + 3*a^4*b^8*x*z^2 -      a^2*b^10*x*z^2 + 6*a^6*b^4*c^2*x*z^2 - 4*a^4*b^6*c^2*x*z^2 - 2*a^2*b^8*c^2*x*z^2 -      3*a^4*b^4*c^4*x*z^2 + 7*a^2*b^6*c^4*x*z^2 - 4*a^2*b^4*c^6*x*z^2 + a^10*b^2*y*z^2 -      3*a^8*b^4*y*z^2 + 3*a^6*b^6*y*z^2 - a^4*b^8*y*z^2 + 2*a^8*b^2*c^2*y*z^2 + 4*a^6*b^4*c^2*y*z^2 -      6*a^4*b^6*c^2*y*z^2 - 7*a^6*b^2*c^4*y*z^2 + 3*a^4*b^4*c^4*y*z^2 + 4*a^4*b^2*c^6*y*z^2");
		exp("K524","a^6*c^2*x^2*y + 2*a^4*b^2*c^2*x^2*y - a^2*b^4*c^2*x^2*y - 2*b^6*c^2*x^2*y - a^4*c^4*x^2*y +      2*b^4*c^4*x^2*y - a^2*c^6*x^2*y - b^2*c^6*x^2*y + c^8*x^2*y + 2*a^6*c^2*x*y^2 + a^4*b^2*c^2*x*y^2 -      2*a^2*b^4*c^2*x*y^2 - b^6*c^2*x*y^2 - 2*a^4*c^4*x*y^2 + b^4*c^4*x*y^2 + a^2*c^6*x*y^2 +      b^2*c^6*x*y^2 - c^8*x*y^2 - a^6*b^2*x^2*z + a^4*b^4*x^2*z + a^2*b^6*x^2*z - b^8*x^2*z -      2*a^4*b^2*c^2*x^2*z + b^6*c^2*x^2*z + a^2*b^2*c^4*x^2*z - 2*b^4*c^4*x^2*z + 2*b^2*c^6*x^2*z +      a^8*y^2*z - a^6*b^2*y^2*z - a^4*b^4*y^2*z + a^2*b^6*y^2*z - a^6*c^2*y^2*z + 2*a^2*b^4*c^2*y^2*z +      2*a^4*c^4*y^2*z - a^2*b^2*c^4*y^2*z - 2*a^2*c^6*y^2*z - 2*a^6*b^2*x*z^2 + 2*a^4*b^4*x*z^2 -      a^2*b^6*x*z^2 + b^8*x*z^2 - a^4*b^2*c^2*x*z^2 - b^6*c^2*x*z^2 + 2*a^2*b^2*c^4*x*z^2 -      b^4*c^4*x*z^2 + b^2*c^6*x*z^2 - a^8*y*z^2 + a^6*b^2*y*z^2 - 2*a^4*b^4*y*z^2 + 2*a^2*b^6*y*z^2 +      a^6*c^2*y*z^2 + a^2*b^4*c^2*y*z^2 + a^4*c^4*y*z^2 - 2*a^2*b^2*c^4*y*z^2 - a^2*c^6*y*z^2");
		exp("K525","-2*a^4*c^2*x^2*y + a^2*b^2*c^2*x^2*y + b^4*c^2*x^2*y + 2*a^2*c^4*x^2*y - b^2*c^4*x^2*y -      a^4*c^2*x*y^2 - a^2*b^2*c^2*x*y^2 + 2*b^4*c^2*x*y^2 + a^2*c^4*x*y^2 - 2*b^2*c^4*x*y^2 +      2*a^4*b^2*x^2*z - 2*a^2*b^4*x^2*z - a^2*b^2*c^2*x^2*z + b^4*c^2*x^2*z - b^2*c^4*x^2*z +      4*a^4*b^2*x*y*z - 4*a^2*b^4*x*y*z - 4*a^4*c^2*x*y*z + 4*b^4*c^2*x*y*z + 4*a^2*c^4*x*y*z -      4*b^2*c^4*x*y*z + 2*a^4*b^2*y^2*z - 2*a^2*b^4*y^2*z - a^4*c^2*y^2*z + a^2*b^2*c^2*y^2*z +      a^2*c^4*y^2*z + a^4*b^2*x*z^2 - a^2*b^4*x*z^2 + a^2*b^2*c^2*x*z^2 + 2*b^4*c^2*x*z^2 -      2*b^2*c^4*x*z^2 + a^4*b^2*y*z^2 - a^2*b^4*y*z^2 - 2*a^4*c^2*y*z^2 - a^2*b^2*c^2*y*z^2 +      2*a^2*c^4*y*z^2");
		exp("K526","a^10*c^2*x^2*y - 2*a^8*b^2*c^2*x^2*y + 2*a^4*b^6*c^2*x^2*y - a^2*b^8*c^2*x^2*y -      3*a^8*c^4*x^2*y + 2*a^6*b^2*c^4*x^2*y + a^4*b^4*c^4*x^2*y + 3*a^6*c^6*x^2*y + a^4*b^2*c^6*x^2*y +      2*a^2*b^4*c^6*x^2*y - a^4*c^8*x^2*y - a^2*b^2*c^8*x^2*y + a^8*b^2*c^2*x*y^2 - 2*a^6*b^4*c^2*x*y^2 +      2*a^2*b^8*c^2*x*y^2 - b^10*c^2*x*y^2 - a^4*b^4*c^4*x*y^2 - 2*a^2*b^6*c^4*x*y^2 + 3*b^8*c^4*x*y^2 -      2*a^4*b^2*c^6*x*y^2 - a^2*b^4*c^6*x*y^2 - 3*b^6*c^6*x*y^2 + a^2*b^2*c^8*x*y^2 + b^4*c^8*x*y^2 -      a^10*b^2*x^2*z + 3*a^8*b^4*x^2*z - 3*a^6*b^6*x^2*z + a^4*b^8*x^2*z + 2*a^8*b^2*c^2*x^2*z -      2*a^6*b^4*c^2*x^2*z - a^4*b^6*c^2*x^2*z + a^2*b^8*c^2*x^2*z - a^4*b^4*c^4*x^2*z -      2*a^2*b^6*c^4*x^2*z - 2*a^4*b^2*c^6*x^2*z + a^2*b^2*c^8*x^2*z - a^10*b^2*x*y*z + 2*a^8*b^4*x*y*z -      2*a^4*b^8*x*y*z + a^2*b^10*x*y*z + a^10*c^2*x*y*z - 3*a^6*b^4*c^2*x*y*z + 3*a^4*b^6*c^2*x*y*z -      b^10*c^2*x*y*z - 2*a^8*c^4*x*y*z + 3*a^6*b^2*c^4*x*y*z - 3*a^2*b^6*c^4*x*y*z + 2*b^8*c^4*x*y*z -      3*a^4*b^2*c^6*x*y*z + 3*a^2*b^4*c^6*x*y*z + 2*a^4*c^8*x*y*z - 2*b^4*c^8*x*y*z - a^2*c^10*x*y*z +      b^2*c^10*x*y*z - a^8*b^4*y^2*z + 3*a^6*b^6*y^2*z - 3*a^4*b^8*y^2*z + a^2*b^10*y^2*z -      a^8*b^2*c^2*y^2*z + a^6*b^4*c^2*y^2*z + 2*a^4*b^6*c^2*y^2*z - 2*a^2*b^8*c^2*y^2*z +      2*a^6*b^2*c^4*y^2*z + a^4*b^4*c^4*y^2*z + 2*a^2*b^4*c^6*y^2*z - a^2*b^2*c^8*y^2*z -      a^8*b^2*c^2*x*z^2 + 2*a^4*b^6*c^2*x*z^2 - a^2*b^8*c^2*x*z^2 + 2*a^6*b^2*c^4*x*z^2 +      a^4*b^4*c^4*x*z^2 + a^2*b^6*c^4*x*z^2 - b^8*c^4*x*z^2 + 2*a^2*b^4*c^6*x*z^2 + 3*b^6*c^6*x*z^2 -      2*a^2*b^2*c^8*x*z^2 - 3*b^4*c^8*x*z^2 + b^2*c^10*x*z^2 + a^8*b^2*c^2*y*z^2 - 2*a^6*b^4*c^2*y*z^2 +      a^2*b^8*c^2*y*z^2 + a^8*c^4*y*z^2 - a^6*b^2*c^4*y*z^2 - a^4*b^4*c^4*y*z^2 - 2*a^2*b^6*c^4*y*z^2 -      3*a^6*c^6*y*z^2 - 2*a^4*b^2*c^6*y*z^2 + 3*a^4*c^8*y*z^2 + 2*a^2*b^2*c^8*y*z^2 - a^2*c^10*y*z^2 ");
		exp("K527","(-a^6)*c^2*x^2*y + a^2*b^4*c^2*x^2*y + 2*a^4*c^4*x^2*y + 2*a^2*b^2*c^4*x^2*y - a^2*c^6*x^2*y -      a^4*b^2*c^2*x*y^2 + b^6*c^2*x*y^2 - 2*a^2*b^2*c^4*x*y^2 - 2*b^4*c^4*x*y^2 + b^2*c^6*x*y^2 +      a^6*b^2*x^2*z - 2*a^4*b^4*x^2*z + a^2*b^6*x^2*z - 2*a^2*b^4*c^2*x^2*z - a^2*b^2*c^4*x^2*z -      a^6*b^2*y^2*z + 2*a^4*b^4*y^2*z - a^2*b^6*y^2*z + 2*a^4*b^2*c^2*y^2*z + a^2*b^2*c^4*y^2*z +      a^4*b^2*c^2*x*z^2 + 2*a^2*b^4*c^2*x*z^2 - b^6*c^2*x*z^2 + 2*b^4*c^4*x*z^2 - b^2*c^6*x*z^2 +      a^6*c^2*y*z^2 - 2*a^4*b^2*c^2*y*z^2 - a^2*b^4*c^2*y*z^2 - 2*a^4*c^4*y*z^2 + a^2*c^6*y*z^2");
		exp("K528","a^14*b^2*c^2*x^2*y - a^12*b^4*c^2*x^2*y - 9*a^10*b^6*c^2*x^2*y + 25*a^8*b^8*c^2*x^2*y -      25*a^6*b^10*c^2*x^2*y + 9*a^4*b^12*c^2*x^2*y + a^2*b^14*c^2*x^2*y - b^16*c^2*x^2*y +      a^14*c^4*x^2*y + 6*a^10*b^4*c^4*x^2*y - 20*a^8*b^6*c^4*x^2*y + 9*a^6*b^8*c^4*x^2*y +      12*a^4*b^10*c^4*x^2*y - 8*a^2*b^12*c^4*x^2*y - 5*a^12*c^6*x^2*y + 18*a^8*b^4*c^6*x^2*y +      28*a^6*b^6*c^6*x^2*y - 51*a^4*b^8*c^6*x^2*y + 10*b^12*c^6*x^2*y + 9*a^10*c^8*x^2*y -      20*a^8*b^2*c^8*x^2*y - 52*a^6*b^4*c^8*x^2*y + 36*a^4*b^6*c^8*x^2*y + 43*a^2*b^8*c^8*x^2*y -      16*b^10*c^8*x^2*y - 5*a^8*c^10*x^2*y + 45*a^6*b^2*c^10*x^2*y + 21*a^4*b^4*c^10*x^2*y -      59*a^2*b^6*c^10*x^2*y - 5*a^6*c^12*x^2*y - 36*a^4*b^2*c^12*x^2*y + 18*a^2*b^4*c^12*x^2*y +      16*b^6*c^12*x^2*y + 9*a^4*c^14*x^2*y + 10*a^2*b^2*c^14*x^2*y - 10*b^4*c^14*x^2*y -      5*a^2*c^16*x^2*y + c^18*x^2*y + a^16*c^2*x*y^2 - a^14*b^2*c^2*x*y^2 - 9*a^12*b^4*c^2*x*y^2 +      25*a^10*b^6*c^2*x*y^2 - 25*a^8*b^8*c^2*x*y^2 + 9*a^6*b^10*c^2*x*y^2 + a^4*b^12*c^2*x*y^2 -      a^2*b^14*c^2*x*y^2 + 8*a^12*b^2*c^4*x*y^2 - 12*a^10*b^4*c^4*x*y^2 - 9*a^8*b^6*c^4*x*y^2 +      20*a^6*b^8*c^4*x*y^2 - 6*a^4*b^10*c^4*x*y^2 - b^14*c^4*x*y^2 - 10*a^12*c^6*x*y^2 +      51*a^8*b^4*c^6*x*y^2 - 28*a^6*b^6*c^6*x*y^2 - 18*a^4*b^8*c^6*x*y^2 + 5*b^12*c^6*x*y^2 +      16*a^10*c^8*x*y^2 - 43*a^8*b^2*c^8*x*y^2 - 36*a^6*b^4*c^8*x*y^2 + 52*a^4*b^6*c^8*x*y^2 +      20*a^2*b^8*c^8*x*y^2 - 9*b^10*c^8*x*y^2 + 59*a^6*b^2*c^10*x*y^2 - 21*a^4*b^4*c^10*x*y^2 -      45*a^2*b^6*c^10*x*y^2 + 5*b^8*c^10*x*y^2 - 16*a^6*c^12*x*y^2 - 18*a^4*b^2*c^12*x*y^2 +      36*a^2*b^4*c^12*x*y^2 + 5*b^6*c^12*x*y^2 + 10*a^4*c^14*x*y^2 - 10*a^2*b^2*c^14*x*y^2 -      9*b^4*c^14*x*y^2 + 5*b^2*c^16*x*y^2 - c^18*x*y^2 - a^14*b^4*x^2*z + 5*a^12*b^6*x^2*z -      9*a^10*b^8*x^2*z + 5*a^8*b^10*x^2*z + 5*a^6*b^12*x^2*z - 9*a^4*b^14*x^2*z + 5*a^2*b^16*x^2*z -      b^18*x^2*z - a^14*b^2*c^2*x^2*z + 20*a^8*b^8*c^2*x^2*z - 45*a^6*b^10*c^2*x^2*z +      36*a^4*b^12*c^2*x^2*z - 10*a^2*b^14*c^2*x^2*z + a^12*b^2*c^4*x^2*z - 6*a^10*b^4*c^4*x^2*z -      18*a^8*b^6*c^4*x^2*z + 52*a^6*b^8*c^4*x^2*z - 21*a^4*b^10*c^4*x^2*z - 18*a^2*b^12*c^4*x^2*z +      10*b^14*c^4*x^2*z + 9*a^10*b^2*c^6*x^2*z + 20*a^8*b^4*c^6*x^2*z - 28*a^6*b^6*c^6*x^2*z -      36*a^4*b^8*c^6*x^2*z + 59*a^2*b^10*c^6*x^2*z - 16*b^12*c^6*x^2*z - 25*a^8*b^2*c^8*x^2*z -      9*a^6*b^4*c^8*x^2*z + 51*a^4*b^6*c^8*x^2*z - 43*a^2*b^8*c^8*x^2*z + 25*a^6*b^2*c^10*x^2*z -      12*a^4*b^4*c^10*x^2*z + 16*b^8*c^10*x^2*z - 9*a^4*b^2*c^12*x^2*z + 8*a^2*b^4*c^12*x^2*z -      10*b^6*c^12*x^2*z - a^2*b^2*c^14*x^2*z + b^2*c^16*x^2*z + a^18*y^2*z - 5*a^16*b^2*y^2*z +      9*a^14*b^4*y^2*z - 5*a^12*b^6*y^2*z - 5*a^10*b^8*y^2*z + 9*a^8*b^10*y^2*z - 5*a^6*b^12*y^2*z +      a^4*b^14*y^2*z + 10*a^14*b^2*c^2*y^2*z - 36*a^12*b^4*c^2*y^2*z + 45*a^10*b^6*c^2*y^2*z -      20*a^8*b^8*c^2*y^2*z + a^2*b^14*c^2*y^2*z - 10*a^14*c^4*y^2*z + 18*a^12*b^2*c^4*y^2*z +      21*a^10*b^4*c^4*y^2*z - 52*a^8*b^6*c^4*y^2*z + 18*a^6*b^8*c^4*y^2*z + 6*a^4*b^10*c^4*y^2*z -      a^2*b^12*c^4*y^2*z + 16*a^12*c^6*y^2*z - 59*a^10*b^2*c^6*y^2*z + 36*a^8*b^4*c^6*y^2*z +      28*a^6*b^6*c^6*y^2*z - 20*a^4*b^8*c^6*y^2*z - 9*a^2*b^10*c^6*y^2*z + 43*a^8*b^2*c^8*y^2*z -      51*a^6*b^4*c^8*y^2*z + 9*a^4*b^6*c^8*y^2*z + 25*a^2*b^8*c^8*y^2*z - 16*a^8*c^10*y^2*z +      12*a^4*b^4*c^10*y^2*z - 25*a^2*b^6*c^10*y^2*z + 10*a^6*c^12*y^2*z - 8*a^4*b^2*c^12*y^2*z +      9*a^2*b^4*c^12*y^2*z + a^2*b^2*c^14*y^2*z - a^2*c^16*y^2*z - a^16*b^2*x*z^2 + 10*a^12*b^6*x*z^2 -      16*a^10*b^8*x*z^2 + 16*a^6*b^12*x*z^2 - 10*a^4*b^14*x*z^2 + b^18*x*z^2 + a^14*b^2*c^2*x*z^2 -      8*a^12*b^4*c^2*x*z^2 + 43*a^8*b^8*c^2*x*z^2 - 59*a^6*b^10*c^2*x*z^2 + 18*a^4*b^12*c^2*x*z^2 +      10*a^2*b^14*c^2*x*z^2 - 5*b^16*c^2*x*z^2 + 9*a^12*b^2*c^4*x*z^2 + 12*a^10*b^4*c^4*x*z^2 -      51*a^8*b^6*c^4*x*z^2 + 36*a^6*b^8*c^4*x*z^2 + 21*a^4*b^10*c^4*x*z^2 - 36*a^2*b^12*c^4*x*z^2 +      9*b^14*c^4*x*z^2 - 25*a^10*b^2*c^6*x*z^2 + 9*a^8*b^4*c^6*x*z^2 + 28*a^6*b^6*c^6*x*z^2 -      52*a^4*b^8*c^6*x*z^2 + 45*a^2*b^10*c^6*x*z^2 - 5*b^12*c^6*x*z^2 + 25*a^8*b^2*c^8*x*z^2 -      20*a^6*b^4*c^8*x*z^2 + 18*a^4*b^6*c^8*x*z^2 - 20*a^2*b^8*c^8*x*z^2 - 5*b^10*c^8*x*z^2 -      9*a^6*b^2*c^10*x*z^2 + 6*a^4*b^4*c^10*x*z^2 + 9*b^8*c^10*x*z^2 - a^4*b^2*c^12*x*z^2 -      5*b^6*c^12*x*z^2 + a^2*b^2*c^14*x*z^2 + b^4*c^14*x*z^2 - a^18*y*z^2 + 10*a^14*b^4*y*z^2 -      16*a^12*b^6*y*z^2 + 16*a^8*b^10*y*z^2 - 10*a^6*b^12*y*z^2 + a^2*b^16*y*z^2 + 5*a^16*c^2*y*z^2 -      10*a^14*b^2*c^2*y*z^2 - 18*a^12*b^4*c^2*y*z^2 + 59*a^10*b^6*c^2*y*z^2 - 43*a^8*b^8*c^2*y*z^2 +      8*a^4*b^12*c^2*y*z^2 - a^2*b^14*c^2*y*z^2 - 9*a^14*c^4*y*z^2 + 36*a^12*b^2*c^4*y*z^2 -      21*a^10*b^4*c^4*y*z^2 - 36*a^8*b^6*c^4*y*z^2 + 51*a^6*b^8*c^4*y*z^2 - 12*a^4*b^10*c^4*y*z^2 -      9*a^2*b^12*c^4*y*z^2 + 5*a^12*c^6*y*z^2 - 45*a^10*b^2*c^6*y*z^2 + 52*a^8*b^4*c^6*y*z^2 -      28*a^6*b^6*c^6*y*z^2 - 9*a^4*b^8*c^6*y*z^2 + 25*a^2*b^10*c^6*y*z^2 + 5*a^10*c^8*y*z^2 +      20*a^8*b^2*c^8*y*z^2 - 18*a^6*b^4*c^8*y*z^2 + 20*a^4*b^6*c^8*y*z^2 - 25*a^2*b^8*c^8*y*z^2 -      9*a^8*c^10*y*z^2 - 6*a^4*b^4*c^10*y*z^2 + 9*a^2*b^6*c^10*y*z^2 + 5*a^6*c^12*y*z^2 +      a^2*b^4*c^12*y*z^2 - a^4*c^14*y*z^2 - a^2*b^2*c^14*y*z^2");
		exp("K529","a^5*c*x^2*y - a^4*b*c*x^2*y - 2*a^3*b^2*c*x^2*y + 2*a^2*b^3*c*x^2*y + a*b^4*c*x^2*y -      b^5*c*x^2*y - a^4*c^2*x^2*y + 3*a^3*b*c^2*x^2*y - 3*a*b^3*c^2*x^2*y + b^4*c^2*x^2*y -      a^3*c^3*x^2*y - 2*a^2*b*c^3*x^2*y + 3*a*b^2*c^3*x^2*y + a^2*c^4*x^2*y - a*b*c^4*x^2*y -      b^2*c^4*x^2*y + b*c^5*x^2*y + a^5*c*x*y^2 - a^4*b*c*x*y^2 - 2*a^3*b^2*c*x*y^2 + 2*a^2*b^3*c*x*y^2 +      a*b^4*c*x*y^2 - b^5*c*x*y^2 - a^4*c^2*x*y^2 + 3*a^3*b*c^2*x*y^2 - 3*a*b^3*c^2*x*y^2 +      b^4*c^2*x*y^2 - 3*a^2*b*c^3*x*y^2 + 2*a*b^2*c^3*x*y^2 + b^3*c^3*x*y^2 + a^2*c^4*x*y^2 +      a*b*c^4*x*y^2 - b^2*c^4*x*y^2 - a*c^5*x*y^2 - a^5*b*x^2*z + a^4*b^2*x^2*z + a^3*b^3*x^2*z -      a^2*b^4*x^2*z + a^4*b*c*x^2*z - 3*a^3*b^2*c*x^2*z + 2*a^2*b^3*c*x^2*z + a*b^4*c*x^2*z -      b^5*c*x^2*z + 2*a^3*b*c^2*x^2*z - 3*a*b^3*c^2*x^2*z + b^4*c^2*x^2*z - 2*a^2*b*c^3*x^2*z +      3*a*b^2*c^3*x^2*z - a*b*c^4*x^2*z - b^2*c^4*x^2*z + b*c^5*x^2*z - a^5*b*x*y*z + 2*a^4*b^2*x*y*z -      2*a^2*b^4*x*y*z + a*b^5*x*y*z + a^5*c*x*y*z - 5*a^3*b^2*c*x*y*z + 5*a^2*b^3*c*x*y*z - b^5*c*x*y*z -      2*a^4*c^2*x*y*z + 5*a^3*b*c^2*x*y*z - 5*a*b^3*c^2*x*y*z + 2*b^4*c^2*x*y*z - 5*a^2*b*c^3*x*y*z +      5*a*b^2*c^3*x*y*z + 2*a^2*c^4*x*y*z - 2*b^2*c^4*x*y*z - a*c^5*x*y*z + b*c^5*x*y*z + a^4*b^2*y^2*z -      a^3*b^3*y^2*z - a^2*b^4*y^2*z + a*b^5*y^2*z + a^5*c*y^2*z - a^4*b*c*y^2*z - 2*a^3*b^2*c*y^2*z +      3*a^2*b^3*c*y^2*z - a*b^4*c*y^2*z - a^4*c^2*y^2*z + 3*a^3*b*c^2*y^2*z - 2*a*b^3*c^2*y^2*z -      3*a^2*b*c^3*y^2*z + 2*a*b^2*c^3*y^2*z + a^2*c^4*y^2*z + a*b*c^4*y^2*z - a*c^5*y^2*z - a^5*b*x*z^2 +      a^4*b^2*x*z^2 - a^2*b^4*x*z^2 + a*b^5*x*z^2 + a^4*b*c*x*z^2 - 3*a^3*b^2*c*x*z^2 +      3*a^2*b^3*c*x*z^2 - a*b^4*c*x*z^2 + 2*a^3*b*c^2*x*z^2 - 2*a*b^3*c^2*x*z^2 + b^4*c^2*x*z^2 -      2*a^2*b*c^3*x*z^2 + 3*a*b^2*c^3*x*z^2 - b^3*c^3*x*z^2 - a*b*c^4*x*z^2 - b^2*c^4*x*z^2 +      b*c^5*x*z^2 - a^5*b*y*z^2 + a^4*b^2*y*z^2 - a^2*b^4*y*z^2 + a*b^5*y*z^2 + a^4*b*c*y*z^2 -      3*a^3*b^2*c*y*z^2 + 3*a^2*b^3*c*y*z^2 - a*b^4*c*y*z^2 - a^4*c^2*y*z^2 + 2*a^3*b*c^2*y*z^2 -      2*a*b^3*c^2*y*z^2 + a^3*c^3*y*z^2 - 3*a^2*b*c^3*y*z^2 + 2*a*b^2*c^3*y*z^2 + a^2*c^4*y*z^2 +      a*b*c^4*y*z^2 - a*c^5*y*z^2");
		exp("K530","a^6*x^2*y - 3*a^4*b^2*x^2*y + 3*a^2*b^4*x^2*y - b^6*x^2*y + a^4*c^2*x^2*y + a^2*b^2*c^2*x^2*y -      2*b^4*c^2*x^2*y - 3*a^2*c^4*x^2*y + 2*b^2*c^4*x^2*y + c^6*x^2*y + a^6*x*y^2 - 3*a^4*b^2*x*y^2 +      3*a^2*b^4*x*y^2 - b^6*x*y^2 + 2*a^4*c^2*x*y^2 - a^2*b^2*c^2*x*y^2 - b^4*c^2*x*y^2 -      2*a^2*c^4*x*y^2 + 3*b^2*c^4*x*y^2 - c^6*x*y^2 - a^6*x^2*z - a^4*b^2*x^2*z + 3*a^2*b^4*x^2*z -      b^6*x^2*z + 3*a^4*c^2*x^2*z - a^2*b^2*c^2*x^2*z - 2*b^4*c^2*x^2*z - 3*a^2*c^4*x^2*z +      2*b^2*c^4*x^2*z + c^6*x^2*z - 4*a^4*b^2*x*y*z + 4*a^2*b^4*x*y*z + 4*a^4*c^2*x*y*z -      4*b^4*c^2*x*y*z - 4*a^2*c^4*x*y*z + 4*b^2*c^4*x*y*z + a^6*y^2*z - 3*a^4*b^2*y^2*z + a^2*b^4*y^2*z +      b^6*y^2*z + 2*a^4*c^2*y^2*z + a^2*b^2*c^2*y^2*z - 3*b^4*c^2*y^2*z - 2*a^2*c^4*y^2*z +      3*b^2*c^4*y^2*z - c^6*y^2*z - a^6*x*z^2 - 2*a^4*b^2*x*z^2 + 2*a^2*b^4*x*z^2 + b^6*x*z^2 +      3*a^4*c^2*x*z^2 + a^2*b^2*c^2*x*z^2 - 3*b^4*c^2*x*z^2 - 3*a^2*c^4*x*z^2 + b^2*c^4*x*z^2 +      c^6*x*z^2 - a^6*y*z^2 - 2*a^4*b^2*y*z^2 + 2*a^2*b^4*y*z^2 + b^6*y*z^2 + 3*a^4*c^2*y*z^2 -      a^2*b^2*c^2*y*z^2 - 3*b^4*c^2*y*z^2 - a^2*c^4*y*z^2 + 3*b^2*c^4*y*z^2 - c^6*y*z^2");
		exp("K531","(-a^4)*c^2*x^2*y + b^4*c^2*x^2*y - b^2*c^4*x^2*y + c^6*x^2*y - a^4*c^2*x*y^2 + b^4*c^2*x*y^2 +      a^2*c^4*x*y^2 - c^6*x*y^2 + a^4*b^2*x^2*z - b^6*x^2*z + b^4*c^2*x^2*z - b^2*c^4*x^2*z + a^6*y^2*z -      a^2*b^4*y^2*z - a^4*c^2*y^2*z + a^2*c^4*y^2*z + a^4*b^2*x*z^2 - a^2*b^4*x*z^2 + b^6*x*z^2 -      b^2*c^4*x*z^2 - a^6*y*z^2 + a^4*b^2*y*z^2 - a^2*b^4*y*z^2 + a^2*c^4*y*z^2");
		exp("K532","(-a^4)*b^2*c^6*x^2*y + b^4*c^8*x^2*y + a^2*b^4*c^6*x*y^2 - a^4*c^8*x*y^2 + a^4*b^6*c^2*x^2*z -      b^8*c^4*x^2*z - a^6*b^4*c^2*y^2*z + a^8*c^4*y^2*z + a^4*b^8*x*z^2 - a^2*b^6*c^4*x*z^2 -      a^8*b^4*y*z^2 + a^6*b^2*c^4*y*z^2");
		exp("K533","(-a^6)*x^2*y + a^4*b^2*x^2*y + a^2*b^4*x^2*y - b^6*x^2*y + a^4*c^2*x^2*y - a^2*b^2*c^2*x^2*y +      a^2*c^4*x^2*y - c^6*x^2*y + a^6*x*y^2 - a^4*b^2*x*y^2 - a^2*b^4*x*y^2 + b^6*x*y^2 +      a^2*b^2*c^2*x*y^2 - b^4*c^2*x*y^2 - b^2*c^4*x*y^2 + c^6*x*y^2 + a^6*x^2*z - a^4*b^2*x^2*z -      a^2*b^4*x^2*z + b^6*x^2*z - a^4*c^2*x^2*z + a^2*b^2*c^2*x^2*z - a^2*c^4*x^2*z + c^6*x^2*z -      a^6*y^2*z + a^4*b^2*y^2*z + a^2*b^4*y^2*z - b^6*y^2*z - a^2*b^2*c^2*y^2*z + b^4*c^2*y^2*z +      b^2*c^4*y^2*z - c^6*y^2*z - a^6*x*z^2 - b^6*x*z^2 + a^4*c^2*x*z^2 - a^2*b^2*c^2*x*z^2 +      b^4*c^2*x*z^2 + a^2*c^4*x*z^2 + b^2*c^4*x*z^2 - c^6*x*z^2 + a^6*y*z^2 + b^6*y*z^2 - a^4*c^2*y*z^2 +      a^2*b^2*c^2*y*z^2 - b^4*c^2*y*z^2 - a^2*c^4*y*z^2 - b^2*c^4*y*z^2 + c^6*y*z^2");
		exp("K534","(-a^4)*c^2*x^2*y + a^2*b^2*c^2*x^2*y + 2*b^4*c^2*x^2*y - 3*b^2*c^4*x^2*y + c^6*x^2*y -      2*a^4*c^2*x*y^2 - a^2*b^2*c^2*x*y^2 + b^4*c^2*x*y^2 + 3*a^2*c^4*x*y^2 - c^6*x*y^2 + a^4*b^2*x^2*z -      b^6*x^2*z - a^2*b^2*c^2*x^2*z + 3*b^4*c^2*x^2*z - 2*b^2*c^4*x^2*z + a^6*y^2*z - a^2*b^4*y^2*z -      3*a^4*c^2*y^2*z + a^2*b^2*c^2*y^2*z + 2*a^2*c^4*y^2*z + 2*a^4*b^2*x*z^2 - 3*a^2*b^4*x*z^2 +      b^6*x*z^2 + a^2*b^2*c^2*x*z^2 - b^2*c^4*x*z^2 - a^6*y*z^2 + 3*a^4*b^2*y*z^2 - 2*a^2*b^4*y*z^2 -      a^2*b^2*c^2*y*z^2 + a^2*c^4*y*z^2");
		exp("K535","(-a^4)*c^2*x^2*y + 3*a^2*b^2*c^2*x^2*y - 2*b^4*c^2*x^2*y - b^2*c^4*x^2*y + c^6*x^2*y +      2*a^4*c^2*x*y^2 - 3*a^2*b^2*c^2*x*y^2 + b^4*c^2*x*y^2 + a^2*c^4*x*y^2 - c^6*x*y^2 + a^4*b^2*x^2*z -      b^6*x^2*z - 3*a^2*b^2*c^2*x^2*z + b^4*c^2*x^2*z + 2*b^2*c^4*x^2*z + a^6*y^2*z - a^2*b^4*y^2*z -      a^4*c^2*y^2*z + 3*a^2*b^2*c^2*y^2*z - 2*a^2*c^4*y^2*z - 2*a^4*b^2*x*z^2 - a^2*b^4*x*z^2 +      b^6*x*z^2 + 3*a^2*b^2*c^2*x*z^2 - b^2*c^4*x*z^2 - a^6*y*z^2 + a^4*b^2*y*z^2 + 2*a^2*b^4*y*z^2 -      3*a^2*b^2*c^2*y*z^2 + a^2*c^4*y*z^2");
		exp("K536","a^6*c^2*x^2*y + 4*a^4*b^2*c^2*x^2*y - a^2*b^4*c^2*x^2*y - 4*b^6*c^2*x^2*y - 3*a^4*c^4*x^2*y +      3*b^4*c^4*x^2*y - a^2*c^6*x^2*y - 2*b^2*c^6*x^2*y + 3*c^8*x^2*y + 4*a^6*c^2*x*y^2 +      a^4*b^2*c^2*x*y^2 - 4*a^2*b^4*c^2*x*y^2 - b^6*c^2*x*y^2 - 3*a^4*c^4*x*y^2 + 3*b^4*c^4*x*y^2 +      2*a^2*c^6*x*y^2 + b^2*c^6*x*y^2 - 3*c^8*x*y^2 - a^6*b^2*x^2*z + 3*a^4*b^4*x^2*z + a^2*b^6*x^2*z -      3*b^8*x^2*z - 4*a^4*b^2*c^2*x^2*z + 2*b^6*c^2*x^2*z + a^2*b^2*c^4*x^2*z - 3*b^4*c^4*x^2*z +      4*b^2*c^6*x^2*z + 4*a^6*b^2*x*y*z - 4*a^2*b^6*x*y*z - 4*a^6*c^2*x*y*z + 4*b^6*c^2*x*y*z +      4*a^2*c^6*x*y*z - 4*b^2*c^6*x*y*z + 3*a^8*y^2*z - a^6*b^2*y^2*z - 3*a^4*b^4*y^2*z + a^2*b^6*y^2*z -      2*a^6*c^2*y^2*z + 4*a^2*b^4*c^2*y^2*z + 3*a^4*c^4*y^2*z - a^2*b^2*c^4*y^2*z - 4*a^2*c^6*y^2*z -      4*a^6*b^2*x*z^2 + 3*a^4*b^4*x*z^2 - 2*a^2*b^6*x*z^2 + 3*b^8*x*z^2 - a^4*b^2*c^2*x*z^2 -      b^6*c^2*x*z^2 + 4*a^2*b^2*c^4*x*z^2 - 3*b^4*c^4*x*z^2 + b^2*c^6*x*z^2 - 3*a^8*y*z^2 +      2*a^6*b^2*y*z^2 - 3*a^4*b^4*y*z^2 + 4*a^2*b^6*y*z^2 + a^6*c^2*y*z^2 + a^2*b^4*c^2*y*z^2 +      3*a^4*c^4*y*z^2 - 4*a^2*b^2*c^4*y*z^2 - a^2*c^6*y*z^2");
		exp("K537","3*a^6*c^2*x^2*y - 2*a^4*b^2*c^2*x^2*y + 3*a^2*b^4*c^2*x^2*y - 4*b^6*c^2*x^2*y - a^4*c^4*x^2*y -      b^4*c^4*x^2*y - 3*a^2*c^6*x^2*y + 4*b^2*c^6*x^2*y + c^8*x^2*y + 4*a^6*c^2*x*y^2 -      3*a^4*b^2*c^2*x*y^2 + 2*a^2*b^4*c^2*x*y^2 - 3*b^6*c^2*x*y^2 + a^4*c^4*x*y^2 + b^4*c^4*x*y^2 -      4*a^2*c^6*x*y^2 + 3*b^2*c^6*x*y^2 - c^8*x*y^2 - 3*a^6*b^2*x^2*z + a^4*b^4*x^2*z + 3*a^2*b^6*x^2*z -      b^8*x^2*z + 2*a^4*b^2*c^2*x^2*z - 4*b^6*c^2*x^2*z - 3*a^2*b^2*c^4*x^2*z + b^4*c^4*x^2*z +      4*b^2*c^6*x^2*z - 4*a^6*b^2*x*y*z + 4*a^2*b^6*x*y*z + 4*a^6*c^2*x*y*z - 4*b^6*c^2*x*y*z -      4*a^2*c^6*x*y*z + 4*b^2*c^6*x*y*z + a^8*y^2*z - 3*a^6*b^2*y^2*z - a^4*b^4*y^2*z + 3*a^2*b^6*y^2*z +      4*a^6*c^2*y^2*z - 2*a^2*b^4*c^2*y^2*z - a^4*c^4*y^2*z + 3*a^2*b^2*c^4*y^2*z - 4*a^2*c^6*y^2*z -      4*a^6*b^2*x*z^2 - a^4*b^4*x*z^2 + 4*a^2*b^6*x*z^2 + b^8*x*z^2 + 3*a^4*b^2*c^2*x*z^2 -      3*b^6*c^2*x*z^2 - 2*a^2*b^2*c^4*x*z^2 - b^4*c^4*x*z^2 + 3*b^2*c^6*x*z^2 - a^8*y*z^2 -      4*a^6*b^2*y*z^2 + a^4*b^4*y*z^2 + 4*a^2*b^6*y*z^2 + 3*a^6*c^2*y*z^2 - 3*a^2*b^4*c^2*y*z^2 +      a^4*c^4*y*z^2 + 2*a^2*b^2*c^4*y*z^2 - 3*a^2*c^6*y*z^2");
		exp("K538","a^4*c^2*x^2*y + a^2*b^2*c^2*x^2*y + b^2*c^4*x^2*y - c^6*x^2*y - a^2*b^2*c^2*x*y^2 -      b^4*c^2*x*y^2 - a^2*c^4*x*y^2 + c^6*x*y^2 - a^4*b^2*x^2*z + b^6*x^2*z - a^2*b^2*c^2*x^2*z -      b^4*c^2*x^2*z - a^6*y^2*z + a^2*b^4*y^2*z + a^4*c^2*y^2*z + a^2*b^2*c^2*y^2*z + a^2*b^4*x*z^2 -      b^6*x*z^2 + a^2*b^2*c^2*x*z^2 + b^2*c^4*x*z^2 + a^6*y*z^2 - a^4*b^2*y*z^2 - a^2*b^2*c^2*y*z^2 -      a^2*c^4*y*z^2");
		exp("K539","(-a^4)*c^2*x^2*y + a^2*b^2*c^2*x^2*y + b^2*c^4*x^2*y + c^6*x^2*y - a^2*b^2*c^2*x*y^2 +      b^4*c^2*x*y^2 - a^2*c^4*x*y^2 - c^6*x*y^2 + a^4*b^2*x^2*z - b^6*x^2*z - a^2*b^2*c^2*x^2*z -      b^4*c^2*x^2*z + a^6*y^2*z - a^2*b^4*y^2*z + a^4*c^2*y^2*z + a^2*b^2*c^2*y^2*z + a^2*b^4*x*z^2 +      b^6*x*z^2 + a^2*b^2*c^2*x*z^2 - b^2*c^4*x*z^2 - a^6*y*z^2 - a^4*b^2*y*z^2 - a^2*b^2*c^2*y*z^2 +      a^2*c^4*y*z^2");
		exp("K540","a^10*x^2*y - 3*a^6*b^4*x^2*y + 5*a^4*b^6*x^2*y - 6*a^2*b^8*x^2*y + 3*b^10*x^2*y - a^8*c^2*x^2*y +      2*a^4*b^4*c^2*x^2*y + 2*a^2*b^6*c^2*x^2*y - 3*b^8*c^2*x^2*y - a^4*b^2*c^4*x^2*y -      a^2*b^4*c^4*x^2*y + 2*b^6*c^4*x^2*y - a^4*c^6*x^2*y + 2*a^2*b^2*c^6*x^2*y - b^4*c^6*x^2*y +      a^2*c^8*x^2*y - b^2*c^8*x^2*y + 3*a^10*x*y^2 - 6*a^8*b^2*x*y^2 + 5*a^6*b^4*x*y^2 -      3*a^4*b^6*x*y^2 + b^10*x*y^2 - 3*a^8*c^2*x*y^2 + 2*a^6*b^2*c^2*x*y^2 + 2*a^4*b^4*c^2*x*y^2 -      b^8*c^2*x*y^2 + 2*a^6*c^4*x*y^2 - a^4*b^2*c^4*x*y^2 - a^2*b^4*c^4*x*y^2 - a^4*c^6*x*y^2 +      2*a^2*b^2*c^6*x*y^2 - b^4*c^6*x*y^2 - a^2*c^8*x*y^2 + b^2*c^8*x*y^2 + a^10*x^2*z - a^8*b^2*x^2*z -      a^4*b^6*x^2*z + a^2*b^8*x^2*z - a^4*b^4*c^2*x^2*z + 2*a^2*b^6*c^2*x^2*z - b^8*c^2*x^2*z -      3*a^6*c^4*x^2*z + 2*a^4*b^2*c^4*x^2*z - a^2*b^4*c^4*x^2*z - b^6*c^4*x^2*z + 5*a^4*c^6*x^2*z +      2*a^2*b^2*c^6*x^2*z + 2*b^4*c^6*x^2*z - 6*a^2*c^8*x^2*z - 3*b^2*c^8*x^2*z + 3*c^10*x^2*z +      a^8*b^2*y^2*z - a^6*b^4*y^2*z - a^2*b^8*y^2*z + b^10*y^2*z - a^8*c^2*y^2*z + 2*a^6*b^2*c^2*y^2*z -      a^4*b^4*c^2*y^2*z - a^6*c^4*y^2*z - a^4*b^2*c^4*y^2*z + 2*a^2*b^4*c^4*y^2*z - 3*b^6*c^4*y^2*z +      2*a^4*c^6*y^2*z + 2*a^2*b^2*c^6*y^2*z + 5*b^4*c^6*y^2*z - 3*a^2*c^8*y^2*z - 6*b^2*c^8*y^2*z +      3*c^10*y^2*z + 3*a^10*x*z^2 - 3*a^8*b^2*x*z^2 + 2*a^6*b^4*x*z^2 - a^4*b^6*x*z^2 - a^2*b^8*x*z^2 -      6*a^8*c^2*x*z^2 + 2*a^6*b^2*c^2*x*z^2 - a^4*b^4*c^2*x*z^2 + 2*a^2*b^6*c^2*x*z^2 + b^8*c^2*x*z^2 +      5*a^6*c^4*x*z^2 + 2*a^4*b^2*c^4*x*z^2 - a^2*b^4*c^4*x*z^2 - b^6*c^4*x*z^2 - 3*a^4*c^6*x*z^2 -      b^2*c^8*x*z^2 + c^10*x*z^2 - a^8*b^2*y*z^2 - a^6*b^4*y*z^2 + 2*a^4*b^6*y*z^2 - 3*a^2*b^8*y*z^2 +      3*b^10*y*z^2 + a^8*c^2*y*z^2 + 2*a^6*b^2*c^2*y*z^2 - a^4*b^4*c^2*y*z^2 + 2*a^2*b^6*c^2*y*z^2 -      6*b^8*c^2*y*z^2 - a^6*c^4*y*z^2 - a^4*b^2*c^4*y*z^2 + 2*a^2*b^4*c^4*y*z^2 + 5*b^6*c^4*y*z^2 -      3*b^4*c^6*y*z^2 - a^2*c^8*y*z^2 + c^10*y*z^2");
		exp("K541","a^14*c^2*x^2*y + 3*a^12*b^2*c^2*x^2*y - 13*a^10*b^4*c^2*x^2*y + 5*a^8*b^6*c^2*x^2*y +      15*a^6*b^8*c^2*x^2*y - 11*a^4*b^10*c^2*x^2*y - 3*a^2*b^12*c^2*x^2*y + 3*b^14*c^2*x^2*y -      4*a^12*c^4*x^2*y + 5*a^10*b^2*c^4*x^2*y + 30*a^8*b^4*c^4*x^2*y - 46*a^6*b^6*c^4*x^2*y -      4*a^4*b^8*c^4*x^2*y + 25*a^2*b^10*c^4*x^2*y - 6*b^12*c^4*x^2*y + 5*a^10*c^6*x^2*y -      30*a^8*b^2*c^6*x^2*y + 5*a^6*b^4*c^6*x^2*y + 59*a^4*b^6*c^6*x^2*y - 34*a^2*b^8*c^6*x^2*y -      5*b^10*c^6*x^2*y + 29*a^6*b^2*c^8*x^2*y - 46*a^4*b^4*c^8*x^2*y - 3*a^2*b^6*c^8*x^2*y +      20*b^8*c^8*x^2*y - 5*a^6*c^10*x^2*y - 2*a^4*b^2*c^10*x^2*y + 22*a^2*b^4*c^10*x^2*y -      15*b^6*c^10*x^2*y + 4*a^4*c^12*x^2*y - 6*a^2*b^2*c^12*x^2*y + 2*b^4*c^12*x^2*y - a^2*c^14*x^2*y +      b^2*c^14*x^2*y + 3*a^14*c^2*x*y^2 - 3*a^12*b^2*c^2*x*y^2 - 11*a^10*b^4*c^2*x*y^2 +      15*a^8*b^6*c^2*x*y^2 + 5*a^6*b^8*c^2*x*y^2 - 13*a^4*b^10*c^2*x*y^2 + 3*a^2*b^12*c^2*x*y^2 +      b^14*c^2*x*y^2 - 6*a^12*c^4*x*y^2 + 25*a^10*b^2*c^4*x*y^2 - 4*a^8*b^4*c^4*x*y^2 -      46*a^6*b^6*c^4*x*y^2 + 30*a^4*b^8*c^4*x*y^2 + 5*a^2*b^10*c^4*x*y^2 - 4*b^12*c^4*x*y^2 -      5*a^10*c^6*x*y^2 - 34*a^8*b^2*c^6*x*y^2 + 59*a^6*b^4*c^6*x*y^2 + 5*a^4*b^6*c^6*x*y^2 -      30*a^2*b^8*c^6*x*y^2 + 5*b^10*c^6*x*y^2 + 20*a^8*c^8*x*y^2 - 3*a^6*b^2*c^8*x*y^2 -      46*a^4*b^4*c^8*x*y^2 + 29*a^2*b^6*c^8*x*y^2 - 15*a^6*c^10*x*y^2 + 22*a^4*b^2*c^10*x*y^2 -      2*a^2*b^4*c^10*x*y^2 - 5*b^6*c^10*x*y^2 + 2*a^4*c^12*x*y^2 - 6*a^2*b^2*c^12*x*y^2 +      4*b^4*c^12*x*y^2 + a^2*c^14*x*y^2 - b^2*c^14*x*y^2 + a^14*b^2*x^2*z - 4*a^12*b^4*x^2*z +      5*a^10*b^6*x^2*z - 5*a^6*b^10*x^2*z + 4*a^4*b^12*x^2*z - a^2*b^14*x^2*z + 3*a^12*b^2*c^2*x^2*z +      5*a^10*b^4*c^2*x^2*z - 30*a^8*b^6*c^2*x^2*z + 29*a^6*b^8*c^2*x^2*z - 2*a^4*b^10*c^2*x^2*z -      6*a^2*b^12*c^2*x^2*z + b^14*c^2*x^2*z - 13*a^10*b^2*c^4*x^2*z + 30*a^8*b^4*c^4*x^2*z +      5*a^6*b^6*c^4*x^2*z - 46*a^4*b^8*c^4*x^2*z + 22*a^2*b^10*c^4*x^2*z + 2*b^12*c^4*x^2*z +      5*a^8*b^2*c^6*x^2*z - 46*a^6*b^4*c^6*x^2*z + 59*a^4*b^6*c^6*x^2*z - 3*a^2*b^8*c^6*x^2*z -      15*b^10*c^6*x^2*z + 15*a^6*b^2*c^8*x^2*z - 4*a^4*b^4*c^8*x^2*z - 34*a^2*b^6*c^8*x^2*z +      20*b^8*c^8*x^2*z - 11*a^4*b^2*c^10*x^2*z + 25*a^2*b^4*c^10*x^2*z - 5*b^6*c^10*x^2*z -      3*a^2*b^2*c^12*x^2*z - 6*b^4*c^12*x^2*z + 3*b^2*c^14*x^2*z - a^14*b^2*y^2*z + 4*a^12*b^4*y^2*z -      5*a^10*b^6*y^2*z + 5*a^6*b^10*y^2*z - 4*a^4*b^12*y^2*z + a^2*b^14*y^2*z + a^14*c^2*y^2*z -      6*a^12*b^2*c^2*y^2*z - 2*a^10*b^4*c^2*y^2*z + 29*a^8*b^6*c^2*y^2*z - 30*a^6*b^8*c^2*y^2*z +      5*a^4*b^10*c^2*y^2*z + 3*a^2*b^12*c^2*y^2*z + 2*a^12*c^4*y^2*z + 22*a^10*b^2*c^4*y^2*z -      46*a^8*b^4*c^4*y^2*z + 5*a^6*b^6*c^4*y^2*z + 30*a^4*b^8*c^4*y^2*z - 13*a^2*b^10*c^4*y^2*z -      15*a^10*c^6*y^2*z - 3*a^8*b^2*c^6*y^2*z + 59*a^6*b^4*c^6*y^2*z - 46*a^4*b^6*c^6*y^2*z +      5*a^2*b^8*c^6*y^2*z + 20*a^8*c^8*y^2*z - 34*a^6*b^2*c^8*y^2*z - 4*a^4*b^4*c^8*y^2*z +      15*a^2*b^6*c^8*y^2*z - 5*a^6*c^10*y^2*z + 25*a^4*b^2*c^10*y^2*z - 11*a^2*b^4*c^10*y^2*z -      6*a^4*c^12*y^2*z - 3*a^2*b^2*c^12*y^2*z + 3*a^2*c^14*y^2*z + 3*a^14*b^2*x*z^2 - 6*a^12*b^4*x*z^2 -      5*a^10*b^6*x*z^2 + 20*a^8*b^8*x*z^2 - 15*a^6*b^10*x*z^2 + 2*a^4*b^12*x*z^2 + a^2*b^14*x*z^2 -      3*a^12*b^2*c^2*x*z^2 + 25*a^10*b^4*c^2*x*z^2 - 34*a^8*b^6*c^2*x*z^2 - 3*a^6*b^8*c^2*x*z^2 +      22*a^4*b^10*c^2*x*z^2 - 6*a^2*b^12*c^2*x*z^2 - b^14*c^2*x*z^2 - 11*a^10*b^2*c^4*x*z^2 -      4*a^8*b^4*c^4*x*z^2 + 59*a^6*b^6*c^4*x*z^2 - 46*a^4*b^8*c^4*x*z^2 - 2*a^2*b^10*c^4*x*z^2 +      4*b^12*c^4*x*z^2 + 15*a^8*b^2*c^6*x*z^2 - 46*a^6*b^4*c^6*x*z^2 + 5*a^4*b^6*c^6*x*z^2 +      29*a^2*b^8*c^6*x*z^2 - 5*b^10*c^6*x*z^2 + 5*a^6*b^2*c^8*x*z^2 + 30*a^4*b^4*c^8*x*z^2 -      30*a^2*b^6*c^8*x*z^2 - 13*a^4*b^2*c^10*x*z^2 + 5*a^2*b^4*c^10*x*z^2 + 5*b^6*c^10*x*z^2 +      3*a^2*b^2*c^12*x*z^2 - 4*b^4*c^12*x*z^2 + b^2*c^14*x*z^2 + a^14*b^2*y*z^2 + 2*a^12*b^4*y*z^2 -      15*a^10*b^6*y*z^2 + 20*a^8*b^8*y*z^2 - 5*a^6*b^10*y*z^2 - 6*a^4*b^12*y*z^2 + 3*a^2*b^14*y*z^2 -      a^14*c^2*y*z^2 - 6*a^12*b^2*c^2*y*z^2 + 22*a^10*b^4*c^2*y*z^2 - 3*a^8*b^6*c^2*y*z^2 -      34*a^6*b^8*c^2*y*z^2 + 25*a^4*b^10*c^2*y*z^2 - 3*a^2*b^12*c^2*y*z^2 + 4*a^12*c^4*y*z^2 -      2*a^10*b^2*c^4*y*z^2 - 46*a^8*b^4*c^4*y*z^2 + 59*a^6*b^6*c^4*y*z^2 - 4*a^4*b^8*c^4*y*z^2 -      11*a^2*b^10*c^4*y*z^2 - 5*a^10*c^6*y*z^2 + 29*a^8*b^2*c^6*y*z^2 + 5*a^6*b^4*c^6*y*z^2 -      46*a^4*b^6*c^6*y*z^2 + 15*a^2*b^8*c^6*y*z^2 - 30*a^6*b^2*c^8*y*z^2 + 30*a^4*b^4*c^8*y*z^2 +      5*a^2*b^6*c^8*y*z^2 + 5*a^6*c^10*y*z^2 + 5*a^4*b^2*c^10*y*z^2 - 13*a^2*b^4*c^10*y*z^2 -      4*a^4*c^12*y*z^2 + 3*a^2*b^2*c^12*y*z^2 + a^2*c^14*y*z^2");
		exp("K542","a^9*c*x^2*y - a^8*b*c*x^2*y - 6*a^5*b^4*c*x^2*y + 6*a^4*b^5*c*x^2*y + 8*a^3*b^6*c*x^2*y -      8*a^2*b^7*c*x^2*y - 3*a*b^8*c*x^2*y + 3*b^9*c*x^2*y - 2*a^8*c^2*x^2*y + 7*a^7*b*c^2*x^2*y -      8*a^6*b^2*c^2*x^2*y + 5*a^5*b^3*c^2*x^2*y + 16*a^4*b^4*c^2*x^2*y - 31*a^3*b^5*c^2*x^2*y +      19*a*b^7*c^2*x^2*y - 6*b^8*c^2*x^2*y - 2*a^7*c^3*x^2*y - 5*a^6*b*c^3*x^2*y + 21*a^5*b^2*c^3*x^2*y -      30*a^4*b^3*c^3*x^2*y + 2*a^3*b^4*c^3*x^2*y + 37*a^2*b^5*c^3*x^2*y - 21*a*b^6*c^3*x^2*y -      2*b^7*c^3*x^2*y + 6*a^6*c^4*x^2*y - 14*a^5*b*c^4*x^2*y - 2*a^4*b^2*c^4*x^2*y +      38*a^3*b^3*c^4*x^2*y - 30*a^2*b^4*c^4*x^2*y - 8*a*b^5*c^4*x^2*y + 10*b^6*c^4*x^2*y +      14*a^4*b*c^5*x^2*y - 26*a^3*b^2*c^5*x^2*y - 4*a^2*b^3*c^5*x^2*y + 20*a*b^4*c^5*x^2*y -      4*b^5*c^5*x^2*y - 6*a^4*c^6*x^2*y + 7*a^3*b*c^6*x^2*y + 12*a^2*b^2*c^6*x^2*y - 11*a*b^3*c^6*x^2*y -      2*b^4*c^6*x^2*y + 2*a^3*c^7*x^2*y - 9*a^2*b*c^7*x^2*y + 5*a*b^2*c^7*x^2*y + 2*b^3*c^7*x^2*y +      2*a^2*c^8*x^2*y - 2*b^2*c^8*x^2*y - a*c^9*x^2*y + b*c^9*x^2*y + 3*a^9*c*x*y^2 - 3*a^8*b*c*x*y^2 -      8*a^7*b^2*c*x*y^2 + 8*a^6*b^3*c*x*y^2 + 6*a^5*b^4*c*x*y^2 - 6*a^4*b^5*c*x*y^2 - a*b^8*c*x*y^2 +      b^9*c*x*y^2 - 6*a^8*c^2*x*y^2 + 19*a^7*b*c^2*x*y^2 - 31*a^5*b^3*c^2*x*y^2 + 16*a^4*b^4*c^2*x*y^2 +      5*a^3*b^5*c^2*x*y^2 - 8*a^2*b^6*c^2*x*y^2 + 7*a*b^7*c^2*x*y^2 - 2*b^8*c^2*x*y^2 - 2*a^7*c^3*x*y^2 -      21*a^6*b*c^3*x*y^2 + 37*a^5*b^2*c^3*x*y^2 + 2*a^4*b^3*c^3*x*y^2 - 30*a^3*b^4*c^3*x*y^2 +      21*a^2*b^5*c^3*x*y^2 - 5*a*b^6*c^3*x*y^2 - 2*b^7*c^3*x*y^2 + 10*a^6*c^4*x*y^2 - 8*a^5*b*c^4*x*y^2 -      30*a^4*b^2*c^4*x*y^2 + 38*a^3*b^3*c^4*x*y^2 - 2*a^2*b^4*c^4*x*y^2 - 14*a*b^5*c^4*x*y^2 +      6*b^6*c^4*x*y^2 - 4*a^5*c^5*x*y^2 + 20*a^4*b*c^5*x*y^2 - 4*a^3*b^2*c^5*x*y^2 -      26*a^2*b^3*c^5*x*y^2 + 14*a*b^4*c^5*x*y^2 - 2*a^4*c^6*x*y^2 - 11*a^3*b*c^6*x*y^2 +      12*a^2*b^2*c^6*x*y^2 + 7*a*b^3*c^6*x*y^2 - 6*b^4*c^6*x*y^2 + 2*a^3*c^7*x*y^2 + 5*a^2*b*c^7*x*y^2 -      9*a*b^2*c^7*x*y^2 + 2*b^3*c^7*x*y^2 - 2*a^2*c^8*x*y^2 + 2*b^2*c^8*x*y^2 + a*c^9*x*y^2 -      b*c^9*x*y^2 + a^9*b*x^2*z - 2*a^8*b^2*x^2*z - 2*a^7*b^3*x^2*z + 6*a^6*b^4*x^2*z - 6*a^4*b^6*x^2*z +      2*a^3*b^7*x^2*z + 2*a^2*b^8*x^2*z - a*b^9*x^2*z - a^8*b*c*x^2*z + 7*a^7*b^2*c*x^2*z -      5*a^6*b^3*c*x^2*z - 14*a^5*b^4*c*x^2*z + 14*a^4*b^5*c*x^2*z + 7*a^3*b^6*c*x^2*z -      9*a^2*b^7*c*x^2*z + b^9*c*x^2*z - 8*a^6*b^2*c^2*x^2*z + 21*a^5*b^3*c^2*x^2*z -      2*a^4*b^4*c^2*x^2*z - 26*a^3*b^5*c^2*x^2*z + 12*a^2*b^6*c^2*x^2*z + 5*a*b^7*c^2*x^2*z -      2*b^8*c^2*x^2*z + 5*a^5*b^2*c^3*x^2*z - 30*a^4*b^3*c^3*x^2*z + 38*a^3*b^4*c^3*x^2*z -      4*a^2*b^5*c^3*x^2*z - 11*a*b^6*c^3*x^2*z + 2*b^7*c^3*x^2*z - 6*a^5*b*c^4*x^2*z +      16*a^4*b^2*c^4*x^2*z + 2*a^3*b^3*c^4*x^2*z - 30*a^2*b^4*c^4*x^2*z + 20*a*b^5*c^4*x^2*z -      2*b^6*c^4*x^2*z + 6*a^4*b*c^5*x^2*z - 31*a^3*b^2*c^5*x^2*z + 37*a^2*b^3*c^5*x^2*z -      8*a*b^4*c^5*x^2*z - 4*b^5*c^5*x^2*z + 8*a^3*b*c^6*x^2*z - 21*a*b^3*c^6*x^2*z + 10*b^4*c^6*x^2*z -      8*a^2*b*c^7*x^2*z + 19*a*b^2*c^7*x^2*z - 2*b^3*c^7*x^2*z - 3*a*b*c^8*x^2*z - 6*b^2*c^8*x^2*z +      3*b*c^9*x^2*z - a^9*b*y^2*z + 2*a^8*b^2*y^2*z + 2*a^7*b^3*y^2*z - 6*a^6*b^4*y^2*z +      6*a^4*b^6*y^2*z - 2*a^3*b^7*y^2*z - 2*a^2*b^8*y^2*z + a*b^9*y^2*z + a^9*c*y^2*z -      9*a^7*b^2*c*y^2*z + 7*a^6*b^3*c*y^2*z + 14*a^5*b^4*c*y^2*z - 14*a^4*b^5*c*y^2*z -      5*a^3*b^6*c*y^2*z + 7*a^2*b^7*c*y^2*z - a*b^8*c*y^2*z - 2*a^8*c^2*y^2*z + 5*a^7*b*c^2*y^2*z +      12*a^6*b^2*c^2*y^2*z - 26*a^5*b^3*c^2*y^2*z - 2*a^4*b^4*c^2*y^2*z + 21*a^3*b^5*c^2*y^2*z -      8*a^2*b^6*c^2*y^2*z + 2*a^7*c^3*y^2*z - 11*a^6*b*c^3*y^2*z - 4*a^5*b^2*c^3*y^2*z +      38*a^4*b^3*c^3*y^2*z - 30*a^3*b^4*c^3*y^2*z + 5*a^2*b^5*c^3*y^2*z - 2*a^6*c^4*y^2*z +      20*a^5*b*c^4*y^2*z - 30*a^4*b^2*c^4*y^2*z + 2*a^3*b^3*c^4*y^2*z + 16*a^2*b^4*c^4*y^2*z -      6*a*b^5*c^4*y^2*z - 4*a^5*c^5*y^2*z - 8*a^4*b*c^5*y^2*z + 37*a^3*b^2*c^5*y^2*z -      31*a^2*b^3*c^5*y^2*z + 6*a*b^4*c^5*y^2*z + 10*a^4*c^6*y^2*z - 21*a^3*b*c^6*y^2*z +      8*a*b^3*c^6*y^2*z - 2*a^3*c^7*y^2*z + 19*a^2*b*c^7*y^2*z - 8*a*b^2*c^7*y^2*z - 6*a^2*c^8*y^2*z -      3*a*b*c^8*y^2*z + 3*a*c^9*y^2*z + 3*a^9*b*x*z^2 - 6*a^8*b^2*x*z^2 - 2*a^7*b^3*x*z^2 +      10*a^6*b^4*x*z^2 - 4*a^5*b^5*x*z^2 - 2*a^4*b^6*x*z^2 + 2*a^3*b^7*x*z^2 - 2*a^2*b^8*x*z^2 +      a*b^9*x*z^2 - 3*a^8*b*c*x*z^2 + 19*a^7*b^2*c*x*z^2 - 21*a^6*b^3*c*x*z^2 - 8*a^5*b^4*c*x*z^2 +      20*a^4*b^5*c*x*z^2 - 11*a^3*b^6*c*x*z^2 + 5*a^2*b^7*c*x*z^2 - b^9*c*x*z^2 - 8*a^7*b*c^2*x*z^2 +      37*a^5*b^3*c^2*x*z^2 - 30*a^4*b^4*c^2*x*z^2 - 4*a^3*b^5*c^2*x*z^2 + 12*a^2*b^6*c^2*x*z^2 -      9*a*b^7*c^2*x*z^2 + 2*b^8*c^2*x*z^2 + 8*a^6*b*c^3*x*z^2 - 31*a^5*b^2*c^3*x*z^2 +      2*a^4*b^3*c^3*x*z^2 + 38*a^3*b^4*c^3*x*z^2 - 26*a^2*b^5*c^3*x*z^2 + 7*a*b^6*c^3*x*z^2 +      2*b^7*c^3*x*z^2 + 6*a^5*b*c^4*x*z^2 + 16*a^4*b^2*c^4*x*z^2 - 30*a^3*b^3*c^4*x*z^2 -      2*a^2*b^4*c^4*x*z^2 + 14*a*b^5*c^4*x*z^2 - 6*b^6*c^4*x*z^2 - 6*a^4*b*c^5*x*z^2 +      5*a^3*b^2*c^5*x*z^2 + 21*a^2*b^3*c^5*x*z^2 - 14*a*b^4*c^5*x*z^2 - 8*a^2*b^2*c^6*x*z^2 -      5*a*b^3*c^6*x*z^2 + 6*b^4*c^6*x*z^2 + 7*a*b^2*c^7*x*z^2 - 2*b^3*c^7*x*z^2 - a*b*c^8*x*z^2 -      2*b^2*c^8*x*z^2 + b*c^9*x*z^2 + a^9*b*y*z^2 - 2*a^8*b^2*y*z^2 + 2*a^7*b^3*y*z^2 - 2*a^6*b^4*y*z^2 -      4*a^5*b^5*y*z^2 + 10*a^4*b^6*y*z^2 - 2*a^3*b^7*y*z^2 - 6*a^2*b^8*y*z^2 + 3*a*b^9*y*z^2 -      a^9*c*y*z^2 + 5*a^7*b^2*c*y*z^2 - 11*a^6*b^3*c*y*z^2 + 20*a^5*b^4*c*y*z^2 - 8*a^4*b^5*c*y*z^2 -      21*a^3*b^6*c*y*z^2 + 19*a^2*b^7*c*y*z^2 - 3*a*b^8*c*y*z^2 + 2*a^8*c^2*y*z^2 - 9*a^7*b*c^2*y*z^2 +      12*a^6*b^2*c^2*y*z^2 - 4*a^5*b^3*c^2*y*z^2 - 30*a^4*b^4*c^2*y*z^2 + 37*a^3*b^5*c^2*y*z^2 -      8*a*b^7*c^2*y*z^2 + 2*a^7*c^3*y*z^2 + 7*a^6*b*c^3*y*z^2 - 26*a^5*b^2*c^3*y*z^2 +      38*a^4*b^3*c^3*y*z^2 + 2*a^3*b^4*c^3*y*z^2 - 31*a^2*b^5*c^3*y*z^2 + 8*a*b^6*c^3*y*z^2 -      6*a^6*c^4*y*z^2 + 14*a^5*b*c^4*y*z^2 - 2*a^4*b^2*c^4*y*z^2 - 30*a^3*b^3*c^4*y*z^2 +      16*a^2*b^4*c^4*y*z^2 + 6*a*b^5*c^4*y*z^2 - 14*a^4*b*c^5*y*z^2 + 21*a^3*b^2*c^5*y*z^2 +      5*a^2*b^3*c^5*y*z^2 - 6*a*b^4*c^5*y*z^2 + 6*a^4*c^6*y*z^2 - 5*a^3*b*c^6*y*z^2 -      8*a^2*b^2*c^6*y*z^2 - 2*a^3*c^7*y*z^2 + 7*a^2*b*c^7*y*z^2 - 2*a^2*c^8*y*z^2 - a*b*c^8*y*z^2 +      a*c^9*y*z^2");
		exp("K543","(-a^18)*x^2*y + 2*a^16*b^2*x^2*y + 11*a^14*b^4*x^2*y - 49*a^12*b^6*x^2*y + 77*a^10*b^8*x^2*y -      49*a^8*b^10*x^2*y - 7*a^6*b^12*x^2*y + 29*a^4*b^14*x^2*y - 16*a^2*b^16*x^2*y + 3*b^18*x^2*y +      3*a^16*c^2*x^2*y - 20*a^14*b^2*c^2*x^2*y + 38*a^12*b^4*c^2*x^2*y + 6*a^10*b^6*c^2*x^2*y -      110*a^8*b^8*c^2*x^2*y + 152*a^6*b^10*c^2*x^2*y - 90*a^4*b^12*c^2*x^2*y + 22*a^2*b^14*c^2*x^2*y -      b^16*c^2*x^2*y + 27*a^12*b^2*c^4*x^2*y - 123*a^10*b^4*c^4*x^2*y + 204*a^8*b^6*c^4*x^2*y -      126*a^6*b^8*c^4*x^2*y - 21*a^4*b^10*c^4*x^2*y + 57*a^2*b^12*c^4*x^2*y - 18*b^14*c^4*x^2*y -      9*a^12*c^6*x^2*y + 26*a^10*b^2*c^6*x^2*y + 11*a^8*b^4*c^6*x^2*y - 144*a^6*b^6*c^6*x^2*y +      209*a^4*b^8*c^6*x^2*y - 106*a^2*b^10*c^6*x^2*y + 13*b^12*c^6*x^2*y + 12*a^10*c^8*x^2*y -      47*a^8*b^2*c^8*x^2*y + 137*a^6*b^4*c^8*x^2*y - 117*a^4*b^6*c^8*x^2*y - 17*a^2*b^8*c^8*x^2*y +      32*b^10*c^8*x^2*y - 9*a^8*c^10*x^2*y - 24*a^6*b^2*c^10*x^2*y - 48*a^4*b^4*c^10*x^2*y +      114*a^2*b^6*c^10*x^2*y - 33*b^8*c^10*x^2*y + 12*a^6*c^12*x^2*y + 53*a^4*b^2*c^12*x^2*y -      49*a^2*b^4*c^12*x^2*y - 14*b^6*c^12*x^2*y - 15*a^4*c^14*x^2*y - 14*a^2*b^2*c^14*x^2*y +      23*b^4*c^14*x^2*y + 9*a^2*c^16*x^2*y - 3*b^2*c^16*x^2*y - 2*c^18*x^2*y - 3*a^18*x*y^2 +      16*a^16*b^2*x*y^2 - 29*a^14*b^4*x*y^2 + 7*a^12*b^6*x*y^2 + 49*a^10*b^8*x*y^2 - 77*a^8*b^10*x*y^2 +      49*a^6*b^12*x*y^2 - 11*a^4*b^14*x*y^2 - 2*a^2*b^16*x*y^2 + b^18*x*y^2 + a^16*c^2*x*y^2 -      22*a^14*b^2*c^2*x*y^2 + 90*a^12*b^4*c^2*x*y^2 - 152*a^10*b^6*c^2*x*y^2 + 110*a^8*b^8*c^2*x*y^2 -      6*a^6*b^10*c^2*x*y^2 - 38*a^4*b^12*c^2*x*y^2 + 20*a^2*b^14*c^2*x*y^2 - 3*b^16*c^2*x*y^2 +      18*a^14*c^4*x*y^2 - 57*a^12*b^2*c^4*x*y^2 + 21*a^10*b^4*c^4*x*y^2 + 126*a^8*b^6*c^4*x*y^2 -      204*a^6*b^8*c^4*x*y^2 + 123*a^4*b^10*c^4*x*y^2 - 27*a^2*b^12*c^4*x*y^2 - 13*a^12*c^6*x*y^2 +      106*a^10*b^2*c^6*x*y^2 - 209*a^8*b^4*c^6*x*y^2 + 144*a^6*b^6*c^6*x*y^2 - 11*a^4*b^8*c^6*x*y^2 -      26*a^2*b^10*c^6*x*y^2 + 9*b^12*c^6*x*y^2 - 32*a^10*c^8*x*y^2 + 17*a^8*b^2*c^8*x*y^2 +      117*a^6*b^4*c^8*x*y^2 - 137*a^4*b^6*c^8*x*y^2 + 47*a^2*b^8*c^8*x*y^2 - 12*b^10*c^8*x*y^2 +      33*a^8*c^10*x*y^2 - 114*a^6*b^2*c^10*x*y^2 + 48*a^4*b^4*c^10*x*y^2 + 24*a^2*b^6*c^10*x*y^2 +      9*b^8*c^10*x*y^2 + 14*a^6*c^12*x*y^2 + 49*a^4*b^2*c^12*x*y^2 - 53*a^2*b^4*c^12*x*y^2 -      12*b^6*c^12*x*y^2 - 23*a^4*c^14*x*y^2 + 14*a^2*b^2*c^14*x*y^2 + 15*b^4*c^14*x*y^2 +      3*a^2*c^16*x*y^2 - 9*b^2*c^16*x*y^2 + 2*c^18*x*y^2 + a^18*x^2*z - 3*a^16*b^2*x^2*z +      9*a^12*b^6*x^2*z - 12*a^10*b^8*x^2*z + 9*a^8*b^10*x^2*z - 12*a^6*b^12*x^2*z + 15*a^4*b^14*x^2*z -      9*a^2*b^16*x^2*z + 2*b^18*x^2*z - 2*a^16*c^2*x^2*z + 20*a^14*b^2*c^2*x^2*z -      27*a^12*b^4*c^2*x^2*z - 26*a^10*b^6*c^2*x^2*z + 47*a^8*b^8*c^2*x^2*z + 24*a^6*b^10*c^2*x^2*z -      53*a^4*b^12*c^2*x^2*z + 14*a^2*b^14*c^2*x^2*z + 3*b^16*c^2*x^2*z - 11*a^14*c^4*x^2*z -      38*a^12*b^2*c^4*x^2*z + 123*a^10*b^4*c^4*x^2*z - 11*a^8*b^6*c^4*x^2*z - 137*a^6*b^8*c^4*x^2*z +      48*a^4*b^10*c^4*x^2*z + 49*a^2*b^12*c^4*x^2*z - 23*b^14*c^4*x^2*z + 49*a^12*c^6*x^2*z -      6*a^10*b^2*c^6*x^2*z - 204*a^8*b^4*c^6*x^2*z + 144*a^6*b^6*c^6*x^2*z + 117*a^4*b^8*c^6*x^2*z -      114*a^2*b^10*c^6*x^2*z + 14*b^12*c^6*x^2*z - 77*a^10*c^8*x^2*z + 110*a^8*b^2*c^8*x^2*z +      126*a^6*b^4*c^8*x^2*z - 209*a^4*b^6*c^8*x^2*z + 17*a^2*b^8*c^8*x^2*z + 33*b^10*c^8*x^2*z +      49*a^8*c^10*x^2*z - 152*a^6*b^2*c^10*x^2*z + 21*a^4*b^4*c^10*x^2*z + 106*a^2*b^6*c^10*x^2*z -      32*b^8*c^10*x^2*z + 7*a^6*c^12*x^2*z + 90*a^4*b^2*c^12*x^2*z - 57*a^2*b^4*c^12*x^2*z -      13*b^6*c^12*x^2*z - 29*a^4*c^14*x^2*z - 22*a^2*b^2*c^14*x^2*z + 18*b^4*c^14*x^2*z +      16*a^2*c^16*x^2*z + b^2*c^16*x^2*z - 3*c^18*x^2*z - 2*a^18*y^2*z + 9*a^16*b^2*y^2*z -      15*a^14*b^4*y^2*z + 12*a^12*b^6*y^2*z - 9*a^10*b^8*y^2*z + 12*a^8*b^10*y^2*z - 9*a^6*b^12*y^2*z +      3*a^2*b^16*y^2*z - b^18*y^2*z - 3*a^16*c^2*y^2*z - 14*a^14*b^2*c^2*y^2*z + 53*a^12*b^4*c^2*y^2*z -      24*a^10*b^6*c^2*y^2*z - 47*a^8*b^8*c^2*y^2*z + 26*a^6*b^10*c^2*y^2*z + 27*a^4*b^12*c^2*y^2*z -      20*a^2*b^14*c^2*y^2*z + 2*b^16*c^2*y^2*z + 23*a^14*c^4*y^2*z - 49*a^12*b^2*c^4*y^2*z -      48*a^10*b^4*c^4*y^2*z + 137*a^8*b^6*c^4*y^2*z + 11*a^6*b^8*c^4*y^2*z - 123*a^4*b^10*c^4*y^2*z +      38*a^2*b^12*c^4*y^2*z + 11*b^14*c^4*y^2*z - 14*a^12*c^6*y^2*z + 114*a^10*b^2*c^6*y^2*z -      117*a^8*b^4*c^6*y^2*z - 144*a^6*b^6*c^6*y^2*z + 204*a^4*b^8*c^6*y^2*z + 6*a^2*b^10*c^6*y^2*z -      49*b^12*c^6*y^2*z - 33*a^10*c^8*y^2*z - 17*a^8*b^2*c^8*y^2*z + 209*a^6*b^4*c^8*y^2*z -      126*a^4*b^6*c^8*y^2*z - 110*a^2*b^8*c^8*y^2*z + 77*b^10*c^8*y^2*z + 32*a^8*c^10*y^2*z -      106*a^6*b^2*c^10*y^2*z - 21*a^4*b^4*c^10*y^2*z + 152*a^2*b^6*c^10*y^2*z - 49*b^8*c^10*y^2*z +      13*a^6*c^12*y^2*z + 57*a^4*b^2*c^12*y^2*z - 90*a^2*b^4*c^12*y^2*z - 7*b^6*c^12*y^2*z -      18*a^4*c^14*y^2*z + 22*a^2*b^2*c^14*y^2*z + 29*b^4*c^14*y^2*z - a^2*c^16*y^2*z -      16*b^2*c^16*y^2*z + 3*c^18*y^2*z + 3*a^18*x*z^2 - a^16*b^2*x*z^2 - 18*a^14*b^4*x*z^2 +      13*a^12*b^6*x*z^2 + 32*a^10*b^8*x*z^2 - 33*a^8*b^10*x*z^2 - 14*a^6*b^12*x*z^2 + 23*a^4*b^14*x*z^2 -      3*a^2*b^16*x*z^2 - 2*b^18*x*z^2 - 16*a^16*c^2*x*z^2 + 22*a^14*b^2*c^2*x*z^2 +      57*a^12*b^4*c^2*x*z^2 - 106*a^10*b^6*c^2*x*z^2 - 17*a^8*b^8*c^2*x*z^2 + 114*a^6*b^10*c^2*x*z^2 -      49*a^4*b^12*c^2*x*z^2 - 14*a^2*b^14*c^2*x*z^2 + 9*b^16*c^2*x*z^2 + 29*a^14*c^4*x*z^2 -      90*a^12*b^2*c^4*x*z^2 - 21*a^10*b^4*c^4*x*z^2 + 209*a^8*b^6*c^4*x*z^2 - 117*a^6*b^8*c^4*x*z^2 -      48*a^4*b^10*c^4*x*z^2 + 53*a^2*b^12*c^4*x*z^2 - 15*b^14*c^4*x*z^2 - 7*a^12*c^6*x*z^2 +      152*a^10*b^2*c^6*x*z^2 - 126*a^8*b^4*c^6*x*z^2 - 144*a^6*b^6*c^6*x*z^2 + 137*a^4*b^8*c^6*x*z^2 -      24*a^2*b^10*c^6*x*z^2 + 12*b^12*c^6*x*z^2 - 49*a^10*c^8*x*z^2 - 110*a^8*b^2*c^8*x*z^2 +      204*a^6*b^4*c^8*x*z^2 + 11*a^4*b^6*c^8*x*z^2 - 47*a^2*b^8*c^8*x*z^2 - 9*b^10*c^8*x*z^2 +      77*a^8*c^10*x*z^2 + 6*a^6*b^2*c^10*x*z^2 - 123*a^4*b^4*c^10*x*z^2 + 26*a^2*b^6*c^10*x*z^2 +      12*b^8*c^10*x*z^2 - 49*a^6*c^12*x*z^2 + 38*a^4*b^2*c^12*x*z^2 + 27*a^2*b^4*c^12*x*z^2 -      9*b^6*c^12*x*z^2 + 11*a^4*c^14*x*z^2 - 20*a^2*b^2*c^14*x*z^2 + 2*a^2*c^16*x*z^2 +      3*b^2*c^16*x*z^2 - c^18*x*z^2 + 2*a^18*y*z^2 + 3*a^16*b^2*y*z^2 - 23*a^14*b^4*y*z^2 +      14*a^12*b^6*y*z^2 + 33*a^10*b^8*y*z^2 - 32*a^8*b^10*y*z^2 - 13*a^6*b^12*y*z^2 + 18*a^4*b^14*y*z^2 +      a^2*b^16*y*z^2 - 3*b^18*y*z^2 - 9*a^16*c^2*y*z^2 + 14*a^14*b^2*c^2*y*z^2 + 49*a^12*b^4*c^2*y*z^2 -      114*a^10*b^6*c^2*y*z^2 + 17*a^8*b^8*c^2*y*z^2 + 106*a^6*b^10*c^2*y*z^2 - 57*a^4*b^12*c^2*y*z^2 -      22*a^2*b^14*c^2*y*z^2 + 16*b^16*c^2*y*z^2 + 15*a^14*c^4*y*z^2 - 53*a^12*b^2*c^4*y*z^2 +      48*a^10*b^4*c^4*y*z^2 + 117*a^8*b^6*c^4*y*z^2 - 209*a^6*b^8*c^4*y*z^2 + 21*a^4*b^10*c^4*y*z^2 +      90*a^2*b^12*c^4*y*z^2 - 29*b^14*c^4*y*z^2 - 12*a^12*c^6*y*z^2 + 24*a^10*b^2*c^6*y*z^2 -      137*a^8*b^4*c^6*y*z^2 + 144*a^6*b^6*c^6*y*z^2 + 126*a^4*b^8*c^6*y*z^2 - 152*a^2*b^10*c^6*y*z^2 +      7*b^12*c^6*y*z^2 + 9*a^10*c^8*y*z^2 + 47*a^8*b^2*c^8*y*z^2 - 11*a^6*b^4*c^8*y*z^2 -      204*a^4*b^6*c^8*y*z^2 + 110*a^2*b^8*c^8*y*z^2 + 49*b^10*c^8*y*z^2 - 12*a^8*c^10*y*z^2 -      26*a^6*b^2*c^10*y*z^2 + 123*a^4*b^4*c^10*y*z^2 - 6*a^2*b^6*c^10*y*z^2 - 77*b^8*c^10*y*z^2 +      9*a^6*c^12*y*z^2 - 27*a^4*b^2*c^12*y*z^2 - 38*a^2*b^4*c^12*y*z^2 + 49*b^6*c^12*y*z^2 +      20*a^2*b^2*c^14*y*z^2 - 11*b^4*c^14*y*z^2 - 3*a^2*c^16*y*z^2 - 2*b^2*c^16*y*z^2 + c^18*y*z^2");
		exp("K544","(-a^16)*c^2*x^2*y + 2*a^14*b^2*c^2*x^2*y + 2*a^10*b^6*c^2*x^2*y - 10*a^8*b^8*c^2*x^2*y +      6*a^6*b^10*c^2*x^2*y + 8*a^4*b^12*c^2*x^2*y - 10*a^2*b^14*c^2*x^2*y + 3*b^16*c^2*x^2*y +      5*a^14*c^4*x^2*y - 10*a^12*b^2*c^4*x^2*y + 11*a^8*b^6*c^4*x^2*y + 5*a^6*b^8*c^4*x^2*y -      36*a^4*b^10*c^4*x^2*y + 38*a^2*b^12*c^4*x^2*y - 13*b^14*c^4*x^2*y - 9*a^12*c^6*x^2*y +      18*a^10*b^2*c^6*x^2*y - 9*a^8*b^4*c^6*x^2*y - 18*a^6*b^6*c^6*x^2*y + 45*a^4*b^8*c^6*x^2*y -      48*a^2*b^10*c^6*x^2*y + 21*b^12*c^6*x^2*y + 5*a^10*c^8*x^2*y - 13*a^8*b^2*c^8*x^2*y +      18*a^6*b^4*c^8*x^2*y - 16*a^4*b^6*c^8*x^2*y + 17*a^2*b^8*c^8*x^2*y - 15*b^10*c^8*x^2*y +      5*a^8*c^10*x^2*y + 2*a^6*b^2*c^10*x^2*y - 6*a^4*b^4*c^10*x^2*y + 8*a^2*b^6*c^10*x^2*y +      5*b^8*c^10*x^2*y - 9*a^6*c^12*x^2*y - 6*a^2*b^4*c^12*x^2*y - 3*b^6*c^12*x^2*y + 5*a^4*c^14*x^2*y +      2*a^2*b^2*c^14*x^2*y + 3*b^4*c^14*x^2*y - a^2*c^16*x^2*y - b^2*c^16*x^2*y - 3*a^16*c^2*x*y^2 +      10*a^14*b^2*c^2*x*y^2 - 8*a^12*b^4*c^2*x*y^2 - 6*a^10*b^6*c^2*x*y^2 + 10*a^8*b^8*c^2*x*y^2 -      2*a^6*b^10*c^2*x*y^2 - 2*a^2*b^14*c^2*x*y^2 + b^16*c^2*x*y^2 + 13*a^14*c^4*x*y^2 -      38*a^12*b^2*c^4*x*y^2 + 36*a^10*b^4*c^4*x*y^2 - 5*a^8*b^6*c^4*x*y^2 - 11*a^6*b^8*c^4*x*y^2 +      10*a^2*b^12*c^4*x*y^2 - 5*b^14*c^4*x*y^2 - 21*a^12*c^6*x*y^2 + 48*a^10*b^2*c^6*x*y^2 -      45*a^8*b^4*c^6*x*y^2 + 18*a^6*b^6*c^6*x*y^2 + 9*a^4*b^8*c^6*x*y^2 - 18*a^2*b^10*c^6*x*y^2 +      9*b^12*c^6*x*y^2 + 15*a^10*c^8*x*y^2 - 17*a^8*b^2*c^8*x*y^2 + 16*a^6*b^4*c^8*x*y^2 -      18*a^4*b^6*c^8*x*y^2 + 13*a^2*b^8*c^8*x*y^2 - 5*b^10*c^8*x*y^2 - 5*a^8*c^10*x*y^2 -      8*a^6*b^2*c^10*x*y^2 + 6*a^4*b^4*c^10*x*y^2 - 2*a^2*b^6*c^10*x*y^2 - 5*b^8*c^10*x*y^2 +      3*a^6*c^12*x*y^2 + 6*a^4*b^2*c^12*x*y^2 + 9*b^6*c^12*x*y^2 - 3*a^4*c^14*x*y^2 -      2*a^2*b^2*c^14*x*y^2 - 5*b^4*c^14*x*y^2 + a^2*c^16*x*y^2 + b^2*c^16*x*y^2 + a^16*b^2*x^2*z -      5*a^14*b^4*x^2*z + 9*a^12*b^6*x^2*z - 5*a^10*b^8*x^2*z - 5*a^8*b^10*x^2*z + 9*a^6*b^12*x^2*z -      5*a^4*b^14*x^2*z + a^2*b^16*x^2*z - 2*a^14*b^2*c^2*x^2*z + 10*a^12*b^4*c^2*x^2*z -      18*a^10*b^6*c^2*x^2*z + 13*a^8*b^8*c^2*x^2*z - 2*a^6*b^10*c^2*x^2*z - 2*a^2*b^14*c^2*x^2*z +      b^16*c^2*x^2*z + 9*a^8*b^6*c^4*x^2*z - 18*a^6*b^8*c^4*x^2*z + 6*a^4*b^10*c^4*x^2*z +      6*a^2*b^12*c^4*x^2*z - 3*b^14*c^4*x^2*z - 2*a^10*b^2*c^6*x^2*z - 11*a^8*b^4*c^6*x^2*z +      18*a^6*b^6*c^6*x^2*z + 16*a^4*b^8*c^6*x^2*z - 8*a^2*b^10*c^6*x^2*z + 3*b^12*c^6*x^2*z +      10*a^8*b^2*c^8*x^2*z - 5*a^6*b^4*c^8*x^2*z - 45*a^4*b^6*c^8*x^2*z - 17*a^2*b^8*c^8*x^2*z -      5*b^10*c^8*x^2*z - 6*a^6*b^2*c^10*x^2*z + 36*a^4*b^4*c^10*x^2*z + 48*a^2*b^6*c^10*x^2*z +      15*b^8*c^10*x^2*z - 8*a^4*b^2*c^12*x^2*z - 38*a^2*b^4*c^12*x^2*z - 21*b^6*c^12*x^2*z +      10*a^2*b^2*c^14*x^2*z + 13*b^4*c^14*x^2*z - 3*b^2*c^16*x^2*z - a^16*b^2*y^2*z + 5*a^14*b^4*y^2*z -      9*a^12*b^6*y^2*z + 5*a^10*b^8*y^2*z + 5*a^8*b^10*y^2*z - 9*a^6*b^12*y^2*z + 5*a^4*b^14*y^2*z -      a^2*b^16*y^2*z - a^16*c^2*y^2*z + 2*a^14*b^2*c^2*y^2*z + 2*a^10*b^6*c^2*y^2*z -      13*a^8*b^8*c^2*y^2*z + 18*a^6*b^10*c^2*y^2*z - 10*a^4*b^12*c^2*y^2*z + 2*a^2*b^14*c^2*y^2*z +      3*a^14*c^4*y^2*z - 6*a^12*b^2*c^4*y^2*z - 6*a^10*b^4*c^4*y^2*z + 18*a^8*b^6*c^4*y^2*z -      9*a^6*b^8*c^4*y^2*z - 3*a^12*c^6*y^2*z + 8*a^10*b^2*c^6*y^2*z - 16*a^8*b^4*c^6*y^2*z -      18*a^6*b^6*c^6*y^2*z + 11*a^4*b^8*c^6*y^2*z + 2*a^2*b^10*c^6*y^2*z + 5*a^10*c^8*y^2*z +      17*a^8*b^2*c^8*y^2*z + 45*a^6*b^4*c^8*y^2*z + 5*a^4*b^6*c^8*y^2*z - 10*a^2*b^8*c^8*y^2*z -      15*a^8*c^10*y^2*z - 48*a^6*b^2*c^10*y^2*z - 36*a^4*b^4*c^10*y^2*z + 6*a^2*b^6*c^10*y^2*z +      21*a^6*c^12*y^2*z + 38*a^4*b^2*c^12*y^2*z + 8*a^2*b^4*c^12*y^2*z - 13*a^4*c^14*y^2*z -      10*a^2*b^2*c^14*y^2*z + 3*a^2*c^16*y^2*z + 3*a^16*b^2*x*z^2 - 13*a^14*b^4*x*z^2 +      21*a^12*b^6*x*z^2 - 15*a^10*b^8*x*z^2 + 5*a^8*b^10*x*z^2 - 3*a^6*b^12*x*z^2 + 3*a^4*b^14*x*z^2 -      a^2*b^16*x*z^2 - 10*a^14*b^2*c^2*x*z^2 + 38*a^12*b^4*c^2*x*z^2 - 48*a^10*b^6*c^2*x*z^2 +      17*a^8*b^8*c^2*x*z^2 + 8*a^6*b^10*c^2*x*z^2 - 6*a^4*b^12*c^2*x*z^2 + 2*a^2*b^14*c^2*x*z^2 -      b^16*c^2*x*z^2 + 8*a^12*b^2*c^4*x*z^2 - 36*a^10*b^4*c^4*x*z^2 + 45*a^8*b^6*c^4*x*z^2 -      16*a^6*b^8*c^4*x*z^2 - 6*a^4*b^10*c^4*x*z^2 + 5*b^14*c^4*x*z^2 + 6*a^10*b^2*c^6*x*z^2 +      5*a^8*b^4*c^6*x*z^2 - 18*a^6*b^6*c^6*x*z^2 + 18*a^4*b^8*c^6*x*z^2 + 2*a^2*b^10*c^6*x*z^2 -      9*b^12*c^6*x*z^2 - 10*a^8*b^2*c^8*x*z^2 + 11*a^6*b^4*c^8*x*z^2 - 9*a^4*b^6*c^8*x*z^2 -      13*a^2*b^8*c^8*x*z^2 + 5*b^10*c^8*x*z^2 + 2*a^6*b^2*c^10*x*z^2 + 18*a^2*b^6*c^10*x*z^2 +      5*b^8*c^10*x*z^2 - 10*a^2*b^4*c^12*x*z^2 - 9*b^6*c^12*x*z^2 + 2*a^2*b^2*c^14*x*z^2 +      5*b^4*c^14*x*z^2 - b^2*c^16*x*z^2 + a^16*b^2*y*z^2 - 3*a^14*b^4*y*z^2 + 3*a^12*b^6*y*z^2 -      5*a^10*b^8*y*z^2 + 15*a^8*b^10*y*z^2 - 21*a^6*b^12*y*z^2 + 13*a^4*b^14*y*z^2 - 3*a^2*b^16*y*z^2 +      a^16*c^2*y*z^2 - 2*a^14*b^2*c^2*y*z^2 + 6*a^12*b^4*c^2*y*z^2 - 8*a^10*b^6*c^2*y*z^2 -      17*a^8*b^8*c^2*y*z^2 + 48*a^6*b^10*c^2*y*z^2 - 38*a^4*b^12*c^2*y*z^2 + 10*a^2*b^14*c^2*y*z^2 -      5*a^14*c^4*y*z^2 + 6*a^10*b^4*c^4*y*z^2 + 16*a^8*b^6*c^4*y*z^2 - 45*a^6*b^8*c^4*y*z^2 +      36*a^4*b^10*c^4*y*z^2 - 8*a^2*b^12*c^4*y*z^2 + 9*a^12*c^6*y*z^2 - 2*a^10*b^2*c^6*y*z^2 -      18*a^8*b^4*c^6*y*z^2 + 18*a^6*b^6*c^6*y*z^2 - 5*a^4*b^8*c^6*y*z^2 - 6*a^2*b^10*c^6*y*z^2 -      5*a^10*c^8*y*z^2 + 13*a^8*b^2*c^8*y*z^2 + 9*a^6*b^4*c^8*y*z^2 - 11*a^4*b^6*c^8*y*z^2 +      10*a^2*b^8*c^8*y*z^2 - 5*a^8*c^10*y*z^2 - 18*a^6*b^2*c^10*y*z^2 - 2*a^2*b^6*c^10*y*z^2 +      9*a^6*c^12*y*z^2 + 10*a^4*b^2*c^12*y*z^2 - 5*a^4*c^14*y*z^2 - 2*a^2*b^2*c^14*y*z^2 +      a^2*c^16*y*z^2");
		exp("K545","(-a^4)*b^2*x^2*y - b^4*c^2*x^2*y - a^2*c^4*x^2*y + a^2*b^4*x*y^2 + a^4*c^2*x*y^2 +      b^2*c^4*x*y^2 + a^2*b^4*x^2*z + a^4*c^2*x^2*z + b^2*c^4*x^2*z - a^4*b^2*y^2*z - b^4*c^2*y^2*z -      a^2*c^4*y^2*z - a^4*b^2*x*z^2 - b^4*c^2*x*z^2 - a^2*c^4*x*z^2 + a^2*b^4*y*z^2 + a^4*c^2*y*z^2 +      b^2*c^4*y*z^2");
		exp("K546","(-a^2)*b^6*c^4*x^2*y - a^4*b^2*c^6*x^2*y - b^4*c^8*x^2*y + a^6*b^2*c^4*x*y^2 +      a^2*b^4*c^6*x*y^2 + a^4*c^8*x*y^2 + a^4*b^6*c^2*x^2*z + b^8*c^4*x^2*z + a^2*b^4*c^6*x^2*z -      a^6*b^4*c^2*y^2*z - a^8*c^4*y^2*z - a^4*b^2*c^6*y^2*z - a^4*b^8*x*z^2 - a^6*b^4*c^2*x*z^2 -      a^2*b^6*c^4*x*z^2 + a^8*b^4*y*z^2 + a^4*b^6*c^2*y*z^2 + a^6*b^2*c^4*y*z^2");
		exp("K547","(-a^8)*b^4*c^2*x^2*y + a^6*b^4*c^4*x^2*y + a^4*b^6*c^4*x^2*y - a^2*b^8*c^4*x^2*y +      a^4*b^4*c^6*x^2*y - a^4*b^2*c^8*x^2*y + a^4*b^8*c^2*x*y^2 + a^8*b^2*c^4*x*y^2 - a^6*b^4*c^4*x*y^2 -      a^4*b^6*c^4*x*y^2 - a^4*b^4*c^6*x*y^2 + a^2*b^4*c^8*x*y^2 + a^4*b^8*c^2*x^2*z + a^8*b^2*c^4*x^2*z -      a^6*b^4*c^4*x^2*z - a^4*b^6*c^4*x^2*z - a^4*b^4*c^6*x^2*z + a^2*b^4*c^8*x^2*z + a^8*b^6*x*y*z -      a^6*b^8*x*y*z + a^8*b^4*c^2*x*y*z - a^4*b^8*c^2*x*y*z - a^8*b^2*c^4*x*y*z + a^2*b^8*c^4*x*y*z -      a^8*c^6*x*y*z + b^8*c^6*x*y*z + a^6*c^8*x*y*z + a^4*b^2*c^8*x*y*z - a^2*b^4*c^8*x*y*z -      b^6*c^8*x*y*z - a^8*b^4*c^2*y^2*z + a^6*b^4*c^4*y^2*z + a^4*b^6*c^4*y^2*z - a^2*b^8*c^4*y^2*z +      a^4*b^4*c^6*y^2*z - a^4*b^2*c^8*y^2*z - a^8*b^4*c^2*x*z^2 + a^6*b^4*c^4*x*z^2 + a^4*b^6*c^4*x*z^2 -      a^2*b^8*c^4*x*z^2 + a^4*b^4*c^6*x*z^2 - a^4*b^2*c^8*x*z^2 + a^4*b^8*c^2*y*z^2 + a^8*b^2*c^4*y*z^2 -      a^6*b^4*c^4*y*z^2 - a^4*b^6*c^4*y*z^2 - a^4*b^4*c^6*y*z^2 + a^2*b^4*c^8*y*z^2");
		exp("K548","a^4*b^2*x^2*y + a^2*b^2*c^2*x^2*y + b^4*c^2*x^2*y + a^2*c^4*x^2*y - a^2*b^4*x*y^2 -      a^4*c^2*x*y^2 - a^2*b^2*c^2*x*y^2 - b^2*c^4*x*y^2 - a^2*b^4*x^2*z - a^4*c^2*x^2*z -      a^2*b^2*c^2*x^2*z - b^2*c^4*x^2*z + a^4*b^2*x*y*z - a^2*b^4*x*y*z - a^4*c^2*x*y*z + b^4*c^2*x*y*z +      a^2*c^4*x*y*z - b^2*c^4*x*y*z + a^4*b^2*y^2*z + a^2*b^2*c^2*y^2*z + b^4*c^2*y^2*z + a^2*c^4*y^2*z +      a^4*b^2*x*z^2 + a^2*b^2*c^2*x*z^2 + b^4*c^2*x*z^2 + a^2*c^4*x*z^2 - a^2*b^4*y*z^2 - a^4*c^2*y*z^2 -      a^2*b^2*c^2*y*z^2 - b^2*c^4*y*z^2");
		exp("K549","a^6*b^8*c^2*x^2*y - a^6*b^6*c^4*x^2*y + a^4*b^8*c^4*x^2*y - 2*a^2*b^10*c^4*x^2*y -      2*a^8*b^2*c^6*x^2*y + a^6*b^4*c^6*x^2*y - a^4*b^6*c^6*x^2*y + a^2*b^8*c^6*x^2*y + b^10*c^6*x^2*y +      a^8*c^8*x^2*y + a^6*b^2*c^8*x^2*y + 2*a^4*b^4*c^8*x^2*y - 2*a^2*b^6*c^8*x^2*y -      2*a^4*b^2*c^10*x^2*y + a^2*b^4*c^10*x^2*y - a^8*b^6*c^2*x*y^2 + 2*a^10*b^2*c^4*x*y^2 -      a^8*b^4*c^4*x*y^2 + a^6*b^6*c^4*x*y^2 - a^10*c^6*x*y^2 - a^8*b^2*c^6*x*y^2 + a^6*b^4*c^6*x*y^2 -      a^4*b^6*c^6*x*y^2 + 2*a^2*b^8*c^6*x*y^2 + 2*a^6*b^2*c^8*x*y^2 - 2*a^4*b^4*c^8*x*y^2 -      a^2*b^6*c^8*x*y^2 - b^8*c^8*x*y^2 - a^4*b^2*c^10*x*y^2 + 2*a^2*b^4*c^10*x*y^2 - a^8*b^8*x^2*z +      2*a^8*b^6*c^2*x^2*z - a^6*b^8*c^2*x^2*z + 2*a^4*b^10*c^2*x^2*z - a^6*b^6*c^4*x^2*z -      2*a^4*b^8*c^4*x^2*z - a^2*b^10*c^4*x^2*z + a^6*b^4*c^6*x^2*z + a^4*b^6*c^6*x^2*z +      2*a^2*b^8*c^6*x^2*z - a^6*b^2*c^8*x^2*z - a^4*b^4*c^8*x^2*z - a^2*b^6*c^8*x^2*z +      2*a^2*b^4*c^10*x^2*z - b^6*c^10*x^2*z + a^8*b^8*y^2*z - 2*a^10*b^4*c^2*y^2*z + a^8*b^6*c^2*y^2*z -      2*a^6*b^8*c^2*y^2*z + a^10*b^2*c^4*y^2*z + 2*a^8*b^4*c^4*y^2*z + a^6*b^6*c^4*y^2*z -      2*a^8*b^2*c^6*y^2*z - a^6*b^4*c^6*y^2*z - a^4*b^6*c^6*y^2*z + a^6*b^2*c^8*y^2*z +      a^4*b^4*c^8*y^2*z + a^2*b^6*c^8*y^2*z + a^6*c^10*y^2*z - 2*a^4*b^2*c^10*y^2*z + a^10*b^6*x*z^2 -      2*a^10*b^4*c^2*x*z^2 + a^8*b^6*c^2*x*z^2 - 2*a^6*b^8*c^2*x*z^2 + a^4*b^10*c^2*x*z^2 +      a^8*b^4*c^4*x*z^2 - a^6*b^6*c^4*x*z^2 + 2*a^4*b^8*c^4*x*z^2 - 2*a^2*b^10*c^4*x*z^2 +      a^8*b^2*c^6*x*z^2 - a^6*b^4*c^6*x*z^2 + a^4*b^6*c^6*x*z^2 + a^2*b^8*c^6*x*z^2 -      2*a^2*b^6*c^8*x*z^2 + b^8*c^8*x*z^2 - a^6*b^10*y*z^2 - a^10*b^4*c^2*y*z^2 + 2*a^8*b^6*c^2*y*z^2 -      a^6*b^8*c^2*y*z^2 + 2*a^4*b^10*c^2*y*z^2 + 2*a^10*b^2*c^4*y*z^2 - 2*a^8*b^4*c^4*y*z^2 +      a^6*b^6*c^4*y*z^2 - a^4*b^8*c^4*y*z^2 - a^8*b^2*c^6*y*z^2 - a^6*b^4*c^6*y*z^2 + a^4*b^6*c^6*y*z^2 -      a^2*b^8*c^6*y*z^2 - a^8*c^8*y*z^2 + 2*a^6*b^2*c^8*y*z^2");
		exp("K550","a^4*b^2*x^2*y - 2*a^2*b^2*c^2*x^2*y + b^4*c^2*x^2*y + a^2*c^4*x^2*y - a^2*b^4*x*y^2 -      a^4*c^2*x*y^2 + 2*a^2*b^2*c^2*x*y^2 - b^2*c^4*x*y^2 - a^2*b^4*x^2*z - a^4*c^2*x^2*z +      2*a^2*b^2*c^2*x^2*z - b^2*c^4*x^2*z - 2*a^4*b^2*x*y*z + 2*a^2*b^4*x*y*z + 2*a^4*c^2*x*y*z -      2*b^4*c^2*x*y*z - 2*a^2*c^4*x*y*z + 2*b^2*c^4*x*y*z + a^4*b^2*y^2*z - 2*a^2*b^2*c^2*y^2*z +      b^4*c^2*y^2*z + a^2*c^4*y^2*z + a^4*b^2*x*z^2 - 2*a^2*b^2*c^2*x*z^2 + b^4*c^2*x*z^2 +      a^2*c^4*x*z^2 - a^2*b^4*y*z^2 - a^4*c^2*y*z^2 + 2*a^2*b^2*c^2*y*z^2 - b^2*c^4*y*z^2");
		exp("K551","a^4*b^2*x^2*y - 4*a^2*b^2*c^2*x^2*y + b^4*c^2*x^2*y + a^2*c^4*x^2*y - a^2*b^4*x*y^2 -      a^4*c^2*x*y^2 + 4*a^2*b^2*c^2*x*y^2 - b^2*c^4*x*y^2 - a^2*b^4*x^2*z - a^4*c^2*x^2*z +      4*a^2*b^2*c^2*x^2*z - b^2*c^4*x^2*z - 4*a^4*b^2*x*y*z + 4*a^2*b^4*x*y*z + 4*a^4*c^2*x*y*z -      4*b^4*c^2*x*y*z - 4*a^2*c^4*x*y*z + 4*b^2*c^4*x*y*z + a^4*b^2*y^2*z - 4*a^2*b^2*c^2*y^2*z +      b^4*c^2*y^2*z + a^2*c^4*y^2*z + a^4*b^2*x*z^2 - 4*a^2*b^2*c^2*x*z^2 + b^4*c^2*x*z^2 +      a^2*c^4*x*z^2 - a^2*b^4*y*z^2 - a^4*c^2*y*z^2 + 4*a^2*b^2*c^2*y*z^2 - b^2*c^4*y*z^2");
		exp("K552","(-a^8)*b^4*x^2*y + a^6*b^6*x^2*y + a^4*b^6*c^2*x^2*y + a^6*b^2*c^4*x^2*y - 3*a^4*b^4*c^4*x^2*y -      b^8*c^4*x^2*y + a^6*c^6*x^2*y + a^2*b^4*c^6*x^2*y + b^6*c^6*x^2*y - a^4*c^8*x^2*y + a^6*b^6*x*y^2 -      a^4*b^8*x*y^2 + a^6*b^4*c^2*x*y^2 - a^8*c^4*x*y^2 - 3*a^4*b^4*c^4*x*y^2 + a^2*b^6*c^4*x*y^2 +      a^6*c^6*x*y^2 + a^4*b^2*c^6*x*y^2 + b^6*c^6*x*y^2 - b^4*c^8*x*y^2 + a^6*b^6*x^2*z - a^4*b^8*x^2*z +      a^6*b^4*c^2*x^2*z - a^8*c^4*x^2*z - 3*a^4*b^4*c^4*x^2*z + a^2*b^6*c^4*x^2*z + a^6*c^6*x^2*z +      a^4*b^2*c^6*x^2*z + b^6*c^6*x^2*z - b^4*c^8*x^2*z + a^10*b^2*x*y*z - 2*a^8*b^4*x*y*z -      2*a^4*b^8*x*y*z + a^2*b^10*x*y*z + a^10*c^2*x*y*z + a^6*b^4*c^2*x*y*z + a^4*b^6*c^2*x*y*z +      b^10*c^2*x*y*z - 2*a^8*c^4*x*y*z + a^6*b^2*c^4*x*y*z + a^2*b^6*c^4*x*y*z - 2*b^8*c^4*x*y*z +      a^4*b^2*c^6*x*y*z + a^2*b^4*c^6*x*y*z - 2*a^4*c^8*x*y*z - 2*b^4*c^8*x*y*z + a^2*c^10*x*y*z +      b^2*c^10*x*y*z - a^8*b^4*y^2*z + a^6*b^6*y^2*z + a^4*b^6*c^2*y^2*z + a^6*b^2*c^4*y^2*z -      3*a^4*b^4*c^4*y^2*z - b^8*c^4*y^2*z + a^6*c^6*y^2*z + a^2*b^4*c^6*y^2*z + b^6*c^6*y^2*z -      a^4*c^8*y^2*z - a^8*b^4*x*z^2 + a^6*b^6*x*z^2 + a^4*b^6*c^2*x*z^2 + a^6*b^2*c^4*x*z^2 -      3*a^4*b^4*c^4*x*z^2 - b^8*c^4*x*z^2 + a^6*c^6*x*z^2 + a^2*b^4*c^6*x*z^2 + b^6*c^6*x*z^2 -      a^4*c^8*x*z^2 + a^6*b^6*y*z^2 - a^4*b^8*y*z^2 + a^6*b^4*c^2*y*z^2 - a^8*c^4*y*z^2 -      3*a^4*b^4*c^4*y*z^2 + a^2*b^6*c^4*y*z^2 + a^6*c^6*y*z^2 + a^4*b^2*c^6*y*z^2 + b^6*c^6*y*z^2 -      b^4*c^8*y*z^2");
		exp("K553","a^4*b^2*x^2*y - 3*a^2*b^2*c^2*x^2*y + b^4*c^2*x^2*y + a^2*c^4*x^2*y - a^2*b^4*x*y^2 -      a^4*c^2*x*y^2 + 3*a^2*b^2*c^2*x*y^2 - b^2*c^4*x*y^2 - a^2*b^4*x^2*z - a^4*c^2*x^2*z +      3*a^2*b^2*c^2*x^2*z - b^2*c^4*x^2*z - 3*a^4*b^2*x*y*z + 3*a^2*b^4*x*y*z + 3*a^4*c^2*x*y*z -      3*b^4*c^2*x*y*z - 3*a^2*c^4*x*y*z + 3*b^2*c^4*x*y*z + a^4*b^2*y^2*z - 3*a^2*b^2*c^2*y^2*z +      b^4*c^2*y^2*z + a^2*c^4*y^2*z + a^4*b^2*x*z^2 - 3*a^2*b^2*c^2*x*z^2 + b^4*c^2*x*z^2 +      a^2*c^4*x*z^2 - a^2*b^4*y*z^2 - a^4*c^2*y*z^2 + 3*a^2*b^2*c^2*y*z^2 - b^2*c^4*y*z^2");
		exp("K554","a^6*b^2*c^4*x^2*y - 2*a^4*b^4*c^4*x^2*y + a^2*b^6*c^4*x^2*y + a^4*b^2*c^6*x^2*y -      2*a^2*b^4*c^6*x^2*y + b^6*c^6*x^2*y - a^6*b^2*c^4*x*y^2 + 2*a^4*b^4*c^4*x*y^2 - a^2*b^6*c^4*x*y^2 -      a^6*c^6*x*y^2 + 2*a^4*b^2*c^6*x*y^2 - a^2*b^4*c^6*x*y^2 - a^6*b^4*c^2*x^2*z - a^4*b^6*c^2*x^2*z +      2*a^4*b^4*c^4*x^2*z + 2*a^2*b^6*c^4*x^2*z - a^2*b^4*c^6*x^2*z - b^6*c^6*x^2*z + a^6*b^4*c^2*y^2*z +      a^4*b^6*c^2*y^2*z - 2*a^6*b^2*c^4*y^2*z - 2*a^4*b^4*c^4*y^2*z + a^6*c^6*y^2*z + a^4*b^2*c^6*y^2*z +      a^6*b^6*x*z^2 + a^6*b^4*c^2*x*z^2 - 2*a^4*b^6*c^2*x*z^2 - 2*a^4*b^4*c^4*x*z^2 + a^2*b^6*c^4*x*z^2 +      a^2*b^4*c^6*x*z^2 - a^6*b^6*y*z^2 + 2*a^6*b^4*c^2*y*z^2 - a^4*b^6*c^2*y*z^2 - a^6*b^2*c^4*y*z^2 +      2*a^4*b^4*c^4*y*z^2 - a^4*b^2*c^6*y*z^2");
		exp("K555","a^4*c^2*x^2*y - a^2*b^2*c^2*x^2*y - a^2*c^4*x^2*y + a^2*b^2*c^2*x*y^2 - b^4*c^2*x*y^2 +      b^2*c^4*x*y^2 - a^4*b^2*x^2*z + a^2*b^4*x^2*z + a^2*b^2*c^2*x^2*z + 2*a^4*b^2*x*y*z -      2*a^2*b^4*x*y*z - 2*a^4*c^2*x*y*z + 2*b^4*c^2*x*y*z + 2*a^2*c^4*x*y*z - 2*b^2*c^4*x*y*z -      a^4*b^2*y^2*z + a^2*b^4*y^2*z - a^2*b^2*c^2*y^2*z - a^2*b^2*c^2*x*z^2 - b^4*c^2*x*z^2 +      b^2*c^4*x*z^2 + a^4*c^2*y*z^2 + a^2*b^2*c^2*y*z^2 - a^2*c^4*y*z^2");
		exp("K556","(-a^8)*b^2*c^2*x^2*y + 2*a^6*b^4*c^2*x^2*y - a^4*b^6*c^2*x^2*y + a^8*c^4*x^2*y -      2*a^4*b^4*c^4*x^2*y + a^2*b^6*c^4*x^2*y - 2*a^6*c^6*x^2*y + 2*a^4*b^2*c^6*x^2*y + a^4*c^8*x^2*y -      a^2*b^2*c^8*x^2*y - a^6*b^4*c^2*x*y^2 + 2*a^4*b^6*c^2*x*y^2 - a^2*b^8*c^2*x*y^2 +      a^6*b^2*c^4*x*y^2 - 2*a^4*b^4*c^4*x*y^2 + b^8*c^4*x*y^2 + 2*a^2*b^4*c^6*x*y^2 - 2*b^6*c^6*x*y^2 -      a^2*b^2*c^8*x*y^2 + b^4*c^8*x*y^2 + a^8*b^4*x^2*z - 2*a^6*b^6*x^2*z + a^4*b^8*x^2*z -      a^8*b^2*c^2*x^2*z + 2*a^4*b^6*c^2*x^2*z - a^2*b^8*c^2*x^2*z + 2*a^6*b^2*c^4*x^2*z -      2*a^4*b^4*c^4*x^2*z - a^4*b^2*c^6*x^2*z + a^2*b^4*c^6*x^2*z + a^10*b^2*x*y*z - 2*a^8*b^4*x*y*z +      2*a^6*b^6*x*y*z - 2*a^4*b^8*x*y*z + a^2*b^10*x*y*z + a^10*c^2*x*y*z - 2*a^8*b^2*c^2*x*y*z +      a^6*b^4*c^2*x*y*z + a^4*b^6*c^2*x*y*z - 2*a^2*b^8*c^2*x*y*z + b^10*c^2*x*y*z - 2*a^8*c^4*x*y*z +      a^6*b^2*c^4*x*y*z + a^2*b^6*c^4*x*y*z - 2*b^8*c^4*x*y*z + 2*a^6*c^6*x*y*z + a^4*b^2*c^6*x*y*z +      a^2*b^4*c^6*x*y*z + 2*b^6*c^6*x*y*z - 2*a^4*c^8*x*y*z - 2*a^2*b^2*c^8*x*y*z - 2*b^4*c^8*x*y*z +      a^2*c^10*x*y*z + b^2*c^10*x*y*z + a^8*b^4*y^2*z - 2*a^6*b^6*y^2*z + a^4*b^8*y^2*z -      a^8*b^2*c^2*y^2*z + 2*a^6*b^4*c^2*y^2*z - a^2*b^8*c^2*y^2*z - 2*a^4*b^4*c^4*y^2*z +      2*a^2*b^6*c^4*y^2*z + a^4*b^2*c^6*y^2*z - a^2*b^4*c^6*y^2*z + a^6*b^4*c^2*x*z^2 -      a^2*b^8*c^2*x*z^2 - a^6*b^2*c^4*x*z^2 - 2*a^4*b^4*c^4*x*z^2 + 2*a^2*b^6*c^4*x*z^2 + b^8*c^4*x*z^2 +      2*a^4*b^2*c^6*x*z^2 - 2*b^6*c^6*x*z^2 - a^2*b^2*c^8*x*z^2 + b^4*c^8*x*z^2 - a^8*b^2*c^2*y*z^2 +      a^4*b^6*c^2*y*z^2 + a^8*c^4*y*z^2 + 2*a^6*b^2*c^4*y*z^2 - 2*a^4*b^4*c^4*y*z^2 - a^2*b^6*c^4*y*z^2 -      2*a^6*c^6*y*z^2 + 2*a^2*b^4*c^6*y*z^2 + a^4*c^8*y*z^2 - a^2*b^2*c^8*y*z^2");
		exp("K557","a^12*x^2*y - 6*a^10*b^2*x^2*y + 15*a^8*b^4*x^2*y - 20*a^6*b^6*x^2*y + 15*a^4*b^8*x^2*y -      6*a^2*b^10*x^2*y + b^12*x^2*y - 3*a^8*c^4*x^2*y + 9*a^6*b^2*c^4*x^2*y - 12*a^4*b^4*c^4*x^2*y +      9*a^2*b^6*c^4*x^2*y - 3*b^8*c^4*x^2*y + a^6*c^6*x^2*y + b^6*c^6*x^2*y - 6*a^2*b^2*c^8*x^2*y +      3*a^2*c^10*x^2*y + 3*b^2*c^10*x^2*y - 2*c^12*x^2*y - a^12*x*y^2 + 6*a^10*b^2*x*y^2 -      15*a^8*b^4*x*y^2 + 20*a^6*b^6*x*y^2 - 15*a^4*b^8*x*y^2 + 6*a^2*b^10*x*y^2 - b^12*x*y^2 +      3*a^8*c^4*x*y^2 - 9*a^6*b^2*c^4*x*y^2 + 12*a^4*b^4*c^4*x*y^2 - 9*a^2*b^6*c^4*x*y^2 +      3*b^8*c^4*x*y^2 - a^6*c^6*x*y^2 - b^6*c^6*x*y^2 + 6*a^2*b^2*c^8*x*y^2 - 3*a^2*c^10*x*y^2 -      3*b^2*c^10*x*y^2 + 2*c^12*x*y^2 - a^12*x^2*z + 3*a^8*b^4*x^2*z - a^6*b^6*x^2*z - 3*a^2*b^10*x^2*z +      2*b^12*x^2*z + 6*a^10*c^2*x^2*z - 9*a^6*b^4*c^2*x^2*z + 6*a^2*b^8*c^2*x^2*z - 3*b^10*c^2*x^2*z -      15*a^8*c^4*x^2*z + 12*a^4*b^4*c^4*x^2*z + 20*a^6*c^6*x^2*z - 9*a^2*b^4*c^6*x^2*z - b^6*c^6*x^2*z -      15*a^4*c^8*x^2*z + 3*b^4*c^8*x^2*z + 6*a^2*c^10*x^2*z - c^12*x^2*z - 9*a^10*b^2*x*y*z +      18*a^8*b^4*x*y*z - 18*a^4*b^8*x*y*z + 9*a^2*b^10*x*y*z + 9*a^10*c^2*x*y*z - 27*a^6*b^4*c^2*x*y*z +      27*a^4*b^6*c^2*x*y*z - 9*b^10*c^2*x*y*z - 18*a^8*c^4*x*y*z + 27*a^6*b^2*c^4*x*y*z -      27*a^2*b^6*c^4*x*y*z + 18*b^8*c^4*x*y*z - 27*a^4*b^2*c^6*x*y*z + 27*a^2*b^4*c^6*x*y*z +      18*a^4*c^8*x*y*z - 18*b^4*c^8*x*y*z - 9*a^2*c^10*x*y*z + 9*b^2*c^10*x*y*z - 2*a^12*y^2*z +      3*a^10*b^2*y^2*z + a^6*b^6*y^2*z - 3*a^4*b^8*y^2*z + b^12*y^2*z + 3*a^10*c^2*y^2*z -      6*a^8*b^2*c^2*y^2*z + 9*a^4*b^6*c^2*y^2*z - 6*b^10*c^2*y^2*z - 12*a^4*b^4*c^4*y^2*z +      15*b^8*c^4*y^2*z + a^6*c^6*y^2*z + 9*a^4*b^2*c^6*y^2*z - 20*b^6*c^6*y^2*z - 3*a^4*c^8*y^2*z +      15*b^4*c^8*y^2*z - 6*b^2*c^10*y^2*z + c^12*y^2*z + a^12*x*z^2 - 3*a^8*b^4*x*z^2 + a^6*b^6*x*z^2 +      3*a^2*b^10*x*z^2 - 2*b^12*x*z^2 - 6*a^10*c^2*x*z^2 + 9*a^6*b^4*c^2*x*z^2 - 6*a^2*b^8*c^2*x*z^2 +      3*b^10*c^2*x*z^2 + 15*a^8*c^4*x*z^2 - 12*a^4*b^4*c^4*x*z^2 - 20*a^6*c^6*x*z^2 +      9*a^2*b^4*c^6*x*z^2 + b^6*c^6*x*z^2 + 15*a^4*c^8*x*z^2 - 3*b^4*c^8*x*z^2 - 6*a^2*c^10*x*z^2 +      c^12*x*z^2 + 2*a^12*y*z^2 - 3*a^10*b^2*y*z^2 - a^6*b^6*y*z^2 + 3*a^4*b^8*y*z^2 - b^12*y*z^2 -      3*a^10*c^2*y*z^2 + 6*a^8*b^2*c^2*y*z^2 - 9*a^4*b^6*c^2*y*z^2 + 6*b^10*c^2*y*z^2 +      12*a^4*b^4*c^4*y*z^2 - 15*b^8*c^4*y*z^2 - a^6*c^6*y*z^2 - 9*a^4*b^2*c^6*y*z^2 + 20*b^6*c^6*y*z^2 +      3*a^4*c^8*y*z^2 - 15*b^4*c^8*y*z^2 + 6*b^2*c^10*y*z^2 - c^12*y*z^2");
		exp("K558","-3*a^10*c^2*x^2*y + 5*a^8*b^2*c^2*x^2*y + 2*a^6*b^4*c^2*x^2*y - 6*a^4*b^6*c^2*x^2*y +      a^2*b^8*c^2*x^2*y + b^10*c^2*x^2*y + 8*a^8*c^4*x^2*y - 8*a^6*b^2*c^4*x^2*y + 4*a^4*b^4*c^4*x^2*y -      4*b^8*c^4*x^2*y - 6*a^6*c^6*x^2*y + 2*a^4*b^2*c^6*x^2*y - 2*a^2*b^4*c^6*x^2*y + 6*b^6*c^6*x^2*y -      4*b^4*c^8*x^2*y + a^2*c^10*x^2*y + b^2*c^10*x^2*y - a^10*c^2*x*y^2 - a^8*b^2*c^2*x*y^2 +      6*a^6*b^4*c^2*x*y^2 - 2*a^4*b^6*c^2*x*y^2 - 5*a^2*b^8*c^2*x*y^2 + 3*b^10*c^2*x*y^2 +      4*a^8*c^4*x*y^2 - 4*a^4*b^4*c^4*x*y^2 + 8*a^2*b^6*c^4*x*y^2 - 8*b^8*c^4*x*y^2 - 6*a^6*c^6*x*y^2 +      2*a^4*b^2*c^6*x*y^2 - 2*a^2*b^4*c^6*x*y^2 + 6*b^6*c^6*x*y^2 + 4*a^4*c^8*x*y^2 - a^2*c^10*x*y^2 -      b^2*c^10*x*y^2 + 3*a^10*b^2*x^2*z - 8*a^8*b^4*x^2*z + 6*a^6*b^6*x^2*z - a^2*b^10*x^2*z -      5*a^8*b^2*c^2*x^2*z + 8*a^6*b^4*c^2*x^2*z - 2*a^4*b^6*c^2*x^2*z - b^10*c^2*x^2*z -      2*a^6*b^2*c^4*x^2*z - 4*a^4*b^4*c^4*x^2*z + 2*a^2*b^6*c^4*x^2*z + 4*b^8*c^4*x^2*z +      6*a^4*b^2*c^6*x^2*z - 6*b^6*c^6*x^2*z - a^2*b^2*c^8*x^2*z + 4*b^4*c^8*x^2*z - b^2*c^10*x^2*z +      4*a^10*b^2*x*y*z - 8*a^8*b^4*x*y*z + 8*a^4*b^8*x*y*z - 4*a^2*b^10*x*y*z - 4*a^10*c^2*x*y*z +      20*a^6*b^4*c^2*x*y*z - 20*a^4*b^6*c^2*x*y*z + 4*b^10*c^2*x*y*z + 8*a^8*c^4*x*y*z -      20*a^6*b^2*c^4*x*y*z + 20*a^2*b^6*c^4*x*y*z - 8*b^8*c^4*x*y*z + 20*a^4*b^2*c^6*x*y*z -      20*a^2*b^4*c^6*x*y*z - 8*a^4*c^8*x*y*z + 8*b^4*c^8*x*y*z + 4*a^2*c^10*x*y*z - 4*b^2*c^10*x*y*z +      a^10*b^2*y^2*z - 6*a^6*b^6*y^2*z + 8*a^4*b^8*y^2*z - 3*a^2*b^10*y^2*z + a^10*c^2*y^2*z +      2*a^6*b^4*c^2*y^2*z - 8*a^4*b^6*c^2*y^2*z + 5*a^2*b^8*c^2*y^2*z - 4*a^8*c^4*y^2*z -      2*a^6*b^2*c^4*y^2*z + 4*a^4*b^4*c^4*y^2*z + 2*a^2*b^6*c^4*y^2*z + 6*a^6*c^6*y^2*z -      6*a^2*b^4*c^6*y^2*z - 4*a^4*c^8*y^2*z + a^2*b^2*c^8*y^2*z + a^2*c^10*y^2*z + a^10*b^2*x*z^2 -      4*a^8*b^4*x*z^2 + 6*a^6*b^6*x*z^2 - 4*a^4*b^8*x*z^2 + a^2*b^10*x*z^2 + a^8*b^2*c^2*x*z^2 -      2*a^4*b^6*c^2*x*z^2 + b^10*c^2*x*z^2 - 6*a^6*b^2*c^4*x*z^2 + 4*a^4*b^4*c^4*x*z^2 +      2*a^2*b^6*c^4*x*z^2 + 2*a^4*b^2*c^6*x*z^2 - 8*a^2*b^4*c^6*x*z^2 - 6*b^6*c^6*x*z^2 +      5*a^2*b^2*c^8*x*z^2 + 8*b^4*c^8*x*z^2 - 3*b^2*c^10*x*z^2 - a^10*b^2*y*z^2 + 4*a^8*b^4*y*z^2 -      6*a^6*b^6*y*z^2 + 4*a^4*b^8*y*z^2 - a^2*b^10*y*z^2 - a^10*c^2*y*z^2 + 2*a^6*b^4*c^2*y*z^2 -      a^2*b^8*c^2*y*z^2 - 2*a^6*b^2*c^4*y*z^2 - 4*a^4*b^4*c^4*y*z^2 + 6*a^2*b^6*c^4*y*z^2 +      6*a^6*c^6*y*z^2 + 8*a^4*b^2*c^6*y*z^2 - 2*a^2*b^4*c^6*y*z^2 - 8*a^4*c^8*y*z^2 -      5*a^2*b^2*c^8*y*z^2 + 3*a^2*c^10*y*z^2");
		exp("K559","a^2*b^2*c^2*x^2*y - a^2*c^4*x^2*y - b^2*c^4*x^2*y + c^6*x^2*y - a^2*b^2*c^2*x*y^2 +      a^2*c^4*x*y^2 + b^2*c^4*x*y^2 - c^6*x*y^2 + a^2*b^4*x^2*z - b^6*x^2*z - a^2*b^2*c^2*x^2*z +      b^4*c^2*x^2*z + 3*a^4*b^2*x*y*z - 3*a^2*b^4*x*y*z - 3*a^4*c^2*x*y*z + 3*b^4*c^2*x*y*z +      3*a^2*c^4*x*y*z - 3*b^2*c^4*x*y*z + a^6*y^2*z - a^4*b^2*y^2*z - a^4*c^2*y^2*z + a^2*b^2*c^2*y^2*z -      a^2*b^4*x*z^2 + b^6*x*z^2 + a^2*b^2*c^2*x*z^2 - b^4*c^2*x*z^2 - a^6*y*z^2 + a^4*b^2*y*z^2 +      a^4*c^2*y*z^2 - a^2*b^2*c^2*y*z^2");
		exp("K560","(-a^6)*b*c^2*x^2*y + 3*a^4*b^3*c^2*x^2*y - 3*a^2*b^5*c^2*x^2*y + b^7*c^2*x^2*y +      6*a^5*b*c^3*x^2*y - 6*a^4*b^2*c^3*x^2*y - 12*a^3*b^3*c^3*x^2*y + 12*a^2*b^4*c^3*x^2*y +      6*a*b^5*c^3*x^2*y - 6*b^6*c^3*x^2*y - 12*a^4*b*c^4*x^2*y + 24*a^3*b^2*c^4*x^2*y -      24*a*b^4*c^4*x^2*y + 12*b^5*c^4*x^2*y + 8*a^3*b*c^5*x^2*y - 24*a^2*b^2*c^5*x^2*y +      24*a*b^3*c^5*x^2*y - 8*b^4*c^5*x^2*y + a^7*c^2*x*y^2 - 3*a^5*b^2*c^2*x*y^2 + 3*a^3*b^4*c^2*x*y^2 -      a*b^6*c^2*x*y^2 - 6*a^6*c^3*x*y^2 + 6*a^5*b*c^3*x*y^2 + 12*a^4*b^2*c^3*x*y^2 -      12*a^3*b^3*c^3*x*y^2 - 6*a^2*b^4*c^3*x*y^2 + 6*a*b^5*c^3*x*y^2 + 12*a^5*c^4*x*y^2 -      24*a^4*b*c^4*x*y^2 + 24*a^2*b^3*c^4*x*y^2 - 12*a*b^4*c^4*x*y^2 - 8*a^4*c^5*x*y^2 +      24*a^3*b*c^5*x*y^2 - 24*a^2*b^2*c^5*x*y^2 + 8*a*b^3*c^5*x*y^2 - a^6*b^2*c*x^2*z +      6*a^5*b^3*c*x^2*z - 12*a^4*b^4*c*x^2*z + 8*a^3*b^5*c*x^2*z - 6*a^4*b^3*c^2*x^2*z +      24*a^3*b^4*c^2*x^2*z - 24*a^2*b^5*c^2*x^2*z + 3*a^4*b^2*c^3*x^2*z - 12*a^3*b^3*c^3*x^2*z +      24*a*b^5*c^3*x^2*z + 12*a^2*b^3*c^4*x^2*z - 24*a*b^4*c^4*x^2*z - 8*b^5*c^4*x^2*z -      3*a^2*b^2*c^5*x^2*z + 6*a*b^3*c^5*x^2*z + 12*b^4*c^5*x^2*z - 6*b^3*c^6*x^2*z + b^2*c^7*x^2*z -      2*a^7*b*c*x*y*z + 6*a^6*b^2*c*x*y*z - 12*a^5*b^3*c*x*y*z + 14*a^4*b^4*c*x*y*z -      12*a^3*b^5*c*x*y*z + 6*a^2*b^6*c*x*y*z - 2*a*b^7*c*x*y*z + 6*a^6*b*c^2*x*y*z -      6*a^5*b^2*c^2*x*y*z + 6*a^4*b^3*c^2*x*y*z + 6*a^3*b^4*c^2*x*y*z - 6*a^2*b^5*c^2*x*y*z +      6*a*b^6*c^2*x*y*z - 12*a^5*b*c^3*x*y*z + 6*a^4*b^2*c^3*x*y*z - 18*a^3*b^3*c^3*x*y*z +      6*a^2*b^4*c^3*x*y*z - 12*a*b^5*c^3*x*y*z + 14*a^4*b*c^4*x*y*z + 6*a^3*b^2*c^4*x*y*z +      6*a^2*b^3*c^4*x*y*z + 14*a*b^4*c^4*x*y*z - 12*a^3*b*c^5*x*y*z - 6*a^2*b^2*c^5*x*y*z -      12*a*b^3*c^5*x*y*z + 6*a^2*b*c^6*x*y*z + 6*a*b^2*c^6*x*y*z - 2*a*b*c^7*x*y*z + 8*a^5*b^3*c*y^2*z -      12*a^4*b^4*c*y^2*z + 6*a^3*b^5*c*y^2*z - a^2*b^6*c*y^2*z - 24*a^5*b^2*c^2*y^2*z +      24*a^4*b^3*c^2*y^2*z - 6*a^3*b^4*c^2*y^2*z + 24*a^5*b*c^3*y^2*z - 12*a^3*b^3*c^3*y^2*z +      3*a^2*b^4*c^3*y^2*z - 8*a^5*c^4*y^2*z - 24*a^4*b*c^4*y^2*z + 12*a^3*b^2*c^4*y^2*z +      12*a^4*c^5*y^2*z + 6*a^3*b*c^5*y^2*z - 3*a^2*b^2*c^5*y^2*z - 6*a^3*c^6*y^2*z + a^2*c^7*y^2*z +      a^7*b^2*x*z^2 - 6*a^6*b^3*x*z^2 + 12*a^5*b^4*x*z^2 - 8*a^4*b^5*x*z^2 + 6*a^5*b^3*c*x*z^2 -      24*a^4*b^4*c*x*z^2 + 24*a^3*b^5*c*x*z^2 - 3*a^5*b^2*c^2*x*z^2 + 12*a^4*b^3*c^2*x*z^2 -      24*a^2*b^5*c^2*x*z^2 - 12*a^3*b^3*c^3*x*z^2 + 24*a^2*b^4*c^3*x*z^2 + 8*a*b^5*c^3*x*z^2 +      3*a^3*b^2*c^4*x*z^2 - 6*a^2*b^3*c^4*x*z^2 - 12*a*b^4*c^4*x*z^2 + 6*a*b^3*c^5*x*z^2 -      a*b^2*c^6*x*z^2 - 8*a^5*b^4*y*z^2 + 12*a^4*b^5*y*z^2 - 6*a^3*b^6*y*z^2 + a^2*b^7*y*z^2 +      24*a^5*b^3*c*y*z^2 - 24*a^4*b^4*c*y*z^2 + 6*a^3*b^5*c*y*z^2 - 24*a^5*b^2*c^2*y*z^2 +      12*a^3*b^4*c^2*y*z^2 - 3*a^2*b^5*c^2*y*z^2 + 8*a^5*b*c^3*y*z^2 + 24*a^4*b^2*c^3*y*z^2 -      12*a^3*b^3*c^3*y*z^2 - 12*a^4*b*c^4*y*z^2 - 6*a^3*b^2*c^4*y*z^2 + 3*a^2*b^3*c^4*y*z^2 +      6*a^3*b*c^5*y*z^2 - a^2*b*c^6*y*z^2");
		exp("K561","a^3*c^3*x^2*y - a^2*b*c^3*x^2*y + a*b^2*c^3*x^2*y - b^3*c^3*x^2*y - a^2*c^4*x^2*y -      b^2*c^4*x^2*y - a*c^5*x^2*y + b*c^5*x^2*y + c^6*x^2*y + a^3*c^3*x*y^2 - a^2*b*c^3*x*y^2 +      a*b^2*c^3*x*y^2 - b^3*c^3*x*y^2 + a^2*c^4*x*y^2 + b^2*c^4*x*y^2 - a*c^5*x*y^2 + b*c^5*x*y^2 -      c^6*x*y^2 - a^3*b^3*x^2*z + a^2*b^4*x^2*z + a*b^5*x^2*z - b^6*x^2*z + a^2*b^3*c*x^2*z -      b^5*c*x^2*z - a*b^3*c^2*x^2*z + b^4*c^2*x^2*z + b^3*c^3*x^2*z - a^5*b*x*y*z + 2*a^4*b^2*x*y*z -      2*a^2*b^4*x*y*z + a*b^5*x*y*z + a^5*c*x*y*z - a^3*b^2*c*x*y*z + a^2*b^3*c*x*y*z - b^5*c*x*y*z -      2*a^4*c^2*x*y*z + a^3*b*c^2*x*y*z - a*b^3*c^2*x*y*z + 2*b^4*c^2*x*y*z - a^2*b*c^3*x*y*z +      a*b^2*c^3*x*y*z + 2*a^2*c^4*x*y*z - 2*b^2*c^4*x*y*z - a*c^5*x*y*z + b*c^5*x*y*z + a^6*y^2*z -      a^5*b*y^2*z - a^4*b^2*y^2*z + a^3*b^3*y^2*z + a^5*c*y^2*z - a^3*b^2*c*y^2*z - a^4*c^2*y^2*z +      a^3*b*c^2*y^2*z - a^3*c^3*y^2*z - a^3*b^3*x*z^2 - a^2*b^4*x*z^2 + a*b^5*x*z^2 + b^6*x*z^2 +      a^2*b^3*c*x*z^2 - b^5*c*x*z^2 - a*b^3*c^2*x*z^2 - b^4*c^2*x*z^2 + b^3*c^3*x*z^2 - a^6*y*z^2 -      a^5*b*y*z^2 + a^4*b^2*y*z^2 + a^3*b^3*y*z^2 + a^5*c*y*z^2 - a^3*b^2*c*y*z^2 + a^4*c^2*y*z^2 +      a^3*b*c^2*y*z^2 - a^3*c^3*y*z^2");
		exp("K562","(-a^10)*c^2*x^2*y + 2*a^8*b^2*c^2*x^2*y - 2*a^4*b^6*c^2*x^2*y + a^2*b^8*c^2*x^2*y +      3*a^8*c^4*x^2*y - 3*a^6*b^2*c^4*x^2*y + a^4*b^4*c^4*x^2*y - a^2*b^6*c^4*x^2*y - 3*a^6*c^6*x^2*y -      a^2*b^4*c^6*x^2*y + a^4*c^8*x^2*y + a^2*b^2*c^8*x^2*y - a^8*b^2*c^2*x*y^2 + 2*a^6*b^4*c^2*x*y^2 -      2*a^2*b^8*c^2*x*y^2 + b^10*c^2*x*y^2 + a^6*b^2*c^4*x*y^2 - a^4*b^4*c^4*x*y^2 +      3*a^2*b^6*c^4*x*y^2 - 3*b^8*c^4*x*y^2 + a^4*b^2*c^6*x*y^2 + 3*b^6*c^6*x*y^2 - a^2*b^2*c^8*x*y^2 -      b^4*c^8*x*y^2 + a^10*b^2*x^2*z - 3*a^8*b^4*x^2*z + 3*a^6*b^6*x^2*z - a^4*b^8*x^2*z -      2*a^8*b^2*c^2*x^2*z + 3*a^6*b^4*c^2*x^2*z - a^2*b^8*c^2*x^2*z - a^4*b^4*c^4*x^2*z +      a^2*b^6*c^4*x^2*z + 2*a^4*b^2*c^6*x^2*z + a^2*b^4*c^6*x^2*z - a^2*b^2*c^8*x^2*z + a^10*b^2*x*y*z -      2*a^8*b^4*x*y*z + 2*a^4*b^8*x*y*z - a^2*b^10*x*y*z - a^10*c^2*x*y*z + 5*a^6*b^4*c^2*x*y*z -      5*a^4*b^6*c^2*x*y*z + b^10*c^2*x*y*z + 2*a^8*c^4*x*y*z - 5*a^6*b^2*c^4*x*y*z +      5*a^2*b^6*c^4*x*y*z - 2*b^8*c^4*x*y*z + 5*a^4*b^2*c^6*x*y*z - 5*a^2*b^4*c^6*x*y*z -      2*a^4*c^8*x*y*z + 2*b^4*c^8*x*y*z + a^2*c^10*x*y*z - b^2*c^10*x*y*z + a^8*b^4*y^2*z -      3*a^6*b^6*y^2*z + 3*a^4*b^8*y^2*z - a^2*b^10*y^2*z + a^8*b^2*c^2*y^2*z - 3*a^4*b^6*c^2*y^2*z +      2*a^2*b^8*c^2*y^2*z - a^6*b^2*c^4*y^2*z + a^4*b^4*c^4*y^2*z - a^4*b^2*c^6*y^2*z -      2*a^2*b^4*c^6*y^2*z + a^2*b^2*c^8*y^2*z + a^8*b^2*c^2*x*z^2 - a^6*b^4*c^2*x*z^2 -      a^4*b^6*c^2*x*z^2 + a^2*b^8*c^2*x*z^2 - 2*a^6*b^2*c^4*x*z^2 + a^4*b^4*c^4*x*z^2 + b^8*c^4*x*z^2 -      3*a^2*b^4*c^6*x*z^2 - 3*b^6*c^6*x*z^2 + 2*a^2*b^2*c^8*x*z^2 + 3*b^4*c^8*x*z^2 - b^2*c^10*x*z^2 -      a^8*b^2*c^2*y*z^2 + a^6*b^4*c^2*y*z^2 + a^4*b^6*c^2*y*z^2 - a^2*b^8*c^2*y*z^2 - a^8*c^4*y*z^2 -      a^4*b^4*c^4*y*z^2 + 2*a^2*b^6*c^4*y*z^2 + 3*a^6*c^6*y*z^2 + 3*a^4*b^2*c^6*y*z^2 - 3*a^4*c^8*y*z^2 -      2*a^2*b^2*c^8*y*z^2 + a^2*c^10*y*z^2");
		exp("K563","(-a^8)*c^4*x^2*y + a^2*b^6*c^4*x^2*y + 3*a^6*c^6*x^2*y + a^4*b^2*c^6*x^2*y - a^2*b^4*c^6*x^2*y -      3*a^4*c^8*x^2*y - a^2*b^2*c^8*x^2*y + a^2*c^10*x^2*y - a^6*b^2*c^4*x*y^2 + b^8*c^4*x*y^2 +      a^4*b^2*c^6*x*y^2 - a^2*b^4*c^6*x*y^2 - 3*b^6*c^6*x*y^2 + a^2*b^2*c^8*x*y^2 + 3*b^4*c^8*x*y^2 -      b^2*c^10*x*y^2 + a^8*b^4*x^2*z - 3*a^6*b^6*x^2*z + 3*a^4*b^8*x^2*z - a^2*b^10*x^2*z -      a^4*b^6*c^2*x^2*z + a^2*b^8*c^2*x^2*z + a^2*b^6*c^4*x^2*z - a^2*b^4*c^6*x^2*z + a^10*b^2*x*y*z -      2*a^8*b^4*x*y*z + 2*a^4*b^8*x*y*z - a^2*b^10*x*y*z - a^10*c^2*x*y*z + 3*a^6*b^4*c^2*x*y*z -      3*a^4*b^6*c^2*x*y*z + b^10*c^2*x*y*z + 2*a^8*c^4*x*y*z - 3*a^6*b^2*c^4*x*y*z +      3*a^2*b^6*c^4*x*y*z - 2*b^8*c^4*x*y*z + 3*a^4*b^2*c^6*x*y*z - 3*a^2*b^4*c^6*x*y*z -      2*a^4*c^8*x*y*z + 2*b^4*c^8*x*y*z + a^2*c^10*x*y*z - b^2*c^10*x*y*z + a^10*b^2*y^2*z -      3*a^8*b^4*y^2*z + 3*a^6*b^6*y^2*z - a^4*b^8*y^2*z - a^8*b^2*c^2*y^2*z + a^6*b^4*c^2*y^2*z -      a^6*b^2*c^4*y^2*z + a^4*b^2*c^6*y^2*z + a^6*b^4*c^2*x*z^2 - a^4*b^6*c^2*x*z^2 - a^2*b^8*c^2*x*z^2 +      b^10*c^2*x*z^2 + a^2*b^6*c^4*x*z^2 - 3*b^8*c^4*x*z^2 + 3*b^6*c^6*x*z^2 - b^4*c^8*x*z^2 -      a^10*c^2*y*z^2 + a^8*b^2*c^2*y*z^2 + a^6*b^4*c^2*y*z^2 - a^4*b^6*c^2*y*z^2 + 3*a^8*c^4*y*z^2 -      a^6*b^2*c^4*y*z^2 - 3*a^6*c^6*y*z^2 + a^4*c^8*y*z^2");
		exp("K564","2*a^12*c^2*x^2*y - 3*a^10*b^2*c^2*x^2*y - 6*a^8*b^4*c^2*x^2*y + 14*a^6*b^6*c^2*x^2*y -      6*a^4*b^8*c^2*x^2*y - 3*a^2*b^10*c^2*x^2*y + 2*b^12*c^2*x^2*y - 3*a^10*c^4*x^2*y +      18*a^8*b^2*c^4*x^2*y - 15*a^6*b^4*c^4*x^2*y - 15*a^4*b^6*c^4*x^2*y + 18*a^2*b^8*c^4*x^2*y -      3*b^10*c^4*x^2*y - 6*a^8*c^6*x^2*y - 15*a^6*b^2*c^6*x^2*y + 42*a^4*b^4*c^6*x^2*y -      15*a^2*b^6*c^6*x^2*y - 6*b^8*c^6*x^2*y + 14*a^6*c^8*x^2*y - 15*a^4*b^2*c^8*x^2*y -      15*a^2*b^4*c^8*x^2*y + 14*b^6*c^8*x^2*y - 6*a^4*c^10*x^2*y + 18*a^2*b^2*c^10*x^2*y -      6*b^4*c^10*x^2*y - 3*a^2*c^12*x^2*y - 3*b^2*c^12*x^2*y + 2*c^14*x^2*y - 2*a^12*c^2*x*y^2 +      3*a^10*b^2*c^2*x*y^2 + 6*a^8*b^4*c^2*x*y^2 - 14*a^6*b^6*c^2*x*y^2 + 6*a^4*b^8*c^2*x*y^2 +      3*a^2*b^10*c^2*x*y^2 - 2*b^12*c^2*x*y^2 + 3*a^10*c^4*x*y^2 - 18*a^8*b^2*c^4*x*y^2 +      15*a^6*b^4*c^4*x*y^2 + 15*a^4*b^6*c^4*x*y^2 - 18*a^2*b^8*c^4*x*y^2 + 3*b^10*c^4*x*y^2 +      6*a^8*c^6*x*y^2 + 15*a^6*b^2*c^6*x*y^2 - 42*a^4*b^4*c^6*x*y^2 + 15*a^2*b^6*c^6*x*y^2 +      6*b^8*c^6*x*y^2 - 14*a^6*c^8*x*y^2 + 15*a^4*b^2*c^8*x*y^2 + 15*a^2*b^4*c^8*x*y^2 -      14*b^6*c^8*x*y^2 + 6*a^4*c^10*x*y^2 - 18*a^2*b^2*c^10*x*y^2 + 6*b^4*c^10*x*y^2 + 3*a^2*c^12*x*y^2 +      3*b^2*c^12*x*y^2 - 2*c^14*x*y^2 - 2*a^12*b^2*x^2*z + 3*a^10*b^4*x^2*z + 6*a^8*b^6*x^2*z -      14*a^6*b^8*x^2*z + 6*a^4*b^10*x^2*z + 3*a^2*b^12*x^2*z - 2*b^14*x^2*z + 3*a^10*b^2*c^2*x^2*z -      18*a^8*b^4*c^2*x^2*z + 15*a^6*b^6*c^2*x^2*z + 15*a^4*b^8*c^2*x^2*z - 18*a^2*b^10*c^2*x^2*z +      3*b^12*c^2*x^2*z + 6*a^8*b^2*c^4*x^2*z + 15*a^6*b^4*c^4*x^2*z - 42*a^4*b^6*c^4*x^2*z +      15*a^2*b^8*c^4*x^2*z + 6*b^10*c^4*x^2*z - 14*a^6*b^2*c^6*x^2*z + 15*a^4*b^4*c^6*x^2*z +      15*a^2*b^6*c^6*x^2*z - 14*b^8*c^6*x^2*z + 6*a^4*b^2*c^8*x^2*z - 18*a^2*b^4*c^8*x^2*z +      6*b^6*c^8*x^2*z + 3*a^2*b^2*c^10*x^2*z + 3*b^4*c^10*x^2*z - 2*b^2*c^12*x^2*z + 3*a^12*b^2*x*y*z -      15*a^10*b^4*x*y*z + 30*a^8*b^6*x*y*z - 30*a^6*b^8*x*y*z + 15*a^4*b^10*x*y*z - 3*a^2*b^12*x*y*z -      3*a^12*c^2*x*y*z + 9*a^8*b^4*c^2*x*y*z - 9*a^4*b^8*c^2*x*y*z + 3*b^12*c^2*x*y*z +      15*a^10*c^4*x*y*z - 9*a^8*b^2*c^4*x*y*z + 9*a^2*b^8*c^4*x*y*z - 15*b^10*c^4*x*y*z -      30*a^8*c^6*x*y*z + 30*b^8*c^6*x*y*z + 30*a^6*c^8*x*y*z + 9*a^4*b^2*c^8*x*y*z -      9*a^2*b^4*c^8*x*y*z - 30*b^6*c^8*x*y*z - 15*a^4*c^10*x*y*z + 15*b^4*c^10*x*y*z + 3*a^2*c^12*x*y*z -      3*b^2*c^12*x*y*z + 2*a^14*y^2*z - 3*a^12*b^2*y^2*z - 6*a^10*b^4*y^2*z + 14*a^8*b^6*y^2*z -      6*a^6*b^8*y^2*z - 3*a^4*b^10*y^2*z + 2*a^2*b^12*y^2*z - 3*a^12*c^2*y^2*z + 18*a^10*b^2*c^2*y^2*z -      15*a^8*b^4*c^2*y^2*z - 15*a^6*b^6*c^2*y^2*z + 18*a^4*b^8*c^2*y^2*z - 3*a^2*b^10*c^2*y^2*z -      6*a^10*c^4*y^2*z - 15*a^8*b^2*c^4*y^2*z + 42*a^6*b^4*c^4*y^2*z - 15*a^4*b^6*c^4*y^2*z -      6*a^2*b^8*c^4*y^2*z + 14*a^8*c^6*y^2*z - 15*a^6*b^2*c^6*y^2*z - 15*a^4*b^4*c^6*y^2*z +      14*a^2*b^6*c^6*y^2*z - 6*a^6*c^8*y^2*z + 18*a^4*b^2*c^8*y^2*z - 6*a^2*b^4*c^8*y^2*z -      3*a^4*c^10*y^2*z - 3*a^2*b^2*c^10*y^2*z + 2*a^2*c^12*y^2*z + 2*a^12*b^2*x*z^2 - 3*a^10*b^4*x*z^2 -      6*a^8*b^6*x*z^2 + 14*a^6*b^8*x*z^2 - 6*a^4*b^10*x*z^2 - 3*a^2*b^12*x*z^2 + 2*b^14*x*z^2 -      3*a^10*b^2*c^2*x*z^2 + 18*a^8*b^4*c^2*x*z^2 - 15*a^6*b^6*c^2*x*z^2 - 15*a^4*b^8*c^2*x*z^2 +      18*a^2*b^10*c^2*x*z^2 - 3*b^12*c^2*x*z^2 - 6*a^8*b^2*c^4*x*z^2 - 15*a^6*b^4*c^4*x*z^2 +      42*a^4*b^6*c^4*x*z^2 - 15*a^2*b^8*c^4*x*z^2 - 6*b^10*c^4*x*z^2 + 14*a^6*b^2*c^6*x*z^2 -      15*a^4*b^4*c^6*x*z^2 - 15*a^2*b^6*c^6*x*z^2 + 14*b^8*c^6*x*z^2 - 6*a^4*b^2*c^8*x*z^2 +      18*a^2*b^4*c^8*x*z^2 - 6*b^6*c^8*x*z^2 - 3*a^2*b^2*c^10*x*z^2 - 3*b^4*c^10*x*z^2 +      2*b^2*c^12*x*z^2 - 2*a^14*y*z^2 + 3*a^12*b^2*y*z^2 + 6*a^10*b^4*y*z^2 - 14*a^8*b^6*y*z^2 +      6*a^6*b^8*y*z^2 + 3*a^4*b^10*y*z^2 - 2*a^2*b^12*y*z^2 + 3*a^12*c^2*y*z^2 - 18*a^10*b^2*c^2*y*z^2 +      15*a^8*b^4*c^2*y*z^2 + 15*a^6*b^6*c^2*y*z^2 - 18*a^4*b^8*c^2*y*z^2 + 3*a^2*b^10*c^2*y*z^2 +      6*a^10*c^4*y*z^2 + 15*a^8*b^2*c^4*y*z^2 - 42*a^6*b^4*c^4*y*z^2 + 15*a^4*b^6*c^4*y*z^2 +      6*a^2*b^8*c^4*y*z^2 - 14*a^8*c^6*y*z^2 + 15*a^6*b^2*c^6*y*z^2 + 15*a^4*b^4*c^6*y*z^2 -      14*a^2*b^6*c^6*y*z^2 + 6*a^6*c^8*y*z^2 - 18*a^4*b^2*c^8*y*z^2 + 6*a^2*b^4*c^8*y*z^2 +      3*a^4*c^10*y*z^2 + 3*a^2*b^2*c^10*y*z^2 - 2*a^2*c^12*y*z^2");
		exp("K565","2*a^6*c^2*x^2*y - 3*a^4*b^2*c^2*x^2*y - 3*a^2*b^4*c^2*x^2*y + 2*b^6*c^2*x^2*y - 3*a^4*c^4*x^2*y +      12*a^2*b^2*c^4*x^2*y - 3*b^4*c^4*x^2*y - 3*a^2*c^6*x^2*y - 3*b^2*c^6*x^2*y + 2*c^8*x^2*y -      2*a^6*c^2*x*y^2 + 3*a^4*b^2*c^2*x*y^2 + 3*a^2*b^4*c^2*x*y^2 - 2*b^6*c^2*x*y^2 + 3*a^4*c^4*x*y^2 -      12*a^2*b^2*c^4*x*y^2 + 3*b^4*c^4*x*y^2 + 3*a^2*c^6*x*y^2 + 3*b^2*c^6*x*y^2 - 2*c^8*x*y^2 -      2*a^6*b^2*x^2*z + 3*a^4*b^4*x^2*z + 3*a^2*b^6*x^2*z - 2*b^8*x^2*z + 3*a^4*b^2*c^2*x^2*z -      12*a^2*b^4*c^2*x^2*z + 3*b^6*c^2*x^2*z + 3*a^2*b^2*c^4*x^2*z + 3*b^4*c^4*x^2*z - 2*b^2*c^6*x^2*z +      9*a^6*b^2*x*y*z - 9*a^2*b^6*x*y*z - 9*a^6*c^2*x*y*z + 9*b^6*c^2*x*y*z + 9*a^2*c^6*x*y*z -      9*b^2*c^6*x*y*z + 2*a^8*y^2*z - 3*a^6*b^2*y^2*z - 3*a^4*b^4*y^2*z + 2*a^2*b^6*y^2*z -      3*a^6*c^2*y^2*z + 12*a^4*b^2*c^2*y^2*z - 3*a^2*b^4*c^2*y^2*z - 3*a^4*c^4*y^2*z -      3*a^2*b^2*c^4*y^2*z + 2*a^2*c^6*y^2*z + 2*a^6*b^2*x*z^2 - 3*a^4*b^4*x*z^2 - 3*a^2*b^6*x*z^2 +      2*b^8*x*z^2 - 3*a^4*b^2*c^2*x*z^2 + 12*a^2*b^4*c^2*x*z^2 - 3*b^6*c^2*x*z^2 - 3*a^2*b^2*c^4*x*z^2 -      3*b^4*c^4*x*z^2 + 2*b^2*c^6*x*z^2 - 2*a^8*y*z^2 + 3*a^6*b^2*y*z^2 + 3*a^4*b^4*y*z^2 -      2*a^2*b^6*y*z^2 + 3*a^6*c^2*y*z^2 - 12*a^4*b^2*c^2*y*z^2 + 3*a^2*b^4*c^2*y*z^2 + 3*a^4*c^4*y*z^2 +      3*a^2*b^2*c^4*y*z^2 - 2*a^2*c^6*y*z^2");
		exp("K566","-2*a^4*c^2*x^2*y - a^2*b^2*c^2*x^2*y + 3*b^4*c^2*x^2*y + a^2*c^4*x^2*y - 4*b^2*c^4*x^2*y +     c^6*x^2*y - 3*a^4*c^2*x*y^2 + a^2*b^2*c^2*x*y^2 + 2*b^4*c^2*x*y^2 + 4*a^2*c^4*x*y^2 -     b^2*c^4*x*y^2 - c^6*x*y^2 + 2*a^4*b^2*x^2*z - a^2*b^4*x^2*z - b^6*x^2*z + a^2*b^2*c^2*x^2*z +     4*b^4*c^2*x^2*z - 3*b^2*c^4*x^2*z + 6*a^4*b^2*x*y*z - 6*a^2*b^4*x*y*z - 6*a^4*c^2*x*y*z +     6*b^4*c^2*x*y*z + 6*a^2*c^4*x*y*z - 6*b^2*c^4*x*y*z + a^6*y^2*z + a^4*b^2*y^2*z - 2*a^2*b^4*y^2*z -     4*a^4*c^2*y^2*z - a^2*b^2*c^2*y^2*z + 3*a^2*c^4*y^2*z + 3*a^4*b^2*x*z^2 - 4*a^2*b^4*x*z^2 +     b^6*x*z^2 - a^2*b^2*c^2*x*z^2 + b^4*c^2*x*z^2 - 2*b^2*c^4*x*z^2 - a^6*y*z^2 + 4*a^4*b^2*y*z^2 -     3*a^2*b^4*y*z^2 - a^4*c^2*y*z^2 + a^2*b^2*c^2*y*z^2 + 2*a^2*c^4*y*z^2");
		exp("K567","a^2*c^2*x^2*y - b^2*c^2*x^2*y - a^2*c^2*x*y^2 + b^2*c^2*x*y^2 + a^2*b^2*x^2*z - b^2*c^2*x^2*z +      2*a^4*x*y*z - 2*a^2*b^2*x*y*z + 2*b^4*x*y*z - 2*a^2*c^2*x*y*z - 2*b^2*c^2*x*y*z + 2*c^4*x*y*z +      a^2*b^2*y^2*z - a^2*c^2*y^2*z - a^2*b^2*x*z^2 + b^2*c^2*x*z^2 - a^2*b^2*y*z^2 + a^2*c^2*y*z^2");
		exp("K568","b^2*c^2*x^2*y - c^4*x^2*y + a^2*c^2*x*y^2 - c^4*x*y^2 - b^4*x^2*z + b^2*c^2*x^2*z - 2*a^4*x*y*z +      2*a^2*b^2*x*y*z - 2*b^4*x*y*z + 2*a^2*c^2*x*y*z + 2*b^2*c^2*x*y*z - 2*c^4*x*y*z - a^4*y^2*z +      a^2*c^2*y^2*z + a^2*b^2*x*z^2 - b^4*x*z^2 - a^4*y*z^2 + a^2*b^2*y*z^2");
		exp("K569","a^4*c^2*x^2*y - 2*a^2*b^2*c^2*x^2*y + b^4*c^2*x^2*y - 3*a^2*c^4*x^2*y - 3*b^2*c^4*x^2*y +      2*c^6*x^2*y - a^4*c^2*x*y^2 + 2*a^2*b^2*c^2*x*y^2 - b^4*c^2*x*y^2 + 3*a^2*c^4*x*y^2 +      3*b^2*c^4*x*y^2 - 2*c^6*x*y^2 - a^4*b^2*x^2*z + 3*a^2*b^4*x^2*z - 2*b^6*x^2*z +      2*a^2*b^2*c^2*x^2*z + 3*b^4*c^2*x^2*z - b^2*c^4*x^2*z + 2*a^4*b^2*x*y*z - 2*a^2*b^4*x*y*z -      2*a^4*c^2*x*y*z + 2*b^4*c^2*x*y*z + 2*a^2*c^4*x*y*z - 2*b^2*c^4*x*y*z + 2*a^6*y^2*z -      3*a^4*b^2*y^2*z + a^2*b^4*y^2*z - 3*a^4*c^2*y^2*z - 2*a^2*b^2*c^2*y^2*z + a^2*c^4*y^2*z +      a^4*b^2*x*z^2 - 3*a^2*b^4*x*z^2 + 2*b^6*x*z^2 - 2*a^2*b^2*c^2*x*z^2 - 3*b^4*c^2*x*z^2 +      b^2*c^4*x*z^2 - 2*a^6*y*z^2 + 3*a^4*b^2*y*z^2 - a^2*b^4*y*z^2 + 3*a^4*c^2*y*z^2 +      2*a^2*b^2*c^2*y*z^2 - a^2*c^4*y*z^2");
		exp("K570","(-a^4)*c^4*x^2*y - b^4*c^4*x^2*y + a^2*c^6*x^2*y + b^2*c^6*x^2*y + a^4*c^4*x*y^2 +      b^4*c^4*x*y^2 - a^2*c^6*x*y^2 - b^2*c^6*x*y^2 + a^4*b^4*x^2*z - a^2*b^6*x^2*z - b^6*c^2*x^2*z +      b^4*c^4*x^2*z + a^6*b^2*x*y*z - a^2*b^6*x*y*z - a^6*c^2*x*y*z + b^6*c^2*x*y*z + a^2*c^6*x*y*z -      b^2*c^6*x*y*z + a^6*b^2*y^2*z - a^4*b^4*y^2*z + a^6*c^2*y^2*z - a^4*c^4*y^2*z - a^4*b^4*x*z^2 +      a^2*b^6*x*z^2 + b^6*c^2*x*z^2 - b^4*c^4*x*z^2 - a^6*b^2*y*z^2 + a^4*b^4*y*z^2 - a^6*c^2*y*z^2 +      a^4*c^4*y*z^2");
		exp("K571","(-a^2)*c*x^2*y + b^2*c*x^2*y + a*c^2*x^2*y + b*c^2*x^2*y - a^2*c*x*y^2 + b^2*c*x*y^2 -      a*c^2*x*y^2 - b*c^2*x*y^2 + a^2*b*x^2*z - a*b^2*x^2*z - b^2*c*x^2*z - b*c^2*x^2*z + 2*a^2*b*x*y*z -      2*a*b^2*x*y*z - 2*a^2*c*x*y*z + 2*b^2*c*x*y*z + 2*a*c^2*x*y*z - 2*b*c^2*x*y*z + a^2*b*y^2*z -      a*b^2*y^2*z + a^2*c*y^2*z + a*c^2*y^2*z + a^2*b*x*z^2 + a*b^2*x*z^2 + b^2*c*x*z^2 - b*c^2*x*z^2 -      a^2*b*y*z^2 - a*b^2*y*z^2 - a^2*c*y*z^2 + a*c^2*y*z^2");
		exp("K572","(-a^3)*c*x^2*y + a^2*b*c*x^2*y + a*b^2*c*x^2*y - b^3*c*x^2*y + 2*a^2*c^2*x^2*y -      2*a*b*c^2*x^2*y - a*c^3*x^2*y + b*c^3*x^2*y - a^3*c*x*y^2 + a^2*b*c*x*y^2 + a*b^2*c*x*y^2 -      b^3*c*x*y^2 - 2*a*b*c^2*x*y^2 + 2*b^2*c^2*x*y^2 + a*c^3*x*y^2 - b*c^3*x*y^2 - a^3*b*x^2*z +      2*a^2*b^2*x^2*z - a*b^3*x^2*z + a^2*b*c*x^2*z - 2*a*b^2*c*x^2*z + b^3*c*x^2*z + a*b*c^2*x^2*z -      b*c^3*x^2*z - 2*a^3*b*x*y*z + 4*a^2*b^2*x*y*z - 2*a*b^3*x*y*z - 2*a^3*c*x*y*z + 2*a^2*b*c*x*y*z +      2*a*b^2*c*x*y*z - 2*b^3*c*x*y*z + 4*a^2*c^2*x*y*z + 2*a*b*c^2*x*y*z + 4*b^2*c^2*x*y*z -      2*a*c^3*x*y*z - 2*b*c^3*x*y*z - a^3*b*y^2*z + 2*a^2*b^2*y^2*z - a*b^3*y^2*z + a^3*c*y^2*z -      2*a^2*b*c*y^2*z + a*b^2*c*y^2*z + a*b*c^2*y^2*z - a*c^3*y^2*z - a^3*b*x*z^2 + a*b^3*x*z^2 +      a^2*b*c*x*z^2 - 2*a*b^2*c*x*z^2 - b^3*c*x*z^2 + a*b*c^2*x*z^2 + 2*b^2*c^2*x*z^2 - b*c^3*x*z^2 +      a^3*b*y*z^2 - a*b^3*y*z^2 - a^3*c*y*z^2 - 2*a^2*b*c*y*z^2 + a*b^2*c*y*z^2 + 2*a^2*c^2*y*z^2 +      a*b*c^2*y*z^2 - a*c^3*y*z^2");
		exp("K573","2*a^6*x^2*y - 6*a^4*b^2*x^2*y + 6*a^2*b^4*x^2*y - 2*b^6*x^2*y - 17*a^5*c*x^2*y +      13*a^4*b*c*x^2*y + 34*a^3*b^2*c*x^2*y - 26*a^2*b^3*c*x^2*y - 17*a*b^4*c*x^2*y + 13*b^5*c*x^2*y +      28*a^4*c^2*x^2*y - 10*a^3*b*c^2*x^2*y - 30*a^2*b^2*c^2*x^2*y + 10*a*b^3*c^2*x^2*y +      2*b^4*c^2*x^2*y - 2*a^3*c^3*x^2*y + 12*a^2*b*c^3*x^2*y - 18*a*b^2*c^3*x^2*y - 24*b^3*c^3*x^2*y -      26*a^2*c^4*x^2*y - 26*a*b*c^4*x^2*y + 4*b^2*c^4*x^2*y + 19*a*c^5*x^2*y + 11*b*c^5*x^2*y -      4*c^6*x^2*y + 2*a^6*x*y^2 - 6*a^4*b^2*x*y^2 + 6*a^2*b^4*x*y^2 - 2*b^6*x*y^2 - 13*a^5*c*x*y^2 +      17*a^4*b*c*x*y^2 + 26*a^3*b^2*c*x*y^2 - 34*a^2*b^3*c*x*y^2 - 13*a*b^4*c*x*y^2 + 17*b^5*c*x*y^2 -      2*a^4*c^2*x*y^2 - 10*a^3*b*c^2*x*y^2 + 30*a^2*b^2*c^2*x*y^2 + 10*a*b^3*c^2*x*y^2 -      28*b^4*c^2*x*y^2 + 24*a^3*c^3*x*y^2 + 18*a^2*b*c^3*x*y^2 - 12*a*b^2*c^3*x*y^2 + 2*b^3*c^3*x*y^2 -      4*a^2*c^4*x*y^2 + 26*a*b*c^4*x*y^2 + 26*b^2*c^4*x*y^2 - 11*a*c^5*x*y^2 - 19*b*c^5*x*y^2 +      4*c^6*x*y^2 - 2*a^6*x^2*z + 17*a^5*b*x^2*z - 28*a^4*b^2*x^2*z + 2*a^3*b^3*x^2*z +      26*a^2*b^4*x^2*z - 19*a*b^5*x^2*z + 4*b^6*x^2*z - 13*a^4*b*c*x^2*z + 10*a^3*b^2*c*x^2*z -      12*a^2*b^3*c*x^2*z + 26*a*b^4*c*x^2*z - 11*b^5*c*x^2*z + 6*a^4*c^2*x^2*z - 34*a^3*b*c^2*x^2*z +      30*a^2*b^2*c^2*x^2*z + 18*a*b^3*c^2*x^2*z - 4*b^4*c^2*x^2*z + 26*a^2*b*c^3*x^2*z -      10*a*b^2*c^3*x^2*z + 24*b^3*c^3*x^2*z - 6*a^2*c^4*x^2*z + 17*a*b*c^4*x^2*z - 2*b^2*c^4*x^2*z -      13*b*c^5*x^2*z + 2*c^6*x^2*z + 36*a^5*b*x*y*z - 72*a^4*b^2*x*y*z + 72*a^2*b^4*x*y*z -      36*a*b^5*x*y*z - 36*a^5*c*x*y*z + 108*a^3*b^2*c*x*y*z - 108*a^2*b^3*c*x*y*z + 36*b^5*c*x*y*z +      72*a^4*c^2*x*y*z - 108*a^3*b*c^2*x*y*z + 108*a*b^3*c^2*x*y*z - 72*b^4*c^2*x*y*z +      108*a^2*b*c^3*x*y*z - 108*a*b^2*c^3*x*y*z - 72*a^2*c^4*x*y*z + 72*b^2*c^4*x*y*z + 36*a*c^5*x*y*z -      36*b*c^5*x*y*z - 4*a^6*y^2*z + 19*a^5*b*y^2*z - 26*a^4*b^2*y^2*z - 2*a^3*b^3*y^2*z +      28*a^2*b^4*y^2*z - 17*a*b^5*y^2*z + 2*b^6*y^2*z + 11*a^5*c*y^2*z - 26*a^4*b*c*y^2*z +      12*a^3*b^2*c*y^2*z - 10*a^2*b^3*c*y^2*z + 13*a*b^4*c*y^2*z + 4*a^4*c^2*y^2*z - 18*a^3*b*c^2*y^2*z -      30*a^2*b^2*c^2*y^2*z + 34*a*b^3*c^2*y^2*z - 6*b^4*c^2*y^2*z - 24*a^3*c^3*y^2*z +      10*a^2*b*c^3*y^2*z - 26*a*b^2*c^3*y^2*z + 2*a^2*c^4*y^2*z - 17*a*b*c^4*y^2*z + 6*b^2*c^4*y^2*z +      13*a*c^5*y^2*z - 2*c^6*y^2*z - 2*a^6*x*z^2 + 13*a^5*b*x*z^2 + 2*a^4*b^2*x*z^2 - 24*a^3*b^3*x*z^2 +      4*a^2*b^4*x*z^2 + 11*a*b^5*x*z^2 - 4*b^6*x*z^2 - 17*a^4*b*c*x*z^2 + 10*a^3*b^2*c*x*z^2 -      18*a^2*b^3*c*x*z^2 - 26*a*b^4*c*x*z^2 + 19*b^5*c*x*z^2 + 6*a^4*c^2*x*z^2 - 26*a^3*b*c^2*x*z^2 -      30*a^2*b^2*c^2*x*z^2 + 12*a*b^3*c^2*x*z^2 - 26*b^4*c^2*x*z^2 + 34*a^2*b*c^3*x*z^2 -      10*a*b^2*c^3*x*z^2 - 2*b^3*c^3*x*z^2 - 6*a^2*c^4*x*z^2 + 13*a*b*c^4*x*z^2 + 28*b^2*c^4*x*z^2 -      17*b*c^5*x*z^2 + 2*c^6*x*z^2 + 4*a^6*y*z^2 - 11*a^5*b*y*z^2 - 4*a^4*b^2*y*z^2 + 24*a^3*b^3*y*z^2 -      2*a^2*b^4*y*z^2 - 13*a*b^5*y*z^2 + 2*b^6*y*z^2 - 19*a^5*c*y*z^2 + 26*a^4*b*c*y*z^2 +      18*a^3*b^2*c*y*z^2 - 10*a^2*b^3*c*y*z^2 + 17*a*b^4*c*y*z^2 + 26*a^4*c^2*y*z^2 -      12*a^3*b*c^2*y*z^2 + 30*a^2*b^2*c^2*y*z^2 + 26*a*b^3*c^2*y*z^2 - 6*b^4*c^2*y*z^2 +      2*a^3*c^3*y*z^2 + 10*a^2*b*c^3*y*z^2 - 34*a*b^2*c^3*y*z^2 - 28*a^2*c^4*y*z^2 - 13*a*b*c^4*y*z^2 +      6*b^2*c^4*y*z^2 + 17*a*c^5*y*z^2 - 2*c^6*y*z^2");
		exp("K574","2*a^4*x^2*y - 4*a^3*b*x^2*y + 4*a*b^3*x^2*y - 2*b^4*x^2*y - 3*a^3*c*x^2*y + 5*a^2*b*c*x^2*y -     a*b^2*c*x^2*y - b^3*c*x^2*y - 4*a^2*c^2*x^2*y - 2*a*b*c^2*x^2*y + 6*b^2*c^2*x^2*y + 9*a*c^3*x^2*y +     b*c^3*x^2*y - 4*c^4*x^2*y + 2*a^4*x*y^2 - 4*a^3*b*x*y^2 + 4*a*b^3*x*y^2 - 2*b^4*x*y^2 +     a^3*c*x*y^2 + a^2*b*c*x*y^2 - 5*a*b^2*c*x*y^2 + 3*b^3*c*x*y^2 - 6*a^2*c^2*x*y^2 + 2*a*b*c^2*x*y^2 +     4*b^2*c^2*x*y^2 - a*c^3*x*y^2 - 9*b*c^3*x*y^2 + 4*c^4*x*y^2 - 2*a^4*x^2*z + 3*a^3*b*x^2*z +     4*a^2*b^2*x^2*z - 9*a*b^3*x^2*z + 4*b^4*x^2*z + 4*a^3*c*x^2*z - 5*a^2*b*c*x^2*z + 2*a*b^2*c*x^2*z -     b^3*c*x^2*z + a*b*c^2*x^2*z - 6*b^2*c^2*x^2*z - 4*a*c^3*x^2*z + b*c^3*x^2*z + 2*c^4*x^2*z -     4*a^4*y^2*z + 9*a^3*b*y^2*z - 4*a^2*b^2*y^2*z - 3*a*b^3*y^2*z + 2*b^4*y^2*z + a^3*c*y^2*z -     2*a^2*b*c*y^2*z + 5*a*b^2*c*y^2*z - 4*b^3*c*y^2*z + 6*a^2*c^2*y^2*z - a*b*c^2*y^2*z - a*c^3*y^2*z +     4*b*c^3*y^2*z - 2*c^4*y^2*z - 2*a^4*x*z^2 - a^3*b*x*z^2 + 6*a^2*b^2*x*z^2 + a*b^3*x*z^2 -     4*b^4*x*z^2 + 4*a^3*c*x*z^2 - a^2*b*c*x*z^2 - 2*a*b^2*c*x*z^2 + 9*b^3*c*x*z^2 + 5*a*b*c^2*x*z^2 -     4*b^2*c^2*x*z^2 - 4*a*c^3*x*z^2 - 3*b*c^3*x*z^2 + 2*c^4*x*z^2 + 4*a^4*y*z^2 - a^3*b*y*z^2 -     6*a^2*b^2*y*z^2 + a*b^3*y*z^2 + 2*b^4*y*z^2 - 9*a^3*c*y*z^2 + 2*a^2*b*c*y*z^2 + a*b^2*c*y*z^2 -     4*b^3*c*y*z^2 + 4*a^2*c^2*y*z^2 - 5*a*b*c^2*y*z^2 + 3*a*c^3*y*z^2 + 4*b*c^3*y*z^2 - 2*c^4*y*z^2");
		exp("K575","a^6*x^2*y - 3*a^4*b^2*x^2*y + 3*a^2*b^4*x^2*y - b^6*x^2*y - 4*a^5*c*x^2*y + 2*a^4*b*c*x^2*y +      8*a^3*b^2*c*x^2*y - 4*a^2*b^3*c*x^2*y - 4*a*b^4*c*x^2*y + 2*b^5*c*x^2*y + 5*a^4*c^2*x^2*y -      5*a^3*b*c^2*x^2*y - 6*a^2*b^2*c^2*x^2*y + 5*a*b^3*c^2*x^2*y + b^4*c^2*x^2*y - a^3*c^3*x^2*y +      6*a^2*b*c^3*x^2*y - 3*b^3*c^3*x^2*y - 4*a^2*c^4*x^2*y - 4*a*b*c^4*x^2*y + 2*b^2*c^4*x^2*y +      5*a*c^5*x^2*y + b*c^5*x^2*y - 2*c^6*x^2*y + a^6*x*y^2 - 3*a^4*b^2*x*y^2 + 3*a^2*b^4*x*y^2 -      b^6*x*y^2 - 2*a^5*c*x*y^2 + 4*a^4*b*c*x*y^2 + 4*a^3*b^2*c*x*y^2 - 8*a^2*b^3*c*x*y^2 -      2*a*b^4*c*x*y^2 + 4*b^5*c*x*y^2 - a^4*c^2*x*y^2 - 5*a^3*b*c^2*x*y^2 + 6*a^2*b^2*c^2*x*y^2 +      5*a*b^3*c^2*x*y^2 - 5*b^4*c^2*x*y^2 + 3*a^3*c^3*x*y^2 - 6*a*b^2*c^3*x*y^2 + b^3*c^3*x*y^2 -      2*a^2*c^4*x*y^2 + 4*a*b*c^4*x*y^2 + 4*b^2*c^4*x*y^2 - a*c^5*x*y^2 - 5*b*c^5*x*y^2 + 2*c^6*x*y^2 -      a^6*x^2*z + 4*a^5*b*x^2*z - 5*a^4*b^2*x^2*z + a^3*b^3*x^2*z + 4*a^2*b^4*x^2*z - 5*a*b^5*x^2*z +      2*b^6*x^2*z - 2*a^4*b*c*x^2*z + 5*a^3*b^2*c*x^2*z - 6*a^2*b^3*c*x^2*z + 4*a*b^4*c*x^2*z -      b^5*c*x^2*z + 3*a^4*c^2*x^2*z - 8*a^3*b*c^2*x^2*z + 6*a^2*b^2*c^2*x^2*z - 2*b^4*c^2*x^2*z +      4*a^2*b*c^3*x^2*z - 5*a*b^2*c^3*x^2*z + 3*b^3*c^3*x^2*z - 3*a^2*c^4*x^2*z + 4*a*b*c^4*x^2*z -      b^2*c^4*x^2*z - 2*b*c^5*x^2*z + c^6*x^2*z + 9*a^5*b*x*y*z - 18*a^4*b^2*x*y*z + 18*a^2*b^4*x*y*z -      9*a*b^5*x*y*z - 9*a^5*c*x*y*z + 27*a^3*b^2*c*x*y*z - 27*a^2*b^3*c*x*y*z + 9*b^5*c*x*y*z +      18*a^4*c^2*x*y*z - 27*a^3*b*c^2*x*y*z + 27*a*b^3*c^2*x*y*z - 18*b^4*c^2*x*y*z +      27*a^2*b*c^3*x*y*z - 27*a*b^2*c^3*x*y*z - 18*a^2*c^4*x*y*z + 18*b^2*c^4*x*y*z + 9*a*c^5*x*y*z -      9*b*c^5*x*y*z - 2*a^6*y^2*z + 5*a^5*b*y^2*z - 4*a^4*b^2*y^2*z - a^3*b^3*y^2*z + 5*a^2*b^4*y^2*z -      4*a*b^5*y^2*z + b^6*y^2*z + a^5*c*y^2*z - 4*a^4*b*c*y^2*z + 6*a^3*b^2*c*y^2*z - 5*a^2*b^3*c*y^2*z +      2*a*b^4*c*y^2*z + 2*a^4*c^2*y^2*z - 6*a^2*b^2*c^2*y^2*z + 8*a*b^3*c^2*y^2*z - 3*b^4*c^2*y^2*z -      3*a^3*c^3*y^2*z + 5*a^2*b*c^3*y^2*z - 4*a*b^2*c^3*y^2*z + a^2*c^4*y^2*z - 4*a*b*c^4*y^2*z +      3*b^2*c^4*y^2*z + 2*a*c^5*y^2*z - c^6*y^2*z - a^6*x*z^2 + 2*a^5*b*x*z^2 + a^4*b^2*x*z^2 -      3*a^3*b^3*x*z^2 + 2*a^2*b^4*x*z^2 + a*b^5*x*z^2 - 2*b^6*x*z^2 - 4*a^4*b*c*x*z^2 +      5*a^3*b^2*c*x*z^2 - 4*a*b^4*c*x*z^2 + 5*b^5*c*x*z^2 + 3*a^4*c^2*x*z^2 - 4*a^3*b*c^2*x*z^2 -      6*a^2*b^2*c^2*x*z^2 + 6*a*b^3*c^2*x*z^2 - 4*b^4*c^2*x*z^2 + 8*a^2*b*c^3*x*z^2 - 5*a*b^2*c^3*x*z^2 -      b^3*c^3*x*z^2 - 3*a^2*c^4*x*z^2 + 2*a*b*c^4*x*z^2 + 5*b^2*c^4*x*z^2 - 4*b*c^5*x*z^2 + c^6*x*z^2 +      2*a^6*y*z^2 - a^5*b*y*z^2 - 2*a^4*b^2*y*z^2 + 3*a^3*b^3*y*z^2 - a^2*b^4*y*z^2 - 2*a*b^5*y*z^2 +      b^6*y*z^2 - 5*a^5*c*y*z^2 + 4*a^4*b*c*y*z^2 - 5*a^2*b^3*c*y*z^2 + 4*a*b^4*c*y*z^2 +      4*a^4*c^2*y*z^2 - 6*a^3*b*c^2*y*z^2 + 6*a^2*b^2*c^2*y*z^2 + 4*a*b^3*c^2*y*z^2 - 3*b^4*c^2*y*z^2 +      a^3*c^3*y*z^2 + 5*a^2*b*c^3*y*z^2 - 8*a*b^2*c^3*y*z^2 - 5*a^2*c^4*y*z^2 - 2*a*b*c^4*y*z^2 +      3*b^2*c^4*y*z^2 + 4*a*c^5*y*z^2 - c^6*y*z^2");
		exp("K576","(-a^6)*b^2*c^6*x^2*y - a^4*b^4*c^6*x^2*y + a^2*b^6*c^6*x^2*y + b^8*c^6*x^2*y +      a^4*b^2*c^8*x^2*y - 2*a^2*b^4*c^8*x^2*y - 3*b^6*c^8*x^2*y + a^2*b^2*c^10*x^2*y + 3*b^4*c^10*x^2*y -      b^2*c^12*x^2*y - a^8*c^6*x*y^2 - a^6*b^2*c^6*x*y^2 + a^4*b^4*c^6*x*y^2 + a^2*b^6*c^6*x*y^2 +      3*a^6*c^8*x*y^2 + 2*a^4*b^2*c^8*x*y^2 - a^2*b^4*c^8*x*y^2 - 3*a^4*c^10*x*y^2 - a^2*b^2*c^10*x*y^2 +      a^2*c^12*x*y^2 + a^6*b^6*c^2*x^2*z - a^4*b^8*c^2*x^2*z - a^2*b^10*c^2*x^2*z + b^12*c^2*x^2*z +      a^4*b^6*c^4*x^2*z + 2*a^2*b^8*c^4*x^2*z - 3*b^10*c^4*x^2*z - a^2*b^6*c^6*x^2*z + 3*b^8*c^6*x^2*z -      b^6*c^8*x^2*z - a^12*c^2*y^2*z + a^10*b^2*c^2*y^2*z + a^8*b^4*c^2*y^2*z - a^6*b^6*c^2*y^2*z +      3*a^10*c^4*y^2*z - 2*a^8*b^2*c^4*y^2*z - a^6*b^4*c^4*y^2*z - 3*a^8*c^6*y^2*z + a^6*b^2*c^6*y^2*z +      a^6*c^8*y^2*z + a^8*b^6*x*z^2 - 3*a^6*b^8*x*z^2 + 3*a^4*b^10*x*z^2 - a^2*b^12*x*z^2 +      a^6*b^6*c^2*x*z^2 - 2*a^4*b^8*c^2*x*z^2 + a^2*b^10*c^2*x*z^2 - a^4*b^6*c^4*x*z^2 +      a^2*b^8*c^4*x*z^2 - a^2*b^6*c^6*x*z^2 + a^12*b^2*y*z^2 - 3*a^10*b^4*y*z^2 + 3*a^8*b^6*y*z^2 -      a^6*b^8*y*z^2 - a^10*b^2*c^2*y*z^2 + 2*a^8*b^4*c^2*y*z^2 - a^6*b^6*c^2*y*z^2 - a^8*b^2*c^4*y*z^2 +      a^6*b^4*c^4*y*z^2 + a^6*b^2*c^6*y*z^2");
		exp("K577","(-a)*c^2*x^2*y + b*c^2*x^2*y + c^3*x^2*y - a*c^2*x*y^2 + b*c^2*x*y^2 - c^3*x*y^2 + a*b^2*x^2*z -      b^3*x^2*z - b^2*c*x^2*z + 2*a^2*b*x*y*z - 2*a*b^2*x*y*z - 2*a^2*c*x*y*z + 2*b^2*c*x*y*z +      2*a*c^2*x*y*z - 2*b*c^2*x*y*z + a^3*y^2*z - a^2*b*y^2*z + a^2*c*y^2*z + a*b^2*x*z^2 + b^3*x*z^2 -      b^2*c*x*z^2 - a^3*y*z^2 - a^2*b*y*z^2 + a^2*c*y*z^2");
		exp("K578","(-a^3)*b*c^2*x^2*y + 2*a^2*b^2*c^2*x^2*y - a*b^3*c^2*x^2*y + a^3*c^3*x^2*y - 2*a*b^2*c^3*x^2*y +      b^3*c^3*x^2*y - 2*a^2*c^4*x^2*y + 2*a*b*c^4*x^2*y + a*c^5*x^2*y - b*c^5*x^2*y - a^3*b*c^2*x*y^2 +      2*a^2*b^2*c^2*x*y^2 - a*b^3*c^2*x*y^2 + a^3*c^3*x*y^2 - 2*a^2*b*c^3*x*y^2 + b^3*c^3*x*y^2 +      2*a*b*c^4*x*y^2 - 2*b^2*c^4*x*y^2 - a*c^5*x*y^2 + b*c^5*x*y^2 + a^3*b^3*x^2*z - 2*a^2*b^4*x^2*z +      a*b^5*x^2*z - a^3*b^2*c*x^2*z + 2*a*b^4*c*x^2*z - b^5*c*x^2*z + 2*a^2*b^2*c^2*x^2*z -      2*a*b^3*c^2*x^2*z - a*b^2*c^3*x^2*z + b^3*c^3*x^2*z + a^5*b*x*y*z - 2*a^4*b^2*x*y*z +      2*a^3*b^3*x*y*z - 2*a^2*b^4*x*y*z + a*b^5*x*y*z + a^5*c*x*y*z - 2*a^4*b*c*x*y*z + a^3*b^2*c*x*y*z +      a^2*b^3*c*x*y*z - 2*a*b^4*c*x*y*z + b^5*c*x*y*z - 2*a^4*c^2*x*y*z + a^3*b*c^2*x*y*z +      a*b^3*c^2*x*y*z - 2*b^4*c^2*x*y*z + 2*a^3*c^3*x*y*z + a^2*b*c^3*x*y*z + a*b^2*c^3*x*y*z +      2*b^3*c^3*x*y*z - 2*a^2*c^4*x*y*z - 2*a*b*c^4*x*y*z - 2*b^2*c^4*x*y*z + a*c^5*x*y*z + b*c^5*x*y*z +      a^5*b*y^2*z - 2*a^4*b^2*y^2*z + a^3*b^3*y^2*z - a^5*c*y^2*z + 2*a^4*b*c*y^2*z - a^2*b^3*c*y^2*z -      2*a^3*b*c^2*y^2*z + 2*a^2*b^2*c^2*y^2*z + a^3*c^3*y^2*z - a^2*b*c^3*y^2*z + a^3*b^3*x*z^2 -      a*b^5*x*z^2 - a^3*b^2*c*x*z^2 - 2*a^2*b^3*c*x*z^2 + 2*a*b^4*c*x*z^2 + b^5*c*x*z^2 +      2*a^2*b^2*c^2*x*z^2 - 2*b^4*c^2*x*z^2 - a*b^2*c^3*x*z^2 + b^3*c^3*x*z^2 - a^5*b*y*z^2 +      a^3*b^3*y*z^2 + a^5*c*y*z^2 + 2*a^4*b*c*y*z^2 - 2*a^3*b^2*c*y*z^2 - a^2*b^3*c*y*z^2 -      2*a^4*c^2*y*z^2 + 2*a^2*b^2*c^2*y*z^2 + a^3*c^3*y*z^2 - a^2*b*c^3*y*z^2");
		exp("K579","(-a^4)*b^4*c^4*x^2*y + a^2*b^6*c^4*x^2*y + a^4*b^2*c^6*x^2*y - b^6*c^6*x^2*y -      a^2*b^2*c^8*x^2*y + b^4*c^8*x^2*y + a^6*b^2*c^4*x*y^2 - a^4*b^4*c^4*x*y^2 - a^6*c^6*x*y^2 +      a^2*b^4*c^6*x*y^2 + a^4*c^8*x*y^2 - a^2*b^2*c^8*x*y^2 + a^4*b^6*c^2*x^2*z - a^2*b^8*c^2*x^2*z -      a^4*b^4*c^4*x^2*z + b^8*c^4*x^2*z + a^2*b^4*c^6*x^2*z - b^6*c^6*x^2*z - 2*a^8*b^2*c^2*x*y*z +      3*a^6*b^4*c^2*x*y*z + 3*a^4*b^6*c^2*x*y*z - 2*a^2*b^8*c^2*x*y*z + 3*a^6*b^2*c^4*x*y*z -      12*a^4*b^4*c^4*x*y*z + 3*a^2*b^6*c^4*x*y*z + 3*a^4*b^2*c^6*x*y*z + 3*a^2*b^4*c^6*x*y*z -      2*a^2*b^2*c^8*x*y*z - a^8*b^2*c^2*y^2*z + a^6*b^4*c^2*y^2*z + a^8*c^4*y^2*z - a^4*b^4*c^4*y^2*z -      a^6*c^6*y^2*z + a^4*b^2*c^6*y^2*z - a^6*b^6*x*z^2 + a^4*b^8*x*z^2 + a^6*b^4*c^2*x*z^2 -      a^2*b^8*c^2*x*z^2 - a^4*b^4*c^4*x*z^2 + a^2*b^6*c^4*x*z^2 + a^8*b^4*y*z^2 - a^6*b^6*y*z^2 -      a^8*b^2*c^2*y*z^2 + a^4*b^6*c^2*y*z^2 + a^6*b^2*c^4*y*z^2 - a^4*b^4*c^4*y*z^2");
		exp("K580","a^6*x^2*y - 3*a^4*b^2*x^2*y + 3*a^2*b^4*x^2*y - b^6*x^2*y - a^4*c^2*x^2*y + a^2*b^2*c^2*x^2*y +      b^2*c^4*x^2*y + a^6*x*y^2 - 3*a^4*b^2*x*y^2 + 3*a^2*b^4*x*y^2 - b^6*x*y^2 - a^2*b^2*c^2*x*y^2 +      b^4*c^2*x*y^2 - a^2*c^4*x*y^2 - a^6*x^2*z + a^4*b^2*x^2*z + 3*a^4*c^2*x^2*z - a^2*b^2*c^2*x^2*z -      b^4*c^2*x^2*z - 3*a^2*c^4*x^2*z + c^6*x^2*z - 2*a^4*b^2*x*y*z + 2*a^2*b^4*x*y*z + 2*a^4*c^2*x*y*z -      2*b^4*c^2*x*y*z - 2*a^2*c^4*x*y*z + 2*b^2*c^4*x*y*z - a^2*b^4*y^2*z + b^6*y^2*z + a^4*c^2*y^2*z +      a^2*b^2*c^2*y^2*z - 3*b^4*c^2*y^2*z + 3*b^2*c^4*y^2*z - c^6*y^2*z - a^6*x*z^2 + a^2*b^4*x*z^2 +      3*a^4*c^2*x*z^2 + a^2*b^2*c^2*x*z^2 - 3*a^2*c^4*x*z^2 - b^2*c^4*x*z^2 + c^6*x*z^2 - a^4*b^2*y*z^2 +      b^6*y*z^2 - a^2*b^2*c^2*y*z^2 - 3*b^4*c^2*y*z^2 + a^2*c^4*y*z^2 + 3*b^2*c^4*y*z^2 - c^6*y*z^2");
		exp("K581","(-a^4)*c^2*x^2*y + 3*a^2*b^2*c^2*x^2*y - 2*b^4*c^2*x^2*y + a^2*c^4*x^2*y + 2*b^2*c^4*x^2*y +      2*a^4*c^2*x*y^2 - 3*a^2*b^2*c^2*x*y^2 + b^4*c^2*x*y^2 - 2*a^2*c^4*x*y^2 - b^2*c^4*x*y^2 +      a^4*b^2*x^2*z - a^2*b^4*x^2*z - 3*a^2*b^2*c^2*x^2*z - 2*b^4*c^2*x^2*z + 2*b^2*c^4*x^2*z +      2*a^4*b^2*x*y*z - 2*a^2*b^4*x*y*z - 2*a^4*c^2*x*y*z + 2*b^4*c^2*x*y*z + 2*a^2*c^4*x*y*z -      2*b^2*c^4*x*y*z + a^4*b^2*y^2*z - a^2*b^4*y^2*z + 2*a^4*c^2*y^2*z + 3*a^2*b^2*c^2*y^2*z -      2*a^2*c^4*y^2*z - 2*a^4*b^2*x*z^2 + 2*a^2*b^4*x*z^2 + 3*a^2*b^2*c^2*x*z^2 + b^4*c^2*x*z^2 -      b^2*c^4*x*z^2 - 2*a^4*b^2*y*z^2 + 2*a^2*b^4*y*z^2 - a^4*c^2*y*z^2 - 3*a^2*b^2*c^2*y*z^2 +      a^2*c^4*y*z^2");
		exp("K582","(-a^6)*c^2*x^2*y - 4*a^4*b^2*c^2*x^2*y + 7*a^2*b^4*c^2*x^2*y - 2*b^6*c^2*x^2*y +      6*a^2*b^2*c^4*x^2*y + a^2*c^6*x^2*y + 2*b^2*c^6*x^2*y + 2*a^6*c^2*x*y^2 - 7*a^4*b^2*c^2*x*y^2 +      4*a^2*b^4*c^2*x*y^2 + b^6*c^2*x*y^2 - 6*a^2*b^2*c^4*x*y^2 - 2*a^2*c^6*x*y^2 - b^2*c^6*x*y^2 +      a^6*b^2*x^2*z - a^2*b^6*x^2*z + 4*a^4*b^2*c^2*x^2*z - 6*a^2*b^4*c^2*x^2*z - 2*b^6*c^2*x^2*z -      7*a^2*b^2*c^4*x^2*z + 2*b^2*c^6*x^2*z + 2*a^6*b^2*x*y*z - 2*a^2*b^6*x*y*z - 2*a^6*c^2*x*y*z +      2*b^6*c^2*x*y*z + 2*a^2*c^6*x*y*z - 2*b^2*c^6*x*y*z + a^6*b^2*y^2*z - a^2*b^6*y^2*z +      2*a^6*c^2*y^2*z + 6*a^4*b^2*c^2*y^2*z - 4*a^2*b^4*c^2*y^2*z + 7*a^2*b^2*c^4*y^2*z -      2*a^2*c^6*y^2*z - 2*a^6*b^2*x*z^2 + 2*a^2*b^6*x*z^2 + 7*a^4*b^2*c^2*x*z^2 + 6*a^2*b^4*c^2*x*z^2 +      b^6*c^2*x*z^2 - 4*a^2*b^2*c^4*x*z^2 - b^2*c^6*x*z^2 - 2*a^6*b^2*y*z^2 + 2*a^2*b^6*y*z^2 -      a^6*c^2*y*z^2 - 6*a^4*b^2*c^2*y*z^2 - 7*a^2*b^4*c^2*y*z^2 + 4*a^2*b^2*c^4*y*z^2 + a^2*c^6*y*z^2 ");
		exp("K583","a^14*x^3 - 4*a^12*b^2*x^3 + 6*a^10*b^4*x^3 - 4*a^8*b^6*x^3 + a^6*b^8*x^3 - 4*a^12*c^2*x^3 +      12*a^10*b^2*c^2*x^3 - 12*a^8*b^4*c^2*x^3 + 4*a^6*b^6*c^2*x^3 + 6*a^10*c^4*x^3 -      12*a^8*b^2*c^4*x^3 + 6*a^6*b^4*c^4*x^3 - 4*a^8*c^6*x^3 + 4*a^6*b^2*c^6*x^3 + a^6*c^8*x^3 +      3*a^12*b^2*x^2*y - 12*a^10*b^4*x^2*y + 18*a^8*b^6*x^2*y - 12*a^6*b^8*x^2*y + 3*a^4*b^10*x^2*y -      4*a^10*b^2*c^2*x^2*y + 12*a^8*b^4*c^2*x^2*y - 12*a^6*b^6*c^2*x^2*y + 4*a^4*b^8*c^2*x^2*y -      2*a^8*b^2*c^4*x^2*y + 4*a^6*b^4*c^4*x^2*y - 2*a^4*b^6*c^4*x^2*y + 4*a^6*b^2*c^6*x^2*y -      4*a^4*b^4*c^6*x^2*y - a^4*b^2*c^8*x^2*y + 3*a^10*b^4*x*y^2 - 12*a^8*b^6*x*y^2 + 18*a^6*b^8*x*y^2 -      12*a^4*b^10*x*y^2 + 3*a^2*b^12*x*y^2 + 4*a^8*b^4*c^2*x*y^2 - 12*a^6*b^6*c^2*x*y^2 +      12*a^4*b^8*c^2*x*y^2 - 4*a^2*b^10*c^2*x*y^2 - 2*a^6*b^4*c^4*x*y^2 + 4*a^4*b^6*c^4*x*y^2 -      2*a^2*b^8*c^4*x*y^2 - 4*a^4*b^4*c^6*x*y^2 + 4*a^2*b^6*c^6*x*y^2 - a^2*b^4*c^8*x*y^2 + a^8*b^6*y^3 -      4*a^6*b^8*y^3 + 6*a^4*b^10*y^3 - 4*a^2*b^12*y^3 + b^14*y^3 + 4*a^6*b^6*c^2*y^3 -      12*a^4*b^8*c^2*y^3 + 12*a^2*b^10*c^2*y^3 - 4*b^12*c^2*y^3 + 6*a^4*b^6*c^4*y^3 -      12*a^2*b^8*c^4*y^3 + 6*b^10*c^4*y^3 + 4*a^2*b^6*c^6*y^3 - 4*b^8*c^6*y^3 + b^6*c^8*y^3 +      3*a^12*c^2*x^2*z - 4*a^10*b^2*c^2*x^2*z - 2*a^8*b^4*c^2*x^2*z + 4*a^6*b^6*c^2*x^2*z -      a^4*b^8*c^2*x^2*z - 12*a^10*c^4*x^2*z + 12*a^8*b^2*c^4*x^2*z + 4*a^6*b^4*c^4*x^2*z -      4*a^4*b^6*c^4*x^2*z + 18*a^8*c^6*x^2*z - 12*a^6*b^2*c^6*x^2*z - 2*a^4*b^4*c^6*x^2*z -      12*a^6*c^8*x^2*z + 4*a^4*b^2*c^8*x^2*z + 3*a^4*c^10*x^2*z + 3*a^10*b^2*c^2*x*y*z -      4*a^8*b^4*c^2*x*y*z + 2*a^6*b^6*c^2*x*y*z - 4*a^4*b^8*c^2*x*y*z + 3*a^2*b^10*c^2*x*y*z -      4*a^8*b^2*c^4*x*y*z + 4*a^6*b^4*c^4*x*y*z + 4*a^4*b^6*c^4*x*y*z - 4*a^2*b^8*c^4*x*y*z +      2*a^6*b^2*c^6*x*y*z + 4*a^4*b^4*c^6*x*y*z + 2*a^2*b^6*c^6*x*y*z - 4*a^4*b^2*c^8*x*y*z -      4*a^2*b^4*c^8*x*y*z + 3*a^2*b^2*c^10*x*y*z - a^8*b^4*c^2*y^2*z + 4*a^6*b^6*c^2*y^2*z -      2*a^4*b^8*c^2*y^2*z - 4*a^2*b^10*c^2*y^2*z + 3*b^12*c^2*y^2*z - 4*a^6*b^4*c^4*y^2*z +      4*a^4*b^6*c^4*y^2*z + 12*a^2*b^8*c^4*y^2*z - 12*b^10*c^4*y^2*z - 2*a^4*b^4*c^6*y^2*z -      12*a^2*b^6*c^6*y^2*z + 18*b^8*c^6*y^2*z + 4*a^2*b^4*c^8*y^2*z - 12*b^6*c^8*y^2*z +      3*b^4*c^10*y^2*z + 3*a^10*c^4*x*z^2 + 4*a^8*b^2*c^4*x*z^2 - 2*a^6*b^4*c^4*x*z^2 -      4*a^4*b^6*c^4*x*z^2 - a^2*b^8*c^4*x*z^2 - 12*a^8*c^6*x*z^2 - 12*a^6*b^2*c^6*x*z^2 +      4*a^4*b^4*c^6*x*z^2 + 4*a^2*b^6*c^6*x*z^2 + 18*a^6*c^8*x*z^2 + 12*a^4*b^2*c^8*x*z^2 -      2*a^2*b^4*c^8*x*z^2 - 12*a^4*c^10*x*z^2 - 4*a^2*b^2*c^10*x*z^2 + 3*a^2*c^12*x*z^2 -      a^8*b^2*c^4*y*z^2 - 4*a^6*b^4*c^4*y*z^2 - 2*a^4*b^6*c^4*y*z^2 + 4*a^2*b^8*c^4*y*z^2 +      3*b^10*c^4*y*z^2 + 4*a^6*b^2*c^6*y*z^2 + 4*a^4*b^4*c^6*y*z^2 - 12*a^2*b^6*c^6*y*z^2 -      12*b^8*c^6*y*z^2 - 2*a^4*b^2*c^8*y*z^2 + 12*a^2*b^4*c^8*y*z^2 + 18*b^6*c^8*y*z^2 -      4*a^2*b^2*c^10*y*z^2 - 12*b^4*c^10*y*z^2 + 3*b^2*c^12*y*z^2 + a^8*c^6*z^3 + 4*a^6*b^2*c^6*z^3 +      6*a^4*b^4*c^6*z^3 + 4*a^2*b^6*c^6*z^3 + b^8*c^6*z^3 - 4*a^6*c^8*z^3 - 12*a^4*b^2*c^8*z^3 -      12*a^2*b^4*c^8*z^3 - 4*b^6*c^8*z^3 + 6*a^4*c^10*z^3 + 12*a^2*b^2*c^10*z^3 + 6*b^4*c^10*z^3 -      4*a^2*c^12*z^3 - 4*b^2*c^12*z^3 + c^14*z^3");
		exp("K584","(-a)*b*c*x*y*z + a*c^2*x*y^2*Cos(A/3) + a*b^2*x*z^2*Cos(A/3) + b*c^2*x^2*y*Cos(B/3) + a^2*b*y*z^2*Cos(B/3) - 2*b^2*c*x^2*z*Cos(A/3)*Cos(B/3) - 2*a^2*c*y^2*z*Cos(A/3)*Cos(B/3) +      b^2*c*x^2*z*Cos(C/3) + a^2*c*y^2*z*Cos(C/3) - 2*b*c^2*x^2*y*Cos(A/3)*Cos(C/3) - 2*a^2*b*y*z^2*Cos(A/3)*Cos(C/3) - 2*a*c^2*x*y^2*Cos(B/3)*Cos(C/3) -      2*a*b^2*x*z^2*Cos(B/3)*Cos(C/3) + 8*a*b*c*x*y*z*Cos(A/3)*Cos(B/3)*Cos(C/3)");
		exp("K585","a*c^2*x*y^2*Cos(A/3) - a*b^2*x*z^2*Cos(A/3) - b*c^2*x^2*y*Cos(B/3) + a^2*b*y*z^2*Cos(B/3) - 2*b^2*c*x^2*z*Cos(A/3)*Cos(B/3) + 2*a^2*c*y^2*z*Cos(A/3)*Cos(B/3) +      b^2*c*x^2*z*Cos(C/3) - a^2*c*y^2*z*Cos(C/3) + 2*b*c^2*x^2*y*Cos(A/3)*Cos(C/3) - 2*a^2*b*y*z^2*Cos(A/3)*Cos(C/3) - 2*a*c^2*x*y^2*Cos(B/3)*Cos(C/3) +      2*a*b^2*x*z^2*Cos(B/3)*Cos(C/3)");
		exp("K586","(-a)*c^2*x*y^2*Cos(A/3) + a*b^2*x*z^2*Cos(A/3) + b*c^2*x^2*y*Cos(B/3) - a^2*b*y*z^2*Cos(B/3) - 4*b*c^2*x^2*y*Cos(A/3)^2*Cos(B/3) + 4*a*c^2*x*y^2*Cos(A/3)*Cos(B/3)^2 -      b^2*c*x^2*z*Cos(C/3) + a^2*c*y^2*z*Cos(C/3) + 4*b^2*c*x^2*z*Cos(A/3)^2*Cos(C/3) - 4*a^2*c*y^2*z*Cos(B/3)^2*Cos(C/3) - 4*a*b^2*x*z^2*Cos(A/3)*Cos(C/3)^2 +      4*a^2*b*y*z^2*Cos(B/3)*Cos(C/3)^2");
		exp("K587","(-a)*b^4*c^4*x^2*y*Cos(A/3) + a*b^2*c^6*x^2*y*Cos(A/3) + a^5*c^4*x*y^2*Cos(A/3) - a^3*c^6*x*y^2*Cos(A/3) + a*b^6*c^2*x^2*z*Cos(A/3) - a*b^4*c^4*x^2*z*Cos(A/3) -      2*a^5*b^2*c^2*x*y*z*Cos(A/3) + a^3*b^4*c^2*x*y*z*Cos(A/3) + a^3*b^2*c^4*x*y*z*Cos(A/3) + a^5*b^4*x*z^2*Cos(A/3) - a^3*b^6*x*z^2*Cos(A/3) - 2*a*b^5*c^3*x^2*y*Cos(A/3)^2 +      2*a*b^3*c^5*x^2*y*Cos(A/3)^2 - 2*a^3*b*c^5*x*y^2*Cos(A/3)^2 + 2*a*b^5*c^3*x^2*z*Cos(A/3)^2 - 2*a*b^3*c^5*x^2*z*Cos(A/3)^2 + 4*a^3*b^3*c^3*x*y*z*Cos(A/3)^2 -      2*a^3*b^5*c*x*z^2*Cos(A/3)^2 + b^5*c^4*x^2*y*Cos(B/3) - b^3*c^6*x^2*y*Cos(B/3) - a^4*b*c^4*x*y^2*Cos(B/3) + a^2*b*c^6*x*y^2*Cos(B/3) + a^4*b^3*c^2*x*y*z*Cos(B/3) -      2*a^2*b^5*c^2*x*y*z*Cos(B/3) + a^2*b^3*c^4*x*y*z*Cos(B/3) + a^6*b*c^2*y^2*z*Cos(B/3) - a^4*b*c^4*y^2*z*Cos(B/3) - a^6*b^3*y*z^2*Cos(B/3) + a^4*b^5*y*z^2*Cos(B/3) -      2*a^2*b^4*c^3*x^2*y*Cos(A/3)*Cos(B/3) + 4*b^6*c^3*x^2*y*Cos(A/3)*Cos(B/3) + 4*a^2*b^2*c^5*x^2*y*Cos(A/3)*Cos(B/3) - 4*b^4*c^5*x^2*y*Cos(A/3)*Cos(B/3) +      4*a^6*c^3*x*y^2*Cos(A/3)*Cos(B/3) - 2*a^4*b^2*c^3*x*y^2*Cos(A/3)*Cos(B/3) - 4*a^4*c^5*x*y^2*Cos(A/3)*Cos(B/3) + 4*a^2*b^2*c^5*x*y^2*Cos(A/3)*Cos(B/3) -      2*a^2*b^4*c^3*x^2*z*Cos(A/3)*Cos(B/3) - 4*a^6*b^2*c*x*y*z*Cos(A/3)*Cos(B/3) + 8*a^4*b^4*c*x*y*z*Cos(A/3)*Cos(B/3) - 4*a^2*b^6*c*x*y*z*Cos(A/3)*Cos(B/3) -      4*a^4*b^2*c^3*x*y*z*Cos(A/3)*Cos(B/3) - 4*a^2*b^4*c^3*x*y*z*Cos(A/3)*Cos(B/3) + 4*a^2*b^2*c^5*x*y*z*Cos(A/3)*Cos(B/3) - 2*a^4*b^2*c^3*y^2*z*Cos(A/3)*Cos(B/3) +      2*a^4*b^4*c*x*z^2*Cos(A/3)*Cos(B/3) + 2*a^4*b^4*c*y*z^2*Cos(A/3)*Cos(B/3) - 4*a^2*b^5*c^2*x^2*y*Cos(A/3)^2*Cos(B/3) + 4*b^7*c^2*x^2*y*Cos(A/3)^2*Cos(B/3) +      8*a^2*b^3*c^4*x^2*y*Cos(A/3)^2*Cos(B/3) - 4*b^5*c^4*x^2*y*Cos(A/3)^2*Cos(B/3) - 8*a^4*b*c^4*x*y^2*Cos(A/3)^2*Cos(B/3) + 4*a^2*b^3*c^4*x*y^2*Cos(A/3)^2*Cos(B/3) +      4*a^2*b^5*c^2*x^2*z*Cos(A/3)^2*Cos(B/3) - 8*a^2*b^3*c^4*x^2*z*Cos(A/3)^2*Cos(B/3) + 8*a^4*b^3*c^2*x*y*z*Cos(A/3)^2*Cos(B/3) - 12*a^2*b^5*c^2*x*y*z*Cos(A/3)^2*Cos(B/3) +      8*a^2*b^3*c^4*x*y*z*Cos(A/3)^2*Cos(B/3) - 2*a*b^3*c^5*x^2*y*Cos(B/3)^2 - 2*a^5*b*c^3*x*y^2*Cos(B/3)^2 + 2*a^3*b*c^5*x*y^2*Cos(B/3)^2 + 4*a^3*b^3*c^3*x*y*z*Cos(B/3)^2 +      2*a^5*b*c^3*y^2*z*Cos(B/3)^2 - 2*a^3*b*c^5*y^2*z*Cos(B/3)^2 - 2*a^5*b^3*c*y*z^2*Cos(B/3)^2 + 4*a^3*b^2*c^4*x^2*y*Cos(A/3)*Cos(B/3)^2 - 8*a*b^4*c^4*x^2*y*Cos(A/3)*Cos(B/3)^2 +      4*a^7*c^2*x*y^2*Cos(A/3)*Cos(B/3)^2 - 4*a^5*b^2*c^2*x*y^2*Cos(A/3)*Cos(B/3)^2 - 4*a^5*c^4*x*y^2*Cos(A/3)*Cos(B/3)^2 + 8*a^3*b^2*c^4*x*y^2*Cos(A/3)*Cos(B/3)^2 -      12*a^5*b^2*c^2*x*y*z*Cos(A/3)*Cos(B/3)^2 + 8*a^3*b^4*c^2*x*y*z*Cos(A/3)*Cos(B/3)^2 + 8*a^3*b^2*c^4*x*y*z*Cos(A/3)*Cos(B/3)^2 + 4*a^5*b^2*c^2*y^2*z*Cos(A/3)*Cos(B/3)^2 -      8*a^3*b^2*c^4*y^2*z*Cos(A/3)*Cos(B/3)^2 + 8*a^3*b^3*c^3*x^2*y*Cos(A/3)^2*Cos(B/3)^2 - 8*a*b^5*c^3*x^2*y*Cos(A/3)^2*Cos(B/3)^2 - 8*a^5*b*c^3*x*y^2*Cos(A/3)^2*Cos(B/3)^2 +      8*a^3*b^3*c^3*x*y^2*Cos(A/3)^2*Cos(B/3)^2 - 8*a^3*b^3*c^3*x^2*z*Cos(A/3)^2*Cos(B/3)^2 + 16*a^3*b^3*c^3*x*y*z*Cos(A/3)^2*Cos(B/3)^2 - 8*a^3*b^3*c^3*y^2*z*Cos(A/3)^2*Cos(B/3)^2 -      b^6*c^3*x^2*z*Cos(C/3) + b^4*c^5*x^2*z*Cos(C/3) + a^4*b^2*c^3*x*y*z*Cos(C/3) + a^2*b^4*c^3*x*y*z*Cos(C/3) - 2*a^2*b^2*c^5*x*y*z*Cos(C/3) - a^6*c^3*y^2*z*Cos(C/3) +      a^4*c^5*y^2*z*Cos(C/3) - a^4*b^4*c*x*z^2*Cos(C/3) + a^2*b^6*c*x*z^2*Cos(C/3) + a^6*b^2*c*y*z^2*Cos(C/3) - a^4*b^4*c*y*z^2*Cos(C/3) - 2*a^2*b^3*c^4*x^2*y*Cos(A/3)*Cos(C/3) +      2*a^4*b*c^4*x*y^2*Cos(A/3)*Cos(C/3) + 4*a^2*b^5*c^2*x^2*z*Cos(A/3)*Cos(C/3) - 2*a^2*b^3*c^4*x^2*z*Cos(A/3)*Cos(C/3) - 4*b^5*c^4*x^2*z*Cos(A/3)*Cos(C/3) +      4*b^3*c^6*x^2*z*Cos(A/3)*Cos(C/3) - 4*a^6*b*c^2*x*y*z*Cos(A/3)*Cos(C/3) - 4*a^4*b^3*c^2*x*y*z*Cos(A/3)*Cos(C/3) + 4*a^2*b^5*c^2*x*y*z*Cos(A/3)*Cos(C/3) +      8*a^4*b*c^4*x*y*z*Cos(A/3)*Cos(C/3) - 4*a^2*b^3*c^4*x*y*z*Cos(A/3)*Cos(C/3) - 4*a^2*b*c^6*x*y*z*Cos(A/3)*Cos(C/3) + 2*a^4*b*c^4*y^2*z*Cos(A/3)*Cos(C/3) +      4*a^6*b^3*x*z^2*Cos(A/3)*Cos(C/3) - 4*a^4*b^5*x*z^2*Cos(A/3)*Cos(C/3) - 2*a^4*b^3*c^2*x*z^2*Cos(A/3)*Cos(C/3) + 4*a^2*b^5*c^2*x*z^2*Cos(A/3)*Cos(C/3) -      2*a^4*b^3*c^2*y*z^2*Cos(A/3)*Cos(C/3) - 8*a^2*b^4*c^3*x^2*y*Cos(A/3)^2*Cos(C/3) + 4*a^2*b^2*c^5*x^2*y*Cos(A/3)^2*Cos(C/3) + 8*a^2*b^4*c^3*x^2*z*Cos(A/3)^2*Cos(C/3) -      4*a^2*b^2*c^5*x^2*z*Cos(A/3)^2*Cos(C/3) - 4*b^4*c^5*x^2*z*Cos(A/3)^2*Cos(C/3) + 4*b^2*c^7*x^2*z*Cos(A/3)^2*Cos(C/3) + 8*a^4*b^2*c^3*x*y*z*Cos(A/3)^2*Cos(C/3) +      8*a^2*b^4*c^3*x*y*z*Cos(A/3)^2*Cos(C/3) - 12*a^2*b^2*c^5*x*y*z*Cos(A/3)^2*Cos(C/3) - 8*a^4*b^4*c*x*z^2*Cos(A/3)^2*Cos(C/3) + 4*a^2*b^4*c^3*x*z^2*Cos(A/3)^2*Cos(C/3) +      2*a*b^4*c^4*x^2*y*Cos(B/3)*Cos(C/3) - 2*a^3*b^2*c^4*x*y^2*Cos(B/3)*Cos(C/3) + 2*a*b^4*c^4*x^2*z*Cos(B/3)*Cos(C/3) + 4*a^5*b^2*c^2*x*y*z*Cos(B/3)*Cos(C/3) -      4*a^3*b^4*c^2*x*y*z*Cos(B/3)*Cos(C/3) - 4*a*b^6*c^2*x*y*z*Cos(B/3)*Cos(C/3) - 4*a^3*b^2*c^4*x*y*z*Cos(B/3)*Cos(C/3) + 8*a*b^4*c^4*x*y*z*Cos(B/3)*Cos(C/3) -      4*a*b^2*c^6*x*y*z*Cos(B/3)*Cos(C/3) + 4*a^5*b^2*c^2*y^2*z*Cos(B/3)*Cos(C/3) - 4*a^5*c^4*y^2*z*Cos(B/3)*Cos(C/3) - 2*a^3*b^2*c^4*y^2*z*Cos(B/3)*Cos(C/3) +      4*a^3*c^6*y^2*z*Cos(B/3)*Cos(C/3) - 2*a^3*b^4*c^2*x*z^2*Cos(B/3)*Cos(C/3) - 4*a^5*b^4*y*z^2*Cos(B/3)*Cos(C/3) + 4*a^3*b^6*y*z^2*Cos(B/3)*Cos(C/3) +      4*a^5*b^2*c^2*y*z^2*Cos(B/3)*Cos(C/3) - 2*a^3*b^4*c^2*y*z^2*Cos(B/3)*Cos(C/3) - 4*a^3*b^3*c^3*x^2*y*Cos(A/3)*Cos(B/3)*Cos(C/3) + 12*a*b^5*c^3*x^2*y*Cos(A/3)*Cos(B/3)*Cos(C/3) -      4*a*b^3*c^5*x^2*y*Cos(A/3)*Cos(B/3)*Cos(C/3) + 12*a^5*b*c^3*x*y^2*Cos(A/3)*Cos(B/3)*Cos(C/3) - 4*a^3*b^3*c^3*x*y^2*Cos(A/3)*Cos(B/3)*Cos(C/3) -      4*a^3*b*c^5*x*y^2*Cos(A/3)*Cos(B/3)*Cos(C/3) - 4*a^3*b^3*c^3*x^2*z*Cos(A/3)*Cos(B/3)*Cos(C/3) - 4*a*b^5*c^3*x^2*z*Cos(A/3)*Cos(B/3)*Cos(C/3) +      12*a*b^3*c^5*x^2*z*Cos(A/3)*Cos(B/3)*Cos(C/3) - 8*a^7*b*c*x*y*z*Cos(A/3)*Cos(B/3)*Cos(C/3) + 8*a^5*b^3*c*x*y*z*Cos(A/3)*Cos(B/3)*Cos(C/3) +      8*a^3*b^5*c*x*y*z*Cos(A/3)*Cos(B/3)*Cos(C/3) - 8*a*b^7*c*x*y*z*Cos(A/3)*Cos(B/3)*Cos(C/3) + 8*a^5*b*c^3*x*y*z*Cos(A/3)*Cos(B/3)*Cos(C/3) -      48*a^3*b^3*c^3*x*y*z*Cos(A/3)*Cos(B/3)*Cos(C/3) + 8*a*b^5*c^3*x*y*z*Cos(A/3)*Cos(B/3)*Cos(C/3) + 8*a^3*b*c^5*x*y*z*Cos(A/3)*Cos(B/3)*Cos(C/3) +      8*a*b^3*c^5*x*y*z*Cos(A/3)*Cos(B/3)*Cos(C/3) - 8*a*b*c^7*x*y*z*Cos(A/3)*Cos(B/3)*Cos(C/3) - 4*a^5*b*c^3*y^2*z*Cos(A/3)*Cos(B/3)*Cos(C/3) -      4*a^3*b^3*c^3*y^2*z*Cos(A/3)*Cos(B/3)*Cos(C/3) + 12*a^3*b*c^5*y^2*z*Cos(A/3)*Cos(B/3)*Cos(C/3) + 12*a^5*b^3*c*x*z^2*Cos(A/3)*Cos(B/3)*Cos(C/3) -      4*a^3*b^5*c*x*z^2*Cos(A/3)*Cos(B/3)*Cos(C/3) - 4*a^3*b^3*c^3*x*z^2*Cos(A/3)*Cos(B/3)*Cos(C/3) - 4*a^5*b^3*c*y*z^2*Cos(A/3)*Cos(B/3)*Cos(C/3) +      12*a^3*b^5*c*y*z^2*Cos(A/3)*Cos(B/3)*Cos(C/3) - 4*a^3*b^3*c^3*y*z^2*Cos(A/3)*Cos(B/3)*Cos(C/3) - 16*a^3*b^4*c^2*x^2*y*Cos(A/3)^2*Cos(B/3)*Cos(C/3) +      16*a*b^6*c^2*x^2*y*Cos(A/3)^2*Cos(B/3)*Cos(C/3) + 16*a^3*b^2*c^4*x^2*y*Cos(A/3)^2*Cos(B/3)*Cos(C/3) - 8*a*b^4*c^4*x^2*y*Cos(A/3)^2*Cos(B/3)*Cos(C/3) -      8*a^3*b^2*c^4*x*y^2*Cos(A/3)^2*Cos(B/3)*Cos(C/3) + 16*a^3*b^4*c^2*x^2*z*Cos(A/3)^2*Cos(B/3)*Cos(C/3) - 16*a^3*b^2*c^4*x^2*z*Cos(A/3)^2*Cos(B/3)*Cos(C/3) -      8*a*b^4*c^4*x^2*z*Cos(A/3)^2*Cos(B/3)*Cos(C/3) + 16*a*b^2*c^6*x^2*z*Cos(A/3)^2*Cos(B/3)*Cos(C/3) + 16*a^5*b^2*c^2*x*y*z*Cos(A/3)^2*Cos(B/3)*Cos(C/3) -      16*a^3*b^4*c^2*x*y*z*Cos(A/3)^2*Cos(B/3)*Cos(C/3) - 16*a*b^6*c^2*x*y*z*Cos(A/3)^2*Cos(B/3)*Cos(C/3) - 16*a^3*b^2*c^4*x*y*z*Cos(A/3)^2*Cos(B/3)*Cos(C/3) +      32*a*b^4*c^4*x*y*z*Cos(A/3)^2*Cos(B/3)*Cos(C/3) - 16*a*b^2*c^6*x*y*z*Cos(A/3)^2*Cos(B/3)*Cos(C/3) + 8*a^3*b^2*c^4*y^2*z*Cos(A/3)^2*Cos(B/3)*Cos(C/3) -      8*a^3*b^4*c^2*x*z^2*Cos(A/3)^2*Cos(B/3)*Cos(C/3) + 8*a^3*b^4*c^2*y*z^2*Cos(A/3)^2*Cos(B/3)*Cos(C/3) - 8*a^4*b^2*c^3*x*y^2*Cos(B/3)^2*Cos(C/3) +      4*a^2*b^2*c^5*x*y^2*Cos(B/3)^2*Cos(C/3) + 8*a^4*b^2*c^3*x*y*z*Cos(B/3)^2*Cos(C/3) + 8*a^2*b^4*c^3*x*y*z*Cos(B/3)^2*Cos(C/3) - 12*a^2*b^2*c^5*x*y*z*Cos(B/3)^2*Cos(C/3) +      8*a^4*b^2*c^3*y^2*z*Cos(B/3)^2*Cos(C/3) - 4*a^4*c^5*y^2*z*Cos(B/3)^2*Cos(C/3) - 4*a^2*b^2*c^5*y^2*z*Cos(B/3)^2*Cos(C/3) + 4*a^2*c^7*y^2*z*Cos(B/3)^2*Cos(C/3) -      8*a^4*b^4*c*y*z^2*Cos(B/3)^2*Cos(C/3) + 4*a^4*b^2*c^3*y*z^2*Cos(B/3)^2*Cos(C/3) - 8*a^2*b^3*c^4*x^2*y*Cos(A/3)*Cos(B/3)^2*Cos(C/3) +      16*a^6*b*c^2*x*y^2*Cos(A/3)*Cos(B/3)^2*Cos(C/3) - 16*a^4*b^3*c^2*x*y^2*Cos(A/3)*Cos(B/3)^2*Cos(C/3) - 8*a^4*b*c^4*x*y^2*Cos(A/3)*Cos(B/3)^2*Cos(C/3) +      16*a^2*b^3*c^4*x*y^2*Cos(A/3)*Cos(B/3)^2*Cos(C/3) + 8*a^2*b^3*c^4*x^2*z*Cos(A/3)*Cos(B/3)^2*Cos(C/3) - 16*a^6*b*c^2*x*y*z*Cos(A/3)*Cos(B/3)^2*Cos(C/3) -      16*a^4*b^3*c^2*x*y*z*Cos(A/3)*Cos(B/3)^2*Cos(C/3) + 16*a^2*b^5*c^2*x*y*z*Cos(A/3)*Cos(B/3)^2*Cos(C/3) + 32*a^4*b*c^4*x*y*z*Cos(A/3)*Cos(B/3)^2*Cos(C/3) -      16*a^2*b^3*c^4*x*y*z*Cos(A/3)*Cos(B/3)^2*Cos(C/3) - 16*a^2*b*c^6*x*y*z*Cos(A/3)*Cos(B/3)^2*Cos(C/3) + 16*a^4*b^3*c^2*y^2*z*Cos(A/3)*Cos(B/3)^2*Cos(C/3) -      8*a^4*b*c^4*y^2*z*Cos(A/3)*Cos(B/3)^2*Cos(C/3) - 16*a^2*b^3*c^4*y^2*z*Cos(A/3)*Cos(B/3)^2*Cos(C/3) + 16*a^2*b*c^6*y^2*z*Cos(A/3)*Cos(B/3)^2*Cos(C/3) +      8*a^4*b^3*c^2*x*z^2*Cos(A/3)*Cos(B/3)^2*Cos(C/3) - 8*a^4*b^3*c^2*y*z^2*Cos(A/3)*Cos(B/3)^2*Cos(C/3) + 16*a^4*b^2*c^3*x^2*y*Cos(A/3)^2*Cos(B/3)^2*Cos(C/3) -      16*a^2*b^4*c^3*x^2*y*Cos(A/3)^2*Cos(B/3)^2*Cos(C/3) - 16*a^4*b^2*c^3*x*y^2*Cos(A/3)^2*Cos(B/3)^2*Cos(C/3) + 16*a^2*b^4*c^3*x*y^2*Cos(A/3)^2*Cos(B/3)^2*Cos(C/3) -      16*a^4*b^2*c^3*x^2*z*Cos(A/3)^2*Cos(B/3)^2*Cos(C/3) + 16*a^2*b^2*c^5*x^2*z*Cos(A/3)^2*Cos(B/3)^2*Cos(C/3) + 16*a^4*b^2*c^3*x*y*z*Cos(A/3)^2*Cos(B/3)^2*Cos(C/3) +      16*a^2*b^4*c^3*x*y*z*Cos(A/3)^2*Cos(B/3)^2*Cos(C/3) - 32*a^2*b^2*c^5*x*y*z*Cos(A/3)^2*Cos(B/3)^2*Cos(C/3) - 16*a^2*b^4*c^3*y^2*z*Cos(A/3)^2*Cos(B/3)^2*Cos(C/3) +      16*a^2*b^2*c^5*y^2*z*Cos(A/3)^2*Cos(B/3)^2*Cos(C/3) - 2*a*b^5*c^3*x^2*z*Cos(C/3)^2 + 4*a^3*b^3*c^3*x*y*z*Cos(C/3)^2 - 2*a^5*b*c^3*y^2*z*Cos(C/3)^2 - 2*a^5*b^3*c*x*z^2*Cos(C/3)^2 +      2*a^3*b^5*c*x*z^2*Cos(C/3)^2 + 2*a^5*b^3*c*y*z^2*Cos(C/3)^2 - 2*a^3*b^5*c*y*z^2*Cos(C/3)^2 + 4*a^3*b^4*c^2*x^2*z*Cos(A/3)*Cos(C/3)^2 - 8*a*b^4*c^4*x^2*z*Cos(A/3)*Cos(C/3)^2 -      12*a^5*b^2*c^2*x*y*z*Cos(A/3)*Cos(C/3)^2 + 8*a^3*b^4*c^2*x*y*z*Cos(A/3)*Cos(C/3)^2 + 8*a^3*b^2*c^4*x*y*z*Cos(A/3)*Cos(C/3)^2 + 4*a^7*b^2*x*z^2*Cos(A/3)*Cos(C/3)^2 -      4*a^5*b^4*x*z^2*Cos(A/3)*Cos(C/3)^2 - 4*a^5*b^2*c^2*x*z^2*Cos(A/3)*Cos(C/3)^2 + 8*a^3*b^4*c^2*x*z^2*Cos(A/3)*Cos(C/3)^2 + 4*a^5*b^2*c^2*y*z^2*Cos(A/3)*Cos(C/3)^2 -      8*a^3*b^4*c^2*y*z^2*Cos(A/3)*Cos(C/3)^2 - 8*a^3*b^3*c^3*x^2*y*Cos(A/3)^2*Cos(C/3)^2 + 8*a^3*b^3*c^3*x^2*z*Cos(A/3)^2*Cos(C/3)^2 - 8*a*b^3*c^5*x^2*z*Cos(A/3)^2*Cos(C/3)^2 +      16*a^3*b^3*c^3*x*y*z*Cos(A/3)^2*Cos(C/3)^2 - 8*a^5*b^3*c*x*z^2*Cos(A/3)^2*Cos(C/3)^2 + 8*a^3*b^3*c^3*x*z^2*Cos(A/3)^2*Cos(C/3)^2 - 8*a^3*b^3*c^3*y*z^2*Cos(A/3)^2*Cos(C/3)^2 +      8*a^4*b^3*c^2*x*y*z*Cos(B/3)*Cos(C/3)^2 - 12*a^2*b^5*c^2*x*y*z*Cos(B/3)*Cos(C/3)^2 + 8*a^2*b^3*c^4*x*y*z*Cos(B/3)*Cos(C/3)^2 + 4*a^4*b^3*c^2*y^2*z*Cos(B/3)*Cos(C/3)^2 -      8*a^4*b*c^4*y^2*z*Cos(B/3)*Cos(C/3)^2 - 8*a^4*b^3*c^2*x*z^2*Cos(B/3)*Cos(C/3)^2 + 4*a^2*b^5*c^2*x*z^2*Cos(B/3)*Cos(C/3)^2 - 4*a^4*b^5*y*z^2*Cos(B/3)*Cos(C/3)^2 +      4*a^2*b^7*y*z^2*Cos(B/3)*Cos(C/3)^2 + 8*a^4*b^3*c^2*y*z^2*Cos(B/3)*Cos(C/3)^2 - 4*a^2*b^5*c^2*y*z^2*Cos(B/3)*Cos(C/3)^2 + 8*a^2*b^4*c^3*x^2*y*Cos(A/3)*Cos(B/3)*Cos(C/3)^2 +      8*a^4*b^2*c^3*x*y^2*Cos(A/3)*Cos(B/3)*Cos(C/3)^2 - 8*a^2*b^4*c^3*x^2*z*Cos(A/3)*Cos(B/3)*Cos(C/3)^2 - 16*a^6*b^2*c*x*y*z*Cos(A/3)*Cos(B/3)*Cos(C/3)^2 +      32*a^4*b^4*c*x*y*z*Cos(A/3)*Cos(B/3)*Cos(C/3)^2 - 16*a^2*b^6*c*x*y*z*Cos(A/3)*Cos(B/3)*Cos(C/3)^2 - 16*a^4*b^2*c^3*x*y*z*Cos(A/3)*Cos(B/3)*Cos(C/3)^2 -      16*a^2*b^4*c^3*x*y*z*Cos(A/3)*Cos(B/3)*Cos(C/3)^2 + 16*a^2*b^2*c^5*x*y*z*Cos(A/3)*Cos(B/3)*Cos(C/3)^2 - 8*a^4*b^2*c^3*y^2*z*Cos(A/3)*Cos(B/3)*Cos(C/3)^2 +      16*a^6*b^2*c*x*z^2*Cos(A/3)*Cos(B/3)*Cos(C/3)^2 - 8*a^4*b^4*c*x*z^2*Cos(A/3)*Cos(B/3)*Cos(C/3)^2 - 16*a^4*b^2*c^3*x*z^2*Cos(A/3)*Cos(B/3)*Cos(C/3)^2 +      16*a^2*b^4*c^3*x*z^2*Cos(A/3)*Cos(B/3)*Cos(C/3)^2 - 8*a^4*b^4*c*y*z^2*Cos(A/3)*Cos(B/3)*Cos(C/3)^2 + 16*a^2*b^6*c*y*z^2*Cos(A/3)*Cos(B/3)*Cos(C/3)^2 +      16*a^4*b^2*c^3*y*z^2*Cos(A/3)*Cos(B/3)*Cos(C/3)^2 - 16*a^2*b^4*c^3*y*z^2*Cos(A/3)*Cos(B/3)*Cos(C/3)^2 - 16*a^4*b^3*c^2*x^2*y*Cos(A/3)^2*Cos(B/3)*Cos(C/3)^2 +      16*a^2*b^5*c^2*x^2*y*Cos(A/3)^2*Cos(B/3)*Cos(C/3)^2 + 16*a^4*b^3*c^2*x^2*z*Cos(A/3)^2*Cos(B/3)*Cos(C/3)^2 - 16*a^2*b^3*c^4*x^2*z*Cos(A/3)^2*Cos(B/3)*Cos(C/3)^2 +      16*a^4*b^3*c^2*x*y*z*Cos(A/3)^2*Cos(B/3)*Cos(C/3)^2 - 32*a^2*b^5*c^2*x*y*z*Cos(A/3)^2*Cos(B/3)*Cos(C/3)^2 + 16*a^2*b^3*c^4*x*y*z*Cos(A/3)^2*Cos(B/3)*Cos(C/3)^2 -      16*a^4*b^3*c^2*x*z^2*Cos(A/3)^2*Cos(B/3)*Cos(C/3)^2 + 16*a^2*b^3*c^4*x*z^2*Cos(A/3)^2*Cos(B/3)*Cos(C/3)^2 + 16*a^2*b^5*c^2*y*z^2*Cos(A/3)^2*Cos(B/3)*Cos(C/3)^2 -      16*a^2*b^3*c^4*y*z^2*Cos(A/3)^2*Cos(B/3)*Cos(C/3)^2 - 8*a^3*b^3*c^3*x*y^2*Cos(B/3)^2*Cos(C/3)^2 + 16*a^3*b^3*c^3*x*y*z*Cos(B/3)^2*Cos(C/3)^2 +      8*a^3*b^3*c^3*y^2*z*Cos(B/3)^2*Cos(C/3)^2 - 8*a^3*b*c^5*y^2*z*Cos(B/3)^2*Cos(C/3)^2 - 8*a^3*b^3*c^3*x*z^2*Cos(B/3)^2*Cos(C/3)^2 - 8*a^3*b^5*c*y*z^2*Cos(B/3)^2*Cos(C/3)^2 +      8*a^3*b^3*c^3*y*z^2*Cos(B/3)^2*Cos(C/3)^2 + 16*a^5*b^2*c^2*x*y^2*Cos(A/3)*Cos(B/3)^2*Cos(C/3)^2 - 16*a^3*b^4*c^2*x*y^2*Cos(A/3)*Cos(B/3)^2*Cos(C/3)^2 -      32*a^5*b^2*c^2*x*y*z*Cos(A/3)*Cos(B/3)^2*Cos(C/3)^2 + 16*a^3*b^4*c^2*x*y*z*Cos(A/3)*Cos(B/3)^2*Cos(C/3)^2 + 16*a^3*b^2*c^4*x*y*z*Cos(A/3)*Cos(B/3)^2*Cos(C/3)^2 +      16*a^3*b^4*c^2*y^2*z*Cos(A/3)*Cos(B/3)^2*Cos(C/3)^2 - 16*a^3*b^2*c^4*y^2*z*Cos(A/3)*Cos(B/3)^2*Cos(C/3)^2 + 16*a^5*b^2*c^2*x*z^2*Cos(A/3)*Cos(B/3)^2*Cos(C/3)^2 -      16*a^3*b^2*c^4*x*z^2*Cos(A/3)*Cos(B/3)^2*Cos(C/3)^2 - 16*a^3*b^4*c^2*y*z^2*Cos(A/3)*Cos(B/3)^2*Cos(C/3)^2 + 16*a^3*b^2*c^4*y*z^2*Cos(A/3)*Cos(B/3)^2*Cos(C/3)^2");
		exp("K588","(-a)*b^3*c^2*x^2*y + b^4*c^2*x^2*y + a*b^2*c^3*x^2*y - b^3*c^3*x^2*y + a^4*c^2*x*y^2 -      a^3*b*c^2*x*y^2 - a^3*c^3*x*y^2 + a^2*b*c^3*x*y^2 + a*b^3*c^2*x^2*z - a*b^2*c^3*x^2*z -      b^3*c^3*x^2*z + b^2*c^4*x^2*z - 2*a^4*b*c*x*y*z + 2*a^3*b^2*c*x*y*z + 2*a^2*b^3*c*x*y*z -      2*a*b^4*c*x*y*z + 2*a^3*b*c^2*x*y*z - 6*a^2*b^2*c^2*x*y*z + 2*a*b^3*c^2*x*y*z + 2*a^2*b*c^3*x*y*z +      2*a*b^2*c^3*x*y*z - 2*a*b*c^4*x*y*z + a^3*b*c^2*y^2*z - a^3*c^3*y^2*z - a^2*b*c^3*y^2*z +      a^2*c^4*y^2*z + a^4*b^2*x*z^2 - a^3*b^3*x*z^2 - a^3*b^2*c*x*z^2 + a^2*b^3*c*x*z^2 - a^3*b^3*y*z^2 +      a^2*b^4*y*z^2 + a^3*b^2*c*y*z^2 - a^2*b^3*c*y*z^2");
		exp("K589","a^8*b^4*x^2*y - 4*a^6*b^6*x^2*y + 6*a^4*b^8*x^2*y - 4*a^2*b^10*x^2*y + b^12*x^2*y -      a^8*b^2*c^2*x^2*y + 3*a^6*b^4*c^2*x^2*y - 5*a^4*b^6*c^2*x^2*y + 5*a^2*b^8*c^2*x^2*y -      2*b^10*c^2*x^2*y + a^6*b^2*c^4*x^2*y - a^4*b^4*c^4*x^2*y - a^2*b^6*c^4*x^2*y + b^8*c^4*x^2*y +      a^12*x*y^2 - 4*a^10*b^2*x*y^2 + 6*a^8*b^4*x*y^2 - 4*a^6*b^6*x*y^2 + a^4*b^8*x*y^2 -      2*a^10*c^2*x*y^2 + 5*a^8*b^2*c^2*x*y^2 - 5*a^6*b^4*c^2*x*y^2 + 3*a^4*b^6*c^2*x*y^2 -      a^2*b^8*c^2*x*y^2 + a^8*c^4*x*y^2 - a^6*b^2*c^4*x*y^2 - a^4*b^4*c^4*x*y^2 + a^2*b^6*c^4*x*y^2 -      a^8*b^2*c^2*x^2*z + a^6*b^4*c^2*x^2*z + a^8*c^4*x^2*z + 3*a^6*b^2*c^4*x^2*z - a^4*b^4*c^4*x^2*z -      4*a^6*c^6*x^2*z - 5*a^4*b^2*c^6*x^2*z - a^2*b^4*c^6*x^2*z + 6*a^4*c^8*x^2*z + 5*a^2*b^2*c^8*x^2*z +      b^4*c^8*x^2*z - 4*a^2*c^10*x^2*z - 2*b^2*c^10*x^2*z + c^12*x^2*z - 2*a^12*x*y*z +      6*a^10*b^2*x*y*z - 6*a^8*b^4*x*y*z + 4*a^6*b^6*x*y*z - 6*a^4*b^8*x*y*z + 6*a^2*b^10*x*y*z -      2*b^12*x*y*z + 6*a^10*c^2*x*y*z - 12*a^8*b^2*c^2*x*y*z + 6*a^6*b^4*c^2*x*y*z +      6*a^4*b^6*c^2*x*y*z - 12*a^2*b^8*c^2*x*y*z + 6*b^10*c^2*x*y*z - 6*a^8*c^4*x*y*z +      6*a^6*b^2*c^4*x*y*z - 6*a^4*b^4*c^4*x*y*z + 6*a^2*b^6*c^4*x*y*z - 6*b^8*c^4*x*y*z +      4*a^6*c^6*x*y*z + 6*a^4*b^2*c^6*x*y*z + 6*a^2*b^4*c^6*x*y*z + 4*b^6*c^6*x*y*z - 6*a^4*c^8*x*y*z -      12*a^2*b^2*c^8*x*y*z - 6*b^4*c^8*x*y*z + 6*a^2*c^10*x*y*z + 6*b^2*c^10*x*y*z - 2*c^12*x*y*z +      a^4*b^6*c^2*y^2*z - a^2*b^8*c^2*y^2*z - a^4*b^4*c^4*y^2*z + 3*a^2*b^6*c^4*y^2*z + b^8*c^4*y^2*z -      a^4*b^2*c^6*y^2*z - 5*a^2*b^4*c^6*y^2*z - 4*b^6*c^6*y^2*z + a^4*c^8*y^2*z + 5*a^2*b^2*c^8*y^2*z +      6*b^4*c^8*y^2*z - 2*a^2*c^10*y^2*z - 4*b^2*c^10*y^2*z + c^12*y^2*z + a^12*x*z^2 -      2*a^10*b^2*x*z^2 + a^8*b^4*x*z^2 - 4*a^10*c^2*x*z^2 + 5*a^8*b^2*c^2*x*z^2 - a^6*b^4*c^2*x*z^2 +      6*a^8*c^4*x*z^2 - 5*a^6*b^2*c^4*x*z^2 - a^4*b^4*c^4*x*z^2 - 4*a^6*c^6*x*z^2 + 3*a^4*b^2*c^6*x*z^2 +      a^2*b^4*c^6*x*z^2 + a^4*c^8*x*z^2 - a^2*b^2*c^8*x*z^2 + a^4*b^8*y*z^2 - 2*a^2*b^10*y*z^2 +      b^12*y*z^2 - a^4*b^6*c^2*y*z^2 + 5*a^2*b^8*c^2*y*z^2 - 4*b^10*c^2*y*z^2 - a^4*b^4*c^4*y*z^2 -      5*a^2*b^6*c^4*y*z^2 + 6*b^8*c^4*y*z^2 + a^4*b^2*c^6*y*z^2 + 3*a^2*b^4*c^6*y*z^2 - 4*b^6*c^6*y*z^2 -      a^2*b^2*c^8*y*z^2 + b^4*c^8*y*z^2");
		exp("K590","a^4*b^10*c^4*x^2*y - 2*a^2*b^12*c^4*x^2*y + b^14*c^4*x^2*y - a^4*b^8*c^6*x^2*y +      5*a^2*b^10*c^6*x^2*y - 4*b^12*c^6*x^2*y - a^4*b^6*c^8*x^2*y - 5*a^2*b^8*c^8*x^2*y +      6*b^10*c^8*x^2*y + a^4*b^4*c^10*x^2*y + 3*a^2*b^6*c^10*x^2*y - 4*b^8*c^10*x^2*y -      a^2*b^4*c^12*x^2*y + b^6*c^12*x^2*y + a^14*c^4*x*y^2 - 2*a^12*b^2*c^4*x*y^2 + a^10*b^4*c^4*x*y^2 -      4*a^12*c^6*x*y^2 + 5*a^10*b^2*c^6*x*y^2 - a^8*b^4*c^6*x*y^2 + 6*a^10*c^8*x*y^2 -      5*a^8*b^2*c^8*x*y^2 - a^6*b^4*c^8*x*y^2 - 4*a^8*c^10*x*y^2 + 3*a^6*b^2*c^10*x*y^2 +      a^4*b^4*c^10*x*y^2 + a^6*c^12*x*y^2 - a^4*b^2*c^12*x*y^2 + a^4*b^10*c^4*x^2*z -      a^2*b^12*c^4*x^2*z - a^4*b^8*c^6*x^2*z + 3*a^2*b^10*c^6*x^2*z + b^12*c^6*x^2*z -      a^4*b^6*c^8*x^2*z - 5*a^2*b^8*c^8*x^2*z - 4*b^10*c^8*x^2*z + a^4*b^4*c^10*x^2*z +      5*a^2*b^6*c^10*x^2*z + 6*b^8*c^10*x^2*z - 2*a^2*b^4*c^12*x^2*z - 4*b^6*c^12*x^2*z +      b^4*c^14*x^2*z - 2*a^14*b^2*c^2*x*y*z + 6*a^12*b^4*c^2*x*y*z - 6*a^10*b^6*c^2*x*y*z +      4*a^8*b^8*c^2*x*y*z - 6*a^6*b^10*c^2*x*y*z + 6*a^4*b^12*c^2*x*y*z - 2*a^2*b^14*c^2*x*y*z +      6*a^12*b^2*c^4*x*y*z - 12*a^10*b^4*c^4*x*y*z + 6*a^8*b^6*c^4*x*y*z + 6*a^6*b^8*c^4*x*y*z -      12*a^4*b^10*c^4*x*y*z + 6*a^2*b^12*c^4*x*y*z - 6*a^10*b^2*c^6*x*y*z + 6*a^8*b^4*c^6*x*y*z -      6*a^6*b^6*c^6*x*y*z + 6*a^4*b^8*c^6*x*y*z - 6*a^2*b^10*c^6*x*y*z + 4*a^8*b^2*c^8*x*y*z +      6*a^6*b^4*c^8*x*y*z + 6*a^4*b^6*c^8*x*y*z + 4*a^2*b^8*c^8*x*y*z - 6*a^6*b^2*c^10*x*y*z -      12*a^4*b^4*c^10*x*y*z - 6*a^2*b^6*c^10*x*y*z + 6*a^4*b^2*c^12*x*y*z + 6*a^2*b^4*c^12*x*y*z -      2*a^2*b^2*c^14*x*y*z - a^12*b^2*c^4*y^2*z + a^10*b^4*c^4*y^2*z + a^12*c^6*y^2*z +      3*a^10*b^2*c^6*y^2*z - a^8*b^4*c^6*y^2*z - 4*a^10*c^8*y^2*z - 5*a^8*b^2*c^8*y^2*z -      a^6*b^4*c^8*y^2*z + 6*a^8*c^10*y^2*z + 5*a^6*b^2*c^10*y^2*z + a^4*b^4*c^10*y^2*z -      4*a^6*c^12*y^2*z - 2*a^4*b^2*c^12*y^2*z + a^4*c^14*y^2*z + a^14*b^4*x*z^2 - 4*a^12*b^6*x*z^2 +      6*a^10*b^8*x*z^2 - 4*a^8*b^10*x*z^2 + a^6*b^12*x*z^2 - 2*a^12*b^4*c^2*x*z^2 +      5*a^10*b^6*c^2*x*z^2 - 5*a^8*b^8*c^2*x*z^2 + 3*a^6*b^10*c^2*x*z^2 - a^4*b^12*c^2*x*z^2 +      a^10*b^4*c^4*x*z^2 - a^8*b^6*c^4*x*z^2 - a^6*b^8*c^4*x*z^2 + a^4*b^10*c^4*x*z^2 + a^12*b^6*y*z^2 -      4*a^10*b^8*y*z^2 + 6*a^8*b^10*y*z^2 - 4*a^6*b^12*y*z^2 + a^4*b^14*y*z^2 - a^12*b^4*c^2*y*z^2 +      3*a^10*b^6*c^2*y*z^2 - 5*a^8*b^8*c^2*y*z^2 + 5*a^6*b^10*c^2*y*z^2 - 2*a^4*b^12*c^2*y*z^2 +      a^10*b^4*c^4*y*z^2 - a^8*b^6*c^4*y*z^2 - a^6*b^8*c^4*y*z^2 + a^4*b^10*c^4*y*z^2");
		exp("K591","(-a^4)*b^2*x^3 + 2*a^2*b^4*x^3 - b^6*x^3 + a^4*c^2*x^3 - b^4*c^2*x^3 - 2*a^2*c^4*x^3 +      b^2*c^4*x^3 + c^6*x^3 - a^4*b^2*x^2*y + 2*a^2*b^4*x^2*y - b^6*x^2*y + a^4*c^2*x^2*y -      2*a^2*b^2*c^2*x^2*y + b^4*c^2*x^2*y + b^2*c^4*x^2*y - c^6*x^2*y + a^6*x*y^2 - 2*a^4*b^2*x*y^2 +      a^2*b^4*x*y^2 - a^4*c^2*x*y^2 + 2*a^2*b^2*c^2*x*y^2 - b^4*c^2*x*y^2 - a^2*c^4*x*y^2 + c^6*x*y^2 +      a^6*y^3 - 2*a^4*b^2*y^3 + a^2*b^4*y^3 + a^4*c^2*y^3 - b^4*c^2*y^3 - a^2*c^4*y^3 + 2*b^2*c^4*y^3 -      c^6*y^3 - a^4*b^2*x^2*z + b^6*x^2*z + a^4*c^2*x^2*z + 2*a^2*b^2*c^2*x^2*z - b^4*c^2*x^2*z -      2*a^2*c^4*x^2*z - b^2*c^4*x^2*z + c^6*x^2*z - 4*a^4*b^2*x*y*z + 4*a^2*b^4*x*y*z + 4*a^4*c^2*x*y*z -      4*b^4*c^2*x*y*z - 4*a^2*c^4*x*y*z + 4*b^2*c^4*x*y*z - a^6*y^2*z + a^2*b^4*y^2*z + a^4*c^2*y^2*z -      2*a^2*b^2*c^2*y^2*z - b^4*c^2*y^2*z + a^2*c^4*y^2*z + 2*b^2*c^4*y^2*z - c^6*y^2*z - a^6*x*z^2 +      a^4*b^2*x*z^2 + a^2*b^4*x*z^2 - b^6*x*z^2 + 2*a^4*c^2*x*z^2 - 2*a^2*b^2*c^2*x*z^2 - a^2*c^4*x*z^2 +      b^2*c^4*x*z^2 + a^6*y*z^2 - a^4*b^2*y*z^2 - a^2*b^4*y*z^2 + b^6*y*z^2 + 2*a^2*b^2*c^2*y*z^2 -      2*b^4*c^2*y*z^2 - a^2*c^4*y*z^2 + b^2*c^4*y*z^2 - a^6*z^3 - a^4*b^2*z^3 + a^2*b^4*z^3 + b^6*z^3 +      2*a^4*c^2*z^3 - 2*b^4*c^2*z^3 - a^2*c^4*z^3 + b^2*c^4*z^3");
		exp("K592","a^12*b^6*x^3 - 4*a^10*b^8*x^3 + 5*a^8*b^10*x^3 - 5*a^4*b^14*x^3 + 4*a^2*b^16*x^3 - b^18*x^3 -      3*a^12*b^4*c^2*x^3 + 8*a^10*b^6*c^2*x^3 - 3*a^8*b^8*c^2*x^3 - 12*a^6*b^10*c^2*x^3 +      19*a^4*b^12*c^2*x^3 - 12*a^2*b^14*c^2*x^3 + 3*b^16*c^2*x^3 + 3*a^12*b^2*c^4*x^3 -      16*a^8*b^6*c^4*x^3 + 24*a^6*b^8*c^4*x^3 - 16*a^4*b^10*c^4*x^3 + 8*a^2*b^12*c^4*x^3 -      3*b^14*c^4*x^3 - a^12*c^6*x^3 - 8*a^10*b^2*c^6*x^3 + 16*a^8*b^4*c^6*x^3 - 12*a^4*b^8*c^6*x^3 +      4*a^2*b^10*c^6*x^3 + 3*b^12*c^6*x^3 + 4*a^10*c^8*x^3 + 3*a^8*b^2*c^8*x^3 - 24*a^6*b^4*c^8*x^3 +      12*a^4*b^6*c^8*x^3 - 6*b^10*c^8*x^3 - 5*a^8*c^10*x^3 + 12*a^6*b^2*c^10*x^3 + 16*a^4*b^4*c^10*x^3 -      4*a^2*b^6*c^10*x^3 + 6*b^8*c^10*x^3 - 19*a^4*b^2*c^12*x^3 - 8*a^2*b^4*c^12*x^3 - 3*b^6*c^12*x^3 +      5*a^4*c^14*x^3 + 12*a^2*b^2*c^14*x^3 + 3*b^4*c^14*x^3 - 4*a^2*c^16*x^3 - 3*b^2*c^16*x^3 +      c^18*x^3 + 3*a^14*b^4*x^2*y - 12*a^12*b^6*x^2*y + 15*a^10*b^8*x^2*y - 15*a^6*b^12*x^2*y +      12*a^4*b^14*x^2*y - 3*a^2*b^16*x^2*y - 6*a^14*b^2*c^2*x^2*y + 12*a^12*b^4*c^2*x^2*y +      11*a^10*b^6*c^2*x^2*y - 53*a^8*b^8*c^2*x^2*y + 60*a^6*b^10*c^2*x^2*y - 26*a^4*b^12*c^2*x^2*y -      a^2*b^14*c^2*x^2*y + 3*b^16*c^2*x^2*y + 3*a^14*c^4*x^2*y + 12*a^12*b^2*c^4*x^2*y -      51*a^10*b^4*c^4*x^2*y + 62*a^8*b^6*c^4*x^2*y - 23*a^6*b^8*c^4*x^2*y - 19*a^4*b^10*c^4*x^2*y +      27*a^2*b^12*c^4*x^2*y - 11*b^14*c^4*x^2*y - 12*a^12*c^6*x^2*y + 9*a^10*b^2*c^6*x^2*y +      30*a^8*b^4*c^6*x^2*y - 67*a^6*b^6*c^6*x^2*y + 60*a^4*b^8*c^6*x^2*y - 32*a^2*b^10*c^6*x^2*y +      14*b^12*c^6*x^2*y + 16*a^10*c^8*x^2*y - 34*a^8*b^2*c^8*x^2*y + 33*a^6*b^4*c^8*x^2*y -      6*a^4*b^6*c^8*x^2*y - 9*a^2*b^8*c^8*x^2*y - 9*b^10*c^8*x^2*y - 5*a^8*c^10*x^2*y +      17*a^6*b^2*c^10*x^2*y - 32*a^4*b^4*c^10*x^2*y + 25*a^2*b^6*c^10*x^2*y + 10*b^8*c^10*x^2*y -      5*a^6*c^12*x^2*y + 9*a^4*b^2*c^12*x^2*y - a^2*b^4*c^12*x^2*y - 13*b^6*c^12*x^2*y +      2*a^4*c^14*x^2*y - 8*a^2*b^2*c^14*x^2*y + 6*b^4*c^14*x^2*y + 2*a^2*c^16*x^2*y + b^2*c^16*x^2*y -      c^18*x^2*y + 3*a^16*b^2*x*y^2 - 12*a^14*b^4*x*y^2 + 15*a^12*b^6*x*y^2 - 15*a^8*b^10*x*y^2 +      12*a^6*b^12*x*y^2 - 3*a^4*b^14*x*y^2 - 3*a^16*c^2*x*y^2 + a^14*b^2*c^2*x*y^2 +      26*a^12*b^4*c^2*x*y^2 - 60*a^10*b^6*c^2*x*y^2 + 53*a^8*b^8*c^2*x*y^2 - 11*a^6*b^10*c^2*x*y^2 -      12*a^4*b^12*c^2*x*y^2 + 6*a^2*b^14*c^2*x*y^2 + 11*a^14*c^4*x*y^2 - 27*a^12*b^2*c^4*x*y^2 +      19*a^10*b^4*c^4*x*y^2 + 23*a^8*b^6*c^4*x*y^2 - 62*a^6*b^8*c^4*x*y^2 + 51*a^4*b^10*c^4*x*y^2 -      12*a^2*b^12*c^4*x*y^2 - 3*b^14*c^4*x*y^2 - 14*a^12*c^6*x*y^2 + 32*a^10*b^2*c^6*x*y^2 -      60*a^8*b^4*c^6*x*y^2 + 67*a^6*b^6*c^6*x*y^2 - 30*a^4*b^8*c^6*x*y^2 - 9*a^2*b^10*c^6*x*y^2 +      12*b^12*c^6*x*y^2 + 9*a^10*c^8*x*y^2 + 9*a^8*b^2*c^8*x*y^2 + 6*a^6*b^4*c^8*x*y^2 -      33*a^4*b^6*c^8*x*y^2 + 34*a^2*b^8*c^8*x*y^2 - 16*b^10*c^8*x*y^2 - 10*a^8*c^10*x*y^2 -      25*a^6*b^2*c^10*x*y^2 + 32*a^4*b^4*c^10*x*y^2 - 17*a^2*b^6*c^10*x*y^2 + 5*b^8*c^10*x*y^2 +      13*a^6*c^12*x*y^2 + a^4*b^2*c^12*x*y^2 - 9*a^2*b^4*c^12*x*y^2 + 5*b^6*c^12*x*y^2 -      6*a^4*c^14*x*y^2 + 8*a^2*b^2*c^14*x*y^2 - 2*b^4*c^14*x*y^2 - a^2*c^16*x*y^2 - 2*b^2*c^16*x*y^2 +      c^18*x*y^2 + a^18*y^3 - 4*a^16*b^2*y^3 + 5*a^14*b^4*y^3 - 5*a^10*b^8*y^3 + 4*a^8*b^10*y^3 -      a^6*b^12*y^3 - 3*a^16*c^2*y^3 + 12*a^14*b^2*c^2*y^3 - 19*a^12*b^4*c^2*y^3 + 12*a^10*b^6*c^2*y^3 +      3*a^8*b^8*c^2*y^3 - 8*a^6*b^10*c^2*y^3 + 3*a^4*b^12*c^2*y^3 + 3*a^14*c^4*y^3 - 8*a^12*b^2*c^4*y^3 +      16*a^10*b^4*c^4*y^3 - 24*a^8*b^6*c^4*y^3 + 16*a^6*b^8*c^4*y^3 - 3*a^2*b^12*c^4*y^3 -      3*a^12*c^6*y^3 - 4*a^10*b^2*c^6*y^3 + 12*a^8*b^4*c^6*y^3 - 16*a^4*b^8*c^6*y^3 +      8*a^2*b^10*c^6*y^3 + b^12*c^6*y^3 + 6*a^10*c^8*y^3 - 12*a^6*b^4*c^8*y^3 + 24*a^4*b^6*c^8*y^3 -      3*a^2*b^8*c^8*y^3 - 4*b^10*c^8*y^3 - 6*a^8*c^10*y^3 + 4*a^6*b^2*c^10*y^3 - 16*a^4*b^4*c^10*y^3 -      12*a^2*b^6*c^10*y^3 + 5*b^8*c^10*y^3 + 3*a^6*c^12*y^3 + 8*a^4*b^2*c^12*y^3 + 19*a^2*b^4*c^12*y^3 -      3*a^4*c^14*y^3 - 12*a^2*b^2*c^14*y^3 - 5*b^4*c^14*y^3 + 3*a^2*c^16*y^3 + 4*b^2*c^16*y^3 -      c^18*y^3 - 3*a^14*b^4*x^2*z + 12*a^12*b^6*x^2*z - 16*a^10*b^8*x^2*z + 5*a^8*b^10*x^2*z +      5*a^6*b^12*x^2*z - 2*a^4*b^14*x^2*z - 2*a^2*b^16*x^2*z + b^18*x^2*z + 6*a^14*b^2*c^2*x^2*z -      12*a^12*b^4*c^2*x^2*z - 9*a^10*b^6*c^2*x^2*z + 34*a^8*b^8*c^2*x^2*z - 17*a^6*b^10*c^2*x^2*z -      9*a^4*b^12*c^2*x^2*z + 8*a^2*b^14*c^2*x^2*z - b^16*c^2*x^2*z - 3*a^14*c^4*x^2*z -      12*a^12*b^2*c^4*x^2*z + 51*a^10*b^4*c^4*x^2*z - 30*a^8*b^6*c^4*x^2*z - 33*a^6*b^8*c^4*x^2*z +      32*a^4*b^10*c^4*x^2*z + a^2*b^12*c^4*x^2*z - 6*b^14*c^4*x^2*z + 12*a^12*c^6*x^2*z -      11*a^10*b^2*c^6*x^2*z - 62*a^8*b^4*c^6*x^2*z + 67*a^6*b^6*c^6*x^2*z + 6*a^4*b^8*c^6*x^2*z -      25*a^2*b^10*c^6*x^2*z + 13*b^12*c^6*x^2*z - 15*a^10*c^8*x^2*z + 53*a^8*b^2*c^8*x^2*z +      23*a^6*b^4*c^8*x^2*z - 60*a^4*b^6*c^8*x^2*z + 9*a^2*b^8*c^8*x^2*z - 10*b^10*c^8*x^2*z -      60*a^6*b^2*c^10*x^2*z + 19*a^4*b^4*c^10*x^2*z + 32*a^2*b^6*c^10*x^2*z + 9*b^8*c^10*x^2*z +      15*a^6*c^12*x^2*z + 26*a^4*b^2*c^12*x^2*z - 27*a^2*b^4*c^12*x^2*z - 14*b^6*c^12*x^2*z -      12*a^4*c^14*x^2*z + a^2*b^2*c^14*x^2*z + 11*b^4*c^14*x^2*z + 3*a^2*c^16*x^2*z - 3*b^2*c^16*x^2*z -      3*a^16*b^2*x*y*z + 7*a^14*b^4*x*y*z + 7*a^12*b^6*x*y*z - 35*a^10*b^8*x*y*z + 35*a^8*b^10*x*y*z -      7*a^6*b^12*x*y*z - 7*a^4*b^14*x*y*z + 3*a^2*b^16*x*y*z + 3*a^16*c^2*x*y*z - 31*a^12*b^4*c^2*x*y*z +      50*a^10*b^6*c^2*x*y*z - 50*a^6*b^10*c^2*x*y*z + 31*a^4*b^12*c^2*x*y*z - 3*b^16*c^2*x*y*z -      7*a^14*c^4*x*y*z + 31*a^12*b^2*c^4*x*y*z - 90*a^8*b^6*c^4*x*y*z + 90*a^6*b^8*c^4*x*y*z -      31*a^2*b^12*c^4*x*y*z + 7*b^14*c^4*x*y*z - 7*a^12*c^6*x*y*z - 50*a^10*b^2*c^6*x*y*z +      90*a^8*b^4*c^6*x*y*z - 90*a^4*b^8*c^6*x*y*z + 50*a^2*b^10*c^6*x*y*z + 7*b^12*c^6*x*y*z +      35*a^10*c^8*x*y*z - 90*a^6*b^4*c^8*x*y*z + 90*a^4*b^6*c^8*x*y*z - 35*b^10*c^8*x*y*z -      35*a^8*c^10*x*y*z + 50*a^6*b^2*c^10*x*y*z - 50*a^2*b^6*c^10*x*y*z + 35*b^8*c^10*x*y*z +      7*a^6*c^12*x*y*z - 31*a^4*b^2*c^12*x*y*z + 31*a^2*b^4*c^12*x*y*z - 7*b^6*c^12*x*y*z +      7*a^4*c^14*x*y*z - 7*b^4*c^14*x*y*z - 3*a^2*c^16*x*y*z + 3*b^2*c^16*x*y*z - a^18*y^2*z +      2*a^16*b^2*y^2*z + 2*a^14*b^4*y^2*z - 5*a^12*b^6*y^2*z - 5*a^10*b^8*y^2*z + 16*a^8*b^10*y^2*z -      12*a^6*b^12*y^2*z + 3*a^4*b^14*y^2*z + a^16*c^2*y^2*z - 8*a^14*b^2*c^2*y^2*z +      9*a^12*b^4*c^2*y^2*z + 17*a^10*b^6*c^2*y^2*z - 34*a^8*b^8*c^2*y^2*z + 9*a^6*b^10*c^2*y^2*z +      12*a^4*b^12*c^2*y^2*z - 6*a^2*b^14*c^2*y^2*z + 6*a^14*c^4*y^2*z - a^12*b^2*c^4*y^2*z -      32*a^10*b^4*c^4*y^2*z + 33*a^8*b^6*c^4*y^2*z + 30*a^6*b^8*c^4*y^2*z - 51*a^4*b^10*c^4*y^2*z +      12*a^2*b^12*c^4*y^2*z + 3*b^14*c^4*y^2*z - 13*a^12*c^6*y^2*z + 25*a^10*b^2*c^6*y^2*z -      6*a^8*b^4*c^6*y^2*z - 67*a^6*b^6*c^6*y^2*z + 62*a^4*b^8*c^6*y^2*z + 11*a^2*b^10*c^6*y^2*z -      12*b^12*c^6*y^2*z + 10*a^10*c^8*y^2*z - 9*a^8*b^2*c^8*y^2*z + 60*a^6*b^4*c^8*y^2*z -      23*a^4*b^6*c^8*y^2*z - 53*a^2*b^8*c^8*y^2*z + 15*b^10*c^8*y^2*z - 9*a^8*c^10*y^2*z -      32*a^6*b^2*c^10*y^2*z - 19*a^4*b^4*c^10*y^2*z + 60*a^2*b^6*c^10*y^2*z + 14*a^6*c^12*y^2*z +      27*a^4*b^2*c^12*y^2*z - 26*a^2*b^4*c^12*y^2*z - 15*b^6*c^12*y^2*z - 11*a^4*c^14*y^2*z -      a^2*b^2*c^14*y^2*z + 12*b^4*c^14*y^2*z + 3*a^2*c^16*y^2*z - 3*b^2*c^16*y^2*z + 3*a^16*b^2*x*z^2 -      11*a^14*b^4*x*z^2 + 14*a^12*b^6*x*z^2 - 9*a^10*b^8*x*z^2 + 10*a^8*b^10*x*z^2 - 13*a^6*b^12*x*z^2 +      6*a^4*b^14*x*z^2 + a^2*b^16*x*z^2 - b^18*x*z^2 - 3*a^16*c^2*x*z^2 - a^14*b^2*c^2*x*z^2 +      27*a^12*b^4*c^2*x*z^2 - 32*a^10*b^6*c^2*x*z^2 - 9*a^8*b^8*c^2*x*z^2 + 25*a^6*b^10*c^2*x*z^2 -      a^4*b^12*c^2*x*z^2 - 8*a^2*b^14*c^2*x*z^2 + 2*b^16*c^2*x*z^2 + 12*a^14*c^4*x*z^2 -      26*a^12*b^2*c^4*x*z^2 - 19*a^10*b^4*c^4*x*z^2 + 60*a^8*b^6*c^4*x*z^2 - 6*a^6*b^8*c^4*x*z^2 -      32*a^4*b^10*c^4*x*z^2 + 9*a^2*b^12*c^4*x*z^2 + 2*b^14*c^4*x*z^2 - 15*a^12*c^6*x*z^2 +      60*a^10*b^2*c^6*x*z^2 - 23*a^8*b^4*c^6*x*z^2 - 67*a^6*b^6*c^6*x*z^2 + 33*a^4*b^8*c^6*x*z^2 +      17*a^2*b^10*c^6*x*z^2 - 5*b^12*c^6*x*z^2 - 53*a^8*b^2*c^8*x*z^2 + 62*a^6*b^4*c^8*x*z^2 +      30*a^4*b^6*c^8*x*z^2 - 34*a^2*b^8*c^8*x*z^2 - 5*b^10*c^8*x*z^2 + 15*a^8*c^10*x*z^2 +      11*a^6*b^2*c^10*x*z^2 - 51*a^4*b^4*c^10*x*z^2 + 9*a^2*b^6*c^10*x*z^2 + 16*b^8*c^10*x*z^2 -      12*a^6*c^12*x*z^2 + 12*a^4*b^2*c^12*x*z^2 + 12*a^2*b^4*c^12*x*z^2 - 12*b^6*c^12*x*z^2 +      3*a^4*c^14*x*z^2 - 6*a^2*b^2*c^14*x*z^2 + 3*b^4*c^14*x*z^2 + a^18*y*z^2 - a^16*b^2*y*z^2 -      6*a^14*b^4*y*z^2 + 13*a^12*b^6*y*z^2 - 10*a^10*b^8*y*z^2 + 9*a^8*b^10*y*z^2 - 14*a^6*b^12*y*z^2 +      11*a^4*b^14*y*z^2 - 3*a^2*b^16*y*z^2 - 2*a^16*c^2*y*z^2 + 8*a^14*b^2*c^2*y*z^2 +      a^12*b^4*c^2*y*z^2 - 25*a^10*b^6*c^2*y*z^2 + 9*a^8*b^8*c^2*y*z^2 + 32*a^6*b^10*c^2*y*z^2 -      27*a^4*b^12*c^2*y*z^2 + a^2*b^14*c^2*y*z^2 + 3*b^16*c^2*y*z^2 - 2*a^14*c^4*y*z^2 -      9*a^12*b^2*c^4*y*z^2 + 32*a^10*b^4*c^4*y*z^2 + 6*a^8*b^6*c^4*y*z^2 - 60*a^6*b^8*c^4*y*z^2 +      19*a^4*b^10*c^4*y*z^2 + 26*a^2*b^12*c^4*y*z^2 - 12*b^14*c^4*y*z^2 + 5*a^12*c^6*y*z^2 -      17*a^10*b^2*c^6*y*z^2 - 33*a^8*b^4*c^6*y*z^2 + 67*a^6*b^6*c^6*y*z^2 + 23*a^4*b^8*c^6*y*z^2 -      60*a^2*b^10*c^6*y*z^2 + 15*b^12*c^6*y*z^2 + 5*a^10*c^8*y*z^2 + 34*a^8*b^2*c^8*y*z^2 -      30*a^6*b^4*c^8*y*z^2 - 62*a^4*b^6*c^8*y*z^2 + 53*a^2*b^8*c^8*y*z^2 - 16*a^8*c^10*y*z^2 -      9*a^6*b^2*c^10*y*z^2 + 51*a^4*b^4*c^10*y*z^2 - 11*a^2*b^6*c^10*y*z^2 - 15*b^8*c^10*y*z^2 +      12*a^6*c^12*y*z^2 - 12*a^4*b^2*c^12*y*z^2 - 12*a^2*b^4*c^12*y*z^2 + 12*b^6*c^12*y*z^2 -      3*a^4*c^14*y*z^2 + 6*a^2*b^2*c^14*y*z^2 - 3*b^4*c^14*y*z^2 - a^18*z^3 + 3*a^16*b^2*z^3 -      3*a^14*b^4*z^3 + 3*a^12*b^6*z^3 - 6*a^10*b^8*z^3 + 6*a^8*b^10*z^3 - 3*a^6*b^12*z^3 +      3*a^4*b^14*z^3 - 3*a^2*b^16*z^3 + b^18*z^3 + 4*a^16*c^2*z^3 - 12*a^14*b^2*c^2*z^3 +      8*a^12*b^4*c^2*z^3 + 4*a^10*b^6*c^2*z^3 - 4*a^6*b^10*c^2*z^3 - 8*a^4*b^12*c^2*z^3 +      12*a^2*b^14*c^2*z^3 - 4*b^16*c^2*z^3 - 5*a^14*c^4*z^3 + 19*a^12*b^2*c^4*z^3 - 16*a^10*b^4*c^4*z^3 -      12*a^8*b^6*c^4*z^3 + 12*a^6*b^8*c^4*z^3 + 16*a^4*b^10*c^4*z^3 - 19*a^2*b^12*c^4*z^3 +      5*b^14*c^4*z^3 - 12*a^10*b^2*c^6*z^3 + 24*a^8*b^4*c^6*z^3 - 24*a^4*b^8*c^6*z^3 +      12*a^2*b^10*c^6*z^3 + 5*a^10*c^8*z^3 - 3*a^8*b^2*c^8*z^3 - 16*a^6*b^4*c^8*z^3 +      16*a^4*b^6*c^8*z^3 + 3*a^2*b^8*c^8*z^3 - 5*b^10*c^8*z^3 - 4*a^8*c^10*z^3 + 8*a^6*b^2*c^10*z^3 -      8*a^2*b^6*c^10*z^3 + 4*b^8*c^10*z^3 + a^6*c^12*z^3 - 3*a^4*b^2*c^12*z^3 + 3*a^2*b^4*c^12*z^3 -      b^6*c^12*z^3");
		exp("K593","-a^8*b^6*x^3 + 5*a^6*b^8*x^3 - 9*a^4*b^10*x^3 + 7*a^2*b^12*x^3 - 2*b^14*x^3 + 3*a^8*b^4*c^2*x^3 -      10*a^6*b^6*c^2*x^3 + 12*a^4*b^8*c^2*x^3 - 8*a^2*b^10*c^2*x^3 + 3*b^12*c^2*x^3 - 3*a^8*b^2*c^4*x^3 +      9*a^4*b^6*c^4*x^3 - 5*a^2*b^8*c^4*x^3 - 3*b^10*c^4*x^3 + a^8*c^6*x^3 + 10*a^6*b^2*c^6*x^3 -      9*a^4*b^4*c^6*x^3 + 8*b^8*c^6*x^3 - 5*a^6*c^8*x^3 - 12*a^4*b^2*c^8*x^3 + 5*a^2*b^4*c^8*x^3 -      8*b^6*c^8*x^3 + 9*a^4*c^10*x^3 + 8*a^2*b^2*c^10*x^3 + 3*b^4*c^10*x^3 - 7*a^2*c^12*x^3 -      3*b^2*c^12*x^3 + 2*c^14*x^3 + 3*a^8*b^6*x^2*y - 9*a^6*b^8*x^2*y + 9*a^4*b^10*x^2*y -      3*a^2*b^12*x^2*y - 9*a^8*b^4*c^2*x^2*y + 20*a^6*b^6*c^2*x^2*y - 16*a^4*b^8*c^2*x^2*y +      2*a^2*b^10*c^2*x^2*y + 3*b^12*c^2*x^2*y + 9*a^8*b^2*c^4*x^2*y - 6*a^6*b^4*c^4*x^2*y -      3*a^4*b^6*c^4*x^2*y + 9*a^2*b^8*c^4*x^2*y - 11*b^10*c^4*x^2*y - 3*a^8*c^6*x^2*y -      12*a^6*b^2*c^6*x^2*y + 15*a^4*b^4*c^6*x^2*y - 6*a^2*b^6*c^6*x^2*y + 16*b^8*c^6*x^2*y +      7*a^6*c^8*x^2*y - 2*a^4*b^2*c^8*x^2*y - 11*a^2*b^4*c^8*x^2*y - 14*b^6*c^8*x^2*y -      3*a^4*c^10*x^2*y + 12*a^2*b^2*c^10*x^2*y + 11*b^4*c^10*x^2*y - 3*a^2*c^12*x^2*y -      7*b^2*c^12*x^2*y + 2*c^14*x^2*y + 3*a^12*b^2*x*y^2 - 9*a^10*b^4*x*y^2 + 9*a^8*b^6*x*y^2 -      3*a^6*b^8*x*y^2 - 3*a^12*c^2*x*y^2 - 2*a^10*b^2*c^2*x*y^2 + 16*a^8*b^4*c^2*x*y^2 -      20*a^6*b^6*c^2*x*y^2 + 9*a^4*b^8*c^2*x*y^2 + 11*a^10*c^4*x*y^2 - 9*a^8*b^2*c^4*x*y^2 +      3*a^6*b^4*c^4*x*y^2 + 6*a^4*b^6*c^4*x*y^2 - 9*a^2*b^8*c^4*x*y^2 - 16*a^8*c^6*x*y^2 +      6*a^6*b^2*c^6*x*y^2 - 15*a^4*b^4*c^6*x*y^2 + 12*a^2*b^6*c^6*x*y^2 + 3*b^8*c^6*x*y^2 +      14*a^6*c^8*x*y^2 + 11*a^4*b^2*c^8*x*y^2 + 2*a^2*b^4*c^8*x*y^2 - 7*b^6*c^8*x*y^2 -      11*a^4*c^10*x*y^2 - 12*a^2*b^2*c^10*x*y^2 + 3*b^4*c^10*x*y^2 + 7*a^2*c^12*x*y^2 +      3*b^2*c^12*x*y^2 - 2*c^14*x*y^2 + 2*a^14*y^3 - 7*a^12*b^2*y^3 + 9*a^10*b^4*y^3 - 5*a^8*b^6*y^3 +      a^6*b^8*y^3 - 3*a^12*c^2*y^3 + 8*a^10*b^2*c^2*y^3 - 12*a^8*b^4*c^2*y^3 + 10*a^6*b^6*c^2*y^3 -      3*a^4*b^8*c^2*y^3 + 3*a^10*c^4*y^3 + 5*a^8*b^2*c^4*y^3 - 9*a^6*b^4*c^4*y^3 + 3*a^2*b^8*c^4*y^3 -      8*a^8*c^6*y^3 + 9*a^4*b^4*c^6*y^3 - 10*a^2*b^6*c^6*y^3 - b^8*c^6*y^3 + 8*a^6*c^8*y^3 -      5*a^4*b^2*c^8*y^3 + 12*a^2*b^4*c^8*y^3 + 5*b^6*c^8*y^3 - 3*a^4*c^10*y^3 - 8*a^2*b^2*c^10*y^3 -      9*b^4*c^10*y^3 + 3*a^2*c^12*y^3 + 7*b^2*c^12*y^3 - 2*c^14*y^3 + 3*a^8*b^6*x^2*z - 7*a^6*b^8*x^2*z +      3*a^4*b^10*x^2*z + 3*a^2*b^12*x^2*z - 2*b^14*x^2*z - 9*a^8*b^4*c^2*x^2*z + 12*a^6*b^6*c^2*x^2*z +      2*a^4*b^8*c^2*x^2*z - 12*a^2*b^10*c^2*x^2*z + 7*b^12*c^2*x^2*z + 9*a^8*b^2*c^4*x^2*z +      6*a^6*b^4*c^4*x^2*z - 15*a^4*b^6*c^4*x^2*z + 11*a^2*b^8*c^4*x^2*z - 11*b^10*c^4*x^2*z -      3*a^8*c^6*x^2*z - 20*a^6*b^2*c^6*x^2*z + 3*a^4*b^4*c^6*x^2*z + 6*a^2*b^6*c^6*x^2*z +      14*b^8*c^6*x^2*z + 9*a^6*c^8*x^2*z + 16*a^4*b^2*c^8*x^2*z - 9*a^2*b^4*c^8*x^2*z -      16*b^6*c^8*x^2*z - 9*a^4*c^10*x^2*z - 2*a^2*b^2*c^10*x^2*z + 11*b^4*c^10*x^2*z + 3*a^2*c^12*x^2*z -      3*b^2*c^12*x^2*z + 6*a^12*b^2*x*y*z - 20*a^10*b^4*x*y*z + 30*a^8*b^6*x*y*z - 30*a^6*b^8*x*y*z +      20*a^4*b^10*x*y*z - 6*a^2*b^12*x*y*z - 6*a^12*c^2*x*y*z + 10*a^8*b^4*c^2*x*y*z -      10*a^4*b^8*c^2*x*y*z + 6*b^12*c^2*x*y*z + 20*a^10*c^4*x*y*z - 10*a^8*b^2*c^4*x*y*z +      10*a^2*b^8*c^4*x*y*z - 20*b^10*c^4*x*y*z - 30*a^8*c^6*x*y*z + 30*b^8*c^6*x*y*z + 30*a^6*c^8*x*y*z +      10*a^4*b^2*c^8*x*y*z - 10*a^2*b^4*c^8*x*y*z - 30*b^6*c^8*x*y*z - 20*a^4*c^10*x*y*z +      20*b^4*c^10*x*y*z + 6*a^2*c^12*x*y*z - 6*b^2*c^12*x*y*z + 2*a^14*y^2*z - 3*a^12*b^2*y^2*z -      3*a^10*b^4*y^2*z + 7*a^8*b^6*y^2*z - 3*a^6*b^8*y^2*z - 7*a^12*c^2*y^2*z + 12*a^10*b^2*c^2*y^2*z -      2*a^8*b^4*c^2*y^2*z - 12*a^6*b^6*c^2*y^2*z + 9*a^4*b^8*c^2*y^2*z + 11*a^10*c^4*y^2*z -      11*a^8*b^2*c^4*y^2*z + 15*a^6*b^4*c^4*y^2*z - 6*a^4*b^6*c^4*y^2*z - 9*a^2*b^8*c^4*y^2*z -      14*a^8*c^6*y^2*z - 6*a^6*b^2*c^6*y^2*z - 3*a^4*b^4*c^6*y^2*z + 20*a^2*b^6*c^6*y^2*z +      3*b^8*c^6*y^2*z + 16*a^6*c^8*y^2*z + 9*a^4*b^2*c^8*y^2*z - 16*a^2*b^4*c^8*y^2*z - 9*b^6*c^8*y^2*z -      11*a^4*c^10*y^2*z + 2*a^2*b^2*c^10*y^2*z + 9*b^4*c^10*y^2*z + 3*a^2*c^12*y^2*z - 3*b^2*c^12*y^2*z +      3*a^12*b^2*x*z^2 - 11*a^10*b^4*x*z^2 + 16*a^8*b^6*x*z^2 - 14*a^6*b^8*x*z^2 + 11*a^4*b^10*x*z^2 -      7*a^2*b^12*x*z^2 + 2*b^14*x*z^2 - 3*a^12*c^2*x*z^2 + 2*a^10*b^2*c^2*x*z^2 + 9*a^8*b^4*c^2*x*z^2 -      6*a^6*b^6*c^2*x*z^2 - 11*a^4*b^8*c^2*x*z^2 + 12*a^2*b^10*c^2*x*z^2 - 3*b^12*c^2*x*z^2 +      9*a^10*c^4*x*z^2 - 16*a^8*b^2*c^4*x*z^2 - 3*a^6*b^4*c^4*x*z^2 + 15*a^4*b^6*c^4*x*z^2 -      2*a^2*b^8*c^4*x*z^2 - 3*b^10*c^4*x*z^2 - 9*a^8*c^6*x*z^2 + 20*a^6*b^2*c^6*x*z^2 -      6*a^4*b^4*c^6*x*z^2 - 12*a^2*b^6*c^6*x*z^2 + 7*b^8*c^6*x*z^2 + 3*a^6*c^8*x*z^2 -      9*a^4*b^2*c^8*x*z^2 + 9*a^2*b^4*c^8*x*z^2 - 3*b^6*c^8*x*z^2 - 2*a^14*y*z^2 + 7*a^12*b^2*y*z^2 -      11*a^10*b^4*y*z^2 + 14*a^8*b^6*y*z^2 - 16*a^6*b^8*y*z^2 + 11*a^4*b^10*y*z^2 - 3*a^2*b^12*y*z^2 +      3*a^12*c^2*y*z^2 - 12*a^10*b^2*c^2*y*z^2 + 11*a^8*b^4*c^2*y*z^2 + 6*a^6*b^6*c^2*y*z^2 -      9*a^4*b^8*c^2*y*z^2 - 2*a^2*b^10*c^2*y*z^2 + 3*b^12*c^2*y*z^2 + 3*a^10*c^4*y*z^2 +      2*a^8*b^2*c^4*y*z^2 - 15*a^6*b^4*c^4*y*z^2 + 3*a^4*b^6*c^4*y*z^2 + 16*a^2*b^8*c^4*y*z^2 -      9*b^10*c^4*y*z^2 - 7*a^8*c^6*y*z^2 + 12*a^6*b^2*c^6*y*z^2 + 6*a^4*b^4*c^6*y*z^2 -      20*a^2*b^6*c^6*y*z^2 + 9*b^8*c^6*y*z^2 + 3*a^6*c^8*y*z^2 - 9*a^4*b^2*c^8*y*z^2 +      9*a^2*b^4*c^8*y*z^2 - 3*b^6*c^8*y*z^2 - 2*a^14*z^3 + 3*a^12*b^2*z^3 - 3*a^10*b^4*z^3 +      8*a^8*b^6*z^3 - 8*a^6*b^8*z^3 + 3*a^4*b^10*z^3 - 3*a^2*b^12*z^3 + 2*b^14*z^3 + 7*a^12*c^2*z^3 -      8*a^10*b^2*c^2*z^3 - 5*a^8*b^4*c^2*z^3 + 5*a^4*b^8*c^2*z^3 + 8*a^2*b^10*c^2*z^3 - 7*b^12*c^2*z^3 -      9*a^10*c^4*z^3 + 12*a^8*b^2*c^4*z^3 + 9*a^6*b^4*c^4*z^3 - 9*a^4*b^6*c^4*z^3 - 12*a^2*b^8*c^4*z^3 +      9*b^10*c^4*z^3 + 5*a^8*c^6*z^3 - 10*a^6*b^2*c^6*z^3 + 10*a^2*b^6*c^6*z^3 - 5*b^8*c^6*z^3 -      a^6*c^8*z^3 + 3*a^4*b^2*c^8*z^3 - 3*a^2*b^4*c^8*z^3 + b^6*c^8*z^3");
		exp("K594","2*a^3*b*c^2*x^2*y - a^2*b^2*c^2*x^2*y - 2*a*b^3*c^2*x^2*y + b^4*c^2*x^2*y - 2*a^2*b*c^3*x^2*y +      2*a*b^2*c^3*x^2*y - b^2*c^4*x^2*y - a^4*c^2*x*y^2 + 2*a^3*b*c^2*x*y^2 + a^2*b^2*c^2*x*y^2 -      2*a*b^3*c^2*x*y^2 - 2*a^2*b*c^3*x*y^2 + 2*a*b^2*c^3*x*y^2 + a^2*c^4*x*y^2 - 2*a^3*b^2*c*x^2*z +      2*a^2*b^3*c*x^2*z + a^2*b^2*c^2*x^2*z - 2*a*b^3*c^2*x^2*z + b^4*c^2*x^2*z + 2*a*b^2*c^3*x^2*z -      b^2*c^4*x^2*z - 4*a^3*b^2*c*x*y*z + 4*a^2*b^3*c*x*y*z + 4*a^3*b*c^2*x*y*z - 4*a*b^3*c^2*x*y*z -      4*a^2*b*c^3*x*y*z + 4*a*b^2*c^3*x*y*z - 2*a^3*b^2*c*y^2*z + 2*a^2*b^3*c*y^2*z - a^4*c^2*y^2*z +      2*a^3*b*c^2*y^2*z - a^2*b^2*c^2*y^2*z - 2*a^2*b*c^3*y^2*z + a^2*c^4*y^2*z + a^4*b^2*x*z^2 -      a^2*b^4*x*z^2 - 2*a^3*b^2*c*x*z^2 + 2*a^2*b^3*c*x*z^2 - a^2*b^2*c^2*x*z^2 - 2*a*b^3*c^2*x*z^2 +      2*a*b^2*c^3*x*z^2 + a^4*b^2*y*z^2 - a^2*b^4*y*z^2 - 2*a^3*b^2*c*y*z^2 + 2*a^2*b^3*c*y*z^2 +      2*a^3*b*c^2*y*z^2 + a^2*b^2*c^2*y*z^2 - 2*a^2*b*c^3*y*z^2");
		exp("K595","2*a^8*b^2*x^2*y - 5*a^6*b^4*x^2*y + 3*a^4*b^6*x^2*y + a^2*b^8*x^2*y - b^10*x^2*y +      2*a^8*c^2*x^2*y - 4*a^6*b^2*c^2*x^2*y + 2*a^4*b^4*c^2*x^2*y - a^2*b^6*c^2*x^2*y + b^8*c^2*x^2*y -      3*a^6*c^4*x^2*y + 5*a^4*b^2*c^4*x^2*y - a^2*b^4*c^4*x^2*y - 2*b^6*c^4*x^2*y + 2*a^4*c^6*x^2*y -      a^2*b^2*c^6*x^2*y + 2*b^4*c^6*x^2*y - 2*a^2*c^8*x^2*y - b^2*c^8*x^2*y + c^10*x^2*y + a^10*x*y^2 -      a^8*b^2*x*y^2 - 3*a^6*b^4*x*y^2 + 5*a^4*b^6*x*y^2 - 2*a^2*b^8*x*y^2 - a^8*c^2*x*y^2 +      a^6*b^2*c^2*x*y^2 - 2*a^4*b^4*c^2*x*y^2 + 4*a^2*b^6*c^2*x*y^2 - 2*b^8*c^2*x*y^2 + 2*a^6*c^4*x*y^2 +      a^4*b^2*c^4*x*y^2 - 5*a^2*b^4*c^4*x*y^2 + 3*b^6*c^4*x*y^2 - 2*a^4*c^6*x*y^2 + a^2*b^2*c^6*x*y^2 -      2*b^4*c^6*x*y^2 + a^2*c^8*x*y^2 + 2*b^2*c^8*x*y^2 - c^10*x*y^2 - 2*a^8*b^2*x^2*z +      3*a^6*b^4*x^2*z - 2*a^4*b^6*x^2*z + 2*a^2*b^8*x^2*z - b^10*x^2*z - 2*a^8*c^2*x^2*z +      4*a^6*b^2*c^2*x^2*z - 5*a^4*b^4*c^2*x^2*z + a^2*b^6*c^2*x^2*z + b^8*c^2*x^2*z + 5*a^6*c^4*x^2*z -      2*a^4*b^2*c^4*x^2*z + a^2*b^4*c^4*x^2*z - 2*b^6*c^4*x^2*z - 3*a^4*c^6*x^2*z + a^2*b^2*c^6*x^2*z +      2*b^4*c^6*x^2*z - a^2*c^8*x^2*z - b^2*c^8*x^2*z + c^10*x^2*z - 2*a^8*b^2*x*y*z + 4*a^6*b^4*x*y*z -      4*a^4*b^6*x*y*z + 2*a^2*b^8*x*y*z + 2*a^8*c^2*x*y*z - 2*b^8*c^2*x*y*z - 4*a^6*c^4*x*y*z +      4*b^6*c^4*x*y*z + 4*a^4*c^6*x*y*z - 4*b^4*c^6*x*y*z - 2*a^2*c^8*x*y*z + 2*b^2*c^8*x*y*z +      a^10*y^2*z - 2*a^8*b^2*y^2*z + 2*a^6*b^4*y^2*z - 3*a^4*b^6*y^2*z + 2*a^2*b^8*y^2*z -      a^8*c^2*y^2*z - a^6*b^2*c^2*y^2*z + 5*a^4*b^4*c^2*y^2*z - 4*a^2*b^6*c^2*y^2*z + 2*b^8*c^2*y^2*z +      2*a^6*c^4*y^2*z - a^4*b^2*c^4*y^2*z + 2*a^2*b^4*c^4*y^2*z - 5*b^6*c^4*y^2*z - 2*a^4*c^6*y^2*z -      a^2*b^2*c^6*y^2*z + 3*b^4*c^6*y^2*z + a^2*c^8*y^2*z + b^2*c^8*y^2*z - c^10*y^2*z - a^10*x*z^2 +      a^8*b^2*x*z^2 - 2*a^6*b^4*x*z^2 + 2*a^4*b^6*x*z^2 - a^2*b^8*x*z^2 + b^10*x*z^2 + a^8*c^2*x*z^2 -      a^6*b^2*c^2*x*z^2 - a^4*b^4*c^2*x*z^2 - a^2*b^6*c^2*x*z^2 - 2*b^8*c^2*x*z^2 + 3*a^6*c^4*x*z^2 +      2*a^4*b^2*c^4*x*z^2 + 5*a^2*b^4*c^4*x*z^2 + 2*b^6*c^4*x*z^2 - 5*a^4*c^6*x*z^2 -      4*a^2*b^2*c^6*x*z^2 - 3*b^4*c^6*x*z^2 + 2*a^2*c^8*x*z^2 + 2*b^2*c^8*x*z^2 - a^10*y*z^2 +      a^8*b^2*y*z^2 - 2*a^6*b^4*y*z^2 + 2*a^4*b^6*y*z^2 - a^2*b^8*y*z^2 + b^10*y*z^2 + 2*a^8*c^2*y*z^2 +      a^6*b^2*c^2*y*z^2 + a^4*b^4*c^2*y*z^2 + a^2*b^6*c^2*y*z^2 - b^8*c^2*y*z^2 - 2*a^6*c^4*y*z^2 -      5*a^4*b^2*c^4*y*z^2 - 2*a^2*b^4*c^4*y*z^2 - 3*b^6*c^4*y*z^2 + 3*a^4*c^6*y*z^2 +      4*a^2*b^2*c^6*y*z^2 + 5*b^4*c^6*y*z^2 - 2*a^2*c^8*y*z^2 - 2*b^2*c^8*y*z^2");
		exp("K596","2*a^8*b^2*x^2*y - 5*a^6*b^4*x^2*y + 3*a^4*b^6*x^2*y + a^2*b^8*x^2*y - b^10*x^2*y -      2*a^8*c^2*x^2*y + 2*a^4*b^4*c^2*x^2*y - 5*a^2*b^6*c^2*x^2*y + 5*b^8*c^2*x^2*y + 5*a^6*c^4*x^2*y +      a^4*b^2*c^4*x^2*y + 3*a^2*b^4*c^4*x^2*y - 10*b^6*c^4*x^2*y - 6*a^4*c^6*x^2*y - a^2*b^2*c^6*x^2*y +      10*b^4*c^6*x^2*y + 2*a^2*c^8*x^2*y - 5*b^2*c^8*x^2*y + c^10*x^2*y + a^10*x*y^2 - a^8*b^2*x*y^2 -      3*a^6*b^4*x*y^2 + 5*a^4*b^6*x*y^2 - 2*a^2*b^8*x*y^2 - 5*a^8*c^2*x*y^2 + 5*a^6*b^2*c^2*x*y^2 -      2*a^4*b^4*c^2*x*y^2 + 2*b^8*c^2*x*y^2 + 10*a^6*c^4*x*y^2 - 3*a^4*b^2*c^4*x*y^2 -      a^2*b^4*c^4*x*y^2 - 5*b^6*c^4*x*y^2 - 10*a^4*c^6*x*y^2 + a^2*b^2*c^6*x*y^2 + 6*b^4*c^6*x*y^2 +      5*a^2*c^8*x*y^2 - 2*b^2*c^8*x*y^2 - c^10*x*y^2 + 2*a^8*b^2*x^2*z - 5*a^6*b^4*x^2*z +      6*a^4*b^6*x^2*z - 2*a^2*b^8*x^2*z - b^10*x^2*z - 2*a^8*c^2*x^2*z - a^4*b^4*c^2*x^2*z +      a^2*b^6*c^2*x^2*z + 5*b^8*c^2*x^2*z + 5*a^6*c^4*x^2*z - 2*a^4*b^2*c^4*x^2*z - 3*a^2*b^4*c^4*x^2*z -      10*b^6*c^4*x^2*z - 3*a^4*c^6*x^2*z + 5*a^2*b^2*c^6*x^2*z + 10*b^4*c^6*x^2*z - a^2*c^8*x^2*z -      5*b^2*c^8*x^2*z + c^10*x^2*z + 6*a^8*b^2*x*y*z - 12*a^6*b^4*x*y*z + 12*a^4*b^6*x*y*z -      6*a^2*b^8*x*y*z - 6*a^8*c^2*x*y*z + 6*b^8*c^2*x*y*z + 12*a^6*c^4*x*y*z - 12*b^6*c^4*x*y*z -      12*a^4*c^6*x*y*z + 12*b^4*c^6*x*y*z + 6*a^2*c^8*x*y*z - 6*b^2*c^8*x*y*z + a^10*y^2*z +      2*a^8*b^2*y^2*z - 6*a^6*b^4*y^2*z + 5*a^4*b^6*y^2*z - 2*a^2*b^8*y^2*z - 5*a^8*c^2*y^2*z -      a^6*b^2*c^2*y^2*z + a^4*b^4*c^2*y^2*z + 2*b^8*c^2*y^2*z + 10*a^6*c^4*y^2*z + 3*a^4*b^2*c^4*y^2*z +      2*a^2*b^4*c^4*y^2*z - 5*b^6*c^4*y^2*z - 10*a^4*c^6*y^2*z - 5*a^2*b^2*c^6*y^2*z + 3*b^4*c^6*y^2*z +      5*a^2*c^8*y^2*z + b^2*c^8*y^2*z - c^10*y^2*z - a^10*x*z^2 + 5*a^8*b^2*x*z^2 - 10*a^6*b^4*x*z^2 +      10*a^4*b^6*x*z^2 - 5*a^2*b^8*x*z^2 + b^10*x*z^2 + a^8*c^2*x*z^2 - 5*a^6*b^2*c^2*x*z^2 +      3*a^4*b^4*c^2*x*z^2 - a^2*b^6*c^2*x*z^2 + 2*b^8*c^2*x*z^2 + 3*a^6*c^4*x*z^2 + 2*a^4*b^2*c^4*x*z^2 +      a^2*b^4*c^4*x*z^2 - 6*b^6*c^4*x*z^2 - 5*a^4*c^6*x*z^2 + 5*b^4*c^6*x*z^2 + 2*a^2*c^8*x*z^2 -      2*b^2*c^8*x*z^2 - a^10*y*z^2 + 5*a^8*b^2*y*z^2 - 10*a^6*b^4*y*z^2 + 10*a^4*b^6*y*z^2 -      5*a^2*b^8*y*z^2 + b^10*y*z^2 - 2*a^8*c^2*y*z^2 + a^6*b^2*c^2*y*z^2 - 3*a^4*b^4*c^2*y*z^2 +      5*a^2*b^6*c^2*y*z^2 - b^8*c^2*y*z^2 + 6*a^6*c^4*y*z^2 - a^4*b^2*c^4*y*z^2 - 2*a^2*b^4*c^4*y*z^2 -      3*b^6*c^4*y*z^2 - 5*a^4*c^6*y*z^2 + 5*b^4*c^6*y*z^2 + 2*a^2*c^8*y*z^2 - 2*b^2*c^8*y*z^2");
		exp("K597","2*a^16*b^2*x^2*y - 13*a^14*b^4*x^2*y + 35*a^12*b^6*x^2*y - 49*a^10*b^8*x^2*y +      35*a^8*b^10*x^2*y - 7*a^6*b^12*x^2*y - 7*a^4*b^14*x^2*y + 5*a^2*b^16*x^2*y - b^18*x^2*y +      2*a^16*c^2*x^2*y - 2*a^14*b^2*c^2*x^2*y - 8*a^12*b^4*c^2*x^2*y + a^10*b^6*c^2*x^2*y +      45*a^8*b^8*c^2*x^2*y - 72*a^6*b^10*c^2*x^2*y + 42*a^4*b^12*c^2*x^2*y - 7*a^2*b^14*c^2*x^2*y -      b^16*c^2*x^2*y - 5*a^14*c^4*x^2*y + 17*a^12*b^2*c^4*x^2*y + 25*a^10*b^4*c^4*x^2*y -      120*a^8*b^6*c^4*x^2*y + 106*a^6*b^8*c^4*x^2*y + 2*a^4*b^10*c^4*x^2*y - 34*a^2*b^12*c^4*x^2*y +      9*b^14*c^4*x^2*y - 4*a^12*c^6*x^2*y - 41*a^10*b^2*c^6*x^2*y + 86*a^8*b^4*c^6*x^2*y +      36*a^6*b^6*c^6*x^2*y - 142*a^4*b^8*c^6*x^2*y + 66*a^2*b^10*c^6*x^2*y - b^12*c^6*x^2*y +      24*a^10*c^8*x^2*y - a^8*b^2*c^8*x^2*y - 136*a^6*b^4*c^8*x^2*y + 128*a^4*b^6*c^8*x^2*y +      10*a^2*b^8*c^8*x^2*y - 26*b^10*c^8*x^2*y - 25*a^8*c^10*x^2*y + 66*a^6*b^2*c^10*x^2*y +      18*a^4*b^4*c^10*x^2*y - 80*a^2*b^6*c^10*x^2*y + 26*b^8*c^10*x^2*y + 3*a^6*c^12*x^2*y -      51*a^4*b^2*c^12*x^2*y + 37*a^2*b^4*c^12*x^2*y + b^6*c^12*x^2*y + 10*a^4*c^14*x^2*y +      9*a^2*b^2*c^14*x^2*y - 9*b^4*c^14*x^2*y - 6*a^2*c^16*x^2*y + b^2*c^16*x^2*y + c^18*x^2*y +      a^18*x*y^2 - 5*a^16*b^2*x*y^2 + 7*a^14*b^4*x*y^2 + 7*a^12*b^6*x*y^2 - 35*a^10*b^8*x*y^2 +      49*a^8*b^10*x*y^2 - 35*a^6*b^12*x*y^2 + 13*a^4*b^14*x*y^2 - 2*a^2*b^16*x*y^2 + a^16*c^2*x*y^2 +      7*a^14*b^2*c^2*x*y^2 - 42*a^12*b^4*c^2*x*y^2 + 72*a^10*b^6*c^2*x*y^2 - 45*a^8*b^8*c^2*x*y^2 -      a^6*b^10*c^2*x*y^2 + 8*a^4*b^12*c^2*x*y^2 + 2*a^2*b^14*c^2*x*y^2 - 2*b^16*c^2*x*y^2 -      9*a^14*c^4*x*y^2 + 34*a^12*b^2*c^4*x*y^2 - 2*a^10*b^4*c^4*x*y^2 - 106*a^8*b^6*c^4*x*y^2 +      120*a^6*b^8*c^4*x*y^2 - 25*a^4*b^10*c^4*x*y^2 - 17*a^2*b^12*c^4*x*y^2 + 5*b^14*c^4*x*y^2 +      a^12*c^6*x*y^2 - 66*a^10*b^2*c^6*x*y^2 + 142*a^8*b^4*c^6*x*y^2 - 36*a^6*b^6*c^6*x*y^2 -      86*a^4*b^8*c^6*x*y^2 + 41*a^2*b^10*c^6*x*y^2 + 4*b^12*c^6*x*y^2 + 26*a^10*c^8*x*y^2 -      10*a^8*b^2*c^8*x*y^2 - 128*a^6*b^4*c^8*x*y^2 + 136*a^4*b^6*c^8*x*y^2 + a^2*b^8*c^8*x*y^2 -      24*b^10*c^8*x*y^2 - 26*a^8*c^10*x*y^2 + 80*a^6*b^2*c^10*x*y^2 - 18*a^4*b^4*c^10*x*y^2 -      66*a^2*b^6*c^10*x*y^2 + 25*b^8*c^10*x*y^2 - a^6*c^12*x*y^2 - 37*a^4*b^2*c^12*x*y^2 +      51*a^2*b^4*c^12*x*y^2 - 3*b^6*c^12*x*y^2 + 9*a^4*c^14*x*y^2 - 9*a^2*b^2*c^14*x*y^2 -      10*b^4*c^14*x*y^2 - a^2*c^16*x*y^2 + 6*b^2*c^16*x*y^2 - c^18*x*y^2 - 2*a^16*b^2*x^2*z +      5*a^14*b^4*x^2*z + 4*a^12*b^6*x^2*z - 24*a^10*b^8*x^2*z + 25*a^8*b^10*x^2*z - 3*a^6*b^12*x^2*z -      10*a^4*b^14*x^2*z + 6*a^2*b^16*x^2*z - b^18*x^2*z - 2*a^16*c^2*x^2*z + 2*a^14*b^2*c^2*x^2*z -      17*a^12*b^4*c^2*x^2*z + 41*a^10*b^6*c^2*x^2*z + a^8*b^8*c^2*x^2*z - 66*a^6*b^10*c^2*x^2*z +      51*a^4*b^12*c^2*x^2*z - 9*a^2*b^14*c^2*x^2*z - b^16*c^2*x^2*z + 13*a^14*c^4*x^2*z +      8*a^12*b^2*c^4*x^2*z - 25*a^10*b^4*c^4*x^2*z - 86*a^8*b^6*c^4*x^2*z + 136*a^6*b^8*c^4*x^2*z -      18*a^4*b^10*c^4*x^2*z - 37*a^2*b^12*c^4*x^2*z + 9*b^14*c^4*x^2*z - 35*a^12*c^6*x^2*z -      a^10*b^2*c^6*x^2*z + 120*a^8*b^4*c^6*x^2*z - 36*a^6*b^6*c^6*x^2*z - 128*a^4*b^8*c^6*x^2*z +      80*a^2*b^10*c^6*x^2*z - b^12*c^6*x^2*z + 49*a^10*c^8*x^2*z - 45*a^8*b^2*c^8*x^2*z -      106*a^6*b^4*c^8*x^2*z + 142*a^4*b^6*c^8*x^2*z - 10*a^2*b^8*c^8*x^2*z - 26*b^10*c^8*x^2*z -      35*a^8*c^10*x^2*z + 72*a^6*b^2*c^10*x^2*z - 2*a^4*b^4*c^10*x^2*z - 66*a^2*b^6*c^10*x^2*z +      26*b^8*c^10*x^2*z + 7*a^6*c^12*x^2*z - 42*a^4*b^2*c^12*x^2*z + 34*a^2*b^4*c^12*x^2*z +      b^6*c^12*x^2*z + 7*a^4*c^14*x^2*z + 7*a^2*b^2*c^14*x^2*z - 9*b^4*c^14*x^2*z - 5*a^2*c^16*x^2*z +      b^2*c^16*x^2*z + c^18*x^2*z - 2*a^16*b^2*x*y*z + 28*a^12*b^6*x*y*z - 70*a^10*b^8*x*y*z +      70*a^8*b^10*x*y*z - 28*a^6*b^12*x*y*z + 2*a^2*b^16*x*y*z + 2*a^16*c^2*x*y*z -      54*a^12*b^4*c^2*x*y*z + 100*a^10*b^6*c^2*x*y*z - 100*a^6*b^10*c^2*x*y*z + 54*a^4*b^12*c^2*x*y*z -      2*b^16*c^2*x*y*z + 54*a^12*b^2*c^4*x*y*z - 260*a^8*b^6*c^4*x*y*z + 260*a^6*b^8*c^4*x*y*z -      54*a^2*b^12*c^4*x*y*z - 28*a^12*c^6*x*y*z - 100*a^10*b^2*c^6*x*y*z + 260*a^8*b^4*c^6*x*y*z -      260*a^4*b^8*c^6*x*y*z + 100*a^2*b^10*c^6*x*y*z + 28*b^12*c^6*x*y*z + 70*a^10*c^8*x*y*z -      260*a^6*b^4*c^8*x*y*z + 260*a^4*b^6*c^8*x*y*z - 70*b^10*c^8*x*y*z - 70*a^8*c^10*x*y*z +      100*a^6*b^2*c^10*x*y*z - 100*a^2*b^6*c^10*x*y*z + 70*b^8*c^10*x*y*z + 28*a^6*c^12*x*y*z -      54*a^4*b^2*c^12*x*y*z + 54*a^2*b^4*c^12*x*y*z - 28*b^6*c^12*x*y*z - 2*a^2*c^16*x*y*z +      2*b^2*c^16*x*y*z + a^18*y^2*z - 6*a^16*b^2*y^2*z + 10*a^14*b^4*y^2*z + 3*a^12*b^6*y^2*z -      25*a^10*b^8*y^2*z + 24*a^8*b^10*y^2*z - 4*a^6*b^12*y^2*z - 5*a^4*b^14*y^2*z + 2*a^2*b^16*y^2*z +      a^16*c^2*y^2*z + 9*a^14*b^2*c^2*y^2*z - 51*a^12*b^4*c^2*y^2*z + 66*a^10*b^6*c^2*y^2*z -      a^8*b^8*c^2*y^2*z - 41*a^6*b^10*c^2*y^2*z + 17*a^4*b^12*c^2*y^2*z - 2*a^2*b^14*c^2*y^2*z +      2*b^16*c^2*y^2*z - 9*a^14*c^4*y^2*z + 37*a^12*b^2*c^4*y^2*z + 18*a^10*b^4*c^4*y^2*z -      136*a^8*b^6*c^4*y^2*z + 86*a^6*b^8*c^4*y^2*z + 25*a^4*b^10*c^4*y^2*z - 8*a^2*b^12*c^4*y^2*z -      13*b^14*c^4*y^2*z + a^12*c^6*y^2*z - 80*a^10*b^2*c^6*y^2*z + 128*a^8*b^4*c^6*y^2*z +      36*a^6*b^6*c^6*y^2*z - 120*a^4*b^8*c^6*y^2*z + a^2*b^10*c^6*y^2*z + 35*b^12*c^6*y^2*z +      26*a^10*c^8*y^2*z + 10*a^8*b^2*c^8*y^2*z - 142*a^6*b^4*c^8*y^2*z + 106*a^4*b^6*c^8*y^2*z +      45*a^2*b^8*c^8*y^2*z - 49*b^10*c^8*y^2*z - 26*a^8*c^10*y^2*z + 66*a^6*b^2*c^10*y^2*z +      2*a^4*b^4*c^10*y^2*z - 72*a^2*b^6*c^10*y^2*z + 35*b^8*c^10*y^2*z - a^6*c^12*y^2*z -      34*a^4*b^2*c^12*y^2*z + 42*a^2*b^4*c^12*y^2*z - 7*b^6*c^12*y^2*z + 9*a^4*c^14*y^2*z -      7*a^2*b^2*c^14*y^2*z - 7*b^4*c^14*y^2*z - a^2*c^16*y^2*z + 5*b^2*c^16*y^2*z - c^18*y^2*z -      a^18*x*z^2 - a^16*b^2*x*z^2 + 9*a^14*b^4*x*z^2 - a^12*b^6*x*z^2 - 26*a^10*b^8*x*z^2 +      26*a^8*b^10*x*z^2 + a^6*b^12*x*z^2 - 9*a^4*b^14*x*z^2 + a^2*b^16*x*z^2 + b^18*x*z^2 +      5*a^16*c^2*x*z^2 - 7*a^14*b^2*c^2*x*z^2 - 34*a^12*b^4*c^2*x*z^2 + 66*a^10*b^6*c^2*x*z^2 +      10*a^8*b^8*c^2*x*z^2 - 80*a^6*b^10*c^2*x*z^2 + 37*a^4*b^12*c^2*x*z^2 + 9*a^2*b^14*c^2*x*z^2 -      6*b^16*c^2*x*z^2 - 7*a^14*c^4*x*z^2 + 42*a^12*b^2*c^4*x*z^2 + 2*a^10*b^4*c^4*x*z^2 -      142*a^8*b^6*c^4*x*z^2 + 128*a^6*b^8*c^4*x*z^2 + 18*a^4*b^10*c^4*x*z^2 - 51*a^2*b^12*c^4*x*z^2 +      10*b^14*c^4*x*z^2 - 7*a^12*c^6*x*z^2 - 72*a^10*b^2*c^6*x*z^2 + 106*a^8*b^4*c^6*x*z^2 +      36*a^6*b^6*c^6*x*z^2 - 136*a^4*b^8*c^6*x*z^2 + 66*a^2*b^10*c^6*x*z^2 + 3*b^12*c^6*x*z^2 +      35*a^10*c^8*x*z^2 + 45*a^8*b^2*c^8*x*z^2 - 120*a^6*b^4*c^8*x*z^2 + 86*a^4*b^6*c^8*x*z^2 -      a^2*b^8*c^8*x*z^2 - 25*b^10*c^8*x*z^2 - 49*a^8*c^10*x*z^2 + a^6*b^2*c^10*x*z^2 +      25*a^4*b^4*c^10*x*z^2 - 41*a^2*b^6*c^10*x*z^2 + 24*b^8*c^10*x*z^2 + 35*a^6*c^12*x*z^2 -      8*a^4*b^2*c^12*x*z^2 + 17*a^2*b^4*c^12*x*z^2 - 4*b^6*c^12*x*z^2 - 13*a^4*c^14*x*z^2 -      2*a^2*b^2*c^14*x*z^2 - 5*b^4*c^14*x*z^2 + 2*a^2*c^16*x*z^2 + 2*b^2*c^16*x*z^2 - a^18*y*z^2 -      a^16*b^2*y*z^2 + 9*a^14*b^4*y*z^2 - a^12*b^6*y*z^2 - 26*a^10*b^8*y*z^2 + 26*a^8*b^10*y*z^2 +      a^6*b^12*y*z^2 - 9*a^4*b^14*y*z^2 + a^2*b^16*y*z^2 + b^18*y*z^2 + 6*a^16*c^2*y*z^2 -      9*a^14*b^2*c^2*y*z^2 - 37*a^12*b^4*c^2*y*z^2 + 80*a^10*b^6*c^2*y*z^2 - 10*a^8*b^8*c^2*y*z^2 -      66*a^6*b^10*c^2*y*z^2 + 34*a^4*b^12*c^2*y*z^2 + 7*a^2*b^14*c^2*y*z^2 - 5*b^16*c^2*y*z^2 -      10*a^14*c^4*y*z^2 + 51*a^12*b^2*c^4*y*z^2 - 18*a^10*b^4*c^4*y*z^2 - 128*a^8*b^6*c^4*y*z^2 +      142*a^6*b^8*c^4*y*z^2 - 2*a^4*b^10*c^4*y*z^2 - 42*a^2*b^12*c^4*y*z^2 + 7*b^14*c^4*y*z^2 -      3*a^12*c^6*y*z^2 - 66*a^10*b^2*c^6*y*z^2 + 136*a^8*b^4*c^6*y*z^2 - 36*a^6*b^6*c^6*y*z^2 -      106*a^4*b^8*c^6*y*z^2 + 72*a^2*b^10*c^6*y*z^2 + 7*b^12*c^6*y*z^2 + 25*a^10*c^8*y*z^2 +      a^8*b^2*c^8*y*z^2 - 86*a^6*b^4*c^8*y*z^2 + 120*a^4*b^6*c^8*y*z^2 - 45*a^2*b^8*c^8*y*z^2 -      35*b^10*c^8*y*z^2 - 24*a^8*c^10*y*z^2 + 41*a^6*b^2*c^10*y*z^2 - 25*a^4*b^4*c^10*y*z^2 -      a^2*b^6*c^10*y*z^2 + 49*b^8*c^10*y*z^2 + 4*a^6*c^12*y*z^2 - 17*a^4*b^2*c^12*y*z^2 +      8*a^2*b^4*c^12*y*z^2 - 35*b^6*c^12*y*z^2 + 5*a^4*c^14*y*z^2 + 2*a^2*b^2*c^14*y*z^2 +      13*b^4*c^14*y*z^2 - 2*a^2*c^16*y*z^2 - 2*b^2*c^16*y*z^2");
		exp("K598","(-a^2)*b^2*c^2*x^3 + b^4*c^2*x^3 + b^2*c^4*x^3 + a^4*c^2*y^3 - a^2*b^2*c^2*y^3 + a^2*c^4*y^3 +      6*a^2*b^2*c^2*x*y*z + a^4*b^2*z^3 + a^2*b^4*z^3 - a^2*b^2*c^2*z^3");
		exp("K599","a^4*c^2*x^2*y - 2*a^2*b^2*c^2*x^2*y + b^4*c^2*x^2*y - 3*a^2*c^4*x^2*y + b^2*c^4*x^2*y +      2*c^6*x^2*y - a^4*c^2*x*y^2 + 2*a^2*b^2*c^2*x*y^2 - b^4*c^2*x*y^2 - a^2*c^4*x*y^2 +      3*b^2*c^4*x*y^2 - 2*c^6*x*y^2 - a^4*b^2*x^2*z + 3*a^2*b^4*x^2*z - 2*b^6*x^2*z +      2*a^2*b^2*c^2*x^2*z - b^4*c^2*x^2*z - b^2*c^4*x^2*z - 2*a^4*b^2*x*y*z + 2*a^2*b^4*x*y*z +      2*a^4*c^2*x*y*z - 2*b^4*c^2*x*y*z - 2*a^2*c^4*x*y*z + 2*b^2*c^4*x*y*z + 2*a^6*y^2*z -      3*a^4*b^2*y^2*z + a^2*b^4*y^2*z + a^4*c^2*y^2*z - 2*a^2*b^2*c^2*y^2*z + a^2*c^4*y^2*z +      a^4*b^2*x*z^2 + a^2*b^4*x*z^2 + 2*b^6*x*z^2 - 2*a^2*b^2*c^2*x*z^2 - 3*b^4*c^2*x*z^2 +      b^2*c^4*x*z^2 - 2*a^6*y*z^2 - a^4*b^2*y*z^2 - a^2*b^4*y*z^2 + 3*a^4*c^2*y*z^2 +      2*a^2*b^2*c^2*y*z^2 - a^2*c^4*y*z^2");
		exp("K600","a^6*x^3 - 2*a^4*b^2*x^3 + a^2*b^4*x^3 - 2*a^4*c^2*x^3 + 2*a^2*b^2*c^2*x^3 + a^2*c^4*x^3 -     a^6*x^2*y + 3*a^2*b^4*x^2*y - 2*b^6*x^2*y + 3*a^4*c^2*x^2*y - 2*a^2*b^2*c^2*x^2*y - b^4*c^2*x^2*y -     3*a^2*c^4*x^2*y + 2*b^2*c^4*x^2*y + c^6*x^2*y - 2*a^6*x*y^2 + 3*a^4*b^2*x*y^2 - b^6*x*y^2 -     a^4*c^2*x*y^2 - 2*a^2*b^2*c^2*x*y^2 + 3*b^4*c^2*x*y^2 + 2*a^2*c^4*x*y^2 - 3*b^2*c^4*x*y^2 +     c^6*x*y^2 + a^4*b^2*y^3 - 2*a^2*b^4*y^3 + b^6*y^3 + 2*a^2*b^2*c^2*y^3 - 2*b^4*c^2*y^3 +     b^2*c^4*y^3 - a^6*x^2*z + 3*a^4*b^2*x^2*z - 3*a^2*b^4*x^2*z + b^6*x^2*z - 2*a^2*b^2*c^2*x^2*z +     2*b^4*c^2*x^2*z + 3*a^2*c^4*x^2*z - b^2*c^4*x^2*z - 2*c^6*x^2*z + 3*a^6*x*y*z - 3*a^4*b^2*x*y*z -     3*a^2*b^4*x*y*z + 3*b^6*x*y*z - 3*a^4*c^2*x*y*z + 6*a^2*b^2*c^2*x*y*z - 3*b^4*c^2*x*y*z -     3*a^2*c^4*x*y*z - 3*b^2*c^4*x*y*z + 3*c^6*x*y*z + a^6*y^2*z - 3*a^4*b^2*y^2*z + 3*a^2*b^4*y^2*z -     b^6*y^2*z + 2*a^4*c^2*y^2*z - 2*a^2*b^2*c^2*y^2*z - a^2*c^4*y^2*z + 3*b^2*c^4*y^2*z - 2*c^6*y^2*z -     2*a^6*x*z^2 - a^4*b^2*x*z^2 + 2*a^2*b^4*x*z^2 + b^6*x*z^2 + 3*a^4*c^2*x*z^2 - 2*a^2*b^2*c^2*x*z^2 -     3*b^4*c^2*x*z^2 + 3*b^2*c^4*x*z^2 - c^6*x*z^2 + a^6*y*z^2 + 2*a^4*b^2*y*z^2 - a^2*b^4*y*z^2 -     2*b^6*y*z^2 - 3*a^4*c^2*y*z^2 - 2*a^2*b^2*c^2*y*z^2 + 3*b^4*c^2*y*z^2 + 3*a^2*c^4*y*z^2 -     c^6*y*z^2 + a^4*c^2*z^3 + 2*a^2*b^2*c^2*z^3 + b^4*c^2*z^3 - 2*a^2*c^4*z^3 - 2*b^2*c^4*z^3 +     c^6*z^3");
		exp("K601","(-a^18)*x^2*y + 4*a^16*b^2*x^2*y - 3*a^14*b^4*x^2*y - 10*a^12*b^6*x^2*y + 25*a^10*b^8*x^2*y -     24*a^8*b^10*x^2*y + 11*a^6*b^12*x^2*y - 2*a^4*b^14*x^2*y + 2*a^16*c^2*x^2*y -     14*a^14*b^2*c^2*x^2*y + 33*a^12*b^4*c^2*x^2*y - 28*a^10*b^6*c^2*x^2*y - 8*a^8*b^8*c^2*x^2*y +     30*a^6*b^10*c^2*x^2*y - 19*a^4*b^12*c^2*x^2*y + 4*a^2*b^14*c^2*x^2*y + 2*a^14*c^4*x^2*y +     4*a^12*b^2*c^4*x^2*y - 39*a^10*b^4*c^4*x^2*y + 80*a^8*b^6*c^4*x^2*y - 68*a^6*b^8*c^4*x^2*y +     18*a^4*b^10*c^4*x^2*y + 5*a^2*b^12*c^4*x^2*y - 2*b^14*c^4*x^2*y - 7*a^12*c^6*x^2*y +     22*a^10*b^2*c^6*x^2*y - 30*a^8*b^4*c^6*x^2*y - 4*a^6*b^6*c^6*x^2*y + 46*a^4*b^8*c^6*x^2*y -     30*a^2*b^10*c^6*x^2*y + 3*b^12*c^6*x^2*y + 5*a^10*c^8*x^2*y - 8*a^8*b^2*c^8*x^2*y +     42*a^6*b^4*c^8*x^2*y - 64*a^4*b^6*c^8*x^2*y + 19*a^2*b^8*c^8*x^2*y + 6*b^10*c^8*x^2*y -     4*a^8*c^10*x^2*y - 20*a^6*b^2*c^10*x^2*y + 18*a^4*b^4*c^10*x^2*y + 20*a^2*b^6*c^10*x^2*y -     14*b^8*c^10*x^2*y + 8*a^6*c^12*x^2*y + 10*a^4*b^2*c^12*x^2*y - 24*a^2*b^4*c^12*x^2*y +     6*b^6*c^12*x^2*y - 7*a^4*c^14*x^2*y + 4*a^2*b^2*c^14*x^2*y + 3*b^4*c^14*x^2*y + 2*a^2*c^16*x^2*y -     2*b^2*c^16*x^2*y - 2*a^14*b^4*x*y^2 + 11*a^12*b^6*x*y^2 - 24*a^10*b^8*x*y^2 + 25*a^8*b^10*x*y^2 -     10*a^6*b^12*x*y^2 - 3*a^4*b^14*x*y^2 + 4*a^2*b^16*x*y^2 - b^18*x*y^2 + 4*a^14*b^2*c^2*x*y^2 -     19*a^12*b^4*c^2*x*y^2 + 30*a^10*b^6*c^2*x*y^2 - 8*a^8*b^8*c^2*x*y^2 - 28*a^6*b^10*c^2*x*y^2 +     33*a^4*b^12*c^2*x*y^2 - 14*a^2*b^14*c^2*x*y^2 + 2*b^16*c^2*x*y^2 - 2*a^14*c^4*x*y^2 +     5*a^12*b^2*c^4*x*y^2 + 18*a^10*b^4*c^4*x*y^2 - 68*a^8*b^6*c^4*x*y^2 + 80*a^6*b^8*c^4*x*y^2 -     39*a^4*b^10*c^4*x*y^2 + 4*a^2*b^12*c^4*x*y^2 + 2*b^14*c^4*x*y^2 + 3*a^12*c^6*x*y^2 -     30*a^10*b^2*c^6*x*y^2 + 46*a^8*b^4*c^6*x*y^2 - 4*a^6*b^6*c^6*x*y^2 - 30*a^4*b^8*c^6*x*y^2 +     22*a^2*b^10*c^6*x*y^2 - 7*b^12*c^6*x*y^2 + 6*a^10*c^8*x*y^2 + 19*a^8*b^2*c^8*x*y^2 -     64*a^6*b^4*c^8*x*y^2 + 42*a^4*b^6*c^8*x*y^2 - 8*a^2*b^8*c^8*x*y^2 + 5*b^10*c^8*x*y^2 -     14*a^8*c^10*x*y^2 + 20*a^6*b^2*c^10*x*y^2 + 18*a^4*b^4*c^10*x*y^2 - 20*a^2*b^6*c^10*x*y^2 -     4*b^8*c^10*x*y^2 + 6*a^6*c^12*x*y^2 - 24*a^4*b^2*c^12*x*y^2 + 10*a^2*b^4*c^12*x*y^2 +     8*b^6*c^12*x*y^2 + 3*a^4*c^14*x*y^2 + 4*a^2*b^2*c^14*x*y^2 - 7*b^4*c^14*x*y^2 - 2*a^2*c^16*x*y^2 +     2*b^2*c^16*x*y^2 - a^18*x^2*z + 2*a^16*b^2*x^2*z + 2*a^14*b^4*x^2*z - 7*a^12*b^6*x^2*z +     5*a^10*b^8*x^2*z - 4*a^8*b^10*x^2*z + 8*a^6*b^12*x^2*z - 7*a^4*b^14*x^2*z + 2*a^2*b^16*x^2*z +     4*a^16*c^2*x^2*z - 14*a^14*b^2*c^2*x^2*z + 4*a^12*b^4*c^2*x^2*z + 22*a^10*b^6*c^2*x^2*z -     8*a^8*b^8*c^2*x^2*z - 20*a^6*b^10*c^2*x^2*z + 10*a^4*b^12*c^2*x^2*z + 4*a^2*b^14*c^2*x^2*z -     2*b^16*c^2*x^2*z - 3*a^14*c^4*x^2*z + 33*a^12*b^2*c^4*x^2*z - 39*a^10*b^4*c^4*x^2*z -     30*a^8*b^6*c^4*x^2*z + 42*a^6*b^8*c^4*x^2*z + 18*a^4*b^10*c^4*x^2*z - 24*a^2*b^12*c^4*x^2*z +     3*b^14*c^4*x^2*z - 10*a^12*c^6*x^2*z - 28*a^10*b^2*c^6*x^2*z + 80*a^8*b^4*c^6*x^2*z -     4*a^6*b^6*c^6*x^2*z - 64*a^4*b^8*c^6*x^2*z + 20*a^2*b^10*c^6*x^2*z + 6*b^12*c^6*x^2*z +     25*a^10*c^8*x^2*z - 8*a^8*b^2*c^8*x^2*z - 68*a^6*b^4*c^8*x^2*z + 46*a^4*b^6*c^8*x^2*z +     19*a^2*b^8*c^8*x^2*z - 14*b^10*c^8*x^2*z - 24*a^8*c^10*x^2*z + 30*a^6*b^2*c^10*x^2*z +     18*a^4*b^4*c^10*x^2*z - 30*a^2*b^6*c^10*x^2*z + 6*b^8*c^10*x^2*z + 11*a^6*c^12*x^2*z -     19*a^4*b^2*c^12*x^2*z + 5*a^2*b^4*c^12*x^2*z + 3*b^6*c^12*x^2*z - 2*a^4*c^14*x^2*z +     4*a^2*b^2*c^14*x^2*z - 2*b^4*c^14*x^2*z + 2*a^16*b^2*y^2*z - 7*a^14*b^4*y^2*z + 8*a^12*b^6*y^2*z -     4*a^10*b^8*y^2*z + 5*a^8*b^10*y^2*z - 7*a^6*b^12*y^2*z + 2*a^4*b^14*y^2*z + 2*a^2*b^16*y^2*z -     b^18*y^2*z - 2*a^16*c^2*y^2*z + 4*a^14*b^2*c^2*y^2*z + 10*a^12*b^4*c^2*y^2*z -     20*a^10*b^6*c^2*y^2*z - 8*a^8*b^8*c^2*y^2*z + 22*a^6*b^10*c^2*y^2*z + 4*a^4*b^12*c^2*y^2*z -     14*a^2*b^14*c^2*y^2*z + 4*b^16*c^2*y^2*z + 3*a^14*c^4*y^2*z - 24*a^12*b^2*c^4*y^2*z +     18*a^10*b^4*c^4*y^2*z + 42*a^8*b^6*c^4*y^2*z - 30*a^6*b^8*c^4*y^2*z - 39*a^4*b^10*c^4*y^2*z +     33*a^2*b^12*c^4*y^2*z - 3*b^14*c^4*y^2*z + 6*a^12*c^6*y^2*z + 20*a^10*b^2*c^6*y^2*z -     64*a^8*b^4*c^6*y^2*z - 4*a^6*b^6*c^6*y^2*z + 80*a^4*b^8*c^6*y^2*z - 28*a^2*b^10*c^6*y^2*z -     10*b^12*c^6*y^2*z - 14*a^10*c^8*y^2*z + 19*a^8*b^2*c^8*y^2*z + 46*a^6*b^4*c^8*y^2*z -     68*a^4*b^6*c^8*y^2*z - 8*a^2*b^8*c^8*y^2*z + 25*b^10*c^8*y^2*z + 6*a^8*c^10*y^2*z -     30*a^6*b^2*c^10*y^2*z + 18*a^4*b^4*c^10*y^2*z + 30*a^2*b^6*c^10*y^2*z - 24*b^8*c^10*y^2*z +     3*a^6*c^12*y^2*z + 5*a^4*b^2*c^12*y^2*z - 19*a^2*b^4*c^12*y^2*z + 11*b^6*c^12*y^2*z -     2*a^4*c^14*y^2*z + 4*a^2*b^2*c^14*y^2*z - 2*b^4*c^14*y^2*z - 2*a^14*b^4*x*z^2 + 3*a^12*b^6*x*z^2 +     6*a^10*b^8*x*z^2 - 14*a^8*b^10*x*z^2 + 6*a^6*b^12*x*z^2 + 3*a^4*b^14*x*z^2 - 2*a^2*b^16*x*z^2 +     4*a^14*b^2*c^2*x*z^2 + 5*a^12*b^4*c^2*x*z^2 - 30*a^10*b^6*c^2*x*z^2 + 19*a^8*b^8*c^2*x*z^2 +     20*a^6*b^10*c^2*x*z^2 - 24*a^4*b^12*c^2*x*z^2 + 4*a^2*b^14*c^2*x*z^2 + 2*b^16*c^2*x*z^2 -     2*a^14*c^4*x*z^2 - 19*a^12*b^2*c^4*x*z^2 + 18*a^10*b^4*c^4*x*z^2 + 46*a^8*b^6*c^4*x*z^2 -     64*a^6*b^8*c^4*x*z^2 + 18*a^4*b^10*c^4*x*z^2 + 10*a^2*b^12*c^4*x*z^2 - 7*b^14*c^4*x*z^2 +     11*a^12*c^6*x*z^2 + 30*a^10*b^2*c^6*x*z^2 - 68*a^8*b^4*c^6*x*z^2 - 4*a^6*b^6*c^6*x*z^2 +     42*a^4*b^8*c^6*x*z^2 - 20*a^2*b^10*c^6*x*z^2 + 8*b^12*c^6*x*z^2 - 24*a^10*c^8*x*z^2 -     8*a^8*b^2*c^8*x*z^2 + 80*a^6*b^4*c^8*x*z^2 - 30*a^4*b^6*c^8*x*z^2 - 8*a^2*b^8*c^8*x*z^2 -     4*b^10*c^8*x*z^2 + 25*a^8*c^10*x*z^2 - 28*a^6*b^2*c^10*x*z^2 - 39*a^4*b^4*c^10*x*z^2 +     22*a^2*b^6*c^10*x*z^2 + 5*b^8*c^10*x*z^2 - 10*a^6*c^12*x*z^2 + 33*a^4*b^2*c^12*x*z^2 +     4*a^2*b^4*c^12*x*z^2 - 7*b^6*c^12*x*z^2 - 3*a^4*c^14*x*z^2 - 14*a^2*b^2*c^14*x*z^2 +     2*b^4*c^14*x*z^2 + 4*a^2*c^16*x*z^2 + 2*b^2*c^16*x*z^2 - c^18*x*z^2 - 2*a^16*b^2*y*z^2 +     3*a^14*b^4*y*z^2 + 6*a^12*b^6*y*z^2 - 14*a^10*b^8*y*z^2 + 6*a^8*b^10*y*z^2 + 3*a^6*b^12*y*z^2 -     2*a^4*b^14*y*z^2 + 2*a^16*c^2*y*z^2 + 4*a^14*b^2*c^2*y*z^2 - 24*a^12*b^4*c^2*y*z^2 +     20*a^10*b^6*c^2*y*z^2 + 19*a^8*b^8*c^2*y*z^2 - 30*a^6*b^10*c^2*y*z^2 + 5*a^4*b^12*c^2*y*z^2 +     4*a^2*b^14*c^2*y*z^2 - 7*a^14*c^4*y*z^2 + 10*a^12*b^2*c^4*y*z^2 + 18*a^10*b^4*c^4*y*z^2 -     64*a^8*b^6*c^4*y*z^2 + 46*a^6*b^8*c^4*y*z^2 + 18*a^4*b^10*c^4*y*z^2 - 19*a^2*b^12*c^4*y*z^2 -     2*b^14*c^4*y*z^2 + 8*a^12*c^6*y*z^2 - 20*a^10*b^2*c^6*y*z^2 + 42*a^8*b^4*c^6*y*z^2 -     4*a^6*b^6*c^6*y*z^2 - 68*a^4*b^8*c^6*y*z^2 + 30*a^2*b^10*c^6*y*z^2 + 11*b^12*c^6*y*z^2 -     4*a^10*c^8*y*z^2 - 8*a^8*b^2*c^8*y*z^2 - 30*a^6*b^4*c^8*y*z^2 + 80*a^4*b^6*c^8*y*z^2 -     8*a^2*b^8*c^8*y*z^2 - 24*b^10*c^8*y*z^2 + 5*a^8*c^10*y*z^2 + 22*a^6*b^2*c^10*y*z^2 -     39*a^4*b^4*c^10*y*z^2 - 28*a^2*b^6*c^10*y*z^2 + 25*b^8*c^10*y*z^2 - 7*a^6*c^12*y*z^2 +     4*a^4*b^2*c^12*y*z^2 + 33*a^2*b^4*c^12*y*z^2 - 10*b^6*c^12*y*z^2 + 2*a^4*c^14*y*z^2 -     14*a^2*b^2*c^14*y*z^2 - 3*b^4*c^14*y*z^2 + 2*a^2*c^16*y*z^2 + 4*b^2*c^16*y*z^2 - c^18*y*z^2");
		exp("K602","a^4*b^2*x^2*y + b^6*x^2*y - a^4*c^2*x^2*y - 2*a^2*b^2*c^2*x^2*y - 3*b^4*c^2*x^2*y +     2*a^2*c^4*x^2*y + 4*b^2*c^4*x^2*y - 2*c^6*x^2*y + a^6*x*y^2 + a^2*b^4*x*y^2 - 3*a^4*c^2*x*y^2 -     2*a^2*b^2*c^2*x*y^2 - b^4*c^2*x*y^2 + 4*a^2*c^4*x*y^2 + 2*b^2*c^4*x*y^2 - 2*c^6*x*y^2 -     a^4*b^2*x^2*z + 2*a^2*b^4*x^2*z - 2*b^6*x^2*z + a^4*c^2*x^2*z - 2*a^2*b^2*c^2*x^2*z +     4*b^4*c^2*x^2*z - 3*b^2*c^4*x^2*z + c^6*x^2*z - 2*a^6*y^2*z + 2*a^4*b^2*y^2*z - a^2*b^4*y^2*z +     4*a^4*c^2*y^2*z - 2*a^2*b^2*c^2*y^2*z + b^4*c^2*y^2*z - 3*a^2*c^4*y^2*z + c^6*y^2*z + a^6*x*z^2 -     3*a^4*b^2*x*z^2 + 4*a^2*b^4*x*z^2 - 2*b^6*x*z^2 - 2*a^2*b^2*c^2*x*z^2 + 2*b^4*c^2*x*z^2 +     a^2*c^4*x*z^2 - b^2*c^4*x*z^2 - 2*a^6*y*z^2 + 4*a^4*b^2*y*z^2 - 3*a^2*b^4*y*z^2 + b^6*y*z^2 +     2*a^4*c^2*y*z^2 - 2*a^2*b^2*c^2*y*z^2 - a^2*c^4*y*z^2 + b^2*c^4*y*z^2");
		exp("K603","a^3*b^2*c*x^2*y - a^2*b^3*c*x^2*y - a*b^4*c*x^2*y + b^5*c*x^2*y - a^3*b*c^2*x^2*y +     5*a*b^3*c^2*x^2*y - 2*b^4*c^2*x^2*y + a^2*b*c^3*x^2*y - 6*a*b^2*c^3*x^2*y - b^3*c^3*x^2*y +     2*a*b*c^4*x^2*y + 4*b^2*c^4*x^2*y - 2*b*c^5*x^2*y + a^5*c*x*y^2 - a^4*b*c*x*y^2 - a^3*b^2*c*x*y^2 +     a^2*b^3*c*x*y^2 - 2*a^4*c^2*x*y^2 + 5*a^3*b*c^2*x*y^2 - a*b^3*c^2*x*y^2 - a^3*c^3*x*y^2 -     6*a^2*b*c^3*x*y^2 + a*b^2*c^3*x*y^2 + 4*a^2*c^4*x*y^2 + 2*a*b*c^4*x*y^2 - 2*a*c^5*x*y^2 -     a^3*b^2*c*x^2*z + a^2*b^3*c*x^2*z + 2*a*b^4*c*x^2*z - 2*b^5*c*x^2*z + a^3*b*c^2*x^2*z -     6*a*b^3*c^2*x^2*z + 4*b^4*c^2*x^2*z - a^2*b*c^3*x^2*z + 5*a*b^2*c^3*x^2*z - b^3*c^3*x^2*z -     a*b*c^4*x^2*z - 2*b^2*c^4*x^2*z + b*c^5*x^2*z - 2*a^5*c*y^2*z + 2*a^4*b*c*y^2*z + a^3*b^2*c*y^2*z -     a^2*b^3*c*y^2*z + 4*a^4*c^2*y^2*z - 6*a^3*b*c^2*y^2*z + a*b^3*c^2*y^2*z - a^3*c^3*y^2*z +     5*a^2*b*c^3*y^2*z - a*b^2*c^3*y^2*z - 2*a^2*c^4*y^2*z - a*b*c^4*y^2*z + a*c^5*y^2*z + a^5*b*x*z^2 -     2*a^4*b^2*x*z^2 - a^3*b^3*x*z^2 + 4*a^2*b^4*x*z^2 - 2*a*b^5*x*z^2 - a^4*b*c*x*z^2 +     5*a^3*b^2*c*x*z^2 - 6*a^2*b^3*c*x*z^2 + 2*a*b^4*c*x*z^2 - a^3*b*c^2*x*z^2 + a*b^3*c^2*x*z^2 +     a^2*b*c^3*x*z^2 - a*b^2*c^3*x*z^2 - 2*a^5*b*y*z^2 + 4*a^4*b^2*y*z^2 - a^3*b^3*y*z^2 -     2*a^2*b^4*y*z^2 + a*b^5*y*z^2 + 2*a^4*b*c*y*z^2 - 6*a^3*b^2*c*y*z^2 + 5*a^2*b^3*c*y*z^2 -     a*b^4*c*y*z^2 + a^3*b*c^2*y*z^2 - a*b^3*c^2*y*z^2 - a^2*b*c^3*y*z^2 + a*b^2*c^3*y*z^2");
		exp("K604","(-a^2)*b^2*c^2*x^2*y - b^4*c^2*x^2*y + 2*b^2*c^4*x^2*y - a^4*c^2*x*y^2 - a^2*b^2*c^2*x*y^2 +     2*a^2*c^4*x*y^2 - a^2*b^2*c^2*x^2*z + 2*b^4*c^2*x^2*z - b^2*c^4*x^2*z + 2*a^4*c^2*y^2*z -     a^2*b^2*c^2*y^2*z - a^2*c^4*y^2*z - a^4*b^2*x*z^2 + 2*a^2*b^4*x*z^2 - a^2*b^2*c^2*x*z^2 +     2*a^4*b^2*y*z^2 - a^2*b^4*y*z^2 - a^2*b^2*c^2*y*z^2");
		exp("K605","a^3*c*x^2*y - a*b^2*c*x^2*y + a*c^3*x^2*y + a^2*b*c*x*y^2 - b^3*c*x*y^2 - b*c^3*x*y^2 -     a^3*b*x^2*z - a*b^3*x^2*z + a*b*c^2*x^2*z + a^3*b*y^2*z + a*b^3*y^2*z - a*b*c^2*y^2*z -     a^2*b*c*x*z^2 + b^3*c*x*z^2 + b*c^3*x*z^2 - a^3*c*y*z^2 + a*b^2*c*y*z^2 - a*c^3*y*z^2");
		exp("K606","a^4*x^2*y - 2*a^2*b^2*x^2*y + b^4*x^2*y + a^2*c^2*x^2*y - b^2*c^2*x^2*y - a^4*x*y^2 +     2*a^2*b^2*x*y^2 - b^4*x*y^2 + a^2*c^2*x*y^2 - b^2*c^2*x*y^2 - a^4*x^2*z - a^2*b^2*x^2*z +     2*a^2*c^2*x^2*z + b^2*c^2*x^2*z - c^4*x^2*z + a^2*b^2*y^2*z + b^4*y^2*z - a^2*c^2*y^2*z -     2*b^2*c^2*y^2*z + c^4*y^2*z + a^4*x*z^2 - a^2*b^2*x*z^2 - 2*a^2*c^2*x*z^2 + b^2*c^2*x*z^2 +     c^4*x*z^2 + a^2*b^2*y*z^2 - b^4*y*z^2 - a^2*c^2*y*z^2 + 2*b^2*c^2*y*z^2 - c^4*y*z^2");
		exp("K607","a^18*x^2*y - 8*a^16*b^2*x^2*y + 28*a^14*b^4*x^2*y - 56*a^12*b^6*x^2*y + 70*a^10*b^8*x^2*y -     56*a^8*b^10*x^2*y + 28*a^6*b^12*x^2*y - 8*a^4*b^14*x^2*y + a^2*b^16*x^2*y + 6*a^14*b^2*c^2*x^2*y -     28*a^12*b^4*c^2*x^2*y + 51*a^10*b^6*c^2*x^2*y - 45*a^8*b^8*c^2*x^2*y + 20*a^6*b^10*c^2*x^2*y -     6*a^4*b^12*c^2*x^2*y + 3*a^2*b^14*c^2*x^2*y - b^16*c^2*x^2*y - 9*a^14*c^4*x^2*y +     29*a^12*b^2*c^4*x^2*y - 36*a^10*b^4*c^4*x^2*y + 21*a^8*b^6*c^4*x^2*y - 7*a^6*b^8*c^4*x^2*y +     9*a^4*b^10*c^4*x^2*y - 12*a^2*b^12*c^4*x^2*y + 5*b^14*c^4*x^2*y + 15*a^12*c^6*x^2*y -     41*a^10*b^2*c^6*x^2*y + 52*a^8*b^4*c^6*x^2*y - 36*a^6*b^6*c^6*x^2*y + 4*a^4*b^8*c^6*x^2*y +     16*a^2*b^10*c^6*x^2*y - 10*b^12*c^6*x^2*y - 9*a^10*c^8*x^2*y + 3*a^8*b^2*c^8*x^2*y -     7*a^6*b^4*c^8*x^2*y + 15*a^4*b^6*c^8*x^2*y - 12*a^2*b^8*c^8*x^2*y + 11*b^10*c^8*x^2*y +     9*a^8*c^10*x^2*y + 20*a^6*b^2*c^10*x^2*y - 18*a^4*b^4*c^10*x^2*y - 6*a^2*b^6*c^10*x^2*y -     10*b^8*c^10*x^2*y - 15*a^6*c^12*x^2*y - 5*a^4*b^2*c^12*x^2*y + 19*a^2*b^4*c^12*x^2*y +     11*b^6*c^12*x^2*y + 9*a^4*c^14*x^2*y - 9*a^2*b^2*c^14*x^2*y - 10*b^4*c^14*x^2*y +     5*b^2*c^16*x^2*y - c^18*x^2*y - a^16*b^2*x*y^2 + 8*a^14*b^4*x*y^2 - 28*a^12*b^6*x*y^2 +     56*a^10*b^8*x*y^2 - 70*a^8*b^10*x*y^2 + 56*a^6*b^12*x*y^2 - 28*a^4*b^14*x*y^2 + 8*a^2*b^16*x*y^2 -     b^18*x*y^2 + a^16*c^2*x*y^2 - 3*a^14*b^2*c^2*x*y^2 + 6*a^12*b^4*c^2*x*y^2 - 20*a^10*b^6*c^2*x*y^2 +     45*a^8*b^8*c^2*x*y^2 - 51*a^6*b^10*c^2*x*y^2 + 28*a^4*b^12*c^2*x*y^2 - 6*a^2*b^14*c^2*x*y^2 -     5*a^14*c^4*x*y^2 + 12*a^12*b^2*c^4*x*y^2 - 9*a^10*b^4*c^4*x*y^2 + 7*a^8*b^6*c^4*x*y^2 -     21*a^6*b^8*c^4*x*y^2 + 36*a^4*b^10*c^4*x*y^2 - 29*a^2*b^12*c^4*x*y^2 + 9*b^14*c^4*x*y^2 +     10*a^12*c^6*x*y^2 - 16*a^10*b^2*c^6*x*y^2 - 4*a^8*b^4*c^6*x*y^2 + 36*a^6*b^6*c^6*x*y^2 -     52*a^4*b^8*c^6*x*y^2 + 41*a^2*b^10*c^6*x*y^2 - 15*b^12*c^6*x*y^2 - 11*a^10*c^8*x*y^2 +     12*a^8*b^2*c^8*x*y^2 - 15*a^6*b^4*c^8*x*y^2 + 7*a^4*b^6*c^8*x*y^2 - 3*a^2*b^8*c^8*x*y^2 +     9*b^10*c^8*x*y^2 + 10*a^8*c^10*x*y^2 + 6*a^6*b^2*c^10*x*y^2 + 18*a^4*b^4*c^10*x*y^2 -     20*a^2*b^6*c^10*x*y^2 - 9*b^8*c^10*x*y^2 - 11*a^6*c^12*x*y^2 - 19*a^4*b^2*c^12*x*y^2 +     5*a^2*b^4*c^12*x*y^2 + 15*b^6*c^12*x*y^2 + 10*a^4*c^14*x*y^2 + 9*a^2*b^2*c^14*x*y^2 -     9*b^4*c^14*x*y^2 - 5*a^2*c^16*x*y^2 + c^18*x*y^2 - a^18*x^2*z + 9*a^14*b^4*x^2*z -     15*a^12*b^6*x^2*z + 9*a^10*b^8*x^2*z - 9*a^8*b^10*x^2*z + 15*a^6*b^12*x^2*z - 9*a^4*b^14*x^2*z +     b^18*x^2*z + 8*a^16*c^2*x^2*z - 6*a^14*b^2*c^2*x^2*z - 29*a^12*b^4*c^2*x^2*z +     41*a^10*b^6*c^2*x^2*z - 3*a^8*b^8*c^2*x^2*z - 20*a^6*b^10*c^2*x^2*z + 5*a^4*b^12*c^2*x^2*z +     9*a^2*b^14*c^2*x^2*z - 5*b^16*c^2*x^2*z - 28*a^14*c^4*x^2*z + 28*a^12*b^2*c^4*x^2*z +     36*a^10*b^4*c^4*x^2*z - 52*a^8*b^6*c^4*x^2*z + 7*a^6*b^8*c^4*x^2*z + 18*a^4*b^10*c^4*x^2*z -     19*a^2*b^12*c^4*x^2*z + 10*b^14*c^4*x^2*z + 56*a^12*c^6*x^2*z - 51*a^10*b^2*c^6*x^2*z -     21*a^8*b^4*c^6*x^2*z + 36*a^6*b^6*c^6*x^2*z - 15*a^4*b^8*c^6*x^2*z + 6*a^2*b^10*c^6*x^2*z -     11*b^12*c^6*x^2*z - 70*a^10*c^8*x^2*z + 45*a^8*b^2*c^8*x^2*z + 7*a^6*b^4*c^8*x^2*z -     4*a^4*b^6*c^8*x^2*z + 12*a^2*b^8*c^8*x^2*z + 10*b^10*c^8*x^2*z + 56*a^8*c^10*x^2*z -     20*a^6*b^2*c^10*x^2*z - 9*a^4*b^4*c^10*x^2*z - 16*a^2*b^6*c^10*x^2*z - 11*b^8*c^10*x^2*z -     28*a^6*c^12*x^2*z + 6*a^4*b^2*c^12*x^2*z + 12*a^2*b^4*c^12*x^2*z + 10*b^6*c^12*x^2*z +     8*a^4*c^14*x^2*z - 3*a^2*b^2*c^14*x^2*z - 5*b^4*c^14*x^2*z - a^2*c^16*x^2*z + b^2*c^16*x^2*z -     12*a^16*b^2*x*y*z + 48*a^14*b^4*x*y*z - 72*a^12*b^6*x*y*z + 60*a^10*b^8*x*y*z - 60*a^8*b^10*x*y*z +     72*a^6*b^12*x*y*z - 48*a^4*b^14*x*y*z + 12*a^2*b^16*x*y*z + 12*a^16*c^2*x*y*z -     84*a^12*b^4*c^2*x*y*z + 120*a^10*b^6*c^2*x*y*z - 120*a^6*b^10*c^2*x*y*z + 84*a^4*b^12*c^2*x*y*z -     12*b^16*c^2*x*y*z - 48*a^14*c^4*x*y*z + 84*a^12*b^2*c^4*x*y*z - 120*a^8*b^6*c^4*x*y*z +     120*a^6*b^8*c^4*x*y*z - 84*a^2*b^12*c^4*x*y*z + 48*b^14*c^4*x*y*z + 72*a^12*c^6*x*y*z -     120*a^10*b^2*c^6*x*y*z + 120*a^8*b^4*c^6*x*y*z - 120*a^4*b^8*c^6*x*y*z + 120*a^2*b^10*c^6*x*y*z -     72*b^12*c^6*x*y*z - 60*a^10*c^8*x*y*z - 120*a^6*b^4*c^8*x*y*z + 120*a^4*b^6*c^8*x*y*z +     60*b^10*c^8*x*y*z + 60*a^8*c^10*x*y*z + 120*a^6*b^2*c^10*x*y*z - 120*a^2*b^6*c^10*x*y*z -     60*b^8*c^10*x*y*z - 72*a^6*c^12*x*y*z - 84*a^4*b^2*c^12*x*y*z + 84*a^2*b^4*c^12*x*y*z +     72*b^6*c^12*x*y*z + 48*a^4*c^14*x*y*z - 48*b^4*c^14*x*y*z - 12*a^2*c^16*x*y*z + 12*b^2*c^16*x*y*z -     a^18*y^2*z + 9*a^14*b^4*y^2*z - 15*a^12*b^6*y^2*z + 9*a^10*b^8*y^2*z - 9*a^8*b^10*y^2*z +     15*a^6*b^12*y^2*z - 9*a^4*b^14*y^2*z + b^18*y^2*z + 5*a^16*c^2*y^2*z - 9*a^14*b^2*c^2*y^2*z -     5*a^12*b^4*c^2*y^2*z + 20*a^10*b^6*c^2*y^2*z + 3*a^8*b^8*c^2*y^2*z - 41*a^6*b^10*c^2*y^2*z +     29*a^4*b^12*c^2*y^2*z + 6*a^2*b^14*c^2*y^2*z - 8*b^16*c^2*y^2*z - 10*a^14*c^4*y^2*z +     19*a^12*b^2*c^4*y^2*z - 18*a^10*b^4*c^4*y^2*z - 7*a^8*b^6*c^4*y^2*z + 52*a^6*b^8*c^4*y^2*z -     36*a^4*b^10*c^4*y^2*z - 28*a^2*b^12*c^4*y^2*z + 28*b^14*c^4*y^2*z + 11*a^12*c^6*y^2*z -     6*a^10*b^2*c^6*y^2*z + 15*a^8*b^4*c^6*y^2*z - 36*a^6*b^6*c^6*y^2*z + 21*a^4*b^8*c^6*y^2*z +     51*a^2*b^10*c^6*y^2*z - 56*b^12*c^6*y^2*z - 10*a^10*c^8*y^2*z - 12*a^8*b^2*c^8*y^2*z +     4*a^6*b^4*c^8*y^2*z - 7*a^4*b^6*c^8*y^2*z - 45*a^2*b^8*c^8*y^2*z + 70*b^10*c^8*y^2*z +     11*a^8*c^10*y^2*z + 16*a^6*b^2*c^10*y^2*z + 9*a^4*b^4*c^10*y^2*z + 20*a^2*b^6*c^10*y^2*z -     56*b^8*c^10*y^2*z - 10*a^6*c^12*y^2*z - 12*a^4*b^2*c^12*y^2*z - 6*a^2*b^4*c^12*y^2*z +     28*b^6*c^12*y^2*z + 5*a^4*c^14*y^2*z + 3*a^2*b^2*c^14*y^2*z - 8*b^4*c^14*y^2*z - a^2*c^16*y^2*z +     b^2*c^16*y^2*z - a^16*b^2*x*z^2 + 5*a^14*b^4*x*z^2 - 10*a^12*b^6*x*z^2 + 11*a^10*b^8*x*z^2 -     10*a^8*b^10*x*z^2 + 11*a^6*b^12*x*z^2 - 10*a^4*b^14*x*z^2 + 5*a^2*b^16*x*z^2 - b^18*x*z^2 +     a^16*c^2*x*z^2 + 3*a^14*b^2*c^2*x*z^2 - 12*a^12*b^4*c^2*x*z^2 + 16*a^10*b^6*c^2*x*z^2 -     12*a^8*b^8*c^2*x*z^2 - 6*a^6*b^10*c^2*x*z^2 + 19*a^4*b^12*c^2*x*z^2 - 9*a^2*b^14*c^2*x*z^2 -     8*a^14*c^4*x*z^2 - 6*a^12*b^2*c^4*x*z^2 + 9*a^10*b^4*c^4*x*z^2 + 4*a^8*b^6*c^4*x*z^2 +     15*a^6*b^8*c^4*x*z^2 - 18*a^4*b^10*c^4*x*z^2 - 5*a^2*b^12*c^4*x*z^2 + 9*b^14*c^4*x*z^2 +     28*a^12*c^6*x*z^2 + 20*a^10*b^2*c^6*x*z^2 - 7*a^8*b^4*c^6*x*z^2 - 36*a^6*b^6*c^6*x*z^2 -     7*a^4*b^8*c^6*x*z^2 + 20*a^2*b^10*c^6*x*z^2 - 15*b^12*c^6*x*z^2 - 56*a^10*c^8*x*z^2 -     45*a^8*b^2*c^8*x*z^2 + 21*a^6*b^4*c^8*x*z^2 + 52*a^4*b^6*c^8*x*z^2 + 3*a^2*b^8*c^8*x*z^2 +     9*b^10*c^8*x*z^2 + 70*a^8*c^10*x*z^2 + 51*a^6*b^2*c^10*x*z^2 - 36*a^4*b^4*c^10*x*z^2 -     41*a^2*b^6*c^10*x*z^2 - 9*b^8*c^10*x*z^2 - 56*a^6*c^12*x*z^2 - 28*a^4*b^2*c^12*x*z^2 +     29*a^2*b^4*c^12*x*z^2 + 15*b^6*c^12*x*z^2 + 28*a^4*c^14*x*z^2 + 6*a^2*b^2*c^14*x*z^2 -     9*b^4*c^14*x*z^2 - 8*a^2*c^16*x*z^2 + c^18*x*z^2 + a^18*y*z^2 - 5*a^16*b^2*y*z^2 +     10*a^14*b^4*y*z^2 - 11*a^12*b^6*y*z^2 + 10*a^10*b^8*y*z^2 - 11*a^8*b^10*y*z^2 + 10*a^6*b^12*y*z^2 -     5*a^4*b^14*y*z^2 + a^2*b^16*y*z^2 + 9*a^14*b^2*c^2*y*z^2 - 19*a^12*b^4*c^2*y*z^2 +     6*a^10*b^6*c^2*y*z^2 + 12*a^8*b^8*c^2*y*z^2 - 16*a^6*b^10*c^2*y*z^2 + 12*a^4*b^12*c^2*y*z^2 -     3*a^2*b^14*c^2*y*z^2 - b^16*c^2*y*z^2 - 9*a^14*c^4*y*z^2 + 5*a^12*b^2*c^4*y*z^2 +     18*a^10*b^4*c^4*y*z^2 - 15*a^8*b^6*c^4*y*z^2 - 4*a^6*b^8*c^4*y*z^2 - 9*a^4*b^10*c^4*y*z^2 +     6*a^2*b^12*c^4*y*z^2 + 8*b^14*c^4*y*z^2 + 15*a^12*c^6*y*z^2 - 20*a^10*b^2*c^6*y*z^2 +     7*a^8*b^4*c^6*y*z^2 + 36*a^6*b^6*c^6*y*z^2 + 7*a^4*b^8*c^6*y*z^2 - 20*a^2*b^10*c^6*y*z^2 -     28*b^12*c^6*y*z^2 - 9*a^10*c^8*y*z^2 - 3*a^8*b^2*c^8*y*z^2 - 52*a^6*b^4*c^8*y*z^2 -     21*a^4*b^6*c^8*y*z^2 + 45*a^2*b^8*c^8*y*z^2 + 56*b^10*c^8*y*z^2 + 9*a^8*c^10*y*z^2 +     41*a^6*b^2*c^10*y*z^2 + 36*a^4*b^4*c^10*y*z^2 - 51*a^2*b^6*c^10*y*z^2 - 70*b^8*c^10*y*z^2 -     15*a^6*c^12*y*z^2 - 29*a^4*b^2*c^12*y*z^2 + 28*a^2*b^4*c^12*y*z^2 + 56*b^6*c^12*y*z^2 +     9*a^4*c^14*y*z^2 - 6*a^2*b^2*c^14*y*z^2 - 28*b^4*c^14*y*z^2 + 8*b^2*c^16*y*z^2 - c^18*y*z^2");
		exp("K608","a^8*b^2*x^2*y - 2*a^6*b^4*x^2*y + 2*a^2*b^8*x^2*y - b^10*x^2*y - 2*a^6*b^2*c^2*x^2*y +     6*a^4*b^4*c^2*x^2*y - 6*a^2*b^6*c^2*x^2*y + 2*b^8*c^2*x^2*y - a^4*b^2*c^4*x^2*y + b^6*c^4*x^2*y +     2*a^2*b^2*c^6*x^2*y - 2*b^4*c^6*x^2*y + a^10*x*y^2 - 2*a^8*b^2*x*y^2 + 2*a^4*b^6*x*y^2 -     a^2*b^8*x*y^2 - 2*a^8*c^2*x*y^2 + 6*a^6*b^2*c^2*x*y^2 - 6*a^4*b^4*c^2*x*y^2 + 2*a^2*b^6*c^2*x*y^2 -     a^6*c^4*x*y^2 + a^2*b^4*c^4*x*y^2 + 2*a^4*c^6*x*y^2 - 2*a^2*b^2*c^6*x*y^2 - a^8*c^2*x^2*z +     2*a^6*b^2*c^2*x^2*z + a^4*b^4*c^2*x^2*z - 2*a^2*b^6*c^2*x^2*z + 2*a^6*c^4*x^2*z -     6*a^4*b^2*c^4*x^2*z + 2*b^6*c^4*x^2*z + 6*a^2*b^2*c^6*x^2*z - b^4*c^6*x^2*z - 2*a^2*c^8*x^2*z -     2*b^2*c^8*x^2*z + c^10*x^2*z + 2*a^6*b^2*c^2*y^2*z - a^4*b^4*c^2*y^2*z - 2*a^2*b^6*c^2*y^2*z +     b^8*c^2*y^2*z - 2*a^6*c^4*y^2*z + 6*a^2*b^4*c^4*y^2*z - 2*b^6*c^4*y^2*z + a^4*c^6*y^2*z -     6*a^2*b^2*c^6*y^2*z + 2*a^2*c^8*y^2*z + 2*b^2*c^8*y^2*z - c^10*y^2*z - a^10*x*z^2 +     2*a^8*b^2*x*z^2 + a^6*b^4*x*z^2 - 2*a^4*b^6*x*z^2 + 2*a^8*c^2*x*z^2 - 6*a^6*b^2*c^2*x*z^2 +     2*a^2*b^6*c^2*x*z^2 + 6*a^4*b^2*c^4*x*z^2 - a^2*b^4*c^4*x*z^2 - 2*a^4*c^6*x*z^2 -     2*a^2*b^2*c^6*x*z^2 + a^2*c^8*x*z^2 + 2*a^6*b^4*y*z^2 - a^4*b^6*y*z^2 - 2*a^2*b^8*y*z^2 +     b^10*y*z^2 - 2*a^6*b^2*c^2*y*z^2 + 6*a^2*b^6*c^2*y*z^2 - 2*b^8*c^2*y*z^2 + a^4*b^2*c^4*y*z^2 -     6*a^2*b^4*c^4*y*z^2 + 2*a^2*b^2*c^6*y*z^2 + 2*b^4*c^6*y*z^2 - b^2*c^8*y*z^2");
		exp("K609","4*b^4*c^2*x^3 - 4*b^2*c^4*x^3 + a^4*c^2*x^2*y + 4*a^2*b^2*c^2*x^2*y - b^4*c^2*x^2*y -     2*a^2*c^4*x^2*y + c^6*x^2*y + a^4*c^2*x*y^2 - 4*a^2*b^2*c^2*x*y^2 - b^4*c^2*x*y^2 +     2*b^2*c^4*x*y^2 - c^6*x*y^2 - 4*a^4*c^2*y^3 + 4*a^2*c^4*y^3 - a^4*b^2*x^2*z + 2*a^2*b^4*x^2*z -     b^6*x^2*z - 4*a^2*b^2*c^2*x^2*z + b^2*c^4*x^2*z + a^6*y^2*z - 2*a^4*b^2*y^2*z + a^2*b^4*y^2*z +     4*a^2*b^2*c^2*y^2*z - a^2*c^4*y^2*z - a^4*b^2*x*z^2 + b^6*x*z^2 + 4*a^2*b^2*c^2*x*z^2 -     2*b^4*c^2*x*z^2 + b^2*c^4*x*z^2 - a^6*y*z^2 + a^2*b^4*y*z^2 + 2*a^4*c^2*y*z^2 -     4*a^2*b^2*c^2*y*z^2 - a^2*c^4*y*z^2 + 4*a^4*b^2*z^3 - 4*a^2*b^4*z^3");
		exp("K610","a^6*b*c*x^2*y + a^5*b^2*c*x^2*y - a^2*b^5*c*x^2*y - a*b^6*c*x^2*y + a^6*c^2*x^2*y +     a^5*b*c^2*x^2*y - a^2*b^4*c^2*x^2*y - a*b^5*c^2*x^2*y - 2*a^4*b*c^3*x^2*y - 2*a^3*b^2*c^3*x^2*y -     2*a^4*c^4*x^2*y - 2*a^3*b*c^4*x^2*y + a^2*b*c^5*x^2*y + a*b^2*c^5*x^2*y + a^2*c^6*x^2*y +     a*b*c^6*x^2*y + a^6*b*c*x*y^2 + a^5*b^2*c*x*y^2 - a^2*b^5*c*x*y^2 - a*b^6*c*x*y^2 +     a^5*b*c^2*x*y^2 + a^4*b^2*c^2*x*y^2 - a*b^5*c^2*x*y^2 - b^6*c^2*x*y^2 + 2*a^2*b^3*c^3*x*y^2 +     2*a*b^4*c^3*x*y^2 + 2*a*b^3*c^4*x*y^2 + 2*b^4*c^4*x*y^2 - a^2*b*c^5*x*y^2 - a*b^2*c^5*x*y^2 -     a*b*c^6*x*y^2 - b^2*c^6*x*y^2 - a^6*b^2*x^2*z + 2*a^4*b^4*x^2*z - a^2*b^6*x^2*z - a^6*b*c*x^2*z -     a^5*b^2*c*x^2*z + 2*a^4*b^3*c*x^2*z + 2*a^3*b^4*c*x^2*z - a^2*b^5*c*x^2*z - a*b^6*c*x^2*z -     a^5*b*c^2*x^2*z + 2*a^3*b^3*c^2*x^2*z - a*b^5*c^2*x^2*z + a^2*b^2*c^4*x^2*z + a^2*b*c^5*x^2*z +     a*b^2*c^5*x^2*z + a*b*c^6*x^2*z + a^6*b^2*y^2*z - 2*a^4*b^4*y^2*z + a^2*b^6*y^2*z + a^6*b*c*y^2*z +     a^5*b^2*c*y^2*z - 2*a^4*b^3*c*y^2*z - 2*a^3*b^4*c*y^2*z + a^2*b^5*c*y^2*z + a*b^6*c*y^2*z +     a^5*b*c^2*y^2*z - 2*a^3*b^3*c^2*y^2*z + a*b^5*c^2*y^2*z - a^2*b^2*c^4*y^2*z - a^2*b*c^5*y^2*z -     a*b^2*c^5*y^2*z - a*b*c^6*y^2*z - a^6*b*c*x*z^2 - a^5*b^2*c*x*z^2 + a^2*b^5*c*x*z^2 +     a*b^6*c*x*z^2 - a^5*b*c^2*x*z^2 - a^4*b^2*c^2*x*z^2 + a*b^5*c^2*x*z^2 + b^6*c^2*x*z^2 -     2*a^2*b^3*c^3*x*z^2 - 2*a*b^4*c^3*x*z^2 - 2*a*b^3*c^4*x*z^2 - 2*b^4*c^4*x*z^2 + a^2*b*c^5*x*z^2 +     a*b^2*c^5*x*z^2 + a*b*c^6*x*z^2 + b^2*c^6*x*z^2 - a^6*b*c*y*z^2 - a^5*b^2*c*y*z^2 +     a^2*b^5*c*y*z^2 + a*b^6*c*y*z^2 - a^6*c^2*y*z^2 - a^5*b*c^2*y*z^2 + a^2*b^4*c^2*y*z^2 +     a*b^5*c^2*y*z^2 + 2*a^4*b*c^3*y*z^2 + 2*a^3*b^2*c^3*y*z^2 + 2*a^4*c^4*y*z^2 + 2*a^3*b*c^4*y*z^2 -     a^2*b*c^5*y*z^2 - a*b^2*c^5*y*z^2 - a^2*c^6*y*z^2 - a*b*c^6*y*z^2");
		exp("K611","a^8*x^2*y - 2*a^6*b^2*x^2*y + 2*a^2*b^6*x^2*y - b^8*x^2*y - a^6*c^2*x^2*y + 2*a^4*b^2*c^2*x^2*y -     3*a^2*b^4*c^2*x^2*y + 2*b^6*c^2*x^2*y + 2*a^2*b^2*c^4*x^2*y - a^2*c^6*x^2*y - 2*b^2*c^6*x^2*y +     c^8*x^2*y + a^8*x*y^2 - 2*a^6*b^2*x*y^2 + 2*a^2*b^6*x*y^2 - b^8*x*y^2 - 2*a^6*c^2*x*y^2 +     3*a^4*b^2*c^2*x*y^2 - 2*a^2*b^4*c^2*x*y^2 + b^6*c^2*x*y^2 - 2*a^2*b^2*c^4*x*y^2 + 2*a^2*c^6*x*y^2 +     b^2*c^6*x*y^2 - c^8*x*y^2 - a^8*x^2*z + a^6*b^2*x^2*z + a^2*b^6*x^2*z - b^8*x^2*z +     2*a^6*c^2*x^2*z - 2*a^4*b^2*c^2*x^2*z - 2*a^2*b^4*c^2*x^2*z + 2*b^6*c^2*x^2*z +     3*a^2*b^2*c^4*x^2*z - 2*a^2*c^6*x^2*z - 2*b^2*c^6*x^2*z + c^8*x^2*z + a^8*y^2*z - a^6*b^2*y^2*z -     a^2*b^6*y^2*z + b^8*y^2*z - 2*a^6*c^2*y^2*z + 2*a^4*b^2*c^2*y^2*z + 2*a^2*b^4*c^2*y^2*z -     2*b^6*c^2*y^2*z - 3*a^2*b^2*c^4*y^2*z + 2*a^2*c^6*y^2*z + 2*b^2*c^6*y^2*z - c^8*y^2*z - a^8*x*z^2 +     2*a^6*b^2*x*z^2 - 2*a^2*b^6*x*z^2 + b^8*x*z^2 + 2*a^6*c^2*x*z^2 - 3*a^4*b^2*c^2*x*z^2 +     2*a^2*b^4*c^2*x*z^2 - b^6*c^2*x*z^2 + 2*a^2*b^2*c^4*x*z^2 - 2*a^2*c^6*x*z^2 - b^2*c^6*x*z^2 +     c^8*x*z^2 - a^8*y*z^2 + 2*a^6*b^2*y*z^2 - 2*a^2*b^6*y*z^2 + b^8*y*z^2 + a^6*c^2*y*z^2 -     2*a^4*b^2*c^2*y*z^2 + 3*a^2*b^4*c^2*y*z^2 - 2*b^6*c^2*y*z^2 - 2*a^2*b^2*c^4*y*z^2 + a^2*c^6*y*z^2 +     2*b^2*c^6*y*z^2 - c^8*y*z^2");
		exp("K612","(-a^6)*c^2*x^2*y + a^4*b^2*c^2*x^2*y + a^2*b^4*c^2*x^2*y - b^6*c^2*x^2*y + 2*a^4*c^4*x^2*y +     2*b^4*c^4*x^2*y - a^2*c^6*x^2*y - b^2*c^6*x^2*y + a^6*c^2*x*y^2 - a^4*b^2*c^2*x*y^2 -     a^2*b^4*c^2*x*y^2 + b^6*c^2*x*y^2 - 2*a^4*c^4*x*y^2 - 2*b^4*c^4*x*y^2 + a^2*c^6*x*y^2 +     b^2*c^6*x*y^2 + a^6*b^2*x^2*z - 2*a^4*b^4*x^2*z + a^2*b^6*x^2*z - a^4*b^2*c^2*x^2*z +     b^6*c^2*x^2*z - a^2*b^2*c^4*x^2*z - 2*b^4*c^4*x^2*z + b^2*c^6*x^2*z - a^6*b^2*y^2*z +     2*a^4*b^4*y^2*z - a^2*b^6*y^2*z - a^6*c^2*y^2*z + a^2*b^4*c^2*y^2*z + 2*a^4*c^4*y^2*z +     a^2*b^2*c^4*y^2*z - a^2*c^6*y^2*z - a^6*b^2*x*z^2 + 2*a^4*b^4*x*z^2 - a^2*b^6*x*z^2 +     a^4*b^2*c^2*x*z^2 - b^6*c^2*x*z^2 + a^2*b^2*c^4*x*z^2 + 2*b^4*c^4*x*z^2 - b^2*c^6*x*z^2 +     a^6*b^2*y*z^2 - 2*a^4*b^4*y*z^2 + a^2*b^6*y*z^2 + a^6*c^2*y*z^2 - a^2*b^4*c^2*y*z^2 -     2*a^4*c^4*y*z^2 - a^2*b^2*c^4*y*z^2 + a^2*c^6*y*z^2");
		exp("K613","(-a^6)*b^4*c^2*x^2*y + 3*a^4*b^6*c^2*x^2*y - 3*a^2*b^8*c^2*x^2*y + b^10*c^2*x^2*y +     a^6*b^2*c^4*x^2*y - 3*a^4*b^4*c^4*x^2*y + 4*a^2*b^6*c^4*x^2*y - 2*b^8*c^4*x^2*y -     a^2*b^4*c^6*x^2*y + 2*b^4*c^8*x^2*y - b^2*c^10*x^2*y - a^10*c^2*x*y^2 + 3*a^8*b^2*c^2*x*y^2 -     3*a^6*b^4*c^2*x*y^2 + a^4*b^6*c^2*x*y^2 + 2*a^8*c^4*x*y^2 - 4*a^6*b^2*c^4*x*y^2 +     3*a^4*b^4*c^4*x*y^2 - a^2*b^6*c^4*x*y^2 + a^4*b^2*c^6*x*y^2 - 2*a^4*c^8*x*y^2 + a^2*c^10*x*y^2 -     a^6*b^4*c^2*x^2*z + b^10*c^2*x^2*z + a^6*b^2*c^4*x^2*z + 3*a^4*b^4*c^4*x^2*z + a^2*b^6*c^4*x^2*z -     2*b^8*c^4*x^2*z - 3*a^4*b^2*c^6*x^2*z - 4*a^2*b^4*c^6*x^2*z + 3*a^2*b^2*c^8*x^2*z +     2*b^4*c^8*x^2*z - b^2*c^10*x^2*z - 6*a^6*b^4*c^2*x*y*z + 6*a^4*b^6*c^2*x*y*z +     6*a^6*b^2*c^4*x*y*z - 6*a^2*b^6*c^4*x*y*z - 6*a^4*b^2*c^6*x*y*z + 6*a^2*b^4*c^6*x*y*z -     a^10*c^2*y^2*z + a^4*b^6*c^2*y^2*z + 2*a^8*c^4*y^2*z - a^6*b^2*c^4*y^2*z - 3*a^4*b^4*c^4*y^2*z -     a^2*b^6*c^4*y^2*z + 4*a^4*b^2*c^6*y^2*z + 3*a^2*b^4*c^6*y^2*z - 2*a^4*c^8*y^2*z -     3*a^2*b^2*c^8*y^2*z + a^2*c^10*y^2*z + a^10*b^2*x*z^2 - 2*a^8*b^4*x*z^2 + 2*a^4*b^8*x*z^2 -     a^2*b^10*x*z^2 - 3*a^8*b^2*c^2*x*z^2 + 4*a^6*b^4*c^2*x*z^2 - a^4*b^6*c^2*x*z^2 +     3*a^6*b^2*c^4*x*z^2 - 3*a^4*b^4*c^4*x*z^2 - a^4*b^2*c^6*x*z^2 + a^2*b^4*c^6*x*z^2 +     a^10*b^2*y*z^2 - 2*a^8*b^4*y*z^2 + 2*a^4*b^8*y*z^2 - a^2*b^10*y*z^2 + a^6*b^4*c^2*y*z^2 -     4*a^4*b^6*c^2*y*z^2 + 3*a^2*b^8*c^2*y*z^2 + 3*a^4*b^4*c^4*y*z^2 - 3*a^2*b^6*c^4*y*z^2 -     a^4*b^2*c^6*y*z^2 + a^2*b^4*c^6*y*z^2");
		exp("K614","a^10*c^2*x^2*y - a^8*b^2*c^2*x^2*y - 4*a^4*b^6*c^2*x^2*y + 7*a^2*b^8*c^2*x^2*y -     3*b^10*c^2*x^2*y - 2*a^8*c^4*x^2*y + 5*a^6*b^2*c^4*x^2*y + 10*a^4*b^4*c^4*x^2*y -     15*a^2*b^6*c^4*x^2*y + 2*b^8*c^4*x^2*y - 2*a^6*c^6*x^2*y - 15*a^4*b^2*c^6*x^2*y +     5*a^2*b^4*c^6*x^2*y + 10*b^6*c^6*x^2*y + 8*a^4*c^8*x^2*y + 10*a^2*b^2*c^8*x^2*y -     12*b^4*c^8*x^2*y - 7*a^2*c^10*x^2*y + b^2*c^10*x^2*y + 2*c^12*x^2*y + 3*a^10*c^2*x*y^2 -     7*a^8*b^2*c^2*x*y^2 + 4*a^6*b^4*c^2*x*y^2 + a^2*b^8*c^2*x*y^2 - b^10*c^2*x*y^2 - 2*a^8*c^4*x*y^2 +     15*a^6*b^2*c^4*x*y^2 - 10*a^4*b^4*c^4*x*y^2 - 5*a^2*b^6*c^4*x*y^2 + 2*b^8*c^4*x*y^2 -     10*a^6*c^6*x*y^2 - 5*a^4*b^2*c^6*x*y^2 + 15*a^2*b^4*c^6*x*y^2 + 2*b^6*c^6*x*y^2 +     12*a^4*c^8*x*y^2 - 10*a^2*b^2*c^8*x*y^2 - 8*b^4*c^8*x*y^2 - a^2*c^10*x*y^2 + 7*b^2*c^10*x*y^2 -     2*c^12*x*y^2 - a^10*b^2*x^2*z + 2*a^8*b^4*x^2*z + 2*a^6*b^6*x^2*z - 8*a^4*b^8*x^2*z +     7*a^2*b^10*x^2*z - 2*b^12*x^2*z + a^8*b^2*c^2*x^2*z - 5*a^6*b^4*c^2*x^2*z + 15*a^4*b^6*c^2*x^2*z -     10*a^2*b^8*c^2*x^2*z - b^10*c^2*x^2*z - 10*a^4*b^4*c^4*x^2*z - 5*a^2*b^6*c^4*x^2*z +     12*b^8*c^4*x^2*z + 4*a^4*b^2*c^6*x^2*z + 15*a^2*b^4*c^6*x^2*z - 10*b^6*c^6*x^2*z -     7*a^2*b^2*c^8*x^2*z - 2*b^4*c^8*x^2*z + 3*b^2*c^10*x^2*z + 18*a^6*b^4*c^2*x*y*z -     18*a^4*b^6*c^2*x*y*z - 18*a^6*b^2*c^4*x*y*z + 18*a^2*b^6*c^4*x*y*z + 18*a^4*b^2*c^6*x*y*z -     18*a^2*b^4*c^6*x*y*z + 2*a^12*y^2*z - 7*a^10*b^2*y^2*z + 8*a^8*b^4*y^2*z - 2*a^6*b^6*y^2*z -     2*a^4*b^8*y^2*z + a^2*b^10*y^2*z + a^10*c^2*y^2*z + 10*a^8*b^2*c^2*y^2*z - 15*a^6*b^4*c^2*y^2*z +     5*a^4*b^6*c^2*y^2*z - a^2*b^8*c^2*y^2*z - 12*a^8*c^4*y^2*z + 5*a^6*b^2*c^4*y^2*z +     10*a^4*b^4*c^4*y^2*z + 10*a^6*c^6*y^2*z - 15*a^4*b^2*c^6*y^2*z - 4*a^2*b^4*c^6*y^2*z +     2*a^4*c^8*y^2*z + 7*a^2*b^2*c^8*y^2*z - 3*a^2*c^10*y^2*z - 3*a^10*b^2*x*z^2 + 2*a^8*b^4*x*z^2 +     10*a^6*b^6*x*z^2 - 12*a^4*b^8*x*z^2 + a^2*b^10*x*z^2 + 2*b^12*x*z^2 + 7*a^8*b^2*c^2*x*z^2 -     15*a^6*b^4*c^2*x*z^2 + 5*a^4*b^6*c^2*x*z^2 + 10*a^2*b^8*c^2*x*z^2 - 7*b^10*c^2*x*z^2 -     4*a^6*b^2*c^4*x*z^2 + 10*a^4*b^4*c^4*x*z^2 - 15*a^2*b^6*c^4*x*z^2 + 8*b^8*c^4*x*z^2 +     5*a^2*b^4*c^6*x*z^2 - 2*b^6*c^6*x*z^2 - a^2*b^2*c^8*x*z^2 - 2*b^4*c^8*x*z^2 + b^2*c^10*x*z^2 -     2*a^12*y*z^2 - a^10*b^2*y*z^2 + 12*a^8*b^4*y*z^2 - 10*a^6*b^6*y*z^2 - 2*a^4*b^8*y*z^2 +     3*a^2*b^10*y*z^2 + 7*a^10*c^2*y*z^2 - 10*a^8*b^2*c^2*y*z^2 - 5*a^6*b^4*c^2*y*z^2 +     15*a^4*b^6*c^2*y*z^2 - 7*a^2*b^8*c^2*y*z^2 - 8*a^8*c^4*y*z^2 + 15*a^6*b^2*c^4*y*z^2 -     10*a^4*b^4*c^4*y*z^2 + 4*a^2*b^6*c^4*y*z^2 + 2*a^6*c^6*y*z^2 - 5*a^4*b^2*c^6*y*z^2 +     2*a^4*c^8*y*z^2 + a^2*b^2*c^8*y*z^2 - a^2*c^10*y*z^2");
		exp("K615","(-a^4)*c^2*x^2*y + 6*a^2*b^2*c^2*x^2*y - 5*b^4*c^2*x^2*y - 2*a^2*c^4*x^2*y + 2*b^2*c^4*x^2*y +     3*c^6*x^2*y + 5*a^4*c^2*x*y^2 - 6*a^2*b^2*c^2*x*y^2 + b^4*c^2*x*y^2 - 2*a^2*c^4*x*y^2 +     2*b^2*c^4*x*y^2 - 3*c^6*x*y^2 + a^4*b^2*x^2*z + 2*a^2*b^4*x^2*z - 3*b^6*x^2*z -     6*a^2*b^2*c^2*x^2*z - 2*b^4*c^2*x^2*z + 5*b^2*c^4*x^2*z + 8*a^4*b^2*x*y*z - 8*a^2*b^4*x*y*z -     8*a^4*c^2*x*y*z + 8*b^4*c^2*x*y*z + 8*a^2*c^4*x*y*z - 8*b^2*c^4*x*y*z + 3*a^6*y^2*z -     2*a^4*b^2*y^2*z - a^2*b^4*y^2*z + 2*a^4*c^2*y^2*z + 6*a^2*b^2*c^2*y^2*z - 5*a^2*c^4*y^2*z -     5*a^4*b^2*x*z^2 + 2*a^2*b^4*x*z^2 + 3*b^6*x*z^2 + 6*a^2*b^2*c^2*x*z^2 - 2*b^4*c^2*x*z^2 -     b^2*c^4*x*z^2 - 3*a^6*y*z^2 - 2*a^4*b^2*y*z^2 + 5*a^2*b^4*y*z^2 + 2*a^4*c^2*y*z^2 -     6*a^2*b^2*c^2*y*z^2 + a^2*c^4*y*z^2");
		exp("K616","a^6*x^2*y - 3*a^4*b^2*x^2*y + 3*a^2*b^4*x^2*y - b^6*x^2*y + 3*a^4*c^2*x^2*y -     2*a^2*b^2*c^2*x^2*y - b^4*c^2*x^2*y - 5*a^2*c^4*x^2*y + b^2*c^4*x^2*y + c^6*x^2*y + a^6*x*y^2 -     3*a^4*b^2*x*y^2 + 3*a^2*b^4*x*y^2 - b^6*x*y^2 + a^4*c^2*x*y^2 + 2*a^2*b^2*c^2*x*y^2 -     3*b^4*c^2*x*y^2 - a^2*c^4*x*y^2 + 5*b^2*c^4*x*y^2 - c^6*x*y^2 - a^6*x^2*z - 3*a^4*b^2*x^2*z +     5*a^2*b^4*x^2*z - b^6*x^2*z + 3*a^4*c^2*x^2*z + 2*a^2*b^2*c^2*x^2*z - b^4*c^2*x^2*z -     3*a^2*c^4*x^2*z + b^2*c^4*x^2*z + c^6*x^2*z - 8*a^4*b^2*x*y*z + 8*a^2*b^4*x*y*z + 8*a^4*c^2*x*y*z -     8*b^4*c^2*x*y*z - 8*a^2*c^4*x*y*z + 8*b^2*c^4*x*y*z + a^6*y^2*z - 5*a^4*b^2*y^2*z +     3*a^2*b^4*y^2*z + b^6*y^2*z + a^4*c^2*y^2*z - 2*a^2*b^2*c^2*y^2*z - 3*b^4*c^2*y^2*z -     a^2*c^4*y^2*z + 3*b^2*c^4*y^2*z - c^6*y^2*z - a^6*x*z^2 - a^4*b^2*x*z^2 + a^2*b^4*x*z^2 +     b^6*x*z^2 + 3*a^4*c^2*x*z^2 - 2*a^2*b^2*c^2*x*z^2 - 5*b^4*c^2*x*z^2 - 3*a^2*c^4*x*z^2 +     3*b^2*c^4*x*z^2 + c^6*x*z^2 - a^6*y*z^2 - a^4*b^2*y*z^2 + a^2*b^4*y*z^2 + b^6*y*z^2 +     5*a^4*c^2*y*z^2 + 2*a^2*b^2*c^2*y*z^2 - 3*b^4*c^2*y*z^2 - 3*a^2*c^4*y*z^2 + 3*b^2*c^4*y*z^2 -     c^6*y*z^2");
		exp("K617","a^6*x^2*y - 3*a^4*b^2*x^2*y + 3*a^2*b^4*x^2*y - b^6*x^2*y + a^4*c^2*x^2*y - b^4*c^2*x^2*y -     3*a^2*c^4*x^2*y + b^2*c^4*x^2*y + c^6*x^2*y + a^6*x*y^2 - 3*a^4*b^2*x*y^2 + 3*a^2*b^4*x*y^2 -     b^6*x*y^2 + a^4*c^2*x*y^2 - b^4*c^2*x*y^2 - a^2*c^4*x*y^2 + 3*b^2*c^4*x*y^2 - c^6*x*y^2 -     a^6*x^2*z - a^4*b^2*x^2*z + 3*a^2*b^4*x^2*z - b^6*x^2*z + 3*a^4*c^2*x^2*z - b^4*c^2*x^2*z -     3*a^2*c^4*x^2*z + b^2*c^4*x^2*z + c^6*x^2*z - 4*a^4*b^2*x*y*z + 4*a^2*b^4*x*y*z + 4*a^4*c^2*x*y*z -     4*b^4*c^2*x*y*z - 4*a^2*c^4*x*y*z + 4*b^2*c^4*x*y*z + a^6*y^2*z - 3*a^4*b^2*y^2*z + a^2*b^4*y^2*z +     b^6*y^2*z + a^4*c^2*y^2*z - 3*b^4*c^2*y^2*z - a^2*c^4*y^2*z + 3*b^2*c^4*y^2*z - c^6*y^2*z -     a^6*x*z^2 - a^4*b^2*x*z^2 + a^2*b^4*x*z^2 + b^6*x*z^2 + 3*a^4*c^2*x*z^2 - 3*b^4*c^2*x*z^2 -     3*a^2*c^4*x*z^2 + b^2*c^4*x*z^2 + c^6*x*z^2 - a^6*y*z^2 - a^4*b^2*y*z^2 + a^2*b^4*y*z^2 +     b^6*y*z^2 + 3*a^4*c^2*y*z^2 - 3*b^4*c^2*y*z^2 - a^2*c^4*y*z^2 + 3*b^2*c^4*y*z^2 - c^6*y*z^2");
		exp("K618","a^6*x^2*y - 3*a^4*b^2*x^2*y + 3*a^2*b^4*x^2*y - b^6*x^2*y + 2*a^4*c^2*x^2*y - a^2*b^2*c^2*x^2*y -     b^4*c^2*x^2*y - 4*a^2*c^4*x^2*y + b^2*c^4*x^2*y + c^6*x^2*y + a^6*x*y^2 - 3*a^4*b^2*x*y^2 +     3*a^2*b^4*x*y^2 - b^6*x*y^2 + a^4*c^2*x*y^2 + a^2*b^2*c^2*x*y^2 - 2*b^4*c^2*x*y^2 - a^2*c^4*x*y^2 +     4*b^2*c^4*x*y^2 - c^6*x*y^2 - a^6*x^2*z - 2*a^4*b^2*x^2*z + 4*a^2*b^4*x^2*z - b^6*x^2*z +     3*a^4*c^2*x^2*z + a^2*b^2*c^2*x^2*z - b^4*c^2*x^2*z - 3*a^2*c^4*x^2*z + b^2*c^4*x^2*z + c^6*x^2*z -     6*a^4*b^2*x*y*z + 6*a^2*b^4*x*y*z + 6*a^4*c^2*x*y*z - 6*b^4*c^2*x*y*z - 6*a^2*c^4*x*y*z +     6*b^2*c^4*x*y*z + a^6*y^2*z - 4*a^4*b^2*y^2*z + 2*a^2*b^4*y^2*z + b^6*y^2*z + a^4*c^2*y^2*z -     a^2*b^2*c^2*y^2*z - 3*b^4*c^2*y^2*z - a^2*c^4*y^2*z + 3*b^2*c^4*y^2*z - c^6*y^2*z - a^6*x*z^2 -     a^4*b^2*x*z^2 + a^2*b^4*x*z^2 + b^6*x*z^2 + 3*a^4*c^2*x*z^2 - a^2*b^2*c^2*x*z^2 - 4*b^4*c^2*x*z^2 -     3*a^2*c^4*x*z^2 + 2*b^2*c^4*x*z^2 + c^6*x*z^2 - a^6*y*z^2 - a^4*b^2*y*z^2 + a^2*b^4*y*z^2 +     b^6*y*z^2 + 4*a^4*c^2*y*z^2 + a^2*b^2*c^2*y*z^2 - 3*b^4*c^2*y*z^2 - 2*a^2*c^4*y*z^2 +     3*b^2*c^4*y*z^2 - c^6*y*z^2");
		exp("K619","(-a^5)*c*x^2*y + a^4*b*c*x^2*y + 2*a^3*b^2*c*x^2*y - 2*a^2*b^3*c*x^2*y - a*b^4*c*x^2*y +     b^5*c*x^2*y - a^4*c^2*x^2*y - a^3*b*c^2*x^2*y + a^2*b^2*c^2*x^2*y + a*b^3*c^2*x^2*y +     a^3*c^3*x^2*y - a*b^2*c^3*x^2*y + a^2*c^4*x^2*y + a*b*c^4*x^2*y - b*c^5*x^2*y - a^5*c*x*y^2 +     a^4*b*c*x*y^2 + 2*a^3*b^2*c*x*y^2 - 2*a^2*b^3*c*x*y^2 - a*b^4*c*x*y^2 + b^5*c*x*y^2 -     a^3*b*c^2*x*y^2 - a^2*b^2*c^2*x*y^2 + a*b^3*c^2*x*y^2 + b^4*c^2*x*y^2 + a^2*b*c^3*x*y^2 -     b^3*c^3*x*y^2 - a*b*c^4*x*y^2 - b^2*c^4*x*y^2 + a*c^5*x*y^2 + a^5*b*x^2*z + a^4*b^2*x^2*z -     a^3*b^3*x^2*z - a^2*b^4*x^2*z - a^4*b*c*x^2*z + a^3*b^2*c*x^2*z - a*b^4*c*x^2*z + b^5*c*x^2*z -     2*a^3*b*c^2*x^2*z - a^2*b^2*c^2*x^2*z + a*b^3*c^2*x^2*z + 2*a^2*b*c^3*x^2*z - a*b^2*c^3*x^2*z +     a*b*c^4*x^2*z - b*c^5*x^2*z + a^5*b*x*y*z + 2*a^4*b^2*x*y*z - 2*a^2*b^4*x*y*z - a*b^5*x*y*z -     a^5*c*x*y*z + a^3*b^2*c*x*y*z - a^2*b^3*c*x*y*z + b^5*c*x*y*z - 2*a^4*c^2*x*y*z - a^3*b*c^2*x*y*z +     a*b^3*c^2*x*y*z + 2*b^4*c^2*x*y*z + a^2*b*c^3*x*y*z - a*b^2*c^3*x*y*z + 2*a^2*c^4*x*y*z -     2*b^2*c^4*x*y*z + a*c^5*x*y*z - b*c^5*x*y*z + a^4*b^2*y^2*z + a^3*b^3*y^2*z - a^2*b^4*y^2*z -     a*b^5*y^2*z - a^5*c*y^2*z + a^4*b*c*y^2*z - a^2*b^3*c*y^2*z + a*b^4*c*y^2*z - a^3*b*c^2*y^2*z +     a^2*b^2*c^2*y^2*z + 2*a*b^3*c^2*y^2*z + a^2*b*c^3*y^2*z - 2*a*b^2*c^3*y^2*z - a*b*c^4*y^2*z +     a*c^5*y^2*z + a^5*b*x*z^2 - a*b^5*x*z^2 - a^4*b*c*x*z^2 + a^3*b^2*c*x*z^2 - a^2*b^3*c*x*z^2 +     a*b^4*c*x*z^2 - 2*a^3*b*c^2*x*z^2 + a^2*b^2*c^2*x*z^2 + b^4*c^2*x*z^2 + 2*a^2*b*c^3*x*z^2 -     a*b^2*c^3*x*z^2 + b^3*c^3*x*z^2 + a*b*c^4*x*z^2 - b^2*c^4*x*z^2 - b*c^5*x*z^2 + a^5*b*y*z^2 -     a*b^5*y*z^2 - a^4*b*c*y*z^2 + a^3*b^2*c*y*z^2 - a^2*b^3*c*y*z^2 + a*b^4*c*y*z^2 - a^4*c^2*y*z^2 -     a^2*b^2*c^2*y*z^2 + 2*a*b^3*c^2*y*z^2 - a^3*c^3*y*z^2 + a^2*b*c^3*y*z^2 - 2*a*b^2*c^3*y*z^2 +     a^2*c^4*y*z^2 - a*b*c^4*y*z^2 + a*c^5*y*z^2");
		exp("K620","(-a^10)*c^2*x^2*y + 4*a^8*b^2*c^2*x^2*y - 6*a^6*b^4*c^2*x^2*y + 4*a^4*b^6*c^2*x^2*y -     a^2*b^8*c^2*x^2*y + 2*a^8*c^4*x^2*y - 6*a^6*b^2*c^4*x^2*y + 6*a^4*b^4*c^4*x^2*y -     2*a^2*b^6*c^4*x^2*y - 2*a^4*c^8*x^2*y + 2*a^2*b^2*c^8*x^2*y + a^2*c^10*x^2*y + a^8*b^2*c^2*x*y^2 -     4*a^6*b^4*c^2*x*y^2 + 6*a^4*b^6*c^2*x*y^2 - 4*a^2*b^8*c^2*x*y^2 + b^10*c^2*x*y^2 +     2*a^6*b^2*c^4*x*y^2 - 6*a^4*b^4*c^4*x*y^2 + 6*a^2*b^6*c^4*x*y^2 - 2*b^8*c^4*x*y^2 -     2*a^2*b^2*c^8*x*y^2 + 2*b^4*c^8*x*y^2 - b^2*c^10*x*y^2 + a^10*b^2*x^2*z - 2*a^8*b^4*x^2*z +     2*a^4*b^8*x^2*z - a^2*b^10*x^2*z - 4*a^8*b^2*c^2*x^2*z + 6*a^6*b^4*c^2*x^2*z -     2*a^2*b^8*c^2*x^2*z + 6*a^6*b^2*c^4*x^2*z - 6*a^4*b^4*c^4*x^2*z - 4*a^4*b^2*c^6*x^2*z +     2*a^2*b^4*c^6*x^2*z + a^2*b^2*c^8*x^2*z + 2*a^10*b^2*x*y*z - 4*a^8*b^4*x*y*z + 4*a^4*b^8*x*y*z -     2*a^2*b^10*x*y*z - 2*a^10*c^2*x*y*z + 10*a^6*b^4*c^2*x*y*z - 10*a^4*b^6*c^2*x*y*z +     2*b^10*c^2*x*y*z + 4*a^8*c^4*x*y*z - 10*a^6*b^2*c^4*x*y*z + 10*a^2*b^6*c^4*x*y*z -     4*b^8*c^4*x*y*z + 10*a^4*b^2*c^6*x*y*z - 10*a^2*b^4*c^6*x*y*z - 4*a^4*c^8*x*y*z + 4*b^4*c^8*x*y*z +     2*a^2*c^10*x*y*z - 2*b^2*c^10*x*y*z + a^10*b^2*y^2*z - 2*a^8*b^4*y^2*z + 2*a^4*b^8*y^2*z -     a^2*b^10*y^2*z + 2*a^8*b^2*c^2*y^2*z - 6*a^4*b^6*c^2*y^2*z + 4*a^2*b^8*c^2*y^2*z +     6*a^4*b^4*c^4*y^2*z - 6*a^2*b^6*c^4*y^2*z - 2*a^4*b^2*c^6*y^2*z + 4*a^2*b^4*c^6*y^2*z -     a^2*b^2*c^8*y^2*z - a^8*b^2*c^2*x*z^2 - 2*a^6*b^4*c^2*x*z^2 + 2*a^2*b^8*c^2*x*z^2 +     b^10*c^2*x*z^2 + 4*a^6*b^2*c^4*x*z^2 + 6*a^4*b^4*c^4*x*z^2 - 2*b^8*c^4*x*z^2 -     6*a^4*b^2*c^6*x*z^2 - 6*a^2*b^4*c^6*x*z^2 + 4*a^2*b^2*c^8*x*z^2 + 2*b^4*c^8*x*z^2 -     b^2*c^10*x*z^2 - a^10*c^2*y*z^2 - 2*a^8*b^2*c^2*y*z^2 + 2*a^4*b^6*c^2*y*z^2 + a^2*b^8*c^2*y*z^2 +     2*a^8*c^4*y*z^2 - 6*a^4*b^4*c^4*y*z^2 - 4*a^2*b^6*c^4*y*z^2 + 6*a^4*b^2*c^6*y*z^2 +     6*a^2*b^4*c^6*y*z^2 - 2*a^4*c^8*y*z^2 - 4*a^2*b^2*c^8*y*z^2 + a^2*c^10*y*z^2");
		exp("K621","a^6*c^2*x^2*y - 2*a^4*b^2*c^2*x^2*y + a^2*b^4*c^2*x^2*y - 2*a^4*c^4*x^2*y + 2*a^2*b^2*c^4*x^2*y +     a^2*c^6*x^2*y - a^4*b^2*c^2*x*y^2 + 2*a^2*b^4*c^2*x*y^2 - b^6*c^2*x*y^2 - 2*a^2*b^2*c^4*x*y^2 +     2*b^4*c^4*x*y^2 - b^2*c^6*x*y^2 - a^6*b^2*x^2*z + 2*a^4*b^4*x^2*z - a^2*b^6*x^2*z +     2*a^4*b^2*c^2*x^2*z - 2*a^2*b^4*c^2*x^2*z - a^2*b^2*c^4*x^2*z + a^6*b^2*y^2*z - 2*a^4*b^4*y^2*z +     a^2*b^6*y^2*z + 2*a^4*b^2*c^2*y^2*z - 2*a^2*b^4*c^2*y^2*z + a^2*b^2*c^4*y^2*z + a^4*b^2*c^2*x*z^2 +     2*a^2*b^4*c^2*x*z^2 + b^6*c^2*x*z^2 - 2*a^2*b^2*c^4*x*z^2 - 2*b^4*c^4*x*z^2 + b^2*c^6*x*z^2 -     a^6*c^2*y*z^2 - 2*a^4*b^2*c^2*y*z^2 - a^2*b^4*c^2*y*z^2 + 2*a^4*c^4*y*z^2 + 2*a^2*b^2*c^4*y*z^2 -     a^2*c^6*y*z^2");
		exp("K622","a^8*b^2*c^2*x^2*y - 2*a^6*b^4*c^2*x^2*y + 2*a^2*b^8*c^2*x^2*y - b^10*c^2*x^2*y - a^8*c^4*x^2*y +     a^6*b^2*c^4*x^2*y + 3*a^4*b^4*c^4*x^2*y - 5*a^2*b^6*c^4*x^2*y + 2*b^8*c^4*x^2*y + 3*a^6*c^6*x^2*y -     4*a^4*b^2*c^6*x^2*y + a^2*b^4*c^6*x^2*y - 3*a^4*c^8*x^2*y + a^2*b^2*c^8*x^2*y - 2*b^4*c^8*x^2*y +     a^2*c^10*x^2*y + b^2*c^10*x^2*y + a^10*c^2*x*y^2 - 2*a^8*b^2*c^2*x*y^2 + 2*a^4*b^6*c^2*x*y^2 -     a^2*b^8*c^2*x*y^2 - 2*a^8*c^4*x*y^2 + 5*a^6*b^2*c^4*x*y^2 - 3*a^4*b^4*c^4*x*y^2 -     a^2*b^6*c^4*x*y^2 + b^8*c^4*x*y^2 - a^4*b^2*c^6*x*y^2 + 4*a^2*b^4*c^6*x*y^2 - 3*b^6*c^6*x*y^2 +     2*a^4*c^8*x*y^2 - a^2*b^2*c^8*x*y^2 + 3*b^4*c^8*x*y^2 - a^2*c^10*x*y^2 - b^2*c^10*x*y^2 +     a^8*b^4*x^2*z - 3*a^6*b^6*x^2*z + 3*a^4*b^8*x^2*z - a^2*b^10*x^2*z - a^8*b^2*c^2*x^2*z -     a^6*b^4*c^2*x^2*z + 4*a^4*b^6*c^2*x^2*z - a^2*b^8*c^2*x^2*z - b^10*c^2*x^2*z +     2*a^6*b^2*c^4*x^2*z - 3*a^4*b^4*c^4*x^2*z - a^2*b^6*c^4*x^2*z + 2*b^8*c^4*x^2*z +     5*a^2*b^4*c^6*x^2*z - 2*a^2*b^2*c^8*x^2*z - 2*b^4*c^8*x^2*z + b^2*c^10*x^2*z + a^10*b^2*x*y*z -     2*a^8*b^4*x*y*z + 2*a^4*b^8*x*y*z - a^2*b^10*x*y*z - a^10*c^2*x*y*z + 5*a^6*b^4*c^2*x*y*z -     5*a^4*b^6*c^2*x*y*z + b^10*c^2*x*y*z + 2*a^8*c^4*x*y*z - 5*a^6*b^2*c^4*x*y*z +     5*a^2*b^6*c^4*x*y*z - 2*b^8*c^4*x*y*z + 5*a^4*b^2*c^6*x*y*z - 5*a^2*b^4*c^6*x*y*z -     2*a^4*c^8*x*y*z + 2*b^4*c^8*x*y*z + a^2*c^10*x*y*z - b^2*c^10*x*y*z + a^10*b^2*y^2*z -     3*a^8*b^4*y^2*z + 3*a^6*b^6*y^2*z - a^4*b^8*y^2*z + a^10*c^2*y^2*z + a^8*b^2*c^2*y^2*z -     4*a^6*b^4*c^2*y^2*z + a^4*b^6*c^2*y^2*z + a^2*b^8*c^2*y^2*z - 2*a^8*c^4*y^2*z + a^6*b^2*c^4*y^2*z +     3*a^4*b^4*c^4*y^2*z - 2*a^2*b^6*c^4*y^2*z - 5*a^4*b^2*c^6*y^2*z + 2*a^4*c^8*y^2*z +     2*a^2*b^2*c^8*y^2*z - a^2*c^10*y^2*z - a^10*b^2*x*z^2 + 2*a^8*b^4*x*z^2 - 2*a^4*b^8*x*z^2 +     a^2*b^10*x*z^2 + 2*a^8*b^2*c^2*x*z^2 - 5*a^6*b^4*c^2*x*z^2 + a^4*b^6*c^2*x*z^2 +     a^2*b^8*c^2*x*z^2 + b^10*c^2*x*z^2 + 3*a^4*b^4*c^4*x*z^2 - 4*a^2*b^6*c^4*x*z^2 - 3*b^8*c^4*x*z^2 -     2*a^4*b^2*c^6*x*z^2 + a^2*b^4*c^6*x*z^2 + 3*b^6*c^6*x*z^2 + a^2*b^2*c^8*x*z^2 - b^4*c^8*x*z^2 -     a^10*b^2*y*z^2 + 2*a^8*b^4*y*z^2 - 2*a^4*b^8*y*z^2 + a^2*b^10*y*z^2 - a^10*c^2*y*z^2 -     a^8*b^2*c^2*y*z^2 - a^6*b^4*c^2*y*z^2 + 5*a^4*b^6*c^2*y*z^2 - 2*a^2*b^8*c^2*y*z^2 +     3*a^8*c^4*y*z^2 + 4*a^6*b^2*c^4*y*z^2 - 3*a^4*b^4*c^4*y*z^2 - 3*a^6*c^6*y*z^2 - a^4*b^2*c^6*y*z^2 +     2*a^2*b^4*c^6*y*z^2 + a^4*c^8*y*z^2 - a^2*b^2*c^8*y*z^2");
		exp("K623","a^3*x^2*y - 2*a^2*b*x^2*y + a*b^2*x^2*y - a^2*c*x^2*y + b^2*c*x^2*y + a*c^2*x^2*y - c^3*x^2*y -     a^2*b*x*y^2 + 2*a*b^2*x*y^2 - b^3*x*y^2 - a^2*c*x*y^2 + b^2*c*x*y^2 - b*c^2*x*y^2 + c^3*x*y^2 -     a^3*x^2*z + a^2*b*x^2*z - a*b^2*x^2*z + b^3*x^2*z + 2*a^2*c*x^2*z - a*c^2*x^2*z - b*c^2*x^2*z -     a^3*y^2*z + a^2*b*y^2*z - a*b^2*y^2*z + b^3*y^2*z - 2*b^2*c*y^2*z + a*c^2*y^2*z + b*c^2*y^2*z +     a^2*b*x*z^2 - b^3*x*z^2 + a^2*c*x*z^2 + b^2*c*x*z^2 - 2*a*c^2*x*z^2 - b*c^2*x*z^2 + c^3*x*z^2 +     a^3*y*z^2 - a*b^2*y*z^2 - a^2*c*y*z^2 - b^2*c*y*z^2 + a*c^2*y*z^2 + 2*b*c^2*y*z^2 - c^3*y*z^2");
		exp("K624","a^6*c^2*x^2*y - 3*a^2*b^4*c^2*x^2*y + 2*b^6*c^2*x^2*y - 2*a^4*c^4*x^2*y + 3*a^2*b^2*c^4*x^2*y -     b^4*c^4*x^2*y + a^2*c^6*x^2*y - b^2*c^6*x^2*y - 2*a^6*c^2*x*y^2 + 3*a^4*b^2*c^2*x*y^2 -     b^6*c^2*x*y^2 + a^4*c^4*x*y^2 - 3*a^2*b^2*c^4*x*y^2 + 2*b^4*c^4*x*y^2 + a^2*c^6*x*y^2 -     b^2*c^6*x*y^2 - a^6*b^2*x^2*z + 2*a^4*b^4*x^2*z - a^2*b^6*x^2*z - 3*a^2*b^4*c^2*x^2*z +     b^6*c^2*x^2*z + 3*a^2*b^2*c^4*x^2*z + b^4*c^4*x^2*z - 2*b^2*c^6*x^2*z + a^6*b^2*y^2*z -     2*a^4*b^4*y^2*z + a^2*b^6*y^2*z - a^6*c^2*y^2*z + 3*a^4*b^2*c^2*y^2*z - a^4*c^4*y^2*z -     3*a^2*b^2*c^4*y^2*z + 2*a^2*c^6*y^2*z + 2*a^6*b^2*x*z^2 - a^4*b^4*x*z^2 - a^2*b^6*x*z^2 -     3*a^4*b^2*c^2*x*z^2 + 3*a^2*b^4*c^2*x*z^2 + b^6*c^2*x*z^2 - 2*b^4*c^4*x*z^2 + b^2*c^6*x*z^2 +     a^6*b^2*y*z^2 + a^4*b^4*y*z^2 - 2*a^2*b^6*y*z^2 - a^6*c^2*y*z^2 - 3*a^4*b^2*c^2*y*z^2 +     3*a^2*b^4*c^2*y*z^2 + 2*a^4*c^4*y*z^2 - a^2*c^6*y*z^2");
		exp("K625","(-a^10)*c^2*x^2*y + a^8*b^2*c^2*x^2*y - a^6*b^4*c^2*x^2*y + a^4*b^6*c^2*x^2*y +     2*a^2*b^8*c^2*x^2*y - 2*b^10*c^2*x^2*y + 3*a^8*c^4*x^2*y - 4*a^6*b^2*c^4*x^2*y +     3*a^4*b^4*c^4*x^2*y - 8*a^2*b^6*c^4*x^2*y + 6*b^8*c^4*x^2*y - a^6*c^6*x^2*y + 3*a^4*b^2*c^6*x^2*y +     3*a^2*b^4*c^6*x^2*y - 5*b^6*c^6*x^2*y - 3*a^4*c^8*x^2*y + 3*b^4*c^8*x^2*y + 2*a^2*c^10*x^2*y -     2*b^2*c^10*x^2*y + 2*a^10*c^2*x*y^2 - 2*a^8*b^2*c^2*x*y^2 - a^6*b^4*c^2*x*y^2 + a^4*b^6*c^2*x*y^2 -     a^2*b^8*c^2*x*y^2 + b^10*c^2*x*y^2 - 6*a^8*c^4*x*y^2 + 8*a^6*b^2*c^4*x*y^2 - 3*a^4*b^4*c^4*x*y^2 +     4*a^2*b^6*c^4*x*y^2 - 3*b^8*c^4*x*y^2 + 5*a^6*c^6*x*y^2 - 3*a^4*b^2*c^6*x*y^2 -     3*a^2*b^4*c^6*x*y^2 + b^6*c^6*x*y^2 - 3*a^4*c^8*x*y^2 + 3*b^4*c^8*x*y^2 + 2*a^2*c^10*x*y^2 -     2*b^2*c^10*x*y^2 + a^10*b^2*x^2*z - 3*a^8*b^4*x^2*z + a^6*b^6*x^2*z + 3*a^4*b^8*x^2*z -     2*a^2*b^10*x^2*z - a^8*b^2*c^2*x^2*z + 4*a^6*b^4*c^2*x^2*z - 3*a^4*b^6*c^2*x^2*z +     2*b^10*c^2*x^2*z + a^6*b^2*c^4*x^2*z - 3*a^4*b^4*c^4*x^2*z - 3*a^2*b^6*c^4*x^2*z -     3*b^8*c^4*x^2*z - a^4*b^2*c^6*x^2*z + 8*a^2*b^4*c^6*x^2*z + 5*b^6*c^6*x^2*z - 2*a^2*b^2*c^8*x^2*z -     6*b^4*c^8*x^2*z + 2*b^2*c^10*x^2*z + 2*a^10*b^2*y^2*z - 3*a^8*b^4*y^2*z - a^6*b^6*y^2*z +     3*a^4*b^8*y^2*z - a^2*b^10*y^2*z - 2*a^10*c^2*y^2*z + 3*a^6*b^4*c^2*y^2*z - 4*a^4*b^6*c^2*y^2*z +     a^2*b^8*c^2*y^2*z + 3*a^8*c^4*y^2*z + 3*a^6*b^2*c^4*y^2*z + 3*a^4*b^4*c^4*y^2*z -     a^2*b^6*c^4*y^2*z - 5*a^6*c^6*y^2*z - 8*a^4*b^2*c^6*y^2*z + a^2*b^4*c^6*y^2*z + 6*a^4*c^8*y^2*z +     2*a^2*b^2*c^8*y^2*z - 2*a^2*c^10*y^2*z - 2*a^10*b^2*x*z^2 + 6*a^8*b^4*x*z^2 - 5*a^6*b^6*x*z^2 +     3*a^4*b^8*x*z^2 - 2*a^2*b^10*x*z^2 + 2*a^8*b^2*c^2*x*z^2 - 8*a^6*b^4*c^2*x*z^2 +     3*a^4*b^6*c^2*x*z^2 + 2*b^10*c^2*x*z^2 + a^6*b^2*c^4*x*z^2 + 3*a^4*b^4*c^4*x*z^2 +     3*a^2*b^6*c^4*x*z^2 - 3*b^8*c^4*x*z^2 - a^4*b^2*c^6*x*z^2 - 4*a^2*b^4*c^6*x*z^2 - b^6*c^6*x*z^2 +     a^2*b^2*c^8*x*z^2 + 3*b^4*c^8*x*z^2 - b^2*c^10*x*z^2 + 2*a^10*b^2*y*z^2 - 3*a^8*b^4*y*z^2 +     5*a^6*b^6*y*z^2 - 6*a^4*b^8*y*z^2 + 2*a^2*b^10*y*z^2 - 2*a^10*c^2*y*z^2 - 3*a^6*b^4*c^2*y*z^2 +     8*a^4*b^6*c^2*y*z^2 - 2*a^2*b^8*c^2*y*z^2 + 3*a^8*c^4*y*z^2 - 3*a^6*b^2*c^4*y*z^2 -     3*a^4*b^4*c^4*y*z^2 - a^2*b^6*c^4*y*z^2 + a^6*c^6*y*z^2 + 4*a^4*b^2*c^6*y*z^2 + a^2*b^4*c^6*y*z^2 -     3*a^4*c^8*y*z^2 - a^2*b^2*c^8*y*z^2 + a^2*c^10*y*z^2");
		exp("K626","(-a^6)*b^2*c^4*x^2*y - a^4*b^4*c^4*x^2*y + a^2*b^6*c^4*x^2*y + b^8*c^4*x^2*y +     5*a^4*b^2*c^6*x^2*y + 2*a^2*b^4*c^6*x^2*y - 3*b^6*c^6*x^2*y - 3*a^2*b^2*c^8*x^2*y +     3*b^4*c^8*x^2*y - b^2*c^10*x^2*y - a^8*c^4*x*y^2 - a^6*b^2*c^4*x*y^2 + a^4*b^4*c^4*x*y^2 +     a^2*b^6*c^4*x*y^2 + 3*a^6*c^6*x*y^2 - 2*a^4*b^2*c^6*x*y^2 - 5*a^2*b^4*c^6*x*y^2 - 3*a^4*c^8*x*y^2 +     3*a^2*b^2*c^8*x*y^2 + a^2*c^10*x*y^2 + a^6*b^4*c^2*x^2*z - 5*a^4*b^6*c^2*x^2*z +     3*a^2*b^8*c^2*x^2*z + b^10*c^2*x^2*z + a^4*b^4*c^4*x^2*z - 2*a^2*b^6*c^4*x^2*z - 3*b^8*c^4*x^2*z -     a^2*b^4*c^6*x^2*z + 3*b^6*c^6*x^2*z - b^4*c^8*x^2*z - 8*a^6*b^4*c^2*x*y*z + 8*a^4*b^6*c^2*x*y*z +     8*a^6*b^2*c^4*x*y*z - 8*a^2*b^6*c^4*x*y*z - 8*a^4*b^2*c^6*x*y*z + 8*a^2*b^4*c^6*x*y*z -     a^10*c^2*y^2*z - 3*a^8*b^2*c^2*y^2*z + 5*a^6*b^4*c^2*y^2*z - a^4*b^6*c^2*y^2*z + 3*a^8*c^4*y^2*z +     2*a^6*b^2*c^4*y^2*z - a^4*b^4*c^4*y^2*z - 3*a^6*c^6*y^2*z + a^4*b^2*c^6*y^2*z + a^4*c^8*y^2*z +     a^8*b^4*x*z^2 - 3*a^6*b^6*x*z^2 + 3*a^4*b^8*x*z^2 - a^2*b^10*x*z^2 + a^6*b^4*c^2*x*z^2 +     2*a^4*b^6*c^2*x*z^2 - 3*a^2*b^8*c^2*x*z^2 - a^4*b^4*c^4*x*z^2 + 5*a^2*b^6*c^4*x*z^2 -     a^2*b^4*c^6*x*z^2 + a^10*b^2*y*z^2 - 3*a^8*b^4*y*z^2 + 3*a^6*b^6*y*z^2 - a^4*b^8*y*z^2 +     3*a^8*b^2*c^2*y*z^2 - 2*a^6*b^4*c^2*y*z^2 - a^4*b^6*c^2*y*z^2 - 5*a^6*b^2*c^4*y*z^2 +     a^4*b^4*c^4*y*z^2 + a^4*b^2*c^6*y*z^2");
		exp("K627","a^10*c^2*x^2*y - 3*a^8*b^2*c^2*x^2*y + 2*a^6*b^4*c^2*x^2*y + 2*a^4*b^6*c^2*x^2*y -     3*a^2*b^8*c^2*x^2*y + b^10*c^2*x^2*y - 4*a^8*c^4*x^2*y + 8*a^6*b^2*c^4*x^2*y -     4*a^4*b^4*c^4*x^2*y + 6*a^6*c^6*x^2*y - 6*a^4*b^2*c^6*x^2*y + 2*a^2*b^4*c^6*x^2*y -     2*b^6*c^6*x^2*y - 4*a^4*c^8*x^2*y + a^2*c^10*x^2*y + b^2*c^10*x^2*y - a^10*c^2*x*y^2 +     3*a^8*b^2*c^2*x*y^2 - 2*a^6*b^4*c^2*x*y^2 - 2*a^4*b^6*c^2*x*y^2 + 3*a^2*b^8*c^2*x*y^2 -     b^10*c^2*x*y^2 + 4*a^4*b^4*c^4*x*y^2 - 8*a^2*b^6*c^4*x*y^2 + 4*b^8*c^4*x*y^2 + 2*a^6*c^6*x*y^2 -     2*a^4*b^2*c^6*x*y^2 + 6*a^2*b^4*c^6*x*y^2 - 6*b^6*c^6*x*y^2 + 4*b^4*c^8*x*y^2 - a^2*c^10*x*y^2 -     b^2*c^10*x*y^2 - a^10*b^2*x^2*z + 4*a^8*b^4*x^2*z - 6*a^6*b^6*x^2*z + 4*a^4*b^8*x^2*z -     a^2*b^10*x^2*z + 3*a^8*b^2*c^2*x^2*z - 8*a^6*b^4*c^2*x^2*z + 6*a^4*b^6*c^2*x^2*z - b^10*c^2*x^2*z -     2*a^6*b^2*c^4*x^2*z + 4*a^4*b^4*c^4*x^2*z - 2*a^2*b^6*c^4*x^2*z - 2*a^4*b^2*c^6*x^2*z +     2*b^6*c^6*x^2*z + 3*a^2*b^2*c^8*x^2*z - b^2*c^10*x^2*z + a^10*b^2*y^2*z - 4*a^8*b^4*y^2*z +     6*a^6*b^6*y^2*z - 4*a^4*b^8*y^2*z + a^2*b^10*y^2*z + a^10*c^2*y^2*z - 6*a^6*b^4*c^2*y^2*z +     8*a^4*b^6*c^2*y^2*z - 3*a^2*b^8*c^2*y^2*z + 2*a^6*b^2*c^4*y^2*z - 4*a^4*b^4*c^4*y^2*z +     2*a^2*b^6*c^4*y^2*z - 2*a^6*c^6*y^2*z + 2*a^2*b^4*c^6*y^2*z - 3*a^2*b^2*c^8*y^2*z +     a^2*c^10*y^2*z + a^10*b^2*x*z^2 - 2*a^6*b^6*x*z^2 + a^2*b^10*x*z^2 - 3*a^8*b^2*c^2*x*z^2 +     2*a^4*b^6*c^2*x*z^2 + b^10*c^2*x*z^2 + 2*a^6*b^2*c^4*x*z^2 - 4*a^4*b^4*c^4*x*z^2 -     6*a^2*b^6*c^4*x*z^2 - 4*b^8*c^4*x*z^2 + 2*a^4*b^2*c^6*x*z^2 + 8*a^2*b^4*c^6*x*z^2 +     6*b^6*c^6*x*z^2 - 3*a^2*b^2*c^8*x*z^2 - 4*b^4*c^8*x*z^2 + b^2*c^10*x*z^2 - a^10*b^2*y*z^2 +     2*a^6*b^6*y*z^2 - a^2*b^10*y*z^2 - a^10*c^2*y*z^2 - 2*a^6*b^4*c^2*y*z^2 + 3*a^2*b^8*c^2*y*z^2 +     4*a^8*c^4*y*z^2 + 6*a^6*b^2*c^4*y*z^2 + 4*a^4*b^4*c^4*y*z^2 - 2*a^2*b^6*c^4*y*z^2 -     6*a^6*c^6*y*z^2 - 8*a^4*b^2*c^6*y*z^2 - 2*a^2*b^4*c^6*y*z^2 + 4*a^4*c^8*y*z^2 +     3*a^2*b^2*c^8*y*z^2 - a^2*c^10*y*z^2");
		exp("K628","(-a^7)*b*x^2*y - a^6*b^2*x^2*y + 3*a^5*b^3*x^2*y + 3*a^4*b^4*x^2*y - 3*a^3*b^5*x^2*y -     3*a^2*b^6*x^2*y + a*b^7*x^2*y + b^8*x^2*y - a^7*c*x^2*y - a^6*b*c*x^2*y + 3*a^5*b^2*c*x^2*y +     3*a^4*b^3*c*x^2*y - 3*a^3*b^4*c*x^2*y - 3*a^2*b^5*c*x^2*y + a*b^6*c*x^2*y + b^7*c*x^2*y -     2*a^3*b^3*c^2*x^2*y - 2*a^2*b^4*c^2*x^2*y + 2*a*b^5*c^2*x^2*y + 2*b^6*c^2*x^2*y -     2*a^3*b^2*c^3*x^2*y - 2*a^2*b^3*c^3*x^2*y + 2*a*b^4*c^3*x^2*y + 2*b^5*c^3*x^2*y +     3*a^3*b*c^4*x^2*y + 3*a^2*b^2*c^4*x^2*y - a*b^3*c^4*x^2*y - b^4*c^4*x^2*y + 3*a^3*c^5*x^2*y +     3*a^2*b*c^5*x^2*y - a*b^2*c^5*x^2*y - b^3*c^5*x^2*y - 2*a*b*c^6*x^2*y - 2*b^2*c^6*x^2*y -     2*a*c^7*x^2*y - 2*b*c^7*x^2*y + a^8*x*y^2 + a^7*b*x*y^2 - 3*a^6*b^2*x*y^2 - 3*a^5*b^3*x*y^2 +     3*a^4*b^4*x*y^2 + 3*a^3*b^5*x*y^2 - a^2*b^6*x*y^2 - a*b^7*x*y^2 + a^7*c*x*y^2 + a^6*b*c*x*y^2 -     3*a^5*b^2*c*x*y^2 - 3*a^4*b^3*c*x*y^2 + 3*a^3*b^4*c*x*y^2 + 3*a^2*b^5*c*x*y^2 - a*b^6*c*x*y^2 -     b^7*c*x*y^2 + 2*a^6*c^2*x*y^2 + 2*a^5*b*c^2*x*y^2 - 2*a^4*b^2*c^2*x*y^2 - 2*a^3*b^3*c^2*x*y^2 +     2*a^5*c^3*x*y^2 + 2*a^4*b*c^3*x*y^2 - 2*a^3*b^2*c^3*x*y^2 - 2*a^2*b^3*c^3*x*y^2 - a^4*c^4*x*y^2 -     a^3*b*c^4*x*y^2 + 3*a^2*b^2*c^4*x*y^2 + 3*a*b^3*c^4*x*y^2 - a^3*c^5*x*y^2 - a^2*b*c^5*x*y^2 +     3*a*b^2*c^5*x*y^2 + 3*b^3*c^5*x*y^2 - 2*a^2*c^6*x*y^2 - 2*a*b*c^6*x*y^2 - 2*a*c^7*x*y^2 -     2*b*c^7*x*y^2 - a^7*b*x^2*z + 3*a^3*b^5*x^2*z - 2*a*b^7*x^2*z - a^7*c*x^2*z - a^6*b*c*x^2*z +     3*a^3*b^4*c*x^2*z + 3*a^2*b^5*c*x^2*z - 2*a*b^6*c*x^2*z - 2*b^7*c*x^2*z - a^6*c^2*x^2*z +     3*a^5*b*c^2*x^2*z - 2*a^3*b^3*c^2*x^2*z + 3*a^2*b^4*c^2*x^2*z - a*b^5*c^2*x^2*z - 2*b^6*c^2*x^2*z +     3*a^5*c^3*x^2*z + 3*a^4*b*c^3*x^2*z - 2*a^3*b^2*c^3*x^2*z - 2*a^2*b^3*c^3*x^2*z - a*b^4*c^3*x^2*z -     b^5*c^3*x^2*z + 3*a^4*c^4*x^2*z - 3*a^3*b*c^4*x^2*z - 2*a^2*b^2*c^4*x^2*z + 2*a*b^3*c^4*x^2*z -     b^4*c^4*x^2*z - 3*a^3*c^5*x^2*z - 3*a^2*b*c^5*x^2*z + 2*a*b^2*c^5*x^2*z + 2*b^3*c^5*x^2*z -     3*a^2*c^6*x^2*z + a*b*c^6*x^2*z + 2*b^2*c^6*x^2*z + a*c^7*x^2*z + b*c^7*x^2*z + c^8*x^2*z +     2*a^8*x*y*z - 2*a^6*b^2*x*y*z - 2*a^2*b^6*x*y*z + 2*b^8*x*y*z - 2*a^6*c^2*x*y*z +     2*a^4*b^2*c^2*x*y*z + 2*a^2*b^4*c^2*x*y*z - 2*b^6*c^2*x*y*z + 2*a^2*b^2*c^4*x*y*z -     2*a^2*c^6*x*y*z - 2*b^2*c^6*x*y*z + 2*c^8*x*y*z - 2*a^7*b*y^2*z + 3*a^5*b^3*y^2*z - a*b^7*y^2*z -     2*a^7*c*y^2*z - 2*a^6*b*c*y^2*z + 3*a^5*b^2*c*y^2*z + 3*a^4*b^3*c*y^2*z - a*b^6*c*y^2*z -     b^7*c*y^2*z - 2*a^6*c^2*y^2*z - a^5*b*c^2*y^2*z + 3*a^4*b^2*c^2*y^2*z - 2*a^3*b^3*c^2*y^2*z +     3*a*b^5*c^2*y^2*z - b^6*c^2*y^2*z - a^5*c^3*y^2*z - a^4*b*c^3*y^2*z - 2*a^3*b^2*c^3*y^2*z -     2*a^2*b^3*c^3*y^2*z + 3*a*b^4*c^3*y^2*z + 3*b^5*c^3*y^2*z - a^4*c^4*y^2*z + 2*a^3*b*c^4*y^2*z -     2*a^2*b^2*c^4*y^2*z - 3*a*b^3*c^4*y^2*z + 3*b^4*c^4*y^2*z + 2*a^3*c^5*y^2*z + 2*a^2*b*c^5*y^2*z -     3*a*b^2*c^5*y^2*z - 3*b^3*c^5*y^2*z + 2*a^2*c^6*y^2*z + a*b*c^6*y^2*z - 3*b^2*c^6*y^2*z +     a*c^7*y^2*z + b*c^7*y^2*z + c^8*y^2*z + a^8*x*z^2 + a^7*b*x*z^2 + 2*a^6*b^2*x*z^2 +     2*a^5*b^3*x*z^2 - a^4*b^4*x*z^2 - a^3*b^5*x*z^2 - 2*a^2*b^6*x*z^2 - 2*a*b^7*x*z^2 + a^7*c*x*z^2 +     a^6*b*c*x*z^2 + 2*a^5*b^2*c*x*z^2 + 2*a^4*b^3*c*x*z^2 - a^3*b^4*c*x*z^2 - a^2*b^5*c*x*z^2 -     2*a*b^6*c*x*z^2 - 2*b^7*c*x*z^2 - 3*a^6*c^2*x*z^2 - 3*a^5*b*c^2*x*z^2 - 2*a^4*b^2*c^2*x*z^2 -     2*a^3*b^3*c^2*x*z^2 + 3*a^2*b^4*c^2*x*z^2 + 3*a*b^5*c^2*x*z^2 - 3*a^5*c^3*x*z^2 -     3*a^4*b*c^3*x*z^2 - 2*a^3*b^2*c^3*x*z^2 - 2*a^2*b^3*c^3*x*z^2 + 3*a*b^4*c^3*x*z^2 +     3*b^5*c^3*x*z^2 + 3*a^4*c^4*x*z^2 + 3*a^3*b*c^4*x*z^2 + 3*a^3*c^5*x*z^2 + 3*a^2*b*c^5*x*z^2 -     a^2*c^6*x*z^2 - a*b*c^6*x*z^2 - a*c^7*x*z^2 - b*c^7*x*z^2 - 2*a^7*b*y*z^2 - 2*a^6*b^2*y*z^2 -     a^5*b^3*y*z^2 - a^4*b^4*y*z^2 + 2*a^3*b^5*y*z^2 + 2*a^2*b^6*y*z^2 + a*b^7*y*z^2 + b^8*y*z^2 -     2*a^7*c*y*z^2 - 2*a^6*b*c*y*z^2 - a^5*b^2*c*y*z^2 - a^4*b^3*c*y*z^2 + 2*a^3*b^4*c*y*z^2 +     2*a^2*b^5*c*y*z^2 + a*b^6*c*y*z^2 + b^7*c*y*z^2 + 3*a^5*b*c^2*y*z^2 + 3*a^4*b^2*c^2*y*z^2 -     2*a^3*b^3*c^2*y*z^2 - 2*a^2*b^4*c^2*y*z^2 - 3*a*b^5*c^2*y*z^2 - 3*b^6*c^2*y*z^2 + 3*a^5*c^3*y*z^2 +     3*a^4*b*c^3*y*z^2 - 2*a^3*b^2*c^3*y*z^2 - 2*a^2*b^3*c^3*y*z^2 - 3*a*b^4*c^3*y*z^2 -     3*b^5*c^3*y*z^2 + 3*a*b^3*c^4*y*z^2 + 3*b^4*c^4*y*z^2 + 3*a*b^2*c^5*y*z^2 + 3*b^3*c^5*y*z^2 -     a*b*c^6*y*z^2 - b^2*c^6*y*z^2 - a*c^7*y*z^2 - b*c^7*y*z^2");
		exp("K629","a^2*b^2*c^2*x^2*y - b^4*c^2*x^2*y - a^4*c^2*x*y^2 + a^2*b^2*c^2*x*y^2 + a^2*b^2*c^2*x^2*z -     b^2*c^4*x^2*z + a^6*x*y*z - a^4*b^2*x*y*z - a^2*b^4*x*y*z + b^6*x*y*z - a^4*c^2*x*y*z +     3*a^2*b^2*c^2*x*y*z - b^4*c^2*x*y*z - a^2*c^4*x*y*z - b^2*c^4*x*y*z + c^6*x*y*z +     a^2*b^2*c^2*y^2*z - a^2*c^4*y^2*z - a^4*b^2*x*z^2 + a^2*b^2*c^2*x*z^2 - a^2*b^4*y*z^2 +     a^2*b^2*c^2*y*z^2");
		exp("K630","(-b^4)*c^2*x^2*y + b^2*c^4*x^2*y - a^4*c^2*x*y^2 + a^2*c^4*x*y^2 + b^4*c^2*x^2*z -     b^2*c^4*x^2*z + a^6*x*y*z - a^4*b^2*x*y*z - a^2*b^4*x*y*z + b^6*x*y*z - a^4*c^2*x*y*z +     3*a^2*b^2*c^2*x*y*z - b^4*c^2*x*y*z - a^2*c^4*x*y*z - b^2*c^4*x*y*z + c^6*x*y*z + a^4*c^2*y^2*z -     a^2*c^4*y^2*z - a^4*b^2*x*z^2 + a^2*b^4*x*z^2 + a^4*b^2*y*z^2 - a^2*b^4*y*z^2");
		exp("K631","(-a)*c^2*x^2*y + b*c^2*x^2*y + c^3*x^2*y - a*c^2*x*y^2 + b*c^2*x*y^2 - c^3*x*y^2 + a*b^2*x^2*z -     b^3*x^2*z - b^2*c*x^2*z + a^3*y^2*z - a^2*b*y^2*z + a^2*c*y^2*z + a*b^2*x*z^2 + b^3*x*z^2 -     b^2*c*x*z^2 - a^3*y*z^2 - a^2*b*y*z^2 + a^2*c*y*z^2");
		exp("K632","(-a^2)*b*c^3*x^2*y + 2*a*b^2*c^3*x^2*y - b^3*c^3*x^2*y + b*c^5*x^2*y + a^3*c^3*x*y^2 -     2*a^2*b*c^3*x*y^2 + a*b^2*c^3*x*y^2 - a*c^5*x*y^2 + a^2*b^3*c*x^2*z - b^5*c*x^2*z -     2*a*b^3*c^2*x^2*z + b^3*c^3*x^2*z + a^5*c*y^2*z - a^3*b^2*c*y^2*z + 2*a^3*b*c^2*y^2*z -     a^3*c^3*y^2*z - a^3*b^3*x*z^2 + a*b^5*x*z^2 + 2*a^2*b^3*c*x*z^2 - a*b^3*c^2*x*z^2 - a^5*b*y*z^2 +     a^3*b^3*y*z^2 - 2*a^3*b^2*c*y*z^2 + a^3*b*c^2*y*z^2");
		exp("K633","-2*a^6*b^2*c^4*x^2*y + a^4*b^4*c^4*x^2*y + b^8*c^4*x^2*y + 3*a^4*b^2*c^6*x^2*y -     3*b^6*c^6*x^2*y + 3*b^4*c^8*x^2*y - b^2*c^10*x^2*y - a^8*c^4*x*y^2 - a^4*b^4*c^4*x*y^2 +     2*a^2*b^6*c^4*x*y^2 + 3*a^6*c^6*x*y^2 - 3*a^2*b^4*c^6*x*y^2 - 3*a^4*c^8*x*y^2 + a^2*c^10*x*y^2 +     2*a^6*b^4*c^2*x^2*z - 3*a^4*b^6*c^2*x^2*z + b^10*c^2*x^2*z - a^4*b^4*c^4*x^2*z - 3*b^8*c^4*x^2*z +     3*b^6*c^6*x^2*z - b^4*c^8*x^2*z - a^10*c^2*y^2*z + 3*a^6*b^4*c^2*y^2*z - 2*a^4*b^6*c^2*y^2*z +     3*a^8*c^4*y^2*z + a^4*b^4*c^4*y^2*z - 3*a^6*c^6*y^2*z + a^4*c^8*y^2*z + a^8*b^4*x*z^2 -     3*a^6*b^6*x*z^2 + 3*a^4*b^8*x*z^2 - a^2*b^10*x*z^2 + a^4*b^4*c^4*x*z^2 + 3*a^2*b^6*c^4*x*z^2 -     2*a^2*b^4*c^6*x*z^2 + a^10*b^2*y*z^2 - 3*a^8*b^4*y*z^2 + 3*a^6*b^6*y*z^2 - a^4*b^8*y*z^2 -     3*a^6*b^2*c^4*y*z^2 - a^4*b^4*c^4*y*z^2 + 2*a^4*b^2*c^6*y*z^2");
		exp("K634","4*b^2*c^2*x^2*y + 4*a^2*c^2*x*y^2 + 4*b^2*c^2*x^2*z - a^4*x*y*z + 2*a^2*b^2*x*y*z - b^4*x*y*z +     2*a^2*c^2*x*y*z + 2*b^2*c^2*x*y*z - c^4*x*y*z + 4*a^2*c^2*y^2*z + 4*a^2*b^2*x*z^2 +     4*a^2*b^2*y*z^2");
		exp("K635","b^2*c^2*x^2*y + a^2*c^2*x*y^2 + b^2*c^2*x^2*z - 2*a^2*b*c*x*y*z - 2*a*b^2*c*x*y*z -     2*a*b*c^2*x*y*z + a^2*c^2*y^2*z + a^2*b^2*x*z^2 + a^2*b^2*y*z^2");
		exp("K636","a^6*x^2*y - b^6*x^2*y - a^4*c^2*x^2*y + a^2*b^2*c^2*x^2*y + 3*b^4*c^2*x^2*y - a^2*c^4*x^2*y -     3*b^2*c^4*x^2*y + c^6*x^2*y + a^6*x*y^2 - b^6*x*y^2 - 3*a^4*c^2*x*y^2 - a^2*b^2*c^2*x*y^2 +     b^4*c^2*x*y^2 + 3*a^2*c^4*x*y^2 + b^2*c^4*x*y^2 - c^6*x*y^2 - a^6*x^2*z + a^4*b^2*x^2*z +     a^2*b^4*x^2*z - b^6*x^2*z - a^2*b^2*c^2*x^2*z + 3*b^4*c^2*x^2*z - 3*b^2*c^4*x^2*z + c^6*x^2*z +     a^6*y^2*z - a^4*b^2*y^2*z - a^2*b^4*y^2*z + b^6*y^2*z - 3*a^4*c^2*y^2*z + a^2*b^2*c^2*y^2*z +     3*a^2*c^4*y^2*z - c^6*y^2*z - a^6*x*z^2 + 3*a^4*b^2*x*z^2 - 3*a^2*b^4*x*z^2 + b^6*x*z^2 +     a^2*b^2*c^2*x*z^2 - b^4*c^2*x*z^2 - b^2*c^4*x*z^2 + c^6*x*z^2 - a^6*y*z^2 + 3*a^4*b^2*y*z^2 -     3*a^2*b^4*y*z^2 + b^6*y*z^2 + a^4*c^2*y*z^2 - a^2*b^2*c^2*y*z^2 + a^2*c^4*y*z^2 - c^6*y*z^2");
		exp("K637","a*c*x^2*y + b*c*x^2*y + 2*c^2*x^2*y - a*c*x*y^2 - b*c*x*y^2 - 2*c^2*x*y^2 - a*b*x^2*z -     2*b^2*x^2*z - b*c*x^2*z + 2*a^2*y^2*z + a*b*y^2*z + a*c*y^2*z + a*b*x*z^2 + 2*b^2*x*z^2 +     b*c*x*z^2 - 2*a^2*y*z^2 - a*b*y*z^2 - a*c*y*z^2");
		exp("K638","a^10*b^2*x^2*y - 3*a^8*b^4*x^2*y + 2*a^6*b^6*x^2*y + 2*a^4*b^8*x^2*y - 3*a^2*b^10*x^2*y +     b^12*x^2*y - a^10*c^2*x^2*y + 5*a^6*b^4*c^2*x^2*y - 7*a^4*b^6*c^2*x^2*y + 4*a^2*b^8*c^2*x^2*y -     b^10*c^2*x^2*y + 3*a^8*c^4*x^2*y - 4*a^6*b^2*c^4*x^2*y - a^4*b^4*c^4*x^2*y + 2*a^2*b^6*c^4*x^2*y -     3*a^6*c^6*x^2*y + 5*a^4*b^2*c^6*x^2*y - a^2*b^4*c^6*x^2*y - b^6*c^6*x^2*y + a^4*c^8*x^2*y -     2*a^2*b^2*c^8*x^2*y + b^4*c^8*x^2*y + a^12*x*y^2 - 3*a^10*b^2*x*y^2 + 2*a^8*b^4*x*y^2 +     2*a^6*b^6*x*y^2 - 3*a^4*b^8*x*y^2 + a^2*b^10*x*y^2 - a^10*c^2*x*y^2 + 4*a^8*b^2*c^2*x*y^2 -     7*a^6*b^4*c^2*x*y^2 + 5*a^4*b^6*c^2*x*y^2 - b^10*c^2*x*y^2 + 2*a^6*b^2*c^4*x*y^2 -     a^4*b^4*c^4*x*y^2 - 4*a^2*b^6*c^4*x*y^2 + 3*b^8*c^4*x*y^2 - a^6*c^6*x*y^2 - a^4*b^2*c^6*x*y^2 +     5*a^2*b^4*c^6*x*y^2 - 3*b^6*c^6*x*y^2 + a^4*c^8*x*y^2 - 2*a^2*b^2*c^8*x*y^2 + b^4*c^8*x*y^2 -     a^10*b^2*x^2*z + 3*a^8*b^4*x^2*z - 3*a^6*b^6*x^2*z + a^4*b^8*x^2*z + a^10*c^2*x^2*z -     4*a^6*b^4*c^2*x^2*z + 5*a^4*b^6*c^2*x^2*z - 2*a^2*b^8*c^2*x^2*z - 3*a^8*c^4*x^2*z +     5*a^6*b^2*c^4*x^2*z - a^4*b^4*c^4*x^2*z - a^2*b^6*c^4*x^2*z + b^8*c^4*x^2*z + 2*a^6*c^6*x^2*z -     7*a^4*b^2*c^6*x^2*z + 2*a^2*b^4*c^6*x^2*z - b^6*c^6*x^2*z + 2*a^4*c^8*x^2*z + 4*a^2*b^2*c^8*x^2*z -     3*a^2*c^10*x^2*z - b^2*c^10*x^2*z + c^12*x^2*z - 2*a^12*x*y*z + 4*a^10*b^2*x*y*z +     2*a^8*b^4*x*y*z - 8*a^6*b^6*x*y*z + 2*a^4*b^8*x*y*z + 4*a^2*b^10*x*y*z - 2*b^12*x*y*z +     4*a^10*c^2*x*y*z - 16*a^8*b^2*c^2*x*y*z + 12*a^6*b^4*c^2*x*y*z + 12*a^4*b^6*c^2*x*y*z -     16*a^2*b^8*c^2*x*y*z + 4*b^10*c^2*x*y*z + 2*a^8*c^4*x*y*z + 12*a^6*b^2*c^4*x*y*z -     30*a^4*b^4*c^4*x*y*z + 12*a^2*b^6*c^4*x*y*z + 2*b^8*c^4*x*y*z - 8*a^6*c^6*x*y*z +     12*a^4*b^2*c^6*x*y*z + 12*a^2*b^4*c^6*x*y*z - 8*b^6*c^6*x*y*z + 2*a^4*c^8*x*y*z -     16*a^2*b^2*c^8*x*y*z + 2*b^4*c^8*x*y*z + 4*a^2*c^10*x*y*z + 4*b^2*c^10*x*y*z - 2*c^12*x*y*z +     a^8*b^4*y^2*z - 3*a^6*b^6*y^2*z + 3*a^4*b^8*y^2*z - a^2*b^10*y^2*z - 2*a^8*b^2*c^2*y^2*z +     5*a^6*b^4*c^2*y^2*z - 4*a^4*b^6*c^2*y^2*z + b^10*c^2*y^2*z + a^8*c^4*y^2*z - a^6*b^2*c^4*y^2*z -     a^4*b^4*c^4*y^2*z + 5*a^2*b^6*c^4*y^2*z - 3*b^8*c^4*y^2*z - a^6*c^6*y^2*z + 2*a^4*b^2*c^6*y^2*z -     7*a^2*b^4*c^6*y^2*z + 2*b^6*c^6*y^2*z + 4*a^2*b^2*c^8*y^2*z + 2*b^4*c^8*y^2*z - a^2*c^10*y^2*z -     3*b^2*c^10*y^2*z + c^12*y^2*z + a^12*x*z^2 - a^10*b^2*x*z^2 - a^6*b^6*x*z^2 + a^4*b^8*x*z^2 -     3*a^10*c^2*x*z^2 + 4*a^8*b^2*c^2*x*z^2 + 2*a^6*b^4*c^2*x*z^2 - a^4*b^6*c^2*x*z^2 -     2*a^2*b^8*c^2*x*z^2 + 2*a^8*c^4*x*z^2 - 7*a^6*b^2*c^4*x*z^2 - a^4*b^4*c^4*x*z^2 +     5*a^2*b^6*c^4*x*z^2 + b^8*c^4*x*z^2 + 2*a^6*c^6*x*z^2 + 5*a^4*b^2*c^6*x*z^2 - 4*a^2*b^4*c^6*x*z^2 -     3*b^6*c^6*x*z^2 - 3*a^4*c^8*x*z^2 + 3*b^4*c^8*x*z^2 + a^2*c^10*x*z^2 - b^2*c^10*x*z^2 +     a^8*b^4*y*z^2 - a^6*b^6*y*z^2 - a^2*b^10*y*z^2 + b^12*y*z^2 - 2*a^8*b^2*c^2*y*z^2 -     a^6*b^4*c^2*y*z^2 + 2*a^4*b^6*c^2*y*z^2 + 4*a^2*b^8*c^2*y*z^2 - 3*b^10*c^2*y*z^2 + a^8*c^4*y*z^2 +     5*a^6*b^2*c^4*y*z^2 - a^4*b^4*c^4*y*z^2 - 7*a^2*b^6*c^4*y*z^2 + 2*b^8*c^4*y*z^2 - 3*a^6*c^6*y*z^2 -     4*a^4*b^2*c^6*y*z^2 + 5*a^2*b^4*c^6*y*z^2 + 2*b^6*c^6*y*z^2 + 3*a^4*c^8*y*z^2 - 3*b^4*c^8*y*z^2 -     a^2*c^10*y*z^2 + b^2*c^10*y*z^2");
		exp("K639","a^8*b^6*c^4*x^2*y - a^6*b^8*c^4*x^2*y - a^2*b^12*c^4*x^2*y + b^14*c^4*x^2*y -     2*a^8*b^4*c^6*x^2*y - a^6*b^6*c^6*x^2*y + 2*a^4*b^8*c^6*x^2*y + 4*a^2*b^10*c^6*x^2*y -     3*b^12*c^6*x^2*y + a^8*b^2*c^8*x^2*y + 5*a^6*b^4*c^8*x^2*y - a^4*b^6*c^8*x^2*y -     7*a^2*b^8*c^8*x^2*y + 2*b^10*c^8*x^2*y - 3*a^6*b^2*c^10*x^2*y - 4*a^4*b^4*c^10*x^2*y +     5*a^2*b^6*c^10*x^2*y + 2*b^8*c^10*x^2*y + 3*a^4*b^2*c^12*x^2*y - 3*b^6*c^12*x^2*y -     a^2*b^2*c^14*x^2*y + b^4*c^14*x^2*y + a^14*c^4*x*y^2 - a^12*b^2*c^4*x*y^2 - a^8*b^6*c^4*x*y^2 +     a^6*b^8*c^4*x*y^2 - 3*a^12*c^6*x*y^2 + 4*a^10*b^2*c^6*x*y^2 + 2*a^8*b^4*c^6*x*y^2 -     a^6*b^6*c^6*x*y^2 - 2*a^4*b^8*c^6*x*y^2 + 2*a^10*c^8*x*y^2 - 7*a^8*b^2*c^8*x*y^2 -     a^6*b^4*c^8*x*y^2 + 5*a^4*b^6*c^8*x*y^2 + a^2*b^8*c^8*x*y^2 + 2*a^8*c^10*x*y^2 +     5*a^6*b^2*c^10*x*y^2 - 4*a^4*b^4*c^10*x*y^2 - 3*a^2*b^6*c^10*x*y^2 - 3*a^6*c^12*x*y^2 +     3*a^2*b^4*c^12*x*y^2 + a^4*c^14*x*y^2 - a^2*b^2*c^14*x*y^2 + a^8*b^8*c^2*x^2*z -     3*a^6*b^10*c^2*x^2*z + 3*a^4*b^12*c^2*x^2*z - a^2*b^14*c^2*x^2*z - 2*a^8*b^6*c^4*x^2*z +     5*a^6*b^8*c^4*x^2*z - 4*a^4*b^10*c^4*x^2*z + b^14*c^4*x^2*z + a^8*b^4*c^6*x^2*z -     a^6*b^6*c^6*x^2*z - a^4*b^8*c^6*x^2*z + 5*a^2*b^10*c^6*x^2*z - 3*b^12*c^6*x^2*z -     a^6*b^4*c^8*x^2*z + 2*a^4*b^6*c^8*x^2*z - 7*a^2*b^8*c^8*x^2*z + 2*b^10*c^8*x^2*z +     4*a^2*b^6*c^10*x^2*z + 2*b^8*c^10*x^2*z - a^2*b^4*c^12*x^2*z - 3*b^6*c^12*x^2*z + b^4*c^14*x^2*z -     2*a^14*b^2*c^2*x*y*z + 4*a^12*b^4*c^2*x*y*z + 2*a^10*b^6*c^2*x*y*z - 8*a^8*b^8*c^2*x*y*z +     2*a^6*b^10*c^2*x*y*z + 4*a^4*b^12*c^2*x*y*z - 2*a^2*b^14*c^2*x*y*z + 4*a^12*b^2*c^4*x*y*z -     16*a^10*b^4*c^4*x*y*z + 12*a^8*b^6*c^4*x*y*z + 12*a^6*b^8*c^4*x*y*z - 16*a^4*b^10*c^4*x*y*z +     4*a^2*b^12*c^4*x*y*z + 2*a^10*b^2*c^6*x*y*z + 12*a^8*b^4*c^6*x*y*z - 30*a^6*b^6*c^6*x*y*z +     12*a^4*b^8*c^6*x*y*z + 2*a^2*b^10*c^6*x*y*z - 8*a^8*b^2*c^8*x*y*z + 12*a^6*b^4*c^8*x*y*z +     12*a^4*b^6*c^8*x*y*z - 8*a^2*b^8*c^8*x*y*z + 2*a^6*b^2*c^10*x*y*z - 16*a^4*b^4*c^10*x*y*z +     2*a^2*b^6*c^10*x*y*z + 4*a^4*b^2*c^12*x*y*z + 4*a^2*b^4*c^12*x*y*z - 2*a^2*b^2*c^14*x*y*z -     a^14*b^2*c^2*y^2*z + 3*a^12*b^4*c^2*y^2*z - 3*a^10*b^6*c^2*y^2*z + a^8*b^8*c^2*y^2*z +     a^14*c^4*y^2*z - 4*a^10*b^4*c^4*y^2*z + 5*a^8*b^6*c^4*y^2*z - 2*a^6*b^8*c^4*y^2*z -     3*a^12*c^6*y^2*z + 5*a^10*b^2*c^6*y^2*z - a^8*b^4*c^6*y^2*z - a^6*b^6*c^6*y^2*z +     a^4*b^8*c^6*y^2*z + 2*a^10*c^8*y^2*z - 7*a^8*b^2*c^8*y^2*z + 2*a^6*b^4*c^8*y^2*z -     a^4*b^6*c^8*y^2*z + 2*a^8*c^10*y^2*z + 4*a^6*b^2*c^10*y^2*z - 3*a^6*c^12*y^2*z -     a^4*b^2*c^12*y^2*z + a^4*c^14*y^2*z + a^14*b^4*x*z^2 - 3*a^12*b^6*x*z^2 + 2*a^10*b^8*x*z^2 +     2*a^8*b^10*x*z^2 - 3*a^6*b^12*x*z^2 + a^4*b^14*x*z^2 - a^12*b^4*c^2*x*z^2 + 4*a^10*b^6*c^2*x*z^2 -     7*a^8*b^8*c^2*x*z^2 + 5*a^6*b^10*c^2*x*z^2 - a^2*b^14*c^2*x*z^2 + 2*a^8*b^6*c^4*x*z^2 -     a^6*b^8*c^4*x*z^2 - 4*a^4*b^10*c^4*x*z^2 + 3*a^2*b^12*c^4*x*z^2 - a^8*b^4*c^6*x*z^2 -     a^6*b^6*c^6*x*z^2 + 5*a^4*b^8*c^6*x*z^2 - 3*a^2*b^10*c^6*x*z^2 + a^6*b^4*c^8*x*z^2 -     2*a^4*b^6*c^8*x*z^2 + a^2*b^8*c^8*x*z^2 + a^14*b^4*y*z^2 - 3*a^12*b^6*y*z^2 + 2*a^10*b^8*y*z^2 +     2*a^8*b^10*y*z^2 - 3*a^6*b^12*y*z^2 + a^4*b^14*y*z^2 - a^14*b^2*c^2*y*z^2 + 5*a^10*b^6*c^2*y*z^2 -     7*a^8*b^8*c^2*y*z^2 + 4*a^6*b^10*c^2*y*z^2 - a^4*b^12*c^2*y*z^2 + 3*a^12*b^2*c^4*y*z^2 -     4*a^10*b^4*c^4*y*z^2 - a^8*b^6*c^4*y*z^2 + 2*a^6*b^8*c^4*y*z^2 - 3*a^10*b^2*c^6*y*z^2 +     5*a^8*b^4*c^6*y*z^2 - a^6*b^6*c^6*y*z^2 - a^4*b^8*c^6*y*z^2 + a^8*b^2*c^8*y*z^2 -     2*a^6*b^4*c^8*y*z^2 + a^4*b^6*c^8*y*z^2");
		exp("K640","3*a^4*b^2*c^4*x^2*y - a^2*b^4*c^4*x^2*y - b^6*c^4*x^2*y - 2*a^2*b^2*c^6*x^2*y + b^2*c^8*x^2*y +     a^6*c^4*x*y^2 + a^4*b^2*c^4*x*y^2 - 3*a^2*b^4*c^4*x*y^2 + 2*a^2*b^2*c^6*x*y^2 - a^2*c^8*x*y^2 -     3*a^4*b^4*c^2*x^2*z + 2*a^2*b^6*c^2*x^2*z - b^8*c^2*x^2*z + a^2*b^4*c^4*x^2*z + b^4*c^6*x^2*z +     a^8*c^2*y^2*z - 2*a^6*b^2*c^2*y^2*z + 3*a^4*b^4*c^2*y^2*z - a^4*b^2*c^4*y^2*z - a^4*c^6*y^2*z -     a^6*b^4*x*z^2 + a^2*b^8*x*z^2 - a^4*b^4*c^2*x*z^2 - 2*a^2*b^6*c^2*x*z^2 + 3*a^2*b^4*c^4*x*z^2 -     a^8*b^2*y*z^2 + a^4*b^6*y*z^2 + 2*a^6*b^2*c^2*y*z^2 + a^4*b^4*c^2*y*z^2 - 3*a^4*b^2*c^4*y*z^2");
		exp("K641","-3*a^8*b^2*c^4*x^2*y + a^6*b^4*c^4*x^2*y - a^4*b^6*c^4*x^2*y + 2*a^2*b^8*c^4*x^2*y +     b^10*c^4*x^2*y + 5*a^6*b^2*c^6*x^2*y - 2*a^2*b^6*c^6*x^2*y - 2*b^8*c^6*x^2*y -     2*a^4*b^2*c^8*x^2*y - a^2*b^4*c^8*x^2*y + a^2*b^2*c^10*x^2*y + 2*b^4*c^10*x^2*y - b^2*c^12*x^2*y -     a^10*c^4*x*y^2 - 2*a^8*b^2*c^4*x*y^2 + a^6*b^4*c^4*x*y^2 - a^4*b^6*c^4*x*y^2 +     3*a^2*b^8*c^4*x*y^2 + 2*a^8*c^6*x*y^2 + 2*a^6*b^2*c^6*x*y^2 - 5*a^2*b^6*c^6*x*y^2 +     a^4*b^2*c^8*x*y^2 + 2*a^2*b^4*c^8*x*y^2 - 2*a^4*c^10*x*y^2 - a^2*b^2*c^10*x*y^2 + a^2*c^12*x*y^2 +     3*a^8*b^4*c^2*x^2*z - 5*a^6*b^6*c^2*x^2*z + 2*a^4*b^8*c^2*x^2*z - a^2*b^10*c^2*x^2*z +     b^12*c^2*x^2*z - a^6*b^4*c^4*x^2*z + a^2*b^8*c^4*x^2*z - 2*b^10*c^4*x^2*z + a^4*b^4*c^6*x^2*z +     2*a^2*b^6*c^6*x^2*z - 2*a^2*b^4*c^8*x^2*z + 2*b^6*c^8*x^2*z - b^4*c^10*x^2*z - a^12*c^2*y^2*z +     a^10*b^2*c^2*y^2*z - 2*a^8*b^4*c^2*y^2*z + 5*a^6*b^6*c^2*y^2*z - 3*a^4*b^8*c^2*y^2*z +     2*a^10*c^4*y^2*z - a^8*b^2*c^4*y^2*z + a^4*b^6*c^4*y^2*z - 2*a^6*b^2*c^6*y^2*z -     a^4*b^4*c^6*y^2*z - 2*a^6*c^8*y^2*z + 2*a^4*b^2*c^8*y^2*z + a^4*c^10*y^2*z + a^10*b^4*x*z^2 -     2*a^8*b^6*x*z^2 + 2*a^4*b^10*x*z^2 - a^2*b^12*x*z^2 + 2*a^8*b^4*c^2*x*z^2 - 2*a^6*b^6*c^2*x*z^2 -     a^4*b^8*c^2*x*z^2 + a^2*b^10*c^2*x*z^2 - a^6*b^4*c^4*x*z^2 - 2*a^2*b^8*c^4*x*z^2 +     a^4*b^4*c^6*x*z^2 + 5*a^2*b^6*c^6*x*z^2 - 3*a^2*b^4*c^8*x*z^2 + a^12*b^2*y*z^2 - 2*a^10*b^4*y*z^2 +     2*a^6*b^8*y*z^2 - a^4*b^10*y*z^2 - a^10*b^2*c^2*y*z^2 + a^8*b^4*c^2*y*z^2 + 2*a^6*b^6*c^2*y*z^2 -     2*a^4*b^8*c^2*y*z^2 + 2*a^8*b^2*c^4*y*z^2 + a^4*b^6*c^4*y*z^2 - 5*a^6*b^2*c^6*y*z^2 -     a^4*b^4*c^6*y*z^2 + 3*a^4*b^2*c^8*y*z^2");
		exp("K642","-2*a^2*b^4*c^2*x^2*y + 2*b^6*c^2*x^2*y + a^4*c^4*x^2*y - 2*a^2*b^2*c^4*x^2*y + b^4*c^4*x^2*y -     2*b^2*c^6*x^2*y - c^8*x^2*y - 2*a^6*c^2*x*y^2 + 2*a^4*b^2*c^2*x*y^2 - a^4*c^4*x*y^2 +     2*a^2*b^2*c^4*x*y^2 - b^4*c^4*x*y^2 + 2*a^2*c^6*x*y^2 + c^8*x*y^2 - a^4*b^4*x^2*z + b^8*x^2*z +     2*a^2*b^4*c^2*x^2*z + 2*b^6*c^2*x^2*z + 2*a^2*b^2*c^4*x^2*z - b^4*c^4*x^2*z - 2*b^2*c^6*x^2*z -     2*a^6*b^2*x*y*z + 2*a^2*b^6*x*y*z + 2*a^6*c^2*x*y*z - 2*b^6*c^2*x*y*z - 2*a^2*c^6*x*y*z +     2*b^2*c^6*x*y*z - a^8*y^2*z + a^4*b^4*y^2*z - 2*a^6*c^2*y^2*z - 2*a^4*b^2*c^2*y^2*z +     a^4*c^4*y^2*z - 2*a^2*b^2*c^4*y^2*z + 2*a^2*c^6*y^2*z + 2*a^6*b^2*x*z^2 + a^4*b^4*x*z^2 -     2*a^2*b^6*x*z^2 - b^8*x*z^2 - 2*a^4*b^2*c^2*x*z^2 - 2*a^2*b^4*c^2*x*z^2 + b^4*c^4*x*z^2 +     a^8*y*z^2 + 2*a^6*b^2*y*z^2 - a^4*b^4*y*z^2 - 2*a^2*b^6*y*z^2 + 2*a^4*b^2*c^2*y*z^2 +     2*a^2*b^4*c^2*y*z^2 - a^4*c^4*y*z^2");
		exp("K643","a^4*b^2*c^2*x^2*y - 2*a^2*b^4*c^2*x^2*y + b^6*c^2*x^2*y - 2*a^2*b^2*c^4*x^2*y - b^2*c^6*x^2*y -     a^6*c^2*x*y^2 + 2*a^4*b^2*c^2*x*y^2 - a^2*b^4*c^2*x*y^2 + 2*a^2*b^2*c^4*x*y^2 + a^2*c^6*x*y^2 -     a^4*b^2*c^2*x^2*z + 2*a^2*b^4*c^2*x^2*z + b^6*c^2*x^2*z + 2*a^2*b^2*c^4*x^2*z - b^2*c^6*x^2*z -     a^6*c^2*y^2*z - 2*a^4*b^2*c^2*y^2*z + a^2*b^4*c^2*y^2*z - 2*a^2*b^2*c^4*y^2*z + a^2*c^6*y^2*z +     a^6*b^2*x*z^2 - a^2*b^6*x*z^2 - 2*a^4*b^2*c^2*x*z^2 - 2*a^2*b^4*c^2*x*z^2 + a^2*b^2*c^4*x*z^2 +     a^6*b^2*y*z^2 - a^2*b^6*y*z^2 + 2*a^4*b^2*c^2*y*z^2 + 2*a^2*b^4*c^2*y*z^2 - a^2*b^2*c^4*y*z^2");
		exp("K644","b^2*c^2*x^2*y + c^4*x^2*y - a^2*c^2*x*y^2 - c^4*x*y^2 - b^4*x^2*z - b^2*c^2*x^2*z + a^4*y^2*z +     a^2*c^2*y^2*z + a^2*b^2*x*z^2 + b^4*x*z^2 - a^4*y*z^2 - a^2*b^2*y*z^2");
		exp("K645","a^10*b^2*x^3 - 5*a^8*b^4*x^3 + 10*a^6*b^6*x^3 - 10*a^4*b^8*x^3 + 5*a^2*b^10*x^3 - b^12*x^3 -     a^10*c^2*x^3 + 2*a^6*b^4*c^2*x^3 + 4*a^4*b^6*c^2*x^3 - 9*a^2*b^8*c^2*x^3 + 4*b^10*c^2*x^3 +     5*a^8*c^4*x^3 - 2*a^6*b^2*c^4*x^3 + 2*a^2*b^6*c^4*x^3 - 5*b^8*c^4*x^3 - 10*a^6*c^6*x^3 -     4*a^4*b^2*c^6*x^3 - 2*a^2*b^4*c^6*x^3 + 10*a^4*c^8*x^3 + 9*a^2*b^2*c^8*x^3 + 5*b^4*c^8*x^3 -     5*a^2*c^10*x^3 - 4*b^2*c^10*x^3 + c^12*x^3 + a^12*x^2*y - 3*a^10*b^2*x^2*y + 10*a^6*b^6*x^2*y -     15*a^4*b^8*x^2*y + 9*a^2*b^10*x^2*y - 2*b^12*x^2*y + 2*a^10*c^2*x^2*y + a^8*b^2*c^2*x^2*y -     16*a^6*b^4*c^2*x^2*y + 22*a^4*b^6*c^2*x^2*y - 10*a^2*b^8*c^2*x^2*y + b^10*c^2*x^2*y -     17*a^8*c^4*x^2*y + 10*a^6*b^2*c^4*x^2*y + 16*a^4*b^4*c^4*x^2*y - 10*a^2*b^6*c^4*x^2*y +     b^8*c^4*x^2*y + 28*a^6*c^6*x^2*y - 6*a^4*b^2*c^6*x^2*y + 16*a^2*b^4*c^6*x^2*y + 10*b^6*c^6*x^2*y -     17*a^4*c^8*x^2*y - 7*a^2*b^2*c^8*x^2*y - 16*b^4*c^8*x^2*y + 2*a^2*c^10*x^2*y + 5*b^2*c^10*x^2*y +     c^12*x^2*y + 2*a^12*x*y^2 - 9*a^10*b^2*x*y^2 + 15*a^8*b^4*x*y^2 - 10*a^6*b^6*x*y^2 +     3*a^2*b^10*x*y^2 - b^12*x*y^2 - a^10*c^2*x*y^2 + 10*a^8*b^2*c^2*x*y^2 - 22*a^6*b^4*c^2*x*y^2 +     16*a^4*b^6*c^2*x*y^2 - a^2*b^8*c^2*x*y^2 - 2*b^10*c^2*x*y^2 - a^8*c^4*x*y^2 +     10*a^6*b^2*c^4*x*y^2 - 16*a^4*b^4*c^4*x*y^2 - 10*a^2*b^6*c^4*x*y^2 + 17*b^8*c^4*x*y^2 -     10*a^6*c^6*x*y^2 - 16*a^4*b^2*c^6*x*y^2 + 6*a^2*b^4*c^6*x*y^2 - 28*b^6*c^6*x*y^2 +     16*a^4*c^8*x*y^2 + 7*a^2*b^2*c^8*x*y^2 + 17*b^4*c^8*x*y^2 - 5*a^2*c^10*x*y^2 - 2*b^2*c^10*x*y^2 -     c^12*x*y^2 + a^12*y^3 - 5*a^10*b^2*y^3 + 10*a^8*b^4*y^3 - 10*a^6*b^6*y^3 + 5*a^4*b^8*y^3 -     a^2*b^10*y^3 - 4*a^10*c^2*y^3 + 9*a^8*b^2*c^2*y^3 - 4*a^6*b^4*c^2*y^3 - 2*a^4*b^6*c^2*y^3 +     b^10*c^2*y^3 + 5*a^8*c^4*y^3 - 2*a^6*b^2*c^4*y^3 + 2*a^2*b^6*c^4*y^3 - 5*b^8*c^4*y^3 +     2*a^4*b^2*c^6*y^3 + 4*a^2*b^4*c^6*y^3 + 10*b^6*c^6*y^3 - 5*a^4*c^8*y^3 - 9*a^2*b^2*c^8*y^3 -     10*b^4*c^8*y^3 + 4*a^2*c^10*y^3 + 5*b^2*c^10*y^3 - c^12*y^3 - a^12*x^2*z - 2*a^10*b^2*x^2*z +     17*a^8*b^4*x^2*z - 28*a^6*b^6*x^2*z + 17*a^4*b^8*x^2*z - 2*a^2*b^10*x^2*z - b^12*x^2*z +     3*a^10*c^2*x^2*z - a^8*b^2*c^2*x^2*z - 10*a^6*b^4*c^2*x^2*z + 6*a^4*b^6*c^2*x^2*z +     7*a^2*b^8*c^2*x^2*z - 5*b^10*c^2*x^2*z + 16*a^6*b^2*c^4*x^2*z - 16*a^4*b^4*c^4*x^2*z -     16*a^2*b^6*c^4*x^2*z + 16*b^8*c^4*x^2*z - 10*a^6*c^6*x^2*z - 22*a^4*b^2*c^6*x^2*z +     10*a^2*b^4*c^6*x^2*z - 10*b^6*c^6*x^2*z + 15*a^4*c^8*x^2*z + 10*a^2*b^2*c^8*x^2*z - b^4*c^8*x^2*z -     9*a^2*c^10*x^2*z - b^2*c^10*x^2*z + 2*c^12*x^2*z + a^12*y^2*z + 2*a^10*b^2*y^2*z -     17*a^8*b^4*y^2*z + 28*a^6*b^6*y^2*z - 17*a^4*b^8*y^2*z + 2*a^2*b^10*y^2*z + b^12*y^2*z +     5*a^10*c^2*y^2*z - 7*a^8*b^2*c^2*y^2*z - 6*a^6*b^4*c^2*y^2*z + 10*a^4*b^6*c^2*y^2*z +     a^2*b^8*c^2*y^2*z - 3*b^10*c^2*y^2*z - 16*a^8*c^4*y^2*z + 16*a^6*b^2*c^4*y^2*z +     16*a^4*b^4*c^4*y^2*z - 16*a^2*b^6*c^4*y^2*z + 10*a^6*c^6*y^2*z - 10*a^4*b^2*c^6*y^2*z +     22*a^2*b^4*c^6*y^2*z + 10*b^6*c^6*y^2*z + a^4*c^8*y^2*z - 10*a^2*b^2*c^8*y^2*z - 15*b^4*c^8*y^2*z +     a^2*c^10*y^2*z + 9*b^2*c^10*y^2*z - 2*c^12*y^2*z - 2*a^12*x*z^2 + a^10*b^2*x*z^2 + a^8*b^4*x*z^2 +     10*a^6*b^6*x*z^2 - 16*a^4*b^8*x*z^2 + 5*a^2*b^10*x*z^2 + b^12*x*z^2 + 9*a^10*c^2*x*z^2 -     10*a^8*b^2*c^2*x*z^2 - 10*a^6*b^4*c^2*x*z^2 + 16*a^4*b^6*c^2*x*z^2 - 7*a^2*b^8*c^2*x*z^2 +     2*b^10*c^2*x*z^2 - 15*a^8*c^4*x*z^2 + 22*a^6*b^2*c^4*x*z^2 + 16*a^4*b^4*c^4*x*z^2 -     6*a^2*b^6*c^4*x*z^2 - 17*b^8*c^4*x*z^2 + 10*a^6*c^6*x*z^2 - 16*a^4*b^2*c^6*x*z^2 +     10*a^2*b^4*c^6*x*z^2 + 28*b^6*c^6*x*z^2 + a^2*b^2*c^8*x*z^2 - 17*b^4*c^8*x*z^2 - 3*a^2*c^10*x*z^2 +     2*b^2*c^10*x*z^2 + c^12*x*z^2 - a^12*y*z^2 - 5*a^10*b^2*y*z^2 + 16*a^8*b^4*y*z^2 -     10*a^6*b^6*y*z^2 - a^4*b^8*y*z^2 - a^2*b^10*y*z^2 + 2*b^12*y*z^2 - 2*a^10*c^2*y*z^2 +     7*a^8*b^2*c^2*y*z^2 - 16*a^6*b^4*c^2*y*z^2 + 10*a^4*b^6*c^2*y*z^2 + 10*a^2*b^8*c^2*y*z^2 -     9*b^10*c^2*y*z^2 + 17*a^8*c^4*y*z^2 + 6*a^6*b^2*c^4*y*z^2 - 16*a^4*b^4*c^4*y*z^2 -     22*a^2*b^6*c^4*y*z^2 + 15*b^8*c^4*y*z^2 - 28*a^6*c^6*y*z^2 - 10*a^4*b^2*c^6*y*z^2 +     16*a^2*b^4*c^6*y*z^2 - 10*b^6*c^6*y*z^2 + 17*a^4*c^8*y*z^2 - a^2*b^2*c^8*y*z^2 - 2*a^2*c^10*y*z^2 +     3*b^2*c^10*y*z^2 - c^12*y*z^2 - a^12*z^3 + 4*a^10*b^2*z^3 - 5*a^8*b^4*z^3 + 5*a^4*b^8*z^3 -     4*a^2*b^10*z^3 + b^12*z^3 + 5*a^10*c^2*z^3 - 9*a^8*b^2*c^2*z^3 + 2*a^6*b^4*c^2*z^3 -     2*a^4*b^6*c^2*z^3 + 9*a^2*b^8*c^2*z^3 - 5*b^10*c^2*z^3 - 10*a^8*c^4*z^3 + 4*a^6*b^2*c^4*z^3 -     4*a^2*b^6*c^4*z^3 + 10*b^8*c^4*z^3 + 10*a^6*c^6*z^3 + 2*a^4*b^2*c^6*z^3 - 2*a^2*b^4*c^6*z^3 -     10*b^6*c^6*z^3 - 5*a^4*c^8*z^3 + 5*b^4*c^8*z^3 + a^2*c^10*z^3 - b^2*c^10*z^3");
		exp("K646","a^4*b^2*c^2*x^2*y - b^6*c^2*x^2*y + a^4*c^4*x^2*y + 2*a^2*b^2*c^4*x^2*y + 3*b^4*c^4*x^2*y -     2*a^2*c^6*x^2*y - 3*b^2*c^6*x^2*y + c^8*x^2*y + a^6*c^2*x*y^2 - a^2*b^4*c^2*x*y^2 -     3*a^4*c^4*x*y^2 - 2*a^2*b^2*c^4*x*y^2 - b^4*c^4*x*y^2 + 3*a^2*c^6*x*y^2 + 2*b^2*c^6*x*y^2 -     c^8*x*y^2 - a^4*b^4*x^2*z + 2*a^2*b^6*x^2*z - b^8*x^2*z - a^4*b^2*c^2*x^2*z - 2*a^2*b^4*c^2*x^2*z +     3*b^6*c^2*x^2*z - 3*b^4*c^4*x^2*z + b^2*c^6*x^2*z + a^8*y^2*z - 2*a^6*b^2*y^2*z + a^4*b^4*y^2*z -     3*a^6*c^2*y^2*z + 2*a^4*b^2*c^2*y^2*z + a^2*b^4*c^2*y^2*z + 3*a^4*c^4*y^2*z - a^2*c^6*y^2*z -     a^6*b^2*x*z^2 + 3*a^4*b^4*x*z^2 - 3*a^2*b^6*x*z^2 + b^8*x*z^2 + 2*a^2*b^4*c^2*x*z^2 -     2*b^6*c^2*x*z^2 + a^2*b^2*c^4*x*z^2 + b^4*c^4*x*z^2 - a^8*y*z^2 + 3*a^6*b^2*y*z^2 -     3*a^4*b^4*y*z^2 + a^2*b^6*y*z^2 + 2*a^6*c^2*y*z^2 - 2*a^4*b^2*c^2*y*z^2 - a^4*c^4*y*z^2 -     a^2*b^2*c^4*y*z^2");
		exp("K647","a^8*x^2*y - 3*a^6*b^2*x^2*y + 3*a^4*b^4*x^2*y - a^2*b^6*x^2*y - a^6*c^2*x^2*y +     2*a^4*b^2*c^2*x^2*y - a^2*b^4*c^2*x^2*y - a^4*c^4*x^2*y + a^2*b^2*c^4*x^2*y + a^2*c^6*x^2*y +     a^6*b^2*x*y^2 - 3*a^4*b^4*x*y^2 + 3*a^2*b^6*x*y^2 - b^8*x*y^2 + a^4*b^2*c^2*x*y^2 -     2*a^2*b^4*c^2*x*y^2 + b^6*c^2*x*y^2 - a^2*b^2*c^4*x*y^2 + b^4*c^4*x*y^2 - b^2*c^6*x*y^2 -     a^8*x^2*z + a^6*b^2*x^2*z + a^4*b^4*x^2*z - a^2*b^6*x^2*z + 3*a^6*c^2*x^2*z - 2*a^4*b^2*c^2*x^2*z -     a^2*b^4*c^2*x^2*z - 3*a^4*c^4*x^2*z + a^2*b^2*c^4*x^2*z + a^2*c^6*x^2*z + a^6*b^2*y^2*z -     a^4*b^4*y^2*z - a^2*b^6*y^2*z + b^8*y^2*z + a^4*b^2*c^2*y^2*z + 2*a^2*b^4*c^2*y^2*z -     3*b^6*c^2*y^2*z - a^2*b^2*c^4*y^2*z + 3*b^4*c^4*y^2*z - b^2*c^6*y^2*z - a^6*c^2*x*z^2 -     a^4*b^2*c^2*x*z^2 + a^2*b^4*c^2*x*z^2 + b^6*c^2*x*z^2 + 3*a^4*c^4*x*z^2 + 2*a^2*b^2*c^4*x*z^2 -     b^4*c^4*x*z^2 - 3*a^2*c^6*x*z^2 - b^2*c^6*x*z^2 + c^8*x*z^2 - a^6*c^2*y*z^2 - a^4*b^2*c^2*y*z^2 +     a^2*b^4*c^2*y*z^2 + b^6*c^2*y*z^2 + a^4*c^4*y*z^2 - 2*a^2*b^2*c^4*y*z^2 - 3*b^4*c^4*y*z^2 +     a^2*c^6*y*z^2 + 3*b^2*c^6*y*z^2 - c^8*y*z^2");
		exp("K648","a^4*b^2*x^3 - b^6*x^3 - a^4*c^2*x^3 + 3*b^4*c^2*x^3 - 3*b^2*c^4*x^3 + c^6*x^3 + 3*a^6*x^2*y -     4*a^4*b^2*x^2*y + 5*a^2*b^4*x^2*y - 4*b^6*x^2*y + a^4*c^2*x^2*y - 4*a^2*b^2*c^2*x^2*y +     5*b^4*c^2*x^2*y - a^2*c^4*x^2*y + 2*b^2*c^4*x^2*y - 3*c^6*x^2*y + 4*a^6*x*y^2 - 5*a^4*b^2*x*y^2 +     4*a^2*b^4*x*y^2 - 3*b^6*x*y^2 - 5*a^4*c^2*x*y^2 + 4*a^2*b^2*c^2*x*y^2 - b^4*c^2*x*y^2 -     2*a^2*c^4*x*y^2 + b^2*c^4*x*y^2 + 3*c^6*x*y^2 + a^6*y^3 - a^2*b^4*y^3 - 3*a^4*c^2*y^3 +     b^4*c^2*y^3 + 3*a^2*c^4*y^3 - c^6*y^3 - 3*a^6*x^2*z - a^4*b^2*x^2*z + a^2*b^4*x^2*z + 3*b^6*x^2*z +     4*a^4*c^2*x^2*z + 4*a^2*b^2*c^2*x^2*z - 2*b^4*c^2*x^2*z - 5*a^2*c^4*x^2*z - 5*b^2*c^4*x^2*z +     4*c^6*x^2*z - 20*a^4*b^2*x*y*z + 20*a^2*b^4*x*y*z + 20*a^4*c^2*x*y*z - 20*b^4*c^2*x*y*z -     20*a^2*c^4*x*y*z + 20*b^2*c^4*x*y*z - 3*a^6*y^2*z - a^4*b^2*y^2*z + a^2*b^4*y^2*z + 3*b^6*y^2*z +     2*a^4*c^2*y^2*z - 4*a^2*b^2*c^2*y^2*z - 4*b^4*c^2*y^2*z + 5*a^2*c^4*y^2*z + 5*b^2*c^4*y^2*z -     4*c^6*y^2*z - 4*a^6*x*z^2 + 5*a^4*b^2*x*z^2 + 2*a^2*b^4*x*z^2 - 3*b^6*x*z^2 + 5*a^4*c^2*x*z^2 -     4*a^2*b^2*c^2*x*z^2 - b^4*c^2*x*z^2 - 4*a^2*c^4*x*z^2 + b^2*c^4*x*z^2 + 3*c^6*x*z^2 + 3*a^6*y*z^2 -     2*a^4*b^2*y*z^2 - 5*a^2*b^4*y*z^2 + 4*b^6*y*z^2 + a^4*c^2*y*z^2 + 4*a^2*b^2*c^2*y*z^2 -     5*b^4*c^2*y*z^2 - a^2*c^4*y*z^2 + 4*b^2*c^4*y*z^2 - 3*c^6*y*z^2 - a^6*z^3 + 3*a^4*b^2*z^3 -     3*a^2*b^4*z^3 + b^6*z^3 + a^2*c^4*z^3 - b^2*c^4*z^3");
		exp("K649","(-a^4)*b^2*x^3 + b^6*x^3 + a^4*c^2*x^3 - 3*b^4*c^2*x^3 + 3*b^2*c^4*x^3 - c^6*x^3 + 3*a^6*x^2*y -     14*a^4*b^2*x^2*y + 13*a^2*b^4*x^2*y - 2*b^6*x^2*y - a^4*c^2*x^2*y - 8*a^2*b^2*c^2*x^2*y +     7*b^4*c^2*x^2*y - 5*a^2*c^4*x^2*y - 8*b^2*c^4*x^2*y + 3*c^6*x^2*y + 2*a^6*x*y^2 -     13*a^4*b^2*x*y^2 + 14*a^2*b^4*x*y^2 - 3*b^6*x*y^2 - 7*a^4*c^2*x*y^2 + 8*a^2*b^2*c^2*x*y^2 +     b^4*c^2*x*y^2 + 8*a^2*c^4*x*y^2 + 5*b^2*c^4*x*y^2 - 3*c^6*x*y^2 - a^6*y^3 + a^2*b^4*y^3 +     3*a^4*c^2*y^3 - b^4*c^2*y^3 - 3*a^2*c^4*y^3 + c^6*y^3 - 3*a^6*x^2*z + a^4*b^2*x^2*z +     5*a^2*b^4*x^2*z - 3*b^6*x^2*z + 14*a^4*c^2*x^2*z + 8*a^2*b^2*c^2*x^2*z + 8*b^4*c^2*x^2*z -     13*a^2*c^4*x^2*z - 7*b^2*c^4*x^2*z + 2*c^6*x^2*z - 4*a^4*b^2*x*y*z + 4*a^2*b^4*x*y*z +     4*a^4*c^2*x*y*z - 4*b^4*c^2*x*y*z - 4*a^2*c^4*x*y*z + 4*b^2*c^4*x*y*z + 3*a^6*y^2*z -     5*a^4*b^2*y^2*z - a^2*b^4*y^2*z + 3*b^6*y^2*z - 8*a^4*c^2*y^2*z - 8*a^2*b^2*c^2*y^2*z -     14*b^4*c^2*y^2*z + 7*a^2*c^4*y^2*z + 13*b^2*c^4*y^2*z - 2*c^6*y^2*z - 2*a^6*x*z^2 +     7*a^4*b^2*x*z^2 - 8*a^2*b^4*x*z^2 + 3*b^6*x*z^2 + 13*a^4*c^2*x*z^2 - 8*a^2*b^2*c^2*x*z^2 -     5*b^4*c^2*x*z^2 - 14*a^2*c^4*x*z^2 - b^2*c^4*x*z^2 + 3*c^6*x*z^2 - 3*a^6*y*z^2 + 8*a^4*b^2*y*z^2 -     7*a^2*b^4*y*z^2 + 2*b^6*y*z^2 + 5*a^4*c^2*y*z^2 + 8*a^2*b^2*c^2*y*z^2 - 13*b^4*c^2*y*z^2 +     a^2*c^4*y*z^2 + 14*b^2*c^4*y*z^2 - 3*c^6*y*z^2 + a^6*z^3 - 3*a^4*b^2*z^3 + 3*a^2*b^4*z^3 -     b^6*z^3 - a^2*c^4*z^3 + b^2*c^4*z^3");
		exp("K650","a^4*b^2*x^3 - b^6*x^3 - a^4*c^2*x^3 + 3*b^4*c^2*x^3 - 3*b^2*c^4*x^3 + c^6*x^3 + 5*a^6*x^2*y -     10*a^4*b^2*x^2*y + 11*a^2*b^4*x^2*y - 6*b^6*x^2*y + a^4*c^2*x^2*y - 8*a^2*b^2*c^2*x^2*y +     9*b^4*c^2*x^2*y - 3*a^2*c^4*x^2*y - 3*c^6*x^2*y + 6*a^6*x*y^2 - 11*a^4*b^2*x*y^2 +     10*a^2*b^4*x*y^2 - 5*b^6*x*y^2 - 9*a^4*c^2*x*y^2 + 8*a^2*b^2*c^2*x*y^2 - b^4*c^2*x*y^2 +     3*b^2*c^4*x*y^2 + 3*c^6*x*y^2 + a^6*y^3 - a^2*b^4*y^3 - 3*a^4*c^2*y^3 + b^4*c^2*y^3 +     3*a^2*c^4*y^3 - c^6*y^3 - 5*a^6*x^2*z - a^4*b^2*x^2*z + 3*a^2*b^4*x^2*z + 3*b^6*x^2*z +     10*a^4*c^2*x^2*z + 8*a^2*b^2*c^2*x^2*z - 11*a^2*c^4*x^2*z - 9*b^2*c^4*x^2*z + 6*c^6*x^2*z -     28*a^4*b^2*x*y*z + 28*a^2*b^4*x*y*z + 28*a^4*c^2*x*y*z - 28*b^4*c^2*x*y*z - 28*a^2*c^4*x*y*z +     28*b^2*c^4*x*y*z - 3*a^6*y^2*z - 3*a^4*b^2*y^2*z + a^2*b^4*y^2*z + 5*b^6*y^2*z -     8*a^2*b^2*c^2*y^2*z - 10*b^4*c^2*y^2*z + 9*a^2*c^4*y^2*z + 11*b^2*c^4*y^2*z - 6*c^6*y^2*z -     6*a^6*x*z^2 + 9*a^4*b^2*x*z^2 - 3*b^6*x*z^2 + 11*a^4*c^2*x*z^2 - 8*a^2*b^2*c^2*x*z^2 -     3*b^4*c^2*x*z^2 - 10*a^2*c^4*x*z^2 + b^2*c^4*x*z^2 + 5*c^6*x*z^2 + 3*a^6*y*z^2 - 9*a^2*b^4*y*z^2 +     6*b^6*y*z^2 + 3*a^4*c^2*y*z^2 + 8*a^2*b^2*c^2*y*z^2 - 11*b^4*c^2*y*z^2 - a^2*c^4*y*z^2 +     10*b^2*c^4*y*z^2 - 5*c^6*y*z^2 - a^6*z^3 + 3*a^4*b^2*z^3 - 3*a^2*b^4*z^3 + b^6*z^3 + a^2*c^4*z^3 -     b^2*c^4*z^3");
		exp("K651","(-a^4)*b^2*x^3 + b^6*x^3 + a^4*c^2*x^3 - 3*b^4*c^2*x^3 + 3*b^2*c^4*x^3 - c^6*x^3 + 2*a^6*x^2*y -     11*a^4*b^2*x^2*y + 10*a^2*b^4*x^2*y - b^6*x^2*y - a^4*c^2*x^2*y - 6*a^2*b^2*c^2*x^2*y +     5*b^4*c^2*x^2*y - 4*a^2*c^4*x^2*y - 7*b^2*c^4*x^2*y + 3*c^6*x^2*y + a^6*x*y^2 - 10*a^4*b^2*x*y^2 +     11*a^2*b^4*x*y^2 - 2*b^6*x*y^2 - 5*a^4*c^2*x*y^2 + 6*a^2*b^2*c^2*x*y^2 + b^4*c^2*x*y^2 +     7*a^2*c^4*x*y^2 + 4*b^2*c^4*x*y^2 - 3*c^6*x*y^2 - a^6*y^3 + a^2*b^4*y^3 + 3*a^4*c^2*y^3 -     b^4*c^2*y^3 - 3*a^2*c^4*y^3 + c^6*y^3 - 2*a^6*x^2*z + a^4*b^2*x^2*z + 4*a^2*b^4*x^2*z -     3*b^6*x^2*z + 11*a^4*c^2*x^2*z + 6*a^2*b^2*c^2*x^2*z + 7*b^4*c^2*x^2*z - 10*a^2*c^4*x^2*z -     5*b^2*c^4*x^2*z + c^6*x^2*z + 3*a^6*y^2*z - 4*a^4*b^2*y^2*z - a^2*b^4*y^2*z + 2*b^6*y^2*z -     7*a^4*c^2*y^2*z - 6*a^2*b^2*c^2*y^2*z - 11*b^4*c^2*y^2*z + 5*a^2*c^4*y^2*z + 10*b^2*c^4*y^2*z -     c^6*y^2*z - a^6*x*z^2 + 5*a^4*b^2*x*z^2 - 7*a^2*b^4*x*z^2 + 3*b^6*x*z^2 + 10*a^4*c^2*x*z^2 -     6*a^2*b^2*c^2*x*z^2 - 4*b^4*c^2*x*z^2 - 11*a^2*c^4*x*z^2 - b^2*c^4*x*z^2 + 2*c^6*x*z^2 -     3*a^6*y*z^2 + 7*a^4*b^2*y*z^2 - 5*a^2*b^4*y*z^2 + b^6*y*z^2 + 4*a^4*c^2*y*z^2 +     6*a^2*b^2*c^2*y*z^2 - 10*b^4*c^2*y*z^2 + a^2*c^4*y*z^2 + 11*b^2*c^4*y*z^2 - 2*c^6*y*z^2 + a^6*z^3 -     3*a^4*b^2*z^3 + 3*a^2*b^4*z^3 - b^6*z^3 - a^2*c^4*z^3 + b^2*c^4*z^3");
		exp("K652","(-a^4)*b^2*x^3 + b^6*x^3 + a^4*c^2*x^3 - 3*b^4*c^2*x^3 + 3*b^2*c^4*x^3 - c^6*x^3 + a^6*x^2*y -     8*a^4*b^2*x^2*y + 7*a^2*b^4*x^2*y - a^4*c^2*x^2*y - 4*a^2*b^2*c^2*x^2*y + 3*b^4*c^2*x^2*y -     3*a^2*c^4*x^2*y - 6*b^2*c^4*x^2*y + 3*c^6*x^2*y - 7*a^4*b^2*x*y^2 + 8*a^2*b^4*x*y^2 - b^6*x*y^2 -     3*a^4*c^2*x*y^2 + 4*a^2*b^2*c^2*x*y^2 + b^4*c^2*x*y^2 + 6*a^2*c^4*x*y^2 + 3*b^2*c^4*x*y^2 -     3*c^6*x*y^2 - a^6*y^3 + a^2*b^4*y^3 + 3*a^4*c^2*y^3 - b^4*c^2*y^3 - 3*a^2*c^4*y^3 + c^6*y^3 -     a^6*x^2*z + a^4*b^2*x^2*z + 3*a^2*b^4*x^2*z - 3*b^6*x^2*z + 8*a^4*c^2*x^2*z + 4*a^2*b^2*c^2*x^2*z +     6*b^4*c^2*x^2*z - 7*a^2*c^4*x^2*z - 3*b^2*c^4*x^2*z + 4*a^4*b^2*x*y*z - 4*a^2*b^4*x*y*z -     4*a^4*c^2*x*y*z + 4*b^4*c^2*x*y*z + 4*a^2*c^4*x*y*z - 4*b^2*c^4*x*y*z + 3*a^6*y^2*z -     3*a^4*b^2*y^2*z - a^2*b^4*y^2*z + b^6*y^2*z - 6*a^4*c^2*y^2*z - 4*a^2*b^2*c^2*y^2*z -     8*b^4*c^2*y^2*z + 3*a^2*c^4*y^2*z + 7*b^2*c^4*y^2*z + 3*a^4*b^2*x*z^2 - 6*a^2*b^4*x*z^2 +     3*b^6*x*z^2 + 7*a^4*c^2*x*z^2 - 4*a^2*b^2*c^2*x*z^2 - 3*b^4*c^2*x*z^2 - 8*a^2*c^4*x*z^2 -     b^2*c^4*x*z^2 + c^6*x*z^2 - 3*a^6*y*z^2 + 6*a^4*b^2*y*z^2 - 3*a^2*b^4*y*z^2 + 3*a^4*c^2*y*z^2 +     4*a^2*b^2*c^2*y*z^2 - 7*b^4*c^2*y*z^2 + a^2*c^4*y*z^2 + 8*b^2*c^4*y*z^2 - c^6*y*z^2 + a^6*z^3 -     3*a^4*b^2*z^3 + 3*a^2*b^4*z^3 - b^6*z^3 - a^2*c^4*z^3 + b^2*c^4*z^3");
		exp("K653","(-a^4)*b^2*c^2*x^2*y + a^2*c^6*x^2*y + a^2*b^4*c^2*x*y^2 - b^2*c^6*x*y^2 - a^2*b^6*x^2*z +     a^4*b^2*c^2*x^2*z + a^6*b^2*y^2*z - a^2*b^4*c^2*y^2*z + b^6*c^2*x*z^2 - a^2*b^2*c^4*x*z^2 -     a^6*c^2*y*z^2 + a^2*b^2*c^4*y*z^2");
		exp("K654","(-a^4)*c^2*x^2*y + 2*a^2*b^2*c^2*x^2*y - b^4*c^2*x^2*y - 2*a^3*c^3*x^2*y + 2*a^2*b*c^3*x^2*y -     2*a*b^2*c^3*x^2*y + 2*b^3*c^3*x^2*y + 2*a*c^5*x^2*y - 2*b*c^5*x^2*y + c^6*x^2*y + a^4*c^2*x*y^2 -     2*a^2*b^2*c^2*x*y^2 + b^4*c^2*x*y^2 - 2*a^3*c^3*x*y^2 + 2*a^2*b*c^3*x*y^2 - 2*a*b^2*c^3*x*y^2 +     2*b^3*c^3*x*y^2 + 2*a*c^5*x*y^2 - 2*b*c^5*x*y^2 - c^6*x*y^2 + a^4*b^2*x^2*z + 2*a^3*b^3*x^2*z -     2*a*b^5*x^2*z - b^6*x^2*z - 2*a^2*b^3*c*x^2*z + 2*b^5*c*x^2*z - 2*a^2*b^2*c^2*x^2*z +     2*a*b^3*c^2*x^2*z - 2*b^3*c^3*x^2*z + b^2*c^4*x^2*z + 2*a^5*b*x*y*z + 4*a^4*b^2*x*y*z -     4*a^2*b^4*x*y*z - 2*a*b^5*x*y*z - 2*a^5*c*x*y*z + 2*a^3*b^2*c*x*y*z - 2*a^2*b^3*c*x*y*z +     2*b^5*c*x*y*z - 4*a^4*c^2*x*y*z - 2*a^3*b*c^2*x*y*z + 2*a*b^3*c^2*x*y*z + 4*b^4*c^2*x*y*z +     2*a^2*b*c^3*x*y*z - 2*a*b^2*c^3*x*y*z + 4*a^2*c^4*x*y*z - 4*b^2*c^4*x*y*z + 2*a*c^5*x*y*z -     2*b*c^5*x*y*z + a^6*y^2*z + 2*a^5*b*y^2*z - 2*a^3*b^3*y^2*z - a^2*b^4*y^2*z - 2*a^5*c*y^2*z +     2*a^3*b^2*c*y^2*z - 2*a^3*b*c^2*y^2*z + 2*a^2*b^2*c^2*y^2*z + 2*a^3*c^3*y^2*z - a^2*c^4*y^2*z -     a^4*b^2*x*z^2 + 2*a^3*b^3*x*z^2 - 2*a*b^5*x*z^2 + b^6*x*z^2 - 2*a^2*b^3*c*x*z^2 + 2*b^5*c*x*z^2 +     2*a^2*b^2*c^2*x*z^2 + 2*a*b^3*c^2*x*z^2 - 2*b^3*c^3*x*z^2 - b^2*c^4*x*z^2 - a^6*y*z^2 +     2*a^5*b*y*z^2 - 2*a^3*b^3*y*z^2 + a^2*b^4*y*z^2 - 2*a^5*c*y*z^2 + 2*a^3*b^2*c*y*z^2 -     2*a^3*b*c^2*y*z^2 - 2*a^2*b^2*c^2*y*z^2 + 2*a^3*c^3*y*z^2 + a^2*c^4*y*z^2");
		exp("K655","-3*a^4*b^2*x^3 + 2*a^2*b^4*x^3 + b^6*x^3 + 3*a^4*c^2*x^3 - 3*b^4*c^2*x^3 - 2*a^2*c^4*x^3 +     3*b^2*c^4*x^3 - c^6*x^3 - 3*a^6*x^2*y - 2*a^4*b^2*x^2*y + a^2*b^4*x^2*y + 4*b^6*x^2*y +     12*a^4*c^2*x^2*y - 2*a^2*b^2*c^2*x^2*y - 10*b^4*c^2*x^2*y - 7*a^2*c^4*x^2*y + 8*b^2*c^4*x^2*y -     2*c^6*x^2*y - 4*a^6*x*y^2 - a^4*b^2*x*y^2 + 2*a^2*b^4*x*y^2 + 3*b^6*x*y^2 + 10*a^4*c^2*x*y^2 +     2*a^2*b^2*c^2*x*y^2 - 12*b^4*c^2*x*y^2 - 8*a^2*c^4*x*y^2 + 7*b^2*c^4*x*y^2 + 2*c^6*x*y^2 -     a^6*y^3 - 2*a^4*b^2*y^3 + 3*a^2*b^4*y^3 + 3*a^4*c^2*y^3 - 3*b^4*c^2*y^3 - 3*a^2*c^4*y^3 +     2*b^2*c^4*y^3 + c^6*y^3 + 3*a^6*x^2*z - 12*a^4*b^2*x^2*z + 7*a^2*b^4*x^2*z + 2*b^6*x^2*z +     2*a^4*c^2*x^2*z + 2*a^2*b^2*c^2*x^2*z - 8*b^4*c^2*x^2*z - a^2*c^4*x^2*z + 10*b^2*c^4*x^2*z -     4*c^6*x^2*z - 16*a^4*b^2*x*y*z + 16*a^2*b^4*x*y*z + 16*a^4*c^2*x*y*z - 16*b^4*c^2*x*y*z -     16*a^2*c^4*x*y*z + 16*b^2*c^4*x*y*z - 2*a^6*y^2*z - 7*a^4*b^2*y^2*z + 12*a^2*b^4*y^2*z -     3*b^6*y^2*z + 8*a^4*c^2*y^2*z - 2*a^2*b^2*c^2*y^2*z - 2*b^4*c^2*y^2*z - 10*a^2*c^4*y^2*z +     b^2*c^4*y^2*z + 4*c^6*y^2*z + 4*a^6*x*z^2 - 10*a^4*b^2*x*z^2 + 8*a^2*b^4*x*z^2 - 2*b^6*x*z^2 +     a^4*c^2*x*z^2 - 2*a^2*b^2*c^2*x*z^2 - 7*b^4*c^2*x*z^2 - 2*a^2*c^4*x*z^2 + 12*b^2*c^4*x*z^2 -     3*c^6*x*z^2 + 2*a^6*y*z^2 - 8*a^4*b^2*y*z^2 + 10*a^2*b^4*y*z^2 - 4*b^6*y*z^2 + 7*a^4*c^2*y*z^2 +     2*a^2*b^2*c^2*y*z^2 - b^4*c^2*y*z^2 - 12*a^2*c^4*y*z^2 + 2*b^2*c^4*y*z^2 + 3*c^6*y*z^2 + a^6*z^3 -     3*a^4*b^2*z^3 + 3*a^2*b^4*z^3 - b^6*z^3 + 2*a^4*c^2*z^3 - 2*b^4*c^2*z^3 - 3*a^2*c^4*z^3 +     3*b^2*c^4*z^3");
		exp("K656","(x+y+z)^3-27*x*y*z");
		
	}
	public static class NullOutputStream extends OutputStream {
		  @Override
		  public void write(int b) throws IOException {
		  }
	}
	private static void exp(String name,String s){
		String cycloCheck = "", symhomCheck ="";
		//System.setOut(new PrintStream(new NullOutputStream()));
		try {
			
			app.getKernel().getGeoGebraCAS().evaluateRaw("g:=coeff3("+s+")");
			//300,210,120,111
			//033,123,213,222
			cycloCheck = app.getKernel().getGeoGebraCAS().evaluateRaw(
					"{normal(g[0][3][3]-cyc(g[3][3][0])),normal(g[0][3][3]-cyc(cyc(g[3][0][3])))," +
					"normal(g[3][2][1]-cyc(g[2][1][3])),normal(g[3][2][1]-cyc(cyc(g[1][3][2])))," +
					"normal(g[3][1][2]-cyc(g[1][2][3])),normal(g[3][1][2]-cyc(cyc(g[2][3][1])))," +
					"normal(g[2][2][2]-cyc(g[2][2][2]))}");
			symhomCheck = app.getKernel().getGeoGebraCAS().evaluateRaw(
					
					"normal(g[2][2][2]-swapab(g[2][2][2]))");
			
			
			
		} catch (Throwable e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		if(!"0".equals(symhomCheck) ){
		//	System.out.println("weird hom"+name+":"+symhomCheck);
		}
		
		if(!"{0,0,0,0,0,0,0}".equals(cycloCheck) ){
			acyc.add(name);
			if(!name.endsWith("A") && !name.endsWith("B") && !name.endsWith("C")){
				throw new IllegalStateException(
						"not cyclic" + name + ":" + cycloCheck);
			}
		}
		
		else{
			String symCheck = "", asymCheck = "";
			try {
				//300,210,120,111
				//033,123,213,222
				symCheck = app.getKernel().getGeoGebraCAS().evaluateRaw(
						"normal(g[3][2][1]-swapab(g[2][3][1]))");		
				asymCheck = app.getKernel().getGeoGebraCAS().evaluateRaw(
						"normal(g[3][2][1]+swapab(g[2][3][1]))");
				// coeffs = app.getKernel().getGeoGebraCAS().evaluateRaw(
				// "{g[0][3][3],g[1][2][3],g[2][2][2]}))");
				
				
			} catch (Throwable e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			if("0".equals(symCheck)){
				sym.add(name);
				//System.out.println("case "+name.substring(1,4)+":return s("+toJava(coeffs)+");");
			}else if("0".equals(asymCheck)){
				asym.add(name);
				//System.out.println("case "+name.substring(1,4)+":return a("+toJava(coeffs)+");");
			}else{
				nosym.add(name+":"+symCheck+"\n"+asymCheck);
			}
		}
	}

}
